/**
 * AuditData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.audit;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.general.BaseGeneralInfo;
import com.smartmatic.waffle.domain.user.UserData;

/**
 * DTO class that represents an auditory object.
 */
@Entity
@org.hibernate.annotations.Entity(mutable = false)
@Table(name = "audit", uniqueConstraints = { @UniqueConstraint(columnNames = "audit_custom_code") })
public class AuditData extends BaseGeneralInfo {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The date. */
	private Date date;

	/** The device ip. */
	private String deviceIp;

	/** The user object. */
	private UserData user;

	/** The auditory results set. */
	private Set<AuditResultData> results = new HashSet<AuditResultData>();

	/**
	 * Full constructor.
	 * 
	 * @param date
	 *            the date
	 * @param deviceIp
	 *            the device ip
	 * @param user
	 *            the user
	 * @param customCode
	 *            the custom code
	 * @param results
	 *            the results
	 */
	public AuditData(Date date, String deviceIp, UserData user,
			String customCode, Set<AuditResultData> results) {
		this.date = date;
		this.deviceIp = deviceIp;
		this.user = user;
		this.customCode = customCode;
		this.results = results;
	}

	/**
	 * Getter for the date attribute.
	 * 
	 * @return the date
	 */
	@Column(name = "audit_date", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	public Date getDate() {
		return this.date;
	}

	/**
	 * Setter for the date attribute.
	 * 
	 * @param date
	 *            the date to set
	 */
	public void setDate(Date date) {
		this.date = date;
	}

	/**
	 * Getter for the deviceIp attribute.
	 * 
	 * @return the deviceIp
	 */
	@Column(name = "audit_device_ip", nullable = false)
	public String getDeviceIp() {
		return this.deviceIp;
	}

	/**
	 * Setter for the deviceIp attribute.
	 * 
	 * @param deviceIp
	 *            the deviceIp to set
	 */
	public void setDeviceIp(String deviceIp) {
		this.deviceIp = deviceIp;
	}

	/**
	 * Getter for the user attribute.
	 * 
	 * @return the user
	 */
	@ManyToOne
	@JoinColumn(name = "user_code", nullable = false, updatable = true)
	public UserData getUser() {
		return this.user;
	}

	/**
	 * Setter for the user attribute.
	 * 
	 * @param user
	 *            the user to set
	 */
	public void setUser(UserData user) {
		this.user = user;
	}

	/**
	 * Getter for the results attribute.
	 * 
	 * @return the results
	 */
	@OneToMany(mappedBy = "audit", cascade = CascadeType.ALL)
	public Set<AuditResultData> getResults() {
		return this.results;
	}

	/**
	 * Setter for the results attribute.
	 * 
	 * @param results
	 *            the results to set
	 */
	public void setResults(Set<AuditResultData> results) {
		this.results = results;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getCode()
	 */
	@Id
	@Column(name = "audit_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_audit")
	@SequenceGenerator(name = "s_audit", sequenceName = "s_audit", allocationSize = 1, initialValue = 1)
	@Override
	public long getCode() {
		return super.getCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#setCode(long)
	 */
	@Override
	public void setCode(long code) {
		super.setCode(code);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#setCustomCode(
	 * java.lang.String)
	 */
	@Override
	public void setCustomCode(String customCode) {
		super.setCustomCode(customCode);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getCustomCode()
	 */
	@Column(name = "audit_custom_code", nullable = false)
	@Override
	public String getCustomCode() {
		return super.getCustomCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.domain.BaseDataObject#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object anObject) {
		if (anObject instanceof AuditData) {
			AuditData otherAudit = (AuditData) anObject;
			return otherAudit.customCode.equals(this.customCode);
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.domain.BaseDataObject#hashCode()
	 */
	@Override
	public int hashCode() {
		return (int) this.code;
	}

}
