/**
 * ContestGroupData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.contest;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.smartmatic.waffle.lang.annotations.basic.ValidatorListener;
import com.smartmatic.waffle.persistence.metadata.MultilingualPropertyAnnotation;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * JavaBean representing a group of contests.
 */
@Entity
@XStreamAlias("contest-group")
@Table(name = "contest_group", uniqueConstraints = { @UniqueConstraint(columnNames = "contest_group_code") })
@ValidatorListener(name = "com.smartmatic.waffle.validator.contest.ContestGroupDataValidator")
public class ContestGroupData extends GeneralInfo {

	/** The Constant serialVersionUID. */
	@XStreamOmitField
	private static final long serialVersionUID = -412085660697774424L;

	/** The regions. */
	@XStreamOmitField
	private Set<RegionData> regions;

	/** The election. */
	@XStreamOmitField
	private ElectionData election;

	/** The contests. */
	@XStreamOmitField
	private Set<ContestData> contests;

	/** The contest codes string. */
	@XStreamImplicit(itemFieldName = "contest-code")
	private List<String> contestCodesString = new ArrayList<String>();

	/** The description. */
	@XStreamAlias("description-key")
	@XStreamAsAttribute
	String description;

	/** The contest group name. */
	@XStreamAlias("name-key")
	@XStreamAsAttribute
	private String contestGroupName;

	/** The logical voting group. */
	@XStreamOmitField
	private Set<LogicalVotingGroupData> logicalVotingGroup;

	/** The original custom code. */
	@XStreamOmitField
	private String originalCustomCode;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getVersion()
	 */
	@Override
	@Transient
	public int getVersion() {
		return this.version;
	}

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "contest_group_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_contest_group")
	@SequenceGenerator(name = "s_contest_group", sequenceName = "s_contest_group", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode.
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "contest_group_custom_code", nullable = false, unique = true, length = 100)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@MultilingualPropertyAnnotation
	@Column(name = "contest_group_name", length = 255)
	public String getContestGroupName() {
		return this.contestGroupName;
	}

	/**
	 * Sets the contest group name.
	 * 
	 * @param contestGroupName
	 *            the contestGroupName to set
	 */
	public void setContestGroupName(String contestGroupName) {
		this.contestGroupName = contestGroupName;
		this.description = contestGroupName;
	}

	/**
	 * Setter of <code>description</code> property.
	 * 
	 * @param description
	 *            The <code>description</code> to set.
	 * @see #description
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@MultilingualPropertyAnnotation
	@Column(name = "contest_group_description", length = 255)
	public String getDescription() {
		return this.description;
	}

	/**
	 * Getter of <code>electionDataParent</code> property.
	 * 
	 * @return <code>electionDataParent</code> property value.
	 * @see #election
	 */
	@Transient
	public ElectionData getElection() {
		return this.election;
	}

	/**
	 * Setter for the <code>electionDataParent</code> attribute.
	 * 
	 * @param electionDataParent
	 *            the electionDataParent to set
	 * @see ContestGroupData#election
	 */
	public void setElection(ElectionData electionDataParent) {
		this.election = electionDataParent;
	}

	/**
	 * Getter for the <code>contests</code> attribute.
	 * 
	 * @return <code>contests</code> attribute value.
	 * @see ContestGroupData#contests
	 */
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "contest_group_contest", inverseJoinColumns = @JoinColumn(name = "contest_code", referencedColumnName = "contest_code"), joinColumns = @JoinColumn(name = "contest_group_code", referencedColumnName = "contest_group_code"))
	public Set<ContestData> getContests() {
		return this.contests;
	}

	/**
	 * Setter for the <code>contests</code> attribute.
	 * 
	 * @param contests
	 *            the contests to set
	 * @see ContestGroupData#contests
	 */
	public void setContests(Set<ContestData> contests) {
		this.contests = contests;
		if (contests != null) {
			for (Iterator iterator = contests.iterator(); iterator.hasNext();) {
				ContestData contestData = (ContestData) iterator.next();

				this.contestCodesString.add(String.valueOf(contestData
						.getCode()));
			}
		}
	}

	/**
	 * Setter of {@link #contestCodesString} field.
	 * 
	 * @param contestCodes
	 *            The List<String> to set.
	 */
	public void setContestCodesString(List<String> contestCodes) {
		this.contestCodesString = contestCodes;
	}

	/**
	 * Getter of {@link #contestCodesString} field.
	 * 
	 * @return {@link #contestCodesString} value.
	 */
	@Transient
	public List<String> getContestCodesString() {
		return this.contestCodesString;
	}

	/**
	 * Getter of {@link #logicalVotingGroup} field.
	 * 
	 * @return {@link #logicalVotingGroup} value.
	 */
	@OneToMany(mappedBy = "contestGroup", targetEntity = LogicalVotingGroupData.class)
	public Set<LogicalVotingGroupData> getLogicalVotingGroup() {
		return this.logicalVotingGroup;
	}

	/**
	 * Setter of {@link #logicalVotingGroup} field.
	 * 
	 * @param logicalVotingGroup
	 *            The LogicalVotingGroupData to set.
	 */
	public void setLogicalVotingGroup(
			Set<LogicalVotingGroupData> logicalVotingGroup) {
		this.logicalVotingGroup = logicalVotingGroup;
	}

	/**
	 * Setter of {@link #regions} field.
	 * 
	 * @param regions
	 *            The regions to set.
	 */
	public void setRegions(Set<RegionData> regions) {
		this.regions = regions;
	}

	/**
	 * Getter of {@link #regions} field.
	 * 
	 * @return {@link #regions} value.
	 */
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "region_contest_group", joinColumns = { @JoinColumn(name = "contest_group_code", referencedColumnName = "contest_group_code") }, inverseJoinColumns = { @JoinColumn(name = "region_code", referencedColumnName = "region_code") })
	public Set<RegionData> getRegions() {
		return this.regions;
	}

	/**
	 * Getter of <code>originalCustomCode</code> property.
	 * 
	 * @return <code>originalCustomCode</code> property value.
	 * @see #originalCustomCode
	 */
	@Transient
	public String getOriginalCustomCode() {
		return this.originalCustomCode;
	}

	/**
	 * Setter of <code>originalCustomCode</code> property.
	 * 
	 * @param originalCustomCode
	 *            The <code>originalCustomCode</code> to set.
	 * @see #originalCustomCode
	 */
	public void setOriginalCustomCode(String originalCustomCode) {
		this.originalCustomCode = originalCustomCode;
	}

	/**
	 * Hash code.
	 * 
	 * @return the int
	 * @inheritDoc
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result
				+ ((this.customCode == null) ? 0 : this.customCode.hashCode());
		return result;
	}

	/**
	 * Equals.
	 * 
	 * @param obj
	 *            the obj
	 * @return true, if successful
	 * @inheritDoc
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!super.equals(obj)) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		ContestGroupData other = (ContestGroupData) obj;
		if (this.customCode == null) {
			if (other.customCode != null) {
				return false;
			}
		} else if (!this.customCode.equals(other.customCode)) {
			return false;
		}
		return true;
	}

}
