/**
 * ElectionData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.demographics;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.ballot.BallotCardData;
import com.smartmatic.waffle.domain.ballot.BallotData;
import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.security.FileChecksumData;
import com.smartmatic.waffle.lang.annotations.basic.ValidatorListener;
import com.smartmatic.waffle.persistence.metadata.MultilingualPropertyAnnotation;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * Represents a formal decision-making process by which a population chooses an
 * individual or set of individuals to hold public office.
 */
@Entity
@Table(name = "election", uniqueConstraints = { @UniqueConstraint(columnNames = "election_code") })
@XStreamAlias("election")
@ValidatorListener(name = "com.smartmatic.waffle.validator.demographics.ElectionDataValidator")
public class ElectionData extends GeneralInfo {

	/** The Constant serialVersionUID. */
	@XStreamOmitField
	private static final long serialVersionUID = -6351060753064338269L;

	/** The publickey. */
	@XStreamOmitField
	private ImageWrapper publickey;

	/** The ballots. */
	@XStreamAlias("checksums")
	Set<FileChecksumData> checksums;

	/** The ballots. */
	@XStreamAlias("ballots")
	Set<BallotData> ballots;

	/** The languages. */
	@XStreamOmitField
	List<LanguageData> languages;

	/** The contest class. */
	@XStreamAlias("contest-classes")
	List<ContestClassData> contestClass;

	/** The contest groups. */
	@XStreamAlias("contest-groups")
	protected Set<ContestGroupData> contestGroups;

	/** The electoral organization. */
	@XStreamAlias("electoral-organization")
	@XStreamAsAttribute
	String electoralOrganization;

	/** The election name. */
	@XStreamAlias("name-key")
	@XStreamAsAttribute
	String electionName;

	/** The default languaje. */
	@XStreamOmitField
	private LanguageData defaultLanguaje;

	/** The status active. */
	@XStreamOmitField
	boolean statusActive;

	/** The voting time out. */
	@XStreamAlias("voting-time-out")
	@XStreamAsAttribute
	private int votingTimeOut;

	/** The is initialized. */
	@XStreamOmitField
	private boolean isInitialized = false;

	/** The party list. */
	@XStreamOmitField
	private Set<PartyData> partyList;

	/** The xmlns xs. */
	@XStreamAlias("xmlns:xs")
	@XStreamAsAttribute
	String xmlnsXs;

	/** The xmlns. */
	@XStreamAlias("xmlns")
	@XStreamAsAttribute
	String xmlns;

	/** The election opening date. */
	@XStreamOmitField
	private Date electionOpeningDate;

	/** The election closing date. */
	@XStreamOmitField
	private Date electionClosingDate;

	/** The keystore. */
	@XStreamOmitField
	private KeystoreData keystore;

	/** The applicationKey. */
	@XStreamOmitField
	private String applicationKey;

	/** Use Region Nis code As Polling Place CustomCode. */
	@XStreamOmitField
	private boolean useRegionNisAsPollingPlaceCode;

	/** root-ca.crt fingerprint. */
	@XStreamAlias("root-fp")
	@XStreamAsAttribute
	private String rootFp;

	/** CertToEncrypt p12 password. */
	@XStreamAlias("results-cert-protection-key")
	@XStreamAsAttribute
	private String certToEncryptPassword;

	/**
	 * Gets the public key.
	 * 
	 * @return the publickey
	 */
	@Column(name = "public_key", nullable = true)
	public ImageWrapper getPublickey() {
		return this.publickey;
	}

	/**
	 * Sets the public key.
	 * 
	 * @param publickey
	 *            the new publickey
	 */
	public void setPublickey(ImageWrapper publickey) {
		this.publickey = publickey;
	}

	/**
	 * Gets the election closing date.
	 * 
	 * @return the electionClosingDate
	 */
	@Column(name = "election_closing_date", nullable = false)
	public Date getElectionClosingDate() {
		return this.electionClosingDate;
	}

	/**
	 * Sets the election closing date.
	 * 
	 * @param electionClosingDate
	 *            the electionClosingDate to set
	 */
	public void setElectionClosingDate(Date electionClosingDate) {
		this.electionClosingDate = electionClosingDate;
	}

	/**
	 * Gets the election opening date.
	 * 
	 * @return the election opening date
	 */
	@Column(name = "election_opening_date", nullable = false)
	public Date getElectionOpeningDate() {
		return this.electionOpeningDate;
	}

	/**
	 * Setter of <code>electionDate</code> property.
	 * 
	 * @param electionDate
	 *            The <code>electionDate</code> to set
	 */
	public void setElectionOpeningDate(Date electionDate) {
		this.electionOpeningDate = electionDate;
	}

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "election_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_election")
	@SequenceGenerator(name = "s_election", sequenceName = "s_election", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "custom_code", nullable = false, unique = true, length = 80)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Gets the party list.
	 * 
	 * @return the party list
	 */
	@Transient
	public Set<PartyData> getPartyList() {
		return this.partyList;
	}

	/**
	 * Setter of <code>partyList</code> property.
	 * 
	 * @param partyList
	 *            The <code>partyList</code> to set
	 */
	public void setPartyList(Set<PartyData> partyList) {
		this.partyList = partyList;
	}

	/**
	 * Gets the xmlns xs.
	 * 
	 * @return the xmlnsXs
	 */
	@Transient
	public String getXmlnsXs() {
		return this.xmlnsXs;
	}

	/**
	 * Sets the xmlns xs.
	 * 
	 * @param xmlnsXs
	 *            the xmlnsXs to set
	 */
	public void setXmlnsXs(String xmlnsXs) {
		this.xmlnsXs = xmlnsXs;
	}

	/**
	 * Gets the xmlns.
	 * 
	 * @return the xmlns
	 */
	@Transient
	public String getXmlns() {
		return this.xmlns;
	}

	/**
	 * Sets the xmlns.
	 * 
	 * @param xmlns
	 *            the xmlns to set
	 */
	public void setXmlns(String xmlns) {
		this.xmlns = xmlns;
	}

	/**
	 * Gets the election name.
	 * 
	 * @return the electionName
	 */
	@Column(name = "election_name", length = 1120)
	@MultilingualPropertyAnnotation
	public String getElectionName() {
		return this.electionName;
	}

	/**
	 * Sets the election name.
	 * 
	 * @param electionName
	 *            the electionName to set
	 */
	public void setElectionName(String electionName) {
		this.electionName = electionName;
	}

	/**
	 * Getter of {@link #statusActive} field.
	 * 
	 * @return {@link #statusActive} value.
	 */
	@Column(name = "status_active")
	public boolean isStatusActive() {
		return this.statusActive;
	}

	/**
	 * Setter of {@link #statusActive} field.
	 * 
	 * @param statusActive
	 *            The boolean to set.
	 */
	public void setStatusActive(boolean statusActive) {
		this.statusActive = statusActive;
	}

	/**
	 * Getter of <code>ballots</code> property.
	 * 
	 * @return <code>ballots</code> property value.
	 * @see #ballots
	 */
	@Transient
	public Set<BallotData> getBallots() {
		return this.ballots;
	}

	/**
	 * Setter of <code>ballots</code> property.
	 * 
	 * @param ballots
	 *            The <code>ballots</code> to set.
	 * @see #ballots
	 */
	public void setBallots(Set<BallotData> ballots) {
		this.ballots = ballots;
	}

	/**
	 * Getter of <code>checksums</code> property.
	 * 
	 * @return <code>checksums</code> property value.
	 * @see #checksums
	 */
	@Transient
	public Set<FileChecksumData> getChecksums() {
		return this.checksums;
	}

	/**
	 * Setter of <code>checksums</code> property.
	 * 
	 * @param ballots
	 *            The <code>checksums</code> to set.
	 * @see #ballots
	 */
	public void setChecksums(Set<FileChecksumData> checksums) {
		this.checksums = checksums;
	}

	/**
	 * Getter of <code>votingTimeOut</code> property.
	 * 
	 * @return <code>votingTimeOut</code> property value.
	 * @see #votingTimeOut
	 */
	@Transient
	public int getVotingTimeOut() {
		return this.votingTimeOut;
	}

	/**
	 * Setter of <code>votingTimeOut</code> property.
	 * 
	 * @param votingTimeOut
	 *            the new voting time out
	 * @see #votingTimeOut
	 */
	public void setVotingTimeOut(int votingTimeOut) {
		this.votingTimeOut = votingTimeOut;
	}

	/**
	 * Getter of <code>languajes</code> property.
	 * 
	 * @return <code>languajes</code> property value.
	 * @see #languages
	 */
	@OneToMany(mappedBy = "election", targetEntity = LanguageData.class)
	public List<LanguageData> getLanguages() {
		return this.languages;
	}

	/**
	 * Setter of <code>languajes</code> property.
	 * 
	 * @param languages
	 *            the new languages
	 * @see #languages
	 */
	public void setLanguages(List<LanguageData> languages) {
		this.languages = languages;
	}

	/**
	 * Gets the keystore.
	 * 
	 * @return the keystore
	 */
	@ManyToOne(optional = true, fetch = FetchType.LAZY)
	@JoinColumn(name = "keystore_code", nullable = true, updatable = true)
	public KeystoreData getKeystore() {
		return this.keystore;
	}

	/**
	 * Sets the keystore.
	 * 
	 * @param keystore
	 *            the keystore to set
	 */
	public void setKeystore(KeystoreData keystore) {
		this.keystore = keystore;
	}

	/**
	 * Getter of <code>contestClass</code> property.
	 * 
	 * @return <code>contestClass</code> property value.
	 * @see #contestClass
	 */
	@Transient
	public List<ContestClassData> getContestClass() {
		return this.contestClass;
	}

	/**
	 * Setter of <code>contestClass</code> property.
	 * 
	 * @param contestClass
	 *            The <code>contestClass</code> to set.
	 * @see #contestClass
	 */
	public void setContestClass(List<ContestClassData> contestClass) {
		this.contestClass = contestClass;
	}

	/**
	 * Getter of <code>contestGroup</code> property.
	 * 
	 * @return <code>contestGroup</code> property value.
	 * @see #contestGroups
	 */
	@Transient
	public Set<ContestGroupData> getContestGroups() {
		return this.contestGroups;
	}

	/**
	 * Setter of <code>contestGroup</code> property.
	 * 
	 * @param contestGroup
	 *            The <code>contestGroup</code> to set.
	 * @see #contestGroups
	 */
	public void setContestGroups(Set<ContestGroupData> contestGroup) {
		this.contestGroups = contestGroup;
	}

	/**
	 * Getter of <code>electoralOrganization</code> property.
	 * 
	 * @return <code>electoralOrganization</code> property value.
	 * @see #electoralOrganization
	 */
	@Column(name = "electoral_organization", length = 1000)
	public String getElectoralOrganization() {
		return this.electoralOrganization;
	}

	/**
	 * Setter of <code>electoralOrganization</code> property.
	 * 
	 * @param electoralOrganization
	 *            The <code>electoralOrganization</code> to set.
	 * @see #electoralOrganization
	 */
	public void setElectoralOrganization(String electoralOrganization) {
		this.electoralOrganization = electoralOrganization;
	}

	/**
	 * Returns the contestData based on the code given.
	 * 
	 * @param code
	 *            the contest data code
	 * @return the matched contestData
	 */
	public ContestData getContestDataByCode(long code) {
		if (this.getBallots() != null) {
			for (BallotData ballot : this.getBallots()) {
				if (ballot != null && ballot.getBallotCards() != null) {
					for (BallotCardData ballotCard : ballot.getBallotCards()) {
						if (ballotCard != null
								&& ballotCard.getContest() != null) {
							for (ContestData contest : ballotCard.getContest()) {
								if (contest.getCode() == code) {
									return contest;
								}
							}
						}
					}
				}
			}
		}
		return null;
	}

	/**
	 * Return the contestClassData based on the code given.
	 * 
	 * @param code
	 *            the code of the constetClassData to get
	 * @return the constestClassData matched
	 */
	public ContestClassData getContestClassDataByCode(long code) {
		Iterator<ContestClassData> iter = this.getContestClass().iterator();
		while (iter.hasNext()) {
			ContestClassData contClass = iter.next();
			if (contClass.getCode() == code) {
				return contClass;
			}
		}
		return null;
	}

	/**
	 * Returns the contestGroupData based on the code given.
	 * 
	 * @param code
	 *            the code of the contestGroupData to get
	 * @return the contestGroupData matched
	 */
	public ContestGroupData getContestGroupDataByCode(long code) {
		Iterator<ContestGroupData> iter = this.getContestGroups().iterator();
		while (iter.hasNext()) {
			ContestGroupData contGroup = iter.next();
			if (contGroup.getCode() == code) {
				return contGroup;
			}

		}
		return null;
	}

	/**
	 * Getter of <code>defaultLanguaje</code> property.
	 * 
	 * @return <code>defaultLanguaje</code> property value.
	 * @see #defaultLanguaje
	 */
	@Transient
	public LanguageData getDefaultLanguaje() {
		return this.defaultLanguaje;
	}

	/**
	 * Getter of <code>isInitialized</code> property.
	 * 
	 * @return <code>isInitialized</code> property value.
	 * @see #isInitialized
	 */
	@Transient
	public boolean isInitialized() {
		return this.isInitialized;
	}

	/**
	 * Gets the application key.
	 *
	 * @return the application key
	 */
	@Column(name = "application_key")
	public String getApplicationKey() {
		return this.applicationKey;
	}

	/**
	 * Sets the application key.
	 *
	 * @param applicationKey
	 *            the new application key
	 */
	public void setApplicationKey(String applicationKey) {
		this.applicationKey = applicationKey;
	}

	/**
	 * Gets the value of useRegionNisAsPollingPlaceCode.
	 *
	 * @return useRegionNisAsPollingPlaceCode
	 */
	@Column(name = "use_region_nis_as_polling_place_code")
	public boolean isUseRegionNisAsPollingPlaceCode() {
		return this.useRegionNisAsPollingPlaceCode;
	}

	/**
	 * Sets the value of useRegionNisAsPollingPlaceCode.
	 *
	 * @param useRegionNisAsPollingPlaceCode
	 *            . the new useRegionNisAsPollingPlaceCode.
	 */
	public void setUseRegionNisAsPollingPlaceCode(
			boolean useRegionNisAsPollingPlaceCode) {
		this.useRegionNisAsPollingPlaceCode = useRegionNisAsPollingPlaceCode;
	}

	/**
	 * Gets the value of root certificate fingerprint.
	 *
	 * @return rootFp
	 */
	@Transient
	public String getRootFp() {
		return this.rootFp;
	}

	/**
	 * Sets the value of root certificate fingerprint.
	 *
	 * @param rootFp
	 *            root certificate fingerprint.
	 */
	public void setRootFp(String rootFp) {
		this.rootFp = rootFp;
	}

	/**
	 * Gets the value of cert to encrypt password
	 *
	 * @return certToEncryptPassword
	 */
	@Transient
	public String getCertToEncryptPassword() {
		return this.certToEncryptPassword;
	}

	/**
	 * Sets the value of cert to encrypt password
	 *
	 * @param certToEncryptPassword
	 *            cert to encrypt password
	 */
	public void setCertToEncryptPassword(String certToEncryptPassword) {
		this.certToEncryptPassword = certToEncryptPassword;
	}

}
