/**
 * LogicalVotingGroupData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.domain.device;

import java.util.List;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.general.BaseGeneralInfo;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * JavaBean representing a Logical Voting Group. A logical voting group is the
 * concept where any group of voters are grouped. Basically this concept allows
 * the configuration to split the voters by any criteria (Selection codes,
 * Voting operation modes among others).
 */
@XStreamAlias("logical-voter-group")
@Entity
@Table(name = "logical_voting_group")
public class LogicalVotingGroupData extends BaseGeneralInfo {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -7469280450463162645L;

	/** The contest group. */
	@XStreamOmitField
	private ContestGroupData contestGroup;

	/** The voting booth. */
	@XStreamOmitField
	private VotingBoothData votingBooth;

	/** The custom attribute. */
	@XStreamAlias("custom-attributes")
	private List<CustomAttribute> customAttribute;

	/** The contest group code. */
	@XStreamAlias("contest-group-code")
	@XStreamAsAttribute
	long contestGroupCode;

	/** The operation mode. */
	@XStreamOmitField
	OperationModeData operationMode;

	/** The operation mode code. */
	@XStreamAlias("operation-mode-code")
	@XStreamAsAttribute
	long operationModeCode;

	/** The voting booths. */
	@XStreamOmitField
	Set<VotingBoothData> votingBooths;

	/** The voting booth code. */
	@XStreamAlias("voting-booth-code")
	@XStreamAsAttribute
	long votingBoothCode;

	/** The region data code. */
	@XStreamAlias("region-code")
	@XStreamAsAttribute
	long regionDataCode;

	/** The name. */
	@XStreamAlias("name-key")
	@XStreamAsAttribute
	String name;

	/**
	 * Instantiates a new logical voting group data.
	 * 
	 * @param code
	 *            the code
	 */
	public LogicalVotingGroupData(Long code) {
		this.code = code;
	}

	/**
	 * Instantiates a new logical voting group data.
	 * 
	 * @param votingBooth
	 *            the voting booth
	 * @param contestGroup
	 *            the contest group
	 * @param operationMode
	 *            the operation mode
	 * @param name
	 *            the name
	 * @param customCode
	 *            the custom code
	 */
	public LogicalVotingGroupData(VotingBoothData votingBooth,
			ContestGroupData contestGroup, OperationModeData operationMode,
			String name, String customCode) {

		this.votingBooth = votingBooth;
		this.contestGroup = contestGroup;
		this.operationMode = operationMode;
		this.name = name;
		this.customCode = customCode;
		this.operationModeCode = operationMode.getCode();
	}

	/**
	 * Instantiates a new logical voting group data.
	 */
	public LogicalVotingGroupData() {
		super();
	}

	/**
	 * Getter of {@link #code} field.
	 * 
	 * @return {@link #code} value.
	 */
	@Override
	@Id
	@Column(name = "logical_vg_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_logical_vg")
	@SequenceGenerator(name = "s_logical_vg", sequenceName = "s_logical_vg", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Setter of {@link #code} field.
	 * 
	 * @param code
	 *            The long to set.
	 */
	@Override
	public void setCode(long code) {
		this.code = code;
	}

	/**
	 * Getter of {@link #customCode} field.
	 * 
	 * @return {@link #customCode} value.
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "logical_vg_custom_code", nullable = false, unique = true, length = 80)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Setter of {@link #customCode} field.
	 * 
	 * @param customCode
	 *            The String to set.
	 */
	@Override
	public void setCustomCode(String customCode) {
		this.customCode = customCode;

	}

	/**
	 * Getter of {@link #name} field.
	 * 
	 * @return {@link #name} value.
	 */
	@Column(name = "logical_vg_name", length = 255)
	public String getName() {
		return this.name;
	}

	/**
	 * Setter of {@link #name} field.
	 * 
	 * @param name
	 *            The String to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Getter of {@link #operationModeCode} field.
	 * 
	 * @return {@link #operationModeCode} value.
	 */
	@Column(name = "logical_vg_opm")
	public long getOperationModeCode() {
		return this.operationModeCode;
	}

	/**
	 * Setter of {@link #operationModeCode} field.
	 * 
	 * @param operationModeCode
	 *            The long to set.
	 */
	public void setOperationModeCode(long operationModeCode) {
		this.operationModeCode = operationModeCode;
	}

	/**
	 * Getter of {@link #contestGroupCode} field.
	 * 
	 * @return {@link #contestGroupCode} value.
	 */
	@Transient
	public long getContestGroupCode() {
		return this.contestGroupCode;
	}

	/**
	 * Setter of {@link #contestGroupCode} field.
	 * 
	 * @param contestGroupCode
	 *            The long to set.
	 */
	public void setContestGroupCode(long contestGroupCode) {
		this.contestGroupCode = contestGroupCode;
	}

	/**
	 * Getter of {@link #operationMode} field.
	 * 
	 * @return {@link #operationMode} value.
	 */
	@Transient
	public OperationModeData getOperationMode() {
		return this.operationMode;
	}

	/**
	 * Setter of {@link #operationMode} field.
	 * 
	 * @param operationMode
	 *            The OperationModeData to set.
	 */
	public void setOperationMode(OperationModeData operationMode) {
		this.operationMode = operationMode;
		this.operationModeCode = operationMode.getCode();
	}

	/**
	 * Getter of {@link #votingBoothCode} field.
	 * 
	 * @return {@link #votingBoothCode} value.
	 */
	@Transient
	public long getVotingBoothCode() {
		return this.votingBoothCode;
	}

	/**
	 * Setter of {@link #votingBoothCode} field.
	 * 
	 * @param votingBoothCode
	 *            The long to set.
	 */
	public void setVotingBoothCode(long votingBoothCode) {
		this.votingBoothCode = votingBoothCode;
	}

	/**
	 * Getter of {@link #regionDataCode} field.
	 * 
	 * @return {@link #regionDataCode} value.
	 */
	@Transient
	public long getRegionDataCode() {
		return this.regionDataCode;
	}

	/**
	 * Setter of {@link #regionDataCode} field.
	 * 
	 * @param regionDataCode
	 *            The long to set.
	 */
	public void setRegionDataCode(long regionDataCode) {
		this.regionDataCode = regionDataCode;
	}

	/**
	 * Gets the custom attribute.
	 * 
	 * @return the custom attribute
	 */
	@Transient
	public List<CustomAttribute> getCustomAttribute() {
		return this.customAttribute;
	}

	/**
	 * Sets the custom attribute.
	 * 
	 * @param customAttribute
	 *            the new custom attribute
	 */
	public void setCustomAttribute(List<CustomAttribute> customAttribute) {
		this.customAttribute = customAttribute;
	}

	/**
	 * Gets the voting booths.
	 * 
	 * @return the voting booths
	 */
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "logical_voting_group_voting_booth", joinColumns = { @JoinColumn(name = "logical_vg_code", referencedColumnName = "logical_vg_code") }, inverseJoinColumns = { @JoinColumn(name = "voting_booth_code", referencedColumnName = "voting_booth_code") })
	public Set<VotingBoothData> getVotingBooths() {
		return this.votingBooths;
	}

	/**
	 * Sets the voting booths.
	 * 
	 * @param votingBooths
	 *            the new voting booths
	 */
	public void setVotingBooths(Set<VotingBoothData> votingBooths) {
		this.votingBooths = votingBooths;
	}

	/**
	 * Getter of {@link #contestGroup} field.
	 * 
	 * @return {@link #contestGroup} value.
	 */
	@ManyToOne(optional = false, targetEntity = ContestGroupData.class)
	@JoinColumn(name = "contest_group_code", nullable = false, updatable = true)
	public ContestGroupData getContestGroup() {
		return this.contestGroup;
	}

	/**
	 * Setter of {@link #contestGroup} field.
	 * 
	 * @param contestGroup
	 *            The ContestGroupData to set.
	 */
	public void setContestGroup(ContestGroupData contestGroup) {
		this.contestGroup = contestGroup;
	}

	/**
	 * Getter of {@link #votingBooth} field.
	 * 
	 * @return {@link #votingBooth} value.
	 */
	@Transient
	public VotingBoothData getVotingBooth() {
		return this.votingBooth;
	}

	/**
	 * Setter of {@link #votingBooth} field.
	 * 
	 * @param votingBooth
	 *            The VotingBoothData to set.
	 */
	public void setVotingBooth(VotingBoothData votingBooth) {
		this.votingBooth = votingBooth;
	}
}
