/**
 * VotingCentersData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.device;

import java.io.Serializable;
import java.util.List;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * Class that represents a voting device index.
 */
@XStreamAlias("VotingCenters")
public class VotingCentersData implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -123117511409150037L;

	/** The xmlns xs. */
	@XStreamAlias("xmlns:xs")
	@XStreamAsAttribute
	String xmlnsXs;

	/** The xmlns. */
	@XStreamAlias("xmlns")
	@XStreamAsAttribute
	String xmlns;

	/** The voting centers. */
	@XStreamImplicit(itemFieldName = "VotingCenter")
	List<VotingCenter> votingCenters;

	/**
	 * Creates a new VotingCentersData.
	 * 
	 * @param xmlnsXs
	 *            the xmlns xs
	 * @param xmlns
	 *            the xmlns
	 * @param votingCenters
	 *            the voting centers
	 */
	public VotingCentersData(String xmlnsXs, String xmlns,
			List<VotingCenter> votingCenters) {
		super();
		this.xmlnsXs = xmlnsXs;
		this.xmlns = xmlns;
		this.votingCenters = votingCenters;
	}

	/**
	 * Getter of {@link #votingCenters} field.
	 * 
	 * @return {@link #votingCenters} value.
	 */
	public List<VotingCenter> getVotingCenters() {
		return this.votingCenters;
	}

	/**
	 * Setter of {@link #votingCenters} field.
	 * 
	 * @param votingCenters
	 *            The List<VotingCenter> to set.
	 */
	public void setVotingCenters(List<VotingCenter> votingCenters) {
		this.votingCenters = votingCenters;
	}

	/**
	 * Gets the xmlns xs.
	 * 
	 * @return the xmlnsXs
	 */
	public String getXmlnsXs() {
		return this.xmlnsXs;
	}

	/**
	 * Sets the xmlns xs.
	 * 
	 * @param xmlnsXs
	 *            the xmlnsXs to set
	 */
	public void setXmlnsXs(String xmlnsXs) {
		this.xmlnsXs = xmlnsXs;
	}

	/**
	 * Gets the xmlns.
	 * 
	 * @return the xmlns
	 */
	public String getXmlns() {
		return this.xmlns;
	}

	/**
	 * Sets the xmlns.
	 * 
	 * @param xmlns
	 *            the xmlns to set
	 */
	public void setXmlns(String xmlns) {
		this.xmlns = xmlns;
	}

	/**
	 * The Class VotingCenter.
	 */
	@XStreamAlias("VotingCenter")
	public static class VotingCenter implements Serializable {

		/** The Constant serialVersionUID. */
		private static final long serialVersionUID = 2872380056542758723L;

		/** The code. */
		@XStreamAlias("code")
		@XStreamAsAttribute
		String code;

		/** The programming unit. */
		@XStreamAlias("rnd")
		@XStreamAsAttribute
		String randomSalt;

		/** The folder name. */
		@XStreamAlias("folder-name")
		@XStreamAsAttribute
		String folderName;

		/** The image file name. */
		@XStreamAlias("image-file-name")
		@XStreamAsAttribute
		String imageFileName;

		/** The programming unit. */
		@XStreamAlias("program-unit")
		@XStreamAsAttribute
		String programmingUnit;

		@XStreamOmitField
		VotingDevicesData votingDevices;

		/**
		 * Creates a new VotingCenter.
		 * 
		 * @param code
		 *            the code
		 * @param randomSalt
		 *            the random salt
		 * @param folderName
		 *            the folder name
		 * @param programmingUnit
		 *            the programming unit
		 */
		public VotingCenter(String code, String randomSalt, String folderName,
				String programmingUnit) {
			super();
			this.code = code;
			this.randomSalt = randomSalt;
			this.folderName = folderName;
			this.imageFileName = "";
			this.programmingUnit = programmingUnit;
		}

		/**
		 * Getter of {@link #code} field.
		 * 
		 * @return {@link #code} value.
		 */
		public String getCode() {
			return this.code;
		}

		/**
		 * Setter of {@link #code} field.
		 * 
		 * @param code
		 *            The String to set.
		 */
		public void setCode(String code) {
			this.code = code;
		}

		/**
		 * Gets the random salt.
		 *
		 * @return the random salt
		 */
		public String getRandomSalt() {
			return this.randomSalt;
		}

		/**
		 * Sets the random salt.
		 *
		 * @param randomSalt
		 *            the new random salt
		 */
		public void setRandomSalt(String randomSalt) {
			this.randomSalt = randomSalt;
		}

		/**
		 * Getter of {@link #folderName} field.
		 * 
		 * @return {@link #folderName} value.
		 */
		public String getFolderName() {
			return this.folderName;
		}

		/**
		 * Setter of {@link #folderName} field.
		 * 
		 * @param folderName
		 *            The String to set.
		 */
		public void setFolderName(String folderName) {
			this.folderName = folderName;
		}

		/**
		 * Getter of {@link #imageFileName} field.
		 * 
		 * @return {@link #imageFileName} value.
		 */
		public String getImageFileName() {
			return this.imageFileName;
		}

		/**
		 * Setter of {@link #imageFileName} field.
		 * 
		 * @param imageFileName
		 *            The String to set.
		 */
		public void setImageFileName(String imageFileName) {
			this.imageFileName = imageFileName;
		}

		/**
		 * Getter of {@link #programmingUnit} field.
		 * 
		 * @return {@link #programmingUnit} value.
		 */
		public String getProgrammingUnit() {
			return this.programmingUnit;
		}

		/**
		 * Setter of {@link #programmingUnit} field.
		 * 
		 * @param programmingUnit
		 *            the new programming unit
		 */
		public void setProgrammingUnit(String programmingUnit) {
			this.programmingUnit = programmingUnit;
		}

		/**
		 * Getter of {@link #votingDevices} field
		 * 
		 * @return {@link #votingDevices} value
		 */
		public VotingDevicesData getVotingDevices() {
			return this.votingDevices;
		}

		/**
		 * Setter of {@link #votingDevices} field
		 * 
		 * @param votingDevices
		 *            the new {@link #votingDevices} value
		 */
		public void setVotingDevices(VotingDevicesData votingDevices) {
			this.votingDevices = votingDevices;
		}

		@Override
		public int hashCode() {
			return this.code.hashCode() * this.folderName.hashCode()
					* this.programmingUnit.hashCode();
		}

		@Override
		public boolean equals(Object o) {
			if (o == null) {
				return false;
			}
			if (!(o instanceof VotingCenter)) {
				return false;
			}

			VotingCenter other = (VotingCenter) o;
			return this.code == other.getCode()
					&& this.folderName == other.getFolderName();
		}
	}
}
