/**
 * VotingDeviceData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.device;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;
import javax.persistence.Version;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.user.UserData;
import com.smartmatic.waffle.lang.annotations.basic.ValidatorListener;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamConverter;
import com.thoughtworks.xstream.annotations.XStreamOmitField;
import com.thoughtworks.xstream.converters.enums.EnumConverter;

/**
 * Class that represents a physical location where the voters are required to go
 * in order to vote.
 */
@Entity
@Table(name = "voting_device", uniqueConstraints = { @UniqueConstraint(columnNames = "voting_device_custom_code") })
@ValidatorListener(name = "com.smartmatic.waffle.validator.device.VotingDeviceDataValidatorForMultilangualKey")
@XStreamAlias("voting-device")
public class VotingDeviceData extends GeneralInfo {
	/**
	 * The serial version.
	 */
	@XStreamOmitField
	private static final long serialVersionUID = 2647256383401745047L;

	/** The user related to the voting device. */
	@XStreamOmitField
	public UserData lockOwner;

	/** The keys. */
	@XStreamAlias("keys")
	KeysData keys;

	/** The operation modes. */
	@XStreamAlias("operation-modes")
	Set<OperationModeData> operationModes;

	/** The Voting Booth related to the voting device. */
	@XStreamOmitField
	private VotingBoothData votingBooth;

	/** Programming counter. */
	@XStreamOmitField
	private int programmingCounter;

	/**
	 * Field that contains a SymmetricKey that the operator needs to to
	 * introduce to unblock it.
	 */
	@XStreamOmitField
	private String transmissionKey;

	/**
	 * Field that contains an alphanumeric Key gave to the president of table.
	 */
	@XStreamOmitField
	private String presidentKey;

	/**
	 * Field that contains an alphanumeric Key gave to the president of table.
	 */
	@XStreamOmitField
	private String recounterKey;

	/**
	 * Field that contains a numeric Key gave to the president of table.
	 */

	@XStreamOmitField
	private String numericKey1;

	/**
	 * Field that contains a numeric Key gave to the president of table.
	 */

	@XStreamOmitField
	private String numericKey2;

	/**
	 * Field that contains the asymmetric keys that each voting machine uses to
	 * encrypt/sign its information. @author Ithamar Torres
	 * (itorres@smartmatic.com)
	 */

	@XStreamOmitField
	private ImageWrapper asymmetricKeys;

	/**
	 * Field whose value is the certificate password. This certificate is needed
	 * to authenticate from client side.
	 */

	@XStreamOmitField
	private String protectionKey;

	/** The name. */
	@XStreamOmitField
	private String name;

	/** The voting booths. */
	@XStreamAlias("voting-booths")
	Set<VotingBoothData> votingBooths;

	/** The logical voter groups data. */
	@XStreamAlias("logical-voter-groups")
	LogicalVoterGroupsData logicalVoterGroupsData;

	/** The type. */
	@XStreamAsAttribute
	Type type;

	/**
	 * The Enum Type.
	 */
	@XStreamAlias("type")
	@XStreamConverter(EnumConverter.class)
	public enum Type {

		/** The voting machine. */
		VOTING_MACHINE,
		/** The omr. */
		OMR
	}

	/** The languages. */
	@XStreamOmitField
	Set<LanguageData> languages;

	/**
	 * Getter of {@link #languages} field.
	 * 
	 * @return {@link #languages} value.
	 */
	@Transient
	public Set<LanguageData> getLanguages() {
		return this.languages;
	}

	/**
	 * Setter of {@link #languages} field.
	 * 
	 * @param languages
	 *            The Set<LanguageData> to set.
	 */
	public void setLanguages(Set<LanguageData> languages) {
		this.languages = languages;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getVersion()
	 */
	@Override
	@Transient
	@Version
	public int getVersion() {
		return this.version;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getCode()
	 */
	@Override
	@Id
	@Column(name = "voting_device_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_voting_device")
	@SequenceGenerator(name = "s_voting_device", sequenceName = "s_voting_device", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#setCode(long)
	 */
	@Override
	public void setCode(long code) {
		this.code = code;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getCustomCode()
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "voting_device_custom_code", nullable = false)
	public String getCustomCode() {
		return this.customCode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.GeneralInfo#setName(java.lang.
	 * String)
	 */
	@Override
	public void setName(String name) {
		this.name = name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.GeneralInfo#getName()
	 */
	@Override
	@Column(name = "voting_device_name", nullable = false)
	public String getName() {
		return this.name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#setCustomCode(
	 * java.lang.String)
	 */
	@Override
	public void setCustomCode(String customCode) {
		this.customCode = customCode;
	}

	/**
	 * Gets the user related to the voting device.
	 * 
	 * @return the user related to the voting device
	 */
	@ManyToOne
	@JoinColumn(name = "lock_owner", nullable = true, updatable = true)
	public UserData getLockOwner() {
		return this.lockOwner;
	}

	/**
	 * Setter of <code>lockOwner</code> property.
	 * 
	 * @param lockOwner
	 *            The <code>lockOwner</code> to set
	 */
	public void setLockOwner(UserData lockOwner) {
		this.lockOwner = lockOwner;
	}

	/**
	 * Gets the programming counter.
	 * 
	 * @return the programming counter
	 */
	@Column(name = "programming_counter", nullable = false)
	public int getProgrammingCounter() {
		return this.programmingCounter;
	}

	/**
	 * Setter of <code>programmingCounter</code> property.
	 * 
	 * @param programmingCounter
	 *            The <code>programmingCounter</code> to set
	 */
	public void setProgrammingCounter(int programmingCounter) {
		this.programmingCounter = programmingCounter;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.domain.BaseDataObject#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object anObject) {
		if (anObject instanceof VotingDeviceData) {
			VotingDeviceData otherDevice = (VotingDeviceData) anObject;
			return otherDevice.code == this.code;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.domain.BaseDataObject#hashCode()
	 */
	@Override
	public int hashCode() {
		return (int) this.code;
	}

	/**
	 * Gets the field that contains the asymmetric keys that each voting machine
	 * uses to encrypt/sign its information.
	 * 
	 * @return the field that contains the asymmetric keys that each voting
	 *         machine uses to encrypt/sign its information
	 */
	@Column(name = "asymmetric_key")
	public ImageWrapper getAsymmetricKeys() {
		return this.asymmetricKeys;
	}

	/**
	 * Setter of <code>asymmetricKeys</code> property.
	 * 
	 * @param asymmetricKeys
	 *            The <code>asymmetricKeys</code> to set
	 */
	public void setAsymmetricKeys(ImageWrapper asymmetricKeys) {
		this.asymmetricKeys = asymmetricKeys;
	}

	/**
	 * Gets the field whose value is the certificate password.
	 * 
	 * @return the field whose value is the certificate password
	 */
	@Column(name = "protection_key")
	public String getProtectionKey() {
		return this.protectionKey;
	}

	/**
	 * Setter of <code>protectionKey</code> property.
	 * 
	 * @param protectionKey
	 *            The <code>protectionKey</code> to set
	 */
	public void setProtectionKey(String protectionKey) {
		this.protectionKey = protectionKey;
	}

	/**
	 * Gets the field that contains a SymmetricKey that the operator needs to to
	 * introduce to unblock it.
	 * 
	 * @return the field that contains a SymmetricKey that the operator needs to
	 *         to introduce to unblock it
	 */
	@Column(name = "transmission_key")
	public String getTransmissionKey() {
		return this.transmissionKey;
	}

	/**
	 * Setter of <code>transmissionKey</code> property.
	 * 
	 * @param transmissionKey
	 *            The <code>transmissionKey</code> to set
	 */
	public void setTransmissionKey(String transmissionKey) {
		this.transmissionKey = transmissionKey;
	}

	/**
	 * Gets the field that contains an alphanumeric Key gave to the president of
	 * table.
	 * 
	 * @return the field that contains an alphanumeric Key gave to the president
	 *         of table
	 */
	@Column(name = "president_key")
	public String getPresidentKey() {
		return this.presidentKey;
	}

	/**
	 * Setter of <code>presidentKey</code> property.
	 * 
	 * @param presidentKey
	 *            The <code>presidentKey</code> to set
	 */
	public void setPresidentKey(String presidentKey) {
		this.presidentKey = presidentKey;
	}

	/**
	 * Gets the field that contains an alphanumeric Key gave to the president of
	 * table.
	 * 
	 * @return the field that contains an alphanumeric Key gave to the president
	 *         of table
	 */
	@Column(name = "recounter_key")
	public String getRecounterKey() {
		return this.recounterKey;
	}

	/**
	 * Sets the field that contains an alphanumeric Key gave to the president of
	 * table.
	 * 
	 * @param recounterKey
	 *            the new field that contains an alphanumeric Key gave to the
	 *            president of table
	 */
	public void setRecounterKey(String recounterKey) {
		this.recounterKey = recounterKey;
	}

	/**
	 * Gets the field that contains a numeric Key gave to the president of
	 * table.
	 * 
	 * @return the field that contains a numeric Key gave to the president of
	 *         table
	 */
	@Column(name = "numeric_key1")
	public String getNumericKey1() {
		return this.numericKey1;
	}

	/**
	 * Setter of <code>numericKey1</code> property.
	 * 
	 * @param numericKey
	 *            The <code>numericKey1</code> to set
	 */
	public void setNumericKey1(String numericKey) {
		this.numericKey1 = numericKey;
	}

	/**
	 * Gets the field that contains a numeric Key gave to the president of
	 * table.
	 * 
	 * @return the field that contains a numeric Key gave to the president of
	 *         table
	 */
	@Column(name = "numeric_key2")
	public String getNumericKey2() {
		return this.numericKey2;
	}

	/**
	 * Setter of <code>numericKey2</code> property.
	 * 
	 * @param numericKey
	 *            the new field that contains a numeric Key gave to the
	 *            president of table
	 */
	public void setNumericKey2(String numericKey) {
		this.numericKey2 = numericKey;
	}

	/**
	 * Getter of <code>votingBooths</code> property.
	 * 
	 * @return <code>votingBooths</code> property value.
	 * @see #votingBooths
	 */
	@Transient
	public Set<VotingBoothData> getVotingBooths() {
		return this.votingBooths;
	}

	/**
	 * Setter of <code>votingBooths</code> property.
	 * 
	 * @param votingBooths
	 *            The <code>votingBooths</code> to set.
	 * @see #votingBooths
	 */
	public void setVotingBooths(Set<VotingBoothData> votingBooths) {
		this.votingBooths = votingBooths;
	}

	/**
	 * Getter of <code>operationModes</code> property.
	 * 
	 * @return <code>operationModes</code> property value.
	 * @see #operationModes
	 */
	@Transient
	public Set<OperationModeData> getOperationModes() {
		return this.operationModes;
	}

	/**
	 * Setter of <code>operationModes</code> property.
	 * 
	 * @param operationModes
	 *            The <code>operationModes</code> to set.
	 * @see #operationModes
	 */
	public void setOperationModes(Set<OperationModeData> operationModes) {
		this.operationModes = operationModes;
	}

	/**
	 * Getter of <code>type</code> property.
	 * 
	 * @return <code>type</code> property value.
	 * @see #type
	 */
	@Transient
	public Type getType() {
		return this.type;
	}

	/**
	 * Setter of <code>type</code> property.
	 * 
	 * @param type
	 *            The <code>type</code> to set.
	 * @see #type
	 */
	public void setType(Type type) {
		this.type = type;
	}

	/**
	 * Getter of <code>votingBooth</code> property.
	 * 
	 * @return <code>votingBooth</code> property value.
	 * @see #votingBooth
	 */
	@OneToOne(targetEntity = VotingBoothData.class, mappedBy = "votingDevice")
	public VotingBoothData getVotingBooth() {
		return this.votingBooth;
	}

	/**
	 * Setter of <code>votingBooth</code> property.
	 * 
	 * @param votingBooth
	 *            The <code>votingBooth</code> to set.
	 * @see #votingBooth
	 */
	public void setVotingBooth(VotingBoothData votingBooth) {
		this.votingBooth = votingBooth;
	}

	/**
	 * Gets the keys.
	 * 
	 * @return the keys
	 */
	@Transient
	public KeysData getKeys() {
		return this.keys;
	}

	/**
	 * Setter of <code>keys</code> property.
	 * 
	 * @param keys
	 *            The <code>keys</code> to set
	 */
	public void setKeys(KeysData keys) {
		this.keys = keys;
	}

	/**
	 * Getter of {@link #logicalVoterGroupsData} field.
	 * 
	 * @return {@link #logicalVoterGroupsData} value.
	 */
	@Transient
	public LogicalVoterGroupsData getLogicalVoterGroupsData() {
		return this.logicalVoterGroupsData;
	}

	/**
	 * Setter of {@link #logicalVoterGroupsData} field.
	 * 
	 * @param logicalVoterGroupsData
	 *            the new logical voter groups data
	 */
	public void setLogicalVoterGroupsData(
			LogicalVoterGroupsData logicalVoterGroupsData) {
		this.logicalVoterGroupsData = logicalVoterGroupsData;
	}

	/**
	 * The Class keysData.
	 */
	@XStreamAlias("keys")
	public static class KeysData implements Serializable {

		/** The Constant serialVersionUID. */
		private static final long serialVersionUID = 1L;

		/** The protection key. */
		@XStreamAlias("protection-key")
		@XStreamAsAttribute
		private String protectionKey;

		/** The transmission key. */
		@XStreamAlias("transmission-key")
		@XStreamAsAttribute
		private String transmissionKey;

		/** The backup zip file password. */
		@XStreamAlias("backup-zip-file-password")
		@XStreamAsAttribute
		private String backupZipFilePassword;

		/**
		 * Creates a new keysData instance.
		 * 
		 * @param protectionKey
		 *            the protection key
		 * @param transmissionKey
		 *            the transmission key
		 * @param backupZipFilePassword
		 *            the backup zip file password
		 */
		public KeysData(String protectionKey, String transmissionKey,
				String backupZipFilePassword) {
			this.protectionKey = protectionKey;
			this.transmissionKey = transmissionKey;
			this.backupZipFilePassword = backupZipFilePassword;
		}

		/**
		 * Getter of <code>protectionKey</code> property.
		 * 
		 * @return <code>protectionKey</code> property value
		 */
		public String getProtectionKey() {
			return this.protectionKey;
		}

		/**
		 * Setter of <code>protectionKey</code> property.
		 * 
		 * @param protectionKey
		 *            <code>protectionKey</code> to set
		 */
		public void setProtectionKey(String protectionKey) {
			this.protectionKey = protectionKey;
		}

		/**
		 * Getter of <code>transmissionKey</code> property.
		 * 
		 * @return The <code>transmissionKey</code> property value
		 */
		public String getTransmissionKey() {
			return this.transmissionKey;
		}

		/**
		 * Setter of <code>transmissionKey</code> property.
		 * 
		 * @param transmissionKey
		 *            <code>transmissionKey</code> to set
		 */
		public void setTransmissionKey(String transmissionKey) {
			this.transmissionKey = transmissionKey;
		}

		/**
		 * Getter of <code>backupZipFilePassword</code> property.
		 * 
		 * @return <code>backupZipFilePassword</code> property value
		 */
		public String getBackupZipFilePassword() {
			return this.backupZipFilePassword;
		}

		/**
		 * Setter of <code>backupZipFilePassword</code> property.
		 * 
		 * @param backupZipFilePassword
		 *            <code>backupZipFilePassword</code> to set
		 */
		public void setBackupZipFilePassword(String backupZipFilePassword) {
			this.backupZipFilePassword = backupZipFilePassword;
		}

	}
}
