/**
 * CandidateData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.domain.eligibleoption;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.smartmatic.waffle.lang.annotations.basic.ValidatorListener;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * Class that represents the prospective recipient of a nomination or a person
 * seeking or being considered for some kind of position.
 */
@XStreamAlias("candidate")
@Entity
@Table(name = "candidate", uniqueConstraints = { @UniqueConstraint(columnNames = "candidate_code") })
@ValidatorListener(name = "com.smartmatic.waffle.validator.eligibleoption.CandidateDataValidator")
public class CandidateData extends GeneralInfo implements Comparable {

	/** The Constant serialVersionUID. */
	@XStreamOmitField
	private static final long serialVersionUID = 844105642709870388L;

	/** The identification. */
	@XStreamOmitField
	private String identification;

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "candidate_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_candidate")
	@SequenceGenerator(name = "s_candidate", sequenceName = "s_candidate", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Setter of <code>code</code> property.
	 * 
	 * @param code
	 *            The <code>code</code> to set.
	 * @see #code
	 */
	@Override
	public void setCode(long code) {
		this.code = code;
	}

	/**
	 * Getter of <code>identification</code> property.
	 * 
	 * @return <code>identification</code> property value.
	 * @see #identification
	 */
	@Column(name = "identification", nullable = true, length = 80)
	public String getIdentification() {
		return this.identification;
	}

	/**
	 * Setter of <code>identification</code> property.
	 * 
	 * @param identification
	 *            The <code>identification</code> to set.
	 * @see #identification
	 */
	public void setIdentification(String identification) {
		this.identification = identification;
	}

	/**
	 * Getter of <code>name</code> property.
	 * 
	 * @return <code>name</code> property value.
	 * @see #name
	 */
	@Override
	// @MultilingualPropertyAnnotation
	@Column(name = "name", length = 100)
	public String getName() {
		return this.name;
	}

	/**
	 * Setter of <code>name</code> property.
	 * 
	 * @param name
	 *            The <code>name</code> to set.
	 * @see #name
	 */
	@Override
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "custom_code", nullable = false, unique = true, length = 80)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Setter of <code>customCode</code> property.
	 * 
	 * @param customCode
	 *            The <code>customCode</code> to set.
	 * @see #customCode
	 */
	@Override
	public void setCustomCode(String customCode) {
		this.customCode = customCode;
	}

	/**
	 * Getter of <code>calculatedName</code> property.
	 * 
	 * @return <code>calculatedName</code> property value.
	 * @see #calculatedName
	 */
	@Transient
	public String getCalculatedName() {
		return "( " + this.customCode + " ) " + this.getName();
	}

	/**
	 * To string.
	 * 
	 * @return the string
	 * @inheritDoc
	 */
	@Override
	public String toString() {
		return this.name;
	}

	/**
	 * Equals.
	 * 
	 * @param o
	 *            the o
	 * @return true, if successful
	 * @inheritDoc
	 */
	@Override
	public boolean equals(Object o) {
		if (o instanceof CandidateData) {
			CandidateData a = (CandidateData) o;
			return new EqualsBuilder().append(this.code, a.code).isEquals();
		}
		return false;
	}

	/**
	 * Hash code.
	 * 
	 * @return the int
	 * @inheritDoc
	 */
	@Override
	public int hashCode() {
		return new HashCodeBuilder().append(this.code).toHashCode();
	}

	/**
	 * Gets the version.
	 * 
	 * @return the version
	 * @inheritDoc
	 */
	@Override
	@Transient
	public int getVersion() {
		return this.version;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(Object arg0) {
		return 0;

	}
}
