/**
 * LanguageCollectionData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.i18n;

import java.io.Serializable;
import java.util.List;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * Class that represents a supported language for ECM application.
 */
@XStreamAlias("languages")
public class LanguageCollectionData implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 3946182202727723062L;

	/** The xmlns xs. */
	@XStreamAlias("xmlns:xs")
	@XStreamAsAttribute
	String xmlnsXs;

	/** The xmlns. */
	@XStreamAlias("xmlns")
	@XStreamAsAttribute
	String xmlns;

	/** The signature. */
	@XStreamAlias("Signature")
	Object signature;

	/**
	 * Instantiates a new language collection data.
	 * 
	 * @param xmlnsXs
	 *            the xmlns xs
	 * @param xmlns
	 *            the xmlns
	 * @param languages
	 *            the languages
	 */
	public LanguageCollectionData(String xmlnsXs, String xmlns,
			List<LanguageData> languages) {
		this.xmlnsXs = xmlnsXs;
		this.xmlns = xmlns;
		this.languages = languages;
	}

	/**
	 * Gets the xmlns xs.
	 * 
	 * @return the xmlnsXs
	 */
	public String getXmlnsXs() {
		return this.xmlnsXs;
	}

	/**
	 * Sets the xmlns xs.
	 * 
	 * @param xmlnsXs
	 *            the xmlnsXs to set
	 */
	public void setXmlnsXs(String xmlnsXs) {
		this.xmlnsXs = xmlnsXs;
	}

	/**
	 * Gets the xmlns.
	 * 
	 * @return the xmlns
	 */
	public String getXmlns() {
		return this.xmlns;
	}

	/**
	 * Sets the xmlns.
	 * 
	 * @param xmlns
	 *            the xmlns to set
	 */
	public void setXmlns(String xmlns) {
		this.xmlns = xmlns;
	}

	/** Set of languages. */
	@XStreamImplicit
	private List<LanguageData> languages;

	/**
	 * Gets the set of languages.
	 * 
	 * @return the languages
	 */
	public List<LanguageData> getLanguages() {
		return this.languages;
	}

	/**
	 * Sets the set of languages.
	 * 
	 * @param languages
	 *            the languages to set
	 */
	public void setLanguages(List<LanguageData> languages) {
		this.languages = languages;
	}
}
