/**
 * MultiLingualKeyData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.multiLanguage;

import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.general.GeneralInfo;

/**
 * Class that represents a translatable entry in the system. Every time the user
 * inserts an entity that is translatable the system automatically creates a
 * multilingual key to mark the object as a translatable entry an those entries
 * are used by the translation module.
 */
@Entity
@Table(name = "multilingual_key", uniqueConstraints = { @UniqueConstraint(columnNames = "multilingual_custom_code") })
public class MultiLingualKeyData extends GeneralInfo {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4828335712949687727L;

	private String customCodeRef;

	/**
	 * Key of the object which originated this multilingual key. Because a
	 * multilingual entry is created from the creation of another entity this
	 * field references the refered object primary key
	 */

	private long foreignKey;

	/**
	 * Field linked to this multilingual key. A Given object can generate
	 * differents multilingual keys and that's why is required to maintain which
	 * filed corresponds to the multilingual key instance.
	 */

	private String field;

	/**
	 * Key of the multilingual key Through this information the system can
	 * obtain which is the values of the information. This is an alternative key
	 * to the multilingual key object and is the key used to reference a
	 * multilingual key through the system.
	 */

	private String key;

	/**
	 * Default value of the translatable element. Usually is the translatable
	 * element in the default language. Because the system is always in a
	 * default language the translations from that language aren't required and
	 * are assumed by the system once the user creates the entities in the
	 * application. This field holds the value of the translatable element in
	 * the application default language.
	 */

	private String valueOfKey = null;

	/**
	 * List of translations of the multilingual key. Usually it will hold a list
	 * of translations for every supported language in the election.
	 */

	private Set<TranslationKeyData> translationKeys;

	@Transient
	public String getCustomCodeRef() {
		return this.customCodeRef;
	}

	public void setCustomCodeRef(String customCodeRef) {
		this.customCodeRef = customCodeRef;
	}

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "multilingual_key_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_multilingual_key")
	@SequenceGenerator(name = "s_multilingual_key", sequenceName = "s_multilingual_key", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Setter of <code>code</code> property.
	 * 
	 * @param code
	 *            The <code>code</code> to set.
	 * @see #code
	 */
	@Override
	public void setCode(long code) {
		this.code = code;
	}

	/**
	 * Getter of <code>field</code> property.
	 * 
	 * @return <code>field</code> property value.
	 * @see #field
	 */
	@Column(name = "field")
	public String getField() {
		return this.field;
	}

	/**
	 * Setter of <code>field</code> property.
	 * 
	 * @param field
	 *            The <code>field</code> to set.
	 * @see #field
	 */

	public void setField(String field) {
		this.field = field;
	}

	/**
	 * Getter of <code>foreignKey</code> property.
	 * 
	 * @return <code>foreignKey</code> property value.
	 * @see #foreignKey
	 */
	@Column(name = "foreign_code")
	public long getForeignKey() {
		return this.foreignKey;
	}

	/**
	 * Setter of <code>foreignKey</code> property.
	 * 
	 * @param foreignKey
	 *            The <code>foreignKey</code> to set.
	 * @see #foreignKey
	 */

	public void setForeignKey(long foreignKey) {
		this.foreignKey = foreignKey;
	}

	/**
	 * Getter of <code>key</code> property.
	 * 
	 * @return <code>key</code> property value.
	 * @see #key
	 */
	@Column(name = "object_class")
	public String getKey() {
		return this.key;
	}

	/**
	 * Setter of <code>key</code> property.
	 * 
	 * @param key
	 *            The <code>key</code> to set.
	 * @see #key
	 */

	public void setKey(String key) {
		this.key = key;
	}

	/**
	 * Overrrides equal method with specific semantics to the MultilingualKey
	 * object
	 * 
	 * @param The
	 *            object to compare
	 * @return true if is the same object or false otherwise
	 */

	@Override
	public boolean equals(Object o) {
		if (o instanceof MultiLingualKeyData) {
			MultiLingualKeyData a = (MultiLingualKeyData) o;
			return new EqualsBuilder().append(this.code, a.code).isEquals();
		}
		return false;
	}

	/**
	 * Overrrides the hashCode method with specific semantics to the
	 * MultilingualKey object
	 * 
	 * @return a hashcode value from this object instance
	 */
	@Override
	public int hashCode() {
		return new HashCodeBuilder().append(this.code).toHashCode();
	}

	/**
	 * Getter of <code>valueOfKey</code> property.
	 * 
	 * @return <code>valueOfKey</code> property value.
	 * @see #valueOfKey
	 */
	@Transient
	public String getValueOfKey() {

		return this.valueOfKey;
	}

	/**
	 * Setter of <code>valueOfKey</code> property.
	 * 
	 * @param theValueOfKey
	 *            The <code>valueOfKey</code> to set.
	 * @see #valueOfKey
	 */
	public void setValueOfKey(String theValueOfKey) {
		this.valueOfKey = theValueOfKey;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "multilingual_custom_code", nullable = true, unique = true)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Setter of <code>customCode</code> property.
	 * 
	 * @param theCustomCode
	 *            The <code>customCode</code> to set.
	 * @see #customCode
	 */
	@Override
	public void setCustomCode(String theCustomCode) {
		this.customCode = theCustomCode;
		return;
	}

}
