/**
 * FunctionalityDefinitionEnum.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.security;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamConverter;
import com.thoughtworks.xstream.converters.enums.EnumConverter;

/**
 * Enum with all the supported FunctionalityDefinitions supported in the system.
 * Each FunctionalityDefinition should specify in witch application is
 * supported.
 */

@XStreamConverter(EnumConverter.class)
@XStreamAlias("func-definition")
public enum FunctionalityDefinitionEnum {

	/** The key generation enum. */
	KEY_GENERATION(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelKeyGeneration"),

	/** The images upload enum. */
	IMAGES_UPLOAD(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelImagesUpload"),

	/** The decrypt enum. */
	DECRYPT(new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelDecrypt"),

	/** The file convertion enum. */
	FILE_CONVERTION(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelFileConvertion"),

	/** The report generation enum. */
	REPORT_GENERATION(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelReportGeneration"),

	/** The data generation enum. */
	DATA_GENERATION(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelDataGeneration"),

	/** The screenshot generation enum. */
	SCREENSHOT_GENERATION(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelScreenshotGeneration"),

	/** The master usb merged enum. */
	MASTER_USB_MERGED(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelMasterUsbMerged"),

	/** The eligible option editor enum. */
	ELIGIBLE_OPTION_EDITOR(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelEligibleOptionEditor"),

	/** The integrity check enum. */
	INTEGRITY_CHECK(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelIntegrityCheck"),

	/** The parameter configuration enum. */
	PARAMETER_CONFIGURATION(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.ECM },
			"panelParameterConfiguration"),

	/** The report panel enum. */
	REPORT_PANEL(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.RTS },
			"panelReport"),

	/** The consolidate enum. */
	CONSOLIDATE(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.RTS },
			"panelConsolidate"),

	/** The diagnose automated enum. */
	DIAGNOSE_AUTOMATED(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.RTS },
			"panelDiagnoseAutomated"),

	/** The auditing enum. */
	AUDITING(new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.RTS },
			"panelAuditing"),

	/** The contingency enum. */
	CONTINGENCY(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.RTS },
			"panelContingency"),

	/** The monitoring panel enum. */
	MONITORING_PANEL(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.RTS },
			"panelMonitoring"),

	/** The diagnose station enum. */
	DIAGNOSE_STATION(
			new ApplicationDefinitionEnum[] { ApplicationDefinitionEnum.DIAGNOSTIC },
			"panelDiagnoseStation");

	/**
	 * List of applications that supports the Functionality Definition.
	 */

	private ApplicationDefinitionEnum[] supportedApplications;

	/** Functionality panel name. */
	private String functionalityMainPanelName;

	/**
	 * Getter of <code>functionalityMainPanelName</code> property.
	 * 
	 * @return <code>functionalityMainPanelName</code> property value.
	 * @see #functionalityMainPanelName
	 */

	public String getFunctionalityMainPanelName() {
		return this.functionalityMainPanelName;
	}

	/**
	 * Creates a Functionality Definition.
	 * 
	 * @param apps
	 *            List of applications that supports the Functionality
	 *            Definition
	 * @param functionalityMainPanelName
	 *            the functionality main panel name
	 */

	private FunctionalityDefinitionEnum(ApplicationDefinitionEnum[] apps,
			String functionalityMainPanelName) {
		this.supportedApplications = (apps != null) ? apps.clone() : null;
		this.functionalityMainPanelName = functionalityMainPanelName;
	}

	/**
	 * Getter for the <code>supportedApplications</code> attribute.
	 * 
	 * @return <code>supportedApplications</code> attribute value.
	 * @see FunctionalityDefinitionEnum#supportedApplications
	 */

	public ApplicationDefinitionEnum[] getSupportedApplications() {
		return this.supportedApplications;
	}

}