/**
 * ProfileData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.security;

import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.general.BaseGeneralInfo;
import com.smartmatic.waffle.domain.user.UserData;

/**
 * Represents a user profile. A given profiles gives access to a list of
 * functionalities
 */
@Entity
@Table(name = "profile", uniqueConstraints = { @UniqueConstraint(columnNames = "profile_code") })
public class ProfileData extends BaseGeneralInfo {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -8941194866958617733L;

	/** The profile name. */

	private String profileName;

	/**
	 * Profile description.
	 */

	private String profileDescription;

	/** Collection of functionalities available in the profile. */

	private Set<FunctionalityData> functionalities;

	/** The users. */
	private Set<UserData> users;

	/** Flag used to indicate if the profile is an administrative profile. */

	boolean adminProfile;

	/**
	 * Gets the code.
	 * 
	 * @return the code
	 * @inheritDoc
	 */
	@Override
	@Id
	@Column(name = "profile_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_profile")
	@SequenceGenerator(name = "s_profile", sequenceName = "s_profile", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Sets the code.
	 * 
	 * @param code
	 *            the new code
	 * @inheritDoc
	 */
	@Override
	public void setCode(long code) {
		this.code = code;
	}

	/**
	 * Getter for the <code>profileName</code> attribute.
	 * 
	 * @return <code>profileName</code> attribute value.
	 * @see ProfileData#profileName
	 */
	@Column(name = "profilename", length = 255)
	public String getProfileName() {
		return this.profileName;
	}

	/**
	 * Setter for the <code>profileName</code> attribute.
	 * 
	 * @param profileName
	 *            the profileName to set
	 * @see ProfileData#profileName
	 */

	public void setProfileName(String profileName) {
		this.profileName = profileName;
	}

	/**
	 * Getter for the <code>profileDescription</code> attribute.
	 * 
	 * @return <code>profileDescription</code> attribute value.
	 * @see ProfileData#profileDescription
	 */
	@Column(name = "profiledescription", length = 255)
	public String getProfileDescription() {
		return this.profileDescription;
	}

	/**
	 * Setter for the <code>profileDescription</code> attribute.
	 * 
	 * @param profileDescription
	 *            the profileDescription to set
	 * @see ProfileData#profileDescription
	 */
	public void setProfileDescription(String profileDescription) {
		this.profileDescription = profileDescription;
	}

	/**
	 * Getter for the <code>functionalities</code> attribute.
	 * 
	 * @return <code>functionalities</code> attribute value.
	 * @see ProfileData#functionalities
	 */
	@ManyToMany(targetEntity = FunctionalityData.class)
	@JoinTable(name = "profile_function", joinColumns = @JoinColumn(name = "profile_code", referencedColumnName = "profile_code"), inverseJoinColumns = @JoinColumn(name = "functionality_code", referencedColumnName = "functionality_code"))
	public Set<FunctionalityData> getFunctionalities() {
		return this.functionalities;
	}

	/**
	 * Setter for the <code>functionalities</code> attribute.
	 * 
	 * @param functionalities
	 *            the functionalities to set
	 * @see ProfileData#functionalities
	 */
	public void setFunctionalities(Set<FunctionalityData> functionalities) {
		this.functionalities = functionalities;
	}

	/**
	 * Getter for the <code>adminProfile</code> attribute.
	 * 
	 * @return <code>adminProfile</code> attribute value.
	 * @see ProfileData#adminProfile
	 */
	@Column(name = "adminprofile")
	public boolean isAdminProfile() {
		return this.adminProfile;
	}

	/**
	 * Setter for the <code>adminProfile</code> attribute.
	 * 
	 * @param adminProfile
	 *            the adminProfile to set
	 * @see ProfileData#adminProfile
	 */
	public void setAdminProfile(boolean adminProfile) {
		this.adminProfile = adminProfile;
	}

	/**
	 * Getter of <code>users</code> property.
	 * 
	 * @return <code>users</code> property value.
	 * @see #users
	 */
	@OneToMany(mappedBy = "profile")
	public Set<UserData> getUsers() {
		return this.users;
	}

	/**
	 * Setter of <code>users</code> property.
	 * 
	 * @param users
	 *            The <code>users</code> to set.
	 * @see #users
	 */
	public void setUsers(Set<UserData> users) {
		this.users = users;

		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getCustomCode()
	 */
	@CustomCodeAnnotation
	@Column(name = "profile_custom_code", nullable = false, unique = true, length = 255)
	@Override
	public String getCustomCode() {
		return super.getCustomCode();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#setCustomCode(
	 * java.lang.String)
	 */
	@Override
	public void setCustomCode(String customCode) {
		super.setCustomCode(customCode);
	}

}
