/**
 * BulkServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.bulk.service.impl;

import java.beans.PropertyDescriptor;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.EJBTransactionRolledbackException;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.persistence.Id;
import javax.transaction.NotSupportedException;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;
import javax.xml.bind.DatatypeConverter;

import net.sf.ehcache.Cache;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.beanutils.converters.BooleanConverter;
import org.apache.commons.beanutils.converters.DoubleConverter;
import org.apache.commons.beanutils.converters.IntegerConverter;
import org.apache.commons.beanutils.converters.LongConverter;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.hibernate.exception.ConstraintViolationException;
import org.xml.sax.InputSource;

import com.smartmatic.waffle.domain.IBaseDataObject;
import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.bulk.IBinaryDataWrapper;
import com.smartmatic.waffle.domain.user.IUser;
import com.smartmatic.waffle.ecm.bulk.service.lang.BulkLoadException;
import com.smartmatic.waffle.ecm.bulk.service.lang.BulkLoadException.ExceptionInfo;
import com.smartmatic.waffle.ecm.bulk.service.spec.IBulkService;
import com.smartmatic.waffle.ecm.bulk.service.spec.IBulkServiceLocal;
import com.smartmatic.waffle.ecm.bulk.service.util.BulkCaches;
import com.smartmatic.waffle.ecm.bulk.service.util.BulkXmlValidator;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.PersistedResutsDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.BulkResultType;
import com.smartmatic.waffle.ecm.data.transfer.type.BulkStoreMode;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.log.progress.ProgressLogger;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.search.transformation.TransformationFactory;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;
import com.smartmatic.waffle.properties.ApplicationProperties;
import com.smartmatic.waffle.properties.ApplicationPropertiesException;

/**
 * Session bean responsible for processing and storing / updating information in
 * chunks of data.
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 */
@Stateless(name = IBulkService.SERVICE_NAME)
@Remote(IBulkService.class)
@Local(IBulkServiceLocal.class)
@TransactionManagement(TransactionManagementType.BEAN)
public class BulkServiceImpl implements IBulkService {

	/**
	 * Static reference to log any info or error which occured executing methods
	 * from this class.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(BulkServiceImpl.class);

	/**
	 * Object responsible for managing all the database accesses that this bean
	 * requires.
	 */

	@EJB
	private IPersistenceServiceLocal persistenceService;

	/**
	 * Instance of UserTransaction to manage database transactions.
	 */
	@Resource
	private UserTransaction userTransaction;

	/** Format used to convert a date in a legible format. */
	private static final String DEFAULT_FORMAT = "yyyy.MM.dd.HH.mm.ss.SSS";

	/**
	 * Object that is responsible of the conversion of the date types into a
	 * standard string representation in the application.
	 */
	private SimpleDateFormat formatter = new SimpleDateFormat(
			BulkServiceImpl.DEFAULT_FORMAT);

	/** Number of elements to be inserted per transaction. */
	private Integer batchSize;

	/** Simple cache to avoid searching each time for custom code and code. */
	private Map<String, String> classPropertiesCache = new HashMap<String, String>();

	/**
	 * @inheritDoc
	 */
	@Override
	public BulkResultDTO bulkLoad(byte[] xmlData, IUser user,
			BulkStoreMode bulkStoreMode) throws BulkLoadException {
		return bulkLoadExecute(xmlData, null, user, bulkStoreMode);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public BulkResultDTO bulkLoad(byte[] xmlData, byte[] data, IUser user,
			BulkStoreMode bulkStoreMode) throws BulkLoadException {

		try {
			File tempFile = null;
			ZipFile zip = null;

			try {
				tempFile = File.createTempFile("bulk", "zip");

				FileUtils.writeByteArrayToFile(tempFile, data);

				zip = new ZipFile(tempFile);

				extractVoiceFile(xmlData, zip);

				return bulkLoadExecute(xmlData, zip, user, bulkStoreMode);
			} finally {
				if (tempFile != null) {
					FileUtils.forceDelete(tempFile);
				}

				if (zip != null) {
					zip.close();
				}
			}
		} catch (BulkLoadException e) {
			throw e;
		} catch (Exception e) {
			logger.error(e, "There is an error executing bulk load process");
			throw new BulkLoadException(ExceptionInfo.UNEXPECTED_PROBLEM, e);
		}
	}

	/**
	 * Extract the voice zip file from the resources zip file.
	 * 
	 * @param zip
	 *            The resources zip file.
	 * @throws IOException
	 * @throws DocumentException
	 * @throws BulkLoadException
	 */
	protected void extractVoiceFile(byte[] xmlData, ZipFile zip)
			throws BulkLoadException, DocumentException, IOException {

		InputStream reader = new ByteArrayInputStream(xmlData);

		try {
			InputSource source = new InputSource(reader);
			Document doc = parseXMLFromByteArray(source, reader);
			Element baseNode = doc.getRootElement();

			String className = baseNode.valueOf("@type");

			InputStream inputStream = null;

			if ("com.smartmatic.waffle.domain.i18n.LanguageData"
					.equals(className)) {
				List<Node> nodes = doc.selectNodes("//field-simple");
				Set<String> configuredFiles = new HashSet<String>();

				for (Node node : nodes) {
					Element element = (Element) node;
					if ("voiceNameHDR".equalsIgnoreCase(element
							.attributeValue("name"))
							|| "voiceNameDAT".equalsIgnoreCase(element
									.attributeValue("name"))) {
						configuredFiles.add(element.attributeValue("value"));
					}
				}

				Enumeration<? extends ZipEntry> entries = zip.entries();

				while (entries.hasMoreElements()) {
					ZipEntry entry = entries.nextElement();
					if (entry
							.getName()
							.toLowerCase()
							.equals(ApplicationProperties
									.getValue("voices.zip.name").toString()
									.toLowerCase())) {
						try {
							File voicesFilesDir = generateECMVoiceFolder();

							inputStream = zip.getInputStream(entry);

							File outputFile = new File(voicesFilesDir.getPath()
									+ File.separator + entry.getName());

							FileUtils.copyInputStreamToFile(inputStream,
									outputFile);

							validateConfiguredFiles(configuredFiles, outputFile);
						} catch (IOException e) {
							logger.warn(
									"An error ocurred extracting the voices zip file",
									e);
						} finally {
							try {
								inputStream.close();
							} catch (IOException e) {
								logger.warn(
										"An error ocurred closing the streams",
										e);
							}
						}
					}

				}
			}
		} finally {
			try {
				reader.close();
			} catch (IOException e) {
				logger.warn("An error ocurred closing the reader streams", e);
			}
		}

	}

	/**
	 * Validate if the configured files are located into the zip file.
	 * 
	 * @param configuredFiles
	 *            The configured files on the smartmatic file.
	 * @param zipFile
	 *            The Zip file with the resources.
	 * @throws IOException
	 *             exception throws if the an error occurs reading the zip file.
	 * @throws BulkLoadException
	 *             exception throws if the zip zip file does not contains all
	 *             the configured files.
	 */
	protected void validateConfiguredFiles(Set<String> configuredFiles,
			File zipFile) throws IOException, BulkLoadException {
		ZipInputStream zipInputStream = null;
		try {
			zipInputStream = new ZipInputStream(new FileInputStream(zipFile));

			boolean zipWithAllEntriesRequired = true;
			ZipEntry zipEntry = zipInputStream.getNextEntry();

			List<String> zipEntries = new ArrayList<String>();

			while (zipEntry != null) {

				zipEntries.add(zipEntry.getName());
				zipEntry = zipInputStream.getNextEntry();
			}

			if (configuredFiles != null && !configuredFiles.isEmpty()) {
				if (zipEntries == null || zipEntries.isEmpty()) {
					throw new BulkLoadException(
							BulkLoadException.ExceptionInfo.VOICE_ZIP_WITHOUT_REQUIRED_FILE,
							new Exception());
				}

				for (String configuredFile : configuredFiles) {
					for (String fileInZip : zipEntries) {

						if (fileInZip.lastIndexOf(File.separator) + 1 >= fileInZip
								.length()) {
							continue;
						}

						zipWithAllEntriesRequired = (fileInZip.substring(
								fileInZip.lastIndexOf(File.separator) + 1,
								fileInZip.length()))
								.equalsIgnoreCase(configuredFile);

						if (zipWithAllEntriesRequired) {
							break;
						}
					}
					if (!zipWithAllEntriesRequired) {
						throw new BulkLoadException(
								BulkLoadException.ExceptionInfo.VOICE_ZIP_WITHOUT_REQUIRED_FILE,
								new Exception());
					}
				}
			}
		} finally {
			if (zipInputStream != null) {
				zipInputStream.close();
			}
		}

	}

	/**
	 * Check if the voice folder does not exist and create it.
	 * 
	 * @return The File to the voices folder.
	 */
	private File generateECMVoiceFolder() {
		String voiceFilesFolder = ApplicationProperties
				.getValue("voices.files.folder");
		File voicesFilesDir = new File(
				System.getProperty("jboss.server.home.dir"), voiceFilesFolder);

		if (!voicesFilesDir.exists()) {
			voicesFilesDir.mkdir();
		}
		return voicesFilesDir;
	}

	/**
	 * Method that handles the persistence of a specific object in the database.
	 * It is responsible for interacting with the persistence service and
	 * ensuring that the database operation is performed in a new transaction.
	 * 
	 * @param objectList
	 *            the object list
	 * @param user
	 *            The user who is performing the operation
	 * @param node
	 *            The xml node that describes the object. It is required to
	 *            store the result of the persistence operation
	 * @param bulkStoreMode
	 *            the bulk store mode
	 * @param report
	 *            the previous bulk upload result
	 * 
	 * @return true, if successful
	 */
	private boolean persistObject(List<IBaseDataObject> objectList, IUser user,
			Element node, BulkStoreMode bulkStoreMode, BulkResultDTO report) {
		try {
			// Obtains the first object from the list
			IBaseDataObject object = objectList.get(0);
			// Persist the object in the database
			if (bulkStoreMode == BulkStoreMode.PERSIST) {
				this.persistenceService.persist(object, user);
			} else if (bulkStoreMode == BulkStoreMode.ADD) {
				this.persistenceService.persist(object, user);
			} else {
				this.persistenceService.update(object, user);
			}

			addToRefCache(object);

			// Updates the xml to store the result of the bulk operation
			Element resultElement = node.addElement("result");
			resultElement.addAttribute("value", BulkResultType.OK.toString());

			return true;
		} catch (BaseException e) {

			// Updates the xml to store the result of the bulk operation
			Element resultElement = node.addElement("result");
			resultElement.addAttribute("value",
					BulkResultType.PROBLEM.toString());
			resultElement.addAttribute("cause", e.getMessage());

			report.addErrorReport(node.asXML(), e);

			StringWriter innerWriter = new StringWriter();

			PrintWriter writer = new PrintWriter(innerWriter, true);

			e.printStackTrace(writer);

			resultElement.addAttribute("stackTrace", innerWriter.toString());

			return false;
		} catch (EJBTransactionRolledbackException e) {
			BulkLoadException ex = null;
			if (e.getCause() instanceof javax.persistence.PersistenceException) {
				if (e.getCause().getCause() instanceof ConstraintViolationException) {

					ConstraintViolationException constraintViolationException = (ConstraintViolationException) e
							.getCause().getCause();

					String SQLExceptionMessage = constraintViolationException
							.getSQLException().getNextException().getMessage();

					String[] tokens = SQLExceptionMessage.split("\"");

					ex = new BulkLoadException(
							BulkLoadException.ExceptionInfo.CONSTRAINT_VIOLATION,
							tokens[1]);
				}
			}
			if (ex == null) {
				ex = new BulkLoadException(
						BulkLoadException.ExceptionInfo.PERSISTENCE_PROBLEM, e);
			}
			Element resultElement = node.addElement("result");
			resultElement.addAttribute("value",
					BulkResultType.PROBLEM.toString());
			resultElement.addAttribute("cause", e.getMessage());

			report.addErrorReport(node.asXML(), ex);

			StringWriter innerWriter = new StringWriter();

			PrintWriter writer = new PrintWriter(innerWriter, true);

			e.printStackTrace(writer);

			resultElement.addAttribute("stackTrace", innerWriter.toString());

			return false;
		} catch (Exception e) {

			// Updates the xml to store the result of the bulk operation
			Element resultElement = node.addElement("result");
			resultElement.addAttribute("value",
					BulkResultType.PROBLEM.toString());
			resultElement.addAttribute("cause", e.getMessage());

			BulkLoadException exception = new BulkLoadException(
					BulkLoadException.ExceptionInfo.PERSISTENCE_PROBLEM);
			report.addErrorReport(node.asXML(), exception);

			StringWriter innerWriter = new StringWriter();

			PrintWriter writer = new PrintWriter(innerWriter, true);

			e.printStackTrace(writer);

			resultElement.addAttribute("stackTrace", innerWriter.toString());

			return false;
		}
	}

	/**
	 * Adds the id and customCode of the object to the cache in order to
	 * accelerate bulk load.
	 * 
	 * @param object
	 *            the object to add
	 * @throws IllegalAccessException
	 *             exception throws by PropertyUtils if the caller does not have
	 *             access to the property accessor method
	 * @throws InvocationTargetException
	 *             exception throws by PropertyUtils if the property accessor
	 *             method throws an exception
	 * @throws NoSuchMethodException
	 *             exception throws by PropertyUtils if an accessor method for
	 *             this property cannot be found
	 */
	@SuppressWarnings("unchecked")
	private void addToRefCache(IBaseDataObject object)
			throws IllegalAccessException, InvocationTargetException,
			NoSuchMethodException {
		String idPropertyName = this
				.getPropertyWithAnnotation(object, Id.class);
		String customCodePropertyName = getPropertyWithAnnotation(object,
				CustomCodeAnnotation.class);

		if (customCodePropertyName != null && idPropertyName != null) {
			Cache cache = BulkCaches.getRefObjects();
			String key = buildObjectKey(
					(Class<IBaseDataObject>) object.getClass(), PropertyUtils
							.getProperty(object, customCodePropertyName)
							.toString());

			net.sf.ehcache.Element refObjectInCache = cache.get(key);
			if (refObjectInCache == null) {
				cache.put(new net.sf.ehcache.Element(key, PropertyUtils
						.getProperty(object, idPropertyName)));
			}
		}
	}

	/**
	 * Checks an xml (received as a parameter in the bulk data loading) against
	 * the basic xsd to which all bulk files must comply.
	 * 
	 * @param xmlData
	 *            The xml to validate
	 * @throws BulkLoadException
	 *             if the xml is not well-formed
	 */
	private void checkXmlSyntax(InputSource xmlData) throws BulkLoadException {
		String errors = BulkXmlValidator.validateBaseXsd(xmlData);
		if (errors.length() != 0) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.XML_NOT_VALID_BASE_XSD,
					errors);
		}
	}

	/**
	 * Checks the xml received as parameter in the bulk data loading with the
	 * custom xsd that the developer created to ensure a stronger validation.
	 * The way to configure a custom xsd consists in creating a property in the
	 * file loaded in the ApplicationProperties class that must match this
	 * criteria: <code>BULK.XSD.PATH. + "fullyQualifiedClassName"</code>.
	 * Example:
	 * <code>BULK.XSD.PATH.com.smartmatic.axis.domain.entity.EntityCategoryData</code>
	 * The value of the property must be the xsd against which the developer
	 * wants to perform the validation.
	 * 
	 * @param xmlData
	 *            The xml to validate
	 * @param className
	 *            the fully qualified name for the class
	 * @throws BulkLoadException
	 *             if the xml is not well-formed
	 */
	private void checkCustomXmlSyntax(InputSource xmlData, String className)
			throws BulkLoadException {

		String customXsdPath = "";
		String propertyKey = "bulk.xsd.path." + className;
		try {
			customXsdPath = ApplicationProperties.getValue(propertyKey);
		} catch (ApplicationPropertiesException ex) {
			logger.warn(
					String.format(
							"Property not found: %s, custom validation is not performed ",
							propertyKey), ex);
			return;
		}

		String errors = BulkXmlValidator.validateXsd(xmlData, customXsdPath);
		if (errors.length() != 0) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.XML_NOT_VALID_CUSTOM_XSD,
					errors);
		}
	}

	/**
	 * Stores a chunk of data in the database. This is the entry point to use
	 * the bulk service.
	 * 
	 * @param xmlData
	 *            The xml where is defined all the information to store.
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is required to load
	 * @param user
	 *            The user who is performing the operation.
	 * @param bulkStoreMode
	 *            the bulk store mode
	 * 
	 * @return the result of bulk upload executed
	 * 
	 * @throws BulkLoadException
	 *             if the xml is not well-formed
	 */
	protected BulkResultDTO bulkLoadExecute(byte[] xmlData, ZipFile zipFile,
			IUser user, BulkStoreMode bulkStoreMode) throws BulkLoadException {
		Document doc = null;
		BulkResultDTO result;
		BulkResultDTO bulkOperationResult = new BulkResultDTO();
		InputStream reader = new ByteArrayInputStream(xmlData);

		try {
			InputSource source = new InputSource(reader);

			// Checks the xml against the base xsd.
			try {
				doc = parseXMLFromByteArray(source, reader);
			} catch (BulkLoadException e) {
				if (ExceptionInfo.XML_NOT_VALID_BASE_XSD == e.getInfo()) {
					result = new BulkResultDTO();
					result.setCorrectCount(0);
					result.setIncorrectCount(1);
					result.addErrorReport(e.getMessage(), e);
					return result;
				}

			}

			reader.reset();
			result = bulkLoadExecute(zipFile, user, bulkStoreMode,
					bulkOperationResult, source, doc);

			return result;

		} catch (BulkLoadException e) {
			throw e;
		} catch (DocumentException e) {
			result = new BulkResultDTO();
			result.setCorrectCount(0);
			result.setIncorrectCount(1);
			result.addErrorReport(e.getMessage(), new BulkLoadException(
					ExceptionInfo.XML_PARSE_EXCEPTION, e));
			return result;
		} catch (ClassNotFoundException e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.CLASS_NOT_FOUND, e);
		} catch (Exception e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.UNEXPECTED_PROBLEM, e);
		} finally {
			try {
				reader.close();
			} catch (IOException exception) {
				throw new BulkLoadException(
						BulkLoadException.ExceptionInfo.UNEXPECTED_PROBLEM,
						exception);
			}
		}
	}

	/**
	 * Parse the byte array
	 * 
	 * @param source
	 * @param reader
	 * @return
	 * @throws DocumentException
	 * @throws BulkLoadException
	 * @throws IOException
	 */
	protected Document parseXMLFromByteArray(InputSource source,
			InputStream reader) throws DocumentException, BulkLoadException,
			IOException {

		checkXmlSyntax(source);

		// Loads and parses the xml now that the module knows that is a
		// valid bulk xml
		SAXReader saxReader = new SAXReader();
		reader.reset();
		return saxReader.read(source);
	}

	/**
	 * Stores a chunk of data in the database. This is the entry point to use
	 * the bulk service.
	 * 
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is being loaded
	 * @param user
	 *            The user who is performing the operation
	 * @param bulkStoreMode
	 *            the bulk store mode
	 * @param bulkOperationResult
	 *            previous bulk upload result
	 * @param source
	 *            input source file
	 * @param doc
	 *            The xml where is defined all the information to store
	 * 
	 * @return DTO with the result of the database storing operation
	 * 
	 * @throws BulkLoadException
	 *             if the xml is not well-formed
	 * 
	 * @throws ClassNotFoundException
	 *             if the class entity to be loaded cannot be located
	 */
	@SuppressWarnings("unchecked")
	private BulkResultDTO bulkLoadExecute(ZipFile zipFile, IUser user,
			BulkStoreMode bulkStoreMode, BulkResultDTO bulkOperationResult,
			InputSource source, Document doc) throws BulkLoadException,
			ClassNotFoundException {

		Element baseNode = doc.getRootElement();

		String className = baseNode.valueOf("@type");

		// Initializes the administrative information required to comunicate
		// once the process has finished
		int correctCount = 0;
		int incorrectCount = 0;
		int loaded = 0;

		// Now that the modules knows the type of the object to load it
		// validates against a custom registered xsd
		try {
			checkCustomXmlSyntax(source, className);
		} catch (BulkLoadException e) {
			logger.error("Error checking xml syntax: " + className, e);
			if (ExceptionInfo.XML_NOT_VALID_CUSTOM_XSD == e.getInfo()) {
				BulkResultDTO result = new BulkResultDTO();
				result.setCorrectCount(0);
				result.setIncorrectCount(1);
				result.addErrorReport(e.getMessage(), e);
				return result;
			} else {
				throw e;
			}

		}

		/* Helps Garbage Collector */
		IOUtils.closeQuietly(source.getByteStream());
		source = null;

		Class<IBaseDataObject> classType = (Class<IBaseDataObject>) Class
				.forName(className);

		// Start the iterative process to store the objects
		Element bulkElement = doc.getRootElement();
		List<Element> list = bulkElement.elements();

		int amount = list.size();
		if (amount == 0) {

			BulkServiceImpl.logger
					.info("There are no elements to be loaded for: "
							+ classType.getSimpleName());

			// Constructs and returns the result of the operation
			bulkOperationResult.setCorrectCount(0);
			bulkOperationResult.setIncorrectCount(0);
			bulkOperationResult.addErrorReport(
					BulkLoadException.ExceptionInfo.INVALID_FILE.toString(),
					new BulkLoadException(
							BulkLoadException.ExceptionInfo.INVALID_FILE));
			return bulkOperationResult;
		}

		/* Measure execution time for logging */
		ProgressLogger progressLogger = new ProgressLogger(logger,
				"HH 'hours' mm 'minutes' ss 'seconds'", "Loading data for: "
						+ classType.getSimpleName(),
				"Loaded %d of %d (%d%%) [elapsed: %s]: "
						+ classType.getSimpleName(), "Loaded %d of %d (%d%%): "
						+ classType.getSimpleName(), "", amount);

		Iterator<Element> iter = bulkElement.elementIterator();
		while (iter.hasNext()) {

			PersistedResutsDTO loadedInPage = persistPage(zipFile, user,
					bulkStoreMode, bulkOperationResult, classType, iter);

			loaded += loadedInPage.getLoaded();
			correctCount += loadedInPage.getCorrectCount();
			incorrectCount += loadedInPage.getIncorrectCount();

			if (loadedInPage.getIncorrectCount() > 0) {
				break;
			}

			progressLogger.step(loadedInPage.getLoaded());
		}

		BulkServiceImpl.logger.info("Finished bulk load for: "
				+ classType.getSimpleName());

		BulkServiceImpl.logger.info("Number of elements to be loaded: "
				+ amount);

		BulkServiceImpl.logger.info("Number of elements loaded: " + loaded);

		bulkOperationResult.setCorrectCount(correctCount);
		bulkOperationResult.setIncorrectCount(incorrectCount);
		if (correctCount == 0 && incorrectCount == 0) {
			bulkOperationResult.addErrorReport(
					BulkLoadException.ExceptionInfo.INVALID_FILE.toString(),
					new BulkLoadException(
							BulkLoadException.ExceptionInfo.INVALID_FILE));
		}
		return bulkOperationResult;
	}

	/**
	 * Method to persist a page of chunks of data.
	 * 
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is required to load
	 * @param user
	 *            the user who is performing the operation.
	 * @param bulkStoreMode
	 *            bulk store mode
	 * @param bulkOperationResult
	 *            DTO with the result of bulk upload
	 * @param classType
	 *            the domain class that is uploading
	 * @param elementIterator
	 *            the iterator of bulk upload elements
	 * 
	 * @return DTO with result of persistence
	 */
	private PersistedResutsDTO persistPage(ZipFile zipFile, IUser user,
			BulkStoreMode bulkStoreMode, BulkResultDTO bulkOperationResult,
			Class<IBaseDataObject> classType, Iterator<Element> elementIterator) {

		PersistedResutsDTO results = null;

		try {
			this.userTransaction.begin();

			results = persistPage(zipFile, user, bulkStoreMode,
					bulkOperationResult, classType, elementIterator,
					this.getBatchSize());

			this.userTransaction.commit();

		} catch (Exception e) {
			try {
				this.userTransaction.rollback();
			} catch (Exception ex) {
				logger.error(ex, "There is an error make rollback transaction");
			}
		}
		return results;
	}

	/**
	 * Method to persist a page of chunks of data.
	 * 
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is required to load
	 * @param user
	 *            the user who is performing the operation.
	 * @param bulkStoreMode
	 *            bulk store mode
	 * @param bulkOperationResult
	 *            DTO with the result of bulk upload
	 * @param classType
	 *            the domain class that is uploading
	 * @param elementIterator
	 *            the iterator of bulk upload elements
	 * @param elementCount
	 *            number of elements the iterator should iterate over
	 * 
	 * @return DTO with result of persistence
	 * 
	 * @throws NotSupportedException
	 *             if the calling thread is already associated with a
	 *             transaction, and nested transactions are not supported.
	 * @throws SystemException
	 *             if the transaction service fails in an unexpected way
	 */
	private PersistedResutsDTO persistPage(ZipFile zipFile, IUser user,
			BulkStoreMode bulkStoreMode, BulkResultDTO bulkOperationResult,
			Class<IBaseDataObject> classType,
			Iterator<Element> elementIterator, int elementCount)
			throws NotSupportedException, SystemException {

		logger.debug("Inserting " + classType.getSimpleName() + " "
				+ elementCount + " elements.");
		PersistedResutsDTO results;
		// Initializes the administrative information required to comunicate
		// once the process has finished
		int correctCount = 0;
		int incorrectCount = 0;
		int loaded = 0;
		List<IBaseDataObject> cacheList = new LinkedList<IBaseDataObject>();
		List<Element> successfulElements = new LinkedList<Element>();

		for (int i = 0; i < elementCount && elementIterator.hasNext(); i++) {
			Element node = elementIterator.next();
			try {

				cacheList.clear();

				// Creates the object
				createObject(classType, zipFile, node, user, bulkStoreMode,
						cacheList, true);
				boolean resultPersist;
				// Stores the object in the database
				resultPersist = persistObject(cacheList, user, node,
						bulkStoreMode, bulkOperationResult);
				// Updates the administrative counters
				if (resultPersist) {
					correctCount++;
					successfulElements.add(node);
					loaded++;
				} else {
					incorrectCount++;
					try {
						this.userTransaction.rollback();
					} catch (Exception ex) {
						logger.error(ex,
								"There is an error make rollback transaction");
					}

					/* Still objects to be inserted? */
					if (i < elementCount && elementIterator.hasNext()
							|| successfulElements.isEmpty() == false) {
						this.userTransaction.begin();
					}
					// Reinserts the previous successful elements
					if (successfulElements.isEmpty() == false) {

						persistPage(zipFile, user, bulkStoreMode,
								new BulkResultDTO(), classType,
								successfulElements.iterator(),
								successfulElements.size());

					}

					loaded++;

					break;
				}
			} catch (BulkLoadException e) {
				incorrectCount++;
				bulkOperationResult.addErrorReport(node.asXML(), e);

				// Updates the xml to inform the cause of the problem.
				Element resultElement = node.addElement("result");
				resultElement.addAttribute("value",
						BulkResultType.PROBLEM.toString());
				resultElement.addAttribute("cause", e.getMessage());

				StringWriter innerWriter = new StringWriter();

				PrintWriter writer = new PrintWriter(innerWriter, true);

				e.printStackTrace(writer);

				resultElement
						.addAttribute("stackTrace", innerWriter.toString());
			} catch (Exception e) {

				// Updates the administrative counters
				incorrectCount++;
				bulkOperationResult
						.addErrorReport(
								node.asXML(),
								new BulkLoadException(
										BulkLoadException.ExceptionInfo.UNEXPECTED_PROBLEM,
										e));

				// Updates the xml to inform the cause of the problem.
				Element resultElement = node.addElement("result");
				resultElement.addAttribute("value",
						BulkResultType.PROBLEM.toString());
				resultElement.addAttribute("cause", e.getMessage());

				StringWriter innerWriter = new StringWriter();

				PrintWriter writer = new PrintWriter(innerWriter, true);

				e.printStackTrace(writer);

				resultElement
						.addAttribute("stackTrace", innerWriter.toString());
			}
		}

		results = new PersistedResutsDTO(correctCount, incorrectCount, loaded);

		return results;
	}

	/**
	 * Method responsible for creating an object from a XML. It returns the
	 * created object in the received list.
	 * 
	 * @param classType
	 *            The class of the object to create
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is required to load
	 * @param node
	 *            The xml node that describes the object. It is required to
	 *            store the result of the persistence operation
	 * @param user
	 *            The user who is performing the operation
	 * @param bulkStoreMode
	 *            the bulk store mode
	 * @param cacheList
	 *            IBaseDataObject Database objects cache (used to reduce DB
	 *            accesses and improve performance)
	 * @param cacheObject
	 *            true to cache the created object
	 * 
	 * @return list of created object
	 * 
	 * @throws BulkLoadException
	 *             if there is a problem creating the object
	 */
	@SuppressWarnings("unchecked")
	private List<IBaseDataObject> createObject(
			Class<IBaseDataObject> classType, ZipFile zipFile, Element node,
			IUser user, BulkStoreMode bulkStoreMode,
			List<IBaseDataObject> cacheList, boolean cacheObject)
			throws BulkLoadException {
		try {
			List<IBaseDataObject> resultList = new ArrayList<IBaseDataObject>();
			IBaseDataObject createdBean = null;

			if (bulkStoreMode == BulkStoreMode.PERSIST) {
				createdBean = classType.newInstance();
			} else if (bulkStoreMode == BulkStoreMode.ADD) {
				try {
					createdBean = createObjectToUpdate(classType, node, user);
				} catch (BulkLoadException e) {
					if (BulkLoadException.ExceptionInfo.OBJECT_REFERENCE_NOT_FOUND == e
							.getInfo()) {
						createdBean = classType.newInstance();
					} else {
						throw e;
					}
				}

			} else {
				createdBean = createObjectToUpdate(classType, node, user);
			}

			List<Element> elementChildren = node.elements();
			for (Element element : elementChildren) {

				final String elementName = element.getName();

				// Convert simple fields
				if ("field-simple".equals(elementName)) {
					convertSimpleField(element, createdBean, user);
				}

				// Convert large fields
				if ("field-large".equals(elementName)) {
					convertLargeField(element, createdBean, user);
				}

				// Convert reference fields
				if ("field-ref".equals(elementName)) {
					convertRefField(element, createdBean, user, cacheList);
				}

				// Convert binary References
				if ("field-binary".equals(elementName)) {
					convertBinary(element, zipFile, createdBean, user);
				}

				// Convert collections
				if ("field-collection".equals(elementName)) {
					convertCollection(element, createdBean, user, cacheList);
				}

				// Convert child collections
				if ("child-bulk-collection".equals(elementName)) {
					convertChildCollection(element, zipFile, createdBean, user,
							cacheList);
				}

				// Convert child dependant collections
				if ("child-bulk-dependant-collection".equals(elementName)) {
					convertChildDepCollection(element, zipFile, createdBean,
							user, cacheList);
				}
			}
			// All created objects created before this line are stored before
			// the root object.
			if (cacheObject) {
				cacheList.add(createdBean);
			}

			resultList.add(createdBean);

			return resultList;
		} catch (BulkLoadException e) {
			throw e;
		} catch (Exception ex) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.OBJECT_CONSTRUCION_PROBLEM,
					ex);
		}
	}

	/**
	 * Loads and sets the binary information located in the xml element into the
	 * bean that will hold the binary data.
	 * 
	 * @param fieldElement
	 *            The xml element where is specified the information to set in
	 *            the bean
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is required to load
	 * @param createdBean
	 *            The bean where the information is going to be set
	 * @param user
	 *            The user who is performing the operation
	 * @throws IllegalAccessException
	 *             if the caller does not have access to the property accessor
	 *             method
	 * @throws InvocationTargetException
	 *             if the property accessor method throws an exception
	 * @throws NoSuchMethodException
	 *             if an accessor method for some property that cannot be found
	 * @throws BulkLoadException
	 *             if the binary is null
	 * @throws InstantiationException
	 *             if the Class that ins trying to instantiate represents an
	 *             abstract class, an interface, an array class, a primitive
	 *             type, or void; or if the class has no null constructor; or if
	 *             the instantiation fails for some other reason.
	 * @throws IOException
	 *             signals exception has occurred opening or processing the
	 *             binary file.
	 */
	@SuppressWarnings("rawtypes")
	private void convertBinary(Element fieldElement, ZipFile zipFile,
			Object createdBean, IUser user) throws IllegalAccessException,
			InvocationTargetException, NoSuchMethodException,
			BulkLoadException, InstantiationException, IOException {

		String fieldName = fieldElement.attributeValue("name");
		String fieldReference = fieldElement.attributeValue("path");

		// Checks if data element in base 64 format was included in the xml
		Node data = fieldElement.selectSingleNode("data");
		String dataBase64 = (data != null) ? data.getStringValue() : null;

		PropertyDescriptor descriptor = PropertyUtils.getPropertyDescriptor(
				createdBean, fieldName);
		Class referenceType = descriptor.getPropertyType();

		// Checks that the field is a valid wrapper to store binary data.
		if (!IBinaryDataWrapper.class.isAssignableFrom(referenceType)) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.BINARY_FIELD_WITHOUT_WRAPPER);
		}

		IBinaryDataWrapper dataWrapper = (IBinaryDataWrapper) referenceType
				.newInstance();

		if (dataBase64 != null) {
			dataWrapper.setBinaryData(DatatypeConverter
					.parseBase64Binary(dataBase64));
		} else {
			if (zipFile == null) {
				throw new BulkLoadException(
						BulkLoadException.ExceptionInfo.BINARY_FIELD_WITHOUT_PROVIDED_ZIP);
			}

			dataWrapper.setBinaryData(obtainBinaryReference(zipFile,
					fieldReference));
		}

		PropertyUtils.setProperty(createdBean, fieldName, dataWrapper);
	}

	/**
	 * Reads the information contained inside a zipfile and returns the raw data
	 * that is inside the inner filesystem that match the reference path.
	 * 
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is required to load
	 * @param referencePath
	 *            The path inside the zip file that is required to load.
	 * @return The raw data that is contained in the file specified by the
	 *         referencePath inside the zipfile
	 * @throws IOException
	 *             signals exception has occurred opening or processing the
	 *             binary file.
	 * @throws BulkLoadException
	 *             if the zip entry inside the binary zip was not found.
	 */
	private byte[] obtainBinaryReference(ZipFile zipFile, String referencePath)
			throws IOException, BulkLoadException {

		byte[] data;
		InputStream dataStream = null;

		try {
			ZipEntry dataEntry = zipFile.getEntry(referencePath);
			if (dataEntry == null) {
				throw new BulkLoadException(
						BulkLoadException.ExceptionInfo.BINARY_FIELD_NOT_FOUND,
						referencePath);
			}
			dataStream = zipFile.getInputStream(dataEntry);
			data = IOUtils.toByteArray(dataStream);
		} finally {
			if (dataStream != null) {
				IOUtils.closeQuietly(dataStream);
			}
		}

		return data;
	}

	/**
	 * Reads from the database the object that holds the refid specified by the
	 * xml element and sets into the property also specified in the xml element
	 * the referenced object.
	 * 
	 * @param fieldElement
	 *            The xml element where is stored the information about the name
	 *            of the property to set and also the "customCode" of the object
	 *            to set
	 * @param createdBean
	 *            The object that will hold the reference object
	 * @param user
	 *            The user who is performing the operation.
	 * @param cacheList
	 *            IBaseDataObject Database objects cache (used to reduce DB
	 *            accesses and improve performance)
	 * @throws IllegalAccessException
	 *             if the caller does not have access to the property accessor
	 *             method
	 * @throws InvocationTargetException
	 *             if the property accessor method throws an exception
	 * @throws NoSuchMethodException
	 *             if an accessor method for this property cannot be found
	 * @throws BulkLoadException
	 *             signals that a problem occurred performing some bulk
	 *             operation
	 */
	@SuppressWarnings("unchecked")
	private void convertRefField(Element fieldElement,
			IBaseDataObject createdBean, IUser user,
			List<IBaseDataObject> cacheList) throws IllegalAccessException,
			InvocationTargetException, NoSuchMethodException, BulkLoadException {

		String fieldName = fieldElement.attributeValue("name");
		String fieldReference = fieldElement.attributeValue("refid");
		PropertyDescriptor descriptor = PropertyUtils.getPropertyDescriptor(
				createdBean, fieldName);
		Class<IBaseDataObject> referenceType = (Class<IBaseDataObject>) descriptor
				.getPropertyType();

		IBaseDataObject referencedObject = createReferredObject(referenceType,
				fieldReference, user, cacheList);
		PropertyUtils.setProperty(createdBean, fieldName, referencedObject);
	}

	/**
	 * Reads from the xml element the information regarding a field and sets the
	 * value into the bean received as parameter.
	 * 
	 * @param fieldElement
	 *            The xml element where is stored the information about the name
	 *            of the property to set and also the "customCode" of the object
	 *            to set
	 * @param createdBean
	 *            The object that will hold the field value
	 * @param user
	 *            The user who is performing the operation.
	 * @throws IllegalAccessException
	 *             if the caller does not have access to the property accessor
	 *             method
	 * @throws InvocationTargetException
	 *             if the property accessor method throws an exception
	 * @throws NoSuchMethodException
	 *             if an accessor method for this property cannot be found
	 * @throws BulkLoadException
	 *             signals that a problem occurred performing some bulk
	 *             operation
	 * @throws ParseException
	 *             if the beginning of the specified string cannot be parsed
	 */
	private void convertSimpleField(Element fieldElement, Object createdBean,
			IUser user) throws IllegalAccessException,
			InvocationTargetException, NoSuchMethodException,
			BulkLoadException, ParseException {
		String fieldName = null;
		Object fieldValue = null;
		try {
			fieldName = fieldElement.attributeValue("name");
			fieldValue = fieldElement.attributeValue("value");

			storeValueOnProperty(createdBean, fieldName, fieldValue);
		} catch (Exception e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.OBJECT_CONSTRUCION_PROBLEM,
					fieldName);
		}

	}

	/**
	 * Reads from the xml element the information regarding a field and sets the
	 * value into the bean received as parameter.
	 * 
	 * @param fieldElement
	 *            The xml element where is stored the information about the name
	 *            of the property to set and also the "customCode" of the object
	 *            to set
	 * @param createdBean
	 *            The object that will hold the field value
	 * @param user
	 *            The user who is performing the operation.
	 * @throws IllegalAccessException
	 *             if the caller does not have access to the property accessor
	 *             method
	 * @throws InvocationTargetException
	 *             if the property accessor method throws an exception
	 * @throws NoSuchMethodException
	 *             if an accessor method for this property cannot be found
	 * @throws BulkLoadException
	 *             signals that a problem occurred performing some bulk
	 *             operation
	 * @throws ParseException
	 *             if the beginning of the specified string cannot be parsed
	 */
	private void convertLargeField(Element fieldElement, Object createdBean,
			IUser user) throws IllegalAccessException,
			InvocationTargetException, NoSuchMethodException,
			BulkLoadException, ParseException {
		String fieldName = fieldElement.attributeValue("name");
		Object fieldValue = fieldElement.getData();

		storeValueOnProperty(createdBean, fieldName, fieldValue);
	}

	/**
	 * Converts the value to the property target type and sets the property.
	 * 
	 * @param createdBean
	 *            The object that will hold the field value
	 * @param fieldName
	 *            the property name where the value will be stored
	 * @param fieldValue
	 *            the value to be stored
	 * @throws IllegalAccessException
	 *             if the caller does not have access to the property accessor
	 *             method
	 * @throws InvocationTargetException
	 *             if the property accessor method throws an exception
	 * @throws NoSuchMethodException
	 *             if an accessor method for this property cannot be found
	 * @throws ParseException
	 *             if the beginning of the specified string cannot be parsed
	 * @throws BulkLoadException
	 *             if the enum reference cannot be found
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void storeValueOnProperty(Object createdBean, String fieldName,
			final Object fieldValue) throws IllegalAccessException,
			InvocationTargetException, NoSuchMethodException, ParseException,
			BulkLoadException {

		PropertyDescriptor descriptor = PropertyUtils.getPropertyDescriptor(
				createdBean, fieldName);
		Class propertyType = descriptor.getPropertyType();

		Object convertedValue;
		// Handle type conversions [Beans utils does not convert primitive
		// types]
		if (propertyType.isAssignableFrom(Boolean.TYPE)) {
			BooleanConverter converter = new BooleanConverter();
			convertedValue = converter.convert(Boolean.class, fieldValue);
		} else if (propertyType.isAssignableFrom(Integer.TYPE)) {
			IntegerConverter converter = new IntegerConverter();
			convertedValue = converter.convert(Integer.class, fieldValue);
		} else if (propertyType.isAssignableFrom(Long.TYPE)) {
			LongConverter converter = new LongConverter();
			convertedValue = converter.convert(Integer.class, fieldValue);
		} else if (propertyType.isAssignableFrom(Double.TYPE)
				|| propertyType.isAssignableFrom(Double.class)) {
			DoubleConverter converter = new DoubleConverter();
			convertedValue = converter.convert(Integer.class, fieldValue);
		} else if (propertyType.isAssignableFrom(Date.class)) {
			convertedValue = this.formatter.parse((String) fieldValue);
		} else if (propertyType.isEnum()) {
			convertedValue = constructEnumValue(propertyType, fieldValue);
		} else {
			convertedValue = fieldValue;
		}

		PropertyUtils.setProperty(createdBean, fieldName, convertedValue);
	}

	/**
	 * Converts the string that is used by the user to a enum instance that can
	 * be used to set the property into the bean used to store the information
	 * into the database.
	 * 
	 * @param <T>
	 *            the generic type
	 * @param enumType
	 *            The class that defines the enum
	 * @param fieldValue
	 *            The value specified by the user
	 * @return The enum instance that represents the value provided by the user
	 * @throws BulkLoadException
	 *             if the enum reference cannot be found
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private <T extends Enum> T constructEnumValue(Class<T> enumType,
			Object fieldValue) throws BulkLoadException {
		try {
			String strValue = (String) fieldValue;
			T enumResult = (T) Enum.valueOf(enumType, strValue);
			return enumResult;
		} catch (Exception e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.INCORRECT_ENUM_REFERENCE,
					e, fieldValue);
		}
	}

	/**
	 * Convert child dep collection.
	 * 
	 * @param fieldElement
	 *            the field element
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is required to load
	 * @param createdBean
	 *            the created bean
	 * @param user
	 *            The user who is performing the operation.
	 * @param cacheList
	 *            IBaseDataObject Database objects cache (used to reduce DB
	 *            accesses and improve performance)
	 * @return list of IBaseDataObject with the operation result
	 * @throws BulkLoadException
	 *             if there is a problem creating the object
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private List<IBaseDataObject> convertChildDepCollection(
			Element fieldElement, ZipFile zipFile, IBaseDataObject createdBean,
			IUser user, List<IBaseDataObject> cacheList)
			throws BulkLoadException {
		try {
			String fieldName = fieldElement.attributeValue("name");
			String reverseField = fieldElement.attributeValue("reverseField");

			PropertyDescriptor descriptor = PropertyUtils
					.getPropertyDescriptor(createdBean, fieldName);

			ParameterizedType parametrizedType = (ParameterizedType) descriptor
					.getReadMethod().getGenericReturnType();
			Class collectionClass = descriptor.getReadMethod().getReturnType();

			Collection<IBaseDataObject> collectionContainer = null;

			if (!collectionClass.isInterface()) {
				// Is a specific class
				collectionContainer = (Collection<IBaseDataObject>) collectionClass
						.newInstance();
			} else {
				if (collectionClass == Set.class) {
					collectionContainer = new HashSet<IBaseDataObject>();
				} else {
					collectionContainer = new ArrayList<IBaseDataObject>();
				}
			}

			// Only supports a single item collection
			Class<IBaseDataObject> referenceType = (Class<IBaseDataObject>) parametrizedType
					.getActualTypeArguments()[0];

			List<Element> list = fieldElement.elements();
			Iterator<Element> iter = list.iterator();
			List<IBaseDataObject> result = new ArrayList<IBaseDataObject>();
			while (iter.hasNext()) {
				Element node = iter.next();

				// Creates the object
				// This line assumes that the child objects are always persisted
				// [not updated]. If this assumption changes this
				// is where is required to update this module
				List<IBaseDataObject> resultList = createObject(referenceType,
						zipFile, node, user, BulkStoreMode.PERSIST, cacheList,
						false);

				// Now adds the information to the parent collection.
				// This could be improved obtaining only the childs in the first
				// level it could represent a problem
				// when there is more than a level in the recursion of object
				// creation.
				for (int i = 0; i < resultList.size(); i++) {
					IBaseDataObject childObject = resultList.get(i);

					if (reverseField != null && reverseField.length() != 0) {
						PropertyUtils.setProperty(childObject, reverseField,
								createdBean);
					}

					collectionContainer.add(childObject);
				}

				result.addAll(resultList);
			}

			PropertyUtils.setProperty(createdBean, fieldName,
					collectionContainer);

			return result;
		} catch (BulkLoadException e) {
			throw e;
		} catch (Exception e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.OBJECT_CONSTRUCION_PROBLEM,
					e);
		}
	}

	/**
	 * Convert child collection.
	 * 
	 * @param fieldElement
	 *            the field element
	 * @param zipFile
	 *            the file where the application will look for the binary
	 *            information that is required to load
	 * @param createdBean
	 *            the entity bean created to load
	 * @param user
	 *            The user who is performing the operation.
	 * @param cacheList
	 *            IBaseDataObject Database objects cache (used to reduce DB
	 *            accesses and improve performance)
	 * 
	 * @return collection of element converted
	 * 
	 * @throws BulkLoadException
	 *             if there is a problem creating the object
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private List<IBaseDataObject> convertChildCollection(Element fieldElement,
			ZipFile zipFile, IBaseDataObject createdBean, IUser user,
			List<IBaseDataObject> cacheList) throws BulkLoadException {
		try {
			String fieldName = fieldElement.attributeValue("name");

			PropertyDescriptor descriptor = PropertyUtils
					.getPropertyDescriptor(createdBean, fieldName);

			ParameterizedType parametrizedType = (ParameterizedType) descriptor
					.getReadMethod().getGenericReturnType();
			Class collectionClass = descriptor.getReadMethod().getReturnType();

			Collection<IBaseDataObject> collectionContainer = null;

			if (!collectionClass.isInterface()) {
				// Is a specific class
				collectionContainer = (Collection<IBaseDataObject>) collectionClass
						.newInstance();
			} else {
				if (collectionClass == Set.class) {
					collectionContainer = new HashSet<IBaseDataObject>();
				} else {
					collectionContainer = new ArrayList<IBaseDataObject>();
				}
			}

			// Only supports a single item collection
			Class<IBaseDataObject> referenceType = (Class<IBaseDataObject>) parametrizedType
					.getActualTypeArguments()[0];

			List<Element> list = fieldElement.elements();
			Iterator<Element> iter = list.iterator();
			List<IBaseDataObject> result = new ArrayList<IBaseDataObject>();
			while (iter.hasNext()) {
				Element node = iter.next();

				// Creeates the object
				// This line assumes that the child objects are always persisted
				// [not updated]. If this assumption changes this
				// is where is required to update this module
				List<IBaseDataObject> resultList = createObject(referenceType,
						zipFile, node, user, BulkStoreMode.PERSIST, cacheList,
						true);

				// Now adds the information to the parent collection.
				// This could be improved obtaining only the childs in the first
				// level it could represent a problem
				// when there is more than a level in the recursion of object
				// creation.
				for (int i = 0; i < resultList.size(); i++) {
					IBaseDataObject childObject = resultList.get(i);
					collectionContainer.add(childObject);
				}

				result.addAll(resultList);
			}

			PropertyUtils.setProperty(createdBean, fieldName,
					collectionContainer);

			return result;
		} catch (BulkLoadException e) {
			throw e;
		} catch (Exception e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.OBJECT_CONSTRUCION_PROBLEM,
					e);
		}
	}

	/**
	 * Reads from the xml element the information from a collection and sets the
	 * collection value into the object received as a paremeter.
	 * 
	 * @param fieldElement
	 *            The xml element where is located the information to set in the
	 *            received object
	 * @param createdBean
	 *            The object where the information has to be set
	 * @param user
	 *            The user who is performing the operation.
	 * @param cacheList
	 *            IBaseDataObject Database objects cache (used to reduce DB
	 *            accesses and improve performance)
	 * @throws IllegalAccessException
	 *             if the caller does not have access to the property accessor
	 *             method
	 * @throws InvocationTargetException
	 *             if the property accessor method throws an exception
	 * @throws NoSuchMethodException
	 *             if an accessor method for this propety cannot be found
	 * @throws InstantiationException
	 *             if the new object instantiation fails for some reason
	 * @throws BulkLoadException
	 *             if the descriptor is null or some problem happened creating
	 *             the referred object
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void convertCollection(Element fieldElement,
			IBaseDataObject createdBean, IUser user,
			List<IBaseDataObject> cacheList) throws IllegalAccessException,
			InvocationTargetException, NoSuchMethodException,
			InstantiationException, BulkLoadException {
		String fieldName = fieldElement.attributeValue("name");

		PropertyDescriptor descriptor = PropertyUtils.getPropertyDescriptor(
				createdBean, fieldName);
		if (descriptor == null) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.INCORRECT_FIELD_NAME,
					fieldName, createdBean.getClass().getCanonicalName());
		}

		ParameterizedType parametrizedType = (ParameterizedType) descriptor
				.getReadMethod().getGenericReturnType();
		Class collectionClass = descriptor.getReadMethod().getReturnType();

		Collection<IBaseDataObject> collectionContainer = null;

		if (!collectionClass.isInterface()) {
			// Is a specific class
			collectionContainer = (Collection<IBaseDataObject>) collectionClass
					.newInstance();
		}

		else {
			if (collectionClass == Set.class) {
				collectionContainer = new HashSet<IBaseDataObject>();
			} else {
				collectionContainer = new ArrayList<IBaseDataObject>();
			}
		}

		// Only supports a single item collection
		Type referenceType = parametrizedType.getActualTypeArguments()[0];

		List<Element> collectionItems = fieldElement.elements();
		Iterator<Element> iterCollection = collectionItems.iterator();
		while (iterCollection.hasNext()) {
			Element actualElement = iterCollection.next();
			if (actualElement.getName().compareTo("ref-object") == 0) {
				// Is a reference object.
				String refId = actualElement.attributeValue("refid");
				IBaseDataObject referenceObject = createReferredObject(
						(Class<IBaseDataObject>) referenceType, refId, user,
						cacheList);
				collectionContainer.add(referenceObject);
			}
		}
		if (PropertyUtils.getProperty(createdBean, fieldName) != null) {
			collectionContainer
					.addAll((Collection<? extends IBaseDataObject>) PropertyUtils
							.getProperty(createdBean, fieldName));
		}

		PropertyUtils.setProperty(createdBean, fieldName, collectionContainer);
	}

	/**
	 * Gets an object from the database which can be modified. All modifications
	 * to this object will be reflected automatically in the database
	 * 
	 * @param <T>
	 *            the generic type
	 * @param objectClass
	 *            the object class
	 * @param node
	 *            The xml node that describes the object. It is required to
	 *            store the result of the persistence operation.
	 * @param user
	 *            The user who is performing the operation.
	 * 
	 * @return An object loaded from the database, of which all updates will be
	 *         reflected in the database
	 * 
	 * @throws BulkLoadException
	 *             signals that a problem occurred performing some bulk
	 *             operation
	 */
	@SuppressWarnings("unchecked")
	private <T extends IBaseDataObject> T createObjectToUpdate(
			Class<T> objectClass, Element node, IUser user)
			throws BulkLoadException {
		try {
			SearchExpression search = new SearchExpression(objectClass);

			T referenceObject = objectClass.newInstance();
			String customCodeProperty = getPropertyWithAnnotation(
					referenceObject, CustomCodeAnnotation.class);

			if (customCodeProperty == null) {
				throw new BulkLoadException(
						BulkLoadException.ExceptionInfo.OBJECT_REFERENCE_WITHOUT_CUSTOM_CODE_OR_ID,
						objectClass);
			}

			Element customCodeElement = (Element) node
					.selectSingleNode("field-simple[@name=\""
							+ customCodeProperty + "\"]");
			if (customCodeElement == null) {
				throw new BulkLoadException(
						BulkLoadException.ExceptionInfo.OBJECT_UPDATE_WITHOUT_CUSTOM_CODE_OR_ID,
						objectClass);
			}

			String value = customCodeElement.attributeValue("value");
			PropertyUtils.setProperty(referenceObject, customCodeProperty,
					value);
			Object castedCustomCodeProperty = PropertyUtils.getProperty(
					referenceObject, customCodeProperty);

			search.addSearchCriteria(CriteriaFactory.eq(customCodeProperty,
					castedCustomCodeProperty));

			T result = (T) this.persistenceService.searchObject(search, null);

			if (result == null) {
				throw new BulkLoadException(
						BulkLoadException.ExceptionInfo.OBJECT_REFERENCE_NOT_FOUND,
						objectClass.getCanonicalName(),
						castedCustomCodeProperty);
			}

			return result;
		} catch (BulkLoadException e) {
			throw e;
		} catch (BaseException e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.DATABASE_OPERATION_PROBLEM,
					e);
		} catch (Exception e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.OBJECT_CONSTRUCION_PROBLEM,
					e);
		}
	}

	/**
	 * Creates a light object of the class received as parameter and with the
	 * customCode received as parameter. This method and object required in the
	 * bulk data loading module and avoid an excessive database load using only
	 * the required fields of the required objects.
	 * 
	 * @param objectClass
	 *            The class of the object to lookup
	 * @param refId
	 *            The custom code of the object to lookup
	 * @param user
	 *            The user who is performing the operation.
	 * @param cacheList
	 *            IBaseDataObject Database objects cache (used to reduce DB
	 *            accesses and improve performance)
	 * @return the referred object created
	 * @throws BulkLoadException
	 *             if the object to load does not contain a valid custom code or
	 *             it reference field was not found
	 */
	private IBaseDataObject createReferredObject(
			Class<IBaseDataObject> objectClass, String refId, IUser user,
			List<IBaseDataObject> cacheList) throws BulkLoadException {
		try {
			IBaseDataObject referenceObject = objectClass.newInstance();
			String customCodeProperty = getPropertyWithAnnotation(
					referenceObject, CustomCodeAnnotation.class);

			if (customCodeProperty == null) {
				throw new BulkLoadException(
						BulkLoadException.ExceptionInfo.OBJECT_REFERENCE_WITHOUT_CUSTOM_CODE_OR_ID,
						objectClass);
			}

			// First searchs the object in the cache list
			for (IBaseDataObject object : cacheList) {
				if (objectClass.isInstance(object)) {
					String property = PropertyUtils.getProperty(object,
							customCodeProperty).toString();
					if (property.compareTo(refId) == 0) {
						return object;
					}
				}
			}

			Object referenceKey;

			/* Look for the object in previous retrived objects cache */
			Cache cache = BulkCaches.getRefObjects();
			String key = buildObjectKey(objectClass, refId);
			net.sf.ehcache.Element refObjectInCache = cache.get(key);
			if (refObjectInCache != null) {

				referenceKey = refObjectInCache.getObjectValue();
			} else {

				// Searchs the object in the database
				SearchExpression search = new SearchExpression(objectClass);
				search.addSearchCriteria(CriteriaFactory.eq(customCodeProperty,
						refId));
				search.setTransformation(TransformationFactory
						.idTransformation());
				referenceKey = this.persistenceService.searchObject(search,
						user);

				if (referenceKey == null) {
					throw new BulkLoadException(
							BulkLoadException.ExceptionInfo.OBJECT_REFERENCE_NOT_FOUND,
							refId);
				}

				setObjectId(referenceObject, referenceKey);
				setObjectCustomCode(referenceObject, refId);

				cache.put(new net.sf.ehcache.Element(key, referenceKey));

			}

			setObjectId(referenceObject, referenceKey);
			setObjectCustomCode(referenceObject, refId);
			return referenceObject;

		} catch (BulkLoadException e) {
			throw e;
		} catch (BaseException e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.DATABASE_OPERATION_PROBLEM,
					e);
		} catch (Exception e) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.OBJECT_CONSTRUCION_PROBLEM,
					e);
		}
	}

	/**
	 * Computes a key for use in caches using the class name plus its
	 * identifier.
	 * 
	 * @param objectClass
	 *            object class which name will be used to construct the key
	 * @param refId
	 *            object's id
	 * @return the string
	 */
	private String buildObjectKey(Class<IBaseDataObject> objectClass,
			String refId) {

		return objectClass.getSimpleName() + "[" + refId + "]";

	}

	/**
	 * Helper method used to obtain the name of the property that hold a
	 * specific annotation. It is used to lookup the name of the customCode
	 * property in a specific class.
	 * 
	 * @param actualObject
	 *            The object to lookup the annotation
	 * @param annotationClass
	 *            The annotation to search
	 * @return The name of the property to lookup
	 */
	private String getPropertyWithAnnotation(Object actualObject,
			Class<? extends Annotation> annotationClass) {

		String key = actualObject.getClass().getSimpleName() + ","
				+ annotationClass.getSimpleName();
		String propertyName = this.classPropertiesCache.get(key);

		if (propertyName != null) {
			return propertyName;
		}

		// Search in methods
		Method idMethod = null;
		Method[] methods = actualObject.getClass().getMethods();
		for (int i = 0; i < methods.length; i++) {
			Method actualMethod = methods[i];
			if (actualMethod.isAnnotationPresent(annotationClass)) {
				idMethod = actualMethod;
				break;
			}
		}

		if (idMethod != null) {
			String getMethodName = idMethod.getName();

			propertyName = getMethodName.charAt(0) == 'g' ? getMethodName
					.substring(3) : getMethodName.substring(2);
			propertyName = Character.toLowerCase(propertyName.charAt(0))
					+ propertyName.substring(1);

			this.classPropertiesCache.put(key, propertyName);
			return propertyName;
		} else {
			return null;
		}
	}

	/**
	 * Sets the id of the object.
	 * 
	 * @param actualObject
	 *            The object to set the key
	 * @param objectKey
	 *            The key
	 * @throws IllegalAccessException
	 *             if the caller does not have access to the property accessor
	 *             method
	 * @throws InvocationTargetException
	 *             if the property accessor method throws an exception
	 * @throws NoSuchMethodException
	 *             if an accessor method for this propety cannot be found
	 * @throws BulkLoadException
	 *             if the object to load does not contain a valid custom code
	 */
	private void setObjectId(Object actualObject, Object objectKey)
			throws IllegalAccessException, InvocationTargetException,
			NoSuchMethodException, BulkLoadException {
		String propertyName = getPropertyWithAnnotation(actualObject, Id.class);
		if (propertyName != null) {
			PropertyUtils.setProperty(actualObject, propertyName, objectKey);
		} else {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.OBJECT_REFERENCE_WITHOUT_CUSTOM_CODE_OR_ID,
					actualObject.getClass());
		}
	}

	/**
	 * Sets the id of the object.
	 * 
	 * @param actualObject
	 *            The object to set the key
	 * @param objectCustomCode
	 *            the object custom code
	 * @throws IllegalAccessException
	 *             if the caller does not have access to the property accessor
	 *             method
	 * @throws InvocationTargetException
	 *             if the property accessor method throws an exception
	 * @throws NoSuchMethodException
	 *             if an accessor method for this propety cannot be found
	 * @throws BulkLoadException
	 *             if the object to load does not contain a valid custom code
	 */
	private void setObjectCustomCode(Object actualObject,
			Object objectCustomCode) throws IllegalAccessException,
			InvocationTargetException, NoSuchMethodException, BulkLoadException {
		String propertyName = getPropertyWithAnnotation(actualObject,
				CustomCodeAnnotation.class);
		if (propertyName != null) {
			PropertyUtils.setProperty(actualObject, propertyName,
					objectCustomCode);
		} else {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.OBJECT_REFERENCE_WITHOUT_CUSTOM_CODE_OR_ID,
					actualObject.getClass());
		}
	}

	/**
	 * Getter of <code>batchSize</code> property.
	 * 
	 * @return <code>batchSize</code> property value.
	 * @see #batchSize
	 */

	public int getBatchSize() {
		if (this.batchSize == null) {
			this.batchSize = ApplicationProperties.getIntValue(
					"BULK.BATCH.SIZE", 25);
		}
		return this.batchSize;
	}

	/**
	 * Setter of <code>batchSize</code> property.
	 * 
	 * @param batchSize
	 *            The <code>batchSize</code> to set.
	 * @see #batchSize
	 */
	public void setBatchSize(int batchSize) {
		this.batchSize = batchSize;

		return;
	}

}