/**
 * ECMApp.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dialog.ModalExclusionType;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.concurrent.TimeoutException;

import javax.ejb.EJB;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Timer;
import javax.swing.UIManager;
import javax.swing.plaf.FontUIResource;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.SystemUtils;
import org.jdesktop.application.Application;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;

import com.smartmatic.waffle.domain.BaseDataObject;
import com.smartmatic.waffle.domain.i18n.LanguageCollectionData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.security.ApplicationDefinitionEnum;
import com.smartmatic.waffle.domain.security.FunctionalityData;
import com.smartmatic.waffle.domain.security.FunctionalityDefinitionEnum;
import com.smartmatic.waffle.domain.security.ProfileData;
import com.smartmatic.waffle.domain.user.UserData;
import com.smartmatic.waffle.ecm.client.core.monitoring.ECMConnectionDataBaseCheckListener;
import com.smartmatic.waffle.ecm.client.core.monitoring.ECMConnectionServerCheckListener;
import com.smartmatic.waffle.ecm.client.core.monitoring.MonitoringResourceType;
import com.smartmatic.waffle.ecm.client.func.CloseButtonActionListener;
import com.smartmatic.waffle.ecm.client.func.ECMDecryptDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMEligibleOptionEditorDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMFileConversionDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMImageUploadDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMIntegrityCheckDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMKeyGenerationDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMMasterUsbMergedDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMParameterConfigurationDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMReportGenerationDispatch;
import com.smartmatic.waffle.ecm.client.func.ECMScreenshotGenerationDispatch;
import com.smartmatic.waffle.ecm.client.ui.controller.IRefreshRegionWidget;
import com.smartmatic.waffle.ecm.client.ui.dates.SettingDateDialog;
import com.smartmatic.waffle.ecm.client.ui.login.PassphraseUserInputDialog;
import com.smartmatic.waffle.ecm.client.ui.login.PassphraseUserUpdateDialog;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.core.battery.BatteryWatcher;
import com.smartmatic.waffle.ecm.core.battery.IBatteryStatusChangedListener;
import com.smartmatic.waffle.ecm.core.battery.IBatteryStatusQuery;
import com.smartmatic.waffle.ecm.core.battery.IBatteryStatusQuery.BatteryState;
import com.smartmatic.waffle.ecm.core.battery.TimerListener;
import com.smartmatic.waffle.ecm.core.monitoring.ConnectivityManagerMultiple;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.ecm.core.printer.IPrinterStatusChangedNotifierListener;
import com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery.StatusPrinter;
import com.smartmatic.waffle.ecm.core.printer.PrinterWatcher;
import com.smartmatic.waffle.ecm.service.spec.IECMBulkDataLoadService;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyService;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralDemographyService;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralOrganizationService;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralPhysicalLocationsRegistryService;
import com.smartmatic.waffle.ecm.service.spec.IECMMasterUSBGenerationService;
import com.smartmatic.waffle.ecm.service.spec.IECMNominationService;
import com.smartmatic.waffle.ecm.service.spec.IECMService;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesService;
import com.smartmatic.waffle.ecm.service.spec.IECMWorkflowService;
import com.smartmatic.waffle.ecm.service.workflow.ECMWorkflowStatus;
import com.smartmatic.waffle.ecm.service.workflow.ECMWorkflowStatus.Validation;
import com.smartmatic.waffle.ecm.ui.BackgroundInitMenu;
import com.smartmatic.waffle.ecm.ui.BackgroundInitScreen;
import com.smartmatic.waffle.ecm.ui.BaseKeyListener;
import com.smartmatic.waffle.ecm.ui.BlockingMessageDialog;
import com.smartmatic.waffle.ecm.ui.ConfigurationEntity;
import com.smartmatic.waffle.ecm.ui.InfoPanel;
import com.smartmatic.waffle.ecm.ui.core.ShutDownMachineActionListener;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.ecm.ui.menu.GeneralMenuDialog;
import com.smartmatic.waffle.ecm.ui.menu.GeneralMenuDialogController;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;
import com.smartmatic.waffle.localpersistence.environment.EnvironmentManager;
import com.smartmatic.waffle.localpersistence.exception.HistoricFileException;
import com.smartmatic.waffle.localpersistence.exception.SAESLocalPersistenceException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Main Class of the ECM application. It is used to run the application on the
 * client side.
 * 
 * @author Carlos Ramírez (cramirez@smartmatic.com)
 */
public class ECMApp extends SingleFrameApplication implements
		IPrinterStatusChangedNotifierListener, IBatteryStatusChangedListener {

	/***
	 * ECM Server's services for general purpose.
	 */
	@EJB
	private static IECMService ecmService;

	/***
	 * ECM Server's services for usb generation.
	 */
	@EJB
	private static IECMMasterUSBGenerationService ecmMasterUSBGenerationService;

	/***
	 * ECM Server's utilities service.
	 */
	@EJB
	private static IECMUtilitiesService ecmUtilitiesService;

	/**
	 * ECM Server's services for workflow control.
	 */
	@EJB
	public static IECMWorkflowService ecmWorkflowService;

	/**
	 * ECM Server's services for Electoral Organization operations.
	 */
	@EJB
	private static IECMElectoralOrganizationService ecmElectoralOrganizationService;

	/**
	 * ECM Server's services for Electoral Physical Locations Registry
	 * operations.
	 */
	@EJB
	private static IECMElectoralPhysicalLocationsRegistryService ecmElectoralPhysicalLocationsRegistryService;

	/**
	 * ECM Server's services for demography operations.
	 */
	@EJB
	private static IECMDemographyService ecmDemographyService;

	/**
	 * ECM Server's services for bulk upload operations.
	 */
	@EJB
	private static IECMBulkDataLoadService ecmBulkDataLoadService;

	/**
	 * ECM Server's service for nomination operations.
	 */
	@EJB
	private static IECMNominationService ecmNominationService;

	/**
	 * ECM Server's service for electoral demography operations.
	 */
	@EJB
	private static IECMElectoralDemographyService ecmElectoralDemographyService;
	/***
	 * Object for logger purpose.
	 */
	private static SMTTLogger logger = SMTTLogger.getLogger(ECMApp.class);

	/** The resource message bundle. */
	private ResourceMap resourceMap;

	/***
	 * Reference to the environment manager service.
	 */
	private EnvironmentManager environmentManager;

	/***
	 * Reference to the configuration entity service.
	 */
	private ConfigurationEntity configurationEntity;

	/***
	 * Languages lists available for the application.
	 */
	private LanguageCollectionData languageCollection;

	/***
	 * Reference to the languages load service.
	 */
	private LanguageController langController;

	/***
	 * Timer used to show flashing notifications about status battery.
	 */
	private Timer timer;

	/***
	 * Listener to notify about changes of the status battery.
	 */
	private TimerListener timerListener;

	/**
	 * blocking popup.
	 */
	private volatile BlockingMessageDialog blockedDialog;

	/**
	 * Used to show a message when there is no connection to app server.
	 */
	private volatile BlockingMessageDialog serverDisconnectedDialog;

	/**
	 * Used to show a message when there is no connection to db server.
	 */
	private volatile BlockingMessageDialog dbDisconnectedDialog;

	/***
	 * Reference to the controller of the menus in the application.
	 */
	private GeneralMenuDialogController menuController;

	/***
	 * Button to go at next step in the workflow.
	 */
	private JButton workflowNextButton;

	/***
	 * Button to go at last step in the workflow.
	 */
	private JButton workflowBackButton;

	/***
	 * Label for show the title to the printer status.
	 */
	private JLabel printerStatusLabel;

	/***
	 * Label for show the current printer status.
	 */
	private JLabel printerStatusDetailLabel;

	/***
	 * Label for show the title to the battery status.
	 */
	private JLabel batteryStatusLabel;

	/** * Label for show the current battery status. */
	private JLabel batteryStatusDetailLabel;

	/** The button export logs. */
	private JButton buttonExportLogs;

	/** The file chooser export logs. */
	private JFileChooser fileChooserExportLogs;

	/** The bg dialog. */
	private BackgroundInitScreen bgDialog;

	/** The server down. */
	private boolean serverDown;

	/** The block exit count. */
	private int functionalitiesBlockingWorkflowButtonsCount;

	/** The block exit count. */
	private int functionalitiesBlockingTabsCount;

	/** The actual step name. */
	private String actualStepName;

	/** The tabs blocked. */
	private volatile boolean tabsBlocked;

	/** The btn back enabled. */
	private boolean btnBackEnabled;

	/** The btn next enabled. */
	private boolean btnNextEnabled;

	/** The dialog to setting time and date. */
	private SettingDateDialog dialogToSettingTimeAndDate;

	/**
	 * Main method launching the application.
	 * 
	 * @param args
	 *            Array of arguments received by the application
	 */
	public static void main(String[] args) {

		ECMServiceProvider.setEcmService(ecmService);
		ECMServiceProvider
				.setEcmMasterUSBGenerationService(ecmMasterUSBGenerationService);
		ECMServiceProvider
				.initializeECMElectoralOrganizationService(ecmElectoralOrganizationService);
		ECMServiceProvider
				.initializeElectoralPhysicalLocationsRegistryService(ecmElectoralPhysicalLocationsRegistryService);
		ECMServiceProvider.initializeDemographyService(ecmDemographyService);

		ECMServiceProvider.initializeUtilitiesService(ecmUtilitiesService);

		ECMServiceProvider
				.initializeBulkDataLoadService(ecmBulkDataLoadService);

		ECMServiceProvider.initializeNominationService(ecmNominationService);

		ECMServiceProvider
				.initializeElectoralDemographyService(ecmElectoralDemographyService);

		ECMServiceProvider.setEcmWorkflowService(ecmWorkflowService);

		UIManager.put("control", Color.WHITE);
		launch(ECMApp.class, args);
	}

	/**
	 * Starts the thread which executes the logic for startup.
	 * 
	 * @see org.jdesktop.application.Application#startup()
	 */
	@Override
	protected void startup() {

		logger.info("Initializing Main Thread");
		MainThread mainThread = new MainThread();
		mainThread.start();

		this.functionalitiesBlockingWorkflowButtonsCount = 0;
		this.functionalitiesBlockingTabsCount = 0;
	}

	/**
	 * Method used to start monitoring thread used by the application.
	 * 
	 * @param dataBaseIp
	 *            the data base ip
	 * @param dataBasePort
	 *            the data base port
	 * @param serverIp
	 *            the server ip
	 * @param serverPort
	 *            the server port
	 */
	private void initMonitoringManager(String dataBaseIp, int dataBasePort,
			String serverIp, int serverPort) {

		logger.info("Initializing to monitoring server and DB connections.");

		ConnectivityManagerMultiple monitoringDBServer = new ConnectivityManagerMultiple(
				dataBaseIp, dataBasePort);
		monitoringDBServer
				.addConnectionListener(new ECMConnectionDataBaseCheckListener());
		monitoringDBServer.start();

		ConnectivityManagerMultiple monitoringAppServer = new ConnectivityManagerMultiple(
				serverIp, serverPort);
		monitoringAppServer
				.addConnectionListener(new ECMConnectionServerCheckListener());
		monitoringAppServer.start();
	}

	/**
	 * Method used to show a unique instance of blocking window.
	 * 
	 * @param keyToMessage
	 *            the key to message
	 * @param translatedAllSupported
	 *            the translated all supported
	 * @param shutdownMachine
	 *            the shutdown machine
	 * @param askBeforeShutdown
	 *            the ask before shutdown
	 * @param addKeyListener
	 *            the add key listener
	 * @param icon
	 *            the icon
	 * @param serverDown
	 *            the server down
	 */

	public void manageBlockScreenOnError(String keyToMessage,
			boolean translatedAllSupported, boolean shutdownMachine,
			boolean askBeforeShutdown, boolean addKeyListener, Icon icon,
			boolean serverDown) {

		if (serverDown) {
			this.serverDown = true;
			blockScreenOnError(keyToMessage, translatedAllSupported,
					shutdownMachine, askBeforeShutdown, addKeyListener, icon);
		}

		if (!this.serverDown) {
			blockScreenOnError(keyToMessage, translatedAllSupported,
					shutdownMachine, askBeforeShutdown, addKeyListener, icon);
		}
	}

	/**
	 * Block screen on error.
	 * 
	 * @param keyToMessage
	 *            the key to message
	 * @param translatedAllSupported
	 *            the translated all supported
	 * @param shutdownMachine
	 *            the shutdown machine
	 * @param askBeforeShutdown
	 *            the ask before shutdown
	 * @param addKeyListener
	 *            the add key listener
	 * @param icon
	 *            the icon
	 */
	private void blockScreenOnError(String keyToMessage,
			boolean translatedAllSupported, boolean shutdownMachine,
			boolean askBeforeShutdown, boolean addKeyListener, Icon icon) {
		this.resourceMap = LanguageController.getInstance().getResourceMap();
		String message = null;
		int index = 0;

		if (this.blockedDialog.isVisible()) {
			this.blockedDialog.setVisible(false);
			ECMApp.this.blockedDialog = new BlockingMessageDialog(
					GeneralMenuDialog.getInstance(), true);
			ECMApp.this.blockedDialog.setAlwaysOnTop(true);
			ECMApp.this.blockedDialog
					.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);
		}

		if (translatedAllSupported) {
			Collection<LanguageData> languages = ECMApp.this.languageCollection
					.getLanguages();
			message = "<html><br><p>";
			for (LanguageData indexlanguage : languages) {
				message += this.resourceMap.getString(indexlanguage
						.getLanguageRFC() + "." + keyToMessage)
						+ (shutdownMachine ? (this.resourceMap
								.getString(indexlanguage.getLanguageRFC() + "."
										+ "ServerMonitorSplash.shutdown.text"))
								: "")
						+ ((++index < languages.size()) ? "<br>" : "");
			}
			message += "</p></html>";
		} else {
			message = this.resourceMap.getString(keyToMessage);
		}

		ShutDownMachineActionListener shutdown = new ShutDownMachineActionListener(
				GeneralMenuDialog.getInstance(), askBeforeShutdown);

		InfoPanel disable = new InfoPanel();
		disable.setIcon(icon);
		disable.setText(message);

		this.blockedDialog.setMessagesPanel(disable);

		if (addKeyListener) {
			this.blockedDialog.setKeyListener(new BaseKeyListener());
		}

		this.blockedDialog.setVisible(true);
		if (shutdownMachine) {
			shutdown.process();
		}
	}

	/**
	 * Shows the disconnected screen for the given resource type.
	 * 
	 * @param type
	 *            the type of resource to show its screen
	 * @param messageKey
	 *            the message key to translate and include in the dialog.
	 */
	public void showResourceDisconnectedScreen(MonitoringResourceType type,
			String messageKey) {
		switch (type) {
		case APPLICATION_SERVER:
			setUpDisconnectedScreen(ECMApp.this.serverDisconnectedDialog,
					messageKey);
			ECMApp.this.serverDisconnectedDialog.setVisible(true);
			break;
		case DATABASE:
			setUpDisconnectedScreen(ECMApp.this.dbDisconnectedDialog,
					messageKey);
			ECMApp.this.dbDisconnectedDialog.setVisible(true);
			break;
		}
	}

	/**
	 * Set the required dialog components, if a message key is provided this
	 * should be included.
	 * 
	 * @param dialog
	 *            the dialog to set up
	 * @param messageKey
	 *            the message key to fetch its translations.
	 */
	private void setUpDisconnectedScreen(BlockingMessageDialog dialog,
			String messageKey) {

		this.resourceMap = LanguageController.getInstance().getResourceMap();
		int index = 0;
		Collection<LanguageData> languages = ECMApp.this.languageCollection
				.getLanguages();

		String message = "<html><br><p>";
		for (LanguageData indexlanguage : languages) {
			message += this.resourceMap.getString(indexlanguage
					.getLanguageRFC() + "." + messageKey)
					+ ((++index < languages.size()) ? "<br>" : "");
		}
		message += "</p></html>";

		InfoPanel infoPanel = new InfoPanel();
		infoPanel.setIcon(this.resourceMap.getIcon("loadingLabel.icon"));
		infoPanel.setText(message);

		dialog.setMessagesPanel(infoPanel);
	}

	/**
	 * Method used to hide the instance of blocking window.
	 */
	public void hideResourceDisconnectedScreen(MonitoringResourceType type) {

		switch (type) {

		case APPLICATION_SERVER:
			ECMApp.this.serverDisconnectedDialog.setVisible(false);
			break;
		case DATABASE:
			ECMApp.this.dbDisconnectedDialog.setVisible(false);
			break;
		}

	}

	/**
	 * At startup create and show the main frame of the application.
	 * 
	 * @see org.jdesktop.application.Application#startup()
	 */

	public class MainThread extends Thread {

		/**
		 * Execute startup.
		 */
		private void executeStartup() {
			try {

				final SAESLocalPersistenceManager persistenceManager = SAESLocalPersistenceManager
						.getInstance();

				SMTTLogger
						.configureLoggingEnvironment(SAESLocalPersistenceManager
								.getInstance().getDefaultDestinations().logsConfigData
								.getAbsolutePath());

				SAESCommonDestinations destinations = persistenceManager
						.getDefaultDestinations();

				SAESLocalPersistenceManager.getInstance().processAnnotations(
						LanguageCollectionData.class);

				List<LanguageData> languages = new ArrayList<LanguageData>();

				LanguageData frenchLanguage = new LanguageData();

				LanguageData dutchLanguage = new LanguageData();

				frenchLanguage.setCode(1);
				frenchLanguage.setCustomCode("FR");
				frenchLanguage.setName("fr");
				frenchLanguage.setDefaultLanguage(true);
				frenchLanguage.setPrecedence(0);
				frenchLanguage.setLanguageRFC("FR");
				frenchLanguage.setCountry("BE");

				languages.add(frenchLanguage);

				dutchLanguage.setCode(2);
				dutchLanguage.setCustomCode("NL");
				dutchLanguage.setName("nl");
				dutchLanguage.setDefaultLanguage(true);
				dutchLanguage.setPrecedence(1);
				dutchLanguage.setLanguageRFC("NL");
				dutchLanguage.setCountry("BE");

				languages.add(dutchLanguage);

				ECMApp.this.languageCollection = new LanguageCollectionData("",
						"", languages);

				SAESLocalPersistenceManager.getInstance().processAnnotations(
						ConfigurationEntity.class);
				ECMApp.this.configurationEntity = (ConfigurationEntity) persistenceManager
						.loadFromXML(destinations.configurationEntityFile);

				SAESLocalPersistenceManager.getInstance().omitField(
						BaseDataObject.class, "version");

				try {
					startEnvironmentManager();
				} catch (SAESLocalPersistenceException e) {
					logger.error(e, StringUtils.defaultString(e.getMessage()));
					throw e;
				}

				ECMApp.this.langController = LanguageController.getInstance();
				ECMApp.this.langController.initApplicationContext(Application
						.getInstance(ECMApp.this.getClass()).getContext());

				if (ECMApp.this.languageCollection == null) {
					ECMApp.this.languageCollection = (LanguageCollectionData) persistenceManager
							.loadFromXML(destinations.defaultLanguagesFile);
				}

				Icon icon = LanguageController.getInstance().getResourceMap()
						.getIcon("loadingLabel.icon");
				InfoPanel pannel = new InfoPanel();
				pannel.setIcon(icon);
				pannel.setText(null);
				ECMApp.this.bgDialog = new BackgroundInitScreen(pannel);

				if (ECMClientSideConstants.USE_BATTERY_WATCHER) {

					ECMApp.this.configureBatteryAlertTimer();
					ECMApp.this.configureBatteryWatcher();
				}

				ECMApp.this.restartUI();

			} catch (Exception e) {

				logger.error(e, "There was an error starting the application: "
						+ StringUtils.defaultString(e.getMessage()));

				ECMApp.getApplication().manageBlockScreenOnError(
						"fatal.error.stating.app",
						true,
						true,
						false,
						true,
						LanguageController.getInstance().getResourceMap()
								.getIcon("ServerMonitorSplash.error.icon"),
						true);
			}
		}

		@Override
		public void run() {
			executeStartup();
		}
	}

	/**
	 * Restarts or resets the application's GUI back to the language selection
	 * dialog.
	 */
	public void restartUI() {
		new Thread(new Runnable() {

			@Override
			public void run() {

				try {

					ECMApp.logger.info("Initializing Language Controller");

					ECMApp.this.blockedDialog = new BlockingMessageDialog(
							GeneralMenuDialog.getInstance(), true);
					ECMApp.this.blockedDialog.setAlwaysOnTop(true);
					ECMApp.this.blockedDialog.setResizable(false);
					ECMApp.this.blockedDialog
							.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);

					ECMApp.this.serverDisconnectedDialog = new BlockingMessageDialog(
							GeneralMenuDialog.getInstance(), true);
					ECMApp.this.serverDisconnectedDialog.setAlwaysOnTop(true);
					ECMApp.this.serverDisconnectedDialog.setResizable(false);
					ECMApp.this.serverDisconnectedDialog
							.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);

					ECMApp.this.dbDisconnectedDialog = new BlockingMessageDialog(
							GeneralMenuDialog.getInstance(), true);
					ECMApp.this.dbDisconnectedDialog.setAlwaysOnTop(true);
					ECMApp.this.dbDisconnectedDialog.setResizable(false);
					ECMApp.this.dbDisconnectedDialog
							.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);

					Thread tbg = new Thread(ECMApp.this.bgDialog);
					tbg.start();

					ECMApp.this.bgDialog.showScreen();

					hide(getMainView());

					UserData user = identifyUser();

					logger.info("Initializing Monitor manager thread");

					String databaseIp = null;
					String serverIp = null;
					int dataBasePort = 0;
					int serverPort = 0;

					databaseIp = ECMClientSideConstants.MASTER_DB_IP_ADDRESS;
					dataBasePort = ECMClientSideConstants.MASTER_DB_PORT;

					serverIp = ECMClientSideConstants.SERVER_IP_ADDRESS;
					serverPort = ECMClientSideConstants.SERVER_PORT;

					ECMApp.this.serverDown = false;

					initMonitoringManager(databaseIp, dataBasePort, serverIp,
							serverPort);

					ECMApp.this.bgDialog.showScreen();

					ProfileData loggedProfile = user.getProfile();
					final HashSet<FunctionalityData> functionalities = new HashSet<FunctionalityData>();

					FunctionalityData functionality;
					Set<FunctionalityData> functionalitiesSet = user
							.getProfile().getFunctionalities();
					Iterator<FunctionalityData> itor = functionalitiesSet
							.iterator();

					boolean userIsAdmin = false;
					while (itor.hasNext()) {
						FunctionalityData f = itor.next();
						if (!f.getFunctionalityName().equals("ADMIN")) {
							functionality = f;
							functionalities.add(functionality);

						} else {
							userIsAdmin = true;
						}
					}

					loggedProfile.setAdminProfile(userIsAdmin);

					showChangeDateDialog();

					ECMApp.this.menuController = configMenu(functionalities,
							userIsAdmin);

					logger.info("Initializing Menu Controller");

					ECMApp.this.menuController.getMenuDialog()
							.setActionListenerCloseButton(
									new CloseButtonActionListener(ECMApp.this));

					ECMApp.this.menuController.setDefaultProfile(loggedProfile,
							ApplicationDefinitionEnum.ECM);

					Thread t = new Thread(new BackgroundInitMenu(
							ECMApp.this.menuController, loggedProfile));
					t.start();

					ECMApp.this.bgDialog.scheduledHideScreen();

				} catch (Exception e) {

					logger.error(e,
							"There was an error starting the application: "
									+ StringUtils.defaultString(e.getMessage()));

					ECMApp.getApplication().manageBlockScreenOnError(
							"fatal.error.stating.app",
							true,
							true,
							false,
							true,
							LanguageController.getInstance().getResourceMap()
									.getIcon("ServerMonitorSplash.error.icon"),
							true);

				}
			}
		}).start();

	}

	/**
	 * Starts user identification process.
	 * 
	 * @return Logged User
	 * @throws BaseException
	 */
	private UserData identifyUser() throws BaseException {

		PassphraseUserInputDialog.resetWrongPasswordCounter();
		UserData user = null;

		do {
			LanguageData selectedLanguage = selectLanguage();

			if (!ECMServiceProvider.isAdminUserConfigured()) {
				boolean adminUpdated = updateAdminPassword(selectedLanguage);
				if (!adminUpdated) {
					continue;
				}
			}
			user = loginUser(selectedLanguage);

		} while (user == null);
		return user;
	}

	/**
	 * Calls Login dialog.
	 * 
	 * @param language
	 * @return Logged User.
	 */
	private UserData loginUser(LanguageData language) {

		UserData userData = null;

		logger.info("Initializing Passphrase Input");
		PassphraseUserInputDialog passphraseInputDialog = new PassphraseUserInputDialog(
				language);
		PassphraseUserInputDialog.logout();
		passphraseInputDialog.setVisible(true);

		if (!passphraseInputDialog.canceled) {
			userData = passphraseInputDialog.getUser();
		}
		return userData;
	}

	/**
	 * Calls update password dialog.
	 * 
	 * @param language
	 * @return user updated
	 */
	private boolean updateAdminPassword(LanguageData language) {

		PassphraseUserUpdateDialog passphraseUpdate = new PassphraseUserUpdateDialog(
				language);
		passphraseUpdate.setVisible(true);

		return passphraseUpdate.isPasswordUpdated();
	}

	/**
	 * Selects language of the user experience.
	 * 
	 * @return LanguageData
	 * @throws Exception
	 */
	private LanguageData selectLanguage() {

		ResourceMap resourceMap = ECMApp.this.langController
				.getApplicationContext().getResourceMap(getClass());
		Font font = resourceMap.getFont("Application.defaultFont");
		FontUIResource fontResource = new FontUIResource(font);
		setUIFont(fontResource);

		Collection<LanguageData> languages = ECMApp.this.languageCollection
				.getLanguages();
		ECMApp.this.langController.initLangSelection(languages);
		ECMApp.this.langController.startLangSelection();
		return ECMApp.this.langController.getSelectedLanguage();

	}

	/**
	 * Reset the GUI in order to show the new allowed functionalities.
	 */
	public void restartTabs() {

		new Thread(new Runnable() {

			@Override
			public void run() {
				try {

					Thread tbg = new Thread(ECMApp.this.bgDialog);
					tbg.start();
					ECMApp.this.bgDialog.showScreen();

					ECMApp.logger.info("Initializing Language Controller");

					hide(getMainView());

					UserData user = ECMServiceProvider.getActualUser();

					ProfileData loggedProfile = user.getProfile();
					final HashSet<FunctionalityData> functionalities = new HashSet<FunctionalityData>();

					FunctionalityData functionality;
					Set<FunctionalityData> functionalitiesSet = user
							.getProfile().getFunctionalities();
					Iterator<FunctionalityData> itor = functionalitiesSet
							.iterator();

					boolean userIsAdmin = false;
					while (itor.hasNext()) {
						FunctionalityData f = itor.next();
						if (!f.getFunctionalityName().equals("ADMIN")) {
							functionality = f;
							functionalities.add(functionality);

						} else {
							userIsAdmin = true;
						}
					}

					loggedProfile.setAdminProfile(userIsAdmin);
					ECMApp.this.menuController = configMenu(functionalities,
							userIsAdmin);

					ECMApp.logger.info("Initializing Menu Controller");

					ECMApp.this.menuController.getMenuDialog()
							.setActionListenerCloseButton(
									new CloseButtonActionListener(ECMApp.this));

					ECMApp.this.menuController.setDefaultProfile(loggedProfile,
							ApplicationDefinitionEnum.ECM);

					Thread t = new Thread(new BackgroundInitMenu(
							ECMApp.this.menuController, loggedProfile));
					t.start();

					ECMApp.this.bgDialog.scheduledHideScreen();
				} catch (Exception e) {
					logger.error(e,
							"There was an error starting the application: "
									+ StringUtils.defaultString(e.getMessage()));

					ECMApp.getApplication().manageBlockScreenOnError(
							"fatal.error.stating.app",
							true,
							true,
							false,
							true,
							LanguageController.getInstance().getResourceMap()
									.getIcon("ServerMonitorSplash.error.icon"),
							true);
				}

			}
		}).start();
	}

	/**
	 * Initializes the application's general menu dialog controller according to
	 * a set of functionalities associated with the user. If the user is and
	 * administrator then all of the application's functionalities are
	 * available, otherwise only the specified ones will be.
	 * 
	 * @param functionalities
	 *            Set of functionalities accessible to the user through the
	 *            general menu dialog controller
	 * @param admin
	 *            Boolean that indicates if the user is an administrator. If
	 *            true then the user is an administrator and has access to all
	 *            of the application's functionalities.
	 * @return The general menu dialog controller of the application with the
	 *         set of functionalities specified according to the user's profile
	 * @throws MenuConfigurationFileException
	 *             the menu configuration file exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws TimeoutException
	 *             the timeout exception
	 */
	private GeneralMenuDialogController configMenu(
			HashSet<FunctionalityData> functionalities, boolean admin)
			throws Exception, IOException, TimeoutException {

		Iterator<FunctionalityData> itor = functionalities.iterator();

		GeneralMenuDialogController menuController = GeneralMenuDialogController
				.getInstance();

		menuController.setApp(ApplicationDefinitionEnum.ECM);

		/**
		 * Dispatches than their panels contain region widget navigator and need
		 * be notified when the database is deleted to refresh the region
		 * widget.
		 */

		/**
		 * Client list to invoke the notification.
		 */
		List<IRefreshRegionWidget> clients = new ArrayList<IRefreshRegionWidget>();

		/**
		 * Dispatch who receive the client list to notified when delete database
		 * button is invoked.
		 */
		if (functionalities.isEmpty() == false) {
			while (itor.hasNext()) {
				FunctionalityData functionality = itor.next();
				if (functionality == null) {
					break;
				}
				if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.FILE_CONVERTION) {

					ECMFileConversionDispatch eCMFileConversionDispatch = new ECMFileConversionDispatch();

					menuController.registerFuncImpl(
							FunctionalityDefinitionEnum.FILE_CONVERTION,
							eCMFileConversionDispatch);

				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.KEY_GENERATION) {

					menuController.registerFuncImpl(
							FunctionalityDefinitionEnum.KEY_GENERATION,
							new ECMKeyGenerationDispatch());

				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.IMAGES_UPLOAD) {

					menuController.registerFuncImpl(
							FunctionalityDefinitionEnum.IMAGES_UPLOAD,
							new ECMImageUploadDispatch());

				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.DECRYPT) {
					menuController.registerFuncImpl(
							FunctionalityDefinitionEnum.DECRYPT,
							new ECMDecryptDispatch());

				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.REPORT_GENERATION) {

					if (ECMClientSideConstants.AVAILABLE_TOTALISATION_REPORT
							|| ECMClientSideConstants.AVAILABLE_VOTING_DEVICE_REPORT) {
						ECMReportGenerationDispatch eCMReportGenerationDispatch = new ECMReportGenerationDispatch();

						clients.add(eCMReportGenerationDispatch);

						menuController.registerFuncImpl(
								FunctionalityDefinitionEnum.REPORT_GENERATION,
								eCMReportGenerationDispatch);
					}

				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.SCREENSHOT_GENERATION) {

					ECMScreenshotGenerationDispatch eCMScreenshotGenerationDispatch = new ECMScreenshotGenerationDispatch();

					clients.add(eCMScreenshotGenerationDispatch);

					menuController.registerFuncImpl(
							FunctionalityDefinitionEnum.SCREENSHOT_GENERATION,
							eCMScreenshotGenerationDispatch);
				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.PARAMETER_CONFIGURATION) {

					menuController
							.registerFuncImpl(
									FunctionalityDefinitionEnum.PARAMETER_CONFIGURATION,
									new ECMParameterConfigurationDispatch());

				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.MASTER_USB_MERGED) {

					ECMMasterUsbMergedDispatch eCMMasterUsbMergedDispatch = new ECMMasterUsbMergedDispatch();

					clients.add(eCMMasterUsbMergedDispatch);

					menuController.registerFuncImpl(
							FunctionalityDefinitionEnum.MASTER_USB_MERGED,
							eCMMasterUsbMergedDispatch);
				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.ELIGIBLE_OPTION_EDITOR) {

					ECMEligibleOptionEditorDispatch eCMEligibleOptionEditorDispatch = new ECMEligibleOptionEditorDispatch();

					clients.add(eCMEligibleOptionEditorDispatch);

					menuController.registerFuncImpl(
							FunctionalityDefinitionEnum.ELIGIBLE_OPTION_EDITOR,
							eCMEligibleOptionEditorDispatch);
				} else if (functionality.getFunctionalityDefinition() == FunctionalityDefinitionEnum.INTEGRITY_CHECK) {
					ECMIntegrityCheckDispatch ecmIntegrityCheckDispatch = new ECMIntegrityCheckDispatch();

					menuController.registerFuncImpl(
							FunctionalityDefinitionEnum.INTEGRITY_CHECK,
							ecmIntegrityCheckDispatch);
				}
			}
		}

		menuController.loadMenuConfiguration(
				this.configurationEntity.getMenu(), this.getMainFrame());

		GeneralMenuDialog menuDialog = menuController.getMenuDialog();

		buildWorkflowButtons(menuDialog);

		if (ECMClientSideConstants.USE_PRINTER_WATCHER) {

			PrinterWatcher printerWatcher = PrinterWatcher.getInstance();

			notifiesPrinterStatusChanged(printerWatcher.getCurrentStatus());

			printerWatcher.addListenerClient(this);
		}

		if (ECMClientSideConstants.USE_BATTERY_WATCHER) {

			configureBatteryAlertTimer();
			configureBatteryWatcher();
		}

		return menuController;
	}

	/**
	 * Configures battery alert timer.
	 */
	private void configureBatteryAlertTimer() {

		this.timerListener = new TimerListener(this.batteryStatusDetailLabel,
				ECMClientSideConstants.PLAY_BATTERY_ALERT_BEEP);

		if (this.timer == null) {
			int time = 500;
			this.timer = new Timer(time, this.timerListener);
			this.timer.setInitialDelay(time);
			this.timer.setRepeats(true);
		} else {
			ActionListener[] listeners = this.timer.getActionListeners();
			for (int i = 0; i < listeners.length; i++) {
				this.timer.removeActionListener(listeners[i]);
			}
			this.timer.addActionListener(this.timerListener);
		}
	}

	private void configureBatteryWatcher() throws IOException, TimeoutException {

		BatteryWatcher batteryWatcher = BatteryWatcher.getInstance();
		batteryWatcher.addListenerClient(ECMApp.this);

		BatteryState batteryState = batteryWatcher.getBatteryState();
		int powerPercent = batteryWatcher.getPowerPercent();

		setBatteryState(batteryState, powerPercent);
	}

	/***
	 * {@inheritDoc} <br>
	 * This implementation gets the current battery status and updates it on the
	 * window.
	 */
	@Override
	public void batteryStatusChanged(Map<String, Object> changed) {

		try {
			logger.debug(changed.toString());

			BatteryState batteryState = null;

			int powerPercent = 0;

			if (changed.containsKey(IBatteryStatusQuery.KEY_BATTERY_STATE)) {
				batteryState = (BatteryState) changed
						.get(IBatteryStatusQuery.KEY_BATTERY_STATE);
			} else {
				batteryState = BatteryWatcher.getInstance().getBatteryState();
			}

			if (changed.containsKey("powerPercentage")) {
				powerPercent = ((Number) changed.get("powerPercentage"))
						.intValue();
			} else {
				powerPercent = BatteryWatcher.getInstance().getPowerPercent();
			}

			setBatteryState(batteryState, powerPercent);

		} catch (Exception e) {
			logger.error(e,
					"There is an error getting the current battery status: "
							+ StringUtils.defaultString(e.getMessage()));
		}
	}

	/***
	 * method used to show the information about battery status.
	 * 
	 * @param batteryState
	 *            - The current battery status to show.
	 * @param powerPercent
	 *            - The percent (%) of battery charge to show.
	 */
	private void setBatteryState(BatteryState batteryState, int powerPercent) {
		if (powerPercent != 0 && powerPercent != 100) {
			if (powerPercent <= ECMClientSideConstants.CRITICAL_POWER_PERCENT_VALUE
					&& batteryState.equals(BatteryState.DISCHARGING)) {
				this.timerListener.setLabelText(String.valueOf(powerPercent));
				if (!this.timer.isRunning()) {
					this.timer.start();
				}
			} else if (powerPercent <= ECMClientSideConstants.CRITICAL_POWER_PERCENT_VALUE
					&& batteryState.equals(BatteryState.CHARGING)) {
				if (this.timer.isRunning()) {
					this.timer.stop();
				}
				setBatteryPercentage(String.valueOf(powerPercent), Color.RED);
			} else if (powerPercent > ECMClientSideConstants.CRITICAL_POWER_PERCENT_VALUE) {
				if (this.timer.isRunning()) {
					this.timer.stop();
				}
				setBatteryPercentage(String.valueOf(powerPercent), Color.GREEN);
			}
		} else if (batteryState != null) {
			if (batteryState.equals(BatteryState.FULLY_CHARGED)
					&& powerPercent == 100) {
				setBatteryStatus(BatteryState.FULLY_CHARGED.toString(),
						Color.GREEN);
			} else if (batteryState.equals(BatteryState.DISCONNECTED)) {
				setBatteryStatus(BatteryState.DISCONNECTED.toString(),
						Color.RED);
			} else if (batteryState.equals(BatteryState.DISCHARGING)) {
				setBatteryPercentage(String.valueOf(powerPercent), Color.GREEN);
			}

		}
	}

	/**
	 * * Set the message to show about the battery status.
	 * 
	 * @param status
	 *            - the key of property file for the message to show.
	 * @param color
	 *            - The color in which will be show the message.
	 */
	private void setBatteryStatus(String status, Color color) {

		if (this.batteryStatusDetailLabel != null) {
			ResourceBundle bundle = java.util.ResourceBundle
					.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");
			this.batteryStatusDetailLabel.setForeground(color);
			this.batteryStatusDetailLabel.setText(bundle
					.getString("battery.status." + status));
		}

	}

	/**
	 * * Set the message to show about the battery charge.
	 * 
	 * @param percentage
	 *            -the percentage of battery charge
	 * @param color
	 *            - The color in which will be show the message.
	 */
	private void setBatteryPercentage(String percentage, Color color) {
		if (this.batteryStatusDetailLabel != null) {
			this.batteryStatusDetailLabel.setForeground(color);
			this.batteryStatusDetailLabel.setText(percentage + "%");
		}
	}

	/***
	 * {@inheritDoc} <br>
	 * This implementation gets the current printer status and updates it on the
	 * window.
	 */
	@Override
	public void notifiesPrinterStatusChanged(String status) {
		if (status.equals(StatusPrinter.READY.toString())
				|| status.equals(StatusPrinter.CONNECTED.toString())) {
			this.setPrinterStatus(status, Color.GREEN);
		} else if (status.equals(StatusPrinter.BUSY.toString())
				|| status.equals(StatusPrinter.CANCELING.toString())) {
			this.setPrinterStatus(status, Color.YELLOW);
		} else {
			this.setPrinterStatus(status, Color.RED);
		}
	}

	/**
	 * * Set the message to show about the printer status.
	 * 
	 * @param status
	 *            -The current printer status to show.
	 * @param color
	 *            - The color in which will be show the message.
	 */
	private void setPrinterStatus(String status, Color color) {
		ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");

		this.printerStatusDetailLabel.setForeground(color);

		this.printerStatusDetailLabel.setText(bundle
				.getString("printer.status." + status));
	}

	/**
	 * Builds the workflow buttons.
	 * 
	 * @param menuDialog
	 *            the menu dialog
	 */
	private void buildWorkflowButtons(GeneralMenuDialog menuDialog) {

		this.actualStepName = ECMServiceProvider.getStepName();

		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N

		JPanel buttons = new JPanel();

		JPanel labels = new JPanel();

		JPanel statusBar = new JPanel();

		this.buttonExportLogs = new JButton();
		this.buttonExportLogs.setText(bundle
				.getString("GeneralMenu.export.button.text")); // NOI18N
		this.buttonExportLogs.setName("buttonExportLogs"); // NOI18N
		this.buttonExportLogs
				.addActionListener(new java.awt.event.ActionListener() {

					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						exportLogs(evt);
					}
				});

		this.fileChooserExportLogs = new JFileChooser();
		this.fileChooserExportLogs.setName("fileChooserExportLogs");

		ClientCommon.initFileChooserLocation(this.fileChooserExportLogs);

		this.printerStatusLabel = new JLabel();
		this.printerStatusLabel.setName("labelPrinterStatus");

		this.batteryStatusLabel = new JLabel();
		this.batteryStatusLabel.setName("labelBatteryStatus");

		this.batteryStatusDetailLabel = new JLabel();
		this.batteryStatusDetailLabel.setName("labelBatteryStatusDetail");

		this.printerStatusDetailLabel = new JLabel();
		this.printerStatusDetailLabel.setName("labelPrinterStatusDetail");

		if (ECMClientSideConstants.USE_PRINTER_WATCHER) {
			this.printerStatusLabel.setText(bundle
					.getString("printer.status.label"));
		}

		if (ECMClientSideConstants.USE_BATTERY_WATCHER) {
			this.batteryStatusLabel.setText(bundle
					.getString("battery.status.label"));
		}

		FlowLayout labelLayout = buildVFlowLayaut(FlowLayout.CENTER, 10);

		labels.setLayout(labelLayout);

		labels.add(this.buttonExportLogs);

		if (ECMClientSideConstants.USE_PRINTER_WATCHER
				&& ECMClientSideConstants.USE_BATTERY_WATCHER) {
			JLabel lbSquareBracketOpen = new JLabel("[");
			labels.add(lbSquareBracketOpen);
		}

		labels.add(this.printerStatusLabel);

		labels.add(this.printerStatusDetailLabel);

		if (ECMClientSideConstants.USE_PRINTER_WATCHER
				&& ECMClientSideConstants.USE_BATTERY_WATCHER) {
			JLabel lbPipe = new JLabel("|");
			labels.add(lbPipe);
		}

		labels.add(this.batteryStatusLabel);

		labels.add(this.batteryStatusDetailLabel);

		if (ECMClientSideConstants.USE_PRINTER_WATCHER
				&& ECMClientSideConstants.USE_BATTERY_WATCHER) {
			JLabel lbSquareBracketClose = new JLabel("]");
			labels.add(lbSquareBracketClose);
		}

		FlowLayout buttonsLayout = buildHFlowLayaut(FlowLayout.RIGHT, 10);

		buttons.setLayout(buttonsLayout);

		this.workflowBackButton = new JButton();
		this.workflowBackButton.setName("buttonWorkflowBack");
		this.workflowBackButton.setText(bundle.getString("btnBack.text"));
		this.workflowBackButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				ECMApp.getApplication().goBack();

			}

		});
		buttons.add(this.workflowBackButton);
		this.workflowNextButton = new JButton();
		this.workflowNextButton.setName("buttonWorkflowNext");
		this.workflowNextButton.setText(bundle.getString("btnNext.text"));
		this.workflowNextButton.addMouseListener(new MouseListener() {

			@Override
			public void mouseReleased(MouseEvent e) {
				return;
			}

			@Override
			public void mousePressed(MouseEvent e) {
				return;
			}

			@Override
			public void mouseExited(MouseEvent e) {
				return;
			}

			@Override
			public void mouseEntered(MouseEvent e) {
				return;
			}

			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 1) {
					if (ECMApp.this.workflowNextButton.isEnabled()) {
						ECMApp.this.workflowNextButton.setEnabled(false);
						ECMApp.getApplication().gotoNext();
					}
				}
			}
		});
		buttons.add(this.workflowNextButton);

		this.btnBackEnabled = true;
		this.btnNextEnabled = true;

		if (this.actualStepName.equals("BulkLoad")) {
			this.workflowBackButton.setEnabled(false);
			this.btnBackEnabled = false;
		} else if (this.actualStepName.equals("USBGeneration")) {
			this.workflowNextButton.setEnabled(false);
			this.btnNextEnabled = false;
		}

		statusBar.setLayout(new BorderLayout());

		statusBar.add(labels, BorderLayout.WEST);
		statusBar.add(buttons, BorderLayout.EAST);

		menuDialog.add(statusBar, BorderLayout.SOUTH);

	}

	/**
	 * Builds the h flow layaut.
	 * 
	 * @param aligment
	 * @param hGap
	 * @return the flow layout
	 */
	private FlowLayout buildHFlowLayaut(int aligment, int hGap) {
		FlowLayout layout = new FlowLayout();
		layout.setAlignment(aligment);
		layout.setHgap(hGap);
		return layout;
	}

	/**
	 * Builds the v flow layaut.
	 * 
	 * @param aligment
	 * @param hGap
	 * @return the flow layout
	 */
	private FlowLayout buildVFlowLayaut(int aligment, int hGap) {
		FlowLayout layout = new FlowLayout();
		layout.setAlignment(aligment);
		layout.setVgap(hGap);
		return layout;
	}

	/**
	 * Initialize the specified window by injecting resources. Windows shown in
	 * our application come fully initialized from the GUI builder, so this
	 * additional configuration is not needed.
	 * 
	 * @param root
	 *            Window to be initialized.
	 */
	@Override
	protected void configureWindow(java.awt.Window root) {
		return;
	}

	/***
	 * A convenient static getter for the application instance.
	 * 
	 * @return the instance of ECMApp
	 */
	public static ECMApp getApplication() {
		return Application.getInstance(ECMApp.class);
	}

	/**
	 * Set font by default in entire application.
	 * 
	 * @param f
	 *            Font to be set as default.
	 */
	public void setUIFont(javax.swing.plaf.FontUIResource f) {
		Enumeration<?> keys = UIManager.getDefaults().keys();
		while (keys.hasMoreElements()) {
			Object key = keys.nextElement();
			Object value = UIManager.get(key);
			if (value instanceof javax.swing.plaf.FontUIResource) {
				UIManager.put(key, f);
			}
		}
	}

	/**
	 * Initializes the environment manager of the application.
	 * 
	 * @throws HistoricFileException
	 *             the historic file exception
	 * @throws SAESLocalPersistenceException
	 *             the sAES local persistence exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public void startEnvironmentManager() throws HistoricFileException,
			SAESLocalPersistenceException, IOException {
		logger.info("Starting the environment manager");

		EnvironmentManager.initialize();
		this.environmentManager = EnvironmentManager.getInstance();

		logger.info("Checking environment");
		this.environmentManager.checkEnvironment();
	}

	/**
	 * Method to set tab focus.
	 * 
	 * @param func
	 *            Functionality to set focus
	 */
	public void focusTab(FunctionalityDefinitionEnum func) {
		this.menuController.getMenuDialog().focusTab(func);
	}

	/**
	 * Sets all the panel tabs other than the one related to an specific
	 * functionality as enabled or disabled.
	 * 
	 * @param funcs
	 *            the funcs
	 * @param block
	 *            Status of the other panel tabs. <code>true</code> blocks all
	 *            the other tabs, <code>false</code> unblocks them.
	 * @author Mario Bislick (mario.bislick@smartmatic.com)
	 */
	public void blockOtherTabsByFunctionality(
			HashSet<FunctionalityDefinitionEnum> funcs, boolean block) {
		if (this.menuController != null) {
			boolean blockTabs = true;
			blockTabs = Boolean
					.valueOf(ECMClientSideConstants.BLOCK_PANEL_TABS);
			if (block) {
				setWorkflowButtonsEnabled(false);
				this.functionalitiesBlockingTabsCount++;
				if (this.functionalitiesBlockingTabsCount == 1) {
					if (blockTabs) {
						this.menuController.getMenuDialog()
								.blockOtherTabsByFunctionality(funcs, true);

						this.tabsBlocked = true;
					}
				}

			} else {
				setWorkflowButtonsEnabled(true);

				if (this.functionalitiesBlockingTabsCount > 0) {
					this.functionalitiesBlockingTabsCount--;
				}

				if (this.functionalitiesBlockingTabsCount == 0) {
					if (blockTabs) {
						this.menuController.getMenuDialog()
								.blockOtherTabsByFunctionality(funcs, false);

						this.tabsBlocked = false;
					}
				} else {
					logger.debug("blockExitCount = "
							+ this.functionalitiesBlockingTabsCount);
				}
			}
		}
	}

	/**
	 * Enable or disable the workflow buttons.
	 * 
	 * @param newStatus
	 *            true to enable and false to disable
	 */
	public void setWorkflowButtonsEnabled(boolean newStatus) {

		if (newStatus) {
			if (this.functionalitiesBlockingWorkflowButtonsCount > 0) {
				this.functionalitiesBlockingWorkflowButtonsCount--;
			}
			if (this.functionalitiesBlockingWorkflowButtonsCount == 0) {

				if (this.btnBackEnabled) {
					this.workflowBackButton.setEnabled(true);
				}

				if (this.btnNextEnabled) {
					this.workflowNextButton.setEnabled(true);
				}

				this.buttonExportLogs.setEnabled(true);

				this.menuController.getMenuDialog()
						.setActionListenerCloseButton(
								new CloseButtonActionListener(this));
				this.menuController.getMenuDialog().blockExitTab(true);
			}

		} else {
			this.functionalitiesBlockingWorkflowButtonsCount++;
			if (this.functionalitiesBlockingWorkflowButtonsCount == 1) {

				this.workflowBackButton.setEnabled(false);
				this.workflowNextButton.setEnabled(false);
				this.buttonExportLogs.setEnabled(false);

				this.menuController.getMenuDialog()
						.setActionListenerCloseButton(null);
				this.menuController.getMenuDialog().blockExitTab(false);
			}
		}
	}

	/***
	 * Method used to move on the workflow to the next step.
	 */
	public void gotoNext() {
		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		if (this.tabsBlocked == false) {

			ECMWorkflowStatus prevStatus = (ECMWorkflowStatus) ECMServiceProvider
					.getWorkflowVariable("status");
			if (prevStatus != null) {
				Validation[] clientSideValidations = prevStatus
						.getClientSideValidations();

				boolean valid = true;

				for (ECMWorkflowStatus.Validation validation : clientSideValidations) {

					switch (validation) {
					case OS_IMAGES_LOADED:
						valid = Validations.validateImagesLoaded();
						break;

					default:
						break;
					}

					if (valid == false) {

						ECMOptionPane.showMessageDialog(
								this.menuController.getMenuDialog(),
								bundle.getString("validation."
										+ validation.name()),
								ECMOptionPane.INFORMATION_DIALOG);

						enableWorkflowButton();

						return;
					}
				}

				if (valid == false) {
					return;
				}
			}
			ECMWorkflowStatus postStatus = ECMServiceProvider.gotoNext();
			final Validation[] validationsFailed = postStatus
					.getValidationsFailed();

			if (validationsFailed.length == 0) {

				hide(getMainView());

				executePostProcessing(prevStatus);

				ECMServiceProvider.moveForward();
			} else {

				StringBuilder message = new StringBuilder();

				for (Validation validation : validationsFailed) {

					message.append(bundle.getString("validation."
							+ validation.name()));
					message.append(SystemUtils.LINE_SEPARATOR);

				}
				ECMOptionPane.showMessageDialog(
						this.menuController.getMenuDialog(),
						message.toString(), ECMOptionPane.INFORMATION_DIALOG);

				enableWorkflowButton();
			}
		} else {

			ECMServiceProvider.moveForward();

			ECMOptionPane.showMessageDialog(
					this.menuController.getMenuDialog(),
					bundle.getString("workflow.inprogress"),
					ECMOptionPane.INFORMATION_DIALOG);
		}
	}

	/**
	 * Enables workflow buttons after show an information message.
	 */
	private void enableWorkflowButton() {
		if (!this.actualStepName.equals("BulkLoad")) {
			this.workflowBackButton.setEnabled(true);
			this.btnBackEnabled = true;
		}
		if (!this.actualStepName.equals("USBGeneration")) {
			this.workflowNextButton.setEnabled(true);
			this.btnNextEnabled = true;
		}

	}

	/***
	 * Method used to execute the postload process.
	 * 
	 * @param status
	 *            the current workflow status.
	 */
	private void executePostProcessing(final ECMWorkflowStatus status) {

		new Thread(new Runnable() {

			@Override
			public void run() {
				try {
					for (ECMWorkflowStatus.PostProcessing postProcessing : status
							.getClientSidePostProcessing()) {
						switch (postProcessing) {
						case GENERATE_LOGICAL_VOTING_GROUP:

							ResourceBundle bundle = java.util.ResourceBundle
									.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");

							InfoPanel dialog = (InfoPanel) ECMApp.this.bgDialog
									.getpannel();

							dialog.setText(
									"\n"
											+ bundle.getString("POSTLOADPROCESS.message.executing"),
									32);

							PostProcessing.generateLogicalVotingGroup();
							PostProcessing.fillLanguageCombinations();
							PostProcessing.downloadReports();
							PostProcessing.setDefaultApplicationKey();

							dialog.setText(StringUtils.EMPTY, 32);

							break;

						default:
							break;
						}
					}
				} catch (Exception e) {
					GeneralMenuDialog menuDialog = GeneralMenuDialogController
							.getInstance().getMenuDialog();

					ResourceBundle bundle = java.util.ResourceBundle
							.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");

					ECMOptionPane.showMessageDialog(menuDialog,
							bundle.getString("ecm.general.unexpected.error"),
							bundle.getString("ErrorMessage.title"),
							ECMOptionPane.ERROR_MESSAGE,
							ECMOptionPane.ERROR_DIALOG);

					ECMServiceProvider.goBack();
				}
				restartTabs();
			}
		}).start();
	}

	/***
	 * Method used to move on the workflow to the last step.
	 */
	public void goBack() {
		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N

		if (this.tabsBlocked == false) {
			Boolean showWarning = (Boolean) ECMServiceProvider
					.getWorkflowVariable("showWarning");

			if (showWarning != null && showWarning == true) {
				String stepName = ECMServiceProvider.getStepName();

				if (ECMOptionPane.showConfirmDialog(
						this.menuController.getMenuDialog(),
						bundle.getString(stepName + ".confirm"),
						bundle.getString("question.title.text"),
						ECMOptionPane.YES_NO_OPTION,
						ECMOptionPane.CONFIRM_DIALOG) == ECMOptionPane.YES_OPTION) {

					performRollback();

					ECMServiceProvider.goBack();

					ECMServiceProvider.moveBackward();

					restartTabs();
				}
			} else {
				performRollback();

				ECMServiceProvider.goBack();

				ECMServiceProvider.moveBackward();

				restartTabs();
			}

		} else {
			ECMOptionPane.showMessageDialog(
					this.menuController.getMenuDialog(),
					bundle.getString("workflow.inprogress"),
					ECMOptionPane.INFORMATION_DIALOG);
		}
	}

	/***
	 * Method used to execute a rollback process in the workflow.
	 */
	private void performRollback() {
		ECMWorkflowStatus status = (ECMWorkflowStatus) ECMServiceProvider
				.getWorkflowVariable("status");
		if (status != null) {
			com.smartmatic.waffle.ecm.service.workflow.ECMWorkflowStatus.Rollback[] clientSideRollback = status
					.getClientSideRollback();
			for (ECMWorkflowStatus.Rollback rollback : clientSideRollback) {
				switch (rollback) {
				case DELETE_OS_IMAGES:
					com.smartmatic.waffle.ecm.client.Rollback.deleteImages();
					break;
				default:
					break;
				}
			}
		}
	}

	/**
	 * Show change date dialog.
	 */
	private void showChangeDateDialog() {
		try {
			if (ECMClientSideConstants.CHANGE_DATE_MANDATORY
					&& !Boolean.getBoolean("test_mode")
					&& !Boolean.getBoolean("development_mode")) {

				Integer datetimeSet = ECMServiceProvider
						.getIntParameter(ECMClientSideConstants.PARAMETER_KEY_DATETIME_SET);

				if (datetimeSet == 0) {
					ECMApp.this.dialogToSettingTimeAndDate = new SettingDateDialog(
							GeneralMenuDialog.getInstance(), true);
					ECMApp.this.dialogToSettingTimeAndDate.setAlwaysOnTop(true);
					ECMApp.this.dialogToSettingTimeAndDate
							.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);
					ECMApp.this.dialogToSettingTimeAndDate.setVisible(true);
				} else {
					SettingDateDialog.syncServerToClientDateTime();
				}
			}
		} catch (Exception e) {
			logger.error(e, "There was an error setting date and time: "
					+ StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Export logs.
	 * 
	 * @param evt
	 */
	private void exportLogs(ActionEvent evt) {
		try {
			ResourceMap resourceMap = LanguageController.getInstance()
					.getResourceMap();

			this.fileChooserExportLogs
					.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

			if (this.fileChooserExportLogs.showSaveDialog(this.menuController
					.getMenuDialog()) == 0) {

				ClientCommon
						.saveFileChooserLocationPreference(this.fileChooserExportLogs
								.getSelectedFile().getAbsolutePath());

				File fileSelected = this.fileChooserExportLogs
						.getSelectedFile();

				if (fileSelected != null) {

					if (fileSelected.canWrite()) {

						logger.info("Downloading log files from server");

						Map<String, byte[]> logFiles = ECMServiceProvider
								.getLogFiles();

						if (logFiles == null) {
							ECMOptionPane
									.showMessageDialog(
											this.menuController.getMenuDialog(),
											resourceMap
													.getString("GeneralMenu.message.error.export.log.size.do.not.allowed"),
											resourceMap
													.getString("ErrorMessage.title"),
											ECMOptionPane.ERROR_MESSAGE,
											ECMOptionPane.ERROR_DIALOG);

							return;
						} else if (!logFiles.isEmpty()) {

							File serverLogFileFolder = new File(fileSelected,
									"ServerLog");

							if (!serverLogFileFolder.exists()) {
								serverLogFileFolder.mkdirs();
							} else {
								FileUtils.forceDelete(serverLogFileFolder);
							}

							Iterator<String> keys = logFiles.keySet()
									.iterator();
							while (keys.hasNext()) {
								String codiFileName = keys.next();
								File codiFile = new File(serverLogFileFolder,
										codiFileName);
								byte[] logFileByte = logFiles.get(codiFileName);
								FileUtils.writeByteArrayToFile(codiFile,
										logFileByte);
							}

							logger.info("Server log files downloaded successfully");
						} else {
							logger.warn("Any log file was found in server");
						}

						logger.info("Downloading log files from client");

						File clientLogFileFolder = new File(fileSelected,
								"ClientLog");

						if (!clientLogFileFolder.exists()) {
							clientLogFileFolder.mkdirs();
						} else {
							FileUtils.forceDelete(clientLogFileFolder);
						}

						File clientLogFilesDir = new File(
								ECMClientSideConstants.CLIENT_SIDE_LOG_FILES_PATH);

						if (clientLogFilesDir.exists()) {
							File[] clientLogFiles = clientLogFilesDir
									.listFiles();

							long maxSizeAllowed = ECMClientSideConstants.MAX_SIZE_TO_EXPORT_LOG;

							for (File logFile : clientLogFiles) {

								long fileLength = (logFile.length() / 1024) / 1024;

								if (fileLength > maxSizeAllowed) {

									logger.error(logFile.getName()
											+ " exceeds the maximum size ("
											+ maxSizeAllowed + " MB) allowed");

									ECMOptionPane
											.showMessageDialog(
													this.menuController
															.getMenuDialog(),
													resourceMap
															.getString("GeneralMenu.message.error.export.log.size.do.not.allowed"),
													resourceMap
															.getString("ErrorMessage.title"),
													ECMOptionPane.ERROR_MESSAGE,
													ECMOptionPane.ERROR_DIALOG);

									return;
								}

								FileUtils
										.copyFile(logFile,
												new File(clientLogFileFolder,
														logFile.getName()));
							}
							PendriveUtil.sync();
							logger.info("Client log files downloaded successfully");
						} else {
							logger.warn("Any log file was found in client");
						}
					} else {
						logger.error("permission denied");
						ECMOptionPane
								.showMessageDialog(
										this.menuController.getMenuDialog(),
										resourceMap
												.getString("GeneralMenu.message.error.export.log.permission.denied"),
										resourceMap
												.getString("ErrorMessage.title"),
										ECMOptionPane.ERROR_MESSAGE,
										ECMOptionPane.ERROR_DIALOG);
						return;
					}

					ECMOptionPane
							.showMessageDialog(
									this.menuController.getMenuDialog(),
									resourceMap
											.getString("GeneralMenu.message.error.export.log.successfully"),
									resourceMap
											.getString("NormalMessage.title"),
									ECMOptionPane.INFORMATION_MESSAGE,
									ECMOptionPane.INFORMATION_DIALOG);
				}
			}

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			ECMOptionPane.showMessageDialog(
					this.menuController.getMenuDialog(), this.resourceMap
							.getString("GeneralMenu.message.error.export.log"),
					this.resourceMap.getString("ErrorMessage.title"),
					ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
		}
	}
}