package com.smartmatic.waffle.ecm.client.lang;

import com.smartmatic.waffle.ecm.core.lang.ECMBaseException;
import com.smartmatic.waffle.ecm.core.lang.IECMExceptionInfo;

public class FileGenerationException extends ECMBaseException {

	public enum FileGenerationExceptionInfo implements IECMExceptionInfo {

		WRITING_SIGNATURE_FILE_FAILURE("writing.signature.file.failure",
				"There was a failure writing signature"), WRITING_CERTIFICATES_CHAIN_FAILURE(
				"writing.certificates.chain.failure",
				"There was a failure writing certificates chain."), WRITING_CERTIFICATE_FILE_FAILURE(
				"writing.certificate.file.failure",
				"There was an failure writing certificate file"), WRITING_XML_FILE_FAILURE(
				"writing.xml.file.failure",
				"There was an error writin XML file"), WRITING_DATABASE_DUMP_FAILURE(
				"writing.database.dump.failure",
				"There was an error writing the database dump."), WRITING_CIPHERED_FILE_FAILURE(
				"writing.ciphered.file.failure",
				"There was an error writing the ciphered file"), WRITING_VOICE_FILE_FAILURE(
				"writing.voice.file.failure",
				"There was an error writing the voice files");

		/**
		 * Variable to hold the exception message.
		 */
		private String exceptionMessage;

		/**
		 * Variable to hold the bundle key to show a user understandable
		 * message.
		 */
		private String bundleKey;

		/**
		 * Gets the variable to hold the exception message.
		 * 
		 * @return the variable to hold the exception message
		 * @inheritDoc
		 */
		@Override
		public String getExceptionMessage() {
			return this.exceptionMessage;
		}

		/**
		 * Gets the variable to hold the bundle key to show a user
		 * understandable message.
		 * 
		 * @return the variable to hold the bundle key to show a user
		 *         understandable message
		 * @inheritDoc
		 */
		@Override
		public String getBundleKey() {
			return this.bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 */
		private FileGenerationExceptionInfo(String bundleKey) {
			this.bundleKey = bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 * @param exceptionMessage
		 *            Exception message
		 */
		private FileGenerationExceptionInfo(String bundleKey,
				String exceptionMessage) {
			this.bundleKey = bundleKey;
			this.exceptionMessage = exceptionMessage;
		}
	}

	/**
	 * Instantiates an FileGenerationException.
	 * 
	 * @param throwable
	 *            a nested exception
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public FileGenerationException(Throwable throwable,
			FileGenerationExceptionInfo exceptionInfo) {
		super(throwable, exceptionInfo);
	}

	/**
	 * Instantiates an FileGenerationException.
	 * 
	 * @param throwable
	 *            a nested exception
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param exceptionMessage
	 *            Exception message of nested exception
	 */
	public FileGenerationException(Throwable throwable,
			FileGenerationExceptionInfo exceptionInfo, String exceptionMessage) {
		super(throwable, exceptionInfo, exceptionMessage);
	}

	/**
	 * Instantiates an FileGenerationException.
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public FileGenerationException(FileGenerationExceptionInfo exceptionInfo) {
		super(exceptionInfo);
	}

	/*
	 * @inheritDoc
	 */
	@Override
	public String getBundleKey() {
		return this.getClass().getSimpleName() + "."
				+ this.exceptionInfo.getBundleKey();
	}
}
