/**
 * ContestBrowserHierarchyProvider.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.providers;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyDTO;
import com.smartmatic.waffle.ecm.ui.controls.Browser.IBrowserHierarchyProvider;
import com.smartmatic.waffle.lang.BaseException;

/**
 * Class that provide the data source for the hierarchy widget.<br>
 * The data is about the contest and contest class
 */
public class ContestBrowserHierarchyProvider implements
		IBrowserHierarchyProvider {

	/***
	 * Pre-loaded map of contest objects by contest class custom code.
	 */
	private Map<String, List<ContestData>> contestMapByContestClassCustomCode;

	/***
	 * Pre-loaded contest class list.
	 */
	private List<ContestClassData> contestClassList;

	/***
	 * Class constructor.
	 * 
	 * @param contestMapByContestClassCustomCode
	 *            -map of contest objects by contest class custom code
	 * @param contestClassList
	 *            -contest class list
	 */
	public ContestBrowserHierarchyProvider(
			Map<String, List<ContestData>> contestMapByContestClassCustomCode,
			List<ContestClassData> contestClassList) {
		this.contestMapByContestClassCustomCode = contestMapByContestClassCustomCode;
		this.contestClassList = contestClassList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ui.controls.Browser.IBrowserHierarchyProvider
	 * #initialLoad()
	 */
	@Override
	public Set<BrowserHierarchyDTO> initialLoad() throws BaseException {
		Set<BrowserHierarchyDTO> list = new HashSet<BrowserHierarchyDTO>();
		BrowserHierarchyDTO dtoAux;

		for (ContestClassData current : this.contestClassList) {
			dtoAux = new BrowserHierarchyDTO();
			dtoAux.setId(current);
			dtoAux.setTextToShow(current.getContestClassNameTranslate());
			list.add(dtoAux);
		}
		return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ui.controls.Browser.IBrowserHierarchyProvider
	 * #nextLevel
	 * (com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyDTO)
	 */
	@Override
	public Set<BrowserHierarchyDTO> nextLevel(
			BrowserHierarchyDTO selectedElement) throws BaseException {
		Set<BrowserHierarchyDTO> list = new HashSet<BrowserHierarchyDTO>();
		if (selectedElement.getId() instanceof ContestClassData) {
			ContestClassData selectedContestClass = (ContestClassData) selectedElement
					.getId();
			List<ContestData> contestList = this.contestMapByContestClassCustomCode
					.get(selectedContestClass.getCustomCode());

			BrowserHierarchyDTO dtoAux;
			for (ContestData current : contestList) {
				dtoAux = new BrowserHierarchyDTO();
				dtoAux.setId(current);
				dtoAux.setTextToShow(current.getContestNameTranslate());
				list.add(dtoAux);
			}
			return list;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ui.controls.Browser.IBrowserHierarchyProvider
	 * #previousLevel
	 * (com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyDTO)
	 */
	@Override
	public Set<BrowserHierarchyDTO> previousLevel(
			BrowserHierarchyDTO selectedElement) throws BaseException {
		return null;
	}

}
