/**
 * BulkUploadCodiFilesPanel.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import java.awt.Color;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateParsingException;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.WindowConstants;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.security.FunctionalityDefinitionEnum;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.ui.BulkUploadFileFilter.FilterType;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.client.util.SignatureException;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDetailDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.UploadedFileDetailDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.EMLFileType;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException.BulkDataLoadExceptionOperationExceptionInfo;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Window to bulk upload of EML files.
 */
public class BulkUploadEMLFilesPanel extends BulkUploadAbstractPanel {

	/**
	 * The object's serialization code.
	 */
	private static final long serialVersionUID = -6600721687869955624L;

	/**
	 * For log messages purpose.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(BulkUploadEMLFilesPanel.class);

	/**
	 * Filter for certificate file chooser.
	 */
	BulkUploadFileFilter certificateFilter;

	/**
	 * Filter for files to upload file chooser.
	 */
	BulkUploadFileFilter filesToUploadFilter;

	/**
	 * Resource file content.
	 */
	protected byte[] resourcesFileContent;

	private boolean lockByEMLError = false;

	/**
	 * Instantiates a new BulkUploadEMLFilesPanel.
	 */
	public BulkUploadEMLFilesPanel() {
		initComponents();

		this.certificateFilter = new BulkUploadFileFilter(
				this.resourceBundle
						.getString("Application.certificate_filter_desc"),
				FilterType.VALIDATION_CERTIFICATE);

		this.filesToUploadFilter = new BulkUploadFileFilter(
				this.resourceBundle
						.getString("Application.max_size_filter_desc"),
				FilterType.FILES_TO_UPLOAD);

		this.functionalityFileChooser
				.addChoosableFileFilter(this.certificateFilter);
		this.functionalityFileChooser
				.addChoosableFileFilter(this.filesToUploadFilter);

		loadFileTypesTranslations();

		loadFilesToUploadCertificate();
	}

	/**
	 * Loads the certificate to bulk upload data validation
	 */
	private void loadFilesToUploadCertificate() {
		ByteArrayInputStream byteArrayInputStream = null;
		try {
			if (this.defaultDestinations.filesToUploadValidationCertificateFile
					.exists()) {
				CertificateFactory certificateFactory = CertificateFactory
						.getInstance("X.509");
				byteArrayInputStream = new ByteArrayInputStream(
						FileUtils
								.readFileToByteArray(this.defaultDestinations.filesToUploadValidationCertificateFile));
				this.dataValidationCertificate = (X509Certificate) certificateFactory
						.generateCertificate(byteArrayInputStream);
				this.certificateStatusDetailLabel
						.setText(this.resourceBundle
								.getString("ImagesUploadPanel.status.votingdeviceimage.loaded"));
				this.certificateStatusDetailLabel.setForeground(Color.GREEN);
			} else {
				this.dataValidationCertificate = null;
				this.certificateStatusDetailLabel
						.setText(this.resourceBundle
								.getString("ImagesUploadPanel.status.votingdeviceimage.notloaded"));
				this.certificateStatusDetailLabel.setForeground(Color.RED);
				this.chooseFilesToUploadButton.setEnabled(false);
				this.deploySmttFilesUploadPanelButton.setEnabled(false);
				this.clearDataBaseButton.setEnabled(false);
			}

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new RuntimeException(e);
		} finally {
			if (byteArrayInputStream != null) {
				try {
					byteArrayInputStream.close();
				} catch (IOException e) {
					logger.error(e, StringUtils.defaultString(e.getMessage()));
				}
			}
		}
	}

	/**
	 * Loads the translations for the EML files type
	 */
	private void loadFileTypesTranslations() {
		EMLFileType[] filesTypes = EMLFileType.values();

		for (int i = 0; i < filesTypes.length; i++) {
			filesTypes[i]
					.setTranslatedName(this.resourceBundle
							.getString("com.smartmatic.waffle.ecm.data.transfer.type.EMLFileType."
									+ filesTypes[i].toString()));
		}
	}

	/**
	 * Update status in progress bar to show count processed files and current
	 * file to being processed.
	 * 
	 * @param count
	 *            - count processed files.
	 * @param nameFile
	 *            - file name being processed.
	 */
	public void refreshStatus(int count, String nameFile) {
		String message = String.valueOf(count) + "/"
				+ String.valueOf(this.countFiles) + "     (" + nameFile + ")";
		refreshStatus(message);
	}

	/**
	 * Updates the status label.
	 * 
	 * @param message
	 *            the new status
	 */
	public void refreshStatus(String message) {
		this.uploadStatusLabel.setText(message);
	}

	/**
	 * Blocks or unblocks the bulk upload dialog.
	 * 
	 * @param block
	 *            true to block otherwise false.
	 */
	public void preventCloseOfBulkUploadDialog(boolean block) {
		if (block) {
			this.smttFilesUploadDialog
					.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		} else {
			this.smttFilesUploadDialog
					.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		}
	}

	public void closeSmttFilesUploadPanel() {
		this.smttFilesUploadDialog.setVisible(false);
	}

	/**
	 * Shows the message error for unexpected errors.
	 * 
	 * @param emlFileToUploadDTO
	 *            file where the error occurred to report it
	 */
	public void showUnexpectedErrorMessageExecutingBulkUpload(
			FileToUploadDTO emlFileToUploadDTO) {

		String errorMessage = this.resourceBundle
				.getString("BulkUploadSmttFilesPanel.message.bulk.upload.error");

		errorMessage = StringUtils.replace(errorMessage, "${file_name}",
				emlFileToUploadDTO.getFileToUploadDetailDTO().getFileInfo()
						.getName());

		showErrorMessage(errorMessage);

		this.clearFilesToUploadTableButton.setEnabled(false);
		this.chooseFilesToUploadButton.setEnabled(false);
		this.clearDataBaseButton.setEnabled(true);
		this.uploadStatusLabel.setText(StringUtils.EMPTY);
		this.updateProgressBar(0);
		this.lockByEMLError = true;
	}

	/**
	 * Updates the GUI components and displays the result tree after bulk upload
	 * process finished.
	 * 
	 * @param fileUploadedDetailList
	 *            list with the result of bulk upload
	 */
	public void finishBulkUploadProcess(
			List<UploadedFileDetailDTO> fileUploadedDetailList) {

		this.panelProgressBar.setValue(this.panelProgressBar.getMaximum());

		if (fileUploadedDetailList != null) {

			if (showResult(this.uploadedFilesResultTree, this.rootTreeNode,
					this.paneWithFilesToUploadTable,
					this.paneWithFilesUploadedResult, fileUploadedDetailList)) {

				this.uploadStatusLabel.setText(this.resourceBundle
						.getString("BulkUploadSmttFilesPanel.finish.message"));

				showInfoMessage(this.resourceBundle
						.getString("BulkUploadSmttFilesPanel.message.success"));

				this.clearFilesToUploadTableButton.setEnabled(true);
				this.chooseFilesToUploadButton.setEnabled(true);
				this.clearDataBaseButton.setEnabled(true);
				this.panelProgressBar.setValue(0);
				this.uploadDataValidationCertificateButton.setEnabled(true);
				this.deploySmttFilesUploadPanelButton.setEnabled(true);
				ECMServiceProvider.setWorkflowButtonsEnabled(true);
			} else {

				ECMOptionPane
						.showMessageDialog(
								this,
								this.resourceBundle
										.getString("BulkUploadSmttFilesPanel.message.warning"),
								this.resourceBundle
										.getString("ErrorMessage.title"),
								ECMOptionPane.ERROR_MESSAGE,
								ECMOptionPane.ERROR_DIALOG);

				this.clearDataBaseButton.setEnabled(true);
				this.clearFilesToUploadTableButton.setEnabled(false);
				this.panelProgressBar.setValue(0);
				this.uploadStatusLabel.setText(StringUtils.EMPTY);
				this.uploadDataValidationCertificateButton.setEnabled(false);
				this.deploySmttFilesUploadPanelButton.setEnabled(false);
				this.lockByEMLError = true;
			}
		}

	}

	/**
	 * Sets the progress bar's current value
	 * 
	 * @param progress
	 *            value to update the progress bar
	 */
	@Override
	public void updateProgressBar(int progress) {
		this.panelProgressBar.setValue(progress);

	}

	/**
	 * Sets the progress bar's maximum value
	 * 
	 * @param progress
	 *            value to update the progress bar
	 */
	@Override
	void setProgressBarMaximumValue(int maxValue) {
		this.panelProgressBar.setMaximum(maxValue);
	}

	/**
	 * Method to update the status message with the actual file processing while
	 * bulk upload process for this file is executing
	 * 
	 * @param filesProcessed
	 *            files processed
	 * @param filesAmount
	 *            amount of files to process
	 * @param nameFile
	 *            string with the file name
	 */
	public void updateStatusDuringBulkUploadOperation(int filesProcessed,
			int filesAmount, String nameFile) {
		this.uploadStatusLabel.setText(String.valueOf(filesProcessed) + "/"
				+ String.valueOf(filesAmount) + "     (" + nameFile + ")");
	}

	/**
	 * Method to update the status message while database records removal
	 * process is executing
	 * 
	 * @param bundleKey
	 *            bundle key used to find the translation of the message
	 */
	@Override
	public void updateStatusMessage(String bundleKey) {
		if (bundleKey.equals(StringUtils.EMPTY)) {
			this.uploadStatusLabel.setText(StringUtils.EMPTY);
		} else {
			this.uploadStatusLabel.setText(this.resourceBundle
					.getString(bundleKey));
		}
	}

	/**
	 * Method called by some controllers to show error or information messages.
	 * 
	 * @param bundleKey
	 *            bundle key to find the translation message
	 * @param messageType
	 *            type of message to show
	 */
	@Override
	void showMessage(String bundleKey, int messageType) {
		if (messageType == ECMOptionPane.ERROR_MESSAGE) {
			showErrorMessage(this.resourceBundle.getString(bundleKey));
		} else if (messageType == ECMOptionPane.INFORMATION_MESSAGE) {
			showInfoMessage(this.resourceBundle
					.getString("BulkUploadSmttFilesPanel.generate.translations.message.success"));
		}
	}

	/**
	 * Updates the GUI components after database records removal process
	 * finished.
	 * 
	 * @param someErrorHasOccurred
	 *            true if some error occurred in the process to show different
	 *            component for each case
	 */
	@Override
	void finishDatabaseClearingProcess(boolean someErrorHasOccurred) {
		this.panelProgressBar.setValue(0);

		this.uploadStatusLabel.setText(StringUtils.EMPTY);

		this.deploySmttFilesUploadPanelButton.setEnabled(true);

		this.clearDataBaseButton.setEnabled(true);

		this.chooseFilesToUploadButton.setEnabled(true);

		this.uploadDataValidationCertificateButton.setEnabled(true);

		resetFormToSelectOtherFiles();

		ECMServiceProvider.setWorkflowButtonsEnabled(true);
	}

	/**
	 * Prepare GUI components for bulk upload execution
	 */
	private void startBulkUploadProcess() {

		this.clearFilesToUploadTableButton.setEnabled(false);
		this.chooseFilesToUploadButton.setEnabled(false);
		this.uploadFilesButton.setEnabled(false);
		this.clearDataBaseButton.setEnabled(false);
		this.filesToUploadTable.setEnabled(false);
		this.uploadDataValidationCertificateButton.setEnabled(false);
		this.deploySmttFilesUploadPanelButton.setEnabled(false);

		BulkUploadEMLFilesController bulkUploadSmttFilesController = new BulkUploadEMLFilesController(
				this, this.emlFilesToUpĺoadList, this.resourcesFileContent);

		bulkUploadSmttFilesController.startWorker();

		this.emlFilesToUpĺoadList = null;
		this.resourcesFileContent = null;

		ECMServiceProvider.setWorkflowButtonsEnabled(false);
	}

	/**
	 * Reads each file header to determine which file type it is
	 * 
	 * @param fileToRead
	 *            file to read
	 * 
	 * @return the file type corresponding with the file
	 * 
	 * @throws BulkDataLoadException
	 *             if it is an invalid file type or some error occurs opening
	 *             the file of which to determine the type
	 */
	private EMLFileType getEMLFileType(File fileToRead)
			throws BulkDataLoadException {
		try {
			String line = FileUtils.readLines(fileToRead).iterator().next();

			if ((line != null) && (!line.isEmpty())) {
				String[] columns = line.trim().split(";");
				if (columns.length > 1) {
					String firstColumn = columns[1].replace("\"", "");
					if (!"S".equalsIgnoreCase(firstColumn)) {
						return EMLFileType.SITES;
					}
				}
			}

			return EMLFileType.ELECTION;

		} catch (Exception e) {
			this.fileWithProblems = fileToRead;
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_GETTING_SMTT_FILE_TYPE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Fill the map with the valid files to bulk upload.
	 * 
	 * @param selectedFiles
	 *            files selected by the user
	 */
	private void startValidatingFilesToUpload(List<File> selectedFiles) {
		try {

			this.emlFilesToUpĺoadList = new ArrayList<FileToUploadDTO>();

			DefaultTableModel defaultTableModel = (DefaultTableModel) this.filesToUploadTable
					.getModel();

			boolean enableButtons = true;

			if (ECMClientSideConstants.VALIDATE_FILES_TO_UPLOAD_SIGNATURE) {
				validateSignature(selectedFiles);
			}

			for (File fileToUpload : selectedFiles) {

				EMLFileType fileType = getEMLFileType(fileToUpload);

				FileToUploadDTO fileToUploadDTO = new FileToUploadDTO();
				FileToUploadDetailDTO fileToUploadDetailDTO = new FileToUploadDetailDTO();

				fileToUploadDetailDTO.setFileContent(FileUtils
						.readFileToByteArray(fileToUpload));
				fileToUploadDetailDTO.setFileInfo(fileToUpload);

				fileToUploadDTO.setEMLFileType(fileType);
				fileToUploadDTO.setFileToUploadDetailDTO(fileToUploadDetailDTO);

				this.emlFilesToUpĺoadList.add(fileToUploadDTO);

				defaultTableModel
						.addRow(new Object[] {
								fileType.getTranslatedName(),
								fileToUpload.getName(),
								this.resourceBundle
										.getString("BulkUploadSmttFilesPanel.jTable2.valid.file") });

			}

			this.countFiles = this.emlFilesToUpĺoadList.size();

			if (this.countFiles == 0) {
				enableButtons = false;
				showErrorMessage(this.resourceBundle
						.getString("BulkUploadSmttFilesPanel.error.at.least.one.smartmatic.file.is.required"));
				cleanFilesToUploadTableActionPerformed();
			}

			if (enableButtons) {
				this.uploadFilesButton.setEnabled(true);
				this.clearFilesToUploadTableButton.setEnabled(true);
				this.panelProgressBar.setMaximum(this.countFiles);
			} else {
				this.uploadFilesButton.setEnabled(false);
				this.clearFilesToUploadTableButton.setEnabled(false);
			}

		} catch (BulkDataLoadException e) {
			logger.error(e, "Error during data bulkload preparation");

			String bundle = "BulkUploadSmttFilesPanel." + e.getBundleKey()
					+ ".message";

			String message = this.resourceBundle.getString(bundle);

			if (this.fileWithProblems != null) {
				message = StringUtils.replace(message, "${file_name}",
						this.fileWithProblems.getName());
			}

			ECMServiceProvider.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);
			showErrorMessage(message);
			cleanFilesToUploadTableActionPerformed();
		} catch (SignatureException e) {
			logger.error(e,
					"Error validating signature during data bulkload preparation");

			String message = this.resourceBundle
					.getString("BulkUploadCodiFilesPanel.error.validating.signature");

			ECMServiceProvider.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);
			showErrorMessage(message);
			cleanFilesToUploadTableActionPerformed();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			ECMServiceProvider.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);
			showErrorMessage(this.resourceBundle
					.getString("BulkUploadSmttFilesPanel.error.UNEXPECTED_PROBLEM"));
		}
	}

	/**
	 * Remove items from files to upload table.
	 */
	private void clearFilesToUploadTable() {
		if (this.filesToUploadTable != null) {
			DefaultTableModel tableModelTmp = (DefaultTableModel) this.filesToUploadTable
					.getModel();
			tableModelTmp.getDataVector().removeAllElements();
			this.filesToUploadTable.repaint();
		}
	}

	/**
	 * Clear current items from files to upload table.
	 */
	private void resetFormToSelectOtherFiles() {
		clearFilesToUploadTable();
		this.paneWithFilesUploadedResult.setVisible(false);
		this.paneWithFilesToUploadTable.setVisible(true);
		this.uploadStatusLabel.setText(StringUtils.EMPTY);
		this.panelProgressBar.setValue(0);
		updateUI();
		repaint();
	}

	private File chooseCertificate() {

		File certFile = null;

		ClientCommon.initFileChooserLocation(this.functionalityFileChooser);
		this.functionalityFileChooser.setFileFilter(this.certificateFilter);

		this.functionalityFileChooser.setMultiSelectionEnabled(false);

		int option = this.functionalityFileChooser.showOpenDialog(this);

		if (option == JFileChooser.APPROVE_OPTION) {
			certFile = this.functionalityFileChooser.getSelectedFile();
			ClientCommon
					.saveFileChooserLocationPreference(this.functionalityFileChooser
							.getCurrentDirectory().getAbsolutePath());
		}

		return certFile;
	}

	// ///////////// Buttons Actions ///////////////

	/**
	 * Method triggered on click event performed by the user to upload the
	 * certificate to validate the data.
	 */
	private void uploadDataValidationCertificateActionPerformed() {

		int result = ECMOptionPane.YES_OPTION;

		boolean certificateExist = this.defaultDestinations.filesToUploadValidationCertificateFile
				.exists();

		if (certificateExist) {
			result = showOverwriteMessage();
		}

		if (result == ECMOptionPane.YES_OPTION) {
			File certFile = chooseCertificate();

			if (certFile != null) {
				try {
					byte[] certificateContent = validateLoadedCertificate(
							certificateExist, certFile);

					if (certificateContent != null) {

						StringBuilder stringBuilder = buildCertificateValidationMessage();

						ECMServiceProvider
								.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);

						result = ECMOptionPane.showConfirmDialog(this,
								stringBuilder.toString(), this.resourceBundle
										.getString("WarningMessage.title"),
								ECMOptionPane.YES_NO_OPTION,
								ECMOptionPane.QUESTION_MESSAGE,
								ECMOptionPane.CONFIRM_DIALOG);

						if (result == ECMOptionPane.YES_OPTION) {
							copyCertificateAndUpdatePanel(certificateContent);
						}
					}

				} catch (CertificateParsingException e) {
					logger.error(e, StringUtils.defaultString(e.getMessage()));
					this.dataValidationCertificate = null;
					ECMServiceProvider
							.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);
					showCertificateParsingError(e, certificateExist);

					loadFilesToUploadCertificate();
				} catch (Exception e) {
					logger.error(e, StringUtils.defaultString(e.getMessage()));
					this.dataValidationCertificate = null;

					ECMServiceProvider
							.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);
					showErrorMessage(this.resourceBundle
							.getString("BulkUploadCodiFilesPanel.error.opening.cert"));

					loadFilesToUploadCertificate();
				}
			}
		}
	}

	private void showCertificateParsingError(CertificateParsingException e,
			boolean certificateExist) {

		if (StringUtils.contains(e.getMessage(),
				"invalid DER-encoded certificate data")) {

			String errorMessage = this.resourceBundle
					.getString("BulkUploadCodiFilesPanel.error.invalid.cert.info");

			if (certificateExist) {
				errorMessage = errorMessage
						+ " "
						+ this.resourceBundle
								.getString("BulkUploadCodiFilesPanel.certificate.wont.be.replaced");
			}
			showErrorMessage(errorMessage);
		}
	}

	/**
	 * Copy certificate and update the panel information.
	 * 
	 * @param certificateContent
	 *            the certificate content to copy to file system
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public void copyCertificateAndUpdatePanel(byte[] certificateContent)
			throws IOException {
		SAESCommonDestinations defaultDestinations = SAESLocalPersistenceManager
				.getInstance().getDefaultDestinations();

		FileUtils.writeByteArrayToFile(
				defaultDestinations.filesToUploadValidationCertificateFile,
				certificateContent);

		this.certificateStatusDetailLabel
				.setText(this.resourceBundle
						.getString("ImagesUploadPanel.status.votingdeviceimage.loaded"));

		this.certificateStatusDetailLabel.setForeground(Color.GREEN);

		this.chooseFilesToUploadButton.setEnabled(true);

		this.clearDataBaseButton.setEnabled(true);

		this.deploySmttFilesUploadPanelButton.setEnabled(true);
	}

	/**
	 * Validates the loaded certificate.
	 * 
	 * @param certificateExists
	 *            indicates if the certificate was previously loaded to include
	 *            an overwrite warning message
	 * @param certFile
	 *            the certificate reference to validate
	 * 
	 * @return the content of the certificate or null if is invalid
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws CertificateException
	 *             CertificateException on parsing errors.
	 */
	public byte[] validateLoadedCertificate(boolean certificateExists,
			File certFile) throws IOException, CertificateException {

		byte[] certificateContent = FileUtils.readFileToByteArray(certFile);
		InputStream byteArrayInputStream = new ByteArrayInputStream(
				certificateContent);
		CertificateFactory certificateFactory = CertificateFactory
				.getInstance("X.509");

		try {
			this.dataValidationCertificate = (X509Certificate) certificateFactory
					.generateCertificate(byteArrayInputStream);
		} finally {
			byteArrayInputStream.close();
		}
		Date openDate = this.dataValidationCertificate.getNotBefore();
		Date endDate = this.dataValidationCertificate.getNotAfter();
		Date now = Calendar.getInstance().getTime();

		if (ECMClientSideConstants.VALIDATE_CERTIFICATE_EXPIRED_DATE) {
			if (endDate.compareTo(now) < 0) {

				String errorMessage = this.resourceBundle
						.getString("KeyGenerationPanel.certificate.expired.msg.text");

				if (certificateExists) {
					errorMessage = errorMessage
							+ " "
							+ this.resourceBundle
									.getString("BulkUploadCodiFilesPanel.certificate.wont.be.replaced");
				}

				showErrorMessage(errorMessage);
				loadFilesToUploadCertificate();
				return null;
			}
		}

		if (ECMClientSideConstants.VALIDATE_CERTIFICATE_DATE) {
			if (openDate.compareTo(now) > 0) {

				String errorMessage = this.resourceBundle
						.getString("KeyGenerationPanel.certificate.validation.before.msg.text");

				if (certificateExists) {
					errorMessage = errorMessage
							+ " "
							+ this.resourceBundle
									.getString("BulkUploadCodiFilesPanel.certificate.wont.be.replaced");
				}
				ECMServiceProvider.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);
				showInfoMessage(errorMessage);
				loadFilesToUploadCertificate();
				return null;
			}
		}
		return certificateContent;
	}

	/**
	 * Builds the certificate validation message with all important information
	 * about it.
	 * 
	 * @return StringBuilder object with the message
	 */
	public StringBuilder buildCertificateValidationMessage() {
		SimpleDateFormat formatDate = new SimpleDateFormat(
				"dd/MM/yyyy HH:mm:ss");

		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(this.resourceBundle
				.getString("BulkUploadCodiFilesPanel.certificate.info.message")
				+ "\n" + "\n");
		stringBuilder
				.append(this.resourceBundle
						.getString("BulkUploadCodiFilesPanel.certificate.info.subject.name")
						+ this.dataValidationCertificate.getSubjectDN()
								.getName() + "\n");
		stringBuilder.append(this.resourceBundle
				.getString("BulkUploadCodiFilesPanel.certificate.info.issuer")
				+ this.dataValidationCertificate.getIssuerDN().toString()
				+ "\n");
		stringBuilder.append(this.resourceBundle
				.getString("BulkUploadCodiFilesPanel.certificate.info.type")
				+ this.dataValidationCertificate.getType() + "\n");

		stringBuilder.append(this.resourceBundle
				.getString("BulkUploadCodiFilesPanel.certificate.info.key")
				+ ClientCommon.addLineBreaks(this.dataValidationCertificate
						.getPublicKey().toString(), 100) + "\n");

		stringBuilder
				.append(this.resourceBundle
						.getString("BulkUploadCodiFilesPanel.certificate.info.algorithm")
						+ this.dataValidationCertificate.getSigAlgName() + "\n");
		stringBuilder
				.append(this.resourceBundle
						.getString("BulkUploadCodiFilesPanel.certificate.info.date.not.after")
						+ formatDate.format(this.dataValidationCertificate
								.getNotBefore()) + "\n");
		stringBuilder
				.append(this.resourceBundle
						.getString("BulkUploadCodiFilesPanel.certificate.info.date.not.before")
						+ formatDate.format(this.dataValidationCertificate
								.getNotAfter()) + "\n" + "\n");
		stringBuilder
				.append(this.resourceBundle
						.getString("BulkUploadCodiFilesPanel.certificate.info.confirmation.message"));
		return stringBuilder;
	}

	/**
	 * Shows a warning message if the certificate was previously loaded
	 * 
	 * @return the user selection
	 */
	public int showOverwriteMessage() {
		int result;
		ECMServiceProvider.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);
		result = ECMOptionPane.showConfirmDialog(this, this.resourceBundle
				.getString("BulkUploadCodiFilesPanel.message.overwriteImage"),
				this.resourceBundle.getString("WarningMessage.title"),
				ECMOptionPane.YES_NO_OPTION, ECMOptionPane.QUESTION_MESSAGE,
				ECMOptionPane.CONFIRM_DIALOG);
		return result;
	}

	/**
	 * Method triggered on click event performed by the user to open the panel
	 * to perform smartmatic bulk upload
	 */
	private void deploySmttFilesUploadPanelActionPerformed() {
		cleanFilesToUploadTableActionPerformed();
		this.bulkUploadSmttFilesPanel
				.setDataValidationCertificate(this.dataValidationCertificate);
		this.bulkUploadSmttFilesPanel.setVisible(true);
		if (this.smttFilesUploadDialog == null) {
			this.smttFilesUploadDialog = new JDialog();
			this.bulkUploadSmttFilesPanel
					.setName("panelStaticBulkDataLoadMain");
			this.bulkUploadSmttFilesPanel.setBulkUploadCodiFilesPanel(this);
		}
		this.smttFilesUploadDialog.setTitle(this.resourceBundle
				.getString("BulkUploadSmttFilesPanel.title.BulkUpload"));
		this.smttFilesUploadDialog.setName("dialogStaticBulkDataLoad");
		this.smttFilesUploadDialog.add(this.bulkUploadSmttFilesPanel);
		this.smttFilesUploadDialog.setSize(800, 600);
		this.smttFilesUploadDialog.setLocationRelativeTo(this);
		this.smttFilesUploadDialog.setResizable(false);
		this.smttFilesUploadDialog.setAlwaysOnTop(true);
		this.smttFilesUploadDialog.setModal(true);
		this.smttFilesUploadDialog.setVisible(true);
	}

	/***
	 * This method allows users to select a file to upload, and shows the
	 * selected files in the table of files.
	 */
	private void searchFilesToUploadActionPerformed() {

		if (this.bulkUploadSmttFilesPanel.isSmttDataIsLoaded()) {

			ClientCommon.initFileChooserLocation(this.functionalityFileChooser);
			this.functionalityFileChooser.setMultiSelectionEnabled(true);

			this.functionalityFileChooser
					.setFileFilter(this.filesToUploadFilter);

			if (this.functionalityFileChooser.showOpenDialog(this) == 0) {
				ClientCommon
						.saveFileChooserLocationPreference(this.functionalityFileChooser
								.getCurrentDirectory().getAbsolutePath());

				if (this.paneWithFilesUploadedResult.isVisible()) {
					resetFormToSelectOtherFiles();
					this.uploadedFilesResultTree.removeAll();
					this.uploadStatusLabel.setText(StringUtils.EMPTY);
				}

				List<File> selectedFileList = Arrays
						.asList(this.functionalityFileChooser
								.getSelectedFiles());

				if (selectedFileList != null) {
					startValidatingFilesToUpload(selectedFileList);
				}
			}
			this.functionalityInternalPanel.repaint();
		} else {
			ECMServiceProvider.focusTab(this.FILE_CONVERTION_FUNCTIONALITY);
			showInfoMessage(this.resourceBundle
					.getString("BulkUploadCodiFilesPanel.isNoAvalible.ToExecute"));
		}
	}

	/***
	 * Execute transformation process.
	 */
	private void executeBulkUploadActionPerformed() {

		int result = ECMOptionPane
				.showConfirmDialog(
						this,
						this.resourceBundle
								.getString("BulkUploadCodiFilesPanel.message.executeTansformation"),
						this.resourceBundle
								.getString("ConfirmationMessage.title"),
						ECMOptionPane.YES_NO_OPTION,
						ECMOptionPane.CONFIRM_DIALOG);

		if (result == 0) {
			startBulkUploadProcess();
		}
	}

	/**
	 * Method triggered on click event performed by the user to remove all
	 * smartmatic files' table records
	 */
	private void clearDataBaseActionPerformed() {

		int result = ECMOptionPane.showConfirmDialog(this, this.resourceBundle
				.getString("BulkUploadSmttFilesPanel.message.clean.database"),
				this.resourceBundle.getString("ConfirmationMessage.title"),
				ECMOptionPane.YES_NO_OPTION, ECMOptionPane.CONFIRM_DIALOG);

		if (result == 0) {
			this.panelProgressBar.setMaximum(100);
			this.panelProgressBar.setStringPainted(true);
			this.clearFilesToUploadTableButton.setEnabled(false);
			this.chooseFilesToUploadButton.setEnabled(false);
			this.uploadFilesButton.setEnabled(false);
			this.clearDataBaseButton.setEnabled(false);
			this.filesToUploadTable.setEnabled(false);
			this.uploadDataValidationCertificateButton.setEnabled(false);
			this.deploySmttFilesUploadPanelButton.setEnabled(false);

			if (this.lockByEMLError) {
				ECMServiceProvider.setWorkflowButtonsEnabled(true);
				this.lockByEMLError = false;
			}

			ECMServiceProvider.setWorkflowButtonsEnabled(false);

			startDatabaseClearingProcess(this, true);
		}
	}

	/**
	 * Removes the list of files to be processed from the table.
	 */
	private void cleanFilesToUploadTableActionPerformed() {
		resetFormToSelectOtherFiles();
		this.uploadFilesButton.setEnabled(false);
		this.clearFilesToUploadTableButton.setEnabled(false);
		this.chooseFilesToUploadButton.setEnabled(true);
		this.clearDataBaseButton.setEnabled(true);
		this.uploadStatusLabel.setText(StringUtils.EMPTY);
		this.emlFilesToUpĺoadList = null;
	}

	/**
	 * Shows error message.
	 * 
	 * @param message
	 *            I18n Parsed message
	 */
	private void showErrorMessage(String message) {
		ECMOptionPane.showMessageDialog(this, message,
				this.resourceBundle.getString("ErrorMessage.title"),
				ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
	}

	/**
	 * Shows informative message.
	 * 
	 * @param message
	 *            I18n Parsed message
	 */
	private void showInfoMessage(String message) {
		ECMOptionPane.showMessageDialog(this, message,
				this.resourceBundle.getString("NormalMessage.title"),
				ECMOptionPane.INFORMATION_MESSAGE,
				ECMOptionPane.INFORMATION_DIALOG);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {

		this.functionalityFileChooser = new javax.swing.JFileChooser();
		this.functionalityInfoLayeredPane = new javax.swing.JLayeredPane();
		this.functionalityIconLabel = new javax.swing.JLabel();
		this.functionalityTitleLabel = new javax.swing.JLabel();
		this.functionalityDescriptionLabel = new javax.swing.JLabel();
		this.functionalityBannerBackgroundLabel = new javax.swing.JLabel();
		this.functionalityExternalPanel = new javax.swing.JPanel();
		this.functionalityInternalPanel = new javax.swing.JPanel();
		this.paneWithFilesToUploadTable = new javax.swing.JScrollPane();
		this.filesToUploadTable = new javax.swing.JTable();
		this.clearFilesToUploadTableButton = new javax.swing.JButton();
		this.panelProgressBar = new javax.swing.JProgressBar();
		this.uploadFilesButton = new javax.swing.JButton();
		this.chooseFilesToUploadButton = new javax.swing.JButton();
		this.paneWithFilesUploadedResult = new javax.swing.JScrollPane();
		this.uploadedFilesResultTree = new javax.swing.JTree();
		this.clearDataBaseButton = new javax.swing.JButton();
		this.uploadStatusLabel = new javax.swing.JLabel();
		this.deploySmttFilesUploadPanelButton = new javax.swing.JButton();
		this.uploadDataValidationCertificateButton = new javax.swing.JButton();
		this.certificateStatusLabel = new javax.swing.JLabel();
		this.certificateStatusDetailLabel = new javax.swing.JLabel();

		this.functionalityFileChooser.setName("fileChooserFrom"); // NOI18N

		ClientCommon.initFileChooserLocation(this.functionalityFileChooser);

		this.functionalityInfoLayeredPane
				.setName("paneFunctionalityInfoLayered"); // NOI18N

		this.functionalityIconLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/carga_masiva.png"))); // NOI18N
		this.functionalityIconLabel.setName("labelFunctionalityIcon"); // NOI18N
		this.functionalityIconLabel.setBounds(10, 10, 120, 90);
		this.functionalityInfoLayeredPane.add(this.functionalityIconLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.functionalityTitleLabel.setFont(new java.awt.Font("DejaVu Sans",
				1, 14));
		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		this.functionalityTitleLabel
				.setText(bundle
						.getString("BulkUploadCodiFilesPanel.file_conversion_title.text")); // NOI18N
		this.functionalityTitleLabel.setName("labelFunctionalityTitle"); // NOI18N
		this.functionalityTitleLabel.setBounds(160, 40, 386, 17);
		this.functionalityInfoLayeredPane.add(this.functionalityTitleLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.functionalityDescriptionLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 11));
		this.functionalityDescriptionLabel
				.setText(bundle
						.getString("BulkUploadCodiFilesPanel.file_conversion_desc.text")); // NOI18N
		this.functionalityDescriptionLabel
				.setName("labelFunctionalityDescription"); // NOI18N
		this.functionalityDescriptionLabel.setBounds(180, 60, 670, 14);
		this.functionalityInfoLayeredPane.add(
				this.functionalityDescriptionLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.functionalityBannerBackgroundLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/barra titulo subtitulo.jpg"))); // NOI18N
		this.functionalityBannerBackgroundLabel.setText(bundle
				.getString("BulkUploadCodiFilesPanel.banner_bg.text")); // NOI18N
		this.functionalityBannerBackgroundLabel
				.setName("labelFunctionalityBannerBackground"); // NOI18N
		this.functionalityBannerBackgroundLabel
				.setPreferredSize(new java.awt.Dimension(1440, 120));
		this.functionalityBannerBackgroundLabel.setBounds(0, 0, 1440, 120);
		this.functionalityInfoLayeredPane.add(
				this.functionalityBannerBackgroundLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.functionalityExternalPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder(""));
		this.functionalityExternalPanel.setName("panelMainUpload"); // NOI18N

		this.functionalityInternalPanel.setBorder(javax.swing.BorderFactory
				.createTitledBorder(""));
		this.functionalityInternalPanel.setName("panelCodiUpload"); // NOI18N

		this.paneWithFilesToUploadTable.setName("paneCodiResultLoadScroll"); // NOI18N

		this.filesToUploadTable
				.setModel(new javax.swing.table.DefaultTableModel(
						new Object[][] {

						}, new String[] { "File Type", "File", "valid " }) {
					private static final long serialVersionUID = 7478114008000753206L;
					boolean[] canEdit = new boolean[] { false, false, true };

					@Override
					public boolean isCellEditable(int rowIndex, int columnIndex) {
						return this.canEdit[columnIndex];
					}
				});
		this.filesToUploadTable.setName("jTable1"); // NOI18N
		this.filesToUploadTable.getTableHeader().setReorderingAllowed(false);
		this.paneWithFilesToUploadTable
				.setViewportView(this.filesToUploadTable);
		this.filesToUploadTable
				.getColumnModel()
				.getColumn(0)
				.setHeaderValue(
						bundle.getString("BulkUploadCodiFilesPanel.jTable1.columnModel.title0")); // NOI18N
		this.filesToUploadTable
				.getColumnModel()
				.getColumn(1)
				.setHeaderValue(
						bundle.getString("BulkUploadCodiFilesPanel.jTable1.columnModel.title1")); // NOI18N
		this.filesToUploadTable
				.getColumnModel()
				.getColumn(2)
				.setHeaderValue(
						bundle.getString("BulkUploadCodiFilesPanel.jTable1.columnModel.title3")); // NOI18N

		this.clearFilesToUploadTableButton.setText(bundle
				.getString("BulkUploadCodiFilesPanel.clear.text")); // NOI18N
		this.clearFilesToUploadTableButton.setName("buttonCleanCodiTable"); // NOI18N
		this.clearFilesToUploadTableButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						cleanFilesToUploadTableActionPerformed();
					}
				});
		this.clearFilesToUploadTableButton.setEnabled(false);

		this.panelProgressBar.setName("jProgressBar2"); // NOI18N

		this.uploadFilesButton.setText(bundle
				.getString("BulkUploadCodiFilesPanel.convertion.text")); // NOI18N
		this.uploadFilesButton.setName("buttonUploadData"); // NOI18N
		this.uploadFilesButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						executeBulkUploadActionPerformed();
					}
				});
		this.uploadFilesButton.setEnabled(false);

		this.chooseFilesToUploadButton.setText(bundle
				.getString("BulkUploadCodiFilesPanel.jButton1.text")); // NOI18N
		this.chooseFilesToUploadButton.setName("buttonGetCodiFiles"); // NOI18N
		this.chooseFilesToUploadButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						searchFilesToUploadActionPerformed();
					}
				});

		this.paneWithFilesUploadedResult.setName("paneResults"); // NOI18N

		this.uploadedFilesResultTree.setName("myTree"); // NOI18N
		this.paneWithFilesUploadedResult
				.setViewportView(this.uploadedFilesResultTree);
		this.paneWithFilesUploadedResult.setVisible(false);

		this.clearDataBaseButton.setText(bundle
				.getString("BulkUploadCodiFilesPanel.clearCodiDb.text")); // NOI18N
		this.clearDataBaseButton.setName("buttonDeleteCodiData"); // NOI18N
		this.clearDataBaseButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						clearDataBaseActionPerformed();
					}
				});

		this.uploadStatusLabel.setName("labelCertificateStatus"); // NOI18N

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(
				this.functionalityInternalPanel);
		this.functionalityInternalPanel.setLayout(jPanel1Layout);
		jPanel1Layout
				.setHorizontalGroup(jPanel1Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel1Layout
										.createSequentialGroup()
										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																jPanel1Layout
																		.createSequentialGroup()
																		.addGap(38,
																				38,
																				38)
																		.addComponent(
																				this.chooseFilesToUploadButton,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				357,
																				Short.MAX_VALUE)
																		.addGap(355,
																				355,
																				355)
																		.addComponent(
																				this.uploadFilesButton,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				357,
																				Short.MAX_VALUE)
																		.addGap(21,
																				21,
																				21))
														.addGroup(
																javax.swing.GroupLayout.Alignment.TRAILING,
																jPanel1Layout
																		.createSequentialGroup()
																		.addGap(2,
																				2,
																				2)
																		.addComponent(
																				this.clearDataBaseButton,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				369,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																				487,
																				Short.MAX_VALUE)
																		.addComponent(
																				this.clearFilesToUploadTableButton,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				270,
																				javax.swing.GroupLayout.PREFERRED_SIZE))
														.addComponent(
																this.paneWithFilesToUploadTable,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																1128,
																Short.MAX_VALUE)
														.addComponent(
																this.paneWithFilesUploadedResult,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																1128,
																Short.MAX_VALUE)
														.addGroup(
																jPanel1Layout
																		.createSequentialGroup()
																		.addGap(2,
																				2,
																				2)
																		.addGroup(
																				jPanel1Layout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addComponent(
																								this.uploadStatusLabel,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								1126,
																								Short.MAX_VALUE)
																						.addComponent(
																								this.panelProgressBar,
																								javax.swing.GroupLayout.DEFAULT_SIZE,
																								1126,
																								Short.MAX_VALUE))))
										.addContainerGap()));
		jPanel1Layout
				.setVerticalGroup(jPanel1Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								jPanel1Layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.chooseFilesToUploadButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																54,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.uploadFilesButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																57,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addGap(29, 29, 29)
										.addComponent(
												this.paneWithFilesToUploadTable,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												187, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.paneWithFilesUploadedResult,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												175, Short.MAX_VALUE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.panelProgressBar,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addGap(4, 4, 4)
										.addComponent(
												this.uploadStatusLabel,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												15,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.clearFilesToUploadTableButton)
														.addComponent(
																this.clearDataBaseButton))
										.addGap(0, 0, 0)));

		this.deploySmttFilesUploadPanelButton.setText(bundle
				.getString("BulkUploadCodiFilesPanel.advanced_btn.text")); // NOI18N
		this.deploySmttFilesUploadPanelButton.setName("buttonStaticDataLoad"); // NOI18N
		this.deploySmttFilesUploadPanelButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						deploySmttFilesUploadPanelActionPerformed();
					}
				});

		this.uploadDataValidationCertificateButton.setText(bundle
				.getString("BulkUploadCodiFilesPanel.jButton3.text")); // NOI18N
		this.uploadDataValidationCertificateButton
				.setName("buttonLoadCertificate"); // NOI18N
		this.uploadDataValidationCertificateButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						uploadDataValidationCertificateActionPerformed();
					}
				});

		this.certificateStatusLabel
				.setText(bundle
						.getString("ImagesUploadPanel.jStatusVotingDeviceImageLabel.text")); // NOI18N
		this.certificateStatusLabel.setName("labelStatus"); // NOI18N

		this.certificateStatusDetailLabel
				.setName("labelCertificateStatusDescription"); // NOI18N

		javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(
				this.functionalityExternalPanel);
		this.functionalityExternalPanel.setLayout(jPanel2Layout);
		jPanel2Layout
				.setHorizontalGroup(jPanel2Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel2Layout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												jPanel2Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.functionalityInternalPanel,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addGroup(
																jPanel2Layout
																		.createSequentialGroup()
																		.addComponent(
																				this.uploadDataValidationCertificateButton,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				214,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addGap(18,
																				18,
																				18)
																		.addComponent(
																				this.certificateStatusLabel,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				50,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				this.certificateStatusDetailLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																				651,
																				Short.MAX_VALUE)
																		.addComponent(
																				this.deploySmttFilesUploadPanelButton,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				213,
																				javax.swing.GroupLayout.PREFERRED_SIZE)))
										.addContainerGap()));
		jPanel2Layout
				.setVerticalGroup(jPanel2Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel2Layout
										.createSequentialGroup()
										.addGap(23, 23, 23)
										.addGroup(
												jPanel2Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.deploySmttFilesUploadPanelButton)
														.addComponent(
																this.uploadDataValidationCertificateButton)
														.addComponent(
																this.certificateStatusLabel)
														.addComponent(
																this.certificateStatusDetailLabel))
										.addGap(33, 33, 33)
										.addComponent(
												this.functionalityInternalPanel,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addGap(45, 45, 45)));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.functionalityInfoLayeredPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 1214,
						Short.MAX_VALUE)
				.addGroup(
						layout.createSequentialGroup()
								.addGap(12, 12, 12)
								.addComponent(this.functionalityExternalPanel,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addGap(14, 14, 14)));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addComponent(
										this.functionalityInfoLayeredPane,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										124,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.functionalityExternalPanel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addContainerGap()));
		this.rootTreeNode = new DefaultMutableTreeNode(
				this.resourceBundle
						.getString("BulkUploadSmttFilesPanel.node.root"));

		DefaultTreeModel defaultTreeModel = new DefaultTreeModel(
				this.rootTreeNode);
		this.uploadedFilesResultTree.setModel(defaultTreeModel);
		this.uploadedFilesResultTree.setCellRenderer(new TreeRenderer());
	}

	private SAESCommonDestinations defaultDestinations = SAESLocalPersistenceManager
			.getInstance().getDefaultDestinations();

	private BulkUploadSmttFilesPanel bulkUploadSmttFilesPanel = new BulkUploadSmttFilesPanel();

	private final FunctionalityDefinitionEnum FILE_CONVERTION_FUNCTIONALITY = FunctionalityDefinitionEnum.FILE_CONVERTION;

	private int countFiles;

	private List<FileToUploadDTO> emlFilesToUpĺoadList;

	private javax.swing.JButton deploySmttFilesUploadPanelButton;

	private javax.swing.JButton clearFilesToUploadTableButton;

	private javax.swing.JButton clearDataBaseButton;

	private javax.swing.JButton uploadFilesButton;

	private javax.swing.JButton chooseFilesToUploadButton;

	private javax.swing.JButton uploadDataValidationCertificateButton;

	private javax.swing.JLabel certificateStatusLabel;

	private javax.swing.JLabel certificateStatusDetailLabel;

	private javax.swing.JLabel uploadStatusLabel;

	private javax.swing.JLabel functionalityIconLabel;

	private javax.swing.JLabel functionalityTitleLabel;

	private javax.swing.JLabel functionalityDescriptionLabel;

	private javax.swing.JLabel functionalityBannerBackgroundLabel;

	private javax.swing.JPanel functionalityExternalPanel;

	private javax.swing.JPanel functionalityInternalPanel;

	private javax.swing.JScrollPane paneWithFilesToUploadTable;

	private javax.swing.JScrollPane paneWithFilesUploadedResult;

	private javax.swing.JFileChooser functionalityFileChooser;

	private javax.swing.JProgressBar panelProgressBar;

	private javax.swing.JTable filesToUploadTable;

	private javax.swing.JTree uploadedFilesResultTree;

	private javax.swing.JDialog smttFilesUploadDialog;

	private javax.swing.JLayeredPane functionalityInfoLayeredPane;

	private javax.swing.tree.DefaultMutableTreeNode rootTreeNode;
}
