/**
 * DecryptController.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import java.io.File;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.DecryptWorker;

/**
 * Controller class that creates and starts the worker thread to decipher a
 * voting device folder
 */
public class DecryptController implements IAwtOperationListener {

	/**
	 * Instantiates a new decrypt controller.
	 * 
	 * @param votingDeviceFolderToDecrypt
	 *            the voting device folder to decrypt
	 * @param passphrase
	 *            the passphrase
	 * @param decryptPanel
	 *            the decrypt panel
	 */
	public DecryptController(File votingDeviceFolderToDecrypt,
			String passphrase, DecryptPanel decryptPanel) {
		this.votingDeviceFolderToDecrypt = votingDeviceFolderToDecrypt;
		this.passphrase = passphrase;
		this.decryptPanel = decryptPanel;
	}

	/**
	 * Method that starts the worker that does the real backend functionality
	 * work.
	 */
	public void startWorker() {

		String applicationKey = ECMServiceProvider.getApplicationKey();

		this.decryptWorker = new DecryptWorker(
				this.votingDeviceFolderToDecrypt, this.passphrase,
				applicationKey, ECMClientSideConstants.LENGTH_KEY);

		this.decryptWorker.addListener(this);

		Thread decryptWorkerThread = new Thread(this.decryptWorker,
				"decryptWorkerThread");

		decryptWorkerThread.start();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener#awtOperation
	 * (com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType)
	 */
	@Override
	public void awtOperation(AwtOperationType awtOperationType) {
		switch (awtOperationType) {
		case SHOW_RESULT_MESSAGE:
			this.decryptPanel.setStatusMessage("DecryptPanel.status.loaded");
			this.decryptPanel
					.showPopupInformationMessage("DecryptPanel.finish.process");
			break;
		default:
			this.decryptPanel.showPopupErrorMessage(
					this.decryptWorker.getErrorBundle(),
					this.decryptWorker.getFileWithProblem());
			break;
		}
	}

	/** The voting device folder to decrypt. */
	private File votingDeviceFolderToDecrypt;

	/** Passphrase used to encrypt the files. */
	private String passphrase;

	/** The instance of decryptWorker. */
	private DecryptWorker decryptWorker;

	/** The instance of decryptPanel. */
	private DecryptPanel decryptPanel;
}
