/**
 * DecryptPanel.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.ui;

import java.io.File;
import java.util.ResourceBundle;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * This controller is the front-end for decrypt files from FileGeneration
 * functionality
 */
public class DecryptPanel extends javax.swing.JPanel {

	/**
	 * Creates new form DecryptPanel.
	 */
	public DecryptPanel() {
		initComponents();
		this.selectionPathField.setEditable(false);
		listenerOnPassword();
	}

	/**
	 * Method used to set the status message in panel
	 * 
	 * @param bundle
	 *            key of message to show the user.
	 */
	public void setStatusMessage(String bundle) {
		this.statusMessage.setText(this.langBundle.getString(bundle));
	}

	/**
	 * Method used to show a popup error message to user
	 * 
	 * @param fileWithProblem
	 *            file name with the problem to include in the message
	 * 
	 * @param bundle
	 *            key of message to show.
	 */
	public void showPopupErrorMessage(String bundle, String fileWithProblem) {

		String message = this.langBundle.getString(bundle);

		if (fileWithProblem != null) {
			message = StringUtils.replace(message, "${file}", fileWithProblem);
		}

		ECMOptionPane.showMessageDialog(this, message,
				this.langBundle.getString("ErrorMessage.title"),
				ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
		enableScreenButtons(true);
	}

	/**
	 * Method used to show a popup info message to user
	 * 
	 * @param bundle
	 *            key of message to show.
	 */
	public void showPopupInformationMessage(String bundle) {
		ECMOptionPane.showMessageDialog(this,
				this.langBundle.getString(bundle),
				this.langBundle.getString("NormalMessage.title"),
				ECMOptionPane.INFORMATION_MESSAGE,
				ECMOptionPane.INFORMATION_DIALOG);
		enableScreenButtons(true);
		resetFields();
	}

	/**
	 * Fills the password fields from clipboard
	 * 
	 * @param value
	 *            password from clipboard
	 */
	public void fillPasswordFieldFromClipboard(String value) {
		this.passwordField1.setText(value.substring(0, 4));
		this.passwordField2.setText(value.substring(4, 8));
		this.passwordField3.setText(value.substring(8, 12));
		this.passwordField4.setText(value.substring(12, 16));
	}

	/**
	 * Method used to enable or disable all controls from panel
	 * 
	 * @param enabled
	 *            value to indicate if enable the controls or not.<br>
	 *            true = Enable controls.<br>
	 *            false = not Enable controls.
	 */
	private void enableScreenButtons(boolean enabled) {
		this.decryptButton.setEnabled(enabled);
		this.passwordField1.setEnabled(enabled);
		this.passwordField2.setEnabled(enabled);
		this.passwordField3.setEnabled(enabled);
		this.passwordField4.setEnabled(enabled);
		this.votingDeviceFolderChooserButton.setEnabled(enabled);
	}

	/**
	 * Set the listener on the set password fields
	 */
	private void listenerOnPassword() {
		this.passwordField1.addKeyListener(new DecryptPanelKeyboardListener(
				this.passwordField1, this));
		this.passwordField2.addKeyListener(new DecryptPanelKeyboardListener(
				this.passwordField2, this));
		this.passwordField3.addKeyListener(new DecryptPanelKeyboardListener(
				this.passwordField3, this));
		this.passwordField4.addKeyListener(new DecryptPanelKeyboardListener(
				this.passwordField4, this));
	}

	/**
	 * Method used to get the password introduced by user.
	 * 
	 * @return the user password introduced, otherwise empty String
	 */
	private String getUserPassphrase() {
		return String.valueOf(this.passwordField1.getPassword())
				+ String.valueOf(this.passwordField2.getPassword())
				+ String.valueOf(this.passwordField3.getPassword())
				+ String.valueOf(this.passwordField4.getPassword());
	}

	/**
	 * Check if some password field is empty
	 * 
	 * @return return true if all password fields are empty, otherwise return
	 *         false.
	 */
	private boolean isPasswordEmpty() {
		return this.passwordField1.getPassword().length == 0
				&& this.passwordField2.getPassword().length == 0
				&& this.passwordField3.getPassword().length == 0
				&& this.passwordField4.getPassword().length == 0;
	}

	/**
	 * Method used to clear fields from panel.<br>
	 * Fields to reset :<br>
	 * <li><code>electionsPathField</code> <li><code>passwordField1</code> <li>
	 * <code>passwordField2</code> <li><code>passwordField3</code> <li>
	 * <code>passwordField4</code> <li><code>filesPathLabel</code>
	 */
	private void resetFields() {
		this.selectionPathField.setText("");
		this.passwordField1.setText("");
		this.passwordField2.setText("");
		this.passwordField3.setText("");
		this.passwordField4.setText("");
		this.mainLabel.setText(this.langBundle
				.getString("DecryptPanel.filesPathLabel.text"));
	}

	/**
	 * Method executed when the user clicks in browse button.<br>
	 * It shows an browse file control
	 */
	private void votingDeviceFolderChooser() {
		this.mainLabel.setText(this.langBundle
				.getString("DecryptPanel.path.not.selected"));

		if (this.functionalityFileChooser.showOpenDialog(this) == 0) {

			ClientCommon
					.saveFileChooserLocationPreference(this.functionalityFileChooser
							.getSelectedFile().getAbsolutePath());

			File selectedFolder = this.functionalityFileChooser
					.getSelectedFile();
			this.selectionPathField.setText(selectedFolder.getPath());
			this.mainLabel.setText(this.langBundle
					.getString("DecryptPanel.path.selected"));
		}
	}

	/**
	 * Method executed when the user clicks in decrypt button.<br>
	 * This method executes the decryption process of folders of voting devices
	 */
	private void decryptFiles() {

		String votingDevicePathToDecrypt = this.selectionPathField.getText();

		if (isPasswordEmpty() || votingDevicePathToDecrypt.isEmpty()) {
			showPopupErrorMessage("DecryptPanel.password.or.path.empty", null);
		} else {
			String passphrase = getUserPassphrase();

			File votingDeviceFolderToDecrypt = new File(
					votingDevicePathToDecrypt);
			enableScreenButtons(false);
			DecryptController decryptController = new DecryptController(
					votingDeviceFolderToDecrypt, passphrase, this);
			decryptController.startWorker();
			setStatusMessage("DecryptPanel.status.loading");
		}
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {

		this.functionalityFileChooser = new javax.swing.JFileChooser();
		this.selectionPathField = new javax.swing.JTextField();
		this.votingDeviceFolderChooserButton = new javax.swing.JButton();
		this.mainLabel = new javax.swing.JLabel();
		this.decryptButton = new javax.swing.JButton();
		this.mainLayeredPane = new javax.swing.JLayeredPane();
		this.decryptLogo = new javax.swing.JLabel();
		this.decryptTitle = new javax.swing.JLabel();
		this.decryptDescriptionMessage = new javax.swing.JLabel();
		this.bannerBackgroundLabel = new javax.swing.JLabel();
		this.passwordLabel = new javax.swing.JLabel();
		this.statusLabel = new javax.swing.JLabel();
		this.statusMessage = new javax.swing.JLabel();
		this.passwordField1 = new javax.swing.JPasswordField();
		this.passwordField2 = new javax.swing.JPasswordField();
		this.passwordField3 = new javax.swing.JPasswordField();
		this.passwordField4 = new javax.swing.JPasswordField();

		this.functionalityFileChooser.setName("fileChooserEncDec"); // NOI18N

		ClientCommon.initFileChooserLocation(this.functionalityFileChooser);

		this.functionalityFileChooser
				.setFileSelectionMode(javax.swing.JFileChooser.DIRECTORIES_ONLY);

		this.selectionPathField.setMaximumSize(new java.awt.Dimension(320, 27));
		this.selectionPathField.setMinimumSize(new java.awt.Dimension(320, 27));
		this.selectionPathField.setName("textFieldElectionsPath"); // NOI18N
		this.selectionPathField
				.setPreferredSize(new java.awt.Dimension(320, 27));
		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		this.votingDeviceFolderChooserButton.setText(bundle
				.getString("DecryptPanel.chooseButton.text")); // NOI18N
		this.votingDeviceFolderChooserButton.setName("buttonChoose"); // NOI18N
		this.votingDeviceFolderChooserButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						votingDeviceFolderChooser();
					}
				});
		this.mainLabel.setText(bundle
				.getString("DecryptPanel.filesPathLabel.text")); // NOI18N
		this.mainLabel.setName("labelFilesPath"); // NOI18N

		this.decryptButton.setText(bundle
				.getString("DecryptPanel.decryptButton.text")); // NOI18N
		this.decryptButton.setName("buttonDecrypt"); // NOI18N
		this.decryptButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						decryptFiles();
					}
				});
		this.mainLayeredPane.setName("layeredPaneDecrypt"); // NOI18N

		this.decryptLogo
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/decrypt.png"))); // NOI18N
		this.decryptLogo.setName("labelEncDecLogo"); // NOI18N
		this.decryptLogo.setBounds(10, 10, 120, 90);
		this.mainLayeredPane.add(this.decryptLogo,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.decryptTitle.setFont(new java.awt.Font("DejaVu Sans", 1, 14));
		this.decryptTitle.setText(bundle
				.getString("DecryptPanel.decryptTitle.text")); // NOI18N
		this.decryptTitle.setName("labelEncDecTitle"); // NOI18N
		this.decryptTitle.setBounds(130, 30, 460, 17);
		this.mainLayeredPane.add(this.decryptTitle,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.decryptDescriptionMessage.setFont(new java.awt.Font("DejaVu Sans",
				0, 11));
		this.decryptDescriptionMessage.setText(bundle
				.getString("DecryptPanel.decryptDescriptionMessage.text")); // NOI18N
		this.decryptDescriptionMessage.setName("labelEncDecDescMsg"); // NOI18N
		this.decryptDescriptionMessage.setBounds(160, 50, 500, 14);
		this.mainLayeredPane.add(this.decryptDescriptionMessage,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.bannerBackgroundLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/barra titulo subtitulo.jpg"))); // NOI18N
		this.bannerBackgroundLabel.setText(bundle
				.getString("DecryptPanel.bannerBg.text")); // NOI18N
		this.bannerBackgroundLabel.setName("labelBannerBg"); // NOI18N
		this.bannerBackgroundLabel.setPreferredSize(new java.awt.Dimension(
				1440, 120));
		this.bannerBackgroundLabel.setBounds(0, 0, 1440, 120);
		this.mainLayeredPane.add(this.bannerBackgroundLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.passwordLabel.setText(bundle
				.getString("DecryptPanel.passwordLabel.text")); // NOI18N
		this.passwordLabel.setName("labelPassword"); // NOI18N

		this.statusLabel.setText(bundle
				.getString("DecryptPanel.statusLabel.text")); // NOI18N
		this.statusLabel.setName("labelStatus"); // NOI18N

		this.statusMessage.setText(bundle
				.getString("DecryptPanel.statusMsg.text")); // NOI18N
		this.statusMessage.setName("labelStatusMsg"); // NOI18N

		this.passwordField1
				.setHorizontalAlignment(javax.swing.JTextField.CENTER);
		this.passwordField1.setName("passwordField1"); // NOI18N

		this.passwordField2
				.setHorizontalAlignment(javax.swing.JTextField.CENTER);
		this.passwordField2.setName("passwordField2"); // NOI18N

		this.passwordField3
				.setHorizontalAlignment(javax.swing.JTextField.CENTER);
		this.passwordField3.setName("passwordField3"); // NOI18N

		this.passwordField4
				.setHorizontalAlignment(javax.swing.JTextField.CENTER);
		this.passwordField4.setName("passwordField4"); // NOI18N

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.mainLayeredPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 875,
						Short.MAX_VALUE)
				.addGroup(
						layout.createSequentialGroup()
								.addGap(33, 33, 33)
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		this.passwordField1,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		55,
																		javax.swing.GroupLayout.PREFERRED_SIZE)
																.addPreferredGap(
																		javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																.addComponent(
																		this.passwordField2,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		57,
																		javax.swing.GroupLayout.PREFERRED_SIZE)
																.addPreferredGap(
																		javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																.addComponent(
																		this.passwordField3,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		54,
																		javax.swing.GroupLayout.PREFERRED_SIZE)
																.addPreferredGap(
																		javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																.addComponent(
																		this.passwordField4,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		58,
																		javax.swing.GroupLayout.PREFERRED_SIZE))
												.addComponent(this.mainLabel)
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				javax.swing.GroupLayout.Alignment.LEADING)
																				.addComponent(
																						this.passwordLabel,
																						javax.swing.GroupLayout.PREFERRED_SIZE,
																						297,
																						javax.swing.GroupLayout.PREFERRED_SIZE)
																				.addGroup(
																						layout.createSequentialGroup()
																								.addComponent(
																										this.selectionPathField,
																										javax.swing.GroupLayout.PREFERRED_SIZE,
																										322,
																										javax.swing.GroupLayout.PREFERRED_SIZE)
																								.addPreferredGap(
																										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																								.addComponent(
																										this.votingDeviceFolderChooserButton)))
																.addGap(186,
																		186,
																		186)
																.addComponent(
																		this.decryptButton,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		163,
																		javax.swing.GroupLayout.PREFERRED_SIZE))
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		this.statusLabel,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		75,
																		javax.swing.GroupLayout.PREFERRED_SIZE)
																.addPreferredGap(
																		javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																.addComponent(
																		this.statusMessage,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		361,
																		javax.swing.GroupLayout.PREFERRED_SIZE)))
								.addContainerGap(109, Short.MAX_VALUE)));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addComponent(this.mainLayeredPane,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										124,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addGap(48, 48, 48)
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		this.mainLabel)
																.addPreferredGap(
																		javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																.addGroup(
																		layout.createParallelGroup(
																				javax.swing.GroupLayout.Alignment.BASELINE)
																				.addComponent(
																						this.selectionPathField,
																						javax.swing.GroupLayout.PREFERRED_SIZE,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						javax.swing.GroupLayout.PREFERRED_SIZE)
																				.addComponent(
																						this.votingDeviceFolderChooserButton))
																.addGap(21, 21,
																		21)
																.addComponent(
																		this.passwordLabel))
												.addGroup(
														layout.createSequentialGroup()
																.addGap(37, 37,
																		37)
																.addComponent(
																		this.decryptButton,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		62,
																		javax.swing.GroupLayout.PREFERRED_SIZE)))
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(
														this.passwordField2,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addComponent(
														this.passwordField3,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addComponent(
														this.passwordField4,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addComponent(
														this.passwordField1,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE))
								.addGap(43, 43, 43)
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(this.statusLabel)
												.addComponent(
														this.statusMessage,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														17,
														javax.swing.GroupLayout.PREFERRED_SIZE))
								.addGap(92, 92, 92)));
	}

	/**
	 * The resource message bundle
	 */
	private ResourceBundle langBundle = ResourceBundle
			.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");

	/**
	 * The logger object used to maintain control of the execution
	 */
	private static SMTTLogger logger = SMTTLogger.getLogger(DecryptPanel.class);

	private javax.swing.JLabel bannerBackgroundLabel;
	private javax.swing.JButton votingDeviceFolderChooserButton;
	private javax.swing.JButton decryptButton;
	private javax.swing.JTextField selectionPathField;
	private javax.swing.JLabel decryptDescriptionMessage;
	private javax.swing.JLabel decryptLogo;
	private javax.swing.JLabel decryptTitle;
	private javax.swing.JFileChooser functionalityFileChooser;
	private javax.swing.JLabel mainLabel;
	private javax.swing.JLayeredPane mainLayeredPane;
	private javax.swing.JPasswordField passwordField1;
	private javax.swing.JPasswordField passwordField2;
	private javax.swing.JPasswordField passwordField3;
	private javax.swing.JPasswordField passwordField4;
	private javax.swing.JLabel passwordLabel;
	private javax.swing.JLabel statusLabel;
	private javax.swing.JLabel statusMessage;
	private static final long serialVersionUID = 7880100335894740457L;
}