/**
 * EligibleOptionAdvancedSearchController.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.EligibleOptionAdvancedSearchWorker;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;

/**
 * Class used to control logic of user actions for the Advanced Eligible Option
 * Search panel.
 */
public class EligibleOptionAdvancedSearchController implements
		IAwtOperationListener {

	private EligibleOptionAdvancedSearchPanel eligibleOptionAdvancedSearchPanel = null;

	EligibleOptionAdvancedSearchWorker eligibleOptionAdvancedSearchWorker;

	boolean searchParties = false;

	/**
	 * Creates a new EligibleOptionAdvancedSearchController.
	 * 
	 * @param panel
	 *            the panel
	 * @param searchParties
	 *            the search type
	 */
	public EligibleOptionAdvancedSearchController(
			EligibleOptionAdvancedSearchPanel panel, boolean searchParties) {
		this.eligibleOptionAdvancedSearchPanel = panel;
		this.searchParties = searchParties;
	}

	/**
	 * Method that starts the worker that does the back-end functionality work.
	 */
	public void startWorker() {
		if (this.searchParties) {
			this.eligibleOptionAdvancedSearchWorker = new EligibleOptionAdvancedSearchWorker(
					this.eligibleOptionAdvancedSearchPanel.getSelectedRegions(),
					this.eligibleOptionAdvancedSearchPanel
							.getSelectedContests(), true);
		} else {
			LanguageData language = LanguageController.getInstance()
					.getSelectedLanguage();
			this.eligibleOptionAdvancedSearchWorker = new EligibleOptionAdvancedSearchWorker(
					this.eligibleOptionAdvancedSearchPanel.getSelectedRegions(),
					this.eligibleOptionAdvancedSearchPanel.getSelectedParties(),
					language);
		}

		this.eligibleOptionAdvancedSearchWorker.addListener(this);

		Thread searchPartyWorkerThread = new Thread(
				this.eligibleOptionAdvancedSearchWorker,
				"EligibleOptionAdvancedSearchWorker");

		searchPartyWorkerThread.start();

	}

	@Override
	public void awtOperation(AwtOperationType awtOperationType) {

		switch (awtOperationType) {
		case ENABLE_COMPONENTS:
			if (this.searchParties) {
				this.eligibleOptionAdvancedSearchPanel
						.blockSearchPartyButton(false);
			} else {
				this.eligibleOptionAdvancedSearchPanel
						.blockSearchContestButton(false);
			}
			break;

		case DISABLE_COMPONENTS:
			if (this.searchParties) {
				this.eligibleOptionAdvancedSearchPanel
						.blockSearchPartyButton(true);
			} else {
				this.eligibleOptionAdvancedSearchPanel
						.blockSearchContestButton(true);
			}
			break;

		case UPDATE_COMPONENTS:
			if (this.searchParties) {
				this.eligibleOptionAdvancedSearchPanel
						.updatePartyTable(this.eligibleOptionAdvancedSearchWorker
								.getPartiesList());
			} else {
				this.eligibleOptionAdvancedSearchPanel
						.updateContestTable(this.eligibleOptionAdvancedSearchWorker
								.getContestsList());
			}
			break;

		default:
			break;
		}
	}

}
