/**
 * EligibleOptionAdvancedSearchPanel.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Set;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.providers.RegionBrowserHierarchyProvider;
import com.smartmatic.waffle.ecm.client.ui.controller.IRefreshRegionWidget;
import com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyDTO;
import com.smartmatic.waffle.ecm.service.lang.DemographyException;
import com.smartmatic.waffle.ecm.ui.controls.Browser.BrowserHierarchy;
import com.smartmatic.waffle.ecm.ui.controls.Browser.BrowserHierarchyEvent;
import com.smartmatic.waffle.ecm.ui.controls.Browser.ClickedNodeListener;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * The Class is used to load and draw all the necessary components for the
 * Advanced Eligible Option Search Panel.
 */
public class EligibleOptionAdvancedSearchPanel extends javax.swing.JPanel
		implements ClickedNodeListener, IRefreshRegionWidget {

	/**
	 * Class constructor.
	 * 
	 * @param eligibleOptionEditorPanel
	 *            Parent component of the current window.
	 */
	public EligibleOptionAdvancedSearchPanel(
			EligibleOptionEditorPanel eligibleOptionEditorPanel) {
		this.eligibleOptionEditorPanel = eligibleOptionEditorPanel;
		RegionBrowserHierarchyProvider provider;

		LanguageData language = LanguageController.getInstance()
				.getSelectedLanguage();

		try {
			provider = new RegionBrowserHierarchyProvider(
					ECMServiceProvider.getAvailableRegionsForWidget(), false,
					language);
		} catch (DemographyException e) {
			logger.error(e, e.getMessage());
			return;
		} catch (ConnectionException e) {
			logger.error(e, e.getMessage());
			return;
		}

		this.resourceBundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N

		String titleName = this.resourceBundle
				.getString("RegionLevelNavigatorWidget.regionTable.header.regionColumn")
				+ " | "
				+ this.resourceBundle
						.getString("RegionLevelNavigatorWidget.regionTable.header.categoryColumn");

		this.browserHierarchy = new BrowserHierarchy(provider, true, titleName);

		initComponents();

		this.browserHierarchy.addOnClickNodeListener(this);

		this.searchCandidateOrPartyButton.setToolTipText(null);
		this.searchContestButton.setToolTipText(null);
		this.selectAllContestsButton.setToolTipText(null);
		this.clearContestTableButton.setToolTipText(null);
		this.searchPartyButton.setToolTipText(null);
		this.clearPartyTableButton.setToolTipText(null);
		this.contestTable.getModel().addTableModelListener(
				new TableModelListener() {
					@Override
					public void tableChanged(TableModelEvent e) {
						EligibleOptionAdvancedSearchPanel.this
								.clearPartyTableActionListener();

					}
				});
	}

	/**
	 * Prepares the panel for updating the information of a party or candidate
	 * 
	 * @param isCandidateSelected
	 *            true to update candidate info, false to update party info.
	 */
	public void searchCandidateOrParty(boolean isCandidateSelected) {

		if (isCandidateSelected) {
			this.searchByNamePanel
					.setBorder(javax.swing.BorderFactory.createTitledBorder(this.resourceBundle
							.getString("EligibleOptionAdvancedSearchPanel.candidatePanel.title")));
			this.searchByPartyPanel.setVisible(true);
		} else {
			this.searchByNamePanel
					.setBorder(javax.swing.BorderFactory.createTitledBorder(this.resourceBundle
							.getString("EligibleOptionAdvancedSearchPanel.partiesPanel.title")));
			this.searchByPartyPanel.setVisible(false);
		}
	}

	/**
	 * Returns the list of the selected parties.
	 * 
	 * @return the list of parties
	 * 
	 */
	public List<PartyData> getSelectedParties() {
		List<PartyData> selectedParties = new ArrayList<PartyData>();
		int rowCount = this.partyTable.getRowCount();

		for (int currentRow = 0; currentRow < rowCount; currentRow++) {
			Object obj = this.partyTable.getModel().getValueAt(currentRow, 1);
			boolean selected = obj.toString().equals("true");

			if (selected) {
				selectedParties.add((PartyData) this.partyTable.getModel()
						.getValueAt(currentRow, 0));
			}
		}

		return selectedParties;

	}

	/**
	 * Returns the list of the selected contests.
	 * 
	 * @return the list of contests.
	 */
	public List<ContestData> getSelectedContests() {
		List<ContestData> selectedContests = new ArrayList<ContestData>();
		int rowCount = this.contestTable.getRowCount();

		for (int currentRow = 0; currentRow < rowCount; currentRow++) {
			Object obj = this.contestTable.getModel().getValueAt(currentRow, 1);
			boolean selected = obj.toString().equals("true");

			if (selected) {
				selectedContests.add((ContestData) this.contestTable.getModel()
						.getValueAt(currentRow, 0));
			}
		}
		return selectedContests;

	}

	/**
	 * Returns a list of selected regions.
	 * 
	 * @return the list of regions
	 */
	public List<RegionData> getSelectedRegions() {

		Set<BrowserHierarchyDTO> selectedObject = this.browserHierarchy
				.getSelectedObject();

		List<RegionData> selectedRegions = new ArrayList<RegionData>();

		for (BrowserHierarchyDTO browserHierarchyDto : selectedObject) {
			RegionData selectedRegion = (RegionData) browserHierarchyDto
					.getId();
			selectedRegions.add(selectedRegion);
		}

		return selectedRegions;
	}

	/**
	 * Updates the party table.
	 * 
	 * @param parties
	 *            the list of parties to update
	 */
	public void updatePartyTable(List<PartyData> parties) {
		DefaultTableModel tableModel = (DefaultTableModel) this.partyTable
				.getModel();

		if (!tableModel.getDataVector().isEmpty()) {
			tableModel.getDataVector().removeAllElements();
		}
		if (parties != null && parties.size() > 0) {

			for (PartyData party : parties) {

				tableModel.addRow(new Object[] { party, false }); //$NON-NLS-1$

			}
		} else {
			ECMOptionPane
					.showMessageDialog(
							this,
							this.resourceBundle
									.getString("EligibleOptionAdvancedSearchPanel.searchParty.noPartyFound.message"),
							this.resourceBundle
									.getString("NormalMessage.title"),
							ECMOptionPane.INFORMATION_MESSAGE,
							ECMOptionPane.INFORMATION_DIALOG);
		}

		this.partyTable.clearSelection();
		this.partyTable.repaint();

	}

	/**
	 * Updates the contest table.
	 * 
	 * @param contests
	 *            the list of contests
	 */
	public void updateContestTable(List<ContestData> contests) {
		DefaultTableModel tableModel = (DefaultTableModel) this.contestTable
				.getModel();

		if (!tableModel.getDataVector().isEmpty()) {
			tableModel.getDataVector().removeAllElements();
		}
		if (contests != null && contests.size() > 0) {

			for (ContestData contest : contests) {
				tableModel.addRow(new Object[] { contest, false }); //$NON-NLS-1$
			}
		} else {
			ECMOptionPane
					.showMessageDialog(
							this,
							this.resourceBundle
									.getString("EligibleOptionAdvancedSearchPanel.searchContest.noContestFound.message"),
							this.resourceBundle
									.getString("NormalMessage.title"),
							ECMOptionPane.INFORMATION_MESSAGE,
							ECMOptionPane.INFORMATION_DIALOG);
		}
		this.contestTable.clearSelection();
		this.contestTable.repaint();
	}

	/**
	 * Blocks the search contest buttons.
	 * 
	 * @param enable
	 *            true to block otherwise false
	 */
	public void blockSearchContestButton(boolean enable) {
		this.contestTable.setEnabled(!enable);
		this.searchContestButton.setEnabled(!enable);
		this.selectAllContestsButton.setEnabled(!enable);
		this.clearContestTableButton.setEnabled(!enable);

		blockSearchPartyButton(enable);
	}

	/**
	 * Blocks the search party buttons.
	 * 
	 * @param enable
	 *            true to block otherwise false
	 */
	public void blockSearchPartyButton(boolean enable) {
		this.partyTable.setEnabled(!enable);
		this.searchPartyButton.setEnabled(!enable);
		this.selectAllPartiesButton.setEnabled(!enable);
		this.clearPartyTableButton.setEnabled(!enable);
	}

	/**
	 * Resets the search filters.
	 */
	public void resetFilters() {
		this.searchByNameTextField.setText("");
		this.browserHierarchy.reset();
		this.clearContestTableActionListener();
	}

	/**
	 * Returns the search text.
	 * 
	 * @return the search text
	 */
	public String getSearchText() {
		return this.searchByNameTextField.getText();
	}

	/**
	 * Clear the contest results on click action.
	 * 
	 * @param evt
	 *            the evt
	 */
	@Override
	public void onClickNode(BrowserHierarchyEvent evt) {
		clearContestTableActionListener();
	}

	/**
	 * Refreshes the widget.
	 */
	@Override
	public void refreshWidget() {
		this.browserHierarchy.reset();
	}

	/**
	 * Called when the user presses any key in the textbox for searching.
	 * 
	 * @param evt
	 *            info about user event.
	 */
	private void searchTextKeyReleased(java.awt.event.KeyEvent evt) {// GEN-FIRST:event_searchTextKeyReleased
		// If user presses the enter key
		if (KeyEvent.VK_ENTER == evt.getKeyCode()) {
			// execute the search
			this.searchCandidateOrPartyActionListener();
		}
	}

	/**
	 * Method used to search the parties related to contests selected by user.
	 * If there is not contests selected by user, a message is shown to the user
	 * The method is executed when the user click in search party button.<br>
	 * 
	 */
	private void searchPartyActionListener() {
		if (!getSelectedContests().isEmpty()) {
			EligibleOptionAdvancedSearchController eligibleOptionAdvancedSearchController = new EligibleOptionAdvancedSearchController(
					this, true);

			eligibleOptionAdvancedSearchController.startWorker();
		} else {
			ECMOptionPane
					.showMessageDialog(
							this,
							this.resourceBundle
									.getString("EligibleOptionAdvancedSearchPanel.noContestSelected.message"),
							this.resourceBundle
									.getString("NormalMessage.title"),
							ECMOptionPane.INFORMATION_MESSAGE,
							ECMOptionPane.INFORMATION_DIALOG);
		}

	}

	/**
	 * Refreshes the contests list and clear old party results.
	 */
	private void searchContestActionListener() {

		EligibleOptionAdvancedSearchController eligibleOptionAdvancedSearchController = new EligibleOptionAdvancedSearchController(
				this, false);

		eligibleOptionAdvancedSearchController.startWorker();

		clearPartyTableActionListener();

	}

	/**
	 * Starts a new search of candidates.
	 */
	private void searchCandidateOrPartyActionListener() {
		this.eligibleOptionEditorPanel.searchCandidateOrParty();
	}

	/**
	 * Selects or deselects all the rows loaded in the party table.
	 */
	private void selectAllPartiesActionListener() {
		this.partyTable.clearSelection();
		int rowCount = this.partyTable.getRowCount();
		boolean selected = false;
		for (int currentRow = 0; currentRow < rowCount; currentRow++) {
			if (currentRow == 0) {
				Object obj = this.partyTable.getModel().getValueAt(currentRow,
						1);
				selected = obj.toString().equals("true");
			}

			this.partyTable.getModel().setValueAt(!selected, currentRow, 1);
		}

	}

	/**
	 * Selects or deselects all the rows loaded in the candidate table.
	 */
	private void selectAllContestsActionListener() {
		this.contestTable.clearSelection();
		int rowCount = this.contestTable.getRowCount();
		boolean selected = false;
		for (int currentRow = 0; currentRow < rowCount; currentRow++) {
			if (currentRow == 0) {
				Object obj = this.contestTable.getModel().getValueAt(
						currentRow, 1);
				selected = obj.toString().equals("true");
			}
			this.contestTable.getModel().setValueAt(!selected, currentRow, 1);
		}

		clearPartyTableActionListener();

	}

	/**
	 * Clears the party results.
	 */
	private void clearPartyTableActionListener() {
		DefaultTableModel tableModel = (DefaultTableModel) this.partyTable
				.getModel();

		if (!tableModel.getDataVector().isEmpty()) {
			tableModel.getDataVector().removeAllElements();
		}

		this.partyTable.repaint();

	}

	/**
	 * Clears the contest results.
	 */
	private void clearContestTableActionListener() {
		DefaultTableModel tableModel = (DefaultTableModel) this.contestTable
				.getModel();

		if (!tableModel.getDataVector().isEmpty()) {
			tableModel.getDataVector().removeAllElements();
		}
		this.contestTable.repaint();
		clearPartyTableActionListener();
	}

	/**
	 * Closes the advanced dialog.
	 */
	private void closeDialogActionListener() {
		this.eligibleOptionEditorPanel.closeAdvancedDialog();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings({ "unchecked", "serial", "rawtypes" })
	private void initComponents() {

		this.searchByNamePanel = new javax.swing.JPanel();
		this.searchByNameTextField = new javax.swing.JTextField();
		this.searchByRegionPanel = new javax.swing.JPanel();
		this.browserHierarchyPanel = new javax.swing.JPanel();
		this.searchByContestPanel = new javax.swing.JPanel();
		this.searchContestButton = new javax.swing.JButton();
		this.contestTableScrollPane = new javax.swing.JScrollPane();
		this.contestTable = new javax.swing.JTable();
		this.selectAllContestsButton = new javax.swing.JButton();
		this.clearContestTableButton = new javax.swing.JButton();
		this.searchByPartyPanel = new javax.swing.JPanel();
		this.searchPartyButton = new javax.swing.JButton();
		this.partyTableScrollPane = new javax.swing.JScrollPane();
		this.partyTable = new javax.swing.JTable();
		this.selectAllPartiesButton = new javax.swing.JButton();
		this.clearPartyTableButton = new javax.swing.JButton();
		this.searchCandidateOrPartyButton = new javax.swing.JButton();
		this.closeButton = new javax.swing.JButton();

		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		this.searchByNamePanel
				.setBorder(javax.swing.BorderFactory.createTitledBorder(bundle
						.getString("EligibleOptionAdvancedSearchPanel.candidatePanel.title"))); // NOI18N
		this.searchByNamePanel.setName("panelName"); // NOI18N

		this.searchByNameTextField.setName("textFieldSearchText"); // NOI18N
		this.searchByNameTextField
				.addKeyListener(new java.awt.event.KeyAdapter() {
					@Override
					public void keyReleased(java.awt.event.KeyEvent evt) {
						searchTextKeyReleased(evt);
					}
				});

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(
				this.searchByNamePanel);
		this.searchByNamePanel.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				jPanel1Layout
						.createSequentialGroup()
						.addContainerGap()
						.addComponent(this.searchByNameTextField,
								javax.swing.GroupLayout.DEFAULT_SIZE, 764,
								Short.MAX_VALUE).addContainerGap()));
		jPanel1Layout.setVerticalGroup(jPanel1Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				jPanel1Layout
						.createSequentialGroup()
						.addContainerGap()
						.addComponent(this.searchByNameTextField,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								javax.swing.GroupLayout.DEFAULT_SIZE,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE)));

		this.searchByRegionPanel
				.setBorder(javax.swing.BorderFactory.createTitledBorder(bundle
						.getString("EligibleOptionAdvancedSearchPanel.regionPanel.title"))); // NOI18N
		this.searchByRegionPanel.setName("panelRegion"); // NOI18N

		this.browserHierarchyPanel.setName("panelRegionNav"); // NOI18N
		this.browserHierarchyPanel.setPreferredSize(new java.awt.Dimension(776,
				140));

		javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(
				this.browserHierarchyPanel);
		this.browserHierarchyPanel.setLayout(jPanel2Layout);
		jPanel2Layout.setHorizontalGroup(jPanel2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 764,
				Short.MAX_VALUE));
		jPanel2Layout.setVerticalGroup(jPanel2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 140,
				Short.MAX_VALUE));

		this.browserHierarchyPanel = this.browserHierarchy;

		javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(
				this.searchByRegionPanel);
		this.searchByRegionPanel.setLayout(jPanel4Layout);
		jPanel4Layout.setHorizontalGroup(jPanel4Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				jPanel4Layout
						.createSequentialGroup()
						.addContainerGap()
						.addComponent(this.browserHierarchyPanel,
								javax.swing.GroupLayout.DEFAULT_SIZE, 764,
								Short.MAX_VALUE).addContainerGap()));
		jPanel4Layout.setVerticalGroup(jPanel4Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addComponent(
				this.browserHierarchyPanel,
				javax.swing.GroupLayout.PREFERRED_SIZE,
				javax.swing.GroupLayout.DEFAULT_SIZE,
				javax.swing.GroupLayout.PREFERRED_SIZE));

		this.browserHierarchyPanel.setVisible(true);
		this.browserHierarchy.setVisible(true);

		this.searchByContestPanel
				.setBorder(javax.swing.BorderFactory.createTitledBorder(bundle
						.getString("EligibleOptionAdvancedSearchPanel.contestPanel.title"))); // NOI18N
		this.searchByContestPanel.setName("panelElections"); // NOI18N

		this.searchContestButton
				.setText(bundle
						.getString("EligibleOptionAdvancedSearchPanel.refreshContestButton.text")); // NOI18N
		this.searchContestButton.setName("buttonSearchContest"); // NOI18N

		this.searchContestButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						searchContestActionListener();
					}
				});

		this.contestTableScrollPane.setName("scrollPaneR"); // NOI18N

		this.contestTable.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] {

				}, new String[] { "Contest name", " " }) {
			Class[] types = new Class[] { java.lang.Object.class,
					java.lang.Boolean.class };
			boolean[] canEdit = new boolean[] { false, true };

			@Override
			public Class getColumnClass(int columnIndex) {
				return this.types[columnIndex];
			}

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return this.canEdit[columnIndex];
			}
		});
		this.contestTable.setName("tableContest"); // NOI18N
		this.contestTable.getTableHeader().setReorderingAllowed(false);
		this.contestTableScrollPane.setViewportView(this.contestTable);
		this.contestTable.getColumnModel().getColumn(0).setResizable(false);
		this.contestTable
				.getColumnModel()
				.getColumn(0)
				.setHeaderValue(
						bundle.getString("EligibleOptionEditorPanel.contestTable.nameColumn.title")); // NOI18N
		this.contestTable.getColumnModel().getColumn(0)
				.setCellRenderer(new DefaultTableCellRenderer() {
					private static final long serialVersionUID = -5586149720542099794L;

					@Override
					public void setValue(Object value) {
						setText((value == null) ? "" : ((ContestData) value)
								.getContestNameTranslate());
					}
				});
		this.contestTable.getColumnModel().getColumn(1).setMinWidth(30);
		this.contestTable.getColumnModel().getColumn(1).setPreferredWidth(30);
		this.contestTable.getColumnModel().getColumn(1).setMaxWidth(30);

		this.selectAllContestsButton
				.setText(bundle
						.getString("EligibleOptionAdvancedSearchPanel.selectAllContestsButton.text")); // NOI18N
		this.selectAllContestsButton.setName("buttonUnselectContest"); // NOI18N

		this.selectAllContestsButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						selectAllContestsActionListener();
					}
				});

		this.clearContestTableButton
				.setText(bundle
						.getString("EligibleOptionAdvancedSearchPanel.clearContestTableButton.text")); // NOI18N
		this.clearContestTableButton.setName("buttonClearContestTable"); // NOI18N

		this.clearContestTableButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						clearContestTableActionListener();
					}
				});

		javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(
				this.searchByContestPanel);
		this.searchByContestPanel.setLayout(jPanel5Layout);
		jPanel5Layout
				.setHorizontalGroup(jPanel5Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								jPanel5Layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.contestTableScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												559, Short.MAX_VALUE)
										.addGap(18, 18, 18)
										.addGroup(
												jPanel5Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.searchContestButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																187,
																Short.MAX_VALUE)
														.addComponent(
																this.selectAllContestsButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																187,
																Short.MAX_VALUE)
														.addComponent(
																this.clearContestTableButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																187,
																Short.MAX_VALUE))
										.addContainerGap()));
		jPanel5Layout
				.setVerticalGroup(jPanel5Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel5Layout
										.createSequentialGroup()
										.addGroup(
												jPanel5Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																jPanel5Layout
																		.createSequentialGroup()
																		.addComponent(
																				this.searchContestButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				this.selectAllContestsButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				this.clearContestTableButton))
														.addComponent(
																this.contestTableScrollPane,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																105,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));

		this.searchByPartyPanel
				.setBorder(javax.swing.BorderFactory.createTitledBorder(bundle
						.getString("EligibleOptionAdvancedSearchPanel.partyPanel.title"))); // NOI18N
		this.searchByPartyPanel.setName("panelPartyTitle"); // NOI18N

		this.searchPartyButton
				.setText(bundle
						.getString("EligibleOptionAdvancedSearchPanel.searchPartyButton.text")); // NOI18N
		this.searchPartyButton.setName("buttonSearchParty"); // NOI18N

		this.searchPartyButton.setName("buttonSearchParty"); // NOI18N

		this.searchPartyButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						searchPartyActionListener();
					}
				});

		this.partyTableScrollPane.setName("scrollPaneParty"); // NOI18N

		this.partyTable.setModel(new javax.swing.table.DefaultTableModel(
				new Object[][] {

				}, new String[] { "Party name", " " }) {
			Class[] types = new Class[] { java.lang.Object.class,
					java.lang.Boolean.class };
			boolean[] canEdit = new boolean[] { false, true };

			@Override
			public Class getColumnClass(int columnIndex) {
				return this.types[columnIndex];
			}

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return this.canEdit[columnIndex];
			}
		});
		this.partyTable.setName("tableParty"); // NOI18N
		this.partyTable.getTableHeader().setReorderingAllowed(false);
		this.partyTableScrollPane.setViewportView(this.partyTable);
		this.partyTable.getColumnModel().getColumn(0).setResizable(false);
		this.partyTable
				.getColumnModel()
				.getColumn(0)
				.setHeaderValue(
						bundle.getString("EligibleOptionEditorPanel.partyTable.nameColumn.title")); // NOI18N
		this.partyTable.getColumnModel().getColumn(0)
				.setCellRenderer(new DefaultTableCellRenderer() {
					private static final long serialVersionUID = 1436186969127681200L;

					@Override
					public void setValue(Object value) {
						setText((value == null) ? "" : ((PartyData) value)
								.getName()
								+ " ("
								+ ((PartyData) value).getCustomCode() + ")");
					}
				});
		this.partyTable.getColumnModel().getColumn(1).setMinWidth(30);
		this.partyTable.getColumnModel().getColumn(1).setPreferredWidth(30);
		this.partyTable.getColumnModel().getColumn(1).setMaxWidth(30);

		this.selectAllPartiesButton
				.setText(bundle
						.getString("EligibleOptionAdvancedSearchPanel.selectAllPartiesButton.text")); // NOI18N
		this.selectAllPartiesButton.setName("buttonUnselectParty"); // NOI18N

		this.selectAllPartiesButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						selectAllPartiesActionListener();
					}
				});

		this.clearPartyTableButton
				.setText(bundle
						.getString("EligibleOptionAdvancedSearchPanel.clearPartyTableButton.text")); // NOI18N
		this.clearPartyTableButton.setName("buttonClearPartyTable"); // NOI18N

		this.clearPartyTableButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						clearPartyTableActionListener();
					}
				});

		javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(
				this.searchByPartyPanel);
		this.searchByPartyPanel.setLayout(jPanel3Layout);
		jPanel3Layout
				.setHorizontalGroup(jPanel3Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								jPanel3Layout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.partyTableScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												558, Short.MAX_VALUE)
										.addGap(18, 18, 18)
										.addGroup(
												jPanel3Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.searchPartyButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																188,
																Short.MAX_VALUE)
														.addComponent(
																this.selectAllPartiesButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																188,
																Short.MAX_VALUE)
														.addComponent(
																this.clearPartyTableButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																188,
																Short.MAX_VALUE))
										.addContainerGap()));
		jPanel3Layout
				.setVerticalGroup(jPanel3Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel3Layout
										.createSequentialGroup()
										.addGroup(
												jPanel3Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																jPanel3Layout
																		.createSequentialGroup()
																		.addComponent(
																				this.searchPartyButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				this.selectAllPartiesButton)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addComponent(
																				this.clearPartyTableButton))
														.addComponent(
																this.partyTableScrollPane,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																105,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addContainerGap(
												javax.swing.GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));

		this.searchCandidateOrPartyButton
				.setText(bundle
						.getString("EligibleOptionEditorPanel.searchCandidatesButton.text")); // NOI18N
		this.searchCandidateOrPartyButton.setName("buttonSearchCandidates"); // NOI18N

		this.searchCandidateOrPartyButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						searchCandidateOrPartyActionListener();
					}
				});

		this.closeButton.setText(bundle
				.getString("BulkUploadSmttFilesPanel.jButton4.text")); // NOI18N
		this.closeButton.setName("buttonClose"); // NOI18N
		this.closeButton.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				closeDialogActionListener();
			}
		});

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				javax.swing.GroupLayout.Alignment.LEADING)
																				.addComponent(
																						this.searchByPartyPanel,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE)
																				.addComponent(
																						this.searchByRegionPanel,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE)
																				.addComponent(
																						this.searchByContestPanel,
																						javax.swing.GroupLayout.Alignment.TRAILING,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE)
																				.addComponent(
																						this.searchByNamePanel,
																						javax.swing.GroupLayout.Alignment.TRAILING,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						javax.swing.GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE))
																.addContainerGap())
												.addGroup(
														javax.swing.GroupLayout.Alignment.TRAILING,
														layout.createSequentialGroup()
																.addComponent(
																		this.searchCandidateOrPartyButton,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		222,
																		javax.swing.GroupLayout.PREFERRED_SIZE)
																.addGap(73, 73,
																		73)
																.addComponent(
																		this.closeButton,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		202,
																		javax.swing.GroupLayout.PREFERRED_SIZE)
																.addGap(162,
																		162,
																		162)))));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(this.searchByNamePanel,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.searchByRegionPanel,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.searchByContestPanel,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.searchByPartyPanel,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addGap(12, 12, 12)
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(
														this.searchCandidateOrPartyButton,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														35, Short.MAX_VALUE)
												.addComponent(
														this.closeButton,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														33, Short.MAX_VALUE))
								.addContainerGap()));
	}

	/**
	 * Serialization code
	 */
	private static final long serialVersionUID = 8943880940873196049L;

	/**
	 * For logging purposes.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(EligibleOptionAdvancedSearchPanel.class);

	/**
	 * Reference to widget regions hierarchy.
	 */
	private BrowserHierarchy browserHierarchy;

	/**
	 * Parent component of the current window.
	 */
	private EligibleOptionEditorPanel eligibleOptionEditorPanel;

	/**
	 * ResourceBundle reference to get the languages properties.
	 */
	private ResourceBundle resourceBundle;

	private javax.swing.JPanel searchByNamePanel;

	private javax.swing.JPanel searchByRegionPanel;

	private javax.swing.JPanel browserHierarchyPanel;

	private javax.swing.JPanel searchByContestPanel;

	private javax.swing.JPanel searchByPartyPanel;

	private javax.swing.JTextField searchByNameTextField;

	private javax.swing.JTable contestTable;

	private javax.swing.JTable partyTable;

	private javax.swing.JScrollPane contestTableScrollPane;

	private javax.swing.JScrollPane partyTableScrollPane;

	private javax.swing.JButton searchContestButton;

	private javax.swing.JButton selectAllContestsButton;

	private javax.swing.JButton clearContestTableButton;

	private javax.swing.JButton searchPartyButton;

	private javax.swing.JButton selectAllPartiesButton;

	private javax.swing.JButton clearPartyTableButton;

	private javax.swing.JButton searchCandidateOrPartyButton;

	private javax.swing.JButton closeButton;
}
