/**
 * EligibleOptionEditorPanel.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.ui;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableModel;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.JTextComponent;
import javax.swing.text.Keymap;

import org.apache.commons.io.FileUtils;

import com.smartmatic.waffle.domain.ballot.BallotOptionData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.security.FunctionalityDefinitionEnum;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.ui.controller.IRefreshRegionWidget;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.client.util.ImageUtilities;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.ecm.data.transfer.objects.EligibleOptionSearchDTO;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * The Class is used to load and draw all the necessary components for the
 * Eligible Option Editor Panel.
 */
public class EligibleOptionEditorPanel extends javax.swing.JPanel
		implements IRefreshRegionWidget {

	/**
	 * Creates new form EligibleOptionEditorPanel.
	 */
	public EligibleOptionEditorPanel() {
		this.currentPage = 0;
		this.totalNumberOfPages = 1;

		this.searchResult = null;
		this.dialogPanel = null;
		this.advancedSearchPanel = new EligibleOptionAdvancedSearchPanel(this);
		this.resourceBundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		initComponents();

		Keymap keyMap = this.searchText.getKeymap();
		JTextComponent.loadKeymap(keyMap, this.newBindings,
				this.searchText.getActions());

		this.searchEligibleOptionsButton.setToolTipText(null);
		this.advancedSearchButton.setToolTipText(null);
		this.clearResultsButton.setToolTipText(null);
		this.nextPageButton.setToolTipText(null);
		this.prevPageButton.setToolTipText(null);
		this.nextPageButton.setEnabled(false);
		this.prevPageButton.setEnabled(false);
		this.searchTypeLabel.setText(this.resourceBundle
				.getString("EligibleOptionEditorPanel.search.type.label"));
		this.currentText = "";
		this.currentRegions = new ArrayList<RegionData>();
		this.currentContests = new ArrayList<ContestData>();
		this.currentParties = new ArrayList<PartyData>();

		this.imageLogoMouseListener = new MouseAdapter() {
			@Override
			public void mouseClicked(java.awt.event.MouseEvent evt) {

				javax.swing.JTable table = EligibleOptionEditorPanel.this.elementsTable;

				int row = table.rowAtPoint(evt.getPoint());
				int col = table.columnAtPoint(evt.getPoint());

				if (row >= 0 && col == 1) {
					EligibleOptionData eligibleOption = EligibleOptionEditorPanel.this.searchResult
							.get(row);
					Dimension dim = new Dimension(900, 500);
					ImageIcon icon = getPartyLogoToDisplay(eligibleOption, dim);
					String fileName = getPartyLogoFileName(eligibleOption);

					if (icon != null) {
						showPartyLogoDialog(icon, fileName);
					}
				}
			}
		};

	}

	/**
	 * Gets file name from party logo
	 * 
	 * @param eligibleOption
	 * @return
	 */
	private String getPartyLogoFileName(EligibleOptionData eligibleOption) {

		PartyData partyData = getPartyFromElegibleOption(eligibleOption);
		String fileName = "";
		if (partyData != null) {
			fileName = partyData.getImageFileName();
		}

		return fileName;
	}

	/**
	 * Updates table using results of a party search.
	 * 
	 * @param resultsList
	 *            The results list with the values of a party search.
	 */
	public void updateResultsTableParty(
			List<EligibleOptionSearchDTO> resultsList) {

		for (EligibleOptionSearchDTO result : resultsList) {

			EligibleOptionData index = result.getEligibleOption();
			Dimension dim = new Dimension(200, 50);
			ImageIcon imageToDisplay = getPartyLogoToDisplay(index, dim);

			if (imageToDisplay == null) {
				imageToDisplay = getDefaultImageIcon();
			}

			Object[] values = new Object[4];
			values[0] = index.getBallotName();
			values[1] = imageToDisplay;
			values[2] = index.getContest().getContestNameTranslate();
			values[3] = index.getContest().getCollegeTranslate();
			result.setValues(values);
		}

		updateResultTable(resultsList);
		this.elementsTable.addMouseListener(this.imageLogoMouseListener);
	}

	/**
	 * Gets the image to display from an eligible option with a bounded
	 * dimension.
	 * 
	 * @param eligibleOption
	 *            Eligible option to obtain the logo.
	 * @param boundDimension
	 *            Constrained dimension.
	 * @return
	 */
	private ImageIcon getPartyLogoToDisplay(EligibleOptionData eligibleOption,
			Dimension boundDimension) {
		PartyData partyData = getPartyFromElegibleOption(eligibleOption);
		ImageIcon toDisplay = null;

		if (partyData != null) {
			ImageWrapper imageWrapper = partyData.getImage();
			Image image = ImageUtilities.imageWrapperToImage(imageWrapper);
			Image scaledImage = ImageUtilities.getScaledImageWithBounds(image,
					boundDimension);
			if (scaledImage != null) {
				toDisplay = new ImageIcon(scaledImage);
			}
		}
		return toDisplay;
	}

	/**
	 * Gets the Image party associated to an ElegibleOption.
	 * 
	 * @param elegibleOptionData
	 *            The eligible option.
	 * @return Party Data.
	 */
	private PartyData getPartyFromElegibleOption(
			EligibleOptionData elegibleOptionData) {

		PartyData partyData = null;

		if (elegibleOptionData.getBallotOptions() != null) {
			Iterator<BallotOptionData> iterator = elegibleOptionData
					.getBallotOptions().iterator();
			while (iterator.hasNext()) {
				BallotOptionData ballotOption = iterator.next();
				if (ballotOption.getParty() != null) {
					partyData = ballotOption.getParty();
					break;
				}
			}
		}
		return partyData;
	}

	/**
	 * Shows dialog with Party Logo Image details.
	 * 
	 * @param image
	 *            ImageIcon to be display in the dialog.
	 * @param imageFileName
	 *            Name of the image source file.
	 */
	private void showPartyLogoDialog(ImageIcon image, String imageFileName) {

		JLabel logo = new JLabel(image);
		JLabel fileName = new JLabel(imageFileName);

		final JComponent[] inputs = new JComponent[] { logo, fileName };

		String title = this.resourceBundle.getString(
				"EligibleOptionEditorPanel.dialog.partyLogoDetail.title");

		String okMessage = this.resourceBundle
				.getString("EligibleOptionCandidateEditorDialog.okButton.text");

		JOptionPane.showOptionDialog(this, inputs, title, JOptionPane.OK_OPTION,
				JOptionPane.PLAIN_MESSAGE, null, new String[] { okMessage },
				"");

	}

	/**
	 * Updates table using results of a candidates search.
	 * 
	 * @param resultsList
	 *            The results list with the values of a candidates search.
	 */
	public void updateResultsTableCandidates(
			List<EligibleOptionSearchDTO> resultsList) {
		updateResultTable(resultsList);
	}

	/**
	 * Updates the results table with the provided list.
	 * 
	 * @param resultsList
	 *            The list with the new values.
	 */
	private void updateResultTable(List<EligibleOptionSearchDTO> resultsList) {

		this.elementsTable.removeMouseListener(this.imageLogoMouseListener);

		DefaultTableModel tableModel = (DefaultTableModel) this.elementsTable
				.getModel();

		if (!tableModel.getDataVector().isEmpty()) {
			tableModel.getDataVector().removeAllElements();
		}
		if (resultsList != null && !resultsList.isEmpty()) {
			this.clearResultsButton.setEnabled(true);
			this.searchResult = new HashMap<Integer, EligibleOptionData>();
			int key = 0;

			for (EligibleOptionSearchDTO indexResult : resultsList) {

				this.searchResult.put(key, indexResult.getEligibleOption());
				tableModel.addRow(indexResult.getValues());
				key++;
				this.elementsTable.clearSelection();
				setColumnsEditors();
			}
		} else {
			this.clearResultsButton.setEnabled(false);
			showMessageNoElegibleOptionFound();
		}

	}

	/**
	 * Shows message dialog related to No Eligible Options Found.
	 */
	private void showMessageNoElegibleOptionFound() {
		ECMOptionPane.showMessageDialog(this, this.resourceBundle.getString(
				"EligibleOptionEditorPanel.noEligibleOptionFound.message"),
				this.resourceBundle.getString("NormalMessage.title"),
				ECMOptionPane.INFORMATION_MESSAGE,
				ECMOptionPane.INFORMATION_DIALOG);
	}

	/**
	 * Gets default image for empty party's logo.
	 * 
	 * @return ImageIcon
	 */
	private ImageIcon getDefaultImageIcon() {
		return (ImageIcon) LanguageController.getInstance().getResourceMap()
				.getIcon("ElegibleOptionEditorPanel.no.image.icon");
	}

	/**
	 * Returns the search text contained in the search text field.
	 * 
	 * @return the search text
	 */
	public String getSearchText() {
		return this.searchText.getText();
	}

	/**
	 * Returns the list of the selected regions.
	 * 
	 * @return the list of regions.
	 */
	public List<RegionData> getSelectedRegions() {
		return this.advancedSearchPanel.getSelectedRegions();
	}

	/**
	 * Returns the list of the selected contests.
	 * 
	 * @return the list of contests.
	 */
	public List<ContestData> getSelectedContests() {
		return this.advancedSearchPanel.getSelectedContests();
	}

	/**
	 * Returns the list of the selected parties.
	 * 
	 * @return the list of parties.
	 */
	public List<PartyData> getSelectedParties() {
		return this.advancedSearchPanel.getSelectedParties();
	}

	/**
	 * Starts a new advanced search of candidates.
	 */
	public void searchCandidateOrParty() {
		this.searchText.setText(this.advancedSearchPanel.getSearchText());

		this.setCurrentSearchFilters(this.advancedSearchPanel.getSearchText(),
				this.getSelectedRegions(), this.getSelectedContests(),
				this.getSelectedParties());

		this.resetCurrentPage();
		this.searchEligibleOption();
	}

	/**
	 * Enables the search buttons.
	 * 
	 * @param enable
	 *            true to unblock the search buttons, false to block.
	 */
	public void enableSearchButtons(boolean enable) {
		HashSet<FunctionalityDefinitionEnum> funcs = new HashSet<FunctionalityDefinitionEnum>();
		funcs.add(FunctionalityDefinitionEnum.ELIGIBLE_OPTION_EDITOR);
		ECMServiceProvider.blockOtherTabsByFunctionality(funcs, !enable);

		this.searchText.setEnabled(enable);
		this.searchEligibleOptionsButton.setEnabled(enable);
		this.advancedSearchButton.setEnabled(enable);
		this.elementsTable.setEnabled(enable);

		if (enable) {
			this.prevPageButton.setEnabled(enable);
			this.nextPageButton.setEnabled(enable);
		} else {
			if (this.currentPage > 0) {
				this.prevPageButton.setEnabled(enable);
			}
			if ((this.currentPage + 1) < this.totalNumberOfPages) {
				this.nextPageButton.setEnabled(enable);
			}
		}
	}

	/**
	 * Hides the advanced search dialog.
	 */
	public void hideAdvancedSearchDialog() {
		if (this.dialogPanel != null) {
			this.advancedSearchPanel.setVisible(false);
			this.dialogPanel.setVisible(false);
		}
	}

	/**
	 * Defines the text for pagination label.
	 * 
	 * @param text
	 *            the new pagination label text
	 */
	public void setPaginationLabelText() {

		String text = "";
		int page = this.currentPage + 1;
		String translatedPageLabel = this.resourceBundle
				.getString("EligibleOptionEditorPanel.pagination.page");
		if (this.totalNumberOfPages > 0) {
			text = translatedPageLabel + " " + page + " / "
					+ this.totalNumberOfPages;
		}
		this.paginationLabel.setText(text);
	}

	/**
	 * Defines the total number of pages to show.
	 * 
	 * @param number
	 *            the new max page
	 */
	public void setTotalNumberOfPages(int number) {
		this.totalNumberOfPages = number;
	}

	/**
	 * Returns the current page number
	 * 
	 * @return the current page number
	 */
	public int getCurrentPage() {
		return this.currentPage;
	}

	/**
	 * Decreases by one the current page.
	 */
	public void decreaseCurrentPage() {
		if (this.currentPage > 0) {
			this.currentPage--;
		}
	}

	/**
	 * Increases by one the current page.
	 */
	public void increaseCurrentPage() {
		if ((this.currentPage + 1) < this.totalNumberOfPages) {
			this.currentPage++;
		}
	}

	/**
	 * Reset the current page.
	 */
	public void resetCurrentPage() {
		this.currentPage = 0;
	}

	/**
	 * Returns the eligible option data from the candidates table at specific
	 * row.
	 * 
	 * @param row
	 *            the row
	 * @return the data or null if invalid row.
	 */
	public EligibleOptionData getEligibleOptionDataAtRow(int row) {
		if (row >= this.elementsTable.getRowCount() || row < 0) {
			return null;
		}
		return this.searchResult.get(row);
	}

	/**
	 * Updates the the candidate data at specific row.
	 * 
	 * @param row
	 *            the row
	 * @param eo
	 *            the eo
	 * @param isCandidateUpdate
	 *            the is candidate update
	 */
	public void updateRow(int row, EligibleOptionData eo,
			boolean isCandidateUpdate) {
		this.searchResult.put(row, eo);
		String value = "";
		if (isCandidateUpdate) {
			String[] ballotsName = eo.getBallotName().trim().split("#");
			if (ballotsName.length > 0) {
				value = ballotsName[0];
			}
			if (ballotsName.length > 1) {
				value += " " + ballotsName[1];
			}
			if (ballotsName.length > 2) {
				value += " " + ballotsName[2];
			}

		} else {
			value = this.searchResult.get(row).getBallotName();
		}
		DefaultTableModel tableModel = (DefaultTableModel) this.elementsTable
				.getModel();

		tableModel.setValueAt(value, row, 0);

		this.elementsTable.repaint();

	}

	/**
	 * Returns the current search text.
	 * 
	 * @return the search text
	 */
	public String getCurrentSearchText() {
		return this.currentText;
	}

	/**
	 * Returns a list of search regions.
	 * 
	 * @return the list of regions
	 */
	public List<RegionData> getCurrentSearchRegions() {
		return this.currentRegions;
	}

	/**
	 * Returns the list of search contests.
	 * 
	 * @return the list of contests
	 */
	public List<ContestData> getCurrentSearchContests() {
		return this.currentContests;
	}

	/**
	 * Returns the list of search parties.
	 * 
	 * @return the list of parties
	 */
	public List<PartyData> getCurrentSearchParties() {
		return this.currentParties;
	}

	/**
	 * Set current search values for filters.
	 * 
	 * @param currentSearchText
	 *            the current search text
	 * @param currentSearchRegions
	 *            the current search regions
	 * @param currentSearchContests
	 *            the current search contests
	 * @param currentSearchParties
	 *            the current search parties
	 */
	public void setCurrentSearchFilters(String currentSearchText,
			List<RegionData> currentSearchRegions,
			List<ContestData> currentSearchContests,
			List<PartyData> currentSearchParties) {
		this.currentText = currentSearchText;
		this.currentRegions = currentSearchRegions;
		this.currentContests = currentSearchContests;
		this.currentParties = currentSearchParties;
	}

	/**
	 * Closes the advanced dialog window.
	 */
	public void closeAdvancedDialog() {
		this.dialogPanel.setVisible(false);
	}

	/**
	 * Refreshes the panel widgets.
	 */
	@Override
	public void refreshWidget() {
		this.advancedSearchPanel.refreshWidget();
	}

	/**
	 * Gets the table model of the region selection table. The table model is
	 * initialized with all the direct child regions of the root region which
	 * represents the country.
	 * 
	 * @return Initialized table model
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private DefaultTableModel getCandidateModel() {
		DefaultTableModel tableModel = null;

		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N

		tableModel = new DefaultTableModel(new Object[][] {}, new String[] {
				bundle.getString(
						"EligibleOptionEditorPanel.candidatesTable.header.candidateNameColumn"),
				bundle.getString(
						"EligibleOptionEditorPanel.candidatesTable.header.candidateIDColumn"),
				bundle.getString(
						"EligibleOptionEditorPanel.candidatesTable.header.candidateTypeColumn"),
				bundle.getString(
						"EligibleOptionEditorPanel.candidatesTable.header.contestColumn"),
				bundle.getString(
						"EligibleOptionEditorPanel.candidatesTable.header.partyColumn"),
				"" }) {
			/**
							 * 
							 */
			private static final long serialVersionUID = -1927678653655107268L;

			Class[] types = new Class[] { java.lang.String.class,
					java.lang.String.class, java.lang.String.class,
					java.lang.String.class, java.lang.String.class,
					java.lang.Object.class };
			boolean[] canEdit = new boolean[] { false, false, false, false,
					false, true };

			@Override
			public Class getColumnClass(int columnIndex) {
				return this.types[columnIndex];
			}

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return this.canEdit[columnIndex];
			}
		};

		return tableModel;
	}

	/**
	 * Gets the party model.
	 * 
	 * @return the party model
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private DefaultTableModel getPartyModel() {
		DefaultTableModel tableModel = null;

		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N

		tableModel = new DefaultTableModel(new Object[][] {}, new String[] {
				bundle.getString("EligibleOptionEditorPanel.table.party.name"),
				bundle.getString("EligibleOptionEditorPanel.table.party.image"),
				bundle.getString("EligibleOptionEditorPanel.table.contest"),
				bundle.getString("EligibleOptionEditorPanel.table.college"),
				"" }) {

			private static final long serialVersionUID = 8052209067147891375L;

			Class[] types = new Class[] { java.lang.String.class,
					javax.swing.ImageIcon.class, java.lang.String.class,
					java.lang.String.class, java.lang.String.class };

			boolean[] canEdit = new boolean[] { false, false, false, false,
					true };

			@Override
			public Class getColumnClass(int columnIndex) {
				return this.types[columnIndex];
			}

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return this.canEdit[columnIndex];
			}
		};

		return tableModel;
	}

	/**
	 * Sets the table's columns renderer and editors.
	 */
	private void setColumnsEditors() {

		// Candidate type column
		this.elementsTable.getColumnModel().getColumn(1).setPreferredWidth(150);

		int size = this.elementsTable.getColumnCount();

		String selectValue = (String) this.candidateOrPartySelectionComboBox
				.getSelectedItem();
		boolean isCandidateSelected;

		if (selectValue.equals(this.resourceBundle
				.getString("EligibleOptionEditorPanel.candidate.search"))) {
			isCandidateSelected = true;
		} else {
			isCandidateSelected = false;
		}

		// Modify button column
		this.elementsTable.getColumn(this.elementsTable.getColumnName(size - 1))
				.setCellEditor(new EligibleOptionCellEditor(this,
						isCandidateSelected));

		this.elementsTable.getColumn(this.elementsTable.getColumnName(size - 1))
				.setCellRenderer(new EligibleOptionCellButtonRenderer(
						this.resourceBundle.getString(
								"EligibleOptionEditorPanel.candidatesTable.modifyButton.text")));
		this.repaint();

	}

	/**
	 * Returns a combo box with the search types.
	 * 
	 * @return the combo box
	 */
	private DefaultComboBoxModel getTypeSearch() {
		String[] searchType = {
				this.resourceBundle.getString(
						"EligibleOptionEditorPanel.candidate.search"),
				this.resourceBundle
						.getString("EligibleOptionEditorPanel.party.search") };
		return new DefaultComboBoxModel(searchType);
	}


    /**
     * Exports the eligible option editor log.
     */
    private void exportLogButtonActionPerformed() {
        final String msgKey;        // Success Message Key
        File fileSelected;          // selected file
        File logFile;               // LogFile
        ResourceBundle rb;          // ResourceBundle Alias

        msgKey = "EligibleOptionEditorPanel.success-messages.EoeLogExported";
        rb     = this.resourceBundle;

        try {
            logFile = new File(
                ECMClientSideConstants.CLIENT_SIDE_LOG_FILES_PATH, "eoe.log"
            );

            if (logFile.exists()) {
                if (this.exportLogFileChooser.showSaveDialog(this) == 0) {
                    ClientCommon.saveFileChooserLocationPreference(
                        this.exportLogFileChooser
                            .getCurrentDirectory()
                            .getAbsolutePath()
                    );
                    fileSelected = this.exportLogFileChooser.getSelectedFile();

                    if (fileSelected != null) {
                        FileUtils.copyFile(logFile, fileSelected);
                        PendriveUtil.sync();

                        ECMOptionPane.showMessageDialog(this,
                            rb.getString(msgKey),
                            rb.getString("NormalMessage.title"),
                            ECMOptionPane.INFORMATION_MESSAGE,
                            ECMOptionPane.INFORMATION_DIALOG
                        );
                    }
                }

            } else {
                ECMOptionPane.showMessageDialog(
                    this,
                    rb.getString("EligibleOptionEditorPanel.logFileNotFound.message"),
                    rb.getString("ErrorMessage.title"),
                    ECMOptionPane.ERROR_MESSAGE,
                    ECMOptionPane.ERROR_DIALOG
                );
            }

        } catch (Exception e) {
            logger.error(e, e.getMessage());
            ECMOptionPane.showMessageDialog(
                this,
                rb.getString("EligibleOptionEditorPanel.errorExportingLog.message"),
                rb.getString("ErrorMessage.title"),
                ECMOptionPane.ERROR_MESSAGE,
                ECMOptionPane.ERROR_DIALOG
            );
        }
    }


	/**
	 * Event triggered when the user changes the selection combo box.
	 */
	private void searchTypeSelection() {

		String selectedItem = (String) this.candidateOrPartySelectionComboBox
				.getSelectedItem();
		String partyName = this.resourceBundle
				.getString("EligibleOptionEditorPanel.party.search");
		if (selectedItem.equals(partyName)) {
			this.candidatePartyLabel.setText(this.resourceBundle
					.getString("EligibleOptionEditorPanel.partyLabel.text"));
		} else {
			this.candidatePartyLabel.setText(this.resourceBundle.getString(
					"EligibleOptionEditorPanel.candidateLabel.text"));
		}
	}

	/**
	 * Opens the advance search window.
	 */
	private void advanceSearchActionPerformed() {

		this.dialogPanel = new JDialog();
		int width = 0;
		int height = 0;
		String selection = this.resourceBundle
				.getString("EligibleOptionEditorPanel.candidate.search");
		boolean isCandidateSelected = this.candidateOrPartySelectionComboBox
				.getSelectedItem().equals(selection);
		width = 820;
		if (isCandidateSelected) {
			height = 670;
		} else {
			height = 510;
		}
		this.dialogPanel.setTitle(this.resourceBundle.getString(
				"EligibleOptionAdvancedSearchPanel.title.AdvancedSearch"));
		this.dialogPanel.add(this.advancedSearchPanel);
		this.dialogPanel.setSize(width, height);
		this.dialogPanel.setLocationRelativeTo(this);
		this.dialogPanel.setResizable(false);
		this.dialogPanel.setAlwaysOnTop(true);
		this.dialogPanel.setModal(true);
		this.advancedSearchPanel.resetFilters();
		this.advancedSearchPanel.setVisible(true);
		this.advancedSearchPanel.searchCandidateOrParty(isCandidateSelected);

		this.dialogPanel.setVisible(true);
	}

	/**
	 * Clear result list.
	 */
	public void clearSearchResults() {
		this.clearResultsButton.setEnabled(false);
		DefaultTableModel tableModel = (DefaultTableModel) this.elementsTable
				.getModel();

		if (!tableModel.getDataVector().isEmpty()) {
			tableModel.getDataVector().removeAllElements();
		}

		this.elementsTable.repaint();
	}

	/**
	 * Reset the components used for the current search.
	 */
	public void resetSearchComponents() {
		resetCurrentPage();
		this.searchText.setText("");
		this.paginationLabel.setText("");
		this.totalNumberOfPages = 0;
		this.currentText = "";
		this.currentRegions = new ArrayList<RegionData>();
		this.currentContests = new ArrayList<ContestData>();
		this.currentParties = new ArrayList<PartyData>();
	}

	/**
	 * Next page.
	 */
	private void nextResultPageActionPerformed() {
		if ((this.currentPage + 1) < this.totalNumberOfPages) {
			this.increaseCurrentPage();
			this.clearSearchResults();
			this.searchEligibleOption();
		}
	}

	/**
	 * Previous page.
	 */
	private void prevResultPageActionPerformed() {
		if (this.currentPage > 0) {
			this.decreaseCurrentPage();
			this.clearSearchResults();
			this.searchEligibleOption();
		}
	}

	/**
	 * Search eligible option.
	 */
	private void searchEligibleOption() {
		String selectedItem = (String) this.candidateOrPartySelectionComboBox
				.getSelectedItem();

		String partyName = this.resourceBundle
				.getString("EligibleOptionEditorPanel.party.search");

		if (selectedItem.equals(partyName)) {
			SearchPartyController searchPartyController = new SearchPartyController(
					this);
			this.elementsTable.setModel(getPartyModel());
			this.elementsTable.setRowHeight(50);
			searchPartyController.startWorker();
		} else {
			SearchCandidateController searchCandidateController = new SearchCandidateController(
					this);
			this.elementsTable.setModel(getCandidateModel());
			this.elementsTable.setRowHeight(this.defaultRowHeight);
			searchCandidateController.startWorker();
		}
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {

		this.mainLayeredPane = new javax.swing.JLayeredPane();
		this.functionalityTitleLabel = new javax.swing.JLabel();
		this.iconLabel = new javax.swing.JLabel();
		this.functionalityDescriptionLabel = new javax.swing.JLabel();
		this.functionalityBannerBackgroundLabel = new javax.swing.JLabel();
		this.mainPanel = new javax.swing.JPanel();
		this.mainScrollPane = new javax.swing.JScrollPane();
		this.elementsTable = new javax.swing.JTable();
		this.defaultRowHeight = this.elementsTable.getRowHeight();
		this.searchEligibleOptionsButton = new javax.swing.JButton();
		this.candidatePartyLabel = new javax.swing.JLabel();
		this.searchText = new javax.swing.JTextField();
		this.advancedSearchButton = new javax.swing.JButton();
		this.paginationLabel = new javax.swing.JLabel();
		this.prevPageButton = new javax.swing.JButton();
		this.nextPageButton = new javax.swing.JButton();
		this.clearResultsButton = new javax.swing.JButton();
		this.candidateOrPartySelectionComboBox = new javax.swing.JComboBox();
		this.searchTypeLabel = new javax.swing.JLabel();
		this.exportLogButton = new javax.swing.JButton();
		this.exportLogFileChooser = new JFileChooser();

		this.exportLogFileChooser.setName("fileChooserExportLogEO");

		this.exportLogFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

		ClientCommon.initFileChooserLocation(this.exportLogFileChooser,
				new File("eoe.log"));

		this.mainLayeredPane.setName("layeredPaneMain"); // NOI18N

		this.functionalityTitleLabel
				.setFont(new java.awt.Font("DejaVu Sans", 1, 14));
		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		this.functionalityTitleLabel.setText(
				bundle.getString("EligibleOptionEditorPanel.jLabel2.text_1")); // NOI18N
		this.functionalityTitleLabel.setName("labelFuncTitle"); // NOI18N
		this.functionalityTitleLabel.setBounds(170, 20, 500, 17);
		this.mainLayeredPane.add(this.functionalityTitleLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.iconLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource(
				"/com/smartmatic/waffle/ecm/client/resources/generacion_maquina.png"))); // NOI18N
		this.iconLabel.setName("labelFuncIcon"); // NOI18N
		this.iconLabel.setBounds(30, 0, 120, 100);
		this.mainLayeredPane.add(this.iconLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.functionalityDescriptionLabel
				.setFont(new java.awt.Font("DejaVu Sans", 0, 11));
		this.functionalityDescriptionLabel.setText(
				bundle.getString("EligibleOptionEditorPanel.jLabel3.text_1")); // NOI18N
		this.functionalityDescriptionLabel.setName("labelFuncDescription"); // NOI18N
		this.functionalityDescriptionLabel.setBounds(190, 40, 720, 17);
		this.mainLayeredPane.add(this.functionalityDescriptionLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.functionalityBannerBackgroundLabel
				.setIcon(new javax.swing.ImageIcon(getClass().getResource(
						"/com/smartmatic/waffle/ecm/client/resources/barra titulo subtitulo.jpg"))); // NOI18N
		this.functionalityBannerBackgroundLabel.setName("labelBanner"); // NOI18N
		this.functionalityBannerBackgroundLabel
				.setPreferredSize(new java.awt.Dimension(1440, 120));
		this.functionalityBannerBackgroundLabel.setBounds(0, -10, 1450, 140);
		this.mainLayeredPane.add(this.functionalityBannerBackgroundLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.mainPanel.setMinimumSize(new java.awt.Dimension(800, 0));
		this.mainPanel.setName("panelMain"); // NOI18N
		this.mainPanel.setPreferredSize(new java.awt.Dimension(1024, 600));

		this.mainScrollPane.setName("scrollPaneMainEO"); // NOI18N

		this.elementsTable.setModel(
				new javax.swing.table.DefaultTableModel(new Object[][] {

				}, new String[] {

				}));

		this.elementsTable.setName("tableEligibleOptionResults"); // NOI18N
		this.elementsTable.getTableHeader().setReorderingAllowed(false);
		this.mainScrollPane.setViewportView(this.elementsTable);

		this.searchEligibleOptionsButton.setText(bundle.getString(
				"EligibleOptionEditorPanel.searchCandidatesButton.text")); // NOI18N
		this.searchEligibleOptionsButton.setName("buttonSearchEligibleOptions"); // NOI18N
		this.searchEligibleOptionsButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(
							java.awt.event.ActionEvent evt) {
						setCurrentSearchFilters(getSearchText(),
								new ArrayList<RegionData>(),
								new ArrayList<ContestData>(),
								new ArrayList<PartyData>());
						resetCurrentPage();
						searchEligibleOption();
					}
				});

		this.candidatePartyLabel.setText(bundle
				.getString("EligibleOptionEditorPanel.candidateLabel.text")); // NOI18N
		this.candidatePartyLabel.setName("labelCandidate"); // NOI18N

		this.searchText.setName("textFieldSearchText"); // NOI18N
		this.searchText.addKeyListener(new java.awt.event.KeyAdapter() {
			@Override
			public void keyReleased(java.awt.event.KeyEvent evt) {
				if (KeyEvent.VK_ENTER == evt.getKeyCode()) {
					resetCurrentPage();
					searchEligibleOption();
				}
			}
		});

		this.advancedSearchButton.setText(bundle.getString(
				"EligibleOptionEditorPanel.advancedSearchButton.text")); // NOI18N
		this.advancedSearchButton.setName("buttonAdvancedSearch"); // NOI18N
		this.advancedSearchButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(
							java.awt.event.ActionEvent evt) {
						advanceSearchActionPerformed();
					}
				});

		this.paginationLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
		this.paginationLabel.setText("  "); // NOI18N
		this.paginationLabel.setName("labelPagination"); // NOI18N

		this.prevPageButton.setText(bundle
				.getString("EligibleOptionEditorPanel.prevPageButton.text")); // NOI18N
		this.prevPageButton.setName("buttonPrevPage"); // NOI18N
		this.prevPageButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(
							java.awt.event.ActionEvent evt) {
						prevResultPageActionPerformed();
					}
				});

		this.nextPageButton.setText(bundle
				.getString("EligibleOptionEditorPanel.nextPageButton.text")); // NOI18N
		this.nextPageButton.setName("buttonNextPage"); // NOI18N
		this.nextPageButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(
							java.awt.event.ActionEvent evt) {
						nextResultPageActionPerformed();
					}
				});

		this.clearResultsButton.setText(bundle.getString(
				"EligibleOptionEditorPanel.clearResultsButton.text")); // NOI18N
		this.clearResultsButton.setEnabled(false);
		this.clearResultsButton.setMaximumSize(new java.awt.Dimension(200, 29));
		this.clearResultsButton.setName("buttonClearResults"); // NOI18N
		this.clearResultsButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(
							java.awt.event.ActionEvent evt) {
						clearSearchResults();
					}
				});

		this.candidateOrPartySelectionComboBox.setModel(getTypeSearch());
		this.candidateOrPartySelectionComboBox.setName("comboBoxType"); // NOI18N
		this.candidateOrPartySelectionComboBox
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(
							java.awt.event.ActionEvent evt) {
						searchTypeSelection();
					}
				});
		this.searchTypeLabel.setText("SearchType");
		this.searchTypeLabel.setName("labelSearchType"); // NOI18N

		this.exportLogButton.setName("buttonExportLogEO"); // NOI18N
		this.exportLogButton.setText(bundle
				.getString("EligibleOptionEditorPanel.exportLogButton.text"));
		this.exportLogButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(
							java.awt.event.ActionEvent evt) {
						exportLogButtonActionPerformed();
					}
				});

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(
				this.mainPanel);
		this.mainPanel.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(javax.swing.GroupLayout.Alignment.TRAILING,
						jPanel1Layout.createSequentialGroup().addContainerGap()
								.addGroup(jPanel1Layout.createParallelGroup(
										javax.swing.GroupLayout.Alignment.LEADING)
										.addComponent(this.mainScrollPane,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												977, Short.MAX_VALUE)
										.addGroup(
												javax.swing.GroupLayout.Alignment.TRAILING,
												jPanel1Layout
														.createSequentialGroup()
														.addComponent(
																this.exportLogButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																272,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addPreferredGap(
																javax.swing.LayoutStyle.ComponentPlacement.RELATED)
														.addComponent(
																this.paginationLabel,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																613,
																Short.MAX_VALUE)
														.addGap(18, 18, 18)
														.addComponent(
																this.prevPageButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																25,
																Short.MAX_VALUE)
														.addPreferredGap(
																javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
														.addComponent(
																this.nextPageButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																25,
																Short.MAX_VALUE))
										.addGroup(
												javax.swing.GroupLayout.Alignment.TRAILING,
												jPanel1Layout
														.createSequentialGroup()
														.addGap(455, 455, 455)
														.addComponent(
																this.clearResultsButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																104,
																Short.MAX_VALUE)
														.addGap(418, 418, 418))
										.addGroup(jPanel1Layout
												.createSequentialGroup()
												.addGroup(jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.candidatePartyLabel)
														.addComponent(
																this.searchTypeLabel,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																130,
																javax.swing.GroupLayout.PREFERRED_SIZE))
												.addGap(18, 18, 18)
												.addGroup(jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.searchText,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																659,
																Short.MAX_VALUE)
														.addComponent(
																this.candidateOrPartySelectionComboBox,
																0, 659,
																Short.MAX_VALUE))
												.addGap(18, 18, 18)
												.addGroup(jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.searchEligibleOptionsButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																152,
																Short.MAX_VALUE)
														.addComponent(
																this.advancedSearchButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																152,
																Short.MAX_VALUE))))
								.addGap(39, 39, 39)));
		jPanel1Layout.setVerticalGroup(jPanel1Layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(jPanel1Layout.createSequentialGroup()
						.addContainerGap()
						.addGroup(jPanel1Layout.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
								.addComponent(this.advancedSearchButton)
								.addGroup(jPanel1Layout.createSequentialGroup()
										.addGap(2, 2, 2)
										.addGroup(jPanel1Layout
												.createParallelGroup(
														javax.swing.GroupLayout.Alignment.BASELINE)
												.addComponent(
														this.candidateOrPartySelectionComboBox,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addComponent(
														this.searchTypeLabel))))
						.addGap(16, 16, 16)
						.addGroup(jPanel1Layout.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
								.addGroup(jPanel1Layout.createParallelGroup(
										javax.swing.GroupLayout.Alignment.BASELINE)
										.addComponent(this.candidatePartyLabel)
										.addComponent(
												this.searchEligibleOptionsButton))
								.addGroup(jPanel1Layout.createSequentialGroup()
										.addGap(2, 2, 2)
										.addComponent(this.searchText,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)))
						.addGap(18, 18, 18)
						.addComponent(this.mainScrollPane,
								javax.swing.GroupLayout.DEFAULT_SIZE, 273,
								Short.MAX_VALUE)
						.addPreferredGap(
								javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addGroup(jPanel1Layout.createParallelGroup(
								javax.swing.GroupLayout.Alignment.BASELINE)
								.addComponent(this.paginationLabel)
								.addComponent(this.nextPageButton)
								.addComponent(this.prevPageButton)
								.addComponent(this.exportLogButton))
						.addPreferredGap(
								javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
						.addComponent(this.clearResultsButton,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								javax.swing.GroupLayout.DEFAULT_SIZE,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addContainerGap()));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.mainPanel,
						javax.swing.GroupLayout.DEFAULT_SIZE, 1028,
						Short.MAX_VALUE)
				.addComponent(this.mainLayeredPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 1028,
						Short.MAX_VALUE));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(layout.createSequentialGroup()
						.addComponent(this.mainLayeredPane,
								javax.swing.GroupLayout.PREFERRED_SIZE, 124,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(
								javax.swing.LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(this.mainPanel,
								javax.swing.GroupLayout.DEFAULT_SIZE, 481,
								Short.MAX_VALUE)));
	}

	private static final long serialVersionUID = -3511513105093185658L;

	private javax.swing.JButton advancedSearchButton;

	private javax.swing.JLabel functionalityBannerBackgroundLabel;

	private javax.swing.JTable elementsTable;

	private int defaultRowHeight;

	private javax.swing.JLabel candidatePartyLabel;

	private javax.swing.JButton clearResultsButton;

	private javax.swing.JButton exportLogButton;

	private javax.swing.JComboBox candidateOrPartySelectionComboBox;

	private javax.swing.JLabel iconLabel;

	private javax.swing.JLabel functionalityTitleLabel;

	private javax.swing.JLabel functionalityDescriptionLabel;

	private javax.swing.JLabel searchTypeLabel;

	private javax.swing.JLayeredPane mainLayeredPane;

	private javax.swing.JPanel mainPanel;

	private javax.swing.JScrollPane mainScrollPane;

	private javax.swing.JButton nextPageButton;

	private javax.swing.JButton prevPageButton;

	private javax.swing.JLabel paginationLabel;

	private javax.swing.JButton searchEligibleOptionsButton;

	private javax.swing.JTextField searchText;

	private javax.swing.JFileChooser exportLogFileChooser;

	private ResourceBundle resourceBundle;

	private HashMap<Integer, EligibleOptionData> searchResult;

	private EligibleOptionAdvancedSearchPanel advancedSearchPanel;

	private JDialog dialogPanel;

	private MouseAdapter imageLogoMouseListener;

	private int currentPage;

	private int totalNumberOfPages;

	private String currentText;

	private List<RegionData> currentRegions;

	private List<ContestData> currentContests;

	private List<PartyData> currentParties;

	private JTextComponent.KeyBinding[] newBindings = {
			new JTextComponent.KeyBinding(
					KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK),
					DefaultEditorKit.beepAction),
			new JTextComponent.KeyBinding(
					KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK),
					DefaultEditorKit.beepAction),
			new JTextComponent.KeyBinding(
					KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK),
					DefaultEditorKit.beepAction) };

	private static SMTTLogger logger = SMTTLogger
			.getLogger(EligibleOptionEditorPanel.class);

}
