/**
 * ImagesUploadGUIComponents.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;

import com.smartmatic.waffle.ecm.client.ui.common.ImageType;

/**
 * Class used to transfer GUI references that needs to be updates by
 * controllers.
 */
public class ImagesUploadGUIComponents {

	/**
	 * Instantiates a new images upload gui components.
	 * 
	 * @param imageType
	 *            OS image type
	 * @param jStatusDetailImageLabel
	 *            jStatusDetailImageLabel reference
	 * @param jFileHashDetailImageLabel
	 *            jFileHashDetailImageLabel reference
	 * @param jSearchImageButton
	 *            jSearchImageButton reference
	 * @param jUploadImageButton
	 *            jUploadImageButton reference
	 * @param jImageProgressBar
	 *            jImageProgressBar reference
	 * @param jLabelValidationResult
	 *            jLabelValidationResult reference
	 * @param jImageFileChooser
	 *            jImageFileChooser reference
	 * @param jImageTextFieldImage
	 *            jImageTextFieldImage reference
	 */
	public ImagesUploadGUIComponents(ImageType imageType,
			JLabel jStatusDetailImageLabel, JLabel jFileHashDetailImageLabel,
			JButton jSearchImageButton, JButton jUploadImageButton,
			JProgressBar jImageProgressBar, JLabel jLabelValidationResult,
			JFileChooser jImageFileChooser, JTextField jImageTextFieldImage) {
		this.imageType = imageType;
		this.jStatusDetailImageLabel = jStatusDetailImageLabel;
		this.jFileHashDetailImageLabel = jFileHashDetailImageLabel;
		this.jSearchImageButton = jSearchImageButton;
		this.jUploadImageButton = jUploadImageButton;
		this.jImageProgressBar = jImageProgressBar;
		this.jLabelValidationResult = jLabelValidationResult;
		this.jImageFileChooser = jImageFileChooser;
		this.jImageTextFieldImage = jImageTextFieldImage;
		this.imageUploaded = false;

	}

	/**
	 * Gets the image type.
	 * 
	 * @return the image type
	 */
	public ImageType getImageType() {
		return this.imageType;
	}

	/**
	 * Gets the file hash detail image label.
	 * 
	 * @return the file hash detail image label
	 */
	public JLabel getjFileHashDetailImageLabel() {
		return this.jFileHashDetailImageLabel;
	}

	/**
	 * Gets the image file chooser.
	 * 
	 * @return the image file chooser
	 */
	public JFileChooser getjImageFileChooser() {
		return this.jImageFileChooser;
	}

	/**
	 * Gets the image progress bar.
	 * 
	 * @return the image progress bar
	 */
	public JProgressBar getjImageProgressBar() {
		return this.jImageProgressBar;
	}

	/**
	 * Gets the image text field image.
	 * 
	 * @return the image text field image
	 */
	public JTextField getjImageTextFieldImage() {
		return this.jImageTextFieldImage;
	}

	/**
	 * Gets the label validation result.
	 * 
	 * @return the label validation result
	 */
	public JLabel getjLabelValidationResult() {
		return this.jLabelValidationResult;
	}

	/**
	 * Gets the search image button.
	 * 
	 * @return the search image button
	 */
	public JButton getjSearchImageButton() {
		return this.jSearchImageButton;
	}

	/**
	 * Gets the status detail image label.
	 * 
	 * @return the status detail image label
	 */
	public JLabel getjStatusDetailImageLabel() {
		return this.jStatusDetailImageLabel;
	}

	/**
	 * Gets the upload image button.
	 * 
	 * @return the upload image button
	 */
	public JButton getjUploadImageButton() {
		return this.jUploadImageButton;
	}

	/**
	 * Checks if is image uploaded.
	 * 
	 * @return true, if is image uploaded
	 */
	public boolean isImageUploaded() {
		return this.imageUploaded;
	}

	/**
	 * Sets the image uploaded.
	 * 
	 * @param imageUploaded
	 *            the new image uploaded
	 */
	public void setImageUploaded(boolean imageUploaded) {
		this.imageUploaded = imageUploaded;
	}

	/** OS image type. */
	private ImageType imageType;

	/** Flag to know if the image is already uploaded. */
	private boolean imageUploaded;

	/** jFileHashDetailImageLabel reference. */
	private JLabel jFileHashDetailImageLabel;

	/** jImageFileChooser reference. */
	private JFileChooser jImageFileChooser;

	/** jImageProgressBar reference. */
	private JProgressBar jImageProgressBar;

	/** jImageTextFieldImage reference. */
	private JTextField jImageTextFieldImage;

	/** jLabelValidationResult reference. */
	private JLabel jLabelValidationResult;

	/** jSearchImageButton reference. */
	private JButton jSearchImageButton;

	/** jStatusDetailImageLabel reference. */
	private JLabel jStatusDetailImageLabel;

	/** jUploadImageButton reference. */
	private JButton jUploadImageButton;
}
