/**
 * ImagesUploadPanel.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.ui;

import java.awt.Color;

import javax.swing.JFileChooser;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.ui.common.ImageType;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;

/**
 * The Class is used to load and draw all the necessary components for the
 * ImagesUploadPanel.
 */
public class ImagesUploadPanel extends javax.swing.JPanel {

	/**
	 * Creates new ImagesUploadPanel instance.
	 */
	public ImagesUploadPanel() {
		this.bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		this.defaultDestinations = SAESLocalPersistenceManager.getInstance()
				.getDefaultDestinations();
		initComponents();
		createImagesUploadGUIComponentsComponents();
		initStatus();

		if (!ECMClientSideConstants.GENERATE_TOTALIZATION_DEVICE) {
			this.jCanvassingStationImagePanel.setVisible(false);
		}
	}

	/**
	 * Updates the images status and calculate the sha256sum.
	 * 
	 * @param guiComponents
	 *            the gui components
	 */
	public void requestImageStatus(ImagesUploadGUIComponents guiComponents) {

		guiComponents.getjStatusDetailImageLabel().setForeground(Color.ORANGE);
		guiComponents
				.getjStatusDetailImageLabel()
				.setText(
						this.bundle
								.getString("ImagesUploadPanel.status.votingdeviceimage.updating"));
		guiComponents.getjFileHashDetailImageLabel().setText(StringUtils.EMPTY);

		ImagesUploadHashGetterController imagesUploadHashGetterController = new ImagesUploadHashGetterController(
				this, this.defaultDestinations, guiComponents);

		imagesUploadHashGetterController.startWorker();
	}

	/**
	 * * Update the status of the voting device image file.
	 * 
	 * @param imageHash
	 *            sha256sum image hash
	 * @param imagesUploadGUIComponents
	 *            ImagesUploadGUIComponents reference to update GUI components.
	 */
	public void updateImageStatus(String imageHash,
			ImagesUploadGUIComponents imagesUploadGUIComponents) {

		Color colorOfMessage;
		String messageToShow;

		if (imageHash != null) {
			colorOfMessage = Color.GREEN;
			messageToShow = this.bundle
					.getString("ImagesUploadPanel.status.votingdeviceimage.loaded");
			imagesUploadGUIComponents.setImageUploaded(true);
		} else {
			colorOfMessage = Color.RED;
			messageToShow = this.bundle
					.getString("ImagesUploadPanel.status.votingdeviceimage.notloaded");
		}

		imagesUploadGUIComponents.getjStatusDetailImageLabel().setText(
				messageToShow);
		imagesUploadGUIComponents.getjStatusDetailImageLabel().setForeground(
				colorOfMessage);

		imagesUploadGUIComponents.getjFileHashDetailImageLabel().setText(
				StringUtils.defaultString(imageHash));
		imagesUploadGUIComponents.getjSearchImageButton().setEnabled(true);
		imagesUploadGUIComponents.getjImageProgressBar().setValue(0);
	}

	/**
	 * * Updates the progress bar.
	 * 
	 * @param value
	 *            Progress bar value to update the control
	 * @param imagesUploadGUIComponents
	 *            ImagesUploadGUIComponents reference to update GUI components.
	 */
	public void updateProgressBar(int value,
			ImagesUploadGUIComponents imagesUploadGUIComponents) {
		imagesUploadGUIComponents.getjImageProgressBar().setValue(value);
	}

	/**
	 * Update GUI components with the validation result.
	 * 
	 * @param isValidImage
	 *            True is sent if is a valid image, otherwise false
	 * @param imagesUploadGUIComponents
	 *            ImagesUploadGUIComponents reference to update GUI components.
	 */
	public void updateComponentsAfterValidation(boolean isValidImage,
			ImagesUploadGUIComponents imagesUploadGUIComponents) {

		Color colorOfMessage;
		String messageToShow;

		if (isValidImage) {
			messageToShow = this.bundle
					.getString("ImagesUploadPanel.message.valid.iso");
			colorOfMessage = Color.GREEN;

			imagesUploadGUIComponents.getjSearchImageButton().setEnabled(true);
			imagesUploadGUIComponents.getjUploadImageButton().setEnabled(true);
		} else {
			messageToShow = this.bundle
					.getString("ImagesUploadPanel.message.invalid.iso");
			colorOfMessage = Color.RED;

			imagesUploadGUIComponents.getjImageFileChooser().setSelectedFile(
					null);
			imagesUploadGUIComponents.getjImageTextFieldImage().setText(
					StringUtils.EMPTY);
			imagesUploadGUIComponents.getjSearchImageButton().setEnabled(true);
			imagesUploadGUIComponents.getjUploadImageButton().setEnabled(false);
		}

		imagesUploadGUIComponents.getjLabelValidationResult().setText(
				messageToShow);
		imagesUploadGUIComponents.getjLabelValidationResult().setForeground(
				colorOfMessage);
		imagesUploadGUIComponents.getjLabelValidationResult().setVisible(true);

		ECMServiceProvider.setWorkflowButtonsEnabled(true);
	}

	/**
	 * Update GUI components with the upload result.
	 * 
	 * @param isValidImage
	 *            True is sent if is a valid image, otherwise false
	 * @param imagesUploadGUIComponents
	 *            ImagesUploadGUIComponents reference to update GUI components.
	 */
	public void updateComponentsAfterUpload(
			ImagesUploadGUIComponents guiComponents) {
		ECMServiceProvider.setWorkflowButtonsEnabled(true);
		requestImageStatus(guiComponents);
	}

	/**
	 * Hide and refresh components after validation.
	 * 
	 * @param imagesUploadGUIComponents
	 *            ImagesUploadGUIComponents reference to update GUI components.
	 */
	public void hideComponentsAfterValidation(
			ImagesUploadGUIComponents imagesUploadGUIComponents) {
		imagesUploadGUIComponents.getjLabelValidationResult().setVisible(false);
		imagesUploadGUIComponents.getjImageProgressBar().setValue(0);
	}

	/**
	 * Method that dispose a message box showing a generic error message.
	 */
	public void showGenericErrorMessage() {
		ECMOptionPane.showMessageDialog(this,
				this.bundle.getString("ErrorMessage.GENERIC_ERROR"),
				this.bundle.getString("ErrorMessage.title"),
				ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
		ECMServiceProvider.setWorkflowButtonsEnabled(true);
	}

	/**
	 * Disables the voting device image panel buttons.
	 */
	private void disableVotingDeviceImageButton() {
		this.jSearchVotingDeviceImageButton.setEnabled(false);
		this.jUploadVotingDeviceImageButton.setEnabled(false);
	}

	/**
	 * Disables the voting device feature image panel buttons.
	 */
	private void disableVotingDeviceFeatureImageButton() {
		this.jVotingDeviceFeatureImageSearchButton.setEnabled(false);
		this.jVotingDeviceFeatureImageUploadButton.setEnabled(false);
	}

	/**
	 * Disables the canvassing station image panel buttons.
	 */
	private void disableCanvassingStationImageButton() {
		this.jCanvassingStationImageSearchButton.setEnabled(false);
		this.jCanvassingStationImageUploadButton.setEnabled(false);
	}

	/**
	 * Creates ImagesUploadGUIComponents references to update GUI controls in
	 * controllers.
	 */
	private void createImagesUploadGUIComponentsComponents() {
		this.votingDeviceImageComponents = new ImagesUploadGUIComponents(
				ImageType.OFFICIAL, this.jStatusDetailVotingDeviceImageLabel,
				this.jFileHashDetailVotingDeviceImageLabel,
				this.jSearchVotingDeviceImageButton,
				this.jUploadVotingDeviceImageButton,
				this.jVotingDeviceImageProgressBar,
				this.labelVotingDeviceValidationResult,
				this.jFileVotingDeviceImageChooser,
				this.jTextFieldVotingDeviceImage);

		this.votingDeviceImageFeatureComponents = new ImagesUploadGUIComponents(
				ImageType.SCREEN_GENERATION,
				this.jStatusDetailVotingDeviceFeatureImageLabel,
				this.jFileHashDetailVotingDeviceFeatureImageLabel,
				this.jVotingDeviceFeatureImageSearchButton,
				this.jVotingDeviceFeatureImageUploadButton,
				this.jVotingDeviceFeatureImageProgressBar,
				this.labelVotingDeviceFeatureValidationResult,
				this.jFileVotingDeviceFeatureImageChooser,
				this.jTextFieldVotingDeviceFeatureImage);

		this.canvassingStationComponents = new ImagesUploadGUIComponents(
				ImageType.CANVASSING_STATION,
				this.jStatusDetailCanvassingStationImageLabel,
				this.jFileHashDetailCanvassingStationImageLabel,
				this.jCanvassingStationImageSearchButton,
				this.jCanvassingStationImageUploadButton,
				this.jCanvassingStationImageProgressBar,
				this.labelCanvassingStationValidationResult,
				this.jFileCanvassingStationImageChooser,
				this.jTextFieldCanvassingStationImage);
	}

	/***
	 * Initializes the different statuses when the panel is instantiated.
	 */
	private void initStatus() {
		requestImageStatus(this.votingDeviceImageComponents);
		this.jUploadVotingDeviceImageButton.setEnabled(false);
		this.jVotingDeviceImageProgressBar.setMaximum(100);

		requestImageStatus(this.votingDeviceImageFeatureComponents);
		this.jVotingDeviceFeatureImageUploadButton.setEnabled(false);
		this.jVotingDeviceFeatureImageProgressBar.setMaximum(100);

		requestImageStatus(this.canvassingStationComponents);
		this.jCanvassingStationImageUploadButton.setEnabled(false);
		this.jCanvassingStationImageProgressBar.setMaximum(100);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		this.jVotingDeviceImagePanel = new javax.swing.JPanel();
		this.jTextFieldVotingDeviceImage = new javax.swing.JTextField();
		this.jVotingDeviceImageProgressBar = new javax.swing.JProgressBar();
		this.jSearchVotingDeviceImageButton = new javax.swing.JButton();
		this.jUploadVotingDeviceImageButton = new javax.swing.JButton();
		this.jStatusVotingDeviceImageLabel = new javax.swing.JLabel();
		this.jFileHashVotingDeviceImageLabel = new javax.swing.JLabel();
		this.jFileHashDetailVotingDeviceImageLabel = new javax.swing.JLabel();
		this.jStatusDetailVotingDeviceImageLabel = new javax.swing.JLabel();
		this.labelVotingDeviceValidationResult = new javax.swing.JLabel();
		this.jCanvassingStationImagePanel = new javax.swing.JPanel();
		this.jTextFieldCanvassingStationImage = new javax.swing.JTextField();
		this.jCanvassingStationImageProgressBar = new javax.swing.JProgressBar();
		this.jCanvassingStationImageSearchButton = new javax.swing.JButton();
		this.jCanvassingStationImageUploadButton = new javax.swing.JButton();
		this.jStatusCanvassingStationImageLabel = new javax.swing.JLabel();
		this.jFileHashCanvassingStationImageLabel = new javax.swing.JLabel();
		this.jStatusDetailCanvassingStationImageLabel = new javax.swing.JLabel();
		this.jFileHashDetailCanvassingStationImageLabel = new javax.swing.JLabel();
		this.labelCanvassingStationValidationResult = new javax.swing.JLabel();
		this.jFileVotingDeviceImageChooser = new javax.swing.JFileChooser();
		this.jFileCanvassingStationImageChooser = new javax.swing.JFileChooser();
		this.bannerBackgroundLayeredPane = new javax.swing.JLayeredPane();
		this.iconLogoLabel = new javax.swing.JLabel();
		this.mainTitleLabel = new javax.swing.JLabel();
		this.mainDescriptionLabel = new javax.swing.JLabel();
		this.bannerBackgroundLabel = new javax.swing.JLabel();
		this.jCanvassingVotingDeviceFeatureImagePanel = new javax.swing.JPanel();
		this.jTextFieldVotingDeviceFeatureImage = new javax.swing.JTextField();
		this.jVotingDeviceFeatureImageProgressBar = new javax.swing.JProgressBar();
		this.jVotingDeviceFeatureImageSearchButton = new javax.swing.JButton();
		this.jVotingDeviceFeatureImageUploadButton = new javax.swing.JButton();
		this.jStatusVotingDeviceFeatureImageLabel = new javax.swing.JLabel();
		this.jFileHashVotingDeviceFeatureImageLabel = new javax.swing.JLabel();
		this.jStatusDetailVotingDeviceFeatureImageLabel = new javax.swing.JLabel();
		this.jFileHashDetailVotingDeviceFeatureImageLabel = new javax.swing.JLabel();
		this.labelVotingDeviceFeatureValidationResult = new javax.swing.JLabel();
		this.jFileVotingDeviceFeatureImageChooser = new javax.swing.JFileChooser();

		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		this.jVotingDeviceImagePanel
				.setBorder(javax.swing.BorderFactory.createTitledBorder(
						javax.swing.BorderFactory.createTitledBorder(""),
						bundle.getString("ImagesUploadPanel.jVotingDeviceImagePanel.border.title"))); // NOI18N
		this.jVotingDeviceImagePanel.setName("panelVotingDeviceImage"); // NOI18N

		this.jTextFieldVotingDeviceImage.setEnabled(false);
		this.jTextFieldVotingDeviceImage.setName("textFieldVotingDeviceImage"); // NOI18N

		this.jVotingDeviceImageProgressBar.setBorderPainted(false);
		this.jVotingDeviceImageProgressBar
				.setName("progressBarVotingDeviceImage"); // NOI18N
		this.jVotingDeviceImageProgressBar.setStringPainted(true);

		this.jSearchVotingDeviceImageButton
				.setText(bundle
						.getString("ImagesUploadPanel.jSearchVotingDeviceImageButton.text")); // NOI18N
		this.jSearchVotingDeviceImageButton
				.setName("buttonSearchVotingDeviceImage"); // NOI18N
		this.jSearchVotingDeviceImageButton
				.setPreferredSize(new java.awt.Dimension(80, 30));
		this.jSearchVotingDeviceImageButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						jSearchVotingDeviceImageButtonActionPerformed(evt);
					}
				});

		this.jUploadVotingDeviceImageButton
				.setText(bundle
						.getString("ImagesUploadPanel.jUploadVotingDeviceImageButton.text")); // NOI18N
		this.jUploadVotingDeviceImageButton
				.setName("buttonUploadVotingDeviceImage"); // NOI18N
		this.jUploadVotingDeviceImageButton
				.setPreferredSize(new java.awt.Dimension(80, 30));
		this.jUploadVotingDeviceImageButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						jUploadVotingDeviceImageButtonActionPerformed(evt);
					}
				});

		this.jStatusVotingDeviceImageLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 12));
		this.jStatusVotingDeviceImageLabel
				.setText(bundle
						.getString("ImagesUploadPanel.jStatusVotingDeviceImageLabel.text")); // NOI18N
		this.jStatusVotingDeviceImageLabel
				.setName("labelStatusVotingDeviceImage"); // NOI18N

		this.jFileHashVotingDeviceImageLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 12));
		this.jFileHashVotingDeviceImageLabel
				.setText(bundle
						.getString("ImagesUploadPanel.jFileHashVotingDeviceImageLabel.text")); // NOI18N
		this.jFileHashVotingDeviceImageLabel
				.setName("labelFileHashVotingDeviceImage"); // NOI18N

		this.jFileHashDetailVotingDeviceImageLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 12));
		this.jFileHashDetailVotingDeviceImageLabel
				.setName("labelFileHashDetailVotingDeviceImage"); // NOI18N

		this.jStatusDetailVotingDeviceImageLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 12));
		this.jStatusDetailVotingDeviceImageLabel
				.setName("labelStatusDetailVotingDeviceImage"); // NOI18N

		this.labelVotingDeviceValidationResult
				.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
		this.labelVotingDeviceValidationResult
				.setName("labelVotingDeviceValidationResult"); // NOI18N

		javax.swing.GroupLayout jVotingDeviceImagePanelLayout = new javax.swing.GroupLayout(
				this.jVotingDeviceImagePanel);
		this.jVotingDeviceImagePanel.setLayout(jVotingDeviceImagePanelLayout);
		jVotingDeviceImagePanelLayout
				.setHorizontalGroup(jVotingDeviceImagePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jVotingDeviceImagePanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												jVotingDeviceImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.jVotingDeviceImageProgressBar,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																908,
																Short.MAX_VALUE)
														.addComponent(
																this.jTextFieldVotingDeviceImage,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																908,
																Short.MAX_VALUE)
														.addGroup(
																jVotingDeviceImagePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				this.jStatusVotingDeviceImageLabel)
																		.addGap(28,
																				28,
																				28)
																		.addComponent(
																				this.jStatusDetailVotingDeviceImageLabel)
																		.addGap(608,
																				608,
																				608)
																		.addComponent(
																				this.labelVotingDeviceValidationResult,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				224,
																				Short.MAX_VALUE))
														.addComponent(
																this.jFileHashVotingDeviceImageLabel)
														.addGroup(
																jVotingDeviceImagePanelLayout
																		.createSequentialGroup()
																		.addGap(153,
																				153,
																				153)
																		.addComponent(
																				this.jFileHashDetailVotingDeviceImageLabel)))
										.addGap(18, 18, 18)
										.addGroup(
												jVotingDeviceImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.TRAILING,
																false)
														.addComponent(
																this.jUploadVotingDeviceImageButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																this.jSearchVotingDeviceImageButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE))
										.addContainerGap()));
		jVotingDeviceImagePanelLayout
				.setVerticalGroup(jVotingDeviceImagePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jVotingDeviceImagePanelLayout
										.createSequentialGroup()
										.addGroup(
												jVotingDeviceImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.jTextFieldVotingDeviceImage,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.jSearchVotingDeviceImageButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												jVotingDeviceImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																jVotingDeviceImagePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				this.jVotingDeviceImageProgressBar,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				Short.MAX_VALUE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				jVotingDeviceImagePanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.BASELINE)
																						.addComponent(
																								this.jStatusDetailVotingDeviceImageLabel)
																						.addComponent(
																								this.jStatusVotingDeviceImageLabel)
																						.addComponent(
																								this.labelVotingDeviceValidationResult))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				Short.MAX_VALUE)
																		.addGroup(
																				jVotingDeviceImagePanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addComponent(
																								this.jFileHashDetailVotingDeviceImageLabel)
																						.addComponent(
																								this.jFileHashVotingDeviceImageLabel,
																								javax.swing.GroupLayout.Alignment.TRAILING)))
														.addComponent(
																this.jUploadVotingDeviceImageButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))));

		this.labelVotingDeviceValidationResult.getAccessibleContext()
				.setAccessibleName("labelVotingDeviceValidationResult"); // NOI18N

		this.jCanvassingStationImagePanel
				.setBorder(javax.swing.BorderFactory.createTitledBorder(
						javax.swing.BorderFactory.createTitledBorder(""),
						bundle.getString("ImagesUploadPanel.jCanvassingStationImagePanel.border.title"))); // NOI18N
		this.jCanvassingStationImagePanel
				.setName("panelCanvassingStationImage"); // NOI18N

		this.jTextFieldCanvassingStationImage.setEnabled(false);
		this.jTextFieldCanvassingStationImage
				.setName("textFieldCanvassingStationImage"); // NOI18N

		this.jCanvassingStationImageProgressBar
				.setName("progressBarCanvassingStationImage"); // NOI18N
		this.jCanvassingStationImageProgressBar.setStringPainted(true);

		this.jCanvassingStationImageSearchButton
				.setText(bundle
						.getString("ImagesUploadPanel.jCanvassingStationImageSearchButton.text")); // NOI18N
		this.jCanvassingStationImageSearchButton
				.setName("buttonCanvassingStationImageSearch"); // NOI18N
		this.jCanvassingStationImageSearchButton
				.setPreferredSize(new java.awt.Dimension(80, 30));

		this.jCanvassingStationImageUploadButton
				.setText(bundle
						.getString("ImagesUploadPanel.jCanvassingStationImageUploadButton.text")); // NOI18N
		this.jCanvassingStationImageUploadButton
				.setName("buttonCanvassingStationImageUpload"); // NOI18N
		this.jCanvassingStationImageUploadButton
				.setPreferredSize(new java.awt.Dimension(80, 30));
		this.jCanvassingStationImageUploadButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						jCanvassingStationImageUploadButtonActionPerformed(evt);
					}
				});

		this.jStatusCanvassingStationImageLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 12));
		this.jStatusCanvassingStationImageLabel
				.setText(bundle
						.getString("ImagesUploadPanel.jStatusCanvassingStationImageLabel.text")); // NOI18N
		this.jStatusCanvassingStationImageLabel
				.setName("labelStatusCanvassingStationImage"); // NOI18N

		this.jFileHashCanvassingStationImageLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 12));
		this.jFileHashCanvassingStationImageLabel
				.setText(bundle
						.getString("ImagesUploadPanel.jFileHashCanvassingStationImageLabel.text")); // NOI18N
		this.jFileHashCanvassingStationImageLabel
				.setName("labelFileHashCanvassingStationImage"); // NOI18N

		this.jStatusDetailCanvassingStationImageLabel
				.setFont(new java.awt.Font("DejaVu Sans", 0, 12));
		this.jStatusDetailCanvassingStationImageLabel
				.setName("labelStatusDetailCanvassingStationImage"); // NOI18N

		this.jFileHashDetailCanvassingStationImageLabel
				.setFont(new java.awt.Font("DejaVu Sans", 0, 12));
		this.jFileHashDetailCanvassingStationImageLabel
				.setName("labelFileHashDetailCanvassingStationImage"); // NOI18N

		this.labelCanvassingStationValidationResult
				.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
		this.labelCanvassingStationValidationResult
				.setName("labelCanvassingStationValidationResult"); // NOI18N

		javax.swing.GroupLayout jCanvassingStationImagePanelLayout = new javax.swing.GroupLayout(
				this.jCanvassingStationImagePanel);
		this.jCanvassingStationImagePanel
				.setLayout(jCanvassingStationImagePanelLayout);
		jCanvassingStationImagePanelLayout
				.setHorizontalGroup(jCanvassingStationImagePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jCanvassingStationImagePanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												jCanvassingStationImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.jCanvassingStationImageProgressBar,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																914,
																Short.MAX_VALUE)
														.addComponent(
																this.jTextFieldCanvassingStationImage,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																914,
																Short.MAX_VALUE)
														.addGroup(
																jCanvassingStationImagePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				this.jFileHashCanvassingStationImageLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																		.addComponent(
																				this.jFileHashDetailCanvassingStationImageLabel))
														.addGroup(
																jCanvassingStationImagePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				this.jStatusCanvassingStationImageLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																				676,
																				Short.MAX_VALUE)
																		.addComponent(
																				this.labelCanvassingStationValidationResult,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				190,
																				javax.swing.GroupLayout.PREFERRED_SIZE))
														.addGroup(
																jCanvassingStationImagePanelLayout
																		.createSequentialGroup()
																		.addGap(73,
																				73,
																				73)
																		.addComponent(
																				this.jStatusDetailCanvassingStationImageLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																				841,
																				Short.MAX_VALUE)))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addGroup(
												jCanvassingStationImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING,
																false)
														.addComponent(
																this.jCanvassingStationImageUploadButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																this.jCanvassingStationImageSearchButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																92,
																Short.MAX_VALUE))
										.addContainerGap()));
		jCanvassingStationImagePanelLayout
				.setVerticalGroup(jCanvassingStationImagePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jCanvassingStationImagePanelLayout
										.createSequentialGroup()
										.addGroup(
												jCanvassingStationImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.jTextFieldCanvassingStationImage,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.jCanvassingStationImageSearchButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												jCanvassingStationImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																jCanvassingStationImagePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				this.jCanvassingStationImageProgressBar,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				jCanvassingStationImagePanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addGroup(
																								jCanvassingStationImagePanelLayout
																										.createParallelGroup(
																												javax.swing.GroupLayout.Alignment.BASELINE)
																										.addComponent(
																												this.jStatusCanvassingStationImageLabel)
																										.addComponent(
																												this.labelCanvassingStationValidationResult))
																						.addComponent(
																								this.jStatusDetailCanvassingStationImageLabel))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				jCanvassingStationImagePanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.BASELINE)
																						.addComponent(
																								this.jFileHashCanvassingStationImageLabel)
																						.addComponent(
																								this.jFileHashDetailCanvassingStationImageLabel)))
														.addComponent(
																this.jCanvassingStationImageUploadButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))));

		this.labelCanvassingStationValidationResult.getAccessibleContext()
				.setAccessibleName("jCanvassingVotingDeviceFeatureImagePanel"); // NOI18N

		this.jFileVotingDeviceImageChooser
				.setName("chooserFileVotingDeviceImage"); // NOI18N

		this.jFileCanvassingStationImageChooser
				.setName("chooserFileCanvassingStationImage"); // NOI18N

		this.bannerBackgroundLayeredPane.setMaximumSize(new java.awt.Dimension(
				1440, 124));
		this.bannerBackgroundLayeredPane.setMinimumSize(new java.awt.Dimension(
				1440, 124));
		this.bannerBackgroundLayeredPane
				.setName("paneFunctionalityInfoLayered"); // NOI18N

		this.iconLogoLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/carga_imagen.png"))); // NOI18N
		this.iconLogoLabel.setName("labelFunctionalityIcon"); // NOI18N
		this.iconLogoLabel.setBounds(20, 0, 120, 90);
		this.bannerBackgroundLayeredPane.add(this.iconLogoLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.mainTitleLabel.setFont(new java.awt.Font("DejaVu Sans", 1, 14));
		this.mainTitleLabel.setText(bundle
				.getString("ImagesUploadPanel.mainTitleLabel.text")); // NOI18N
		this.mainTitleLabel.setName("labelImagesUploadTitle"); // NOI18N
		this.mainTitleLabel.setBounds(170, 20, 410, 17);
		this.bannerBackgroundLayeredPane.add(this.mainTitleLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.mainDescriptionLabel.setFont(new java.awt.Font("DejaVu Sans", 0,
				11)); // NOI18N
		this.mainDescriptionLabel.setText(bundle
				.getString("ImagesUploadPanel.mainDescriptionLabel.text")); // NOI18N
		this.mainDescriptionLabel.setName("labelImagesUploadDescription"); // NOI18N
		this.mainDescriptionLabel.setBounds(190, 40, 690, 17);
		this.bannerBackgroundLayeredPane.add(this.mainDescriptionLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.bannerBackgroundLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/barra titulo subtitulo.jpg"))); // NOI18N
		this.bannerBackgroundLabel
				.setName("labelFunctionalityBannerBackground"); // NOI18N
		this.bannerBackgroundLabel.setPreferredSize(new java.awt.Dimension(
				1440, 120));
		this.bannerBackgroundLabel.setBounds(0, 0, 1450, 120);
		this.bannerBackgroundLayeredPane.add(this.bannerBackgroundLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.jCanvassingVotingDeviceFeatureImagePanel
				.setBorder(javax.swing.BorderFactory.createTitledBorder(
						javax.swing.BorderFactory.createTitledBorder(""),
						bundle.getString("ImagesUploadPanel.jCanvassingVotingDeviceFeatureImagePanel.border.title"))); // NOI18N
		this.jCanvassingVotingDeviceFeatureImagePanel
				.setName("panelCanvassingVotingDeviceFeatureImage"); // NOI18N

		this.jTextFieldVotingDeviceFeatureImage.setEnabled(false);
		this.jTextFieldVotingDeviceFeatureImage
				.setName("textFieldVotingDeviceFeatureImage"); // NOI18N

		this.jVotingDeviceFeatureImageProgressBar
				.setName("progressBarVotingDeviceFeatureImage"); // NOI18N
		this.jVotingDeviceFeatureImageProgressBar.setStringPainted(true);

		this.jVotingDeviceFeatureImageSearchButton
				.setText(bundle
						.getString("ImagesUploadPanel.jVotingDeviceFeatureImageSearchButton.text")); // NOI18N
		this.jVotingDeviceFeatureImageSearchButton
				.setName("buttonVotingDeviceFeatureImageSearch"); // NOI18N
		this.jVotingDeviceFeatureImageSearchButton
				.setPreferredSize(new java.awt.Dimension(80, 30));
		this.jVotingDeviceFeatureImageSearchButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						jVotingDeviceFeatureImageSearchButtonActionPerformed(evt);
					}
				});

		this.jVotingDeviceFeatureImageUploadButton
				.setText(bundle
						.getString("ImagesUploadPanel.jVotingDeviceFeatureImageUploadButton.text")); // NOI18N
		this.jVotingDeviceFeatureImageUploadButton
				.setName("buttonVotingDeviceFeatureImageUpload"); // NOI18N
		this.jVotingDeviceFeatureImageUploadButton
				.setPreferredSize(new java.awt.Dimension(80, 30));
		this.jVotingDeviceFeatureImageUploadButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						jVotingDeviceFeatureImageUploadButtonActionPerformed(evt);
					}
				});

		this.jStatusVotingDeviceFeatureImageLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 12));
		this.jStatusVotingDeviceFeatureImageLabel
				.setText(bundle
						.getString("ImagesUploadPanel.jStatusVotingDeviceFeatureImageLabel.text")); // NOI18N
		this.jStatusVotingDeviceFeatureImageLabel
				.setName("labelStatusVotingDeviceFeatureImage"); // NOI18N

		this.jFileHashVotingDeviceFeatureImageLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 12));
		this.jFileHashVotingDeviceFeatureImageLabel
				.setText(bundle
						.getString("ImagesUploadPanel.jFileHashVotingDeviceFeatureImageLabel.text")); // NOI18N
		this.jFileHashVotingDeviceFeatureImageLabel
				.setName("labelFileHashVotingDeviceFeatureImage"); // NOI18N

		this.jStatusDetailVotingDeviceFeatureImageLabel
				.setFont(new java.awt.Font("DejaVu Sans", 0, 12));
		this.jStatusDetailVotingDeviceFeatureImageLabel
				.setName("labelStatusDetailVotingDeviceFeatureImage"); // NOI18N

		this.jFileHashDetailVotingDeviceFeatureImageLabel
				.setFont(new java.awt.Font("DejaVu Sans", 0, 12));
		this.jFileHashDetailVotingDeviceFeatureImageLabel
				.setName("labelFileHashDetailVotingDeviceFeatureImage"); // NOI18N

		this.labelVotingDeviceFeatureValidationResult
				.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
		this.labelVotingDeviceFeatureValidationResult
				.setName("labelVotingDeviceFeatureValidationResult"); // NOI18N

		javax.swing.GroupLayout jCanvassingVotingDeviceFeatureImagePanelLayout = new javax.swing.GroupLayout(
				this.jCanvassingVotingDeviceFeatureImagePanel);
		this.jCanvassingVotingDeviceFeatureImagePanel
				.setLayout(jCanvassingVotingDeviceFeatureImagePanelLayout);
		jCanvassingVotingDeviceFeatureImagePanelLayout
				.setHorizontalGroup(jCanvassingVotingDeviceFeatureImagePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jCanvassingVotingDeviceFeatureImagePanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												jCanvassingVotingDeviceFeatureImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.jVotingDeviceFeatureImageProgressBar,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																896,
																Short.MAX_VALUE)
														.addComponent(
																this.jTextFieldVotingDeviceFeatureImage,
																javax.swing.GroupLayout.Alignment.TRAILING,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																896,
																Short.MAX_VALUE)
														.addGroup(
																jCanvassingVotingDeviceFeatureImagePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				this.jFileHashVotingDeviceFeatureImageLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
																		.addComponent(
																				this.jFileHashDetailVotingDeviceFeatureImageLabel))
														.addGroup(
																jCanvassingVotingDeviceFeatureImagePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				this.jStatusVotingDeviceFeatureImageLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																				675,
																				Short.MAX_VALUE)
																		.addComponent(
																				this.labelVotingDeviceFeatureValidationResult,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				173,
																				javax.swing.GroupLayout.PREFERRED_SIZE))
														.addGroup(
																jCanvassingVotingDeviceFeatureImagePanelLayout
																		.createSequentialGroup()
																		.addGap(73,
																				73,
																				73)
																		.addComponent(
																				this.jStatusDetailVotingDeviceFeatureImageLabel)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED,
																				823,
																				Short.MAX_VALUE)))
										.addGap(18, 18, 18)
										.addGroup(
												jCanvassingVotingDeviceFeatureImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING,
																false)
														.addComponent(
																this.jVotingDeviceFeatureImageUploadButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																this.jVotingDeviceFeatureImageSearchButton,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																92,
																Short.MAX_VALUE))
										.addContainerGap()));
		jCanvassingVotingDeviceFeatureImagePanelLayout
				.setVerticalGroup(jCanvassingVotingDeviceFeatureImagePanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jCanvassingVotingDeviceFeatureImagePanelLayout
										.createSequentialGroup()
										.addGroup(
												jCanvassingVotingDeviceFeatureImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.BASELINE)
														.addComponent(
																this.jTextFieldVotingDeviceFeatureImage,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.jVotingDeviceFeatureImageSearchButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
										.addGroup(
												jCanvassingVotingDeviceFeatureImagePanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addGroup(
																jCanvassingVotingDeviceFeatureImagePanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				this.jVotingDeviceFeatureImageProgressBar,
																				javax.swing.GroupLayout.PREFERRED_SIZE,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				javax.swing.GroupLayout.PREFERRED_SIZE)
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				jCanvassingVotingDeviceFeatureImagePanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.LEADING)
																						.addGroup(
																								jCanvassingVotingDeviceFeatureImagePanelLayout
																										.createParallelGroup(
																												javax.swing.GroupLayout.Alignment.BASELINE)
																										.addComponent(
																												this.jStatusVotingDeviceFeatureImageLabel)
																										.addComponent(
																												this.labelVotingDeviceFeatureValidationResult))
																						.addComponent(
																								this.jStatusDetailVotingDeviceFeatureImageLabel))
																		.addPreferredGap(
																				javax.swing.LayoutStyle.ComponentPlacement.RELATED)
																		.addGroup(
																				jCanvassingVotingDeviceFeatureImagePanelLayout
																						.createParallelGroup(
																								javax.swing.GroupLayout.Alignment.BASELINE)
																						.addComponent(
																								this.jFileHashVotingDeviceFeatureImageLabel)
																						.addComponent(
																								this.jFileHashDetailVotingDeviceFeatureImageLabel)))
														.addComponent(
																this.jVotingDeviceFeatureImageUploadButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))));

		this.labelVotingDeviceFeatureValidationResult.getAccessibleContext()
				.setAccessibleName("labelVotingDeviceFeatureValidationResult"); // NOI18N

		this.jFileVotingDeviceFeatureImageChooser
				.setName("chooserVotingDeviceFeatureImageFile"); // NOI18N

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.bannerBackgroundLayeredPane,
						javax.swing.GroupLayout.Alignment.TRAILING,
						javax.swing.GroupLayout.PREFERRED_SIZE, 1074,
						Short.MAX_VALUE)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(this.jVotingDeviceImagePanel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addContainerGap())
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(
										this.jCanvassingStationImagePanel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addContainerGap())
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addComponent(
										this.jCanvassingVotingDeviceFeatureImagePanel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addContainerGap())
				.addGroup(
						layout.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
								.addGroup(
										layout.createSequentialGroup()
												.addGap(58, 58, 58)
												.addComponent(
														this.jFileVotingDeviceFeatureImageChooser,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														24,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addContainerGap(992,
														Short.MAX_VALUE)
												.addComponent(
														this.jFileVotingDeviceImageChooser,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														24,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addContainerGap(992,
														Short.MAX_VALUE)
												.addComponent(
														this.jFileCanvassingStationImageChooser,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														24,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addContainerGap(992,
														Short.MAX_VALUE))));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addComponent(this.bannerBackgroundLayeredPane,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										123,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.jVotingDeviceImagePanel,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(
										this.jCanvassingStationImagePanel,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(
										this.jCanvassingVotingDeviceFeatureImagePanel,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(
										javax.swing.LayoutStyle.ComponentPlacement.RELATED,
										8, Short.MAX_VALUE)
								.addGroup(
										layout.createParallelGroup(
												javax.swing.GroupLayout.Alignment.TRAILING)
												.addGroup(
														layout.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
																.addComponent(
																		this.jFileVotingDeviceImageChooser,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		9,
																		javax.swing.GroupLayout.PREFERRED_SIZE)
																.addComponent(
																		this.jFileCanvassingStationImageChooser,
																		javax.swing.GroupLayout.PREFERRED_SIZE,
																		9,
																		javax.swing.GroupLayout.PREFERRED_SIZE))))
				.addGroup(
						layout.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
								.addGroup(
										javax.swing.GroupLayout.Alignment.TRAILING,
										layout.createSequentialGroup()
												.addContainerGap(575,
														Short.MAX_VALUE)
												.addComponent(
														this.jFileVotingDeviceFeatureImageChooser,
														javax.swing.GroupLayout.PREFERRED_SIZE,
														9,
														javax.swing.GroupLayout.PREFERRED_SIZE)
												.addContainerGap())));
	}// </editor-fold>//GEN-END:initComponents

	/**
	 * Fires a file chooser to select the voting device image to be loaded then
	 * starts the process to validate the selected image.
	 * 
	 * @param evt
	 *            A semantic event which indicates that a component-defined
	 *            action occurred
	 */
	private void jSearchVotingDeviceImageButtonActionPerformed(
			java.awt.event.ActionEvent evt) {

		ClientCommon.initFileChooserLocation(
				this.jFileVotingDeviceImageChooser,
				ClientCommon.FILE_CHOOSER_ISO);

		int result = this.jFileVotingDeviceImageChooser.showOpenDialog(this);

		if (result == JFileChooser.APPROVE_OPTION) {

			ClientCommon.saveFileChooserPreference(
					this.jFileVotingDeviceImageChooser.getCurrentDirectory()
							.getAbsolutePath(), ClientCommon.FILE_CHOOSER_ISO);

			this.jTextFieldVotingDeviceImage.setText(String
					.valueOf(this.jFileVotingDeviceImageChooser
							.getSelectedFile()));

			if (ECMClientSideConstants.VALIDATE_FOLDERS_ISO_VOTING_MACHINE) {
				disableVotingDeviceImageButton();

				this.labelVotingDeviceValidationResult.setVisible(true);
				this.labelVotingDeviceValidationResult
						.setForeground(Color.ORANGE);
				this.labelVotingDeviceValidationResult.setText(this.bundle
						.getString("ImagesUploadPanel.message.loading.iso"));

				ECMServiceProvider.setWorkflowButtonsEnabled(false);

				ImagesUploadValidatorController imageUploadValidatorController = new ImagesUploadValidatorController(
						this,
						this.jFileVotingDeviceImageChooser.getSelectedFile(),
						ECMClientSideConstants.REQUIRED_FOLDERS_ISO_VOTING_MACHINE,
						this.votingDeviceImageComponents);

				imageUploadValidatorController.startWorker();
			} else {
				this.jUploadVotingDeviceImageButton.setEnabled(true);
			}

		}
	}// GEN-LAST:event_jSearchVotingDeviceImageButtonActionPerformed

	/**
	 * Fires a file chooser to select the voting device screenshot image to be
	 * loaded then starts the process to validate the selected image.
	 * 
	 * @param evt
	 *            A semantic event which indicates that a component-defined
	 *            action occurred
	 */
	private void jVotingDeviceFeatureImageSearchButtonActionPerformed(
			java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jVotingDeviceFeatureImageSearchButtonActionPerformed

		ClientCommon.initFileChooserLocation(
				this.jFileVotingDeviceFeatureImageChooser,
				ClientCommon.FILE_CHOOSER_ISO);

		int result = this.jFileVotingDeviceFeatureImageChooser
				.showOpenDialog(this);

		if (result == JFileChooser.APPROVE_OPTION) {

			ClientCommon.saveFileChooserPreference(
					this.jFileVotingDeviceFeatureImageChooser
							.getCurrentDirectory().getAbsolutePath(),
					ClientCommon.FILE_CHOOSER_ISO);

			this.jTextFieldVotingDeviceFeatureImage.setText(String
					.valueOf(this.jFileVotingDeviceFeatureImageChooser
							.getSelectedFile()));

			if (ECMClientSideConstants.VALIDATE_FOLDERS_ISO_SCREENSHOT) {
				disableVotingDeviceFeatureImageButton();

				this.labelVotingDeviceFeatureValidationResult.setVisible(true);
				this.labelVotingDeviceFeatureValidationResult
						.setForeground(Color.ORANGE);
				this.labelVotingDeviceFeatureValidationResult
						.setText(this.bundle
								.getString("ImagesUploadPanel.message.loading.iso"));

				ECMServiceProvider.setWorkflowButtonsEnabled(false);

				ImagesUploadValidatorController imageUploadValidatorController = new ImagesUploadValidatorController(
						this,
						this.jFileVotingDeviceFeatureImageChooser
								.getSelectedFile(),
						ECMClientSideConstants.REQUIRED_FOLDERS_ISO_SCREENSHOT,
						this.votingDeviceImageFeatureComponents);

				imageUploadValidatorController.startWorker();

			} else {
				this.jVotingDeviceFeatureImageUploadButton.setEnabled(true);
			}
		}

	}// GEN-LAST:event_jVotingDeviceFeatureImageSearchButtonActionPerformed

	/**
	 * Upload the selected voting device image. Then start the process to update
	 * the image status.
	 * 
	 * @param evt
	 *            A semantic event which indicates that a component-defined
	 *            action occurred
	 */
	private void jUploadVotingDeviceImageButtonActionPerformed(
			java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jUploadVotingDeviceImageButtonActionPerformed
		try {
			if (!this.jTextFieldVotingDeviceImage.getText().equals("")) {
				int result = 0;
				if (this.votingDeviceImageComponents.isImageUploaded()) {
					result = ECMOptionPane
							.showConfirmDialog(
									this,
									this.bundle
											.getString("ImagesUploadPanel.message.overwriteImage"),
									this.bundle
											.getString("WarningMessage.title"),
									ECMOptionPane.YES_NO_OPTION,
									ECMOptionPane.WARNING_MESSAGE,
									ECMOptionPane.CONFIRM_DIALOG);
				}
				if (result == 0) {
					this.jStatusDetailVotingDeviceImageLabel
							.setForeground(Color.BLACK);
					this.jStatusDetailVotingDeviceImageLabel
							.setText(this.bundle
									.getString("ImagesUploadPanel.status.votingdeviceimage.uploading"));
					this.jStatusDetailVotingDeviceImageLabel
							.setForeground(Color.ORANGE);
					this.jFileHashDetailVotingDeviceImageLabel.setText("");

					this.jUploadVotingDeviceImageButton.setEnabled(false);

					disableVotingDeviceImageButton();

					ECMServiceProvider.setWorkflowButtonsEnabled(false);

					ImagesUploadController controller = new ImagesUploadController(
							this, this.jTextFieldVotingDeviceImage.getText(),
							this.defaultDestinations.votingDeviceImage,
							this.votingDeviceImageComponents);

					controller.startWorker();
				}
			} else {
				ECMOptionPane
						.showMessageDialog(
								this,
								this.bundle
										.getString("ImagesUploadPanel.message.notvalidselectedfile"),
								this.bundle.getString("ErrorMessage.title"),
								ECMOptionPane.ERROR_MESSAGE,
								ECMOptionPane.ERROR_DIALOG);
			}

		} catch (Exception e) {
			ECMOptionPane.showMessageDialog(this, this.bundle
					.getString("ImagesUploadPanel.message.error.uploading.so"),
					this.bundle.getString("ErrorMessage.title"),
					ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);

		}
	}// GEN-LAST:event_jUploadVotingDeviceImageButtonActionPerformed

	/**
	 * Uploads the selected voting device screenshot image. Then starts the
	 * process to update the image status.
	 * 
	 * @param evt
	 *            A semantic event which indicates that a component-defined
	 *            action occurred
	 */
	private void jVotingDeviceFeatureImageUploadButtonActionPerformed(
			java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jVotingDeviceFeatureImageUploadButtonActionPerformed
		try {
			if (!this.jTextFieldVotingDeviceFeatureImage.getText().equals("")) {
				int result = 0;
				if (this.votingDeviceImageFeatureComponents.isImageUploaded()) {
					result = ECMOptionPane
							.showConfirmDialog(
									this,
									this.bundle
											.getString("ImagesUploadPanel.message.overwriteImage"),
									this.bundle
											.getString("WarningMessage.title"),
									ECMOptionPane.YES_NO_OPTION,
									ECMOptionPane.WARNING_MESSAGE,
									ECMOptionPane.CONFIRM_DIALOG);
				}
				if (result == 0) {
					this.jStatusDetailVotingDeviceFeatureImageLabel
							.setText(this.bundle
									.getString("ImagesUploadPanel.status.votingdeviceimage.uploading"));
					this.jStatusDetailVotingDeviceFeatureImageLabel
							.setForeground(Color.ORANGE);

					disableVotingDeviceFeatureImageButton();

					ECMServiceProvider.setWorkflowButtonsEnabled(false);

					ImagesUploadController controller = new ImagesUploadController(
							this,
							this.jTextFieldVotingDeviceFeatureImage.getText(),
							this.defaultDestinations.votingDeviceScreenShotImage,
							this.votingDeviceImageFeatureComponents);

					controller.startWorker();
				}
			} else {
				ECMOptionPane
						.showMessageDialog(
								this,
								this.bundle
										.getString("ImagesUploadPanel.message.notvalidselectedfile"),
								this.bundle.getString("ErrorMessage.title"),
								ECMOptionPane.ERROR_MESSAGE,
								ECMOptionPane.ERROR_DIALOG);
			}

		} catch (Exception e) {
			ECMOptionPane.showMessageDialog(this, this.bundle
					.getString("ImagesUploadPanel.message.error.uploading.so"),
					this.bundle.getString("ErrorMessage.title"),
					ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
		}

	}// GEN-LAST:event_jVotingDeviceFeatureImageUploadButtonActionPerformed

	/**
	 * Upload the selected canvassing station image. Then start the process to
	 * update the image status.
	 * 
	 * @param evt
	 *            A semantic event which indicates that a component-defined
	 *            action occurred
	 */
	private void jCanvassingStationImageUploadButtonActionPerformed(
			java.awt.event.ActionEvent evt) {// GEN-FIRST:event_jCanvassingStationImageUploadButtonActionPerformed
		try {
			if (!this.jTextFieldCanvassingStationImage.getText().equals("")) {
				int result = 0;
				if (this.canvassingStationComponents.isImageUploaded()) {
					result = ECMOptionPane
							.showConfirmDialog(
									this,
									this.bundle
											.getString("ImagesUploadPanel.message.overwriteImage"),
									this.bundle
											.getString("WarningMessage.title"),
									ECMOptionPane.YES_NO_OPTION,
									ECMOptionPane.WARNING_MESSAGE,
									ECMOptionPane.CONFIRM_DIALOG);
				}
				if (result == 0) {
					this.jStatusDetailCanvassingStationImageLabel
							.setText(this.bundle
									.getString("ImagesUploadPanel.status.votingdeviceimage.uploading"));
					this.jStatusDetailCanvassingStationImageLabel
							.setForeground(Color.ORANGE);

					disableCanvassingStationImageButton();

					ECMServiceProvider.setWorkflowButtonsEnabled(false);

					ImagesUploadController controller = new ImagesUploadController(
							this,
							this.jTextFieldCanvassingStationImage.getText(),
							this.defaultDestinations.canvassingStationImage,
							this.canvassingStationComponents);

					controller.startWorker();
				}
			} else {
				ECMOptionPane
						.showMessageDialog(
								this,
								this.bundle
										.getString("ImagesUploadPanel.message.notvalidselectedfile"),
								this.bundle.getString("ErrorMessage.title"),
								ECMOptionPane.ERROR_MESSAGE,
								ECMOptionPane.ERROR_DIALOG);
			}

		} catch (Exception e) {
			ECMOptionPane.showMessageDialog(this, this.bundle
					.getString("ImagesUploadPanel.message.error.uploading.so"),
					this.bundle.getString("ErrorMessage.title"),
					ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
		}

	}// GEN-LAST:event_jCanvassingStationImageUploadButtonActionPerformed

	/**
	 * SAESCommonDestinations reference to reference and handle files and
	 * folders used by the application.
	 */
	private SAESCommonDestinations defaultDestinations;

	/** Serial Version UID constant. */
	private static final long serialVersionUID = 1L;

	/** Bundle to access the languages resources for GUI controls. */
	private java.util.ResourceBundle bundle;

	// Variables declaration - do not modify//GEN-BEGIN:variables
	/** Background label control. */
	private javax.swing.JLabel bannerBackgroundLabel;

	/** Background layered pane control. */
	private javax.swing.JLayeredPane bannerBackgroundLayeredPane;

	/** Functionality icon logo label. */
	private javax.swing.JLabel iconLogoLabel;

	/** Functionality title label. */
	private javax.swing.JLabel mainTitleLabel;

	/** Functionality description label. */
	private javax.swing.JLabel mainDescriptionLabel;

	/** Canvassing station image panel control. */
	private javax.swing.JPanel jCanvassingStationImagePanel;

	/** Canvassing station image progress bar. */
	private javax.swing.JProgressBar jCanvassingStationImageProgressBar;

	/** Canvassing station image search button. */
	private javax.swing.JButton jCanvassingStationImageSearchButton;

	/** Canvassing station image upload button. */
	private javax.swing.JButton jCanvassingStationImageUploadButton;

	/** Canvassing voting device feature image panel. */
	private javax.swing.JPanel jCanvassingVotingDeviceFeatureImagePanel;

	/** Canvassing station image file chooser. */
	private javax.swing.JFileChooser jFileCanvassingStationImageChooser;

	/** Canvassing station image hash label. */
	private javax.swing.JLabel jFileHashCanvassingStationImageLabel;

	/** Canvassing station image hash detailed label. */
	private javax.swing.JLabel jFileHashDetailCanvassingStationImageLabel;

	/** Voting device feature image hash detailed label. */
	private javax.swing.JLabel jFileHashDetailVotingDeviceFeatureImageLabel;

	/** Voting device image hash detailed label. */
	private javax.swing.JLabel jFileHashDetailVotingDeviceImageLabel;

	/** Voting device feature hash image label. */
	private javax.swing.JLabel jFileHashVotingDeviceFeatureImageLabel;

	/** Voting device hash image label. */
	private javax.swing.JLabel jFileHashVotingDeviceImageLabel;

	/** Voting device feature image file chooser. */
	private javax.swing.JFileChooser jFileVotingDeviceFeatureImageChooser;

	/** Voting device image file chooser. */
	private javax.swing.JFileChooser jFileVotingDeviceImageChooser;

	/** Voting device image search button. */
	private javax.swing.JButton jSearchVotingDeviceImageButton;

	/** Canvassing station image status label. */
	private javax.swing.JLabel jStatusCanvassingStationImageLabel;

	/** Canvassing station image status detail label. */
	private javax.swing.JLabel jStatusDetailCanvassingStationImageLabel;

	/** Voting device feature image status detail label. */
	private javax.swing.JLabel jStatusDetailVotingDeviceFeatureImageLabel;

	/** Voting device image status detail label. */
	private javax.swing.JLabel jStatusDetailVotingDeviceImageLabel;

	/** Voting device feature status image label. */
	private javax.swing.JLabel jStatusVotingDeviceFeatureImageLabel;

	/** Voting device status image label. */
	private javax.swing.JLabel jStatusVotingDeviceImageLabel;

	/** Canvassing station image text field. */
	private javax.swing.JTextField jTextFieldCanvassingStationImage;

	/** Voting device feature image text field. */
	private javax.swing.JTextField jTextFieldVotingDeviceFeatureImage;

	/** Voting device image text field. */
	private javax.swing.JTextField jTextFieldVotingDeviceImage;

	/** Voting device upload image button. */
	private javax.swing.JButton jUploadVotingDeviceImageButton;

	/** Voting device feature image progress bar. */
	private javax.swing.JProgressBar jVotingDeviceFeatureImageProgressBar;

	/** Voting device feature image search button. */
	private javax.swing.JButton jVotingDeviceFeatureImageSearchButton;

	/** Voting device feature image upload button. */
	private javax.swing.JButton jVotingDeviceFeatureImageUploadButton;

	/** Voting device image panel. */
	private javax.swing.JPanel jVotingDeviceImagePanel;

	/** Voting device image progress bar. */
	private javax.swing.JProgressBar jVotingDeviceImageProgressBar;

	/** Canvassing station validation result label. */
	private javax.swing.JLabel labelCanvassingStationValidationResult;

	/** Voting Device Feature validation result label. */
	private javax.swing.JLabel labelVotingDeviceFeatureValidationResult;

	/** Voting device validation result label. */
	private javax.swing.JLabel labelVotingDeviceValidationResult;

	/** Voting device image components. */
	private ImagesUploadGUIComponents votingDeviceImageComponents;

	/** Voting device image feature components. */
	private ImagesUploadGUIComponents votingDeviceImageFeatureComponents;

	/** Canvassing Station image components. */
	private ImagesUploadGUIComponents canvassingStationComponents;

	// End of variables declaration//GEN-END:variables
}
