/**
 * ImageUploadValidatorController.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.ImagesUploadValidatorWorker;

/**
 * Controller class that creates and starts the worker thread to validate the
 * uploaded image.
 */
public class ImagesUploadValidatorController implements IAwtOperationListener {

	/** ImagesUploadPanel reference. */
	private ImagesUploadPanel imagesUploadPanel;

	/** ImagesUploadGUIComponents reference. */
	private ImagesUploadGUIComponents imagesUploadGUIComponents;

	/** ImageUploadValidatorWorker reference. */
	ImagesUploadValidatorWorker imageUploadValidatorWorker;

	/** Contains all file names necessary to consider that the image is valid. */
	private List<String> mandatoryFiles;

	/** tar.gz image to validate. */
	private File srcImage;

	/**
	 * Instantiates a new ImageUploadValidatorController.
	 * 
	 * @param imagesUploadPanel
	 *            ImagesUploadPanel reference
	 * @param srcImage
	 *            tar.gz image to validate
	 * @param mandatoryFoldersForVotingMachineIso
	 *            Mandatory folders for voting machine iso
	 * @param imagesUploadGUIComponents
	 *            ImagesUploadGUIComponents reference
	 */
	public ImagesUploadValidatorController(ImagesUploadPanel imagesUploadPanel,
			File srcImage, String mandatoryFoldersForVotingMachineIso,
			ImagesUploadGUIComponents imagesUploadGUIComponents) {
		this.imagesUploadPanel = imagesUploadPanel;
		this.srcImage = srcImage;
		this.mandatoryFiles = Arrays.asList(StringUtils.split(
				mandatoryFoldersForVotingMachineIso, ":"));
		this.imagesUploadGUIComponents = imagesUploadGUIComponents;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener#awtOperation
	 * (com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType)
	 */
	@Override
	public void awtOperation(AwtOperationType awtOperationType) {
		switch (awtOperationType) {

		case UPDATE_PROGRESS_BAR:
			this.imagesUploadPanel.updateProgressBar(
					this.imageUploadValidatorWorker.getProgressBarValue(),
					this.imagesUploadGUIComponents);
			break;

		case UPDATE_COMPONENTS:
			this.imagesUploadPanel.updateComponentsAfterValidation(
					this.imageUploadValidatorWorker.isValidImage(),
					this.imagesUploadGUIComponents);
			break;

		case DISABLE_COMPONENTS:
			this.imagesUploadPanel
					.hideComponentsAfterValidation(this.imagesUploadGUIComponents);
			break;

		case SHOW_ERROR_MESSAGE:
			this.imagesUploadPanel.showGenericErrorMessage();
			break;

		}

	}

	/**
	 * Method that starts the worker that does the real backend functionality
	 * work.
	 */
	public void startWorker() {

		this.imageUploadValidatorWorker = new ImagesUploadValidatorWorker(
				this.mandatoryFiles, this.srcImage);

		this.imageUploadValidatorWorker.addListener(this);

		Thread imageUploadValidatorWorkerThread = new Thread(
				this.imageUploadValidatorWorker, "ImageUploadValidatorWorker");

		imageUploadValidatorWorkerThread.start();
	}

}
