/**
 * MasterUsbMergedPanel.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Set;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.ParallelGroup;
import javax.xml.datatype.Duration;

import org.jdesktop.application.Action;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.security.FunctionalityDefinitionEnum;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.providers.RegionBrowserHierarchyProvider;
import com.smartmatic.waffle.ecm.client.ui.controller.IRefreshRegionWidget;
import com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyDTO;
import com.smartmatic.waffle.ecm.ui.controls.Browser.BrowserHierarchy;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Front-end for the MasterUsbGeneration when the image combine RTS+PM.
 */
public class MasterUsbMergedPanel extends javax.swing.JPanel implements
		GUIGenerationInterface, IRefreshRegionWidget {

	/**
	 * the number to serialization.
	 */
	private static final long serialVersionUID = 8655735137515717393L;

	/** The resource message bundle. */
	private ResourceBundle bundle = ResourceBundle
			.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");

	/** * For log messages purpose. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(MasterUsbMergedPanel.class);

	/**
	 * Indicates if the master usb generation- process has ed. Flag used to know
	 * if the process has been canceled
	 */
	private boolean generateDeviceStarted = false;

	private BrowserHierarchy regionNavigation;

	MasterUSBGenerationController masterUsbGenerationController;

	/***
	 * Class constructor.<br>
	 * Creates new form MasterUsbMergedPanel
	 */
	public MasterUsbMergedPanel() {

		String titleName = this.bundle
				.getString("RegionLevelNavigatorWidget.regionTable.header.regionColumn")
				+ " | "
				+ this.bundle
						.getString("RegionLevelNavigatorWidget.regionTable.header.categoryColumn");

		RegionBrowserHierarchyProvider provider = getRegionsWidget();
		this.regionNavigation = new BrowserHierarchy(provider, true, titleName);

		initComponents();
		hideResources();
		showLabelStatusProccess(false);
		setPendriveOkLabelVisible(false);
	}

	/**
	 * Resets the master generation panel to initial state.
	 */
	private void resetScreen() {

		setPendriveOkLabelVisible(false);
		showLabelStatusProccess(false);
		enableGenerateButton(true);
		hideResources();
		setProgressBarValue(0);
		this.generateMasterButton
				.setText(this.bundle
						.getString("MasterUsbGenerationPanel.generar_masterusb_btn.text"));
		this.generateDeviceStarted = false;
		if (this.masterUsbGenerationController != null) {
			this.masterUsbGenerationController.stopDriveFinderWorker();
		}
		this.masterUsbGenerationController = null;

		unLockPanelButtons();
	}

	/**
	 * Gets Regions Provider to be used on Hierarchy widget
	 * 
	 * @return
	 */
	private RegionBrowserHierarchyProvider getRegionsWidget() {
		RegionBrowserHierarchyProvider provider = null;

		LanguageData language = LanguageController.getInstance()
				.getSelectedLanguage();
		try {
			provider = new RegionBrowserHierarchyProvider(
					ECMServiceProvider.getAvailableRegionsForWidget(), false,
					language);
		} catch (Exception e) {
			logger.error(e, "Error loading RegionBrowserHierarchyProvider");
			// There is not possible to recover from this error, it is
			// executed during panel constructor
			throw new RuntimeException(
					"Error loading RegionBrowserHierarchyProvider", e);
		}
		return provider;
	}

	/**
	 * Validates master generation pre-conditions.
	 *
	 * @return true, if all preconditions are done
	 */
	public boolean validateMasterGenerationPreConditions() {

		if (this.getSelectedObjects().size() == 0) {
			showErrorMessage("MasterUsbGenerationPanel.message.no.option.selected");
			return false;
		}

		SAESCommonDestinations defaultDestinations = SAESLocalPersistenceManager
				.getInstance().getDefaultDestinations();

		if (!defaultDestinations.votingDeviceImage.exists()) {
			showErrorMessage("MasterUsbMergedPanel.image_not_loaded");
			return false;
		}

		return true;
	}

	/**
	 * Generates master usb.
	 */
	@Action
	public void GenerateMasterUsb() {

		HashSet<FunctionalityDefinitionEnum> funcs = new HashSet<FunctionalityDefinitionEnum>();
		funcs.add(FunctionalityDefinitionEnum.MASTER_USB_MERGED);

		if (!this.generateDeviceStarted) {
			if (validateMasterGenerationPreConditions()) {
				setExplicitDownMessage(this.bundle
						.getString("MasterUsbMergedPanel.down_msg_label.text"));
				this.downloadMessageLabel.setVisible(true);
				this.showPendriveInsertLabel(true);

				ECMServiceProvider.blockOtherTabsByFunctionality(funcs, true);

				this.generateDeviceStarted = true;
				this.generateMasterButton.setText(this.bundle
						.getString("FileGenerationPanel.cancel"));

				this.masterUsbGenerationController = new MasterUSBGenerationController(
						this);
				this.masterUsbGenerationController.startDriveFinderWorker();
			}
		} else {
			showCanceledByUserMessage();
		}
	}

	@Override
	public void setProgressBarValue(int value) {
		this.mainProgressBar.setValue(value);
		this.mainProgressBar.setString(String.valueOf(value) + "%");
	}

	/**
	 * Sets the explicit down message.
	 * 
	 * @param message
	 *            the new explicit down message
	 */
	private void setExplicitDownMessage(String message) {
		this.downloadMessageLabel.setText(message);
	}

	/**
	 * Shows pendrive insert label.
	 * 
	 * @param visibility
	 *            the visibility
	 */
	private void showPendriveInsertLabel(boolean visibility) {
		this.insertPendriveLabel.setVisible(visibility);
	}

	/**
	 * Shows document label.
	 * 
	 * @param visibility
	 *            the visibility
	 */
	private void showDocumentLabel(boolean visibility) {
		this.downloadGifLabel.setVisible(visibility);
	}

	/**
	 * Enables generate button.
	 * 
	 * @param value
	 *            the value
	 */
	private void enableGenerateButton(boolean value) {
		this.generateMasterButton.setEnabled(value);
		this.generateNewInitKeyCheckBox.setEnabled(value);
	}

	/**
	 * Shows canceled by user message.
	 */
	private void showCanceledByUserMessage() {
		resetScreen();
		showMessage(this.bundle
				.getString("FileGenerationPanel.message.process.cancel"));

	}

	@Override
	public void finishProcess(Duration duration) {
		resetScreen();

		showResultMessage(duration);
	}

	private void showResultMessage(Duration duration) {

		StringBuilder message = new StringBuilder();
		message.append(this.bundle
				.getString("FileGenerationPanel.message.success"));
		message.append(" ").append(duration.getHours()).append(" ");
		message.append(this.bundle.getString("FileGenerationPanel.time.hour"));
		message.append(" ").append(duration.getMinutes()).append(" ");
		message.append(this.bundle.getString("FileGenerationPanel.time.minute"));
		message.append(" ").append(duration.getSeconds()).append(" ");
		message.append(this.bundle.getString("FileGenerationPanel.time.second"));

		showResultMessage(message.toString());
	}

	/**
	 * Shows Result Message.
	 * 
	 * @param message
	 *            raw message formated by bundle
	 */
	private void showResultMessage(String message) {
		ECMOptionPane.showMessageDialog(this, message,
				this.bundle.getString("NormalMessage.title"),
				ECMOptionPane.INFORMATION_MESSAGE,
				ECMOptionPane.INFORMATION_DIALOG);
		this.generateNewInitKeyCheckBox.setSelected(true);
	}

	/**
	 * Shows Informative Message.
	 * 
	 * @param message
	 *            raw message formated by bundle
	 */
	private void showMessage(String message) {
		ECMOptionPane.showMessageDialog(this, message,
				this.bundle.getString("NormalMessage.title"),
				ECMOptionPane.INFORMATION_MESSAGE,
				ECMOptionPane.INFORMATION_DIALOG);
	}

	/**
	 * Shows message of many devices connected
	 */
	@Override
	public void showMessageManyDevicesConnected() {
		ECMOptionPane.showMessageDialog(this, this.bundle
				.getString("MasterUsbMergedPanel.many.devices.connected"),
				this.bundle.getString("WarningMessage.title"),
				ECMOptionPane.WARNING_MESSAGE, ECMOptionPane.WARNING_DIALOG);
		resetScreen();
	}

	/**
	 * Shows message of unique memories.
	 */
	public void showMessageUniqueMemories() {
		ECMOptionPane.showMessageDialog(this,
				this.bundle.getString("MasterUsbMergedPanel.unique.memories"),
				this.bundle.getString("WarningMessage.title"),
				ECMOptionPane.WARNING_MESSAGE, ECMOptionPane.WARNING_DIALOG);
	}

	/**
	 * Starts generation process.
	 * 
	 * @param mountedPath
	 *            the mounted path
	 * @param partitionName
	 *            the partition name
	 */
	@Override
	public void preparePanelToProcess() {

		this.enableGenerateButton(false);
		this.showPendriveInsertLabel(false);
		this.showDocumentLabel(false);
		this.setExplicitDownMessage(this.bundle
				.getString("MasterUsbGenerationPanel.down_msg_label_dont_remove.text"));
	}

	@Override
	public boolean getGenerateNewInitKey() {
		return this.generateNewInitKeyCheckBox.isSelected();
	}

	@Override
	public int showConfirmDialogStartGeneration(String mountedPath) {
		int result = ECMOptionPane
				.showConfirmDialog(
						this,
						this.bundle
								.getString("FileGenerationPanel.message.format.device")
								+ mountedPath
								+ this.bundle
										.getString("FileGenerationPanel.message.format.device.quetion"),
						this.bundle
								.getString("FileGenerationPanel.title.format.device"),
						ECMOptionPane.YES_NO_OPTION,
						ECMOptionPane.CONFIRM_DIALOG);

		if (result != 0) {
			showCanceledByUserMessage();
		}

		return result;
	}

	@Override
	public void showErrorMessage(String messageErrorKey) {
		ECMOptionPane.showMessageDialog(this,
				this.bundle.getString(messageErrorKey),
				this.bundle.getString("ErrorMessage.title"),
				ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
		resetScreen();
	}

	@Override
	public void setLabelStatusProcessingContests(int processed, int toProcess) {

		if (processed > 0) {
			StringBuilder messageBuilder = new StringBuilder();
			messageBuilder
					.append(this.bundle
							.getString("MasterUsbMergedPanel.status_message.procesing.contest"));
			messageBuilder.append("[").append(processed);
			messageBuilder.append("/").append(toProcess).append("]");

			setLabelStatusProcess(messageBuilder.toString());
		} else {
			setLabelStatusProcess(this.bundle
					.getString("MasterUsbMergedPanel.status_message.getting.contest"));
		}
		showLabelStatusProccess(true);
	}

	@Override
	public List<Object> getSelectedObjects() {

		List<Object> objectList = new ArrayList<Object>();

		Set<BrowserHierarchyDTO> selectedObjects = this.regionNavigation
				.getSelectedObject();

		for (BrowserHierarchyDTO browserHierarchyDto : selectedObjects) {
			objectList.add(browserHierarchyDto.getId());
		}

		return objectList;
	}

	@Override
	public void setLabelStatusProcess(String message) {
		this.generationStatusLabel.setText(message);
	}

	@Override
	public void showLabelStatusProccess(boolean value) {
		this.generationStatusLabel.setVisible(value);
	}

	@Override
	public void setPendriveOkLabelVisible(boolean value) {
		this.pendriveOkImageLabel.setVisible(value);
	}

	@Override
	public void setVerificationIconInPendriveOkLabel() {
		this.pendriveOkImageLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/usb_find_errors.gif")));
		this.repaint();
	}

	@Override
	public void setCopyOsIconInPendriveOkLabel() {
		this.pendriveOkImageLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/upload_memory.gif")));
		this.repaint();
	}

	@Override
	public void setPrepareContestIconInPendriveOkLabel() {
		this.pendriveOkImageLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/build_contest.gif")));
		this.repaint();
	}

	@Override
	public void setSyncAndUnmountIconInPendriveOkLabel() {
		this.pendriveOkImageLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/syncAndUmountDrive.gif")));
		this.repaint();
	}

	@Override
	public void setDocumentImageLabelVisible(boolean value) {
		this.downloadGifLabel.setVisible(value);
	}

	@Override
	public boolean isDocumentImageLabelVisible() {
		return this.downloadGifLabel.isVisible();
	}

	@Override
	public void setMainProgressBarVisible(boolean value) {
		this.mainProgressBar.setVisible(value);
	}

	@Override
	public boolean isMainProgressBarVisible() {
		return this.mainProgressBar.isVisible();
	}

	@Override
	public void refreshWidget() {
		this.regionNavigation.reset();
	}

	/**
	 * Unlock panel buttons.
	 */
	public void unLockPanelButtons() {
		HashSet<FunctionalityDefinitionEnum> funcs = new HashSet<FunctionalityDefinitionEnum>();
		funcs.add(FunctionalityDefinitionEnum.MASTER_USB_MERGED);
		ECMServiceProvider.blockOtherTabsByFunctionality(funcs, false);
	}

	@Override
	public ResourceBundle getBundle() {
		return this.bundle;
	}

	/***
	 * Hides elements used during processing.
	 */
	public void hideResources() {
		this.insertPendriveLabel.setVisible(false);
		this.mainProgressBar.setVisible(false);
		this.downloadGifLabel.setVisible(false);
		this.downloadMessageLabel.setVisible(false);
	}

	private void initComponents() {

		this.headerLayeredPane = new javax.swing.JLayeredPane();
		this.masterGenerationTitleLabel = new javax.swing.JLabel();
		this.generationIconLabel = new javax.swing.JLabel();
		this.masterGenerationDescriptionLabel = new javax.swing.JLabel();
		this.bannerBackgroundLabel = new javax.swing.JLabel();
		this.containerPanel = new javax.swing.JPanel();
		this.mainPanel = new javax.swing.JPanel();
		this.containerRegionWidgetPanel = new javax.swing.JPanel();
		this.regionWidgetPanel = new javax.swing.JPanel();
		this.mainLayeredPane = new javax.swing.JLayeredPane();
		this.downloadGifLabel = new javax.swing.JLabel();
		this.generateMasterButton = new javax.swing.JButton();
		this.generateNewInitKeyCheckBox = new javax.swing.JCheckBox();
		this.mainProgressBar = new javax.swing.JProgressBar();
		this.insertPendriveLabel = new javax.swing.JLabel();
		this.downloadMessageLabel = new javax.swing.JLabel();
		this.generationStatusLabel = new javax.swing.JLabel();
		this.pendriveOkImageLabel = new javax.swing.JLabel();

		setPreferredSize(new java.awt.Dimension(1024, 768));

		this.headerLayeredPane.setName("layeredPaneBanner"); // NOI18N

		this.masterGenerationTitleLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 1, 14));
		java.util.ResourceBundle bundle = java.util.ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
		this.masterGenerationTitleLabel.setText(bundle
				.getString("MasterUsbMergedPanel.jLabel2.text_1")); // NOI18N
		this.masterGenerationTitleLabel.setName("labelMasterUsbMergedTitle"); // NOI18N
		this.masterGenerationTitleLabel.setBounds(170, 30, 500, 17);
		this.headerLayeredPane.add(this.masterGenerationTitleLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.generationIconLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/generacion_maquina.png"))); // NOI18N
		this.generationIconLabel.setName("labelMasterUsbGenerationIcon"); // NOI18N
		this.generationIconLabel.setBounds(30, 10, 120, 100);
		this.headerLayeredPane.add(this.generationIconLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.masterGenerationDescriptionLabel.setFont(new java.awt.Font(
				"DejaVu Sans", 0, 11));
		this.masterGenerationDescriptionLabel.setText(bundle
				.getString("MasterUsbMergedPanel.jLabel3.text_1")); // NOI18N
		this.masterGenerationDescriptionLabel
				.setName("labelMasterUsbMergedDescription"); // NOI18N
		this.masterGenerationDescriptionLabel.setBounds(190, 50, 720, 17);
		this.headerLayeredPane.add(this.masterGenerationDescriptionLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.bannerBackgroundLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/barra titulo subtitulo.jpg"))); // NOI18N
		this.bannerBackgroundLabel.setName("labelBannerBackground"); // NOI18N
		this.bannerBackgroundLabel.setPreferredSize(new java.awt.Dimension(
				1440, 120));
		this.bannerBackgroundLabel.setBounds(0, -10, 1450, 140);
		this.headerLayeredPane.add(this.bannerBackgroundLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.containerPanel.setName("panelMasterUsbGeneratorMain");
		this.containerPanel.setLayout(new javax.swing.BoxLayout(
				this.containerPanel, javax.swing.BoxLayout.PAGE_AXIS));

		this.mainPanel.setName("panelMasterUsbGenerator"); // NOI18N

		this.containerRegionWidgetPanel.setMaximumSize(new java.awt.Dimension(
				32767, 160));
		this.containerRegionWidgetPanel.setMinimumSize(new java.awt.Dimension(
				0, 150));
		this.containerRegionWidgetPanel
				.setName("panelRegionWidgetNavigatorMain"); // NOI18N
		this.containerRegionWidgetPanel
				.setPreferredSize(new java.awt.Dimension(1036, 160));

		this.regionWidgetPanel
				.setMaximumSize(new java.awt.Dimension(1336, 150));
		this.regionWidgetPanel.setMinimumSize(new java.awt.Dimension(591, 150));
		this.regionWidgetPanel
				.setPreferredSize(new java.awt.Dimension(614, 150));
		this.regionWidgetPanel.setName("panelRegionWidgetNavigator");

		javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(
				this.regionWidgetPanel);
		this.regionWidgetPanel.setLayout(jPanel2Layout);
		jPanel2Layout.setHorizontalGroup(jPanel2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 766,
				Short.MAX_VALUE));
		jPanel2Layout.setVerticalGroup(jPanel2Layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 121,
				Short.MAX_VALUE));

		this.regionWidgetPanel = this.regionNavigation;

		this.generateNewInitKeyCheckBox.setSelected(true);
		String checkboxText = this.bundle
				.getString("MasterUsbGenerationPanel.generate_new_init_key.text");
		this.generateNewInitKeyCheckBox.setText(checkboxText);

		this.generateNewInitKeyCheckBox.setBounds(315, 120, 290, 40);
		this.containerRegionWidgetPanel.add(this.generateNewInitKeyCheckBox,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		GroupLayout jPanel1Layout = new GroupLayout(
				this.containerRegionWidgetPanel);
		this.containerRegionWidgetPanel.setLayout(jPanel1Layout);

		ParallelGroup parallelGroup1 = jPanel1Layout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGroup(
				GroupLayout.Alignment.TRAILING,
				jPanel1Layout
						.createSequentialGroup()
						.addGap(310, 310, 310)
						.addComponent(this.regionWidgetPanel,
								GroupLayout.PREFERRED_SIZE, 766,
								GroupLayout.PREFERRED_SIZE).addGap(58, 58, 58));

		jPanel1Layout.setHorizontalGroup(parallelGroup1);
		jPanel1Layout
				.setVerticalGroup(jPanel1Layout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel1Layout
										.createSequentialGroup()
										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																GroupLayout.Alignment.LEADING)
														.addGroup(
																jPanel1Layout
																		.createSequentialGroup()
																		.addContainerGap()
																		.addComponent(
																				this.regionWidgetPanel,
																				GroupLayout.PREFERRED_SIZE,
																				121,
																				GroupLayout.PREFERRED_SIZE)))
										.addContainerGap(27, Short.MAX_VALUE)));

		this.regionWidgetPanel.setVisible(true);
		this.regionNavigation.setVisible(true);

		this.mainLayeredPane.setCursor(new java.awt.Cursor(
				java.awt.Cursor.DEFAULT_CURSOR));
		this.mainLayeredPane.setName("layeredPaneMain"); // NOI18N
		this.mainLayeredPane
				.setPreferredSize(new java.awt.Dimension(1036, 508));

		this.downloadGifLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.downloadGifLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/download.gif"))); // NOI18N
		this.downloadGifLabel.setName("labelDocumentImage"); // NOI18N
		this.downloadGifLabel.setBounds(610, 40, 130, 130);
		this.mainLayeredPane.add(this.downloadGifLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		final javax.swing.ActionMap actionMap = org.jdesktop.application.Application
				.getInstance().getContext()
				.getActionMap(MasterUsbMergedPanel.class, this);
		this.generateMasterButton.setAction(actionMap.get("GenerateMasterUsb"));
		this.generateMasterButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (!MasterUsbMergedPanel.this.generateDeviceStarted) {
					showMessageUniqueMemories();
				}
			}
		});
		this.generateMasterButton.setText(bundle
				.getString("MasterUsbMergedPanel.generar_masterusb_btn.text")); // NOI18N
		this.generateMasterButton
				.setMaximumSize(new java.awt.Dimension(135, 29));
		this.generateMasterButton
				.setMinimumSize(new java.awt.Dimension(135, 29));
		this.generateMasterButton.setName("buttonGenerateMasterUsb"); // NOI18N
		this.generateMasterButton.setPreferredSize(new java.awt.Dimension(135,
				29));
		this.generateMasterButton.setBounds(530, 240, 290, 40);
		this.mainLayeredPane.add(this.generateMasterButton,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.mainProgressBar.setName("progressBarMain"); // NOI18N
		this.mainProgressBar.setStringPainted(true);
		this.mainProgressBar.setBounds(540, 200, 270, 20);
		this.mainLayeredPane.add(this.mainProgressBar,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.insertPendriveLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.insertPendriveLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/connect_usb_horizontal.gif"))); // NOI18N
		this.insertPendriveLabel.setName("labelInsertPenDrive"); // NOI18N
		this.insertPendriveLabel.setBounds(560, 0, 230, 220);
		this.mainLayeredPane.add(this.insertPendriveLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.downloadMessageLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.downloadMessageLabel.setText(bundle
				.getString("MasterUsbMergedPanel.down_msg_label.text")); // NOI18N
		this.downloadMessageLabel.setName("labelDownMessage"); // NOI18N
		this.downloadMessageLabel.setBounds(340, 280, 670, 30);
		this.mainLayeredPane.add(this.downloadMessageLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.generationStatusLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.generationStatusLabel
				.setText(bundle
						.getString("MasterUsbGenerationPanel.jLabelStatusProccess.proccess.text")); // NOI18N
		this.generationStatusLabel.setName("labelStatusProccess"); // NOI18N
		this.generationStatusLabel.setBounds(230, 220, 890, 30);
		this.mainLayeredPane.add(this.generationStatusLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		this.pendriveOkImageLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.pendriveOkImageLabel
				.setIcon(new javax.swing.ImageIcon(
						getClass()
								.getResource(
										"/com/smartmatic/waffle/ecm/client/resources/upload_memory.gif"))); // NOI18N
		this.pendriveOkImageLabel.setName("labelPendriveOk"); // NOI18N
		this.pendriveOkImageLabel.setBounds(560, 0, 230, 220);
		this.mainLayeredPane.add(this.pendriveOkImageLabel,
				javax.swing.JLayeredPane.DEFAULT_LAYER);

		javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(
				this.mainPanel);
		this.mainPanel.setLayout(jPanel3Layout);
		jPanel3Layout.setHorizontalGroup(jPanel3Layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.containerRegionWidgetPanel,
						javax.swing.GroupLayout.DEFAULT_SIZE, 1024,
						Short.MAX_VALUE)
				.addComponent(this.mainLayeredPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 1024,
						Short.MAX_VALUE));
		jPanel3Layout
				.setVerticalGroup(jPanel3Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel3Layout
										.createSequentialGroup()
										.addComponent(
												this.containerRegionWidgetPanel,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(
												this.mainLayeredPane,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												366,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addContainerGap(95, Short.MAX_VALUE)));
		this.containerPanel.add(this.mainPanel);

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.headerLayeredPane,
						javax.swing.GroupLayout.DEFAULT_SIZE, 1024,
						Short.MAX_VALUE)
				.addGroup(
						layout.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
								.addComponent(this.containerPanel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										1024, Short.MAX_VALUE)));
		layout.setVerticalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addComponent(this.headerLayeredPane,
										javax.swing.GroupLayout.PREFERRED_SIZE,
										124,
										javax.swing.GroupLayout.PREFERRED_SIZE)
								.addContainerGap(644, Short.MAX_VALUE))
				.addGroup(
						layout.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
								.addGroup(
										layout.createSequentialGroup()
												.addGap(129, 129, 129)
												.addComponent(
														this.containerPanel,
														javax.swing.GroupLayout.DEFAULT_SIZE,
														627, Short.MAX_VALUE)
												.addContainerGap())));
	}

	/** The banner background label. */
	private javax.swing.JLabel bannerBackgroundLabel;

	/** The download GIF label. */
	private javax.swing.JLabel downloadGifLabel;

	/** The download message label. */
	private javax.swing.JLabel downloadMessageLabel;

	/** The generar_masterusb_btn. */
	private javax.swing.JButton generateMasterButton;

	/** The Generate new initialization key checkbox. */
	private javax.swing.JCheckBox generateNewInitKeyCheckBox;

	/** The insert pendrive label. */
	private javax.swing.JLabel insertPendriveLabel;

	/** The generation icon label. */
	private javax.swing.JLabel generationIconLabel;

	/** The master generation title label */
	private javax.swing.JLabel masterGenerationTitleLabel;

	/** The master generation description label. */
	private javax.swing.JLabel masterGenerationDescriptionLabel;

	/** The generation status label. */
	private javax.swing.JLabel generationStatusLabel;

	/** The header layered pane. */
	private javax.swing.JLayeredPane headerLayeredPane;

	/** The container region widget panel. */
	private javax.swing.JPanel containerRegionWidgetPanel;

	/** The region widget panel. */
	private javax.swing.JPanel regionWidgetPanel;

	/** The main panel. */
	private javax.swing.JPanel mainPanel;

	/** The main layered pane. */
	private javax.swing.JLayeredPane mainLayeredPane;

	/** The main progress bar. */
	private javax.swing.JProgressBar mainProgressBar;

	/** The pendrive ok image icon . */
	private javax.swing.JLabel pendriveOkImageLabel;

	/** The container panel. */
	private javax.swing.JPanel containerPanel;

}