package com.smartmatic.waffle.ecm.client.ui;

import java.util.ResourceBundle;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.SearchPartyWorker;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;

/**
 * Controller class used to search parties.
 * 
 */
public class SearchPartyController implements IAwtOperationListener {

	EligibleOptionEditorPanel eligibleOptionPanel;

	SearchPartyWorker searchPartyWorker;

	ResourceBundle bundle = ResourceBundle
			.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");

	/**
	 * Creates a new instance of SearchPartyController
	 * 
	 * @param eligibleOptionPanel
	 *            the panel which called this controller.
	 */
	public SearchPartyController(EligibleOptionEditorPanel eligibleOptionPanel) {
		this.eligibleOptionPanel = eligibleOptionPanel;
	}

	/**
	 * Method that starts the worker that does the real backend functionality
	 * work.
	 */
	public void startWorker() {
		LanguageData language = LanguageController.getInstance()
				.getSelectedLanguage();

		this.searchPartyWorker = new SearchPartyWorker(
				this.eligibleOptionPanel.getSearchText(),
				this.eligibleOptionPanel.getCurrentSearchRegions(),
				this.eligibleOptionPanel.getCurrentSearchContests(), language,
				this.eligibleOptionPanel.getCurrentPage());

		this.searchPartyWorker.addListener(this);

		Thread searchPartyWorkerThread = new Thread(this.searchPartyWorker,
				"SearchPartyWorker");

		searchPartyWorkerThread.start();
	}

	@Override
	public void awtOperation(AwtOperationType awtOperationType) {

		switch (awtOperationType) {

		case ENABLE_COMPONENTS:
			this.eligibleOptionPanel.enableSearchButtons(true);
			break;

		case DISABLE_COMPONENTS:
			this.eligibleOptionPanel.enableSearchButtons(false);
			break;

		case UPDATE_COMPONENTS:
			this.eligibleOptionPanel.clearSearchResults();
			this.eligibleOptionPanel
					.setTotalNumberOfPages(this.searchPartyWorker
							.getTotalNumberOfPages());
			this.eligibleOptionPanel.setPaginationLabelText();

			break;
		case SHOW_RESULT_MESSAGE:
			this.eligibleOptionPanel
					.updateResultsTableParty(this.searchPartyWorker
							.getPartiesList());
			break;
		default:
			break;
		}
	}
}
