/**
 * SecurityComponentsExportController.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import java.io.File;

import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.SecurityComponentsExportWorker;

/**
 * Controller class that creates and starts the worker thread to export the
 * voting devices security components.
 */
public class SecurityComponentsExportController implements
		IAwtOperationListener {

	/** KeyGenerationPanel reference. */
	private KeyGenerationPanel keyGenerationPanel;

	/** SecurityComponentsExportWorker reference. */
	private SecurityComponentsExportWorker securityComponentsExportWorker;

	/** Destination file where the zip file will be uploaded. */
	private File destFile;

	/**
	 * Instantiates a new SecurityComponentsExportController.
	 * 
	 * @param keyGenerationPanel
	 *            the panel reference
	 * @param destImageFile
	 *            Destination file where the zip file will be uploaded
	 */
	public SecurityComponentsExportController(
			KeyGenerationPanel keyGenerationPanel, File destFile) {
		this.keyGenerationPanel = keyGenerationPanel;
		this.destFile = destFile;
	}

	/**
	 * Method that starts the worker that does the real backend functionality
	 * work.
	 */
	public void startWorker() {

		this.securityComponentsExportWorker = new SecurityComponentsExportWorker();

		this.securityComponentsExportWorker.setDestFile(this.destFile);

		this.securityComponentsExportWorker.addListener(this);

		Thread securityComponentsExportWorkerThread = new Thread(
				this.securityComponentsExportWorker,
				"SecurityComponentsExportWorker");

		securityComponentsExportWorkerThread.start();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener#awtOperation
	 * (com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType)
	 */
	@Override
	public void awtOperation(AwtOperationType awtOperationType) {

		switch (awtOperationType) {

		case UPDATE_PROGRESS_BAR:
			this.keyGenerationPanel
					.updateProgressBar(this.securityComponentsExportWorker
							.getProgressBarValue());
			break;

		case ENABLE_COMPONENTS: {
			this.keyGenerationPanel.enableButton(true);
			this.keyGenerationPanel.enableTabsAndWorkflowButton(true);
		}
			break;

		case DISABLE_COMPONENTS: {
			this.keyGenerationPanel.enableButton(false);
			this.keyGenerationPanel.enableTabsAndWorkflowButton(false);
		}
			break;

		case UPDATE_COMPONENTS:
			this.keyGenerationPanel
					.setMaxProgressBar(this.securityComponentsExportWorker
							.getMaxProgressBar());
			break;

		case SHOW_RESULT_MESSAGE:
			this.keyGenerationPanel
					.showMessage("KeyGenerationPanel.message.success");
			break;

		case SHOW_ERROR_MESSAGE:
			this.keyGenerationPanel
					.showErrorMessage(this.securityComponentsExportWorker
							.getErrorBundleKey());
			break;
		}

	}

}
