/**
 * KeyGenerationController.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.SecurityComponentsGenerationWorker;

/**
 * This control the execution of the KeyGeneration Functionality.
 */
public class SecurityComponentsGenerationController implements
		IAwtOperationListener {

	/** The keyGenerationPanel. */
	private KeyGenerationPanel keyGenerationPanel;

	SecurityComponentsGenerationWorker securityComponentsGenerationWorker;

	/**
	 * Class constructor.
	 * 
	 * @param keyGenerationPanel
	 *            Key Generation panel who created this controller
	 */
	public SecurityComponentsGenerationController(
			KeyGenerationPanel keyGenerationPanel) {
		this.keyGenerationPanel = keyGenerationPanel;
	}

	/**
	 * Shows a pop-up with information about the amount of time that the key
	 * generation task took to successfully complete the process.
	 * 
	 * @param start
	 *            Time when the process started
	 * @param stop
	 *            Time when the process ended
	 */
	private void showResultMessage(long start, long stop) {
		long miliseconds = stop - start;

		long hour, minute, second;
		long restHour, restMinute, restSecond;

		hour = miliseconds / 3600000;
		restHour = miliseconds % 3600000;

		minute = restHour / 60000;
		restMinute = restHour % 60000;

		second = restMinute / 1000;
		restSecond = restMinute % 1000;

		this.keyGenerationPanel.showResultMessage(hour, minute, second,
				restSecond);
	}

	/**
	 * Method that start the worker to perform back-end operations.
	 */
	public void startWorker() {

		this.securityComponentsGenerationWorker = new SecurityComponentsGenerationWorker();

		this.securityComponentsGenerationWorker.addListener(this);

		Thread keyGenerationWorkerThread = new Thread(
				this.securityComponentsGenerationWorker, "keyGenerationWorker");

		keyGenerationWorkerThread.start();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener#awtOperation
	 * (com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType)
	 */
	@Override
	public void awtOperation(AwtOperationType awtOperationType) {

		switch (awtOperationType) {
		case UPDATE_PROGRESS_BAR:
			this.keyGenerationPanel
					.updateProgressBar(this.securityComponentsGenerationWorker
							.getProgressBarValue());
			break;

		case SHOW_ERROR_MESSAGE:
			this.keyGenerationPanel
					.showErrorMessage(this.securityComponentsGenerationWorker
							.getErrorBundleKey());
			break;
		case SHOW_RESULT_MESSAGE:
			showResultMessage(
					this.securityComponentsGenerationWorker.getStartTime(),
					this.securityComponentsGenerationWorker.getStopTime());
			break;

		case ENABLE_COMPONENTS:
			this.keyGenerationPanel.enableButton(true);
			this.keyGenerationPanel.enableTabsAndWorkflowButton(true);
			break;

		case DISABLE_COMPONENTS: {
			this.keyGenerationPanel.enableButton(false);
			this.keyGenerationPanel.enableTabsAndWorkflowButton(false);
		}
			break;

		case UPDATE_COMPONENTS: {
			this.keyGenerationPanel
					.setMaxProgressBar(this.securityComponentsGenerationWorker
							.getMaxProgressBar());
		}
			break;
		}

	}

}
