/**
 * TreeRenderer.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui;

import java.awt.Component;
import java.util.ResourceBundle;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.jdesktop.application.ResourceMap;

import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;

/**
 * Class to specifies how a tree component should be rendered.
 */
class TreeRenderer extends DefaultTreeCellRenderer {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 263968887486818046L;

	/** The ok icon. */
	private ImageIcon okIcon;

	/** The error icon. */
	private ImageIcon errorIcon;

	/** The warning icon. */
	private ImageIcon warningIcon;

	/** The original leaf icon. */
	private Icon originalLeafIcon;

	/** The original closed icon. */
	private Icon originalClosedIcon;

	/** The original open icon. */
	private Icon originalOpenIcon;

	/** The resource map. */
	protected ResourceMap resourceMap = LanguageController.getInstance()
			.getResourceMap();

	/**
	 * Instantiates a new tree renderer.
	 */
	public TreeRenderer() {

		this.okIcon = this.resourceMap.getImageIcon("upload.ok.icon");
		this.errorIcon = this.resourceMap.getImageIcon("upload.error.icon");
		this.warningIcon = this.resourceMap.getImageIcon("upload.warning.icon");

		this.originalLeafIcon = this.leafIcon;
		this.originalClosedIcon = this.closedIcon;
		this.originalOpenIcon = this.openIcon;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.swing.tree.DefaultTreeCellRenderer#getTreeCellRendererComponent
	 * (javax.swing.JTree, java.lang.Object, boolean, boolean, boolean, int,
	 * boolean)
	 */
	@Override
	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean sel, boolean expanded, boolean leaf, int row,
			boolean hasFocus) {

		if (value instanceof DefaultMutableTreeNode) {
			ResourceBundle bundle = java.util.ResourceBundle
					.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp"); // NOI18N
			String unknownStatusText = bundle
					.getString("BulkUploadSmttFilesPanel.error.unknown.status");
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
			if (node.isRoot()) { // folder image
				this.setLeafIcon(null);
				this.setClosedIcon(this.originalClosedIcon);
				this.setOpenIcon(this.originalOpenIcon);
			} else if (node.getLevel() == 1) { // file image
				this.setLeafIcon(this.originalLeafIcon);
				boolean sw = true;
				for (int i = 0; i < node.getChildCount(); i++) {
					if (node.getChildAt(i).getChildCount() != 0
							|| ((String) ((DefaultMutableTreeNode) node
									.getChildAt(i)).getUserObject())
									.equals(unknownStatusText)) {
						if (((String) ((DefaultMutableTreeNode) node
								.getChildAt(i)).getUserObject())
								.equals(unknownStatusText)) {
							this.setClosedIcon(this.warningIcon);
							this.setOpenIcon(this.warningIcon);
						} else {
							this.setClosedIcon(this.errorIcon);
							this.setOpenIcon(this.errorIcon);
						}
						sw = false;
						break;
					}
				}
				if (sw) {
					this.setClosedIcon(this.okIcon);
					this.setOpenIcon(this.okIcon);
				}

			} else if (node.getLevel() == 2) // ok and error image
			{
				if (((String) (node.getUserObject())).equals(unknownStatusText)) {
					this.setClosedIcon(this.warningIcon);
					this.setOpenIcon(this.warningIcon);
					this.setLeafIcon(this.warningIcon);
				} else {
					this.setClosedIcon(this.errorIcon);
					this.setOpenIcon(this.errorIcon);
					this.setLeafIcon(this.okIcon);
				}
			} else {
				if (((String) (node.getUserObject())).equals(unknownStatusText)) {
					this.setClosedIcon(this.warningIcon);
					this.setOpenIcon(this.warningIcon);
					this.setLeafIcon(this.warningIcon);
				} else {
					this.setLeafIcon(null);
					this.setClosedIcon(null);
					this.setOpenIcon(null);
				}
			}
		}

		super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf,
				row, hasFocus);

		return this;
	}
}
