/**
 * ECMFileChooser.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui.common;

import java.awt.Component;
import java.awt.Container;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JTextField;

/**
 * The Class ECMFileChooser.
 */
public class ECMFileChooser extends JFileChooser {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -5913848285622641280L;

	/**
	 * Show open dialog.
	 * 
	 * @param parent
	 *            the parent
	 * @param buttonOpenText
	 *            the button open text
	 * @param buttonCancelText
	 *            the button cancel text
	 * @return the int
	 */
	public int showOpenDialog(Component parent, String buttonOpenText,
			String buttonCancelText) {
		this.setButtonsNames(this, buttonOpenText, buttonCancelText);
		this.setTextFieldName(this);
		this.repaint();
		return super.showOpenDialog(parent);
	}

	/**
	 * Show save dialog.
	 * 
	 * @param parent
	 *            the parent
	 * @param buttonOpenText
	 *            the button open text
	 * @param buttonCancelText
	 *            the button cancel text
	 * @return the int
	 */
	public int showSaveDialog(Component parent, String buttonOpenText,
			String buttonCancelText) {
		this.setButtonsNames(this, buttonOpenText, buttonCancelText);
		this.setTextFieldName(this);
		this.repaint();
		return super.showOpenDialog(parent);
	}

	/**
	 * Sets the buttons names.
	 * 
	 * @param c
	 *            the c
	 * @param buttonOpenText
	 *            the button open text
	 * @param buttonCancelText
	 *            the button cancel text
	 */
	private void setButtonsNames(Container c, String buttonOpenText,
			String buttonCancelText) {
		int len = c.getComponentCount();
		for (int i = 0; i < len; i++) {
			Component comp = c.getComponent(i);

			if (comp instanceof JButton) {
				JButton b = (JButton) comp;

				if (b != null && b.getText() != null
						&& b.getText().equals(buttonOpenText)) {
					b.setName("buttonOpenFile");
				}

				if (b != null && b.getText() != null
						&& b.getText().equals(buttonCancelText)) {
					b.setName("buttonCancelFile");
				}

			} else if (comp instanceof Container) {
				setButtonsNames((Container) comp, buttonOpenText,
						buttonCancelText);
			}
		}
	}

	/**
	 * Sets the text field name.
	 * 
	 * @param c
	 *            the container to find
	 */
	private void setTextFieldName(Container c) {
		int len = c.getComponentCount();
		for (int i = 0; i < len; i++) {
			Component comp = c.getComponent(i);

			if (comp instanceof JTextField) {
				JTextField b = (JTextField) comp;

				if (b != null) {
					b.setName("textFieldFilename");
				}

			} else if (comp instanceof Container) {
				setTextFieldName((Container) comp);
			}
		}
	}

}
