/**
 * SettingDateDialog.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui.dates;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.plaf.basic.BasicArrowButton;
import javax.swing.plaf.basic.BasicSpinnerUI;

import org.apache.commons.io.IOUtils;

import com.jidesoft.spinner.DateSpinner;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.core.util.SAESCommandUtil;
import com.smartmatic.waffle.ecm.ui.BaseFullScreenJDialog;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * The Class is used to show and set a datetime.
 */
public class SettingDateDialog extends BaseFullScreenJDialog {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -1288506625819345269L;

	/** For log messages purpose. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(SettingDateDialog.class);

	/** The date time picker panel. */
	private javax.swing.JPanel dateTimePickerPanel;

	/** The spinner date. */
	private DateSpinner spinnerDate;

	/** The spinner time. */
	private DateSpinner spinnerTime;

	/** The message label. */
	private JLabel messageLabel;

	/** The date panel. */
	private JPanel datePanel;

	/** The parent. */
	private JDialog parent;

	/**
	 * Creates new form SettingDateDialog.
	 * 
	 * @param parent
	 *            the parent
	 * @param modal
	 *            the modal
	 */
	public SettingDateDialog(JDialog parent, boolean modal) {
		super(parent, modal);
		this.initComponents();
		this.parent = this;

		this.ChangeDateButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				Date date = SettingDateDialog.this.getDatePicked();

				if (SettingDateDialog.this.initialDate.compareTo(date) <= 0) {
					SettingDateDialog.this.ChangeDateButton.setEnabled(false);

					SimpleDateFormat dateformatYYYYMMDD = new SimpleDateFormat(
							ECMClientSideConstants.DATE_PATTERN);

					int result = ECMOptionPane
							.showOptionDialog(
									SettingDateDialog.this.parent,
									SettingDateDialog.this.resourceMap
											.getString(
													"SettingDateDialog.change.date.confir.message",
													dateformatYYYYMMDD
															.format(date)),
									SettingDateDialog.this.resourceMap
											.getString("SettingDateDialog.change.date.title"),
									ECMOptionPane.YES_NO_OPTION,
									ECMOptionPane.QUESTION_MESSAGE, null, null,
									null, ECMOptionPane.CONFIRM_DIALOG);

					if (result == 0) {
						ThreadToSetSystemDate threat = new ThreadToSetSystemDate();
						threat.start();
					} else {
						SettingDateDialog.this.ChangeDateButton
								.setEnabled(true);
					}
				} else {
					SimpleDateFormat dateformatYYYYMMDD = new SimpleDateFormat(
							ECMClientSideConstants.DATE_PATTERN);
					String message = SettingDateDialog.this.resourceMap
							.getString(
									"SettingDateDialog.change.date.invalid.date.message",
									dateformatYYYYMMDD
											.format(SettingDateDialog.this.initialDate));

					String title = SettingDateDialog.this.resourceMap
							.getString("SettingDateDialog.change.date.title");

					ECMOptionPane.showMessageDialog(
							SettingDateDialog.this.parent, message, title,
							ECMOptionPane.WARNING_MESSAGE,
							ECMOptionPane.WARNING_DIALOG);
				}
			}
		});
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
	// <editor-fold defaultstate="collapsed"
	// desc="Generated Code">//GEN-BEGIN:initComponents
	private void initComponents() {

		this.dateControlsPanel = new javax.swing.JPanel();
		this.ChangeDateButton = new javax.swing.JButton();
		this.headerLabel = new JLabel();

		this.datePanel = this.getDatePickerPanel();

		this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

		this.dateControlsPanel.setName("dateControlsPanel"); // NOI18N

		javax.swing.GroupLayout dateControlsPanelLayout = new javax.swing.GroupLayout(
				this.dateControlsPanel);
		this.dateControlsPanel.setLayout(dateControlsPanelLayout);

		dateControlsPanelLayout.setHorizontalGroup(dateControlsPanelLayout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						javax.swing.GroupLayout.Alignment.TRAILING,
						dateControlsPanelLayout
								.createSequentialGroup()
								.addGap(42, 42, 42)
								.addComponent(this.datePanel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										651, Short.MAX_VALUE)
								.addGap(50, 50, 50)));

		dateControlsPanelLayout
				.setVerticalGroup(dateControlsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								dateControlsPanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addComponent(
												this.datePanel,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												161, Short.MAX_VALUE)
										.addContainerGap()));

		this.ChangeDateButton.setText(this.resourceMap
				.getString("SettingDateDialog.change.date.button.text"));
		this.ChangeDateButton.setName("ChangeDateButton"); // NOI18N

		this.headerLabel.setText(this.resourceMap
				.getString("SettingDateDialog.change.date.info.message"));
		this.headerLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.headerLabel.setFont(new java.awt.Font("DejaVu Sans", 1, 14));
		this.headerLabel.setName("messageLabel"); // NOI18N

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(
				this.getContentPane());
		this.getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addGap(50, 50, 50)
								.addComponent(this.dateControlsPanel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addGap(50, 50, 50))

				.addGroup(
						layout.createSequentialGroup()
								.addGap(262, 262, 262)
								.addComponent(this.ChangeDateButton,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addGap(278, 278, 278))
				.addGroup(
						javax.swing.GroupLayout.Alignment.TRAILING,
						layout.createSequentialGroup()
								.addGap(50, 50, 50)
								.addComponent(this.headerLabel,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										javax.swing.GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE).addGap(50, 50, 50)));

		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup()
						.addGap(32, 32, 32)
						.addComponent(this.headerLabel,
								javax.swing.GroupLayout.PREFERRED_SIZE, 52,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(60, 60, 60)
						.addComponent(this.dateControlsPanel,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								javax.swing.GroupLayout.DEFAULT_SIZE,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addGap(136, 136, 136)
						.addComponent(this.ChangeDateButton,
								javax.swing.GroupLayout.PREFERRED_SIZE, 52,
								javax.swing.GroupLayout.PREFERRED_SIZE)
						.addContainerGap(157, Short.MAX_VALUE)));

		this.pack();
	}// </editor-fold>//GEN-END:initComponents

	// Variables declaration - do not modify//GEN-BEGIN:variables
	/** The Change date button. */
	private javax.swing.JButton ChangeDateButton;

	/** The date controls panel. */
	private javax.swing.JPanel dateControlsPanel;

	/** The header label. */
	private javax.swing.JLabel headerLabel;

	// End of variables declaration//GEN-END:variables

	/** The initial date. */
	private Date initialDate;

	/**
	 * Gets the date picker panel.
	 * 
	 * @return the date picker panel
	 */
	private JPanel getDatePickerPanel() {

		this.dateTimePickerPanel = new javax.swing.JPanel();

		this.spinnerDate = new DateSpinner(
				ECMClientSideConstants.DATE_DATE_PATTERN);

		this.spinnerTime = new DateSpinner(
				ECMClientSideConstants.DATE_TIME_PATTERN);

		Dimension spnSz = new Dimension(330, 100), pnlSz = new java.awt.Dimension(
				350, 230);

		this.spinnerDate.setPreferredSize(spnSz);
		this.spinnerDate.setMaximumSize(new Dimension(Short.MAX_VALUE, 100));
		this.spinnerDate.setMinimumSize(spnSz);

		this.spinnerTime.setPreferredSize(spnSz);
		this.spinnerTime.setMaximumSize(new Dimension(Short.MAX_VALUE, 100));
		this.spinnerTime.setMinimumSize(spnSz);

		this.spinnerDate.setFont(this.resourceMap
				.getFont("date.diagnostic.spinner.font"));
		this.spinnerTime.setFont(this.resourceMap
				.getFont("date.diagnostic.spinner.font"));

		this.setName("Form"); // NOI18N

		this.dateTimePickerPanel.setName("dateTimePickerPanel"); // NOI18N
		this.dateTimePickerPanel.setLayout(new javax.swing.BoxLayout(
				this.dateTimePickerPanel, javax.swing.BoxLayout.PAGE_AXIS));

		this.changedSpinnerStyle(this.spinnerDate);

		this.changedSpinnerStyle(this.spinnerTime);

		this.dateTimePickerPanel.add(this.spinnerDate);
		this.spinnerDate
				.setEnabled(ECMClientSideConstants.CHANGE_DATE_AVAILABLE);

		this.dateTimePickerPanel.add(this.spinnerTime);
		this.spinnerTime
				.setEnabled(ECMClientSideConstants.CHANGE_TIME_AVAILABLE);
		this.messageLabel = new JLabel();
		this.messageLabel.setPreferredSize(new Dimension(Short.MAX_VALUE, 30));
		this.messageLabel.setMaximumSize(new Dimension(Short.MAX_VALUE, 30));
		this.messageLabel.setMinimumSize(new Dimension(Short.MAX_VALUE, 30));
		this.messageLabel
				.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.messageLabel.setForeground(Color.RED);

		this.dateTimePickerPanel.add(this.messageLabel);

		this.dateTimePickerPanel.setPreferredSize(pnlSz);
		this.dateTimePickerPanel.setMaximumSize(new Dimension(Short.MAX_VALUE,
				100));
		this.dateTimePickerPanel.setMinimumSize(pnlSz);

		Calendar dateInitialTmp = Calendar.getInstance();
		dateInitialTmp.setTime(this.getDatePicked());
		dateInitialTmp.set(Calendar.SECOND, 0);
		dateInitialTmp.set(Calendar.MILLISECOND, 0);

		SimpleDateFormat sdf = new SimpleDateFormat(
				ECMClientSideConstants.DATE_DATE_PATTERN);

		this.spinnerDate.setValue(dateInitialTmp.getTime());
		this.spinnerTime.setValue(dateInitialTmp.getTime());

		this.initialDate = dateInitialTmp.getTime();

		return this.dateTimePickerPanel;
	}

	/**
	 * Creates the arrow button.
	 * 
	 * @param direction
	 *            the direction
	 * @return the component
	 */
	private Component createArrowButton(int direction) {
		JButton b = new BasicArrowButton(direction) {
			@Override
			public Dimension getPreferredSize() {
				return new Dimension(30, 30);
			}
		};

		b.setBorder(null);

		b.setInheritsPopupMenu(true);
		return b;
	}

	/**
	 * Changed spinner style.
	 * 
	 * @param spinner
	 *            the spinner
	 */
	private void changedSpinnerStyle(DateSpinner spinner) {
		spinner.setUI(new BasicSpinnerUI() {
			@Override
			protected Component createPreviousButton() {
				Component c = SettingDateDialog.this
						.createArrowButton(SwingConstants.SOUTH);
				c.setName("Spinner.previousButton");
				this.installPreviousButtonListeners(c);
				return c;
			}

			@Override
			protected Component createNextButton() {
				Component c = SettingDateDialog.this
						.createArrowButton(SwingConstants.NORTH);
				c.setName("Spinner.nextButton");
				this.installNextButtonListeners(c);
				return c;
			}
		});
		spinner._timeEditor.getTextField().setHorizontalAlignment(
				JTextField.CENTER);
		spinner._timeEditor.getTextField().setAlignmentY(
				JTextField.CENTER_ALIGNMENT);

		spinner.setFont(spinner.getFont().deriveFont(
				spinner.getFont().getStyle() | java.awt.Font.BOLD,
				spinner.getFont().getSize() + 5));
	}

	/**
	 * Gets the date picked.
	 * 
	 * @return the date picked
	 */
	public Date getDatePicked() {
		Calendar time = Calendar.getInstance();
		Calendar date = Calendar.getInstance();
		time.setTime((Date) this.spinnerTime.getValue());
		date.setTime((Date) this.spinnerDate.getValue());

		int day = date.get(Calendar.DAY_OF_MONTH);
		int month = date.get(Calendar.MONTH);
		int year = date.get(Calendar.YEAR);
		int hour = time.get(Calendar.HOUR_OF_DAY);
		int minute = time.get(Calendar.MINUTE);
		int second = time.get(Calendar.SECOND);

		Calendar newCalendar = Calendar.getInstance();
		newCalendar.clear();
		newCalendar.set(year, month, day, hour, minute, second);

		return newCalendar.getTime();

	}

	/**
	 * Sync server to client date time.
	 */
	public static void syncServerToClientDateTime() {

		Process process = null;
		try {

			SAESCommonDestinations defaultDestinations = SAESCommonDestinations
					.getInstance();

			String serverDateTime = ECMServiceProvider.getServerDateTime(
					defaultDestinations.deviceUtilScript.getAbsolutePath(),
					"MACHINE_DATE_TIME");

			process = SAESCommandUtil.execute(
					defaultDestinations.deviceUtilScript.getAbsolutePath(),
					true, "MACHINE_DATE", "\"" + serverDateTime + "\"");
			String aux = IOUtils.toString(process.getErrorStream());

			if (aux != null && aux.length() > 0) {
				logger.error("Error synchronizing date");
			} else {
				logger.info("Synchronizing client date time to:"
						+ serverDateTime);
			}

		} catch (BaseException e) {
			logger.error(e, "Error getting the date from server");
		} catch (IOException e) {
			logger.error(e, "Error setting the new date ");
		} finally {
			if (process != null) {
				process.destroy();
			}
		}
	}

	/**
	 * The Class ThreadToSetSystemDate.
	 */
	public class ThreadToSetSystemDate extends Thread {

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Thread#run()
		 */
		@Override
		public void run() {
			super.run();
			this.syncDateMachine();
			SettingDateDialog.this.setVisible(false);
			SettingDateDialog.this.dispose();
		}

		/***
		 * Method used to change the time of the machine.
		 * 
		 */
		private void syncDateMachine() {
			BufferedReader br = null;
			InputStream is = null;
			Process process = null;
			try {

				SAESCommonDestinations defaultDestinations = SAESCommonDestinations
						.getInstance();

				process = SAESCommandUtil.execute(
						defaultDestinations.deviceUtilScript.getAbsolutePath(),
						true, "MACHINE_DATE", "\""
								+ SettingDateDialog.this.getDatePicked()
										.toString() + "\"");
				is = process.getErrorStream();
				br = new BufferedReader(new InputStreamReader(is));
				String aux = br.readLine();

				if (aux != null
						&& aux.length() > 0
						&& aux != SettingDateDialog.this.getDatePicked()
								.toString()) {
					SettingDateDialog.logger
							.error("error setting the date = "
									+ aux
									+ " "
									+ SettingDateDialog.this.getDatePicked()
											.toString());
					ECMOptionPane
							.showMessageDialog(
									SettingDateDialog.this.parent,
									SettingDateDialog.this.resourceMap
											.getString("SettingDateDialog.change.date.error.message"),
									SettingDateDialog.this.resourceMap
											.getString("SettingDateDialog.change.date.title"),
									ECMOptionPane.ERROR_MESSAGE,
									ECMOptionPane.ERROR_DIALOG);

				} else {

					ECMServiceProvider.setServerDateTime(SettingDateDialog.this
							.getDatePicked(),
							defaultDestinations.deviceUtilScript
									.getAbsolutePath(), "MACHINE_DATE");

					ECMServiceProvider.setParameter(
							ECMClientSideConstants.PARAMETER_KEY_DATETIME_SET,
							"1");

					SettingDateDialog.logger
							.info("success setting the date to :"
									+ SettingDateDialog.this.getDatePicked()
											.toString());
					ECMOptionPane
							.showMessageDialog(
									SettingDateDialog.this.parent,
									SettingDateDialog.this.resourceMap
											.getString("SettingDateDialog.change.date.successfull.message"),
									SettingDateDialog.this.resourceMap
											.getString("change.date.title"),
									ECMOptionPane.INFORMATION_MESSAGE,
									ECMOptionPane.INFORMATION_DIALOG);

				}
			} catch (IOException e) {
				SettingDateDialog.logger.error(e, "error setting the date");
			} catch (BaseException e) {
				SettingDateDialog.logger.error(e,
						"error persisting date status");
			} finally {
				if (process != null) {
					process.destroy();
				}
				SettingDateDialog.this.ChangeDateButton.setEnabled(true);
				try {

					if (is != null) {
						is.close();
					}

					if (br != null) {
						br.close();
					}
				} catch (IOException e) {
					SettingDateDialog.logger.error(e, "error setting the date");
				}
			}
		}

	}
}
