/**
 * PassphraseUserInputDialog.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui.login;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.GroupLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import org.jboss.security.client.SecurityClient;
import org.jboss.security.client.SecurityClientFactory;
import org.jdesktop.application.Action;
import org.jdesktop.application.Application;
import org.jdesktop.application.Task;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.ui.core.TextFieldLimit;
import com.smartmatic.waffle.ecm.ui.util.DotDashBorder;
import com.smartmatic.waffle.ecm.ui.util.ECMOptionPane;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to display the screen that ask the user and password to provide
 * access to the application.
 * 
 * @version 1.0 (Oct 23, 2009)
 * @author Hector Rodriguez <hector.rodriguez@smartmatic.com>
 */

public class PassphraseUserInputDialog extends PassphraseInputDialog {

	/** The logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(PassphraseUserInputDialog.class);

	private static int wrongPasswordCounter = 0;

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 2253416462456125127L;

	/** The user label. */
	private JLabel userLabel;

	/** The password label. */
	private JLabel passwordLabel;

	/** Task that will check if the user has access to this application. */
	protected PassphraseUserCheckTask task;

	/** Language code that identifies this application language. */
	private LanguageData languageData;

	/** The adapter. */
	private KeyAdapter adapter = new java.awt.event.KeyAdapter() {

		@Override
		public void keyReleased(KeyEvent e) {
			PassphraseUserInputDialog.this.passphraseTextFieldKeyReleased(e);
		}
	};

	/**
	 * Constructor.
	 * 
	 * @param parent
	 *            , parent Frame
	 * @param modal
	 *            , boolean that indicates whether this dialog is modal or not
	 * @param inputType
	 *            , type of input (keyboard or screen keyboard)
	 * @param language
	 *            , language chosen
	 */
	public PassphraseUserInputDialog(LanguageData language) {
		super(new JFrame());
		this.setModal(true);
		this.languageData = language;
		this.rebuildPanel();
		this.clearFields(false);
	}

	public void setErrorMessage(String messageKey) {
		this.errorLabel.setForeground(Color.RED);
		this.errorLabel.setFont(this.getResourceMap()
				.getFont("errorLabel.font"));
		this.errorLabel.setText(this.getString(messageKey));
	}

	/**
	 * Rebuilds the current panel.
	 */
	private void rebuildPanel() {
		Dimension languageLabelSize, languageButtonSize = null;
		Color lblColor = null;

		this.mainFrame.setBorder(null);

		languageLabelSize = new Dimension(550, 90);
		languageButtonSize = new Dimension(200, 75);

		if (this.languageData != null) {
			lblColor = this.getResourceMap().getColor(
					this.languageData.getCustomCode()
							+ ".selection.label.background.color");
		}
		if (lblColor == null) {
			lblColor = Color.WHITE;
		}

		Font tknFont = null;
		if (this.languageData != null) {
			tknFont = this.getResourceMap()
					.getFont(
							this.languageData.getCustomCode()
									+ ".selection.label.font");
		}
		if (tknFont == null) {
			tknFont = this.passphraseLabel.getFont().deriveFont(
					this.passphraseLabel.getFont().getStyle()
							| java.awt.Font.BOLD,
					this.passphraseLabel.getFont().getSize() + 4);
		}

		this.passphraseLabel.setOpaque(true);
		this.passphraseLabel.setBackground(lblColor);
		this.passphraseLabel
				.setBorder(new DotDashBorder(
						DotDashBorder.PatternType.DASH_LONG.getPattern(), 1,
						Color.GRAY));
		this.passphraseLabel.setFont(tknFont);
		this.passphraseLabel.setPreferredSize(languageLabelSize);
		this.passphraseLabel.setMaximumSize(languageLabelSize);
		this.passphraseLabel.setMinimumSize(languageLabelSize);

		if (this.languageData != null) {
			tknFont = this.getResourceMap().getFont(
					this.languageData.getCustomCode()
							+ ".selection.button.font");
		}

		if (tknFont == null) {
			tknFont = this.okButton.getFont().deriveFont(
					this.okButton.getFont().getStyle() | java.awt.Font.BOLD,
					this.okButton.getFont().getSize() + 4);
		}

		this.okButton.setFont(tknFont);
		this.okButton.setPreferredSize(languageButtonSize);
		this.okButton.setMaximumSize(languageButtonSize);
		this.okButton.setMinimumSize(languageButtonSize);

		this.resetButton.setFont(tknFont);
		this.resetButton.setPreferredSize(languageButtonSize);
		this.resetButton.setMaximumSize(languageButtonSize);
		this.resetButton.setMinimumSize(languageButtonSize);

		this.userField.setDocument(new TextFieldLimit(50));
		this.passphraseTextField.setDocument(new TextFieldLimit(50));
		this.passphraseTextField.addKeyListener(this.adapter);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ui.passphrase.PassphraseInputDialog#initComponents
	 * ()
	 */
	@Override
	protected void initComponents() {
		super.initComponents();
		this.passphraseLabel.setText(this.getResourceMap().getString(
				"passphraseUserLabel.text"));
		this.userLabel = new JLabel(this.getResourceMap().getString(
				"passphrase.userName"));
		this.userLabel.setName("labelUser");
		this.userLabel.setFont(this.getResourceMap().getFont(
				"passphraseTextField.font")); // NOI18N
		this.passwordLabel = new JLabel(this.getResourceMap().getString(
				"passphrase.password"));
		this.passwordLabel.setName("labelPassword");
		this.passwordLabel.setFont(this.getResourceMap().getFont(
				"passphraseTextField.font")); // NOI18N
		this.userField = new JTextField();
		this.userField.setFont(this.getResourceMap().getFont(
				"passphraseTextField.font")); // NOI18N
		this.userField.setHorizontalAlignment(SwingConstants.CENTER);
		this.userField.setName("textFieldUser"); // NOI18N

		GroupLayout jPanel1Layout = new GroupLayout(this.jPanel1);
		this.jPanel1.setLayout(jPanel1Layout);

		jPanel1Layout
				.setHorizontalGroup(jPanel1Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								jPanel1Layout
										.createSequentialGroup()
										.addContainerGap(34, Short.MAX_VALUE)
										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.CENTER)

														.addComponent(
																this.userLabel,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																400,
																javax.swing.GroupLayout.PREFERRED_SIZE)

														.addComponent(
																this.userField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																200,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.passwordLabel,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																400,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.passphraseTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																200,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.errorLabel,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addContainerGap(34, Short.MAX_VALUE)));
		jPanel1Layout
				.setVerticalGroup(jPanel1Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								jPanel1Layout
										.createSequentialGroup()
										.addGap(12, 12, 12)
										.addComponent(
												this.errorLabel,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												34, Short.MAX_VALUE)
										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.CENTER)
														.addComponent(
																this.userLabel,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																10,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.userField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))

										.addGroup(
												jPanel1Layout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.CENTER)
														.addComponent(
																this.passwordLabel,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																20,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.passphraseTextField,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addContainerGap()));
		this.pack();
	}

	@SuppressWarnings("rawtypes")
	@Override
	@Action
	/**
	 * Task that disables the OK and RESET buttons while the user inputs the information
	 */
	public Task checkPassphrase() {
		Thread check = new Thread(new Runnable() {

			@Override
			public void run() {
				PassphraseUserInputDialog.this.okButton.setEnabled(false);
				PassphraseUserInputDialog.this.resetButton.setEnabled(false);
				PassphraseUserInputDialog.this.task = new PassphraseUserCheckTask(
						Application.getInstance(),
						PassphraseUserInputDialog.this);
			}
		});
		check.run();
		return this.task;
	}

	/**
	 * Method used to clear the <code>username</code> and <code>password</code>
	 * fields.
	 */
	public void clearFields(boolean keepUser) {

		if (keepUser) {
			this.passphraseTextField.setText("");
			this.passphraseTextField.requestFocusInWindow();
		} else {
			this.userField.setText("");
			this.passphraseTextField.setText("");
			this.userField.requestFocusInWindow();
		}

	}

	/**
	 * Logout the current user.
	 */
	public static void logout() {
		SecurityClient securedClient = null;
		try {
			securedClient = SecurityClientFactory.getSecurityClient();
			securedClient.setSimple(null, null);
			securedClient.logout();
		} catch (Exception e) {
			securedClient.logout();
			logger.error(e, "An error occurred while trying to log out");
		}
		logger.info("The user was successfully logged out the server");
	}

	/**
	 * Getter of <code>wrongPasswordCOunter</code> property.
	 * 
	 * @return <code>wrongPasswordCOunter</code> property value.
	 * @see #wrongPasswordCounter
	 */
	protected static int getWrongPasswordCounter() {
		return wrongPasswordCounter;
	}

	/**
	 * Setter of <code>wrongPasswordCOunter</code> property.
	 * 
	 * @param newPasswordCounterValue
	 *            the new define the number of password attempts available for
	 *            login
	 * @see #wrongPasswordCounter
	 */
	public static void resetWrongPasswordCounter() {
		wrongPasswordCounter = 0;
	}

	/**
	 * Update counter.
	 */
	protected void updateCounter() {
		wrongPasswordCounter++;
	}

	/**
	 * Shows Dialog alert after many login attempts
	 */
	public void showDialogMaxAttempsPermitted() {
		ECMOptionPane.showMessageDialog(
				this,
				this.getResourceMap().getString(
						"Application.shutdown.dialog.information_message"),
				this.getResourceMap().getString("ErrorMessage.title"),
				ECMOptionPane.ERROR_MESSAGE, ECMOptionPane.ERROR_DIALOG);
	}

	/**
	 * Passphrase text field key released.
	 * 
	 * @param evt
	 *            the evt
	 */
	private void passphraseTextFieldKeyReleased(KeyEvent evt) {
		if (this.passphraseTextField.getPassword().length > 0
				&& evt.getKeyChar() != '\n') {
			this.okButton.setEnabled(true);
			this.resetButton.setEnabled(true);
			this.errorLabel.setText(" ");
		} else {
			this.okButton.setEnabled(false);
			// this.resetButton.setEnabled(false);
		}
	}

}
