/**
 * FileUtilities.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.codec.digest.DigestUtils;

import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class to execute utility functionalities to files.
 */
public class FileUtilities {

	/** For log messages purpose. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(FileUtilities.class);

	/**
	 * Method used to get the SHA-256 from a file passed as param.
	 * 
	 * @param file
	 *            - file at wich is calculates the SHA-256.
	 * 
	 * @return the SHA-256 of the file passed as param.
	 */
	public static String getSHA256sum(File file) {
		InputStream fileInputStream = null;
		try {
			if (file.exists()) {
				fileInputStream = new FileInputStream(file);
				return DigestUtils.sha256Hex(fileInputStream);
			}
		} catch (FileNotFoundException e) {
			FileUtilities.logger.error(e, e.getMessage());
		} catch (IOException e) {
			FileUtilities.logger.error(e, e.getMessage());
		} finally {
			if (fileInputStream != null) {
				try {
					fileInputStream.close();
				} catch (IOException e) {
					FileUtilities.logger.error(e, e.getMessage());
				}
			}
		}
		return null;
	}
}