/**
 * ImagesUploadWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class that execute the main process to upload Voting Device image in a
 * backend thread.
 */
public class ImagesUploadWorker extends AsynchronousOperation {

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ImagesUploadWorker.class);

	/** progressBar value. */
	private int progressBarValue;

	/** Source file to upload. */
	private File srcFile;

	/** Destination file where the image will be uploaded. */
	private File destFile;

	/**
	 * Instantiates a new ImagesUploadWorker.
	 * 
	 * @param srcFile
	 *            Source file to upload
	 * @param destFile
	 *            Destination file where the image will be uploaded
	 */
	public ImagesUploadWorker(File srcFile, File destFile) {
		this.srcFile = srcFile;
		this.destFile = destFile;
	}

	/**
	 * Gets the progress bar value.
	 * 
	 * @return the progress bar value
	 */
	public int getProgressBarValue() {
		return this.progressBarValue;
	}

	/**
	 * Sets the progress bar value.
	 * 
	 * @param progressBarValue
	 *            the new progress bar value
	 */
	public void setProgressBarValue(int progressBarValue) {
		this.progressBarValue = progressBarValue;
	}

	/**
	 * Method that copy byte per byte from source to destination.
	 * 
	 * @param srcFile
	 *            Source file to upload
	 * @param destFile
	 *            Destination file where the image will be uploaded
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public void copyImage(File srcFile, File destFile) throws IOException {

		FileInputStream in = new FileInputStream(srcFile);

		long imageSize = (long) (srcFile.length() * 1.1);

		long sizeprocessed = 0;

		FileOutputStream imageForDrive = new FileOutputStream(destFile);

		byte[] buffer = new byte[4096];

		try {
			do {
				int len = in.read(buffer);
				if (len == -1) {
					break;
				}

				imageForDrive.write(buffer, 0, len);

				setProgressBarValue((int) ((sizeprocessed * 100) / imageSize));

				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);

				sizeprocessed = sizeprocessed + len;

			} while (true);
		} finally {
			in.close();
			imageForDrive.close();
		}

		setProgressBarValue(100);

		fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {
		try {
			copyImage(this.srcFile, this.destFile);
		} catch (IOException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
	}
}
