/**
 * IntegrityValidationWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.data.transfer.objects.QueryResultDTO;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * The Class IntegrityValidationExecutorWorker used to perform server side SQL
 * queries.
 */
public class IntegrityValidationExecutorWorker extends AsynchronousOperation {

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(IntegrityValidationExecutorWorker.class);

	/** The SQL Query to execute. */
	private String sqlQuery;

	/** Object which contains the result of the query. */
	private QueryResultDTO queryResult;

	/** The error bundle key. */
	private String errorBundleKey;

	/**
	 * Gets the error bundle key.
	 * 
	 * @return the error bundle key
	 */
	public String getErrorBundleKey() {
		return this.errorBundleKey;
	}

	/**
	 * Sets the error bundle key.
	 * 
	 * @param errorBundleKey
	 *            the new error bundle key
	 */
	public void setErrorBundleKey(String errorBundleKey) {
		this.errorBundleKey = errorBundleKey;
	}

	/**
	 * Gets the SQL Query to execute.
	 * 
	 * @return the SQL Query to execute
	 */
	public String getSqlQuery() {
		return this.sqlQuery;
	}

	/**
	 * Sets the SQL Query to execute.
	 * 
	 * @param sqlQuery
	 *            the new SQL Query to execute
	 */
	public void setSqlQuery(String sqlQuery) {
		this.sqlQuery = sqlQuery;
	}

	/**
	 * Gets the object which contains the result of the query.
	 * 
	 * @return the object which contains the result of the query
	 */
	public QueryResultDTO getQueryResult() {
		return this.queryResult;
	}

	/**
	 * Sets the object which contains the result of the query.
	 * 
	 * @param queryResult
	 *            the new object which contains the result of the query
	 */
	public void setQueryResult(QueryResultDTO queryResult) {
		this.queryResult = queryResult;
	}

	/**
	 * Executes the selected query.
	 * 
	 * @throws UtilitiesException
	 *             the integrity exception
	 * @throws ConnectionException
	 *             the connection exception
	 */
	private void executeQuery() throws UtilitiesException, ConnectionException {
		this.queryResult = ECMServiceProvider.executeQuery(this.sqlQuery);
		fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {

		fireAwtEvent(AwtOperationType.DISABLE_COMPONENTS);
		try {
			executeQuery();
		} catch (UtilitiesException e) {

			logger.error(e, StringUtils.defaultString(e.getMessage()));
			this.errorBundleKey = e.getBundleKey();
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (ConnectionException e) {

			logger.error(e, StringUtils.defaultString(e.getMessage()));
			this.errorBundleKey = e.getBundleKey();
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (Exception e) {

			logger.error(e, StringUtils.defaultString(e.getMessage()));
			this.errorBundleKey = "ecm.generic.error";
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} finally {
			fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
	}

}
