package com.smartmatic.waffle.ecm.client.worker;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * This class is used to update the eligible option data sending the request to
 * the server and reports the status to the client.
 * 
 */
public class UpdateEligibleOptionWorker extends AsynchronousOperation {

	/**
	 * Static reference to log any info or error which occurred executing
	 * methods from this class.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(UpdateEligibleOptionWorker.class);

	private boolean isParty;
	private EligibleOptionData eligibleOption;

	/**
	 * Creates a new instance of the UpdateEligibleOptionWorker
	 * 
	 * @param eligibleOption
	 *            the eligible option object to persist.
	 */
	public UpdateEligibleOptionWorker(EligibleOptionData eligibleOption) {
		this.eligibleOption = eligibleOption;
		this.isParty = false;
	}

	/**
	 * Creates a new instance of the UpdateEligibleOptionWorker
	 * 
	 * @param eligibleOption
	 *            the eligible option object to persist.
	 * @param isParty
	 *            true if the current eligible option object refers to a party.
	 */
	public UpdateEligibleOptionWorker(EligibleOptionData eligibleOption,
			boolean isParty) {
		this.eligibleOption = eligibleOption;
		this.isParty = isParty;
	}

	/**
	 * Updates the eligible option and party objects.
	 * 
	 * @throws NominationException
	 *             a signal that an error occurred updating the eligible option
	 *             or party.
	 * @throws ConnectionException
	 *             a signal that a connection error occurred
	 */
	private void updateParty() throws NominationException, ConnectionException {
		ECMServiceProvider.updateEligibleOptionAndParty(this.eligibleOption);
	}

	/**
	 * Updates the eligible option object
	 * 
	 * @throws NominationException
	 *             a signal that an error occurred updating the eligible option
	 *             or party.
	 * @throws ConnectionException
	 *             a signal that a connection error occurred
	 */
	private void updateCandidate() throws NominationException,
			ConnectionException {
		ECMServiceProvider.updateEligibleOption(this.eligibleOption);
	}

	@Override
	protected void performOperation() {
		try {
			if (this.isParty) {
				updateParty();
			} else {
				updateCandidate();
			}
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	@Override
	protected void onCompleted() {
	}

}
