/**
 * BulkUploadDownloadTranslationsTemplateWorkerTest.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;

/**
 * Class to test BulkUploadDownloadTranslationsTemplateWorker.
 */
@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class,
		BulkUploadDownloadTranslationsTemplateWorker.class, PendriveUtil.class })
public class BulkUploadDownloadTranslationsTemplateWorkerTest {

	/**
	 * Prepares the test
	 * 
	 * @throws Exception
	 *             signals that an exception has occurred preparing the test.
	 *             This test should not throws any exception.
	 */
	@Before
	public void setUp() throws Exception {
		mockStaticMethods();
	}

	/**
	 * Test generateFileLines method.
	 * 
	 * @throws Exception
	 */
	@Test
	public final void testGenerateFileLines() throws Exception {

		// arrange
		File fileToExport = new File("test/resources/export.txt");
		BulkUploadDownloadTranslationsTemplateWorker bulkUploadDownloadTranslationsTemplateWorker = new BulkUploadDownloadTranslationsTemplateWorker(
				fileToExport);

		PowerMockito.mockStatic(PendriveUtil.class);

		PowerMockito.doNothing().when(PendriveUtil.class, "sync");

		// act
		bulkUploadDownloadTranslationsTemplateWorker.generateFileLines();

		// assert
		PowerMockito.verifyStatic();
		ECMServiceProvider.getTranslationkeysToDownload(0, 300);
		assertEquals(
				bulkUploadDownloadTranslationsTemplateWorker
						.getMultilingualKeysPagesAmount(),
				1);
		assertTrue(fileToExport.exists());
		assertTrue(fileToExport.delete());
	}

	/**
	 * Prepares mock for ECMServiceProvider class
	 * 
	 * @throws Exception
	 *             signals that an exception has occurred preparing mock. This
	 *             test should not throws any exception.
	 */
	private void mockStaticMethods() throws Exception {
		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.when(ECMServiceProvider.retrieveMultilingualKeysAmount())
				.thenReturn(1);

		List<String> lineList = new ArrayList<String>();

		lineList.add("line1");
		lineList.add("line2");

		PowerMockito.when(
				ECMServiceProvider.getTranslationkeysToDownload(0, 300))
				.thenReturn(lineList);
	}

}
