/**
 * BulkUploadCodiFilesWorkerTest.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDetailDTO;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;

/**
 * Class to test BulkUploadCodiFilesWorker.
 */
@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class, BulkUploadEMLFilesWorker.class })
public class BulkUploadEMLFilesWorkerTest {

	private List<FileToUploadDTO> emlFilesToUpĺoadList = new ArrayList<FileToUploadDTO>();

	/**
	 * Prepares the test
	 * 
	 * @throws Exception
	 *             signals that an exception has occurred preparing the test.
	 *             This test should not throws any exception.
	 */
	@Before
	public void setUp() throws Exception {
		mockStaticMethods();
	}

	/**
	 * Test startBulkUploadOperation method.
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred executing bulk upload.
	 *             This test should not throws any exception.
	 * 
	 * @throws ConnectionException
	 *             signals that a connection problem has occurred. This test
	 *             should not throws any exception.
	 */
	@Test
	public final void testStartBulkUploadOperation()
			throws BulkDataLoadException, ConnectionException {

		// arrange
		BulkUploadEMLFilesWorker bulkUploadCodiFilesWorker = new BulkUploadEMLFilesWorker(
				this.emlFilesToUpĺoadList, null);

		// act
		bulkUploadCodiFilesWorker.startBulkUploadOperation();

		// assert
		PowerMockito.verifyStatic(Mockito.times(2));
		ECMServiceProvider.uploadEMLFileIntoDataBase("TestFile.txt", null,
				null, null);
		assertEquals(bulkUploadCodiFilesWorker.getFileUploadedDetailList()
				.size(), 2);
		assertEquals(bulkUploadCodiFilesWorker.getNumberOfProcessedFiles(), 2);
	}

	/**
	 * Prepares mock for ECMServiceProvider class
	 * 
	 * @throws Exception
	 *             signals that an exception has occurred preparing mock. This
	 *             test should not throws any exception.
	 */
	private void mockStaticMethods() throws Exception {
		File file = new File("TestFile.txt");

		FileToUploadDetailDTO fileToUploadDetailDTO = new FileToUploadDetailDTO();

		fileToUploadDetailDTO.setFileInfo(file);

		FileToUploadDTO codiFileToUpload1 = new FileToUploadDTO();

		codiFileToUpload1.setFileToUploadDetailDTO(fileToUploadDetailDTO);

		FileToUploadDTO codiFileToUpload2 = new FileToUploadDTO();

		codiFileToUpload2.setFileToUploadDetailDTO(fileToUploadDetailDTO);

		this.emlFilesToUpĺoadList.add(codiFileToUpload1);

		this.emlFilesToUpĺoadList.add(codiFileToUpload2);

		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.when(
				ECMServiceProvider.uploadEMLFileIntoDataBase("TestFile.txt",
						null, null, null)).thenReturn(new BulkResultDTO());
	}
}
