package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.service.lang.NominationException;

/**
 * 
 * Test class for EligibleOptionAdvancedSearchWorker
 */
@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class,
		EligibleOptionAdvancedSearchWorker.class })
public class EligibleOptionAdvancedSearchWorkerTest {

	@Test
	public void testGetParties() throws ConnectionException,
			NominationException {
		EligibleOptionAdvancedSearchWorker worker = new EligibleOptionAdvancedSearchWorker(
				null, null, true);

		List<PartyData> partiesList = new ArrayList<PartyData>();
		List<PartyData> partiesListResult;

		PartyData party1 = new PartyData();
		party1.setCode(1);
		party1.setName("party 1");

		PartyData party2 = new PartyData();
		party2.setCode(2);
		party2.setName("party 2");

		partiesList.add(party1);
		partiesList.add(party2);

		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.when(ECMServiceProvider.getParties(null, null))
				.thenReturn(partiesList);

		worker.performOperation();

		partiesListResult = worker.getPartiesList();

		assertEquals(2, partiesListResult.size());

		for (int i = 0; i < partiesListResult.size(); i++) {
			assertEquals(partiesList.get(i).getCode(), partiesListResult.get(i)
					.getCode());
			assertEquals(partiesList.get(i).getName(), partiesListResult.get(i)
					.getName());
		}
	}

	@Test
	public void testGetContests() throws ConnectionException,
			NominationException {

		EligibleOptionAdvancedSearchWorker worker = new EligibleOptionAdvancedSearchWorker(
				null, null, null);

		List<ContestData> contestsList = new ArrayList<ContestData>();
		List<ContestData> contestsListResult;

		ContestData contest1 = new ContestData();
		contest1.setCode(1);
		contest1.setName("contests 1");

		ContestData contest2 = new ContestData();
		contest2.setCode(2);
		contest2.setName("contests 2");

		contestsList.add(contest1);
		contestsList.add(contest2);

		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.when(ECMServiceProvider.getContests(null, null, null))
				.thenReturn(contestsList);

		worker.performOperation();

		contestsListResult = worker.getContestsList();

		assertEquals(2, contestsListResult.size());

		for (int i = 0; i < contestsListResult.size(); i++) {
			assertEquals(contestsList.get(i).getCode(),
					contestsListResult.get(i).getCode());
			assertEquals(contestsList.get(i).getName(),
					contestsListResult.get(i).getName());
		}
	}

}
