package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.authority.ElectoralAuthorityData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.ProgrammingUnitData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.VotingCentersData.VotingCenter;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.device.VotingDevicesData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.domain.general.ReportTemplateData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.util.MasterFileGenerationUtils;
import com.smartmatic.waffle.ecm.client.worker.data.FileData;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionObjectDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionXmlComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ProgrammingUnitElectionObjectDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceDTO;
import com.thoughtworks.xstream.XStream;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ MasterUSBFileGenerationWorker.class, ECMServiceProvider.class,
		MasterFileGenerationUtils.class })
public class MasterUSBFileGenerationWorkerTest {

	MasterUSBFileGenerationWorker masterUSBFileGenerationWorker;

	@Before
	public void setUp() {

		ProgrammingUnitElectionObjectDTO programmingUnitElectionObject = new ProgrammingUnitElectionObjectDTO();

		Set<ProgrammingUnitData> programmingUnitSet = new HashSet<ProgrammingUnitData>();

		ProgrammingUnitData programmingUnit = new ProgrammingUnitData();
		programmingUnit.setCode(1);
		programmingUnit.setProgrammingUnitName("FED-TEST");

		programmingUnitSet.add(programmingUnit);

		programmingUnitElectionObject.setProgrammingUnitSet(programmingUnitSet);

		Map<Long, ElectionObjectDTO> electionObjectMap = new HashMap<Long, ElectionObjectDTO>();

		ElectionObjectDTO electionObjectDto = new ElectionObjectDTO();

		ElectionXmlComponentsDTO electionComponents = new ElectionXmlComponentsDTO();
		electionComponents.setElection(new ElectionData());

		electionObjectDto.setElectionDto(electionComponents);

		electionObjectMap.put((long) 1, electionObjectDto);

		programmingUnitElectionObject.setElectionObjectMap(electionObjectMap);

		String mountedPath = "test/resources/master_folder";

		File mountedFolder = new File(mountedPath);

		mountedFolder.mkdirs();

		this.masterUSBFileGenerationWorker = PowerMockito
				.spy(new MasterUSBFileGenerationWorker(mountedPath,
						programmingUnitElectionObject, true,
						"UgwzYIdmOlj/7h+7dlEHdpxtree2XUahPKf62tIbvtw="));
	}

	@Test
	public void testPerformOperation() throws Exception {

		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito
				.when(ECMServiceProvider
						.getServerPropertyAsString(Mockito.anyString()))
				.thenReturn("1");

		PowerMockito.when(ECMServiceProvider.getApplicationKey())
				.thenReturn("csdfsdfsd+fsfsf1");

		PowerMockito.mockStatic(ECMClientSideConstants.class);

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"processLastLevelRegions", Mockito.any(ElectionObjectDTO.class),
				Mockito.anyListOf(RegionData.class),
				Mockito.any(ProgrammingUnitData.class),
				Mockito.any(File.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"generateVotingCenters", Mockito.any(ElectionData.class),
				Mockito.anyListOf(VotingCenter.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"generateDefaultLanguages", Mockito.any(ElectionData.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"generateInstallationFolder");

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"generateCertificates", Mockito.any(ElectionData.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"generateSignatureFile");

		PowerMockito.doReturn(1).when(this.masterUSBFileGenerationWorker,
				"getVotingDevicesTotal", Mockito.anyListOf(RegionData.class));

		PowerMockito.doReturn("test/resources/master_generation")
				.when(this.masterUSBFileGenerationWorker, "getTargetPath");

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"copyGeneratedFiles");

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"mergeSaesFiles");

		// act
		this.masterUSBFileGenerationWorker.performOperation();

		// assert

		PowerMockito.verifyPrivate(this.masterUSBFileGenerationWorker).invoke(
				"generateDefaultLanguages", Mockito.any(ElectionData.class));

		PowerMockito.verifyPrivate(this.masterUSBFileGenerationWorker)
				.invoke("generateInstallationFolder");

		PowerMockito.verifyPrivate(this.masterUSBFileGenerationWorker).invoke(
				"generateCertificates", Mockito.any(ElectionData.class));

		PowerMockito.verifyPrivate(this.masterUSBFileGenerationWorker)
				.invoke("generateSignatureFile");

	}

	@Test @Ignore
	public void testProcessLastLevelRegions() throws Exception {

		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"setElectoralAuthorityLanguages", Mockito.any(RegionData.class),
				Mockito.anySetOf(LanguageData.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker)
				.processVotingDeviceList(Mockito.any(RegionData.class),
						Mockito.anySetOf(LanguageData.class),
						Mockito.any(ElectionObjectDTO.class),
						Mockito.any(ProgrammingUnitData.class),
						Mockito.anyListOf(ContestGroupData.class),
						Mockito.anyListOf(File.class), Mockito.anyBoolean(),
						Mockito.any(File.class), Mockito.anyListOf(File.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"generateElection", Mockito.any(ElectionData.class),
				Mockito.any(ProgrammingUnitData.class));

		PowerMockito.doReturn(null).when(this.masterUSBFileGenerationWorker,
				"generateCommonAudioResources", Mockito.any(ElectionData.class),
				Mockito.any(ProgrammingUnitData.class));

		PowerMockito.doReturn(null).when(this.masterUSBFileGenerationWorker,
				"downloadVoiceFiles", Mockito.anyString(),
				Mockito.any(Set.class));

		PowerMockito
				.when(ECMServiceProvider
						.getParentRegions(Mockito.any(RegionData.class)))
				.thenReturn(null);

		Map<Long, Set<LanguageData>> regionLanguageMap = new HashMap<Long, Set<LanguageData>>();

		Set<LanguageData> languages = new HashSet<LanguageData>();

		LanguageData language = new LanguageData();
		language.setCode(1);

		languages.add(language);

		regionLanguageMap.put((long) 1, languages);

		ElectionObjectDTO electionObjectDto = new ElectionObjectDTO();

		electionObjectDto.setRegionLanguageMap(regionLanguageMap);

		List<RegionData> lastLevelRegions = new ArrayList<RegionData>();

		RegionData region = new RegionData();
		region.setName("region1");
		region.setCustomCode("region1-custom");

		lastLevelRegions.add(region);

		ProgrammingUnitData programmingUnit = new ProgrammingUnitData();
		programmingUnit.setCustomCode("PU 12");

		File mockFile = new File("mockFile.txt");

		ElectionXmlComponentsDTO electionComponents = new ElectionXmlComponentsDTO();
		electionComponents.setElection(new ElectionData());

		electionObjectDto.setElectionDto(electionComponents);

		this.masterUSBFileGenerationWorker.processLastLevelRegions(
				electionObjectDto, lastLevelRegions, programmingUnit, mockFile);

		PowerMockito.verifyPrivate(this.masterUSBFileGenerationWorker).invoke(
				"setElectoralAuthorityLanguages", Mockito.any(RegionData.class),
				Mockito.anySetOf(LanguageData.class));

		PowerMockito.verifyPrivate(this.masterUSBFileGenerationWorker).invoke(
				"generateElection", Mockito.any(ElectionData.class),
				Mockito.any(ProgrammingUnitData.class));
	}

	@Test
	public void testProcessVotingDeviceList() throws Exception {

		PowerMockito.mockStatic(ECMServiceProvider.class);

		RegionData region = new RegionData();
		region.setCode(1);

		Set<LanguageData> regionLanguageSet = new HashSet<LanguageData>();

		ElectionObjectDTO electionObject = new ElectionObjectDTO();
		ElectionXmlComponentsDTO electionComponents = new ElectionXmlComponentsDTO();
		electionComponents.setElection(new ElectionData());

		electionObject.setElectionDto(electionComponents);

		Map<Long, List<ContestGroupData>> regionContestGroup = new HashMap<Long, List<ContestGroupData>>();

		regionContestGroup.put((long) 1, new ArrayList<ContestGroupData>());

		electionObject.setRegionContestGroup(regionContestGroup);

		ProgrammingUnitData programmingUnit = new ProgrammingUnitData();
		programmingUnit.setCustomCode("programming unit 1");

		List<ContestGroupData> contestGroupForElection = new ArrayList<ContestGroupData>();

		List<VotingDeviceData> votingDevices = new ArrayList<VotingDeviceData>();

		List<File> voiceFiles = new ArrayList<File>();

		VotingDeviceData votingDevice1 = new VotingDeviceData();
		VotingDeviceData votingDevice2 = new VotingDeviceData();
		votingDevice1.setName("Voting Device 1");
		votingDevice1.setCustomCode("device 1");
		votingDevice2.setName("Voting Device 2");
		votingDevice2.setCustomCode("device 2");
		votingDevices.add(votingDevice1);
		votingDevices.add(votingDevice2);

		VotingDeviceDTO votingDeviceDTO = new VotingDeviceDTO();
		votingDeviceDTO.setVotingDevicesData(new VotingDevicesData());

		List<VotingCenter> votingCenterList = new ArrayList<VotingCenter>();

		File mockFile = new File("mockFile.txt");

		List<File> generalAudioResources = new ArrayList<File>();

		PowerMockito
				.when(ECMServiceProvider.getVotingDevicesByRegion(
						Mockito.any(RegionData.class)))
				.thenReturn(votingDevices);

		PowerMockito
				.when(ECMServiceProvider.generateVotingDeviceForRegion(
						Mockito.any(VotingDeviceData.class),
						Mockito.any(RegionData.class),
						Mockito.anyListOf(ContestGroupData.class),
						anyStringMap(), Mockito.anyBoolean(),
						Mockito.any(ElectionData.class), Mockito.anyBoolean()))
				.thenReturn(votingDeviceDTO);

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker)
				.generateXML(Mockito.anyString(),
						Mockito.any(ProgrammingUnitData.class),
						Mockito.any(VotingDevicesData.class),
						Mockito.any(ElectionData.class), Mockito.anyString(),
						Mockito.anyListOf(File.class), Mockito.any(File.class),
						Mockito.anyListOf(File.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"generatePVTemplates",
				Mockito.anyListOf(ReportTemplateData.class),
				Mockito.anyString());

		PowerMockito
				.when(ECMServiceProvider
						.getParentRegions(Mockito.any(RegionData.class)))
				.thenReturn(null);

		this.masterUSBFileGenerationWorker
				.setVotingCentersList(votingCenterList);

		this.masterUSBFileGenerationWorker.processVotingDeviceList(region,
				regionLanguageSet, electionObject, programmingUnit,
				contestGroupForElection, voiceFiles, false, mockFile,
				generalAudioResources);

		// assert
		assertEquals(2, votingCenterList.size());

		assertEquals(votingDevice1.getName(),
				votingCenterList.get(0).getCode());

		assertEquals(votingDevice1.getCustomCode().replace(" ", "_"),
				votingCenterList.get(0).getFolderName());

		assertEquals(programmingUnit.getCustomCode().replace(" ", "_"),
				votingCenterList.get(0).getProgrammingUnit());

		assertEquals(votingDevice2.getName(),
				votingCenterList.get(1).getCode());

		assertEquals(votingDevice2.getCustomCode().replace(" ", "_"),
				votingCenterList.get(1).getFolderName());

		assertEquals(programmingUnit.getCustomCode().replace(" ", "_"),
				votingCenterList.get(1).getProgrammingUnit());

	}

	@Test
	public void testGenerateXML() throws Exception {

		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.mockStatic(MasterFileGenerationUtils.class);

		String votingDeviceCustomCode = "voting_device_1";

		ProgrammingUnitData programmingUnit = new ProgrammingUnitData();
		programmingUnit.setCustomCode("programming unit test");

		VotingDevicesData votingDevices = new VotingDevicesData();

		VotingDeviceData votingDevice = new VotingDeviceData();

		String protectionKey = "protectionKey";
		Set<LanguageData> languages = new HashSet<LanguageData>();

		votingDevice.setName("VotingDevice001");
		votingDevice.setProtectionKey(protectionKey);
		votingDevice.setLanguages(languages);

		votingDevices.setVotingDeviceData(votingDevice);

		List<File> voiceFiles = new ArrayList<File>();

		ElectionData election = new ElectionData();

		File mockFile = new File("mockFile.txt");

		List<File> generalAudioResources = new ArrayList<File>();

		Map<ElectoralAuthorityData, Set<LanguageData>> electoralAuthorityLanguages = new HashMap<ElectoralAuthorityData, Set<LanguageData>>();

		PowerMockito.doReturn(new FileData()).when(
				MasterFileGenerationUtils.class, "writeXML",
				Mockito.any(XStream.class), Mockito.anyObject(),
				Mockito.any(File.class), Mockito.anyString(),
				Mockito.any(KeystoreData.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"writeVotingDeviceCertificate", Mockito.any(ImageWrapper.class),
				Mockito.anyString(), Mockito.any(File.class));

		PowerMockito.doReturn(new ArrayList<File>()).when(
				this.masterUSBFileGenerationWorker, "writePartyImages",
				Mockito.anySetOf(PartyData.class));

		PowerMockito.doNothing().when(this.masterUSBFileGenerationWorker,
				"generateVotingDeviceSignatureFromCache",
				Mockito.any(File.class), Mockito.anyListOf(File.class),
				Mockito.anyListOf(File.class), Mockito.anyString(),
				Mockito.any(ElectionData.class), Mockito.any(File.class),
				Mockito.anyListOf(File.class), Mockito.any(File.class));

		this.masterUSBFileGenerationWorker
				.setElectoralAuthorityLanguages(electoralAuthorityLanguages);

		this.masterUSBFileGenerationWorker.generateXML(votingDeviceCustomCode,
				programmingUnit, votingDevices, election, "ASFSDF232123DA23R",
				voiceFiles, mockFile, generalAudioResources);

		PowerMockito
				.when(ECMServiceProvider
						.getParentRegions(Mockito.any(RegionData.class)))
				.thenReturn(null);

		PowerMockito.verifyPrivate(this.masterUSBFileGenerationWorker).invoke(
				"generateVotingDeviceSignatureFromCache",
				Mockito.any(File.class), Mockito.anyListOf(File.class),
				Mockito.anyListOf(File.class), Mockito.anyString(),
				Mockito.any(ElectionData.class), Mockito.any(File.class),
				Mockito.anyListOf(File.class), Mockito.any(File.class));

	}

	@After
	public void clean() {
		String mountedPath = "test/resources/master_folder";

		File mountedFolder = new File(mountedPath);

		FileUtils.deleteQuietly(mountedFolder);
	}

	private static Map<String, Map<String, String>> anyStringMap() {
		return Matchers.any();
	}

}
