package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.ProgrammingUnitData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionObjectDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ProgrammingUnitElectionObjectDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ProgrammingUnitLastLevelRegionDTO;
import com.smartmatic.waffle.ecm.service.lang.DemographyException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class,
		MasterUSBPrepareContestsWorker.class })
public class MasterUSBPrepareContestsWorkerTest {

	MasterUSBPrepareContestsWorker masterUSBPrepareContestsWorker;

	@Before
	public void setUp() {

		LanguageData language = LanguageController.getInstance()
				.getSelectedLanguage();

		RegionData region1 = new RegionData();
		RegionData region2 = new RegionData();
		region1.setCode(1);
		region2.setCode(2);

		List<Object> selectedObjects = new ArrayList<Object>();

		selectedObjects.add(region1);
		selectedObjects.add(region2);

		this.masterUSBPrepareContestsWorker = new MasterUSBPrepareContestsWorker(
				selectedObjects, language, true);
	}

	@Test
	public void testGetContestsForElection() throws NominationException,
			ConnectionException {

		List<ContestData> contests = new ArrayList<ContestData>();
		ContestData contest1 = new ContestData();
		ContestData contest2 = new ContestData();

		contest1.setCode(1);
		contest1.setContestNameTranslate("Contest 1");
		contest2.setCode(2);
		contest2.setContestNameTranslate("Contest 2");

		contests.add(contest1);
		contests.add(contest2);

		Map<Long, ContestData> contestsMap = new HashMap<Long, ContestData>();

		List<ContestData> contestsResult = this.masterUSBPrepareContestsWorker
				.getContestsForElection(contests, contestsMap);

		assertEquals(2, contestsResult.size());

		assertEquals(2, contestsMap.entrySet().size());
	}

	@Test
	public void testProcessSelectedObjects() throws ConnectionException,
			ElectoralOrganizationOperationException, DemographyException,
			MasterUSBGenerationException, NominationException {

		LanguageData language = LanguageController.getInstance()
				.getSelectedLanguage();

		RegionData region1 = new RegionData();
		RegionData region2 = new RegionData();
		region1.setCode(1);
		region2.setCode(2);

		List<Object> selectedObjects = new ArrayList<Object>();

		selectedObjects.add(region1);
		selectedObjects.add(region2);

		PowerMockito.mockStatic(ECMServiceProvider.class);
		MasterUSBPrepareContestsWorker worker = PowerMockito
				.spy(new MasterUSBPrepareContestsWorker(selectedObjects,
						language, true));

		Set<ProgrammingUnitLastLevelRegionDTO> programmingUnitLastLevelRegionSet = new HashSet<ProgrammingUnitLastLevelRegionDTO>();

		ProgrammingUnitLastLevelRegionDTO pU1 = new ProgrammingUnitLastLevelRegionDTO();
		ProgrammingUnitLastLevelRegionDTO pU2 = new ProgrammingUnitLastLevelRegionDTO();

		List<RegionData> lastLevelRegions = new ArrayList<RegionData>();
		lastLevelRegions.add(region1);
		lastLevelRegions.add(region2);

		ProgrammingUnitData programmingUnit1 = new ProgrammingUnitData();
		ProgrammingUnitData programmingUnit2 = new ProgrammingUnitData();

		programmingUnit1.setCode(1);
		programmingUnit2.setCode(2);

		programmingUnit1.setProgrammingUnitName("Programming Unit 1");
		programmingUnit2.setProgrammingUnitName("Programming Unit 2");

		pU1.setLastLevelRegionForProgrammingUnit(lastLevelRegions);
		pU1.setProgrammingUnit(programmingUnit1);

		pU2.setLastLevelRegionForProgrammingUnit(lastLevelRegions);
		pU2.setProgrammingUnit(programmingUnit2);

		programmingUnitLastLevelRegionSet.add(pU1);
		programmingUnitLastLevelRegionSet.add(pU2);

		PowerMockito.when(
				ECMServiceProvider.getProgrammingUnitsByRegions(Mockito
						.anyListOf(RegionData.class))).thenReturn(
				programmingUnitLastLevelRegionSet);

		ElectionObjectDTO electionObject = new ElectionObjectDTO();

		PowerMockito
				.doReturn(electionObject)
				.when(worker)
				.getElection(Mockito.anyListOf(RegionData.class),
						Mockito.anyMapOf(Long.class, ContestData.class),
						Mockito.any(ElectionData.class), Mockito.anyInt());

		PowerMockito.doCallRealMethod().when(worker).processSelectedObjects();

		// act
		worker.processSelectedObjects();

		// assert
		ProgrammingUnitElectionObjectDTO programminUnitElectionObjectResult = worker
				.getProgrammingUnitElectionObject();

		assertEquals(electionObject, programminUnitElectionObjectResult
				.getElectionObjectMap().get(1L));

		assertEquals(electionObject, programminUnitElectionObjectResult
				.getElectionObjectMap().get(2L));

		assertEquals(4, programminUnitElectionObjectResult.getTotalRegions());

	}
}
