package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.ballot.BallotOptionData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.data.transfer.objects.EligibleOptionSearchDTO;
import com.smartmatic.waffle.ecm.service.lang.NominationException;

/**
 * Test class for SearchPartyWorker
 * 
 */
@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class, SearchPartyWorker.class })
public class SearchPartyWorkerTest {

	private SearchPartyWorker searchPartyWorker;

	@Before
	public void setUp() {
		this.searchPartyWorker = new SearchPartyWorker("parties", null, null,
				null, 1);
	}

	@Test
	public void testSetTotalNumberOfPages() throws NominationException,
			ConnectionException {
		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.when(
				ECMServiceProvider.retrieveNumberOfParties("parties", null,
						null)).thenReturn(17);

		this.searchPartyWorker.calculateTotalNumberOfPages(17);

		assertEquals(1, this.searchPartyWorker.getTotalNumberOfPages());

	}

	@Test
	public void testGetParties() throws ConnectionException,
			NominationException {

		PowerMockito.mockStatic(ECMServiceProvider.class);

		List<EligibleOptionData> partiesList = new ArrayList<EligibleOptionData>();

		EligibleOptionData eligibleOption1 = new EligibleOptionData();
		eligibleOption1.setBallotName("Party 1");
		eligibleOption1.setContest(new ContestData());
		eligibleOption1.setBallotOptions(new HashSet<BallotOptionData>());
		partiesList.add(eligibleOption1);

		PowerMockito.when(
				ECMServiceProvider
						.getParties(1, 1, "parties", null, null, null))
				.thenReturn(partiesList);

		List<EligibleOptionSearchDTO> partiesResultList;

		partiesResultList = this.searchPartyWorker.getParties(1);

		for (EligibleOptionSearchDTO eligibleOptionDTO : partiesResultList) {
			assertNotNull(eligibleOptionDTO.getValues());
		}
	}
}
