package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.service.lang.NominationException;

/**
 * 
 * Test class for UpdateEligibleOptionWorker
 */
@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class, UpdateEligibleOptionWorker.class })
public class UpdateEligibleOptionWorkerTest {

	private EligibleOptionData eligibleOption;

	@Before
	public void setUp() {
		this.eligibleOption = new EligibleOptionData();
		this.eligibleOption.setBallotName("EO 1");
	}

	@Test
	public void testUpdateCandidate() throws NominationException,
			ConnectionException {

		PowerMockito.mockStatic(ECMServiceProvider.class);

		UpdateEligibleOptionWorker updateEligibleOptionWorker = new UpdateEligibleOptionWorker(
				this.eligibleOption);

		EligibleOptionData eligibleOptionUpdated = new EligibleOptionData();
		eligibleOptionUpdated.setBallotName("EO 1");

		PowerMockito.when(
				ECMServiceProvider.updateEligibleOption(this.eligibleOption))
				.thenReturn(eligibleOptionUpdated);

		updateEligibleOptionWorker.performOperation();

		assertEquals(this.eligibleOption.getBallotName(),
				eligibleOptionUpdated.getBallotName());
	}

	@Test
	public void testUpdateParty() throws NominationException,
			ConnectionException {

		PowerMockito.mockStatic(ECMServiceProvider.class);

		UpdateEligibleOptionWorker updateEligibleOptionWorker = new UpdateEligibleOptionWorker(
				this.eligibleOption, true);

		PartyData partyUpdated = new PartyData();
		partyUpdated.setName("EO 1");

		PowerMockito.when(
				ECMServiceProvider
						.updateEligibleOptionAndParty(this.eligibleOption))
				.thenReturn(partyUpdated);

		updateEligibleOptionWorker.performOperation();

		assertEquals(this.eligibleOption.getBallotName(),
				partyUpdated.getName());

	}
}
