/**
 * BatteryWatcher.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.battery;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import com.smartmatic.waffle.ecm.core.battery.IBatteryStatusQuery.BatteryState;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Listener class used to manage clients interested in know about battery
 * status.
 */
public class BatteryWatcher implements IBatteryStatusChangedListener {

	/** For logging purposes. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(BatteryWatcher.class);

	/**
	 * The singleton Instance.
	 */
	private static BatteryWatcher singleton;

	/** The battery status. */
	private BatteryStatus batteryStatus;

	/** The clients. */
	private List<IBatteryStatusChangedListener> clients = new ArrayList<IBatteryStatusChangedListener>();

	/**
	 * Instantiates a new battery watcher.
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws TimeoutException
	 *             the timeout exception
	 */
	private BatteryWatcher() throws IOException, TimeoutException {
		this.batteryStatus = new BatteryStatus();
	}

	/**
	 * Adds the listener client.
	 * 
	 * @param client
	 *            the client
	 */
	public void addListenerClient(IBatteryStatusChangedListener client) {
		if (!this.clients.contains(client)) {
			this.clients.add(client);
			this.batteryStatus.addListener(this);
		}
	}

	/**
	 * * Method used to get the singleton instance.
	 * 
	 * @return the singleton instance.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws TimeoutException
	 *             the timeout exception
	 */
	public static BatteryWatcher getInstance() throws IOException,
			TimeoutException {
		if (singleton == null) {
			singleton = new BatteryWatcher();
		}
		return singleton;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.core.battery.IBatteryStatusChangedListener
	 * #batteryStatusChanged(java.util.Map)
	 */
	@Override
	public void batteryStatusChanged(Map<String, Object> changed) {

		for (IBatteryStatusChangedListener listener : this.clients) {
			listener.batteryStatusChanged(changed);
		}

	}

	/**
	 * Gets the battery state.
	 * 
	 * @return the battery state
	 */
	public BatteryState getBatteryState() {
		return this.batteryStatus.getBatteryState();
	}

	/**
	 * Gets the power percent.
	 * 
	 * @return the power percent
	 */
	public int getPowerPercent() {
		return this.batteryStatus.getPowerPercent();
	}
}
