/**
 * IBatteryStatusQuery.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.battery;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.TimeoutException;

/**
 * Interface thats holds the battery status. This interface implement all
 * methods related to battery status.
 */
public interface IBatteryStatusQuery {

	/** The Constant KEY_BATTERY_STATE. */
	public static final String KEY_BATTERY_STATE = "battery-state";

	/** The Constant KEY_POWER_PERCENTAGE. */
	public static final String KEY_POWER_PERCENTAGE = "power-percentage";

	/**
	 * The Enum BatteryState.
	 */
	public enum BatteryState {

		/** The fully charged. */
		FULLY_CHARGED,
		/** The charging. */
		CHARGING,
		/** The discharging. */
		DISCHARGING,
		/** The disconnected. */
		DISCONNECTED;
	}

	/**
	 * Gets the battery state.
	 * 
	 * @return the battery state
	 */
	BatteryState getBatteryState();

	/**
	 * Update status.
	 * 
	 * @return the map
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws TimeoutException
	 *             the timeout exception
	 */
	Map<String, Object> updateStatus() throws IOException, TimeoutException;

	/**
	 * Gets the power percent.
	 * 
	 * @return the power percent
	 */
	int getPowerPercent();
}
