/**
 * TimerListener.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.core.battery;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.JLabel;

import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Timer used to monitoring of battery status.
 * 
 * @version 1.0 (Dec 15, 2009)
 * @author Hector Rodriguez (hector.rodriguez@smartmatic.com)
 */

public class TimerListener implements ActionListener {

	/** For log messages purpose. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(TimerListener.class);

	/** Label where the blinking message will be shown. */
	private JLabel label = new JLabel();

	/***
	 * Property to indicate whether an beep sound must be produced when the
	 * charge battery is low.
	 */
	boolean playBeep;

	/**
	 * Class constructor.
	 * 
	 * @param messageLabel
	 *            , the label where the blinking message will be shown
	 * @param playBeep
	 *            flag to indicate whether an beep sound must be produced when
	 *            the charge battery is low.
	 */
	public TimerListener(JLabel messageLabel, boolean playBeep) {

		if (messageLabel != null) {
			this.label = messageLabel;
		}

		this.playBeep = playBeep;

	}

	/***
	 * Method used to set the text will be shown to the user.
	 * 
	 * @param text
	 *            - The text to shown.
	 * @see TimerListener#label
	 */
	public void setLabelText(String text) {
		this.label.setText(text + "%");
	}

	/**
	 * Action performed.
	 * 
	 * @param e
	 *            the e
	 * @inheritDoc
	 */
	@Override
	public void actionPerformed(ActionEvent e) {

		try {
			if (this.label.getForeground() == Color.RED) {
				this.label.setForeground(Color.WHITE);
			} else {
				this.label.setForeground(Color.RED);
				if (this.playBeep) {
					new ProcessBuilder("beep").start();
				}
			}
		} catch (IOException e1) {
			logger.error(e1, "Error performing battery beep alert");
		}
	}
}