/**
 * ECMBaseException.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.lang;

/**
 * Abstract base class for all application's exceptions. All exceptions
 * belonging to all packages and any of its sub-packages of ECM application must
 * extend from this class.
 */
public class ECMBaseException extends Exception {

	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 */
	private static final long serialVersionUID = -5828117959728885091L;

	/**
	 * <p>
	 * An enumerated value regarding the cause of the exception.
	 * </p>
	 * 
	 * @see #IECMExceptionInfo
	 */
	protected IECMExceptionInfo exceptionInfo;

	/**
	 * Variable to hold the exception message.
	 */
	protected String exceptionMessage;

	/** Variable to hold the bundle key to show a user understandable message. */
	private String bundleKey;

	/**
	 * <p>
	 * Constructor for an instance of this class having only the enumerated
	 * cause of the exception.
	 * </p>
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public ECMBaseException(IECMExceptionInfo exceptionInfo) {
		super();
		this.exceptionInfo = exceptionInfo;
		this.exceptionMessage = this.exceptionInfo.getExceptionMessage();
		this.bundleKey = this.exceptionInfo.getBundleKey();
	}

	/**
	 * <p>
	 * Constructor for an instance of this class having only the enumerated
	 * cause of the exception.
	 * </p>
	 * 
	 * @param throwable
	 *            a nested exception
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * 
	 * 
	 */
	public ECMBaseException(Throwable throwable,
			IECMExceptionInfo exceptionInfo, String exceptionMessage) {
		super(throwable);
		this.exceptionInfo = exceptionInfo;
		this.bundleKey = this.exceptionInfo.getBundleKey();
		this.exceptionMessage = exceptionMessage;
	}

	/**
	 * <p>
	 * Constructor for an instance of this class having only the enumerated
	 * cause of the exception.
	 * </p>
	 * 
	 * @param throwable
	 *            a nested exception
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * 
	 * 
	 */
	public ECMBaseException(IECMExceptionInfo exceptionInfo,
			String exceptionMessage) {
		this.exceptionInfo = exceptionInfo;
		this.bundleKey = this.exceptionInfo.getBundleKey();
		this.exceptionMessage = exceptionMessage;
	}

	/**
	 * <p>
	 * Constructor for an instance of this class having both the enumerated
	 * cause of the exception and the nested <code>Throwable</code> that might
	 * have triggered this exception.
	 * </p>
	 * 
	 * @param throwable
	 *            a nested exception
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public ECMBaseException(Throwable throwable, IECMExceptionInfo exceptionInfo) {
		super(throwable);
		this.exceptionInfo = exceptionInfo;
		this.exceptionMessage = this.exceptionInfo.getExceptionMessage();
		this.bundleKey = this.exceptionInfo.getBundleKey();
	}

	/**
	 * Gets the variable to hold the bundle key to show a user understandable
	 * message.
	 * 
	 * @return bundle key to show a user understandable message
	 */
	public String getBundleKey() {
		return this.bundleKey;
	}

	/**
	 * Gets the variable to hold the exception message.
	 * 
	 * @return exception message
	 */
	@Override
	public String getMessage() {
		return this.exceptionMessage;
	}

	/**
	 * <p>
	 * Getter. Returns information regarding the cause of the exception.
	 * </p>
	 */
	public IECMExceptionInfo getInfo() {
		return this.exceptionInfo;
	}
}
