/**
 * PendriveException.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.lang;

/**
 * Exception class used to manage errors related with USB stick tasks.
 */
public class PendriveException extends ECMBaseException {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -6224193978959384743L;

	/**
	 * The Enum WritingInPenDriveInfo.
	 */
	public enum PendriveExceptionInfo implements IECMExceptionInfo {

		PENDRIVE_CHECK_FAILURE(
				"pendrive.check.failure",
				"The generation of the pendrive has finished but there were errors on the filesystem"), PENDRIVE_DAMAGED_LABEL(
				"pendrive.damaged.label",
				"An exception occurred validating the pendrive label or because of an invalid label"), PENDRIVE_IO_ERROR(
				"pendrive.io.error",
				"An exception occurred during the read/write validation in the pendrive"), UNEXPECTED_ERROR_COPYING_OS(
				"unexpected.error.copying.os.in.pendrive",
				"Unexpected exception copying the operating system to the pendrive"), ERROR_EXECUTING_TAR_COMMAND(
				"unexpected.error.copying.os.in.pendrive",
				"Error executing tar command during OS-install"), BAD_BLOCKS_DETECTED(
				"bad.block.detected",
				"Bad sectors were found on the pendrive filesystem"), UNEXPECTED_ERROR_SEARCHING_BAD_BLOCKS(
				"unexpected.error.searching.bad.blocks",
				"There was an error checking bad sectors on the flash drive"), PENDRIVE_NOT_FOUND(
				"pendrive.not.found", "Pendrive not found"), INTEGRITY_GENERATION_FAILURE(
				"integrity.generation.failure",
				"An error was found while checking the integrity of the files"), CHANGING_PERMISSIONS_FAILURE(
				"changing.permissions.failure",
				"Failed to change directory permission"), UNEXPECTED_ERROR_EXECUTING_COMMAND(
				"unexpected.error.executing.command",
				"Unexpected error has occurred executing a command");

		/**
		 * Variable to hold the exception message.
		 */
		private String exceptionMessage;

		/**
		 * Variable to hold the bundle key to show a user understandable
		 * message.
		 */
		private String bundleKey;

		/**
		 * Gets the variable to hold the exception message.
		 * 
		 * @return the variable to hold the exception message
		 * @inheritDoc
		 */
		@Override
		public String getExceptionMessage() {
			return this.exceptionMessage;
		}

		/**
		 * Gets the variable to hold the bundle key to show a user
		 * understandable message.
		 * 
		 * @return the variable to hold the bundle key to show a user
		 *         understandable message
		 * @inheritDoc
		 */
		@Override
		public String getBundleKey() {
			return this.bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 */
		private PendriveExceptionInfo(String bundleKey) {
			this.bundleKey = bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 * @param exceptionMessage
		 *            Exception message
		 */
		private PendriveExceptionInfo(String bundleKey, String exceptionMessage) {
			this.bundleKey = bundleKey;
			this.exceptionMessage = exceptionMessage;
		}
	}

	/**
	 * Instantiates a PendriveException.
	 * 
	 * @param throwable
	 *            a nested exception
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public PendriveException(Throwable throwable,
			PendriveExceptionInfo exceptionInfo) {
		super(throwable, exceptionInfo);
	}

	/**
	 * Instantiates a PendriveException.
	 * 
	 * @param throwable
	 *            a nested exception
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param exceptionMessage
	 *            Exception message of nested exception
	 */
	public PendriveException(Throwable throwable,
			PendriveExceptionInfo exceptionInfo, String exceptionMessage) {
		super(throwable, exceptionInfo, exceptionMessage);
	}

	/**
	 * Instantiates a PendriveException.
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public PendriveException(PendriveExceptionInfo exceptionInfo) {
		super(exceptionInfo);
	}

	/**
	 * Instantiates a PendriveException.
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public PendriveException(PendriveExceptionInfo exceptionInfo,
			String standardError) {
		super(exceptionInfo, exceptionInfo.getExceptionMessage()
				+ ". Standard error message:\n" + standardError);
	}

	/**
	 * Gets the exception info.
	 * 
	 * @return the exception info
	 */
	public IECMExceptionInfo getExceptionInfo() {
		return this.exceptionInfo;
	}

	/*
	 * @inheritDoc
	 */
	@Override
	public String getBundleKey() {
		return this.getClass().getSimpleName() + "."
				+ this.exceptionInfo.getBundleKey();
	}
}