/**
 * IPrinterStatusQuery.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.printer;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.TimeoutException;

/**
 * Class used to holds each possible printer status and implement method related
 * with any status change.
 */
public interface IPrinterStatusQuery {

	/**
	 * The Enum DeviceState.
	 */
	public enum DeviceState {

		/** The not found. */
		NOT_FOUND,
		/** The found. */
		FOUND,
		/** The just found. */
		JUST_FOUND,
		/** The disconnected. */
		DISCONNECTED;
	}

	/**
	 * The Enum StatusPrinter.
	 */
	public enum StatusPrinter {

		/** The ready. */
		READY,
		/** The canceling. */
		CANCELING,
		/** The busy. */
		BUSY,
		/** The out of paper. */
		OUT_OF_PAPER,
		/** The error. */
		ERROR,
		/** The out of ink. */
		OUT_OF_INK,
		/** The low ink. */
		LOW_INK,
		/** The low paper. */
		LOW_PAPER,
		/** The offline. */
		OFFLINE,
		/** The battery very low. */
		BATTERY_VERY_LOW,
		/** The door open. */
		DOOR_OPEN,
		/** The disconnected. */
		DISCONNECTED,
		/** The connected. */
		CONNECTED,
		/** The connected. */
		PAPER_DOOR_OPEN;
	}

	/**
	 * Gets the device state.
	 * 
	 * @return the device state
	 */
	DeviceState getDeviceState();

	/**
	 * Gets the printer status.
	 * 
	 * @return the printer status
	 */
	StatusPrinter getPrinterStatus();

	/**
	 * Update status.
	 * 
	 * @return the map
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws TimeoutException
	 *             the timeout exception
	 */
	Map<String, Object> updateStatus() throws IOException, TimeoutException;

	/**
	 * Gets the ink level percent.
	 * 
	 * @return the ink level percent
	 */
	int getInkLevelPercent();
}