/**
 * ComparatorAlphaNum.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.util;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.smartmatic.waffle.domain.device.VotingDeviceData;

/**
 * Comparator class used to sort Alpha Numeric elements.
 */
public class ComparatorAlphaNum implements Comparator<VotingDeviceData> {

	/**
	 * Method used to separate the items numbers and letters within a string.
	 * example: The "letters12words44444phrase" String el method split the
	 * string into a vector of length 5. "letters" "12" "words" "44444" "phrase"
	 * 
	 * @param string
	 *            the string
	 * @return the string[]
	 */
	public String[] buildVector(String string) {
		String[] numbers = string.split("\\D+");

		String[] words = string.split("\\d+");

		if (numbers == null || numbers.length == 0) {
			return words;
		}

		if (words == null || words.length == 0) {
			return numbers;
		}

		String[] startsVector;
		String[] endVector;
		if (numbers[0] != null && !numbers[0].isEmpty()
				&& string.startsWith(numbers[0])) {
			startsVector = numbers;
			endVector = Arrays.copyOfRange(words, 1, words.length);
		} else {
			startsVector = words;

			endVector = Arrays.copyOfRange(numbers, 1, numbers.length);
		}
		Vector<String> v = new Vector<String>();

		for (int indexStart = 0, indexEnd = 0; indexEnd < endVector.length
				|| indexStart < startsVector.length; indexEnd++, indexStart++) {
			if (indexStart < startsVector.length) {
				v.add(startsVector[indexStart]);
			}
			if (indexEnd < endVector.length) {
				v.add(endVector[indexEnd]);
			}
		}

		return v.toArray(new String[v.size()]);
	}

	/***
	 * {@inheritDoc}
	 */
	@Override
	public int compare(VotingDeviceData string1, VotingDeviceData string2) {
		String[] v1 = this.buildVector(string1.getName());
		String[] v2 = this.buildVector(string2.getName());

		for (int index1 = 0, index2 = 0; index1 < v1.length
				|| index2 < v2.length; index2++, index1++) {

			if (index2 < v1.length && index1 < v1.length) {
				if (this.areNumebers(v1[index1], v2[index2])) {
					int n1 = Integer.parseInt(v1[index1]);
					int n2 = Integer.parseInt(v2[index2]);
					if (n1 != n2) {
						return n1 - n2;
					}
				} else {
					if (!v1[index1].equals(v2[index2])) {
						return v1[index1].compareTo(v2[index2]);
					}
				}

			} else {
				return v1.length - v2.length;
			}
		}
		return 0;
	}

	/**
	 * Method used to check if the two strings are numbers.
	 * 
	 * @param p1
	 *            - String to check
	 * @param p2
	 *            String to check
	 * @return boolean value. True if both string are numbers or false otherwise
	 */
	private boolean areNumebers(String p1, String p2) {
		Pattern p = Pattern.compile("\\d+");
		Matcher m1, m2;
		m1 = p.matcher(p1);
		m2 = p.matcher(p2);
		return m1.find() && m2.find();
	}
}