/**
 * InetUtility.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.util;

import java.io.IOException;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.Socket;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * Utility class used to get information regarding net.
 * 
 * @version 1.0 (11/03/2011)
 * @author Hector Rodri­guez (hector.rodriguez@smartmatic.com)
 */
public class InetUtility {

	/**
	 * Checks if is server reachable.
	 * 
	 * @param host
	 *            the host
	 * @param port
	 *            the port
	 * @return true, if is server reachable
	 */
	public static boolean isServerReachable(String host, Integer port) {
		Socket socket = null;
		try {
			socket = new Socket(host, port);
			return true;
		} catch (Exception e) {
			System.err
					.println("There is an error testing conecction to server Exception :"
							+ e.getMessage());
		} finally {
			if (socket != null) {
				try {
					socket.close();
				} catch (IOException e) {
					System.err
							.println("There is an error closing the socket Exception :"
									+ e.getMessage());
				}
				socket = null;
			}
		}
		return false;
	}

	/**
	 * Method the obtains the server IP.
	 * 
	 * @return the server IP
	 */
	public static String obtainServerIp() {
		String address = "UNKNOWN";
		try {
			address = InetAddress.getLocalHost().getHostAddress();
		} catch (Exception e) {
			System.err.println("Exception:: " + e.getMessage());
		}
		return address;
	}

	/**
	 * Gets the client ip.
	 * 
	 * @return the client ip
	 */
	public static String getClientIP() {
		String address = "UNKNOWN";
		try {
			InetAddress ipAddress = InetAddress.getByName(InetAddress
					.getLocalHost().getHostName());
			address = ipAddress.getHostAddress();
		} catch (Exception e) {
			System.err.println("Exception:: " + e.getMessage());
		}
		return address;
	}

	/**
	 * Gets the mac address.
	 * 
	 * @return the mac address
	 */
	public static String getMacAddress() {
		try {
			Enumeration<NetworkInterface> nis = NetworkInterface
					.getNetworkInterfaces();
			while (nis.hasMoreElements()) {
				StringBuilder macBuilder = new StringBuilder();
				NetworkInterface ni = nis.nextElement();
				byte[] mac = ni.getHardwareAddress();
				if (!ni.isLoopback() && !ni.isPointToPoint() && !ni.isVirtual()
						&& ni.supportsMulticast() && mac != null) {
					for (int i = 0; i < mac.length; i++) {
						macBuilder.append(String.format("%02X%s", mac[i],
								(i < mac.length - 1) ? "-" : ""));
					}
					return macBuilder.toString();
				}
			}
		} catch (Exception ex) {
			System.err.println("Exception:: " + ex.getMessage());
		}
		return null;
	}

	/**
	 * Gets the mac addresses.
	 * 
	 * @return the mac addresses
	 */
	public static Map<String, String> getMacAddresses() {
		try {
			Map<String, String> map = new HashMap<String, String>();
			Enumeration<NetworkInterface> nis = NetworkInterface
					.getNetworkInterfaces();
			while (nis.hasMoreElements()) {
				StringBuilder macBuilder = new StringBuilder();

				NetworkInterface ni = nis.nextElement();
				byte[] mac = ni.getHardwareAddress();
				if (mac != null) {
					for (int i = 0; i < mac.length; i++) {
						macBuilder.append(String.format("%02X%s", mac[i],
								(i < mac.length - 1) ? "-" : ""));
					}
					map.put(ni.getDisplayName(), macBuilder.toString());
				}
			}
			return map;

		} catch (Exception ex) {
			System.err.println("Exception:: " + ex.getMessage());
		}

		return new HashMap<String, String>();
	}

}
