/**
 * SAESZipUtil.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.commons.lang.StringUtils;

/**
 * Class used to perform a zip compression operations.
 */
public class SAESZipUtil {

	/**
	 * Zip folder.
	 * 
	 * @param sourceFolder
	 *            the source folder
	 * @param outFile
	 *            the out file
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static void zipFolder(String sourceFolder, String outFile)
			throws IOException {

		List<String> fileList = new ArrayList<String>();

		fillFileList(new File(sourceFolder), sourceFolder, fileList);

		generateZipFile(sourceFolder, outFile, fileList);
	}

	/**
	 * Zip it.
	 * 
	 * @param sourceFolderAbsolutePath
	 *            the source folder absolute path
	 * @param zipFile
	 *            the zip file
	 * @param fileList
	 *            the file list
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static void generateZipFile(String sourceFolderAbsolutePath,
			String zipFile, List<String> fileList) throws IOException {

		byte[] buffer = new byte[1024];

		FileOutputStream fos = new FileOutputStream(zipFile);
		ZipOutputStream zos = new ZipOutputStream(fos);

		for (String file : fileList) {
			ZipEntry ze = new ZipEntry(file);
			zos.putNextEntry(ze);

			FileInputStream in = new FileInputStream(sourceFolderAbsolutePath
					+ File.separator + file);

			int len;
			while ((len = in.read(buffer)) > 0) {
				zos.write(buffer, 0, len);
			}

			in.close();
		}

		zos.closeEntry();
		zos.close();
	}

	/**
	 * Traverse a directory and get all files, and add the file into fileList.
	 * 
	 * @param node
	 *            file or directory
	 * @param sourceFolderAbsolutePath
	 *            the source folder absolute path
	 * @param fileList
	 *            the file list
	 */
	private static void fillFileList(File node,
			String sourceFolderAbsolutePath, List<String> fileList) {

		if (node.isFile()) {
			fileList.add(StringUtils.remove(node.getAbsoluteFile().toString(),
					sourceFolderAbsolutePath));
		}

		if (node.isDirectory()) {
			String[] subNote = node.list();
			for (String filename : subNote) {
				fillFileList(new File(node, filename),
						sourceFolderAbsolutePath, fileList);
			}
		}

	}

	/**
	 * Unzips a zip file in a output folder.
	 * 
	 * @param zipFile
	 *            zip file to unzip
	 * @param outputFolder
	 *            folder in which to unzip
	 * @param desiredFiles
	 *            List with the desired file to unzip.
	 * @throws Exception
	 *             signals that a problem occurred running the method
	 */
	public static void unZipFile(File zipFile, File outputFolder,
			List<String> desiredFiles) throws Exception {

		boolean extractFile = true;

		byte[] buffer = new byte[1024];

		ZipInputStream zipInputStream = new ZipInputStream(new FileInputStream(
				zipFile));

		ZipEntry zipEntry = zipInputStream.getNextEntry();

		while (zipEntry != null) {

			String fileName = zipEntry.getName();

			if (desiredFiles != null && !desiredFiles.isEmpty()) {
				for (String desiredFile : desiredFiles) {
					extractFile = fileName.contains(desiredFile);
					if (extractFile) {
						break;
					}
				}
			}

			if (extractFile) {
				File newFile = new File(outputFolder, fileName);

				if (!newFile.exists()) {
					new File(newFile.getParent()).mkdirs();

					if (zipEntry.isDirectory()) {
						newFile.mkdir();
					} else {
						FileOutputStream fileOutputStream = new FileOutputStream(
								newFile);

						int len;
						while ((len = zipInputStream.read(buffer)) > 0) {
							fileOutputStream.write(buffer, 0, len);
						}

						fileOutputStream.close();
					}
				}
			}

			zipEntry = zipInputStream.getNextEntry();
		}

		zipInputStream.closeEntry();
		zipInputStream.close();

	}

}
