package com.smartmatic.waffle.ecm.core.pendrive.util;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.core.util.SAESCommandUtil;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ PendriveUtil.class, SAESCommandUtil.class })
public class PendriveUtilTest {

	@Test
	public void testCheckPendriveLabelValidLabel() throws Exception {

		// arrange
		String pendrivePath = "/media/smartmatic/a0d3a871-52ce-4dd9-8465-07a82e6ee628";

		// act
		boolean validLabel = PendriveUtil.checkPendriveLabel(pendrivePath);

		// assert
		assertTrue(validLabel);
	}

	@Test(expected = PendriveException.class)
	public void testCheckPendriveLabelInvalidLabel() throws PendriveException {

		// arrange
		String pendrivePath = "/media/smartmatic/MASTER_USB";

		try {
			// act
			PendriveUtil.checkPendriveLabel(pendrivePath);
		} catch (PendriveException e) {
			// assert
			assertTrue(e.getExceptionInfo().equals(
					PendriveExceptionInfo.PENDRIVE_DAMAGED_LABEL));
			throw e;
		}
	}

	@Test
	public void testMergeSAESLocalSAESLocalUnsigned() throws Exception {

		// arrange
		String pathToMedia = "/media/smartmatic/MASTER_USB";
		PowerMockito.mockStatic(PendriveUtil.class);
		PowerMockito.doNothing().when(PendriveUtil.class,
				"executePendriveUtilScript", "merge", pathToMedia, null, true);

		// act
		PowerMockito.doCallRealMethod().when(PendriveUtil.class,
				"mergeSAESLocalSAESLocalUnsigned", pathToMedia, true);
		PendriveUtil.mergeSAESLocalSAESLocalUnsigned(pathToMedia, true);

		// assert
		PowerMockito.verifyPrivate(PendriveUtil.class).invoke(
				"executePendriveUtilScript", "syncAndMerge", pathToMedia, null,
				true);
	}

	@Test
	public void testIsAnExpectedOutput() {
		List<String> errorPermitedList = new ArrayList<String>();
		errorPermitedList.add("time stamp");
		errorPermitedList.add("blocks");
		errorPermitedList.add("chmod: changing permissions");
		errorPermitedList.add("cannot remove");
		errorPermitedList.add("mke2fs");
		errorPermitedList.add("cannot create regular file");
		errorPermitedList.add("/boot/syslinux/ldlinux.sys");

		String cadidateError1 = "invalid time stamp";
		String cadidateError2 = "cp: cannot create regular file";
		String cadidateError3 = "invalid file format";

		assertTrue(PendriveUtil.isAnExpectedOutput(new String[] {
				cadidateError1, cadidateError2 }, errorPermitedList));

		assertFalse(PendriveUtil.isAnExpectedOutput(new String[] {
				cadidateError1, cadidateError2, cadidateError3 },
				errorPermitedList));

	}

	@Test
	public void testSynchronizeAndUnmountPendrive() throws Exception {

		List<String> authorizedErrorsList = new ArrayList<String>();
		authorizedErrorsList.add("lsof: WARNING: can't stat()");
		authorizedErrorsList.add("Output information may be incomplete.");

		// arrange
		String partitionName = "/dev/sdb";
		PowerMockito.mockStatic(SAESCommandUtil.class);
		PowerMockito.mockStatic(PendriveUtil.class);

		PowerMockito.doNothing().when(PendriveUtil.class,
				"executePendriveUtilScript", "sync", StringUtils.EMPTY, null,
				true);

		// act
		PowerMockito.doCallRealMethod().when(PendriveUtil.class, "sync");
		PowerMockito.doCallRealMethod().when(PendriveUtil.class,
				"unmountPendrive", partitionName, true);
		PendriveUtil.sync();
		PendriveUtil.unmountPendrive(partitionName, true);

		// assert
		PowerMockito.verifyStatic();
		PendriveUtil.sync();
		PowerMockito.verifyPrivate(PendriveUtil.class).invoke(
				"executePendriveUtilScript", "umount", partitionName,
				authorizedErrorsList, true);
	}

}
