/**
 * BulkResultInfo.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.data.transfer.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.smartmatic.waffle.lang.BaseException;

/**
 * Object used to store and notify the result of a bulk loading process.
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class BulkResultDTO implements Serializable {

	/**
	 * Holds the serialization id.
	 */

	private static final long serialVersionUID = 1693103938797870272L;

	/**
	 * Holds the count of correct operations performed by the bulk data loading
	 * module.
	 */

	private int correctCount;

	/**
	 * Holds the count of incorrect operations performed by the bulk data
	 * loading module.
	 */

	private int incorrectCount;

	/** The error report list. */
	private List<BulkErrorReportDTO> errorReportList = new ArrayList<BulkErrorReportDTO>();

	/**
	 * Adds the error report.
	 * 
	 * @param text
	 *            the text
	 * @param exception
	 *            the exception
	 */
	public void addErrorReport(String text, BaseException exception) {
		BulkErrorReportDTO report = new BulkErrorReportDTO(text, exception);
		this.errorReportList.add(report);
	}

	/**
	 * Gets the holds the count of correct operations performed by the bulk data
	 * loading module.
	 * 
	 * @return the holds the count of correct operations performed by the bulk
	 *         data loading module
	 */
	public int getCorrectCount() {
		return this.correctCount;
	}

	/**
	 * Sets the holds the count of correct operations performed by the bulk data
	 * loading module.
	 * 
	 * @param correctCount
	 *            the new holds the count of correct operations performed by the
	 *            bulk data loading module
	 */
	public void setCorrectCount(int correctCount) {
		this.correctCount = correctCount;
	}

	/**
	 * Gets the holds the count of incorrect operations performed by the bulk
	 * data loading module.
	 * 
	 * @return the holds the count of incorrect operations performed by the bulk
	 *         data loading module
	 */
	public int getIncorrectCount() {
		return this.incorrectCount;
	}

	/**
	 * Sets the holds the count of incorrect operations performed by the bulk
	 * data loading module.
	 * 
	 * @param incorrectCount
	 *            the new holds the count of incorrect operations performed by
	 *            the bulk data loading module
	 */
	public void setIncorrectCount(int incorrectCount) {
		this.incorrectCount = incorrectCount;
	}

	/**
	 * Gets the error report list.
	 * 
	 * @return the error report list
	 */
	public List<BulkErrorReportDTO> getErrorReportList() {
		return this.errorReportList;
	}

	/**
	 * Sets the error report list.
	 * 
	 * @param errorReportList
	 *            the new error report list
	 */
	public void setErrorReportList(List<BulkErrorReportDTO> errorReportList) {
		this.errorReportList = errorReportList;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public String toString() {
		int total = this.correctCount + this.incorrectCount;
		return "Total processed entries: " + total + ". " + "Correct count: "
				+ this.correctCount + ". " + "Incorrect count: "
				+ this.incorrectCount + ". " + "Errors found: "
				+ this.errorReportList.size() + ".";

	}

}
