/**
 * QueryResultDTO.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.data.transfer.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Class that holds the result of data integrity query.
 */
public class QueryResultDTO implements Serializable {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -1835467838945627868L;

    /** The columns names. */
    private List<String> columnsNames;

    /** The result data rows. */
    private List<ArrayList<String>> rows;

    /**
     * Instantiates a new query result DTO.
     */
    public QueryResultDTO() {
        super();
    }

    /**
     * Returns a string array that contains this query result's columns names.
     * 
     * @return The String array with the columns names
     */
    public String[] getColumnsNames() {
        return columnsNames.toArray(new String[columnsNames.size()]);
    }

    /**
     * Sets the columns names.
     * 
     * @param columnNames
     *            the new columns names
     */
    public void setColumnsNames(List<String> columnNames) {
        this.columnsNames = columnNames;
    }

    /**
     * Returns an object array that contains this query result data rows.
     * 
     * @return The object array with the rows data.
     */
    public Object[][] getRows() {

        if (rows != null) {
            Object[][] rowsObject = new Object[rows.size()][getColumnsSize()];
            int rowIndex = 0;
            for (ArrayList<String> row : rows) {
                int columnIndex = 0;
                for (String column : row) {
                    rowsObject[rowIndex][columnIndex] = column;
                    columnIndex++;
                }
                rowIndex++;
            }
            return rowsObject;
        }
        return null;
    }

    /**
     * Set this query result object rows data.
     * 
     * @param rows
     *            The rows object.
     */
    public void setRows(List<ArrayList<String>> rows) {
        this.rows = rows;
    }

    /**
     * Return the maximum columns size.
     * 
     * @return The maximum columns size.
     */
    private int getColumnsSize() {
        int maxSize = 0;
        for (ArrayList<String> row : rows) {
            if (row.size() > maxSize) {
                maxSize = row.size();
            }
        }
        return maxSize;
    }
}
