/**
 * ECMInitServiceImpl.java
 * 
 * $Revision: 1.11 $ ($Date: 2011-05-18 16:45:22 $)
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.service.impl;

import java.util.Properties;

import org.jboss.ejb3.annotation.Depends;
import org.jboss.ejb3.annotation.Management;
import org.jboss.ejb3.annotation.Service;

import com.smartmatic.waffle.client.ClientJNDIWrapper;
import com.smartmatic.waffle.client.IContextProvider;
import com.smartmatic.waffle.client.JNDICommunicationException;
import com.smartmatic.waffle.client.JNDILookupModeEnum;
import com.smartmatic.waffle.ecm.service.spec.IECMInitService;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.properties.ApplicationProperties;
import com.smartmatic.waffle.properties.SmartProperties;

/**
 * Class that implements the initial Management to the Listener Service
 * 
 * @version 1.0 (13/10/2008)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

@Service(name = IECMInitService.SERVICE_NAME)
@Management(IECMInitService.class)
@Depends({ "jboss.j2ee:ear=com.smartmatic.waffle.ecm.ear,jar=com.smartmatic.waffle.persistence.jar,name=com.smartmatic.waffle.persistence.service.spec.IPersistenceService,service=EJB3" })
public class ECMInitServiceImpl implements IECMInitService {

	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 */
	private static final long serialVersionUID = -7914373086355111399L;

	private SMTTLogger logger = SMTTLogger.getLogger(ECMInitServiceImpl.class);

	/**
	 * <p>
	 * Method used to initialize the application. This method is responsible of
	 * initialize all the environment
	 * </p>
	 * 
	 * @version 1.0 (31/03/2006)
	 */
	@Override
	public void start() {
		try {
			// Initialize the server resources
			SmartProperties.init();
			String basePath = System.getProperty("jboss.server.home.dir");
			for (String propertyName : ApplicationProperties.getPropertyNames()) {
				this.addBasePath(propertyName, "bulk.base.xsd.path", basePath);
				this.addBasePath(propertyName, "bulk.xsd.path", basePath);
			}

			Properties properties = new Properties();
			try {
				ClientJNDIWrapper.init(properties, JNDILookupModeEnum.LOCAL,
						new IContextProvider() {
							@Override
							public String getContext() {
								return (String) ApplicationProperties
										.getValue("jndi.ear.name");
							}
						});
			} catch (JNDICommunicationException e) {
				this.logger.error(e, e.getMessage());
			}

		} catch (Exception e) {
			this.logger.error(e, e.getMessage());
		}
	}

	/**
	 * Adds the base path to the value's beginning if wasn't added before
	 */
	private void addBasePath(String thePropertyName, String theKeyToSearch,
			String theBasePath) {

		if (thePropertyName.startsWith(theKeyToSearch)) {
			String theValue = (String) ApplicationProperties
					.getValue(thePropertyName);

			if (theValue.startsWith(theBasePath) == false) {

				ApplicationProperties.setValue(thePropertyName, theBasePath
						+ theValue);
			}

		}
	}
}
