/**
 * ECMMasterUSBGenerationServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.ballot.BallotCardData;
import com.smartmatic.waffle.domain.ballot.BallotData;
import com.smartmatic.waffle.domain.ballot.BallotOptionCategoryEnum;
import com.smartmatic.waffle.domain.ballot.BallotOptionData;
import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.CategoryData;
import com.smartmatic.waffle.domain.demographics.CategoryData.Level;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.PathData;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.PollingStationData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.demographics.RegionHierarchy;
import com.smartmatic.waffle.domain.demographics.TotalizationNisCode;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.device.CustomAttribute;
import com.smartmatic.waffle.domain.device.CustomAttributeDefinition;
import com.smartmatic.waffle.domain.device.CustomAttributeDefinitionValue;
import com.smartmatic.waffle.domain.device.LogicalVoterGroupsData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.device.OperationModeData;
import com.smartmatic.waffle.domain.device.RoleData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.device.VotingDeviceData.KeysData;
import com.smartmatic.waffle.domain.device.VotingDevicesData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.multiLanguage.StringResourceData;
import com.smartmatic.waffle.ecm.core.util.TimestampUtil;
import com.smartmatic.waffle.ecm.data.transfer.comparators.ContestClassPrecedenceComparator;
import com.smartmatic.waffle.ecm.data.transfer.comparators.ContestGroupByOriginalCustomCodeComparator;
import com.smartmatic.waffle.ecm.data.transfer.comparators.ContestPrecedenceComparator;
import com.smartmatic.waffle.ecm.data.transfer.comparators.CustomAttributeDefinitionValueComparator;
import com.smartmatic.waffle.ecm.data.transfer.comparators.CustomAttributePrecedenceComparator;
import com.smartmatic.waffle.ecm.data.transfer.comparators.LanguagePrecedenceComparator;
import com.smartmatic.waffle.ecm.data.transfer.comparators.LogicalVotingGroupComparator;
import com.smartmatic.waffle.ecm.data.transfer.comparators.TallyPositionComparator;
import com.smartmatic.waffle.ecm.data.transfer.objects.ActiveContestForElectionDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionXmlComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceDTO;
import com.smartmatic.waffle.ecm.service.lang.ElectoralDemographyException;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException.MasterUSBGenerationExceptionInfo;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralDemographyServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMMasterUSBGenerationService;
import com.smartmatic.waffle.ecm.service.spec.IECMMasterUSBGenerationServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMNominationServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMProcessedContestsSingletonLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesServiceLocal;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;
import com.smartmatic.waffle.properties.ApplicationProperties;

/**
 * Session beans that handles all operations related with master USB generation.
 * This class implements all the server services defined in
 * <code>IECMMasterUSBGenerationService</code>
 */
@Stateless(name = IECMMasterUSBGenerationService.SERVICE_NAME)
@Remote(IECMMasterUSBGenerationService.class)
@Local(IECMMasterUSBGenerationServiceLocal.class)
public class ECMMasterUSBGenerationServiceImpl implements
		IECMMasterUSBGenerationService, IECMMasterUSBGenerationServiceLocal {

	/** The logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMMasterUSBGenerationServiceImpl.class);

	/** Instance of IPersistenceServiceLocal. */
	@EJB
	private IPersistenceServiceLocal persistenceService;

	/** Instance of IECMElectoralDemographyServiceLocal. */
	@EJB
	private IECMElectoralDemographyServiceLocal electoralDemographyService;

	/** Instance of IECMDemographyServiceLocal. */
	@EJB
	private IECMDemographyServiceLocal demographyService;

	/** Instance of IECMUtilitiesServiceLocal. */
	@EJB
	private IECMUtilitiesServiceLocal utilitiesService;

	@EJB
	private IECMNominationServiceLocal nominationService;

	@EJB
	private IECMProcessedContestsSingletonLocal processedContestsService;

	/**
	 * Sets the instance of IPersistenceServiceLocal.
	 * 
	 * @param persistenceService
	 *            the new instance of IPersistenceServiceLocal
	 */
	public void setPersistenceService(
			IPersistenceServiceLocal persistenceService) {
		this.persistenceService = persistenceService;
	}

	/**
	 * Sets the instance of IECMElectoralDemographyServiceLocal.
	 * 
	 * @param electoralDemographyService
	 *            the new instance of IECMElectoralDemographyServiceLocal
	 */
	public void setElectoralDemographyService(
			IECMElectoralDemographyServiceLocal electoralDemographyService) {
		this.electoralDemographyService = electoralDemographyService;
	}

	/**
	 * Sets the instance of IECMDemographyServiceLocal.
	 * 
	 * @param demographyService
	 *            the new instance of IECMDemographyServiceLocal
	 */
	public void setDemographyService(
			IECMDemographyServiceLocal demographyService) {
		this.demographyService = demographyService;
	}

	/**
	 * Sets the instance of IECMUtilitiesServiceLocal.
	 * 
	 * @param utilitiesService
	 *            the new instance of IECMUtilitiesServiceLocal
	 */
	public void setUtilitiesService(IECMUtilitiesServiceLocal utilitiesService) {
		this.utilitiesService = utilitiesService;
	}

	/**
	 * Sets the instance of IECMNominationServiceLocal
	 * 
	 * @param nominationService
	 *            the new instance of IECMNominationServiceLocal
	 */
	public void setNominationService(
			IECMNominationServiceLocal nominationService) {
		this.nominationService = nominationService;
	}

	/**
	 * Sets the instance of IECMProcessedContestsSingletonLocal
	 * 
	 * @param processedContestsService
	 *            the new instance of IECMProcessedContestsSingletonLocal
	 */
	public void setProcessedContestsService(
			IECMProcessedContestsSingletonLocal processedContestsService) {
		this.processedContestsService = processedContestsService;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
	public ActiveContestForElectionDTO getActiveContestForElection(
			List<RegionData> lastLevelRegions, LanguageData appLanguage)
			throws MasterUSBGenerationException {

		try {

			ActiveContestForElectionDTO activeContestForElection = new ActiveContestForElectionDTO();

			List<Long> lastLevelRegionCodes = extractRegionCodes(lastLevelRegions);

			// Get the list of last level regions with their parents
			List<RegionData> regionDataList = this.demographyService
					.getRegionsWithParentsByRegionCodes(lastLevelRegionCodes);

			Map<Long, Set<LanguageData>> regionLanguageMap = new HashMap<Long, Set<LanguageData>>();

			Set<ContestData> contestDataSet = new HashSet<ContestData>();

			Map<Long, List<ContestGroupData>> regionContestGroup = new HashMap<Long, List<ContestGroupData>>();

			Map<String, Map<String, String>> resourcesLanguages = new HashMap<String, Map<String, String>>();

			// searching all languages loaded in database
			Set<LanguageData> languages = getLanguages();

			// Get the max contest group code and increment it to generate the
			// next value
			long contestGroupCodeReference = this.electoralDemographyService
					.searchMaxContestGroupCode() + 1;

			for (RegionData regionData : regionDataList) {

				Set<DistrictData> regionDistrictSet = new HashSet<DistrictData>();
				Set<LanguageData> regionLanguageSet = new HashSet<LanguageData>();

				Set<RegionData> regionParentSet = regionData.getAllParents();

				for (RegionData regionParentData : regionParentSet) {
					regionDistrictSet.addAll(regionParentData.getDistricts());
					regionLanguageSet.addAll(regionParentData
							.getElectionsLanguage());
				}

				// At this point regionDistrictSet has the districts of all
				// region hierarchies and regionLanguageSet contains the
				// languages used in these regions.

				// Searching the contest by district. This list contains the
				// contests allowed to vote in the last level region that ir
				// processing
				List<ContestData> contestRegionList = this.electoralDemographyService
						.getContestByDistrict(regionDistrictSet);

				// This loop translate the contest names
				for (ContestData contestData : contestRegionList) {
					contestData = this.utilitiesService.translateContestName(
							contestData, appLanguage);

					this.utilitiesService.processMultilingualObject(languages,
							resourcesLanguages, contestData);
				}

				contestDataSet.addAll(contestRegionList);

				// This list contains the possible region-level contest groups
				// that a voter can select
				List<ContestGroupData> contestContestGroup = this.electoralDemographyService
						.makeRegionSpecificContestGroups(regionData,
								contestRegionList, contestGroupCodeReference);

				contestGroupCodeReference += contestContestGroup.size();

				// This loop translates the contest group names
				for (ContestGroupData contestGroupData : contestContestGroup) {
					this.utilitiesService.processMultilingualObject(languages,
							resourcesLanguages, contestGroupData);
				}

				regionLanguageMap.put(regionData.getCode(), regionLanguageSet);

				regionContestGroup.put(regionData.getCode(),
						contestContestGroup);

			}

			// At this point all last level region were processed and the
			// result of this processing is loaded in a dTO to be sent to
			// client

			List<ContestData> contestList = new ArrayList<ContestData>(
					contestDataSet);

			Collections.sort(contestList, new ContestPrecedenceComparator());

			activeContestForElection.setContestList(contestList);

			activeContestForElection.setRegionLanguageMap(regionLanguageMap);

			activeContestForElection.setRegionContestGroup(regionContestGroup);

			activeContestForElection.setResourcesLanguages(resourcesLanguages);

			activeContestForElection.setElectionLanguages(languages);

			return activeContestForElection;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new MasterUSBGenerationException(
					e,
					MasterUSBGenerationExceptionInfo.PROBLEM_RETRIEVING_ACTIVE_CONTESTS_FOR_ELECTION,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	@TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
	public ElectionXmlComponentsDTO generateElectionForContest(
			List<ContestData> contestList, int objectCodePrefix,
			ElectionData election,
			Map<String, Map<String, String>> resourcesLanguages,
			List<ContestGroupData> contestGroups)
			throws MasterUSBGenerationException {

		try {

			ElectionXmlComponentsDTO electionDto = new ElectionXmlComponentsDTO();

			election.setXmlns("http://www.smartmatic.com/Saes/Voting/Election");

			election.setXmlnsXs("http://www.w3.org/2001/XMLSchema");

			HashSet<PartyData> partySet = new HashSet<PartyData>();

			Set<LanguageData> electionLanguages = getLanguages();

			List<ContestClassData> contestClassList = buildContestClassData(
					contestList, electionLanguages, resourcesLanguages,
					partySet);

			electionDto
					.setContestEligibleOptionResources(getContestEligibleOptionsResources(new HashSet<ContestData>(
							contestList)));

			Collections.sort(contestClassList,
					new ContestClassPrecedenceComparator());

			Set<BallotCardData> ballotCardSet = buildBallotCardSet(
					objectCodePrefix, contestList);

			Set<BallotData> ballotSet = buildBallotSet(objectCodePrefix,
					ballotCardSet);

			election.setBallots(ballotSet);

			election.setContestClass(contestClassList);

			election.setVotingTimeOut(ApplicationProperties
					.getIntValue("election.time.out"));

			election.setPartyList(partySet);

			List<LanguageData> languagesList = new ArrayList<LanguageData>(
					electionLanguages);

			Collections.sort(languagesList, new LanguagePrecedenceComparator());

			election.setLanguages(languagesList);

			this.utilitiesService.processMultilingualObject(electionLanguages,
					resourcesLanguages, election);

			List<ContestGroupData> contestGroupList;
			if (contestGroups == null) {
				contestGroupList = this.electoralDemographyService
						.makeContestsSpecificContestGroups(contestList);
			} else {
				contestGroupList = contestGroups;
			}

			electionDto.setContestGroup(contestGroupList);

			List<ContestGroupData> contestGroupListToElection = getTranslatedContestGroupList(
					contestGroupList, electionLanguages, resourcesLanguages);

			Set<ContestGroupData> contestGroupSet = new HashSet<ContestGroupData>(
					contestGroupListToElection);

			election.setContestGroups(contestGroupSet);

			electionDto.setElection(election);

			electionDto.setResourcesLanguages(resourcesLanguages);

			electionDto.setElectionLanguages(electionLanguages);

			return electionDto;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new MasterUSBGenerationException(
					e,
					MasterUSBGenerationExceptionInfo.PROBLEM_GENERATING_ELECTION_FOR_CONTEST,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	@TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
	public VotingDeviceDTO generateVotingDeviceForRegion(
			VotingDeviceData votingDeviceData, RegionData region,
			List<ContestGroupData> contestGroupList,
			Map<String, Map<String, String>> resourcesLanguages,
			boolean generateDummyVotingDevice, ElectionData election,
			boolean audioVotingEnabled) throws MasterUSBGenerationException {

		try {
			logger.info("Processing " + votingDeviceData.getName());

			long votingDeviceTime = System.currentTimeMillis();

			VotingDeviceDTO votingDeviceDto = new VotingDeviceDTO();

			VotingDevicesData votingDevicesData = new VotingDevicesData();

			votingDevicesData
					.setXmlns("http://www.smartmatic.com/Saes/Voting/VotingDevice");

			votingDevicesData.setXmlnsXs("http://www.w3.org/2001/XMLSchema");

			KeysData keys = getKeysFromVotingDevice(votingDeviceData);

			votingDeviceData.setKeys(keys);

			Set<LanguageData> electionLanguage = votingDeviceData
					.getLanguages();

			setVotingBoothData(votingDeviceData, contestGroupList, region,
					generateDummyVotingDevice, electionLanguage,
					resourcesLanguages, audioVotingEnabled);

			Set<OperationModeData> operationModes = new HashSet<OperationModeData>();

			OperationModeData operationMode = getOperationMode(
					votingDeviceData, election.getElectionOpeningDate(),
					election.getElectionClosingDate(), keys);

			operationModes.add(operationMode);

			votingDeviceData.setOperationModes(operationModes);

			Set<VotingBoothData> votingBooths = new HashSet<VotingBoothData>();

			votingBooths.add(votingDeviceData.getVotingBooth());

			votingDeviceData.setVotingBooths(votingBooths);

			SortedSet<LogicalVotingGroupData> logicalVotingGroupsData = new TreeSet<LogicalVotingGroupData>(
					new LogicalVotingGroupComparator());

			Map<String, List<String>> customAttributeDefinitions = new HashMap<String, List<String>>();

			List<ContestGroupData> sortedContestGroupList = new ArrayList<ContestGroupData>(
					contestGroupList);

			Collections.sort(sortedContestGroupList,
					new ContestGroupByOriginalCustomCodeComparator());

			for (ContestGroupData contestGroup : sortedContestGroupList) {

				Set<CustomAttribute> attributeList = new HashSet<CustomAttribute>();

				LogicalVotingGroupData logicalVotingGroupData = null;

				List<LogicalVotingGroupData> logicalVotingGroupList = new ArrayList<LogicalVotingGroupData>();

				if (generateDummyVotingDevice) {

					logicalVotingGroupList = this.electoralDemographyService
							.getLogicalVotingGroupByContestGroup(
									contestGroup.getCustomCode(),
									contestGroup.getContestGroupName());
				} else {

					logicalVotingGroupData = this.electoralDemographyService
							.getLogicalVotingGroup(contestGroup.getCustomCode());
					logicalVotingGroupList.add(logicalVotingGroupData);
				}

				for (LogicalVotingGroupData indexLogicalVotingGroupData : logicalVotingGroupList) {
					if (indexLogicalVotingGroupData != null) {
						indexLogicalVotingGroupData.setRegionDataCode(region
								.getCode());
						indexLogicalVotingGroupData
								.setVotingBoothCode(votingDeviceData
										.getVotingBooth().getCode());
						indexLogicalVotingGroupData
								.setContestGroupCode(contestGroup.getCode());

						processContestAttributes(resourcesLanguages,
								electionLanguage, customAttributeDefinitions,
								attributeList, contestGroup.getContests());

						processContestGroupAttributes(resourcesLanguages,
								electionLanguage, customAttributeDefinitions,
								attributeList, contestGroup);

						indexLogicalVotingGroupData
								.setCustomAttribute(new ArrayList<CustomAttribute>(
										attributeList));
						logicalVotingGroupsData
								.add(indexLogicalVotingGroupData);

					}
				}

			}

			List<CustomAttributeDefinition> customAttributeDefinitionList = getCustomAttributeDefinitionList(customAttributeDefinitions);

			LogicalVoterGroupsData logicalVoterGroupsData = new LogicalVoterGroupsData();

			logicalVoterGroupsData
					.setCustomAttributeDefinition(customAttributeDefinitionList);
			logicalVoterGroupsData
					.setLogicalVotingGroupData(logicalVotingGroupsData);

			this.utilitiesService.processMultilingualObject(electionLanguage,
					resourcesLanguages, election);

			votingDeviceData.setLogicalVoterGroupsData(logicalVoterGroupsData);

			votingDeviceData.setLanguages(electionLanguage);

			votingDevicesData.setVotingDeviceData(votingDeviceData);

			votingDeviceDto.setVotingDevicesData(votingDevicesData);

			votingDeviceDto.setResourcesLanguages(resourcesLanguages);

			logger.info("Creation time of Voting Device "
					+ TimestampUtil.prettyTimeDifference(votingDeviceTime,
							System.currentTimeMillis()));

			return votingDeviceDto;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new MasterUSBGenerationException(
					e,
					MasterUSBGenerationExceptionInfo.PROBLEM_GENERATING_VOTING_DEVICE_FOR_REGION,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public Set<LanguageData> getLanguages() throws MasterUSBGenerationException {

		SearchExpression search = new SearchExpression(LanguageData.class);
		try {

			Set<LanguageData> languages = new HashSet<LanguageData>(
					this.persistenceService.searchObjects(search, null));

			return languages;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new MasterUSBGenerationException(e,
					MasterUSBGenerationExceptionInfo.PROBLEM_GETTING_LANGUAGES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Method used to iterate the contest list and build the contest class data
	 * required to generate the election object.
	 * 
	 * @param contestList
	 *            the contest list.
	 * @param electionLanguages
	 *            a list of the application languages
	 * @param resourcesLanguages
	 *            the resources languages used by this generation.
	 * @param partySet
	 *            party set reference
	 * @return a list of contest class data objects.
	 * @throws UtilitiesException
	 *             A signal that an error occurred processing languages.
	 * @throws NominationException
	 */
	private List<ContestClassData> buildContestClassData(
			List<ContestData> contestList, Set<LanguageData> electionLanguages,
			Map<String, Map<String, String>> resourcesLanguages,
			HashSet<PartyData> partySet) throws UtilitiesException,
			NominationException {

		List<ContestClassData> contestClassList = new ArrayList<ContestClassData>();

		List<EligibleOptionData> standInList = null;

		List<EligibleOptionData> chiefList = null;

		int processedContests = 0;
		for (ContestData contestData : contestList) {

			logger.info(" Processing contest: "
					+ contestData.getContestNameTranslate());

			setEligibleOptions(contestData);

			List<EligibleOptionData> eligibleOptionDataList = contestData
					.getEligibleOptions();

			standInList = new ArrayList<EligibleOptionData>();
			chiefList = new ArrayList<EligibleOptionData>();

			for (EligibleOptionData eligibleOptionData : eligibleOptionDataList) {

				if (contestData.isQuestion()) {
					this.utilitiesService.processMultilingualObject(
							electionLanguages, resourcesLanguages,
							eligibleOptionData);
				}

				Set<BallotOptionData> ballotOptionDataSet = eligibleOptionData
						.getBallotOptions();

				processBallotOptionSet(partySet, standInList, chiefList,
						eligibleOptionData, ballotOptionDataSet);
			}

			Collections.sort(standInList, new TallyPositionComparator());

			Collections.sort(chiefList, new TallyPositionComparator());

			eligibleOptionDataList = new ArrayList<EligibleOptionData>();

			eligibleOptionDataList.addAll(chiefList);

			eligibleOptionDataList.addAll(standInList);

			contestData.setEligibleOptions(eligibleOptionDataList);

			contestData.setAbstractKey(String.valueOf(contestData.getCode()));

			ContestClassData contestClass = contestData.getContestClass();

			contestClass.setDescription(contestClass.getContestClassName());

			contestClass.setStandCountChief(1);

			contestClass.setPeriod(1);

			contestClass.setUnit(ContestClassData.UnitType.YEAR);

			contestClass
					.setElectionMode(ContestClassData.ElectionModeType.OPENLIST);

			this.utilitiesService.processMultilingualObject(electionLanguages,
					resourcesLanguages, contestClass);

			if (!contestClassList.contains(contestClass)) {
				contestClassList.add(contestClass);
			}

			this.utilitiesService.processMultilingualObject(electionLanguages,
					resourcesLanguages, contestData);

			this.utilitiesService.processMultilingualObject(electionLanguages,
					resourcesLanguages, contestData.getDistrict());

			contestData.setHelp(contestData.getCollege());

			this.processedContestsService
					.setProcessedContests(++processedContests);
		}
		return contestClassList;
	}

	private void setEligibleOptions(ContestData contestData)
			throws NominationException {
		List<EligibleOptionData> eligibleOptions = this.nominationService
				.getEligibleOptionsByContest(contestData);

		List<EligibleOptionData> eligibleOptionsWithEligibleOptionRole = this.nominationService
				.getEligibleOptionRoleByEligibleOption(eligibleOptions);

		List<EligibleOptionData> eligibleOptionsWithBallotOptions = this.nominationService
				.getBallotOptionByEligibleOptions(eligibleOptionsWithEligibleOptionRole);

		contestData.setEligibleOptions(eligibleOptionsWithBallotOptions);
	}

	/**
	 * Returns a map which contains the eligible options candidates names as
	 * <code>StringResourceData</code> objects, associated with each of the
	 * contests given as parameter.
	 * 
	 * @param contests
	 *            the contest set to fetch its eligible options data
	 * @return the map with the information of each of its eligible options as
	 *         StringResourceData
	 * @throws NominationException
	 *             if an error occurs during the service request to fetch the
	 *             eligible options.
	 */
	private Map<Long, Set<StringResourceData>> getContestEligibleOptionsResources(
			Set<ContestData> contests) throws NominationException {

		Map<Long, Set<StringResourceData>> contestEligibleOptionOResources = new HashMap<Long, Set<StringResourceData>>();

		for (ContestData contest : contests) {

			Set<StringResourceData> resources = new HashSet<StringResourceData>();

			List<EligibleOptionData> eligibleOptions = this.nominationService
					.getEligibleOptionsByContest(contest);

			List<EligibleOptionData> eligibleOptionsWithRole = this.nominationService
					.getEligibleOptionRoleByEligibleOption(eligibleOptions);

			for (EligibleOptionData eligibleOption : eligibleOptionsWithRole) {
				StringResourceData resource = new StringResourceData();
				resource.setKey(eligibleOption.getBallotName());
				String value = (!eligibleOption.getParentCode().isEmpty()) ? eligibleOption
						.getEligibleOptionRoleData().first().getCandidate()
						.getName()
						: eligibleOption.getBallotName();
				resource.setValue(value);
				resources.add(resource);
			}
			contestEligibleOptionOResources.put(contest.getCode(), resources);
		}

		return contestEligibleOptionOResources;
	}

	@Override
	public int getProcessedContests() {
		return this.processedContestsService.getProcessedContests();
	}

	@Override
	public void resetProcessedContests() {
		this.processedContestsService.setProcessedContests(0);
	}

	/**
	 * Iterates through the set of ballot options.
	 * 
	 * @param partySet
	 *            party set object reference
	 * @param standInList
	 *            list to fill with stand in eligible options
	 * @param chiefList
	 *            list to fill with chief eligible options
	 * @param eligibleOptionData
	 *            eligible option object reference
	 * @param ballotOptionDataSet
	 *            ballot option set to process.
	 */
	private void processBallotOptionSet(HashSet<PartyData> partySet,
			List<EligibleOptionData> standInList,
			List<EligibleOptionData> chiefList,
			EligibleOptionData eligibleOptionData,
			Set<BallotOptionData> ballotOptionDataSet) {

		for (BallotOptionData ballotOptionData : ballotOptionDataSet) {

			ballotOptionData.getEligibleOption().setCandidateId(
					eligibleOptionData.getCandidateId());

			ballotOptionData.setImageFileName(ballotOptionData.getParty()
					.getImageFileName());

			String partyCustomCode = ballotOptionData.getParty()
					.getCustomCode();

			String[] partyCustomCodeTokens = partyCustomCode.split("\\_");

			partyCustomCode = partyCustomCodeTokens[partyCustomCodeTokens.length - 1];

			ballotOptionData.getParty().setCustomCode(partyCustomCode);

			partySet.add(ballotOptionData.getParty());

			BallotOptionCategoryEnum category = ballotOptionData.getCategory();

			if (category == BallotOptionCategoryEnum.STAND_IN) {
				standInList.add(eligibleOptionData);
			} else if (category == BallotOptionCategoryEnum.PRINCIPAL) {
				chiefList.add(eligibleOptionData);
			}
		}
	}

	/**
	 * Build a ballot card set with a ballot card object data.
	 * 
	 * @param generationCount
	 *            generation count reference
	 * @param contestList
	 *            the contest list to set.
	 * @return the set of ballot card data objects
	 */
	private Set<BallotCardData> buildBallotCardSet(int generationCount,
			List<ContestData> contestList) {

		Set<BallotCardData> ballotCardSet = new HashSet<BallotCardData>();
		BallotCardData ballotCard = new BallotCardData();

		ballotCard.setCode(generationCount);
		ballotCard.setCustomCode(String.valueOf(generationCount));
		ballotCard.setName(String.valueOf(generationCount));
		ballotCard.setAbstractKey(String.valueOf(generationCount));
		ballotCard.setHelp(String.valueOf(generationCount));
		ballotCard.setContest(contestList);

		ballotCardSet.add(ballotCard);

		return ballotCardSet;
	}

	/**
	 * Build a ballot card set with a ballot object data.
	 * 
	 * @param generationCount
	 *            generation count reference
	 * @param ballotCardSet
	 *            the ballot card set to the ballot data object.
	 * @return the set of ballot data objects
	 */
	private Set<BallotData> buildBallotSet(int generationCount,
			Set<BallotCardData> ballotCardSet) {

		BallotData ballot = new BallotData();
		Set<BallotData> ballotSet = new HashSet<BallotData>();

		ballot.setCode(generationCount);
		ballot.setName(String.valueOf(generationCount));
		ballot.setCustomCode(String.valueOf(generationCount));
		ballot.setAbstractKey(String.valueOf(generationCount));
		ballot.setHelp(String.valueOf(generationCount));
		ballot.setBallotCards(ballotCardSet);

		ballotSet.add(ballot);

		return ballotSet;
	}

	/**
	 * Method that translates a contest group data list.
	 * 
	 * @param contestGroupList
	 *            the contest group data list to translate
	 * @param electionLanguages
	 *            election languages set that the list must be translated
	 * @param resourcesLanguages
	 *            resources languages translations references.
	 * @return the contest group data list translated.
	 * @throws UtilitiesException
	 *             a signal that an error occurred translating the list.
	 */
	private List<ContestGroupData> getTranslatedContestGroupList(
			List<ContestGroupData> contestGroupList,
			Set<LanguageData> electionLanguages,
			Map<String, Map<String, String>> resourcesLanguages)
			throws UtilitiesException {

		List<ContestGroupData> constestGroupListTranslated = new ArrayList<ContestGroupData>();

		for (ContestGroupData contestGroupData : contestGroupList) {

			ContestGroupData contestGroupDataNew = new ContestGroupData();

			contestGroupDataNew.setCode(contestGroupData.getCode());
			contestGroupDataNew.setCustomCode(contestGroupData.getCustomCode());
			contestGroupDataNew.setContestGroupName(contestGroupData
					.getContestGroupName());
			contestGroupDataNew.setDescription(contestGroupData
					.getDescription());

			contestGroupDataNew.setContestCodesString(contestGroupData
					.getContestCodesString());

			this.utilitiesService.processMultilingualObject(electionLanguages,
					resourcesLanguages, contestGroupDataNew);

			constestGroupListTranslated.add(contestGroupDataNew);
		}

		return constestGroupListTranslated;
	}

	/**
	 * Gets the keys from a voting device and creates a new KeysData object with
	 * these keys.
	 * 
	 * @param votingDevice
	 *            the voting device to extract the keys.
	 * @return a KeysData object which contains the keys.
	 */
	private KeysData getKeysFromVotingDevice(VotingDeviceData votingDevice) {

		String votingDeviceProtectionKey = votingDevice.getProtectionKey() != null ? votingDevice
				.getProtectionKey() : "";
		String votingDeviceTransmissionKey = votingDevice.getTransmissionKey() != null ? votingDevice
				.getTransmissionKey() : "";
		String backupZipFilePassword = votingDevice.getNumericKey1() != null ? votingDevice
				.getNumericKey1() : "";

		KeysData keys = new KeysData(votingDeviceProtectionKey,
				votingDeviceTransmissionKey, backupZipFilePassword);

		return keys;
	}

	/**
	 * Set the required fields for the voting booth object
	 * 
	 * @param votingDevice
	 *            the voting device object from where the data fields are
	 *            extracted.
	 * @param region
	 *            the region which the voting booth belongs.
	 * @param generateDummyVotingDevice
	 *            true if is a dummy voting device and the region hierarchies
	 *            must be filled with dummy data, otherwise false.
	 * @param electionLanguage
	 *            the election language for this voting booth
	 * @param resourcesLanguages
	 *            the resources languages reference
	 * @param audioVotingEnabled
	 *            The audio voting flag
	 * @throws Exception
	 *             Signal that an error has occurred setting voting booth data.
	 */
	private void setVotingBoothData(VotingDeviceData votingDevice,
			List<ContestGroupData> contestGroupList, RegionData region,
			boolean generateDummyVotingDevice,
			Set<LanguageData> electionLanguage,
			Map<String, Map<String, String>> resourcesLanguages,
			boolean audioVotingEnabled) throws Exception {

		VotingBoothData votingBooth = votingDevice.getVotingBooth();

		PollingPlaceData pollingPlace = votingBooth.getPollingStation()
				.getPollingPlace();

		PollingStationData pollingStation = votingBooth.getPollingStation();

		Set<RegionHierarchy> regionHierarchies;

		if (generateDummyVotingDevice) {
			regionHierarchies = getRegionHierarchiesDummy(region);
			votingBooth
					.setTotalizationMatches(new HashSet<TotalizationNisCode>());
		} else {

			RegionData regionAllParents = this.demographyService
					.getAllParentsRegions(region);

			regionHierarchies = getRegionHierarchies(region, regionAllParents,
					electionLanguage, resourcesLanguages);
			votingBooth.setTotalizationMatches(getTotalizationMatches(
					pollingPlace.getCantonNisCode(), contestGroupList,
					regionAllParents));
		}

		votingBooth.setRegionHierarchies(regionHierarchies);

		votingBooth.setPollingPlaceCode(pollingPlace.getCode());

		votingBooth.setPollingPlaceName(pollingPlace.getName());

		votingBooth.setPollingPlaceAddress(pollingPlace.getAddress());

		votingBooth.setPollingPlaceRegisteredElectors(1000);

		votingBooth.setPollingStationCode(pollingStation.getCode());

		votingBooth
				.setPollingStationCriteria(pollingStation.getSplitCriteria());

		votingBooth.setAudioVotingEnable(audioVotingEnabled);

		votingBooth.setPollingStationNumber(1000);

		votingBooth.setAmountVoters(1000);

	}

	/**
	 * Calculates the totalization NIS codes per contest in the voting device
	 * 
	 * @param votingDeviceCustomCode
	 *            The voting device custom code
	 * @param contestGroupList
	 *            The contests group list of the voting device.
	 * @param region
	 *            The region asociated to the voting device.
	 * @return TotalizationMatches object to add in voting-devices.xml
	 * @throws Exception
	 */
	private Set<TotalizationNisCode> getTotalizationMatches(
			String cantonNisCode, List<ContestGroupData> contestGroupList,
			RegionData region) throws Exception {

		Map<String, TotalizationNisCode> totalizationMatches = new HashMap<String, TotalizationNisCode>();

		String nisCodeLastLevelRegion = region.getNisCode();

		String shortCode = "K";
		Level level = region.getCategory().getCategoryLevel();
		if (level == Level.EIGHTH_LEVEL) {
			shortCode = "I";
		} else if (level == Level.SEVENTH_LEVEL) {
			shortCode = "M";
		}

		for (ContestGroupData contestGroup : contestGroupList) {

			for (ContestData contest : contestGroup.getContests()) {

				if (!totalizationMatches.containsKey(contest.getCustomCode())) {

					TotalizationNisCode totNisCode = new TotalizationNisCode();
					totNisCode.setContestCustomCode(contest.getCustomCode());

					if (contest.isUseLastLevelRegionAsTotNisCode()) {
						totNisCode.setNisCode(nisCodeLastLevelRegion);
						totNisCode.setShortCode(shortCode);
					} else {
						if (cantonNisCode == null || cantonNisCode.isEmpty()) {
							throw new Exception(
									"Error creating the Totalization NIS code, "
											+ "The configuration of the contest:"
											+ contest.getCustomCode()
											+ " is wrong, there is not Canton NIS Code "
											+ "defined for this polling place");
						}
						totNisCode.setNisCode(cantonNisCode);
						totNisCode.setShortCode("K");
					}

					totalizationMatches
							.put(contest.getCustomCode(), totNisCode);
				}
			}
		}
		return new HashSet<TotalizationNisCode>(totalizationMatches.values());
	}

	/**
	 * Get the operation mode data object which contains the information
	 * required by the voting device that is being generated.
	 * 
	 * @param votingDevice
	 *            the voting device data object
	 * @param electionOpeningDate
	 *            the opening date for the election
	 * @param electionClosingDate
	 *            the closing date for the election
	 * @param keys
	 *            the keys data object
	 * @return the operation mode data object filled with the required
	 *         information.
	 */
	private OperationModeData getOperationMode(VotingDeviceData votingDevice,
			Date electionOpeningDate, Date electionClosingDate, KeysData keys) {

		OperationModeData operationMode = new OperationModeData();

		operationMode.setCode(1);
		operationMode.setCustomCode("1");
		operationMode.setOperationModeName("1");
		operationMode.setDescription("1");
		operationMode.setFolderName("1");
		operationMode.setDefault(false);
		operationMode.setUnblockingKey(keys.getBackupZipFilePassword());
		operationMode.setAudioUnblockingKey(0);
		operationMode.setScannerUnblockingKey(0);
		operationMode.setSynchronizationUnblockingKey(0);

		if (ApplicationProperties
				.getBooleanValue("set.default.value.for.election.date")) {

			String dateValue = ApplicationProperties
					.getValue("default.value.for.election.date");

			operationMode.setOpeningDateTimeString(dateValue);
			operationMode.setClosingDateTimeString(dateValue);
		} else {
			operationMode.setOpeningDateTime(electionOpeningDate);
			operationMode.setClosingDateTime(electionClosingDate);
		}

		operationMode.getOpeningDateTimeString();
		operationMode.getClosingDateTimeString();

		Set<RoleData> roles = new HashSet<RoleData>();

		RoleData role = new RoleData();

		String roleKey = votingDevice.getNumericKey1() != null ? votingDevice
				.getNumericKey2() : "";

		role.setKey(roleKey);
		role.setName("Consolidate Rol");
		role.setDescription("Consolidate Rol");
		role.setSystemRole(false);

		roles.add(role);

		operationMode.setRoles(roles);

		return operationMode;
	}

	/**
	 * Iterates through the specified contest list and adds the custom
	 * attributes for each checking if the attribute is already inserted into
	 * the map.
	 * 
	 * @param resourcesLanguages
	 *            the resources languages map reference
	 * @param electionLanguage
	 *            the current election language
	 * @param customAttributeDefinitionsMap
	 *            the custom attributes definition map reference to fill.
	 * @param attributeList
	 *            a list of attributes
	 * @param contestList
	 *            the contest list to iterate through
	 * @throws UtilitiesException
	 *             a signal that an error has occurred processing contest
	 *             attributes.
	 */
	private void processContestAttributes(
			Map<String, Map<String, String>> resourcesLanguages,
			Set<LanguageData> electionLanguage,
			Map<String, List<String>> customAttributeDefinitionsMap,
			Set<CustomAttribute> attributeList, Set<ContestData> contestList)
			throws UtilitiesException {

		List<ContestData> sortedContestList = new ArrayList<ContestData>(
				contestList);

		Collections.sort(sortedContestList, new ContestPrecedenceComparator());

		CustomAttribute attribute;

		for (ContestData contestIndex : sortedContestList) {

			attribute = new CustomAttribute();
			attribute.setCode(contestIndex.getContestClass().getCustomCode());
			attribute.setValue(contestIndex.getCollege());

			attributeList.add(attribute);

			this.utilitiesService.processMultilingualObject(electionLanguage,
					resourcesLanguages, contestIndex);

			this.utilitiesService.processMultilingualObject(electionLanguage,
					resourcesLanguages, contestIndex.getContestClass());

			String attributeKey = contestIndex.getContestClass()
					.getCustomCode()
					+ "$"
					+ contestIndex.getContestClass().getContestClassName()
					+ "$" + contestIndex.getContestClass().getPrecedence();

			List<String> customAttributeDefinitionsList;

			if (customAttributeDefinitionsMap.containsKey(attributeKey)) {

				customAttributeDefinitionsList = customAttributeDefinitionsMap
						.get(attributeKey);

				if (!customAttributeDefinitionsList.contains(contestIndex
						.getCollege())) {
					customAttributeDefinitionsList.add(contestIndex
							.getCollege());
				}

			} else {
				customAttributeDefinitionsList = new ArrayList<String>();
				customAttributeDefinitionsList.add(contestIndex.getCollege());
			}
			customAttributeDefinitionsMap.put(attributeKey,
					customAttributeDefinitionsList);
		}
	}

	/**
	 * Process the attributes for the specified contest group and adds the
	 * custom attributes for each checking if the attribute is already inserted
	 * into the map.
	 * 
	 * @param resourcesLanguages
	 *            the resources languages map reference
	 * @param electionLanguage
	 *            the current election language
	 * @param customAttributeDefinitions
	 *            the custom attributes definition map reference to fill.
	 * @param attributeList
	 *            a list of attributes
	 * @param contestGroupData
	 *            the contest group data to fetch the attributes
	 * @throws UtilitiesException
	 *             A signal that an error has occurred processing attributes for
	 *             the contest group data.
	 * @throws ElectoralDemographyException
	 */
	private void processContestGroupAttributes(
			Map<String, Map<String, String>> resourcesLanguages,
			Set<LanguageData> electionLanguage,
			Map<String, List<String>> customAttributeDefinitions,
			Set<CustomAttribute> attributeList,
			ContestGroupData contestGroupData) throws UtilitiesException,
			ElectoralDemographyException {

		ContestGroupData contestGroupDataOriginal = this.electoralDemographyService
				.getContestGroupByCustomCode(contestGroupData
						.getOriginalCustomCode());

		this.utilitiesService.processMultilingualObject(electionLanguage,
				resourcesLanguages, contestGroupDataOriginal);

		CustomAttribute attribute = new CustomAttribute();

		attribute.setCode(CustomAttribute.Codes.NATIONALITY.toString());
		attribute.setValue(contestGroupDataOriginal.getContestGroupName());

		attributeList.add(attribute);

		String attributeKey = CustomAttribute.Codes.NATIONALITY.toString()
				+ "$" + CustomAttribute.Codes.NATIONALITY.toString() + "$" + 0;

		List<String> customAttributeDefinitionsList;

		if (customAttributeDefinitions.containsKey(attributeKey)) {
			customAttributeDefinitionsList = customAttributeDefinitions
					.get(attributeKey);
			if (!customAttributeDefinitionsList
					.contains(contestGroupDataOriginal.getContestGroupName())) {
				customAttributeDefinitionsList.add(contestGroupDataOriginal
						.getContestGroupName());
			}
			customAttributeDefinitions.remove(attributeKey);
		} else {
			customAttributeDefinitionsList = new ArrayList<String>();
			customAttributeDefinitionsList.add(contestGroupDataOriginal
					.getContestGroupName());

		}

		customAttributeDefinitions.put(attributeKey,
				customAttributeDefinitionsList);
	}

	/**
	 * Gets a custom attribute definition list with its values from a map of
	 * custom attributes.
	 * 
	 * @param customAttributeDefinitionsMap
	 *            the custom attribute definition map
	 * @return the custom attribute definition list to be attached to the voting
	 *         device object
	 * @throws ElectoralDemographyException
	 *             a signal that an error has occurred getting the list.
	 */
	private List<CustomAttributeDefinition> getCustomAttributeDefinitionList(
			Map<String, List<String>> customAttributeDefinitionsMap)
			throws ElectoralDemographyException {

		String separator = "\\$";

		List<CustomAttributeDefinition> customAttributeDefinitionList = new ArrayList<CustomAttributeDefinition>();

		for (Map.Entry<String, List<String>> entry : customAttributeDefinitionsMap
				.entrySet()) {

			CustomAttributeDefinition customAttributeDefinition = new CustomAttributeDefinition();

			boolean isVotingMachine = true;

			String entryKey = entry.getKey().toString();

			customAttributeDefinition.setCode(entryKey.split(separator)[0]);
			customAttributeDefinition.setName(entryKey.split(separator)[1]);
			customAttributeDefinition.setDescriptionKey(entryKey
					.split(separator)[1]);
			customAttributeDefinition.setPrecedence(Long.valueOf(entryKey
					.split(separator)[2]));

			String customAttributeDefinitionValueString = entryKey
					.split(separator)[1];

			if (customAttributeDefinitionValueString.equals("NATIONALITY")) {

				customAttributeDefinition.setName("NATIONALITY_NAME_KEY");
				customAttributeDefinition
						.setDescriptionKey("NATIONALITY_DESCRIPTION_KEY");

				customAttributeDefinition.setMachineType("PM");
				isVotingMachine = false;
			} else {
				customAttributeDefinition.setMachineType("VM");
			}

			List<String> customAttributeDefinitionsValuesStringList = entry
					.getValue();

			List<CustomAttributeDefinitionValue> customAttributeDefinitionValuesList = new ArrayList<CustomAttributeDefinitionValue>();

			for (String customAttributeDefinitionsValuesString : customAttributeDefinitionsValuesStringList) {
				CustomAttributeDefinitionValue customAttributeDefinitionValue = new CustomAttributeDefinitionValue();

				customAttributeDefinitionValue
						.setValue(customAttributeDefinitionsValuesString);

				if (isVotingMachine) {

					long contestCode = Long
							.parseLong(customAttributeDefinitionsValuesString
									.replaceAll("[\\D]", ""));

					ContestData contest = this.electoralDemographyService
							.getContestByCode(contestCode);

					boolean setLanguagesForAllContest = ApplicationProperties
							.getBooleanValue("set.languages.for.all.contest");

					if (setLanguagesForAllContest || contest.getType() != null
							&& contest.getCollegeLanguageRFC() != null) {
						customAttributeDefinitionValue.setLanguage(contest
								.getCollegeLanguageRFC());
					}
				}

				customAttributeDefinitionValuesList
						.add(customAttributeDefinitionValue);
			}

			if (customAttributeDefinition.getCode().equals("K")) {
				Collections.sort(customAttributeDefinitionValuesList,
						new CustomAttributeDefinitionValueComparator());
			}

			customAttributeDefinition
					.setCustomAttributeDefinitionValues(customAttributeDefinitionValuesList);

			customAttributeDefinitionList.add(customAttributeDefinition);
		}

		Collections.sort(customAttributeDefinitionList,
				new CustomAttributePrecedenceComparator());

		return customAttributeDefinitionList;
	}

	/**
	 * Gets the region hierarchies set for a given region.
	 * 
	 * @param region
	 *            the region to get the hierarchies
	 * @param electionLanguage
	 *            the current election language
	 * @param resourcesLanguages
	 *            the resources languages reference
	 * @return the region hierarchies set
	 * @throws Exception
	 *             a signal that an error has occurred getting the hierarchies.
	 */
	private Set<RegionHierarchy> getRegionHierarchies(RegionData region,
			RegionData allParentsRegions, Set<LanguageData> electionLanguage,
			Map<String, Map<String, String>> resourcesLanguages)
			throws Exception {

		List<PathData> pathsList = new ArrayList<PathData>(allParentsRegions
				.getCategory().getPaths());

		List<RegionData> parentsList = new ArrayList<RegionData>(
				allParentsRegions.getAllParents());

		Collections.sort(parentsList);

		region.setAllParents(new HashSet<RegionData>(parentsList));

		RegionData regionBigParent = parentsList.iterator().next();

		Set<RegionHierarchy> regionHierarchies = new HashSet<RegionHierarchy>();

		for (PathData pathData : pathsList) {

			RegionHierarchy regionHierarchy = new RegionHierarchy();

			regionHierarchy.setCode(pathData.getCode());
			regionHierarchy.setName(pathData.getName());
			regionHierarchy.setDefault(pathData.isDefaultPath());

			CategoryData category = null;
			RegionData regionTmp = null;
			boolean invalidRegionPath = false;

			List<RegionData> regionList = new ArrayList<RegionData>();
			regionList.add(cloneRegion(allParentsRegions));

			do {
				regionTmp = this.demographyService.checkRegionParentPath(
						regionTmp == null ? allParentsRegions : regionTmp,
						pathData);

				if (regionTmp != null) {
					regionList.add(cloneRegion(regionTmp));
				} else {
					invalidRegionPath = true;
					break;
				}
			} while (regionBigParent.getCode() != regionTmp.getCode());

			if (!invalidRegionPath) {
				Collections.sort(regionList);

				for (RegionData regionData : regionList) {
					category = setCategoryParent(category, regionData,
							electionLanguage, resourcesLanguages);
				}

				regionHierarchy.setCategory(category);
				regionHierarchies.add(regionHierarchy);
			}
		}

		return regionHierarchies;
	}

	/**
	 * Method used to create another region object with the information of the
	 * region provided.
	 * 
	 * @param region
	 *            the region to clone
	 * @return to region object cloned
	 */
	private RegionData cloneRegion(RegionData region) {

		RegionData newRegion = new RegionData();
		CategoryData newCategory = new CategoryData();
		Set<PathData> newPathSet = new HashSet<PathData>();

		newRegion.setCode(region.getCode());
		newRegion.setRegionName(region.getRegionName());

		newCategory.setCode(region.getCategory().getCode());
		newCategory.setCategoryName(region.getCategory().getCategoryName());
		newCategory.setCategoryLevel(region.getCategory().getCategoryLevel());

		for (PathData pathData : region.getCategory().getPaths()) {

			PathData newPath = new PathData();

			newPath.setCode(pathData.getCode());
			newPath.setName(pathData.getName());
			newPathSet.add(newPath);
		}

		newCategory.setPaths(newPathSet);

		newRegion.setCategory(newCategory);

		return newRegion;
	}

	/**
	 * Method that build a set of region hierarchies with dummy data for
	 * screenshot functionality.
	 * 
	 * @param regionDummy
	 *            Region dummy to build the Region Hierarchy
	 * 
	 * @return Set of Region Hierarchy dummy.
	 */
	private Set<RegionHierarchy> getRegionHierarchiesDummy(
			RegionData regionDummy) {

		Set<RegionHierarchy> regionHierarchies = new HashSet<RegionHierarchy>();
		RegionHierarchy regionHierarchy = new RegionHierarchy();

		regionHierarchy.setCode(regionDummy.getCode());
		regionHierarchy.setName(regionDummy.getRegionName());
		regionHierarchy.setDefault(true);

		PathData defaultpath = new PathData();
		defaultpath.setCode(regionDummy.getCode());
		defaultpath.setCustomCode(regionDummy.getCustomCode());
		defaultpath.setName(regionDummy.getRegionName());
		defaultpath.setDefaultPath(true);

		Set<PathData> pathSet = new HashSet<PathData>();
		pathSet.add(defaultpath);

		CategoryData category = new CategoryData();
		category.setCategoryName(regionDummy.getRegionName());
		category.setPaths(pathSet);

		RegionData regionForXml = new RegionData();
		regionForXml.setCode(regionDummy.getCode());
		regionForXml.setRegionName(regionDummy.getRegionName());

		category.setRegion(regionForXml);
		regionHierarchy.setCategory(category);
		regionHierarchies.add(regionHierarchy);

		return regionHierarchies;
	}

	/**
	 * Recursive method to set and process the languages for a category and its
	 * parents.
	 * 
	 * @param category
	 *            the category to set
	 * @param region
	 *            the region to set to the category
	 * @param electionLanguage
	 *            the current election language
	 * @param resourcesLanguages
	 *            the resources languages object reference
	 * @return the category data object with its parents set.
	 * @throws UtilitiesException
	 *             a signal that an error has occurred setting the category.
	 */
	private CategoryData setCategoryParent(CategoryData category,
			RegionData region, Set<LanguageData> electionLanguage,
			Map<String, Map<String, String>> resourcesLanguages)
			throws UtilitiesException {

		if (category != null) {
			category.setCategory(setCategoryParent(category.getCategory(),
					region, electionLanguage, resourcesLanguages));

			return category;
		} else {
			CategoryData regionCategory = region.getCategory();
			this.utilitiesService.processMultilingualObject(electionLanguage,
					resourcesLanguages, regionCategory);
			this.utilitiesService.processMultilingualObject(electionLanguage,
					resourcesLanguages, region);
			regionCategory.setRegion(region);

			return regionCategory;
		}
	}

	// END OF REFACTORED CODE // TODO: Remove this line at the end of refactor

	/**
	 * Method to get a List of Region codes giving a List of region data.
	 * 
	 * @param regionDataList
	 *            Region Data List to get their codes.
	 * 
	 * @return List of Region codes.
	 */
	private List<Long> extractRegionCodes(List<RegionData> regionDataList) {
		List<Long> regionCodes = new ArrayList<Long>();

		for (Iterator<RegionData> iterator = regionDataList.iterator(); iterator
				.hasNext();) {
			RegionData regionData = iterator.next();
			regionCodes.add(regionData.getCode());
		}

		return regionCodes;
	}
}
