/**
 * ECMUtilitiesServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import static com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException.BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.sql.DataSource;

import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperPrint;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.IBaseDataObject;
import com.smartmatic.waffle.domain.authority.ElectoralAuthorityData;
import com.smartmatic.waffle.domain.ballot.BallotOptionData;
import com.smartmatic.waffle.domain.contest.BallotLayoutData;
import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.CategoryData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.PathData;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.PollingStationData;
import com.smartmatic.waffle.domain.demographics.ProgrammingUnitData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.device.DeviceParameterData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.eligibleoption.CandidateData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionRoleData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.general.BaseGeneralInfo;
import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.domain.general.ReportTemplateData;
import com.smartmatic.waffle.domain.i18n.ContestLanguagesPairData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.multiLanguage.MultiLingualKeyData;
import com.smartmatic.waffle.domain.multiLanguage.StringResourceData;
import com.smartmatic.waffle.domain.multiLanguage.TranslationKeyData;
import com.smartmatic.waffle.domain.security.ProfileData;
import com.smartmatic.waffle.domain.user.UserData;
import com.smartmatic.waffle.ecm.bulk.service.util.BulkCaches;
import com.smartmatic.waffle.ecm.core.printing.PrinterAssistant;
import com.smartmatic.waffle.ecm.data.transfer.comparators.ContestClassPrecedenceComparator;
import com.smartmatic.waffle.ecm.data.transfer.objects.ContestClassTranslationsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.EntityRecordsQuantitiesDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.QueryResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.SmttFileType;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException.BulkDataLoadExceptionOperationExceptionInfo;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException.ElectoralPhysicalLocationsRegistryExceptionInfo;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException.UtilitiesExceptionInfo;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralOrganizationServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesService;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesServiceLocal;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.lang.util.ReflectHelper;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.persistence.metadata.MultilingualPropertyAnnotation;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.search.criteria.OrderCriteria;
import com.smartmatic.waffle.persistence.search.transformation.TransformationFactory;
import com.smartmatic.waffle.persistence.search.transformation.TransformationsList;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;
import com.smartmatic.waffle.properties.ApplicationProperties;
import com.smartmatic.waffle.properties.ECMApplicationProperties;
import com.smartmatic.waffle.security.sign.SignXMLUtil;

/**
 * Session beans that handles utilities server side operations.This class
 * implements all the server services defined in
 * <code>IECMUtilitiesService</code>
 */
@Stateless(name = IECMUtilitiesService.SERVICE_NAME)
@Remote(IECMUtilitiesService.class)
@Local(IECMUtilitiesServiceLocal.class)
public class ECMUtilitiesServiceImpl implements IECMUtilitiesServiceLocal {

	/** Object to generate logs during the application execution. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMUtilitiesServiceImpl.class);
	/**
	 * ECM_READ_ONLY Data source
	 */
	@Resource(mappedName = "java:ECM_READ_ONLY_DS")
	private DataSource dataSourceReadOnly;

	/**
	 * ECM Data source
	 */
	@Resource(mappedName = "java:ECM_DS")
	private DataSource dataSource;

	/**
	 * Instance of IPersistenceServiceLocal
	 */
	@EJB
	private IPersistenceServiceLocal persistenceService;

	/**
	 * Instance of IECMElectoralOrganizationServiceLocal
	 */
	@EJB
	private IECMElectoralOrganizationServiceLocal ecmElectoralOrganizationService;

	/**
	 * @inheritDoc
	 */
	@Override
	public QueryResultDTO executeQuery(String sqlQuery)
			throws UtilitiesException {

		Connection connection = null;
		PreparedStatement preparedStatement = null;
		ResultSet resultSet = null;
		QueryResultDTO queryResult = new QueryResultDTO();
		try {
			connection = this.dataSourceReadOnly.getConnection();
			preparedStatement = connection.prepareStatement(sqlQuery);
			resultSet = preparedStatement.executeQuery();

			if (resultSet != null) {
				ResultSetMetaData resultSetMetaData = resultSet.getMetaData();

				List<ArrayList<String>> rows = new ArrayList<ArrayList<String>>();

				ArrayList<String> columns;

				boolean columnNamesSet = false;

				while (resultSet.next()) {
					if (!columnNamesSet) {
						setColumnNames(queryResult, resultSet,
								resultSetMetaData);
						columnNamesSet = true;
					}
					columns = new ArrayList<String>();

					fillColumnValues(resultSet, resultSetMetaData, columns);

					rows.add(columns);
				}
				if (rows.size() > 0) {
					queryResult.setRows(rows);
				}
			}

			return queryResult;

		} catch (SQLException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_EXECUTING_QUERY,
					StringUtils.defaultString(e.getMessage()));
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_EXECUTING_QUERY,
					StringUtils.defaultString(e.getMessage()));
		} finally {
			try {
				if (resultSet != null) {
					resultSet.close();
				}
				if (preparedStatement != null) {
					preparedStatement.close();
				}
				if (connection != null) {
					connection.close();
				}
			} catch (SQLException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				throw new UtilitiesException(e,
						UtilitiesExceptionInfo.PROBLEM_CLOSING_CONNECTION,
						StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public List<EntityRecordsQuantitiesDTO> getEntityRecordsQuantities()
			throws BulkDataLoadException {
		try {

			List<EntityRecordsQuantitiesDTO> result = new ArrayList<EntityRecordsQuantitiesDTO>();

            result.add(
                this.getEntityRecordsQuantities(
                    SmttFileType.DEVICE_PARAMETER, DeviceParameterData.class
                )
            );
			result.add(getEntityRecordsQuantities(SmttFileType.KEYSTORE,
					KeystoreData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.ELECTION,
					ElectionData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.CATEGORY,
					CategoryData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.REGION,
					RegionData.class));
			result.add(getEntityRecordsQuantities(
					SmttFileType.PROGRAMMING_UNIT, ProgrammingUnitData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.PATH,
					PathData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.DISTRICT,
					DistrictData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.BALLOT_LAYOUT,
					BallotLayoutData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.CONTEST_CLASS,
					ContestClassData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.CONTEST,
					ContestData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.CONTEST_GROUP,
					ContestGroupData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.LANGUAGE,
					LanguageData.class));
			result.add(getEntityRecordsQuantities(
					SmttFileType.ELECTORAL_AUTHORITY,
					ElectoralAuthorityData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.PROFILE,
					ProfileData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.USER,
					UserData.class));
			result.add(getEntityRecordsQuantities(SmttFileType.REPORT_TEMPLATE,
					ReportTemplateData.class));

			return result;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_RETRIEVING_ENTITY_RECORDS_QUANTITIES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public int retrieveMultilingualKeysAmount() throws BulkDataLoadException {
		try {
			SearchExpression search = new SearchExpression(
					MultiLingualKeyData.class);
			return this.persistenceService.countObjects(search, null);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.RETRIEVING_MULTILINGUAL_AMOUNT_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<String> getTranslationkeysToDownload(int pageNumber,
			int pageSize) throws BulkDataLoadException {

		try {

			List<String> linesToFileList = new ArrayList<String>();

			SearchExpression searchExpression = new SearchExpression(
					MultiLingualKeyData.class);

			searchExpression.addOrder(OrderCriteria.asc("customCode"));

			List<MultiLingualKeyData> multilingualKeysList = (List<MultiLingualKeyData>) this.persistenceService
					.searchPagedObjects(searchExpression, null, pageNumber,
							pageSize);

			searchExpression = new SearchExpression(LanguageData.class);

			List<LanguageData> languagesList = (List<LanguageData>) this.persistenceService
					.searchObjects(searchExpression, null);

			for (MultiLingualKeyData multilingualKey : multilingualKeysList) {

				searchExpression = new SearchExpression(
						Class.forName(multilingualKey.getKey()));

				searchExpression.addSearchCriteria(CriteriaFactory.eq("code",
						multilingualKey.getForeignKey()));

				Object objectResult = this.persistenceService.searchObject(
						searchExpression, null);

				for (LanguageData language : languagesList) {

					if (!language.getDefaultLanguage()) {

						String originalValue = (String) PropertyUtils
								.getProperty(objectResult,
										multilingualKey.getField());

						String customCode = (String) PropertyUtils.getProperty(
								objectResult, "customCode");

						String className = objectResult.getClass()
								.getSimpleName();

						String multilingualCustomCode = className.toUpperCase()
								+ "__"
								+ multilingualKey.getField().toUpperCase()
								+ "_" + customCode.toUpperCase();

						if (originalValue != null && !originalValue.isEmpty()) {

							linesToFileList.add(multilingualCustomCode + "_"
									+ language.getCustomCode() + "|"
									+ language.getCustomCode() + "|"
									+ multilingualCustomCode + "|" + "|"
									+ originalValue);
						}
					}
				}

			}

			return linesToFileList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_EXPORTING_MULTILINGUAL_KEYS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateLanguageTables() throws BulkDataLoadException {

		try {
			this.persistenceService
					.executeUpdateNativeQuery("truncate table translation_key cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table multilingual_key cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table region_election_language cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table election_language cascade");

			removeRefObjectFromBulkCache();

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateNominationTables() throws BulkDataLoadException {

		try {

			this.persistenceService
					.executeUpdateNativeQuery("truncate table region_contest_group cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table contest_group_contest cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table contest_group cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table contest cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table contest_class cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table region_district cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table district cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table ballot_layout cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table candidate cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table party cascade");

			removeRefObjectFromBulkCache();

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}

	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateUserTables() throws BulkDataLoadException {

		try {

			this.persistenceService
					.executeUpdateQuery("delete from "
							+ UserData.class.getSimpleName()
							+ " user where user.code in "
							+ "(select user2.code from "
							+ UserData.class.getSimpleName()
							+ " user2 left outer join user2.profile  where ((not user2.profile.adminProfile = true) "
							+ "or (user2.profile is null) or (user2.electoralAuthority is not null)))");

			this.persistenceService
					.executeUpdateNativeQuery("delete from profile_function where profile_code in("
							+ " select pf.profile_code from profile_function pf inner join profile p"
							+ " on pf.profile_code = p.profile_code"
							+ " where not p.adminprofile)");

			this.persistenceService.executeUpdateQuery("delete from "
					+ ProfileData.class.getSimpleName()
					+ " profile WHERE profile.adminProfile != true");

			this.persistenceService.executeUpdateQuery("delete from "
					+ ElectoralAuthorityData.class.getSimpleName());

			removeRefObjectFromBulkCache();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncatePhysicalLocationsTables() throws BulkDataLoadException {
		try {

			this.persistenceService
					.executeUpdateNativeQuery("truncate voting_booth cascade");

			this.persistenceService
					.executeUpdateNativeQuery("delete from voting_device cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate polling_station cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate polling_place cascade");

			removeRefObjectFromBulkCache();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateDemographyTables() throws BulkDataLoadException {
		try {
			this.persistenceService
					.executeUpdateNativeQuery("truncate table category_path");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table category_parent");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table region_programming_unit cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table programming_unit cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table region_all_parent");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table region_parent");

			this.persistenceService
					.executeUpdateNativeQuery("delete from region");

			this.persistenceService
					.executeUpdateNativeQuery("delete from category");

			this.persistenceService
					.executeUpdateNativeQuery("delete from path");

			this.persistenceService
					.executeUpdateNativeQuery("delete from election");

			this.persistenceService.executeUpdateQuery("delete from "
					+ KeystoreData.class.getSimpleName()
					+ " where is_default = false");

			removeRefObjectFromBulkCache();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateReportTemplatesTables() throws BulkDataLoadException {
		try {
			this.persistenceService
					.executeUpdateNativeQuery("truncate table report_template_region");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table report_template");

			deleteReportFiles();

			removeRefObjectFromBulkCache();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateLogicalVotingGroup() throws BulkDataLoadException {
		try {

			this.persistenceService
					.executeUpdateNativeQuery("truncate table logical_voting_group_voting_booth");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table logical_voting_group cascade");

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateEligibleOption() throws BulkDataLoadException {
		try {
			this.persistenceService
					.executeUpdateNativeQuery("truncate table ballot_option cascade");

			this.persistenceService
					.executeUpdateNativeQuery("truncate table eligible_option_role cascade");

			removeRefObjectFromBulkCache();

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateCodiNominationTables() throws BulkDataLoadException {
		try {

			truncateContestLanguagesPair();

			this.persistenceService
					.executeUpdateNativeQuery("truncate table eligible_option cascade");

			deleteLanguageKeyForCodiTables(EligibleOptionData.class
					.getSimpleName());

			this.persistenceService
					.executeUpdateNativeQuery("truncate table party cascade");

			deleteLanguageKeyForCodiTables(PartyData.class.getSimpleName());

			this.persistenceService
					.executeUpdateNativeQuery("truncate table candidate cascade");

			deleteLanguageKeyForCodiTables(CandidateData.class.getSimpleName());

			removeRefObjectFromBulkCache();

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}

	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void truncateContestLanguagesPair() throws BulkDataLoadException {
		try {
			this.persistenceService
					.executeUpdateNativeQuery("truncate table contest_languages_pair");
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void deleteReportFiles() throws BulkDataLoadException {
		try {
			File reportFilesDir = getReportFilesDir();

			FileFilter fileFilter = new FileFilter() {
				@Override
				public boolean accept(File file) {
					return file.isFile();
				}
			};

			File[] reportFiles = reportFilesDir.listFiles(fileFilter);

			if (reportFiles != null) {
				for (File file : reportFiles) {
					FileUtils.forceDelete(file);
				}
			}
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void deleteEMLFiles() throws BulkDataLoadException {
		try {

			this.persistenceService
					.executeUpdateNativeQuery("update contest set codi_file = null");

			File emlFilesDir = getEMLFilesDir();
			deleteDirectory(emlFilesDir);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	private void deleteDirectory(File directory) throws IOException {

		if (directory.exists()) {
			File[] files = directory.listFiles();

			for (File fileToDelete : files) {
				FileUtils.forceDelete(fileToDelete);
			}
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void deleteVoiceFiles() throws BulkDataLoadException {
		try {
			FileUtils.deleteDirectory(getVoiceFolder());

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Gets the voice files directory.
	 * 
	 * @return the voice files directory
	 */
	protected File getVoiceFolder() {
		String voiceFilesFolder = getServerPropertyString("voices.files.folder");
		File voicesFilesDir = new File(
				System.getProperty("jboss.server.home.dir"), voiceFilesFolder);
		return voicesFilesDir;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public IBaseDataObject updateObject(IBaseDataObject baseDataObject)
			throws BulkDataLoadException {
		try {
			return this.persistenceService.update(baseDataObject, null);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_UPDATING_OBJECT_IN_DATABASE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public boolean getServerPropertyAsBoolean(String key) {
		return ApplicationProperties.getBooleanValue(key);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public String getServerPropertyAsString(String key) {
		return ApplicationProperties.getValue(key);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("rawtypes")
	public ContestData getItemByCustomCode(String customCode, Class clazz)
			throws UtilitiesException {
		try {
			SearchExpression search = new SearchExpression(clazz);

			search.addSearchCriteria(CriteriaFactory.eq("customCode",
					customCode));

			ContestData contestResult;

			contestResult = (ContestData) this.persistenceService.searchObject(
					search, null);

			return contestResult;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(
					e,
					UtilitiesExceptionInfo.RETRIEVING_ITEM_BY_CUSTOM_CODE_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public boolean checkIfSecurityComponentsAreGenerated()
			throws ElectoralPhysicalLocationsRegistryException {

		try {

			SearchExpression search = new SearchExpression(
					VotingDeviceData.class);

			search.addSearchCriteria(CriteriaFactory
					.isNotNull("asymmetricKeys"));

			int numberOfVotingDevices = this.persistenceService.countObjects(
					search, null);

			search = new SearchExpression(ElectoralAuthorityData.class);

			search.addSearchCriteria(CriteriaFactory
					.isNotNull("asymmetricKeys"));

			int numberOfElectoralAuthorities = this.persistenceService
					.countObjects(search, null);

			return (numberOfVotingDevices + numberOfElectoralAuthorities) > 0;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralPhysicalLocationsRegistryException(
					e,
					ElectoralPhysicalLocationsRegistryExceptionInfo.SEARCHING_SECURITY_COMPONENTS_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public boolean checkIfDataIsLoaded() throws BaseException {
		return this.checkIfStaticDataIsLoaded()
				&& this.checkIfCodiDataIsLoaded();
	}

	/**
	 * @inheritDoc
	 */
	@SuppressWarnings("unchecked")
	@Override
	public ContestData translateContestName(ContestData contest,
			LanguageData language) throws UtilitiesException {
		try {
			SearchExpression expression = new SearchExpression(
					TranslationKeyData.class);

			expression.createAlias("multilingualKey", "multilingualKey");
			expression.createAlias("electionLanguage", "electionLanguage");

			expression.addSearchCriteria(CriteriaFactory.eq(
					"multilingualKey.customCode", "CONTESTDATA__NAME_"
							+ contest.getCustomCode()));

			expression.addSearchCriteria(CriteriaFactory.eq(
					"electionLanguage.languageRFC", language.getLanguageRFC()));

			List<TranslationKeyData> translations = (List<TranslationKeyData>) this.persistenceService
					.searchObjects(expression, null);

			if (translations != null && !translations.isEmpty()) {
				TranslationKeyData translation = translations.iterator().next();
				contest.setContestNameTranslate(translation
						.getTranslationText());
			} else {
				contest.setContestNameTranslate(new String(contest.getName()));
			}

			if (contest.getDistrict() != null) {
				DistrictData disctrictToTraslate = translateDistrictName(
						contest.getDistrict(), language);

				contest.setDistrict(disctrictToTraslate);
			}
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.TRANSLATING_CONTEST_NAME_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
		return contest;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public DistrictData translateDistrictName(DistrictData district,
			LanguageData language) throws UtilitiesException {

		try {
			SearchExpression expression = new SearchExpression(
					TranslationKeyData.class);

			expression.createAlias("multilingualKey", "multilingualKey");
			expression.createAlias("electionLanguage", "electionLanguage");

			expression.addSearchCriteria(CriteriaFactory.eq(
					"multilingualKey.customCode", "DISTRICTDATA__DISTRICTNAME_"
							+ district.getCustomCode()));

			expression.addSearchCriteria(CriteriaFactory.eq(
					"electionLanguage.customCode", language.getCustomCode()));

			TranslationKeyData translation = (TranslationKeyData) this.persistenceService
					.searchObject(expression, null);

			if (translation != null) {
				district.setDistrictNameTraslated(translation
						.getTranslationText());
			} else {
				district.setDistrictNameTraslated(district.getDistrictName());
			}
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.TRANSLATING_DISTRICT_NAME_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
		return district;

	}

	/**
	 * @inheritDoc
	 */
	@Override
	public ContestClassData translateContestClassName(
			ContestClassData contestClass, LanguageData language)
			throws UtilitiesException {

		try {
			SearchExpression expression = new SearchExpression(
					TranslationKeyData.class);

			expression.createAlias("multilingualKey", "multilingualKey");
			expression.createAlias("electionLanguage", "electionLanguage");

			expression.addSearchCriteria(CriteriaFactory.eq(
					"multilingualKey.customCode",
					"CONTESTCLASSDATA_CONTESTCLASSNAME_"
							+ contestClass.getCustomCode()));

			expression.addSearchCriteria(CriteriaFactory.eq(
					"electionLanguage.customCode", language.getCustomCode()));

			TranslationKeyData translation = (TranslationKeyData) this.persistenceService
					.searchObject(expression, null);

			if (translation != null) {
				contestClass.setContestClassNameTranslate(translation
						.getTranslationText());
			} else {
				contestClass.setContestClassNameTranslate(contestClass
						.getContestClassName());
			}
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(
					e,
					UtilitiesExceptionInfo.TRANSLATING_CONTEST_CLASS_NAME_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
		return contestClass;

	}

	/**
	 * @inheritDoc
	 */
	@Override
	public Map<String, Map<String, String>> processMultilingualObject(
			Set<LanguageData> languagesToTranslate,
			Map<String, Map<String, String>> currentResourcesLanguages,
			BaseGeneralInfo objectToTranslate) throws UtilitiesException {
		try {

			// Lists the attributes annotated with
			// @MultilingualPropertyAnnotation
			List<MultiLingualKeyData> multiLingualKeyDataList = getMultilingualFields(objectToTranslate);

			for (MultiLingualKeyData multiLingualKeyData : multiLingualKeyDataList) {

				// Gets the translationKey for the attribute in available
				// languages
				List<TranslationKeyData> translationKey = getTranslationKeyPerLaguages(
						multiLingualKeyData, languagesToTranslate);

				for (TranslationKeyData translationKeyData : translationKey) {

					StringResourceData stringResources = new StringResourceData();

					stringResources.setKey(getKey(translationKeyData));

					Map<String, String> languageMap = null;

					Map<String, String> defaultMap = null;

					// Building the key for final map with the language custom
					// code
					String keyMap = translationKeyData.getElectionLanguage()
							.getCustomCode();

					if (currentResourcesLanguages.containsKey(keyMap)) {
						languageMap = currentResourcesLanguages.get(keyMap);
					} else {
						languageMap = new HashMap<String, String>();
						currentResourcesLanguages.put(keyMap, languageMap);
					}

					// Creates the map for default language
					if (currentResourcesLanguages.containsKey("DEFAULT")) {
						defaultMap = currentResourcesLanguages.get("DEFAULT");
					} else {
						defaultMap = new HashMap<String, String>();
						currentResourcesLanguages.put("DEFAULT", defaultMap);
					}

					// if the attribute don't contains a translation in
					// database, its default value is used
					if (translationKeyData.getTranslationText() == null) {
						String value = (String) ReflectHelper.invokeGetter(
								objectToTranslate, translationKeyData
										.getMultilingualKey().getField());

						if (value == null) {
							value = "";
						}

						if (!defaultMap.containsKey(getKey(translationKeyData))) {
							defaultMap.put(getKey(translationKeyData), value);
						} else {
							value = defaultMap.get(getKey(translationKeyData));
						}

						if (!languageMap
								.containsKey(getKey(translationKeyData))) {
							languageMap.put(getKey(translationKeyData), value);
						} else {
							value = languageMap.get(getKey(translationKeyData));
						}

						stringResources.setValue(value);

						// else the translation value in
						// database is used
					} else {

						String valueDefault = (String) ReflectHelper
								.invokeGetter(objectToTranslate,
										translationKeyData.getMultilingualKey()
												.getField());

						if (valueDefault != null) {
							if (!defaultMap
									.containsKey(getKey(translationKeyData))) {
								defaultMap.put(getKey(translationKeyData),
										valueDefault);
							}
						}

						String value = translationKeyData.getTranslationText();

						if (value == null) {
							value = "";
						}

						if (!languageMap
								.containsKey(getKey(translationKeyData))) {
							languageMap.put(getKey(translationKeyData), value);
						} else {
							value = languageMap.get(getKey(translationKeyData));
						}

						stringResources.setValue(value);
					}

					for (LanguageData electionLanguage : languagesToTranslate) {
						if (electionLanguage.getCode() == translationKeyData
								.getElectionLanguage().getCode()) {
							electionLanguage
									.setStringResources(stringResources);
						}
					}
				}

				for (TranslationKeyData translationKeyData : translationKey) {
					ReflectHelper.invokeSetter(objectToTranslate,
							translationKeyData.getMultilingualKey().getField(),
							getKey(translationKeyData));
				}
			}

			return currentResourcesLanguages;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(
					e,
					UtilitiesExceptionInfo.PROBLEM_PROCESSING_MULTILINGUAL_OBJECT,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public ContestData translateCollege(ContestData contest,
			LanguageData language) throws UtilitiesException {

		try {
			SearchExpression expression = new SearchExpression(
					TranslationKeyData.class);

			expression.createAlias("multilingualKey", "multilingualKey");
			expression.createAlias("electionLanguage", "electionLanguage");

			expression.addSearchCriteria(CriteriaFactory.eq(
					"multilingualKey.customCode", "CONTESTDATA__COLLEGE_"
							+ contest.getCustomCode()));

			expression.addSearchCriteria(CriteriaFactory.eq(
					"electionLanguage.languageRFC", language.getLanguageRFC()));

			List<TranslationKeyData> translations = (List<TranslationKeyData>) this.persistenceService
					.searchObjects(expression, null);

			if (translations != null && !translations.isEmpty()) {
				TranslationKeyData translation = translations.iterator().next();
				contest.setCollegeTranslate(translation.getTranslationText());
			} else {
				if (contest.getCollege() != null) {
					contest.setCollegeTranslate(new String(contest.getCollege()));
				}
			}

			return contest;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.TRANSLATING_CONTEST_COLLEGE_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<LanguageData> getParentLanguages(ContestData contest)
			throws UtilitiesException {

		List<LanguageData> parentLanguages;

		try {
			SearchExpression search = new SearchExpression(LanguageData.class);
			search.createAlias("regions", "regions");
			search.createAlias("allParents", "regions.allParents");
			search.createAlias("districts", "allParents.districts");
			search.createAlias("contests", "districts.contests");

			search.addSearchCriteria(CriteriaFactory.eq("contests.code",
					contest.getCode()));

			parentLanguages = (List<LanguageData>) this.persistenceService
					.searchObjects(search, null);

			return parentLanguages;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_LANGUAGES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<LanguageData> getChildrenLanguages(ContestData contest)
			throws UtilitiesException {

		List<LanguageData> parentLanguages;

		try {
			SearchExpression search = new SearchExpression(LanguageData.class);
			search.createAlias("regions", "regions");
			search.createAlias("allChildren", "regions.allChildren");
			search.createAlias("districts", "allChildren.districts");
			search.createAlias("contests", "districts.contests");

			search.addSearchCriteria(CriteriaFactory.eq("contests.code",
					contest.getCode()));

			search.addSearchCriteria(CriteriaFactory.eq("officialLanguage",
					true));

			parentLanguages = (List<LanguageData>) this.persistenceService
					.searchObjects(search, null);

			return parentLanguages;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_LANGUAGES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<ContestLanguagesPairData> getLanguageCombinationsByContest(
			ContestData contest) throws UtilitiesException {

		SearchExpression search = new SearchExpression(
				ContestLanguagesPairData.class);

		search.addSearchCriteria(CriteriaFactory.eq("contest.code",
				contest.getCode()));
		try {

			List<ContestLanguagesPairData> contestLanguagesPairList = (List<ContestLanguagesPairData>) this.persistenceService
					.searchObjects(search, null);

			return contestLanguagesPairList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_LANGUAGES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<ReportTemplateData> getReportsTemplatesByRegion(long regionCode)
			throws UtilitiesException {

		try {
			SearchExpression searchReports = new SearchExpression(
					ReportTemplateData.class);
			searchReports.addSearchCriteria(CriteriaFactory.eq("app",
					ReportTemplateData.Application.PM));

			searchReports.createAlias("regions", "regions");
			searchReports.createAlias("allChildren", "regions.allChildren");

			searchReports.addSearchCriteria(CriteriaFactory.eq(
					"allChildren.code", regionCode));

			searchReports.addOrder(OrderCriteria.asc("regions.category"));

			List<ReportTemplateData> reports = (List<ReportTemplateData>) this.persistenceService
					.searchObjects(searchReports, null);

			return reports;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_REPORTS_TEMPLATES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<ReportTemplateData> getCommonAudioResources()
			throws UtilitiesException {

		try {
			SearchExpression searchReports = new SearchExpression(
					ReportTemplateData.class);
			searchReports.addSearchCriteria(CriteriaFactory.eq("app",
					ReportTemplateData.Application.ECM));

			searchReports.addSearchCriteria(CriteriaFactory.eq("type",
					ReportTemplateData.Type.AUDIO_RESOURCE));

			List<ReportTemplateData> reports = (List<ReportTemplateData>) this.persistenceService
					.searchObjects(searchReports, null);

			return reports;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_REPORTS_TEMPLATES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public Map<String, byte[]> getEMLFiles() throws UtilitiesException {

		File emlFilesDir = getEMLFilesDir();

		if (emlFilesDir.exists()) {

			File[] emlFiles = emlFilesDir.listFiles();
			if (emlFiles.length == 0) {
				logger.info("No EML files available");
				return null;
			}

			FileFilter fileFilter = new FileFilter() {
				@Override
				public boolean accept(File file) {
					return file.isFile();
				}
			};

			emlFiles = emlFilesDir.listFiles(fileFilter);
			HashMap<String, byte[]> emlFilesMap = new HashMap<String, byte[]>();

			try {
				for (int i = 0; i < emlFiles.length; ++i) {
					File emlFile = emlFiles[i];
					byte[] emlBytes = FileUtils.readFileToByteArray(emlFile);
					emlFilesMap.put(emlFile.getName(), emlBytes);
				}

				return emlFilesMap;

			} catch (IOException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				throw new UtilitiesException(e,
						UtilitiesExceptionInfo.PROBLEM_GETTING_EML_FILES,
						StringUtils.defaultString(e.getMessage()));
			}
		}
		return null;
	}

	/**
	 * @inheritDoc
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<LanguageData> getParentLanguages(RegionData region)
			throws UtilitiesException {

		SearchExpression search = new SearchExpression(LanguageData.class);
		try {
			search.createAlias("regions", "regions");
			search.createAlias("allChildren", "regions.allChildren");
			search.addSearchCriteria(CriteriaFactory.eq("allChildren.code",
					region.getCode()));
			search.addSearchCriteria(CriteriaFactory.eq("officialLanguage",
					true));
			List<LanguageData> languageList = (List<LanguageData>) this.persistenceService
					.searchObjects(search, null);

			return languageList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_LANGUAGES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@SuppressWarnings("unchecked")
	@Override
	public ContestClassTranslationsDTO getContestClassTranslations(
			RegionData region, List<LanguageData> languages,
			Map<String, ContestClassData> translations)
			throws UtilitiesException {

		SearchExpression expression = new SearchExpression(ContestData.class);

		try {
			expression.createAlias("district", "district");
			expression.createAlias("regions", "district.regions");
			expression.createAlias("allChildren", "regions.allChildren");
			expression.createAlias("eligibleOptions", "eligibleOptions");

			expression.addSearchCriteria(CriteriaFactory.eq("allChildren.code",
					region.getCode()));

			expression.addSearchCriteria(CriteriaFactory.collectionSizeNotEq(
					"eligibleOptions", 0));

			String[] pathsToInit = { "contestClass" };

			List<ContestData> contestList = (List<ContestData>) this.persistenceService
					.searchObjects(expression, pathsToInit, null);

			List<ContestClassData> contestClassList = new ArrayList<ContestClassData>();
			Map<LanguageData, String> languageContestClassTmp;

			for (ContestData contest : contestList) {
				languageContestClassTmp = new HashMap<LanguageData, String>();

				if (!translations.containsKey(contest.getContestClass()
						.getCustomCode())) {

					for (LanguageData language : languages) {
						ContestClassData contestClassTranslated = translateContestClassName(
								contest.getContestClass(), language);

						if (!languageContestClassTmp.containsKey(language)) {
							languageContestClassTmp.put(language,
									contestClassTranslated
											.getContestClassNameTranslate());
						}
					}

					contest.getContestClass().setTranslationsList(
							languageContestClassTmp);
					contestClassList.add(contest.getContestClass());
					translations.put(contest.getContestClass().getCustomCode(),
							contest.getContestClass());
				} else {
					contestClassList.add(translations.get(contest
							.getContestClass().getCustomCode()));
				}
			}

			ContestClassTranslationsDTO returnDto = new ContestClassTranslationsDTO();
			Collections.sort(contestClassList,
					new ContestClassPrecedenceComparator());
			returnDto.setListContestClass(contestClassList);
			returnDto.setTranslations(translations);

			return returnDto;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(
					e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_CONTEST_CLASS_TRANSLATIONS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<LanguageData> getOfficialLanguages() throws UtilitiesException {

		SearchExpression search = new SearchExpression(LanguageData.class);
		try {
			search.addSearchCriteria(CriteriaFactory.eq("officialLanguage",
					true));

			List<LanguageData> languages = (List<LanguageData>) this.persistenceService
					.searchObjects(search, null);

			return languages;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_LANGUAGES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public List<JasperPrint> generatePasswordReports(LanguageData language,
			List<VotingDeviceData> votingDevices, boolean isVotingDeviceReport,
			boolean printReportWithoutPassword) throws UtilitiesException {

		List<JasperPrint> reportsList = new ArrayList<JasperPrint>();

		try {
			if (!votingDevices.isEmpty()) {

				Map<String, Object> reportParameters = getElectionReportParameters(
						language, false);

				Collection<Map<String, Object>> data = new ArrayList<Map<String, Object>>();

				Collections.sort(votingDevices,
						new Comparator<VotingDeviceData>() {

							@Override
							public int compare(VotingDeviceData o1,
									VotingDeviceData o2) {

								Long s1 = o1.getVotingBooth()
										.getPollingStation().getPollingPlace()
										.getCode();
								Long s2 = o2.getVotingBooth()
										.getPollingStation().getPollingPlace()
										.getCode();
								int sComp = s1.compareTo(s2);

								if (sComp != 0) {
									return sComp;
								} else {
									Integer i1 = o1.getVotingBooth()
											.getPollingStation()
											.getPollingPlace()
											.getPollingPlaceNumber();
									Integer i2 = o2.getVotingBooth()
											.getPollingStation()
											.getPollingPlace()
											.getPollingPlaceNumber();
									return i1.compareTo(i2);
								}
							}
						});

				int devicesCount = 0;
				for (VotingDeviceData votingDevice : votingDevices) {

					PollingStationData pollingStation = votingDevice
							.getVotingBooth().getPollingStation();

					if (pollingStation != null) {

						reportParameters.put("REGION_CODE", pollingStation
								.getPollingPlace().getRegion().getCode());

						Map<String, Object> parametersWithPassword = getVotingDeviceParameters(
								votingDevice, pollingStation,
								isVotingDeviceReport, true);

						boolean isLastElement = (votingDevices.size() == ++devicesCount);

						if (printReportWithoutPassword) {

							parametersWithPassword.put("LAST_ELEMENT", false);
							data.add(parametersWithPassword);

							Map<String, Object> parametersWithoutPassword = getVotingDeviceParameters(
									votingDevice, pollingStation,
									isVotingDeviceReport, false);

							parametersWithoutPassword.put("LAST_ELEMENT",
									isLastElement);

							data.add(parametersWithoutPassword);
						} else {
							parametersWithPassword.put("LAST_ELEMENT",
									isLastElement);
							data.add(parametersWithPassword);

						}
					}
				}

				if (data != null && !data.isEmpty()) {
					JasperPrint print = null;
					String reportTemplateType = "";

					if (isVotingDeviceReport) {
						reportTemplateType = "ecm_pen_reception_";
					} else {
						reportTemplateType = "ecm_pen_recount_";
					}
					print = PrinterAssistant.generatePreview(
							new File(System
									.getProperty("jboss.server.home.dir")
									+ "/reports"), reportTemplateType
									+ language.getLanguageRFC(), data,
							reportParameters);
					reportsList.add(print);
				}
			}
			return reportsList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GENERATING_REPORTS,
					StringUtils.defaultString(e.getMessage()));
		}

	}

	/**
	 * @inheritDoc
	 */
	@Override
	public List<JasperPrint> generateTotalizationReports(LanguageData language,
			Set<ElectoralAuthorityData> electoralAuthorities)
			throws UtilitiesException {

		List<JasperPrint> reportsList = new ArrayList<JasperPrint>();

		try {

			Map<String, Object> reportParameters = getElectionReportParameters(
					language, true);

			Collection<Map<String, Object>> data = new ArrayList<Map<String, Object>>();

			for (ElectoralAuthorityData electoralAuthority : electoralAuthorities) {

				Map<String, Object> entry = new HashMap<String, Object>();

				entry.put("authority_name",
						electoralAuthority.getAuthorityUsername());
				entry.put("city", electoralAuthority.getCity());
				entry.put("ea_username",
						electoralAuthority.getAuthorityUsername());
				entry.put("ea_address", electoralAuthority.getAddress());

				String authorityPassword = getPasswordGrouped(
						electoralAuthority.getAuthorityKey(),
						this.getServerPropertyString("number.of.character.password.group"));

				entry.put("authority_key", authorityPassword.toString());
				data.add(entry);

			}

			if (data != null && !data.isEmpty()) {
				JasperPrint print = PrinterAssistant.generatePreview(new File(
						System.getProperty("jboss.server.home.dir")
								+ "/reports"),
						"ecm_rts_password_" + language.getLanguageRFC(), data,
						reportParameters);

				reportsList.add(print);
			}

			return reportsList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(e,
					UtilitiesExceptionInfo.PROBLEM_GENERATING_REPORTS,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void setDefaultApplicationKey()
			throws ElectoralOrganizationOperationException, BaseException {
		String defaultApplicationKey = getServerPropertyAsString("default.application.key");
		setApplicationKey(defaultApplicationKey);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void generateRandomApplicationKey()
			throws ElectoralOrganizationOperationException, BaseException {
		String applicationKey = ECMServerCommonImpl.generateSymmetricKey();
		setApplicationKey(applicationKey);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public String getApplicationKey()
			throws ElectoralOrganizationOperationException {
		return getActiveElection().getApplicationKey();
	}

	/**
	 * Sets the application key.
	 *
	 * @param applicationKey
	 *            the new application key
	 * @throws ElectoralOrganizationOperationException
	 *             the electoral organization operation exception
	 * @throws BaseException
	 *             the base exception
	 */
	private void setApplicationKey(String applicationKey)
			throws ElectoralOrganizationOperationException, BaseException {

		ElectionData activeElection = getActiveElection();

		activeElection.setApplicationKey(applicationKey);

		this.persistenceService.update(activeElection, null);

	}

	/**
	 * Sets the data source for testing proposes.
	 * 
	 * @param dataSource
	 *            the data source
	 */
	public void setDataSource(DataSource dataSource) {
		this.dataSourceReadOnly = dataSource;
	}

	/**
	 * Sets the instance of IPersistenceServiceLocal for testing proposes.
	 * 
	 * @param persistenceService
	 *            the new instance of IPersistenceServiceLocal
	 */
	public void setPersistenceService(
			IPersistenceServiceLocal persistenceService) {
		this.persistenceService = persistenceService;
	}

	/**
	 * Returns the number of records loaded for each smartmatic file type.
	 * 
	 * @param smttFileType
	 *            smtt file type
	 * 
	 * @param classToCountRecords
	 *            class to count records
	 * 
	 * @return dto that hold entity records quantities
	 * 
	 * @throws BaseException
	 *             signals that a problem occur retrieving entities records
	 */
	@SuppressWarnings("rawtypes")
	protected EntityRecordsQuantitiesDTO getEntityRecordsQuantities(
			SmttFileType smttFileType, Class classToCountRecords)
			throws BaseException {

		SearchExpression search = new SearchExpression(classToCountRecords);

		int recordsAmount = this.persistenceService.countObjects(search, null);

		EntityRecordsQuantitiesDTO entityRecordsQuantities = new EntityRecordsQuantitiesDTO();
		entityRecordsQuantities.setRecordsQuantities(recordsAmount);
		entityRecordsQuantities.setSmttFileType(smttFileType);

		return entityRecordsQuantities;
	}

	/**
	 * Removes the ref object from bulk cache.
	 */
	protected void removeRefObjectFromBulkCache() {
		BulkCaches.getRefObjects().removeAll();
	}

	/**
	 * Gets the report files directory.
	 * 
	 * @return the report files directory
	 */
	protected File getReportFilesDir() {
		File reportFilesDir = new File(
				System.getProperty("jboss.server.home.dir"),
				getServerPropertyAsString("reports.files.folder.name"));
		return reportFilesDir;
	}

	/**
	 * Gets the EML files directory.
	 * 
	 * @return the EML files directory
	 */
	@Override
	public File getEMLFilesDir() {
		File emlFilesDir = new File(
				System.getProperty("jboss.server.home.dir"),
				getServerPropertyAsString("eml.files.folder"));
		return emlFilesDir;
	}

	/**
	 * Gets the active election.
	 * 
	 * @return the active election
	 * 
	 * @throws ElectoralOrganizationOperationException
	 *             signals that an exception has occurred retrieving the active
	 *             election
	 */
	@Override
	public ElectionData getActiveElection()
			throws ElectoralOrganizationOperationException {
		ElectionData activeElection = this.ecmElectoralOrganizationService
				.retrieveActiveElection();
		return activeElection;
	}

	/**
	 * Signs a xml detached.
	 * 
	 * @param electionPkcs12
	 *            the pkcs12 to sign
	 * 
	 * @param electionAliasToSign
	 *            the pkcs12 alias to sign
	 * 
	 * @param electionPasswordToSign
	 *            the pkcs12 password to sign
	 * 
	 * @param filesByUriBase
	 *            the files to sign
	 * 
	 * @param signatureAlgorithm
	 *            the signature algorithm
	 * 
	 * @return the xml signed
	 * 
	 * @throws Exception
	 *             signal that some problem occurs signing the xml
	 */
	protected String signDetachedXML(byte[] electionPkcs12,
			String electionAliasToSign, String electionPasswordToSign,
			Map<String, List<File>> filesByUriBase, String signatureAlgorithm)
			throws Exception {

		String xml = SignXMLUtil.signDetachedXML(electionPkcs12,
				electionAliasToSign, electionPasswordToSign, filesByUriBase,
				signatureAlgorithm);

		return xml;
	}

	/**
	 * Method that giving a BaseGeneralInfo object, find all
	 * MultilingualPropertyAnnotation and create MultiLingualKeyData for each
	 * attribute annotated.
	 * 
	 * @param object
	 *            Object to find the MultilingualPropertyAnnotation
	 * @return A list of MultiLingualKeyData with each
	 *         MultilingualPropertyAnnotation
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private List<MultiLingualKeyData> getMultilingualFields(
			BaseGeneralInfo object) {

		List<MultiLingualKeyData> multiLingualKeyDataList = new ArrayList<MultiLingualKeyData>();

		// Process all methods finding the multilingual ones
		Method[] methods = object.getClass().getMethods();
		Class multilingualAnnotationClass = MultilingualPropertyAnnotation.class;

		for (Method theElement : methods) {
			MultilingualPropertyAnnotation annotation = (MultilingualPropertyAnnotation) theElement
					.getAnnotation(multilingualAnnotationClass);
			if (annotation != null) {
				// Save the multilingual resource
				MultiLingualKeyData multilingualKey = new MultiLingualKeyData();
				multilingualKey.setKey(object.getClass().getName());
				multilingualKey.setField(ReflectHelper
						.getPropertyNameFromGetter(theElement));
				multilingualKey.setForeignKey(object.getCode());

				multiLingualKeyDataList.add(multilingualKey);
			}
		}

		return multiLingualKeyDataList;
	}

	/**
	 * Method that build the key for the String Resource.
	 * 
	 * @param translationKeyData
	 *            object to build the key
	 * 
	 * @return key for the String Resource
	 */
	private String getKey(TranslationKeyData translationKeyData) {
		return translationKeyData.getMultilingualKey().getForeignKey() + "|"
				+ translationKeyData.getMultilingualKey().getKey() + "|"
				+ translationKeyData.getMultilingualKey().getField();
	}

	/**
	 * Method that giving multiLingualKeyData load the translation keys for each
	 * language from data base.
	 * 
	 * @param multiLingualKeyData
	 *            The MultiLingualKeyData to find any translation in data base
	 * 
	 * @param languages
	 *            languages to find the translation
	 * 
	 * @return list of TranslationKeyData with the translation if exist
	 * 
	 * @throws UtilitiesException
	 *             signals that some problem occurred loading the translation
	 *             keys
	 */
	@SuppressWarnings("unchecked")
	private List<TranslationKeyData> getTranslationKeyPerLaguages(
			MultiLingualKeyData multiLingualKeyData, Set<LanguageData> languages)
			throws UtilitiesException {

		SearchExpression search = new SearchExpression(TranslationKeyData.class);

		search.addSearchCriteria(CriteriaFactory.in("electionLanguage",
				languages));

		search.createAlias("multilingualKey", "multilingualKey");

		search.createAlias("electionLanguage", "electionLanguage");

		search.addSearchCriteria(CriteriaFactory.eq("multilingualKey.key",
				multiLingualKeyData.getKey()));

		search.addSearchCriteria(CriteriaFactory.eq("multilingualKey.field",
				multiLingualKeyData.getField()));

		search.addSearchCriteria(CriteriaFactory.eq(
				"electionLanguage.defaultLanguage", false));

		search.addSearchCriteria(CriteriaFactory.eq(
				"multilingualKey.foreignKey",
				multiLingualKeyData.getForeignKey()));

		try {

			List<TranslationKeyData> translationKeyDataList = (List<TranslationKeyData>) this.persistenceService
					.searchObjects(search, null);

			if (translationKeyDataList.size() == 0) {
				translationKeyDataList = new ArrayList<TranslationKeyData>();

				for (Iterator<LanguageData> iterator = languages.iterator(); iterator
						.hasNext();) {
					LanguageData electionLanguageData = iterator.next();

					TranslationKeyData translationKeyData = new TranslationKeyData();
					translationKeyData
							.setElectionLanguage(electionLanguageData);
					translationKeyData.setMultilingualKey(multiLingualKeyData);

					translationKeyDataList.add(translationKeyData);
				}
			} else {
				for (Iterator<LanguageData> iterator = languages.iterator(); iterator
						.hasNext();) {
					LanguageData electionLanguageData = iterator.next();

					if (electionLanguageData.getDefaultLanguage()) {
						TranslationKeyData translationKeyData = new TranslationKeyData();
						translationKeyData
								.setElectionLanguage(electionLanguageData);
						translationKeyData
								.setMultilingualKey(multiLingualKeyData);

						translationKeyDataList.add(translationKeyData);
					}
				}
			}

			return translationKeyDataList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(
					e,
					UtilitiesExceptionInfo.PROBLEM_GETTING_TRASTATION_KEY_PER_LANGUAGE,
					StringUtils.defaultString(e.getMessage()));
		}

	}

	/**
	 * Fills the values for each column in the resultset.
	 * 
	 * @param resultSet
	 *            The result set object returned by the executed query.
	 * @param resultSetMetaData
	 *            The result set metadata.
	 * @param columns
	 *            The columns object.
	 * @throws SQLException
	 *             Throws if there is a failure in the executed query.
	 */
	private void fillColumnValues(ResultSet resultSet,
			ResultSetMetaData resultSetMetaData, ArrayList<String> columns)
			throws SQLException {

		for (int i = 1; i <= resultSetMetaData.getColumnCount(); i++) {

			if (resultSetMetaData.getColumnType(i) == Types.ARRAY) {

				ResultSetMetaData arrayMetaData = resultSet.getArray(i)
						.getResultSet().getMetaData();

				ResultSet arrayResultSet = resultSet.getArray(i).getResultSet();

				while (arrayResultSet.next()) {
					for (int j = 2; j <= arrayMetaData.getColumnCount(); j += 2) {
						columns.add(arrayResultSet.getString(j));
					}
				}
			} else {
				columns.add(resultSet.getString(i));
			}
		}
	}

	/**
	 * Set the columns names for the queryresult object.
	 * 
	 * @param queryResult
	 *            The query result object to set the column names.
	 * @param resultSet
	 *            The result set object returned by the executed query.
	 * @param resultSetMetaData
	 *            The result set metadata.
	 * @throws SQLException
	 *             Throws if there is a failure in the executed query.
	 */
	private void setColumnNames(QueryResultDTO queryResult,
			ResultSet resultSet, ResultSetMetaData resultSetMetaData)
			throws SQLException {

		List<String> columnNames = new ArrayList<String>();

		for (int i = 1; i <= resultSetMetaData.getColumnCount(); i++) {

			if (resultSetMetaData.getColumnType(i) == Types.ARRAY) {

				ResultSetMetaData arrayMetaData = resultSet.getArray(i)
						.getResultSet().getMetaData();

				for (int j = 0; j < arrayMetaData.getColumnCount(); j++) {
					columnNames.add(resultSetMetaData.getColumnName(i));
				}
			} else {
				columnNames.add(resultSetMetaData.getColumnName(i));
			}
		}

		queryResult.setColumnsNames(columnNames);
	}

	/**
	 * Method to delete the data of a specific object
	 * 
	 * @param className
	 *            Name of the class
	 * 
	 * @throws BaseException
	 *             Abstract base class to notify the client that a problem
	 *             occurred
	 */
	private void deleteLanguageKeyForCodiTables(String className)
			throws BaseException {
		this.persistenceService
				.executeUpdateNativeQuery("delete from translation_key where translation_key_code in (select translation_key_code from "
						+ "translation_key where multilingual_key_code in (select multilingual_key_code from multilingual_key where "
						+ "object_class like '%" + className + "'))");

		this.persistenceService
				.executeUpdateNativeQuery("delete from multilingual_key where object_class like '%"
						+ className + "'");
	}

	/**
	 * Method to check if the smartmatic files were loaded into the database.
	 * 
	 * @return True if the CODI were loaded, False in other case.
	 * 
	 * @throws BaseException
	 *             Abstract base class to notify the client that a problem
	 *             occurred
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private boolean checkIfStaticDataIsLoaded() throws BaseException {
		Class[] authorizedClasses = { KeystoreData.class, ElectionData.class,
				CategoryData.class, RegionData.class,
				ProgrammingUnitData.class, DistrictData.class,
				ContestClassData.class, BallotLayoutData.class,
				ContestData.class, ContestGroupData.class, LanguageData.class,
				ElectoralAuthorityData.class, ProfileData.class,
				UserData.class, ReportTemplateData.class };

		for (Class<BaseGeneralInfo> baseGeneralInfoObject : authorizedClasses) {
			if (!checkIfEntityIsLoaded(baseGeneralInfoObject)) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Method to check if the CODI files were loaded into the database.
	 * 
	 * @return True if the CODI were loaded, False in other case.
	 * 
	 * @throws BaseException
	 *             Abstract base class to notify the client that a problem
	 *             occurred
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private boolean checkIfCodiDataIsLoaded() throws BaseException {

		Class[] authorizedClasses = { CandidateData.class, PartyData.class,
				EligibleOptionData.class, EligibleOptionRoleData.class,
				BallotOptionData.class, PollingPlaceData.class,
				PollingStationData.class, VotingDeviceData.class,
				VotingBoothData.class, ElectionData.class };

		for (Class<BaseGeneralInfo> baseGeneralInfoObject : authorizedClasses) {
			if (!checkIfEntityIsLoaded(baseGeneralInfoObject)) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Method that check if the entity table is empty.
	 * 
	 * @param classEntiy
	 *            Entity to check.
	 * @return False if the table is empty or true in other case.
	 * @throws BaseException
	 *             Abstract base class to notify the client that a problem
	 *             occurred
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private boolean checkIfEntityIsLoaded(Class classEntiy)
			throws BaseException {

		int count;

		if (classEntiy.equals(UserData.class)) {
			SearchExpression search = new SearchExpression(UserData.class);
			SearchExpression subQuery = new SearchExpression(UserData.class);

			subQuery.createAlias("profile", "profile");

			subQuery.addSearchCriteria(CriteriaFactory.or(CriteriaFactory
					.not(CriteriaFactory.eq("profile.adminProfile", true)),
					CriteriaFactory.or(CriteriaFactory.isNull("profile"),
							CriteriaFactory.isNotNull("electoralAuthority"))));

			subQuery.setTransformation(TransformationFactory
					.distinctTransformation(TransformationFactory
							.propertyTransformation("code")));

			search.addSearchCriteria(CriteriaFactory.propertyIn("code",
					subQuery));

			count = this.persistenceService.countObjects(search, null);
		} else if (classEntiy.equals(ProfileData.class)) {
			SearchExpression search = new SearchExpression(ProfileData.class);

			search.addSearchCriteria(CriteriaFactory.not(CriteriaFactory.eq(
					"adminProfile", true)));

			search.setTransformation(TransformationFactory
					.distinctTransformation(TransformationFactory
							.propertyTransformation("code")));

			count = this.persistenceService.countObjects(search, null);
		} else {
			SearchExpression search = new SearchExpression(classEntiy);
			TransformationsList trans = new TransformationsList();
			trans.add(TransformationFactory
					.propertyTransformation("customCode"));
			search.setMaxResults(1);

			List<String> objectsCustomCodes = (List<String>) this.persistenceService
					.searchObjects(search, null);
			count = 0;
			if (objectsCustomCodes != null) {
				count = objectsCustomCodes.size();
			}
		}

		if (count > 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Method used to get a server property in boolean.
	 * 
	 * @param key
	 *            The key of the property to search.
	 */
	private String getServerPropertyString(String key) {
		return ApplicationProperties.getValue(key);
	}

	/**
	 * Return the current database connection
	 * 
	 * @return the connection
	 * @throws SQLException
	 *             if an error occurred getting the connection
	 */
	private Connection getConnection() throws SQLException {
		return this.dataSource.getConnection();
	}

	/**
	 * Translates the election name to its given languages, if a merged language
	 * is included it should generate a single name with the translations.
	 * 
	 * @param electionToTranslate
	 *            the election data object
	 * @param language
	 *            the language to translate
	 * @return the election name translated
	 * @throws UtilitiesException
	 *             if an error has occurred translating the election
	 */
	private ElectionData translateElectionName(
			ElectionData electionToTranslate, LanguageData language)
			throws UtilitiesException {

		try {

			String[] languageCustomCodes = StringUtils.split(
					language.getCustomCode(), '|');

			StringBuilder electionName = new StringBuilder();

			for (String languageCustomCode : languageCustomCodes) {

				SearchExpression expression = new SearchExpression(
						TranslationKeyData.class);

				expression.createAlias("multilingualKey", "multilingualKey");
				expression.createAlias("electionLanguage", "electionLanguage");

				expression.addSearchCriteria(CriteriaFactory.eq(
						"multilingualKey.customCode",
						"ELECTIONDATA__ELECTIONNAME_"
								+ electionToTranslate.getCustomCode()));

				expression.addSearchCriteria(CriteriaFactory.eq(
						"electionLanguage.customCode", languageCustomCode));

				TranslationKeyData translation = (TranslationKeyData) this.persistenceService
						.searchObject(expression, null);

				if (translation != null) {
					if (electionName.length() != 0) {
						electionName.append(" / ");
					}
					electionName.append(translation.getTranslationText());
				} else {
					if (electionName.length() != 0) {
						electionName.append(" / ");
					}
					electionName.append(electionToTranslate.getElectionName());
				}

			}

			electionToTranslate.setName(electionName.toString());

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new UtilitiesException(
					e,
					UtilitiesExceptionInfo.PROBLEM_PROCESSING_MULTILINGUAL_OBJECT,
					StringUtils.defaultString(e.getMessage()));
		}

		return electionToTranslate;
	}

	/**
	 * Return a map with the voting device parameters to use in the reports
	 * 
	 * @param votingDevice
	 *            the voting device object reference
	 * @param pollingStation
	 *            the polling station object reference
	 * @param isVotingDeviceReport
	 *            true if is voting device report otherwise false
	 * @param includePassword
	 *            true if the parameters map must include the password
	 * @return the map with the parameter name as a key and parameter object as
	 *         value
	 */
	private Map<String, Object> getVotingDeviceParameters(
			VotingDeviceData votingDevice, PollingStationData pollingStation,
			boolean isVotingDeviceReport, boolean includePassword) {

		Map<String, Object> votingDeviceParameters = new HashMap<String, Object>();

		votingDeviceParameters.put("polling_place_custom_code", pollingStation
				.getPollingPlace().getCustomCode());

		String password = "";

		String alarmCode = "";

		if (isVotingDeviceReport) {
			if (includePassword && votingDevice.getPresidentKey() != null) {

				password = getPasswordGrouped(
						votingDevice.getPresidentKey(),
						this.getServerPropertyString("number.of.character.password.group"));
				alarmCode = votingDevice.getNumericKey1();

			}
			votingDeviceParameters.put("president_key", password);
			votingDeviceParameters.put("alarm_code", alarmCode);
		} else {
			if (includePassword && votingDevice.getRecounterKey() != null) {

				password = getPasswordGrouped(
						votingDevice.getRecounterKey(),
						this.getServerPropertyString("number.of.character.password.group"));
				alarmCode = votingDevice.getNumericKey1();
			}
			votingDeviceParameters.put("recounter_key", password);
			votingDeviceParameters.put("alarm_code", alarmCode);
		}

		votingDeviceParameters.put("voting_device_custom_code",
				votingDevice.getName());

		votingDeviceParameters.put("polling_place_address", pollingStation
				.getPollingPlace().getAddress());

		votingDeviceParameters.put("polling_place_city", pollingStation
				.getPollingPlace().getCity());

		votingDeviceParameters.put("polling_place_zip_code", pollingStation
				.getPollingPlace().getZipCode());

		return votingDeviceParameters;

	}

	/**
	 * Returns a map with the election information parameters to use in the
	 * reports
	 * 
	 * @param language
	 *            the language the parameters must be translated
	 * @param isTotalizationReport
	 *            true if is the totalization report, otherwise false
	 * @return the map with the parameter string as key and the value the
	 *         parameter object
	 * @throws SQLException
	 *             if an error with the database connection
	 * @throws UtilitiesException
	 *             if an error occurred translating the election name
	 * @throws ElectoralOrganizationOperationException
	 *             if an error occurred getting the active election
	 */
	private Map<String, Object> getElectionReportParameters(
			LanguageData language, boolean isTotalizationReport)
			throws SQLException, UtilitiesException,
			ElectoralOrganizationOperationException {

		Connection connection = null;
		try {
			Map<String, Object> reportParameters = new HashMap<String, Object>();

			ElectionData activeElection = translateElectionName(
					this.getActiveElection(), language);

			connection = getConnection();

			String reportCode;
			if (isTotalizationReport) {
				reportCode = this
						.getServerPropertyString("code.totalization.report");
			} else {
				reportCode = this
						.getServerPropertyString("code.president.machine.report");
			}
			reportParameters.put(JRParameter.REPORT_CONNECTION, connection);
			reportParameters.put("SUBREPORT_DIR",
					System.getProperty("jboss.server.home.dir") + "/reports/");
			reportParameters.put("ELECTION_DATE",
					activeElection.getElectionOpeningDate());

			reportParameters.put("DEFAULT_LANGUAGE",
					language.getDefaultLanguage());
			reportParameters.put("LANGUAGE_CODE", language.getCode());
			reportParameters.put("REPORT_CODE", reportCode);
			reportParameters.put("ELECTION_NAME",
					StringUtils.replace(activeElection.getName(), "/ ", "/\n"));

			return reportParameters;

		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException e) {
					logger.error(e, StringUtils.defaultString(e.getMessage()));
					throw e;
				}
			}
		}
	}

	/**
	 * Returns the given password grouped by the required number of characters
	 * 
	 * @param password
	 *            the password to group
	 * @param charactersToGroup
	 *            the number of characters to group the password
	 * @return the password grouped
	 */
	private String getPasswordGrouped(String password, String charactersToGroup) {

		StringBuilder passwordGrouped = new StringBuilder();

		String[] characters = password.split("(?<=\\G.{" + charactersToGroup
				+ "})");

		boolean continueAppending = false;

		for (String character : characters) {
			if (continueAppending) {
				passwordGrouped.append('-');
			}
			passwordGrouped.append(character);
			continueAppending = true;
		}

		return passwordGrouped.toString();
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public String getECMProperty(String key) {
		return ECMApplicationProperties.getValue(key);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public byte[] getVoiceFiles() throws UtilitiesException {
		File voicesFilesDir = getVoiceFolder();

		if (voicesFilesDir.exists()) {

			File[] voicesFiles = voicesFilesDir.listFiles();
			if (voicesFiles.length == 0) {
				logger.info("No voices files available");
				return null;
			} else if (voicesFiles.length > 1) {
				logger.info("Too much voices files");
				return null;
			}

			FileFilter fileFilter = new FileFilter() {
				@Override
				public boolean accept(File file) {
					return file.isFile();
				}
			};

			voicesFiles = voicesFilesDir.listFiles(fileFilter);

			try {
				byte[] voiceBytes = null;
				for (int i = 0; i < voicesFiles.length; ++i) {
					File voiceFile = voicesFiles[i];

					voiceBytes = FileUtils.readFileToByteArray(voiceFile);
				}

				return voiceBytes;

			} catch (IOException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				throw new UtilitiesException(e,
						UtilitiesExceptionInfo.PROBLEM_GETTING_VOICE_FILES,
						StringUtils.defaultString(e.getMessage()));
			}
		}
		return null;
	}


    /**
     * {@inheritDoc}
     */
    @Override
    public void truncateDeviceParametersTable() throws BulkDataLoadException {
        final String query = "truncate table device_parameter cascade";
        try {
            this.persistenceService.executeUpdateNativeQuery(query);
            this.removeRefObjectFromBulkCache();

        } catch (Exception ex) {
            logger.error(ex, StringUtils.defaultString( ex.getMessage() ));
            throw new BulkDataLoadException(
                ex,
                UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS,
                StringUtils.defaultString( ex.getMessage() )
            );
        }
    }
}