/**
 * ElectoralPhysicalLocationsRegistryException.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.lang;

import com.smartmatic.waffle.ecm.core.lang.ECMBaseException;
import com.smartmatic.waffle.ecm.core.lang.IECMExceptionInfo;

/**
 * Class to handle any exception that may occur executing an electoral physical
 * locations registry operation.
 */
public class ElectoralPhysicalLocationsRegistryException extends
		ECMBaseException {

	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 */
	private static final long serialVersionUID = 1042224267124632834L;

	/**
	 * Enum that contains all the possible reasons because the
	 * ElectoralPhysicalLocationsRegistryException can be thrown.
	 */
	public enum ElectoralPhysicalLocationsRegistryExceptionInfo implements
			IECMExceptionInfo {

		RETRIEVING_VOTING_DEVICES_AMOUNT_FAILURE(
				"retrieving.voting.devices.amount.failure"), SETTING_SECURITY_COMPONENTS_TO_VOTING_DEVICES_FAILURE(
				"setting.security.components.to.voting.devices.failure"), GENERATING_ASYMMETRIC_KEY_FOR_VOTING_DEVICE_FAILURE(
				"generating.asymmetric.key.for.voting.device.failure"), RETRIEVING_VOTING_DEVICES_SECURITY_COMPONENTS_FAILURE(
				"retrieving.voting.devices.security.components.failure"), COPYING_VOTING_DEVICES_SECURITY_COMPONENTS_ZIP_FILE_TO_FINAL_DESTINATION_FAILURE(
				"copying.voting.devices.security.components.zip.file.to.final.destination.failure"), COMPRESSING_VOTING_DEVICES_SECURITY_COMPONENTS_FILE_FAILURE(
				"compressing.voting.devices.security.components.file.failure"), EXPORTING_VOTING_DEVICES_SECURITY_COMPONENTS_FILE_FAILURE(
				"compressing.voting.devices.security.components.zip.file.failure"), DELETING_VOTING_DEVICES_SECURITY_COMPONENTS_FAILURE(
				"deleting.voting.devices.security.components.failure"), SEARCHING_SECURITY_COMPONENTS_FAILURE(
				"searching.security.components.failure"), GETTING_VOTING_DEVICES_BY_REGION_FAILURE(
				"getting.voting.devices.by.region.failure");

		/**
		 * Variable to hold the exception message.
		 */
		private String exceptionMessage;

		/**
		 * Variable to hold the bundle key to show a user understandable
		 * message.
		 */
		private String bundleKey;

		/**
		 * Gets the variable to hold the exception message.
		 * 
		 * @return the variable to hold the exception message
		 * @inheritDoc
		 */
		@Override
		public String getExceptionMessage() {
			return this.exceptionMessage;
		}

		/**
		 * Gets the variable to hold the bundle key to show a user
		 * understandable message.
		 * 
		 * @return the variable to hold the bundle key to show a user
		 *         understandable message
		 * @inheritDoc
		 */
		@Override
		public String getBundleKey() {
			return this.bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 */
		private ElectoralPhysicalLocationsRegistryExceptionInfo(String bundleKey) {
			this.bundleKey = bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 * @param exceptionMessage
		 *            Exception message
		 */
		private ElectoralPhysicalLocationsRegistryExceptionInfo(
				String bundleKey, String exceptionMessage) {
			this.bundleKey = bundleKey;
			this.exceptionMessage = exceptionMessage;
		}

	}

	/**
	 * Instantiates an ElectoralPhysicalLocationsRegistryException.
	 * 
	 * @param throwable
	 *            a nested exception
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public ElectoralPhysicalLocationsRegistryException(Throwable throwable,
			ElectoralPhysicalLocationsRegistryExceptionInfo exceptionInfo) {
		super(throwable, exceptionInfo);
	}

	/**
	 * Instantiates an ElectoralPhysicalLocationsRegistryException.
	 * 
	 * @param throwable
	 *            a nested exception
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param exceptionMessage
	 *            Exception message of nested exception
	 */
	public ElectoralPhysicalLocationsRegistryException(Throwable throwable,
			ElectoralPhysicalLocationsRegistryExceptionInfo exceptionInfo,
			String exceptionMessage) {
		super(throwable, exceptionInfo, exceptionMessage);
	}

	/**
	 * Instantiates an ElectoralPhysicalLocationsRegistryException.
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public ElectoralPhysicalLocationsRegistryException(
			ElectoralPhysicalLocationsRegistryExceptionInfo exceptionInfo) {
		super(exceptionInfo);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public String getBundleKey() {
		return this.getClass().getSimpleName() + "."
				+ this.exceptionInfo.getBundleKey();
	}
}
