/**
 * KeyGenerationException.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.lang;

import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.lang.IExceptionInfo;

/**
 * Manage exceptions in the keyGeneration process.
 */
public class KeyGenerationException extends BaseException {

	/** Constant for serialVersionUID. */
	private static final long serialVersionUID = 1919387363152878220L;

	/**
	 * Enum that contains all the possible reasons because the
	 * KeyGenerationException can be thrown.
	 * 
	 * @version $Revision: 1.2 $ ($Date: 2011-04-06 04:24:17 $)
	 * @author Ronald Arias (ronald.arias@smartmatic.com)
	 */
	public enum KeyExceptionInfo implements IExceptionInfo {

		/** The problem updating devices. */
		PROBLEM_UPDATING_DEVICES("PROBLEM_UPDATING_DEVICES"),
		/** The problem updating authorities. */
		PROBLEM_UPDATING_AUTHORITIES("PROBLEM_UPDATING_AUTHORITIES"),
		/** The problem getting devices. */
		PROBLEM_GETTING_DEVICES("PROBLEM_GETTING_DEVICES"),
		/** The problem getting authorities. */
		PROBLEM_GETTING_AUTHORITIES("PROBLEM_GETTING_AUTHORITIES"),
		/** The problem generating certificate. */
		PROBLEM_GENERATING_CERTIFICATE("PROBLEM_GENERATING_CERTIFICATE"),
		/** The problem storing certificate. */
		PROBLEM_STORING_CERTIFICATE("PROBLEM_GENERATING_CERTIFICATE"),
		/** The problem generating protectionkey. */
		PROBLEM_GENERATING_PROTECTIONKEY("PROBLEM_GENERATING_PROTECTIONKEY"),
		/** The problem generating presidentkey. */
		PROBLEM_GENERATING_PRESIDENTKEY("PROBLEM_GENERATING_PRESIDENTKEY"),
		/** The problem generating transmissionkey. */
		PROBLEM_GENERATING_TRANSMISSIONKEY("PROBLEM_GENERATING_TRANSMISSIONKEY"),
		/** The problem generating numerickeys. */
		PROBLEM_GENERATING_NUMERICKEYS("PROBLEM_GENERATING_NUMERICKEYS"),
		/** The problem getting programming units. */
		PROBLEM_GETTING_PROGRAMMING_UNITS("PROBLEM_GETTING_PROGRAMMING_UNITS"),
		/** The problem updating programming units. */
		PROBLEM_UPDATING_PROGRAMMING_UNITS("PROBLEM_UPDATING_PROGRAMMING_UNITS");
		;
		/**
		 * <p>
		 * Each enumerator's resource bundle's key message.
		 * </p>
		 */

		private String message;

		/**
		 * Getter of <code>message</code> property.
		 * 
		 * @return <code>message</code> property value.
		 * @see #message
		 * 
		 * @author Ronald Arias (ronald.arias@smartmatic.com)
		 */

		@Override
		public String getMessage() {
			return this.message;
		}

		/**
		 * Constructor.
		 * 
		 * @param msg
		 *            the resource bundle's key message
		 * @author Ronald Arias (ronald.arias@smartmatic.com)
		 */

		private KeyExceptionInfo(String msg) {
			this.message = this.getClass().getName() + "." + msg;
		}
	}

	/**
	 * Instantiates a new key generation exception.
	 * 
	 * @param exceptionInfo
	 *            the exception info
	 */
	public KeyGenerationException(IExceptionInfo exceptionInfo) {
		super(exceptionInfo);
	}

	/**
	 * Instantiates a new key generation exception.
	 * 
	 * @param exceptionInfo
	 *            the exception info
	 * @param e
	 *            the e
	 */
	public KeyGenerationException(IExceptionInfo exceptionInfo, Throwable e) {
		super(exceptionInfo, e);
	}

}
