/**
 * MasterUSBGenerationException.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.lang;

import com.smartmatic.waffle.ecm.core.lang.ECMBaseException;
import com.smartmatic.waffle.ecm.core.lang.IECMExceptionInfo;

/**
 * Class to handle any exception that may occur generation Master USB
 */
public class MasterUSBGenerationException extends ECMBaseException {

	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 */
	private static final long serialVersionUID = 1042224267124632834L;

	/**
	 * Enum that contains all the possible reasons because the
	 * MasterUSBGenerationException can be thrown.
	 */
	public enum MasterUSBGenerationExceptionInfo implements IECMExceptionInfo {

		PROBLEM_RETRIEVING_ACTIVE_CONTESTS_FOR_ELECTION(
				"problem.retrieving.contests.for.election"), PROBLEM_GENERATING_ELECTION_FOR_CONTEST(
				"problem.generating.election.for.contest"), PROBLEM_GENERATING_VOTING_DEVICE_FOR_REGION(
				"problem.generating.voting.device.for.region"), PROBLEM_GETTING_LANGUAGES(
				"problem.getting.languages");

		/**
		 * Variable to hold the exception message.
		 */
		private String exceptionMessage;

		/**
		 * Variable to hold the bundle key to show a user understandable
		 * message.
		 */
		private String bundleKey;

		/**
		 * Gets the variable to hold the exception message.
		 * 
		 * @return the variable to hold the exception message
		 * @inheritDoc
		 */
		@Override
		public String getExceptionMessage() {
			return this.exceptionMessage;
		}

		/**
		 * Gets the variable to hold the bundle key to show a user
		 * understandable message.
		 * 
		 * @return the variable to hold the bundle key to show a user
		 *         understandable message
		 * @inheritDoc
		 */
		@Override
		public String getBundleKey() {
			return this.bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 */
		private MasterUSBGenerationExceptionInfo(String bundleKey) {
			this.bundleKey = bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 * @param exceptionMessage
		 *            Exception message
		 */
		private MasterUSBGenerationExceptionInfo(String bundleKey,
				String exceptionMessage) {
			this.bundleKey = bundleKey;
			this.exceptionMessage = exceptionMessage;
		}

	}

	/**
	 * Instantiates an MasterUSBGenerationException.
	 * 
	 * @param throwable
	 *            a nested exception
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public MasterUSBGenerationException(Throwable throwable,
			MasterUSBGenerationExceptionInfo exceptionInfo) {
		super(throwable, exceptionInfo);
	}

	/**
	 * Instantiates an MasterUSBGenerationException.
	 * 
	 * @param throwable
	 *            a nested exception
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param exceptionMessage
	 *            Exception message of nested exception
	 */
	public MasterUSBGenerationException(Throwable throwable,
			MasterUSBGenerationExceptionInfo exceptionInfo,
			String exceptionMessage) {
		super(throwable, exceptionInfo, exceptionMessage);
	}

	/**
	 * Instantiates an MasterUSBGenerationException.
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 */
	public MasterUSBGenerationException(
			MasterUSBGenerationExceptionInfo exceptionInfo) {
		super(exceptionInfo);
	}

	/*
	 * @inheritDoc
	 */
	@Override
	public String getBundleKey() {
		return this.getClass().getSimpleName() + "."
				+ this.exceptionInfo.getBundleKey();
	}
}
