package com.smartmatic.waffle.ecm.service.lang;

import com.smartmatic.waffle.ecm.core.lang.ECMBaseException;
import com.smartmatic.waffle.ecm.core.lang.IECMExceptionInfo;

public class NominationException extends ECMBaseException {

	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 */
	private static final long serialVersionUID = 5105195361127774884L;

	public enum NominationExceptionInfo implements IECMExceptionInfo {
		RETRIEVING_CANDIDATES_FAILURE("retrieving.candidates.failure"), RETRIEVING_CANDIDATES_AMOUNT_FAILURE(
				"retrieving.candidates.amount.failure"), RETRIEVING_PARTIES_AMOUNT_FAILURE(
				"retrieving.parties.amount.failure"), RETRIEVING_PARTIES_FAILURE(
				"retrieving.parties.failure"), UPDATING_PARTY_FAILURE(
				"updating.party.failure"), UPDATING_ELIGIBLE_OPTION_FAILURE(
				"updating.eligible.option.failure"), RETRIEVING_CONTESTS_FAILURE(
				"retrieving.contest.failure"), GETTING_BALLOT_OPTIONS_BY_ELIGIBLE_OPTION_FAILURE(
				"getting.ballot.options.by.eligible.option.failure"), GETTING_ELIGIBLE_OPTION_ROLE_BY_ELIGIBLE_OPTION_FAILURE(
				"getting.eligible.option.role.by.eligible.option.failure"), GETTING_ELIGIBLE_OPTION_BY_CONTEST_FAILURE(
				"getting.eligible.option.by.contest.failure");

		/**
		 * Variable to hold the exception message.
		 */
		private String exceptionMessage;

		/**
		 * Variable to hold the bundle key to show a user understandable
		 * message.
		 */
		private String bundleKey;

		/**
		 * Gets the variable to hold the exception message.
		 * 
		 * @return the variable to hold the exception message
		 * @inheritDoc
		 */
		@Override
		public String getExceptionMessage() {
			return this.exceptionMessage;
		}

		/**
		 * Gets the variable to hold the bundle key to show a user
		 * understandable message.
		 * 
		 * @return the variable to hold the bundle key to show a user
		 *         understandable message
		 * @inheritDoc
		 */
		@Override
		public String getBundleKey() {
			return this.bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 */
		private NominationExceptionInfo(String bundleKey) {
			this.bundleKey = bundleKey;
		}

		/**
		 * Constructor.
		 * 
		 * @param bundleKey
		 *            Resource bundle's key message
		 * @param exceptionMessage
		 *            Exception message
		 */
		private NominationExceptionInfo(String bundleKey,
				String exceptionMessage) {
			this.bundleKey = bundleKey;
			this.exceptionMessage = exceptionMessage;
		}

	}

	/**
	 * Constructor
	 * 
	 * @param exceptionInfo
	 */
	public NominationException(IECMExceptionInfo exceptionInfo) {
		super(exceptionInfo);
	}

	/**
	 * Constructor
	 * 
	 * @param throwable
	 * @param exceptionInfo
	 * @param exceptionMessage
	 */
	public NominationException(Throwable throwable,
			IECMExceptionInfo exceptionInfo, String exceptionMessage) {
		super(throwable, exceptionInfo, exceptionMessage);
	}

	/**
	 * Constructor
	 * 
	 * @param throwable
	 * @param exceptionInfo
	 */
	public NominationException(Throwable throwable,
			IECMExceptionInfo exceptionInfo) {
		super(throwable, exceptionInfo);
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public String getBundleKey() {
		return this.getClass().getSimpleName() + "."
				+ this.exceptionInfo.getBundleKey();
	}

}
