/**
 * IECMBulkDataLoadService.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.spec;

import javax.ejb.Remote;

import com.smartmatic.waffle.ecm.bulk.service.lang.BulkLoadException;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.EMLFileType;
import com.smartmatic.waffle.ecm.data.transfer.type.SmttFileType;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;

/**
 * Remote interface which establishes the methods contract that must be
 * implemented by <code>ECMBulkDataLoadServiceImpl</code> class.
 */
@Remote
public interface IECMBulkDataLoadService {
	/** Service name Constant. */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.ecm.service.spec.IECMBulkDataLoadService";

	/**
	 * Executes bulk upload of smartmatic files into database.
	 * 
	 * @param fileToUploadName
	 *            Name of file to upload
	 * @param fileToUploadContent
	 *            Content of file to upload
	 * @param fileType
	 *            Type of file to upload
	 * @param resourceFileContent
	 *            Content of resources needed by file to upload
	 * 
	 * @return List of DTO with the result of bulk upload generation.
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred executing bulk upload
	 */
	public BulkResultDTO uploadSmttFileIntoDataBase(String fileToUploadName,
			byte[] fileToUploadContent, SmttFileType fileType,
			byte[] resourceFileContent) throws BulkDataLoadException;

	/**
	 * Executes bulk upload of codi and csv sites files into database.
	 * 
	 * @param fileToUploadName
	 *            Name of file to upload
	 * @param fileToUploadContent
	 *            Content of file to upload
	 * @param emlFileType
	 *            Type of file to upload
	 * @param resourceFileContent
	 *            Content of resources needed by file to upload
	 * 
	 * @return List of DTO with the result of bulk upload generation.
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred executing bulk upload
	 */
	public BulkResultDTO uploadEMLFilesIntoDataBase(String fileToUploadName,
			byte[] fileToUploadContent, EMLFileType emlFileType,
			byte[] resourceFileContent) throws BulkDataLoadException;

	/**
	 * Validate that the language of the regions with audio voting enabled is
	 * configured correctly.
	 * 
	 * @throws BulkLoadException
	 *             signals that at least a region does not have perfectly
	 *             configured it language.
	 */
	public void checkConfiguredRegionLaguages() throws BulkDataLoadException;
}
