/**
 * IECMUtilitiesService.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.spec;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remote;

import net.sf.jasperreports.engine.JasperPrint;

import com.smartmatic.waffle.domain.IBaseDataObject;
import com.smartmatic.waffle.domain.authority.ElectoralAuthorityData;
import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.general.BaseGeneralInfo;
import com.smartmatic.waffle.domain.general.ReportTemplateData;
import com.smartmatic.waffle.domain.i18n.ContestLanguagesPairData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.data.transfer.objects.ContestClassTranslationsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.EntityRecordsQuantitiesDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.QueryResultDTO;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.lang.BaseException;

@Remote
public interface IECMUtilitiesService {

	/** SERVICE_NAME Constant. */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesService";

	/**
	 * Executes a SQL query and return the result data.
	 * 
	 * @param sqlQuery
	 * @return The query result object.
	 * @throws UtilitiesException
	 *             Throw in case a failure occurs during the query execution.
	 */
	QueryResultDTO executeQuery(String sqlQuery) throws UtilitiesException;

	/**
	 * Gets the amount of records stored in database for each entity type.
	 * 
	 * @return List with the amount of records stored in database for each
	 *         entities
	 * 
	 * @throws BulkDataLoadException
	 *             signals that a problem occur retrieving entities records
	 */
	List<EntityRecordsQuantitiesDTO> getEntityRecordsQuantities()
			throws BulkDataLoadException;

	/**
	 * Retrieves the number of multilingual keys loaded in database.
	 * 
	 * @return number of multilingual keys loaded in database
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred retrieving the
	 *             multilingual keys amount
	 */
	int retrieveMultilingualKeysAmount() throws BulkDataLoadException;

	/**
	 * Method to get the multilingual keys for each language needed to export
	 * the translations file.
	 * 
	 * @param pageNumber
	 *            Current page number
	 * @param pageSize
	 *            Page size
	 * 
	 * @return list of strings, each holding the lines used to export the
	 *         translations file
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred retrieving the
	 *             multilingual keys to export the translations file
	 */
	List<String> getTranslationkeysToDownload(int pageNumber, int pageSize)
			throws BulkDataLoadException;

	/**
	 * Method to clear all records of translation_key, multilingual_key,
	 * region_election_language and election_language tables.
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncateLanguageTables() throws BulkDataLoadException;

	/**
	 * Method to clear all records of region_contest_group,
	 * contest_group_contest, contest_group, contest, contest_class,
	 * region_district, district, ballot_layout, candidate and party
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncateNominationTables() throws BulkDataLoadException;

	/**
	 * Method to clear all records of user, profile_function, profile and
	 * electoral_authority
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncateUserTables() throws BulkDataLoadException;

	/**
	 * Method to clear all records of voting_booth, voting_device,
	 * polling_station and polling_place
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncatePhysicalLocationsTables() throws BulkDataLoadException;

	/**
	 * Method to clear all records of category_path, category_parent,
	 * region_programming_unit, programming_unit, region_all_parent,
	 * region_parent, region, category, path and election
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncateDemographyTables() throws BulkDataLoadException;

	/**
	 * Method to clear all records of report_template_region and
	 * report_template. This method also delete the report templated saved in
	 * filesystem
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncateReportTemplatesTables() throws BulkDataLoadException;

	/**
	 * Updates an instance in database.
	 * 
	 * @param baseDataObject
	 *            the IBaseDataObject to update
	 * 
	 * @return the IBaseDataObject updated
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred updating the
	 *             IBaseDataObject object in the database
	 */
	IBaseDataObject updateObject(IBaseDataObject baseDataObject)
			throws BulkDataLoadException;

	/**
	 * Method to clear all records of logical_voting_group_voting_booth
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncateLogicalVotingGroup() throws BulkDataLoadException;

	/**
	 * Method to clear all records of ballot_option and eligible_option_role
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncateEligibleOption() throws BulkDataLoadException;

	/**
	 * Method to clear all records of eligible_option, party and candidate
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing data for some
	 *             table
	 */
	void truncateCodiNominationTables() throws BulkDataLoadException;

	/**
	 * Method to remove codi files in server side
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing codi files
	 */
	void deleteEMLFiles() throws BulkDataLoadException;

	/**
	 * Method to remove voice files in server side
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing codi files
	 */
	void deleteVoiceFiles() throws BulkDataLoadException;

	/**
	 * Method to clear all records of contest_languages_pair
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing codi files
	 */
	void truncateContestLanguagesPair() throws BulkDataLoadException;

	/**
	 * Method to remove the report files in server side
	 * 
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred removing the report
	 *             files
	 */
	void deleteReportFiles() throws BulkDataLoadException;

	/**
	 * Retrieves a <code>ContestData</code> object given the custom code
	 * 
	 * @param customCode
	 *            custom code to retrieve the object
	 * 
	 * @return the object corresponding the customCode parameter
	 * 
	 * @throws UtilitiesException
	 *             signals that an exception has occurred retrieving the object
	 */
	public ContestData getItemByCustomCode(String customCode, Class clazz)
			throws UtilitiesException;

	/**
	 * Check if security components for voting devices are already generated.
	 * 
	 * @return True is there are security components information generated
	 *         otherwise false
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             signals that an exception has occurred searching security
	 *             components
	 */
	public boolean checkIfSecurityComponentsAreGenerated()
			throws ElectoralPhysicalLocationsRegistryException;

	/**
	 * Checks that all the tables needed to perform the ECM functionalities are
	 * loaded
	 * 
	 * @return <code>false</code> if none of the needed database tables is
	 *         empty,<code>false</code> if there is at least one empty table
	 * 
	 * @throws BaseException
	 *             signals that an exception has occurred counting data elements
	 */
	public abstract boolean checkIfDataIsLoaded() throws BaseException;

	/**
	 * Method used to get a boolean property from config-properties.xml.
	 * 
	 * @param key
	 *            The key of the property to search.
	 * 
	 * @return The boolean value of the property with key:<code>key</code>
	 */
	public boolean getServerPropertyAsBoolean(String key);

	/**
	 * Method used to get a string property from config-properties.xml.
	 * 
	 * @param key
	 *            The key of the property to search.
	 * 
	 * @return The string value of the property with key:<code>key</code>
	 */
	public String getServerPropertyAsString(String key);

	/**
	 * Method used to translate a contest name to an available election
	 * language.
	 * 
	 * @param contest
	 *            The contest to translate
	 * @param language
	 *            The language the given contest must be translated
	 * 
	 * @return The contest data with the translated name
	 * 
	 * @throws UtilitiesException
	 *             A signal that an error occurred translating the contest.
	 */
	ContestData translateContestName(ContestData contest, LanguageData language)
			throws UtilitiesException;

	/**
	 * Method used to translate a district name to an available election
	 * language.
	 * 
	 * @param district
	 *            The district to translate
	 * @param language
	 *            The language the given contest must be translated
	 * 
	 * @return The district data with the translated name
	 * 
	 * @throws UtilitiesException
	 *             A signal that an error occurred translating the district.
	 */
	DistrictData translateDistrictName(DistrictData district,
			LanguageData language) throws UtilitiesException;

	/**
	 * Method used to translate a contest college to an available election
	 * language.
	 * 
	 * @param contest
	 *            The contest to translate
	 * @param language
	 *            The language the given contest must be translated
	 * @return The contest data with the translated college
	 * 
	 * @throws UtilitiesException
	 *             A signal that an error occurred translating the contest.
	 */
	ContestData translateCollege(ContestData contest, LanguageData language)
			throws UtilitiesException;

	/**
	 * Retrieves a map of resources of translatable components for Master USB
	 * generation. The object is translatable if some attribute have @MultilingualPropertyAnnotation
	 * eg the attribute name for ContestData is translatable. If, before running
	 * the method contest.getName() = Parlement flamand Antwerpen, then the
	 * value of name is changed by
	 * com.smartmatic.waffle.domain.contest.ContestData and in the map of
	 * languages a registry NL={6|ContestData|name=Vlaams Parlement} is added
	 * and in the default map we add the default value
	 * (DEFAULT={6|ContestData|name=Parlement flamand Antwerpen}). Maps for
	 * language NL and DEFAULT are nested maps.
	 * 
	 * @param languagesToTranslate
	 *            a set of languages in which will be translated the object. eg
	 *            NL (Nederlands)
	 * 
	 * @param currentResourcesLanguages
	 *            current map with previous objects already translated eg
	 *            {NON-NL ={6|ContestData|name =Vlaams Parlement}, DEFAULT={6|
	 *            .ContestData|name=Parlement flamand Antwerpen}}
	 * 
	 * @param objectToTranslate
	 *            the object to translate
	 * 
	 * @return map with object already translated plus the object to translate
	 *         eg {NON-NL={6|ContestData| name=Vlaams Parlement},
	 *         NL={6|ContestData|name=Vlaams Parlement},
	 *         DEFAULT={6|ContestData|name=Parlement flamand Antwerpen}}
	 * 
	 * @throws UtilitiesException
	 *             signals that an error occurred creating the resource.
	 */
	Map<String, Map<String, String>> processMultilingualObject(
			Set<LanguageData> languagesToTranslate,
			Map<String, Map<String, String>> currentResourcesLanguages,
			BaseGeneralInfo objectToTranslate) throws UtilitiesException;

	/**
	 * Method that return from all the parent regions of the contest the list of
	 * languages
	 * 
	 * @param contest
	 *            the contest to search its parent languages
	 * @return a list of languages
	 * @throws UtilitiesException
	 *             signal that an error occurred getting the parent languages
	 */
	List<LanguageData> getParentLanguages(ContestData contest)
			throws UtilitiesException;

	/**
	 * Method that return from all the children regions of the contest the list
	 * of languages
	 * 
	 * @param contest
	 *            the contest to search its children languages
	 * @return a list of languages
	 * @throws UtilitiesException
	 *             signal that an error occurred getting the children languages
	 */
	List<LanguageData> getChildrenLanguages(ContestData contest)
			throws UtilitiesException;

	/**
	 * Returns a list of contest language pair objects for a given contest
	 * 
	 * @param contest
	 *            the contest to search its languages combinations
	 * @return a list of combination objects
	 * @throws UtilitiesException
	 *             signal that an error occurred getting the language
	 *             combinations
	 */
	List<ContestLanguagesPairData> getLanguageCombinationsByContest(
			ContestData contest) throws UtilitiesException;

	/**
	 * Method used to translate a contest class name to an available election
	 * language.
	 * 
	 * @param contestClass
	 *            The contestClass to translate
	 * @param language
	 *            The language the given contest must be translated
	 * 
	 * @return The contest class data with the translated name
	 * 
	 * @throws UtilitiesException
	 *             A signal that an error occurred translating the contest
	 *             class.
	 */
	ContestClassData translateContestClassName(ContestClassData contestClass,
			LanguageData language) throws UtilitiesException;

	/**
	 * Return a list of report templates used by the PM/VM
	 * 
	 * @param regionCode
	 *            the region to get the reports
	 * @return a list of report template data
	 * @throws UtilitiesException
	 *             if an error has occurred getting the templates
	 */
	List<ReportTemplateData> getReportsTemplatesByRegion(long regionCode)
			throws UtilitiesException;

	/**
	 * Gets the EML files from the server into a hashMap.
	 * 
	 * @return A HashMap with the inputStreams
	 * @throws UtilitiesException
	 *             an error occurred getting the EML files
	 * @throws IOException
	 *             signals that an I/O exception of some sort has occurred.
	 */
	Map<String, byte[]> getEMLFiles() throws UtilitiesException;

	/**
	 * From a region this method return the parent languages
	 * 
	 * @param region
	 *            the region to get its parent languages
	 * @return the list of languages
	 * @throws UtilitiesException
	 *             if an error has occurred getting the languages
	 */
	List<LanguageData> getParentLanguages(RegionData region)
			throws UtilitiesException;

	/**
	 * Return an object that contains the contest class translations for a
	 * region
	 * 
	 * @param region
	 * @param languages
	 *            the language list to translate the contest classes
	 * @param translations
	 *            the translations object reference when iterating multiple
	 *            regions
	 * @return the contest class translation object
	 * @throws UtilitiesException
	 *             an error occurred translating the contest classes
	 * 
	 */
	ContestClassTranslationsDTO getContestClassTranslations(RegionData region,
			List<LanguageData> languages,
			Map<String, ContestClassData> translations)
			throws UtilitiesException;

	/**
	 * Return a list of the official languages loaded
	 * 
	 * @return the list of languages
	 * @throws UtilitiesException
	 *             if an error occurred getting the languages
	 */
	List<LanguageData> getOfficialLanguages() throws UtilitiesException;

	/**
	 * Generates the reports for voting devices and recount machines.
	 * 
	 * @param language
	 *            the language for the report
	 * @param votingDevices
	 *            the list of voting devices
	 * @param isVotingDeviceReport
	 *            true if the reports if for voting devices, otherwise false
	 * @param printReportWithoutPassword
	 *            true if the reports with password should be printed, otherwise
	 *            false
	 * @return the list of generated reports
	 * @throws UtilitiesException
	 *             if an error has occurred generating the reports
	 */
	List<JasperPrint> generatePasswordReports(LanguageData language,
			List<VotingDeviceData> votingDevices, boolean isVotingDeviceReport,
			boolean printReportWithoutPassword) throws UtilitiesException;

	/**
	 * Generates the reports for totalization machines
	 * 
	 * @param language
	 *            the language for the report
	 * @param electoralAuthorities
	 *            the set of electoral authorities
	 * @return the list of generated reports
	 * @throws UtilitiesException
	 *             if an error has occurred generating the reports
	 * 
	 */
	List<JasperPrint> generateTotalizationReports(LanguageData language,
			Set<ElectoralAuthorityData> electoralAuthorities)
			throws UtilitiesException;

	/**
	 * Sets the default application key.
	 *
	 * @throws ElectoralOrganizationOperationException
	 *             the electoral organization operation exception
	 * @throws BaseException
	 *             the base exception
	 */
	void setDefaultApplicationKey()
			throws ElectoralOrganizationOperationException, BaseException;

	/**
	 * Generate random application key.
	 *
	 * @throws ElectoralOrganizationOperationException
	 *             the electoral organization operation exception
	 * @throws BaseException
	 *             the base exception
	 */
	void generateRandomApplicationKey()
			throws ElectoralOrganizationOperationException, BaseException;

	/**
	 * Gets the application key.
	 *
	 * @return the application key
	 * @throws ElectoralOrganizationOperationException
	 *             the electoral organization operation exception
	 */
	String getApplicationKey() throws ElectoralOrganizationOperationException;

	/**
	 * Method used to get a string property from os-regex.properties.
	 * 
	 * @param key
	 *            The key of the property to search.
	 * 
	 * @return The string value of the property with key:<code>key</code>
	 */
	public String getECMProperty(String key);

	/**
	 * Returns a list of common audio resources XML files, which are loaded as
	 * resources.
	 * 
	 * @return the list of audio resources.
	 * @throws UtilitiesException
	 *             if an error occurs loading the files from the database.
	 */
	public List<ReportTemplateData> getCommonAudioResources()
			throws UtilitiesException;

	/**
	 * Gets the voices resources from the server.
	 * 
	 * @return A byte array with the inputStream
	 * @throws UtilitiesException
	 *             an error occurred getting the resource files
	 * @throws IOException
	 *             signals that an I/O exception of some sort has occurred.
	 */
	byte[] getVoiceFiles() throws UtilitiesException;

	/**
	 * Gets the EML files directory.
	 * 
	 * @return the EML files directory
	 */
	public File getEMLFilesDir();

	/**
	 * Gets Active election.
	 * 
	 * @return the active election.
	 * @throws ElectoralOrganizationOperationException
	 */
	public ElectionData getActiveElection()
			throws ElectoralOrganizationOperationException;


    /**
     * Truncates and deletes the content of the table: {@code device_parameter}.
     */
    void truncateDeviceParametersTable() throws BulkDataLoadException;
}
