package com.smartmatic.waffle.ecm.service.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.ballot.BallotCardData;
import com.smartmatic.waffle.domain.ballot.BallotData;
import com.smartmatic.waffle.domain.ballot.BallotOptionCategoryEnum;
import com.smartmatic.waffle.domain.ballot.BallotOptionData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.CategoryData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.PathData;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.PollingStationData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.device.CustomAttributeDefinition;
import com.smartmatic.waffle.domain.device.CustomAttributeDefinitionValue;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionRoleData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.user.IUser;
import com.smartmatic.waffle.ecm.data.transfer.objects.ActiveContestForElectionDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionXmlComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceDTO;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralDemographyServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMNominationServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMProcessedContestsSingletonLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesServiceLocal;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;
import com.smartmatic.waffle.properties.ApplicationProperties;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMUtilitiesServiceImpl.class, ApplicationProperties.class,
		ECMDemographyServiceImpl.class, ECMElectoralDemographyServiceImpl.class })
public class ECMMasterUSBGenerationServiceImplTest {

	ECMMasterUSBGenerationServiceImpl masterUSBGenerationServiceImpl;

	@Before
	public void setUp() throws Exception {
		this.masterUSBGenerationServiceImpl = new ECMMasterUSBGenerationServiceImpl();
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public final void testGetActiveContestForElection() throws Exception {

		// arrange
		List<RegionData> lastLevelRegions = prepateMockForGetActiveContestForElection();

		LanguageData appLanguage = new LanguageData();

		// act
		ActiveContestForElectionDTO activeContestForElection = this.masterUSBGenerationServiceImpl
				.getActiveContestForElection(lastLevelRegions, appLanguage);

		// assert
		assertNotNull(activeContestForElection.getContestList());
		assertNotNull(activeContestForElection.getElectionLanguages());
		assertNotNull(activeContestForElection.getRegionContestGroup());
		assertNotNull(activeContestForElection.getRegionLanguageMap());
		assertNotNull(activeContestForElection.getResourcesLanguages());

	}

	@Test
	public void testGenerateElectionForContest() throws Exception {

		// arrange
		IECMUtilitiesServiceLocal utilitiesService = PowerMockito
				.mock(IECMUtilitiesServiceLocal.class);

		IECMElectoralDemographyServiceLocal electoralDemography = PowerMockito
				.mock(IECMElectoralDemographyServiceLocal.class);

		IPersistenceServiceLocal persistenceService = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		IECMNominationServiceLocal nominationService = PowerMockito
				.mock(IECMNominationServiceLocal.class);

		IECMProcessedContestsSingletonLocal processedContestsService = PowerMockito
				.mock(IECMProcessedContestsSingletonLocal.class);

		PowerMockito.mockStatic(ApplicationProperties.class);

		List<ContestData> contestList = CommonTestMethods
				.getContestLoadedInDatabase();

		List<EligibleOptionData> eligibleOptionList = getEligibleOptionList();

		for (ContestData contest : contestList) {
			contest.setEligibleOptions(eligibleOptionList);
		}

		ElectionData election = new ElectionData();

		Map<String, Map<String, String>> resourcesLanguages = new HashMap<String, Map<String, String>>();

		PowerMockito.when(
				ApplicationProperties.getIntValue(Mockito.anyString()))
				.thenReturn(1);

		this.masterUSBGenerationServiceImpl
				.setUtilitiesService(utilitiesService);

		this.masterUSBGenerationServiceImpl
				.setPersistenceService(persistenceService);

		this.masterUSBGenerationServiceImpl
				.setElectoralDemographyService(electoralDemography);

		this.masterUSBGenerationServiceImpl
				.setNominationService(nominationService);

		this.masterUSBGenerationServiceImpl
				.setProcessedContestsService(processedContestsService);
		// act
		ElectionXmlComponentsDTO electionDto = this.masterUSBGenerationServiceImpl
				.generateElectionForContest(contestList, 1, election,
						resourcesLanguages, null);

		// assert
		Set<BallotData> ballotSet = electionDto.getElection().getBallots();

		assertEquals(election, electionDto.getElection());
		assertEquals(1, ballotSet.size());

		for (BallotData ballot : ballotSet) {
			Set<BallotCardData> ballotCards = ballot.getBallotCards();

			for (BallotCardData ballotCard : ballotCards) {
				assertEquals(ballotCard.getContest(), contestList);
			}
		}

		assertEquals(resourcesLanguages, electionDto.getResourcesLanguages());

	}

	@Test
	public void testGenerateVotingDeviceForRegion() throws Exception {
		// arrange
		IECMUtilitiesServiceLocal utilitiesService = PowerMockito
				.mock(IECMUtilitiesServiceLocal.class);

		ECMDemographyServiceImpl demographyService = PowerMockito
				.mock(ECMDemographyServiceImpl.class);

		ECMElectoralDemographyServiceImpl electoralDemographyService = PowerMockito
				.mock(ECMElectoralDemographyServiceImpl.class);

		PowerMockito.mockStatic(ApplicationProperties.class);

		VotingDeviceData votingDevice = new VotingDeviceData();
		VotingBoothData votingBooth = new VotingBoothData();
		PollingPlaceData pollingPlace = new PollingPlaceData();
		PollingStationData pollingStation = new PollingStationData();

		pollingStation.setPollingPlace(pollingPlace);
		votingBooth.setPollingStation(pollingStation);
		votingDevice.setVotingBooth(votingBooth);

		RegionData region = new RegionData();
		Set<RegionData> regionParents = new HashSet<RegionData>();
		RegionData parent = new RegionData();

		CategoryData category1 = new CategoryData();
		CategoryData category2 = new CategoryData();
		category1.setCategoryLevel(CategoryData.Level.FIRST_LEVEL);
		category2.setCategoryLevel(CategoryData.Level.SECOND_LEVEL);

		Set<PathData> paths = new HashSet<PathData>();

		PathData path = new PathData();
		path.setCode(1);
		path.setName("path1");
		path.setDefaultPath(true);
		paths.add(path);

		category1.setPaths(paths);
		category2.setPaths(paths);
		regionParents.add(parent);
		region.setAllParents(regionParents);
		parent.setCategory(category1);
		region.setCategory(category2);

		List<ContestGroupData> contestGroupList = new ArrayList<ContestGroupData>();
		ContestGroupData contestGroup = new ContestGroupData();
		contestGroup.setCode(1);
		contestGroup.setCustomCode("1-ConstestGroup");
		contestGroup.setContestGroupName("ConstestGroup");

		Set<ContestData> contests = new HashSet<ContestData>(
				CommonTestMethods.getContestLoadedInDatabase());
		contestGroup.setContests(contests);

		contestGroupList.add(contestGroup);

		Map<String, Map<String, String>> resourcesLanguages = new TreeMap<String, Map<String, String>>();

		ElectionData election = new ElectionData();
		election.setElectionOpeningDate(new Date());
		election.setElectionClosingDate(new Date());

		votingDevice.setName("Voting device test");

		ContestData contest = new ContestData();
		contest.setCode(1);
		contest.setCollegeLanguageRFC("FR");

		PowerMockito.when(
				ApplicationProperties.getBooleanValue(Mockito.anyString()))
				.thenReturn(true);

		PowerMockito.doReturn(region).when(demographyService)
				.getAllParentsRegions(region);

		PowerMockito.doReturn(parent).when(demographyService)
				.checkRegionParentPath(region, path);

		PowerMockito.doReturn(new LogicalVotingGroupData())
				.when(electoralDemographyService)
				.getLogicalVotingGroup(Mockito.anyString());

		PowerMockito.doReturn(contest).when(electoralDemographyService)
				.getContestByCode(Mockito.anyLong());

		PowerMockito.doReturn(contestGroup).when(electoralDemographyService)
				.getContestGroupByCustomCode(Mockito.anyString());

		this.masterUSBGenerationServiceImpl
				.setUtilitiesService(utilitiesService);

		this.masterUSBGenerationServiceImpl
				.setDemographyService(demographyService);

		this.masterUSBGenerationServiceImpl
				.setElectoralDemographyService(electoralDemographyService);

		// act
		VotingDeviceDTO votingDeviceDto = this.masterUSBGenerationServiceImpl
				.generateVotingDeviceForRegion(votingDevice, region,
						contestGroupList, resourcesLanguages, false, election,
						false);

		// assert

		VotingDeviceData votingDeviceResult = votingDeviceDto
				.getVotingDevicesData().getVotingDeviceData();

		List<CustomAttributeDefinition> customAttributeDefinitionListResult = votingDeviceResult
				.getLogicalVoterGroupsData().getCustomAttributeDefinition();

		for (CustomAttributeDefinition customAttributeDefinition : customAttributeDefinitionListResult) {
			assertNotNull(customAttributeDefinition.getCode());
			assertNotNull(customAttributeDefinition.getDescriptionKey());
			assertNotNull(customAttributeDefinition.getMachineType());
			assertNotNull(customAttributeDefinition
					.getCustomAttributeDefinitionValues());

			for (CustomAttributeDefinitionValue customAttributeDefinitionValue : customAttributeDefinition
					.getCustomAttributeDefinitionValues()) {
				assertNotNull(customAttributeDefinitionValue.getValue());
			}
		}

	}

	@Test
	public void testGetLanguages() throws BaseException,
			MasterUSBGenerationException {

		// arrange
		IPersistenceServiceLocal persistenceService = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		Set<LanguageData> languages = new HashSet<LanguageData>();

		LanguageData language1 = new LanguageData();
		language1.setCode(1);
		language1.setCountry("BE");
		language1.setCustomCode("FR");

		LanguageData language2 = new LanguageData();
		language2.setCode(2);
		language2.setCountry("BE");
		language2.setCustomCode("NL");

		languages.add(language1);
		languages.add(language2);

		PowerMockito
				.doReturn(languages)
				.when(persistenceService)
				.searchObjects(Mockito.any(SearchExpression.class),
						Mockito.any(IUser.class));

		this.masterUSBGenerationServiceImpl
				.setPersistenceService(persistenceService);

		// act
		Set<LanguageData> languagesResult = this.masterUSBGenerationServiceImpl
				.getLanguages();

		// assert
		assertEquals(2, languagesResult.size());

		assertEquals(languages, languagesResult);
	}

	private List<RegionData> prepateMockForGetActiveContestForElection()
			throws UtilitiesException, Exception {
		IECMDemographyServiceLocal demographyService = PowerMockito
				.mock(IECMDemographyServiceLocal.class);

		IECMElectoralDemographyServiceLocal electoralDemography = PowerMockito
				.mock(IECMElectoralDemographyServiceLocal.class);

		IPersistenceServiceLocal persistenceService = PowerMockito
				.mock(IPersistenceServiceLocal.class);

		IECMUtilitiesServiceLocal utilitiesService = PowerMockito
				.mock(IECMUtilitiesServiceLocal.class);

		PowerMockito.when(
				utilitiesService.translateContestName(
						Mockito.any(ContestData.class),
						Mockito.any(LanguageData.class))).thenAnswer(
				new Answer<ContestData>() {
					@Override
					public ContestData answer(InvocationOnMock invocation)
							throws Throwable {
						Object[] args = invocation.getArguments();
						return (ContestData) args[0];
					}
				});

		Set<LanguageData> languages = new HashSet<LanguageData>();

		PowerMockito.doReturn(languages).when(persistenceService,
				"searchObjects", Mockito.any(SearchExpression.class),
				Mockito.any(IUser.class));

		List<RegionData> lastLevelRegions = getRegionObject();

		PowerMockito.doReturn(lastLevelRegions).when(demographyService,
				"getRegionsWithParentsByRegionCodes", Mockito.anyCollection());

		List<ContestGroupData> contestGroupLoaded = new ArrayList<ContestGroupData>(
				CommonTestMethods.getContestGroupResult());

		PowerMockito.doReturn(contestGroupLoaded).when(electoralDemography,
				"getContestGroupsLoadedInDatabase");

		List<ContestData> contestList = CommonTestMethods
				.getContestLoadedInDatabase();

		PowerMockito.doReturn(contestList).when(electoralDemography,
				"getContestByDistrict", Mockito.anyObject());

		this.masterUSBGenerationServiceImpl
				.setDemographyService(demographyService);

		this.masterUSBGenerationServiceImpl
				.setElectoralDemographyService(electoralDemography);

		this.masterUSBGenerationServiceImpl
				.setPersistenceService(persistenceService);

		this.masterUSBGenerationServiceImpl
				.setUtilitiesService(utilitiesService);

		return lastLevelRegions;
	}

	private List<RegionData> getRegionObject() {
		List<RegionData> lastLevelRegions = new ArrayList<RegionData>();

		RegionData region = new RegionData();

		region.setCode(1);

		region.setCustomCode("REGIONTEST1");

		region.setName("RegionTest");

		Set<RegionData> regionParentsSet = new HashSet<RegionData>();

		Set<LanguageData> languages = new HashSet<LanguageData>();

		LanguageData language1 = new LanguageData();

		languages.add(language1);

		LanguageData language2 = new LanguageData();

		languages.add(language2);

		Set<DistrictData> districts = new HashSet<DistrictData>();

		DistrictData district1 = new DistrictData();

		districts.add(district1);

		DistrictData district2 = new DistrictData();

		districts.add(district2);

		RegionData regionParent1 = new RegionData();

		regionParent1.setDistricts(districts);

		regionParent1.setElectionsLanguage(languages);

		regionParentsSet.add(regionParent1);

		region.setAllParents(regionParentsSet);

		lastLevelRegions.add(region);

		return lastLevelRegions;
	}

	private List<EligibleOptionData> getEligibleOptionList() {
		List<EligibleOptionData> eligibleOptionList = new ArrayList<EligibleOptionData>();

		EligibleOptionData eligibleOption1 = new EligibleOptionData();
		eligibleOption1.setCode(1);
		eligibleOption1.setBallotName("eligible option 1");
		eligibleOption1.setBallotOptions(getBallotOptionSet());
		eligibleOptionList.add(eligibleOption1);

		EligibleOptionData eligibleOption2 = new EligibleOptionData();
		eligibleOption2.setCode(2);
		eligibleOption2.setBallotName("eligible option 2");
		eligibleOption2.setBallotOptions(getBallotOptionSet());
		eligibleOptionList.add(eligibleOption2);

		EligibleOptionData eligibleOption3 = new EligibleOptionData();
		eligibleOption3.setCode(3);
		eligibleOption3.setBallotName("eligible option 3");
		eligibleOption3.setBallotOptions(getBallotOptionSet());
		eligibleOptionList.add(eligibleOption3);

		return eligibleOptionList;
	}

	private Set<BallotOptionData> getBallotOptionSet() {
		Set<BallotOptionData> ballotOptionSet = new HashSet<BallotOptionData>();

		EligibleOptionData eligibleOption = new EligibleOptionData();
		eligibleOption.setCandidateId("1234");

		SortedSet<EligibleOptionRoleData> eligibleOptionRolSet = new TreeSet<EligibleOptionRoleData>();
		eligibleOption.setEligibleOptionRoleData(eligibleOptionRolSet);

		BallotOptionData ballot1 = new BallotOptionData();
		ballot1.setCode(1);
		ballot1.setCategory(BallotOptionCategoryEnum.PRINCIPAL);
		ballot1.setEligibleOption(eligibleOption);

		PartyData party1 = new PartyData();
		party1.setCustomCode("CK11002A1_1");

		ballot1.setParty(party1);

		ballotOptionSet.add(ballot1);

		BallotOptionData ballot2 = new BallotOptionData();
		ballot2.setCode(2);
		ballot2.setCategory(BallotOptionCategoryEnum.STAND_IN);
		ballot2.setEligibleOption(eligibleOption);

		PartyData party2 = new PartyData();
		party2.setCustomCode("CK11002A1_5");

		ballot2.setParty(party2);

		ballotOptionSet.add(ballot2);

		return ballotOptionSet;
	}

}
