/**
 * InfoPanel.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.ui;

import javax.swing.Icon;

import org.jdesktop.application.ResourceMap;

import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;

/**
 * Blocking panel is shown to user when the app isn't available.
 */
public class InfoPanel extends javax.swing.JPanel {

	/**
	 * the serial version.
	 */
	private static final long serialVersionUID = 201947972981358284L;

	/**
	 * The icon that is shown in panel.
	 */
	private Icon icon;

	/**
	 * The message that is shown in panel.
	 */
	private String message;

	/** The resource message bundle. */
	private ResourceMap resourceMap = LanguageController.getInstance()
			.getResourceMap();

	/**
	 * * the default class constructor.
	 */
	public InfoPanel() {
		this.initComponents();
	}

	/**
	 * Instantiates a new info panel.
	 * 
	 * @param msg
	 *            the msg
	 */
	public InfoPanel(String msg) {
		this();
		this.message = msg;
		this.icon = this.resourceMap.getIcon("RTS.disable.icon");
		this.setIcon(this.icon);
		this.setText(this.message);
	}

	/**
	 * Instantiates a new info panel.
	 * 
	 * @param parameter
	 *            the parameter
	 */
	public InfoPanel(Icon parameter) {
		this();
		this.setIcon(parameter);
	}

	/**
	 * Instantiates a new info panel.
	 * 
	 * @param parameter
	 *            the parameter
	 * @param msg
	 *            the msg
	 */
	public InfoPanel(Icon parameter, String msg) {
		this();
		this.icon = parameter;
		this.message = msg;
		this.setIcon(this.icon);
		this.setText(this.message);
	}

	/**
	 * Method used to set the icon to shown.
	 * 
	 * @param icon
	 *            the icon to shown.
	 */
	public void setIcon(Icon icon) {
		this.jLabel1.setIcon(icon);
	}

	/**
	 * Method used to set the message to shown.
	 * 
	 * @param text
	 *            the message to shown.
	 */
	public void setText(String text) {
		this.jLabel2.setFont(this.jLabel2.getFont().deriveFont(
				this.jLabel2.getFont().getStyle() | java.awt.Font.BOLD,
				this.jLabel2.getFont().getSize() + 8));
		this.jLabel2.setText(text);
	}

	/**
	 * Method used to set the message to shown.
	 * 
	 * @param text
	 *            the message to shown.
	 * @param fontSize
	 *            the font size
	 */
	public void setText(String text, int fontSize) {
		this.jLabel2.setFont(this.jLabel2.getFont().deriveFont(
				this.jLabel2.getFont().getStyle() | java.awt.Font.BOLD,
				fontSize));
		this.jLabel2.setText(text);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {

		this.jPanel1 = new javax.swing.JPanel();
		this.jLabel1 = new javax.swing.JLabel();
		this.jLabel2 = new javax.swing.JLabel();

		this.setLayout(new javax.swing.BoxLayout(this,
				javax.swing.BoxLayout.PAGE_AXIS));

		this.jPanel1.setName("jPanel1");

		this.jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.jLabel1.setName("jLabel1");

		this.jLabel2.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		this.jLabel2.setName("jLabel2");

		javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(
				this.jPanel1);
		this.jPanel1.setLayout(jPanel1Layout);
		jPanel1Layout
				.setHorizontalGroup(jPanel1Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGap(0, 400, Short.MAX_VALUE)
						.addGroup(
								jPanel1Layout
										.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
										.addGroup(
												jPanel1Layout
														.createSequentialGroup()
														.addGap(101, 101, 101)
														.addGroup(
																jPanel1Layout
																		.createParallelGroup(
																				javax.swing.GroupLayout.Alignment.CENTER)
																		.addComponent(
																				this.jLabel1)
																		.addComponent(
																				this.jLabel2,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				javax.swing.GroupLayout.DEFAULT_SIZE,
																				Short.MAX_VALUE))
														.addGap(101, 101, 101))));
		jPanel1Layout
				.setVerticalGroup(jPanel1Layout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGap(0, 300, Short.MAX_VALUE)
						.addGroup(
								jPanel1Layout
										.createParallelGroup(
												javax.swing.GroupLayout.Alignment.LEADING)
										.addGroup(
												jPanel1Layout
														.createSequentialGroup()
														.addGap(0, 150,
																Short.MAX_VALUE)
														.addComponent(
																this.jLabel1)
														.addComponent(
																this.jLabel2)
														.addGap(0, 150,
																Short.MAX_VALUE))));

		this.add(this.jPanel1);
	}

	/** The j label1. */
	private javax.swing.JLabel jLabel1;

	/** The j label2. */
	private javax.swing.JLabel jLabel2;

	/** The j panel1. */
	private javax.swing.JPanel jPanel1;

}
