/**
 * LanguageConfirmationDialog.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.jdesktop.application.ResourceMap;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.ui.core.ShutDownMachineActionListener;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.ecm.ui.util.DotDashBorder;
import com.smartmatic.waffle.ecm.ui.util.GUIComponentUtil;

/**
 * Class used to build and show a language confirmation dialog, message will be
 * drawn in all the available application languages.
 */
public class LanguageConfirmationDialog extends JPanel {

	/** The resource map. */
	protected ResourceMap resourceMap = LanguageController.getInstance()
			.getResourceMap();

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -5645053128144753476L;

	/** The shutdown button panel. */
	private javax.swing.JPanel shutdownButtonPanel;

	/**
	 * Creates new form LanguageSelectionDialog.
	 * 
	 */
	public LanguageConfirmationDialog() {
		super();
		this.initComponents();
		this.shutdownButtonPanel.removeAll();
		JButton shutDown = new JButton();
		shutDown.addActionListener(new ShutDownMachineActionListener(this));

		Dimension shutDownDimension = new Dimension(60, 60);
		shutDown.setPreferredSize(shutDownDimension);
		shutDown.setMaximumSize(shutDownDimension);
		shutDown.setMinimumSize(shutDownDimension);
		shutDown.setAlignmentY(CENTER_ALIGNMENT);
		shutDown.setAlignmentX(CENTER_ALIGNMENT);
		shutDown.setIcon(this.resourceMap.getIcon("Application.shutdown.ico"));
		shutDown.setFocusable(false);
		shutDown.setName("buttonShutdown");

		this.shutdownButtonPanel.add(shutDown);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {
		Dimension languageButtonSize, panelSize;
		int hGap = 0, hCenterGap = 0, vGap = 0, vCenterGap = 0;
		Dimension shutDownPanelSize;

		boolean isLandscape = GUIComponentUtil.isLandscape(this);

		if (isLandscape) {
			languageButtonSize = new Dimension(250, 110);
			panelSize = new java.awt.Dimension(1000, 250);
			hGap = 43;
			hCenterGap = 300;
			vGap = 30;
			vCenterGap = 50;
			shutDownPanelSize = new Dimension(900, 60);
		} else {
			languageButtonSize = new Dimension(200, 100);
			panelSize = new Dimension(760, 400);
			hGap = 75;
			hCenterGap = 230;
			vGap = 102;
			vCenterGap = 300;
			shutDownPanelSize = new Dimension(700, 60);
		}

		this.languagesLabelsPanel = new javax.swing.JPanel();
		this.languagesButtonsPanel = new javax.swing.JPanel();
		this.shutdownButtonPanel = new javax.swing.JPanel();
		this.cancelButton = new javax.swing.JButton();
		this.confirmButton = new javax.swing.JButton();

		Font tknFont = null;
		tknFont = this.resourceMap
				.getFont("default.language.confirmation.buttons.font");

		if (tknFont == null) {
			tknFont = this.confirmButton.getFont().deriveFont(
					this.confirmButton.getFont().getStyle()
							| java.awt.Font.BOLD,
					this.confirmButton.getFont().getSize() + 4);
		}

		this.cancelButton.setFont(tknFont);
		this.cancelButton.setFocusPainted(false);
		this.cancelButton.setName("buttonCancel");
		this.cancelButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						LanguageConfirmationDialog.this
								.cancelButtonActionPerformed(evt);
					}
				});
		this.cancelButton.setPreferredSize(languageButtonSize);

		this.confirmButton.setFont(tknFont);
		this.confirmButton.setFocusPainted(false);
		this.confirmButton.setName("buttonConfirm");
		this.confirmButton
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent evt) {
						LanguageConfirmationDialog.this
								.confirmButtonActionPerformed(evt);
					}
				});
		this.confirmButton.setPreferredSize(languageButtonSize);

		this.setBackground(this.resourceMap.getColor("mainPane.background"));

		this.languagesLabelsPanel.setMaximumSize(panelSize);
		this.languagesLabelsPanel.setMinimumSize(panelSize);
		this.languagesLabelsPanel.setName("panelLanguagesLabels");
		this.languagesLabelsPanel.setOpaque(false);
		this.languagesLabelsPanel.setPreferredSize(panelSize);

		this.languagesButtonsPanel.setMaximumSize(panelSize);
		this.languagesButtonsPanel.setMinimumSize(panelSize);
		this.languagesButtonsPanel.setName("panelLanguagesButtons");
		this.languagesButtonsPanel.setOpaque(false);

		this.shutdownButtonPanel.setMaximumSize(shutDownPanelSize);
		this.shutdownButtonPanel.setMinimumSize(shutDownPanelSize);
		this.shutdownButtonPanel.setName("panelShutdownButton");
		this.shutdownButtonPanel.setOpaque(false);
		this.shutdownButtonPanel.setPreferredSize(shutDownPanelSize);
		javax.swing.GroupLayout languagesButtonsPanelLayout = new javax.swing.GroupLayout(
				this.languagesButtonsPanel);
		this.languagesButtonsPanel.setLayout(languagesButtonsPanelLayout);
		languagesButtonsPanelLayout
				.setHorizontalGroup(languagesButtonsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								languagesButtonsPanelLayout
										.createSequentialGroup()
										.addGap(hGap)
										.addComponent(
												this.cancelButton,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(
												javax.swing.LayoutStyle.ComponentPlacement.RELATED,
												hCenterGap, Short.MAX_VALUE)
										.addComponent(
												this.confirmButton,
												javax.swing.GroupLayout.PREFERRED_SIZE,
												javax.swing.GroupLayout.DEFAULT_SIZE,
												javax.swing.GroupLayout.PREFERRED_SIZE)
										.addGap(hGap)));
		languagesButtonsPanelLayout
				.setVerticalGroup(languagesButtonsPanelLayout
						.createParallelGroup(
								javax.swing.GroupLayout.Alignment.LEADING)
						.addGroup(
								javax.swing.GroupLayout.Alignment.TRAILING,
								languagesButtonsPanelLayout
										.createSequentialGroup()
										.addContainerGap(vCenterGap,
												Short.MAX_VALUE)
										.addGroup(
												languagesButtonsPanelLayout
														.createParallelGroup(
																javax.swing.GroupLayout.Alignment.LEADING)
														.addComponent(
																this.cancelButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE)
														.addComponent(
																this.confirmButton,
																javax.swing.GroupLayout.PREFERRED_SIZE,
																javax.swing.GroupLayout.DEFAULT_SIZE,
																javax.swing.GroupLayout.PREFERRED_SIZE))
										.addGap(vGap)));

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
				.addComponent(this.languagesLabelsPanel,
						javax.swing.GroupLayout.PREFERRED_SIZE,
						javax.swing.GroupLayout.PREFERRED_SIZE, Short.MAX_VALUE)
				.addComponent(this.languagesButtonsPanel,
						javax.swing.GroupLayout.PREFERRED_SIZE,
						javax.swing.GroupLayout.PREFERRED_SIZE, Short.MAX_VALUE)
				.addComponent(this.shutdownButtonPanel,
						javax.swing.GroupLayout.PREFERRED_SIZE,
						javax.swing.GroupLayout.PREFERRED_SIZE, Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(
				javax.swing.GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup()
						.addComponent(this.languagesLabelsPanel,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								Short.MAX_VALUE)
						.addComponent(this.languagesButtonsPanel,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								Short.MAX_VALUE)
						.addComponent(this.shutdownButtonPanel,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								javax.swing.GroupLayout.PREFERRED_SIZE,
								Short.MAX_VALUE)));

	}

	/**
	 * Confirm button action performed.
	 * 
	 * @param evt
	 *            the evt
	 */
	private void confirmButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
		if (this.listener != null) {
			this.listener.ok();
		}
	}

	/**
	 * Cancel button action performed.
	 * 
	 * @param evt
	 *            the evt
	 */
	private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
		if (this.listener != null) {
			this.listener.cancel();
		}

	}

	/** The cancel button. */
	private javax.swing.JButton cancelButton;

	/** The confirm button. */
	private javax.swing.JButton confirmButton;

	/** The languages buttons panel. */
	private javax.swing.JPanel languagesButtonsPanel;

	/** The languages labels panel. */
	private javax.swing.JPanel languagesLabelsPanel;

	/** The listener. */
	private DialogListener listener;

	/**
	 * Sets the language.
	 * 
	 * @param language
	 *            the language
	 * @param languages
	 *            the languages
	 */
	public void setLanguage(String language, Collection<LanguageData> languages) {

		StringBuilder confirmText = new StringBuilder(), cancelText = new StringBuilder();

		this.languagesLabelsPanel.removeAll();
		this.languagesLabelsPanel.setLayout(new GridBagLayout());

		this.confirmButton.setLayout(new GridBagLayout());
		this.cancelButton.setLayout(new GridBagLayout());

		java.awt.GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints.gridx = 0;
		gridBagConstraints.insets = new Insets(0, 0, 15, 0);

		int idx = 0;

		for (LanguageData languageData : languages) {

			Font tknFont = null;
			Color tknColor = null;

			String translateLanguage = this.getString(languageData
					.getCustomCode() + "." + language);

			String keyName = "label." + languageData.getCustomCode()
					+ ".langConfirmation";

			JLabel langLabel = new JLabel(this.getString(
					languageData.getCustomCode() + ".confirmText",
					translateLanguage));

			langLabel.setName(keyName);

			tknFont = this.resourceMap.getFont(languageData.getCustomCode()
					+ ".selection.label.font");
			if (tknFont == null) {
				tknFont = langLabel.getFont().deriveFont(
						langLabel.getFont().getStyle() | java.awt.Font.BOLD,
						langLabel.getFont().getSize() + 4);
			}
			langLabel.setFont(tknFont);
			langLabel.setHorizontalAlignment(SwingConstants.CENTER);
			langLabel.setVerticalAlignment(SwingConstants.CENTER);
			langLabel.setHorizontalTextPosition(SwingConstants.CENTER);
			langLabel.setBorder(new DotDashBorder(
					DotDashBorder.PatternType.DASH_LONG.getPattern(), 1,
					Color.GRAY));
			langLabel.setOpaque(true);
			tknColor = this.resourceMap.getColor(languageData.getCustomCode()
					+ ".selection.label.background.color");
			if (tknColor == null) {
				tknColor = Color.WHITE;
			}

			langLabel.setBackground(tknColor);
			langLabel.setPreferredSize(new Dimension(650, 75));
			langLabel.setMaximumSize(new Dimension(650, 75));
			langLabel.setMinimumSize(new Dimension(650, 75));

			gridBagConstraints.gridy = idx;

			this.languagesLabelsPanel.add(langLabel, gridBagConstraints);

			JLabel confirmLabel = new JLabel(
					this.resourceMap.getString(languageData.getCustomCode()
							+ ".confirm.button.text"));

			confirmLabel.setName("button." + languageData.getCustomCode()
					+ ".langConfirm");

			confirmLabel.setFont(tknFont);
			confirmLabel.setHorizontalAlignment(SwingConstants.CENTER);
			confirmLabel.setVerticalAlignment(SwingConstants.CENTER);
			confirmLabel.setHorizontalTextPosition(SwingConstants.CENTER);

			JLabel cancelLabel = new JLabel(
					this.resourceMap.getString(languageData.getCustomCode()
							+ ".cancel.button.text"));

			cancelLabel.setName("button." + languageData.getCustomCode()
					+ ".langCancel");

			cancelLabel.setFont(tknFont);
			cancelLabel.setHorizontalAlignment(SwingConstants.CENTER);
			cancelLabel.setVerticalAlignment(SwingConstants.CENTER);
			cancelLabel.setHorizontalTextPosition(SwingConstants.CENTER);
			this.confirmButton.add(confirmLabel, gridBagConstraints);
			this.cancelButton.add(cancelLabel, gridBagConstraints);

			idx++;
		}

		if (cancelText.length() > 0 && confirmText.length() > 0) {
			cancelText.replace(0, 3, "");
			confirmText.replace(0, 3, "");

			this.cancelButton.setText(cancelText.toString());
			this.confirmButton.setText(confirmText.toString());
		}

	}

	/**
	 * Returns a string that matches with the given key.
	 * 
	 * @param key
	 *            the key to find
	 * @param arg
	 *            additional arguments
	 * @return the matched key
	 */
	public String getString(String key, Object... arg) {
		String result = this.resourceMap.getString(key, arg);
		if (result == null || result.length() == 0) {
			result = key;
		}
		return result;
	}

	/**
	 * Sets the listener for the dialog actions.
	 * 
	 * @param theListener
	 *            the new listener
	 */
	public void setListener(DialogListener theListener) {
		this.listener = theListener;
	}

}
