/**
 * CellCheckRenderer.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.controls.Browser;

import java.awt.Component;

import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.TableCellRenderer;

import com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyTreeDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.BrowserHierarchyStatusType;

/**
 * Class that specifies how to render the cells of the status column of the
 * table in the <code>RegionLevelNavigationWidget</code>.
 * 
 * @author Mario Bislick (mario.bislick@smartmatic.com)
 */
public class CellCheckRenderer extends JCheckBox implements TableCellRenderer {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -5683832234856558707L;

	/** The selected object. */
	private BrowserHierarchyDTO selectedObject;

	/** The tree memory. */
	private BrowserHierarchyTreeDTO treeMemory;

	/**
	 * Instantiates a new cell check renderer.
	 * 
	 * @param treeMemory
	 *            the tree memory
	 */
	public CellCheckRenderer(BrowserHierarchyTreeDTO treeMemory) {
		super();
		setOpaque(true);
		this.treeMemory = treeMemory;
	}

	/**
	 * Instantiates a new cell check renderer.
	 */
	public CellCheckRenderer() {
		super();
		setOpaque(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax
	 * .swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		if (isSelected) {
			setForeground(table.getSelectionForeground());
			setBackground(table.getSelectionBackground());
		} else {
			setForeground(table.getForeground());
			setBackground(UIManager.getColor("Button.background"));
		}
		this.selectedObject = (BrowserHierarchyDTO) value;
		String checkboxIconName;
		BrowserHierarchyStatusType currentStatus = this.treeMemory
				.getCurrentStatus(this.selectedObject);

		if (BrowserHierarchyStatusType.FULL_SELECTED.equals(currentStatus)) {
			checkboxIconName = "/com/smartmatic/waffle/ecm/client/resources/checkbox_true.jpg";
		} else if (BrowserHierarchyStatusType.PARTIAL_SELECTED
				.equals(currentStatus)) {
			checkboxIconName = "/com/smartmatic/waffle/ecm/client/resources/checkbox_transition.jpg";
		} else {
			checkboxIconName = "/com/smartmatic/waffle/ecm/client/resources/checkbox_false.jpg";
		}

		setIcon(new javax.swing.ImageIcon(getClass().getResource(
				checkboxIconName)));
		return this;
	}
}
