/**
 * AbstractGeneralMenuView.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.menu;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

import org.jdesktop.application.ResourceMap;

import com.smartmatic.waffle.domain.security.ApplicationDefinitionEnum;
import com.smartmatic.waffle.domain.security.FunctionalityDefinitionEnum;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.ecm.ui.layout.SpringLayoutUtil;

/**
 * Abstract class define a general menu view panel.
 */
public abstract class AbstractGeneralMenuView extends javax.swing.JPanel
		implements IGeneralMenuView {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/**
	 * Holds the panels that should be displayed once the user select a menu
	 * category.
	 */
	protected Map<BasicMenuItem, JPanel> categoryPanelMap;

	/** The close action listener. */
	protected ActionListener closeActionListener;

	/** The map buttons. */
	protected Map<FunctionalityDefinitionEnum, JButton> mapButtons = new HashMap<FunctionalityDefinitionEnum, JButton>();

	/** The func selected dispatcher. */
	protected IFunctionalitySelected funcSelectedDispatcher;

	/**
	 * Holds the resource map used to obtain keys values required by this
	 * dialog.
	 */
	protected ResourceMap resourceMap = LanguageController.getInstance()
			.getResourceMap();

	/**
	 * Builds the panel related to a given category.
	 * 
	 * @param category
	 *            The category to create the panel
	 * @param funcButtonActionListener
	 *            the func button action listener
	 * @param app
	 *            the app
	 * @return A panel with the supported functionalities that are inside the
	 *         category as buttons
	 */
	protected JPanel buildPanelByMenuCategory(MenuCategoryItem category,
			ActionListener funcButtonActionListener,
			ApplicationDefinitionEnum app) {
		JPanel container = new JPanel();
		container.setLayout(new GridBagLayout());
		container.setBackground(Color.WHITE);

		/* Builds constraints to be used with inside panel */
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		JPanel insidePanel = new JPanel(new SpringLayout());
		insidePanel.setBackground(Color.WHITE);

		List<FunctionalityDefinitionEnum> toSort = new ArrayList<FunctionalityDefinitionEnum>(
				category.getFunctionalityDefinitions().keySet());
		Collections.sort(toSort);
		Iterator<FunctionalityDefinitionEnum> functionalities = toSort
				.iterator();

		/* Adds the buttons in the panel where it will be displayed */
		int numPairs = 0;
		while (functionalities.hasNext()) {
			FunctionalityDefinitionEnum actualFunctionality = functionalities
					.next();
			String functionalityString = this.resourceMap
					.getString(actualFunctionality.toString());
			JButton funcButton = new JButton(functionalityString.toUpperCase());
			funcButton.setName(actualFunctionality.toString());
			funcButton.setAlignmentY(Component.CENTER_ALIGNMENT);
			funcButton.setAlignmentX(Component.CENTER_ALIGNMENT);

			/* Estimated size */
			funcButton.setPreferredSize(new Dimension(400, 50));

			insidePanel.add(funcButton);
			numPairs++;

			/* Adds a listener to the button */
			funcButton.addActionListener(funcButtonActionListener);
			this.mapButtons.put(actualFunctionality, funcButton);
		}
		SpringLayoutUtil.makeCompactGrid(insidePanel, numPairs, 1, 0, 0, 5, 5);
		container.add(insidePanel, gridBagConstraints);

		this.checkStatusPanel(app);
		return container;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.ui.menu.IGeneralMenuView#checkStatusPanel
	 * (com.smartmatic.waffle.domain.security.ApplicationDefinitionEnum)
	 */
	@Override
	public void checkStatusPanel(ApplicationDefinitionEnum app) {
		Iterator<FunctionalityDefinitionEnum> keys = this.mapButtons.keySet()
				.iterator();
		for (; keys.hasNext();) {
			FunctionalityDefinitionEnum key = keys.next();
			@SuppressWarnings("unused")
			JButton button = this.mapButtons.get(key);
			switch (app) {
			case RTS:
				break;
			default:
				break;
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.ui.menu.IGeneralMenuView#setCloseActionListener
	 * (java.awt.event.ActionListener)
	 */
	@Override
	public void setCloseActionListener(ActionListener listener) {
		this.closeActionListener = listener;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.ui.menu.IGeneralMenuView#
	 * setFuncSelectedDispatcher
	 * (com.smartmatic.waffle.ecm.ui.menu.IFunctionalitySelected)
	 */
	@Override
	public void setFuncSelectedDispatcher(
			IFunctionalitySelected funcSelectedDispatcher) {
		this.funcSelectedDispatcher = funcSelectedDispatcher;
	}
}
