/**
 * BasicMenuItem.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.menu;

import java.util.List;
import java.util.Map;

import com.smartmatic.waffle.domain.security.FunctionalityDefinitionEnum;
import com.smartmatic.waffle.domain.security.IActionData;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;

/**
 * Class used to create menu items.
 */
public abstract class BasicMenuItem {

	/** The category title. */
	@XStreamAlias("category-title-key")
	@XStreamAsAttribute
	private String categoryTitle;

	/** The icon resource. */
	@XStreamAlias("icon-resource")
	@XStreamAsAttribute
	private String iconResource;

	/**
	 * Functionality count.
	 * 
	 * @return the int
	 */
	public abstract int functionalityCount();

	/**
	 * Clone restriced menu item.
	 * 
	 * @param availableFunctionalities
	 *            the available functionalities
	 * @return the basic menu item
	 */
	public BasicMenuItem cloneRestricedMenuItem(
			Map<FunctionalityDefinitionEnum, List<IActionData>> availableFunctionalities) {
		BasicMenuItem clonedInstance = this
				.createAndCloneCustomInformation(availableFunctionalities);
		clonedInstance.setCategoryTitle(this.categoryTitle);
		clonedInstance.setIconResource(this.iconResource);
		return clonedInstance;
	}

	/**
	 * Creates the and clone custom information.
	 * 
	 * @param availableFunctionalities
	 *            the available functionalities
	 * @return the basic menu item
	 */
	protected abstract BasicMenuItem createAndCloneCustomInformation(
			Map<FunctionalityDefinitionEnum, List<IActionData>> availableFunctionalities);

	/**
	 * Getter for the <code>categoryTitle</code> attribute.
	 * 
	 * @return <code>categoryTitle</code> attribute value.
	 * @see BasicMenuItem#categoryTitle
	 */
	public String getCategoryTitle() {
		return this.categoryTitle;
	}

	/**
	 * Setter for the <code>categoryTitle</code> attribute.
	 * 
	 * @param categoryTitle
	 *            the categoryTitle to set
	 * @see BasicMenuItem#categoryTitle
	 */
	public void setCategoryTitle(String categoryTitle) {
		this.categoryTitle = categoryTitle;
	}

	/**
	 * Getter for the <code>iconResource</code> attribute.
	 * 
	 * @return <code>iconResource</code> attribute value.
	 * @see BasicMenuItem#iconResource
	 */
	public String getIconResource() {
		return this.iconResource;
	}

	/**
	 * Setter for the <code>iconResource</code> attribute.
	 * 
	 * @param iconResource
	 *            the iconResource to set
	 * @see BasicMenuItem#iconResource
	 */
	public void setIconResource(String iconResource) {
		this.iconResource = iconResource;
	}

}
