/**
 * IGeneralMenuView.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.menu;

import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JPanel;

import com.smartmatic.waffle.domain.security.ApplicationDefinitionEnum;
import com.smartmatic.waffle.domain.security.FunctionalityDefinitionEnum;

/**
 * Interface used to define the methods for the general menu.
 */
public interface IGeneralMenuView {

	/**
	 * Define the actions to show or not the close button.
	 * 
	 * @param showCloseButton
	 *            true to show, false otherwise
	 */
	void showCloseButton(boolean showCloseButton);

	/**
	 * Set the listener to close button.
	 * 
	 * @param listener
	 *            the action listener
	 */
	void setCloseActionListener(java.awt.event.ActionListener listener);

	/**
	 * Set the dispatcher for the selected function.
	 * 
	 * @param funcSelectedDispatcher
	 *            the new func selected dispatcher
	 */
	public void setFuncSelectedDispatcher(
			IFunctionalitySelected funcSelectedDispatcher);

	/**
	 * Configures the menu with the items.
	 * 
	 * @param categoryList
	 *            a collection of menu items
	 * @param applicationDefinitionEnum
	 *            application definition
	 */
	public void setMenuConfiguration(Collection<BasicMenuItem> categoryList,
			ApplicationDefinitionEnum applicationDefinitionEnum);

	/**
	 * Checks the status of the panel for the given application.
	 * 
	 * @param app
	 *            the application to check
	 */
	public void checkStatusPanel(ApplicationDefinitionEnum app);

	/**
	 * Shows functionality.
	 * 
	 * @param funcToShow
	 *            the func to show
	 */
	public void showFunctionalityByEnumValue(
			FunctionalityDefinitionEnum funcToShow);

	/**
	 * Shows a popup panel.
	 * 
	 * @param panel
	 *            the parent panel
	 * @param fixedSize
	 *            boolean to define if the panel should have a fixed size
	 */
	public void showPopupPanel(JPanel panel, boolean fixedSize);

	/**
	 * Shows a popup panel and adds a close button.
	 * 
	 * @param panel
	 *            the parent panel
	 * @param fixedSize
	 *            boolean to define if the panel should have a fixed size
	 * @param closeButton
	 *            the close button
	 */
	public void showPopupPanel(JPanel panel, boolean fixedSize,
			JButton closeButton);

}
