/**
 * MenuConfiguration.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.menu;

import java.util.List;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * Class used to configure the general menu.
 */
@XStreamAlias("menu-conf")
public class MenuConfiguration {

	/**
	 * The Enum MenuType.
	 */
	public static enum MenuType {

		/** The buttons. */
		BUTTONS,
		/** The tabs. */
		TABS;
	}

	/** The menu type. */
	@XStreamAsAttribute
	@XStreamAlias(value = "menu-type")
	private MenuType menuType;

	/** The perform workflow. */
	@XStreamAsAttribute
	@XStreamAlias(value = "perform-workflow-on-startup")
	private boolean performWorkflow;

	/** The func shortcut. */
	@XStreamImplicit(itemFieldName = "func-shortcut")
	private List<FunctionalityShortcutMenuItem> funcShortcut;

	/** The menu category. */
	@XStreamImplicit(itemFieldName = "menu-category")
	private List<MenuCategoryItem> menuCategory;

	/**
	 * Instantiates a new menu configuration.
	 */
	public MenuConfiguration() {
		super();
	}

	/**
	 * Getter of <code>performWorkflow</code> property.
	 * 
	 * @return <code>performWorkflow</code> property value.
	 * @see #performWorkflow
	 */

	public boolean isPerformWorkflow() {
		return this.performWorkflow;
	}

	/**
	 * Setter of <code>performWorkflow</code> property.
	 * 
	 * @param performWorkflow
	 *            The <code>performWorkflow</code> to set.
	 * @see #performWorkflow
	 */
	public void setPerformWorkflow(boolean performWorkflow) {
		this.performWorkflow = performWorkflow;
	}

	/**
	 * Getter of <code>funcShortcut</code> property.
	 * 
	 * @return <code>funcShortcut</code> property value.
	 * @see #funcShortcut
	 */

	public List<FunctionalityShortcutMenuItem> getFuncShortcut() {
		return this.funcShortcut;
	}

	/**
	 * Setter of <code>funcShortcut</code> property.
	 * 
	 * @param funcShortcut
	 *            The <code>funcShortcut</code> to set.
	 * @see #funcShortcut
	 */
	public void setFuncShortcut(List<FunctionalityShortcutMenuItem> funcShortcut) {
		this.funcShortcut = funcShortcut;
	}

	/**
	 * Getter of <code>menuCategory</code> property.
	 * 
	 * @return <code>menuCategory</code> property value.
	 * @see #menuCategory
	 */

	public List<MenuCategoryItem> getMenuCategory() {
		return this.menuCategory;
	}

	/**
	 * Setter of <code>menuCategory</code> property.
	 * 
	 * @param menuCategory
	 *            The <code>menuCategory</code> to set.
	 * @see #menuCategory
	 */
	public void setMenuCategory(List<MenuCategoryItem> menuCategory) {
		this.menuCategory = menuCategory;
	}

	/**
	 * Getter of <code>menuType</code> property.
	 * 
	 * @return <code>menuType</code> property value.
	 * @see #menuType
	 */

	public MenuType getMenuType() {
		if (this.menuType == null) {
			return MenuType.BUTTONS;
		} else {
			return this.menuType;
		}
	}

	/**
	 * Setter of <code>menuType</code> property.
	 * 
	 * @param theMenuType
	 *            The <code>menuType</code> to set.
	 * @see #menuType
	 */
	public void setMenuType(MenuType theMenuType) {
		this.menuType = theMenuType;
	}

}
