/**
 * PopUpOptionDialog.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.util;

import java.awt.Component;
import java.util.List;
import java.util.Locale;

import javax.swing.Icon;
import javax.swing.JOptionPane;

import org.jdesktop.application.ResourceMap;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;

/**
 * Util class that build a custom popup using JOptionPane.
 * 
 * @version 1.0 (Aug 11, 2011)
 * @author Mauricio Herrera (mauricio.herrera@smartmatic.com)
 */

public class PopUpOptionDialog extends JOptionPane {

	/**
	 * The Serial Version UID.
	 */
	private static final long serialVersionUID = 8396511404168334342L;

	/**
	 * The options key of the buttons by default.
	 */
	private static String[] optionsKey = { "general.answer.yes",
			"general.answer.no" };

	/**
	 * Brings up a dialog with a specified icon, where the number of choices is
	 * determined by the <code>optionType</code> parameter.
	 * <p>
	 * If <code>optionType</code> is <code>YES_NO_OPTION</code>, or
	 * <code>YES_NO_CANCEL_OPTION</code> and the <code>options</code> parameter
	 * is <code>null</code>, then the options are supplied by the look and feel.
	 * <p>
	 * The <code>messageType</code> parameter is primarily used to supply a
	 * default icon from the look and feel.
	 * 
	 * @param parentComponent
	 *            determines the <code>Frame</code> in which the dialog is
	 *            displayed; if <code>null</code>, or if the
	 *            <code>parentComponent</code> has no <code>Frame</code>, a
	 *            default <code>Frame</code> is used
	 * @param message
	 *            the <code>Object</code> to display
	 * @param title
	 *            the title string for the dialog
	 * @param optionType
	 *            an integer designating the options available on the dialog:
	 *            <code>DEFAULT_OPTION</code>, <code>YES_NO_CANCEL_OPTION</code>
	 *            , <code>YES_NO_OPTION</code> or <code>OK_CANCEL_OPTION</code>
	 * @param messageType
	 *            an integer designating the kind of message this is, primarily
	 *            used to determine the icon from the pluggable Look and Feel:
	 *            <code>ERROR_MESSAGE</code>, <code>INFORMATION_MESSAGE</code>,
	 *            <code>WARNING_MESSAGE</code>, <code>QUESTION_MESSAGE</code>,
	 *            or <code>PLAIN_MESSAGE</code>
	 * @param icon
	 *            the icon to display in the dialog
	 * @param options
	 *            an array of objects indicating the possible choices the user
	 *            can make; if the objects are components, they are rendered
	 *            properly; non-<code>String</code> objects are rendered using
	 *            their <code>toString</code> methods; if this parameter is
	 *            <code>null</code>, the options are determined by the Look and
	 *            Feel
	 * @return an integer indicating the option chosen by the user, or
	 *         <code>CLOSED_OPTION</code> if the user closed the dialog
	 * @see javax.swing.JOptionPane
	 */
	public static int showPopUpOptionDialog(Component parentComponent,
			Object message, String title, int optionType, int messageType,
			Icon icon, Object[] options) {

		int result = ECMOptionPane.showOptionDialog(parentComponent, message,
				title, optionType, messageType, icon, options, null,
				ECMOptionPane.CONFIRM_DIALOG);

		return result;
	}

	/**
	 * Brings up a dialog with a specified icon, where the number of choices is
	 * determined by the <code>optionType</code> parameter.
	 * <p>
	 * If <code>optionType</code> is <code>YES_NO_OPTION</code>, or
	 * <code>YES_NO_CANCEL_OPTION</code> and the <code>options</code> parameter
	 * is <code>null</code>, then the options are supplied by the look and feel.
	 * <p>
	 * The <code>messageType</code> parameter is primarily used to supply a
	 * default icon from the look and feel.
	 * 
	 * @param parentComponent
	 *            determines the <code>Frame</code> in which the dialog is
	 *            displayed; if <code>null</code>, or if the
	 *            <code>parentComponent</code> has no <code>Frame</code>, a
	 *            default <code>Frame</code> is used
	 * @param messageKey
	 *            the key of the string message to display
	 * @param titleKey
	 *            the key of the string title for the dialog
	 * @param optionType
	 *            an integer designating the options available on the dialog:
	 *            <code>DEFAULT_OPTION</code>, <code>YES_NO_OPTION</code> or
	 *            <code>OK_CANCEL_OPTION</code>
	 * @param messageType
	 *            an integer designating the kind of message this is, primarily
	 *            used to determine the icon from the pluggable Look and Feel:
	 *            <code>ERROR_MESSAGE</code>, <code>INFORMATION_MESSAGE</code>,
	 *            <code>WARNING_MESSAGE</code>, <code>QUESTION_MESSAGE</code>,
	 *            or <code>PLAIN_MESSAGE</code>
	 * @param icon
	 *            the icon to display in the dialog
	 * @param optionsKey
	 *            an array of key of the strings indicating the possible choices
	 *            the user can make; if this parameter is <code>null</code>, the
	 *            options are determined by the <code>optionsKey</code> array.
	 * 
	 * @param languages
	 *            the language list to use in the popup.
	 * @return an integer indicating the option chosen by the user, or
	 *         <code>CLOSED_OPTION</code> if the user closed the dialog
	 * @see javax.swing.JOptionPane
	 */
	public static int showPopUpOptionDialog(Component parentComponent,
			String messageKey, String titleKey, int optionType,
			int messageType, Icon icon, String[] optionsKey,
			List<LanguageData> languages) {

		ResourceMap resourceMap = null;

		String title = "";
		String question = "";
		String ok = "";
		String cancel = "";

		LanguageController lc = LanguageController.getInstance();
		LanguageData currentLanguage = lc.getSelectedLanguage();
		Locale currentLocal = Locale.getDefault();
		boolean isFirst = true;
		Locale selectedLocale = null;

		if (optionsKey == null) {
			optionsKey = PopUpOptionDialog.optionsKey;
		}

		for (LanguageData lang : lc.getLanguageList()) {
			if (!isFirst) {
				title += " / ";
				question += "\n\n";
				ok += " / ";
				cancel += " / ";
			} else {
				isFirst = false;
			}
			lc.setSelectedLanguage(lang);
			selectedLocale = lc.getLocaleFromLanguage(lang);
			Locale.setDefault(selectedLocale);
			resourceMap = lc.getResourceMap();
			title += resourceMap.getString(titleKey);
			question += resourceMap.getString(messageKey);
			ok += resourceMap.getString(optionsKey[0]);

			cancel += resourceMap.getString(optionsKey[1]);
		}
		if (currentLanguage == null) {
			lc.setSelectedLanguage(null);
		} else {
			lc.setSelectedLanguage(currentLanguage);
		}
		Locale.setDefault(currentLocal);

		String[] options = { ok, cancel };

		int result = ECMOptionPane.showOptionDialog(parentComponent, question,
				title, optionType, messageType, icon, options, null,
				ECMOptionPane.CONFIRM_DIALOG);

		return result;
	}

	/**
	 * Brings up a dialog with a specified icon, where the number of choices is
	 * determined by the <code>optionType</code> parameter.
	 * <p>
	 * If <code>optionType</code> is <code>YES_NO_OPTION</code>, or
	 * <code>YES_NO_CANCEL_OPTION</code> and the <code>options</code> parameter
	 * is <code>null</code>, then the options are supplied by the look and feel.
	 * <p>
	 * The <code>messageType</code> parameter is primarily used to supply a
	 * default icon from the look and feel.
	 * 
	 * @param parentComponent
	 *            determines the <code>Frame</code> in which the dialog is
	 *            displayed; if <code>null</code>, or if the
	 *            <code>parentComponent</code> has no <code>Frame</code>, a
	 *            default <code>Frame</code> is used
	 * @param messageKey
	 *            the key of the string message to display
	 * @param titleKey
	 *            the key of the string title for the dialog
	 * @param optionType
	 *            an integer designating the options available on the dialog:
	 *            <code>DEFAULT_OPTION</code>, <code>YES_NO_OPTION</code> or
	 *            <code>OK_CANCEL_OPTION</code>
	 * @param messageType
	 *            an integer designating the kind of message this is, primarily
	 *            used to determine the icon from the pluggable Look and Feel:
	 *            <code>ERROR_MESSAGE</code>, <code>INFORMATION_MESSAGE</code>,
	 *            <code>WARNING_MESSAGE</code>, <code>QUESTION_MESSAGE</code>,
	 *            or <code>PLAIN_MESSAGE</code>
	 * @param icon
	 *            the icon to display in the dialog
	 * @param optionsKey
	 *            an array of key of the strings indicating the possible choices
	 *            the user can make; if this parameter is <code>null</code>, the
	 *            options are determined by the <code>optionsKey</code> array.
	 * 
	 * @param resourceMap
	 *            the resource map of the language to use in the popup.
	 * @return an integer indicating the option chosen by the user, or
	 *         <code>CLOSED_OPTION</code> if the user closed the dialog
	 * @see javax.swing.JOptionPane
	 */
	public static int showPopUpOptionDialog(Component parentComponent,
			String messageKey, String titleKey, int optionType,
			int messageType, Icon icon, String[] optionsKey,
			ResourceMap resourceMap) {

		String title = resourceMap.getString(titleKey);
		String message = resourceMap.getString(messageKey);

		if (optionsKey == null) {
			optionsKey = PopUpOptionDialog.optionsKey;
		}

		String option0 = resourceMap.getString(optionsKey[0]);
		String option1 = resourceMap.getString(optionsKey[1]);
		String options[] = { option0, option1 };

		int result = ECMOptionPane.showOptionDialog(parentComponent, message,
				title, optionType, messageType, icon, options, null,
				ECMOptionPane.CONFIRM_DIALOG);

		return result;
	}
}
