/**
 * SAESCommonDestinations.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.localpersistence;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager.MountedDrive;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * Class used to reference and handle files and folders used by the application.
 */
@XStreamAlias(value = "saes-common-destinations")
public class SAESCommonDestinations extends ApplianceCommonDestinations {

	/** The status. */
	@XStreamAlias(value = "status")
	@CommonDestinationInfo(existence = Existence.OPTIONAL_VALID, existenceLevel = 0, type = DestinationType.HISTORIC_XML_FILE)
	public final File status;

	/** The create os image script. */
	@XStreamAlias(value = "create-os-image-script")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_PLAIN)
	public final File createOsImageScript;

	/** The script to write the application key in the master usb. */
	@XStreamAlias(value = "write-application-key")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_PLAIN)
	public final File writeApplicationKeyScript;

	/** The pen drive util script. */
	@XStreamAlias(value = "pen-drive-util-script")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_PLAIN)
	public final File pendriveUtilScript;

	/** The temp document modified. */
	@XStreamAlias(value = "temp-document-modified")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_PLAIN)
	public final File tempDocumentModified;

	/** The signature register. */
	@XStreamAlias(value = "signature-register")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_PLAIN)
	public final File signatureRegister;

	/** The voting device image. */
	@XStreamAlias(value = "voting-machine-image")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE)
	public final File votingDeviceImage;

	/** The canvassing station image. */
	@XStreamAlias(value = "canvassing-Station-Image")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE)
	public final File canvassingStationImage;

	/** The voting device screen shot image. */
	@XStreamAlias(value = "voting-machine-screen-shot-image")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE)
	public final File votingDeviceScreenShotImage;

	/** The eml dir. */
	@XStreamAlias(value = "emlDir")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File emlDir;

	/** The eml file. */
	@XStreamAlias(value = "emlFile")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File emlFile;

	/** The default languages file. */
	@XStreamAlias(value = "defaultLanguagesFile")
	@CommonDestinationInfo(existence = Existence.MANDATORY, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File defaultLanguagesFile;

	/** The codi temp dir. */
	@XStreamAlias(value = "codeTempDir")
	@CommonDestinationInfo(existence = Existence.OPTIONAL_AUTOCREATE, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File codiTempDir;

	/** The codi temp dir results. */
	@XStreamAlias(value = "codiTempDirResults")
	@CommonDestinationInfo(existence = Existence.OPTIONAL_AUTOCREATE, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File codiTempDirResults;

	/** The codi sign crt. */
	@XStreamAlias(value = "filesToUploadValidationCertificateFile")
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE)
	public final File filesToUploadValidationCertificateFile;

	/** The device info. */
	@XStreamOmitField
	protected MountedDrive deviceInfo;

	/**
	 * *************************************************************************
	 * ***************************************.
	 */

	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File saesLocalDir;

	/** The saes dir. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File saesDir;

	/** The saes docs. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File saesDocs;

	/** The voting device. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File votingDevice;

	/** The transmission package. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File transmissionPackage;

	/** The transmission package key. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File transmissionPackageKey;

	/** The voting centers. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File votingCenters;

	/** The installation code. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File installationCode;

	/** The watch dog lock file. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE)
	public final File watchDogLockFile;

	/** The unsaved. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE)
	public final File unsaved;

	/** The integrity folder. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File integrityFolder;

	/** The integrity file. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FILE_XML)
	public final File integrityFile;

	/** The pvs folder. */
	@CommonDestinationInfo(existence = Existence.OPTIONAL, existenceLevel = 0, type = DestinationType.FOLDER)
	public final File pvsFolder;

	/**
	 * Instantiates a new sAES common destinations.
	 * 
	 * @param theBaseDir
	 *            the the base dir
	 * @param dynamicDirName
	 *            the dynamic dir name
	 * @param theDeviceInfo
	 *            the the device info
	 * @throws IllegalArgumentException
	 *             the illegal argument exception
	 * @throws IllegalAccessException
	 *             the illegal access exception
	 */
	public SAESCommonDestinations(File theBaseDir, String dynamicDirName,
			MountedDrive theDeviceInfo) throws IllegalArgumentException,
			IllegalAccessException {
		super(theBaseDir, dynamicDirName);

		File dynamicDirFile = this.buildDynamicDirFile(dynamicDirName);

		this.status = new File(dynamicDirFile, "status.xml");

		this.createOsImageScript = new File(this.scriptsDir, "OS-install.sh");

		this.writeApplicationKeyScript = new File(this.scriptsDir,
				"writeApplicationKey.sh");

		this.filesToUploadValidationCertificateFile = new File(this.scriptsDir,
				"CodiSignCertificate.crt");

		this.pendriveUtilScript = new File(this.scriptsDir, "pendriveUtil.sh");

		this.tempDocumentModified = new File(this.reportsDir,
				"document_modified.pdf");

		this.signatureRegister = new File(dynamicDirFile,
				"signature-register.xml");

		this.votingDeviceImage = new File(this.scriptsDir, "OS.tar.gz");

		this.canvassingStationImage = new File(this.scriptsDir,
				"RTS-installer.tar.gz");

		this.votingDeviceScreenShotImage = new File(this.scriptsDir,
				"OS_VD_CS.tar.gz");

		this.emlDir = new File(this.confDir, "eml");
		this.emlFile = new File(this.emlDir, "460-votes-v5-0.xsd");

		this.defaultLanguagesFile = new File(this.confDir,
				"default-languages.xml");

		this.codiTempDir = new File(this.confDir, "codiTempDir");

		this.codiTempDirResults = new File(this.confDir, "codiTempDirResults");

		this.deviceInfo = theDeviceInfo;

		this.saesLocalDir = new File(this.baseDir, "SAESLocal");
		this.saesDir = new File(this.saesLocalDir, "SAES");
		this.saesDocs = new File(this.saesDir, "Docs");
		this.votingDevice = new File(this.saesDir, "voting-devices.xml");
		this.transmissionPackage = new File(this.saesDocs,
				"transmission-package.xml");
		this.transmissionPackageKey = new File(this.saesDocs,
				"transmission-package.key");

		this.votingCenters = new File(this.saesLocalDir, "voting-centers.xml");
		this.installationCode = new File(this.confDir, "installation-code.xml");

		// added file used as reference for watch-dog script
		this.watchDogLockFile = new File(this.confDir, "lock");

		this.unsaved = new File(this.confDir, "unsaved");

		this.integrityFolder = new File(this.confDir, "integrity");

		this.integrityFile = new File(this.integrityFolder,
				"integrity-check.xml");

		this.pvsFolder = new File(this.codiTempDir, "PVS");

		this.processAnnotations();

		return;

	}

	/**
	 * Composed constructor.
	 * 
	 * @param theBaseDir
	 *            the the base dir
	 * @param dynamicDirName
	 *            the dynamic dir name
	 * @param confFile
	 *            the conf file
	 * @param theDeviceInfo
	 *            the the device info
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws IllegalArgumentException
	 *             the illegal argument exception
	 * @throws IllegalAccessException
	 *             the illegal access exception
	 */
	public SAESCommonDestinations(File theBaseDir, String dynamicDirName,
			InputStream confFile, MountedDrive theDeviceInfo)
			throws IOException, IllegalArgumentException,
			IllegalAccessException {
		super(theBaseDir, dynamicDirName, confFile);

		confFile.reset();

		SAESCommonDestinations instance = (SAESCommonDestinations) LocalPersistenceManager
				.getInstance().loadFromXML(confFile);
		SAESCommonDestinations defaultInstance = new SAESCommonDestinations(
				theBaseDir, dynamicDirName, this.deviceInfo);

		File dynamicDirFile = this.buildDynamicDirFile(dynamicDirName);

		this.status = this.composePath(dynamicDirFile, instance.status,
				defaultInstance.status);

		this.writeApplicationKeyScript = this.composePath(this.scriptsDir,
				instance.writeApplicationKeyScript,
				defaultInstance.writeApplicationKeyScript);

		this.createOsImageScript = this.composePath(this.scriptsDir,
				instance.createOsImageScript,
				defaultInstance.createOsImageScript);

		this.filesToUploadValidationCertificateFile = this.composePath(
				this.scriptsDir,
				instance.filesToUploadValidationCertificateFile,
				defaultInstance.filesToUploadValidationCertificateFile);

		this.pendriveUtilScript = this
				.composePath(this.scriptsDir, instance.pendriveUtilScript,
						defaultInstance.pendriveUtilScript);

		this.tempDocumentModified = this.composePath(this.reportsDir,
				instance.tempDocumentModified,
				defaultInstance.tempDocumentModified);

		this.signatureRegister = this.composePath(dynamicDirFile,
				instance.signatureRegister, defaultInstance.signatureRegister);

		this.votingDeviceImage = this.composePath(this.scriptsDir,
				instance.votingDeviceImage, defaultInstance.votingDeviceImage);

		this.canvassingStationImage = this.composePath(this.scriptsDir,
				instance.canvassingStationImage,
				defaultInstance.canvassingStationImage);

		this.votingDeviceScreenShotImage = this.composePath(this.scriptsDir,
				instance.votingDeviceScreenShotImage,
				defaultInstance.votingDeviceScreenShotImage);

		this.emlDir = this.composePath(this.confDir, instance.emlDir,
				defaultInstance.emlDir);

		this.emlFile = this.composePath(this.emlDir, instance.emlFile,
				defaultInstance.emlFile);

		this.defaultLanguagesFile = this.composePath(this.confDir,
				instance.defaultLanguagesFile,
				defaultInstance.defaultLanguagesFile);

		this.codiTempDir = this.composePath(this.confDir, instance.codiTempDir,
				defaultInstance.codiTempDir);

		this.codiTempDirResults = this
				.composePath(this.confDir, instance.codiTempDirResults,
						defaultInstance.codiTempDirResults);

		this.deviceInfo = theDeviceInfo;

		this.saesLocalDir = this.composePath(this.baseDir,
				instance.saesLocalDir, defaultInstance.saesLocalDir);
		this.saesDir = this.composePath(this.baseDir, instance.saesDir,
				defaultInstance.saesDir);
		this.saesDocs = this.composePath(this.saesDir, instance.saesDocs,
				defaultInstance.saesDocs);
		this.votingDevice = this.composePath(this.saesDir,
				instance.votingDevice, defaultInstance.votingDevice);
		this.transmissionPackage = this.composePath(this.saesDocs,
				instance.transmissionPackage,
				defaultInstance.transmissionPackage);
		this.transmissionPackageKey = this.composePath(this.saesDocs,
				instance.transmissionPackageKey,
				defaultInstance.transmissionPackageKey);

		this.votingCenters = this.composePath(this.saesLocalDir,
				instance.votingCenters, defaultInstance.votingCenters);

		this.installationCode = this.composePath(this.confDir,
				instance.installationCode, defaultInstance.installationCode);

		// added file used as reference for watch-dog script
		this.watchDogLockFile = this.composePath(this.confDir,
				instance.watchDogLockFile, defaultInstance.watchDogLockFile);

		this.unsaved = this.composePath(this.confDir, instance.unsaved,
				defaultInstance.unsaved);

		this.integrityFolder = this.composePath(this.confDir,
				instance.integrityFile, defaultInstance.integrityFolder);

		this.integrityFile = this.composePath(this.integrityFolder,
				instance.integrityFile, defaultInstance.integrityFolder);

		this.pvsFolder = this.composePath(this.pvsFolder, instance.pvsFolder,
				defaultInstance.pvsFolder);

		this.processAnnotations();
		return;
	}

	/**
	 * Builds the dynamic dir file.
	 * 
	 * @param dynamicDirName
	 *            the dynamic dir name
	 * @return the file
	 */
	private File buildDynamicDirFile(String dynamicDirName) {
		File dynamicDirFile;
		if (dynamicDirName != null) {
			dynamicDirFile = new File(this.appStateDir, dynamicDirName);
		} else {
			dynamicDirFile = this.appStateDir;

		}
		return dynamicDirFile;
	}

	/**
	 * Gets the single instance of SAESCommonDestinations.
	 * 
	 * @return single instance of SAESCommonDestinations
	 */
	public static SAESCommonDestinations getInstance() {
		if (CommonDestinations.instance == null) {
			CommonDestinations.setInstance(SAESLocalPersistenceManager
					.getInstance().getDefaultDestinations());
		}
		return (SAESCommonDestinations) CommonDestinations.getInstance();
	}

	/**
	 * Getter of <code>deviceInfo</code> property.
	 * 
	 * @return <code>deviceInfo</code> property value.
	 * @see #deviceInfo
	 */

	public MountedDrive getDeviceInfo() {
		return this.deviceInfo;
	}

	/**
	 * Setter of <code>deviceInfo</code> property.
	 * 
	 * @param theDeviceInfo
	 *            The <code>deviceInfo</code> to set.
	 * @see #deviceInfo
	 */
	public void setDeviceInfo(MountedDrive theDeviceInfo) {
		if (this.deviceInfo == null
				&& this.baseDir.getAbsolutePath().startsWith(
						theDeviceInfo.mountedPath.getAbsolutePath())) {

			this.deviceInfo = theDeviceInfo;
		} else {
			throw new IllegalStateException(
					"the deviceInfo field was already set");
		}

		return;
	}

}
