/**
 * SMTTAppender.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.log;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.log4j.jdbc.JDBCAppender;

/**
 * Extension of the default JDBCAppender created to support the data-source as
 * the input configuration to the database.
 * 
 * @see org.apache.log4j.jdbc.JDBCAppender
 * 
 * @author Daniel Izquierdo
 */
public class SMTTAppender extends JDBCAppender {

	/**
	 * The configured jndi datasource name.
	 */

	private String dataSourceJNDIName;

	/**
	 * The datasource where all the logging events are stored
	 */

	private DataSource dataSource;

	/**
	 * Empty constructor.
	 */
	public SMTTAppender() {
		super();
	}

	/**
	 * Method who creates the connection to the database.
	 */
	@Override
	public Connection getConnection() throws SQLException {
		/*
		 * the datasource may not be available when activateOptions() is called,
		 * so we initialize it here
		 */
		if (this.dataSource == null) {
			if (this.getDataSourceJNDIName() == null) {
				System.err
						.println("There is not a datasource jndi name configured");
				return null;
			}

			try {
				Context context = new InitialContext();
				this.dataSource = (DataSource) context.lookup(this
						.getDataSourceJNDIName());
			} catch (NamingException e) {
				System.err
						.println("There is a problem obtaining the configured datasource");
				e.printStackTrace();
				throw new SQLException(e);
			}
		}

		try {
			return this.dataSource.getConnection();
		} catch (SQLException e) {
			System.err
					.println("There is a problem obtaining the connection from the configured datasource");
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * Override this to return the connection to a pool, or to clean up the
	 * resource.
	 * 
	 * The default behavior holds a single connection open until the appender is
	 * closed (typically when garbage collected).
	 * 
	 * @param conn
	 *            The connection to close
	 */
	@Override
	public void closeConnection(Connection conn) {
		if (conn != null) {
			try {
				conn.close();
			} catch (SQLException e) {
				System.err
						.println("There is a problem closing the connection from the logger");
				e.printStackTrace();
			}
		}
	}

	/**
	 * Getter of <code>dataSourceJNDIName</code> property.
	 * 
	 * @return <code>dataSourceJNDIName</code> property value.
	 * @see #dataSourceJNDIName
	 */
	public String getDataSourceJNDIName() {
		return this.dataSourceJNDIName;
	}

	/**
	 * Setter of <code>dataSourceJNDIName</code> property.
	 * 
	 * @param dataSourceJNDIName
	 *            The <code>dataSourceJNDIName</code> to set.
	 * @see #dataSourceJNDIName
	 */
	public void setDataSourceJNDIName(String dataSourceJNDIName) {
		this.dataSourceJNDIName = dataSourceJNDIName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.log4j.jdbc.JDBCAppender#execute(java.lang.String)
	 */
	@Override
	protected void execute(String sql) throws SQLException {
		Connection con = null;
		Statement stmt = null;

		try {
			con = this.getConnection();

			stmt = con.createStatement();
			stmt.executeUpdate(sql);
		} catch (SQLException e) {

			throw e;
		} finally {
			if (stmt != null) {
				stmt.close();
			}
			if (con != null) {
				this.closeConnection(con);
			}

		}

	}

}