/**
 * ConfigurationParameter.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.log.configuration;

import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Represent the configuration parameter used as a XML node
 * 
 * @author Carlos Lameda (carlos.lameda@smartmatic.com)
 * @version 1.0 (Nov 11, 2009)
 */
public class ConfigurationParameter {

	/**
	 * Attribute's name inside the tag
	 */
	public static final String NAME_ATRIBUTE_NAME = "name";

	/**
	 * Attribute's value inside the tag
	 */
	public static final String VALUE_ATRIBUTE_NAME = "value";

	/**
	 * Attribute's type inside the tag
	 */
	public static final String TYPE_ATRIBUTE_NAME = "type";

	/**
	 * Attribute's reference inside the tag
	 */
	public static final String REFERENCE_ATRIBUTE_NAME = "ref";

	/**
	 * Attribute's additivity inside the tag
	 */
	public static final String ADDITIVITY_ATRIBUTE_NAME = "additivity";

	/**
	 * Node custom name
	 */
	private String customName;

	/**
	 * Node type
	 */
	private String type;

	/**
	 * Node value
	 */
	private String value;

	/**
	 * Inner parameters as children
	 */
	private LinkedList<ConfigurationParameter> innerParams;

	/**
	 * Use custom name for local element name
	 */
	private boolean useCostumNameForLocalElementName = false;

	/**
	 * The default element name
	 */
	private String defaultElementName = "param";

	/**
	 * Attribute's reference value
	 */
	private String reference;

	/**
	 * Attribute's additivity value
	 */
	private String additivity;

	/**
	 * Getter of <code>defaultElementName</code> property.
	 * 
	 * @return <code>defaultElementName</code> property value.
	 * @see #defaultElementName
	 */
	public String getDefaultElementName() {
		return this.defaultElementName;
	}

	/**
	 * Setter of <code>defaultElementName</code> property.
	 * 
	 * @param defaultElementName
	 *            The <code>defaultElementName</code> to set.
	 * @see #defaultElementName
	 */
	public void setDefaultElementName(String defaultElementName) {
		this.defaultElementName = defaultElementName;
	}

	/**
	 * Getter of <code>customName</code> property.
	 * 
	 * @return <code>customName</code> property value.
	 * @see #customName
	 */
	public String getCustomName() {
		return this.customName;
	}

	/**
	 * Getter of <code>type</code> property.
	 * 
	 * @return <code>type</code> property value.
	 * @see #type
	 */
	public String getType() {
		return this.type;
	}

	/**
	 * Getter of <code>value</code> property.
	 * 
	 * @return <code>value</code> property value.
	 * @see #value
	 */
	public String getValue() {
		return this.value;
	}

	/**
	 * Getter of <code>innerParams</code> property.
	 * 
	 * @return <code>innerParams</code> property value.
	 * @see #innerParams
	 */
	public LinkedList<ConfigurationParameter> getInnerParams() {
		return this.innerParams;
	}

	/**
	 * Getter of <code>reference</code> property.
	 * 
	 * @return <code>reference</code> property value.
	 * @see #reference
	 */
	public String getReference() {
		return this.reference;
	}

	/**
	 * Getter of <code>additivity</code> property.
	 * 
	 * @return <code>additivity</code> property value.
	 * @see #additivity
	 */
	public String getAdditivity() {
		return this.additivity;
	}

	/**
	 * Getter of <code>useCostumNameForLocalElementName</code> property.
	 * 
	 * @return <code>useCostumNameForLocalElementName</code> property value.
	 * @see #useCostumNameForLocalElementName
	 */
	public boolean isUseCostumNameForLocalElementName() {
		return this.useCostumNameForLocalElementName;
	}

	/**
	 * ConfigurationParameter Class constructor
	 * 
	 * @param costumName
	 *            The custom name to set
	 * @param value
	 *            The node value
	 */
	public ConfigurationParameter(String costumName, String value) {
		this(costumName, value, null, null, null, false);
	}

	/**
	 * ConfigurationParameter Class constructor
	 * 
	 * @param costumName
	 *            The custom name to set
	 * @param value
	 *            The node value
	 * @param type
	 *            The node type
	 */
	public ConfigurationParameter(String costumName, String value, String type) {
		this(costumName, value, type, null, null, false);
	}

	/**
	 * ConfigurationParameter Class constructor
	 * 
	 * @param costumName
	 *            The custom name
	 * @param value
	 *            The node value
	 * @param type
	 *            The node type
	 * @param reference
	 *            The node reference
	 * @param additivity
	 *            The node additivity
	 * @param useCostumNameForLocalElementName
	 *            If use custom name for local element name
	 */
	public ConfigurationParameter(String costumName, String value, String type,
			String reference, String additivity,
			boolean useCostumNameForLocalElementName) {
		this.customName = costumName;
		this.value = value;
		this.type = type;
		this.reference = reference;
		this.additivity = additivity;
		this.useCostumNameForLocalElementName = useCostumNameForLocalElementName;
	}

	/**
	 * Add a child to the element
	 * 
	 * @param childParameter
	 *            The child node to add
	 */
	public void addChildParameter(ConfigurationParameter childParameter) {
		addChildParameter(childParameter, false);
	}

	/**
	 * Add a child to the element
	 * 
	 * @param childParameter
	 *            The child node to add
	 * @param replace
	 *            Indicate if replace a existence node
	 */
	public void addChildParameter(ConfigurationParameter childParameter,
			boolean replace) {
		if (this.innerParams == null) {
			this.innerParams = new LinkedList<ConfigurationParameter>();
		}
		if (replace) {
			addOrReplace(childParameter);
		} else {
			this.innerParams.add(childParameter);
		}
	}

	/**
	 * Replace an element child
	 * 
	 * @param childParameter
	 *            The child to replace
	 */
	public void addOrReplace(ConfigurationParameter childParameter) {
		/*
		 * Removes (if there exists) a configuration parameter, with the same
		 * identification as 'childParameter'.
		 */
		this.innerParams.remove(childParameter);
		/* Adds the new 'childParameter'. */
		this.innerParams.add(childParameter);
	}

	/**
	 * Build a XML document with this element as root
	 * 
	 * @return The XML document
	 */
	public Element getXMLElement() {
		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		Document doc;
		try {
			// Build the document and set the root
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			doc = builder.newDocument();

			return getXMLElement(doc);
		} catch (ParserConfigurationException e) {
			return null;
		}
	}

	/**
	 * append XML node to a document
	 * 
	 * @param doc
	 *            Document to append the node
	 * @return the element
	 */
	public Element getXMLElement(Document doc) {
		// create the element
		String elementName = this.useCostumNameForLocalElementName ? this.customName
				: this.defaultElementName;
		Element rootElement = doc.createElement(elementName);
		// Load the element attributes
		if (!this.useCostumNameForLocalElementName && this.customName != null
				&& this.customName.length() > 0) {
			rootElement.setAttribute(NAME_ATRIBUTE_NAME, this.customName);
		}
		if (this.value != null && this.value.length() > 0) {
			rootElement.setAttribute(VALUE_ATRIBUTE_NAME, this.value);
		}
		if (this.type != null && this.type.length() > 0) {
			rootElement.setAttribute(TYPE_ATRIBUTE_NAME, this.type);
		}
		if (this.reference != null && this.reference.length() > 0) {
			rootElement.setAttribute(REFERENCE_ATRIBUTE_NAME, this.reference);
		}
		if (this.additivity != null && this.additivity.length() > 0) {
			rootElement.setAttribute(ADDITIVITY_ATRIBUTE_NAME, this.additivity);
		}
		if (this.innerParams != null) {
			for (ConfigurationParameter configParam : this.innerParams) {
				rootElement.appendChild(configParam.getXMLElement(doc));
			}
		}
		return rootElement;
	}

	/**
	 * Override the equals method, in this implementation customName and
	 * defaultElementName are used
	 * 
	 * @param obj
	 *            The other object to compare
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof ConfigurationParameter) {
			ConfigurationParameter conf = (ConfigurationParameter) obj;
			return this.customName.equalsIgnoreCase(conf.customName)
					&& this.defaultElementName
							.equalsIgnoreCase(conf.defaultElementName);
		} else {
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return super.hashCode();
	}

}
