/**
 * CriteriaDefinitionException.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:45:12 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.persistence.search;

import com.smartmatic.waffle.lang.BaseRuntimeException;
import com.smartmatic.waffle.lang.IExceptionInfo;

/**
 * Exception thrown when an attempt to create an alias with the same path is
 * made using different join types.
 * 
 * @version 1.0 (31/03/2006)
 * @author Carlos Ramirez (cramirez@smartmatic.com)
 */
public class CriteriaDefinitionException extends BaseRuntimeException {

	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */

	static final long serialVersionUID = 3928789772152502094L;

	/**
	 * <p>
	 * <code>Enumeration</code> of all possible causes by which the
	 * <code>SipXTapiBaseException</code> might be thrown.
	 * </p>
	 * 
	 * @see IExceptionInfo
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @revision initial release (04/05/2006) (submitted by
	 *           itorres@smartmatic.com)
	 */

	public enum ExceptionInfo implements IExceptionInfo {

		CONFLICTING_JOIN_TYPE("CONFLICTING_JOIN_TYPE");

		/**
		 * <p>
		 * Each enumerator's resource bundle's key message.
		 * </p>
		 */

		private String message;

		/**
		 * Getter of <code>message</code> property.
		 * 
		 * @return <code>message</code> property value.
		 * @see #message
		 * 
		 * 
		 * @author Ithamar Torres (itorres@smartmatic.com)
		 */

		@Override
		public String getMessage() {
			return this.message;
		}

		/**
		 * Setter of <code>message</code> property.
		 * 
		 * @param theMessage
		 *            The <code>message</code> to set.
		 * @see #message
		 * 
		 * @author Ithamar Torres (itorres@smartmatic.com)
		 */

		public void setMessage(String theMessage) {
			this.message = theMessage;
			return;
		}

		/**
		 * Constructor
		 * 
		 * @param msg
		 *            the resource bundle's key message
		 * 
		 * @author Ithamar Torres (itorres@smartmatic.com)
		 */

		private ExceptionInfo(String msg) {
			this.message = this.getClass().getName() + "." + msg;
		}
	}

	/**
	 * Constructor of ExceptionInfo with an interface description
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	public CriteriaDefinitionException(IExceptionInfo exceptionInfo) {
		super(exceptionInfo);
	}

	/**
	 * <p>
	 * Constructor for an instance of this class having the cause of the
	 * exception, and an array of placeholder values used as parameter for this
	 * exception's message contained in the <code>info</code> attribute.
	 * </p>
	 * 
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param parameters
	 *            the array of placeholder values used as parameters for this
	 *            exception's enumerated cause's related message
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	public CriteriaDefinitionException(IExceptionInfo exceptionInfo,
			Object... parameters) {
		super(exceptionInfo, parameters);
	}
}
