/**
 * LikeCriteria.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:45:08 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.criteria;

/**
 * A "like" expression. This expression could be case-sensitive or not
 * 
 * @author Ithamar Torres
 */

public class LikeCriteria extends SimpleCriteria {

	/**
	 * Holds the serialization id of the class
	 */
	private static final long serialVersionUID = 7940363021853695150L;

	/**
	 * Constructor of a LikeExpression
	 * 
	 * @param propertyName
	 *            Property that will be used in the like comparison
	 * @param value
	 *            Value that will be used in the comparison process
	 * @param ignoreCase
	 *            parameter that indicates that the like is case sensitive or
	 *            not
	 */

	public LikeCriteria(String propertyName, Object value, boolean ignoreCase) {
		super(propertyName, value, ignoreCase);
	}

	/**
	 * Constructor of a LikeExpression
	 * 
	 * @param propertyName
	 *            Property that will be used in the like comparison
	 * @param value
	 *            Value that will be used in the comparison process
	 */

	LikeCriteria(String propertyName, Object value) {
		super(propertyName, value);
	}

	/**
	 * Constructor of a LikeExpression
	 * 
	 * @param propertyName
	 *            Property that will be used in the like comparison
	 * @param value
	 *            Value that will be used in the comparison process
	 * @param matchMode
	 *            mode that will be used in the comparison process
	 * 
	 * @see com.smartmatic.waffle.persistence.search.criteria.MatchCriteriaMode
	 */

	LikeCriteria(String propertyName, String value, MatchCriteriaMode matchMode) {
		this(propertyName, matchMode.toMatchString(value));
	}

	/**
	 * Operator applied to the logical expression
	 * 
	 * @return The operator used in the logical operation
	 */

	@Override
	String getOp() {
		return " like ";
	}

}
