/**
 * LogicalCriteria.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:45:06 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.criteria;

/**
 * Superclass of binary logical expressions.
 * 
 * @author Ithamar Torres
 */

public abstract class LogicalCriteria implements ISearchCriteria {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Getter of <code>lhs</code> property.
	 * 
	 * @return <code>lhs</code> property value.
	 * @see #lhs
	 */

	public ISearchCriteria getLhs() {
		return this.lhs;
	}

	/**
	 * Setter of <code>lhs</code> property.
	 * 
	 * @param lhs
	 *            The <code>lhs</code> to set.
	 * @see #lhs
	 */
	public void setLhs(ISearchCriteria lhs) {
		this.lhs = lhs;
	}

	/**
	 * Getter of <code>rhs</code> property.
	 * 
	 * @return <code>rhs</code> property value.
	 * @see #rhs
	 */

	public ISearchCriteria getRhs() {
		return this.rhs;
	}

	/**
	 * Setter of <code>rhs</code> property.
	 * 
	 * @param rhs
	 *            The <code>rhs</code> to set.
	 * @see #rhs
	 */
	public void setRhs(ISearchCriteria rhs) {
		this.rhs = rhs;
	}

	/**
	 * Left hand side parameter of the binary expression.
	 */

	private ISearchCriteria lhs;

	/**
	 * Right hand side parameter of the binary expression.
	 */

	private ISearchCriteria rhs;

	/**
	 * Constructor of this abstract class. This constructor is called by the
	 * subclasses of this class to pass the parameters used by this class. This
	 * is the class where all the logic is coded the subclasses only contain the
	 * operator and all the invocations are handled by this class
	 * 
	 * @param lhs
	 *            Left hand side parameter of the binary expression.
	 * @param rhs
	 *            Right hand side parameter of the binary expression.
	 */

	LogicalCriteria(ISearchCriteria lhs, ISearchCriteria rhs) {
		this.lhs = lhs;
		this.rhs = rhs;
	}

	/**
	 * Abstract method that each subclass must implement. In this method each
	 * subclass should return the operator applied to the logical expression
	 * 
	 * @return The operator used in the logical operation
	 */

	abstract String getOp();

	/**
	 * Returns a string representation of the object
	 * 
	 * @return a string representation of the object
	 */

	@Override
	public String toString() {
		return this.lhs.toString() + ' ' + this.getOp() + ' '
				+ this.rhs.toString();
	}
}
