/**
 * OrderCriteria.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:45:07 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.criteria;

import java.io.Serializable;

/**
 * Represents an order imposed upon a <tt>Criteria</tt> result set
 * 
 * @author Ithamar Torres
 */

public class OrderCriteria implements Serializable {

	/**
	 * Holds the serialization id of the class
	 */
	private static final long serialVersionUID = -7144763194878846942L;

	/**
	 * Flag that indicates which order is used. Is this flag is true means that
	 * the results should be ordered ascending else descending
	 */

	private boolean ascending;

	/**
	 * Property that will be used in the ordering process
	 */

	private String propertyName;

	/**
	 * Constructor for Order.
	 * 
	 * @param propertyName
	 *            Property that will be used in the ordering process
	 * @param ascending
	 *            Flag that indicates which order is used
	 */

	protected OrderCriteria(String propertyName, boolean ascending) {
		this.propertyName = propertyName;
		this.ascending = ascending;
	}

	/**
	 * Ascending order
	 * 
	 * @param propertyName
	 *            propertyName Property that will be used in the ordering
	 *            process
	 * @return Order a new instance of Order configured to order the information
	 *         ascending
	 */

	public static OrderCriteria asc(String propertyName) {
		return new OrderCriteria(propertyName, true);
	}

	/**
	 * Descending order
	 * 
	 * @param propertyName
	 *            propertyName Property that will be used in the ordering
	 *            process
	 * @return Order a new instance of Order configured to order the information
	 *         descending
	 */
	public static OrderCriteria desc(String propertyName) {
		return new OrderCriteria(propertyName, false);
	}

	/**
	 * Getter of <code>ascending</code> property.
	 * 
	 * @return <code>ascending</code> property value.
	 * @see #ascending
	 */

	public boolean isAscending() {
		return this.ascending;
	}

	/**
	 * Setter of <code>ascending</code> property.
	 * 
	 * @param ascending
	 *            The <code>ascending</code> to set.
	 * @see #ascending
	 */
	public void setAscending(boolean ascending) {
		this.ascending = ascending;
	}

	/**
	 * Getter of <code>propertyName</code> property.
	 * 
	 * @return <code>propertyName</code> property value.
	 * @see #propertyName
	 */
	public String getPropertyName() {
		return this.propertyName;
	}

	/**
	 * Setter of <code>propertyName</code> property.
	 * 
	 * @param propertyName
	 *            The <code>propertyName</code> to set.
	 * @see #propertyName
	 */
	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof OrderCriteria)) {
			return false;
		}
		OrderCriteria oc = (OrderCriteria) o;
		return this.propertyName.equals(oc.propertyName)
				&& this.ascending == oc.ascending;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (this.ascending ? 1231 : 1237);
		result = prime
				* result
				+ ((this.propertyName == null) ? 0 : this.propertyName
						.hashCode());
		return result;
	}

}
