/**
 * SizeCollectionCriteria.java
 * 
 * $Revision: 1.4 $ ($Date: 2011-04-06 04:44:55 $)
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.criteria;

/**
 * Criteria used to create search expressions that could restrict the results
 * using the size of a collection.
 * 
 * @version 1.0 (May 21, 2007)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class SizeCollectionCriteria implements ISearchCriteria {

	/**
	 * Holds the serialization id of the class
	 */

	private static final long serialVersionUID = -2406140412482967742L;

	/**
	 * Enumerate that maintains all the modes that this collection criteria can
	 * be used
	 * 
	 * @version 1.0 (May 21, 2007)
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	public enum RestrictMode {
		/**
		 * Represent the equal size mode
		 */
		EQUAL,
		/**
		 * Represent the Not equal size mode
		 */
		NOTEQUAL
	}

	/**
	 * The mode to filter the search expression.
	 */

	private RestrictMode mode;

	/**
	 * The parameter to filter the collection
	 */

	private int size;

	/**
	 * The property name
	 */

	private String propertyName;

	/**
	 * Creates a new instance of a SizeCollectionCriteria
	 * 
	 * @param mode
	 *            The mode used to restrict the search expression
	 * @param propertyName
	 *            The property name
	 * @param size
	 *            The parameter used to filter the collection
	 */

	public SizeCollectionCriteria(RestrictMode mode, String propertyName,
			int size) {
		this.mode = mode;
		this.size = size;
		this.propertyName = propertyName;
	}

	/**
	 * Getter of <code>mode</code> property.
	 * 
	 * @return <code>mode</code> property value.
	 * @see #mode
	 */

	public RestrictMode getMode() {
		return this.mode;
	}

	/**
	 * Getter of <code>size</code> property.
	 * 
	 * @return <code>size</code> property value.
	 * @see #size
	 */

	public int getSize() {
		return this.size;
	}

	/**
	 * Getter of <code>propertyName</code> property.
	 * 
	 * @return <code>propertyName</code> property value.
	 * @see #propertyName
	 */

	public String getPropertyName() {
		return this.propertyName;
	}

}
