/**
 * SubqueryCriteria.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:44:54 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.criteria;

import com.smartmatic.waffle.persistence.search.SearchExpression;

/**
 * Constraint using the value returned by a sub-query
 * 
 * @author Carlos Rafael Ramirez
 * 
 */
public abstract class SubqueryCriteria implements ISearchCriteria {

	/**
	 * Denotes the data structure's version
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Sub-query to be executed and whose value will be used in a WHERE
	 * comparison
	 */
	private final SearchExpression subquery;

	/**
	 * Getter of <code>subquery</code> property.
	 * 
	 * @return <code>subquery</code> property value.
	 * @see #subquery
	 */
	public SearchExpression getSubquery() {
		return this.subquery;
	}

	/**
	 * Constructor of SubqueryCriteria
	 * 
	 * @param subquery
	 *            the sub query to be executed
	 */
	public SubqueryCriteria(SearchExpression subquery) {
		super();
		this.subquery = subquery;
	}

}
