/**
 * SubqueryPropertyCriteria.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:45:08 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.criteria;

import com.smartmatic.waffle.persistence.search.SearchExpression;

/**
 * Constraints a property using the value returned by a sub-query
 * 
 * @author Carlos Rafael Ramirez
 * 
 */
public class SubqueryPropertyCriteria extends SubqueryCriteria {

	/**
	 * Denotes the data structure's version
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Property that will be used in the evaluation of the expression
	 */

	private final String propertyName;

	/**
	 * Construct a subqueryPropertyCriteria using the provided sub-query and
	 * propertyName
	 * 
	 * @param subquery
	 *            Sub-query to be executed and whose value will be used in a
	 *            WHERE comparison
	 * @param propertyName
	 *            Property that will be used in the evaluation of the expression
	 */
	public SubqueryPropertyCriteria(SearchExpression subquery,
			String propertyName) {

		super(subquery);
		this.propertyName = propertyName;
	}

	/**
	 * Getter of <code>propertyName</code> property.
	 * 
	 * @return <code>propertyName</code> property value.
	 * @see #propertyName
	 */
	public String getPropertyName() {
		return this.propertyName;
	}

}
