/**
 * AliasTransformationsMap.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:45:09 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.transformation;

import java.util.HashMap;
import java.util.Map;

/**
 * Class that implements a list of projections that have alias and also stores
 * the bean's class that will be used to generate the result objects.
 * 
 * @version 1.0 (16/04/2005)
 * @author Ithamar Torres
 */

public class AliasTransformationsMap implements ITransformation {

	/**
	 * Holds the serialization id of the class
	 */

	private static final long serialVersionUID = -645320907771343431L;

	/**
	 * the bean's class that will be used to generate the result objects.
	 */
	private Class beanClass;

	/**
	 * Map of transformation grouped by this transformation with alias
	 */

	private Map<String, ITransformation> transformationMap = new HashMap<String, ITransformation>();

	/**
	 * Method that allows to add new transformations to this transformation list
	 * 
	 * @param theTranformation
	 *            the transformation to be added
	 * @param theAlias
	 *            the name that identifies the provided transformation that will
	 *            be the map key or the bean property
	 */

	public void add(ITransformation theTranformation, String theAlias) {
		this.transformationMap.put(theAlias, theTranformation);
	}

	/**
	 * Constructor that accepts the bean's class that will be used to generate
	 * the result objects.
	 * 
	 * @param theBeanClass
	 *            the bean's class that will be used to generate the result
	 *            objects.
	 */
	public AliasTransformationsMap(Class theBeanClass) {
		this.beanClass = theBeanClass;
	}

	/**
	 * Default constructor with default values.
	 */
	public AliasTransformationsMap() {
		return;
	}

	/**
	 * Getter of <code>beanClass</code> property.
	 * 
	 * @return <code>beanClass</code> property value.
	 * @see #beanClass
	 */

	public Class getBeanClass() {
		return this.beanClass;
	}

	/**
	 * Method that allows to obtain the list of transformations contained in
	 * this transformation list
	 * 
	 * @return list of transformations contained in this transformation list
	 */

	public Map<String, ITransformation> getTransformations() {
		return this.transformationMap;
	}

	/**
	 * Setter of <code>beanClass</code> property.
	 * 
	 * @param theBeanClass
	 *            The <code>beanClass</code> to set.
	 * @see #beanClass
	 */
	public void setBeanClass(Class theBeanClass) {
		this.beanClass = theBeanClass;
	}

}
