/**
 * PropertyTransformation.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:45:09 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.transformation;

/**
 * 
 * Transformation used to obtain a property from a query.
 * 
 * @version 1.0 (16/04/2005)
 * @author Ithamar Torres
 */

public class PropertyTransformation implements ITransformation {

	/**
	 * Holds the serialization id of the class
	 */

	private static final long serialVersionUID = 1946924817426420361L;

	/**
	 * Name of the property to obtain
	 */

	private String propertyName;

	/**
	 * Indicates if the transformation is grouped or not
	 */

	private boolean grouped;

	/**
	 * Constructor used to create a PropertyTransformation with the property
	 * used, and indicating if it is grouped
	 * 
	 * @param prop
	 *            Property name used in transformation
	 * @param grouped
	 *            Indicate if group results
	 */
	protected PropertyTransformation(String prop, boolean grouped) {
		this.propertyName = prop;
		this.grouped = grouped;
	}

	/**
	 * Constructor used to create a PropertyTransformation with the property
	 * used, and indicating if it is grouped
	 * 
	 * @param prop
	 *            Property name used in transformation
	 */
	protected PropertyTransformation(String prop) {
		this(prop, false);
	}

	/**
	 * Getter of <code>propertyName</code> property.
	 * 
	 * @return <code>propertyName</code> property value.
	 * @see #propertyName
	 */
	public String getPropertyName() {
		return this.propertyName;
	}

	/**
	 * Getter of <code>grouped</code> property.
	 * 
	 * 
	 * @return <code>grouped</code> property value.
	 * @see #grouped
	 */
	public boolean isGrouped() {
		return this.grouped;
	}
}
