/**
 * SmartProperties.java.java
 *
 * $Revision: 1.4 $ ($Date: 2011-04-06 04:46:26 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.properties;

import java.io.File;
import java.io.Serializable;

/**
 * Class that do the operations related to the application properties
 * initialization
 * 
 * @author Carlos Lameda (carlos.lameda@smartmatic.com)
 * @author Daniel Bastidas (daniel.bastidas@smartmatic.com)
 * @version 1.0 (Dec 17, 2009)
 */
public final class SmartProperties implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6223722840702630279L;

	/**
	 * Create the instance
	 */
	private static SmartProperties instance = new SmartProperties();

	/**
	 * Constructor
	 */
	private SmartProperties() {
	}

	/**
	 * Enumerated representing the the property file type
	 * 
	 * @author Argenis Cabeza
	 * @version 1.0 (May 11, 2006)
	 */
	public static enum Type {

		/**
		 * ENCRYPTED file type
		 */
		ENCRYPTED,

		/**
		 * PLAIN file type
		 */
		PLAIN;

	}

	/**
	 * Return the instance implementing a singleton
	 * 
	 * @return The instance of this class
	 */
	public static SmartProperties getInstance() {
		return SmartProperties.instance;
	}

	/**
	 * Initialize the property file according to the type
	 * 
	 * @param type
	 *            The type of the property
	 * @return the result of initializing process
	 */
	public static boolean init(Type type) {
		return ApplicationProperties.getInstance().init(type);
	}

	/**
	 * Initialize the properties
	 */
	public static void init() {
		File fileEncrypted = new File(
				System.getProperty("jboss.server.home.dir"),
				"conf/saes.server.properties.xml");
		if (fileEncrypted.exists() == true) {

			SmartProperties.init(SmartProperties.Type.ENCRYPTED);

		} else {
			SmartProperties.init(SmartProperties.Type.PLAIN);
		}
	}

}
