package com.smartmatic.waffle.properties;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.igfay.jfig.JFig;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

/**
 * Class used to write unit tests for ApplicationProperties.
 */
@RunWith(PowerMockRunner.class)
@PrepareForTest({ JFig.class })
public class ApplicationPropertiesTest {

	private JFig jFigInstance;

	@Before
	public void setup() {
		this.jFigInstance = (JFig) JFig.getInstance();
	}

	@Test
	public void noApplicationPropertiesInstantiatedTest() {
		boolean result = ApplicationProperties
				.isApplicationPropertiesInstantiated();

		assertFalse(result);
	}

	@Test
	public void applicationPropertiesInstantiatedTest() {

		ApplicationProperties.getInstance();

		boolean result = ApplicationProperties
				.isApplicationPropertiesInstantiated();

		assertTrue(result);
	}

	@Test
	public void applicationPropertiesNotInitTest() {

		PowerMockito.mockStatic(JFig.class);

		PowerMockito.when(
				JFig.getInstance(Mockito.any(SmartPropertiesLocator.class)))
				.thenReturn(null);

		ApplicationProperties.getInstance();

		boolean result = ApplicationProperties.getInstance().init(
				SmartProperties.Type.PLAIN);

		assertFalse(result);
	}

	@Test
	public void applicationPropertiesInitTest() {

		PowerMockito.mockStatic(JFig.class);

		PowerMockito.when(
				JFig.getInstance(Mockito.any(SmartPropertiesLocator.class)))
				.thenReturn(this.jFigInstance);

		ApplicationProperties.getInstance();

		boolean result = ApplicationProperties.getInstance().init(
				SmartProperties.Type.PLAIN);

		assertTrue(result);
	}
}
