/**
 * DistrictDataValidator.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.validator.demographics;

import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.validator.ObjectDataValidator;
import com.smartmatic.waffle.validator.ObjectDataValidatorException;
import com.smartmatic.waffle.validator.Validator;

/**
 * Class used to validate District Data.
 */
public class DistrictDataValidator extends ObjectDataValidator<DistrictData> {

	/**
	 * Check unassocited region.
	 * 
	 * @param component
	 *            the component
	 * @return true, if successful
	 */
	private boolean checkUnassocitedRegion(DistrictData component) {
		return component.getRegions() == null
				|| component.getRegions().isEmpty();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.validator.ObjectDataValidator#prePersist(java.lang.Object)
	 */
	@Override
	public void prePersist(DistrictData component)
			throws ObjectDataValidatorException {
		if (this.checkUnassocitedRegion(component)) {
			throw new DistrictDataValidationException(
					DistrictDataValidationException.ExceptionInfo.UNASSOCIATED_REGION,
					new Exception());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.validator.ObjectDataValidator#preUpdate(java.lang.Object)
	 */
	@Override
	public void preUpdate(DistrictData component)
			throws ObjectDataValidatorException {
		if (this.checkUnassocitedRegion(component)) {
			throw new DistrictDataValidationException(
					DistrictDataValidationException.ExceptionInfo.UNASSOCIATED_REGION,
					new Exception());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.validator.ObjectDataValidator#postPersist(java.lang.Object
	 * )
	 */
	@Override
	public void postPersist(DistrictData component)
			throws ObjectDataValidatorException {
		Validator v = new Validator();
		v.prePersist(component);
	}
}
