package com.smartmatic.waffle.validator.i18;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.validator.ObjectDataValidator;
import com.smartmatic.waffle.validator.ObjectDataValidatorException;

/**
 * Class used to validate language data.
 */
public class LanguageDataValidator extends ObjectDataValidator<LanguageData> {
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.validator.ObjectDataValidator#prePersist(java.lang.Object)
	 */
	@Override
	public void prePersist(LanguageData language)
			throws ObjectDataValidatorException {
		if (checkVoiceFileNames(language)) {
			throw new LanguageDataValidationException(
					LanguageDataValidationException.ExceptionInfo.INCOMPLETE_VOICE_INFORMATION,
					new Exception());
		}
	}
	
	/**
	 * Check for HDR and DAT files are present for voices.
	 * 
	 * @param language
	 *            The language to persist.
	 * @return True if one file was setted and the other one was missed.
	 */
	private boolean checkVoiceFileNames(LanguageData language) {
		if (language.getVoiceNameDAT() == null
				|| language.getVoiceNameDAT().isEmpty()) {
			return language.getVoiceNameHDR() != null
					&& !language.getVoiceNameHDR().isEmpty();
		} else {
			return language.getVoiceNameHDR() == null
					|| language.getVoiceNameHDR().isEmpty();
		}
	}

}
