/**
 * JNDICommunicationException.java
 *
 * $Revision: 1.4 $ ($Date: 2011-04-06 05:08:28 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.client;

import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.lang.IExceptionInfo;

/**
 * Exception Java Naming and Directory Interface Communication
 * 
 * @author Carlos Lameda (carlos.lameda@smartmatic.com)
 * @version 1.0 (Nov 5, 2009)
 */
public class JNDICommunicationException extends BaseException {
	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	private static final long serialVersionUID = -7351410891184276848L;

	/**
	 * Enum that contains all the possible reasons because the
	 * JNICommunicationException can be thrown
	 * 
	 * 
	 * @version 1.0 (23/10/2006)
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	public enum ExceptionInfo implements IExceptionInfo {

		CLIENT_NOT_INITIALIZED("CLIENT_NOT_INITIALIZED"), CLIENT_ALREADY_INITIALIZED(

		"CLIENT_ALREADY_INITIALIZED"), NAMING_EXCEPTION("NAMING_EXCEPTION");
		/**
		 * <p>
		 * Each enumerator's resource bundle's key message.
		 * </p>
		 */

		private String message;

		/**
		 * Getter of <code>message</code> property.
		 * 
		 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
		 * @return <code>message</code> property value.
		 * @see #message
		 */
		@Override
		public String getMessage() {
			return this.message;
		}

		/**
		 * Constructor
		 * 
		 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
		 * @param msg
		 *            the resource bundle's key message
		 */
		private ExceptionInfo(String msg) {
			this.message = this.getClass().getName() + "." + msg;
		}
	}

	/**
	 * Creates a instance of a JNICommunicationException
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param exceptionInfo
	 *            The reason because the exception is being thrown
	 */
	public JNDICommunicationException(IExceptionInfo exceptionInfo) {
		super(exceptionInfo);
		return;
	}

	/**
	 * Creates a instance of a JNICommunicationException
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param exceptionInfo
	 *            The reason because the exception is being thrown
	 * @param e
	 *            params used to give information about the exception
	 */
	public JNDICommunicationException(IExceptionInfo exceptionInfo, Throwable e) {
		super(exceptionInfo, e);
		return;
	}

	/**
	 * Creates a instance of a JNICommunicationException
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param exceptionInfo
	 *            The reason because the exception is being thrown
	 * @param parameters
	 *            params used to give information about the exception
	 */
	public JNDICommunicationException(IExceptionInfo exceptionInfo,
			Object... parameters) {
		super(exceptionInfo, parameters);
		return;
	}
}
