/**
 * ExceptionHandler.java
 *
 * $Revision: 1.5 $ ($Date: 2011-04-06 05:08:29 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.lang.util;

import java.lang.reflect.InvocationTargetException;

import com.smartmatic.waffle.lang.BaseException;

/**
 * Class used to wrap a RuntimeExcception into a BaseException if it has one
 * inside.
 * 
 * @author Armando Torres (armando.torres@smartmatic.com)
 * @version 1.0 (Nov 02, 2006)
 */
public class ExceptionHandler {

	/**
	 * Method used to wrap an Exception into a BaseException
	 * 
	 * @author Armando Torres (armando.torres@smartmatic.com)
	 * @param Exception
	 *            ex The exception to be wrapped.
	 * @return BaseException The exception wrapped.
	 */
	public static BaseException wrapException(Exception ex) {
		BaseException exception = null;
		InvocationTargetException invTargetEx = (InvocationTargetException) ex
				.getCause();
		exception = (BaseException) invTargetEx.getTargetException();
		return exception;
	}

	/**
	 * Method used to verify that an Exception contains a BaseException wrapped.
	 * 
	 * @author Armando Torres (armando.torres@smartmatic.com)
	 * @param Exception
	 *            ex The exception to verify.
	 * @return boolean true if it has been wrapped, false if it's an
	 *         RuntimeException.
	 */
	public static boolean isWrapOfBaseException(Exception ex) {
		if (ex instanceof RuntimeException) {
			if (ex.getCause() != null
					&& ex.getCause() instanceof InvocationTargetException) {
				InvocationTargetException invTargetEx = (InvocationTargetException) ex
						.getCause();
				if (invTargetEx.getTargetException() != null
						&& invTargetEx.getTargetException() instanceof BaseException) {
					return true;
				}
			}
		} else if (ex instanceof BaseException) {
			return true;
		}
		return false;
	}
}
