/**
 * BundleKeyFormatter.java
 *
 * $Revision: 1.5 $ ($Date: 2011-04-06 05:08:31 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.text.impl;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;

import com.smartmatic.waffle.text.spec.ITextFormatter;

/**
 * Formats all the messages required based in a properties file that contains
 * the messages that has to be used to match the received key.
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 * @version 1.0 (15/11/2006)
 */
public class BundleKeyFormatter implements ITextFormatter {
	/**
	 * Holds the class used to locate the resources required by this class.
	 */

	private ResourceBundle resBun;

	/**
	 * Creates a new instance of a BundleKeyFormatter.
	 * 
	 * @param fileName
	 *            The file name where are located the bundle keys
	 */

	public BundleKeyFormatter(String fileName) {
		this.resBun = ResourceBundle.getBundle(fileName);
		// TODO: Implement locale specific behavior.
		// this.resBun = ResourceBundle.getBundle(fileName,
		// LocaleUtil.getConfiguredLocale());
	}

	/**
	 * Loads a java ResourceBundle given the file name and the specific region
	 * 
	 * @param fileName
	 *            Resource file name
	 * @param locale
	 *            The Specific region
	 */
	public BundleKeyFormatter(String fileName, Locale locale) {
		this.resBun = ResourceBundle.getBundle(fileName, locale);
	}

	/**
	 * Method that returns a key formatted.
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param key
	 *            The message key that needs to be formatted.
	 * @return The text formatted.
	 */
	@Override
	public String getText(String key) {
		return this.getText(key, "");
	}

	/**
	 * Method that returns a key formatted.
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param key
	 *            The message key that needs to be formatted.
	 * @param params
	 *            The params used to match the parameters founded in the message
	 * @return The text formatted.
	 */
	@Override
	public String getText(String key, Object... params) {
		String text = key;
		try {
			text = this.resBun.getString(key);
		} catch (Exception e) {
		}
		return MessageFormat.format(text, params);
	}

}
