/**
 * LocalContainerAppVersion.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.util;

/**
 * Class used to associate a local implementation for each jboss container
 * version.
 * 
 * @version 1.0 (Nov 2, 2010)
 * @author Hector Rodriguez (hector.rodriguez@smartmatic.com)
 */

public enum LocalContainerAppVersion {

	/** The v5_1_0_ ga. */
	v5_1_0_GA("org.jboss.ejb3.mdb.MessagingDelegateWrapper", "startDelivery",
			"stopDelivery", "DeliveryActive", "shutdown"),

	/** The v4_2_2_ ga. */
	v4_2_2_GA("org.jboss.ejb3.mdb.MdbDelegateWrapper", "start", "stop",
			"StateString", "exit");

	/** The mdb delegate wrapper. */
	private String mdbDelegateWrapper;

	/** The start method name. */
	private String startMethodName;

	/** The stop method name. */
	private String stopMethodName;

	/** The state name. */
	private String stateName;

	/** The shutdown method name. */
	private String shutdownMethodName;

	/**
	 * Instantiates a new local container app version.
	 * 
	 * @param MDBDelegateWrapperClass
	 *            the mDB delegate wrapper class
	 * @param startName
	 *            the start name
	 * @param stopName
	 *            the stop name
	 * @param status
	 *            the status
	 * @param shutdown
	 *            the shutdown
	 */
	private LocalContainerAppVersion(String MDBDelegateWrapperClass,
			String startName, String stopName, String status, String shutdown) {
		this.mdbDelegateWrapper = MDBDelegateWrapperClass;
		this.startMethodName = startName;
		this.stopMethodName = stopName;
		this.stateName = status;
		this.shutdownMethodName = shutdown;
	}

	/**
	 * Gets the mdb delegate wrapper.
	 * 
	 * @return the mdb delegate wrapper
	 */
	public String getMdbDelegateWrapper() {
		return this.mdbDelegateWrapper;
	}

	/**
	 * Gets the start method name.
	 * 
	 * @return the start method name
	 */
	public String getStartMethodName() {
		return this.startMethodName;
	}

	/**
	 * Gets the stop method name.
	 * 
	 * @return the stop method name
	 */
	public String getStopMethodName() {
		return this.stopMethodName;
	}

	/**
	 * Gets the state name.
	 * 
	 * @return the state name
	 */
	public String getStateName() {
		return this.stateName;
	}

	/**
	 * Gets the shutdown method name.
	 * 
	 * @return the shutdown method name
	 */
	public String getShutdownMethodName() {
		return this.shutdownMethodName;
	}

	/**
	 * Gets the instance by version.
	 * 
	 * @param version
	 *            the version
	 * @return the instance by version
	 */
	public static LocalContainerAppVersion getInstanceByVersion(Object version) {
		String vInstance = (String) version;
		vInstance = "v" + vInstance.replace(".", "_");
		LocalContainerAppVersion reference = Enum.valueOf(
				LocalContainerAppVersion.class, vInstance);
		return reference;
	}

}
