/**
 * IObjectDataValidator.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 05:08:28 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.validator;

/**
 * Contract that defines the required methods in a validator.
 * 
 * @author Julio Ynojosa (jynojosa@smartmatic.com)
 * @version 1.0 (04/09/2006)
 */
public interface IObjectDataValidator<T> {

	/**
	 * Method to execute before persisting a component. It must implements
	 * business rules.
	 * 
	 * @author Julio Ynojosa (jynojosa@smartmatic.com)
	 * @param component
	 *            to be saved.
	 */
	void prePersist(T component) throws ObjectDataValidatorException;

	/**
	 * Method to execute after persisting a component. It must implements
	 * business rules.
	 * 
	 * @author Julio Ynojosa (jynojosa@smartmatic.com)
	 * @param the
	 *            saved component.
	 */
	void postPersist(T component) throws ObjectDataValidatorException;

	/**
	 * Method to execute before removing a component. It must implements
	 * business rules.
	 * 
	 * @author Julio Ynojosa (jynojosa@smartmatic.com)
	 * @param the
	 *            component to remove.
	 */
	void preRemove(T component) throws ObjectDataValidatorException;

	/**
	 * Method to execute after removing a component. It must implements business
	 * rules.
	 * 
	 * @author Julio Ynojosa (jynojosa@smartmatic.com)
	 * @param the
	 *            removed component.
	 */
	void postRemove(T component) throws ObjectDataValidatorException;

	/**
	 * Method to execute before updating a component. It must implements
	 * business rules.
	 * 
	 * @author Julio Ynojosa (jynojosa@smartmatic.com)
	 * @param the
	 *            component to update.
	 */
	void preUpdate(T component) throws ObjectDataValidatorException;

	/**
	 * Method to execute after updating a component. It must implements business
	 * rules.
	 * 
	 * @author Julio Ynojosa (jynojosa@smartmatic.com)
	 * @param the
	 *            updated component.
	 */
	void postUpdate(T component) throws ObjectDataValidatorException;
}
