package com.smartmatic.waffle.lang.util;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashSet;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.powermock.modules.junit4.PowerMockRunner;

/**
 * Class used to write unit tests for ReflectHelper.
 */
@RunWith(PowerMockRunner.class)
public class ReflectHelperTest {

	@Test
	public void getFieldFromClassTest() throws NoSuchFieldException {
		Field field = ReflectHelper.getFieldFromClass(WaffleClass.class,
				"waffleAttribute");
		Assert.assertNotNull(field);
		Assert.assertEquals("waffleAttribute", field.getName());

	}

	@Test
	public void getFieldFromClassSuperTest() throws NoSuchFieldException {
		Field field = ReflectHelper.getFieldFromClass(WaffleClass.class,
				"superAttribute");
		Assert.assertNotNull(field);
		Assert.assertEquals("superAttribute", field.getName());

	}

	@Test(expected = NoSuchFieldException.class)
	public void getFieldFromClassNoAttributeTest() throws NoSuchFieldException {
		Field field = ReflectHelper.getFieldFromClass(WaffleClass.class,
				"newWaffleAttribute");

	}

	@Test
	public void getPropertyNameFromGetterTest() {
		HashSet<String> propertyNames = new HashSet<String>();
		WaffleClass waffleClass = new WaffleClass();
		String propertyName = "";
		for (Method method : waffleClass.getClass().getMethods()) {
			propertyName = ReflectHelper.getPropertyNameFromGetter(method);
			if (!propertyNames.contains(propertyName)
					&& ("superAttribute".equals(propertyName) || "waffleAttribute"
							.equalsIgnoreCase(propertyName))) {
				propertyNames.add(propertyName);
			}
		}

		Assert.assertEquals(propertyNames.size(), 2);
		Assert.assertTrue(propertyNames.remove("superAttribute"));
		Assert.assertTrue(propertyNames.remove("waffleAttribute"));
	}

	@Test
	public void invokeGetterTest() throws SecurityException,
			IllegalArgumentException, NoSuchMethodException,
			IllegalAccessException, InvocationTargetException {
		WaffleClass waffleClass = new WaffleClass(1);

		int returnValue = (Integer) ReflectHelper.invokeGetter(waffleClass,
				"waffleAttribute");

		Assert.assertEquals(returnValue, waffleClass.getWaffleAttribute());
	}

	@Test
	public void invokeSetterTest() throws SecurityException, IllegalArgumentException, NoSuchMethodException, NoSuchFieldException, IllegalAccessException, InvocationTargetException {
		WaffleClass waffleClass = new WaffleClass();

		ReflectHelper.invokeSetter(waffleClass, "waffleAttribute", 1);

		Assert.assertEquals(waffleClass.getWaffleAttribute(), 1);
	}
}

class SuperWaffleClass {
	private boolean superAttribute;

	/**
	 * @return the superAttribute
	 */
	public boolean isSuperAttribute() {
		return this.superAttribute;
	}

	/**
	 * @param superAttribute
	 *            the superAttribute to set
	 */
	public void setSuperAttribute(boolean superAttribute) {
		this.superAttribute = superAttribute;
	}
}

class WaffleClass extends SuperWaffleClass {
	private int waffleAttribute;

	public WaffleClass() {
		super();
	}

	public WaffleClass(int waffleAttribute) {
		super();
		this.waffleAttribute = waffleAttribute;
	}

	/**
	 * @return the waffleAttribute
	 */
	public int getWaffleAttribute() {
		return this.waffleAttribute;
	}

	/**
	 * @param waffleAttribute
	 *            the waffleAttribute to set
	 */
	public void setWaffleAttribute(int waffleAttribute) {
		this.waffleAttribute = waffleAttribute;
	}

}
