/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * main.cc
 * Copyright (C) Freddy Gomez 2011 <fgomez@smartmatic.com>
 *
 * BelgiumVM is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Election is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <Operation/functionality-initiator.h>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Devices/SmartcardReaderPowerManagerFactory.hxx>
#include <Card/ISmartcardReaderPowerManager.hxx>
#include <Operation/Devices/PDBController.hxx>
#include <Operation/Devices/IOBController.hxx>
#include <Printing/IB31K1PrintingBackend.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#ifndef GIT_COMMIT_HASH
#define GIT_COMMIT_HASH "unavailable"
#endif

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

#include <stdio.h>
#include <signal.h>
#include <unistd.h>

#include "Config.hxx"
#include "RunFunctionality.hxx"
#include <System/StackTraceManager.hxx>

using namespace Smartmatic::SAES::Election;

static void handleSignal(int signal)
{
    if (signal == SIGUSR1)
    {
        ISmartcardReaderPowerManager* smartcardPower =
            SmartcardReaderPowerManagerFactory::getSmartcardReaderPowerManager();

        smartcardPower->doReaderPowerCycle();
    }
}

static void configureNeopSensor()
{
    Smartmatic::SAES::Printing::IB31K1PrintingBackend printer;

    printer.configureNeopSensor();
    printer.End();
}

int main ( int argc, char * argv[] )
{
    struct sigaction sact;
    sigemptyset ( &sact.sa_mask );
    sact.sa_flags = SA_RESTART;
    sact.sa_handler = &handleSignal;
    sigaction ( SIGUSR1, &sact, NULL );
    sigaction ( SIGUSR2, &sact, NULL );

    if ( !Glib::thread_supported() )
    {
        Glib::thread_init();
    }

    Gtk::Main app ( argc, argv );

    Smartmatic::System::StackTraceManager::getInstance()->start();

    OptParser::getInstance()->initialize ( argc, argv );

    std::string pathLocaleDir;
    pathLocaleDir = OptParser::getInstance()->isSet ( OptParser::LOCALE_DIR ) && OptParser::getInstance()->getArg ( OptParser::LOCALE_DIR, pathLocaleDir ) ? pathLocaleDir : GNOMELOCALEDIR;

    bindtextdomain ( GETTEXT_PACKAGE, pathLocaleDir.c_str() );
    bind_textdomain_codeset ( GETTEXT_PACKAGE, "UTF-8" );
    textdomain ( pathLocaleDir.c_str() );
    setlocale ( LC_ALL, "" );

    OptParser::getInstance()->runApplicationProcess ( PACKAGE_NAME, CONFDIR );

    Config::start();
    ISMTTLog * logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Election", GETTEXT_PACKAGE );
    logger->Debug ( _ ( "Program name: " ) + std::string ( PACKAGE_NAME ) );
    logger->Debug ( _ ( "Version: " ) + std::string ( VERSION ) );
    logger->Debug ( _ ( "Build: " ) + std::string ( GIT_COMMIT_HASH ) );

    FunctionalityInitiator::Register();

    bool loadLanguage = SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getLoadSelectionLanguage();

    if ( loadLanguage )
    {
        logger->Debug ( "Load language file " );
        Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration()->loadDefaultLanguages();
    }
    else
    {
        logger->Debug ( "Skip language file " );
    }

    RunFunctionality::start();

    std::string machineModel;
    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

    if (StringUtils::toUpper(machineModel) == "VM-A4517")
    {
        PDBController::getInstance()->getFWVersion();
        PDBController::getInstance()->turnAllLedsOff();
        Glib::Thread::create(sigc::ptr_fun(&configureNeopSensor), false);
    }
    else if (StringUtils::toUpper(machineModel) == "PM-VIU805")
    {
        IOBController::getInstance()->getFWVersion();
    }

    Gtk::Main::run();

    return 0;
}
