/**
 * @file	DiagnosticLogConfigurationManager.hxx
 * @brief	Declaration of the diagnostic log configuration manager.
 * @author	Yadickson Soto
 * @date	15/04/2014
 */

#ifndef DIAGNOSTICLOGCONFIGURATIONMANAGER_HXX_
#define DIAGNOSTICLOGCONFIGURATIONMANAGER_HXX_

#include <Configuration/diagnostic-log-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Configuration
        {
            /**
             * @class	DiagnosticLogConfigurationManager
             * @brief	The diagnostic log configuration manager.
             * @author	Yadickson Soto
             * @date	15/04/2014
             */
            class DiagnosticLogConfigurationManager
            {
                public:
                    /**
                     * @fn	static DiagnosticLogConfigurationManager * getInstance( DiagnosticLogConfigurationManager * externManager = NULL );
                     * @brief	Gets the current Diagnostic Log Configuration Manager.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     * @param 	externManager external manager to unit test
                     * @return	instance of the class.
                     */
                    static DiagnosticLogConfigurationManager * getInstance ( DiagnosticLogConfigurationManager * externManager = NULL );

                    /**
                     * @fn	virtual bool shouldPersistDiagnosticLogFile();
                     * @brief	Whether to persist diagnostic file.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     * @return	true if persist diagnostic file.
                     */
                    virtual bool shouldPersistDiagnosticLogFile();

                    /**
                     * @fn	virtual std::string getSeparator();
                     * @brief	Gets the field separator value.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     * @return	the field separator value.
                     */
                    virtual std::string getSeparator();

                    /**
                     * @fn	virtual std::string getStringOk();
                     * @brief	Gets the status value if diagnostic is success.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     * @return	the status value if diagnostic is success.
                     */
                    virtual std::string getStringOk();

                    /**
                     * @fn	virtual std::string getSeparator();
                     * @brief	Gets the status value if diagnostic is fail.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     * @return	the status value if diagnostic is fail.
                     */
                    virtual std::string getStringNotOk();

                    /**
                     * @fn	virtual std::string getSeparator();
                     * @brief	Gets the status value if diagnostic N/A.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     * @return	the status value if diagnostic N/A.
                     */
                    virtual std::string getStringNA();

                    /**
                     * @fn	virtual ~DiagnosticLogConfigurationManager();
                     * @brief	Class destructor.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     */
                    virtual ~DiagnosticLogConfigurationManager();

                protected:

                    /**
                     * @fn	DiagnosticLogConfigurationManager();
                     * @brief	Class constructor.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     */
                    DiagnosticLogConfigurationManager();

                    /**
                     * @fn	virtual DiagnosticLogConfiguration & getDiagnosticLogConfiguration ();
                     * @brief	Gets a reference to the context of the diagnostic log configuration.
                     * @author	Yadickson Soto
                     * @date	15/04/2014
                     * @return	The reference to the context of the diagnostic log configuration
                     */
                    virtual DiagnosticLogConfiguration & getDiagnosticLogConfiguration ();

                private:

                    static DiagnosticLogConfigurationManager * instance; ///< The pointer to the singleton instance of the diagnostic log configuration manager

            };

        } /* namespace Configuration */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* DIAGNOSTICLOGCONFIGURATIONMANAGER_HXX_ */
