/**
 * @file	s-a-e-s-conf-manager.cc
 * @brief	saes conf manager class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#include "s-a-e-s-conf-manager.h"
#include <Configuration/configuration-persistence-manager.h>
#include <System/ZenityWrapper.hxx>
#include <System/Runtime/ExitProcess.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Runtime;

SAESConfigurationManager * SAESConfigurationManager::instance = NULL;

SAESConfigurationManager::SAESConfigurationManager()
{
    configuration = NULL;
}

SAESConfigurationManager::~SAESConfigurationManager()
{
    if ( configuration != NULL )
    {
        delete ( configuration );
        configuration = NULL;
    }

    instance = NULL;
}

SaesConfiguration * SAESConfigurationManager::getConfiguration()
{
    if ( configuration == NULL )
    {
        initialize();
    }

    return configuration;
}

void SAESConfigurationManager::initialize()
{
    try
    {
        std::string semiFile ( ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "SAES" ) );
        std::istringstream ss ( semiFile );
        configuration = new SaesConfiguration ( * parseSaesConfiguration ( ss, ConfigurationTypes::Flags::dont_validate ).get() );
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << e;
        ZenityWrapper::showError ( "Config SAES Error \n\n" + ss.str(), "Error" );
        ExitProcess::getInstance()->exitProcess();
    }
}

void SAESConfigurationManager::loadDefault()
{
    SaesConfiguration configSAES;

    ConfigurationReportPrinting reportConf;
    ConfigurationReportFile fileConfig;

    fileConfig.setFileDescription ( N_ ( "Smartmatic.SAES.Printing.ClosedPrintDocument.PVDescription" ) );
    fileConfig.setFileName ( "pv-report.pdf" );
    fileConfig.setReportTitle ( N_ ( "Smartmatic.SAES.Printing.ClosedPrintDocument.PVTitle" ) );
    fileConfig.setNumCopies ( 1 );

    ConfigurationReportPrinting::ReportFilesSequence rceq;

    rceq.push_back ( fileConfig );
    reportConf.setReportFiles ( rceq );

    ConfigurationPrinter configurationPrinter;
    ConfigurationScreenPreview configurationScreenPreview;
    ConfigurationCard configurationCard;
    ConfigurationVote configurationVote;
    ConfigurationTimeout configurationTimeout;
    ConfigurationSmartcardRemoved configurationSmartcardRemoved;
    ConfigurationInstallation configurationInstallation;
    ConfigurationAudioResourceTreatment configurationAudioResourceTreatment;

    LogPrintDocumentConfiguration logPrintDocConf;
    TextConfiguration textHeader1;
    TextConfiguration textHeader2;
    TextConfiguration textHeader3;
    TextConfiguration textHeader4;
    TextConfiguration textHeader5;
    TextConfiguration textHeader6;
    TextConfiguration textHeader7;
    TextConfiguration textHeaderCustomAtt;
    TextConfiguration textBallotName;
    TextConfiguration textRegionName;
    TextConfiguration textContestName1;
    TextConfiguration textContestName2;
    TextConfiguration textContestName3;
    TextConfiguration textParentOptionSelected;
    TextConfiguration textPrincipalsMessage;
    TextConfiguration textStandInMessage;
    TextConfiguration textBallotOption;
    TextConfiguration textGeneric;
    TextConfiguration textBlankVote;
    TextConfiguration textTitleTimeout;
    TextConfiguration textElectionNameTimeout;
    TextConfiguration textVotingBoothTimeout;
    TextConfiguration textVoteIDTimeout;
    TextConfiguration textDescriptionTimeout;
    TextConfiguration textTitleSmartcardRemoved;
    TextConfiguration textElectionNameSmartcardRemoved;
    TextConfiguration textVotingBoothSmartcardRemoved;
    TextConfiguration textVoteIDSmartcardRemoved;
    TextConfiguration textDescriptionSmartcardRemoved;
    TextConfiguration textInstallationTitleLevel1;
    TextConfiguration textInstallationTitleLevel2;
    TextConfiguration textInstallationTitleLevel3;
    TextConfiguration textInstallationInfo;
    TextConfiguration textLogTitleLevel1;
    TextConfiguration textLogTitleLevel2;
    TextConfiguration textLogTitleLevel3;
    TextConfiguration textLogTitleLevel4;
    TextConfiguration textLogTitleLevel5;
    TextConfiguration textLogEntry;

    LayoutContestConfiguration layoutContestConfiguration;

    textHeader1.setSize ( 12 );
    textHeader1.setFontFamily ( "FreeSans" );
    textHeader1.setBold ( false );
    textHeader1.setItalic ( false );
    textHeader1.setHPandding ( 0 );
    textHeader1.setVPandding ( 1 );
    textHeader1.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textHeader1.setLevelPaddingSpaceChar ( 0 );
    textHeader1.setVisible ( true );

    textHeader2.setSize ( 8 );
    textHeader2.setFontFamily ( "FreeSans" );
    textHeader2.setBold ( true );
    textHeader2.setItalic ( true );
    textHeader2.setHPandding ( 0 );
    textHeader2.setVPandding ( 1 );
    textHeader2.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textHeader2.setLevelPaddingSpaceChar ( 0 );
    textHeader2.setVisible ( true );

    textHeader3.setSize ( 9 );
    textHeader3.setFontFamily ( "FreeSans" );
    textHeader3.setBold ( false );
    textHeader3.setItalic ( false );
    textHeader3.setHPandding ( 0 );
    textHeader3.setVPandding ( 0 );
    textHeader3.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textHeader3.setLevelPaddingSpaceChar ( 0 );
    textHeader3.setVisible ( true );

    textHeader4.setSize ( 9 );
    textHeader4.setFontFamily ( "FreeSans" );
    textHeader4.setBold ( false );
    textHeader4.setItalic ( false );
    textHeader4.setHPandding ( 0 );
    textHeader4.setVPandding ( 1 );
    textHeader4.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textHeader4.setLevelPaddingSpaceChar ( 0 );
    textHeader4.setVisible ( true );

    textHeader5.setSize ( 9 );
    textHeader5.setFontFamily ( "FreeSans" );
    textHeader5.setBold ( false );
    textHeader5.setItalic ( false );
    textHeader5.setHPandding ( 0 );
    textHeader5.setVPandding ( 1 );
    textHeader5.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textHeader5.setLevelPaddingSpaceChar ( 0 );
    textHeader5.setVisible ( true );

    textHeader6.setSize ( 9 );
    textHeader6.setFontFamily ( "FreeSans" );
    textHeader6.setBold ( false );
    textHeader6.setItalic ( false );
    textHeader6.setHPandding ( 0 );
    textHeader6.setVPandding ( 1 );
    textHeader6.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textHeader6.setLevelPaddingSpaceChar ( 0 );
    textHeader6.setVisible ( true );

    textHeader7.setSize ( 9 );
    textHeader7.setFontFamily ( "FreeSans" );
    textHeader7.setBold ( false );
    textHeader7.setItalic ( false );
    textHeader7.setHPandding ( 0 );
    textHeader7.setVPandding ( 1 );
    textHeader7.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textHeader7.setLevelPaddingSpaceChar ( 0 );
    textHeader7.setVisible ( true );

    textHeaderCustomAtt.setSize ( 9 );
    textHeaderCustomAtt.setFontFamily ( "FreeSans" );
    textHeaderCustomAtt.setBold ( false );
    textHeaderCustomAtt.setItalic ( false );
    textHeaderCustomAtt.setHPandding ( 0 );
    textHeaderCustomAtt.setVPandding ( 1 );
    textHeaderCustomAtt.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textHeaderCustomAtt.setLevelPaddingSpaceChar ( 0 );
    textHeaderCustomAtt.setVisible ( true );

    textBallotName.setSize ( 12 );
    textBallotName.setFontFamily ( "FreeSans" );
    textBallotName.setBold ( true );
    textBallotName.setItalic ( false );
    textBallotName.setHPandding ( 0 );
    textBallotName.setVPandding ( 1 );
    textBallotName.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textBallotName.setLevelPaddingSpaceChar ( 0 );
    textBallotName.setVisible ( false );

    textRegionName.setSize ( 5 );
    textRegionName.setFontFamily ( "FreeSans" );
    textRegionName.setBold ( false );
    textRegionName.setItalic ( false );
    textRegionName.setHPandding ( 0 );
    textRegionName.setVPandding ( 1 );
    textRegionName.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textRegionName.setLevelPaddingSpaceChar ( 1 );
    textRegionName.setVisible ( true );

    textContestName1.setSize ( 9 );
    textContestName1.setFontFamily ( "FreeSans" );
    textContestName1.setBold ( true );
    textContestName1.setItalic ( false );
    textContestName1.setHPandding ( 0 );
    textContestName1.setVPandding ( 1 );
    textContestName1.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textContestName1.setLevelPaddingSpaceChar ( 2 );
    textContestName1.setVisible ( true );

    textContestName2.setSize ( 9 );
    textContestName2.setFontFamily ( "FreeSans" );
    textContestName2.setBold ( true );
    textContestName2.setItalic ( false );
    textContestName2.setHPandding ( 0 );
    textContestName2.setVPandding ( 1 );
    textContestName2.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_RIGHT );
    textContestName2.setLevelPaddingSpaceChar ( 2 );
    textContestName2.setVisible ( true );

    textContestName3.setSize ( 9 );
    textContestName3.setFontFamily ( "FreeSans" );
    textContestName3.setBold ( true );
    textContestName3.setItalic ( false );
    textContestName3.setHPandding ( 0 );
    textContestName3.setVPandding ( 1 );
    textContestName3.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textContestName3.setLevelPaddingSpaceChar ( 2 );
    textContestName3.setVisible ( true );

    textParentOptionSelected.setSize ( 12 );
    textParentOptionSelected.setFontFamily ( "FreeSans" );
    textParentOptionSelected.setBold ( true );
    textParentOptionSelected.setItalic ( false );
    textParentOptionSelected.setHPandding ( 0 );
    textParentOptionSelected.setVPandding ( 1 );
    textParentOptionSelected.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textParentOptionSelected.setLevelPaddingSpaceChar ( 2 );
    textParentOptionSelected.setVisible ( true );

    textPrincipalsMessage.setSize ( 12 );
    textPrincipalsMessage.setFontFamily ( "FreeSans" );
    textPrincipalsMessage.setBold ( true );
    textPrincipalsMessage.setItalic ( false );
    textPrincipalsMessage.setHPandding ( 0 );
    textPrincipalsMessage.setVPandding ( 1 );
    textPrincipalsMessage.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textPrincipalsMessage.setLevelPaddingSpaceChar ( 2 );
    textPrincipalsMessage.setVisible ( true );

    textStandInMessage.setSize ( 12 );
    textStandInMessage.setFontFamily ( "FreeSans" );
    textStandInMessage.setBold ( true );
    textStandInMessage.setItalic ( false );
    textStandInMessage.setHPandding ( 0 );
    textStandInMessage.setVPandding ( 1 );
    textStandInMessage.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textStandInMessage.setLevelPaddingSpaceChar ( 2 );
    textStandInMessage.setVisible ( true );

    textBallotOption.setSize ( 9 );
    textBallotOption.setFontFamily ( "FreeSans" );
    textBallotOption.setBold ( false );
    textBallotOption.setItalic ( true );
    textBallotOption.setHPandding ( 14 );
    textBallotOption.setVPandding ( 1 );
    textBallotOption.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textBallotOption.setLevelPaddingSpaceChar ( 1 );
    textBallotOption.setVisible ( true );

    textGeneric.setSize ( 9 );
    textGeneric.setFontFamily ( "FreeSans" );
    textGeneric.setBold ( false );
    textGeneric.setItalic ( false );
    textGeneric.setHPandding ( 0 );
    textGeneric.setVPandding ( 1 );
    textGeneric.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textGeneric.setLevelPaddingSpaceChar ( 0 );
    textGeneric.setVisible ( true );

    textBlankVote.setSize ( 12 );
    textBlankVote.setFontFamily ( "FreeSans" );
    textBlankVote.setBold ( true );
    textBlankVote.setItalic ( false );
    textBlankVote.setHPandding ( 0 );
    textBlankVote.setVPandding ( 1 );
    textBlankVote.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textBlankVote.setLevelPaddingSpaceChar ( 0 );
    textBlankVote.setVisible ( true );

    layoutContestConfiguration.setLayoutContestType ( Smartmatic::SAES::Configuration::LayoutContestType::CONFIGURATION_TRUNCATED );
    layoutContestConfiguration.setCompleteWithSpaces ( true );
    layoutContestConfiguration.setTextBallotOption ( textBallotOption );

    configurationVote.setTextHeader1 ( textHeader1 );
    configurationVote.setTextHeader2 ( textHeader2 );
    configurationVote.setTextHeader3 ( textHeader3 );
    configurationVote.setTextHeader4 ( textHeader4 );
    configurationVote.setTextHeader5 ( textHeader5 );
    configurationVote.setTextHeader6 ( textHeader6 );
    configurationVote.setTextHeader7 ( textHeader7 );
    configurationVote.setTextHeaderCustomAttributes ( textHeaderCustomAtt );
    configurationVote.setTextBallotName ( textBallotName );
    configurationVote.setTextRegionName ( textRegionName );
    configurationVote.setTextContestName1 ( textContestName1 );
    configurationVote.setTextContestName2 ( textContestName2 );
    configurationVote.setTextContestName3 ( textContestName3 );
    configurationVote.setIsTextContestName1CustomAttribute ( false );
    configurationVote.setIsTextContestName2CustomAttribute ( true );
    configurationVote.setIsTextContestName3CustomAttribute ( false );
    configurationVote.setTextParentOptionSelected ( textParentOptionSelected );
    configurationVote.setTextPrincipalsMessage ( textPrincipalsMessage );
    configurationVote.setTextStandInMessage ( textStandInMessage );
    configurationVote.setTextGeneric ( textGeneric );
    configurationVote.setTextBlankVote ( textBlankVote );
    configurationVote.setLayoutContestConfiguration ( layoutContestConfiguration );
    configurationVote.setPaddingChar ( " " );
    configurationVote.setPrintToPdf ( false );
    configurationVote.setQRcodePosition ( Smartmatic::SAES::Configuration::QRcodePosition::CONFIGURATION_TOP_ALL );
    configurationVote.setQRCodePrintingMode ( Smartmatic::SAES::Configuration::QRCodePrintingModeType::CONFIGURATION_IMAGE_BASED );
    configurationVote.setMaxBallotOptionLastNameChars ( 22 );
    configurationVote.setMaxBallotOptionNameChars ( 1 );
    configurationVote.setMaxBallotOptionCodeChars ( 4 );
    configurationVote.setBallotOptionNameSeparator ( "#" );
    configurationVote.setDirectPrinting ( true );
    configurationVote.setLevelPaddingSpaceCharSize ( 2 );
    configurationVote.setCompleteVoteWithSpaces ( true );
    configurationVote.setUseMaxSizeBlankVote ( false );
    configurationVote.setShowBallotLines ( true );
    configurationVote.setLayoutBallotOptiontType ( Smartmatic::SAES::Configuration::LayoutBallotOptiontType::CONFIGURATION_SEPARATEPRINCIPALANDSTANDIN );
    configurationVote.setLayoutSuperBlankType ( Smartmatic::SAES::Configuration::LayoutSuperBlankType::CONFIGURATION_JUSTCONTESTCLASS );
    configurationVote.setAddBlankSpaceFullContest ( 1 );

    configurationAudioResourceTreatment.setSeparator("#");
    configurationAudioResourceTreatment.setEnableTreatment(false);

    MultiLanguageConfiguration multiLanguageConfiguration ( false, "/", 25 );
    configurationVote.setMultiLanguageConfiguration ( multiLanguageConfiguration );

    QRCodeVoteList qrCodeVoteList;
    QRCodeVoteConfig qrCodeVoteConfig1;
    QRCodeVoteConfig qrCodeVoteConfig2;

    QRCodeVoteList::QRCodeVoteConfigSequence seqQRCodeVote;

    QRCodeConfig qrConfig1;
    qrConfig1.setSizeQRCode ( 3 );
    qrConfig1.setMarginQRCode ( 2 );
    qrConfig1.setCorrectionModeQRCode ( CorrectionModeQRCode::CONFIGURATION_QR_ECLEVEL_L );
    qrConfig1.setVersionQRCode ( 0 );

    QRCodeConfig qrConfig2;
    qrConfig2.setSizeQRCode ( 3 );
    qrConfig2.setMarginQRCode ( 0 );
    qrConfig2.setCorrectionModeQRCode ( CorrectionModeQRCode::CONFIGURATION_QR_ECLEVEL_L );
    qrConfig2.setVersionQRCode ( 21 );

    qrCodeVoteConfig1.setMaxSize ( 1000 );
    qrCodeVoteConfig1.setQRCodeConfig ( qrConfig1 );
    qrCodeVoteConfig2.setMaxSize ( 3500 );
    qrCodeVoteConfig2.setQRCodeConfig ( qrConfig2 );

    seqQRCodeVote.push_back ( qrCodeVoteConfig1 );
    seqQRCodeVote.push_back ( qrCodeVoteConfig2 );

    qrCodeVoteList.setQRCodeVoteConfig ( seqQRCodeVote );

    configurationVote.setVoteQRCodeMode ( Smartmatic::SAES::Configuration::VoteQRCodeModeEnum::CONFIGURATION_XMLFULL );
    configurationVote.setQRCodeVoteList ( qrCodeVoteList );

    VoteCodeConfig voteCodeConfig;
    voteCodeConfig.setCountGroupVoteCode ( 11 );
    voteCodeConfig.setCharSeparatorVoteCode ( " " );

    configurationVote.setVoteCodeConfig ( voteCodeConfig );

    textTitleTimeout.setSize ( 12 );
    textTitleTimeout.setFontFamily ( "FreeSans" );
    textTitleTimeout.setBold ( true );
    textTitleTimeout.setItalic ( false );
    textTitleTimeout.setHPandding ( 0 );
    textTitleTimeout.setVPandding ( 1 );
    textTitleTimeout.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textTitleTimeout.setLevelPaddingSpaceChar ( 0 );
    textTitleTimeout.setVisible ( true );

    textElectionNameTimeout.setSize ( 12 );
    textElectionNameTimeout.setFontFamily ( "FreeSans" );
    textElectionNameTimeout.setBold ( true );
    textElectionNameTimeout.setItalic ( true );
    textElectionNameTimeout.setHPandding ( 0 );
    textElectionNameTimeout.setVPandding ( 1 );
    textElectionNameTimeout.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textElectionNameTimeout.setLevelPaddingSpaceChar ( 0 );
    textElectionNameTimeout.setVisible ( true );

    textVotingBoothTimeout.setSize ( 12 );
    textVotingBoothTimeout.setFontFamily ( "FreeSans" );
    textVotingBoothTimeout.setBold ( false );
    textVotingBoothTimeout.setItalic ( false );
    textVotingBoothTimeout.setHPandding ( 0 );
    textVotingBoothTimeout.setVPandding ( 0 );
    textVotingBoothTimeout.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textVotingBoothTimeout.setLevelPaddingSpaceChar ( 0 );
    textVotingBoothTimeout.setVisible ( true );

    textVoteIDTimeout.setSize ( 12 );
    textVoteIDTimeout.setFontFamily ( "FreeSans" );
    textVoteIDTimeout.setBold ( false );
    textVoteIDTimeout.setItalic ( false );
    textVoteIDTimeout.setHPandding ( 0 );
    textVoteIDTimeout.setVPandding ( 1 );
    textVoteIDTimeout.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textVoteIDTimeout.setLevelPaddingSpaceChar ( 0 );
    textVoteIDTimeout.setVisible ( true );

    textDescriptionTimeout.setSize ( 9 );
    textDescriptionTimeout.setFontFamily ( "FreeSans" );
    textDescriptionTimeout.setBold ( false );
    textDescriptionTimeout.setItalic ( false );
    textDescriptionTimeout.setHPandding ( 0 );
    textDescriptionTimeout.setVPandding ( 1 );
    textDescriptionTimeout.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textDescriptionTimeout.setLevelPaddingSpaceChar ( 0 );
    textDescriptionTimeout.setVisible ( true );

    configurationTimeout.setTextTitle ( textTitleTimeout );
    configurationTimeout.setTextElectionName ( textElectionNameTimeout );
    configurationTimeout.setTextVotingBooth ( textVotingBoothTimeout );
    configurationTimeout.setTextVoteID ( textVoteIDTimeout );
    configurationTimeout.setTextDescription ( textDescriptionTimeout );

    textTitleSmartcardRemoved.setSize ( 12 );
    textTitleSmartcardRemoved.setFontFamily ( "FreeSans" );
    textTitleSmartcardRemoved.setBold ( true );
    textTitleSmartcardRemoved.setItalic ( false );
    textTitleSmartcardRemoved.setHPandding ( 0 );
    textTitleSmartcardRemoved.setVPandding ( 1 );
    textTitleSmartcardRemoved.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textTitleSmartcardRemoved.setLevelPaddingSpaceChar ( 0 );
    textTitleSmartcardRemoved.setVisible ( true );

    textElectionNameSmartcardRemoved.setSize ( 12 );
    textElectionNameSmartcardRemoved.setFontFamily ( "FreeSans" );
    textElectionNameSmartcardRemoved.setBold ( true );
    textElectionNameSmartcardRemoved.setItalic ( true );
    textElectionNameSmartcardRemoved.setHPandding ( 0 );
    textElectionNameSmartcardRemoved.setVPandding ( 1 );
    textElectionNameSmartcardRemoved.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textElectionNameSmartcardRemoved.setLevelPaddingSpaceChar ( 0 );
    textElectionNameSmartcardRemoved.setVisible ( true );

    textVotingBoothSmartcardRemoved.setSize ( 12 );
    textVotingBoothSmartcardRemoved.setFontFamily ( "FreeSans" );
    textVotingBoothSmartcardRemoved.setBold ( false );
    textVotingBoothSmartcardRemoved.setItalic ( false );
    textVotingBoothSmartcardRemoved.setHPandding ( 0 );
    textVotingBoothSmartcardRemoved.setVPandding ( 0 );
    textVotingBoothSmartcardRemoved.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textVotingBoothSmartcardRemoved.setLevelPaddingSpaceChar ( 0 );
    textVotingBoothSmartcardRemoved.setVisible ( true );

    textVoteIDSmartcardRemoved.setSize ( 12 );
    textVoteIDSmartcardRemoved.setFontFamily ( "FreeSans" );
    textVoteIDSmartcardRemoved.setBold ( false );
    textVoteIDSmartcardRemoved.setItalic ( false );
    textVoteIDSmartcardRemoved.setHPandding ( 0 );
    textVoteIDSmartcardRemoved.setVPandding ( 1 );
    textVoteIDSmartcardRemoved.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textVoteIDSmartcardRemoved.setLevelPaddingSpaceChar ( 0 );
    textVoteIDSmartcardRemoved.setVisible ( true );

    textDescriptionSmartcardRemoved.setSize ( 9 );
    textDescriptionSmartcardRemoved.setFontFamily ( "FreeSans" );
    textDescriptionSmartcardRemoved.setBold ( false );
    textDescriptionSmartcardRemoved.setItalic ( false );
    textDescriptionSmartcardRemoved.setHPandding ( 0 );
    textDescriptionSmartcardRemoved.setVPandding ( 1 );
    textDescriptionSmartcardRemoved.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textDescriptionSmartcardRemoved.setLevelPaddingSpaceChar ( 0 );
    textDescriptionSmartcardRemoved.setVisible ( true );

    configurationSmartcardRemoved.setTextTitle ( textTitleSmartcardRemoved );
    configurationSmartcardRemoved.setTextElectionName ( textElectionNameSmartcardRemoved );
    configurationSmartcardRemoved.setTextVotingBooth ( textVotingBoothSmartcardRemoved );
    configurationSmartcardRemoved.setTextVoteID ( textVoteIDSmartcardRemoved );
    configurationSmartcardRemoved.setTextDescription ( textDescriptionSmartcardRemoved );

    textInstallationTitleLevel1.setSize ( 12 );
    textInstallationTitleLevel1.setFontFamily ( "FreeSans" );
    textInstallationTitleLevel1.setBold ( false );
    textInstallationTitleLevel1.setItalic ( false );
    textInstallationTitleLevel1.setHPandding ( 0 );
    textInstallationTitleLevel1.setVPandding ( 1 );
    textInstallationTitleLevel1.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textInstallationTitleLevel1.setLevelPaddingSpaceChar ( 0 );
    textInstallationTitleLevel1.setVisible ( true );

    textInstallationTitleLevel2.setSize ( 12 );
    textInstallationTitleLevel2.setFontFamily ( "FreeSans" );
    textInstallationTitleLevel2.setBold ( false );
    textInstallationTitleLevel2.setItalic ( false );
    textInstallationTitleLevel2.setHPandding ( 0 );
    textInstallationTitleLevel2.setVPandding ( 1 );
    textInstallationTitleLevel2.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textInstallationTitleLevel2.setLevelPaddingSpaceChar ( 0 );
    textInstallationTitleLevel2.setVisible ( true );

    textInstallationTitleLevel3.setSize ( 9 );
    textInstallationTitleLevel3.setFontFamily ( "FreeSans" );
    textInstallationTitleLevel3.setBold ( false );
    textInstallationTitleLevel3.setItalic ( false );
    textInstallationTitleLevel3.setHPandding ( 0 );
    textInstallationTitleLevel3.setVPandding ( 10 );
    textInstallationTitleLevel3.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_RIGHT );
    textInstallationTitleLevel3.setLevelPaddingSpaceChar ( 0 );
    textInstallationTitleLevel3.setVisible ( true );

    textInstallationInfo.setSize ( 9 );
    textInstallationInfo.setFontFamily ( "FreeSans" );
    textInstallationInfo.setBold ( false );
    textInstallationInfo.setItalic ( false );
    textInstallationInfo.setHPandding ( 0 );
    textInstallationInfo.setVPandding ( 10 );
    textInstallationInfo.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_LEFT );
    textInstallationInfo.setLevelPaddingSpaceChar ( 0 );
    textInstallationInfo.setVisible ( true );

    ConfigurationTestPrinting configurationTestPrinting;
    TextConfiguration textTitle;

    textTitle.setSize ( 12 );
    textTitle.setFontFamily ( "FreeSans" );
    textTitle.setBold ( true );
    textTitle.setItalic ( false );
    textTitle.setHPandding ( 0 );
    textTitle.setVPandding ( 1 );
    textTitle.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textTitle.setLevelPaddingSpaceChar ( 0 );
    textTitle.setVisible ( true );

    configurationTestPrinting.setTextInstallationTitle ( textTitle );
    configurationTestPrinting.setShowBarCode ( true );

    ConfigurationCalibrationPrinting configurationCalibrationPrinting;
    configurationCalibrationPrinting.setTitle ( textTitle );
    configurationCalibrationPrinting.setInfo ( textTitle );

    configurationInstallation.setTextInstallationTitleLevel1 ( textInstallationTitleLevel1 );
    configurationInstallation.setTextInstallationTitleLevel2 ( textInstallationTitleLevel2 );
    configurationInstallation.setTextInstallationTitleLevel3 ( textInstallationTitleLevel3 );
    configurationInstallation.setTextInstallationInfo ( textInstallationInfo );
    configurationInstallation.setShowBarCode ( true );
    configurationInstallation.setBase64QRCode ( false );
    configurationInstallation.setQRcodePosition ( Smartmatic::SAES::Configuration::QRcodePosition::CONFIGURATION_BOTTOM_ALL );

    RegionCategoryLevel regionCategoryLevelSeq;
    regionCategoryLevelSeq.setDepth ( 8 );

    RegionCategoryLevel::CategoryLevelSequence categoryLevelSeq;
    categoryLevelSeq.push_back ( CategoryLevel::VOTING_DEVICES_FIRST_LEVEL );
    categoryLevelSeq.push_back ( CategoryLevel::VOTING_DEVICES_SECOND_LEVEL );
    categoryLevelSeq.push_back ( CategoryLevel::VOTING_DEVICES_THIRD_LEVEL );
    categoryLevelSeq.push_back ( CategoryLevel::VOTING_DEVICES_FOURTH_LEVEL );
    categoryLevelSeq.push_back ( CategoryLevel::VOTING_DEVICES_FIFTH_LEVEL );
    categoryLevelSeq.push_back ( CategoryLevel::VOTING_DEVICES_SIXTH_LEVEL );
    categoryLevelSeq.push_back ( CategoryLevel::VOTING_DEVICES_SEVENTH_LEVEL );
    categoryLevelSeq.push_back ( CategoryLevel::VOTING_DEVICES_EIGHTH_LEVEL );
    regionCategoryLevelSeq.setCategoryLevel ( categoryLevelSeq );

    configurationInstallation.setRegionCategoryLevel ( regionCategoryLevelSeq );

    ConfigurationDiagnosticReport configurationDiagnosticReport;
    configurationDiagnosticReport.setMultilanguage ( true );
    configurationDiagnosticReport.setTextInstallationTitleLevel1 ( textInstallationTitleLevel1 );
    configurationDiagnosticReport.setTextInstallationTitleLevel2 ( textInstallationTitleLevel2 );
    configurationDiagnosticReport.setTextInstallationTitleLevel3 ( textInstallationTitleLevel3 );
    configurationDiagnosticReport.setTextInstallationInfo ( textInstallationInfo );
    configurationDiagnosticReport.setShowBarCode ( true );
    configurationDiagnosticReport.setQRCodeConfig ( qrConfig1 );
    configurationDiagnosticReport.setProcessQRCode ( false );

    QRCodeConfig qrConfigInstall;

    qrConfigInstall.setSizeQRCode ( 3 );
    qrConfigInstall.setMarginQRCode ( 0 );
    qrConfigInstall.setCorrectionModeQRCode ( CorrectionModeQRCode::CONFIGURATION_QR_ECLEVEL_L );
    qrConfigInstall.setVersionQRCode ( 21 );

    configurationInstallation.setQRCodeConfig ( qrConfigInstall );

    textLogTitleLevel1.setSize ( 10 );
    textLogTitleLevel1.setFontFamily ( "FreeSans" );
    textLogTitleLevel1.setBold ( true );
    textLogTitleLevel1.setItalic ( false );
    textLogTitleLevel1.setHPandding ( 0 );
    textLogTitleLevel1.setVPandding ( 10 );
    textLogTitleLevel1.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textLogTitleLevel1.setLevelPaddingSpaceChar ( 0 );
    textLogTitleLevel1.setVisible ( true );

    textLogTitleLevel2.setSize ( 8 );
    textLogTitleLevel2.setFontFamily ( "FreeSans" );
    textLogTitleLevel2.setBold ( false );
    textLogTitleLevel2.setItalic ( false );
    textLogTitleLevel2.setHPandding ( 0 );
    textLogTitleLevel2.setVPandding ( 10 );
    textLogTitleLevel2.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textLogTitleLevel2.setLevelPaddingSpaceChar ( 0 );
    textLogTitleLevel2.setVisible ( true );

    textLogTitleLevel3.setSize ( 8 );
    textLogTitleLevel3.setFontFamily ( "FreeSans" );
    textLogTitleLevel3.setBold ( false );
    textLogTitleLevel3.setItalic ( false );
    textLogTitleLevel3.setHPandding ( 0 );
    textLogTitleLevel3.setVPandding ( 10 );
    textLogTitleLevel3.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textLogTitleLevel3.setLevelPaddingSpaceChar ( 0 );
    textLogTitleLevel3.setVisible ( true );

    textLogTitleLevel4.setSize ( 8 );
    textLogTitleLevel4.setFontFamily ( "FreeSans" );
    textLogTitleLevel4.setBold ( false );
    textLogTitleLevel4.setItalic ( false );
    textLogTitleLevel4.setHPandding ( 0 );
    textLogTitleLevel4.setVPandding ( 10 );
    textLogTitleLevel4.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textLogTitleLevel4.setLevelPaddingSpaceChar ( 0 );
    textLogTitleLevel4.setVisible ( true );

    textLogTitleLevel5.setSize ( 8 );
    textLogTitleLevel5.setFontFamily ( "FreeSans" );
    textLogTitleLevel5.setBold ( false );
    textLogTitleLevel5.setItalic ( false );
    textLogTitleLevel5.setHPandding ( 0 );
    textLogTitleLevel5.setVPandding ( 10 );
    textLogTitleLevel5.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_CENTER );
    textLogTitleLevel5.setLevelPaddingSpaceChar ( 0 );
    textLogTitleLevel5.setVisible ( true );

    textLogEntry.setSize ( 6 );
    textLogEntry.setFontFamily ( "FreeSans" );
    textLogEntry.setBold ( false );
    textLogEntry.setItalic ( false );
    textLogEntry.setHPandding ( 0 );
    textLogEntry.setVPandding ( 10 );
    textLogEntry.setAlign ( Smartmatic::SAES::Configuration::AlignType::CONFIGURATION_RIGHT );
    textLogEntry.setLevelPaddingSpaceChar ( 0 );
    textLogEntry.setVisible ( true );

    logPrintDocConf.setTextLogTitleLevel1 ( textLogTitleLevel1 );
    logPrintDocConf.setTextLogTitleLevel2 ( textLogTitleLevel2 );
    logPrintDocConf.setTextLogTitleLevel3 ( textLogTitleLevel3 );
    logPrintDocConf.setTextLogTitleLevel4 ( textLogTitleLevel4 );
    logPrintDocConf.setTextLogTitleLevel5 ( textLogTitleLevel5 );
    logPrintDocConf.setTextLogEntry ( textLogEntry );
    logPrintDocConf.setColumnLayout ( DocumentColumnLayout::CONFIGURATION_SINGLECOLUMN );

    configurationPrinter.setConfigurationTimeout ( configurationTimeout );
    configurationPrinter.setConfigurationVote ( configurationVote );
    configurationPrinter.setConfigurationInstallation ( configurationInstallation );
    configurationPrinter.setConfigurationTestPrinting ( configurationTestPrinting );
    configurationPrinter.setConfigurationCalibrationPrinting ( configurationCalibrationPrinting );
    configurationPrinter.setConfigurationDiagnosticReport ( configurationDiagnosticReport );
    configurationPrinter.setConfigurationSmartcardRemoved ( configurationSmartcardRemoved );
    configurationPrinter.setLogPrintDocConfiguration ( logPrintDocConf );

    configurationScreenPreview.setConfigurationVote ( configurationVote );

    configurationCard.setTimeOfValidity ( 0 );

    TransmissionPackageConfiguration transmissionConfig;
    transmissionConfig.setSaveDemoVotesInTransmissionPackage ( false );

    EmlFileConfiguration emlFileConfiguration;
    emlFileConfiguration.setEnableEml(true);
    emlFileConfiguration.setPersistenceEml(true);
    emlFileConfiguration.setSeparatorVotingDevice("_");
    emlFileConfiguration.setFirstStringFileName("V");
    emlFileConfiguration.setPersistenceX7SEml(true);
    emlFileConfiguration.setX7SAddPVReport(true);
    emlFileConfiguration.setX7SSigned(X7SSigned::CONFIGURATION_PKCS7_SIGNER);
    emlFileConfiguration.setX7SCrypto(X7SCrypto::CONFIGURATION_PKCS7_CRYPTO);
    emlFileConfiguration.setX7SSignedCertified(X7SSecurityType::CONFIGURATION_MACHINE_KEYS_FILE);
    emlFileConfiguration.setX7SCryptoCertified(X7SSecurityType::CONFIGURATION_PEM_FILES);

    FileHashConfiguration fileHashConfiguration;
    fileHashConfiguration.setEnableHashVerification(true);

    DiagnosticLogConfiguration diagnosticLogConfiguration;
    diagnosticLogConfiguration.setPersistence ( true );
    diagnosticLogConfiguration.setSeparator ( ";" );
    diagnosticLogConfiguration.setStringOk ( "OK" );
    diagnosticLogConfiguration.setStringNOk ( "NOK" );
    diagnosticLogConfiguration.setStringNA ( "N/A" );

    QRCodeConfig qrConfigBarcode;
    qrConfigBarcode.setSizeQRCode ( 3 );
    qrConfigBarcode.setMarginQRCode ( 2 );
    qrConfigBarcode.setCorrectionModeQRCode ( CorrectionModeQRCode::CONFIGURATION_QR_ECLEVEL_L );
    qrConfigBarcode.setVersionQRCode ( 0 );

    PVFileConfiguration pvFileConfiguration;
    pvFileConfiguration.setPersistenceBaseFile ( false );

    ConfigurationAudioResourceTreatment audioResourceTreatment;
    audioResourceTreatment.setEnableTreatment(false);
    audioResourceTreatment.setSeparator("#");

    configSAES.setAudioResourceTreatmentConfiguration(audioResourceTreatment);
    configSAES.setBarcodeQRCodeConfig ( qrConfigBarcode );
    configSAES.setTransmissionPackageConfiguration ( transmissionConfig );
    configSAES.setEmlFileConfiguration ( emlFileConfiguration );
    configSAES.setFileHashConfiguration ( fileHashConfiguration );
    configSAES.setDiagnosticLogConfiguration ( diagnosticLogConfiguration );
    configSAES.setConfigurationPrinter ( configurationPrinter );
    configSAES.setConfigurationScreenPreview ( configurationScreenPreview );
    configSAES.setConfigurationCard ( configurationCard );
    configSAES.setReportPrinting ( reportConf );
    configSAES.setPVFileConfiguration ( pvFileConfiguration );
    configSAES.setAudioResourceTreatmentConfiguration ( configurationAudioResourceTreatment );

    Smartmatic::ConfigurationTypes::NamespaceInfomap map;
#ifdef NAMESPACE_CONFIGURATION
    map[""].name = NAMESPACE_CONFIGURATION;
#endif
    map[""].schema = "";

    std::stringstream ss;
    serializeSaesConfiguration ( ss, configSAES, map, "UTF-8" );

    ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "SAES", ss.str() );
}

unsigned int SAESConfigurationManager::getCardValidity()
{
    return getConfiguration()->getConfigurationCard().getTimeOfValidity();
}

bool SAESConfigurationManager::getSaveDemoVotesInTransmissionPackage()
{
    return getConfiguration()->getTransmissionPackageConfiguration().getSaveDemoVotesInTransmissionPackage();
}

EmlFileConfiguration & SAESConfigurationManager::getEmlFileConfiguration()
{
    return getConfiguration()->getEmlFileConfiguration();
}

FileHashConfiguration & SAESConfigurationManager::getFileHashConfiguration()
{
    return getConfiguration()->getFileHashConfiguration();
}

PVFileConfiguration & SAESConfigurationManager::getPVFileConfiguration()
{
    return getConfiguration()->getPVFileConfiguration();
}

ConfigurationVote & SAESConfigurationManager::getPrintVoteConfiguration()
{
    return ( getConfiguration()->getConfigurationPrinter().getConfigurationVote() );
}

ConfigurationVote & SAESConfigurationManager::getPrintScreenVoteConfiguration()
{
    return ( getConfiguration()->getConfigurationScreenPreview().getConfigurationVote() );
}

QRCodeConfig SAESConfigurationManager::getQRCodeBarcodeConfig()
{
    return ( getConfiguration()->getBarcodeQRCodeConfig() );
}

QRCodeConfig SAESConfigurationManager::getQRCodeVoteConfig ( int size )
{
    QRCodeVoteList & qrCodeVoteList ( getConfiguration()->getConfigurationPrinter().getConfigurationVote().getQRCodeVoteList() );
    QRCodeVoteList::QRCodeVoteConfigSequence & seqQRCodeVoteConfig ( qrCodeVoteList.getQRCodeVoteConfig() );
    QRCodeVoteList::QRCodeVoteConfigIterator it;
    QRCodeConfig qrCodeConfig;
    std::map <int, QRCodeConfig> mapConfig;
    std::map <int, QRCodeConfig>::iterator mapIt;

    for ( it = seqQRCodeVoteConfig.begin();
            it != seqQRCodeVoteConfig.end();
            it++ )
    {
        mapConfig[ ( *it ).getMaxSize()] = ( *it ).getQRCodeConfig();
    }

    bool found = false;

    for ( mapIt = mapConfig.begin();
            mapIt != mapConfig.end();
            ++mapIt )
    {
        qrCodeConfig = ( ( *mapIt ).second );

        if ( size <= ( ( *mapIt ).first ) )
        {
            found = true;
            break;
        }
    }

    if ( !found )
    {
        qrCodeConfig.setVersionQRCode ( 0 );
    }

    return qrCodeConfig;
}

QRCodeConfig SAESConfigurationManager::getQRCodeInstallConfig()
{
    return ( getConfiguration()->getConfigurationPrinter().getConfigurationInstallation().getQRCodeConfig() );
}

QRecLevel SAESConfigurationManager::GetCorrectionModeQRCode ( CorrectionModeQRCode & correction )
{
    if ( correction == CorrectionModeQRCode::CONFIGURATION_QR_ECLEVEL_L )
    {
        return QR_ECLEVEL_L;
    }
    else if ( correction == CorrectionModeQRCode::CONFIGURATION_QR_ECLEVEL_M )
    {
        return QR_ECLEVEL_M;
    }
    else if ( correction == CorrectionModeQRCode::CONFIGURATION_QR_ECLEVEL_Q )
    {
        return QR_ECLEVEL_Q;
    }
    else if ( correction == CorrectionModeQRCode::CONFIGURATION_QR_ECLEVEL_H )
    {
        return QR_ECLEVEL_H;
    }

    return QR_ECLEVEL_L;
}

bool SAESConfigurationManager::showBarCodeInstallationReport()
{
    return getConfiguration()->getConfigurationPrinter().getConfigurationInstallation().getShowBarCode();
}

VoteCodeConfig & SAESConfigurationManager::getVoteCodeConfig()
{
    return getConfiguration()->getConfigurationPrinter().getConfigurationVote().getVoteCodeConfig();
}

LayoutContestConfiguration & SAESConfigurationManager::getLayoutContestConfiguration()
{
    return getConfiguration()->getConfigurationPrinter().getConfigurationVote().getLayoutContestConfiguration();
}

LayoutBallotOptiontType & SAESConfigurationManager::getLayoutBallotOptiontType()
{
    return getConfiguration()->getConfigurationPrinter().getConfigurationVote().getLayoutBallotOptiontType();
}

LayoutContestConfiguration & SAESConfigurationManager::getLayoutContestConfigurationPreviewScreen()
{
    return getPrintScreenVoteConfiguration().getLayoutContestConfiguration();
}

LayoutBallotOptiontType & SAESConfigurationManager::getLayoutBallotOptiontTypePreviewScreen()
{
    return getPrintScreenVoteConfiguration().getLayoutBallotOptiontType();
}

SAESConfigurationManager * SAESConfigurationManager::getInstance ( SAESConfigurationManager * externManager )
{
    if ( ! instance )
    {
        instance = ( externManager == NULL ? new SAESConfigurationManager() : externManager );
    }

    return instance;
}
