/**
 * @file   EmlElementNotFoundException.hxx
 * @brief  Declaration of EmlElementNotFound class.
 * @author manuel.arguelles
 * @date   Jan 12, 2018
 */

#ifndef EML_ELEMENT_NOT_FOUND_EXCEPTION_HXX
#define EML_ELEMENT_NOT_FOUND_EXCEPTION_HXX

#include <System/Exception/BaseException.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Exceptions
        {
            namespace Voting
            {
                namespace Eml
                {
                    /**
                     * @class  EmlElementNotFoundException
                     * @brief  Exception when an requested element of EML file is not found.
                     * @author manuel.arguelles
                     * @date   Jan 12, 2018
                     */
                    class EmlElementNotFoundException : public Smartmatic::System::Exception::BaseException
                    {
                        public:

                            /**
                             * @brief  Creates an instance of the exception.
                             * @author manuel.arguelles
                             * @date   Jan 12, 2018
                             * @param  message The exception message.
                             * @param  code    The exception code.
                             * @param  logger  The logger.
                             */
                            EmlElementNotFoundException(std::string message, std::string code,
                                                        Smartmatic::Log::ISMTTLog* logger = NULL);
                    };

                } /* namespace Eml */
            } /* namespace Voting */
        } /* namespace Exceptions */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* EML_ELEMENT_NOT_FOUND_EXCEPTION_HXX */
