/**
 * @file CalibrationPrinting.cxx
 * @brief Body class calibration test report printing
 * @date 10/09/2013
 * @author Yadickson Soto
 */

#include "CalibrationPrinting.hxx"
#include <Environment/Configurations/resource-path-provider.h>
#include "Printing/TranslatorTextConfiguration.h"
#include <Voting/OperationStatus.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting;

CalibrationPrinting::CalibrationPrinting ( OperationStatus * operationStatus )
    : BasePrintDocument ( false ), operationStatus ( operationStatus )
{
    configuration = SAESConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getConfigurationCalibrationPrinting();
}

CalibrationPrinting::~CalibrationPrinting()
{
}

void CalibrationPrinting::PrintDoc()
{
    DocumentTextOptions titleOption = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configuration.getTitle() );
    DocumentTextOptions infoOption = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configuration.getInfo() );

    std::list<DocumentTextOptions> listDocumentTextOptions;
    listDocumentTextOptions.push_back ( infoOption );
    listDocumentTextOptions.push_back ( infoOption );

    std::string serialDetected = operationStatus->getVotingMachineStatusWrapper()->getSerialDetected();

    Glib::ustring title = Glib::ustring::compose ( _ ( "Calibration Test" ), PACKAGE_VERSION );

    this->NewTextBlock ( titleOption );
    this->DrawString ( " ", true );
    this->DrawString ( title, true );
    this->DrawString ( " ", true );
    this->AddBreak();

    this->NewTextColunmsBlock ( listDocumentTextOptions );
    this->DrawString ( _ ( "Serial Detected" ), true );
    this->DrawString ( serialDetected, true );
    this->DrawString ( " ", true );
    this->AddBreak();

    this->NewTextBlock ( infoOption );
    this->DrawString ( _ ( "Calibration test fail, please call for service." ), true );
    this->DrawString ( " ", true );
    this->DrawString ( " ", true );
    this->DrawString ( " ", true );
    this->AddBreak();

    PrintQR();

    this->DrawString ( " ", true );
    this->DrawString ( " ", true );
    this->DrawString ( " ", true );
    this->AddBreak();
}

void CalibrationPrinting::BeginPrint()
{
}

void CalibrationPrinting::EndPrint()
{
}

void CalibrationPrinting::PrintQR()
{
    SafeBuffer toEncode;
    bool process = SAESConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getConfigurationDiagnosticReport().getProcessQRCode();
    toEncode = operationStatus->getQRDiagnosticResult ( process );

    Glib::RefPtr<Gdk::Pixbuf> backgroundImage;
    QRCodeConfig config = SAESConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getConfigurationDiagnosticReport().getQRCodeConfig();
    int pixel_size = config.getSizeQRCode();
    int margin = config.getMarginQRCode();
    int version = config.getVersionQRCode();
    QRecLevel correctionLevel = SAESConfigurationManager::getInstance()->GetCorrectionModeQRCode ( config.getCorrectionModeQRCode() );

    if ( toEncode.GetSize() > 0 )
    {
        this->DrawQRCode (
            toEncode
            , pixel_size
            , margin
            , correctionLevel
            , version
            // Image content
            , backgroundImage
            , Center );
    }

}
