/**
 * @file DiagnosticPrintDocument.hxx
 * @brief Header diagnostic print document class
 * @date 19/09/2012
 * @author Yadickson Soto
 */

#ifndef _DIAGNOSTICPRINTDOCUMENT_H_
#define _DIAGNOSTICPRINTDOCUMENT_H_

#include <gtkmm.h>
#include <Printing/BasePrintDocument.h>
#include <Printing/DocumentOptions.h>
#include <Configuration/s-a-e-s-conf-manager.h>
#include "Voting/Status/voting-machine-status-schema.hxx"
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            class OperationStatus;
        }
        namespace Printing
        {
            /**
             * @class	DiagnosticPrintDocument
             * @brief	diagnostic print document class.
             * @author	Yadickson Soto
             * @date	19/09/2012
             */
            class DiagnosticPrintDocument : public BasePrintDocument
            {
                public:

                    /**
                     * @fn	DiagnosticPrintDocument::DiagnosticPrintDocument( Smartmatic::SAES::Voting::OperationStatus * operationStatus );
                     * @brief	class constructor.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     * @param operationStatus The operation status
                     */
                    explicit DiagnosticPrintDocument ( Smartmatic::SAES::Voting::OperationStatus * operationStatus );
                    /**
                     * @fn	void DiagnosticPrintDocument::PrintDoc();
                     * @brief	method print document.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     */
                    void PrintDoc();
                    /**
                     * @fn	void DiagnosticPrintDocument::BeginPrint();
                     * @brief	method begin print document.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     */
                    void BeginPrint();
                    /**
                     * @fn	void DiagnosticPrintDocument::EndPrint();
                     * @brief	method end print document.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     */
                    void EndPrint();

                protected:

                    /**
                     * @fn	void DiagnosticPrintDocument::processDocument();
                     * @brief	method process print document.
                     * @author	Yadickson Soto
                     * @date	20/09/2012
                     */
                    void processDocument();
                    /**
                     * @fn	std::string DiagnosticPrintDocument::getType(Smartmatic::SAES::Common::Configuration::DeviceType & type);
                     * @brief	method get string type.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     * @param	type	type.
                     * @return	string type.
                     */
                    std::string getType ( Smartmatic::SAES::Common::Configuration::DeviceType & type );
                    /**
                     * @fn	std::string DiagnosticPrintDocument::getStatus(Smartmatic::SAES::Voting::Status::VMStatus & status);
                     * @brief	method get string status.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     * @param	status	status.
                     * @return	string status.
                     */
                    std::string getStatus ( Smartmatic::SAES::Voting::Status::VMStatus & status );
                    /**
                     * @fn	void DiagnosticPrintDocument::setQR();
                     * @brief	Method getter set qr code.
                     * @author	Yadickson Soto
                     * @date	24/10/2011
                     */
                    void setQR();
                    /**
                     * @fn	void DiagnosticPrintDocument::setQRCodeImage();
                     * @brief	Method getter set qr code image.
                     * @author	Yadickson Soto
                     * @date	24/10/2011
                     */
                    void setQRCodeImage();
                    /**
                     * @fn	void DiagnosticPrintDocument::setQRCode();
                     * @brief	Method getter set qr code.
                     * @author	Yadickson Soto
                     * @date	24/10/2011
                     */
                    void setQRCode();

                private:

                    /**
                     * @brief  Adds a line or two lines with entry and value. Entry line is
                     * evaluated and split if needed.
                     * @author manuel.arguelles
                     * @date   May  7, 2018
                     * @param  entry The entry title.
                     * @param  value The value of the entry.
                     */
                    void printEntryValue(const std::string& entry, const std::string& value);

                private:

                    Smartmatic::System::SafeBuffer data; ///< data qr code
                    Glib::RefPtr<Gdk::Pixbuf> pixBuff; ///< pix buf
                    bool multilanguage; ///< multilanguage
                    Smartmatic::SAES::Configuration::ConfigurationDiagnosticReport configuration; ///< The configuration
                    Smartmatic::SAES::Voting::OperationStatus * operationStatus; ///< The operation status
                    static Smartmatic::Log::ISMTTLog * logger; ///< logger
            };
        }
    }
}

#endif // _DIAGNOSTICPRINTDOCUMENT_H_
