/**
 * @file	ErrorLogPrintDocument.h
 * @brief	Declares the error log print document class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef ERRORLOGPRINTDOCUMENT_H_
#define ERRORLOGPRINTDOCUMENT_H_

#include "Printing/LogPrintDocument.h"

namespace Smartmatic
{

    namespace SAES
    {

        namespace Printing
        {

            /**
             * @class	ErrorLogPrintDocument
             * @brief	Error log print document.
             * @author	Youssef.tourkmani
             * @date	31/10/2012
             */
            class ErrorLogPrintDocument : public LogPrintDocument
            {
                public:

                    /**
                     * @fn	ErrorLogPrintDocument::ErrorLogPrintDocument(Smartmatic::SAES::Log::LogReader* reader,
                     * 			Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration);
                     * @brief	Constructor.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param [in,out]	reader									If non-null, the reader.
                     * @param [in,out]	electoralConfiguration	If non-null, the electoral configuration.
                     */
                    ErrorLogPrintDocument ( Smartmatic::SAES::Log::LogReader * reader,
                                            Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );

                    /**
                     * @fn	virtual ErrorLogPrintDocument::~ErrorLogPrintDocument();
                     * @brief	Destructor.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    virtual ~ErrorLogPrintDocument();

                protected:

                    /**
                     * @fn	virtual std::string ErrorLogPrintDocument::GetLogDocName ();
                     * @brief	Gets log document name.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	The log document name.
                     */
                    virtual std::string GetLogDocName ();

            };

        }

    }

}

#endif /* ERRORLOGPRINTDOCUMENT_H_ */
