/**
 * @file   ElectionAudioResourcesManager.cxx
 * @brief  Implementation of ElectionAudioResourcesManager class.
 * @author manuel.arguelles
 * @date   Oct 12, 2017
 */

#include "ElectionAudioResourcesManager.hxx"
#include <Configuration/s-a-e-s-conf-manager.h>
#include <System/Utils/StringUtils.hxx>

using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::System::Utils;

ElectionAudioResourcesManager* ElectionAudioResourcesManager::m_instance = NULL;

ElectionAudioResourcesManager::ElectionAudioResourcesManager()
{
}

ElectionAudioResourcesManager::~ElectionAudioResourcesManager()
{
    delete m_instance;
    m_instance = NULL;
}

ElectionAudioResourcesManager* ElectionAudioResourcesManager::getInstance(ElectionAudioResourcesManager* externResource)
{
    if (m_instance == NULL)
    {
        m_instance = (externResource == NULL ? new ElectionAudioResourcesManager() : externResource);
    }

    return m_instance;
}


void ElectionAudioResourcesManager::init(SaesDirectories* directories, BasePathType type)
{
    m_fileName = directories->getElectionResourcesFileName();
    m_folderPath = directories->getElectoralResourceFullFolderPath();
    Initialize();
}

std::string& ElectionAudioResourcesManager::getFileName()
{
    return m_fileName;
}

std::string& ElectionAudioResourcesManager::getFullFolderPath()
{
    return m_folderPath;
}

std::string ElectionAudioResourcesManager::getResource ( const std::string & key )
{
    std::string resource = StringResourcesManagerBase::getResource ( key, currentCulture );

    if (SAESConfigurationManager::getInstance()->getConfiguration()->getAudioResourceTreatmentConfiguration().getEnableTreatment())
    {
        std::string separator = SAESConfigurationManager::getInstance()->getConfiguration()->getAudioResourceTreatmentConfiguration().getSeparator();
        std::size_t found = resource.find(separator);

        if (found != std::string::npos)
        {
            std::vector<std::string> listOfWords;
            StringUtils::splitToVector(resource, separator, listOfWords);

            if (listOfWords.size() >= 3)
            {
                resource = StringUtils::replaceLast(resource, listOfWords[listOfWords.size()-1], "" );
            }

            resource = StringUtils::replaceAll( resource, separator, "  " );
            resource = StringUtils::capitalizeUtf8(resource);
        }
    }

    return resource;
}
