/**
 * @file IDManager.hxx
 * @brief Header class to generate, find and remove uuid values
 * @date 08/05/2013
 * @author Yadickson Soto
 */

#ifndef IDMANAGER_HXX_
#define IDMANAGER_HXX_

#include <set>
#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            /**
             * @class IDManager
             * @brief Contains the logic to generate, find and remove uuid values
             * @date 08/05/2013
             * @author Yadickson Soto
             */
            class IDManager
            {
                public:
                    /**
                     * @fn IDManager();
                     * @brief Class constructor
                     * @date 08/05/2013
                     * @author Yadickson Soto
                     */
                    IDManager();
                    /**
                     * @fn virtual ~IDManager();
                     * @brief Class destructor
                     * @date 08/05/2013
                     * @author Yadickson Soto
                     */
                    virtual ~IDManager();
                    /**
                     * @fn std::string generate();
                     * @brief Generate new uuid value, add to uuid list
                     * @date 08/05/2013
                     * @author Yadickson Soto
                     * @return new uuid value in string base64 of uuid
                     */
                    std::string generate();
                    /**
                     * @fn unsigned int size();
                     * @brief Getter method: size of elements in uuid list
                     * @date 08/05/2013
                     * @author Yadickson Soto
                     * @return size
                     */
                    unsigned int size();
                    /**
                     * @fn bool find( const std::string & uuid );
                     * @brief Find uuid inside of uuid list
                     * @date 08/05/2013
                     * @author Yadickson Soto
                     * @param	uuid	uuid to find
                     * @return true if find success
                     */
                    bool find ( const std::string & uuid );
                    /**
                     * @fn bool remove( const std::string & uuid );
                     * @brief Remove uuid inside of uuid list
                     * @date 08/05/2013
                     * @author Yadickson Soto
                     * @param	uuid	uuid to find
                     * @return true if find and remove is success
                     */
                    bool remove ( const std::string & uuid );
                    /**
                     * @fn void clear();
                     * @brief Clear all uuid value of uuid list
                     * @date 08/05/2013
                     * @author Yadickson Soto
                     */
                    void clear();

                private:

                    std::set < std::string > uuidList; ///< uuid list values
            };

        } /* namespace Voting */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* IDMANAGER_HXX_ */
