/**
 * @file PVContestWrapper.cxx
 * @brief Body PV Contest Wrapper
 * @date 05/06/2013
 * @author Yadickson Soto
 */

#include "PVContestWrapper.hxx"
#include <System/Utils/StringUtils.hxx>
#include <Voting/Eml/EmlInfo.hxx>
#include <Voting/Election/election-schema.hxx>

using namespace Smartmatic::SAES::Voting::Eml;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::System::Utils;

PVContestWrapper* PVContestWrapper::instance = NULL;

PVContestWrapper::PVContestWrapper()
{
}

PVContestWrapper::~PVContestWrapper()
{
    instance = NULL;
    clear();
}

void PVContestWrapper::init()
{
    clear();
}

void PVContestWrapper::clear()
{
    std::list<Contest*>::iterator it;

    for (it = contestList.begin(); it != contestList.end(); ++it)
    {
        Contest* c = *it;
        delete c;
        c = NULL;
    }

    contestList.clear();
    contestMap.clear();
}

void PVContestWrapper::setContest(const Smartmatic::SAES::Voting::Election::Contest* contest)
{
    std::string code = contest->getCode();

    if (contestMap.find(code) == contestMap.end())
    {
        Contest*    c                = new Contest();
        std::string customCode       = contest->getCustom_code();
        std::string electionType     = Voting::Eml::EmlInfo::getElectionType(customCode);
        std::string linguisticRegime = Voting::Eml::EmlInfo::getLanguage(customCode);
        int         eligibleAmount   = (contest->getEligible_amount().present()
                                        ? contest->getEligible_amount().get()
                                        : 0);

        c->setCode(code);
        c->setCustom_code(customCode);
        c->setElection_type ( electionType );
        c->setLinguistic_regime ( linguisticRegime );
        c->setEligible_amount(eligibleAmount);
        c->setChecksum(0);

        contestMap[customCode] = c;
        contestList.push_back(c);
    }
}

Contest* PVContestWrapper::getContest(const std::string& customCode)
{
    return contestMap[customCode];
}

std::list<Contest*>& PVContestWrapper::getContestList()
{
    return contestList;
}

PVContestWrapper* PVContestWrapper::getInstance()
{
    if (instance == NULL)
    {
        instance = new PVContestWrapper();
    }

    return instance;
}
