/**
 * @file   VoteCounterByContest.hxx
 * @brief  Declaration of VoteCounterByContest class.
 * @author eduardo.fernandez
 * @date   Dec 05, 2018
 */

#ifndef _VOTE_COUNTER_BY_CONTEST_HXX_
#define _VOTE_COUNTER_BY_CONTEST_HXX_

#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            namespace PBReports
            {
                /**
                 * @class  VoteCounterByContest
                 * @brief  Vote counter by contest.
                 * @author eduardo.fernandez
                 * @date   05/12/2018
                 */
                class VoteCounterByContest
                {
                    public:

                        /**
                         * @brief  Constructor.
                         * @author eduardo.fernandez
                         * @date   05/12/2018
                         * @param  customCode The contest custom code.
                         * @param  initialCounter The initial counter.
                         */
                        VoteCounterByContest(const std::string& customCode, const size_t initialCounter = 0);

                        /**
                         * @brief  Finaliser.
                         * @author eduardo.fernandez
                         * @date   05/12/2018
                         */
                        virtual ~VoteCounterByContest();

                        /**
                         * @brief  Increment counter.
                         * @author eduardo.fernandez
                         * @date   05/12/2018
                         */
                        void incrementCounter();

                        /**
                         * @brief  Gets the ca code.
                         * @author eduardo.fernandez
                         * @date   05/12/2018
                         * @return The ca code.
                         */
                        std::string getCustomCode();

                        /**
                         * @brief  Gets the counter.
                         * @author eduardo.fernandez
                         * @date   05/12/2018
                         * @return The counter.
                         */
                        size_t getCounter();

                        /**
                         * @brief  Clears the counter.
                         * @author eduardo.fernandez
                         * @date   05/12/2018
                         */
                        void clearCounter();

                    private:

                        std::string  m_customCode;
                        size_t m_counter;
                };
            }
        }
    }
}

#endif /* _VOTE_COUNTER_BY_CONTEST_HXX_ */
