/**
 * @file RecountVoteStatistics.h
 * @brief Header of recount vote statistics class, responsible for the voting counter by nationality.
 * @date 27/08/2013
 * @author Yadickson Soto
 */

#ifndef RECOUNT_VOTE_STATISTICS_H_
#define RECOUNT_VOTE_STATISTICS_H_

#include <list>
#include <string>
#include <Voting/VotingDevice/logical-voter-group-schema.hxx>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/VotingDevice/VotingDeviceWrapper.hxx>

#include <map>
#include <set>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {

            class RecountVoteStatistics
            {
                private:
                    std::string m_nationalityKeyName;

                    std::set < std::string > voterTypes;

                    Smartmatic::SAES::Voting::VotingDevice::VotingDeviceWrapper * votingDeviceWrapper; ///< voting device wrapper
                    std::map<std::string, int> m_sessionNums;

                    //	typedef std::vector< std::pair<int, std::list<std::string> > > SessionContainer;
                    std::list< std::vector< std::pair<int, std::list<std::string> > > > m_sessionResults;

                    std::list< std::pair<int, int> > m_updateQueue;

                    std::set<std::string> m_processedVotes;

                    int GetVoterIndex ( const std::string & voter );
                    std::vector< std::pair<int, std::list<std::string> > > & GetSessionResults ( const std::string & sessionId );

                public:
                    RecountVoteStatistics ();
                    virtual ~RecountVoteStatistics ();
                    void initialize ( Smartmatic::SAES::Voting::VotingDevice::VotingDeviceWrapper * votingDeviceWrapper );

                    virtual std::list<std::string> GetVoterTypes();
                    int GetSessionNum ( const std::string & sessionId );
                    std::string GetVoter ( Smartmatic::SAES::Voting::Vote::Vote & vote );
                    void ProcessVote ( Smartmatic::SAES::Voting::Vote::Vote & vote );

                    bool GetUpdateQueue ( std::list< std::pair<int, int> > & updates, std::list<std::string> & updateSessionNames );

                    void Persist ( std::list< std::pair<int, int> > deleteQueue );
                    void clear();
            };
        }
    }
}

#endif
