/**
 * @file	GeneratedImagesWrapper.cxx
 * @brief	generated images wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "GeneratedImagesWrapper.hxx"
#include <System/IO/Path.h>
#include <System/IO/File.h>
#include <Log/SMTTLogManager.h>
#include <Resources/election-string-resources-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Screen;
using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::SAES::Resources;

ISMTTLog * GeneratedImagesWrapper::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.Screen.GeneratedImagesWrapper", GETTEXT_PACKAGE );

GeneratedImagesWrapper::GeneratedImagesWrapper()
{
}

GeneratedImagesWrapper::~GeneratedImagesWrapper()
{
}

void GeneratedImagesWrapper::Persist ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoral, std::string folder )
throw ( CryptoException,
        SignedException,
        XmlException )
{
    Smartmatic::SAES::Runtime::Environment::SaesDirectories directories;
    std::string fullFileName = Path::Combine ( folder, directories.getXMLImageFileName() );
    logger->Debug ( "Creating File: " + fullFileName );

    Smartmatic::SAES::Voting::Screen::NamespaceInfomap map;
    map[""].name = NAMESPACE_SCREEN;
    map[""].schema = "";
    std::ostringstream os;

    try
    {
        if ( !images.empty() )
        {
            generatedImages.setImage ( images );
        }

        if ( !confirmationImages.empty() )
        {
            generatedImages.setConfirmationImage ( confirmationImages );
        }

        if ( !customAttributeImages.empty() )
        {
            generatedImages.setCustomAttributeImage ( customAttributeImages );
        }

        if ( !partySelectionImages.empty() )
        {
            generatedImages.setPartySelectionImage ( partySelectionImages );
        }

        if ( !ballotSelectionImages.empty() )
        {
            generatedImages.setBallotSelectionImage ( ballotSelectionImages );
        }

        generatedImages.setVoting_center ( electoral->getVotingDevice()->getFirstVotingDevice().getCustom_code() );

        serializeGeneratedImages ( os, generatedImages, map, "UTF-8" );
        logger->Debug ( "File " + fullFileName + " parsed." );
    }
    catch ( ... )
    {
        throw XmlException (
            _ ( "Smartmatic.SAES.Voting.Screen.GeneratedImagesWrapper.Error" ),
            N_ ( "Smartmatic.SAES.Voting.Screen.GeneratedImagesWrapper.Error" ),
            logger );
    }

    SafeBuffer buffer ( os.str() );
    File::getInstance()->saveBufferToFile ( buffer, fullFileName );
}

void GeneratedImagesWrapper::addBasicImage ( BasicImage & image )
{
    images.push_back ( image );
}

void GeneratedImagesWrapper::addSelectionImage ( SelectionImage & image )
{
    confirmationImages.push_back ( image );
}

void GeneratedImagesWrapper::addCustomAttributeImage ( CustomAttributeImage & image )
{
    customAttributeImages.push_back ( image );
}

void GeneratedImagesWrapper::addPartySelectionImage ( PartySelectionImage & image )
{
    partySelectionImages.push_back ( image );
}

void GeneratedImagesWrapper::addBallotSelectionImage ( BallotSelectionImage & image )
{
    ballotSelectionImages.push_back ( image );
}
