/*
 * CheckListDiagnose.cxx
 *
 *  Created on: 17/09/2013
 *      Author: soto
 */

#include "CheckListDiagnose.hxx"
#include <Voting/Status/check-list-diagnose-status-schema.hxx>
#include <Environment/Configurations/resource-path-provider.h>
#include <Runtime/Environment/saes-directories.h>
#include <System/IO/File.h>

using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;

CheckListDiagnose::CheckListDiagnose()
{

}

CheckListDiagnose::~CheckListDiagnose()
{

}

void CheckListDiagnose::getCheckList ( std::list < DiagnoseStep > & list )
{
    list.clear();

    try
    {
        std::string str = getContentFile();
        std::istringstream istr ( str );
        CheckListDiagnoseStatus checkList = * ( parseCheckListDiagnoseStatus ( istr, CheckListDiagnoseTypes::Flags::dont_validate ) );

        if ( checkList.getView() )
        {
            CheckListDiagnoseStatus::ItemSequence & seq ( checkList.getItem() );
            CheckListDiagnoseStatus::ItemIterator it;

            for ( it = seq.begin();
                    it != seq.end();
                    ++it )
            {
                if ( ( *it ).getView() )
                {
                    DiagnoseStep step;
                    step.setCode ( DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_C ) );
                    step.setCustomCode ( ( *it ).getCustomCode() );
                    step.setValue ( ( *it ).getText() );
                    list.push_back ( step );
                }
            }
        }
    }
    catch ( ... )
    {

    }
}

std::string CheckListDiagnose::getContentFile()
{
    SaesDirectories directory;

    std::string fileDom = directory.getCheckListDiagnoseFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string fileFlash = directory.getCheckListDiagnoseFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string fileBackup = directory.getCheckListDiagnoseFileFullPath ( BasePathType::CONFIGURATION_BACKUP );

    fileDom = ResourcePathProvider::getInstance()->getResourcePath ( fileDom );
    fileFlash = ResourcePathProvider::getInstance()->getResourcePath ( fileFlash );
    fileBackup = ResourcePathProvider::getInstance()->getResourcePath ( fileBackup );

    SafeBuffer buffer;
    std::string result;

    if ( File::getInstance()->openFileToBuffer ( fileDom, buffer ) ||
            File::getInstance()->openFileToBuffer ( fileFlash, buffer ) ||
            File::getInstance()->openFileToBuffer ( fileBackup, buffer ) )
    {
        result.assign ( ( const char * ) buffer.GetData(), buffer.GetSize() );
    }

    return result;
}
