/**
 * @file	StationTallyWrapper.hxx
 * @brief	station tally wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef __STATION_TALLY_WRAPPER_H__
#define __STATION_TALLY_WRAPPER_H__

#include <map>
#include <list>
#include <string>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            class ElectoralConfiguration;

            namespace Election
            {
                class Ballot_option;
            }

            namespace Vote
            {
                class Register;
                class StationTally;
                class Vote;

                /**
                 * @class	StationTallyWrapper
                 * @brief	Station tally wrapper.
                 * @author	Eduardo.campione
                 * @date	28/04/2011
                 */

                class StationTallyWrapper
                {
                    public:

                        /**
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	28/04/2011
                         */
                        StationTallyWrapper ();

                        /**
                         * @brief	Finaliser.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */
                        virtual ~StationTallyWrapper();

                        /**
                         * @brief	initialize class.
                         * @author	Yadickson Soto
                         * @date	28/04/2011
                         * @param	electoralConfiguration	electoral configuration.
                         */
                        void init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );

                        /**
                         * @brief Method add vote for update info
                         * @param vote vote
                         * @date 17/02/2012
                         * @author Yadickson Soto
                         */
                        void update ( Vote & vote );

                        /**
                         * @brief	Method init info.
                         * @author	Yadickson Soto
                         * @date	17/02/2012
                         */
                        void initialize();

                        /**
                         * @brief	Method clear info.
                         * @author	Yadickson Soto
                         * @date	17/02/2012
                         */
                        void clear();

                        /**
                         * @brief	Gets station tally list.
                         * @author	Yadickson Soto
                         * @date	26/03/2014
                         * @return	station tally list.
                         */
                        virtual std::list <StationTally *> getStationTally ();

                    private:

                        /**
                         * @typedef	std::map <std::string, Smartmatic::SAES::Voting::Vote::Register *> registerMap
                         * @brief	Defines an alias representing the register map.
                         */
                        typedef std::map <std::string, Smartmatic::SAES::Voting::Vote::Register *> registerMap;
                        /**
                         * @typedef	std::map <std::string, registerMap > fullRegisterMap
                         * @brief	Defines an alias representing the full register map.
                         */
                        typedef std::map <std::string, registerMap > fullRegisterMap;

                        /**
                         * @brief	Init egister map.
                         * @author	Yadickson Soto
                         * @date	31/10/2012
                         * @param	code	The code.
                         * @param	map	The map.
                         * @param	list	The list.
                         */
                        void initRegisterMap ( std::string code, registerMap & map, std::list<Smartmatic::SAES::Voting::Election::Ballot_option * > & list );

                        /**
                         * @brief	Clean register map.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	map	The map.
                         */
                        void cleanRegisterMap ( registerMap & map );

                    private:

                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;	///< The electoral configuration
                        std::map <std::string, StationTally *> stationTallyMap;

                        fullRegisterMap allRegisterMap; ///< all register map
                        fullRegisterMap listRegisterMap;	///< The list register map
                        fullRegisterMap principalRegisterMap; ///< The principal register map
                        fullRegisterMap standInRegisterMap; ///< The stand in register map

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /*__STATION_TALLY_WRAPPER_H__*/
