/**
 * @file VoteOptimizedSelection.hxx
 * @brief Header vote optimized selection ballots
 * @date 21/05/2013
 * @author Yadickson Soto
 */

#ifndef VOTEOPTIMIZEDSELECTION_HXX_
#define VOTEOPTIMIZEDSELECTION_HXX_

#include <list>
#include <map>
#include <string>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            namespace Vote
            {
                /**
                 * @class VoteOptimizedSelection
                 * @brief This class have the logic of transformation of ballots to checksum
                 * @date 21/05/2013
                 * @author Yadickson Soto
                 */
                class VoteOptimizedSelection
                {
                    public:
                        /**
                         * @fn VoteOptimizedSelection();
                         * @brief Class constructor
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         */
                        VoteOptimizedSelection();
                        /**
                         * @fn VoteOptimizedSelection( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                         * @brief Class constructor
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         * @param electoralConfiguration electoral configuration object
                         */
                        explicit VoteOptimizedSelection ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                        /**
                         * @fn virtual ~VoteOptimizedSelection();
                         * @brief Class destructor
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         */
                        virtual ~VoteOptimizedSelection();
                        /**
                         * @fn virtual std::string getChecksum ( const Smartmatic::SAES::Voting::Vote::Vote * vote, std::map < std::string, std::string > & mapResult );
                         * @brief This method have the logic of transformation of list ballots to checksum
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         * @param	vote vote to checksum calculation for all selections
                         * @param 	mapResult relationship context code and checksum value, zero "0" if not selections for context
                         */
                        virtual void getChecksum ( const Smartmatic::SAES::Voting::Vote::Vote * vote, std::map < std::string, std::string > & mapResult );
                        /**
                         * @fn virtual void getBallots ( std::map < std::string, std::string > & mapResult, Smartmatic::SAES::Voting::Vote::Vote * vote );
                         * @brief This method have the logic of transformation of checksum to list ballots
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         * @param	vote vote to add all selections
                         * @param	mapResult relationship context code and checksum value
                         */
                        virtual void getBallots ( std::map < std::string, std::string > & mapResult, Smartmatic::SAES::Voting::Vote::Vote * vote );

                    private:

                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration; ///< electoral configuration
                };
            } /* namespace Vote */
        } /* namespace Voting */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* VOTEOPTIMIZEDSELECTION_HXX_ */
