/**
 * @file VoteOptimizedSigned.cxx
 * @brief Body class for signed and validate vote optimized
 * @date 21/05/2013
 * @author Yadickson Soto
 */

#include "VoteOptimizedSigned.hxx"
#include <System/Encoding/Base64Encoder.h>
#include <System/Utils/StringUtils.hxx>
#include <Security/Encryption.hxx>

using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::Security;

VoteOptimizedSigned::VoteOptimizedSigned()
{

}

VoteOptimizedSigned::~VoteOptimizedSigned()
{

}

bool VoteOptimizedSigned::validateBuffer ( Smartmatic::System::SafeBuffer & buffer )
{
    if ( buffer.GetSize() == 0 )
    {
        return false;
    }

    std::string stringVote;
    stringVote.assign ( ( const char * ) buffer.GetData(), buffer.GetSize() );

    std::vector <std::string> result;
    StringUtils::splitToVector ( stringVote, ",", result );

    if ( result.size() < 2 )
    {
        return false;
    }

    bool validate = false;

    std::string signature = result [ result.size() - 1 ];
    std::string data = StringUtils::remove ( stringVote , signature );

    SafeBuffer dataBuffer ( data );
    SafeBuffer signatureBuffer;

    if ( Base64Encoder::ConvertFromBase64 ( signature, signatureBuffer ) )
    {
        validate = Encryption::getInstance()->VerifySignatureWithPersonalCertificate ( dataBuffer, signatureBuffer );
    }

    return validate;
}

bool VoteOptimizedSigned::signedBuffer ( Smartmatic::System::SafeBuffer & buffer )
{
    if ( buffer.GetSize() == 0 )
    {
        return false;
    }

    bool validate;

    SafeBuffer signature;
    validate = Encryption::getInstance()->SignDataWithPersonalCertificate ( buffer, signature );
    buffer.AddData ( signature.GetBase64Data() );

    return validate;
}
