/**
 * @file LogicalVoterGroupFactory.hxx
 * @brief Header class tool for create logical voter groups
 * @date 30/04/2013
 * @author Yadickson Soto
 */

#ifndef LOGICALVOTERGROUPFACTORY_HXX_
#define LOGICALVOTERGROUPFACTORY_HXX_

#include <Voting/VotingDevice/voting-devices-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            namespace VotingDevice
            {
                /**
                 * @class LogicalVoterGroupFactory
                 * @brief class tool for create logical voter groups
                 * @date 30/04/2013
                 * @author Yadickson Soto
                 */
                class LogicalVoterGroupFactory
                {
                    private:

                        /**
                         * @fn LogicalVoterGroupFactory ();
                         * @brief Class constructor
                         * @date 30/04/2013
                         * @author Yadickson Soto
                         */
                        LogicalVoterGroupFactory ();

                    public:

                        /**
                         * @fn virtual ~LogicalVoterGroupFactory();
                         * @brief Class destructor
                         * @date 30/04/2013
                         * @author Yadickson Soto
                         */
                        virtual ~LogicalVoterGroupFactory();

                        /**
                         * @fn static Logical_voter_group createLogicalVoterGroup ( Custom_attribute & attribute );
                         * @brief Method create new logical voter group without custom attribute
                         * @date 10/05/2013
                         * @author Yadickson Soto
                         * @return new logical voter group
                         */
                        static Logical_voter_group createLogicalVoterGroup ();

                        /**
                         * @fn static Logical_voter_group createLogicalVoterGroup ( Custom_attribute & attribute );
                         * @brief Method create new logical voter group by custom attribute
                         * @date 10/05/2013
                         * @author Yadickson Soto
                         * @param attribute attribute to add in logical voter group
                         * @return new logical voter group
                         */
                        static Logical_voter_group createLogicalVoterGroup ( Custom_attribute & attribute );

                        /**
                         * @fn static Logical_voter_group createLogicalVoterGroup ( std::vector < Custom_attribute > & attributes );
                         * @brief Method create new logical voter group by custom attribute list
                         * @date 30/04/2013
                         * @author Yadickson Soto
                         * @param attributes attributes list for add in logical voter group
                         * @return new logical voter group
                         */
                        static Logical_voter_group createLogicalVoterGroup ( std::vector < Custom_attribute > & attributes );
                };
            } /* namespace VotingDevice */
        } /* namespace Voting */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* LOGICALVOTERGROUPFACTORY_HXX_ */
