/**
 * @file	set-language-functionality.cc
 * @brief	set language functionality class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#include "Voting/set-language-functionality.h"
#include "Voting/Election/ElectionInstalationLanguages.h"
#include "Environment/Configurations/base-language-controller.h"
#include "Resources/election-string-resources-manager.h"
#include "Resources/ElectionAudioResourcesManager.hxx"
#include "Resources/GeneralAudioResourcesManager.hxx"
#include <gtkmm.h>

using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Resources;

static std::string currentLanguageCode = "en"; ///< current language

SetLanguageFunctionality::SetLanguageFunctionality()
{
}

SetLanguageFunctionality::~SetLanguageFunctionality()
{
}

std::string SetLanguageFunctionality::getCurrentLanguageCode ()
{
    return currentLanguageCode;
}

bool SetLanguageFunctionality::SetNewVotingLanguage ( std::string codeLanguage )
{
    std::list<Smartmatic::SAES::Voting::Election::Language> listLang =
        Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->getLanguages();
    std::list<Smartmatic::SAES::Voting::Election::Language>::iterator it;

    for ( it = listLang.begin(); it != listLang.end(); ++it )
    {
        if ( it->getCustom_code() == codeLanguage )
        {
            currentLanguageCode = codeLanguage;
            BaseLanguageController::LocalizeApplication ( it->getLanguage(), it->getCountry() );
            ElectionStringResourcesManager::getInstance()->setCurrentCulture ( it->getCustom_code() );
            ElectionAudioResourcesManager::getInstance()->setCurrentCulture ( it->getCustom_code() );
            GeneralAudioResourcesManager::getInstance()->setCurrentCulture ( it->getCustom_code() );
            return true;
        }
    }

    return false;
}

void SetLanguageFunctionality::getCurrentLanguageCountry ( std::string & language, std::string & country )
{
    BaseLanguageController::GetActualLanguage ( language, country );
}

bool SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( std::string language, std::string country )
{
    std::list<Smartmatic::SAES::Voting::Election::Language> listLang =
        Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->getLanguages();
    std::list<Smartmatic::SAES::Voting::Election::Language>::iterator it;
    Glib::ustring li = language;
    Glib::ustring ci = country;
    li = li.lowercase();
    ci = ci.uppercase();
    //Set environment languages
    BaseLanguageController::LocalizeApplication ( li.c_str(), ci.c_str() );

    for ( it = listLang.begin(); it != listLang.end(); ++it )
    {
        Glib::ustring lf = it->getLanguage();
        Glib::ustring cf = it->getCountry();
        lf = lf.lowercase();
        cf = cf.uppercase();

        if ( li == lf && ci == cf )
        {
            ElectionStringResourcesManager::getInstance()->setCurrentCulture ( it->getCustom_code() );
            return true;
        }
    }

    return false;
}
