/**
 * @file	set-language-functionality.h
 * @brief	Declares the set language functionality class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#ifndef _SET_LANGUAGE_FUNCTIONALITY_H_
#define _SET_LANGUAGE_FUNCTIONALITY_H_

#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            /**
             * @class	SetLanguageFunctionality
             * @brief	Set language functionality.
             * @author	Eduardo.campione
             * @date	28/04/2011
             */

            class SetLanguageFunctionality
            {
                public:

                    /**
                     * @fn	static std::string SetLanguageFunctionality::getCurrentLanguageCode ();
                     * @brief	Gets the current language code.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	The current language code.
                     */

                    static std::string getCurrentLanguageCode ();

                    /**
                     * @fn	static bool SetLanguageFunctionality::SetNewVotingLanguage(std::string codeLanguage);
                     * @brief	Sets a new voting language.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	codeLanguage	The code language.
                     * @return	true if it succeeds, false if it fails.
                     */

                    static bool SetNewVotingLanguage ( std::string codeLanguage );

                    /**
                     * @fn	static void SetLanguageFunctionality::getCurrentLanguageCountry (std::string & language,
                     * 			std::string & country);
                     * @brief	Gets a current language country.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	language	The language.
                     * @param	country 	The country.
                     */

                    static void getCurrentLanguageCountry ( std::string & language, std::string & country );

                    /**
                     * @fn	static bool SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry(std::string language,
                     * 			std::string country);
                     * @brief	Sets a new voting language by language country.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	language	The language.
                     * @param	country 	The country.
                     * @return	true if it succeeds, false if it fails.
                     */

                    static bool setNewVotingLanguageByLanguageCountry ( std::string language, std::string country );
                protected:

                    /**
                     * @fn	SetLanguageFunctionality::SetLanguageFunctionality();
                     * @brief	Default constructor.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    SetLanguageFunctionality();

                    /**
                     * @fn	virtual SetLanguageFunctionality::~SetLanguageFunctionality();
                     * @brief	Finaliser.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    virtual ~SetLanguageFunctionality();
            };
        }
    }
}

#endif // _SET_LANGUAGE_FUNCTIONALITY_H_
