#ifndef TEST_DIAGNOSTIC_LOG_CONF_MANAGER_TEST
#define TEST_DIAGNOSTIC_LOG_CONF_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockDiagnosticLogConfigurationManager.hxx>
#include <tests/MockSaesConfigurationManager.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>

class DiagnosticLogConfigurationManagerTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::SAES::Configuration::DiagnosticLogConfigurationManager DiagnosticLogConfigurationManager;
        typedef Smartmatic::SAES::Configuration::SAESConfigurationManager SAESConfigurationManager;
        typedef Smartmatic::SAES::Configuration::DiagnosticLogConfiguration DiagnosticLogConfiguration;
        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;

        DiagnosticLogConfigurationManagerTest()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testGetDiagnosticLogConfigurationPresent()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesConfigurationManager confManager;
            SAESConfigurationManager::getInstance ( &confManager );

            CPPUNIT_ASSERT ( true == SAESConfigurationManager::getInstance()->getConfiguration()->getDiagnosticLogConfiguration().present() );

            persistenceManager.verify();
        }

        void testGetDiagnosticLogConfigurationDefault()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesConfigurationManager confManager;
            SAESConfigurationManager::getInstance ( &confManager );

            SAESConfigurationManager::getInstance()->getConfiguration()->getDiagnosticLogConfiguration().reset();

            DiagnosticLogConfigurationManager * diagnosticLogManager = DiagnosticLogConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( false == diagnosticLogManager->shouldPersistDiagnosticLogFile() );
            CPPUNIT_ASSERT ( "," == diagnosticLogManager->getSeparator() );
            CPPUNIT_ASSERT ( "OK" == diagnosticLogManager->getStringOk() );
            CPPUNIT_ASSERT ( "NOK" == diagnosticLogManager->getStringNotOk() );
            CPPUNIT_ASSERT ( "N/A" == diagnosticLogManager->getStringNA() );

            delete diagnosticLogManager;

            persistenceManager.verify();
        }

        void testGetPersistence()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesConfigurationManager configurationManager;
            SAESConfigurationManager::getInstance ( &configurationManager );

            DiagnosticLogConfiguration & config ( SAESConfigurationManager::getInstance()->getConfiguration()->getDiagnosticLogConfiguration().get() );

            DiagnosticLogConfigurationManager * diagnosticLogManager = DiagnosticLogConfigurationManager::getInstance();

            config.setPersistence ( true );
            CPPUNIT_ASSERT ( true == diagnosticLogManager->shouldPersistDiagnosticLogFile() );

            config.setPersistence ( false );
            CPPUNIT_ASSERT ( false == diagnosticLogManager->shouldPersistDiagnosticLogFile() );

            delete diagnosticLogManager;

            persistenceManager.verify();
        }

        void testGetSeparator()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesConfigurationManager configurationManager;
            SAESConfigurationManager::getInstance ( &configurationManager );

            DiagnosticLogConfiguration & config ( SAESConfigurationManager::getInstance()->getConfiguration()->getDiagnosticLogConfiguration().get() );

            DiagnosticLogConfigurationManager * diagnosticLogManager = DiagnosticLogConfigurationManager::getInstance();

            config.setSeparator ( ":" );
            CPPUNIT_ASSERT ( ":" == diagnosticLogManager->getSeparator() );

            config.setSeparator ( ";" );
            CPPUNIT_ASSERT ( ";" == diagnosticLogManager->getSeparator() );

            delete diagnosticLogManager;

            persistenceManager.verify();
        }

        void testGetStringOk()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesConfigurationManager configurationManager;
            SAESConfigurationManager::getInstance ( &configurationManager );

            DiagnosticLogConfiguration & config ( SAESConfigurationManager::getInstance()->getConfiguration()->getDiagnosticLogConfiguration().get() );

            DiagnosticLogConfigurationManager * diagnosticLogManager = DiagnosticLogConfigurationManager::getInstance();

            config.setStringOk ( "OK" );
            CPPUNIT_ASSERT ( "OK" == diagnosticLogManager->getStringOk() );

            config.setStringOk ( "ok" );
            CPPUNIT_ASSERT ( "ok" == diagnosticLogManager->getStringOk() );

            delete diagnosticLogManager;

            persistenceManager.verify();
        }

        void testGetStringNOk()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesConfigurationManager configurationManager;
            SAESConfigurationManager::getInstance ( &configurationManager );

            DiagnosticLogConfiguration & config ( SAESConfigurationManager::getInstance()->getConfiguration()->getDiagnosticLogConfiguration().get() );

            DiagnosticLogConfigurationManager * diagnosticLogManager = DiagnosticLogConfigurationManager::getInstance();

            config.setStringNOk ( "NOK" );
            CPPUNIT_ASSERT ( "NOK" == diagnosticLogManager->getStringNotOk() );

            config.setStringNOk ( "nok" );
            CPPUNIT_ASSERT ( "nok" == diagnosticLogManager->getStringNotOk() );

            delete diagnosticLogManager;

            persistenceManager.verify();
        }

        void testGetStringNA()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesConfigurationManager configurationManager;
            SAESConfigurationManager::getInstance ( &configurationManager );

            DiagnosticLogConfiguration & config ( SAESConfigurationManager::getInstance()->getConfiguration()->getDiagnosticLogConfiguration().get() );

            DiagnosticLogConfigurationManager * diagnosticLogManager = DiagnosticLogConfigurationManager::getInstance();

            config.setStringNA ( "N/A" );
            CPPUNIT_ASSERT ( "N/A" == diagnosticLogManager->getStringNA() );

            config.setStringNA ( "n-a" );
            CPPUNIT_ASSERT ( "n-a" == diagnosticLogManager->getStringNA() );

            delete diagnosticLogManager;

            persistenceManager.verify();
        }

        CPPUNIT_TEST_SUITE ( DiagnosticLogConfigurationManagerTest );

        CPPUNIT_TEST ( testGetDiagnosticLogConfigurationPresent );
        CPPUNIT_TEST ( testGetDiagnosticLogConfigurationDefault );

        CPPUNIT_TEST ( testGetPersistence );
        CPPUNIT_TEST ( testGetSeparator );
        CPPUNIT_TEST ( testGetStringOk );
        CPPUNIT_TEST ( testGetStringNOk );
        CPPUNIT_TEST ( testGetStringNA );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_DIAGNOSTIC_LOG_CONF_MANAGER_TEST */
