#ifndef TEST_CUSTOM_ATTRIBUTE_DEFINITION_VALUE_TEST
#define TEST_CUSTOM_ATTRIBUTE_DEFINITION_VALUE_TEST

#ifdef ECUT_MAIN

#include <mockpp/framework/VerifyingTestCase.h>
#include <mockpp/ExpectationValue.h>
#include <mockpp/compat/AssertionFailedError.h>

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>

#include <Voting/VotingDevice/CustomAttributeDefinitionWrapper.hxx>

class CustomAttributeDefinitionValueTest : public CppUnit::TestFixture
{
    private:

        Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition * cat;

    public:

        CustomAttributeDefinitionValueTest ()
        {
            cat = NULL;
        }

        void setUp()
        {
            cat = new Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition();
            cat->setCode ( "" );
            cat->setDescription_key ( "" );
            cat->setMachine_type ( Smartmatic::SAES::Voting::VotingDevice::Machine_type::VOTING_DEVICES_VM );
            cat->setName_key ( "" );

            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_values::Custom_attribute_definition_valueSequence newList;
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_values values;
            values.setCustom_attribute_definition_value ( newList );
            cat->setCustom_attribute_definition_values ( values );
        }

        void tearDown()
        {
            delete cat;
            cat = NULL;
        }

        void test_1CustomAttributeDefinition_NotLanguageDefined()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value ( std::string ( "Code" ) );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat );

            CPPUNIT_ASSERT ( 1 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
        }

        void test_1CustomAttributeDefinition_LanguageDefined_FindSameLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value ( std::string ( "Code" ) );
            value.setLanguage ( "FR" );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "FR" );

            CPPUNIT_ASSERT ( 1 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
        }

        void test_1CustomAttributeDefinition_NotLanguageDefined_FindSameLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value ( std::string ( "Code" ) );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "NL" );

            CPPUNIT_ASSERT ( 1 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
        }

        void test_2CustomAttributeDefinition_NotLanguageDefined()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value1 ( std::string ( "Code1" ) );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value2 ( std::string ( "Code2" ) );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value1 );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value2 );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat );

            CPPUNIT_ASSERT ( 2 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code1" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
            CPPUNIT_ASSERT ( std::string ( "Code2" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [1].getValue() );
        }

        void test_2CustomAttributeDefinition_LanguageDefined_FindFirstLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value1 ( std::string ( "Code1" ) );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value2 ( std::string ( "Code2" ) );

            value1.setLanguage ( "FR" );
            value2.setLanguage ( "NL" );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value1 );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value2 );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "FR" );

            CPPUNIT_ASSERT ( 2 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code1" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
            CPPUNIT_ASSERT ( std::string ( "Code2" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [1].getValue() );
        }

        void test_2CustomAttributeDefinition_LanguageDefined_FindSecondLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value1 ( std::string ( "Code1" ) );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value2 ( std::string ( "Code2" ) );

            value1.setLanguage ( "FR" );
            value2.setLanguage ( "NL" );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value1 );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value2 );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "NL" );

            CPPUNIT_ASSERT ( 2 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code2" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
            CPPUNIT_ASSERT ( std::string ( "Code1" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [1].getValue() );
        }

        void test_2CustomAttributeDefinition_CombineLanguageDefined_FindFirstLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value1 ( std::string ( "Code1" ) );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value2 ( std::string ( "Code2" ) );

            value1.setLanguage ( "NL;FR" );
            value2.setLanguage ( "NL" );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value1 );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value2 );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "FR" );

            CPPUNIT_ASSERT ( 2 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code1" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
            CPPUNIT_ASSERT ( std::string ( "Code2" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [1].getValue() );
        }

        void test_2CustomAttributeDefinition_CombineLanguageDefined_FindSecondLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value1 ( std::string ( "Code1" ) );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value2 ( std::string ( "Code2" ) );

            value1.setLanguage ( "NL;FR" );
            value2.setLanguage ( "NL" );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value1 );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value2 );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "NL" );

            CPPUNIT_ASSERT ( 2 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code2" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
            CPPUNIT_ASSERT ( std::string ( "Code1" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [1].getValue() );
        }

        void test_2CustomAttributeDefinition_OneLanguageDefined_FindFirstLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value1 ( "Code1" );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value2 ( "Code2" );

            value1.setLanguage ( "FR" );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value1 );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value2 );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "FR" );

            CPPUNIT_ASSERT ( 2 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code1" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
            CPPUNIT_ASSERT ( std::string ( "Code2" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [1].getValue() );
        }

        void test_2CustomAttributeDefinition_OneLanguageDefined_FindSecondLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value1 ( "Code1" );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value2 ( "Code2" );

            value2.setLanguage ( "NL" );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value1 );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value2 );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "NL" );

            CPPUNIT_ASSERT ( 2 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code2" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
            CPPUNIT_ASSERT ( std::string ( "Code1" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [1].getValue() );
        }

        void test_2CustomAttributeDefinition_NotLanguageDefined_FindSomeLanguage()
        {
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value1 ( std::string ( "Code1" ) );
            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition_value value2 ( std::string ( "Code2" ) );

            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value1 );
            cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().push_back ( value2 );

            Smartmatic::SAES::Voting::VotingDevice::CustomAttributeDefinitionWrapper::sortByLanguage ( *cat, "NL" );

            CPPUNIT_ASSERT ( 2 == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value ().size() );
            CPPUNIT_ASSERT ( std::string ( "Code1" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [0].getValue() );
            CPPUNIT_ASSERT ( std::string ( "Code2" ) == cat->getCustom_attribute_definition_values().getCustom_attribute_definition_value () [1].getValue() );
        }

        CPPUNIT_TEST_SUITE ( CustomAttributeDefinitionValueTest );

        CPPUNIT_TEST ( test_1CustomAttributeDefinition_NotLanguageDefined );
        CPPUNIT_TEST ( test_1CustomAttributeDefinition_NotLanguageDefined_FindSameLanguage );
        CPPUNIT_TEST ( test_1CustomAttributeDefinition_LanguageDefined_FindSameLanguage );

        CPPUNIT_TEST ( test_2CustomAttributeDefinition_NotLanguageDefined );
        CPPUNIT_TEST ( test_2CustomAttributeDefinition_LanguageDefined_FindFirstLanguage );
        CPPUNIT_TEST ( test_2CustomAttributeDefinition_LanguageDefined_FindSecondLanguage );
        CPPUNIT_TEST ( test_2CustomAttributeDefinition_OneLanguageDefined_FindFirstLanguage );
        CPPUNIT_TEST ( test_2CustomAttributeDefinition_OneLanguageDefined_FindSecondLanguage );
        CPPUNIT_TEST ( test_2CustomAttributeDefinition_NotLanguageDefined_FindSomeLanguage );

        CPPUNIT_TEST ( test_2CustomAttributeDefinition_CombineLanguageDefined_FindFirstLanguage );
        CPPUNIT_TEST ( test_2CustomAttributeDefinition_CombineLanguageDefined_FindSecondLanguage );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_CUSTOM_ATTRIBUTE_DEFINITION_VALUE_TEST */
