#ifndef TEST_SESSIONID_MANAGER_TEST
#define TEST_SESSIONID_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Voting/IDManager/SessionIDManager.hxx>
#include <Voting/OperationStatus.hxx>

class SessionIDManagerTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::SAES::Voting::SessionIDManager SessionIDManager;
        typedef Smartmatic::SAES::Voting::OperationStatus OperationStatus;

        SessionIDManager * manager;

        SessionIDManagerTest ()
        {
            manager = NULL;
        }

        void setUp()
        {
            manager = SessionIDManager::getInstance();
        }

        void tearDown()
        {
            delete manager;
            manager = NULL;
        }

        void testGenerationSameUUID()
        {
            std::string uuid1 = manager->getID ( NULL );
            std::string uuid2 = manager->getID ( NULL );

            CPPUNIT_ASSERT ( false == uuid1.empty() );
            CPPUNIT_ASSERT ( false == uuid2.empty() );
            CPPUNIT_ASSERT ( uuid1 == uuid2 );
        }

        void testGenerationSameUUIDWithOperationStatus()
        {
            const std::list<std::string> runFunctionalityDescriptors;
            OperationStatus operationStatus ( NULL, runFunctionalityDescriptors );

            std::string uuid1 = manager->getID ( &operationStatus );
            std::string uuid2 = manager->getID ( &operationStatus );

            CPPUNIT_ASSERT ( false == uuid1.empty() );
            CPPUNIT_ASSERT ( false == uuid2.empty() );
            CPPUNIT_ASSERT ( uuid1 == uuid2 );
        }

        CPPUNIT_TEST_SUITE ( SessionIDManagerTest );

        CPPUNIT_TEST ( testGenerationSameUUID );
        CPPUNIT_TEST ( testGenerationSameUUIDWithOperationStatus );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SESSIONID_MANAGER_TEST */
