#ifndef OPERATION_STATUS_TEST_H_
#define OPERATION_STATUS_TEST_H_

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Voting/Status/DiagnoseStepWrapper.hxx>
#include <tests/MockDiagnosticLogConfigurationManager.hxx>
#include <Voting/OperationStatus.hxx>
#include <System/Compression/compression-class.h>
#include <System/Encoding/Base64Encoder.h>

class OperationStatusTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::Voting::OperationStatus OperationStatus;
        typedef Smartmatic::SAES::Voting::Status::VotingMachineStatusWrapper VotingMachineStatusWrapper;
        typedef Smartmatic::SAES::Common::Configuration::DeviceType DeviceType;
        typedef Smartmatic::SAES::Voting::Status::VMStatus VMStatus;
        typedef Smartmatic::SAES::Configuration::DiagnosticLogConfigurationManager DiagnosticLogConfigurationManager;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;
        typedef Smartmatic::System::Compression::CompressionClass CompressionClass;
        typedef Smartmatic::System::Encoding::Base64Encoder Base64Encoder;
        typedef Smartmatic::SAES::Runtime::Environment::SaesDirectories SaesDirectories;

    public:

        OperationStatusTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testVotingMachineStatusResultWithoutProcess()
        {
            MockDiagnosticLogConfigurationManager mock;
            DiagnosticLogConfigurationManager::getInstance ( &mock );

            MOCKPP_CHAINER_FOR ( MockDiagnosticLogConfigurationManager, getSeparator ) getSeparator_mock ( &mock );
            MOCKPP_CHAINER_FOR ( MockDiagnosticLogConfigurationManager, getStringOk ) getStringOk_mock ( &mock );
            MOCKPP_CHAINER_FOR ( MockDiagnosticLogConfigurationManager, getStringNotOk ) getStringNotOk_mock ( &mock );
            MOCKPP_CHAINER_FOR ( MockDiagnosticLogConfigurationManager, getStringNA ) getStringNA_mock ( &mock );

            getSeparator_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "," ) );

            getStringOk_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "OK" ) );

            getStringNotOk_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "NOK" ) );

            getStringNA_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "N/A" ) );

            const std::list<std::string> runFunctionalityDescriptors;
            OperationStatus operationStatus ( NULL, runFunctionalityDescriptors );

            VotingMachineStatusWrapper * wrapper = operationStatus.getVotingMachineStatusWrapper();

            std::stringstream expected;

            expected << "V:" << PACKAGE_VERSION << ",";
            expected << "SND:ABCD1234,";
            expected << "SNM:ABCD4321,";
            expected << "D:01/01/2013,";
            expected << "Z:1234,";
            expected << "A:NOK,";
            expected << "B:OK,";
            expected << "S:NOK,";
            expected << "P:OK,";
            expected << "R:Hello,";

            wrapper->setSerialDetected ( "ABCD1234" );
            wrapper->setSerial ( "ABCD4321" );
            wrapper->setDate ( "01/01/2013" );
            wrapper->setZipCode ( "1234" );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_ALARMBOX, VMStatus::VOTING_MACHINE_STATUS_ERROR );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_BARCODE, VMStatus::VOTING_MACHINE_STATUS_OK );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_SMARTCARD, VMStatus::VOTING_MACHINE_STATUS_ERROR );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_PRINTER, VMStatus::VOTING_MACHINE_STATUS_OK );
            wrapper->setObservation ( "Hello", 10 );

            SafeBuffer bufferExpected ( expected.str() );
            SafeBuffer bufferResult;

            bufferResult = operationStatus.getQRDiagnosticResult ( false );

            CPPUNIT_ASSERT ( bufferExpected == bufferResult );

            mock.verify();
        }

        void testVotingMachineStatusResultWithProcess()
        {
            MockDiagnosticLogConfigurationManager mock;
            DiagnosticLogConfigurationManager::getInstance ( &mock );

            MOCKPP_CHAINER_FOR ( MockDiagnosticLogConfigurationManager, getSeparator ) getSeparator_mock ( &mock );
            MOCKPP_CHAINER_FOR ( MockDiagnosticLogConfigurationManager, getStringOk ) getStringOk_mock ( &mock );
            MOCKPP_CHAINER_FOR ( MockDiagnosticLogConfigurationManager, getStringNotOk ) getStringNotOk_mock ( &mock );
            MOCKPP_CHAINER_FOR ( MockDiagnosticLogConfigurationManager, getStringNA ) getStringNA_mock ( &mock );

            getSeparator_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "," ) );

            getStringOk_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "OK" ) );

            getStringNotOk_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "NOK" ) );

            getStringNA_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "N/A" ) );

            const std::list<std::string> runFunctionalityDescriptors;
            OperationStatus operationStatus ( NULL, runFunctionalityDescriptors );

            VotingMachineStatusWrapper * wrapper = operationStatus.getVotingMachineStatusWrapper();

            std::stringstream expected;

            expected << "V:" << PACKAGE_VERSION << ",";
            expected << "SND:ABCD1234,";
            expected << "SNM:ABCD4321,";
            expected << "D:01/01/2013,";
            expected << "Z:1234,";
            expected << "A:NOK,";
            expected << "B:OK,";
            expected << "S:NOK,";
            expected << "P:OK,";
            expected << "R:Hello,";

            wrapper->setSerialDetected ( "ABCD1234" );
            wrapper->setSerial ( "ABCD4321" );
            wrapper->setDate ( "01/01/2013" );
            wrapper->setZipCode ( "1234" );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_ALARMBOX, VMStatus::VOTING_MACHINE_STATUS_ERROR );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_BARCODE, VMStatus::VOTING_MACHINE_STATUS_OK );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_SMARTCARD, VMStatus::VOTING_MACHINE_STATUS_ERROR );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_PRINTER, VMStatus::VOTING_MACHINE_STATUS_OK );
            wrapper->setObservation ( "Hello", 10 );

            SafeBuffer bufferExpected ( expected.str() );
            SafeBuffer bufferResult;

            bufferResult = operationStatus.getQRDiagnosticResult ( true );

            CPPUNIT_ASSERT ( 0 != bufferResult.GetSize() );
            CPPUNIT_ASSERT ( bufferExpected != bufferResult );

            std::string base64String;
            base64String.assign ( ( const char * ) bufferResult.GetData(), bufferResult.GetSize() );

            SafeBuffer decode;

            CPPUNIT_ASSERT ( true == Base64Encoder::ConvertFromBase64 ( base64String, decode ) );
            CPPUNIT_ASSERT ( bufferExpected != decode );

            SafeBuffer inflate;

            CPPUNIT_ASSERT ( true == CompressionClass::getInstance()->Inflate ( decode, inflate ) );
            CPPUNIT_ASSERT ( bufferExpected == inflate );

            mock.verify();
        }

        CPPUNIT_TEST_SUITE ( OperationStatusTest );

        CPPUNIT_TEST ( testVotingMachineStatusResultWithoutProcess );
        CPPUNIT_TEST ( testVotingMachineStatusResultWithProcess );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* OPERATION_STATUS_TEST_H_ */
