#ifndef TEST_VOTE_RECOVERYTEST
#define TEST_VOTE_RECOVERYTEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Voting/Vote/VoteFactory.hxx>
#include <Voting/Vote/VoteValidate.hxx>
#include "MockVoteRecovery.hxx"
#include <Voting/Vote/VoteRecovery.h>

class VoteRecoveryTest : public CPPUNIT_NS::TestFixture
{
    private:

        Smartmatic::SAES::Voting::Vote::Vote * vote;
        Smartmatic::SAES::Voting::Vote::Vote * vote2;
        MockVoteRecovery mock;
        Smartmatic::SAES::Voting::Vote::VoteRecovery * recovery;

    public:

        VoteRecoveryTest ()
        {
            vote = NULL;
            vote2 = NULL;
            recovery = NULL;
        }

        void setUp()
        {
            vote = Smartmatic::SAES::Voting::Vote::VoteFactory::createVote();
            vote2 = Smartmatic::SAES::Voting::Vote::VoteFactory::createVote();
            recovery = new Smartmatic::SAES::Voting::Vote::VoteRecovery ( &mock );
        }

        void tearDown()
        {
            if ( vote )
            {
                delete vote;
                vote = NULL;
            }

            if ( vote2 )
            {
                delete vote2;
                vote2 = NULL;
            }

            if ( recovery )
            {
                delete recovery;
                recovery = NULL;
            }
        }

        void testValidateVoteSelecctionNull()
        {
            Smartmatic::SAES::Voting::Vote::Selection s ( "C1" );

            vote->getSelections().push_back ( s );

            std::map<std::string, std::string> result = recovery->LogicalToRecovery ( *vote );

            CPPUNIT_ASSERT ( 1 == result.size() );
            CPPUNIT_ASSERT ( "C1" == result.begin()->first );
            CPPUNIT_ASSERT ( "0" == result.begin()->second );
        }

        void testValidateVoteTwoSelecctionNull()
        {
            Smartmatic::SAES::Voting::Vote::Selection s1 ( "C1" );
            Smartmatic::SAES::Voting::Vote::Selection s2 ( "C2" );

            vote->getSelections().push_back ( s1 );
            vote->getSelections().push_back ( s2 );

            std::map<std::string, std::string> result = recovery->LogicalToRecovery ( *vote );

            CPPUNIT_ASSERT ( 2 == result.size() );

            std::map<std::string, std::string>::iterator it = result.begin();

            CPPUNIT_ASSERT ( "C1" == it->first );
            CPPUNIT_ASSERT ( "0" == it->second );

            ++it;

            CPPUNIT_ASSERT ( "C2" == it->first );
            CPPUNIT_ASSERT ( "0" == it->second );
        }

        void testValidateVoteSelecctionOneBallot()
        {
            Smartmatic::SAES::Voting::Vote::Selection s ( "C1" );
            s.getBallotOptions().push_back ( "1x1xL1" );

            vote->getSelections().push_back ( s );

            std::map<std::string, std::string> result = recovery->LogicalToRecovery ( *vote );

            CPPUNIT_ASSERT ( 1 == result.size() );
            CPPUNIT_ASSERT ( "C1" == result.begin()->first );
            CPPUNIT_ASSERT ( "0" != result.begin()->second );
        }

        void testValidateVoteSelecctionTwoBallot()
        {
            Smartmatic::SAES::Voting::Vote::Selection s ( "C1" );
            s.getBallotOptions().push_back ( "1x1xL1" );
            s.getBallotOptions().push_back ( "1x1xS1" );

            vote->getSelections().push_back ( s );

            std::map<std::string, std::string> result = recovery->LogicalToRecovery ( *vote );

            CPPUNIT_ASSERT ( 1 == result.size() );
            CPPUNIT_ASSERT ( "C1" == result.begin()->first );
            CPPUNIT_ASSERT ( "0" != result.begin()->second );
        }

        void testValidateRegretion()
        {
            Smartmatic::SAES::Voting::Vote::Selection s ( "C1" );
            s.getBallotOptions().push_back ( "1x1xL1" );
            s.getBallotOptions().push_back ( "1x1xS1" );

            vote->getSelections().push_back ( s );

            std::map<std::string, std::string> result = recovery->LogicalToRecovery ( *vote );

            recovery->RecoveryToLogical ( result, *vote2 );

            std::map<std::string, std::string> result2 = recovery->LogicalToRecovery ( *vote2 );

            CPPUNIT_ASSERT ( 1 == result.size() );
            CPPUNIT_ASSERT ( "C1" == result.begin()->first );
            CPPUNIT_ASSERT ( "0" != result.begin()->second );

            CPPUNIT_ASSERT ( result.size() == result2.size() );
            CPPUNIT_ASSERT ( result.begin()->first == result2.begin()->first );
            CPPUNIT_ASSERT ( result.begin()->second == result2.begin()->second );
        }

        void testValidateRegretionNull()
        {
            Smartmatic::SAES::Voting::Vote::Selection s ( "C1" );

            vote->getSelections().push_back ( s );

            std::map<std::string, std::string> result = recovery->LogicalToRecovery ( *vote );

            recovery->RecoveryToLogical ( result, *vote2 );

            std::map<std::string, std::string> result2 = recovery->LogicalToRecovery ( *vote2 );

            CPPUNIT_ASSERT ( 1 == result.size() );
            CPPUNIT_ASSERT ( "C1" == result.begin()->first );
            CPPUNIT_ASSERT ( "0" == result.begin()->second );

            CPPUNIT_ASSERT ( result.size() == result2.size() );
            CPPUNIT_ASSERT ( result.begin()->first == result2.begin()->first );
            CPPUNIT_ASSERT ( result.begin()->second == result2.begin()->second );
        }

        CPPUNIT_TEST_SUITE ( VoteRecoveryTest );
        CPPUNIT_TEST ( testValidateVoteSelecctionNull );
        CPPUNIT_TEST ( testValidateVoteTwoSelecctionNull );
        CPPUNIT_TEST ( testValidateVoteSelecctionOneBallot );
        CPPUNIT_TEST ( testValidateVoteSelecctionTwoBallot );
        CPPUNIT_TEST ( testValidateRegretion );
        CPPUNIT_TEST ( testValidateRegretionNull );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_VOTE_RECOVERYTEST */
