/**
 * @file   DefaultSmartcardReaderPowerManager.hxx
 * @brief  Implementation of DefaultSmartcardReaderPowerManager class.
 * @author manuel.arguelles
 * @date   Nov  1, 2017
 */

#include "DefaultSmartcardReaderPowerManager.hxx"
#include <Configuration/SmartcardConfigurationManager.hxx>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Runtime/ProcessLauncher.h>
#include <pcscd/PCSCDManager.hxx>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::PCSCD;
using namespace Smartmatic::System::Runtime;

ISMTTLog* DefaultSmartcardReaderPowerManager::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Card.DefaultSmartcardReaderPowerManager",
    GETTEXT_PACKAGE);

DefaultSmartcardReaderPowerManager::DefaultSmartcardReaderPowerManager()
{
}

DefaultSmartcardReaderPowerManager::~DefaultSmartcardReaderPowerManager()
{
}

void DefaultSmartcardReaderPowerManager::doReaderPowerCycle()
{
    if (!SmartcardConfigurationManager::getInstance()->getForceRestartSmartcardReader() &&
        !SmartcardConfigurationManager::getInstance()->getForceRestartPCSCD() )
    {
        m_logger->Debug("PCSCD restart and power cycle are disabled, so no corrective action is applied.");
        return;
    }

    if (SmartcardConfigurationManager::getInstance()->getForceRestartPCSCD())
    {
        m_logger->Debug("Stop PCSCD");

        PCSCDManager::stop(true);

        sleep(SmartcardConfigurationManager::getInstance()->getWaitStopPCSCD());
    }

    if (SmartcardConfigurationManager::getInstance()->getForceRestartSmartcardReader())
    {
        m_logger->Debug("Restart smartcard reader");

        callRestartSmartcard();

        sleep(SmartcardConfigurationManager::getInstance()->getDelayAfterOn());
    }

    if (SmartcardConfigurationManager::getInstance()->getForceRestartPCSCD())
    {
        m_logger->Debug("Start PCSCD");

        PCSCDManager::start(true);

        sleep(SmartcardConfigurationManager::getInstance()->getWaitStartPCSCD());
    }
}

void DefaultSmartcardReaderPowerManager::callRestartSmartcard(const bool background)
{

}

void DefaultSmartcardReaderPowerManager::callPowerOffSmartcard()
{

}

void DefaultSmartcardReaderPowerManager::callPowerOnSmartcard()
{

}

DefaultSmartcardReaderPowerManager::PowerCycledSignal DefaultSmartcardReaderPowerManager::signalPowerCycled()
{
    return m_powerCycledSignal;
}
