/**
 * @file GlobalPlatformModeGP211.hxx
 * @brief The global platform connection mode: GP_211
 * @author Yadickson Soto
 * @date 20/05/2014
 */

#ifndef GLOBALPLATFORMMODEGP211_HXX_
#define GLOBALPLATFORMMODEGP211_HXX_

#include <Card/GlobalPlatformMode.hxx>
#include <globalplatform/globalplatform.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Card
        {
            /**
             * @class GlobalPlatformModeGP211
             * @brief The global platform connection mode: GP_211
             * @author Yadickson Soto
             * @date 20/05/2014
             */
            class GlobalPlatformModeGP211 : public GlobalPlatformMode
            {
                public:

                    /**
                     * @fn GlobalPlatformModeGP211();
                     * @brief Class constructor.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @param cardInfo Information describing the card
                     * @param cardContext Context for the card
                     */
                    GlobalPlatformModeGP211 ( OPGP_CARD_INFO & cardInfo, OPGP_CARD_CONTEXT & cardContext );

                    /**
                     * @fn virtual ~GlobalPlatformModeGP211();
                     * @brief Class destructor.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     */
                    virtual ~GlobalPlatformModeGP211();

                    /**
                     * @fn virtual unsigned int getPlatformMode() = 0;
                     * @brief Gets the platform mode value.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return The platform mode value
                     */
                    virtual unsigned int getPlatformMode();

                protected:

                    /**
                     * @fn virtual OPGP_ERROR_STATUS setupSecureChannel();
                     * @brief Implements the mutual authentication.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return result status code
                     */
                    virtual OPGP_ERROR_STATUS setupSecureChannel();

                    /**
                     * @fn virtual OPGP_ERROR_STATUS processCommand ( const Smartmatic::System::SafeBuffer & command, unsigned char * response, unsigned long & length );
                     * @brief Sends a command to the card reader and gets the response. Platform-mode specific implementation.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @param command Command sent by the reader to the card.
                     * @param response buffer for the response
                     * @param length response buffer length
                     * @return result status code
                     */
                    virtual OPGP_ERROR_STATUS processCommand ( const Smartmatic::System::SafeBuffer & command, unsigned char * response, unsigned long & length );

                private:

                    GP211_SECURITY_INFO securityInfo; ///< The first security information

                    unsigned int securityLevel; ///< The security level.
                    unsigned int securityChannelProtocol; ///< The security channel protocol.
                    unsigned int securityChannelProtocolImpl; ///< The security channel protocol implementation.
            };

        } /* namespace Card */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* GLOBALPLATFORMMODEGP211_HXX_ */
