/**
 * @file   ISmartcardReaderPowerManager.hxx
 * @brief  Declaration of ISmartcardReaderPowerManager interface.
 * @author manuel.arguelles
 * @date   Nov  1, 2017
 */

#ifndef I_SMARTCARD_READER_POWER_MANAGER_HXX
#define I_SMARTCARD_READER_POWER_MANAGER_HXX

#include <sigc++/sigc++.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Card
        {
            /**
             * @class  ISmartcardReaderPowerManager
             * @brief  Interface for the Smartcard reader power manager.
             * @author manuel.arguelles
             * @date   Nov  1, 2017
             */
            class ISmartcardReaderPowerManager
            {
                public:

                    /**
                     * @brief  Definition of power cycled signal.
                     * @author manuel.arguelles
                     * @date   Nov 11, 2017
                     */
                    typedef sigc::signal<void> PowerCycledSignal;

                public:

                    /**
                     * @brief  Destroys an instance of the class.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    virtual ~ISmartcardReaderPowerManager() {}

                public:

                    /**
                     * @brief  Restart the reader with a full power cycle.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    virtual void doReaderPowerCycle() = 0;

                    /**
                     * @brief  Restarts the reader.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     * @param  background If the operation should be done asynchronous.
                     */
                    virtual void callRestartSmartcard(bool background = false) = 0;

                    /**
                     * @brief  Powers off the reader.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    virtual void callPowerOffSmartcard() = 0;

                    /**
                     * @brief  Powers on the reader.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    virtual void callPowerOnSmartcard() = 0;

                    /**
                     * @brief  Emitted when the smartcard reader was power cycled.
                     * @author manuel.arguelles
                     * @date   Nov 11, 2017
                     */
                    virtual PowerCycledSignal signalPowerCycled() = 0;
            };

        } /* namespace Card */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* I_SMARTCARD_READER_POWER_MANAGER_HXX */
