/**
 * @file   P168ASmartcardReaderPowerManager.hxx
 * @brief  Implementation of P168ASmartcardReaderPowerManager class.
 * @author manuel.arguelles
 * @date   Nov  1, 2017
 */

#include "P168ASmartcardReaderPowerManager.hxx"
#include <Configuration/SmartcardConfigurationManager.hxx>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Runtime/ProcessLauncher.h>
#include <pcscd/PCSCDManager.hxx>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::PCSCD;
using namespace Smartmatic::System::Runtime;

ISMTTLog* P168ASmartcardReaderPowerManager::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Card.P168ASmartcardReaderPowerManager",
    GETTEXT_PACKAGE);

P168ASmartcardReaderPowerManager::P168ASmartcardReaderPowerManager()
{
}

P168ASmartcardReaderPowerManager::~P168ASmartcardReaderPowerManager()
{
}

void P168ASmartcardReaderPowerManager::doReaderPowerCycle()
{
    if (!SmartcardConfigurationManager::getInstance()->getForceRestartSmartcardReader() &&
        !SmartcardConfigurationManager::getInstance()->getForceRestartPCSCD() )
    {
        m_logger->Debug("PCSCD restart and power cycle are disabled, so no corrective action is applied.");
        return;
    }

    if (SmartcardConfigurationManager::getInstance()->getForceRestartPCSCD())
    {
        m_logger->Debug("Stop PCSCD");

        PCSCDManager::stop(true);

        sleep(SmartcardConfigurationManager::getInstance()->getWaitStopPCSCD());
    }

    if (SmartcardConfigurationManager::getInstance()->getForceRestartSmartcardReader())
    {
        m_logger->Debug("Restart smartcard reader");

        callRestartSmartcard();

        sleep(SmartcardConfigurationManager::getInstance()->getDelayAfterOn());
    }

    if (SmartcardConfigurationManager::getInstance()->getForceRestartPCSCD())
    {
        m_logger->Debug("Start PCSCD");

        PCSCDManager::start(true);

        sleep(SmartcardConfigurationManager::getInstance()->getWaitStartPCSCD());
    }
}

void P168ASmartcardReaderPowerManager::callRestartSmartcard(const bool background)
{
    std::string command = SAESCommonConfigurationManager::getInstance()->getOSConfigurationFiles().getScriptRunSmartcard();

    if (ProcessLauncher::getInstance()->RunProcessThroughSystem(command, background) == -1)
    {
        m_logger->Debug(_("Error calling smartcard restart command"));
    }
}

void P168ASmartcardReaderPowerManager::callPowerOffSmartcard()
{
    std::string command = SAESCommonConfigurationManager::getInstance()->getOSConfigurationFiles().getScriptRunSmartcard();
    command.append(" OFF");

    if (ProcessLauncher::getInstance()->RunProcessThroughSystem(command) == -1)
    {
        m_logger->Debug(_("Error calling smartcard power off command"));
    }

}

void P168ASmartcardReaderPowerManager::callPowerOnSmartcard()
{
    std::string command = SAESCommonConfigurationManager::getInstance()->getOSConfigurationFiles().getScriptRunSmartcard();
    command.append (" ON");

    if (ProcessLauncher::getInstance()->RunProcessThroughSystem(command) == -1)
    {
        m_logger->Debug(_("Error calling smartcard power on command"));
    }
}

P168ASmartcardReaderPowerManager::PowerCycledSignal P168ASmartcardReaderPowerManager::signalPowerCycled()
{
    return m_powerCycledSignal;
}
