/**
 * @file   P168ASmartcardReaderPowerManager.hxx
 * @brief  Declaration of P168ASmartcardReaderPowerManager class.
 * @author manuel.arguelles
 * @date   Nov  1, 2017
 */

#ifndef P168A_SMARTCARD_READER_POWER_MANAGER_HXX
#define P168A_SMARTCARD_READER_POWER_MANAGER_HXX

#include <Card/ISmartcardReaderPowerManager.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Card
        {
            /**
             * @class  P168ASmartcardReaderPowerManager
             * @brief  Handles the smartcard reader power via P168A.
             * @author manuel.arguelles
             * @date   Nov  1, 2017
             */
            class P168ASmartcardReaderPowerManager : public ISmartcardReaderPowerManager
            {
                public:

                    /**
                     * @brief  Creates an instance of the class.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    P168ASmartcardReaderPowerManager();

                    /**
                     * @brief  Destroys an instance of the class.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    virtual ~P168ASmartcardReaderPowerManager();

                public:

                    /**
                     * @brief  Restart the reader with a full power cycle.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    virtual void doReaderPowerCycle();

                    /**
                     * @brief  Restarts the reader.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     * @param  background If the operation should be done asynchronous.
                     */
                    virtual void callRestartSmartcard(bool background = false);

                    /**
                     * @brief  Powers off the reader.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    virtual void callPowerOffSmartcard();

                    /**
                     * @brief  Powers on the reader.
                     * @author manuel.arguelles
                     * @date   Nov  1, 2017
                     */
                    virtual void callPowerOnSmartcard();

                    /**
                     * @brief  Emitted when the power was cycled.
                     * @author manuel.arguelles
                     * @date   Nov 11, 2017
                     */
                    virtual PowerCycledSignal signalPowerCycled();

                private:

                    static Smartmatic::Log::ISMTTLog* m_logger;
                    PowerCycledSignal m_powerCycledSignal;
            };

        } /* namespace Card */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* P168A_SMARTCARD_READER_POWER_MANAGER_HXX */
