/**
 * @file SmartcardCodeManager.hxx
 * @brief Manager class to save error code from smartcard reader
 * @date 10/01/2014
 * @author Yadickson Soto
 */

#ifndef SMARTCARDCODE_MANAGER_HXX_
#define SMARTCARDCODE_MANAGER_HXX_

#include <string>
#include <list>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Card
        {

            /**
             * @class SmartcardCodeManager
             * @brief Manager class to save error code from smartcard reader
             * @date 10/01/2014
             * @author Yadickson Soto
             */
            class SmartcardCodeManager
            {
                protected:
                    /**
                     * @fn SmartcardCodeManager();
                     * @brief Class constructor
                     * @date 10/01/2014
                     * @author Yadickson Soto
                     */
                    SmartcardCodeManager();

                    static SmartcardCodeManager * instance; ///< instance of the class

                public:
                    /**
                     * @fn virtual ~SmartcardCodeManager();
                     * @brief Class destructorer
                     * @date 10/01/2014
                     * @author Yadickson Soto
                     */
                    virtual ~SmartcardCodeManager();
                    /**
                     * @fn void start();
                     * @brief Initialize save code process
                     * @date 10/01/2014
                     * @author Yadickson Soto
                     */
                    void start();
                    /**
                     * @fn void add( unsigned long code );
                     * @brief Add new code to save, only if start method is call
                     * @param code new code to save
                     * @date 10/01/2014
                     * @author Yadickson Soto
                     */
                    void add ( unsigned long code );
                    /**
                     * @fn const std::list < std::string > & getCodes() const ;
                     * @brief Getter all hexadecimal codes string
                     * @return all hexadecimal codes string
                     * @date 10/01/2014
                     * @author Yadickson Soto
                     */
                    const std::list < std::string > & getCodes() const ;
                    /**
                     * @fn void clear();
                     * @brief Clear all code saves
                     * @date 10/01/2014
                     * @author Yadickson Soto
                     */
                    void clear();
                    /**
                     * @fn void stop();
                     * @brief Stop save code process
                     * @date 10/01/2014
                     * @author Yadickson Soto
                     */
                    void stop();
                    /**
                     * @fn static SmartcardCodeManager * getInstance();
                     * @brief Getter method to instance of the class
                     * @return instance of the class
                     * @date 10/01/2014
                     * @author Yadickson Soto
                     */
                    static SmartcardCodeManager * getInstance();

                private:

                    bool saveProcess; ///< flag save process
                    std::list < std::string > codes; ///< code list
            };

        } /* namespace Card */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* SMARTCARDCODE_MANAGER_HXX_ */
