/**
 * @file	DeviceTypeMonitorConfigurationManager.cxx
 * @brief	Implementation of the device type monitor configuration manager.
 * @author	Yadickson Soto
 * @date	07/05/2014
 */

#include "DeviceTypeMonitorConfigurationManager.hxx"
#include "s-a-e-s-common-conf-manager.h"
#include <System/Utils/StringUtils.hxx>

using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::Utils;

DeviceTypeMonitorConfigurationManager * DeviceTypeMonitorConfigurationManager::instance = NULL;

DeviceTypeMonitorConfigurationManager::DeviceTypeMonitorConfigurationManager()
{

}

DeviceTypeMonitorConfigurationManager::~DeviceTypeMonitorConfigurationManager()
{
    instance = NULL;
}

DeviceTypeMonitorConfiguration DeviceTypeMonitorConfigurationManager::getDeviceTypeMonitorConfiguration ( DeviceType type )
{
    DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence & seq ( getDeviceTypeMonitorSequence() );
    DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorIterator it;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        if ( ( *it ).getType() == type )
        {
            return ( *it );
        }
    }

    DeviceTypeMonitorConfiguration config ( type, " ## " );
    DeviceTypeMonitorConfiguration::PropertySequence sequ;
    config.setProperty ( sequ );

    return config;
}

DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence & DeviceTypeMonitorConfigurationManager::getDeviceTypeMonitorSequence()
{
    return getDeviceTypeMonitorPoolConfiguration().getDeviceTypeMonitor();
}

DeviceTypeMonitorPoolConfiguration & DeviceTypeMonitorConfigurationManager::getDeviceTypeMonitorPoolConfiguration ()
{
    if ( !SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().present() )
    {
        DeviceTypeMonitorPoolConfiguration config;

        DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence seq;
        config.setDeviceTypeMonitor ( seq );

        SAESCommonConfigurationManager::getInstance()->getConfiguration()->setDeviceTypeMonitorPoolConfiguration ( config );
    }

    return SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().get();
}

std::string DeviceTypeMonitorConfigurationManager::getCustomNode ( const std::string & node, DeviceType type )
{
    std::string command = node;

    DeviceTypeMonitorConfiguration config = getDeviceTypeMonitorConfiguration ( type );

    if ( config.getCustomNode().present() )
    {
        std::string placeholder = config.getCustomNode().get();
        command = StringUtils::replace ( placeholder, "%1", node );
    }

    return command;
}

std::string DeviceTypeMonitorConfigurationManager::getSimulatorFilePath ()
{
    std::string file;

    if ( getDeviceTypeMonitorPoolConfiguration ().getSimulatorEventDeviceFilePath().present() )
    {
        file = getDeviceTypeMonitorPoolConfiguration ().getSimulatorEventDeviceFilePath().get();
    }

    return file;
}

bool DeviceTypeMonitorConfigurationManager::simulationEnabled ()
{
    return !getSimulatorFilePath ().empty();
}

DeviceTypeMonitorConfigurationManager * DeviceTypeMonitorConfigurationManager::getInstance ( DeviceTypeMonitorConfigurationManager * externManager )
{
    if ( ! instance )
    {
        instance = ( externManager == NULL ? new DeviceTypeMonitorConfigurationManager() : externManager );
    }

    return instance;
}
