/**
 * @file	SmartcardConfigurationManager.cxx
 * @brief	Implementation of the smartcard configuration manager.
 * @author	Yadickson Soto
 * @date	28/03/2014
 */

#include "SmartcardConfigurationManager.hxx"
#include "s-a-e-s-common-conf-manager.h"

using namespace Smartmatic::SAES::Common::Configuration;

SmartcardConfigurationManager * SmartcardConfigurationManager::instance = NULL;

SmartcardConfigurationManager::SmartcardConfigurationManager()
{

}

SmartcardConfigurationManager::~SmartcardConfigurationManager()
{
    instance = NULL;
}

std::string SmartcardConfigurationManager::getAppletID()
{
    return getSmartcardConfiguration().getAppletID();
}

std::string SmartcardConfigurationManager::getGlobalPIN()
{
    return getSmartcardConfiguration().getGlobalPIN();
}

PlatformMode SmartcardConfigurationManager::getPlatformMode()
{
    return getSmartcardConfiguration().getPlatformMode();
}

bool SmartcardConfigurationManager::hasSecureChannel()
{
    return getSmartcardConfiguration().getSecureChannelEnabled();
}

int SmartcardConfigurationManager::getSecurityKeySetVersion()
{
    return getSmartcardConfiguration().getSecurityKeySetVersion();
}

int SmartcardConfigurationManager::getSecurityKeyIndex()
{
    return getSmartcardConfiguration().getSecurityKeyIndex();
}

SecurityChannelProtocol SmartcardConfigurationManager::getSecurityChannelProtocol()
{
    if ( !getSmartcardConfiguration().getSecurityChannelProtocol().present() )
    {
        getSmartcardConfiguration().setSecurityChannelProtocol ( SecurityChannelProtocol::CONFIGURATION_SCP01 );
    }

    return getSmartcardConfiguration().getSecurityChannelProtocol().get();
}

SecurityChannelProtocolImpl SmartcardConfigurationManager::getSecurityChannelProtocolImpl()
{
    if ( !getSmartcardConfiguration().getSecurityChannelProtocolImpl().present() )
    {
        getSmartcardConfiguration().setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I05 );
    }

    return getSmartcardConfiguration().getSecurityChannelProtocolImpl().get();
}

SecurityLevel SmartcardConfigurationManager::getSecurityLevel()
{
    if ( !getSmartcardConfiguration().getSecurityLevel().present() )
    {
        getSmartcardConfiguration().setSecurityLevel ( SecurityLevel::CONFIGURATION_C_MAC );
    }

    return getSmartcardConfiguration().getSecurityLevel().get();
}

SecurityKeyDerivation SmartcardConfigurationManager::getSecurityKeyDerivation()
{
    if ( !getSmartcardConfiguration().getSecurityKeyDerivation().present() )
    {
        getSmartcardConfiguration().setSecurityKeyDerivation ( SecurityKeyDerivation::CONFIGURATION_NONE );
    }

    return getSmartcardConfiguration().getSecurityKeyDerivation().get();
}

bool SmartcardConfigurationManager::getForceRestartSmartcardReader()
{
    return getSmartcardConfiguration().getForceRestartSmartcardReader();
}

bool SmartcardConfigurationManager::getForceRestartPCSCD()
{
    return getSmartcardConfiguration().getForceRestartPCSCD();
}

unsigned int SmartcardConfigurationManager::getDelayAfterOn()
{
    return getSmartcardConfiguration().getDelayAfterOn();
}

unsigned int SmartcardConfigurationManager::getWaitStopPCSCD()
{
    return getSmartcardConfiguration().getWaitStopPCSCD();
}

unsigned int SmartcardConfigurationManager::getWaitStartPCSCD()
{
    return getSmartcardConfiguration().getWaitStartPCSCD();
}

bool SmartcardConfigurationManager::getCallRestartBeforeInitialize()
{
    return getSmartcardConfiguration().getCallRestartBeforeInitialize();
}

unsigned int SmartcardConfigurationManager::getNumberOfTriesToRead()
{
    return getSmartcardConfiguration().getReadRetry();
}

std::string SmartcardConfigurationManager::getConnectionPluginName()
{
    if ( !getSmartcardConfiguration().getConnectionPluginName().present() )
    {
        getSmartcardConfiguration().setConnectionPluginName ( "gppcscconnectionplugin" );
    }

    return getSmartcardConfiguration().getConnectionPluginName().get();
}

std::string SmartcardConfigurationManager::getConnectionPluginVersion()
{
    if ( !getSmartcardConfiguration().getConnectionPluginVersion().present() )
    {
        getSmartcardConfiguration().setConnectionPluginVersion ( "1.0.1" );
    }

    return getSmartcardConfiguration().getConnectionPluginVersion().get();
}

bool SmartcardConfigurationManager::isErrorCode ( unsigned long code )
{
    bool error = false;

    SmartcardConfiguration::FatalErrorCodesSequence & errorCodes ( getSmartcardConfiguration().getFatalErrorCodes() );
    SmartcardConfiguration::FatalErrorCodesIterator it;

    for ( it = errorCodes.begin();
            it != errorCodes.end() && !error;
            ++it )
    {
        unsigned long x;

        std::stringstream ss;
        ss << std::hex << ( *it );
        ss >> x;

        if ( code == x )
        {
            error = true;
        }
    }

    return error;
}

bool SmartcardConfigurationManager::simulationEnabled ()
{
    return getSmartcardConfiguration().getSmartcardSimulation().present();
}

std::string SmartcardConfigurationManager::getSimulatorEventFile ()
{
    return getSmartcardSimulationConfiguration().getEventFile();
}

std::string SmartcardConfigurationManager::getSimulatorContentFile ()
{
    return getSmartcardSimulationConfiguration().getContentFile();
}

unsigned int SmartcardConfigurationManager::getSimulationTimeAfterReadContentFile ()
{
    return getSmartcardSimulationConfiguration().getTimeAfterRead();
}

unsigned int SmartcardConfigurationManager::getSimulationTimeBeforeWriteContentFile ()
{
    return getSmartcardSimulationConfiguration().getTimeBeforeWrite();
}

SmartcardConfiguration & SmartcardConfigurationManager::getSmartcardConfiguration ()
{
    if ( !SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() )
    {
        SmartcardConfiguration config (
            "aid", // applet id
            "pin", // global pin
            PlatformMode::CONFIGURATION_GP_211, // platform mode
            false, // secure channel enabled
            0, // security key set version
            0, // security key index
            false, // force restart smartcard reader
            0, // delay after on
            false, // force restart PCSCD
            0, // wait stop PCSCD
            0, // wait start PCSCD
            0, // read retry
            false // call restart before initialize
        );

        SmartcardConfiguration::FatalErrorCodesSequence errorCodes;
        config.setFatalErrorCodes ( errorCodes );

        SAESCommonConfigurationManager::getInstance()->getConfiguration()->setSmartcardConfiguration ( config );
    }

    return SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get();
}

SmartcardSimulation & SmartcardConfigurationManager::getSmartcardSimulationConfiguration ()
{
    if ( !getSmartcardConfiguration().getSmartcardSimulation().present() )
    {
        SmartcardSimulation config ( "", "", 0, 0 );
        getSmartcardConfiguration().setSmartcardSimulation ( config );
    }

    return getSmartcardConfiguration ().getSmartcardSimulation().get();
}

SmartcardConfigurationManager * SmartcardConfigurationManager::getInstance ( SmartcardConfigurationManager * externManager )
{
    if ( ! instance )
    {
        instance = ( externManager == NULL ? new SmartcardConfigurationManager() : externManager );
    }

    return instance;
}
