/**
 * @file	LogBasePath.hxx
 * @brief	Declaration of the class that models the required data to establish the base directory.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef LOGBASEPATH_HXX_
#define LOGBASEPATH_HXX_

#include <Log/BasePathInterface.hxx>
#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <Runtime/Environment/saes-directories.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Log
        {
            /**
             * @class	LogBasePath
             * @brief	Class base that contains the configured directory from machine operational context.
             * @author	Youssef.tourkmani
             * @date	30/10/2012
             */
            class LogBasePath : public Smartmatic::Log::BasePathInterface
            {
                public:
                    /**
                     * @fn	LogBasePath::LogBasePath ( Smartmatic::SAES::Common::Configuration::BasePathType type );
                     * @brief	Default constructor initializes an instance of the LogBasePath class.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	type	The type.
                     */
                    explicit LogBasePath ( Smartmatic::SAES::Common::Configuration::BasePathType type );

                    /**
                     * @fn	std::string LogBasePath::getFileFullPath( std::string & file );
                     * @brief	Gets full path.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	file	The file name.
                     * @return	The full path of the merge between base directory and relative path of the file.
                     */
                    std::string getFileFullPath ( std::string & file );
                    /**
                     * @fn	bool LogBasePath::canWriter();
                     * @brief	  Gets whether exists the base directory and if has write permissions.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	 <tt>true</tt> if exists successful; otherwise <tt>false</tt>.
                     */
                    bool canWriter();

                private:
                    Smartmatic::SAES::Common::Configuration::BasePathType type; ///<  The Enumerator reference that represents the printer backend type.
                    Smartmatic::SAES::Runtime::Environment::SaesDirectories directories;	///< The context reference of the directory hierarchy.
            };
        }
    }
}

#endif /* LOGBASEPATH_HXX_ */
