/**
 * @file	LogCryptoBase64.hxx
 * @brief	Declaration of the class that contains the cryptographic methods for logs.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef LOGCRYPTOBASE64_HXX_
#define LOGCRYPTOBASE64_HXX_

#include <Log/ICryptoLayoutTransformer.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Log
        {
            /**
             * @class	LogCryptoBase64
             * @brief	Class that contains the encoding and decoding methods for logs.
             * @author	Youssef.tourkmani
             * @date	30/10/2012
             */
            class LogCryptoBase64 : public Smartmatic::Log::ICryptoLayoutTransformer
            {
                public:

                    /**
                     * @fn	LogCryptoBase64::LogCryptoBase64 ();
                     * @brief	Default constructor.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    LogCryptoBase64 ();

                    /**
                     * @fn	std::string LogCryptoBase64::encode(std::string & line);
                     * @brief	Gets the resulting string from codification.
                     * This method encode the string using INIT_KEY and gives a base64 result.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	line The original string  (preferably one-line).
                     * @return	The encode string in base64 format.
                     */
                    std::string encode ( std::string & line );

                    /**
                     * @fn	std::string LogCryptoBase64::decode(std::string & line);
                     * @brief	Gets the resulting string from decodification.
                     * This method decodes using INIT_KEY.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	line	The base64 string.
                     * @return	A readable string..
                     */
                    std::string decode ( std::string & line );
            };
        }
    }
}

#endif /* LOGCRYPTOBASE64_HXX_ */
