/**
 * @file	LogCryptoGlobal.hxx
 * @brief	Declaration of the mediator that exposes the cryptographic methods for logs.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef LOGCRYPTOGLOBAL_HXX_
#define LOGCRYPTOGLOBAL_HXX_

#include <Log/ICryptoLayoutTransformer.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Log
        {
            /**
             * @class	LogCryptoGlobal
             * @brief	Mediator that exposes the encoding and decoding methods for logs.
             * This class is implemented in terms of cryptoStringToB64.
             * @author	Youssef.tourkmani
             * @date	30/10/2012
             */
            class LogCryptoGlobal : public Smartmatic::Log::ICryptoLayoutTransformer
            {
                public:

                    /**
                     * @fn	LogCryptoGlobal::LogCryptoGlobal ();
                     * @brief	Default constructor initializes an instance of the LogCryptoGlobal class.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    LogCryptoGlobal ();

                    /**
                     * @fn	std::string LogCryptoGlobal::encode(std::string & line);
                     * @brief	Gets the resulting string from codification.
                     * This method encode the string using INIT_KEY and gives a base64 result.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	line The original string  (preferably one-line).
                     * @return	The encode string in base64 format.
                     */
                    std::string encode ( std::string & line );

                    /**
                     * @fn	std::string LogCryptoGlobal::decode(std::string & line);
                     * @brief	Gets the resulting string from decodification. This method decodes using INIT_KEY.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	line	The base64 string.
                     * @return	A readable string.
                     */
                    std::string decode ( std::string & line );
            };
        }
    }
}

#endif /* LOGCRYPTOGLOBAL_HXX_ */
