/**
 * @file	LogLine.h
 * @brief	Declaration of the entity that creates a log line that contains: date, time, params, level and others.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#ifndef _LOGLINE_H_
#define _LOGLINE_H_
#include <string>
#include <vector>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Log
        {
            /**
             * @class	LogLine
             * @brief	Entity that retrieves more detailed information about the logs.
             * It exposes that information through getters and setters.
             * @author	Rafael Ferrero
             * @date	30/10/2012
             */
            class LogLine
            {
                public:

                    /**
                     * @enum	LevelLog
                     * @brief	Values that represent LevelLog.
                     */
                    enum LevelLog
                    {
                        TRACE,
                        DEBUG,
                        INFO,
                        WARNING,
                        ERROR,
                        FATAL
                    };

                    /**
                     * @fn	LogLine::LogLine()
                     * @brief	Default constructor initializes an instance of the LogLine class.
                     * @author	Yadickson Soto
                     * @date	31/10/2012
                     */
                    LogLine();

                    /**
                     * @fn	std::string LogLine::getDate();
                     * @brief	Gets the date.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The date.
                     */

                    std::string getDate();

                    /**
                     * @fn	void LogLine::setDate(const std::string& date);
                     * @brief	Sets a date.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	date	The date.
                     */

                    void setDate ( const std::string& date );

                    /**
                     * @fn	std::string LogLine::getLibrary();
                     * @brief	Gets the library.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The library.
                     */

                    std::string getLibrary();

                    /**
                     * @fn	void LogLine::setLibrary(std::string library);
                     * @brief	Sets a library.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	library	The library.
                     */

                    void setLibrary ( const std::string& library );

                    /**
                     * @fn	std::string LogLine::getMessage();
                     * @brief	Gets the message.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The message.
                     */

                    std::string getMessage();

                    /**
                     * @fn	void LogLine::setMessage(std::string message);
                     * @brief	Sets a message.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	message	The message.
                     */

                    void setMessage ( const std::string& message );

                    /**
                     * @fn	std::vector<std::string> LogLine::getMessageParams();
                     * @brief	Gets the message parameters.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The message parameters.
                     */

                    std::vector<std::string> getMessageParams();

                    /**
                     * @fn	void LogLine::setMessageParams(std::vector<std::string> messageParams);
                     * @brief	Sets a message parameters.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	messageParams	Options for controlling the message.
                     */

                    void setMessageParams ( const std::vector<std::string>& messageParams );

                    /**
                     * @fn	LevelLog LogLine::getLevel();
                     * @brief	Gets the level.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The level.
                     */

                    LevelLog getLevel();

                    /**
                     * @fn	void LogLine::setLevel(LevelLog level);
                     * @brief	Sets a level.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	level	The level.
                     */

                    void setLevel ( const LevelLog& level );

                    /**
                     * @fn	std::string LogLine::getTimeSinceStart();
                     * @brief	Gets the time since start.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The time since start.
                     */

                    std::string getTimeSinceStart();

                    /**
                     * @fn	void LogLine::setTimeSinceStart(std::string time);
                     * @brief	Sets a time since start.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	time	The time.
                     */

                    void setTimeSinceStart ( const std::string& time );
                protected:

                private:
                    std::string date;   ///< The date
                    std::string library;	///< The library
                    std::string message;	///< The message
                    std::vector<std::string> messageParams; ///< Options for controlling the message
                    LevelLog level; ///< The level
                    std::string timeSinceStart; ///< The time since start
            };
        }
    }
}

#endif // _LOGLINE_H_
