/**
 * @file	CheckCODIResourceFiles.hxx
 * @brief	Declaration of the class that contains the verification of signature the resources.
 * @author	Yadickson Soto
 * @date	31/05/2013
 */

#ifndef _CHECK_CODI_RESOURCE_FILES_H_
#define _CHECK_CODI_RESOURCE_FILES_H_

#include "ICheckCondition.hxx"
#include <Configuration/SaesCommonConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Initiator
            {
                /**
                 * @class	CheckCODIResourceFiles
                 * @brief	Class validator to analyze the correctness of signatures the resources in the primary hierarchy.
                 * @author	Yadickson Soto
                 * @date	31/05/2013
                 */
                class CheckCODIResourceFiles: public ICheckCondition
                {
                    public:
                        /**
                         * @fn	CheckCODIResourceFiles::CheckCODIResourceFiles(
                         * 			Smartmatic::SAES::Common::Configuration::BasePathType type =
                        			Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Default constructor initializes an instance of the CheckCODIResourceFiles class.
                         * @author	Yadickson Soto
                         * @date	31/05/2013
                         * @param	type is the selection of the primary hierarchy which contains the file resources.
                         */
                        explicit CheckCODIResourceFiles ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                          Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	virtual CheckCODIResourceFiles::~CheckCODIResourceFiles();
                         * @brief	Finalizes the instance of the CheckCODIResourceFiles class.
                         * @author	Yadickson Soto
                         * @date	31/05/2013
                         */
                        virtual ~CheckCODIResourceFiles();

                        /**
                         * @fn	virtual TestStatus::Tests CheckCODIResourceFiles::check ();
                         * @brief	Checking of signature the resources contains in the hierarchy.
                         * @author	Yadickson Soto
                         * @date	31/05/2013
                         * @return <tt>TestStatus::Passed</tt> if it checks successful; otherwise <tt>TestStatus::Failed</tt>.
                         */
                        virtual TestStatus::Tests check ();

                    private:

                        Smartmatic::SAES::Common::Configuration::BasePathType type; /**< Value indicate the primary hierarchy which contains the file resources.*/
                        static Smartmatic::Log::ISMTTLog * logger; /**< The reference pointer of a logger instance.*/
                };
            }
        }
    }
}

#endif // _CHECK_CODI_RESOURCE_FILES_H_
