/**
 * @file	CheckDirectoryStructure.hxx
 * @brief	Declaration of the class that contains the verification of the directories structure.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef _CHECK_DIRECTORY_STRUCTURE_H_
#define _CHECK_DIRECTORY_STRUCTURE_H_

#include "ICheckCondition.hxx"
#include <Runtime/Environment/saes-directories.h>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Initiator
            {
                /**
                 * @class	CheckDirectoryStructure
                 * @brief	Class validator to analyze the directory structure of the <tt>SAES</tt> or <tt>ElectoralResource</tt>
                 * @author	Yadickson Soto
                 * @date	03-03-2011
                 */
                class CheckDirectoryStructure: public ICheckCondition
                {
                    public:
                        /**
                         * @fn	CheckDirectoryStructure::CheckDirectoryStructure(Smartmatic::SAES::Runtime::Environment::SaesDirectories *directories,
                         * 			Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Default constructor initializes an instance of the CheckDirectoryStructure class.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @param	directories The pointer reference to environment context of the directories hierarchy.
                         * @param	type (optional) The type is the selection of the primary hierarchy which contains the file resources.
                         */
                        CheckDirectoryStructure ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
                                                  Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	virtual CheckDirectoryStructure::~CheckDirectoryStructure();
                         * @brief	Finalizes the instance of the CheckDirectoryStructure class.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         */
                        virtual ~CheckDirectoryStructure();

                        /**
                         * @fn	virtual TestStatus::Tests CheckDirectoryStructure::check ();
                         * @brief	Checks if exists the directory <tt>SAES</tt> or <tt>ElectoralResource</tt>.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	<tt>TestStatus::Passed</tt> if exists one of the directories; otherwise <tt>TestStatus::Failed</tt>.
                         */
                        virtual TestStatus::Tests check ();

                    private:
                        Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories; ///< The pinter reference to environment context of the directory hierarchy.
                        Smartmatic::SAES::Common::Configuration::BasePathType type; ///< The base selection of the primary hierarchy.
                        static Smartmatic::Log::ISMTTLog * logger; ///< The reference pointer of a logger instance.
                };
            }
        }
    }
}

#endif // _CHECK_DIRECTORY_STRUCTURE_H_
