/**
 * @file   CheckEmlResourceFiles.cxx
 * @brief  Implementation of CheckEmlResourceFiles class.
 * @author manuel.arguelles
 * @date   Feb  7, 2018
 */

#include "CheckEmlResourceFiles.hxx"
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/saes-directories.h>
#include <Security/XmlSecResource.hxx>
#include <string>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common;
using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Security;

ISMTTLog* CheckEmlResourceFiles::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.Initiator.CheckEmlResourceFiles",
    GETTEXT_PACKAGE);

CheckEmlResourceFiles::CheckEmlResourceFiles(Configuration::BasePathType type)
    : ICheckCondition(N_("Smartmatic.SAES.Operation.Initiator.CheckEmlResourceFiles.CheckEmlResourceFiles"))
{
    m_type = type;
}

CheckEmlResourceFiles::~CheckEmlResourceFiles()
{
}

TestStatus::Tests CheckEmlResourceFiles::check()
{
    bool success = true;

    if (SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getVerifyEmlFolder())
    {
        SaesDirectories directories;
        std::string path = directories.getEmlFilesFolderFullPath(m_type);
        std::string file = directories.getSignatureEmlFileFullPath(m_type);

        m_logger->Debug("Check EML resource files in " + m_type);

        try
        {
            success = XmlSecResource::getInstance()->validateResource(
                path,
                file,
                Encryption::INSTALL_FILE,
                KeyType::CONFIGURATION_SECURITY_KEY);
        }
        catch (...)
        {
            success = false;
            m_logger->Audit(N_("Smartmatic.SAES.Operation.Initiator.CheckEmlResourceFiles.ErrorCheckSAESFolder"));
        }
    }

    return (success ? TestStatus::Passed : TestStatus::Failed);
}


