/**
 * @file	CheckFlashStructure.cxx
 * @brief	check flash structure class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#include "CheckFlashStructure.hxx"
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/IO/Directory.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::Common::Configuration;

ISMTTLog * CheckFlashStructure::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Initiator.CheckFlashStructure", GETTEXT_PACKAGE );

CheckFlashStructure::CheckFlashStructure ( Runtime::Environment::SaesDirectories & directories,
        Smartmatic::SAES::Common::Configuration::BasePathType type )
    : ICheckCondition ( N_ ( "Smartmatic.SAES.Operation.Initiator.CheckingFlashStructure.CheckingFlashStructure" ) )
{
    this->directories = directories;
    this->type = type;
}

CheckFlashStructure::~CheckFlashStructure()
{
}

TestStatus::Tests CheckFlashStructure::check()
{
    std::string basePath = directories.getDevicePath ( type );
    std::string bootPath = Smartmatic::System::IO::Path::Combine ( basePath, "/boot" );
    logger->Debug ( "validating boot in: " + bootPath );
    std::string votingCenters = directories.getVotingCentersInstalationFileFullPath ( type );
    logger->Debug ( "validating votingCenters in: " + votingCenters );

    bool success =
        Directory::getInstance()->DirectoryExists ( bootPath ) &&
        File::getInstance()->canOpenFile ( votingCenters );

    std::string line = "CheckFlashStructure result = ";
    line.append ( success ? "Success" : "Failed" );
    line.append ( " in " + type );
    logger->Debug ( line );

    if ( !success )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.CheckFlashStructureFailed" ) );
    }

    return ( success ? TestStatus::Passed : TestStatus::Failed );
}
