/**
 * @file	CheckSAESResourceFiles.cxx
 * @brief	Implementation of the class that contains the verify authenticity  for <tt>SAES</tt> directory.
 * @author	Yadickson Soto
 * @date	31/10/2012
 */

#include "CheckSAESResourceFiles.hxx"
#include <Log/SMTTLogManager.h>
#include <Security/XmlSecResource.hxx>
#include <Configuration/s-a-e-s-common-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Common::Configuration;

ISMTTLog * CheckSAESResourceFiles::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Initiator.CheckSAESResourceFiles", GETTEXT_PACKAGE );

CheckSAESResourceFiles::CheckSAESResourceFiles ( Smartmatic::SAES::Common::Configuration::BasePathType type )
    : ICheckCondition ( N_ ( "Smartmatic.SAES.Operation.Initiator.CheckResourceFiles.CheckResourceFiles" ) )
{
    this->type = type;
}

CheckSAESResourceFiles::~CheckSAESResourceFiles()
{
}

TestStatus::Tests CheckSAESResourceFiles::check()
{
    bool success = true;

    if ( SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getVerifySAESFolder() )
    {

        logger->Debug ( "Check SAES resource files in " + type );

        try
        {
            Smartmatic::SAES::Runtime::Environment::SaesDirectories directories;

            std::string path = directories.getBaseInstallationPath ( type );
            std::string file = directories.getSignatureRootFileFullPath ( type );

            success = XmlSecResource::getInstance()->validateResource ( path, file,
                      Smartmatic::SAES::Security::Encryption::INSTALL_FILE,
                      Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECURITY_KEY );
        }
        catch ( ... )
        {
            success = false;
        }

        std::string line = "CheckSAESResourceFiles result = ";
        logger->Debug ( line.append ( success ? "Success" : "Failed" ) );

        if ( !success )
        {
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.CheckSAESResourceFiles.ErrorCheckSAESFolder" ) );
        }
    }

    return ( success ? TestStatus::Passed : TestStatus::Failed );
}
