/**
 * @file DeviceTypeMonitor.cxx
 * @brief Implementation of the common interface that abstracts any type of monitored device.
 * @date 12/12/2012
 * @author Yadickson Soto
 */

#include "DeviceTypeMonitor.hxx"

using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;

DeviceTypeMonitor::DeviceTypeMonitor ( DeviceType type )
    : type ( type )
{

}

DeviceTypeMonitor::~DeviceTypeMonitor()
{
    nodeList.clear();
}

DeviceType DeviceTypeMonitor::getType()
{
    return type;
}

unsigned int DeviceTypeMonitor::getNumberOfDevices()
{
    return nodeList.size();
}

bool DeviceTypeMonitor::addNode ( const std::string & node )
{
    bool notFound = nodeList.find ( node ) == nodeList.end();

    if ( notFound )
    {
        nodeList.insert ( node );
    }

    return notFound;
}

bool DeviceTypeMonitor::removeNode ( const std::string & node )
{
    bool found = nodeList.find ( node ) != nodeList.end();

    if ( found )
    {
        nodeList.erase ( node );
    }

    return found;
}
