/**
 * @file MonitorInfo.hxx
 * @brief Header for the MonitorInfo class to save action and properties.
 * @date 11/12/2012
 * @author Yadickson Soto
 */

#ifndef MONITOR_INFO_HXX_
#define MONITOR_INFO_HXX_

#include <string>
#include <map>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Common
        {
            namespace Operation
            {
                namespace Monitor
                {
                    /**
                     * @class	MonitorInfo
                     * @brief   MonitorInfo class to save action and properties.
                     * @author	Yadickson Soto
                     * @date	21/05/2015
                     */
                    class MonitorInfo
                    {
                        public:

                            /**
                             * @brief Class constructor
                             * @date 21/05/2015
                             * @author Yadickson Soto
                             * @param action flag to identify if action is add (true) or remove (false)
                             */
                            explicit MonitorInfo ( bool action );

                            /**
                             * @brief Class destructor
                             * @date 21/05/2015
                             * @author Yadickson Soto
                             */
                            virtual ~MonitorInfo();

                            /**
                             * @fn void push(const std::string & name, const std::string & value);
                             * @brief push the new par attribute and value.
                             * @date 21/05/2015
                             * @author Yadickson Soto
                             * @param name Attribute name
                             * @param value Attribute value
                             */
                            void push ( const std::string & name, const std::string & value );

                            /**
                             * @fn const std::string getValue ( const std::string & name ) const;
                             * @brief Gets the property value by name.
                             * @date 21/05/2015
                             * @author Yadickson Soto
                             * @return the value by name
                             */
                            const std::string getValue ( const std::string & name ) const;

                            /**
                             * @fn bool isAdded();
                             * @brief Gets when action is true or added.
                             * @date 21/05/2015
                             * @author Yadickson Soto
                             * @return true if action ( added or true )
                             */
                            bool isAdded();

                            /**
                             * @fn virtual void viewInfo();
                             * @brief Method to view all par attribute and value
                             * @date 21/05/2015
                             * @author Yadickson Soto
                             */
                            void viewInfo();

                        private:

                            bool action; ///< flag to identify if action is add (true) or remove (false)
                            std::map < std::string, std::string > infoMap;  ///< the list of attributes and values
                            static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                    };
                }
            }
        }
    }
}

#endif /* MONITOR_INFO_HXX_ */
