/**
 * @file	IBasePrintDocument.hxx
 * @brief	Declaration of the common interface that abstracts to the print entity.
 * @author	Yadickson Soto
 * @date	30/11/2012
 */

#ifndef IBASEPRINTDOCUMENT_H_
#define IBASEPRINTDOCUMENT_H_

#include "Printing/PrintDocumentClient.h"
#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <System/SafeBuffer.h>
#include <System/SafeList.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Printing
        {
            /**
             * @class	IBasePrintDocument
             * @brief	Interface that abstracts to the print entity.
             * @author	Cesar.bonilla
             * @date	4/28/2011
             */
            class IBasePrintDocument
            {
                public:

                    /**
                     * @fn	IBasePrintDocument();
                     * @brief	Default constructor initializes an instance of the IBasePrintDocument class.
                     * @author	Yadickson Soto
                     * @date	05/12/2012
                     */
                    IBasePrintDocument();

                    /**
                     * @fn	virtual IBasePrintDocument::~IBasePrintDocument();
                     * @brief	Finalizes the instance of the IBasePrintDocument class.
                     * @author	Yadickson Soto
                     * @date	30/11/2012
                     */
                    virtual ~IBasePrintDocument();

                    /**
                     * @fn	virtual void Print() = 0;
                     * @brief	Prints the document using the assigned backend.
                     * @author	Yadickson Soto
                     * @date	30/11/2012
                     */
                    virtual void Print() = 0;

                    /**
                     * @fn	virtual void setClient ( PrintDocumentClient * client ) = 0;
                     * @brief	Sets the reference to the client used to retrieve the printer result.
                     * @author	Yadickson Soto
                     * @date	30/11/2012
                     * @param	client	The pointer reference to client instance.
                     */
                    virtual void setClient ( PrintDocumentClient * client ) = 0;

                    /**
                     * @fn	virtual Smartmatic::System::SafeBuffer getBuffer ();
                     * @brief	Get buffer to print.
                     * @author	Yadickson Soto
                     * @date	06/08/2015
                     * @result	The buffer to print
                     */
                    virtual Smartmatic::System::SafeBuffer getBuffer ();

                    /**
                     * @fn	Smartmatic::SAES::Common::Configuration::PrinterBackendType IBasePrintDocument::getBackendType();
                     * @brief	Gets backend type.
                     * @author	Yadickson Soto
                     * @date	05/12/2012
                     * @return	The backend type.
                     */
                    Smartmatic::SAES::Common::Configuration::PrinterBackendType getBackendType();

                private:

                    Smartmatic::SAES::Common::Configuration::PrinterBackendType backendType;	///< Type of the backend
            };
        }
    }
}

#endif /* IBASEPRINTDOCUMENT_H_ */
