/**
 * @file	filename-hasher.cc
 * @brief	filename hasher class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#include <algorithm>
#include "Runtime/Environment/filename-hasher.h"

using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::Security::Cryptography;

FileNameHasher::FileNameHasher()
{
}

FileNameHasher::~FileNameHasher()
{
}

std::string FileNameHasher::GetFileNameHasher ( std::string name )
{
    SafeBuffer buffer ( name );
    SafeBuffer result;
    MsgDigestUtil digest ( buffer );
    digest.getDigest ( result, MsgDigestUtil::getDefaultFileNameDigest() );
    return FileNameHasher::GetValidBase64 ( result );
}

std::string FileNameHasher::GetFileNameHasher ( unsigned char * name, int nameLeng )
{
    SafeBuffer buffer ( name, nameLeng );
    SafeBuffer result;
    MsgDigestUtil digest ( buffer );
    digest.getDigest ( result, MsgDigestUtil::getDefaultFileNameDigest() );
    return FileNameHasher::GetValidBase64 ( result );
}


std::string FileNameHasher::GetValidBase64 ( SafeBuffer & buffer_inner )
{
    std::string invalidName = buffer_inner.GetBase64Data();
    std::string validName = FileNameHasher::ReplaceInvalidCharacters ( invalidName );
    return validName;
}

std::string FileNameHasher::ReplaceInvalidCharacters ( std::string name )
{
    char invalidCharacters[12] = {'/', '\\', ':', '*', '\"', '\'', '<', '>', '\?', '|', ',', '.'};

    for ( int i = 0; i < ( int ) sizeof ( invalidCharacters ); i++ )
    {
        char newVal = ( char ) ( ( int ) 'A' + ( i / 2 ) );
        replace ( name.begin(), name.end(), invalidCharacters[i], newVal );
    }

    return name;
}

