/**
 * @file	saes-directories.h
 * @brief	Declares the saes directories class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef _SAES_DIRECTORIES_H_
#define _SAES_DIRECTORIES_H_

#include <string>
#include <Configuration/SaesCommonConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Runtime
        {
            namespace Environment
            {
                /**
                 * @class	SaesDirectories
                 * @brief	Saes directories.
                 * @author	Cesar.bonilla
                 * @date	4/28/2011
                 */

                class SaesDirectories
                {
                    public:

                        /**
                         * @fn	virtual std::string SaesDirectories::getDevicePath ( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief	Gets a base path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The base path.
                         */

                        virtual std::string getDevicePath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getBasePath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter base path.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @param	type	base path type.
                         * @return	base path.
                         */
                        std::string
                        getBasePath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                          Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getLocalBasePath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter local base path.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @param	type	base path type.
                         * @return	local base path.
                         */
                        std::string
                        getLocalBasePath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                               Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	virtual std::string SaesDirectories::getElectionFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets an election file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The election file full path.
                         */

                        virtual std::string
                        getElectionFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                      Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	virtual std::string SaesDirectories::getDeviceParametersFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the device parameters file full path.
                         * @author	angel.ramirez
                         * @date	05/07/2022
                         * @param	type	The type.
                         * @return	The device parameters file full path.
                         */
                        virtual std::string
                        getDeviceParametersFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                            Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	virtual std::string SaesDirectories::getElectionReduceFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets an election reduce file full path for VM.
                         * @author	Yadickson Soto
                         * @date	24/05/2013
                         * @param	type	The type.
                         * @return	The election reduce file full path.
                         */

                        virtual std::string
                        getElectionReduceFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                            Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getElectionStatusFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets an election status file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The election status file full path.
                         */

                        std::string
                        getElectionStatusFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                            Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getVotingDeviceFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a voting device file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The voting device file full path.
                         */

                        std::string
                        getVotingDeviceFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                          Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getElectoralResourceFullFolderPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets an electoral resource full folder path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The electoral resource full folder path.
                         */

                        std::string
                        getElectoralResourceFullFolderPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	virtual std::string SaesDirectories::getElectoralImagesFullFolderPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets an electoral images full folder path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The electoral images full folder path.
                         */

                        virtual std::string getElectoralImagesFullFolderPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getLocalElectoralImagesFullFolderPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets local electoral images full folder path.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	type	(optional) the type.
                         * @return	The local electoral images full folder path.
                         */
                        std::string getLocalElectoralImagesFullFolderPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getElectoralSAESFullFolderPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets an electoral saes full folder path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The electoral saes full folder path.
                         */

                        std::string
                        getElectoralSAESFullFolderPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                             Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getTemporaryFilesFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a temporary files folder full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The temporary files folder full path.
                         */

                        std::string
                        getTemporaryFilesFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                              Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getTemporaryFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a temporary file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The temporary file full path.
                         */

                        std::string
                        getTemporaryFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                       Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getLanguageFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a language file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The language file full path.
                         */

                        std::string
                        getLanguageFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                      Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getActivatedCardStatisticsFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets an activated card statistics file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The activated card statistics file full path.
                         */

                        std::string
                        getActivatedCardStatisticsFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );


                        /**
                         * @fn	std::string SaesDirectories::getXMLImageFileName();
                         * @brief	Gets a xml image file full path.
                         * @author	Juan.Delgado
                         * @date	18/07/2011
                         * @return	The xml image file name.
                         */

                        std::string getXMLImageFileName();

                        /**
                         * @fn	std::string SaesDirectories::getBaseInstallationPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a base installation path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The base installation path.
                         */

                        std::string getBaseInstallationPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getSaesDataFolderName();
                         * @brief	Gets the saes data folder name.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The saes data folder name.
                         */

                        std::string getSaesDataFolderName();

                        /**
                         * @fn	std::string SaesDirectories::getSaesDataFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the saes data folder full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The saes data folder full path.
                         */

                        std::string
                        getSaesDataFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                        Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getLocalSaesDataFolderFullPath( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter saes local data folder full path.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @param	type	base path type.
                         * @return	saes local data folder full path.
                         */
                        std::string getLocalSaesDataFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getElectionResourcesFileName();
                         * @brief	Gets the election resources file name.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The election resources file name.
                         */

                        std::string getElectionResourcesFileName();

                        /**
                         * @fn	virtual std::string SaesDirectories::getVotingCentersLocalFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a voting centers local file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The voting centers local file full path.
                         */

                        virtual std::string getVotingCentersLocalFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	virtual std::string SaesDirectories::getVotingCentersInstalationFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a voting centers instalation file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The voting centers instalation file full path.
                         */

                        virtual std::string getVotingCentersInstalationFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getLanguageInstalationFileFullPath();
                         * @brief	Gets the language instalation file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The language instalation file full path.
                         */

                        std::string getLanguageInstalationFileFullPath();

                        /**
                         * @fn	std::string SaesDirectories::getLanguageLocalFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a language local file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The language local file full path.
                         */

                        std::string
                        getLanguageLocalFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                           Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getLanguageImageInstalationDirectoryFullPath();
                         * @brief	Gets the language image instalation directory full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The language image instalation directory full path.
                         */

                        std::string getLanguageImageInstalationDirectoryFullPath();

                        /**
                         * @fn	std::string SaesDirectories::getLanguageImageDirectoryFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a language image directory full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The language image directory full path.
                         */

                        std::string
                        getLanguageImageDirectoryFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getImageElectoralResource(std::string imageName);
                         * @brief	Gets an image electoral resource.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	imageName	Name of the image.
                         * @return	The image electoral resource.
                         */

                        std::string
                        getImageElectoralResource ( std::string imageName );

                        /**
                         * @fn	virtual std::string SaesDirectories::getWorkingDirectoryFolderName();
                         * @brief	Gets the working directory folder name, usually SAESLocal.
                         * @author	Freddy.Gomez
                         * @date	4/28/2011
                         * @return	The master configuration base path.
                         */
                        virtual std::string getWorkingDirectoryFolderName();

                        /**
                         * @fn	virtual std::string SaesDirectories::getDOMMemoryBasePath();
                         * @brief	Gets the DOM memory base path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The master configuration base path.
                         */

                        virtual std::string getDOMMemoryBasePath();

                        /**
                         * @fn	virtual std::string SaesDirectories::getFlashMemoryBasePath();
                         * @brief	Gets the flash memory base path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The backup configuration base path.
                         */

                        virtual std::string getFlashMemoryBasePath();


                        /**
                         * @fn	std::string SaesDirectories::getBackupFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets backup file full path.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	type	(optional) the type.
                         * @return	The backup file full path.
                         */
                        std::string getBackupFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );


                        /**
                        * @fn    std::string SaesDirectories::getSecretKeyFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                        * @brief    Gets secret key file full path.
                        * @author    Youssef.tourkmani
                        * @date    31/10/2012
                        * @param    type    (optional) the type.
                        * @return    The secret key file full path.
                        */
                        std::string getSecretKeyFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	virtual std::string SaesDirectories::getUPKeyFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a up key file full path.
                         * @author	Yadickson Soto
                         * @date	13/02/2012
                         * @param	type	The type.
                         * @return	The up key file full path.
                         */

                        virtual std::string getUPKeyFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getMachineKeysFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a machine keys file full path.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	5/24/2011
                         * @param	type	The type.
                         * @return	The machine keys file full path.
                         */

                        std::string getMachineKeysFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getX7SSignedKeyCertifiedFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	method getter x7s signed certified primary key file full path.
                         * @author	Yadickson Soto
                         * @date	21/03/2012
                         * @param	type	type.
                         * @return	signed certified primary key file full path.
                         */
                        std::string getX7SSignedKeyCertifiedFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getX7SSignedCertifiedFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	method getter x7s signed certified file full path.
                         * @author	Yadickson Soto
                         * @date	21/03/2012
                         * @param	type	type.
                         * @return	signed certified file full path.
                         */
                        std::string getX7SSignedCertifiedFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn  std::string SaesDirectories::getX7SCryptoKeystoreInstallationFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief   method getter x7s crypto keystore installation file full path.
                         * @author  Eduardo Fernández
                         * @date    12/12/2018
                         * @param   type    type.
                         * @return  crypto certified file full path.
                         */
                        std::string getX7SCryptoKeystoreInstallationFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn  std::string SaesDirectories::getX7SCryptoKeystoreFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief   method getter x7s crypto keystore file full path.
                         * @author  Eduardo Fernández
                         * @date    18/10/2018
                         * @param   type    type.
                         * @return  crypto certified file full path.
                         */
                        std::string getX7SCryptoKeystoreFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getIdHardwareFileFullPath();
                         * @brief	Gets the file path identifier hardware.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The file path identifier hardware.
                         */

                        std::string getIdHardwareFileFullPath();

                        /**
                         * @fn	std::string SaesDirectories::getVotingExperienceValidationFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a voting experience validation folder full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The voting experience validation folder full path.
                         */

                        std::string getVotingExperienceValidationFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getVotesFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the votes folder full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The votes folder full path.
                         */

                        std::string getVotesFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getDocsFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the documents folder full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The documents folder full path.
                         */

                        virtual std::string getDocsFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief  Gets the EML folder path.
                         * @author manuel.arguelles
                         * @date   Jan  8, 2018
                         * @return The EML folder path.
                         */
                        std::string getEmlFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                         Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);

                        /**
                         * @brief  Gets the EML temp folder path.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2019
                         * @return The EML temp folder path.
                         */
                        std::string getEmlTempFolderFullPath();

                        /**
                         * @fn	std::string SaesDirectories::getTransmissionPackagesFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a transmission packages file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The transmission packages file full path.
                         */
                        std::string getTransmissionPackagesFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getTransmissionPackagesKeyFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter a transmission packages key file full path.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @param	type	base path type.
                         * @return	transmission packages key file full path.
                         */
                        std::string getTransmissionPackagesKeyFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getTransmissionPackagesCrtFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter a transmission packages crt file full path.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @param	type	base path type.
                         * @return	transmission packages crt file full path.
                         */
                        std::string getTransmissionPackagesCrtFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getTransmissionPackagesHashFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter a transmission packages hash file full path.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @param	type	base path type.
                         * @return	transmission packages hash file full path.
                         */
                        std::string getTransmissionPackagesHashFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getPVFormFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets a pv form file full path.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	05/05/2011
                         * @param	type	The type.
                         * @return	The pv form file full path.
                         */
                        std::string getPVFormFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn  std::string getIssuerCertificatesFolderFullPath();
                         * @brief   Gets the issuer certificates folder full path.
                         * @author  eduardo.fernandez
                         * @date    29/11/2018
                         * @return  The issuer certificates folder full path.
                         */
                        std::string getIssuerCertificatesFolderFullPath();

                        /**
                         * @fn	std::string SaesDirectories::getCACertificateFileFullPath();
                         * @brief	Gets the root ca certificate file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The root ca certificate file full path.
                         */

                        std::string getRootCACertificateFileFullPath();

                        /**
                         * @fn	std::string SaesDirectories::getItermediateCACertificatesFileFullPath();
                         * @brief	Gets the intermediate ca certificate file full path.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @param	type	The type.
                         * @return	The intermediate ca certificate file full path.
                         */

                        std::string getIntermediateCACertificateFileFullPath();

                        /**
                         * @fn	std::string SaesDirectories::getPVTemplateFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the full path of the PV template folder.
                         * @author	Freddy.Gomez
                         * @date	06/05/2011
                         * @param	type	(optional) the type.
                         * @return	The full path of the PV template folder.
                         */
                        std::string getPVTemplateFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );


                        /**
                         * @fn	std::string SaesDirectories::getTestPVTemplateFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the full path of the test PV template folder.
                         * @author	Freddy.Gomez
                         * @date	06/05/2011
                         * @param	type	(optional) the type.
                         * @return	The full path of the test PV template folder.
                         */
                        std::string getTestPVTemplateFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );


                        /**
                         * @fn	std::string SaesDirectories::getGlobalPVTemplateFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the full path of the global (non localized) PV template file.
                         * @author	Freddy.Gomez
                         * @date	06/05/2011
                         * @param	type	(optional) the type.
                         * @return	The full path of the global (non localized) PV template file.
                         */

                        std::string getGlobalPVTemplateFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getGeneratedPVReportFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the full path of the pv report file.
                         * @author	Freddy.Gomez
                         * @date	06/05/2011
                         * @param	type	(optional) the type.
                         * @return	The full path of the global pv report file.
                         */

                        std::string getGeneratedPVReportFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );


                        /**
                         * @fn	std::string SaesDirectories::getGeneratedPVReportTestFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the full path of the test pv report file.
                         * @author	Freddy.Gomez
                         * @date	06/05/2011
                         * @param	type	(optional) the type.
                         * @return	The full path of the global test pv report file.
                         */
                        std::string getGeneratedPVReportTestFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getLogFolderName();
                         * @brief	Gets the log folder name.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	20/05/2011
                         * @return	The log folder name.
                         */

                        std::string getLogFolderName();

                        /**
                         * @fn	std::string SaesDirectories::getLogFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the log folder full path.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	20/05/2011
                         * @param	type	the type.
                         * @return	The log folder full path.
                         */
                        std::string getLogFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                               Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                        /**
                         * @fn	std::string SaesDirectories::getLogErrorFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the log error file full path.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	20/05/2011
                         * @param	type	the type.
                         * @return	The log error file full path.
                         */
                        std::string getLogErrorFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                        /**
                         * @fn	std::string SaesDirectories::getLogAuditFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the log audit file full path.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	20/05/2011
                         * @param	type	the type.
                         * @return	The log audit file full path.
                         */
                        std::string getLogAuditFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getElectoralResourcesFolderName();
                         * @brief	Gets the electoral resources folder name.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	19/05/2011
                         * @return	The electoral resources folder name.
                         */

                        std::string getElectoralResourcesFolderName();

                        /**
                         * @fn	std::string SaesDirectories::getVoteStatisticsFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the vote statistics file full path.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	20/05/2011
                         * @param	type	the type.
                         * @return	The vote statistics file full path.
                         */
                        std::string getVoteStatisticsFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	SaesDirectories::SaesDirectories();
                         * @brief	Default constructor.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         */
                        SaesDirectories();

                        /**
                         * @fn	virtual SaesDirectories::~SaesDirectories();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         */
                        virtual ~SaesDirectories();

                        /**
                         * @fn	std::string SaesDirectories::getSignatureResourceFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter signature resource file full path.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @param	type	base path type.
                         * @return	signature resource file full path.
                         */
                        std::string getSignatureResourceFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getSignatureRootFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter signature root resource file full path.
                         * @author	Yadickson Soto
                         * @date	02/11/2012
                         * @param	type	base path type.
                         * @return	signature root resource file full path.
                         */
                        std::string getSignatureRootFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief	Method getter signature EML resource file full path.
                         * @author	Yadickson Soto
                         * @date	31/05/2013
                         * @param	type	base path type.
                         * @return	signature EML resource file full path.
                         */
                        std::string getSignatureEmlFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief	Method getter signature SAESLocal resource file full path.
                         * @author	Yadickson Soto
                         * @date	31/05/2013
                         * @param	type	base path type.
                         * @return	signature SAESLocal resource file full path.
                         */
                        std::string getSignatureSaesLocalFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief  Gets the EML files folder full path.
                         * @author manuel.arguelles
                         * @date   Jan  8, 2018
                         */
                        std::string getEmlFilesFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getDiagnosticResultFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter diagnostic result folder folder full path.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @param	type	base path type.
                         * @return	Diagnostic result files folder full path.
                         */
                        std::string getDiagnosticResultFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn	std::string SaesDirectories::getMachineDateFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Gets the machine's date file full path.
                         * @author	Daniel Ampuero
                         * @date	24/09/2012
                         * @param	type	base path type.
                         * @return	A string with the full path to machine's date file.
                         */
                        std::string getMachineDateFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                        /**
                         * @fn	std::string SaesDirectories::getZipCodeFileFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                         * @brief	Method getter zip code file full path.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         * @param	type	base path type.
                         * @return	zip code file full path.
                         */
                        std::string getZipCodeFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                        /**
                         * @brief Method getter statistic config file full path
                         * @param type base path type
                         * @return statistic config file full path
                         *
                         * @author  Yadickson Soto
                         * @date    30/10/2012
                         */
                        std::string getStatisticConfigFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief Method getter wizard config file full path
                         * @param type base path type
                         * @return wizard config file full path
                         *
                         * @author  Yadickson Soto
                         * @date    11/01/2013
                         */
                        std::string getWizardConfigFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief Method getter menu config file full path
                         * @param type base path type
                         * @return menu config file full path
                         *
                         * @author  Yadickson Soto
                         * @date    11/01/2013
                         */
                        std::string getMenuConfigFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief Method getter voting experience config file full path
                         * @param type base path type
                         * @return voting experience config file full path
                         *
                         * @author  Yadickson Soto
                         * @date    11/01/2013
                         */
                        std::string getVotingExperienceConfigFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief Method getter process status results file full path
                         * @param type base path type
                         * @return process status results file full path
                         *
                         * @author  Yadickson Soto
                         * @date    12/03/2013
                         */
                        std::string getProcessStatusResultsFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn std::string getLoginKeyFileName ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief Getter Method : login key file full path
                         * @param type base path type
                         * @return login key file full path
                         *
                         * @author  Yadickson Soto
                         * @date    20/05/2013
                         */
                        std::string getLoginKeyFileName ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                              Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn std::string getRecountSecretKeyFileName ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief Getter Method : recount secret key file full path
                         * @param type base path type
                         * @return recount secret key file full path
                         *
                         * @author  Yadickson Soto
                         * @date    20/05/2013
                         */
                        std::string getRecountSecretKeyFileName ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn std::string getRecountUPKeyFileName ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief Getter Method : recount up key file full path
                         * @param type base path type
                         * @return up secret key file full path
                         *
                         * @author  Yadickson Soto
                         * @date    20/05/2013
                         */
                        std::string getRecountUPKeyFileName ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn std::string getVotingResultsFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief Getter Method : voting results file full path
                         * @param type base path type
                         * @return voting results file full path
                         *
                         * @author  Yadickson Soto
                         * @date    27/08/2013
                         */
                        std::string getVotingResultsFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                        /**
                         * @fn std::string getCheckListDiagnoseFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief Getter Method : check list diagnose file full path
                         * @param type base path type
                         * @return check list diagnose file full path
                         *
                         * @author  Yadickson Soto
                         * @date    17/09/2013
                         */
                        std::string getCheckListDiagnoseFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                        /**
                         * @fn std::string getSkipCalibrationToolFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief Getter Method : skip calibration tool diagnose file full path
                         * @param type base path type
                         * @return skip calibration tool diagnose file full path
                         *
                         * @author  Yadickson Soto
                         * @date    11/10/2013
                         */
                        std::string getSkipCalibrationToolFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn virtual std::string getVotingResultsFilePlainFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief Getter Method : voting results file plain full path
                         * @param type base path type
                         * @return voting results file plain full path
                         *
                         * @author  Yadickson Soto
                         * @date    22/11/2013
                         */
                        virtual std::string getVotingResultsFilePlainFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @fn virtual std::string getNationalityDefinitionTypeFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
                         * @brief Getter Method : nationality code file full path
                         * @param type base path type
                         * @return nationality code file full path
                         *
                         * @author  Yadickson Soto
                         * @date    22/11/2013
                         */
                        virtual std::string getNationalityDefinitionTypeFileFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType type =
                                    Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                        /**
                         * @brief  Gets the full path of the voices for the text to speech engine.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         * @return The absolute path of the voices.
                         */
                        std::string getTextToSpeechVoicesFolderFullPath(Smartmatic::SAES::Common::Configuration::BasePathType type =
                                                                        Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_LIVE);

                        /**
                         * @brief  Gets the election's audio resources file name.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         * @return The name of the file with the audio resources.
                         */
                        std::string getElectionAudioResourcesFileName();

                        /**
                         * @brief  Gets the general audio resources file name.
                         * @author manuel.arguelles
                         * @date   Dec 18, 2017
                         */
                        std::string getGeneralAudioResourcesFileName();

                    protected:

                        std::string languageImageDirectory; ///< Pathname of the language image directory
                        std::string backupFileName; ///< Filename of the backup file

                        /**
                         * @fn	std::string SaesDirectories::getProjectName();
                         * @brief	Method getter project name.
                         * @author	Yadickson Soto
                         * @date	14/02/2012
                         * @return	project name.
                         */
                        std::string getProjectName();

                    private:

                        //Folders name
                        std::string LiveFileSystem; ///< Live file system
                        std::string ActivatedCardStatisticsFileName;	///< Filename of the activated card statistics file
                        std::string XMLImageFileName;   ///< Filename of the closing notes file

                        std::string votingCentersFileName;  ///< Filename of the voting centers file
                        std::string secretKeyFileName;  ///< Filename of the secret key file
                        std::string machineKeysFileName;	///< Filename of the machine keys file

                        std::string IssuerCertificatesFolderName;  ///< Folder name of the issuer certificates.
                        std::string RootCACertificateFileName;  ///< Filename of the root ca certificate file
                        std::string IntermediateCACertificateFileName;  ///< Filename of the intermediate ca certificate file

                        std::string ElectoralResourcesFolderName;	///< Name of the electoral resources folder
                        std::string ElectionResourcesFileName;  ///< Filename of the election resources file
                        std::string TemporaryFilesFolderName;	///< Pathname of the temporary files folder

                        std::string projectName;	///< Name of the project
                        std::string languageFileName;   ///< Filename of the language file

                        //Files Name

                        std::string ElectionStatusFileName; ///< Filename of the election status file
                        std::string VotingDeviceFileName;   ///< Filename of the voting device file
                        std::string PVFormFileName;   ///< Filename of the pv form
                        std::string VoteStatisticsFileName; ///< Vote statistic file name
                        std::string SignatureResourceFileName; ///< Signature resource file name
                        std::string SignatureRootFileName; ///< Signature root file name
                        std::string SignatureEmlFileName; ///< Signature eml file name
                        std::string SignatureSaesLocalFileName; ///< Signature SAESLocal file name

                        //File IdHardware
                        std::string IdHardwareFileFullPath; ///< The file path identifier hardware

                        //Voting Experience Simulation
                        std::string votingExperienceValidationFolderName;   ///< Pathname of the voting experience validation folder
                        std::string VotesFolder;	///< Pathname of the votes folder
                        std::string PVTemplateFolder; ///< Pathname of the PVTemplate folder
                        std::string TransmissionPackageFileName;	///< Filename of the transmission package file
                        std::string TransmissionPackageHashFileName;    ///< Filename of the transmission package hash file
                        std::string TransmissionPackageKeyFileName; ///< Filename of the transmission package key file
                        std::string TransmissionPackageCrtFileName; ///< Filename of the transmission package crt file
                        std::string PVTemplateFileName;	///< Filename of the nonlocalized PV template
                        std::string GeneratedPVReportFileName;	///< Filename of the nonlocalized PV template

                        //Log configuration
                        std::string logFolderName; ///< log folder name
                        std::string logErrorFileName; ///< log error file name
                        std::string logAuditFileName; ///< log audit file name

                        //X7S certified files
                        std::string X7SSignedKeyCertifiedFileName;	///< Filename of the 7 s signed key certified file
                        std::string X7SSignedCertifiedFileName; ///< Filename of the 7 s signed certified file
                        std::string X7SCryptoKeystoreFileName; ///< Filename of the 7 s crypto keystore file

                        std::string DiagnosticResultFolder; ///< Pathname of the diagnostic result folder

                        std::string DateDiagnosticFile; ///< The date diagnostic file
                        std::string ZipCodeFileName; ///< zip code file name
                        std::string StatisticConfigFileName; ///< statistic config file name
                        std::string WizardConfigFileName; ///< wizard config file name
                        std::string MenuConfigFileName; ///< menu config file name
                        std::string VotingExperienceConfigFileName; ///< voting experience config file name
                        std::string ProcessStatusResultsFileName; ///< Process status results file name

                        std::string LoginKeyFileName; ///< login key file name
                        std::string RecountSecretKeyFileName; ///< recount secret key file name
                        std::string RecountUPKeyFileName; ///< recount up key file name

                        std::string SecurityKeyFileName; ///< security key file name
                        std::string SecurityRecountKeyFileName; ///< security recount key file name

                        std::string ElectionReduceFileName; ///< election reduce file name
                        std::string VotingResultsFileName; ///< voting results file name
                        std::string CheckListDiagnoseFileName; ///< check list diagnose file name
                        std::string SkipCalibrationToolFileName; ///< skip calibration tool diagnose file name
                        std::string VotingResultsFilePlainName; ///< voting results file plain name
                        std::string NationalityDefinitionTypeFileName; ///< nationality code file name

                        std::string TextToSpeechVoicesFolder; //< Text to speech voices folder
                        std::string ElectionAudioResourcesFileName; ///< Filename of the election audio resources file
                        std::string GeneralAudioResourcesFileName; ///< Filename of the general audio resources file

                    protected:

                        std::string SaesDataFolderName; ///< Pathname of the saes data folder
                        std::string workingDirectory; ///< Pathname of the working directory
                        std::string ElectoralImageFolderName;   ///< Pathname of the electoral image folder
                        std::string ElectionFileName;   ///< Filename of the election file
                        std::string DeviceParametersFileName;   ///< Filename of the device parameters file
                        std::string upKeyFileName; ///< Filename of the up key
                        std::string DocsFolder; ///< Pathname of the documents folder
                        std::string EmlFilesFolder; ///< EML file folder name
                        std::string EmlFolder; ///< Path name of the EML folder
                        std::string EmlTempFolder; ///< Path name of the EML temp folder.
                };
            }
        }
    }
}

#endif // _SAES_DIRECTORIES_H_
