#ifndef TEST_SMARTCARD_TEST
#define TEST_SMARTCARD_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Card/Smartcard.hxx>
#include "MockConnectionBehavior.hxx"
#include "MockJavaCardOperations.hxx"

class SmartcardTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::Card::Smartcard Smartcard;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

    public:

        SmartcardTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testOpenDevice()
        {
            MockConnectionBehavior * mockConnectionBehavior = new MockConnectionBehavior();
            MockJavaCardOperations * mockJavaCardOperations = new MockJavaCardOperations();

            Smartcard manager ( mockConnectionBehavior, mockJavaCardOperations, SafeBuffer(), SafeBuffer() );

            MOCKPP_CHAINER_FOR ( MockConnectionBehavior, establishContext ) establishContext_mock ( mockConnectionBehavior );

            establishContext_mock
            .expects ( MOCKPP_NS::atLeastOnce() )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            CPPUNIT_ASSERT ( false == manager.open ( "" ) );
            CPPUNIT_ASSERT ( true == manager.open ( "" ) );

            mockConnectionBehavior->verify();
        }

        void testOpenTwoTimeDevice()
        {
            MockConnectionBehavior * mockConnectionBehavior = new MockConnectionBehavior();
            MockJavaCardOperations * mockJavaCardOperations = new MockJavaCardOperations();

            Smartcard manager ( mockConnectionBehavior, mockJavaCardOperations, SafeBuffer(), SafeBuffer() );

            MOCKPP_CHAINER_FOR ( MockConnectionBehavior, establishContext ) establishContext_mock ( mockConnectionBehavior );

            establishContext_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            CPPUNIT_ASSERT ( true == manager.open ( "" ) );
            CPPUNIT_ASSERT ( true == manager.open ( "" ) );

            mockConnectionBehavior->verify();
        }

        void testCloseDevice()
        {
            MockConnectionBehavior * mockConnectionBehavior = new MockConnectionBehavior();
            MockJavaCardOperations * mockJavaCardOperations = new MockJavaCardOperations();

            Smartcard manager ( mockConnectionBehavior, mockJavaCardOperations, SafeBuffer(), SafeBuffer() );

            MOCKPP_CHAINER_FOR ( MockConnectionBehavior, establishContext ) establishContext_mock ( mockConnectionBehavior );
            MOCKPP_CHAINER_FOR ( MockConnectionBehavior, releaseContext ) releaseContext_mock ( mockConnectionBehavior );

            establishContext_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            releaseContext_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            CPPUNIT_ASSERT ( true == manager.open ( "" ) );

            CPPUNIT_ASSERT ( true == manager.close() );
            CPPUNIT_ASSERT ( true == manager.close() );

            mockConnectionBehavior->verify();
        }

        void testCloseDeviceFail()
        {
            MockConnectionBehavior * mockConnectionBehavior = new MockConnectionBehavior();
            MockJavaCardOperations * mockJavaCardOperations = new MockJavaCardOperations();

            Smartcard manager ( mockConnectionBehavior, mockJavaCardOperations, SafeBuffer(), SafeBuffer() );

            MOCKPP_CHAINER_FOR ( MockConnectionBehavior, establishContext ) establishContext_mock ( mockConnectionBehavior );
            MOCKPP_CHAINER_FOR ( MockConnectionBehavior, releaseContext ) releaseContext_mock ( mockConnectionBehavior );

            establishContext_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            releaseContext_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( false ) );

            CPPUNIT_ASSERT ( true == manager.open ( "" ) );

            CPPUNIT_ASSERT ( false == manager.close() );
            CPPUNIT_ASSERT ( true == manager.close() );

            mockConnectionBehavior->verify();
        }

        void testCloseTwoTimeDevice()
        {
            MockConnectionBehavior * mockConnectionBehavior = new MockConnectionBehavior();
            MockJavaCardOperations * mockJavaCardOperations = new MockJavaCardOperations();

            Smartcard manager ( mockConnectionBehavior, mockJavaCardOperations, SafeBuffer(), SafeBuffer() );

            MOCKPP_CHAINER_FOR ( MockConnectionBehavior, establishContext ) establishContext_mock ( mockConnectionBehavior );
            MOCKPP_CHAINER_FOR ( MockConnectionBehavior, releaseContext ) releaseContext_mock ( mockConnectionBehavior );

            establishContext_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            releaseContext_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            CPPUNIT_ASSERT ( true == manager.open ( "" ) );

            CPPUNIT_ASSERT ( true == manager.close() );
            CPPUNIT_ASSERT ( true == manager.close() );

            mockConnectionBehavior->verify();
        }

        CPPUNIT_TEST_SUITE ( SmartcardTest );

        CPPUNIT_TEST ( testOpenDevice );
        CPPUNIT_TEST ( testOpenTwoTimeDevice );
        CPPUNIT_TEST ( testCloseDevice );
        CPPUNIT_TEST ( testCloseDeviceFail );
        CPPUNIT_TEST ( testCloseTwoTimeDevice );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SMARTCARD_TEST */
