#ifndef TEST_DEVICE_MONITOR_CONF_MANAGER_TEST
#define TEST_DEVICE_MONITOR_CONF_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockSaesCommonConfigurationManager.hxx>
#include <tests/MockDeviceTypeMonitorConfigurationManager.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>

class DeviceTypeMonitorConfigurationManagerTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;
        typedef Smartmatic::SAES::Common::Configuration::SAESCommonConfigurationManager SAESCommonConfigurationManager;
        typedef Smartmatic::SAES::Common::Configuration::DeviceTypeMonitorConfigurationManager DeviceTypeMonitorConfigurationManager;
        typedef Smartmatic::SAES::Common::Configuration::DeviceTypeMonitorPoolConfiguration DeviceTypeMonitorPoolConfiguration;
        typedef Smartmatic::SAES::Common::Configuration::DeviceTypeMonitorConfiguration DeviceTypeMonitorConfiguration;
        typedef Smartmatic::SAES::Common::Configuration::DeviceType DeviceType;

        DeviceTypeMonitorConfigurationManagerTest()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testGetDeviceMonitorConfigurationPresent()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().present() );
            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().get().getSimulatorEventDeviceFilePath().present() );

            persistenceManager.verify();
        }

        void testGetDeviceMonitorConfigurationWithoutDevices()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().reset();

            DeviceTypeMonitorConfigurationManager * deviceMonitorManager = DeviceTypeMonitorConfigurationManager::getInstance();
            DeviceTypeMonitorConfiguration device1 = deviceMonitorManager->getDeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_FLASH );

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_FLASH == device1.getType() );
            CPPUNIT_ASSERT ( " ## " == device1.getNode() );
            CPPUNIT_ASSERT ( true == device1.getProperty().empty() );

            DeviceTypeMonitorConfiguration device2 = deviceMonitorManager->getDeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_BARCODE );

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_BARCODE == device2.getType() );
            CPPUNIT_ASSERT ( " ## " == device2.getNode() );
            CPPUNIT_ASSERT ( true == device2.getProperty().empty() );

            delete deviceMonitorManager;

            persistenceManager.verify();
        }

        void testGetDeviceMonitorConfigurationWithDevices()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().reset();

            DeviceTypeMonitorPoolConfiguration deviceMonitorConfig;
            DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence deviceSeq;

            deviceSeq.push_back ( DeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_BARCODE, "bcr-node" ) );
            deviceSeq.push_back ( DeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_FLASH, "flash-node" ) );

            deviceMonitorConfig.setDeviceTypeMonitor ( deviceSeq );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->setDeviceTypeMonitorPoolConfiguration ( deviceMonitorConfig );

            DeviceTypeMonitorConfigurationManager * deviceMonitorManager = DeviceTypeMonitorConfigurationManager::getInstance();
            DeviceTypeMonitorConfiguration device1 = deviceMonitorManager->getDeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_FLASH );

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_FLASH == device1.getType() );
            CPPUNIT_ASSERT ( "flash-node" == device1.getNode() );
            CPPUNIT_ASSERT ( true == device1.getProperty().empty() );

            DeviceTypeMonitorConfiguration device2 = deviceMonitorManager->getDeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_BARCODE );

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_BARCODE == device2.getType() );
            CPPUNIT_ASSERT ( "bcr-node" == device2.getNode() );
            CPPUNIT_ASSERT ( true == device2.getProperty().empty() );

            delete deviceMonitorManager;

            persistenceManager.verify();
        }

        void testGetCustomNodeWithoutPlaceHolder()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().reset();

            DeviceTypeMonitorPoolConfiguration deviceMonitorConfig;
            DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence deviceSeq;

            deviceSeq.push_back ( DeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_FLASH, "flash-node" ) );

            deviceMonitorConfig.setDeviceTypeMonitor ( deviceSeq );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->setDeviceTypeMonitorPoolConfiguration ( deviceMonitorConfig );

            DeviceTypeMonitorConfigurationManager * deviceMonitorManager = DeviceTypeMonitorConfigurationManager::getInstance();
            DeviceTypeMonitorConfiguration device1 = deviceMonitorManager->getDeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_FLASH );

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_FLASH == device1.getType() );
            CPPUNIT_ASSERT ( "flash-node" == device1.getNode() );
            CPPUNIT_ASSERT ( false == device1.getCustomNode().present() );
            CPPUNIT_ASSERT ( true == device1.getProperty().empty() );

            std::string command = deviceMonitorManager->getCustomNode ( "node", DeviceType::CONFIGURATION_FLASH );

            CPPUNIT_ASSERT ( false == command.empty() );
            CPPUNIT_ASSERT ( "node" == command );

            delete deviceMonitorManager;

            persistenceManager.verify();
        }

        void testGetCustomNodeWithPlaceHolder()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().reset();

            DeviceTypeMonitorPoolConfiguration deviceMonitorConfig;
            DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence deviceSeq;

            DeviceTypeMonitorConfiguration flash ( DeviceType::CONFIGURATION_FLASH, "flash-node" );
            flash.setCustomNode ( "command %1" );

            deviceSeq.push_back ( flash );

            deviceMonitorConfig.setDeviceTypeMonitor ( deviceSeq );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->setDeviceTypeMonitorPoolConfiguration ( deviceMonitorConfig );

            DeviceTypeMonitorConfigurationManager * deviceMonitorManager = DeviceTypeMonitorConfigurationManager::getInstance();
            DeviceTypeMonitorConfiguration device1 = deviceMonitorManager->getDeviceTypeMonitorConfiguration ( DeviceType::CONFIGURATION_FLASH );

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_FLASH == device1.getType() );
            CPPUNIT_ASSERT ( "flash-node" == device1.getNode() );
            CPPUNIT_ASSERT ( true == device1.getCustomNode().present() );
            CPPUNIT_ASSERT ( true == device1.getProperty().empty() );

            std::string command = deviceMonitorManager->getCustomNode ( "node", DeviceType::CONFIGURATION_FLASH );

            CPPUNIT_ASSERT ( false == command.empty() );
            CPPUNIT_ASSERT ( "command node" == command );

            delete deviceMonitorManager;

            persistenceManager.verify();
        }

        void testCheckSimulatorEnabledFalse()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().reset();

            DeviceTypeMonitorConfigurationManager * deviceMonitorManager = DeviceTypeMonitorConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( false == deviceMonitorManager->simulationEnabled() );

            delete deviceMonitorManager;

            persistenceManager.verify();
        }

        void testCheckSimulatorEnabledAndGetFilePath()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorPoolConfiguration().reset();

            DeviceTypeMonitorPoolConfiguration deviceMonitorConfig;
            deviceMonitorConfig.setSimulatorEventDeviceFilePath ( "FilePath" );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->setDeviceTypeMonitorPoolConfiguration ( deviceMonitorConfig );

            DeviceTypeMonitorConfigurationManager * deviceMonitorManager = DeviceTypeMonitorConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( true == deviceMonitorManager->simulationEnabled() );
            CPPUNIT_ASSERT ( "FilePath" == deviceMonitorManager->getSimulatorFilePath() );

            delete deviceMonitorManager;

            persistenceManager.verify();
        }

        CPPUNIT_TEST_SUITE ( DeviceTypeMonitorConfigurationManagerTest );

        CPPUNIT_TEST ( testGetDeviceMonitorConfigurationPresent );
        CPPUNIT_TEST ( testGetDeviceMonitorConfigurationWithoutDevices );
        CPPUNIT_TEST ( testGetDeviceMonitorConfigurationWithDevices );

        CPPUNIT_TEST ( testGetCustomNodeWithoutPlaceHolder );
        CPPUNIT_TEST ( testGetCustomNodeWithPlaceHolder );

        CPPUNIT_TEST ( testCheckSimulatorEnabledFalse );
        CPPUNIT_TEST ( testCheckSimulatorEnabledAndGetFilePath );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_DEVICE_MONITOR_CONF_MANAGER_TEST */
