#ifndef TEST_SMARTCARD_CONF_MANAGER_TEST
#define TEST_SMARTCARD_CONF_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockSmartcardConfigurationManager.hxx>
#include <tests/MockSaesCommonConfigurationManager.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>

class SmartcardConfigurationManagerTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::SAES::Common::Configuration::SmartcardConfigurationManager SmartcardConfigurationManager;
        typedef Smartmatic::SAES::Common::Configuration::SAESCommonConfigurationManager SAESCommonConfigurationManager;
        typedef Smartmatic::SAES::Common::Configuration::SmartcardConfiguration SmartcardConfiguration;
        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;
        typedef Smartmatic::SAES::Common::Configuration::PlatformMode PlatformMode;
        typedef Smartmatic::SAES::Common::Configuration::SecurityChannelProtocol SecurityChannelProtocol;
        typedef Smartmatic::SAES::Common::Configuration::SecurityChannelProtocolImpl SecurityChannelProtocolImpl;
        typedef Smartmatic::SAES::Common::Configuration::SecurityLevel SecurityLevel;
        typedef Smartmatic::SAES::Common::Configuration::SecurityKeyDerivation SecurityKeyDerivation;
        typedef Smartmatic::SAES::Common::Configuration::SmartcardSimulation SmartcardSimulation;

        SmartcardConfigurationManagerTest()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testGetSmartcardConfigurationPresent()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );
            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().present() );

            CPPUNIT_ASSERT ( false == smartcardManager->getAppletID().empty() );
            CPPUNIT_ASSERT ( false == smartcardManager->getGlobalPIN().empty() );
            CPPUNIT_ASSERT ( PlatformMode::CONFIGURATION_GP_211 == smartcardManager->getPlatformMode() );
            CPPUNIT_ASSERT ( false == smartcardManager->hasSecureChannel() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getSecurityKeySetVersion() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getSecurityKeyIndex() );
            CPPUNIT_ASSERT ( SecurityChannelProtocol::CONFIGURATION_SCP02 == smartcardManager->getSecurityChannelProtocol() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I15 == smartcardManager->getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC == smartcardManager->getSecurityLevel() );
            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_NONE == smartcardManager->getSecurityKeyDerivation() );
            CPPUNIT_ASSERT ( false == smartcardManager->getForceRestartSmartcardReader() );
            CPPUNIT_ASSERT ( false == smartcardManager->getForceRestartPCSCD() );
            CPPUNIT_ASSERT ( 2 == smartcardManager->getDelayAfterOn() );
            CPPUNIT_ASSERT ( 2 == smartcardManager->getWaitStopPCSCD() );
            CPPUNIT_ASSERT ( 4 == smartcardManager->getWaitStartPCSCD() );
            CPPUNIT_ASSERT ( false == smartcardManager->getCallRestartBeforeInitialize() );
            CPPUNIT_ASSERT ( false == smartcardManager->getConnectionPluginName().empty() );
            CPPUNIT_ASSERT ( false == smartcardManager->getConnectionPluginVersion().empty() );
            CPPUNIT_ASSERT ( 3 == smartcardManager->getNumberOfTriesToRead() );
            CPPUNIT_ASSERT ( false == smartcardManager->isErrorCode ( 0 ) );

            CPPUNIT_ASSERT ( true == smartcardManager->simulationEnabled() );
            CPPUNIT_ASSERT ( "/tmp/SmartcardEvent" == smartcardManager->getSimulatorEventFile() );
            CPPUNIT_ASSERT ( "/tmp/SmartcardContent" == smartcardManager->getSimulatorContentFile() );
            CPPUNIT_ASSERT ( 1 == smartcardManager->getSimulationTimeAfterReadContentFile() );
            CPPUNIT_ASSERT ( 1 == smartcardManager->getSimulationTimeBeforeWriteContentFile() );

            persistenceManager.verify();
        }

        void testGetSmartcardConfigurationDefault()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().reset();

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( "aid" == smartcardManager->getAppletID() );
            CPPUNIT_ASSERT ( "pin" == smartcardManager->getGlobalPIN() );
            CPPUNIT_ASSERT ( PlatformMode::CONFIGURATION_GP_211 == smartcardManager->getPlatformMode() );
            CPPUNIT_ASSERT ( false == smartcardManager->hasSecureChannel() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getSecurityKeySetVersion() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getSecurityKeyIndex() );
            CPPUNIT_ASSERT ( SecurityChannelProtocol::CONFIGURATION_SCP01 == smartcardManager->getSecurityChannelProtocol() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I05 == smartcardManager->getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_MAC == smartcardManager->getSecurityLevel() );
            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_NONE == smartcardManager->getSecurityKeyDerivation() );
            CPPUNIT_ASSERT ( false == smartcardManager->getForceRestartSmartcardReader() );
            CPPUNIT_ASSERT ( false == smartcardManager->getForceRestartPCSCD() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getDelayAfterOn() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getWaitStopPCSCD() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getWaitStartPCSCD() );
            CPPUNIT_ASSERT ( false == smartcardManager->getCallRestartBeforeInitialize() );
            CPPUNIT_ASSERT ( "gppcscconnectionplugin" == smartcardManager->getConnectionPluginName() );
            CPPUNIT_ASSERT ( "1.0.1" == smartcardManager->getConnectionPluginVersion() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getNumberOfTriesToRead() );
            CPPUNIT_ASSERT ( false == smartcardManager->isErrorCode ( 0 ) );

            CPPUNIT_ASSERT ( false == smartcardManager->simulationEnabled() );
            CPPUNIT_ASSERT ( true == smartcardManager->getSimulatorEventFile().empty() );
            CPPUNIT_ASSERT ( true == smartcardManager->getSimulatorContentFile().empty() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getSimulationTimeAfterReadContentFile() );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getSimulationTimeBeforeWriteContentFile() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetAppletID()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setAppletID ( "12345" );
            CPPUNIT_ASSERT ( "12345" == smartcardManager->getAppletID() );

            config.setAppletID ( "54321" );
            CPPUNIT_ASSERT ( "54321" == smartcardManager->getAppletID() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetGlobalPIN()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setGlobalPIN ( "pin1" );
            CPPUNIT_ASSERT ( "pin1" == smartcardManager->getGlobalPIN() );

            config.setGlobalPIN ( "pin2" );
            CPPUNIT_ASSERT ( "pin2" == smartcardManager->getGlobalPIN() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetPlatformMode()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setPlatformMode ( PlatformMode::CONFIGURATION_GP_211 );
            CPPUNIT_ASSERT ( PlatformMode::CONFIGURATION_GP_211 == smartcardManager->getPlatformMode() );

            config.setPlatformMode ( PlatformMode::CONFIGURATION_GP_211 );
            CPPUNIT_ASSERT ( PlatformMode::CONFIGURATION_GP_211 == smartcardManager->getPlatformMode() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSecurityChannel()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setSecureChannelEnabled ( false );
            CPPUNIT_ASSERT ( false == smartcardManager->hasSecureChannel() );

            config.setSecureChannelEnabled ( true );
            CPPUNIT_ASSERT ( true == smartcardManager->hasSecureChannel() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSecurityKeySetVersion()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setSecurityKeySetVersion ( 1 );
            CPPUNIT_ASSERT ( 1 == smartcardManager->getSecurityKeySetVersion() );

            config.setSecurityKeySetVersion ( 2 );
            CPPUNIT_ASSERT ( 2 == smartcardManager->getSecurityKeySetVersion() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSecurityKeyIndex()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setSecurityKeyIndex ( 10 );
            CPPUNIT_ASSERT ( 10 == smartcardManager->getSecurityKeyIndex() );

            config.setSecurityKeyIndex ( 20 );
            CPPUNIT_ASSERT ( 20 == smartcardManager->getSecurityKeyIndex() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSecurityChannelProtocol()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setSecurityChannelProtocol ( SecurityChannelProtocol::CONFIGURATION_SCP01 );
            CPPUNIT_ASSERT ( SecurityChannelProtocol::CONFIGURATION_SCP01 == smartcardManager->getSecurityChannelProtocol() );

            config.setSecurityChannelProtocol ( SecurityChannelProtocol::CONFIGURATION_SCP02 );
            CPPUNIT_ASSERT ( SecurityChannelProtocol::CONFIGURATION_SCP02 == smartcardManager->getSecurityChannelProtocol() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSecurityChannelProtocolmpl()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I05 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I05 == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I15 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I15 == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I04 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I04 == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I05 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I05 == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I0A );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I0A == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I0B );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I0B == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I14 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I14 == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I15 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I15 == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I1A );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I1A == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I1B );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I1B == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I44 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I44 == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I45 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I45 == smartcardManager->getSecurityChannelProtocolImpl() );

            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I55 );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I55 == smartcardManager->getSecurityChannelProtocolImpl() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSecurityLevel()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setSecurityLevel ( SecurityLevel::CONFIGURATION_R_MAC );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_R_MAC == smartcardManager->getSecurityLevel() );

            config.setSecurityLevel ( SecurityLevel::CONFIGURATION_C_MAC );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_MAC == smartcardManager->getSecurityLevel() );

            config.setSecurityLevel ( SecurityLevel::CONFIGURATION_C_MAC_R_MAC );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_MAC_R_MAC == smartcardManager->getSecurityLevel() );

            config.setSecurityLevel ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC == smartcardManager->getSecurityLevel() );

            config.setSecurityLevel ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC_R_MAC );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC_R_MAC == smartcardManager->getSecurityLevel() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSecurityKeyDerivation()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setSecurityKeyDerivation ( SecurityKeyDerivation::CONFIGURATION_NONE );
            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_NONE == smartcardManager->getSecurityKeyDerivation() );

            config.setSecurityKeyDerivation ( SecurityKeyDerivation::CONFIGURATION_VISA2 );
            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_VISA2 == smartcardManager->getSecurityKeyDerivation() );

            config.setSecurityKeyDerivation ( SecurityKeyDerivation::CONFIGURATION_EMVCPS11 );
            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_EMVCPS11 == smartcardManager->getSecurityKeyDerivation() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSuspendResumeOnErrorCondition()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setForceRestartSmartcardReader ( false );
            config.setForceRestartPCSCD ( false );

            CPPUNIT_ASSERT ( false == smartcardManager->getForceRestartSmartcardReader() );
            CPPUNIT_ASSERT ( false == smartcardManager->getForceRestartPCSCD() );

            config.setForceRestartSmartcardReader ( true );
            config.setForceRestartPCSCD ( false );

            CPPUNIT_ASSERT ( true == smartcardManager->getForceRestartSmartcardReader() );
            CPPUNIT_ASSERT ( false == smartcardManager->getForceRestartPCSCD() );

            config.setForceRestartSmartcardReader ( false );
            config.setForceRestartPCSCD ( true );

            CPPUNIT_ASSERT ( false == smartcardManager->getForceRestartSmartcardReader() );
            CPPUNIT_ASSERT ( true == smartcardManager->getForceRestartPCSCD() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetNumberOfTriesToRead()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setReadRetry ( 0 );
            CPPUNIT_ASSERT ( 0 == smartcardManager->getNumberOfTriesToRead() );

            config.setReadRetry ( 10 );
            CPPUNIT_ASSERT ( 10 == smartcardManager->getNumberOfTriesToRead() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testErrorCode_EmptyErrorCodeList()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            config.getFatalErrorCodes().clear();

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( false == smartcardManager->isErrorCode ( 0 ) );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testErrorCode_NotFoundCode()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            config.getFatalErrorCodes().clear();
            config.getFatalErrorCodes().push_back ( "0010" );
            config.getFatalErrorCodes().push_back ( "0020" );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( false == smartcardManager->isErrorCode ( 0x15 ) );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testErrorCode_FoundCode()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );
            config.getFatalErrorCodes().clear();
            config.getFatalErrorCodes().push_back ( "0010" );
            config.getFatalErrorCodes().push_back ( "0015" );
            config.getFatalErrorCodes().push_back ( "0020" );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( true == smartcardManager->isErrorCode ( 0x15 ) );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSimulationModeDisabled()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().reset();

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( false == smartcardManager->simulationEnabled() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSimulationModeEnabled()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().reset();

            SmartcardConfiguration config;
            SmartcardSimulation simulator ( "", "", 0, 0 );
            config.setSmartcardSimulation ( simulator );

            SAESCommonConfigurationManager::getInstance()->getConfiguration()->setSmartcardConfiguration ( config );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( true == smartcardManager->simulationEnabled() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSimulationEventFile()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().present() );

            SmartcardSimulation & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setEventFile ( "xxyy" );
            CPPUNIT_ASSERT ( "xxyy" == smartcardManager->getSimulatorEventFile() );

            config.setEventFile ( "yyxx" );
            CPPUNIT_ASSERT ( "yyxx" == smartcardManager->getSimulatorEventFile() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSimulationContentFile()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().present() );

            SmartcardSimulation & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setContentFile ( "xxyy" );
            CPPUNIT_ASSERT ( "xxyy" == smartcardManager->getSimulatorContentFile() );

            config.setContentFile ( "yyxx" );
            CPPUNIT_ASSERT ( "yyxx" == smartcardManager->getSimulatorContentFile() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testSimulationTimeAfterReadContentFile()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().present() );

            SmartcardSimulation & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setTimeAfterRead ( 2 );
            CPPUNIT_ASSERT ( 2 == smartcardManager->getSimulationTimeAfterReadContentFile() );

            config.setTimeAfterRead ( 3 );
            CPPUNIT_ASSERT ( 3 == smartcardManager->getSimulationTimeAfterReadContentFile() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetSimulationTimeBeforeWriteContentFile()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().present() );

            SmartcardSimulation & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get().getSmartcardSimulation().get() );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            config.setTimeBeforeWrite ( 4 );
            CPPUNIT_ASSERT ( 4 == smartcardManager->getSimulationTimeBeforeWriteContentFile() );

            config.setTimeBeforeWrite ( 5 );
            CPPUNIT_ASSERT ( 5 == smartcardManager->getSimulationTimeBeforeWriteContentFile() );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testGetAllConfigurationValues()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesCommonConfigurationManager configurationManager;
            SAESCommonConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().present() );

            SmartcardConfiguration & config ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getSmartcardConfiguration().get() );

            config.setAppletID ( "12345" );
            config.setGlobalPIN ( "F503" );
            config.setPlatformMode ( PlatformMode::CONFIGURATION_OP_201 );
            config.setSecureChannelEnabled ( false );
            config.setSecurityKeySetVersion ( 1 );
            config.setSecurityKeyIndex ( 2 );
            config.setSecurityChannelProtocol ( SecurityChannelProtocol::CONFIGURATION_SCP01 );
            config.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I05 );
            config.setSecurityLevel ( SecurityLevel::CONFIGURATION_C_MAC );
            config.setSecurityKeyDerivation ( SecurityKeyDerivation::CONFIGURATION_NONE );

            SmartcardConfigurationManager * smartcardManager = SmartcardConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( "12345" == smartcardManager->getAppletID() );
            CPPUNIT_ASSERT ( "F503" == smartcardManager->getGlobalPIN() );
            CPPUNIT_ASSERT ( PlatformMode::CONFIGURATION_OP_201 == smartcardManager->getPlatformMode() );
            CPPUNIT_ASSERT ( false == smartcardManager->hasSecureChannel() );
            CPPUNIT_ASSERT ( 1 == smartcardManager->getSecurityKeySetVersion() );
            CPPUNIT_ASSERT ( 2 == smartcardManager->getSecurityKeyIndex() );
            CPPUNIT_ASSERT ( SecurityChannelProtocol::CONFIGURATION_SCP01 == smartcardManager->getSecurityChannelProtocol() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I05 == smartcardManager->getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_MAC == smartcardManager->getSecurityLevel() );
            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_NONE == smartcardManager->getSecurityKeyDerivation () );
            CPPUNIT_ASSERT ( false == smartcardManager->isErrorCode ( 0 ) );

            delete smartcardManager;

            persistenceManager.verify();
        }

        void testMockGetAppletID()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getAppletID ) getAppletID_mock ( &smartcardManager );

            getAppletID_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<std::string> ( "12345" ),
                        new MOCKPP_NS::ReturnStub<std::string> ( "54321" ) ) );

            CPPUNIT_ASSERT ( "12345" == smartcardManager.getAppletID() );
            CPPUNIT_ASSERT ( "54321" == smartcardManager.getAppletID() );
        }

        void testMockGetGlobalPIN()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getGlobalPIN ) getGlobalPIN_mock ( &smartcardManager );

            getGlobalPIN_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<std::string> ( "F501" ),
                        new MOCKPP_NS::ReturnStub<std::string> ( "F502" ) ) );

            CPPUNIT_ASSERT ( "F501" == smartcardManager.getGlobalPIN() );
            CPPUNIT_ASSERT ( "F502" == smartcardManager.getGlobalPIN() );
        }

        void testMockGetPlatformMode()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getPlatformMode ) getPlatformMode_mock ( &smartcardManager );

            getPlatformMode_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<PlatformMode> ( PlatformMode::CONFIGURATION_OP_201 ),
                        new MOCKPP_NS::ReturnStub<PlatformMode> ( PlatformMode::CONFIGURATION_GP_211 ) ) );

            CPPUNIT_ASSERT ( PlatformMode::CONFIGURATION_OP_201 == smartcardManager.getPlatformMode() );
            CPPUNIT_ASSERT ( PlatformMode::CONFIGURATION_GP_211 == smartcardManager.getPlatformMode() );
        }

        void testMockGetSecureChannel()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, hasSecureChannel ) hasSecureChannel_mock ( &smartcardManager );

            hasSecureChannel_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            CPPUNIT_ASSERT ( false == smartcardManager.hasSecureChannel() );
            CPPUNIT_ASSERT ( true == smartcardManager.hasSecureChannel() );
        }

        void testMockGetSecurityKeySetVersion()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getSecurityKeySetVersion ) getSecurityKeySetVersion_mock ( &smartcardManager );

            getSecurityKeySetVersion_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <int> ( 1 ),
                        MOCKPP_NS::returnValue <int> ( 2 ) ) );

            CPPUNIT_ASSERT ( 1 == smartcardManager.getSecurityKeySetVersion() );
            CPPUNIT_ASSERT ( 2 == smartcardManager.getSecurityKeySetVersion() );
        }

        void testMockGetSecurityKeyIndex()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getSecurityKeyIndex ) getSecurityKeyIndex_mock ( &smartcardManager );

            getSecurityKeyIndex_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <int> ( 10 ),
                        MOCKPP_NS::returnValue <int> ( 20 ) ) );

            CPPUNIT_ASSERT ( 10 == smartcardManager.getSecurityKeyIndex() );
            CPPUNIT_ASSERT ( 20 == smartcardManager.getSecurityKeyIndex() );
        }

        void testMockGetSecurityChannelProtocol()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getSecurityChannelProtocol ) getSecurityChannelProtocol_mock ( &smartcardManager );

            getSecurityChannelProtocol_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocol> ( SecurityChannelProtocol::CONFIGURATION_SCP01 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocol> ( SecurityChannelProtocol::CONFIGURATION_SCP02 ) ) );

            CPPUNIT_ASSERT ( SecurityChannelProtocol::CONFIGURATION_SCP01 == smartcardManager.getSecurityChannelProtocol() );
            CPPUNIT_ASSERT ( SecurityChannelProtocol::CONFIGURATION_SCP02 == smartcardManager.getSecurityChannelProtocol() );
        }

        void testMockGetSecurityChannelProtocolImpl_SCP01()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getSecurityChannelProtocolImpl ) getSecurityChannelProtocolImpl_mock ( &smartcardManager );

            getSecurityChannelProtocolImpl_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I05 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I15 ) ) ) ;

            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I05 == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP01_IMPL_I15 == smartcardManager.getSecurityChannelProtocolImpl() );
        }

        void testMockGetSecurityChannelProtocolImpl_SCP02_A()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getSecurityChannelProtocolImpl ) getSecurityChannelProtocolImpl_mock ( &smartcardManager );

            getSecurityChannelProtocolImpl_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I04 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I05 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I0A ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I0B ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I14 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I15 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I1A ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I1B ) ) );

            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I04 == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I05 == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I0A == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I0B == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I14 == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I15 == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I1A == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I1B == smartcardManager.getSecurityChannelProtocolImpl() );
        }

        void testMockGetSecurityChannelProtocolImpl_SCP02_B()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getSecurityChannelProtocolImpl ) getSecurityChannelProtocolImpl_mock ( &smartcardManager );

            getSecurityChannelProtocolImpl_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I44 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I45 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I54 ),
                        new MOCKPP_NS::ReturnStub<SecurityChannelProtocolImpl> ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I55 ) ) );

            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I44 == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I45 == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I54 == smartcardManager.getSecurityChannelProtocolImpl() );
            CPPUNIT_ASSERT ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I55 == smartcardManager.getSecurityChannelProtocolImpl() );
        }

        void testMockGetSecurityLevel()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getSecurityLevel ) getSecurityLevel_mock ( &smartcardManager );

            getSecurityLevel_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<SecurityLevel> ( SecurityLevel::CONFIGURATION_R_MAC ),
                        new MOCKPP_NS::ReturnStub<SecurityLevel> ( SecurityLevel::CONFIGURATION_C_MAC ),
                        new MOCKPP_NS::ReturnStub<SecurityLevel> ( SecurityLevel::CONFIGURATION_C_MAC_R_MAC ),
                        new MOCKPP_NS::ReturnStub<SecurityLevel> ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC ),
                        new MOCKPP_NS::ReturnStub<SecurityLevel> ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC_R_MAC ) ) );

            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_R_MAC == smartcardManager.getSecurityLevel() );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_MAC == smartcardManager.getSecurityLevel() );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_MAC_R_MAC == smartcardManager.getSecurityLevel() );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC == smartcardManager.getSecurityLevel() );
            CPPUNIT_ASSERT ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC_R_MAC == smartcardManager.getSecurityLevel() );
        }

        void testMockGetSecurityKeyDerivation()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getSecurityKeyDerivation ) getSecurityKeyDerivation_mock ( &smartcardManager );

            getSecurityKeyDerivation_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<SecurityKeyDerivation> ( SecurityKeyDerivation::CONFIGURATION_NONE ),
                        new MOCKPP_NS::ReturnStub<SecurityKeyDerivation> ( SecurityKeyDerivation::CONFIGURATION_VISA2 ),
                        new MOCKPP_NS::ReturnStub<SecurityKeyDerivation> ( SecurityKeyDerivation::CONFIGURATION_EMVCPS11 ) ) );

            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_NONE == smartcardManager.getSecurityKeyDerivation() );
            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_VISA2 == smartcardManager.getSecurityKeyDerivation() );
            CPPUNIT_ASSERT ( SecurityKeyDerivation::CONFIGURATION_EMVCPS11 == smartcardManager.getSecurityKeyDerivation() );
        }

        void testMockGetNumberOfTriesToRead ()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR ( MockSmartcardConfigurationManager, getNumberOfTriesToRead ) getNumberOfTriesToRead_mock ( &smartcardManager );

            getNumberOfTriesToRead_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <unsigned int> ( 0 ),
                        MOCKPP_NS::returnValue <unsigned int> ( 2 ),
                        MOCKPP_NS::returnValue <unsigned int> ( 5 ) ) );

            CPPUNIT_ASSERT ( 0 == smartcardManager.getNumberOfTriesToRead() );
            CPPUNIT_ASSERT ( 2 == smartcardManager.getNumberOfTriesToRead() );
            CPPUNIT_ASSERT ( 5 == smartcardManager.getNumberOfTriesToRead() );
        }

        void testMockGetErrorCondition ()
        {
            MockSmartcardConfigurationManager smartcardManager;
            SmartcardConfigurationManager::getInstance ( &smartcardManager );

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcardConfigurationManager, isErrorCode, ext ) isErrorCode_mock ( &smartcardManager );

            isErrorCode_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            CPPUNIT_ASSERT ( false == smartcardManager.isErrorCode ( 0x10 ) );
            CPPUNIT_ASSERT ( true == smartcardManager.isErrorCode ( 0x20 ) );
        }

        CPPUNIT_TEST_SUITE ( SmartcardConfigurationManagerTest );

        CPPUNIT_TEST ( testGetSmartcardConfigurationPresent );
        CPPUNIT_TEST ( testGetSmartcardConfigurationDefault );

        CPPUNIT_TEST ( testGetAppletID );
        CPPUNIT_TEST ( testGetGlobalPIN );
        CPPUNIT_TEST ( testGetPlatformMode );
        CPPUNIT_TEST ( testGetSecurityChannel );
        CPPUNIT_TEST ( testGetSecurityKeySetVersion );
        CPPUNIT_TEST ( testGetSecurityKeyIndex );
        CPPUNIT_TEST ( testGetSecurityChannelProtocol );
        CPPUNIT_TEST ( testGetSecurityChannelProtocolmpl );
        CPPUNIT_TEST ( testGetSecurityLevel );
        CPPUNIT_TEST ( testGetSecurityKeyDerivation );
        CPPUNIT_TEST ( testGetSuspendResumeOnErrorCondition );
        CPPUNIT_TEST ( testGetNumberOfTriesToRead );

        CPPUNIT_TEST ( testErrorCode_EmptyErrorCodeList );
        CPPUNIT_TEST ( testErrorCode_NotFoundCode );
        CPPUNIT_TEST ( testErrorCode_FoundCode );

        CPPUNIT_TEST ( testGetSimulationModeDisabled );
        CPPUNIT_TEST ( testGetSimulationModeEnabled );

        CPPUNIT_TEST ( testGetSimulationEventFile );
        CPPUNIT_TEST ( testGetSimulationContentFile );
        CPPUNIT_TEST ( testSimulationTimeAfterReadContentFile );
        CPPUNIT_TEST ( testGetSimulationTimeBeforeWriteContentFile );

        CPPUNIT_TEST ( testGetAllConfigurationValues );

        CPPUNIT_TEST ( testMockGetAppletID );
        CPPUNIT_TEST ( testMockGetGlobalPIN );
        CPPUNIT_TEST ( testMockGetPlatformMode );
        CPPUNIT_TEST ( testMockGetSecureChannel );
        CPPUNIT_TEST ( testMockGetSecurityKeySetVersion );
        CPPUNIT_TEST ( testMockGetSecurityKeyIndex );
        CPPUNIT_TEST ( testMockGetSecurityChannelProtocol );
        CPPUNIT_TEST ( testMockGetSecurityChannelProtocolImpl_SCP01 );
        CPPUNIT_TEST ( testMockGetSecurityChannelProtocolImpl_SCP02_A );
        CPPUNIT_TEST ( testMockGetSecurityChannelProtocolImpl_SCP02_B );
        CPPUNIT_TEST ( testMockGetSecurityLevel );
        CPPUNIT_TEST ( testMockGetSecurityKeyDerivation );
        CPPUNIT_TEST ( testMockGetNumberOfTriesToRead );
        CPPUNIT_TEST ( testMockGetErrorCondition );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SMARTCARD_CONF_MANAGER_TEST */
