#ifndef TEST_PASSPHRASE_TEST
#define TEST_PASSPHRASE_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Security/Passphrase.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include "MockKeyManager.hxx"

class PassphraseTest : public CPPUNIT_NS::TestFixture
{
    private:

        Smartmatic::SAES::Common::Security::Passphrase * passManager;
        MockKeyManager* keyManager;

    public:

        PassphraseTest ()
        {
            passManager = NULL;
            keyManager  = NULL;
        }

        void setUp()
        {
            keyManager = new MockKeyManager();

            Smartmatic::SAES::Common::Security::KeyManager::getInstance(keyManager);
            Smartmatic::System::Security::OpenSSLSecurity::init();
            passManager = Smartmatic::SAES::Common::Security::Passphrase::getInstance();
        }

        void tearDown()
        {
            if ( passManager )
            {
                delete passManager;
                passManager = NULL;
            }

            delete keyManager;
            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testPlainPassphrase()
        {
            std::string plain = "ABCDEF";
            passManager->setPlainPassphrase ( plain );

            CPPUNIT_ASSERT ( plain == passManager->getPlainPassphrase() );
        }

        void testCryptoPlainPassphraseIfNotPlainPassphrase()
        {
            Smartmatic::System::SafeBuffer buffer;

            CPPUNIT_ASSERT ( false == passManager->getCryptoPassphrase ( buffer ) );
        }

        void testCryptoPlainPassphrase()
        {
            std::string plain = "ABCDEF";
            passManager->setPlainPassphrase ( plain );
            Smartmatic::System::SafeBuffer buffer;

            CPPUNIT_ASSERT ( true == passManager->getCryptoPassphrase ( buffer ) );
        }

        void testDecryptEmptyPlainPassphrase()
        {
            std::string plain;
            Smartmatic::System::SafeBuffer buffer;
            CPPUNIT_ASSERT ( false == passManager->getDecryptPassphrase ( buffer, plain ) );
        }

        void testDecryptPlainPassphrase()
        {
            std::string plain;
            Smartmatic::System::SafeBuffer buffer ( "zasdcxsddddd" );
            CPPUNIT_ASSERT ( false == passManager->getDecryptPassphrase ( buffer, plain ) );
        }

        void testRegretion()
        {
            std::string plain = "ABCDEF";
            std::string result;

            passManager->setPlainPassphrase ( plain );
            Smartmatic::System::SafeBuffer bufferCrypto;

            CPPUNIT_ASSERT ( true == passManager->getCryptoPassphrase ( bufferCrypto ) );
            CPPUNIT_ASSERT ( true == passManager->getDecryptPassphrase ( bufferCrypto , result ) );
            CPPUNIT_ASSERT ( plain == result );
        }

        CPPUNIT_TEST_SUITE ( PassphraseTest );
        CPPUNIT_TEST ( testPlainPassphrase );
        CPPUNIT_TEST ( testCryptoPlainPassphraseIfNotPlainPassphrase );
        CPPUNIT_TEST ( testCryptoPlainPassphrase );
        CPPUNIT_TEST ( testDecryptEmptyPlainPassphrase );
        CPPUNIT_TEST ( testDecryptPlainPassphrase );
        CPPUNIT_TEST ( testRegretion );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_PASSPHRASE_TEST */
