/**
 * @file saes-functionality.h
 * @brief Declares the SAESFunctionality class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */


#ifndef _SAES_FUNCTIONALITY_H_
#define _SAES_FUNCTIONALITY_H_


#include "Functionalities/functionality.h"
#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Functionality
        {
            template <class TMenuOptionDescriptor>

            /**
             * @class SAESFunctionality
             * @brief Represents a SAES functionality.
             * @author Youssef.tourkmani
             * @date 01/11/2012
             */

            class SAESFunctionality: public Smartmatic::Functionality::Functionality<TMenuOptionDescriptor>
            {

                public:

                    /**
                     * @fn SAESFunctionality::SAESFunctionality<TMenuOptionDescriptor>()
                     * @brief Default constructor for the SAESFunctionality class.
                     * @author Youssef.tourkmani
                     * @date 01/11/2012
                     * @tparam TMenuOptionDescriptor Type of the menu option descriptor.
                     */

                    SAESFunctionality<TMenuOptionDescriptor>()
                    {
                    }

                    /**
                     * @fn virtual SAESFunctionality::~SAESFunctionality<TMenuOptionDescriptor>()
                     * @brief Finalizes an instance of the SAESFunctionality class.
                     * @author Youssef.tourkmani
                     * @date 01/11/2012
                     * @tparam TMenuOptionDescriptor Type of the menu option descriptor.
                     */

                    virtual ~SAESFunctionality<TMenuOptionDescriptor>()
                    {
                    }

                    //Pure Virtual Function (Abstract Members)

                    /**
                     * @fn virtual void SAESFunctionality::Execute() = 0;
                     * @brief Executes this functionality.
                     * @author Youssef.tourkmani
                     * @date 01/11/2012
                     */

                    virtual void Execute() = 0;

                    /**
                     * @fn virtual bool SAESFunctionality::CanExecute() = 0;
                     * @brief Verifies if this functionality can be executed.
                     * @author Youssef.tourkmani
                     * @date 01/11/2012
                     * @return true if this functionality can be executed. Otherwise, returns false.
                     */

                    virtual bool CanExecute() = 0;

                    /**
                     * @fn virtual bool SAESFunctionality::Enable() = 0;
                     * @brief Enables this functionality.
                     * @author Youssef.tourkmani
                     * @date 01/11/2012
                     * @return true if it succeeds, false if it fails.
                     */

                    virtual bool Enable() = 0;

                    /**
                     * @fn virtual std::string SAESFunctionality::GetWindowTitle() = 0;
                     * @brief Gets the window title.
                     * @author Youssef.tourkmani
                     * @date 01/11/2012
                     * @return The window title.
                     */

                    virtual std::string GetWindowTitle() = 0;

                    /**
                     * @fn virtual bool SAESFunctionality::ShowWindowTitle()
                     * @brief Verifies if the window title for this functionality must be shown.
                     * @author Youssef.tourkmani
                     * @date 01/11/2012
                     * @return true if the window title must be shown.
                     */

                    virtual bool ShowWindowTitle()
                    {
                        return true;
                    }

                    /**
                     * @fn virtual bool SAESFunctionality::ShowHelp()
                     * @brief Verifies if the help for this functionality must be shown.
                     * @author Yadickson Soto
                     * @date 04/10/2012
                     * @return true if the help must be shown. Otherwise, false.
                     */

                    virtual bool ShowHelp()
                    {
                        return false;
                    }

                    /**
                     * @fn virtual TMenuOptionDescriptor SAESFunctionality::GetMenuOptionType() = 0;
                     * @brief Gets the menu option type.
                     * @author Youssef.tourkmani
                     * @date 01/11/2012
                     * @return The menu option type.
                     */

                    virtual TMenuOptionDescriptor GetMenuOptionType() = 0;
            };
        }
    }
}

#endif // _SAES_FUNCTIONALITY_H_
