/**
 * @file CheckDataManager.hxx
 * @brief Header class to check data and synchronize
 * @data 29/07/2015
 * @author Yadickson Soto
 */

#ifndef _CHECK_DATA_MANAGER_HXX_
#define _CHECK_DATA_MANAGER_HXX_

#include <System/SafeThread.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            /**
             * @class CheckDataManager
             * @brief class to check data and synchronize
             * @data 29/07/2015
             * @author Yadickson Soto
             */
            class CheckDataManager : private Smartmatic::System::SafeThread
            {
                protected:

                    /**
                     * @fn CheckDataManager();
                     * @brief Class constructor
                     * @data 29/07/2015
                     * @author Yadickson Soto
                     */
                    CheckDataManager();

                    static CheckDataManager * instance; ///< The instance of the class

                public:

                    /**
                     * @fn virtual ~CheckDataManager();
                     * @brief Class destroyer
                     * @data 29/07/2015
                     * @author Yadickson Soto
                     */
                    virtual ~CheckDataManager();

                    /**
                     * @fn	void startDetection();
                     * @brief	Starts a detection.
                     * @author	Yadickson Soto
                     * @date	29/07/2015
                     */
                    void startDetection();

                    /**
                     * @fn	void stopDetection();
                     * @brief	Stop detection.
                     * @author	Yadickson Soto
                     * @date	29/07/2015
                     */
                    void stopDetection();

                    /**
                     * @fn	void checkInsertion();
                     * @brief	Check extract.
                     * @author	Juan Valera
                     * @date	15/03/2017
                     * @param numberOfDevices Number of devices
                     */
                    void checkInsertion( unsigned int numberOfDevices );

                    /**
                     * @fn	void checkExtraction( unsigned int numberOfDevices );
                     * @brief	Check extract.
                     * @author	Yadickson Soto
                     * @date	29/07/2015
                     * @param numberOfDevices Number of devices
                     */
                    void checkExtraction ( unsigned int numberOfDevices );

                    /**
                     * @fn	void startSafeSynchronize();
                     * @brief	Call initSafeSynchronize
                     * @author	Yadickson Soto
                     * @date	18/06/2013
                     */
                    void startSafeSynchronize();

                    /**
                     * @fn	void clearSynchronize();
                     * @brief	Only delete login-key.
                     * @author	Yadickson Soto
                     * @date	18/06/2013
                     */
                    void clearSynchronize();

                    /**
                     * @fn	static CheckDataManager * getInstance();
                     * @brief	Getter instance of the class
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     * @return The instance of the class
                     */
                    static CheckDataManager * getInstance();

                protected:

                    /**
                     * @fn	void MachineOperationManager::checkDataDevice();
                     * @brief	Check data device.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     */
                    void checkDataDevice();

                    /**
                     * @fn	void safeSynchronize();
                     * @brief	Call method synchronize()
                     * @author	Yadickson Soto
                     * @date	18/06/2013
                     */
                    void safeSynchronize();

                    /**
                     * @fn	    virtual bool process ();
                     * @brief	Callback from thread any time.
                     * @author	Yadickson Soto
                     * @date	30/07/2015
                     * @return true for continue thread
                     */
                    virtual bool process ();

                private:

                    bool isCheckData; ///< true if this object is check data
                    bool isSynchronize; ///< true if synchronize usb data
                    bool isSafeEject; ///< true if synchronize usb data if success and allow
                    bool isWaitingMount; ///< true if we are waiting for usb to be mounted
                    bool isInterrupted; ///< true if the synchronization or data checking was interrupted
                    bool hasNoMemories; ///< true if both memories are absent
            };

        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* _CHECK_DATA_MANAGER_HXX_ */
