/**
 * @file   PDBController.hxx
 * @brief  Declaration of PDBController class.
 * @author manuel.arguelles
 * @date   Nov  2, 2017
 */

#ifndef PDB_CONTROLLER_HXX
#define PDB_CONTROLLER_HXX

#include <glibmm.h>
#include <memory>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                class PDBDevice;
            }
        }

        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class  PDBController
                 * @brief  Controls the PDB device.
                 * @author manuel.arguelles
                 * @date   Nov  2, 2017
                 */
                class PDBController
                {
                    public:

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        ~PDBController();

                    public:

                        /**
                         * @brief  Returns the instance of controller.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         * @return The instance of the controller.
                         */
                        static PDBController* getInstance();

                    public:

                        /**
                         * @brief  Disable smartcard reader.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        void disableSmartCardReader();

                        /**
                         * @brief  Enable smartcard reader.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        void enableSmartCardReader();

                        /**
                         * @brief  Gets the FW version from the PDB's.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                        */
                        std::string getFWVersion();

                        /**
                         * @brief  Gets the serial number from the PDB's EEPROM.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                        */
                        std::string getSerialNumber();

                        /**
                         * @brief  Sets the serial number to the PDB's EEPROM.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        void setSerialNumber ( const std::string & serial, bool save );

                        /**
                         * @brief  turn on All LEDs in the PDB
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        bool turnAllLedsOn();

                        /**
                         * @brief  turn off All LEDs in the PDB
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        bool turnAllLedsOff();

                    private:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        PDBController();

                        /**
                         * @brief  Reconnects to the PDB device.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        void reconnect();

                    private:

                        Glib::Mutex                                 m_mutex;
                        static PDBController*                       m_instance;
                        static Smartmatic::Log::ISMTTLog*           m_logger;
                        std::auto_ptr<SAES::HW::Devices::PDBDevice> m_device;
                };

            } /* namespace Devices */
        } /* namespace Operation */
    } /*  namespace SAES */
} /* namespace Smartmatic */

#endif /* PDB_CONTROLLER_HXX */
