/**
 * @file   SmartcardReaderPowerManagerFactory.cxx
 * @brief  Implementation of SmartcardReaderPowerManagerFactory.
 * @author manuel.arguelles
 * @date   Nov  2, 2017
 */

#include "SmartcardReaderPowerManagerFactory.hxx"
#include <Card/P168ASmartcardReaderPowerManager.hxx>
#include <Operation/Devices/PDBSmartcardReaderPowerManager.hxx>
#include <Operation/Devices/IOBSmartcardReaderPowerManager.hxx>
#include <Card/DefaultSmartcardReaderPowerManager.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>

using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

ISmartcardReaderPowerManager* SmartcardReaderPowerManagerFactory::getSmartcardReaderPowerManager()
{
    std::string machineModel;
    ISmartcardReaderPowerManager* powerManager;

    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

    if (StringUtils::toUpper(machineModel) == "PM-NOTEBOOK-A" ||
        StringUtils::toUpper(machineModel) == "PM-NOTEBOOK-B" )
    {
        powerManager = new DefaultSmartcardReaderPowerManager();
    }
    else if (StringUtils::toUpper(machineModel) == "VM-SAES3370")
    {
        powerManager = new P168ASmartcardReaderPowerManager();
    }
    else if (StringUtils::toUpper(machineModel) == "PM-VIU805")
    {
        powerManager = new IOBSmartcardReaderPowerManager();
    }
    else
    {
        powerManager = new PDBSmartcardReaderPowerManager();
    }

    return powerManager;
}
