/**
 * @file BarCodeDiagnosis.cxx
 * @brief Implementation of the BarCodeDiagnosis.
 * @date 20/10/2011
 * @author Yadickson Soto
 */

#include "BarCodeDiagnosis.hxx"
#include <Operation/Devices/BCRController.hxx>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/BCRController.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::Diagnosis;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;

ISMTTLog * BarCodeDiagnosis::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Diagnosis.BarCodeDiagnosis", GETTEXT_PACKAGE );

BarCodeDiagnosis::BarCodeDiagnosis ( std::string command )
    : IDiagnosis ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE ),
      command ( command )
{
    runcommand = !command.empty();
    updateMessage();
    BCRController::getInstance()->allowReconfigureCommand();

    m_memoryEventConnection = MemoryEventManager::getInstance()->getMemorySignal().connect(
        sigc::mem_fun(*this, &BarCodeDiagnosis::onMemoryEvent));
}

BarCodeDiagnosis::~BarCodeDiagnosis()
{

}

void BarCodeDiagnosis::run()
{
    DeviceTypeMonitor * deviceTypeMonitor = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( DeviceType::CONFIGURATION_BARCODE );

    bool validate = deviceTypeMonitor != NULL && deviceTypeMonitor->getNumberOfDevices() > 0;

    if ( validate && runcommand )
    {
        validate = BCRController::getInstance()->sendReconfigureCommand ( command );
    }

    setStatus ( validate ? VMStatus::VOTING_MACHINE_STATUS_OK : VMStatus::VOTING_MACHINE_STATUS_ERROR );

    m_memoryEventConnection.disconnect();
}

void BarCodeDiagnosis::updateMessage()
{
    const std::string messageFooter = N_ ( "Smartmatic.SAES.Operation.Diagnosis.BarCodeDiagnosis.Info" );

    switch ( getStatus() )
    {
        default:
        case VMStatus::VOTING_MACHINE_STATUS_RUN:

            setMessageFooter(messageFooter);

            if ( runcommand )
            {
                setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.BarCodeDiagnosis.ReconfigureDevice" ) );
            }
            else
            {
                setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.BarCodeDiagnosis.InitializingDiagnostic" ) );
            }

            break;

        case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
            setMessageFooter(messageFooter);
            setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.BarCodeDiagnosis.InitializingDiagnostic" ) );
            break;

        case VMStatus::VOTING_MACHINE_STATUS_OK:
            setMessageFooter(messageFooter);
            setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.BarCodeDiagnosis.DiagnosticSuccessful" ) );
            break;

        case VMStatus::VOTING_MACHINE_STATUS_ERROR:

            setMessageFooter(" ");

            if ( runcommand )
            {
                setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.BarCodeDiagnosis.CouldNotReachBarcodeReaderDiagnosticFailed" ) );
            }
            else
            {
                setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.BarCodeDiagnosis.DiagnosticFailed" ) );
            }

            break;
    }
}

void BarCodeDiagnosis::onMemoryEvent(MemoryEventManager::MemoryEvent event)
{
    if ( event == MemoryEventManager::ONE_MEMORY )
        BCRController::getInstance()->stopReconfigureCommand();
}
