/**
 * @file BarCodeDiagnosis.hxx
 * @brief Declaration of the BarCodeDiagnosis class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#ifndef BARCODEDIAGNOSIS_HXX_
#define BARCODEDIAGNOSIS_HXX_

#include "IDiagnosis.hxx"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Diagnosis
            {
                /**
                 * @class BarCodeDiagnosis
                 * @brief Represents a diagnostic for the barcode reader device.
                 * @author Youssef.tourkmani
                 * @date 01/11/2012
                 */
                class BarCodeDiagnosis : public IDiagnosis
                {
                    public:

                        /**
                         * @fn BarCodeDiagnosis::BarCodeDiagnosis ( std::string command );
                         * @brief Constructor for the BarCodeDiagnosis class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param command The command to send to the barcode reader device.
                         */

                        explicit BarCodeDiagnosis ( std::string command );

                        /**
                         * @fn virtual BarCodeDiagnosis::~BarCodeDiagnosis();
                         * @brief Finalizes an instance of the BarCodeDiagnosis class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual ~BarCodeDiagnosis();

                    protected:

                        /**
                         * @fn virtual void BarCodeDiagnosis::run();
                         * @brief Runs this diagnostic.
                         * @author Yadickson Soto
                         * @date 28/09/2012
                         */

                        virtual void run();

                    private:

                        std::string command; ///< The command
                        bool runcommand; ///< The running command flag
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                        sigc::connection m_memoryEventConnection; ///< memory event connection

                    protected:

                        /**
                         * @fn virtual void BarCodeDiagnosis::updateMessage();
                         * @brief Updates the status message of this diagnostic.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual void updateMessage();

                    private:

                        void onMemoryEvent(MemoryEventManager::MemoryEvent event);
                };
            }
        }
    }
}

#endif /* BARCODEDIAGNOSIS_HXX_ */
