/**
 * @file CalibrationDiagnosis.cxx
 * @brief Implementation of the CalibrationDiagnosis class.
 * @date 20/10/2011
 * @author Yadickson Soto
 */

#include "CalibrationDiagnosis.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Diagnosis;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Voting::Status;

ISMTTLog * CalibrationDiagnosis::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Diagnosis.CalibrationDiagnosis", GETTEXT_PACKAGE );

CalibrationDiagnosis::CalibrationDiagnosis ( int numberPoints, bool skipConditional )
    : IDiagnosis ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION ),
      ICalibrationDiagnosis ( numberPoints, skipConditional )
{
    updateMessage();
}

CalibrationDiagnosis::~CalibrationDiagnosis()
{
}

void CalibrationDiagnosis::run()
{
    bool validate = calibrationTool ();
    setStatus ( validate ? VMStatus::VOTING_MACHINE_STATUS_OK : VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED );
}

void CalibrationDiagnosis::updateMessage()
{
    switch ( getStatus() )
    {
        default:
        case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
        case VMStatus::VOTING_MACHINE_STATUS_RUN:
            setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.CalibrationDiagnosis.InitializingDiagnostic" ) );
            break;

        case VMStatus::VOTING_MACHINE_STATUS_OK:
            setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.CalibrationDiagnosis.DiagnosticSuccessful" ) );
            break;

        case VMStatus::VOTING_MACHINE_STATUS_ERROR:
            setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.CalibrationDiagnosis.DiagnosticFailed" ) );
            break;
    }
}
