/**
 * @file ICalibrationDiagnosis.hxx
 * @brief Declaration of the ICalibrationDiagnosis class.
 * @author Yadickson Soto
 * @date 11/10/2012
 */

#ifndef ICALIBRATIONDIAGNOSIS_HXX_
#define ICALIBRATIONDIAGNOSIS_HXX_

#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Diagnosis
            {
                /**
                 * @class ICalibrationDiagnosis
                 * @brief Call a touchscreen calibration tool.
                 * @author Yadickson Soto
                 * @date 11/10/2012
                 */
                class ICalibrationDiagnosis
                {
                    public:

                        /**
                         * @fn ICalibrationDiagnosis::ICalibrationDiagnosis ( int numberPoints = 5, bool skipConditional = false );
                         * @brief Constructor for the ICalibrationDiagnosis class.
                         * @author Yadickson Soto
                         * @date 11/10/2013
                         * @param numberPoints Number of points.
                         * @param skipConditional Skip calibration if file "skip-calibration-tool" if present
                         */
                        ICalibrationDiagnosis ( int numberPoints = 5, bool skipConditional = false );

                        /**
                         * @fn virtual ICalibrationDiagnosis::~ICalibrationDiagnosis();
                         * @brief Finalizes an instance of the ICalibrationDiagnosis class.
                         * @author Yadickson Soto
                         * @date 11/10/2012
                         */
                        virtual ~ICalibrationDiagnosis();

                        /**
                         * @fn bool calibrationTool();
                         * @brief Method logic to run calibration tool
                         * @author Yadickson Soto
                         * @date 11/10/2012
                         */
                        bool calibrationTool();

                    protected:

                        int numberPoints; ///< Number of points
                        bool skipConditional; ///< Skip conditional flag

                        /**
                         * @fn virtual bool getPenmountFileConditional();
                         * @brief Method check penmount.dat file, not run calibration tool if file is present
                         * @author Yadickson Soto
                         * @date 11/10/2012
                         * @return true if file is present
                         */
                        virtual bool getPenmountFileConditional();

                        /**
                         * @fn virtual bool getSkipCalibrationToolConditional();
                         * @brief Method check skit-calibration-tool file, not run calibration tool if file is present
                         * @author Yadickson Soto
                         * @date 11/10/2012
                         * @return true if file is present
                         */
                        virtual bool getSkipCalibrationToolConditional();

                        /**
                         * @fn virtual bool executeCalibrationTool();
                         * @brief Call gCal calibration tool
                         * @author Yadickson Soto
                         * @date 11/10/2012
                         * @return true if calibration tool execute is successfully
                         */
                        virtual bool executeCalibrationTool();
                };
            }
        }
    }
}

#endif /* ICALIBRATIONDIAGNOSIS_HXX_ */
