/**
 * @file IMachineInitiator.cxx
 * @brief Implementation of the IMachineInitiator class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#include "IMachineInitiator.hxx"

using namespace Smartmatic::SAES::Operation::Initiator;

IMachineInitiator::IMachineInitiator()
{
    this->mode = FIRST_TIME;
    this->electoralConfiguration = NULL;

    signalSendProcessStatus.getSyncSignal().connect ( sigc::mem_fun ( *this, &IMachineInitiator::onRawObject ) );
}

IMachineInitiator::~IMachineInitiator()
{

}

void IMachineInitiator::addCondition ( ICheckCondition * condition )
{
    conditions.push_back ( condition );
}

void IMachineInitiator::init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration )
{
    this->electoralConfiguration = electoralConfiguration;
    mode = FIRST_TIME;
}

void IMachineInitiator::setMode ( RunModeType mode )
{
    this->mode = mode;
}

void IMachineInitiator::addMessage ( const InitializationStatus & status, bool finished )
{
    InitiatorSignal * initiator = new InitiatorSignal ( status, finished );
    getLogger()->Debug ( "Add message : " + initiator->getStatus().getMessage() + ( !finished ? " Continue" : " Finished" ) );
    signalSendProcessStatus.emit ( initiator );
}

void IMachineInitiator::onRawObject ( void * object )
{
    InitiatorSignal * initiator = static_cast < InitiatorSignal * > ( object );
    getLogger()->Debug ( "Process message : " + initiator->getStatus().getMessage() );
    signalDetected.emit ( initiator );
    delete initiator;
}

IMachineInitiator::SignalInitiatorProcess IMachineInitiator::getSignalDetected()
{
    return signalDetected;
}
