/**
 * @file LoginManager.hxx
 * @brief Header login manager class
 * @date 18/05/2013
 * @author Yadickson Soto
 */

#ifndef LOGINMANAGER_HXX_
#define LOGINMANAGER_HXX_

#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            /**
             * @class LoginManager
             * @brief login manager to getter password from opt parser or login key file
             * @date 18/05/2013
             * @author Yadickson Soto
             */
            class LoginManager
            {
                public:
                    /**
                     * @fn LoginManager();
                     * @brief Class constructor
                     * @date 18/05/2013
                     * @author Yadickson Soto
                     */
                    LoginManager();
                    /**
                     * @fn virtual ~LoginManager();
                     * @brief Class destructor
                     * @date 18/05/2013
                     * @author Yadickson Soto
                     */
                    virtual ~LoginManager();
                    /**
                     * @fn bool getPassword ( std::string & pass );
                     * @brief Getter method: password from opt parser or login key file, priority opt password
                     * @date 18/05/2013
                     * @author Yadickson Soto
                     * @param pass password string result
                     * @return true is password is found
                     */
                    bool getPassword ( std::string & pass );

                protected:

                    /**
                     * @fn virtual bool getOptPassword ( std::string & pass );
                     * @brief Getter method: password from opt parser
                     * @date 18/05/2013
                     * @author Yadickson Soto
                     * @param pass password string result
                     * @return true is password is found
                     */
                    virtual bool getOptPassword ( std::string & pass );
                    /**
                     * @fn virtual bool getFilePassword ( std::string & pass );
                     * @brief Getter method: password from login key file
                     * @date 18/05/2013
                     * @author Yadickson Soto
                     * @param pass password string result
                     * @return true is password is found
                     */
                    virtual bool getFilePassword ( std::string & pass );
            };
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* LOGINMANAGER_HXX_ */
