/*
 * MemoryEventManager.cxx
 *
 *  Created on: Jul 29, 2015
 *      Author: soto
 */

#include <Operation/MemoryEventManager.hxx>
#include <System/Sounds/smartmatic-sounds-manager.h>

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::System::Sounds;

MemoryEventManager * MemoryEventManager::instance = NULL;

MemoryEventManager::MemoryEventManager()
{
    flagEmitSignal = false;
    rawSignal.getSyncSignal().connect ( sigc::mem_fun ( *this, &MemoryEventManager::onRawObject ) );
}

MemoryEventManager::~MemoryEventManager()
{
    instance = NULL;
}

void MemoryEventManager::startEmitSignal()
{
    flagEmitSignal = true;
}

void MemoryEventManager::stopEmitSignal()
{
    flagEmitSignal = false;
}

bool MemoryEventManager::isCanEmitSignal ()
{
    return flagEmitSignal;
}

MemoryEventManager::MemorySignal MemoryEventManager::getMemorySignal()
{
    return memoryEventSignal;
}

MemoryEventManager * MemoryEventManager::getInstance()
{
    if ( instance == NULL )
    {
        instance = new MemoryEventManager();
    }

    return instance;
}

void MemoryEventManager::addMemoryEvent ( MemoryEventManager::MemoryEvent event )
{
    if ( event == ONE_MEMORY || event == INTERRUPTED ||
         event == INTERRUPTED_NO_MEMORY || event == CHECK_MEMORY_ERROR )
    {
        SMTTSoundsManager::getInstance()->Stop ();
        SMTTSoundsManager::getInstance()->Sound ( "SoundOneMemory" );
    }
    else if ( event == TWO_MEMORY || event == START_SYNCHRONIZE )
    {
        SMTTSoundsManager::getInstance()->Stop ();
    }

    if ( flagEmitSignal )
    {
        rawSignal.emit ( new MemoryEvent ( event ) );
    }

    if ( event == CHECK_MEMORY_ERROR )
    {
        flagEmitSignal = false;
    }
}

void MemoryEventManager::onRawObject ( void * object )
{
    MemoryEvent * event = static_cast < MemoryEvent * > ( object );
    getMemorySignal().emit ( *event );
    delete event;
}
