/*
 * AbledMonitor.hxx
 *
 *  Created on: Jan 13, 2018
 *      Author: eduardo.fernandez
 */

#ifndef ABLED_MONITOR_HXX
#define ABLED_MONITOR_HXX
#include <Operation/Monitor/DeviceTypeMonitor.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Monitor
            {
                /**
                 * @class AbledMonitor
                 * @brief Represents an abled device monitor.
                 * @author eduardo.fernandez
                 * @date Jan 13, 2018
                 */
                class AbledMonitor : public Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor
                {
                    public:

                        /**
                         * @fn AbledMonitor();
                         * @brief Default constructor for the AbledMonitor class.
                         * @author eduardo.fernandez
                         * @date Jan 13, 2018
                         */

                        AbledMonitor();

                        /**
                         * @fn virtual ~AbledMonitor();
                         * @brief Finalizes an instance of the AbledMonitor class.
                         * @author eduardo.fernandez
                         * @date Jan 13, 2018
                         */

                        virtual ~AbledMonitor();

                        /**
                         * @fn	virtual void added ( const std::string & node );
                         * @brief	Notifies that a Eurn device was added to the machine.
                         * @author	eduardo.fernandez
                         * @date Jan 13, 2018
                         * @param 	node event device path
                         */
                        virtual void added ( const std::string & node );

                        /**
                         * @fn	virtual void removed ( const std::string & node );
                         * @brief	Notifies that a Eurn device was removed from the machine.
                         * @author	eduardo.fernandez
                         * @date Jan 13, 2018
                         * @param 	node event device path
                         */
                        virtual void removed ( const std::string & node );

                    protected:

                        /**
                         * @fn	virtual std::string getDeviceMounted ( const std::string & node, const std::string & customNode );
                         * @brief	Gets when device mounted.
                         * @author	eduardo.fernandez
                         * @date Jan 13, 2018
                         * @param 	node the node to check
                         * @param	customNode the custom node to check device
                         * @return  path device mounted
                         */
                        virtual std::string getDeviceMounted ( const std::string & node, const std::string & customNode );

                    private:

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}




#endif /* ABLED_MONITOR_HXX */
