/**
 * @file BCRMonitor.cxx
 * @brief Implementation of the BCRMonitor class.
 * @date 12/12/2012
 * @author Yadickson Soto
 */

#include "BCRMonitor.hxx"
#include <Operation/Devices/BCRController.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <unistd.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;

ISMTTLog * BCRMonitor::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.BCRMonitor", GETTEXT_PACKAGE );

BCRMonitor::BCRMonitor()
    : DeviceTypeMonitor ( DeviceType::CONFIGURATION_BARCODE )
{

}

BCRMonitor::~BCRMonitor()
{

}

void BCRMonitor::added ( const std::string & node )
{
    logger->Debug ( "Added " + node );
    BCRController::getInstance()->open ( node );

    std::string outCommand = SAESFunctionalityConfigurationManager::getInstance()->getScanVoteConfiguration().getBarcodeOutParams();
    BCRController::getInstance()->sendReconfigureCommand(outCommand);
}

void BCRMonitor::removed ( const std::string & node )
{
    logger->Debug ( "Removed " + node );
    BCRController::getInstance()->close();
}

std::string BCRMonitor::getDeviceMounted ( const std::string & node, const std::string & customNode )
{
    usleep ( 250000 );
    return customNode;
}
