/**
 * @file DeviceMonitorSignal.hxx
 * @brief Header class to safe status, node and device for emit signal
 * @date 28/07/2015
 * @author Yadickson Soto
 */

#ifndef _DEVICEMONITORSIGNAL_HXX_
#define _DEVICEMONITORSIGNAL_HXX_

#include <Operation/Monitor/DeviceTypeMonitor.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Monitor
            {

                /**
                 * @class DeviceMonitorSignal
                 * @brief class to safe status, node and device for emit signal
                 * @date 28/07/2015
                 * @author Yadickson Soto
                 */
                class DeviceMonitorSignal
                {
                    public:
                        /**
                         * @fn DeviceMonitorSignal( bool status, const std::string & node, Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * device );
                         * @brief Class constructor
                         * @date 28/07/2015
                         * @author Yadickson Soto
                         * @param status The status to emit signal, true is added and false is removed
                         * @param node The string path
                         * @param device The device to change
                         */
                        DeviceMonitorSignal ( bool status,
                                              const std::string & node,
                                              Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * device );

                        /**
                         * @fn virtual ~DeviceMonitorSignal();
                         * @brief Class destroyer
                         * @date 28/07/2015
                         * @author Yadickson Soto
                         */
                        virtual ~DeviceMonitorSignal();

                        /**
                         * @fn bool isAdded();
                         * @brief Gets if device is added or removed
                         * @date 28/07/2015
                         * @author Yadickson Soto
                         * @return true is device is added
                         */
                        bool isAdded();

                        /**
                         * @fn std::string getNode();
                         * @brief Gets path node
                         * @date 28/07/2015
                         * @author Yadickson Soto
                         * @return path node
                         */
                        std::string getNode();

                        /**
                         * @fn Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * getDevice();
                         * @brief Gets device
                         * @date 28/07/2015
                         * @author Yadickson Soto
                         * @return device
                         */
                        Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * getDevice();

                    private:

                        bool status;	///< The status to emit signal, true is added and false is removed
                        std::string node;	///< The string path node
                        Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * device;	///< The device
                };

            } /* namespace Monitor */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* _DEVICEMONITORSIGNAL_HXX_ */
