/**
 * @file EthernetMonitor.cxx
 * @brief Declaration the Ethernet Monitor class.
 * @date 07/04/2014
 * @author Yadickson Soto
 */

#include "EthernetMonitor.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/File.h>
#include <System/Utils/SafeBufferUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

ISMTTLog * EthernetMonitor::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.EthernetMonitor", GETTEXT_PACKAGE );

EthernetMonitor::EthernetMonitor ()
    : DeviceTypeMonitor ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ETHERNET )
{
    countDevices = 0;
}

EthernetMonitor::~EthernetMonitor()
{

}

void EthernetMonitor::added ( const std::string & node )
{
    logger->Debug ( "Added " + node );
}

void EthernetMonitor::removed ( const std::string & node )
{
    logger->Debug ( "Removed " + node );
}

std::string EthernetMonitor::getDeviceMounted ( const std::string & node, const std::string & customNode )
{
    interface = node;
    file = customNode;
    return node;
}

bool EthernetMonitor::checkDeviceChanged ()
{
    SafeBuffer line;
    bool status = false;

    if ( File::getInstance()->openFileToBuffer ( file, line ) )
    {
        SafeBufferUtils buffUtil ( line );
        std::vector < SafeBuffer > result = buffUtil.splitByEndLine();

        if ( ! result.empty() )
        {
            status = result.front() == SafeBuffer ( "1" );
        }
    }

    if ( status && countDevices == 0 )
    {
        countDevices++;
    }
    else if ( !status && countDevices > 0 )
    {
        countDevices--;
    }

    return status;
}

unsigned int EthernetMonitor::getNumberOfDevices()
{
    return countDevices;
}

std::string EthernetMonitor::getInterface()
{
    return interface;
}
