/*
 * EurnMonitor.hxx
 *
 *  Created on: Oct 25, 2017
 *      Author: juanvalera
 */

#ifndef EURNMONITOR_HXX_
#define EURNMONITOR_HXX_
#include <Operation/Monitor/DeviceTypeMonitor.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Monitor
            {
                /**
                 * @class EurnMonitor
                 * @brief Represents a barcode reader device monitor.
                 * @date 25/10/2017
                 * @author Juan Valera
                 */
                class EurnMonitor : public Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor
                {
                    public:

                        /**
                         * @fn EurnMonitor();
                         * @brief Default constructor for the EurnMonitor class.
                         * @date 25/10/2017
                         * @author Juan Valera
                         */

                        EurnMonitor();

                        /**
                         * @fn virtual ~EurnMonitor();
                         * @brief Finalizes an instance of the EurnMonitor class.
                         * @date 25/10/2017
                         * @author Juan Valera
                         */

                        virtual ~EurnMonitor();

                        /**
                         * @fn	virtual void added ( const std::string & node );
                         * @brief	Notifies that a Eurn device was added to the machine.
                         * @author	Juan Valera
                         * @date 25/10/2017
                         * @param 	node event device path
                         */
                        virtual void added ( const std::string & node );

                        /**
                         * @fn	virtual void removed ( const std::string & node );
                         * @brief	Notifies that a Eurn device was removed from the machine.
                         * @author	Juan Valera
                         * @date 25/10/2017
                         * @param 	node event device path
                         */
                        virtual void removed ( const std::string & node );

                    protected:

                        /**
                         * @fn	virtual std::string getDeviceMounted ( const std::string & node, const std::string & customNode );
                         * @brief	Gets when device mounted.
                         * @author	Juan Valera
                         * @date 25/10/2017
                         * @param 	node the node to check
                         * @param	customNode the custom node to check device
                         * @return  path device mounted
                         */
                        virtual std::string getDeviceMounted ( const std::string & node, const std::string & customNode );

                    private:

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}




#endif /* EURNMONITOR_HXX_ */
