/**
 * @file MouseMonitor.hxx
 * @brief Declaration of the MouseMonitor class.
 * @date 12/12/2012
 * @author Yadickson Soto
 */

#ifndef MOUSEMONITOR_HXX_
#define MOUSEMONITOR_HXX_

#include <Operation/Monitor/DeviceTypeMonitor.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Monitor
            {

                /**
                 * @class MouseMonitor
                 * @brief  Represents a mouse device monitor.
                 * @date 12/12/2012
                 * @author Yadickson Soto
                 */

                class MouseMonitor : public Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor
                {
                    public:

                        /**
                         * @fn MouseMonitor();
                         * @brief Constructor for the MouseMonitor class.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         */

                        MouseMonitor();

                        /**
                         * @fn virtual ~MouseMonitor();
                         * @brief Finalizes an instance of the MouseMonitor class.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         */

                        virtual ~MouseMonitor();

                        /**
                         * @fn	virtual void added ( const std::string & node );
                         * @brief	Notifies that a mouse reader device was added to the machine.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @param 	node event device path
                         */
                        virtual void added ( const std::string & node );

                        /**
                         * @fn	virtual void removed ( const std::string & node );
                         * @brief	Notifies that a mouse reader device was removed from the machine.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @param 	node event device path
                         */
                        virtual void removed ( const std::string & node );

                    protected:

                        /**
                         * @fn	virtual std::string getDeviceMounted ( const std::string & node, const std::string & customNode );
                         * @brief	Gets when device mounted.
                         * @author	Yadickson Soto
                         * @date	09/07/2015
                         * @param 	node the node to check
                         * @param	customNode the custom node to check device
                         * @return  path device mounted
                         */
                        virtual std::string getDeviceMounted ( const std::string & node, const std::string & customNode );

                    private:

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif /* MOUSEMONITOR_HXX_ */
