/**
 * @file PrinterDiagnoseStatus.cxx
 * @brief Represents a printer device diagnostic status.
 * @date 08/01/2014
 * @author Yadickson Soto
 */

#include "PrinterDiagnoseStatus.hxx"

using namespace Smartmatic::SAES::Operation::Printer;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Printing;

PrinterDiagnoseStatus::PrinterDiagnoseStatus ( VotingMachineStatusWrapper * vmStatus )
    : vmStatus ( vmStatus )
{

}

PrinterDiagnoseStatus::~PrinterDiagnoseStatus()
{

}

void PrinterDiagnoseStatus::addPrinterStatusNotifier ( PrinterStatusNotifier::PhysicalStateEnumType status )
{
    clear();

    for ( unsigned int i = 0; i < 14 ; i++ )
    {
        switch ( ( PrinterStatusNotifier::PhysicalStateEnumType ) ( status & ( 1 << i ) ) )
        {
            default:
            case PrinterStatusNotifier::PSTATE_NONE:
            case PrinterStatusNotifier::PSTATE_UNKNOWN:
            case PrinterStatusNotifier::PSTATE_READY:
            case PrinterStatusNotifier::PSTATE_PROCESSING:
            case PrinterStatusNotifier::PSTATE_CANCELING:
            case PrinterStatusNotifier::PSTATE_REJECTING_JOBS:
                break;

            case PrinterStatusNotifier::PSTATE_OUT_OF_PAPER:
                vmStatus->addStatusPrinterNoPaper();
                break;

            case PrinterStatusNotifier::PSTATE_PAPER_JAM:
                vmStatus->addStatusPrinterPaperJam();
                break;

            case PrinterStatusNotifier::PSTATE_COVER_OPEN:
                vmStatus->addStatusPrinterCoverOpen();
                break;

            case PrinterStatusNotifier::PSTATE_FEEDER_OPEN:
                vmStatus->addStatusPrinterFeederOpen();
                break;

            case PrinterStatusNotifier::PSTATE_OFFLINE:
                vmStatus->setStatusPrinterNotDetected();
                break;

            case PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER:
                vmStatus->addStatusPrinterNearEndOfPaper();
                break;

            case PrinterStatusNotifier::PSTATE_SUPPLYLOW_WARN:
                vmStatus->addStatusPrinterSupplyLowWarn();
                break;

            case PrinterStatusNotifier::PSTATE_OUT_OF_TONER:
                vmStatus->addStatusPrinterNoToner();
                break;

            case PrinterStatusNotifier::PSTATE_CARTRIDGE_NOT_PRESENT:
                vmStatus->addStatusPrinterNoCartridge();
                break;
        }
    }
}

void PrinterDiagnoseStatus::addPrintDocumentResult ( PrintDocumentResult::PrintingResultEnumType status )
{
    clear();

    switch ( status )
    {
        default:
        case PrintDocumentResult::PRINTING_COMPLETED:
        case PrintDocumentResult::PRINTING_NONE:
        case PrintDocumentResult::PRINTING_NO_JOBS:
        case PrintDocumentResult::PRINTING_PENDING:
        case PrintDocumentResult::PRINTING_HELD:
        case PrintDocumentResult::PRINTING_PROCESSING:
        case PrintDocumentResult::PRINTING_PREPARING:
        case PrintDocumentResult::PRINTING_GENERATING_DATA:
        case PrintDocumentResult::PRINTING_SENDING_DATA:
        case PrintDocumentResult::PRINTING_JOB_NOT_SENT:
            break;

        case PrintDocumentResult::PRINTING_NO_PRINTER_FOUND:
            vmStatus->setStatusPrinterNotDetected();
            break;

        case PrintDocumentResult::PRINTING_STOPPED:
        case PrintDocumentResult::PRINTING_CANCELED:
        case PrintDocumentResult::PRINTING_ABORTED:
        case PrintDocumentResult::PRINTING_JOB_ABORTED_BY_SYSTEM:
            vmStatus->addStatusPrintingCanceled();
            break;

        case PrintDocumentResult::PRINTING_SERVER_ERROR:
        case PrintDocumentResult::PRINTING_INTERNAL_ERROR:
        case PrintDocumentResult::PRINTING_DEVICE_ERROR:
        case PrintDocumentResult::PRINTING_UNEXPECTED_ERROR:
        case PrintDocumentResult::PRINTING_PREPARING_ERROR:
            vmStatus->addStatusPrintingError();
            break;
    }
}

void PrinterDiagnoseStatus::clear()
{
    vmStatus->clearPrinterStatus();
}
