/**
 * @file CryptoSmartcard.cxx
 * @brief Implementation of the CryptoSmartcard class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#include "CryptoSmartcard.hxx"
#include <Security/KeyManager.hxx>
#include <System/guid-class.h>
#include <System/Security/Cryptography/PBEKeyDerivatorHMAC.h>
#include <System/Security/ComposeKey.hxx>

using namespace Smartmatic::SAES::Operation::Security;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Security;

CryptoSmartcard::CryptoSmartcard()
    : Smartmatic::SAES::Security::CryptoData()
{
}

CryptoSmartcard::~CryptoSmartcard()
{
}

void CryptoSmartcard::generatePin()
{
    SafeBuffer key;
    SafeBuffer pin;

    KeyManager::getInstance()->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, key );

    Smartmatic::System::Security::ComposeKey compose ( key );
    compose.getCompose ( PASSWORD_PIN_LEN, pin );

    setPin ( pin );
}

void CryptoSmartcard::generateKeys()
{
    PBEKeyDerivatorHMAC pbe;
    SafeBuffer passPhrase ( GuidClass::NewGuid().ToString() );
    SafeBuffer salt ( GuidClass::NewGuid().ToString() );
    SafeBuffer globalKey;

    pbe.SetSalt ( salt );
    pbe.GenerateKey ( passPhrase, globalKey, MsgDigestUtil::getDefaultGenerateKeyDigest() );

    SafeBuffer key;

    Smartmatic::System::Security::ComposeKey compose ( globalKey );
    compose.getCompose ( PASSWORD_FILE_KEY, key );

    setKey ( key );
}
