/**
 * @file   TextToSpeechFactory.hxx
 * @brief  Declaration of TextToSpeechFactory class.
 * @author manuel.arguelles
 * @date   Oct 11, 2017
 */

#ifndef TEXT_TO_SPEECH_FACTORY_HXX
#define TEXT_TO_SPEECH_FACTORY_HXX

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                class ITextToSpeech;
            }
        }
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace AudioVoting
            {

                /**
                 * @class  TextToSpeechFactory
                 * @brief  Factory class to create TextToSpeech objects.
                 * @author manuel.arguelles
                 * @date   Oct 11, 2017
                 */
                class TextToSpeechFactory
                {
                    public:

                        /**
                         * @brief  Creates a TextToSpeech object.
                         * @author manuel.arguelles
                         * @date   Oct 11, 2017
                         * @return A newly allocated TextToSpeech object.
                         */
                        static System::Audio::TextToSpeech::ITextToSpeech* createTextToSpeech();

                    private:

                        /**
                         * @brief  Disable instances of the class.
                         * @author manuel.arguelles
                         * @date   Oct 11, 2017
                         */
                        TextToSpeechFactory();

                    private:

                        static Smartmatic::Log::ISMTTLog* logger;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* TEXT_TO_SPEECH_FACTORY_HXX */

