/*
 * AfterScanningVoteComplianceVerificator.cxx
 *
 *  Created on: Feb 09, 2018
 *      Author: eduardo.fernandez
 */

#include "AfterScanningVoteComplianceVerificator.hxx"

#include "VoteComplianceVerificationManager.hxx"
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <Voting/ElectoralConfiguration.hxx>
#include <Resources/election-string-resources-manager.h>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/Election/election-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::VoteCompliance;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Log;

ISMTTLog * AfterScanningVoteComplianceVerificator::m_logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Voting.AfterScanningVoteComplianceVerificator", GETTEXT_PACKAGE );

AfterScanningVoteComplianceVerificator::AfterScanningVoteComplianceVerificator ( ElectoralConfiguration *electoralConfiguration )
: m_voteComplianceVerificationManager(new VoteComplianceVerificationManager(electoralConfiguration))
{
}

AfterScanningVoteComplianceVerificator::~AfterScanningVoteComplianceVerificator()
{
}

bool AfterScanningVoteComplianceVerificator::checkMinimumNumberOfScannings(const Vote::Vote &scannedVote)
{
    bool success = true;

    if (SAESFunctionalityConfigurationManager::getInstance()->getAfterScanningVoteComplianceVerificationConfiguration().getCheckMinimumNumberOfScannings())
    {
        success = m_voteComplianceVerificationManager->checkMinimumNumberOfSelections(scannedVote);
    }

    return success;
}

bool AfterScanningVoteComplianceVerificator::checkScanningsBelongToSameParty(const Vote::Vote &scannedVote)
{
    bool success = true;

    if (SAESFunctionalityConfigurationManager::getInstance()->getAfterScanningVoteComplianceVerificationConfiguration().getCheckScanningsBelongToSameParty())
    {
        success = m_voteComplianceVerificationManager->checkSelectionsBelongToSameParty(scannedVote);
    }

    return success;
}

bool AfterScanningVoteComplianceVerificator::checkScanningsApplicableToConstituency(const Vote::Vote &scannedVote)
{
    bool success = true;

    if (SAESFunctionalityConfigurationManager::getInstance()->getAfterScanningVoteComplianceVerificationConfiguration().getCheckScanningsApplicableToConstituency())
    {
        success = m_voteComplianceVerificationManager->checkSelectionsApplicableToConstituency(scannedVote);
    }

    return success;
}

bool AfterScanningVoteComplianceVerificator::checkUniquenessInScannings(const Vote::Vote &scannedVote)
{
    bool success = true;

    if (SAESFunctionalityConfigurationManager::getInstance()->getAfterScanningVoteComplianceVerificationConfiguration().getCheckUniquenessInScannings())
    {
        success = m_voteComplianceVerificationManager->checkUniquenessInSelections(scannedVote);
    }

    return success;
}

bool AfterScanningVoteComplianceVerificator::checkCumulativeNumberOfScannings(const Vote::Vote &scannedVote)
{
    bool success = true;

    if (SAESFunctionalityConfigurationManager::getInstance()->getAfterScanningVoteComplianceVerificationConfiguration().getCheckCumulativeNumberOfScannings())
    {
        success = m_voteComplianceVerificationManager->checkCumulativeNumberOfSelections(scannedVote);
    }

    return success;
}
