/*
 * VoteFromSafeBuffer.hxx
 *
 *  Created on: Aug 5, 2015
 *      Author: soto
 */

#ifndef _VOTING_VOTEFROMSAFEBUFFER_HXX_
#define _VOTING_VOTEFROMSAFEBUFFER_HXX_

#include <System/SafeBuffer.h>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Voting
            {

                class VoteFromSafeBuffer
                {
                    public:
                        /**
                         * @enum	VoteFromSafeBufferResult
                         * @brief	Values that represent VoteFromSafeBufferResult.
                         */
                        enum VoteFromSafeBufferResult
                        {
                            VOTE_FROM_BUFFER_SUCCESS,
                            VOTE_FROM_BUFFER_DECOMPRESSION_ERROR,
                            VOTE_FROM_BUFFER_DECRYPTO_ERROR,
                            VOTE_FROM_BUFFER_SIGNATURE_ERROR,
                            VOTE_FROM_BUFFER_SIGNATURE_TRUST_ERROR,
                            VOTE_FROM_BUFFER_INVALID_VOTING_DEVICE,
                            VOTE_FROM_BUFFER_INVALID_LVG,
                            VOTE_FROM_BUFFER_INVALID_PROCESS_INSTANCE,
                            VOTE_FROM_BUFFER_UNEXPECTED_ERROR
                        };

                        explicit VoteFromSafeBuffer ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                        virtual ~VoteFromSafeBuffer();


                        /**
                         * @fn	VoteFromSafeBufferResult getVoteFromSafeBuffer ( Smartmatic::System::SafeBuffer * safeBuffer,
                         * 			Smartmatic::SAES::Voting::Vote::Vote *& vote );
                         * @brief	creates a vote from a SafeBuffer.
                         * @author	Yadickson Soto
                         * @date	05/08/2015
                         * @param	safeBuffer	from which to  extract vote, vote by reference to store data.
                         * @param	vote		The vote result.
                         * @return	The vote status.
                         */

                        VoteFromSafeBufferResult getVote (
                            Smartmatic::System::SafeBuffer * safeBuffer,
                            Smartmatic::SAES::Voting::Vote::Vote *& vote );

                    private:

                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;  ///< instance of object electoral configuration
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };

            } /* namespace Voting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* _VOTING_VOTEFROMSAFEBUFFER_HXX_ */
