/**
 * @file	VotingMemento.hxx
 * @brief	voting memento class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#ifndef __VOTING_MEMENTO_H__
#define __VOTING_MEMENTO_H__

#include "VotingState.hxx"

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Voting
            {
                /**
                 * @class	VotingMemento
                 * @brief	Voting memento.
                 * @author	Yadickson Soto
                 * @date	07-03-2011
                 */
                class VotingMemento
                {
                    private:
                        VotingState * state;	///< The voting state
                        int position; ///< position in creation

                    public:

                        /**
                         * @fn	VotingMemento::VotingMemento ( VotingState * state , int position );
                         * @brief	class constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	state			If non-null, the state.
                         * @param	position	The position.
                         */

                        VotingMemento ( VotingState * state , int position );

                        /**
                         * @fn	virtual VotingMemento::~VotingMemento ();
                         * @brief	Finaliser.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */

                        virtual ~VotingMemento ();

                        /**
                         * @fn	VotingState * VotingMemento::getState();
                         * @brief	gets the state of memento.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	state of memento.
                         */

                        VotingState * getState();

                        /**
                         * @fn	int VotingMemento::getPosition();
                         * @brief	gets the position of memento.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	position of memento.
                         */

                        int getPosition();
                };
            }
        }
    }
}

#endif /*__VOTING_MEMENTO_H__*/
