/**
 * @file	VotingOriginator.hxx
 * @brief	voting originator class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#ifndef __VOTING_ORIGINATOR_H__
#define __VOTING_ORIGINATOR_H__

#include "VotingState.hxx"
#include "VotingMemento.hxx"

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Voting
            {
                /**
                 * @class	VotingOriginator
                 * @brief	Voting originator.
                 * @author	Yadickson Soto
                 * @date	07-03-2011
                 */

                class VotingOriginator
                {
                    private:
                        VotingState * state;	///< The voting state
                        static int count; ///< count creation

                    public:

                        /**
                         * @fn	VotingOriginator::VotingOriginator();
                         * @brief	Class construnctor.
                         * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date	31/10/2012
                         */
                        VotingOriginator();

                        /**
                         * @fn	VotingMemento * VotingOriginator::createMemento();
                         * @brief	create memento.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	new memento.
                         */

                        VotingMemento * createMemento();

                        /**
                         * @fn	void VotingOriginator::setState ( VotingState * state );
                         * @brief	restore	state from new memento.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	state	new state memento.
                         */

                        void setState ( VotingState * state );
                };
            }
        }
    }
}

#endif /*__VOTING_ORIGINATOR_H__*/
