#ifndef TEST_PRINTER_CONTROLLER_CONF_MANAGER_TEST
#define TEST_PRINTER_CONTROLLER_CONF_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockPrinterControllerConfigurationManager.hxx>
#include <tests/MockSaesFunctionalityConfigurationManager.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>

class PrinterControllerConfigurationManagerTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::SAES::Functionality::Configuration::PrinterControllerConfigurationManager PrinterControllerConfigurationManager;
        typedef Smartmatic::SAES::Functionality::Configuration::SAESFunctionalityConfigurationManager SAESFunctionalityConfigurationManager;
        typedef Smartmatic::SAES::Functionality::Configuration::PrinterControllerConfiguration PrinterControllerConfiguration;
        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;

        PrinterControllerConfigurationManagerTest()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testGetPrinterControllerConfigurationPresent()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesFunctionalityConfigurationManager configurationManager;
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            CPPUNIT_ASSERT ( true == SAESFunctionalityConfigurationManager::getInstance()->getConfiguration()->getPrinterControllerConfiguration().present() );

            persistenceManager.verify();
        }

        void testGetPrinterControllerConfigurationDefault()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesFunctionalityConfigurationManager configurationManager;
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            SAESFunctionalityConfigurationManager::getInstance()->getConfiguration()->getPrinterControllerConfiguration().reset();

            PrinterControllerConfigurationManager * printerControllerManager = PrinterControllerConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( false == printerControllerManager->shouldSkipPrinter() );
            CPPUNIT_ASSERT ( 2 == printerControllerManager->getTimeoutPrint() );
            CPPUNIT_ASSERT ( true == printerControllerManager->getResultPrintFile().empty() );
            CPPUNIT_ASSERT ( true == printerControllerManager->getStatusPrintFile().empty() );

            delete printerControllerManager;

            persistenceManager.verify();
        }

        void testGetSkipPrinter()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesFunctionalityConfigurationManager configurationManager;
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            PrinterControllerConfiguration & config ( SAESFunctionalityConfigurationManager::getInstance()->getConfiguration()->getPrinterControllerConfiguration().get() );

            PrinterControllerConfigurationManager * printerControllerManager = PrinterControllerConfigurationManager::getInstance();

            config.setSkipPrinter ( true );
            CPPUNIT_ASSERT ( true == printerControllerManager->shouldSkipPrinter() );

            config.setSkipPrinter ( false );
            CPPUNIT_ASSERT ( false == printerControllerManager->shouldSkipPrinter() );

            delete printerControllerManager;

            persistenceManager.verify();
        }

        void testGetTimeoutPrinter()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesFunctionalityConfigurationManager configurationManager;
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            PrinterControllerConfiguration & config ( SAESFunctionalityConfigurationManager::getInstance()->getConfiguration()->getPrinterControllerConfiguration().get() );

            PrinterControllerConfigurationManager * printerControllerManager = PrinterControllerConfigurationManager::getInstance();

            config.setTimeoutPrint ( 0 );
            CPPUNIT_ASSERT ( 0 == printerControllerManager->getTimeoutPrint() );

            config.setTimeoutPrint ( 1 );
            CPPUNIT_ASSERT ( 1 == printerControllerManager->getTimeoutPrint() );

            delete printerControllerManager;

            persistenceManager.verify();
        }

        void testGetResultPrinter()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesFunctionalityConfigurationManager configurationManager;
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            PrinterControllerConfiguration & config ( SAESFunctionalityConfigurationManager::getInstance()->getConfiguration()->getPrinterControllerConfiguration().get() );

            PrinterControllerConfigurationManager * printerControllerManager = PrinterControllerConfigurationManager::getInstance();

            config.setResultPrintFile ( "Result1" );
            CPPUNIT_ASSERT ( "Result1" == printerControllerManager->getResultPrintFile() );

            config.setResultPrintFile ( "Result2" );
            CPPUNIT_ASSERT ( "Result2" == printerControllerManager->getResultPrintFile() );

            delete printerControllerManager;

            persistenceManager.verify();
        }

        void testGetStatusPrinter()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSaesFunctionalityConfigurationManager configurationManager;
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            PrinterControllerConfiguration & config ( SAESFunctionalityConfigurationManager::getInstance()->getConfiguration()->getPrinterControllerConfiguration().get() );

            PrinterControllerConfigurationManager * printerControllerManager = PrinterControllerConfigurationManager::getInstance();

            config.setStatusPrintFile ( "Status1" );
            CPPUNIT_ASSERT ( "Status1" == printerControllerManager->getStatusPrintFile() );

            config.setStatusPrintFile ( "Status2" );
            CPPUNIT_ASSERT ( "Status2" == printerControllerManager->getStatusPrintFile() );

            delete printerControllerManager;

            persistenceManager.verify();
        }

        void testMockGetSkipPrinter()
        {
            MockPrinterControllerConfigurationManager printerControllerManager;
            PrinterControllerConfigurationManager::getInstance ( &printerControllerManager );

            MOCKPP_CHAINER_FOR ( MockPrinterControllerConfigurationManager, shouldSkipPrinter ) shouldSkipPrinter_mock ( &printerControllerManager );

            shouldSkipPrinter_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            CPPUNIT_ASSERT ( false == printerControllerManager.shouldSkipPrinter() );
            CPPUNIT_ASSERT ( true == printerControllerManager.shouldSkipPrinter() );
        }

        void testMockGetTimeoutPrinter()
        {
            MockPrinterControllerConfigurationManager printerControllerManager;
            PrinterControllerConfigurationManager::getInstance ( &printerControllerManager );

            MOCKPP_CHAINER_FOR ( MockPrinterControllerConfigurationManager, getTimeoutPrint ) getTimeoutPrint_mock ( &printerControllerManager );

            getTimeoutPrint_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <unsigned int> ( 0 ),
                        MOCKPP_NS::returnValue <unsigned int> ( 1 ) ) );

            CPPUNIT_ASSERT ( 0 == printerControllerManager.getTimeoutPrint() );
            CPPUNIT_ASSERT ( 1 == printerControllerManager.getTimeoutPrint() );
        }

        void testMockGetResultPrinter()
        {
            MockPrinterControllerConfigurationManager printerControllerManager;
            PrinterControllerConfigurationManager::getInstance ( &printerControllerManager );

            MOCKPP_CHAINER_FOR ( MockPrinterControllerConfigurationManager, getResultPrintFile ) getResultPrintFile_mock ( &printerControllerManager );

            getResultPrintFile_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<std::string> ( "Result1" ),
                        new MOCKPP_NS::ReturnStub<std::string> ( "Result2" ) ) );

            CPPUNIT_ASSERT ( "Result1" == printerControllerManager.getResultPrintFile() );
            CPPUNIT_ASSERT ( "Result2" == printerControllerManager.getResultPrintFile() );
        }

        void testMockGetStatusPrinter()
        {
            MockPrinterControllerConfigurationManager printerControllerManager;
            PrinterControllerConfigurationManager::getInstance ( &printerControllerManager );

            MOCKPP_CHAINER_FOR ( MockPrinterControllerConfigurationManager, getStatusPrintFile ) getStatusPrintFile_mock ( &printerControllerManager );

            getStatusPrintFile_mock
            .stubs()
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<std::string> ( "Status1" ),
                        new MOCKPP_NS::ReturnStub<std::string> ( "Status2" ) ) );

            CPPUNIT_ASSERT ( "Status1" == printerControllerManager.getStatusPrintFile() );
            CPPUNIT_ASSERT ( "Status2" == printerControllerManager.getStatusPrintFile() );
        }

        CPPUNIT_TEST_SUITE ( PrinterControllerConfigurationManagerTest );

        CPPUNIT_TEST ( testGetPrinterControllerConfigurationPresent );
        CPPUNIT_TEST ( testGetPrinterControllerConfigurationDefault );

        CPPUNIT_TEST ( testGetSkipPrinter );
        CPPUNIT_TEST ( testGetTimeoutPrinter );
        CPPUNIT_TEST ( testGetResultPrinter );
        CPPUNIT_TEST ( testGetStatusPrinter );

        CPPUNIT_TEST ( testMockGetSkipPrinter );
        CPPUNIT_TEST ( testMockGetTimeoutPrinter );
        CPPUNIT_TEST ( testMockGetResultPrinter );
        CPPUNIT_TEST ( testMockGetStatusPrinter );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_PRINTER_CONTROLLER_CONF_MANAGER_TEST */
