#ifndef TEST_PRINTER_DIAGNOSE_STATUS_TEST
#define TEST_PRINTER_DIAGNOSE_STATUS_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Operation/Printer/PrinterDiagnoseStatus.hxx>

class PrinterDiagnoseStatusTest : public CPPUNIT_NS::TestFixture
{

    public:

        typedef Smartmatic::SAES::Operation::Printer::PrinterDiagnoseStatus PrinterDiagnoseStatus;
        typedef Smartmatic::SAES::Voting::Status::VotingMachineStatusWrapper VotingMachineStatusWrapper;
        typedef Smartmatic::SAES::Printing::PrinterStatusNotifier PrinterStatusNotifier;
        typedef Smartmatic::SAES::Printing::PrintDocumentResult PrintDocumentResult;

        PrinterDiagnoseStatus * printerDiagnoseStatus;
        VotingMachineStatusWrapper * vmStatus;

        PrinterDiagnoseStatusTest ()
        {
            printerDiagnoseStatus = NULL;
            vmStatus = NULL;
        }

        void setUp()
        {
            const std::list<std::string> runFunctionalityDescriptors;
            vmStatus = new VotingMachineStatusWrapper ( runFunctionalityDescriptors );
            printerDiagnoseStatus = new PrinterDiagnoseStatus ( vmStatus );
        }

        void tearDown()
        {
            if ( printerDiagnoseStatus )
            {
                delete printerDiagnoseStatus;
                printerDiagnoseStatus = NULL;
            }

            if ( vmStatus )
            {
                delete vmStatus;
                vmStatus = NULL;
            }
        }

        void testPrinterNoDetected_PrinterStatusNotifier()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-ND,";

            status = PrinterStatusNotifier::PSTATE_OFFLINE;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterNoDetected_PrintDocument()
        {
            std::string result;
            std::stringstream expected;
            PrintDocumentResult::PrintingResultEnumType status;

            expected << "P:NOK-ND,";

            status = PrintDocumentResult::PRINTING_NO_PRINTER_FOUND;

            printerDiagnoseStatus->addPrintDocumentResult ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterPaperJam()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-PJ,";

            status = PrinterStatusNotifier::PSTATE_PAPER_JAM;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterNoPaper()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-NP,";

            status = PrinterStatusNotifier::PSTATE_OUT_OF_PAPER;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterCoverOpen()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-CO,";

            status = PrinterStatusNotifier::PSTATE_COVER_OPEN;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterFeederOpen()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-FO,";

            status = PrinterStatusNotifier::PSTATE_FEEDER_OPEN;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterNearEndOfPaper()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-NE,";

            status = PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterSupplyLow()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-SL,";

            status = PrinterStatusNotifier::PSTATE_SUPPLYLOW_WARN;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterNoToner()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-NT,";

            status = PrinterStatusNotifier::PSTATE_OUT_OF_TONER;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterNoCartridge()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-NC,";

            status = PrinterStatusNotifier::PSTATE_CARTRIDGE_NOT_PRESENT;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterPrintingCanceled()
        {
            std::string result;
            std::stringstream expected;
            PrintDocumentResult::PrintingResultEnumType status;

            expected << "P:NOK-PC,";

            status = PrintDocumentResult::PRINTING_CANCELED;

            printerDiagnoseStatus->addPrintDocumentResult ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterPrintingError()
        {
            std::string result;
            std::stringstream expected;
            PrintDocumentResult::PrintingResultEnumType status;

            expected << "P:NOK-PE,";

            status = PrintDocumentResult::PRINTING_DEVICE_ERROR;

            printerDiagnoseStatus->addPrintDocumentResult ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterNearEndOfPaperAndFeederOpen()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:NOK-FO-NE,";

            status = ( PrinterStatusNotifier::PhysicalStateEnumType ) ( PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER | PrinterStatusNotifier::PSTATE_FEEDER_OPEN );

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterNoneError_PrinterStatusNotifier()
        {
            std::string result;
            std::stringstream expected;
            PrinterStatusNotifier::PhysicalStateEnumType status;

            expected << "P:N/A,";

            status = PrinterStatusNotifier::PSTATE_NONE;

            printerDiagnoseStatus->addPrinterStatusNotifier ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK", "N/A" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testPrinterNoneError_PrintDocument()
        {
            std::string result;
            std::stringstream expected;
            PrintDocumentResult::PrintingResultEnumType status;

            expected << "P:N/A,";

            status = PrintDocumentResult::PRINTING_NONE;

            printerDiagnoseStatus->addPrintDocumentResult ( status );

            result = vmStatus->getResult ( ",", ":", "OK", "NOK", "N/A" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        CPPUNIT_TEST_SUITE ( PrinterDiagnoseStatusTest );

        CPPUNIT_TEST ( testPrinterNoDetected_PrinterStatusNotifier );
        CPPUNIT_TEST ( testPrinterNoDetected_PrintDocument );
        CPPUNIT_TEST ( testPrinterPaperJam );
        CPPUNIT_TEST ( testPrinterNoPaper );
        CPPUNIT_TEST ( testPrinterNoPaper );
        CPPUNIT_TEST ( testPrinterCoverOpen );
        CPPUNIT_TEST ( testPrinterFeederOpen );
        CPPUNIT_TEST ( testPrinterNearEndOfPaper );
        CPPUNIT_TEST ( testPrinterSupplyLow );
        CPPUNIT_TEST ( testPrinterNoToner );
        CPPUNIT_TEST ( testPrinterNoCartridge );
        CPPUNIT_TEST ( testPrinterPrintingCanceled );
        CPPUNIT_TEST ( testPrinterPrintingError );
        CPPUNIT_TEST ( testPrinterNearEndOfPaperAndFeederOpen );
        CPPUNIT_TEST ( testPrinterNoneError_PrinterStatusNotifier );
        CPPUNIT_TEST ( testPrinterNoneError_PrintDocument );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_PRINTER_DIAGNOSE_STATUS_TEST */
