#ifndef BEFORE_GENERATING_RESULTS_VERIFICATION_TEST
#define BEFORE_GENERATING_RESULTS_VERIFICATION_TEST

#ifdef ECUT_MAIN

#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <cppunit/extensions/HelperMacros.h>
#include <Operation/Voting/VoteCompliance/VoteComplianceVerificator.hxx>
#include <tests/MockSaesFunctionalityConfigurationManager.hxx>
#include <tests/MockVoteStatistics.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Eml/510.hxx>
#include <Voting/PBReports/vote-results-schema.hxx>
#include <Voting/PBReports/VoteCounterByContest.hxx>
#include <Voting/Vote/vote-schema.hxx>

using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::VoteCompliance;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting::Eml;
using namespace Smartmatic::System::Utils;

static const std::string CONTEST_CODE                                 = "147";
static const std::string PARTY_CODE                                   = "1712";

static const std::string PRINCIPAL_VOTE_BALLOT_OPTION_CODE            = "48359";
static const std::string STAND_IN_VOTE_BALLOT_OPTION_CODE             = "48364";

static const size_t      SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES = 3;
static const size_t      SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES  = 3;

static const size_t      FAILURE_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES = 6;
static const size_t      FAILURE_STAND_IN_CANDIDATE_PREFERENCE_VOTES  = 6;

static const size_t      SUCCESS_PARTY_TOTAL_VOTES                    = 3;
static const size_t      FAILURE_PARTY_TOTAL_VOTES                    = 6;

static const size_t      SUCCESS_PARTY_LIST_VOTES                     = 3;
static const size_t      FAILURE_PARTY_LIST_VOTES                     = 6;

static const size_t      TOTAL_VOTES                                  = 5;

static const std::string EML_FILE_NAME                                = "eml-file-name";

static const std::string EUROPEAN_VOTERS_IDENTIFIER                   = "E_EuropeanVoters";
static const std::string NATIONAL_VOTERS_IDENTIFIER                   = "N_NationalVoters";
static const std::string NATIONAL_VOTERS_ABROAD_IDENTIFIER            = "S_NationalVotersAbroad";
static const std::string TOTAL_VOTERS_IDENTIFIER                      = "T_TotalVoters";

static const size_t      BLANK_AND_INVALID_VOTES                      = 300;
static const std::string DEFAULT_CUSTOM_CODE                          = "BR210181";
static const std::string SECOND_CUSTOM_CODE                           = "BR210182";
static const std::string THIRD_CUSTOM_CODE                            = "BR210183";
static const size_t      DEFAULT_VALID_VOTES                          = 800;
static const size_t      DEFAULT_VOTES_COUNTER                        = 700;
static const size_t      EUROPEAN_VOTERS                              = 450;
static const size_t      INCORRECT_TOTAL_VOTERS                       = 325;
static const size_t      INCORRECT_VOTES_DEPOSITED                    = 1400;
static const size_t      NATIONAL_VOTERS                              = 550;
static const size_t      NATIONAL_VOTERS_ABROAD                       = 600;
static const size_t      SECOND_VALID_VOTES                           = 400;
static const size_t      SECOND_VOTES_COUNTER                         = 500;
static const size_t      THIRD_VALID_VOTES                            = 100;
static const size_t      TOTAL_VOTERS                                 = 1600;
static const size_t      THIRD_VOTES_COUNTER                          = 400;
static const size_t      VOTES_DEPOSITED                              = 1600;

static const std::string BLANK_AND_INVALID_VOTES_METRIC_IDENTIFIER    = "BlankAndInvalidVotes";
static const std::string VOTES_DEPOSITED_METRIC_IDENTIFIER            = "VotesDeposited";
static const std::string DEFAULT_CATEGORY_IDENTIFIER                  = "R210181";
static const std::string SECOND_CATEGORY_IDENTIFIER                   = "R210182";
static const std::string THIRD_CATEGORY_IDENTIFIER                    = "R210183";
static const std::string DEFAULT_ELECTION_CATEGORY                    = "BR";

class BeforeGeneratingResultsVerificationTest : public CPPUNIT_NS::TestFixture
{
    public:

        BeforeGeneratingResultsVerificationTest ()
        {

        }

        void setUp()
        {
            MockSaesFunctionalityConfigurationManager configurationManager;
            configurationManager.initialize();
            SAESFunctionalityConfigurationManager::getInstance ( &configurationManager );

            m_candidatesSequence.clear();
            m_partiesSequence.clear();
            m_contestsSequence.clear();

            m_electionSequence.clear();
            m_electronicVotingCountersSequence.clear();

            m_partyResults.setCode(PARTY_CODE);

            m_contestResults.setCode(CONTEST_CODE);
        }

        void tearDown()
        {
            VoteComplianceVerificator::getInstance()->dispose();
        }

        void checkTotalCumulativeSelections_partyVotesExceedTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setTotal_votes(FAILURE_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkTotalCumulativeSelections(TOTAL_VOTES, m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkTotalCumulativeSelections_principalCandidateVotesExceedTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, FAILURE_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkTotalCumulativeSelections(TOTAL_VOTES, m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkTotalCumulativeSelections_standInCandidatesVotesExceedTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, FAILURE_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkTotalCumulativeSelections(TOTAL_VOTES, m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkTotalCumulativeSelections_categorizedVotesDoNotExceedTotalVotes_validVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkTotalCumulativeSelections(TOTAL_VOTES, m_voteResults);

            CPPUNIT_ASSERT(check);
        }

        void checkIndividualCumulativeSelections_partyListVotesExceedPartyTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, FAILURE_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setList_votes(FAILURE_PARTY_LIST_VOTES);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkIndividualCumulativeSelections(m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkIndividualCumulativeSelections_principalCandidateVotesExceedPartyTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, FAILURE_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setList_votes(SUCCESS_PARTY_LIST_VOTES);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkIndividualCumulativeSelections(m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkIndividualCumulativeSelections_standInCandidateVotesExceedPartyTotalVotes_invalidVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setList_votes(SUCCESS_PARTY_LIST_VOTES);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, FAILURE_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkIndividualCumulativeSelections(m_voteResults);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkIndividualCumulativeSelections_categorizedVotesDoNotExceedPartyTotalVotes_validVoteResults()
        {
            m_candidateResult = buildItemResult(PRINCIPAL_VOTE_BALLOT_OPTION_CODE, SUCCESS_PRINCIPAL_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setList_votes(SUCCESS_PARTY_LIST_VOTES);
            m_partyResults.setTotal_votes(SUCCESS_PARTY_TOTAL_VOTES);
            m_partyResults.setCandidateResults(m_itemsResults);

            m_candidatesSequence.clear();
            m_candidateResult = buildItemResult(STAND_IN_VOTE_BALLOT_OPTION_CODE, SUCCESS_STAND_IN_CANDIDATE_PREFERENCE_VOTES);
            m_candidatesSequence.push_back(m_candidateResult);
            m_itemsResults.setCandidate(m_candidatesSequence);
            m_partyResults.setSubstituteResults(m_itemsResults);

            m_partiesSequence.push_back(m_partyResults);
            m_contestResults.setPartyResults(m_partiesSequence);

            m_contestsSequence.push_back(m_contestResults);
            m_voteResults.setContestResults(m_contestsSequence);

            bool check = VoteComplianceVerificator::getInstance()->checkIndividualCumulativeSelections(m_voteResults);

            CPPUNIT_ASSERT(check);
        }

        void checkEmlVotesDeposited_votesDepositedDoNotMatchSumOfValidAndInvalidVotes_failedValidation()
        {
            Selection::CountMetricType metricVotesDeposited(VOTES_DEPOSITED, VOTES_DEPOSITED_METRIC_IDENTIFIER);
            Selection::CountMetricType metricBlankAndInvalidVotes(BLANK_AND_INVALID_VOTES, BLANK_AND_INVALID_VOTES_METRIC_IDENTIFIER);

            Selection::CountMetricSequence metricSequence;
            metricSequence.push_back(metricVotesDeposited);
            metricSequence.push_back(metricBlankAndInvalidVotes);

            TotalVotes::SelectionSequence selectionSequence;
            selectionSequence.push_back(buildSelection(DEFAULT_CATEGORY_IDENTIFIER, metricSequence, DEFAULT_VALID_VOTES));
            selectionSequence.push_back(buildSelection(SECOND_CATEGORY_IDENTIFIER, metricSequence, SECOND_VALID_VOTES));

            m_electionSequence = buildElectionSequence(selectionSequence);

            Count count;
            count.setElection(m_electionSequence);

            EML eml;
            eml.setCount(count);

            std::map<std::string, EML> plainEmlFiles;
            plainEmlFiles[EML_FILE_NAME] = eml;

            bool check = VoteComplianceVerificator::getInstance()->checkEmlVotesDeposited(plainEmlFiles);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkEmlVotesDeposited_votesDepositedMatchSumOfValidAndInvalidVotes_successfulValidation()
        {
            Selection::CountMetricType metricVotesDeposited(VOTES_DEPOSITED, VOTES_DEPOSITED_METRIC_IDENTIFIER);
            Selection::CountMetricType metricBlankAndInvalidVotes(BLANK_AND_INVALID_VOTES, BLANK_AND_INVALID_VOTES_METRIC_IDENTIFIER);

            Selection::CountMetricSequence metricSequence;
            metricSequence.push_back(metricVotesDeposited);
            metricSequence.push_back(metricBlankAndInvalidVotes);

            TotalVotes::SelectionSequence selectionSequence;
            selectionSequence.push_back(buildSelection(DEFAULT_CATEGORY_IDENTIFIER, metricSequence, DEFAULT_VALID_VOTES));
            selectionSequence.push_back(buildSelection(SECOND_CATEGORY_IDENTIFIER, metricSequence, SECOND_VALID_VOTES));
            selectionSequence.push_back(buildSelection(THIRD_CATEGORY_IDENTIFIER, metricSequence, THIRD_VALID_VOTES));

            m_electionSequence = buildElectionSequence(selectionSequence);

            Count count;
            count.setElection(m_electionSequence);

            EML eml;
            eml.setCount(count);

            std::map<std::string, EML> plainEmlFiles;
            plainEmlFiles[EML_FILE_NAME] = eml;

            bool check = VoteComplianceVerificator::getInstance()->checkEmlVotesDeposited(plainEmlFiles);

            CPPUNIT_ASSERT(check);
        }

        void checkEmlVotesByVoterType_sumOfTypesOfVotersDoesNotMatchTotalVoters_failedValidation()
        {
            Count::ElectronicVotingCountersType totalVotersCounter(StringUtils::intToString(INCORRECT_TOTAL_VOTERS));
            totalVotersCounter.setType(TOTAL_VOTERS_IDENTIFIER);

            Count::ElectronicVotingCountersType europeanVotersCounter(StringUtils::intToString(EUROPEAN_VOTERS));
            europeanVotersCounter.setType(EUROPEAN_VOTERS_IDENTIFIER);

            Count::ElectronicVotingCountersType nationalVotersCounter(StringUtils::intToString(NATIONAL_VOTERS));
            nationalVotersCounter.setType(NATIONAL_VOTERS_IDENTIFIER);

            Count::ElectronicVotingCountersType nationalVotersAbroadCounter(StringUtils::intToString(NATIONAL_VOTERS_ABROAD));
            nationalVotersAbroadCounter.setType(NATIONAL_VOTERS_ABROAD_IDENTIFIER);

            m_electronicVotingCountersSequence.push_back(europeanVotersCounter);
            m_electronicVotingCountersSequence.push_back(nationalVotersCounter);
            m_electronicVotingCountersSequence.push_back(nationalVotersAbroadCounter);
            m_electronicVotingCountersSequence.push_back(totalVotersCounter);

            Count count;
            count.setElectronicVotingCounters(m_electronicVotingCountersSequence);

            EML eml;
            eml.setCount(count);

            std::map<std::string, EML> plainEmlFiles;
            plainEmlFiles[EML_FILE_NAME] = eml;

            bool check = VoteComplianceVerificator::getInstance()->checkEmlVotesByVoterType(plainEmlFiles);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkEmlVotesByVoterType_sumOfTypesOfVotersMatchesTotalVoters_successfulValidation()
        {
            Count::ElectronicVotingCountersType totalVotersCounter(StringUtils::intToString(TOTAL_VOTERS));
            totalVotersCounter.setType(TOTAL_VOTERS_IDENTIFIER);

            Count::ElectronicVotingCountersType europeanVotersCounter(StringUtils::intToString(EUROPEAN_VOTERS));
            europeanVotersCounter.setType(EUROPEAN_VOTERS_IDENTIFIER);

            Count::ElectronicVotingCountersType nationalVotersCounter(StringUtils::intToString(NATIONAL_VOTERS));
            nationalVotersCounter.setType(NATIONAL_VOTERS_IDENTIFIER);

            Count::ElectronicVotingCountersType nationalVotersAbroadCounter(StringUtils::intToString(NATIONAL_VOTERS_ABROAD));
            nationalVotersAbroadCounter.setType(NATIONAL_VOTERS_ABROAD_IDENTIFIER);

            Count::ElectronicVotingCountersSequence electronicVotingCountersSequence;
            electronicVotingCountersSequence.push_back(europeanVotersCounter);
            electronicVotingCountersSequence.push_back(nationalVotersCounter);
            electronicVotingCountersSequence.push_back(nationalVotersAbroadCounter);
            electronicVotingCountersSequence.push_back(totalVotersCounter);

            Count count;
            count.setElectronicVotingCounters(electronicVotingCountersSequence);

            EML eml;
            eml.setCount(count);

            std::map<std::string, EML> plainEmlFiles;
            plainEmlFiles[EML_FILE_NAME] = eml;

            bool check = VoteComplianceVerificator::getInstance()->checkEmlVotesByVoterType(plainEmlFiles);

            CPPUNIT_ASSERT(check);
        }

        void checkEmlVotesCounter_votesDepositedIsLessThanSumOfVotesCounter_failedValidation()
        {
            Selection::CountMetricType metricVotesDeposited(INCORRECT_VOTES_DEPOSITED, VOTES_DEPOSITED_METRIC_IDENTIFIER);

            Selection::CountMetricSequence metricSequence;
            metricSequence.push_back(metricVotesDeposited);

            TotalVotes::SelectionSequence selectionSequence;
            selectionSequence.push_back(buildSelection(DEFAULT_CATEGORY_IDENTIFIER, metricSequence));
            selectionSequence.push_back(buildSelection(SECOND_CATEGORY_IDENTIFIER, metricSequence));
            selectionSequence.push_back(buildSelection(THIRD_CATEGORY_IDENTIFIER, metricSequence));

            m_electionSequence = buildElectionSequence(selectionSequence);

            MockVoteStatistics voteStatistics;
            voteStatistics.votesCounterByContest.push_back(VoteCounterByContest(DEFAULT_CUSTOM_CODE, DEFAULT_VOTES_COUNTER));
            voteStatistics.votesCounterByContest.push_back(VoteCounterByContest(SECOND_CUSTOM_CODE, SECOND_VOTES_COUNTER));
            voteStatistics.votesCounterByContest.push_back(VoteCounterByContest(THIRD_CUSTOM_CODE, THIRD_VOTES_COUNTER));

            Count count;
            count.setElection(m_electionSequence);

            EML eml;
            eml.setCount(count);

            std::map<std::string, EML> plainEmlFiles;
            plainEmlFiles[EML_FILE_NAME] = eml;

            bool check = VoteComplianceVerificator::getInstance()->checkEmlVotesCounter(plainEmlFiles, &voteStatistics);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkEmlVotesCounter_votesDepositedIsGreaterThanSumOfVotesCounter_failedValidation()
        {
            Selection::CountMetricType metricVotesDeposited(VOTES_DEPOSITED, VOTES_DEPOSITED_METRIC_IDENTIFIER);

            Selection::CountMetricSequence metricSequence;
            metricSequence.push_back(metricVotesDeposited);

            TotalVotes::SelectionSequence selectionSequence;
            selectionSequence.push_back(buildSelection(DEFAULT_CATEGORY_IDENTIFIER, metricSequence));
            selectionSequence.push_back(buildSelection(SECOND_CATEGORY_IDENTIFIER, metricSequence));
            selectionSequence.push_back(buildSelection(THIRD_CATEGORY_IDENTIFIER, metricSequence));

            m_electionSequence = buildElectionSequence(selectionSequence);

            MockVoteStatistics voteStatistics;
            voteStatistics.votesCounterByContest.push_back(VoteCounterByContest(DEFAULT_CUSTOM_CODE, DEFAULT_VOTES_COUNTER));
            voteStatistics.votesCounterByContest.push_back(VoteCounterByContest(SECOND_CUSTOM_CODE, SECOND_VOTES_COUNTER));

            Count count;
            count.setElection(m_electionSequence);

            EML eml;
            eml.setCount(count);

            std::map<std::string, EML> plainEmlFiles;
            plainEmlFiles[EML_FILE_NAME] = eml;

            bool check = VoteComplianceVerificator::getInstance()->checkEmlVotesCounter(plainEmlFiles, &voteStatistics);

            CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(check));
        }

        void checkEmlVotesCounter_votesDepositedMatchSumOfVotesCounter_successfulValidation()
        {
            Selection::CountMetricType metricVotesDeposited(VOTES_DEPOSITED, VOTES_DEPOSITED_METRIC_IDENTIFIER);

            Selection::CountMetricSequence metricSequence;
            metricSequence.push_back(metricVotesDeposited);

            TotalVotes::SelectionSequence selectionSequence;
            selectionSequence.push_back(buildSelection(DEFAULT_CATEGORY_IDENTIFIER, metricSequence));
            selectionSequence.push_back(buildSelection(SECOND_CATEGORY_IDENTIFIER, metricSequence));
            selectionSequence.push_back(buildSelection(THIRD_CATEGORY_IDENTIFIER, metricSequence));

            m_electionSequence = buildElectionSequence(selectionSequence);

            MockVoteStatistics voteStatistics;
            voteStatistics.votesCounterByContest.push_back(VoteCounterByContest(DEFAULT_CUSTOM_CODE, DEFAULT_VOTES_COUNTER));
            voteStatistics.votesCounterByContest.push_back(VoteCounterByContest(SECOND_CUSTOM_CODE, SECOND_VOTES_COUNTER));
            voteStatistics.votesCounterByContest.push_back(VoteCounterByContest(THIRD_CUSTOM_CODE, THIRD_VOTES_COUNTER));

            Count count;
            count.setElection(m_electionSequence);

            EML eml;
            eml.setCount(count);

            std::map<std::string, EML> plainEmlFiles;
            plainEmlFiles[EML_FILE_NAME] = eml;

            bool check = VoteComplianceVerificator::getInstance()->checkEmlVotesCounter(plainEmlFiles, &voteStatistics);

            CPPUNIT_ASSERT(check);
        }

        CPPUNIT_TEST_SUITE(BeforeGeneratingResultsVerificationTest);

        CPPUNIT_TEST(checkTotalCumulativeSelections_partyVotesExceedTotalVotes_invalidVoteResults);
        CPPUNIT_TEST(checkTotalCumulativeSelections_principalCandidateVotesExceedTotalVotes_invalidVoteResults);
        CPPUNIT_TEST(checkTotalCumulativeSelections_standInCandidatesVotesExceedTotalVotes_invalidVoteResults);
        CPPUNIT_TEST(checkTotalCumulativeSelections_categorizedVotesDoNotExceedTotalVotes_validVoteResults);

        CPPUNIT_TEST(checkIndividualCumulativeSelections_partyListVotesExceedPartyTotalVotes_invalidVoteResults);
        CPPUNIT_TEST(checkIndividualCumulativeSelections_principalCandidateVotesExceedPartyTotalVotes_invalidVoteResults);
        CPPUNIT_TEST(checkIndividualCumulativeSelections_standInCandidateVotesExceedPartyTotalVotes_invalidVoteResults);
        CPPUNIT_TEST(checkIndividualCumulativeSelections_categorizedVotesDoNotExceedPartyTotalVotes_validVoteResults);

        CPPUNIT_TEST(checkEmlVotesDeposited_votesDepositedDoNotMatchSumOfValidAndInvalidVotes_failedValidation);
        CPPUNIT_TEST(checkEmlVotesDeposited_votesDepositedMatchSumOfValidAndInvalidVotes_successfulValidation);
        CPPUNIT_TEST(checkEmlVotesByVoterType_sumOfTypesOfVotersDoesNotMatchTotalVoters_failedValidation);
        CPPUNIT_TEST(checkEmlVotesByVoterType_sumOfTypesOfVotersMatchesTotalVoters_successfulValidation);
        CPPUNIT_TEST(checkEmlVotesCounter_votesDepositedIsLessThanSumOfVotesCounter_failedValidation);
        CPPUNIT_TEST(checkEmlVotesCounter_votesDepositedIsGreaterThanSumOfVotesCounter_failedValidation);
        CPPUNIT_TEST(checkEmlVotesCounter_votesDepositedMatchSumOfVotesCounter_successfulValidation);

        CPPUNIT_TEST_SUITE_END();

    private:

        Smartmatic::SAES::Voting::PBReports::ItemResult buildItemResult(
            const std::string &ballotOptionCode,
            const size_t preferenceVotes)
        {
            Smartmatic::SAES::Voting::PBReports::ItemResult candidateResult;
            candidateResult.setCode(ballotOptionCode);
            candidateResult.setPreference_votes(preferenceVotes);

            return candidateResult;
        }

        Selection buildSelection(
            const std::string& selectionCategory,
            const Selection::CountMetricSequence& metricSequence,
            const size_t& validVotes = 0)
        {
            Selection selection;
            selection.setValidVotes(validVotes);
            selection.setCategory(selectionCategory);
            selection.setCountMetric(metricSequence);

            return selection;
        }

        Count::ElectionSequence buildElectionSequence(const TotalVotes::SelectionSequence& selectionSequence)
        {
            TotalVotes totalVotes;
            totalVotes.setSelection(selectionSequence);

            Smartmatic::SAES::Voting::Eml::Contest contest;
            contest.setTotalVotes(totalVotes);

            Smartmatic::SAES::Voting::Eml::Contests::ContestSequence contestSequence;
            contestSequence.push_back(contest);

            Smartmatic::SAES::Voting::Eml::Contests contests;
            contests.setContest(contestSequence);

            Election::ElectionIdentifierType electionIdentifier;
            electionIdentifier.setElectionCategory(DEFAULT_ELECTION_CATEGORY);

            Election election;
            election.setContests(contests);
            election.setElectionIdentifier(electionIdentifier);

            Count::ElectionSequence electionSequence;
            electionSequence.push_back(election);

            return electionSequence;
        }

        Smartmatic::SAES::Voting::PBReports::VoteResults m_voteResults;
        ItemsResults::CandidateSequence m_candidatesSequence;
        ItemResult m_candidateResult;
        ItemsResults m_itemsResults;
        PartyResults m_partyResults;
        ContestResults::PartyResultsSequence m_partiesSequence;
        VoteResults::ContestResultsType m_contestResults;
        VoteResults::ContestResultsSequence m_contestsSequence;

        Count::ElectionSequence m_electionSequence;
        Count::ElectronicVotingCountersSequence m_electronicVotingCountersSequence;
};

#endif /* ECUT_MAIN */

#endif /* BEFORE_GENERATING_RESULTS_VERIFICATION_TEST */
