/**
 * @file DeviceTypeMonitorWindowConfigurationManager.cxx
 * @brief Implementation of the device type monitor window configuration manager.
 * @date 12/05/2014
 * @author Yadickson Soto
 */

#include "DeviceTypeMonitorWindowConfigurationManager.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>

using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Common::Configuration;

DeviceTypeMonitorWindowConfigurationManager * DeviceTypeMonitorWindowConfigurationManager::instance = NULL;

DeviceTypeMonitorWindowConfigurationManager::DeviceTypeMonitorWindowConfigurationManager()
{

}

DeviceTypeMonitorWindowConfigurationManager::~DeviceTypeMonitorWindowConfigurationManager()
{
    instance = NULL;
}

bool DeviceTypeMonitorWindowConfigurationManager::isVisible()
{
    return getDeviceTypeMonitorWindowConfiguration().getVisible();
}

unsigned int DeviceTypeMonitorWindowConfigurationManager::getTimeOut()
{
    return getDeviceTypeMonitorWindowConfiguration().getTimeOut();
}

unsigned int DeviceTypeMonitorWindowConfigurationManager::getOpacity()
{
    return getDeviceTypeMonitorWindowConfiguration().getOpacity();
}

unsigned int DeviceTypeMonitorWindowConfigurationManager::getIconWidth()
{
    return getDeviceTypeMonitorWindowConfiguration().getIconWidth();
}

unsigned int DeviceTypeMonitorWindowConfigurationManager::getIconHeight()
{
    return getDeviceTypeMonitorWindowConfiguration().getIconHeight();
}

PositionConfiguration & DeviceTypeMonitorWindowConfigurationManager::getPosition()
{
    return getDeviceTypeMonitorWindowConfiguration().getPosition();
}

unsigned int DeviceTypeMonitorWindowConfigurationManager::getBorder()
{
    return getDeviceTypeMonitorWindowConfiguration().getBorder();
}

bool DeviceTypeMonitorWindowConfigurationManager::isDeviceWidgetShowing ( const DeviceType & type )
{
    bool show = false;
    std::string machineModel;

    DevicesToShowConfiguration& config(getDeviceTypeMonitorWindowConfiguration().getDevicesToShow());
    DevicesToShowConfiguration::DeviceTypeMonitorSequence seq = config.getDeviceTypeMonitor();
    DevicesToShowConfiguration::DeviceTypeMonitorIterator it;

    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

    if (StringUtils::toUpper(machineModel) == "PM-VIU805" && type == DeviceType::CONFIGURATION_SMARTCARD)
    {
        return false;
    }

    for (it = seq.begin(); it != seq.end(); ++it)
    {
        if (*it == type)
        {
            show = true;
            break;
        }
    }

    return show;
}

DeviceTypeMonitorWindowConfiguration & DeviceTypeMonitorWindowConfigurationManager::getDeviceTypeMonitorWindowConfiguration ()
{
    if ( !SAESGUIConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorWindowConfiguration().present() )
    {
        DevicesToShowConfiguration devicesToShow;
        DevicesToShowConfiguration::DeviceTypeMonitorSequence devices;

        devicesToShow.setDeviceTypeMonitor ( devices );

        DeviceTypeMonitorWindowConfiguration config (
            false,
            0,
            0,
            100,
            100,
            PositionConfiguration ( PositionConfiguration::CONFIGURATION_BOTTOM_RIGHT ),
            0,
            devicesToShow );


        SAESGUIConfigurationManager::getInstance()->getConfiguration()->setDeviceTypeMonitorWindowConfiguration ( config );
    }

    return SAESGUIConfigurationManager::getInstance()->getConfiguration()->getDeviceTypeMonitorWindowConfiguration().get();
}

DeviceTypeMonitorWindowConfigurationManager * DeviceTypeMonitorWindowConfigurationManager::getInstance ( DeviceTypeMonitorWindowConfigurationManager * externManager )
{
    if ( ! instance )
    {
        instance = ( externManager == NULL ? new DeviceTypeMonitorWindowConfigurationManager() : externManager );
    }

    return instance;
}
