/**
 * @file AbleDiagnosticFunctionality.hxx
 * @brief Header class able diagnostic functionality
 * @date 05/09/2012
 * @author Yadickson Soto
 */

#ifndef ABLEDIAGNOSTICFUNCTIONALITY_HXX_
#define ABLEDIAGNOSTICFUNCTIONALITY_HXX_

#include <gtkmm.h>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/AbleDiagnosticWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	AbleDiagnosticFunctionality
                 * @brief	Class able diagnostic functionality.
                 * @author	Yadickson Soto
                 * @date	05/09/2012
                 */
                class AbleDiagnosticFunctionality : public GraphicDisplayFunctionality
                {
                    private:
                        /**
                         * @fn	AbleDiagnosticFunctionality::AbleDiagnosticFunctionality();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        AbleDiagnosticFunctionality();

                        static AbleDiagnosticFunctionality * instance; ///< instance of class
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                        Smartmatic::SAES::GUI::Widgets::AbleDiagnosticWidget * ableDiagnosticWidget; ///< voting for disabled
                        sigc::connection connDiag;	///< connection diagnosis

                    protected:

                        /**
                         * @fn	std::string AbleDiagnosticFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         * @return	The window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType AbleDiagnosticFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         * @return	The menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	Gtk::Widget * AbleDiagnosticFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         * @return	null if it fails, else the frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	void AbleDiagnosticFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void AbleDiagnosticFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * AbleDiagnosticFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         * @return	null if it fails, else the logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring AbleDiagnosticFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         * @return	The start log message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring AbleDiagnosticFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         * @return	The finish log message.
                         */
                        Glib::ustring getFinishLogMessage();
                        /**
                         * @fn	virtual void AbleDiagnosticFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        virtual void deleteElements();
                        /**
                         * @fn	virtual void AbleDiagnosticFunctionality::block();
                         * @brief	method block functionality.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        virtual void block();
                        /**
                         * @fn	virtual void AbleDiagnosticFunctionality::partial();
                         * @brief	method partial functionality.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        virtual void partial();
                        /**
                         * @fn	virtual void AbleDiagnosticFunctionality::resume();
                         * @brief	method resume functionality.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        virtual void resume();
                        /**
                         * @fn	void AbleDiagnosticFunctionality::endDiagnosis();
                         * @brief	method end diagnosis.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         */
                        void endDiagnosis();

                    public:
                        /**
                         * @fn	static AbleDiagnosticFunctionality * AbleDiagnosticFunctionality::getInstance();
                         * @brief	Method getter instance.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         * @return	instance of class.
                         */
                        static AbleDiagnosticFunctionality * getInstance();
                        /**
                         * @fn	virtual AbleDiagnosticFunctionality::~AbleDiagnosticFunctionality();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        virtual ~AbleDiagnosticFunctionality();
                };
            }
        }
    }
}
#endif /* ABLEDIAGNOSTICFUNCTIONALITY_HXX_ */
