/**
 * @file	ActivationConsolidationFunctionality.h
 * @brief	Declares the activation consolidation functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef ACTIVATIONCONSOLIDATIONFUNCTIONALITY_H_
#define ACTIVATIONCONSOLIDATIONFUNCTIONALITY_H_

#include <gtkmm.h>
#include <Log/ISMTTLog.h>
#include <Functionalities/graphic-display-functionality.h>
#include <Configuration/label-configuration.h>
#include <Gui/Widgets/ConsolidationStatsWidget.h>
#include <Gui/Widgets/ActivationStatsWidget.h>
#include <Gui/Widgets/CardActivatorEventBox.h>
#include <Voting/VotingDevice/voting-devices-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	ActivationConsolidationFunctionality
                 * @brief	Activation consolidation functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class ActivationConsolidationFunctionality : public GraphicDisplayFunctionality
                {
                    public:
                        /**
                         * @fn	static ActivationConsolidationFunctionality * ActivationConsolidationFunctionality::getInstance();
                         * @brief	Gets the instance of class.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         * @return	null if it fails, else instance of class.
                         */
                        static ActivationConsolidationFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	virtual void ActivationConsolidationFunctionality::block();
                         * @brief	method block functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void block();
                        /**
                         * @fn	virtual void ActivationConsolidationFunctionality::partial();
                         * @brief	method partial functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void partial();
                        /**
                         * @fn	virtual void ActivationConsolidationFunctionality::resume();
                         * @brief	method resume functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void resume();
                        /**
                         * @fn	virtual void ActivationConsolidationFunctionality::refresh();
                         * @brief	method refresh.
                         * @author	Yadickson Soto
                         * @date	14/06/2012
                         */
                        virtual void refresh();

                        /**
                         * @fn	virtual void ActivationConsolidationFunctionality::onHelpShown();
                         * @brief	Method on help shown.
                         * @author	Juan Valera
                         * @date	09/02/2017
                         */
                        virtual void onHelpShown();

                        /**
                         * @fn	virtual void ActivationConsolidationFunctionality:: onHelpClosed();
                         * @brief	Method on help closed.
                         * @author	Juan Valera
                         * @date	09/02/2017
                         */
                        virtual void onHelpClosed();

                    private:
                        static ActivationConsolidationFunctionality * current_instance; ///< The current instance

                        /**
                         * @fn	ActivationConsolidationFunctionality::ActivationConsolidationFunctionality();
                         * @brief	Default constructor.
                         * @author	Juan Delgado
                         * @date	02/09/2011d
                         */
                        ActivationConsolidationFunctionality();

                        /**
                         * @fn	virtual ActivationConsolidationFunctionality::~ActivationConsolidationFunctionality();
                         * @brief	Default destructor.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         */
                        virtual ~ActivationConsolidationFunctionality();

                        /**
                         * @fn	std::string ActivationConsolidationFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType ActivationConsolidationFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         * @return	The menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                        /**
                         * @fn	Gtk::Widget * ActivationConsolidationFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         * @return	null if it fails, else the frame.
                         */
                        Gtk::Widget * GetFrame();

                        /**
                         * @fn	void ActivationConsolidationFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         */
                        void Dispose();

                        /**
                         * @fn	void ActivationConsolidationFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         */
                        void Initialize();

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * ActivationConsolidationFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         * @return	null if it fails, else the logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring ActivationConsolidationFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         * @return	The start log message.
                         */
                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring ActivationConsolidationFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         * @return	The finish log message.
                         */
                        Glib::ustring getFinishLogMessage();
                        /**
                         * @fn	virtual void ActivationConsolidationFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                        /**
                         * @fn	void ActivationConsolidationFunctionality::SetCustomAttributeVector ( std::vector< Smartmatic::SAES::Voting::VotingDevice::Custom_attribute > vecCA );
                         * @brief	Sets the custom attribute vector.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         * @param	vecCA	custom attribute vector.
                         */
                        void SetCustomAttributeVector ( std::vector< Smartmatic::SAES::Voting::VotingDevice::Custom_attribute > vecCA );

                        /**
                         * @fn  void ActivationConsolidationFunctionality::writeVotingCard();
                         * @brief   Writes information into a voting card.
                         * @author  Eduardo Fernández
                         * @date    04/01/2018
                         */
                        void writeVotingCard();

                        /**
                         * @fn	void ActivationConsolidationFunctionality::BarcodeConnected ( bool isConnected );
                         * @brief	Handles if barcode is connected.
                         * @author	Juan Delgado
                         * @date	02/09/2011
                         * @param	isConnected	if barcode is connected.
                         */
                        void BarcodeConnected ( bool isConnected );

                        /**
                         * @fn	void ActivationConsolidationFunctionality::statusSendCommands ( bool status );
                         * @brief	Status send commands.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	status	true to status.
                         */
                        void statusSendCommands ( bool status );

                        /**
                         * @fn	void ActivationConsolidationFunctionality::hideBarcodeConnected();
                         * @brief	Hides the barcode connected.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void hideBarcodeConnected();

                        /**
                         * @fn	virtual void ActivationConsolidationFunctionality::onBeforeExit();
                         * @brief	virtual method before exit.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        virtual void onBeforeExit();

                        /**
                         * @fn	void ActivationConsolidationFunctionality::onSignalStatusActivated(bool status);
                         * @brief	method on signal status activated.
                         * @author	Yadickson Soto
                         * @date	13/11/2012
                         * @param status status activated
                         */
                        void onSignalStatusActivated ( bool status );

                        /**
                         * @fn  void ActivationConsolidationFunctionality::onActivationSignalDone();
                         * @brief   method on activation done signal.
                         * @author  Eduardo Fernández
                         * @date    04/01/2018
                         */
                        void onActivationSignalDone();

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                        Smartmatic::SAES::GUI::Widgets::ActivationStatsWidget * activationWidget; ///< The activation widget
                        Smartmatic::SAES::GUI::Widgets::ConsolidationStatsWidget * consolidationWidget; ///< The consolidation widget
                        Smartmatic::SAES::GUI::Widgets::CardActivatorEventBox * cardActivatorBox; ///< The card activator box

                        Gtk::HBox * outsideBox; ///< The outside box
                };
            }
        }
    }
}

#endif /* ACTIVATIONCONSOLIDATIONFUNCTIONALITY_H_ */
