/**
 * @file AutoBarcodeDiagnosisFunctionality.hxx
 * @brief Header auto barcode diagnosis functionality class
 * @date 20/06/2013
 * @author Yadickson Soto
 */

#ifndef _AUTO_BARCODE_DIAGNOSIS_FUNCTIONALITY_HXX
#define _AUTO_BARCODE_DIAGNOSIS_FUNCTIONALITY_HXX

#include <gtkmm.h>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/AutoBarcodeDiagnosisWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	AutoBarcodeDiagnosisFunctionality
                 * @brief	auto barcode diagnostic functionality class.
                 * @author	Yadickson Soto
                 * @date	20/06/2013
                 */
                class AutoBarcodeDiagnosisFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual AutoBarcodeDiagnosisFunctionality::~AutoBarcodeDiagnosisFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        virtual ~AutoBarcodeDiagnosisFunctionality();
                        /**
                         * @fn	static AutoBarcodeDiagnosisFunctionality * AutoBarcodeDiagnosisFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	instance of class.
                         */
                        static AutoBarcodeDiagnosisFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * AutoBarcodeDiagnosisFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring AutoBarcodeDiagnosisFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring AutoBarcodeDiagnosisFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static AutoBarcodeDiagnosisFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Smartmatic::SAES::GUI::Widgets::AutoBarcodeDiagnosisWidget * widget; ///< The widget

                        /**
                         * @fn	AutoBarcodeDiagnosisFunctionality::AutoBarcodeDiagnosisFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        AutoBarcodeDiagnosisFunctionality();
                        /**
                         * @fn	void AutoBarcodeDiagnosisFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        void Dispose();
                        /**
                         * @fn	void AutoBarcodeDiagnosisFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * AutoBarcodeDiagnosisFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string AutoBarcodeDiagnosisFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType AutoBarcodeDiagnosisFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	void AutoBarcodeDiagnosisFunctionality::end();
                         * @brief	method end diagnostic.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        void end();
                        /**
                         * @fn	virtual void AutoBarcodeDiagnosisFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        virtual void deleteElements();

                        /**
                         * @fn	virtual bool ForceExecute();
                         * @brief	Method force execute functionality
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	true if force execute
                         */
                        virtual bool ForceExecute();
                };
            }
        }
    }
}

#endif // _AUTO_BARCODE_DIAGNOSIS_FUNCTIONALITY_HXX
