/**
 * @file AutoDiagnosticFunctionality.hxx
 * @brief Header auto diagnostic functionality class
 * @date 24/09/2012
 * @author Yadickson Soto
 */

#ifndef _AUTO_DIAGNOSTIC_FUNCTIONALITY_HXX
#define _AUTO_DIAGNOSTIC_FUNCTIONALITY_HXX

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/AutomaticDiagnosticWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	AutoDiagnosticFunctionality
                 * @brief	auto diagnostic functionality class.
                 * @author	Yadickson Soto
                 * @date	24/09/2012
                 */
                class AutoDiagnosticFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual AutoDiagnosticFunctionality::~AutoDiagnosticFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         */
                        virtual ~AutoDiagnosticFunctionality();
                        /**
                         * @fn	static AutoDiagnosticFunctionality * AutoDiagnosticFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         * @return	instance of class.
                         */
                        static AutoDiagnosticFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * AutoDiagnosticFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring AutoDiagnosticFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring AutoDiagnosticFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static AutoDiagnosticFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Smartmatic::SAES::GUI::Widgets::AutomaticDiagnosticWidget * widget; ///< The widget
                        std::string serial; ///< current serial

                        /**
                         * @fn	AutoDiagnosticFunctionality::AutoDiagnosticFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         */
                        AutoDiagnosticFunctionality();
                        /**
                         * @fn	void AutoDiagnosticFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void AutoDiagnosticFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * AutoDiagnosticFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string AutoDiagnosticFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType AutoDiagnosticFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	void AutoDiagnosticFunctionality::end();
                         * @brief	method end diagnostic.
                         * @author	Yadickson Soto
                         * @date	24/09/2012
                         */
                        void end();
                        /**
                         * @fn	virtual void AutoDiagnosticFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                        /**
                         * @fn	virtual bool ForceExecute();
                         * @brief	Method force execute functionality
                         * @author	Yadickson Soto
                         * @date	16/11/2012
                         * @return	true if force execute
                         */
                        virtual bool ForceExecute();
                };
            }
        }
    }
}

#endif // _AUTO_DIAGNOSTIC_FUNCTIONALITY_HXX
