/**
 * @file AutoPrinterDiagnosisFunctionality.hxx
 * @brief Header auto printer diagnosis functionality class
 * @date 20/06/2013
 * @author Yadickson Soto
 */

#ifndef _AUTO_PRINTER_DIAGNOSIS_FUNCTIONALITY_HXX
#define _AUTO_PRINTER_DIAGNOSIS_FUNCTIONALITY_HXX

#include <gtkmm.h>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/AutoPrinterDiagnosisWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	AutoPrinterDiagnosisFunctionality
                 * @brief	auto printer diagnostic functionality class.
                 * @author	Yadickson Soto
                 * @date	20/06/2013
                 */
                class AutoPrinterDiagnosisFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual AutoPrinterDiagnosisFunctionality::~AutoPrinterDiagnosisFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        virtual ~AutoPrinterDiagnosisFunctionality();
                        /**
                         * @fn	static AutoPrinterDiagnosisFunctionality * AutoPrinterDiagnosisFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	instance of class.
                         */
                        static AutoPrinterDiagnosisFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * AutoPrinterDiagnosisFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring AutoPrinterDiagnosisFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring AutoPrinterDiagnosisFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static AutoPrinterDiagnosisFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Smartmatic::SAES::GUI::Widgets::AutoPrinterDiagnosisWidget * widget; ///< The widget

                        /**
                         * @fn	AutoPrinterDiagnosisFunctionality::AutoPrinterDiagnosisFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        AutoPrinterDiagnosisFunctionality();
                        /**
                         * @fn	void AutoPrinterDiagnosisFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        void Dispose();
                        /**
                         * @fn	void AutoPrinterDiagnosisFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * AutoPrinterDiagnosisFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string AutoPrinterDiagnosisFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType AutoPrinterDiagnosisFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	void AutoPrinterDiagnosisFunctionality::end();
                         * @brief	method end diagnostic.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        void end();
                        /**
                         * @fn	virtual void AutoPrinterDiagnosisFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         */
                        virtual void deleteElements();

                        /**
                         * @fn	virtual bool ForceExecute();
                         * @brief	Method force execute functionality
                         * @author	Yadickson Soto
                         * @date	20/06/2013
                         * @return	true if force execute
                         */
                        virtual bool ForceExecute();
                };
            }
        }
    }
}

#endif // _AUTO_BARCODE_DIAGNOSIS_FUNCTIONALITY_HXX
