/**
 * @file CheckVoteFilesFunctionality.cxx
 * @brief Body class check vote files functionality
 * @date 05/09/2012
 * @author Yadickson Soto
 */

#include "CheckVoteFilesFunctionality.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::SAES::Operation;

ISMTTLog * CheckVoteFilesFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.CheckVoteFilesFunctionality", GETTEXT_PACKAGE );

CheckVoteFilesFunctionality * CheckVoteFilesFunctionality::instance = NULL;

CheckVoteFilesFunctionality::CheckVoteFilesFunctionality()
{
    widget = NULL;
    isInterrupted = false;
}

CheckVoteFilesFunctionality::~CheckVoteFilesFunctionality()
{
    deleteElements();
}

CheckVoteFilesFunctionality * CheckVoteFilesFunctionality::getInstance()
{
    if ( !instance )
    {
        instance = new CheckVoteFilesFunctionality();
    }

    return instance;
}

std::string CheckVoteFilesFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.CheckVoteFilesFunctionality.Title" );
}

Smartmatic::Log::ISMTTLog * CheckVoteFilesFunctionality::getLogger()
{
    return logger;
}

Glib::ustring CheckVoteFilesFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.CheckVoteFilesFunctionality.LogStartFunctionality" );
}

Glib::ustring CheckVoteFilesFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.CheckVoteFilesFunctionality.LogCloseFunctionality" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType CheckVoteFilesFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMCHECKVOTESFUNCTIONALITY;
}

Gtk::Widget * CheckVoteFilesFunctionality::GetFrame()
{
    widget = AbstractFactoryWidget::createValidateVotesWidget();
    widget->getSignalEnd().connect ( sigc::mem_fun ( this, &CheckVoteFilesFunctionality::ExitFunctionality ) );
    widget->getSignalError().connect ( sigc::mem_fun ( this, &CheckVoteFilesFunctionality::onError ) );
    return widget;
}

void CheckVoteFilesFunctionality::Dispose()
{

}

void CheckVoteFilesFunctionality::Initialize()
{
    if ( MachineOperationManager::getInstance()->isCloseElection() )
    {
        logger->Debug ( "Could not perform functionality in a closed election." );
        ExitFunctionality();
        return;
    }

    disableExitButton();
    widget->initialize ();
}

void CheckVoteFilesFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void CheckVoteFilesFunctionality::block()
{
    logger->Debug ( "Block functionality" );
    isInterrupted = true;
}

void CheckVoteFilesFunctionality::partial()
{
    logger->Debug ( "Partial functionality" );
}

void CheckVoteFilesFunctionality::resume()
{
    logger->Debug ( "Resume functionality" );
    isInterrupted = false;
}

void CheckVoteFilesFunctionality::onError()
{
    Glib::signal_timeout().connect ( sigc::mem_fun ( this, &CheckVoteFilesFunctionality::checkResume ), 500 );
}

bool CheckVoteFilesFunctionality::checkResume()
{
    bool keepChecking = true;

    if (!isInterrupted)
    {
        keepChecking = false;
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( this, &CheckVoteFilesFunctionality::Initialize ), 100 );
    }

    return keepChecking;
}
