/**
 * @file DataUtilsValidatorFunctionality.hxx
 * @brief Header class data utils validator functionality
 * @date 18/07/2012
 * @author Yadickson Soto
 */

#ifndef DATAUTILSVALIDATORFUNCTIONALITY_HXX_
#define DATAUTILSVALIDATORFUNCTIONALITY_HXX_

#include <gtkmm.h>
#include <Functionalities/graphic-display-functionality.h>
#include <Runtime/Environment/SaesInstallDirectories.hxx>
#include <Runtime/Environment/SaesTempValidatorDirectories.hxx>
#include <Voting/VotingCenters/voting-centers-schema.hxx>
#include <Operation/Voting/VotingExperienceController.hxx>
#include <Gui/Widgets/DataUtilsValidatorWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	DataUtilsValidatorFunctionality
                 * @brief	Class data utils validator functionality.
                 * @author	Yadickson Soto
                 * @date	18/07/2012
                 */
                class DataUtilsValidatorFunctionality : public GraphicDisplayFunctionality
                {
                    private:
                        /**
                         * @fn	DataUtilsValidatorFunctionality::DataUtilsValidatorFunctionality();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        DataUtilsValidatorFunctionality();

                        static DataUtilsValidatorFunctionality * instance; ///< instance of class
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                        Smartmatic::SAES::GUI::Widgets::DataUtilsValidatorWidget * dataUtilsWidget; ///< data utils widget

                        Smartmatic::SAES::Runtime::Environment::SaesInstallDirectories * currentDirectories;   ///< The current directories
                        Smartmatic::SAES::Runtime::Environment::SaesTempValidatorDirectories * directoriesTempValidation; ///< directories temp validation

                        std::vector< Smartmatic::SAES::Voting::VotingCenters::VotingCenter > votingCenters; ///< all voting centers
                        Smartmatic::SAES::Voting::VotingCenters::VotingCenter center; ///< current center

                        std::map <std::string, std::string> keyMap; ///< key map

                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;  ///< The electoral configuration
                        Smartmatic::SAES::Operation::Voting::VotingExperienceController * votingExperienceController; ///< voting experience controller

                        unsigned int currentCenter; ///< current center
                        std::string folderPath;	///< The folder path
                        bool centerCaseSensitive; ///< center case sensitive
                        std::string currentLanguage; ///< current language
                        std::string currentCountry; ///< current country

                        Glib::Dispatcher signalStartValidate; ///< signal start validate
                        Glib::Dispatcher signalFailValidate; ///< signal fail validate
                        Glib::Dispatcher signalNextVotingCenter; ///< signal next voting center
                        Glib::Dispatcher signalEndFunctionality; ///< signal end functionality

                        Glib::Thread * m_thread_check_data; ///< The thread pointer to check data
                        Glib::Thread * m_thread_validate_center; ///< The thread pointer to validate center
                        Glib::Thread * m_thread_export_results; ///< The thread pointer to export results

                    protected:

                        /**
                         * @fn	virtual std::string DataUtilsValidatorFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	The window title.
                         */
                        virtual std::string GetWindowTitle();
                        /**
                         * @fn	virtual bool DataUtilsValidatorFunctionality::ShowWindowTitle();
                         * @brief	getter isshow window title.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	true is show window title.
                         */
                        virtual bool ShowWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType DataUtilsValidatorFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	The menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	Gtk::Widget * DataUtilsValidatorFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	null if it fails, else the frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	void DataUtilsValidatorFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void DataUtilsValidatorFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * DataUtilsValidatorFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	null if it fails, else the logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring DataUtilsValidatorFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	The start log message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring DataUtilsValidatorFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	The finish log message.
                         */
                        Glib::ustring getFinishLogMessage();
                        /**
                         * @fn	virtual void DataUtilsValidatorFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        virtual void deleteElements();
                        /**
                         * @fn	virtual void DataUtilsValidatorFunctionality::block();
                         * @brief	method block functionality.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        virtual void block();
                        /**
                         * @fn	virtual void DataUtilsValidatorFunctionality::partial();
                         * @brief	method partial functionality.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        virtual void partial();
                        /**
                         * @fn	virtual void DataUtilsValidatorFunctionality::resume();
                         * @brief	method resume functionality.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        virtual void resume();
                        /**
                         * @fn	void DataUtilsValidatorFunctionality::startValidate();
                         * @brief	method start validate.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        void startValidate();
                        /**
                         * @fn	void DataUtilsValidatorFunctionality::nextVotingCenter();
                         * @brief	method validate next voting center.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        void nextVotingCenter();
                        /**
                         * @fn	void DataUtilsValidatorFunctionality::validateCenter();
                         * @brief	method validate voting center.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        void validateCenter();
                        /**
                         * @fn	bool DataUtilsValidatorFunctionality::loadElectoralConfiguration();
                         * @brief	method validate voting center.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	true is load config success.
                         */
                        bool loadElectoralConfiguration();
                        /**
                         * @fn	bool DataUtilsValidatorFunctionality::getPassword ( std::string & password );
                         * @brief	method getter password.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @param	password	password.
                         * @return	true is load password success.
                         */
                        bool getPassword ( std::string & password );
                        /**
                         * @fn	bool DataUtilsValidatorFunctionality::loadKeys();
                         * @brief	method load keys.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	true is load keys success.
                         */
                        bool loadKeys();

                        /**
                         * @fn	void checkData ()
                         * @brief	Method check data files
                         * @author	Yadickson Soto
                         * @date	02/11/2012
                         */
                        void checkData();
                        /**
                         * @fn	bool checkSAESResources ()
                         * @brief	Method check SAES files
                         * @author	Yadickson Soto
                         * @date	02/11/2012
                         * @return	True is check result
                         */
                        bool checkSAESResources ( );
                        /**
                         * @fn	bool validateCenterResources ()
                         * @brief	Method check center files
                         * @author	Yadickson Soto
                         * @date	12/11/2012
                         * @return	True is check result
                         */
                        bool validateCenterResources();
                        bool loadLanguages();
                        /**
                         * @fn	bool validateCloseElection ()
                         * @brief	Method check close election process
                         * @author	Yadickson Soto
                         * @date	31/05/2013
                         * @return	True is check result
                         */
                        bool validateCloseElection();

                        /**
                         * @fn	void failValidate ()
                         * @brief	Method show faild validate data
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         */
                        void failValidate();

                        /**
                         * @fn	virtual bool ForceExecute();
                         * @brief	Method force execute functionality
                         * @author	Yadickson Soto
                         * @date	16/11/2012
                         * @return	true if force execute
                         */
                        virtual bool ForceExecute();

                        void endFunctionalityProcess();
                        void exportResults();

                    public:
                        /**
                         * @fn	static DataUtilsValidatorFunctionality * DataUtilsValidatorFunctionality::getInstance();
                         * @brief	Method getter instance.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	instance of class.
                         */
                        static DataUtilsValidatorFunctionality * getInstance();
                        /**
                         * @fn	virtual DataUtilsValidatorFunctionality::~DataUtilsValidatorFunctionality();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         */
                        virtual ~DataUtilsValidatorFunctionality();
                };
            }
        }
    }
}
#endif /* DATAUTILSVALIDATORFUNCTIONALITY_HXX_ */
