/**
 * @file DateDiagnosticFunctionality.hxx
 * @brief Header date setup functionality class
 * This class will provide date setup functionality,
 * providing the user a graphic way to observe, modify
 * and confirm the current date, and storing it into a file.
 * @date 18/09/2012
 * @author Daniel Ampuero
 */

#ifndef _DATE_DIAGNOSTIC_FUNCTIONALITY_H_
#define _DATE_DIAGNOSTIC_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/DateDiagnosticWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	DateDiagnosticFunctionality
                 * @brief	Date setup functionality class.
                 * @author	Daniel Ampuero
                 * @date	19/09/2012
                 */
                class DateDiagnosticFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual DateDiagnosticFunctionality::~DateDiagnosticFunctionality();
                         * @brief	Class destructorer.
                         * @author	Daniel Ampuero
                         * @date	189/09/2012
                         */
                        virtual ~DateDiagnosticFunctionality();
                        /**
                         * @fn	static DateDiagnosticFunctionality * DateDiagnosticFunctionality::getInstance();
                         * @brief	Singleton instance getter.
                         * @author	Daniel Ampuero
                         * @date	18/09/2012
                         * @return	instance of class.
                         */
                        static DateDiagnosticFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * DateDiagnosticFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Daniel Ampuero
                         * @date	18/09/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring DateDiagnosticFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Daniel Ampuero
                         * @date	18/09/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring DateDiagnosticFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Daniel Ampuero
                         * @date	18/09/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static DateDiagnosticFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Smartmatic::SAES::GUI::Widgets::DateDiagnosticWidget * widget; ///< The widget

                        /**
                         * @fn	DateDiagnosticFunctionality::DateDiagnosticFunctionality();
                         * @brief	class constructor.
                         * @author	Daniel Ampuero
                         * @date	19/09/2012
                         */
                        DateDiagnosticFunctionality();
                        /**
                         * @fn	void DateDiagnosticFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Daniel Ampuero
                         * @date	19/09/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void DateDiagnosticFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Daniel Ampuero
                         * @date	19/09/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * DateDiagnosticFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Daniel Ampuero
                         * @date	19/09/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string DateDiagnosticFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Daniel Ampuero
                         * @date	19/09/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType DateDiagnosticFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Daniel Ampuero
                         * @date	19/09/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	virtual void DateDiagnosticFunctionality::deleteElements();
                         * @brief	Deletes all elementss.
                         * @author	Daniel Ampuero
                         * @date	19/09/2012
                         */
                        virtual void deleteElements();
                        /**
                         * @fn	virtual bool DateDiagnosticFunctionality::ShowWindowTitle();
                         * @brief	Gets whether window title is visible.
                         * @author	Daniel Ampuero
                         * @date	19/09/2012
                         * @return	true is show window title.
                         */
                        virtual bool ShowWindowTitle();

                        /**
                         * @fn	void DateDiagnosticFunctionality::ProcessDate();
                         * @brief	Process the date introduced by the user in the widget.
                         * @author	Daniel Ampuero
                         * @date	21/09/2012
                         */
                        void ProcessDate();


                };
            }
        }
    }
}

#endif // _DATE_DIAGNOSTIC_FUNCTIONALITY_H_
