/**
 * @file DeviceDiagnosticResultFunctionality.cxx
 * @brief Body device diagnostic result functionality class
 * @date 19/09/2012
 * @author Yadickson Soto
 */

#include "DeviceDiagnosticResultFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/IO/File.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/FactoryMethodWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * DeviceDiagnosticResultFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality", GETTEXT_PACKAGE );

DeviceDiagnosticResultFunctionality * DeviceDiagnosticResultFunctionality::current_instance = NULL;
DeviceDiagnosticResultFunctionality * DeviceDiagnosticResultFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new DeviceDiagnosticResultFunctionality();
    }

    return current_instance;
}

DeviceDiagnosticResultFunctionality::DeviceDiagnosticResultFunctionality()
{
    widget = NULL;
}

void DeviceDiagnosticResultFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void DeviceDiagnosticResultFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality.ExitFunctionality" ) );
    disableExitButton();
    widget->refresh();
    widget->getSignalEnd().connect ( sigc::mem_fun ( *this, &DeviceDiagnosticResultFunctionality::endShowResult ) );
}

void DeviceDiagnosticResultFunctionality::endShowResult()
{
    enableExitButton();
    ExitFunctionality();
}

Gtk::Widget * DeviceDiagnosticResultFunctionality::GetFrame()
{
    int labelWidth = SAESGUIConfigurationManager::getInstance()->getDeviceDiagnosticResultConfiguration().getDiagnosticReviewWidth();
    int labelHeight = SAESGUIConfigurationManager::getInstance()->getDeviceDiagnosticResultConfiguration().getDiagnosticReviewHeight();
    widget = new DiagnosticResultWidget ( true, labelWidth, labelHeight );
    return widget;
}

std::string DeviceDiagnosticResultFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality.Title" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType DeviceDiagnosticResultFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICRESULTFUNCTIONALITY;
}

void DeviceDiagnosticResultFunctionality::Dispose()
{
}

DeviceDiagnosticResultFunctionality::~DeviceDiagnosticResultFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * DeviceDiagnosticResultFunctionality::getLogger()
{
    return logger;
}

Glib::ustring DeviceDiagnosticResultFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality.LogStartFunctionality" );
}

Glib::ustring DeviceDiagnosticResultFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality.LogCloseFunctionality" );
}

bool DeviceDiagnosticResultFunctionality::ShowWindowTitle()
{
    return true;
}

bool DeviceDiagnosticResultFunctionality::ForceExecute()
{
    return true;
}
