/**
 * @file DeviceDiagnosticResultFunctionality.hxx
 * @brief Header device diagnostic result functionality class
 * @date 19/09/2012
 * @author Yadickson Soto
 */

#ifndef _DEVICE_DIAGNOSTIC_FUNCTIONALITY_H_
#define _DEVICE_DIAGNOSTIC_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/DiagnosticResultWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	DeviceDiagnosticResultFunctionality
                 * @brief	device diagnostic result functionality class.
                 * @author	Yadickson Soto
                 * @date	19/09/2012
                 */
                class DeviceDiagnosticResultFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual DeviceDiagnosticResultFunctionality::~DeviceDiagnosticResultFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         */
                        virtual ~DeviceDiagnosticResultFunctionality();
                        /**
                         * @fn	static DeviceDiagnosticResultFunctionality * DeviceDiagnosticResultFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         * @return	instance of class.
                         */
                        static DeviceDiagnosticResultFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * DeviceDiagnosticResultFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring DeviceDiagnosticResultFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring DeviceDiagnosticResultFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static DeviceDiagnosticResultFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Smartmatic::SAES::GUI::Widgets::DiagnosticResultWidget * widget; ///< The widget

                        /**
                         * @fn	DeviceDiagnosticResultFunctionality::DeviceDiagnosticResultFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         */
                        DeviceDiagnosticResultFunctionality();
                        /**
                         * @fn	void DeviceDiagnosticResultFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void DeviceDiagnosticResultFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * DeviceDiagnosticResultFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string DeviceDiagnosticResultFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType DeviceDiagnosticResultFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	virtual bool DeviceDiagnosticResultFunctionality::ShowWindowTitle();
                         * @brief	Gets whether window title is visible.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	true is show window title.
                         */
                        virtual bool ShowWindowTitle();
                        /**
                         * @fn	virtual void DeviceDiagnosticResultFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();
                        /**
                         * @fn	void DeviceDiagnosticResultFunctionality::endShowResult();
                         * @brief	method end show result.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         */
                        void endShowResult();
                        /**
                         * @fn	virtual bool ForceExecute();
                         * @brief	Method force execute functionality
                         * @author	Yadickson Soto
                         * @date	26/06/2013
                         * @return	true if force execute
                         */
                        virtual bool ForceExecute();
                };
            }
        }
    }
}

#endif // _DEVICE_DIAGNOSTIC_FUNCTIONALITY_H_
