/**
 * @file   DeviceDiagnosticResultListOnlyFunctionality.cxx
 * @brief  Implementation of DeviceDiagnosticResultListOnlyFunctionality class.
 * @author manuel.arguelles
 * @date   Sep  4, 2017
 */

#include "DeviceDiagnosticResultListOnlyFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/IO/File.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/FactoryMethodWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog* DeviceDiagnosticResultListOnlyFunctionality::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality",
    GETTEXT_PACKAGE);

DeviceDiagnosticResultListOnlyFunctionality* DeviceDiagnosticResultListOnlyFunctionality::m_currentInstance = NULL;

DeviceDiagnosticResultListOnlyFunctionality* DeviceDiagnosticResultListOnlyFunctionality::getInstance()
{
    if ( !m_currentInstance )
    {
        m_currentInstance = new DeviceDiagnosticResultListOnlyFunctionality();
    }

    return m_currentInstance;
}

DeviceDiagnosticResultListOnlyFunctionality::DeviceDiagnosticResultListOnlyFunctionality()
    : m_widget(NULL)
{
}

void DeviceDiagnosticResultListOnlyFunctionality::deleteElements()
{
    delete m_widget;
    m_widget = NULL;
}

void DeviceDiagnosticResultListOnlyFunctionality::Initialize()
{
    exitWidget.setInfo(
        N_("Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResulFunctionality.ExitFunctionality"));
    disableExitButton();
    m_widget->refresh();
    m_widget->getSignalEnd().connect(
        sigc::mem_fun(*this,
                      &DeviceDiagnosticResultListOnlyFunctionality::endShowResult));
}

void DeviceDiagnosticResultListOnlyFunctionality::endShowResult()
{
    enableExitButton();
    ExitFunctionality();
}

Gtk::Widget* DeviceDiagnosticResultListOnlyFunctionality::GetFrame()
{
    DeviceDiagnosticResultConfiguration& resultConfiguration = \
        SAESGUIConfigurationManager::getInstance()->getDeviceDiagnosticResultConfiguration();
    int labelWidth  = resultConfiguration.getDiagnosticReviewWidth();
    int labelHeight = resultConfiguration.getDiagnosticReviewHeight();

    m_widget = new DiagnosticResultWidget(false, labelWidth, labelHeight);

    if (resultConfiguration.getDiagnosticReviewTimeToShow().present())
    {
        int timeToShow = resultConfiguration.getDiagnosticReviewTimeToShow().get();

        m_widget->setTimeToShow(timeToShow);
    }

    return m_widget;
}

std::string DeviceDiagnosticResultListOnlyFunctionality::GetWindowTitle()
{
    return N_("Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality.Title");
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType DeviceDiagnosticResultListOnlyFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICRESULTLISTONLYFUNCTIONALITY;
}

void DeviceDiagnosticResultListOnlyFunctionality::Dispose()
{
}

DeviceDiagnosticResultListOnlyFunctionality::~DeviceDiagnosticResultListOnlyFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * DeviceDiagnosticResultListOnlyFunctionality::getLogger()
{
    return m_logger;
}

Glib::ustring DeviceDiagnosticResultListOnlyFunctionality::getStartLogMessage()
{
    return N_("Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality.LogStartFunctionality");
}

Glib::ustring DeviceDiagnosticResultListOnlyFunctionality::getFinishLogMessage()
{
    return N_("Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticResultFunctionality.LogCloseFunctionality");
}

bool DeviceDiagnosticResultListOnlyFunctionality::ShowWindowTitle()
{
    return true;
}

bool DeviceDiagnosticResultListOnlyFunctionality::ForceExecute()
{
    return true;
}

