/**
 * @file DeviceDiagnosticSaveFunctionality.cxx
 * @brief Body device diagnostic result functionality class
 * @date 19/09/2012
 * @author Yadickson Soto
 */

#include "DeviceDiagnosticSaveFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/FactoryMethodWidget.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Voting/AudioVoting/AudioVotingCheckUtility.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

static const size_t TIME_TO_EXIT_WITH_NO_ERRORS = 1000;
static const size_t TIME_TO_EXIT_WITH_ERRORS    = 3000;

ISMTTLog * DeviceDiagnosticSaveFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticSaveFunctionality", GETTEXT_PACKAGE );

DeviceDiagnosticSaveFunctionality * DeviceDiagnosticSaveFunctionality::current_instance = NULL;
DeviceDiagnosticSaveFunctionality * DeviceDiagnosticSaveFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new DeviceDiagnosticSaveFunctionality();
    }

    return current_instance;
}

DeviceDiagnosticSaveFunctionality::DeviceDiagnosticSaveFunctionality()
{
    widget = NULL;
    m_exportThread = NULL;
}

void DeviceDiagnosticSaveFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }

    if ( m_exportThread )
    {
        m_exportThread->join();
        m_exportThread = NULL;
    }
}

void DeviceDiagnosticSaveFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticSaveFunctionality.ExitFunctionality" ) );
    disableExitButton();
    widget->setInfoText ( _ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticSaveFunctionality.Save" ) );
    widget->setInfoTitle ( "  " );
    widget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );

    if (m_exportThread != NULL)
    {
        m_exportThread->join();
        m_exportThread = NULL;
    }

    m_exportThread = Glib::Thread::create(sigc::mem_fun(*this, &DeviceDiagnosticSaveFunctionality::exportResults));
}

Gtk::Widget * DeviceDiagnosticSaveFunctionality::GetFrame()
{
    widget = FactoryMethodWidget::createProgressWidget();
    return widget;
}

std::string DeviceDiagnosticSaveFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticSaveFunctionality.Title" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType DeviceDiagnosticSaveFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICSAVEFUNCTIONALITY;
}

void DeviceDiagnosticSaveFunctionality::Dispose()
{
}

DeviceDiagnosticSaveFunctionality::~DeviceDiagnosticSaveFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * DeviceDiagnosticSaveFunctionality::getLogger()
{
    return logger;
}

Glib::ustring DeviceDiagnosticSaveFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticSaveFunctionality.LogStartFunctionality" );
}

Glib::ustring DeviceDiagnosticSaveFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticSaveFunctionality.LogCloseFunctionality" );
}

bool DeviceDiagnosticSaveFunctionality::ShowWindowTitle()
{
    return false;
}

void DeviceDiagnosticSaveFunctionality::exportResults()
{
    AudioVotingCheckUtility audioVotingCheckUtility;
    bool able20IsConnected = false;
    size_t timeToExit = TIME_TO_EXIT_WITH_NO_ERRORS;

    if ((audioVotingCheckUtility.ableIsConnected(true)) &&
        (audioVotingCheckUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED20))
    {
        able20IsConnected = true;
    }

    const bool success = MachineOperationManager::getInstance()->getOperationStatus()->persistDiagnosticFile(able20IsConnected);

    if (success == false)
    {
        timeToExit = TIME_TO_EXIT_WITH_ERRORS;
        widget->setInfoText ( _ ( "Smartmatic.SAES.GUI.Functionality.DeviceDiagnosticSaveFunctionality.ErrorPersistingResults" ) );
    }

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &DeviceDiagnosticSaveFunctionality::ExitFunctionality ), timeToExit );
}
