/**
 * @file DiagnosticsInfoFunctionality.cxx
 * @brief Body class diagnostic info functionality
 * @date 15/01/2014
 * @author Yadickson Soto
 */

#include "DiagnosticsInfoFunctionality.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::Functionality;

ISMTTLog * DiagnosticsInfoFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.DiagnosticsInfoFunctionality", GETTEXT_PACKAGE );

DiagnosticsInfoFunctionality * DiagnosticsInfoFunctionality::instance = NULL;

DiagnosticsInfoFunctionality::DiagnosticsInfoFunctionality()
{
    diagnosticsInfoWidget = NULL;
}

DiagnosticsInfoFunctionality::~DiagnosticsInfoFunctionality()
{
    deleteElements();
}

DiagnosticsInfoFunctionality * DiagnosticsInfoFunctionality::getInstance()
{
    if ( !instance )
    {
        instance = new DiagnosticsInfoFunctionality();
    }

    return instance;
}

std::string DiagnosticsInfoFunctionality::GetWindowTitle()
{
    return N_ ( "Diagnostics Info Functionality Title" );
}

Smartmatic::Log::ISMTTLog * DiagnosticsInfoFunctionality::getLogger()
{
    return logger;
}

Glib::ustring DiagnosticsInfoFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DiagnosticsInfoFunctionality.LogStartFunctionality" );
}

Glib::ustring DiagnosticsInfoFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DiagnosticsInfoFunctionality.LogCloseFunctionality" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType DiagnosticsInfoFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMDIAGNOSTICSINFOFUNCTIONALITY;
}

Gtk::Widget * DiagnosticsInfoFunctionality::GetFrame()
{
    diagnosticsInfoWidget = new DiagnosticsInfoWidget ();
    return diagnosticsInfoWidget;
}

void DiagnosticsInfoFunctionality::Dispose()
{
    connDiag.disconnect();
}

void DiagnosticsInfoFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.DiagnosticsInfoFunctionality.FinalMessage" ) );
    disableExitButton();
    connDiag.disconnect();
    connDiag = diagnosticsInfoWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &DiagnosticsInfoFunctionality::endDiagnosis ) );
    diagnosticsInfoWidget->start();
}

void DiagnosticsInfoFunctionality::endDiagnosis()
{
    ExitFunctionality();
}

void DiagnosticsInfoFunctionality::deleteElements()
{
    if ( diagnosticsInfoWidget )
    {
        delete diagnosticsInfoWidget;
        diagnosticsInfoWidget = NULL;
    }
}

void DiagnosticsInfoFunctionality::block()
{
    logger->Debug ( "Block functionality" );
}

void DiagnosticsInfoFunctionality::partial()
{
    logger->Debug ( "Partial functionality" );
}

void DiagnosticsInfoFunctionality::resume()
{
    logger->Debug ( "Resume functionality" );
}
