/**
 * @file DiagnosticsInfoFunctionality.hxx
 * @brief Header class diagnostics info functionality
 * @date 15/01/2014
 * @author Yadickson Soto
 */

#ifndef INFO_DIAGNOSE_FUNCTIONALITY_HXX_
#define INFO_DIAGNOSE_FUNCTIONALITY_HXX_

#include <gtkmm.h>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/DiagnosticsInfoWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	DiagnosticsInfoFunctionality
                 * @brief	Class diagnostic info functionality.
                 * @author	Yadickson Soto
                 * @date	15/01/2014
                 */
                class DiagnosticsInfoFunctionality : public GraphicDisplayFunctionality
                {
                    private:
                        /**
                         * @fn	DiagnosticsInfoFunctionality::DiagnosticsInfoFunctionality();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         */
                        DiagnosticsInfoFunctionality();

                        static DiagnosticsInfoFunctionality * instance; ///< instance of class
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                        Smartmatic::SAES::GUI::Widgets::DiagnosticsInfoWidget * diagnosticsInfoWidget; ///< diagnostics info widget
                        sigc::connection connDiag;	///< connection diagnosis

                    protected:

                        /**
                         * @fn	std::string DiagnosticsInfoFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         * @return	The window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType DiagnosticsInfoFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         * @return	The menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	Gtk::Widget * DiagnosticsInfoFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         * @return	null if it fails, else the frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	void DiagnosticsInfoFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         */
                        void Dispose();
                        /**
                         * @fn	void DiagnosticsInfoFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         */
                        void Initialize();
                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * DiagnosticsInfoFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         * @return	null if it fails, else the logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring DiagnosticsInfoFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         * @return	The start log message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring DiagnosticsInfoFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         * @return	The finish log message.
                         */
                        Glib::ustring getFinishLogMessage();
                        /**
                         * @fn	virtual void DiagnosticsInfoFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         */
                        virtual void deleteElements();
                        /**
                         * @fn	virtual void DiagnosticsInfoFunctionality::block();
                         * @brief	method block functionality.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         */
                        virtual void block();
                        /**
                         * @fn	virtual void DiagnosticsInfoFunctionality::partial();
                         * @brief	method partial functionality.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         */
                        virtual void partial();
                        /**
                         * @fn	virtual void DiagnosticsInfoFunctionality::resume();
                         * @brief	method resume functionality.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         */
                        virtual void resume();
                        /**
                         * @fn	void DiagnosticsInfoFunctionality::endDiagnosis();
                         * @brief	method end diagnosis.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         */
                        void endDiagnosis();

                    public:
                        /**
                         * @fn	static DiagnosticsInfoFunctionality * DiagnosticsInfoFunctionality::getInstance();
                         * @brief	Method getter instance.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         * @return	instance of class.
                         */
                        static DiagnosticsInfoFunctionality * getInstance();
                        /**
                         * @fn	virtual DiagnosticsInfoFunctionality::~DiagnosticsInfoFunctionality();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	15/01/2014
                         */
                        virtual ~DiagnosticsInfoFunctionality();
                };
            }
        }
    }
}
#endif /* INFO_DIAGNOSE_FUNCTIONALITY_HXX_ */
