/**
 * @file InitialCalibrationFunctionality.hxx
 * @brief Header initial calibration functionality class
 * @date 18/09/2012
 * @author Yadickson Soto
 */

#ifndef _INITIAL_CALIBRATION_FUNCTIONALITY_H_
#define _INITIAL_CALIBRATION_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/AppCalibrationWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	InitialCalibrationFunctionality
                 * @brief	initial calibration functionality class.
                 * @author	Yadickson Soto
                 * @date	18/09/2012
                 */
                class InitialCalibrationFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual InitialCalibrationFunctionality::~InitialCalibrationFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         */
                        virtual ~InitialCalibrationFunctionality();
                        /**
                         * @fn	static InitialCalibrationFunctionality * InitialCalibrationFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	instance of class.
                         */
                        static InitialCalibrationFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * InitialCalibrationFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring InitialCalibrationFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring InitialCalibrationFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        Smartmatic::SAES::GUI::Widgets::AppCalibrationWidget * diagnostic; ///< The calibration tool
                        static InitialCalibrationFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                        /**
                         * @fn	InitialCalibrationFunctionality::InitialCalibrationFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         */
                        InitialCalibrationFunctionality();
                        /**
                         * @fn	void InitialCalibrationFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void InitialCalibrationFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * InitialCalibrationFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string InitialCalibrationFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType InitialCalibrationFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	virtual void InitialCalibrationFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();
                        /**
                         * @fn	virtual bool InitialCalibrationFunctionality::ShowWindowTitle();
                         * @brief	Gets whether window title is visible.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         * @return	true is show window title.
                         */
                        virtual bool ShowWindowTitle();

                };
            }
        }
    }
}

#endif // _INITIAL_CALIBRATION_FUNCTIONALITY_H_
