/**
 * @file InitiatorFunctionality.cxx
 * @brief Body initiator functionality class
 * @date 03/10/2012
 * @author Yadickson Soto
 */

#include "InitiatorFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * InitiatorFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.InitiatorFunctionality", GETTEXT_PACKAGE );

InitiatorFunctionality * InitiatorFunctionality::current_instance = NULL;
InitiatorFunctionality * InitiatorFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new InitiatorFunctionality();
    }

    return current_instance;
}

InitiatorFunctionality::InitiatorFunctionality()
{
    widget = NULL;
}

void InitiatorFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void InitiatorFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.InitiatorFunctionality.ExitFunctionality" ) );
    widget->getSignalEnd().connect ( sigc::mem_fun ( this, &InitiatorFunctionality::end ) );
    widget->start();
    disableExitButton();
}

Gtk::Widget * InitiatorFunctionality::GetFrame()
{
    widget = AbstractFactoryWidget::createInitiatorWidget();
    return widget;
}

std::string InitiatorFunctionality::GetWindowTitle()
{
    return widget->getTitleText();
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType InitiatorFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMINITIATORFUNCTIONALITY;
}

void InitiatorFunctionality::Dispose()
{
}

void InitiatorFunctionality::end()
{
    ExitFunctionality();
}

InitiatorFunctionality::~InitiatorFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * InitiatorFunctionality::getLogger()
{
    return logger;
}

Glib::ustring InitiatorFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.InitiatorFunctionality.LogStartFunctionality" );
}

Glib::ustring InitiatorFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.InitiatorFunctionality.LogCloseFunctionality" );
}

bool InitiatorFunctionality::ForceExecute()
{
    return true;
}
