/**
 * @file LanguageInstallFunctionality.cxx
 * @brief Body language installation functionality class
 * @date 18/09/2012
 * @author Yadickson Soto
 */

#include "LanguageInstallFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Environment/Configurations/base-language-controller.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * LanguageInstallFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.LanguageInstallFunctionality", GETTEXT_PACKAGE );

LanguageInstallFunctionality * LanguageInstallFunctionality::current_instance = NULL;
LanguageInstallFunctionality * LanguageInstallFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new LanguageInstallFunctionality();
    }

    return current_instance;
}

LanguageInstallFunctionality::LanguageInstallFunctionality()
{
    changeLanguageWidget = NULL;
}

void LanguageInstallFunctionality::deleteElements()
{
    if ( changeLanguageWidget )
    {
        delete changeLanguageWidget;
        changeLanguageWidget = NULL;
    }
}

void LanguageInstallFunctionality::Initialize()
{
    changeLanguageWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &LanguageInstallFunctionality::onLanguageChange ) );
    changeLanguageWidget->setSaveFile ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getPersistLanguage() );
    changeLanguageWidget->start ( false );
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.LanguageInstallFunctionality.LanguageNotChanged" ) );
    enableExitButton();
}

Gtk::Widget * LanguageInstallFunctionality::GetFrame()
{
    changeLanguageWidget = new LanguageInstallWidget();
    return changeLanguageWidget;
}

std::string LanguageInstallFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.LanguageInstallFunctionality.Title" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType LanguageInstallFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMLANGUAGEINSTALLFUNCTIONALITY;
}

void LanguageInstallFunctionality::Dispose()
{
}

void LanguageInstallFunctionality::onLanguageChange()
{
    if ( BaseLanguageController::isSetLanguage() )
    {
        exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.LanguageInstallFunctionality.LanguageChanged" ) );
    }
    else
    {
        exitWidget.setInfo ( " " );
    }

    RefreshInfo();
    ExitFunctionality();
}

LanguageInstallFunctionality::~LanguageInstallFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * LanguageInstallFunctionality::getLogger()
{
    return logger;
}

Glib::ustring LanguageInstallFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.LanguageInstallFunctionality.LogStartFunctionality" );
}

Glib::ustring LanguageInstallFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.LanguageInstallFunctionality.LogCloseFunctionality" );
}

bool LanguageInstallFunctionality::ShowWindowTitle()
{
    return BaseLanguageController::isSetLanguage();
}
