/**
 * @file	LogReportFunctionality.cc
 * @brief	log report functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "LogReportFunctionality.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Log/LogReader.h>
#include <System/Exception/LogReaderException.h>
#include <System/IO/File.h>
#include <Runtime/Environment/saes-directories.h>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Log;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;

LogReportFunctionality * LogReportFunctionality::current_instance = NULL;

ISMTTLog * LogReportFunctionality::logger =
    SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.ReportLogFunctionality",
                                GETTEXT_PACKAGE );

LogReportFunctionality * LogReportFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new LogReportFunctionality();
    }

    return current_instance;
}

LogReportFunctionality::LogReportFunctionality()
{
    genericPrintingWidget = NULL;
    logReaderAudit = NULL;
    logReaderError = NULL;
    reportLogWidget = NULL;
}

LogReportFunctionality::~LogReportFunctionality()
{
    deleteElements();
}

void LogReportFunctionality::deleteElements()
{
    if ( logReaderAudit )
    {
        delete logReaderAudit;
        logReaderAudit = NULL;
    }

    if ( logReaderError )
    {
        delete logReaderError;
        logReaderError = NULL;
    }

    if ( reportLogWidget )
    {
        delete reportLogWidget;
        reportLogWidget = NULL;
    }

    if ( genericPrintingWidget )
    {
        delete genericPrintingWidget;
        genericPrintingWidget = NULL;
    }
}

void LogReportFunctionality::Initialize()
{
    enableExitButton();
}

Gtk::Widget * LogReportFunctionality::GetFrame()
{
    SaesDirectories directories;
    std::string auditPath = directories.getLogAuditFileFullPath();
    std::string errorPath = directories.getLogErrorFileFullPath();
    this->pageSize = SAESGUIConfigurationManager::getInstance()->GetLogReportFunctionalityConfiguration().getPageSize();
    LogConfigurationReader & logConf ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getLogConfigurationReader() );

    logReaderAudit = new LogReader ( auditPath, pageSize, logConf.getLogAuditReader() );
    logReaderError = new LogReader ( errorPath, pageSize, logConf.getLogErrorReader() );

    genericPrintingWidget = AbstractFactoryWidget::createGenericPrintingWidget();
    reportLogWidget = new ReportLogWidget ( logReaderAudit, logReaderError, genericPrintingWidget );
    reportLogWidget->on_print_begin().connect ( sigc::mem_fun ( *this, &LogReportFunctionality::on_disable_exit ) );
    reportLogWidget->on_print_end().connect ( sigc::mem_fun ( *this, &LogReportFunctionality::on_enable_exit ) );

    return reportLogWidget;
}

void LogReportFunctionality::on_disable_exit()
{
    setExitButtonSensitive ( false );
    setHelpButtonSensitive ( false );
}

void LogReportFunctionality::on_enable_exit()
{
    setExitButtonSensitive ( true );
    setHelpButtonSensitive ( true );
}

void LogReportFunctionality::on_close_clicked()
{
    ExitFunctionality();
}

std::string LogReportFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ReportLogFunctionality.ReportLogTitle" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType LogReportFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMLOGREPORTFUNCTIONALITY;
}

void LogReportFunctionality::Dispose()
{
}

Smartmatic::Log::ISMTTLog * LogReportFunctionality::getLogger()
{
    return logger;
}
Glib::ustring LogReportFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.LogReportFunctionality.LogStartFunctionality" );
}

Glib::ustring LogReportFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.LogReportFunctionality.LogCloseFunctionality" );
}

