/**
 * @file MenuFunctionality.cxx
 * @brief Body menu functionality class
 * @date 21/09/2012
 * @author Yadickson Soto
 */

#include "MenuFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/IO/File.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/FunctionalityWindowManager.hxx>
#include <Gui/Windows/VoteProcessWindow.hxx>
#include <Gui/Windows/BarcodeReconfigureWindow.hxx>
#include <Gui/Windows/DeviceMonitorWindow.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Operation;

ISMTTLog * MenuFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.MenuFunctionality", GETTEXT_PACKAGE );

MenuFunctionality * MenuFunctionality::current_instance = NULL;
MenuFunctionality * MenuFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new MenuFunctionality();
    }

    return current_instance;
}

MenuFunctionality::MenuFunctionality()
{
    widget = NULL;
}

void MenuFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void MenuFunctionality::Initialize()
{
    VoteProcessWindow::getInstance()->start();
    BarcodeReconfigureWindow::getInstance()->start();
    DeviceMonitorWindow::getInstance()->start();
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.MenuFunctionality.ExitFunctionality" ) );
    enableExitButton();
}

Gtk::Widget * MenuFunctionality::GetFrame()
{
    widget = new MenuWidget();
    widget->reloadMenu = true;
    widget->refresh();
    return widget;
}

std::string MenuFunctionality::GetWindowTitle()
{
    return widget->getTitleText();
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType MenuFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMENU;
}

void MenuFunctionality::Dispose()
{
}

MenuFunctionality::~MenuFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * MenuFunctionality::getLogger()
{
    return logger;
}

Glib::ustring MenuFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.MenuFunctionality.LogStartFunctionality" );
}

Glib::ustring MenuFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.MenuFunctionality.LogCloseFunctionality" );
}

void MenuFunctionality::refresh()
{
    widget->refresh();
    GraphicDisplayFunctionality::refresh();
}

void MenuFunctionality::onSetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value )
{
    if ( widget )
    {
        widget->SetKey ( value );
    }
}
