/**
 * @file MouseFunctionality.hxx
 * @brief Header mouse functionality class
 * @date 22/03/2019
 * @author eduardo.fernandez
 */

#ifndef _MOUSE_FUNCTIONALITY_HXX
#define _MOUSE_FUNCTIONALITY_HXX

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/MouseWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	MouseFunctionality
                 * @brief	mouse functionality class.
                 * @author	eduardo.fernandez
                 * @date	22/03/2019
                 */
                class MouseFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual MouseFunctionality::~MouseFunctionality();
                         * @brief	class destructor.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         */
                        virtual ~MouseFunctionality();
                        /**
                         * @fn	static MouseFunctionality * MouseFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         * @return	instance of class.
                         */
                        static MouseFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * MouseFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring MouseFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring MouseFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static MouseFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Smartmatic::SAES::GUI::Widgets::MouseWidget * widget; ///< The widget

                        /**
                         * @fn	MouseFunctionality::MouseFunctionality();
                         * @brief	class constructor.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         */
                        MouseFunctionality();
                        /**
                         * @fn	void MouseFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         */
                        void Dispose();
                        /**
                         * @fn	void MouseFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * MouseFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string MouseFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType MouseFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	virtual void MouseFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author  eduardo.fernandez
                         * @date    22/03/2019
                         */
                        virtual void deleteElements();
                };
            }
        }
    }
}

#endif // _MOUSE_FUNCTIONALITY_HXX
