/**
 * @file ProcessStatusResultsShutdownFunctionality.hxx
 * @brief Header voting machine shutdown class
 * @date 08/11/2012
 * @author Yadickson Soto
 */

#ifndef _PROCESS_STATUS_RESULT_SHUTDOWN_FUNCTIONALITY_HXX
#define _PROCESS_STATUS_RESULT_SHUTDOWN_FUNCTIONALITY_HXX

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/IconShutdownWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	ProcessStatusResultsShutdownFunctionality
                 * @brief	auto diagnostic functionality class.
                 * @author	Yadickson Soto
                 * @date	08/11/2012
                 */
                class ProcessStatusResultsShutdownFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual ProcessStatusResultsShutdownFunctionality::~ProcessStatusResultsShutdownFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         */
                        virtual ~ProcessStatusResultsShutdownFunctionality();
                        /**
                         * @fn	static ProcessStatusResultsShutdownFunctionality * ProcessStatusResultsShutdownFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         * @return	instance of class.
                         */
                        static ProcessStatusResultsShutdownFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * ProcessStatusResultsShutdownFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring ProcessStatusResultsShutdownFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring ProcessStatusResultsShutdownFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static ProcessStatusResultsShutdownFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Smartmatic::SAES::GUI::Widgets::IconShutdownWidget * widget; ///< The widget

                        /**
                         * @fn	ProcessStatusResultsShutdownFunctionality::ProcessStatusResultsShutdownFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         */
                        ProcessStatusResultsShutdownFunctionality();
                        /**
                         * @fn	void ProcessStatusResultsShutdownFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void ProcessStatusResultsShutdownFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * ProcessStatusResultsShutdownFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string ProcessStatusResultsShutdownFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	virtual bool ProcessStatusResultsShutdownFunctionality::ShowWindowTitle()
                         * @brief	getter isshow window title.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         * @return	true is show window title.
                         */
                        virtual bool ShowWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType ProcessStatusResultsShutdownFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	void ProcessStatusResultsShutdownFunctionality::end();
                         * @brief	method end diagnostic.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         */
                        void end();
                        /**
                         * @fn	virtual void ProcessStatusResultsShutdownFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	08/11/2012
                         */
                        virtual void deleteElements();

                        /**
                         * @fn	virtual bool ForceExecute();
                         * @brief	Method force execute functionality
                         * @author	Yadickson Soto
                         * @date	16/11/2012
                         * @return	true if force execute
                         */
                        virtual bool ForceExecute();
                };
            }
        }
    }
}

#endif // _PROCESS_STATUS_RESULT_SHUTDOWN_FUNCTIONALITY_HXX
