/*
 * RecountFunctionalityController.h
 *
 *  Created on: Feb 13, 2017
 *      Author: Juan Valera
 */

#ifndef RECOUNTFUNCTIONALITYCONTROLLER_H_
#define RECOUNTFUNCTIONALITYCONTROLLER_H_

#include <Gui/Widgets/RecountWidget.h>
#include <Operation/Voting/VotePersistence.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {

                class RecountFunctionalityController
                {

                   public:
                      /**
                      * @fn	RecountFunctionalityController::RecountFunctionalityController();
                      * @brief	Class constructor.
                      * @author	Juan Valera
                      * @date	13/02/2017
                      */
                       RecountFunctionalityController();

                       /**
                       * @fn	RecountFunctionalityController::RecountFunctionalityController(Smartmatic::SAES::GUI::Widgets::RecountWidget * widget);
                       * @brief	Class constructor.
                       * @author	Juan Valera
                       * @date	13/02/2017
                       */
                        explicit RecountFunctionalityController(Smartmatic::SAES::GUI::Widgets::RecountWidget * widgetC);
                    private:
                        Smartmatic::SAES::GUI::Widgets::RecountWidget * widget; ///< The widget
                        sigc::connection signalStatusVote;
                        sigc::connection signalVote;

                        /**
                         * @fn	void RecountFunctionalityController::onSignalState(Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status);
                         * @brief	Callback on signal vote.
                         * @author	Juan Valera
                         * @date	13/02/2017
                         */
                        void onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );
                        /**
                         * @fn	void RecountFunctionalityController::onSignalVote(Smartmatic::SAES::Voting::Vote::Vote * vote);
                         * @brief	Callback on signal vote.
                         * @author	Juan Valera
                         * @date	13/02/2017
                         */
                        void onSignalVote ( Smartmatic::SAES::Voting::Vote::Vote * vote );

                        Smartmatic::Log::ISMTTLog * getLogger();
                    public:
                        /**
                         * Destructor
                         */
                        virtual ~RecountFunctionalityController();
                        /**
                         * @fn	void RecountFunctionalityController::Initialize();
                         * @brief	Initializes this object.
                         * @author	Juan Valera
                         * @date	13/02/2017
                         */
                        void Initialize();
                        /**
                         * @fn	void RecountFunctionalityController::Stop();
                         * @brief	stop this object.
                         * @author	Juan Valera
                         * @date	13/02/2017
                         */
                        void Stop();

                        /**
                         * @brief  Pause the recount functionality.
                         * @author manuel.arguelles
                         * @date   Apr  2, 2018
                         */
                        void pause();

                        /**
                         * @brief  Resume the recount functionality.
                         * @author manuel.arguelles
                         * @date   Apr  2, 2018
                         */
                        void resume();
                };

            } /* namespace Functionality */
        } /* namespace GUI */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* RECOUNTFUNCTIONALITYCONTROLLER_H_ */
