/**
 * @file ReportInstallFunctionality.cxx
 * @brief Body report install functionality class
 * @date 15/11/2012
 * @author Yadickson Soto
 */

#include "ReportInstallFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <System/Runtime/PowerDown.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::System::Runtime;

ISMTTLog * ReportInstallFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.ReportInstallFunctionality", GETTEXT_PACKAGE );

ReportInstallFunctionality * ReportInstallFunctionality::current_instance = NULL;
ReportInstallFunctionality * ReportInstallFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new ReportInstallFunctionality();
    }

    return current_instance;
}

ReportInstallFunctionality::ReportInstallFunctionality()
{
    widget = NULL;
}

void ReportInstallFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void ReportInstallFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.ReportInstallFunctionality.ExitFunctionality" ) );

    if ( !MachineOperationManager::getInstance()->isPrintingInstallReport() )
    {
        logger->Debug ( "Start report install" );
        widget->start ( true, true, true );
    }
    else
    {
        end();
    }

    disableExitButton();
}

Gtk::Widget * ReportInstallFunctionality::GetFrame()
{
    widget = AbstractFactoryWidget::createReportInstallWidget();
    widget->getSignalEnd().connect ( sigc::mem_fun ( this, &ReportInstallFunctionality::end ) );
    return widget;
}

std::string ReportInstallFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ReportInstallFunctionality.Title" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType ReportInstallFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMREPORTINSTALLFUNCTIONALITY;
}

void ReportInstallFunctionality::Dispose()
{
}

void ReportInstallFunctionality::end()
{
    if ( MachineOperationManager::getInstance()->isPrintingInstallReport() )
    {
        ExitFunctionality();
    }
    else
    {
        PowerDown::shutdown();
    }
}

ReportInstallFunctionality::~ReportInstallFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * ReportInstallFunctionality::getLogger()
{
    return logger;
}

Glib::ustring ReportInstallFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ReportInstallFunctionality.LogStartFunctionality" );
}

Glib::ustring ReportInstallFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ReportInstallFunctionality.LogCloseFunctionality" );
}
