/**
 * @file ReprintReportInstallFunctionality.cxx
 * @brief Body class printer report install functionality
 * @date 05/09/2012
 * @author Yadickson Soto
 */

#include "ReprintReportInstallFunctionality.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::SAES::Operation;

ISMTTLog * ReprintReportInstallFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.ReprintReportInstallFunctionality", GETTEXT_PACKAGE );

ReprintReportInstallFunctionality * ReprintReportInstallFunctionality::instance = NULL;

ReprintReportInstallFunctionality::ReprintReportInstallFunctionality()
{
    widget = NULL;
}

ReprintReportInstallFunctionality::~ReprintReportInstallFunctionality()
{
    deleteElements();
}

ReprintReportInstallFunctionality * ReprintReportInstallFunctionality::getInstance()
{
    if ( !instance )
    {
        instance = new ReprintReportInstallFunctionality();
    }

    return instance;
}

std::string ReprintReportInstallFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ReprintReportInstallFunctionality.Title" );
}

Smartmatic::Log::ISMTTLog * ReprintReportInstallFunctionality::getLogger()
{
    return logger;
}

Glib::ustring ReprintReportInstallFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ReprintReportInstallFunctionality.LogStartFunctionality" );
}

Glib::ustring ReprintReportInstallFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ReprintReportInstallFunctionality.LogCloseFunctionality" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType ReprintReportInstallFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMREPRINTINSTALLFUNCTIONALITY;
}

Gtk::Widget * ReprintReportInstallFunctionality::GetFrame()
{
    widget = AbstractFactoryWidget::createReportInstallWidget();
    return widget;
}

void ReprintReportInstallFunctionality::Dispose()
{

}

void ReprintReportInstallFunctionality::Initialize()
{
    disableExitButton();
    widget->start ( false, false, false );
    widget->getSignalEnd().connect ( sigc::mem_fun ( this, &ReprintReportInstallFunctionality::ExitFunctionality ) );
}

void ReprintReportInstallFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void ReprintReportInstallFunctionality::block()
{
    logger->Debug ( "Block functionality" );
}

void ReprintReportInstallFunctionality::partial()
{
    logger->Debug ( "Partial functionality" );
}

void ReprintReportInstallFunctionality::resume()
{
    logger->Debug ( "Resume functionality" );
}
