/**
 * @file ResultsPackageTransmissionFunctionality.cxx
 * @brief Body results package transmission functionality class
 * @date 07/04/2014
 * @author Yadickson Soto
 */

#include <Functionalities/ResultsPackageTransmissionFunctionality.hxx>
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/FactoryMethodWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * ResultsPackageTransmissionFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.ResultsPackageTransmissionFunctionality", GETTEXT_PACKAGE );

ResultsPackageTransmissionFunctionality * ResultsPackageTransmissionFunctionality::instance = NULL;

ResultsPackageTransmissionFunctionality * ResultsPackageTransmissionFunctionality::getInstance()
{
    if ( !instance )
    {
        instance = new ResultsPackageTransmissionFunctionality();
    }

    return instance;
}

ResultsPackageTransmissionFunctionality::ResultsPackageTransmissionFunctionality()
{
    widget = NULL;
}

void ResultsPackageTransmissionFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void ResultsPackageTransmissionFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.ResultsPackageTransmissionFunctionality.ExitFunctionality" ) );
    widget->getSignalEnd().connect ( sigc::mem_fun ( this, &ResultsPackageTransmissionFunctionality::ExitFunctionality ) );

    disableExitButton();
    widget->initialize();
}

Gtk::Widget * ResultsPackageTransmissionFunctionality::GetFrame()
{
    widget = FactoryMethodWidget::createResultsPackageTransmissionWidget();
    return widget;
}

std::string ResultsPackageTransmissionFunctionality::GetWindowTitle()
{
    return N_ ( "Results Package Transmission Title" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType ResultsPackageTransmissionFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMRESULTSPACKAGETRANSMISSIONFUNCTIONALITY;
}

void ResultsPackageTransmissionFunctionality::Dispose()
{

}

ResultsPackageTransmissionFunctionality::~ResultsPackageTransmissionFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * ResultsPackageTransmissionFunctionality::getLogger()
{
    return logger;
}

Glib::ustring ResultsPackageTransmissionFunctionality::getStartLogMessage()
{
    return N_ ( "Starting Results Package Transmission Functionality" );
}

Glib::ustring ResultsPackageTransmissionFunctionality::getFinishLogMessage()
{
    return N_ ( "Closing Results Package Transmission Functionality" );
}
