/**
 * @file ResultsPackageTransmissionFunctionality.hxx
 * @brief Header results package transmission functionality class
 * @date 21/09/2012
 * @author Yadickson Soto
 */

#ifndef SENDRESULTSPACKAGEFUNCTIONALITY_HXX_
#define SENDRESULTSPACKAGEFUNCTIONALITY_HXX_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Transmission/ResultsPackageTransmissionWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	ResultsPackageTransmissionFunctionality
                 * @brief	Results package transmission functionality class.
                 * @author	Yadickson Soto
                 * @date	21/09/2012
                 */
                class ResultsPackageTransmissionFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual ResultsPackageTransmissionFunctionality::~ResultsPackageTransmissionFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        virtual ~ResultsPackageTransmissionFunctionality();
                        /**
                         * @fn	static ResultsPackageTransmissionFunctionality * ResultsPackageTransmissionFunctionality::getInstance();
                         * @brief	Singleton instance getter method..
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	instance of class.
                         */
                        static ResultsPackageTransmissionFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * ResultsPackageTransmissionFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring ResultsPackageTransmissionFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring ResultsPackageTransmissionFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static ResultsPackageTransmissionFunctionality * instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                        Smartmatic::SAES::GUI::Widgets::ResultsPackageTransmissionWidget * widget; ///< The widget

                        /**
                         * @fn	ResultsPackageTransmissionFunctionality::ResultsPackageTransmissionFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        ResultsPackageTransmissionFunctionality();
                        /**
                         * @fn	void ResultsPackageTransmissionFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void ResultsPackageTransmissionFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * ResultsPackageTransmissionFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string ResultsPackageTransmissionFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType ResultsPackageTransmissionFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	virtual void ResultsPackageTransmissionFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();
                };
            }
        }
    }
}

#endif /* SENDRESULTSPACKAGEFUNCTIONALITY_HXX_ */
