/**
 * @file SerialNumberFunctionality.hxx
 * @brief Header serial number functionality class
 * @date 21/09/2012
 * @author Yadickson Soto
 */

#ifndef _SERIAL_NUMBER_FUNCTIONALITY_HXX
#define _SERIAL_NUMBER_FUNCTIONALITY_HXX

#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/SerialNumberWidget.hxx>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	SerialNumberFunctionality
                 * @brief	serial number functionality class.
                 * @author	Yadickson Soto
                 * @date	21/09/2012
                 */
                class SerialNumberFunctionality : public GraphicDisplayFunctionality, private Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @fn	virtual SerialNumberFunctionality::~SerialNumberFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        virtual ~SerialNumberFunctionality();
                        /**
                         * @fn	static SerialNumberFunctionality * SerialNumberFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	instance of class.
                         */
                        static SerialNumberFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * SerialNumberFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring SerialNumberFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring SerialNumberFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static SerialNumberFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                        Smartmatic::SAES::GUI::Widgets::SerialNumberWidget * widget; ///< The widget

                        bool processSerial; ///< True to gets and false to safe
                        std::string m_serial; ///> The serial to safe

                        Smartmatic::System::SafeDispatcher signalGetDispatcher; ///< signal dispatcher end get serial
                        Smartmatic::System::SafeDispatcher signalSaveDispatcher; ///< signal dispatcher end save serial

                        /**
                         * @fn	SerialNumberFunctionality::SerialNumberFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        SerialNumberFunctionality();
                        /**
                         * @fn	void SerialNumberFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void SerialNumberFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * SerialNumberFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string SerialNumberFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType SerialNumberFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                        /**
                         * @fn	    virtual bool process ();
                         * @brief	Callback from thread any time. Gets the serial number in background
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @return true for continue thread
                         */
                        virtual bool process ();

                        /**
                         * @fn	    void onGetSerial ( void * object );
                         * @brief	Callback when emit sync signal. End get saerial
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @param 	object The object to process
                         */
                        void onGetSerial ( void * object );

                        /**
                         * @fn	    void onSaveSerial ( void * object );
                         * @brief	Callback when emit sync signal. End save serial
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @param 	object The object to process
                         */
                        void onSaveSerial ( void * object );

                        /**
                         * @fn	void SerialNumberFunctionality::saveSerial();
                         * @brief	method save serial number.
                         * @author	Yadickson Soto
                         * @date	21/09/2012
                         */
                        void saveSerial();

                        /**
                         * @fn	virtual void SerialNumberFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();
                };
            }
        }
    }
}

#endif // _SERIAL_NUMBER_FUNCTIONALITY_HXX
