/**
 * @file SimulatorVotesFunctionality.cxx
 * @brief Body simulator votes functionality class
 * @date 05/10/2012
 * @author Yadickson Soto
 */

#include "SimulatorVotesFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/Voting/FactoryVotingWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * SimulatorVotesFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.SimulatorVotesFunctionality", GETTEXT_PACKAGE );

SimulatorVotesFunctionality * SimulatorVotesFunctionality::current_instance = NULL;
SimulatorVotesFunctionality * SimulatorVotesFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new SimulatorVotesFunctionality();
    }

    return current_instance;
}

SimulatorVotesFunctionality::SimulatorVotesFunctionality()
{
    widget = NULL;
}

void SimulatorVotesFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void SimulatorVotesFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.SimulatorVotesFunctionality.ExitFunctionality" ) );
    widget->getSignalEnd().connect ( sigc::mem_fun ( this, &SimulatorVotesFunctionality::end ) );
    widget->stopPrint.connect ( sigc::mem_fun ( this, &SimulatorVotesFunctionality::enableExitButton ) );
    widget->startPrint.connect ( sigc::mem_fun ( this, &SimulatorVotesFunctionality::disableExitButton ) );
    enableExitButton();
}

Gtk::Widget * SimulatorVotesFunctionality::GetFrame()
{
    widget = FactoryVotingWidget::createSimulatorVotesWidget();
    return widget;
}

std::string SimulatorVotesFunctionality::GetWindowTitle()
{
    return widget->getTitleText();
}

bool SimulatorVotesFunctionality::ShowWindowTitle()
{
    return widget->isTitle();
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType SimulatorVotesFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMSIMULATORVOTESFUNCTIONALITY;
}

void SimulatorVotesFunctionality::Dispose()
{
}

void SimulatorVotesFunctionality::end()
{
    ExitFunctionality();
}

SimulatorVotesFunctionality::~SimulatorVotesFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * SimulatorVotesFunctionality::getLogger()
{
    return logger;
}

Glib::ustring SimulatorVotesFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.SimulatorVotesFunctionality.LogStartFunctionality" );
}

Glib::ustring SimulatorVotesFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.SimulatorVotesFunctionality.LogCloseFunctionality" );
}

void SimulatorVotesFunctionality::refresh()
{
    widget->refresh();
    GraphicDisplayFunctionality::refresh();
}
