/**
 * @file SkipVoteProcessDemoFunctionality.hxx
 * @brief Header skip vote demo functionality class
 * @date 22/10/2012
 * @author Yadickson Soto
 */

#ifndef _SKIP_VOTE_PROCESS_DEMO_FUNCTIONALITY_H_
#define _SKIP_VOTE_PROCESS_DEMO_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	SkipVoteProcessDemoFunctionality
                 * @brief	skip vote demo functionality class.
                 * @author	Yadickson Soto
                 * @date	22/10/2012
                 */
                class SkipVoteProcessDemoFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual SkipVoteProcessDemoFunctionality::~SkipVoteProcessDemoFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         */
                        virtual ~SkipVoteProcessDemoFunctionality();
                        /**
                         * @fn	static SkipVoteProcessDemoFunctionality * SkipVoteProcessDemoFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         * @return	instance of class.
                         */
                        static SkipVoteProcessDemoFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * SkipVoteProcessDemoFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring SkipVoteProcessDemoFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring SkipVoteProcessDemoFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static SkipVoteProcessDemoFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Gtk::Frame * widget; ///< widget

                        /**
                         * @fn	SkipVoteProcessDemoFunctionality::SkipVoteProcessDemoFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         */
                        SkipVoteProcessDemoFunctionality();
                        /**
                         * @fn	void SkipVoteProcessDemoFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void SkipVoteProcessDemoFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * SkipVoteProcessDemoFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string SkipVoteProcessDemoFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType SkipVoteProcessDemoFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	virtual void SkipVoteProcessDemoFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         */
                        virtual void deleteElements();
                };
            }
        }
    }
}

#endif // _SKIP_VOTE_PROCESS_DEMO_FUNCTIONALITY_H_
