/**
 * @file SkipVotingMachineInstallationFunctionality.cxx
 * @brief Body skip install machine functionality class
 * @date 18/10/2012
 * @author Yadickson Soto
 */

#include "SkipVotingMachineInstallationFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/IO/File.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * SkipVotingMachineInstallationFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.SkipVotingMachineInstallationFunctionality", GETTEXT_PACKAGE );

SkipVotingMachineInstallationFunctionality * SkipVotingMachineInstallationFunctionality::current_instance = NULL;
SkipVotingMachineInstallationFunctionality * SkipVotingMachineInstallationFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new SkipVotingMachineInstallationFunctionality();
    }

    return current_instance;
}

SkipVotingMachineInstallationFunctionality::SkipVotingMachineInstallationFunctionality()
{
    widget = NULL;
}

void SkipVotingMachineInstallationFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void SkipVotingMachineInstallationFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.SkipVotingMachineInstallationFunctionality.ErrorSkipInstallMachine" ) );
    disableExitButton();

    try
    {
        if ( MachineOperationManager::getInstance()->isNotInstalled() )
        {
            exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.SkipVotingMachineInstallationFunctionality.SkipInstallMachine" ) );
            MachineOperationManager::getInstance()->skipInstallMachine();
        }
        else
        {
            exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.SkipVotingMachineInstallationFunctionality.NotSkipInstallMachine" ) );
        }
    }
    catch ( ... )
    {
        logger->Debug ( "ERROR - skip install machine" );
    }

    ExitFunctionality();
}

Gtk::Widget * SkipVotingMachineInstallationFunctionality::GetFrame()
{
    widget = new Gtk::Frame();
    return widget;
}

std::string SkipVotingMachineInstallationFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.SkipVotingMachineInstallationFunctionality.Title" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType SkipVotingMachineInstallationFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMSKIPVOTINGMACHINEINSTALLATIONFUNCTIONALITY;
}

void SkipVotingMachineInstallationFunctionality::Dispose()
{
}

SkipVotingMachineInstallationFunctionality::~SkipVotingMachineInstallationFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * SkipVotingMachineInstallationFunctionality::getLogger()
{
    return logger;
}

Glib::ustring SkipVotingMachineInstallationFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.SkipVotingMachineInstallationFunctionality.LogStartFunctionality" );
}

Glib::ustring SkipVotingMachineInstallationFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.SkipVotingMachineInstallationFunctionality.LogCloseFunctionality" );
}
