/**
 * @file SkipVotingMachineInstallationFunctionality.hxx
 * @brief Header skip install machine functionality class
 * @date 18/10/2012
 * @author Yadickson Soto
 */

#ifndef _SKIP_INSTALL_MACHINE_FUNCTIONALITY_H_
#define _SKIP_INSTALL_MACHINE_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	SkipVotingMachineInstallationFunctionality
                 * @brief	skip install machine functionality class.
                 * @author	Yadickson Soto
                 * @date	18/10/2012
                 */
                class SkipVotingMachineInstallationFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	virtual SkipVotingMachineInstallationFunctionality::~SkipVotingMachineInstallationFunctionality();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         */
                        virtual ~SkipVotingMachineInstallationFunctionality();
                        /**
                         * @fn	static SkipVotingMachineInstallationFunctionality * SkipVotingMachineInstallationFunctionality::getInstance();
                         * @brief	Singleton instance getter method.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         * @return	instance of class.
                         */
                        static SkipVotingMachineInstallationFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * SkipVotingMachineInstallationFunctionality::getLogger();
                         * @brief	Logger getter method.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         * @return	logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring SkipVotingMachineInstallationFunctionality::getStartLogMessage();
                         * @brief	Gets the logger starting message.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         * @return	message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring SkipVotingMachineInstallationFunctionality::getFinishLogMessage();
                         * @brief	Gets the logger ending message.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         * @return	message.
                         */
                        Glib::ustring getFinishLogMessage();

                    private:

                        static SkipVotingMachineInstallationFunctionality * current_instance;  ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Gtk::Frame * widget; ///< widget

                        /**
                         * @fn	SkipVotingMachineInstallationFunctionality::SkipVotingMachineInstallationFunctionality();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         */
                        SkipVotingMachineInstallationFunctionality();
                        /**
                         * @fn	void SkipVotingMachineInstallationFunctionality::Dispose();
                         * @brief	method dispose.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         */
                        void Dispose();
                        /**
                         * @fn	void SkipVotingMachineInstallationFunctionality::Initialize();
                         * @brief	method initialize.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         */
                        void Initialize();
                        /**
                         * @fn	Gtk::Widget * SkipVotingMachineInstallationFunctionality::GetFrame();
                         * @brief	Frame getter method.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         * @return	new frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	std::string SkipVotingMachineInstallationFunctionality::GetWindowTitle();
                         * @brief	Window title getter method.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         * @return	the window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType SkipVotingMachineInstallationFunctionality::GetMenuOptionType();
                         * @brief	Menu option type getter method.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         * @return	the menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	virtual void SkipVotingMachineInstallationFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	18/10/2012
                         */
                        virtual void deleteElements();
                };
            }
        }
    }
}

#endif // _SKIP_INSTALL_MACHINE_FUNCTIONALITY_H_
