/**
 * @file SystemFilesIntegrityCheckFunctionality.hxx
 * @brief Header class system files integrity check
 * @date 15/01/2019
 * @author eduardo.fernandez
 */

#ifndef SYSTEM_FILES_INTEGRITY_CHECK_FUNCTIONALITY_HXX_
#define SYSTEM_FILES_INTEGRITY_CHECK_FUNCTIONALITY_HXX_

#include <gtkmm.h>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/SystemFilesIntegrityCheckWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	SystemFilesIntegrityCheckFunctionality
                 * @brief	Class system files integrity check.
                 * @author eduardo.fernandez
                 * @date 15/01/2019
                 */
                class SystemFilesIntegrityCheckFunctionality : public GraphicDisplayFunctionality
                {
                    private:
                        /**
                         * @fn	SystemFilesIntegrityCheckFunctionality::SystemFilesIntegrityCheckFunctionality();
                         * @brief	Class constructor.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        SystemFilesIntegrityCheckFunctionality();

                        static SystemFilesIntegrityCheckFunctionality * instance; ///< instance of class
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                        Smartmatic::SAES::GUI::Widgets::SystemFilesIntegrityCheckWidget * systemFilesIntegrityCheckWidget; ///< system files integrity check widget

                    protected:

                        /**
                         * @fn	std::string SystemFilesIntegrityCheckFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         * @return	The window title.
                         */
                        std::string GetWindowTitle();
                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType SystemFilesIntegrityCheckFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         * @return	The menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	Gtk::Widget * RemoveUSBFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         * @return	null if it fails, else the frame.
                         */
                        Gtk::Widget * GetFrame();
                        /**
                         * @fn	void SystemFilesIntegrityCheckFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        void Dispose();
                        /**
                         * @fn	void SystemFilesIntegrityCheckFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        void Initialize();
                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * SystemFilesIntegrityCheckFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         * @return	null if it fails, else the logger.
                         */
                        Smartmatic::Log::ISMTTLog * getLogger();
                        /**
                         * @fn	Glib::ustring SystemFilesIntegrityCheckFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         * @return	The start log message.
                         */
                        Glib::ustring getStartLogMessage();
                        /**
                         * @fn	Glib::ustring SystemFilesIntegrityCheckFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         * @return	The finish log message.
                         */
                        Glib::ustring getFinishLogMessage();
                        /**
                         * @fn	virtual void SystemFilesIntegrityCheckFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        virtual void deleteElements();
                        /**
                         * @fn	virtual void SystemFilesIntegrityCheckFunctionality::block();
                         * @brief	method block functionality.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        virtual void block();
                        /**
                         * @fn	virtual void SystemFilesIntegrityCheckFunctionality::partial();
                         * @brief	method partial functionality.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        virtual void partial();
                        /**
                         * @fn	virtual void SystemFilesIntegrityCheckFunctionality::resume();
                         * @brief	method resume functionality.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        virtual void resume();
                        /**
                         * @fn	void SystemFilesIntegrityCheckFunctionality::endFunctionality();
                         * @brief	method end functionality, call this method if no verifications have to be made.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        void endFunctionality();
                        /**
                         * @fn	virtual bool SystemFilesIntegrityCheckFunctionality::ShowWindowTitle()
                         * @brief	getter is show window title.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         * @return	true is show window title.
                         */
                        virtual bool ShowWindowTitle();

                    public:
                        /**
                         * @fn	static SystemFilesIntegrityCheckFunctionality * SystemFilesIntegrityCheckFunctionality::getInstance();
                         * @brief	Method getter instance.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         * @return	instance of class.
                         */
                        static SystemFilesIntegrityCheckFunctionality * getInstance();
                        /**
                         * @fn	virtual SystemFilesIntegrityCheckFunctionality::~SystemFilesIntegrityCheckFunctionality();
                         * @brief	Class destructor.
                         * @author eduardo.fernandez
                         * @date 15/01/2019
                         */
                        virtual ~SystemFilesIntegrityCheckFunctionality();
                };
            }
        }
    }
}
#endif /* SYSTEM_FILES_INTEGRITY_CHECK_FUNCTIONALITY_HXX_ */
