/**
 * @file VotingMachineProcessStatusFunctionality.cxx
 * @brief Body voting machine process status functionality class
 * @date 08/11/2012
 * @author Yadickson Soto
 */

#include "VotingMachineProcessStatusFunctionality.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * VotingMachineProcessStatusFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.VotingMachineProcessStatusFunctionality", GETTEXT_PACKAGE );

VotingMachineProcessStatusFunctionality * VotingMachineProcessStatusFunctionality::current_instance = NULL;
VotingMachineProcessStatusFunctionality * VotingMachineProcessStatusFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new VotingMachineProcessStatusFunctionality();
    }

    return current_instance;
}

VotingMachineProcessStatusFunctionality::VotingMachineProcessStatusFunctionality()
{
    widget = NULL;
}

void VotingMachineProcessStatusFunctionality::deleteElements()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void VotingMachineProcessStatusFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.VotingMachineProcessStatusFunctionality.ExitFunctionality" ) );
    widget->getSignalEnd().connect ( sigc::mem_fun ( this, &VotingMachineProcessStatusFunctionality::end ) );
    disableExitButton();
}

Gtk::Widget * VotingMachineProcessStatusFunctionality::GetFrame()
{
    widget = new ProcessResultWidget();
    return widget;
}

std::string VotingMachineProcessStatusFunctionality::GetWindowTitle()
{
    return widget->getTitleText();
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType VotingMachineProcessStatusFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMVOTINGMACHINEPROCESSSTATUSRESULTSFUNCTIONALITY;
}

void VotingMachineProcessStatusFunctionality::Dispose()
{
}

void VotingMachineProcessStatusFunctionality::end()
{
    ExitFunctionality();
}

VotingMachineProcessStatusFunctionality::~VotingMachineProcessStatusFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * VotingMachineProcessStatusFunctionality::getLogger()
{
    return logger;
}

Glib::ustring VotingMachineProcessStatusFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.VotingMachineProcessStatusFunctionality.LogStartFunctionality" );
}

Glib::ustring VotingMachineProcessStatusFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.VotingMachineProcessStatusFunctionality.LogCloseFunctionality" );
}

bool VotingMachineProcessStatusFunctionality::ShowWindowTitle()
{
    return false;
}

bool VotingMachineProcessStatusFunctionality::ForceExecute()
{
    return true;
}
