/**
 * @file WaitingVotingExperienceFunctionality.cxx
 * @brief Body class waiting voting experience functionality
 * @date 17/10/2012
 * @author Yadickson Soto
 */

#include "WaitingVotingExperienceFunctionality.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Windows/VoteProcessWindow.hxx>
#include <Gui/Windows/BarcodeReconfigureWindow.hxx>
#include <Gui/Windows/DeviceMonitorWindow.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::SAES::GUI::Windows;

ISMTTLog * WaitingVotingExperienceFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.WaitingVotingExperienceFunctionality", GETTEXT_PACKAGE );

WaitingVotingExperienceFunctionality * WaitingVotingExperienceFunctionality::instance = NULL;

WaitingVotingExperienceFunctionality::WaitingVotingExperienceFunctionality()
{
    widget = NULL;
}

WaitingVotingExperienceFunctionality::~WaitingVotingExperienceFunctionality()
{
    deleteElements();
}

WaitingVotingExperienceFunctionality * WaitingVotingExperienceFunctionality::getInstance()
{
    if ( !instance )
    {
        instance = new WaitingVotingExperienceFunctionality();
    }

    return instance;
}

std::string WaitingVotingExperienceFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.WaitingVotingExperienceFunctionality.Title" );
}

Smartmatic::Log::ISMTTLog * WaitingVotingExperienceFunctionality::getLogger()
{
    return logger;
}

Glib::ustring WaitingVotingExperienceFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.WaitingVotingExperienceFunctionality.LogStartFunctionality" );
}

Glib::ustring WaitingVotingExperienceFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.WaitingVotingExperienceFunctionality.LogCloseFunctionality" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType WaitingVotingExperienceFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMWAITINGVOTINGEXPERIENCEFUNCTIONALITY;
}

Gtk::Widget * WaitingVotingExperienceFunctionality::GetFrame()
{
    if ( !widget )
    {
        widget = new MachineVotingWidget();
        widget->getSignalEnd().connect ( sigc::mem_fun ( this, &WaitingVotingExperienceFunctionality::end ) );
    }

    return widget;
}

void WaitingVotingExperienceFunctionality::Dispose()
{
    widget->stopAll();
}

void WaitingVotingExperienceFunctionality::Initialize()
{
    baseWindow->set_name ( SAESGUIConfigurationManager::getInstance()->GetVotingExperienceWindowName() );
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.WaitingVotingExperienceFunctionality.FinalMessage" ) );
    disableExitButton();

    BarcodeReconfigureWindow::getInstance()->stop();
    DeviceMonitorWindow::getInstance()->stop();
    VoteProcessWindow::getInstance()->stop();

    widget->start();
}

void WaitingVotingExperienceFunctionality::deleteElements()
{
}

bool WaitingVotingExperienceFunctionality::ShowWindowTitle()
{
    return false;
}

void WaitingVotingExperienceFunctionality::refresh()
{
    widget->refresh();
    baseWindow->set_name ( SAESGUIConfigurationManager::getInstance()->GetVotingExperienceWindowName() );
    GraphicDisplayFunctionality::refresh();
}

void WaitingVotingExperienceFunctionality::end()
{
    ExitFunctionality();
}
