/**
 * @file	barcode-diagnosis-functionality.cc
 * @brief	barcode diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include <Voting/OperationStatus.hxx>
#include "Functionalities/barcode-diagnosis-functionality.h"
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/Polling/BCRPolling.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Operation::Devices;

ISMTTLog * BarcodeDiagnosisFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.BarcodeDiagnosisFunctionality", GETTEXT_PACKAGE );

BarcodeDiagnosisFunctionality * BarcodeDiagnosisFunctionality::instance = NULL;


BarcodeDiagnosisFunctionality * BarcodeDiagnosisFunctionality::getInstance()
{
    if ( !instance )
    {
        instance = new BarcodeDiagnosisFunctionality();
    }

    return instance;
}

BarcodeDiagnosisFunctionality::BarcodeDiagnosisFunctionality()
{
    readerWidget = NULL;
}

BarcodeDiagnosisFunctionality::~BarcodeDiagnosisFunctionality()
{
    deleteElements();
    instance = NULL;
}

void BarcodeDiagnosisFunctionality::Initialize()
{
    BCRPolling::getInstance()->pause();

    connBCDiag.disconnect();
    connBCDiag = readerWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &BarcodeDiagnosisFunctionality::endDiagnosis ) );
    readerWidget->start();
}

void BarcodeDiagnosisFunctionality::endDiagnosis()
{
    enableExitButton();
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.BarcodeDiagnosisFunctionality.BarcodeDiagnosisExecuted" ) );
    ExitFunctionality();
}

Gtk::Widget * BarcodeDiagnosisFunctionality::GetFrame()
{
    readerWidget = new Smartmatic::SAES::GUI::Widgets::BarcodeReaderDiagnoseWidget();
    return readerWidget;
}

std::string BarcodeDiagnosisFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.BarcodeDiagnosisFunctionality.WindowTitle" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType BarcodeDiagnosisFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMBARCODEDIAGNOSISFUNCTIONALITY;
}

void BarcodeDiagnosisFunctionality::onBeforeExit()
{
    BCRPolling::getInstance()->resume();
}

void BarcodeDiagnosisFunctionality::Dispose()
{
    connBCDiag.disconnect();
    readerWidget->dispose();
}

Smartmatic::Log::ISMTTLog * BarcodeDiagnosisFunctionality::getLogger()

{
    return logger;
}
Glib::ustring BarcodeDiagnosisFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.BarcodeDiagnosisFunctionality.LogStartFunctionality" );
}

Glib::ustring BarcodeDiagnosisFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.BarcodeDiagnosisFunctionality.LogCloseFunctionality" );
}

void BarcodeDiagnosisFunctionality::deleteElements()
{
    if ( readerWidget )
    {
        delete readerWidget;
        readerWidget = NULL;
    }
}
