/**
 * @file	barcode-diagnosis-functionality.h
 * @brief	Declares the barcode diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#ifndef _BARCODE_DIAGNOSIS_FUNCTIONALITY_H_
#define _BARCODE_DIAGNOSIS_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/BarcodeReaderDiagnoseWidget.hxx>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	BarcodeDiagnosisFunctionality
                 * @brief	Barcode diagnosis functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class BarcodeDiagnosisFunctionality: public GraphicDisplayFunctionality
                {

                    public:

                        /**
                         * @fn	bool BarcodeDiagnosisFunctionality::DontAllowHelp();
                         * @brief	Gets the dont allow help.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowHelp();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> BarcodeDiagnosisFunctionality::getHelpSeqKey();
                         * @brief	Gets the help sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The help sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getHelpSeqKey();

                        /**
                         * @fn	bool BarcodeDiagnosisFunctionality::DontAllowExit();
                         * @brief	Gets the dont allow exit.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowExit();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> BarcodeDiagnosisFunctionality::getSeqKey();
                         * @brief	Gets the sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getSeqKey();

                        /**
                         * @fn	static BarcodeDiagnosisFunctionality * BarcodeDiagnosisFunctionality::getInstance();
                         * @brief	Gets the current.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else.
                         */

                        static BarcodeDiagnosisFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * BarcodeDiagnosisFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring BarcodeDiagnosisFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring BarcodeDiagnosisFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        Glib::ustring getFinishLogMessage();

                        /**
                         * @fn	virtual void BarcodeDiagnosisFunctionality::deleteElements();
                         * @brief	virtual Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                    protected:

                        /**
                         * @fn	BarcodeDiagnosisFunctionality::BarcodeDiagnosisFunctionality();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        BarcodeDiagnosisFunctionality();

                        /**
                         * @fn	virtual BarcodeDiagnosisFunctionality::~BarcodeDiagnosisFunctionality();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual ~BarcodeDiagnosisFunctionality();

                        /**
                         * @fn	void BarcodeDiagnosisFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Dispose();

                        /**
                         * @fn	void BarcodeDiagnosisFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Initialize();

                        /**
                         * @fn	Gtk::Widget * BarcodeDiagnosisFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        Gtk::Widget * GetFrame();

                        /**
                         * @fn	std::string BarcodeDiagnosisFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType BarcodeDiagnosisFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	void BarcodeDiagnosisFunctionality::endDiagnosis();
                         * @brief	method end diagnosis.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         */
                        void endDiagnosis();
                        /**
                         * @fn	virtual void BarcodeDiagnosisFunctionality::onBeforeExit();
                         * @brief	virtual method before exit.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        virtual void onBeforeExit();

                    private:

                        Smartmatic::SAES::GUI::Widgets::BarcodeReaderDiagnoseWidget * readerWidget;  ///< The reader widget
                        sigc::connection connBCDiag;	///< connection barcode diagnosis

                        static BarcodeDiagnosisFunctionality * instance; ///< The current instance
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                };
            }
        }
    }
}


#endif // _BARCODE_DIAGNOSIS_FUNCTIONALITY_H_
