/**
 * @file	change-language-functionality.cc
 * @brief	change language functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "change-language-functionality.h"
#include "Operation/MachineOperationManager.hxx"
#include <Log/SMTTLogManager.h>
#include <Voting/Election/ElectionInstalationLanguages.h>
#include <Voting/ElectoralConfiguration.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Environment/Configurations/base-language-controller.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * ChangeLanguageFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.ChangeLanguageFunctionality", GETTEXT_PACKAGE );

ChangeLanguageFunctionality * ChangeLanguageFunctionality::current_instance = NULL;
ChangeLanguageFunctionality * ChangeLanguageFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new ChangeLanguageFunctionality();
    }

    return current_instance;
}

ChangeLanguageFunctionality::ChangeLanguageFunctionality()
{
    changeLanguageWidget = NULL;
}

void ChangeLanguageFunctionality::deleteElements()
{
    if ( changeLanguageWidget )
    {
        delete changeLanguageWidget;
        changeLanguageWidget = NULL;
    }
}

void ChangeLanguageFunctionality::Initialize()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.ChangeLanguageFunctionality.LanguageNotChanged" ) );
    enableExitButton();
}

Gtk::Widget * ChangeLanguageFunctionality::GetFrame()
{
    changeLanguageWidget = new ChangeLanguageWidget();
    changeLanguageWidget->init ( MachineOperationManager::getInstance()->getSaesDirectories() );
    changeLanguageWidget->SetShowMessage ( true );
    changeLanguageWidget->LanguageChanged().connect ( sigc::mem_fun ( this, &ChangeLanguageFunctionality::OnLanguageChange ) );
    Smartmatic::SAES::Voting::ElectoralConfiguration * elecConfig =
        Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration();
    std::list<Smartmatic::SAES::Voting::Election::Language> listLang;

    if ( elecConfig != NULL )
    {
        listLang = elecConfig->getLanguageList();
    }
    else
    {
        listLang = Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->getLanguages();
    }

    changeLanguageWidget->initialize ( listLang, true ); //False to save the file in both memories
    changeLanguageWidget->set_visible ( true );
    this->changeLanguageWidget->SetSaveFile ( true );
    return changeLanguageWidget;
}

std::string ChangeLanguageFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ChangeLanguageFunctionality.Title" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType ChangeLanguageFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMCHANGELANGUAGEFUNCTIONALITY;
}

void ChangeLanguageFunctionality::Dispose()
{
}

void ChangeLanguageFunctionality::OnLanguageChange()
{
    if ( BaseLanguageController::isSetLanguage() )
    {
        exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.ChangeLanguageFunctionality.LanguageChanged" ) );
    }
    else
    {
        exitWidget.setInfo ( " " );
    }

    RefreshInfo();
    ExitFunctionality();
}

ChangeLanguageFunctionality::~ChangeLanguageFunctionality()
{
    deleteElements();
}

Smartmatic::Log::ISMTTLog * ChangeLanguageFunctionality::getLogger()
{
    return logger;
}

Glib::ustring ChangeLanguageFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ChangeLanguageFunctionality.LogStartFunctionality" );
}

Glib::ustring ChangeLanguageFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ChangeLanguageFunctionality.LogCloseFunctionality" );
}

bool ChangeLanguageFunctionality::ShowWindowTitle()
{
    return BaseLanguageController::isSetLanguage();
}
