/**
 * @file	flash-diagnosis-functionality.cc
 * @brief	flash diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include <Voting/OperationStatus.hxx>
#include "Functionalities/flash-diagnosis-functionality.h"
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;

ISMTTLog * FlashDiagnosisFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.FlashDiagnosisFunctionality", GETTEXT_PACKAGE );

FlashDiagnosisFunctionality * FlashDiagnosisFunctionality::current_instance = NULL;

FlashDiagnosisFunctionality * FlashDiagnosisFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new FlashDiagnosisFunctionality();
    }

    return current_instance;
}

FlashDiagnosisFunctionality::FlashDiagnosisFunctionality()
{
    flashWidget = NULL;
}

FlashDiagnosisFunctionality::~FlashDiagnosisFunctionality()
{
    deleteElements();
}

void FlashDiagnosisFunctionality::Initialize()
{
    flashWidget->initialize();
    connFlashDiag = flashWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &FlashDiagnosisFunctionality::on_FlashDiagFinished ) );
}

void FlashDiagnosisFunctionality::on_FlashDiagFinished()
{
    enableExitButton();
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.FlashDiagnosisFunctionality.FlashDiagnosisExecuted" ) );
    ExitFunctionality();
}

Gtk::Widget * FlashDiagnosisFunctionality::GetFrame()
{
    flashWidget = new Smartmatic::SAES::GUI::Widgets::FlashDiagnosisWidget ();
    return flashWidget;
}

std::string FlashDiagnosisFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.FlashDiagnosisFunctionality.FlashDiagnosis" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType FlashDiagnosisFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMFLASHDIAGNOSISFUNCTIONALITY;
}

void FlashDiagnosisFunctionality::Dispose()
{
    connFlashDiag.disconnect();
    flashWidget->Dispose();
}

Smartmatic::Log::ISMTTLog * FlashDiagnosisFunctionality::getLogger()
{
    return logger;
}
Glib::ustring FlashDiagnosisFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.FlashDiagnosisFunctionality.LogStartFunctionality" );
}

Glib::ustring FlashDiagnosisFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.FlashDiagnosisFunctionality.LogCloseFunctionality" );
}

void FlashDiagnosisFunctionality::deleteElements()
{
    if ( flashWidget )
    {
        delete ( flashWidget );
        flashWidget = NULL;
    }
}
