/**
 * @file	printer-diagnosis-functionality.cc
 * @brief	printer diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "printer-diagnosis-functionality.h"
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * PrinterDiagnosisFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.PrinterDiagnosisFunctionality", GETTEXT_PACKAGE );

PrinterDiagnosisFunctionality * PrinterDiagnosisFunctionality::current_instance = NULL;

PrinterDiagnosisFunctionality * PrinterDiagnosisFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new PrinterDiagnosisFunctionality();
    }

    return current_instance;
}

PrinterDiagnosisFunctionality::PrinterDiagnosisFunctionality()
{
    printerWidget = NULL;
}

PrinterDiagnosisFunctionality::~PrinterDiagnosisFunctionality()
{
    deleteElements();
}

void PrinterDiagnosisFunctionality::Initialize()
{
    printerWidget->initialize();
    //connPrinterDiag = printerWidget->signalContinue_detected.connect ( sigc::mem_fun ( this, &PrinterDiagnosisFunctionality::on_PrinterDiagFinished ) );
    connPrinterDiagOnOk = printerWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &PrinterDiagnosisFunctionality::on_PrinterDiagFinished ) );
    //connPrinterDiagOnCancel = printerWidget->signalCancel_detected.connect ( sigc::mem_fun ( this, &PrinterDiagnosisFunctionality::on_PrinterDiagFinished ) );
}

void PrinterDiagnosisFunctionality::on_PrinterDiagFinished()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.PrinterDiagnosisFunctionality.PrinterDiagnosisExecuted" ) );
    enableExitButton();
    ExitFunctionality();
}

Gtk::Widget * PrinterDiagnosisFunctionality::GetFrame()
{
    printerWidget = new Smartmatic::SAES::GUI::Widgets::PrinterDiagnosisWidget ();
    return printerWidget;
}

std::string PrinterDiagnosisFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.PrinterDiagnosisFunctionality.PrinterDiagnosis" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType PrinterDiagnosisFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMPRINTERDIAGNOSISFUNCTIONALITY;
}

void PrinterDiagnosisFunctionality::Dispose()
{
    printerWidget->dispose();
}

Smartmatic::Log::ISMTTLog * PrinterDiagnosisFunctionality::getLogger()
{
    return logger;
}

Glib::ustring PrinterDiagnosisFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.PrinterDiagnosisFunctionality.LogStartFunctionality" );
}

Glib::ustring PrinterDiagnosisFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.PrinterDiagnosisFunctionality.LogCloseFunctionality" );
}

void PrinterDiagnosisFunctionality::deleteElements()
{
    if ( printerWidget )
    {
        delete printerWidget;
        printerWidget = NULL;
    }
}
