/**
 * @file BallotOptionButtonWidget.cxx
 * @brief Body class ballot option button widget
 * @date 12/03/2012
 * @author Yadickson Soto
 */

#include "BallotOptionButtonWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Operation/Voting/VotingOperation.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::GUI::Operation::Voting;

BallotOptionButtonWidget::BallotOptionButtonWidget ( bool expand )
    : ButtonsWidget ( expand )
{
    int size;

    if ( expand )
    {
        int spacing = SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getExpandButton().getSpacing();
        setSpacing ( spacing );
        size = SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getExpandButton().getFontSize();
    }
    else
    {
        int clearWidth = SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getSeparateButton().getClearButtonWidth();
        int acceptWidth = SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getSeparateButton().getAcceptButtonWidth();
        int backWidth = SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getSeparateButton().getBackButtonWidth();
        clearButton->set_size_request ( clearWidth, -1 );
        acceptButton->set_size_request ( acceptWidth, -1 );
        backButton->set_size_request ( backWidth, -1 );
        size = SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getSeparateButton().getFontSize();
    }

    std::string soundBallotS = SAESGUIConfigurationManager::getInstance()->getBallotOptionSelectSound();

    clearButton->WidgetsPosition ( 5, 0, 0 );
    clearButton->SetSound ( soundBallotS );
    acceptButton->WidgetsPosition ( 5, 0, 0 );
    acceptButton->SetSound ( soundBallotS );
    backButton->WidgetsPosition ( 5, 0, 0 );
    backButton->SetSound ( soundBallotS );

    modifyFontSize ( size );

    setAccessibleName ( "BallotOptionButtonsContainer" );
    acceptButton->setAccessibleName ( "ConfirmBallotOptionSelected" );
    clearButton->setAccessibleName ( "CancelBallotOptionSelected" );
    backButton->setAccessibleName ( "BackBallotOptionSelected" );
}

BallotOptionButtonWidget::~BallotOptionButtonWidget()
{
}

void BallotOptionButtonWidget::update()
{
    clearButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.BallotOptionSelectionWidget.ClearSelection" ) );
    acceptButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.BallotOptionSelectionWidget.AcceptSelection" ) );
    backButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.BallotOptionSelectionWidget.Back" ) );
    clearButton->SetCustomName ( VotingOperation::getButtonName() );
    acceptButton->SetCustomName ( VotingOperation::getButtonName() );
    backButton->SetCustomName ( VotingOperation::getButtonName() );

    clearSignals();
}
