/**
 * @file BallotOptionTableWidget.cxx
 * @brief Body class ballot option button widget
 * @date 13/03/2012
 * @author Yadickson Soto
 */

#include "BallotOptionTableWidget.hxx"
#include "FillTableWidget.hxx"
#include <Gui/Widgets/Voting/ContestLayout.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::GUI::Widgets;

BallotOptionTableWidget::BallotOptionTableWidget ( FillTableWidget * fillTableWidget )
    : SmartmaticBasicEventBox()
{
    this->fillTableWidget = fillTableWidget;
    this->add ( *fillTableWidget );
    this->show_all_children();
    this->show();

    setAccessibleName ( "MainBallotOptionSelectionContainer" );
}

BallotOptionTableWidget::~BallotOptionTableWidget()
{
    this->remove();

    if ( fillTableWidget )
    {
        delete fillTableWidget;
        fillTableWidget = NULL;
    }
}

void BallotOptionTableWidget::addPrincipal ( Smartmatic::GUI::Widgets::ISizeWidget * widget )
{
    fillTableWidget->addPrincipal ( widget );
}

void BallotOptionTableWidget::addStandIn ( Smartmatic::GUI::Widgets::ISizeWidget * widget )
{
    fillTableWidget->addStandIn ( widget );
}

void BallotOptionTableWidget::fill ( Contest * contest, int totalW, int totalH )
{
    getTableWidget()->setSelectionQuantitiesBoundaries ( contest->getMin_votes(), contest->getMax_votes() );

    ContestLayout layout ( contest );
    fillTableWidget->fill ( &layout, totalW, totalH );
}

void BallotOptionTableWidget::clear()
{
    fillTableWidget->clear();
}

void BallotOptionTableWidget::update()
{
    fillTableWidget->update();
}

void BallotOptionTableWidget::clearSelections()
{
    getTableWidget()->clear();
}

Smartmatic::GUI::Widgets::TableBoxSelectionWidget * BallotOptionTableWidget::getTableWidget()
{
    return fillTableWidget->getTableWidget();
}
