/**
 * @file	CardActivatorEventBox.cpp
 * @brief	Implements the card activator event box class.
 * @author	Youssef.tourkmani
 * @date	05/11/2012
 */

#include "CardActivatorEventBox.h"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Resources/election-string-resources-manager.h>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/SaesGUIConfiguration-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define P_(Singular, Plural, Number) ((Number == 1) ? (Singular) : (Plural))///<macro without ngettext
#define N_(String) (String) ///<macro for gettext not translate

using namespace Gtk;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::VotingDevice;

CardActivatorEventBox::CardActivatorEventBox ( int width )
{
    static ActivationConsolidationFunctionalityConfiguration config =
        SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration();
    static LabelConfiguration selectedLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getLabelSelectedConfiguration() );
    static LabelConfiguration unselectedLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getLabelUnselectedConfiguration() );
    static LabelConfiguration titleLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getTitleFontConfiguration() );

    Gdk::Color selectedBoxColor = SAESGUIConfigurationManager::ConvertToGdkColor ( config.getSelectedBoxColor() );
    Gdk::Color unselectedBoxColor = SAESGUIConfigurationManager::ConvertToGdkColor ( config.getUnselectedBoxColor() );
    Gdk::Color selectedFontColor = SAESGUIConfigurationManager::ConvertToGdkColor ( config.getSelectedFontColor() );
    Gdk::Color unselectedFontColor = SAESGUIConfigurationManager::ConvertToGdkColor ( config.getUnselectedFontColor() );

    CardPolling::getInstance()->getSignalCard().connect ( sigc::mem_fun ( *this, &CardActivatorEventBox::onSignalCardChange ) );

    std::list< std::map < std::string, std::list < std::map < std::string, std::string > > > > eventboxSelectionMap;
    //std::map< string, std::map< string, string > > eventboxSelectionMap;

    if ( MachineOperationManager::getInstance()->getElectoralConfiguration() != NULL )
    {
        try
        {
            listCAD = MachineOperationManager::getInstance()->getElectoralConfiguration()->getVotingDevice()->getCustomAttributeDefinitionList (
                          MachineOperationManager::getInstance()->getElectoralConfiguration()->getVotingDevice()->getFirstVotingDevice(),
                          Machine_type::VOTING_DEVICES_PM );
        }
        catch ( ... )
        {
        }

        std::map < std::string, std::list < std::map < std::string, std::string > > > customAttrMap;

        for ( std::list<Custom_attribute_definition>::iterator iter = listCAD.begin(); iter != listCAD.end(); ++iter )
        {
            std::string s = Glib::ustring::compose (
                                P_ ( _ ( "Smartmatic.SAES.GUI.Widgets.CardActivactorEventBox.TitleCustomAttributeSingular" ),
                                     _ ( "Smartmatic.SAES.GUI.Widgets.CardActivactorEventBox.TitleCustomAttribute" ),
                                     listCAD.size() ), ElectionStringResourcesManager::getInstance()->getResource ( iter->getName_key() ) );
            std::list < std::map < std::string, std::string > > item;

            for ( Custom_attribute_definition_values::Custom_attribute_definition_valueIterator cad = iter->getCustom_attribute_definition_values().getCustom_attribute_definition_value().begin();
                    cad != iter->getCustom_attribute_definition_values().getCustom_attribute_definition_value().end(); ++cad )
            {
                std::map < std::string, std::string > mapi;
                mapi[ ( *cad ).getValue()] = ElectionStringResourcesManager::getInstance()->getResource ( ( *cad ).getValue() );
                item.push_back ( mapi );
            }

            customAttrMap[s] = item;
        }

        eventboxSelectionMap.push_back ( customAttrMap );
    }

    eventboxSelectionWidget = manage ( new EventboxSelectionWidget ( eventboxSelectionMap, "TypeOfVoterTitle", "VoterTypeBox" ) );
    eventboxSelectionWidget->SetSelectedBoxColor ( selectedBoxColor );
    eventboxSelectionWidget->SetUnselectedBoxColor ( unselectedBoxColor );
    eventboxSelectionWidget->SetSelectedFontColor ( selectedFontColor );
    eventboxSelectionWidget->SetUnselectedFontColor ( unselectedFontColor );
    eventboxSelectionWidget->SetTitlesFontDescription ( titleLabelConfiguration );
    eventboxSelectionWidget->SetSelectedFontDescription ( selectedLabelConfiguration );
    eventboxSelectionWidget->SetUnselectedFontDescription ( unselectedLabelConfiguration );
    eventboxSelectionWidget->set_size_request ( width, -1 );
    eventboxSelectionWidget->SetTitlesHeight ( config.getTitleHeight() );
    m_eventBoxConnection = eventboxSelectionWidget->signal_box_clicked().connect ( sigc::mem_fun ( *this, &CardActivatorEventBox::on_EventboxClicked ) );
    eventboxSelectionWidget->getDisconnectionSignal().connect(sigc::mem_fun(*this, &CardActivatorEventBox::on_SignalDisconnectionEvent));
    eventboxSelectionWidget->SetFirstSelected();
    eventboxSelectionWidget->show();
    pack_start ( *eventboxSelectionWidget );
}

CardActivatorEventBox::~CardActivatorEventBox()
{
}

void CardActivatorEventBox::on_EventboxClicked ( std::list< std::string > valuesSelected )
{
    m_eventBoxConnection.disconnect();
    attributes.clear();

    for ( std::list< std::string >::iterator iter = valuesSelected.begin();
            iter != valuesSelected.end(); ++iter )
    {
        for ( std::list < Custom_attribute_definition >::iterator iter2 = listCAD.begin();
                iter2 != listCAD.end(); ++iter2 )
        {
            for ( Custom_attribute_definition_values::Custom_attribute_definition_valueIterator cad = iter2->getCustom_attribute_definition_values().getCustom_attribute_definition_value().begin();
                    cad != iter2->getCustom_attribute_definition_values().getCustom_attribute_definition_value().end(); ++cad )
            {
                if ( ( *cad ).getValue() == *iter )
                {
                    Custom_attribute ca;
                    ca.setCode ( iter2->getCode() );
                    ca.setValue ( *iter );
                    attributes.push_back ( ca );
                    eventboxSelectionWidget->setInitializing(false);
                    m_customAttributeSelectedSignal.emit();
                    break;
                }
            }
        }
    }

    m_type_vector_changed.emit ( attributes );
}

void CardActivatorEventBox::on_SignalDisconnectionEvent(const bool initializing)
{
    if ((CardPolling::getInstance()->isCardPresent() == false) || (initializing))
    {
        eventboxSelectionWidget->reconnectSignals();
    }
    else if (initializing == false)
    {
        if (CardPolling::getInstance()->isCardPresent() && CardPolling::getInstance()->certaintyOfCardPresence())
        {
            on_EventboxClicked(eventboxSelectionWidget->GetValuesSelected());
        }
        else
        {
            eventboxSelectionWidget->reconnectSignals();
        }
    }
}

void CardActivatorEventBox::onSignalCardChange(CardPolling::CardPollingStatus status)
{
    if (status == CardPolling::INSERTED)
    {
        eventboxSelectionWidget->disconnectSignals();
    }
}

CardActivatorEventBox::CustomAttributeSelectedSignal CardActivatorEventBox::getCustomAttributeSelectedSignal()
{
    return m_customAttributeSelectedSignal;
}

std::vector< Custom_attribute > CardActivatorEventBox::GetCustomAttributes()
{
    return attributes;
}

CardActivatorEventBox::type_vectorCA_changed CardActivatorEventBox::signal_vectorCustomAttribute_changed()
{
    return m_type_vector_changed;
}

void CardActivatorEventBox::setFont ( LabelConfiguration & titleLabelConfiguration, LabelConfiguration & selectedLabelConfiguration, LabelConfiguration & unselectedLabelConfiguration )
{
    eventboxSelectionWidget->SetTitlesFontDescription ( titleLabelConfiguration );
    eventboxSelectionWidget->SetSelectedFontDescription ( selectedLabelConfiguration );
    eventboxSelectionWidget->SetUnselectedFontDescription ( unselectedLabelConfiguration );
    eventboxSelectionWidget->SetFirstSelected();
    eventboxSelectionWidget->show();
}

void CardActivatorEventBox::reset ()
{
    eventboxSelectionWidget->SetFirstSelected();
}

void CardActivatorEventBox::reconnectEventBox()
{
    m_eventBoxConnection = eventboxSelectionWidget->signal_box_clicked().connect ( sigc::mem_fun ( *this, &CardActivatorEventBox::on_EventboxClicked ) );
    eventboxSelectionWidget->reconnectSignals();
}

void CardActivatorEventBox::disconnectEventBox()
{
    m_eventBoxConnection.disconnect();
    eventboxSelectionWidget->disconnectSignals(false);
}
