/**
 * @file AbleA4517ButtonWidget.cxx
 * @brief Body class able a4517 widget
 * @author eduardo.fernandez
 * @date 22/01/2018
 */

#include "AbleA4517ButtonWidget.hxx"
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Configuration;

ISMTTLog * AbleA4517ButtonWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.AbleA4517ButtonWidget", GETTEXT_PACKAGE );

AbleA4517ButtonWidget::AbleA4517ButtonWidget()
    : IAbleButtonWidget()
{
    initialize();
    add ( *container );
    show_all_children();
}

AbleA4517ButtonWidget::~AbleA4517ButtonWidget()
{
}

void AbleA4517ButtonWidget::initialize()
{
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/AbleWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - AbleA4517AButtonWidget Initialize" );
    }

    builder->get_widget ( "mainFrame", container );
    builder->get_widget ( "image-base", imageBase );
    imageBase->set ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/Able20Buttons.png" ) );
    Gtk::Image * image = NULL;
    builder->get_widget ( "image-help", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_HELP, image );
    image = NULL;
    builder->get_widget ( "image-volume-down", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_DOWN, image );
    image = NULL;
    builder->get_widget ( "image-arrow-down", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_DOWN, image );
    image = NULL;
    builder->get_widget ( "image-speed-down", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_DOWN, image );
    image = NULL;
    builder->get_widget ( "image-arrow-left", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_LEFT, image );
    image = NULL;
    builder->get_widget ( "image-select", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_SELECT, image );
    image = NULL;
    builder->get_widget ( "image-arrow-right", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_RIGHT, image );
    image = NULL;
    builder->get_widget ( "image-volume-up", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_UP, image );
    image = NULL;
    builder->get_widget ( "image-arrow-up", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_UP, image );
    image = NULL;
    builder->get_widget ( "image-speed-up", image );
    addButton ( A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_UP, image );

    std::list <A4517AbleButtonType>::iterator it;

    for ( it = getA4517Buttons().begin();
            it != getA4517Buttons().end();
            ++it )
    {
        Gtk::Image * image = getButton ( *it );

        if ( image == NULL )
        {
            continue;
        }

        std::string file;
        image->set_visible ( false );
        image->set_no_show_all();

        switch ( *it )
        {
            case A4517AbleButtonType::A4517_ABLED_DEVICE_HELP:
                file = "ui/Able20Help.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_DOWN:
                file = "ui/Able20VolumeDown.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_DOWN:
                file = "ui/Able20ArrowDown.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_DOWN:
                file = "ui/Able20SpeedDown.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_LEFT:
                file = "ui/Able20ArrowLeft.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_SELECT:
                file = "ui/Able20Select.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_RIGHT:
                file = "ui/Able20ArrowRight.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_UP:
                file = "ui/Able20VolumeUp.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_UP:
                file = "ui/Able20ArrowUp.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_UP:
                file = "ui/Able20SpeedUp.png";
                break;

            case A4517AbleButtonType::A4517_ABLED_DEVICE_UNKNOWN:
            default:
                break;
        }

        file = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, file );
        image->set ( Gdk::PixbufAnimation::create_from_file ( file ) );
    }
}
