/**
 * @file AbleButtonDiagnosticWidget.hxx
 * @brief Header class able button diagnostic widget
 * @date 05/09/2012
 * @author Yadickson Soto
 */

#ifndef ABLEBUTTONDIAGNOSTICWIDGET_HXX_
#define ABLEBUTTONDIAGNOSTICWIDGET_HXX_

#include <Widgets/StepWidget.hxx>
#include "AbleButtonWidget.hxx"
#include <Operation/AbleDButtonState.hxx>
#include <Configuration/s-a-e-s-hardware-conf-manager.h>
#include <Operation/Voting/AudioVoting/AbledController.hxx>
#include <Operation/Voting/AudioVoting/AudioVotingCheckUtility.hxx>
#include <set>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	AbleButtonDiagnosticWidget
                 * @brief	Class able button diagnostic widget.
                 * @author	Yadickson Soto
                 * @date	05/09/2012
                 */
                class AbleButtonDiagnosticWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:

                        typedef sigc::signal<void> SignalAllButtonsPressed;

                    public:
                        /**
                         * @fn	AbleButtonDiagnosticWidget::AbleButtonDiagnosticWidget();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        AbleButtonDiagnosticWidget();
                        /**
                         * @fn	virtual AbleButtonDiagnosticWidget::~AbleButtonDiagnosticWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        virtual ~AbleButtonDiagnosticWidget();
                        /**
                         * @fn	virtual void AbleButtonDiagnosticWidget::initialize();
                         * @brief	Initializer method.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        virtual void initialize();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::connectAbleSignals();
                         * @brief   Connect button signals.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void connectAbleSignals();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleHelpPress();
                         * @brief   Handles help button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleHelpPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleVolumeDownPress();
                         * @brief   Handles volume down button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleVolumeDownPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleDownPress();
                         * @brief   Handles down button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleDownPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleSpeedDownPress();
                         * @brief   Handles speed down button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleSpeedDownPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleLeftPress();
                         * @brief   Handles left button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleLeftPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleSelectPress();
                         * @brief   Handles select button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleSelectPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleRightPress();
                         * @brief   Handles right button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleRightPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleVolumeUpPress();
                         * @brief   Handles volume up button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleVolumeUpPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleUpPress();
                         * @brief   Handles up button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleUpPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleSpeedUpPress();
                         * @brief   Handles speed up button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void handleSpeedUpPress();
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleButtonPress(Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button);
                         * @brief   Handles button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button Button whose pressing is handled.
                         */
                        virtual void handleButtonPress(Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button);
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::handleButtonPress(Smartmatic::SAES::HW::Configuration::AbleButtonType button);
                         * @brief   Handles button press.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button Button whose pressing is handled.
                         */
                        virtual void handleButtonPress(Smartmatic::SAES::HW::Configuration::AbleButtonType button);
                        /**
                         * @fn  virtual void AbleButtonDiagnosticWidget::validateButtonsPressing();
                         * @brief   Validates whether all buttons have been pressed.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        virtual void validateButtonsPressing();
                        /**
                         * @fn  virtual AbleButtonDiagnosticWidget::SignalAllButtonsPressed AbleButtonDiagnosticWidget::getSignalAllButtonsPressed();
                         * @brief   Gets signal all buttons pressed.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         */
                        SignalAllButtonsPressed getSignalAllButtonsPressed();

                        /**
                         * @fn	virtual void AbleButtonDiagnosticWidget::dispose();
                         * @brief	virtual method stop.
                         * @author	Yadickson Soto
                         * @date	05/09/2012
                         */
                        virtual void dispose();
                        /**
                         * @fn	sigc::signal<void> AbleButtonDiagnosticWidget::getSignalError();
                         * @brief	method getter signal error.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         * @return	signal error.
                         */
                        sigc::signal<void> getSignalError();

                    protected:

                        /**
                         * @fn	void AbleButtonDiagnosticWidget::endDiagnosis();
                         * @brief	method end diagnosis.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         */
                        void endDiagnosis();
                        /**
                         * @fn	void AbleButtonDiagnosticWidget::startCheckButton();
                         * @brief	method start check button.
                         * @author	Yadickson Soto
                         * @date	10/09/2012
                         */
                        void startCheckButton();
                        /**
                         * @fn	void AbleButtonDiagnosticWidget::errorButton();
                         * @brief	method show error button.
                         * @author	Yadickson Soto
                         * @date	10/09/2012
                         */
                        void errorButton();
                        /**
                         * @fn	void AbleButtonDiagnosticWidget::emitSignalError();
                         * @brief	method emit signal error.
                         * @author	Yadickson Soto
                         * @date	11/09/2012
                         */
                        void emitSignalError();
                        /**
                         * @fn  void AbleButtonDiagnosticWidget::setAbledButtonWidgetText();
                         * @brief   method to set the abled button widget text.
                         * @author  eduardo.fernandez
                         * @date    17/01/2018
                         */
                        void setAbledButtonWidgetText();
                        /**
                         * @fn  void AbleButtonDiagnosticWidget::setA4517AbledButtonWidgetText();
                         * @brief   method to set the A4517 abled button widget text.
                         * @author  eduardo.fernandez
                         * @date    17/01/2018
                         */
                        void setA4517AbledButtonWidgetText();
                        /**
                         * @fn  void AbleButtonDiagnosticWidget::showMessage(Smartmatic::SAES::HW::Configuration::AbleButtonType button);
                         * @brief   method show message.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button Button whose message will be shown.
                         */
                        void showMessage(Smartmatic::SAES::HW::Configuration::AbleButtonType button);
                        /**
                         * @fn  void AbleButtonDiagnosticWidget::showMessage(Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button);
                         * @brief   method show message.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button Button whose message will be shown.
                         */
                        void showMessage(Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button);
                        /**
                         * @fn	void AbleButtonDiagnosticWidget::skipDiagnostic();
                         * @brief	method skip diagnostic.
                         * @author	Yadickson Soto
                         * @date	11/09/2012
                         */
                        void skipDiagnostic();

                    private:

                        AbleButtonWidget * ableButtonWidget; ///< able button widgets
                        int timerRefresh; ///< timer refresh time out
                        bool status; ///< status
                        bool keepButtonsPressed; ///< flag keep buttons pressed
                        bool checkButtonsInSequence; ///< flag check buttons in sequence
                        std::string errorMessage; ///< error message
                        sigc::connection conn;	///< The signal connection
                        sigc::connection connTimeout;	///< The signal connection time out
                        sigc::signal<void> signalError; ///< signal error
                        Smartmatic::SAES::HW::Configuration::P168ADeviceConfiguration::ButtonTypeSequence buttonsToPress; ///< press buttons
                        Smartmatic::SAES::HW::Configuration::A4517AbledDeviceConfiguration::ButtonTypeSequence a4517AbledButtonsToPress;
                        unsigned int currentButton; ///< current button
                        Glib::Mutex mutex; ///< mutex
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                        SignalAllButtonsPressed signalAllButtonsPressed;
                        Smartmatic::SAES::Operation::AudioVoting::AbledController abledController;
                        Smartmatic::SAES::Operation::AudioVoting::AudioVotingCheckUtility audioVotingCheck;

                        bool helpButtonPressed;
                        bool volumeDownButtonPressed;
                        bool downButtonPressed;
                        bool speedDownButtonPressed;
                        bool leftButtonPressed;
                        bool selectButtonPressed;
                        bool rightButtonPressed;
                        bool volumeUpButtonPressed;
                        bool upButtonPressed;
                        bool speedUpButtonPressed;

                        sigc::connection helpButtonConnection;
                        sigc::connection volumeDownButtonConnection;
                        sigc::connection downButtonConnection;
                        sigc::connection speedDownButtonConnection;
                        sigc::connection leftButtonConnection;
                        sigc::connection selectButtonConnection;
                        sigc::connection rightButtonConnection;
                        sigc::connection volumeUpButtonConnection;
                        sigc::connection upButtonConnection;
                        sigc::connection speedUpButtonConnection;
                };
            }
        }
    }
}
#endif /* ABLEBUTTONDIAGNOSTICWIDGET_HXX_ */
