/**
 * @file AbleButtonWidget.hxx
 * @brief Header class able button widget
 * @date 06/09/2012
 * @author Yadickson Soto
 */

#ifndef ABLEBUTTONWIDGET_HXX_
#define ABLEBUTTONWIDGET_HXX_

#include <Gui/Widgets/Diagnose/BasicDiagnosticWidget.hxx>
#include <Configuration/s-a-e-s-hardware-conf-manager.h>
#include "IAbleButtonWidget.hxx"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	AbleButtonWidget
                 * @brief	Class able button widget.
                 * @author	Yadickson Soto
                 * @date	06/09/2012
                 */
                class AbleButtonWidget : public Smartmatic::SAES::GUI::Widgets::BasicDiagnosticWidget
                {
                    public:
                        /**
                         * @fn	AbleButtonWidget::AbleButtonWidget();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         */
                        AbleButtonWidget();
                        /**
                         * @fn	virtual AbleButtonWidget::~AbleButtonWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         */
                        virtual ~AbleButtonWidget();
                        /**
                         * @fn  bool AbleButtonWidget::setAbleWidget();
                         * @brief   Method set able widget.
                         * @author  eduardo.fernandez
                         * @date    16/02/2018
                         */
                        void setAbleWidget();
                        /**
                         * @fn	bool AbleButtonWidget::pressButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                         * @brief	Method show press button.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         * @param	button	button to press.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool pressButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                        /**
                         * @fn  bool AbleButtonWidget::pressButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button );
                         * @brief   Method show press button.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button  button to press.
                         * @return  true if it succeeds, false if it fails.
                         */
                        bool pressButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button );
                        /**
                         * @fn	bool AbleButtonWidget::releaseButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                         * @brief	Method hide press button.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         * @param	button	button to release.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool releaseButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                        /**
                         * @fn	void AbleButtonWidget::setText ( const std::string & text );
                         * @brief	Method setter bottom text.
                         * @author	Yadickson Soto
                         * @date	10/09/2012
                         * @param	text	new bottom text.
                         */
                        void setText ( const std::string & text );
                        /**
                         * @fn	std::list <Smartmatic::SAES::HW::Configuration::AbleButtonType> & AbleButtonWidget::getButtons();
                         * @brief	Method getter orden button.
                         * @author	Yadickson Soto
                         * @date	10/09/2012
                         * @return	button button to release.
                         */
                        std::list <Smartmatic::SAES::HW::Configuration::AbleButtonType> & getButtons();
                        /**
                         * @fn	sigc::signal<void> AbleButtonWidget::getSignalCancel();
                         * @brief	method getter signal end.
                         * @author	Yadickson Soto
                         * @date	27/04/2012
                         * @return	signal end.
                         */
                        sigc::signal<void> getSignalCancel();
                        /**
                         * @fn	void AbleButtonWidget::viewButton ( bool view = true );
                         * @brief	method view button.
                         * @author	Yadickson Soto
                         * @date	27/04/2012
                         * @param	view	view button if true.
                         */
                        void viewButton ( bool view = true );

                    protected:

                        /**
                         * @fn	void AbleButtonWidget::initialize();
                         * @brief	Method initialize widget.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         */
                        void initialize();

                    private:

                        Gtk::EventBox * mainFrame; ///< main frame
                        Gtk::VBox * mainVbox; ///< main vbox
                        Gtk::HBox * mainHbox; ///< main hbox
                        Gtk::Label * infoText; ///< info text
                        Gtk::EventBox * container; ///< container

                        IAbleButtonWidget * ableWidget; ///< able widget
                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                };
            }
        }
    }
}
#endif /* ABLEBUTTONWIDGET_HXX_ */
