/**
 * @file	AppCalibrationWidget.cxx
 * @brief	Body of the call and wait from application for calibration.
 * @author	Yadickson Soto
 * @date	28/05/2015
 */

#include "AppCalibrationWidget.hxx"
#include <Log/SMTTLogManager.h>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation::Diagnosis;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting::Status;

ISMTTLog * AppCalibrationWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUi.Widgets.AppCalibrationWidget", GETTEXT_PACKAGE );

AppCalibrationWidget::AppCalibrationWidget ( ProgressWidget * progress, bool skipConditional )
    : progress ( progress )
{
    progress->setInfoTitle ( "   " );
    progress->setInfoText ( "  " );
    progress->showInfoFooter ( false );

    TouchScreenCalibrationConfiguration & config ( SAESGUIConfigurationManager::getInstance()->getTouchScreenCalibrationConfiguration() );

    diagnostic = new CalibrationDiagnosis ( config.getNumberOfCalibrationPoints(), skipConditional );
    diagnostic->getSignalEnd().connect ( sigc::mem_fun ( *this, &AppCalibrationWidget::onEndCalibration ) );

    timeOut = config.getWaitingTimeAfterCalibration().present() ? config.getWaitingTimeAfterCalibration().get() : 5000;

    setWidget ( progress );
}

AppCalibrationWidget::~AppCalibrationWidget()
{
    remove();

    if ( progress != NULL )
    {
        delete ( progress );
        progress = NULL;
    }

    if ( diagnostic != NULL )
    {
        delete ( diagnostic );
        diagnostic = NULL;
    }
}

bool AppCalibrationWidget::calibrate()
{
    logger->Debug ( "Run Calibration" );
    progress->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    progress->setInfoText ( "  " );
    diagnostic->test();
    return false;
}

void AppCalibrationWidget::onEndCalibration()
{
    progress->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    progress->setInfoText ( _ ( "Wait while calibration settings are saved" ) );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &AppCalibrationWidget::emit ) , timeOut );
}
