/**
 * @file	AutoDiagnosisBaseWidget.cxx
 * @brief	automatic barcode diagnostic widget class.
 * @author	Yadickson Soto
 * @date	20/06/2013
 */

#include "AutoDiagnosisBaseWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/Diagnosis/PrinterDiagnosis.hxx>
#include <Configuration/label-configuration.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation::Diagnosis;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting::Status;

ISMTTLog * AutoDiagnosisBaseWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.AutoDiagnosisBaseWidget", GETTEXT_PACKAGE );

AutoDiagnosisBaseWidget::AutoDiagnosisBaseWidget ( ProgressWidget * progress )
    : WizardWidgetBase()
    , diagnostic ( NULL )
    , progress ( progress )
{
    setWidget ( progress );

    LabelConfiguration diagConf = SAESGUIConfigurationManager::getInstance()->GetDiagnosisLabelConfiguration();
    progress->setInfoTextConfig ( diagConf );
    LabelConfiguration diagFooterConf = SAESGUIConfigurationManager::getInstance()->getDiagnosisFooterLabelConfiguration();
    progress->setInfoFooterConfig ( diagFooterConf );

    if ( SAESGUIConfigurationManager::getInstance()->getShowFooterInAutoDiagnosis() )
    {
        progress->showInfoFooter();
    }
    else
    {
        progress->hideInfoFooter();
    }

    progress->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::NONE );
    progress->setInfoText ( " " );
    progress->setInfoFooter ( " " );
}

AutoDiagnosisBaseWidget::~AutoDiagnosisBaseWidget()
{
    dispose();
}

void AutoDiagnosisBaseWidget::initialize()
{
    firstMessage();
    conn.disconnect();

    DiagnosticType device = SAESGUIConfigurationManager::getInstance()->getAutoDiagnosisDiagnosticType ( diagnostic->getType() );
    diagnostic->setTime ( device.getWaitTime() );
    diagnostic->setMandatory ( device.getMandatory() );

    conn = diagnostic->getSignalEnd().connect ( sigc::mem_fun ( *this, &AutoDiagnosisBaseWidget::showResult ) );
}

void AutoDiagnosisBaseWidget::dispose()
{
    this->remove();
    conn.disconnect();

    if ( progress )
    {
        delete progress;
        progress = NULL;
    }

    if ( diagnostic )
    {
        delete diagnostic;
        diagnostic = NULL;
    }
}

void AutoDiagnosisBaseWidget::end()
{
    dispose();
    emit();
}

void AutoDiagnosisBaseWidget::firstMessage()
{
    diagnostic->updateMessage();
    progress->setInfoText ( diagnostic->getMessage() );
    progress->setInfoFooter ( diagnostic->getMessageFooter() );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &AutoDiagnosisBaseWidget::runningMessage ), 1000 );
}

void AutoDiagnosisBaseWidget::runningMessage()
{
    diagnostic->setStatus ( VMStatus::VOTING_MACHINE_STATUS_RUN );
    progress->setInfoText ( diagnostic->getMessage() );
    progress->setInfoFooter ( diagnostic->getMessageFooter() );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &AutoDiagnosisBaseWidget::test ), 1000 );
}

void AutoDiagnosisBaseWidget::test()
{
    diagnostic->test();
}

void AutoDiagnosisBaseWidget::showResult()
{
    progress->setInfoText ( diagnostic->getMessage() );
    progress->setInfoFooter ( diagnostic->getMessageFooter() );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &AutoDiagnosisBaseWidget::end ), diagnostic->getTime() );
}

std::string AutoDiagnosisBaseWidget::getTitleText()
{
    return N_ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.Title" );
}
