/**
 * @file	BasicDiagnosticWidget.hxx
 * @brief	Declares the basic diagnostic widget class.
 * @author	Yadickson Soto
 * @date	01/11/2012
 */


#ifndef _BASIC_DIAGNOSTIC_WIDGET_H_
#define _BASIC_DIAGNOSTIC_WIDGET_H_

#include <Widgets/wizard-widget-base.h>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <Gui/Widgets/Diagnose/DiagnosticButtonWidget.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	BasicDiagnosticWidget
                 * @brief	Basic diagnostic widget.
                 * @author	Cesar.bonilla
                 * @date	4/29/2011
                 */

                class BasicDiagnosticWidget: public Smartmatic::GUI::Widgets::WizardWidgetBase
                {

                    protected:

                        sigc::signal<void> signalOkDetected;   ///< The signal ok detected
                        sigc::signal<void> signalCancelDetected;   ///< The signal cancel detected
                        sigc::signal<void> signalRetryDetected;	///< The signal retry detected

                        /**
                         * @fn	void BasicDiagnosticWidget::setOkButtonSensitive ( bool sensitive );
                         * @brief	Sets an ok button sensitive.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	sensitive	true to sensitive.
                         */

                        void setOkButtonSensitive ( bool sensitive );

                        /**
                         * @fn	void BasicDiagnosticWidget::setCancelButtonSensitive ( bool sensitive );
                         * @brief	Sets a cancel button sensitive.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	sensitive	true to sensitive.
                         */

                        void setCancelButtonSensitive ( bool sensitive );

                        /**
                         * @fn	void BasicDiagnosticWidget::setRetryButtonSensitive ( bool sensitive );
                         * @brief	Sets the retry button sensitive property.
                         * @author	Daniel Ampuero
                         * @date	25/09/2012
                         * @param	sensitive	true to sensitive.
                         */
                        void setRetryButtonSensitive ( bool sensitive );
                        /**
                         * @fn	void BasicDiagnosticWidget::setOkButtonLabel ( std::string label );
                         * @brief	Sets an ok button label.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	label	The label.
                         */

                        void setOkButtonLabel ( std::string label );

                        /**
                         * @fn	void BasicDiagnosticWidget::setCancelButtonLabel ( std::string label );
                         * @brief	Sets a cancel button label.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	label	The label.
                         */

                        void setCancelButtonLabel ( std::string label );

                        /**
                         * @fn	void BasicDiagnosticWidget::setRetryButtonLabel ( std::string label );
                         * @brief	Sets the retry button label.
                         * @author	Daniel Ampuero
                         * @date	25/09/2012
                         * @param	label	The label.
                         */
                        void setRetryButtonLabel ( std::string label );

                        /**
                         * @fn	virtual void setWidget(Gtk::Widget * widget);
                         * @brief	widget setter method.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @param	widget	widget.
                         */
                        virtual void setWidget ( Gtk::Widget * widget );

                        /**
                         * @fn	BasicDiagnosticWidget::BasicDiagnosticWidget ( bool showOk = false, bool showCancel = false, bool hasRetry = false );
                         * @brief	Constructor.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	showOk		 	true to show, false to hide the ok.
                         * @param	showCancel 	true to show, false to hide the cancel.
                         * @param	hasRetry	 	(optional) the has retry.
                         */

                        BasicDiagnosticWidget ( bool showOk = false, bool showCancel = false, bool hasRetry = false );

                        /**
                         * @fn	BasicDiagnosticWidget::~BasicDiagnosticWidget ( );
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        virtual ~BasicDiagnosticWidget ( );

                        /**
                         * @fn	void showButtons ( bool showOk, bool showCancel, bool showRetry = false );
                         * @brief	Show or hide buttons.
                         * @author	Yadickson Soto
                         * @date	06/09/2013
                         * @param	showOk		 	true to show, false to hide the ok.
                         * @param	showCancel 	true to show, false to hide the cancel.
                         * @param	showRetry	 	(optional) the has retry.
                         */
                        void showButtons ( bool showOk, bool showCancel, bool showRetry = false );

                    private:

                        /**
                         * @fn	void BasicDiagnosticWidget::on_buttonOk_clicked();
                         * @brief	Handles button ok clicked signals.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void onButtonOkClicked();

                        /**
                         * @fn	void BasicDiagnosticWidget::on_buttonCancel_clicked();
                         * @brief	Handles button cancel clicked signals.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void onButtonCancelClicked();

                        /**
                         * @fn	virtual void BasicDiagnosticWidget::on_buttonRetry_clicked();
                         * @brief	Handles button retry clicked signals.
                         * @author	Daniel Ampuero
                         * @date	25/09/2012
                         */
                        void onButtonRetryClicked();

                        Gtk::VBox vboxPrincipal;	///< The vbox principal
                        Gtk::Frame frameDiagnostic; ///< The frame diagnostic

                        Smartmatic::SAES::GUI::Widgets::DiagnosticButtonWidget * buttons;	///< The button panels
                };
            }
        }
    }
}

#endif // _BASIC_DIAGNOSTIC_WIDGET_H_
