/*
 * CheckListDiagnoseWidget.cxx
 *
 *  Created on: 17/09/2013
 *      Author: soto
 */

#include "CheckListDiagnoseWidget.hxx"
#include <Voting/Status/CheckListDiagnose.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Operation/MachineOperationManager.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::System::Utils;

CheckListDiagnoseWidget::CheckListDiagnoseWidget()
    : BasicDiagnosticWidget ( true, false, false )
{
    signalOkDetected.connect ( mem_fun ( *this, &CheckListDiagnoseWidget::end ) );

    CheckListDiagnose checkListObj;
    std::list < DiagnoseStep > list;
    std::list < DiagnoseStep >::iterator it;

    checkListObj.getCheckList ( list );

    labelMessage = new SmartmaticBasicLabel();
    vbox = new SmartmaticBasicVBox ( false, 10 );
    table = new TableCenterWidget ( false, false );
    table->setSpacings ( 10, 20 );

    vbox->pack_start ( *labelMessage, false, false, 20 );

    LabelConfiguration infoConf = SAESGUIConfigurationManager::getInstance()->GetDiagnosisLabelConfiguration();
    LabelConfiguration::SetLabelConfiguration ( labelMessage, infoConf );

    int row = 0;

    for ( it = list.begin();
            it != list.end();
            ++it, row++ )
    {
        SmartmaticBasicCheckButton * check = manage ( new SmartmaticBasicCheckButton () );
        check->set_label ( "" );

        indexs.push_back ( ( *it ).getCustomCode().get() );
        maps[ ( *it ).getCustomCode().get() ] = check;

        SmartmaticBasicLabel * l = manage ( new SmartmaticBasicLabel() );

        l->set_text ( StringUtils::convertEndLine ( ( *it ).getValue().get() ) );

        Pango::FontDescription font = l->get_style()->get_font();
        font.set_size ( infoConf.GetSize() * 0.8 * Pango::SCALE );
        l->modify_font ( font );
        l->set_alignment ( 0.0, 0.5 );

        l->set_line_wrap ( true );
        l->set_line_wrap_mode ( Pango::WRAP_CHAR );
        l->set_size_request ( 800, -1 );

        table->put ( l, row, 0, Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK );
        table->put ( check, row, 1, Gtk::SHRINK, Gtk::SHRINK );
    }

    vbox->pack_end ( *table, true, true, 0 );

    setWidget ( vbox );
}

CheckListDiagnoseWidget::~CheckListDiagnoseWidget()
{
    if ( vbox )
    {
        delete vbox;
        vbox = NULL;
    }

    if ( table )
    {
        delete table;
        table = NULL;
    }

    if ( labelMessage )
    {
        delete labelMessage;
        labelMessage = NULL;
    }

    std::map < std::string, SmartmaticBasicCheckButton * >::iterator it;

    for ( it = maps.begin();
            it != maps.end();
            ++it )
    {
        SmartmaticBasicCheckButton * check = it->second;
        delete check;
        check = NULL;
    }

    indexs.clear();
    maps.clear();
}

void CheckListDiagnoseWidget::initialize()
{
    labelMessage->set_text ( _ ( "Please check diagnose list" ) );

    if ( maps.empty() )
    {
        end();
    }
}

void CheckListDiagnoseWidget::end()
{
    std::list < std::string >::iterator it;

    for ( it = indexs.begin();
            it != indexs.end();
            ++it )
    {
        std::string customCode = ( *it );
        SmartmaticBasicCheckButton * check = maps [ ( *it ) ];

        MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->addCustomCheckList ( customCode, check->get_active() );
    }

    emit();
}
