/**
 * @file	DateDiagnosticWidget.hxx
 * @brief	date diagnostic widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef DateDiagnosticWidget_HXX_
#define DateDiagnosticWidget_HXX_

#include <gtkmm.h>
#include <glibmm/date.h>
#include <Widgets/smartmatic-button.h>
#include <Widgets/smartmatic-entry.h>
#include <Gui/Widgets/Diagnose/BasicDiagnosticWidget.hxx>
#include <Configuration/SaesGUIConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	DateDiagnosticWidget
                 * @brief	Date diagnostic widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class DateDiagnosticWidget : public Smartmatic::SAES::GUI::Widgets::BasicDiagnosticWidget
                {
                    public:
                        /**
                         * @fn	DateDiagnosticWidget::DateDiagnosticWidget ( Smartmatic::SAES::Configuration::DateDiagnosticWidgetConfiguration & config );
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	config	The configuration.
                         */
                        explicit DateDiagnosticWidget ( Smartmatic::SAES::Configuration::DateDiagnosticWidgetConfiguration & config );
                        /**
                         * @fn	virtual DateDiagnosticWidget::~DateDiagnosticWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~DateDiagnosticWidget();

                        /**
                         * @fn	void DateDiagnosticWidget::getDayText ( std::string & text );
                         * @brief	Gets day text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	text	The text.
                         */
                        void getDayText ( std::string & text );
                        /**
                         * @fn	void DateDiagnosticWidget::getMonthText ( std::string & text );
                         * @brief	Gets month text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	text	The text.
                         */
                        void getMonthText ( std::string & text );
                        /**
                         * @fn	void DateDiagnosticWidget::getYearText ( std::string & text );
                         * @brief	Gets year text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	text	The text.
                         */
                        void getYearText ( std::string & text );

                    private:

                        Gtk::Frame * mainFrame; ///< The main frame
                        Gtk::Table * tableDate; ///< Date of the table
                        Gtk::Label * labelTitle;	///< The label title
                        Gtk::Label * labelDay;	///< The label day
                        Gtk::Label * labelMonth;	///< The label month
                        Gtk::Label * labelYear; ///< The label year

                        Smartmatic::GUI::Widgets::SmartmaticEntry * entryDay; ///< The entry day value
                        Smartmatic::GUI::Widgets::SmartmaticEntry * entryMonth; ///< The entry monht value
                        Smartmatic::GUI::Widgets::SmartmaticEntry * entryYear; ///< The entry year value

                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonDayUp; ///< The button day up
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonDayDown; ///< The button day down
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonMonthUp; ///< The button month up
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonMonthDown; ///< The button month down
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonYearUp; ///< The button year up
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonYearDown; ///< The button year down

                        int buttonArrowWidth; ///< The button arrow width
                        int buttonArrowHeight; ///< The button arrow height
                        int entryWidth; ///< The entry width
                        int entryHeight; ///< The entry height
                        int minYear;	///< The minimum year
                        int maxYear;	///< The maximum year
                        int inputFontSize; ///< input font size

                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                        Smartmatic::SAES::Configuration::DateDiagnosticWidgetConfiguration config;	///< The configuration

                    protected:
                        /**
                         * @fn  void DateDiagnosticWidget::setCurrentDate();
                         * @brief   Sets the current date.
                         * @author  eduardo.fernandez
                         * @date    21/09/2017
                         */
                        void setCurrentDate();

                        /**
                         * @fn	void DateDiagnosticWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void initialize();
                        /**
                         * @fn	void DateDiagnosticWidget::dispose();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void dispose();
                        /**
                         * @fn  std::string DateDiagnosticWidget::checkStringAndAddZero ( std::string input );
                         * @brief   Check string and add zero.
                         * @author  Luiggi.mendez
                         * @date    28/04/2011
                         * @param   input   The input.
                         * @return  .
                         */
                        std::string checkStringAndAddZero ( std::string input );
                        /**
                         * @fn  void DateDiagnosticWidget::on_buttonDayUp_clicked();
                         * @brief   Handles button day up clicked signals.
                         * @author  Luiggi.mendez
                         * @date    28/04/2011
                         */

                        void on_buttonDayUp_clicked();

                        /**
                         * @fn  void DateDiagnosticWidget::on_buttonDayDown_clicked();
                         * @brief   Handles button day down clicked signals.
                         * @author  Luiggi.mendez
                         * @date    28/04/2011
                         */

                        void on_buttonDayDown_clicked();

                        /**
                         * @fn  void DateDiagnosticWidget::on_buttonMonthUp_clicked();
                         * @brief   Handles button month up clicked signals.
                         * @author  Luiggi.mendez
                         * @date    28/04/2011
                         */

                        void on_buttonMonthUp_clicked();

                        /**
                         * @fn  void DateDiagnosticWidget::on_buttonMonthDown_clicked();
                         * @brief   Handles button month down clicked signals.
                         * @author  Luiggi.mendez
                         * @date    28/04/2011
                         */

                        void on_buttonMonthDown_clicked();

                        /**
                         * @fn  void DateDiagnosticWidget::on_buttonYearUp_clicked();
                         * @brief   Handles button year up clicked signals.
                         * @author  Luiggi.mendez
                         * @date    28/04/2011
                         */

                        void on_buttonYearUp_clicked();

                        /**
                         * @fn  void DateDiagnosticWidget::on_buttonYearDown_clicked();
                         * @brief   Handles button year down clicked signals.
                         * @author  Luiggi.mendez
                         * @date    28/04/2011
                         */
                        void on_buttonYearDown_clicked();
                        /**
                         * @fn	void DateDiagnosticWidget::validate();
                         * @brief	Validates this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void validate();
                        /**
                         * @fn	void end ();
                         * @brief	End diagnose.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void end ();
                };
            }
        }
    }
}

#endif /* DateDiagnosticWidget_HXX_ */
