/**
 * @file DeviceDiagnosticPrintWidget.cxx
 * @brief Body device diagnostic print widget
 * @date 19/09/2012
 * @author Yadickson Soto
 */

#include "DeviceDiagnosticPrintWidget.hxx"
#include <Gui/Widgets/FactoryMethodWidget.hxx>
#include <Operation/Devices/P168AController.hxx>
#include <Operation/Devices/PDBController.hxx>
#include <Operation/ZipCodeManager.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Environment/Configurations/resource-path-provider.h>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;

DeviceDiagnosticPrintWidget::DeviceDiagnosticPrintWidget ( GenericPrintingWidget * widget )
    : widget ( widget )
{
    print = NULL;

    initialize();

    setWidget ( widget );
}

DeviceDiagnosticPrintWidget::~DeviceDiagnosticPrintWidget()
{
    this->remove();

    if ( widget )
    {
        delete widget;
        widget = NULL;
    }

    if ( print )
    {
        delete print;
        print = NULL;
    }
}

void DeviceDiagnosticPrintWidget::initialize()
{
    widget->getSignalEnd().connect ( sigc::mem_fun ( *this, &DeviceDiagnosticPrintWidget::endPrint ) );
    widget->getSignalError().connect ( sigc::mem_fun ( *this, &DeviceDiagnosticPrintWidget::endPrint ) );

    setWidget ( widget );
}

void DeviceDiagnosticPrintWidget::start()
{
    widget->setInfoTitle ( _ ( "Smartmatic.SAES.GUI.Widgets.DeviceDiagnosticPrintWidget.Print" ) );

    if ( print )
    {
        delete print;
        print = NULL;
    }

    if ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getInfoMachine() == InfoMachineConfiguration::CONFIGURATION_VM )
    {
      std::string serial = MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->getSerial();
      if ( serial.empty() )
      {
          std::string machineModel;
          std::string serialNumber;
          OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);
          if (StringUtils::toUpper(machineModel) == "VM-A4517")
          {
              serialNumber = PDBController::getInstance()->getSerialNumber();
              PDBController::getInstance()->setSerialNumber ( serialNumber, false );
          }
          else
          {
              serialNumber = P168AController::getInstance()->getSerialNumber();
              P168AController::getInstance()->setSerialNumber ( serialNumber, false );
          }
      }
    }

    std::string zipCode = ZipCodeManager::getZipCode();
    ZipCodeManager::setZipCode ( zipCode, false );

    print = new DiagnosticPrintDocument ( MachineOperationManager::getInstance()->getOperationStatus() );

    widget->start ( print , true, true );
}

void DeviceDiagnosticPrintWidget::endPrint()
{
    emit();
}
