/**
 * @file DeviceResultWidget.hxx
 * @brief Body class device result widget
 * @date 24/10/2011
 * @author Yadickson Soto
 */

#ifndef DEVICERESULTWIDGET_HXX_
#define DEVICERESULTWIDGET_HXX_

#include <gtkmm.h>
#include <Voting/Status/voting-machine-status-schema.hxx>
#include <Widgets/StepWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	DeviceResultWidget
                 * @brief	Device result widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class DeviceResultWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:
                        /**
                         * @fn	DeviceResultWidget::DeviceResultWidget ( Smartmatic::SAES::Voting::Status::DeviceStatus * diagnosis,
                         * 			int labelWidth, int labelHeight );
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	diagnosis	If non-null, the diagnosis.
                         * @param	labelWidth				 	Width of the label.
                         * @param	labelHeight				 	Height of the label.
                         */
                        DeviceResultWidget ( Smartmatic::SAES::Voting::Status::DeviceStatus & diagnosis, int labelWidth, int labelHeight );
                        /**
                         * @fn	virtual DeviceResultWidget::~DeviceResultWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~DeviceResultWidget();
                        /**
                         * @fn	Smartmatic::SAES::Voting::Status::DeviceStatus * DeviceResultWidget::getDiagnosis();
                         * @brief	Gets the diagnosis.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	null if it fails, else the diagnosis.
                         */
                        Smartmatic::SAES::Voting::Status::DeviceStatus & getDiagnosis();
                        /**
                         * @fn	void DeviceResultWidget::update();
                         * @brief	Updates this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void update();

                    protected:
                        /**
                         * @fn	void DeviceResultWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void initialize();
                        /**
                         * @fn	void DeviceResultWidget::dispose();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void dispose();

                    private:
                        Gtk::Frame * mainFrame; ///< The main frame
                        Gtk::Label * label; ///< The label
                        Gtk::Image * image; ///< The image
                        int labelWidth; ///< Width of the label
                        int labelHeight;	///< Height of the label
                        Smartmatic::SAES::Voting::Status::DeviceStatus & diagnosis; ///< The diagnosis
                        static Smartmatic::Log::ISMTTLog * logger;	///< The logger
                };
            }
        }
    }
}

#endif /* DEVICERESULTWIDGET_HXX_ */
