/**
 * @file DiagnosisBlockedWidget.hxx
 * @brief Header class diagnosis blocked
 * @date 12/06/2012
 * @author Yadickson Soto
 */

#ifndef DIAGNOSISBLOCKEDWIDGET_HXX_
#define DIAGNOSISBLOCKEDWIDGET_HXX_

#include <Widgets/StepWidget.hxx>
#include <Widgets/smartmatic-button.h>
#include <Gui/Widgets/ProgressWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	DiagnosisBlockedWidget
                 * @brief	Class diagnosis blocked.
                 * @author	Yadickson Soto
                 * @date	12/06/2012
                 */
                class DiagnosisBlockedWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:
                        /**
                         * @fn	DiagnosisBlockedWidget::DiagnosisBlockedWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress );
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	12/06/2012
                         * @param	progress	progress.
                         */
                        explicit DiagnosisBlockedWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress );
                        /**
                         * @fn	virtual DiagnosisBlockedWidget::~DiagnosisBlockedWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	12/06/2012
                         */
                        virtual ~DiagnosisBlockedWidget();
                        /**
                         * @fn	Glib::SignalProxy0<void> DiagnosisBlockedWidget::getSignalClicked();
                         * @brief	Method getter signal clicked.
                         * @author	Yadickson Soto
                         * @date	12/06/2012
                         * @return	signal clicked.
                         */
                        Glib::SignalProxy0<void> getSignalClicked();

                    protected:

                        /**
                         * @fn	virtual void DiagnosisBlockedWidget::refresh();
                         * @brief	refresh widget method.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         */
                        virtual void refresh();

                    private:

                        Gtk::VBox * mainVBox; ///< main vbox
                        Gtk::EventBox * mainEventBox; ///< main event box
                        Gtk::HButtonBox * hbuttonbox; ///< hbuttonbox
                        Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress; ///< progress widget
                        Smartmatic::GUI::Widgets::SmartmaticButton * button; ///< button
                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                };
            }
        }
    }
}

#endif /* DIAGNOSISBLOCKEDWIDGET_HXX_ */
