/**
 * @file	DiagnosticResultWidget.hxx
 * @brief	diagnostic result widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef DIAGNOSTICRESULTWIDGET_HXX_
#define DIAGNOSTICRESULTWIDGET_HXX_

#include <gtkmm.h>
#include <Widgets/smartmatic-button.h>
#include "DeviceResultWidget.hxx"
#include <list>
#include <Widgets/StepWidget.hxx>
#include <Voting/Status/voting-machine-status-schema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	DiagnosticResultWidget
                 * @brief	Diagnostic result widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class DiagnosticResultWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:
                        /**
                         * @fn	DiagnosticResultWidget::DiagnosticResultWidget ( bool forceButtonFailed, int labelWidth,
                         * 			int labelHeight );
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	forceButtonFailed	true if force button failed.
                         * @param	labelWidth			 	Width of the label.
                         * @param	labelHeight			 	Height of the label.
                         */
                        DiagnosticResultWidget ( bool forceButtonFailed, int labelWidth, int labelHeight );
                        /**
                         * @fn	virtual DiagnosticResultWidget::~DiagnosticResultWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~DiagnosticResultWidget();

                        /**
                         * @fn	virtual void DiagnosticResultWidget::refresh();
                         * @brief	Refreshes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual void refresh();

                        /**
                         * @brief  Sets the time to show the results when action buttons are disabled.
                         * @author manuel.arguelles
                         * @date   Sep  5, 2017
                         * @param  timeToShow The time to show the results in milliseconds.
                         */
                        void setTimeToShow(int timeToShow);

                    protected:
                        /**
                         * @fn	void DiagnosticResultWidget::end();
                         * @brief	Ends this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void end();
                        /**
                         * @fn	void DiagnosticResultWidget::fail();
                         * @brief	Fails this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void fail();
                        /**
                         * @fn	void DiagnosticResultWidget::shutDown();
                         * @brief	Shuts down this object and frees any resources it is using.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void shutDown();
                        /**
                         * @fn	void DiagnosticResultWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void initialize();
                        /**
                         * @fn	void DiagnosticResultWidget::dispose();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void dispose();

                    private:

                        /**
                         * @fn  void DiagnosticResultWidget::onSignalBlock();
                         * @brief   Signal block handler.
                         * @author  eduardo.fernandez
                         * @date    10/08/2017
                         */
                        void onSignalBlock();

                        /**
                         * @property	std::list <Smartmatic::SAES::Voting::Status::DeviceStatus *> diagnosis
                         * @brief	Gets the diagnosis.
                         * @return	The diagnosis.
                         */
                        std::list <Smartmatic::SAES::Voting::Status::DeviceStatus > diagnosis;
                        /**
                         * @property	std::list < DeviceResultWidget * > deviceList
                         * @brief	Gets a list of devices.
                         * @return	A List of devices.
                         */
                        std::list < DeviceResultWidget * > deviceList;
                        bool mandatoryFail; ///< true to mandatory fail
                        bool forceButtonFailed; ///< force use buttons
                        Gtk::Frame * mainFrame; ///< The main frame
                        Gtk::Frame * centerFrame; ///< The center frame
                        Gtk::VBox * mainVbox; ///< The main vbox
                        Gtk::VBox * centerVbox; ///< The center vbox
                        Gtk::HBox * centerHbox; ///< The center hbox
                        Gtk::Label * infoLabel; ///< The information label
                        Gtk::VButtonBox * buttonbox;	///< The buttonbox
                        Gtk::HBox * buttonHBox;
                        int labelWidth; ///< Width of the label
                        int labelHeight;	///< Height of the label
                        Smartmatic::GUI::Widgets::SmartmaticButton * actionButton;	///< The action button
                        Smartmatic::GUI::Widgets::SmartmaticButton * shutdownButton;
                        static Smartmatic::Log::ISMTTLog * logger;	///< The logger
                        sigc::connection memorySignal;
                        bool showInstallationScreen;
                        int timeToShow;
                };
            }
        }
    }
}

#endif /* DIAGNOSTICRESULTWIDGET_HXX_ */
