/*
 * EurnActionsWidget.hxx
 *
 *  Created on: Nov 7, 2017
 *      Author: juanvalera
 */

#ifndef EURNACTIONSWIDGET_HXX_
#define EURNACTIONSWIDGET_HXX_

#include <Gui/Widgets/Diagnose/BasicDiagnosticWidget.hxx>
#include <Gui/Widgets/Diagnose/EurnEndDiagnoseWidget.hxx>
#include <Gui/Widgets/Diagnose/AlarmBoxDiagnoseStepWidget.hxx>
#include <Operation/Devices/EurnController.hxx>
#include <Widgets/SmartmaticBasicFrame.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/smartmatic-button.h>
#include <set>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	EurnActionsWidget
                 * @brief	eUrn actions diagnosis widget.
                 * @author	Juan Valera
                 * @date	07/11/2017
                 */

                class EurnActionsWidget: public BasicDiagnosticWidget, private Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @fn	EurnActionsWidget::EurnActionsWidget ();
                         * @brief	Constructor.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        EurnActionsWidget ();

                        /**
                         * @fn	virtual ~EurnActionsWidget();
                         * @brief	Destroy.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        virtual ~EurnActionsWidget();

                        /**
                         * @fn	void EurnActionsWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void initialize();
                        /**
                         * @fn	virtual void dispose();
                         * @brief	virtual method stop.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        virtual void dispose();

                        /**
                         * @fn	void onSignalEurnStatus ( Smartmatic::SAES::Operation::Devices::EurnController::DeviceStatus status );
                         * @brief	Callback signal Eurn status changed
                         * @author	Juan Valera
                         * @date	08/11/2017
                         * @param	status	status Eurn.
                         */
                        void onSignalEurnStatus ( Smartmatic::SAES::Operation::Devices::EurnController::DeviceStatus status );

                        /**
                         * @fn	void updateEurnStatus();
                         * @brief	Updates the Eurn status.
                         * @author	Juan Valera
                         * @date	08/11/2017
                         */
                        void updateEurnStatus(Smartmatic::SAES::Operation::Devices::EurnController::DeviceStatus status);

                    private:

                        /**
                         * @fn	enum ButtonStateTest
                         * @brief	Button action ON or OFF
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        enum ButtonStateTest
                        {
                            ON,
                            OFF
                        };

                        int timeOk;   ///< The time diagnose successfully
                        int timeFail;   ///< The time diagnose fail

                        Smartmatic::GUI::Widgets::SmartmaticBasicFrame  frame; ///< The principal frame
                        Gtk::Alignment  alignment; ///< The principal container of labels and buttons

                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox  vboxOptions; ///< The vbox container

                        Smartmatic::GUI::Widgets::SmartmaticButton  buttonOpenDoor; ///< Button open test
                        Smartmatic::GUI::Widgets::SmartmaticButton  buttonCloseDoor; ///< Button close test
                        Smartmatic::GUI::Widgets::SmartmaticButton  buttonRedLed; ///< Button red led test
                        Smartmatic::GUI::Widgets::SmartmaticButton  buttonGreenLed; ///< Button green led test

                        Smartmatic::SAES::GUI::Widgets::AlarmBoxDiagnoseStepWidget  openDoorOption; ///< buzzer test
                        Smartmatic::SAES::GUI::Widgets::AlarmBoxDiagnoseStepWidget  redLedOption; ///< red led test
                        Smartmatic::SAES::GUI::Widgets::AlarmBoxDiagnoseStepWidget  greenLedOption; ///< green led test
                        Smartmatic::SAES::GUI::Widgets::AlarmBoxDiagnoseStepWidget  closeDoorOption; ///< button test

                        Smartmatic::SAES::GUI::Widgets::EurnEndDiagnoseWidget * endWidget; ///< End widget

                        ButtonStateTest openDoorState; ///< The buzzer state
                        ButtonStateTest redLedState; ///< The red led state
                        ButtonStateTest greenLedState; ///< The green led state
                        ButtonStateTest closeDoorState; ///< The button state

                        sigc::connection connOk; ///< The connection signal dispatcher ok
                        sigc::connection connError; ///< The connection signal dispatcher error
                        sigc::connection conn_updateEurnStatus;///< The connection signal dispatcher Eurn status changed;

                        Smartmatic::System::SafeDispatcher signalEnableButtons; ///< The signal to enabled buttons
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger

                        bool showOpenDoorButton; ///< flag show open door button test
                        bool showRedLedButton; ///< flag show red led button test
                        bool showGreenLedButton; ///< flag show red led button test
                        bool showCloseDoorButton; ///< flag show close door button test

                        /**
                         * @fn	void onRedLedAction ();
                         * @brief	Action ON or OFF Red Led.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void onRedLedAction ();

                        /**
                         * @fn	void onGreenLedAction ();
                         * @brief	Action ON or OFF Green Led.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void onGreenLedAction ();

                        /**
                         * @fn	void onOpenDoorAction ();
                         * @brief	Action Open door.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void onOpenDoorAction ();

                        /**
                         * @fn	void onCloseDoorAction ();
                         * @brief	Action Close Door.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void onCloseDoorAction ();

                        /**
                         * @fn	void validate();
                         * @brief	Validate if running all test in true.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void validate();

                        /**
                         * @fn	void end();
                         * @brief	End test diagnostic.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void end();

                        /**
                         * @fn	void error();
                         * @brief	End test diagnostic fail.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void error();

                        /**
                         * @fn	void viewSuccess();
                         * @brief	View diagnose successfully
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void viewSuccess();

                        /**
                         * @fn	void viewCancel();
                         * @brief	View diagnose failed
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void viewCancel();

                        /**
                         * @fn	    virtual bool process ();
                         * @brief	Callback from thread only one time. Send command in background
                         * @author	Juan Valera
                         * @date	07/11/2017
                         * @return always false
                         */
                        virtual bool process ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                        /**
                         * @fn	    void setStatus();
                         * @brief	End diagnoses to set status result.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void setStatus();
                        /**
                         * @fn	    void restoreEurnState();
                         * @brief	restore eUrn state.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void restoreEurnState();
};
            }
        }
    }
}
#endif /* EURNACTIONSWIDGET_HXX_ */
