/*
 * EurnDetectedWidget.hxx
 *
 *  Created on: Nov 7, 2017
 *      Author: juanvalera
 */

#ifndef EURNDETECTEDWIDGET_HXX_
#define EURNDETECTEDWIDGET_HXX_

#include <Widgets/wizard-widget-base.h>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	EurnDetectedWidget
                 * @brief	eUrn detected widget.
                 * @author	Juan Valera
                 * @date	07/11/2017
                 */

                class EurnDetectedWidget: public Smartmatic::GUI::Widgets::WizardWidgetBase, private Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @fn	EurnDetectedWidget::EurnDetectedWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * widget );
                         * @brief	Constructor.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         * @param 	widget the visual container
                         */
                        explicit EurnDetectedWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * widget );

                        /**
                         * @fn	EurnDetectedWidget::~EurnDetectedWidget();
                         * @brief	Destroy.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        virtual ~EurnDetectedWidget();

                        /**
                         * @fn	void EurnDetectedWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void initialize();

                    private:

                        int timeOK; ///< Timeout end eUrn detected successfully
                        int timeFail; ///< Timeout end eUrn detected fail

                        Smartmatic::System::SafeDispatcher signalEndDetected; ///< The signal end detected eUrn box

                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                        Smartmatic::SAES::GUI::Widgets::ProgressWidget * widget; ///< The visual container

                    protected:

                        /**
                         * @fn	    virtual bool process ();
                         * @brief	Callback from thread one time. Start detection eUrn
                         * @author	Juan Valera
                         * @date	07/11/2017
                         * @return false always
                         */
                        virtual bool process ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                        /**
                         * @fn	void EurnDetectedWidget::error();
                         * @brief	End test diagnostic fail.
                         * @author	Juan Valera
                         * @date	07/11/2017
                         */
                        void error();
                };
            }
        }
    }
}
#endif /* EURNDETECTEDWIDGET_HXX_ */
