/**
 * @file IAbleButtonWidget.cxx
 * @brief Body class able widget
 * @date 06/09/2012
 * @author Yadickson Soto
 */

#include "IAbleButtonWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::Log;

ISMTTLog * IAbleButtonWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.IAbleButtonWidget", GETTEXT_PACKAGE );

IAbleButtonWidget::IAbleButtonWidget()
    : Gtk::EventBox()
{
    imageBase = NULL;
    container = NULL;
    mapButtons.clear();
    a4517MapButtons.clear();
}

IAbleButtonWidget::~IAbleButtonWidget()
{
    remove();

    if ( imageBase )
    {
        delete imageBase;
        imageBase = NULL;
    }

    if ( container )
    {
        delete container;
        container = NULL;
    }

    std::map <AbleButtonType, Gtk::Image *>::iterator it;

    for ( it = mapButtons.begin();
            it != mapButtons.end();
            ++it )
    {
        Gtk::Image * image = it->second;

        if ( image )
        {
            delete image;
            image = NULL;
        }
    }

    std::map <A4517AbleButtonType, Gtk::Image *>::iterator iterator;

    for ( iterator = a4517MapButtons.begin();
            iterator != a4517MapButtons.end();
            ++iterator )
    {
        Gtk::Image * image = iterator->second;

        if ( image )
        {
            delete image;
            image = NULL;
        }
    }

    buttons.clear();
    a4517Buttons.clear();
    mapButtons.clear();
    a4517MapButtons.clear();
}

Gtk::Image * IAbleButtonWidget::getButton ( AbleButtonType button )
{
    Gtk::Image * image = NULL;

    if ( mapButtons.find ( button ) != mapButtons.end() )
    {
        image = mapButtons[button];
    }

    return image;
}

Gtk::Image * IAbleButtonWidget::getButton ( A4517AbleButtonType button )
{
    Gtk::Image * image = NULL;

    if ( a4517MapButtons.find ( button ) != a4517MapButtons.end() )
    {
        image = a4517MapButtons[button];
    }

    return image;
}

bool IAbleButtonWidget::setStateButton ( AbleButtonType button, bool state )
{
    Gtk::Image * image = getButton ( button );

    if ( image )
    {
        image->set_visible ( state );
    }
    else
    {
        logger->Debug ( "Image Button " + button + " not found." );
    }

    return image != NULL;
}

bool IAbleButtonWidget::setStateButton ( A4517AbleButtonType button, bool state )
{
    Gtk::Image * image = getButton ( button );

    if ( image )
    {
        image->set_visible ( state );
    }
    else
    {
        logger->Debug ( "Image Button " + button + " not found." );
    }

    return image != NULL;
}

bool IAbleButtonWidget::pressButton ( AbleButtonType button )
{
    return setStateButton ( button, true );
}

bool IAbleButtonWidget::pressButton ( A4517AbleButtonType button )
{
    return setStateButton ( button, true );
}

bool IAbleButtonWidget::releaseButton ( AbleButtonType button )
{
    return setStateButton ( button, false );
}

std::list <AbleButtonType> & IAbleButtonWidget::getButtons()
{
    return buttons;
}

std::list <A4517AbleButtonType> & IAbleButtonWidget::getA4517Buttons()
{
    return a4517Buttons;
}

void IAbleButtonWidget::addButton ( AbleButtonType button, Gtk::Image * image )
{
    if ( image != NULL && mapButtons.find ( button ) == mapButtons.end() )
    {
        buttons.push_back ( button );
        mapButtons[button] = image;
    }
}

void IAbleButtonWidget::addButton ( A4517AbleButtonType button, Gtk::Image * image )
{
    if ( image != NULL && a4517MapButtons.find ( button ) == a4517MapButtons.end() )
    {
        a4517Buttons.push_back ( button );
        a4517MapButtons[button] = image;
    }
}
