/**
 * @file IAbleButtonWidget.hxx
 * @brief Header class able p168a widget
 * @date 06/09/2012
 * @author Yadickson Soto
 */

#ifndef IABLEBUTTONWIDGET_HXX_
#define IABLEBUTTONWIDGET_HXX_

#include <gtkmm.h>
#include <map>
#include <Configuration/s-a-e-s-hardware-conf-manager.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	IAbleButtonWidget
                 * @brief	Class able widget.
                 * @author	Yadickson Soto
                 * @date	06/09/2012
                 */
                class IAbleButtonWidget : public Gtk::EventBox
                {
                    public:
                        /**
                         * @fn	IAbleButtonWidget::IAbleButtonWidget();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         */
                        IAbleButtonWidget();
                        /**
                         * @fn	virtual IAbleButtonWidget::~IAbleButtonWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         */
                        virtual ~IAbleButtonWidget();
                        /**
                         * @fn	bool IAbleButtonWidget::pressButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                         * @brief	Method show press button.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         * @param	button	button to press.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool pressButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                        /**
                         * @fn  bool IAbleButtonWidget::pressButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button );
                         * @brief   Method show press button.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button  button to press.
                         * @return  true if it succeeds, false if it fails.
                         */
                        bool pressButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button );
                        /**
                         * @fn	bool IAbleButtonWidget::releaseButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                         * @brief	Method hide press button.
                         * @author	Yadickson Soto
                         * @date	06/09/2012
                         * @param	button	button to release.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool releaseButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                        /**
                         * @fn	std::list <Smartmatic::SAES::HW::Configuration::AbleButtonType> & IAbleButtonWidget::getButtons();
                         * @brief	Method getter orden button.
                         * @author	Yadickson Soto
                         * @date	10/09/2012
                         * @return	button button to release.
                         */
                        std::list <Smartmatic::SAES::HW::Configuration::AbleButtonType> & getButtons();
                        /**
                         * @fn  std::list <Smartmatic::SAES::HW::Configuration::A4517AbleButtonType> & IAbleButtonWidget::getA4517Buttons();
                         * @brief   Method getter orden button.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @return  buttons to release.
                         */
                        std::list <Smartmatic::SAES::HW::Configuration::A4517AbleButtonType> & getA4517Buttons();

                    protected:
                        /**
                         * @fn	virtual Gtk::Image * IAbleButtonWidget::getButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                         * @brief	Method get button.
                         * @author	Yadickson Soto
                         * @date	10/09/2012
                         * @param	button	button to search.
                         * @return	image.
                         */
                        virtual Gtk::Image * getButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button );
                        /**
                         * @fn  virtual Gtk::Image * IAbleButtonWidget::getButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button );
                         * @brief   Method get button.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button  button to search.
                         * @return  image.
                         */
                        virtual Gtk::Image * getButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button );
                        /**
                         * @fn	virtual bool IAbleButtonWidget::setStateButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button,
                         * 			bool state );
                         * @brief	Method setter state button.
                         * @author	Yadickson Soto
                         * @date	10/09/2012
                         * @param	button	button to search.
                         * @param	state 	new state.
                         * @return	true is success.
                         */
                        virtual bool setStateButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button, bool state );
                        /**
                         * @fn  virtual bool IAbleButtonWidget::setStateButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button,
                         *          bool state );
                         * @brief   Method setter state button.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button  button to search.
                         * @param   state   new state.
                         * @return  true is success.
                         */
                        virtual bool setStateButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button, bool state );
                        /**
                         * @fn	virtual void IAbleButtonWidget::addButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button,
                         * 			Gtk::Image * image );
                         * @brief	Method add button.
                         * @author	Yadickson Soto
                         * @date	10/09/2012
                         * @param	button	new button type.
                         * @param	image 	new image.
                         */
                        virtual void addButton ( Smartmatic::SAES::HW::Configuration::AbleButtonType button, Gtk::Image * image );
                        /**
                         * @fn  virtual void IAbleButtonWidget::addButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button,
                         *          Gtk::Image * image );
                         * @brief   Method add button.
                         * @author  eduardo.fernandez
                         * @date    22/01/2018
                         * @param   button  new button type.
                         * @param   image   new image.
                         */
                        virtual void addButton ( Smartmatic::SAES::HW::Configuration::A4517AbleButtonType button, Gtk::Image * image );

                        Gtk::Image * imageBase; ///< image base
                        Gtk::Fixed * container; ///< container

                    private:

                        std::list <Smartmatic::SAES::HW::Configuration::AbleButtonType> buttons; ///< all buttons
                        std::list <Smartmatic::SAES::HW::Configuration::A4517AbleButtonType> a4517Buttons; ///< all a4517 abled buttons
                        std::map <Smartmatic::SAES::HW::Configuration::AbleButtonType, Gtk::Image *> mapButtons; ///< map buttons
                        std::map <Smartmatic::SAES::HW::Configuration::A4517AbleButtonType, Gtk::Image *> a4517MapButtons; ///< a45147 map buttons
                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                };
            }
        }
    }
}
#endif /* IABLEBUTTONWIDGET_HXX_ */
