/**
 * @file	SmartcardDetectedWidget.cxx
 * @brief	smartcard reader device detected widget class.
 * @author	Yadickson Soto
 * @date	12/09/2013
 */

#include "SmartcardDetectedWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Operation::Monitor;

ISMTTLog * SmartcardDetectedWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.SmartcardDetectedWidget", GETTEXT_PACKAGE );

SmartcardDetectedWidget::SmartcardDetectedWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget )
    : WizardWidgetBase (), progressWidget ( progressWidget )
{
    timeStep = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getTimeStep();
    timeFail = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getTimeFail();

    setWidget ( progressWidget );
}

SmartcardDetectedWidget::~SmartcardDetectedWidget()
{
    remove();

    if ( progressWidget )
    {
        delete progressWidget;
        progressWidget = NULL;
    }
}

void SmartcardDetectedWidget::initialize()
{
    progressWidget->setInfoText ( _ ( "Detecting Smartcard Reader" ) );
    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );

    unsigned int numberOfDevices = DeviceTypeMonitorPool::getInstance()->getNumberOfDevicesByType ( DeviceType::CONFIGURATION_SMARTCARD );

    if ( numberOfDevices != 0 )
    {
        dispose();
        progressWidget->setInfoText ( _ ( "Smartcard Reader Device Detected" ) );
        progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_DETECT );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardDetectedWidget::emit ), timeStep );
    }
    else
    {
        progressWidget->setInfoText ( _ ( "Smartcard Reader Device Not Found" ) );
        progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_DISABLED );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardDetectedWidget::error ), timeFail );
    }
}

void SmartcardDetectedWidget::error()
{
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setStatusSmartcardReaderNotDetected();
    on_stop_signal_emit().emit();
}

void SmartcardDetectedWidget::dispose()
{

}
