/**
 * @file WizardDiagnoseWidget.cxx
 * @brief Body wizard diagnose widget
 * @date 09/09/2013
 * @author Yadickson Soto
 */

#include "WizardDiagnoseWidget.hxx"
#include "AlarmBoxActionsWidget.hxx"
#include <Operation/MachineOperationManager.hxx>

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting;

WizardDiagnoseWidget::WizardDiagnoseWidget ( Smartmatic::SAES::Common::Configuration::DeviceType type )
    : WizardWidgetBase()
{
    this->type = type;

    manager = new WizardWidgetManager ();

    setWidget ( manager );
}

WizardDiagnoseWidget::~WizardDiagnoseWidget()
{
    this->remove();
    dispose();

    std::vector < WizardWidgetBase * >::iterator it;

    for ( it = wizardVector.begin();
            it != wizardVector.end();
            ++it )
    {
        WizardWidgetBase * widget = ( *it );
        delete widget;
        widget = NULL;
    }

    wizardVector.clear();

    if ( manager )
    {
        delete manager;
        manager = NULL;
    }
}

void WizardDiagnoseWidget::start()
{
    dispose();

    std::vector < Smartmatic::GUI::Widgets::WizardWidgetBase * >::iterator it;

    for ( it = wizardVector.begin();
            it != wizardVector.end();
            ++it )
    {
        connections.push_back ( ( *it )->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &WizardDiagnoseWidget::error ) ) );
        connections.push_back ( ( *it )->on_cancel_signal_emit().connect ( sigc::mem_fun ( *this, &WizardDiagnoseWidget::cancel ) ) );
    }

    managerConnection = manager->on_finish_signal_emit().connect ( sigc::mem_fun ( *this, &WizardDiagnoseWidget::end ) );
    manager->Start ( wizardVector );
}

void WizardDiagnoseWidget::dispose()
{
    managerConnection.disconnect();

    std::vector < sigc::connection >::iterator it;

    for ( it = connections.begin();
            it != connections.end();
            ++it )
    {
        ( *it ).disconnect();
    }

    connections.clear();
}

void WizardDiagnoseWidget::end()
{
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setDiagnosisStatus ( type, VMStatus::VOTING_MACHINE_STATUS_OK, false );
    emit();
}

void WizardDiagnoseWidget::error()
{
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setDiagnosisStatus ( type, VMStatus::VOTING_MACHINE_STATUS_ERROR, false );
    emit();
}

void WizardDiagnoseWidget::cancel()
{
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setDiagnosisStatus ( type, VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED, false );
    emit();
}
