/**
 * @file ZipCodeWidget.hxx
 * @brief Header class zip code widget
 * @date 27/09/2012
 * @author Yadickson Soto
 */

#ifndef _ZIP_CODE_WIDGET_HXX_
#define _ZIP_CODE_WIDGET_HXX_

#include <Gui/Widgets/Diagnose/BasicDiagnosticWidget.hxx>
#include <Widgets/smartmatic-entry.h>
#include <Widgets/keypad-widget.h>
#include <Widgets/key-pad-widget-interface.h>
#include <Widgets/smartmatic-button.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	ZipCodeWidget
                 * @brief	Class zip code widget.
                 * @author	Yadickson Soto
                 * @date	27/09/2012
                 */
                class ZipCodeWidget : public BasicDiagnosticWidget,
                    public Smartmatic::GUI::Widgets::KeypadInterface
                {
                    public:
                        /**
                         * @fn	ZipCodeWidget::ZipCodeWidget ( bool showKeyBoard );
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         * @param	showKeyBoard	show key board.
                         */
                        explicit ZipCodeWidget ( bool showKeyBoard );
                        /**
                         * @fn	virtual ZipCodeWidget::~ZipCodeWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         */
                        virtual ~ZipCodeWidget();
                        /**
                         * @fn	std::string ZipCodeWidget::getZipCode();
                         * @brief	Method getter zip code.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         * @return	zip code.
                         */
                        std::string getZipCode();
                        /**
                         * @fn	void ZipCodeWidget::setZipCode ( std::string zipCode );
                         * @brief	Method setter zip code.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         * @param	zipCode	zip code.
                         */
                        void setZipCode ( std::string zipCode );

                        /**
                         * @fn	void ZipCodeWidget::initialize();
                         * @brief	Method initialize widget.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         */
                        virtual void initialize();

                    protected:
                        /**
                         * @fn	virtual void ZipCodeWidget::SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value );
                         * @brief	virtual method setter key.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         * @param	value	value key.
                         */
                        virtual void SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value );
                        /**
                         * @fn	virtual void ZipCodeWidget::OnKeyPress ( int data );
                         * @brief	virtual method key press.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         * @param	data	key value.
                         */
                        virtual void OnKeyPress ( int data );
                        /**
                         * @fn	virtual void ZipCodeWidget::OnKeyPressT9 ( char data );
                         * @brief	virtual method key press.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         * @param	data	key value.
                         */
                        virtual void OnKeyPressT9 ( char data );

                        /**
                         * @fn	void ZipCodeWidget::validate();
                         * @brief	Method validate zip code.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         */
                        void validate();
                        /**
                         * @fn	void ZipCodeWidget::clearMessage();
                         * @brief	Method clear message.
                         * @author	Yadickson Soto
                         * @date	27/09/2012
                         */
                        void clearMessage();

                    private:

                        Gtk::EventBox * mainFrame; ///< main frame
                        Gtk::Alignment * mainAlignment; ///< main alignment
                        Gtk::Alignment * tableAlignment; ///< table alignment
                        Gtk::Alignment * keyBoardAlignment; ///< key board alignment
                        Gtk::VBox * mainVbox; ///< main vbox
                        Gtk::Label * titleText; ///< info text
                        Gtk::Label * zipCodeText; ///< info text
                        Gtk::Label * messageText; ///< info text
                        Gtk::Table * table; ///< container
                        Gtk::HSeparator * separator; ///< separator
                        Smartmatic::GUI::Widgets::KeypadWidget * keyboard; ///< keypad test
                        Smartmatic::GUI::Widgets::SmartmaticEntry * zipCodeEntry; ///< zip code entry

                        bool showKeyBoard; ///< show key board
                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                };
            }
        }
    }
}
#endif /* _ZIP_CODE_WIDGET_HXX_ */
