/**
 * @file   ExternalPrintWidget.hxx
 * @brief  Declaration of ExternalPrintWidget class.
 *         This is a wrapper for ExternalPrintWizardWidget.
 * @author manuel.arguelles
 * @date   Jun  9, 2017
 */

#ifndef _EXTERNAL_PRINT_WIDGET_H_
#define _EXTERNAL_PRINT_WIDGET_H_

#include <Widgets/wizard-widget-base.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                class ExternalPrintWizardWidget;

                /**
                 * @class  ExternalPrintWidget
                 * @brief  Wrapper adapter for ExternalPrintWizardWidget.
                 * @author manuel.arguelles
                 * @date   Jun  9, 2017
                 */
                class ExternalPrintWidget : public Smartmatic::GUI::Widgets::WizardWidgetBase
                {
                    public:
                        /**
                         * @brief  Class constructor.
                         * @author manuel.arguelles
                         * @date   Jun  9, 2017
                         */
                        ExternalPrintWidget();

                        /**
                         * @brief  Class destructor.
                         * @author manuel.arguelles
                         * @date   Jun  9, 2017
                         */
                        virtual ~ExternalPrintWidget();

                        /**
                         * @brief  Starts the external printer wizard.
                         * @author manuel.arguelles
                         * @date   Jun  9, 2017
                         */
                        virtual void initialize();

                    private:
                        /**
                         * @brief  Emits signal when done.
                         * @author manuel.arguelles
                         * @date   Jun  9, 2017
                         */
                        void onWizardDone();

                    private:
                        ExternalPrintWizardWidget* m_wizardWidget;
                        sigc::connection m_connection;
                };
            }
        }
    }
}

#endif /* _EXTERNAL_PRINT_WIDGET_H_ */
