/**
 * @file   ExternalPrintWizardWidget.hxx
 * @brief  Declaration of ExternalPrintWizardWidget class.
 * @author manuel.arguelles
 * @date   Jun  6, 2017
 */

#ifndef _EXTERNAL_PRINT_WIZARD_WIDGET_H_
#define _EXTERNAL_PRINT_WIZARD_WIDGET_H_

#include <Operation/MemoryEventManager.hxx>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Widgets/SmartmaticBasicHBox.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/StepWidget.hxx>
#include <Widgets/smartmatic-button.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                class ProgressWidget;

                /**
                 * @class  ExternalPrintWizardWidget
                 * @brief  External print wizard widget.
                 * @author manuel.arguelles
                 * @date   Jun  6, 2017
                 */
                class ExternalPrintWizardWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:
                        /**
                         * @brief  Class constructor.
                         * @author manuel.arguelles
                         * @date   Jun  6, 2017
                         * param   progress The progress widget.
                         */
                        explicit ExternalPrintWizardWidget(Smartmatic::SAES::GUI::Widgets::ProgressWidget* progress);

                        /**
                         * @brief  Class destructor.
                         * @author manuel.arguelles
                         * @date   Jun  6, 2017
                         */
                        virtual ~ExternalPrintWizardWidget();

                        /**
                         * @brief  Initialize the wizard widget.
                         * @author manuel.arguelles
                         * @date   Jun  7, 2017
                         */
                        void Initialize();

                        typedef sigc::signal<void> FinishSignal; /**< Finish signal type */

                        /**
                         * @brief  Signal when wizard finalizes.
                         * @author manuel.arguelles
                         * @date   Jun  8, 2017
                         */
                        FinishSignal getFinishSignal();

                    private:
                        /**
                         * @brief  Method to handle memory events.
                         * @author manuel.arguelles
                         * @date   Jun  8, 2017
                         * @param  event The event type.
                         */
                        void onMemoryEvent(Smartmatic::SAES::Operation::MemoryEventManager::MemoryEvent event);

                        /**
                         * @brief  Method to cancel the wizard.
                         * @author manuel.arguelles
                         * @date   Jun  8, 2017
                         */
                        void onCancel();

                        /**
                         * @brief  Method to complete the wizard.
                         * @author manuel.arguelles
                         * @date   Jun  8, 2017
                         */
                        void onComplete();

                        /**
                         * @brief  Method while waiting for usb removal.
                         * @author manuel.arguelles
                         * @date   Jun  8, 2017
                         */
                        void onWaitingToPrint();

                        /**
                         * @brief  Method while printing.
                         * @author manuel.arguelles
                         * @date   Jun  8, 2017
                         */
                        void onPrinting();

                        /**
                         * @brief  Restore the memory monitor.
                         * @author manuel.arguelles
                         * @date   Jun  9, 2017
                         */
                        void restoreMonitor();

                        /**
                         * @brief  Occurs when the final button is clicked.
                         * @author manuel.arguelles
                         * @date   Jul  6, 2022
                         */
                        void onOkButtonClicked();

                    private:
                        FinishSignal                                    m_finishSignal;
                        Smartmatic::GUI::Widgets::SmartmaticBasicHBox   m_okHBox;
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox   m_mainVBox;
                        Smartmatic::GUI::Widgets::SmartmaticButton      m_mainButton;
                        Smartmatic::GUI::Widgets::SmartmaticButton      m_okButton;
                        Smartmatic::SAES::GUI::Widgets::ProgressWidget* m_progress;
                        sigc::connection                                m_buttonConnection;
                        sigc::connection                                m_memoryEventConnection;
                        static bool                                     m_firstRun;
                };
            }
        }
    }
}

#endif /* _EXTERNAL_PRINT_WIZARD_WIDGET_H_ */
