/**
 * @file FillTableWidget.hxx
 * @brief Header class fill table mode
 * @date 15/03/2012
 * @author Yadickson Soto
 */

#ifndef FILLTABLEWIDGET_HXX_
#define FILLTABLEWIDGET_HXX_

#include <gtkmm.h>
#include <list>
#include <Widgets/BoxSelectionWidget.hxx>
#include <Widgets/TableBoxSelectionWidget.hxx>
#include <Gui/Widgets/IFillTableWidget.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	FillTableWidget
                 * @brief	Class FillTableWidget.
                 * @author	Yadickson Soto
                 * @date	15/03/2012
                 */
                class FillTableWidget : public Smartmatic::GUI::Widgets::SmartmaticBasicEventBox, public IFillTableWidget
                {
                    public:
                        /**
                         * @fn	FillTableWidget::FillTableWidget ( Smartmatic::GUI::Widgets::TableBoxSelectionWidget * tableWidget );
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	15/03/2012
                         * @param [in,out]	tableWidget	If non-null, the table widget.
                         */
                        explicit FillTableWidget ( Smartmatic::GUI::Widgets::TableBoxSelectionWidget * tableWidget );
                        /**
                         * @fn	virtual FillTableWidget::~FillTableWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	15/03/2012
                         */
                        virtual ~FillTableWidget();
                        /**
                         * @fn	void FillTableWidget::update();
                         * @brief	method update.
                         * @author	Yadickson Soto
                         * @date	15/03/2012
                         */
                        void update();
                        /**
                         * @fn	virtual void putStandInTitle ();
                         * @brief	method fill stand in title.
                         * @author	Yadickson Soto
                         * @date	23/04/2013
                         */
                        virtual void putStandInTitle ();
                        /**
                         * @fn	Smartmatic::GUI::Widgets::TableBoxSelectionWidget * getTableWidget();
                         * @brief	Gets the ballot options TableBoxSelectionWidget.
                         * @author	Yadickson Soto
                         * @date	25/02/2014
                         */
                        Smartmatic::GUI::Widgets::TableBoxSelectionWidget * getTableWidget();

                    protected:

                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * standInLabel; ///< label stand in
                        Smartmatic::GUI::Widgets::BoxSelectionWidget * standInEventBox; ///< box stand in
                };
            }
        }
    }
}

#endif /* FillTableWidget_HXX_ */
