/**
 * @file	InfoMessageWidget.hxx
 * @brief	information message widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef INFOMESSAGEWIDGET_HXX_
#define INFOMESSAGEWIDGET_HXX_

#include <gtkmm.h>
#include <Widgets/smartmatic-button.h>
#include <Widgets/SmartmaticBasicFrame.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	InfoMessageWidget
                 * @brief	Information message widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class InfoMessageWidget: public Smartmatic::GUI::Widgets::SmartmaticBasicFrame
                {
                    public:
                        /**
                         * @fn	InfoMessageWidget::InfoMessageWidget();
                         * @brief	Default constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        InfoMessageWidget();
                        /**
                         * @fn	virtual InfoMessageWidget::~InfoMessageWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~InfoMessageWidget();

                        /**
                         * @fn	Glib::SignalProxy0< void > InfoMessageWidget::getSignalButtonOkClicked();
                         * @brief	Gets signal button ok clicked.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	The signal button ok clicked.
                         */
                        Glib::SignalProxy0< void > getSignalButtonOkClicked();
                        /**
                         * @fn	Glib::SignalProxy0< void > InfoMessageWidget::getSignalButtonBackClicked();
                         * @brief	Gets signal button back clicked.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	The signal button back clicked.
                         */
                        Glib::SignalProxy0< void > getSignalButtonBackClicked();
                        /**
                         * @fn	Glib::SignalProxy0< void > InfoMessageWidget::getSignalButtonPrintClicked();
                         * @brief	Gets signal button print clicked.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	The signal button print clicked.
                         */
                        Glib::SignalProxy0< void > getSignalButtonPrintClicked();

                        /**
                         * @fn	void InfoMessageWidget::setInfoWidget ( Gtk::Widget * info );
                         * @brief	Sets information widget.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	info	If non-null, the information.
                         */
                        void setInfoWidget ( Gtk::Widget * info );
                        /**
                         * @fn	void InfoMessageWidget::enableButtonBack ( bool enable );
                         * @brief	Enables the button back.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	enable	true to enable, false to disable.
                         */
                        void enableButtonBack ( bool enable );
                        /**
                         * @fn	void InfoMessageWidget::enableButtonOk ( bool enable );
                         * @brief	Enables the button ok.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	enable	true to enable, false to disable.
                         */
                        void enableButtonOk ( bool enable );
                        /**
                         * @fn	void InfoMessageWidget::enableButtonPrint ( bool enable );
                         * @brief	Enables the button print.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	enable	true to enable, false to disable.
                         */
                        void enableButtonPrint ( bool enable );

                        /**
                         * @fn	void InfoMessageWidget::hideButtonBack();
                         * @brief	Hides the button back.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void hideButtonBack();
                        /**
                         * @fn	void InfoMessageWidget::showButtonBack();
                         * @brief	Shows the button back.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void showButtonBack();

                        /**
                         * @fn	void InfoMessageWidget::hideButtonOk();
                         * @brief	Hides the button ok.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void hideButtonOk();
                        /**
                         * @fn	void InfoMessageWidget::showButtonOk();
                         * @brief	Shows the button ok.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void showButtonOk();

                        /**
                         * @fn	void InfoMessageWidget::hideButtonPrint();
                         * @brief	Hides the button print.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void hideButtonPrint();
                        /**
                         * @fn	void InfoMessageWidget::showButtonPrint();
                         * @brief	Shows the button print.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void showButtonPrint();

                        /**
                         * @fn	void InfoMessageWidget::hideText();
                         * @brief	Hides the text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void hideText();
                        /**
                         * @fn	void InfoMessageWidget::showText();
                         * @brief	Shows the text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void showText();

                    private:

                        Smartmatic::GUI::Widgets::SmartmaticBasicFrame * mainFrame; ///< The main frame
                        Smartmatic::GUI::Widgets::SmartmaticBasicFrame * infoFrame; ///< The information frame
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox * mainVBox; ///< The main v box
                        Gtk::VButtonBox * vbuttonboxPrint;	///< The vbuttonbox print

                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonBack;	///< The button back
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonOk;	///< The button ok
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonPrint; ///< The button print

                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger

                    protected:
                        /**
                         * @fn	void InfoMessageWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void initialize();
                        /**
                         * @fn	void InfoMessageWidget::dispose();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void dispose();
                };
            }
        }
    }
}

#endif /* INFOMESSAGEWIDGET_HXX_ */
