/**
 * @file	InitiatorWidget.hxx
 * @brief	initiator widget class.
 * @author	Yadickson Soto
 * @date	01/11/2012
 */


#ifndef INITIATORWIDGET_HXX_
#define INITIATORWIDGET_HXX_

#include <gtkmm.h>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <Gui/Widgets/ShutdownWidget.hxx>
#include <Operation/Initiator/InitializationStatus.hxx>
#include <Widgets/StepWidget.hxx>
#include <Operation/Initiator/IMachineInitiator.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	InitiatorWidget
                 * @brief	Initiator widget.
                 * @author	Yadickson Soto
                 * @date	01/11/2012
                 */
                class InitiatorWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:

                        /**
                         * @fn	InitiatorWidget::InitiatorWidget ( ProgressWidget * progress,
                         * 			Smartmatic::SAES::Operation::Initiator::IMachineInitiator * initiator );
                         * @brief	Constructor.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @param [in,out]	progress 	If non-null, the progress.
                         * @param [in,out]	initiator	The initiator process machine.
                         */
                        InitiatorWidget ( ProgressWidget * progress, Smartmatic::SAES::Operation::Initiator::IMachineInitiator * initiator );
                        /**
                         * @fn	virtual InitiatorWidget::~InitiatorWidget();
                         * @brief	Destructor.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         */
                        virtual ~InitiatorWidget();

                        /**
                         * @fn	void InitiatorWidget::start();
                         * @brief	Starts this object.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         */
                        void start();
                        /**
                         * @fn	virtual std::string InitiatorWidget::getTitleText();
                         * @brief	virtual method getter title.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @return	title.
                         */
                        virtual std::string getTitleText();

                    protected:
                        /**
                         * @fn	virtual void InitiatorWidget::initialize();
                         * @brief	Initializer method.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void initialize();
                        /**
                         * @fn	virtual void InitiatorWidget::dispose();
                         * @brief	virtual method stop.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void dispose();

                        /**
                         * @fn	void InitiatorWidget::end();
                         * @brief	Ends this object.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         */
                        void end();
                        /**
                         * @fn	void InitiatorWidget::statusReport ( Smartmatic::SAES::Operation::Initiator::InitializationStatus & status,
                         * 			bool finalized );
                         * @brief	Status report.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @param 	status	The status.
                         */
                        void statusReport ( Smartmatic::SAES::Operation::Initiator::InitiatorSignal * status );
                        /**
                         * @fn	void InitiatorWidget::showMessage( Smartmatic::SAES::Operation::Initiator::InitiatorSignal * status );
                         * @brief	Shows the message.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @param 	status	The status.
                         */
                        void showMessage ( Smartmatic::SAES::Operation::Initiator::InitiatorSignal * status );
                        /**
                         * @fn	void InitiatorWidget::showEndMessage(Smartmatic::SAES::Operation::Initiator::InitiatorSignal * status);
                         * @brief	Shows the end message.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @param 	status	The status.
                         */
                        void showEndMessage ( Smartmatic::SAES::Operation::Initiator::InitiatorSignal * status );

                    private:

                        Smartmatic::SAES::Operation::Initiator::IMachineInitiator * initiator; ///< The initiator process machine
                        ProgressWidget * progress;	///< The progress
                        ShutdownWidget shutdownWidget;
                        sigc::connection conn;	///< The connection
                        static Smartmatic::Log::ISMTTLog * logger;	///< The logger
                };
            }
        }
    }
}

#endif /* INITIATORWIDGET_HXX_ */
