/**
 * @file	LanguageInstallWidget.cxx
 * @brief	language install widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "LanguageInstallWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <Voting/set-language-functionality.h>
#include "Voting/Election/ElectionInstalationLanguages.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::GUI::Widgets;

ISMTTLog * LanguageInstallWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.LanguageInstallWidget", GETTEXT_PACKAGE );

LanguageInstallWidget::LanguageInstallWidget()
    : StepWidget()
{
    this->languageWidget = NULL;
    this->initialize();
    setWidget ( languageWidget );
    this->setAccessibleName ( "LanguageInstallWidget" );
}

LanguageInstallWidget::~LanguageInstallWidget()
{
    dispose();
}

void LanguageInstallWidget::initialize()
{
    dispose();
    languageWidget = new ChangeLanguageWidget();
    languageWidget->SetSaveFile ( true );
    languageWidget->LanguageChanged().connect ( sigc::mem_fun ( *this, &LanguageInstallWidget::end ) );
}

void LanguageInstallWidget::setSaveFile ( bool save )
{
    languageWidget->SetSaveFile ( save );
}

void LanguageInstallWidget::dispose()
{
    this->remove();

    if ( languageWidget )
    {
        delete languageWidget;
        languageWidget = NULL;
    }
}

void LanguageInstallWidget::start ( bool force )
{
    this->remove();
    std::list<Election::Language> languages;

    if ( force )
    {
        bool firstTime = !VotingCenters::VotingCentersWrapper::getInstance()->isInstall();

        if ( firstTime )
        {
            logger->Debug ( "load language election" );
            languages = MachineOperationManager::getInstance()->getLanguagesElection();
        }
        else
        {
            logger->Debug ( "load language install" );
            languages = MachineOperationManager::getInstance()->getLanguagesInstall();
            setSaveFile ( false );
        }
    }
    else
    {
        languages = Election::ElectionInstalationLanguages::getInstance()->getLanguages();
    }

    if ( !languages.empty() )
    {
        logger->Debug ( "Show selected language" );
        languageWidget->init ( MachineOperationManager::getInstance()->getSaesDirectories() );
        languageWidget->initialize ( languages, false );
        this->add ( *languageWidget );
        this->show_all_children();
    }
    else
    {
        logger->Debug ( "Language not found" );
        signalLanguageNotFound.emit();
    }
}

void LanguageInstallWidget::end()
{
    emit();
}

sigc::signal<void> LanguageInstallWidget::getSignalLanguageNotFound()
{
    return signalLanguageNotFound;
}
