/**
 * @file	ListResultWidget.hxx
 * @brief	list result widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef LISTRESULTWIDGET_HXX_
#define LISTRESULTWIDGET_HXX_

#include <gtkmm.h>
#include <list>
#include <Widgets/StepWidget.hxx>
#include <Widgets/TableCenterWidget.hxx>
#include <Widgets/smartmatic-button.h>
#include "ResultWidget.hxx"
#include <Widgets/SmartmaticBasicScrollContainer.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	ListResultWidget
                 * @brief	List result widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class ListResultWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:
                        /**
                         * @fn	ListResultWidget::ListResultWidget ( int columns, int width = 250, int height = 40 );
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	columns	The columns.
                         * @param	width	 	(optional) the width.
                         * @param	height 	(optional) the height.
                         */
                        ListResultWidget ( int columns, int width = 250, int height = 40 );
                        /**
                         * @fn	virtual ListResultWidget::~ListResultWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~ListResultWidget();
                        /**
                         * @fn	void ListResultWidget::addWidget ( Smartmatic::SAES::GUI::Widgets::ResultWidget * widget, bool enableSignal );
                         * @brief	Adds a widget.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	widget	If non-null, the widget.
                         * @param enableSignal flag enable signal
                         */
                        void addWidget ( Smartmatic::SAES::GUI::Widgets::ResultWidget * widget, bool enableSignal );
                        /**
                         * @fn	int ListResultWidget::getCountWidget();
                         * @brief	Gets count widget.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	The count widget.
                         */
                        int getCountWidget();

                    protected:

                        /**
                         * @fn	void ListResultWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void initialize();
                        /**
                         * @fn	void ListResultWidget::dispose();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void dispose();

                        /**
                         * @fn	void ListResultWidget::putWidget ( Smartmatic::SAES::GUI::Widgets::ResultWidget * widget );
                         * @brief	Puts a widget.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	widget	If non-null, the widget.
                         */
                        void putWidget ( Smartmatic::SAES::GUI::Widgets::ResultWidget * widget );
                        /**
                         * @fn	void ListResultWidget::viewInfo ( Smartmatic::SAES::GUI::Widgets::ResultWidget * result );
                         * @brief	View information.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [out]	result	if non-null, the result.
                         */
                        void viewInfo ( Smartmatic::SAES::GUI::Widgets::ResultWidget * result );

                        /**
                         * @fn	void ListResultWidget::initInfo();
                         * @brief	Initialises the information.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void initInfo();
                        /**
                         * @fn	void ListResultWidget::clearList();
                         * @brief	Clears the list.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void clearList();

                    private:

                        int columns;	///< The columns
                        int width;	///< The width
                        int height; ///< The height

                        int i;	///< Zero-based index of the
                        int j;	///< The int to process

                        Smartmatic::GUI::Widgets::SmartmaticBasicScrollContainer * scrollFrame;	///< The scroll frame
                        Gtk::EventBox * centerFrame;	///< The center frame

                        Smartmatic::GUI::Widgets::TableCenterWidget * tableWidget;	///< The table widget

                        Gtk::Label * center;	///< The center
                        Smartmatic::GUI::Widgets::TableCenterWidget * resultWidget; ///< The result widget
                        Smartmatic::GUI::Widgets::SmartmaticButton * button;	///< The button

                        /**
                         * @property	std::vector <Smartmatic::SAES::GUI::Widgets::ResultWidget *> resultWidgets
                         * @brief	Gets the result widgets.
                         * @return	The result widgets.
                         */
                        std::vector <Smartmatic::SAES::GUI::Widgets::ResultWidget *> resultWidgets;
                        /**
                         * @property	std::list <Smartmatic::SAES::GUI::Widgets::ResultWidget *> resultList
                         * @brief	Gets a list of results.
                         * @return	A List of results.
                         */
                        std::list <Smartmatic::SAES::GUI::Widgets::ResultWidget *> resultList;
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                };
            }
        }
    }
}

#endif /* LISTRESULTWIDGET_HXX_ */
