/**
 * @file	LogPagingTreeViewWidget.h
 * @brief	Declares the log paging tree view widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _LOGPAGINGTREEVIEWWIDGET_H_
#define _LOGPAGINGTREEVIEWWIDGET_H_

#include <Widgets/StepWidget.hxx>
#include <Gui/Widgets/LogTreeViewWidget.h>
#include <Log/LogReader.h>
#include "Widgets/smartmatic-button.h"

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	LogPagingTreeViewWidget
                 * @brief	Log paging tree view widget.
                 * @author	Cesar.bonilla
                 * @date	4/29/2011
                 */

                class LogPagingTreeViewWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:

                        /**
                         * @fn	LogPagingTreeViewWidget::LogPagingTreeViewWidget ( Smartmatic::SAES::Log::LogReader * logReader, bool enablePrint );
                         * @brief	Constructor.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	logReader	If non-null, the log reader.
                         * @param	enablePrint Enable print button
                         */

                        LogPagingTreeViewWidget ( Smartmatic::SAES::Log::LogReader * logReader, bool enablePrint );

                        /**
                         * @fn	virtual LogPagingTreeViewWidget::~LogPagingTreeViewWidget();
                         * @brief	Finaliser.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        virtual ~LogPagingTreeViewWidget();

                        /**
                         * @fn	void LogPagingTreeViewWidget::SetPrintButtonSensitive ( bool sensitive );
                         * @brief	Sets print button sensitive.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	sensitive	true to sensitive.
                         */
                        void SetPrintButtonSensitive ( bool sensitive );

                        sigc::signal<void> signalPrint_detected;   ///< The signal print detected

                    private:

                        /**
                         * @fn	void LogPagingTreeViewWidget::LoadPage ( unsigned int page );
                         * @brief	Loads a page.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	page	The page.
                         */

                        void LoadPage ( unsigned int page );

                        /**
                         * @fn	void LogPagingTreeViewWidget::LoadGUI ();
                         * @brief	Loads the graphical user interface.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void LoadGUI ();

                        /**
                         * @fn	void LogPagingTreeViewWidget::on_buttonBack_clicked();
                         * @brief	Handles button back clicked signals.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void on_buttonBack_clicked();

                        /**
                         * @fn	void LogPagingTreeViewWidget::on_buttonNext_clicked();
                         * @brief	Handles button next clicked signals.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void on_buttonNext_clicked();

                        /**
                         * @fn	void LogPagingTreeViewWidget::on_buttonPrint_clicked();
                         * @brief	Handles button print clicked signals.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void on_buttonPrint_clicked();

                        bool enablePrint; ///< Enable print button

                        Gtk::Table * m_VTable;  ///< The v table
                        Gtk::VBox * vBoxRight;	///< The box right
                        Gtk::ButtonBox	* m_ButtonBoxNav;	///< The button box nav
                        Gtk::Label	* m_LabelPage;  ///< The label page
                        Gtk::EventBox * eventboxLog; ///< eventbox
                        LogTreeViewWidget * logTreeViewWidget;  ///< The log tree view widget

                        Smartmatic::SAES::Log::LogReader * logReader;	///< The log reader
                        Smartmatic::SAES::Log::LogPage logPage; ///< The log page
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonBack;  ///< The back button
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonNext;  ///< The next button
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonPrint; ///< The button print
                };
            }
        }
    }
}

#endif // _LOGPAGINGTREEVIEWWIDGET_H_
