/**
 * @file	LoginWidget.hxx
 * @brief	login widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef LOGINWIDGET_HXX_
#define LOGINWIDGET_HXX_

#include <gtkmm.h>
#include <Widgets/key-board-widget.h>
#include <Widgets/smartmatic-entry.h>
#include <Widgets/smartmatic-entry-divided.h>
#include <Widgets/smartmatic-button.h>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicFrame.hxx>
#include <Widgets/SmartmaticBasicTable.hxx>
#include <Widgets/SmartmaticBasicImage.hxx>
#include <Configuration/SaesGUIConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	LoginWidget
                 * @brief	Login widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class LoginWidget: public Smartmatic::GUI::Widgets::SmartmaticBasicFrame
                {
                    public:
                        /**
                         * @fn	LoginWidget::LoginWidget ( bool onlyIcons,
                         * 			Smartmatic::SAES::Configuration::EntryDividedWidgetConfiguration & config );
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	onlyIcons					true to only icons.
                         * @param [in,out]	config	The configuration.
                         */
                        LoginWidget ( bool onlyIcons, Smartmatic::SAES::Configuration::EntryDividedWidgetConfiguration & config );
                        /**
                         * @fn	virtual LoginWidget::~LoginWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~LoginWidget();

                        /**
                         * @fn	Glib::SignalProxy0< void > LoginWidget::getSignalButtonOkClicked();
                         * @brief	Gets signal button ok clicked.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	The signal button ok clicked.
                         */
                        Glib::SignalProxy0< void > getSignalButtonOkClicked();

                        /**
                         * @fn	void LoginWidget::enabledLogin ( bool enabled );
                         * @brief	Enabled login.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	enabled	true to enable, false to disable.
                         */
                        void enabledLogin ( bool enabled );
                        /**
                         * @fn	void LoginWidget::getLoginText ( std::string & text );
                         * @brief	Gets login text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	text	The text.
                         */
                        void getLoginText ( std::string & text );
                        /**
                         * @fn	void LoginWidget::setLoginText ( std::string & text );
                         * @brief	Sets login text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	text	The text.
                         */
                        void setLoginText ( std::string & text );
                        /**
                         * @fn	void LoginWidget::getPasswordText ( std::string & text );
                         * @brief	Gets password text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	text	The text.
                         */
                        void getPasswordText ( std::string & text );
                        /**
                         * @fn	void LoginWidget::clearLogin();
                         * @brief	Clears the login.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void clearLogin();
                        /**
                         * @fn	void LoginWidget::clearPassword();
                         * @brief	Clears the password.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void clearPassword();
                        /**
                         * @fn	void LoginWidget::focusPassword();
                         * @brief	set focus to the password.
                         * @author	Juan Valera
                         * @date	23/02/2017
                         */
                        void focusPassword();
                        /**
                         * @fn	void LoginWidget::focusLogin();
                         * @brief	set focus to the login.
                         * @author	Juan Valera
                         * @date	23/02/2017
                         */
                        void focusLogin();

                    private:

                        /**
                         * @brief  Activate the login button.
                         * @author manuel.arguelles
                         * @date   Jul  6, 2017
                         */
                        void pressLoginButton();

                    private:

                        Smartmatic::GUI::Widgets::SmartmaticBasicFrame * mainFrame; ///< The main frame
                        Smartmatic::GUI::Widgets::SmartmaticBasicTable * tableLogin;	///< The table login
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * labelTitle;	///< The label title
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * labelLogin;	///< The label login
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * labelPassword; ///< The label password
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * imageLogin;	///< The image login
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * imagePassword; ///< The image password

                        Smartmatic::GUI::Widgets::SmartmaticEntry * login;	///< The login
                        Smartmatic::GUI::Widgets::SmartmaticEntryDivided * password;	///< The password
                        Smartmatic::GUI::Widgets::SmartmaticButton * buttonOk;	///< The button ok
                        Smartmatic::GUI::Widgets::SmartmaticBasicTable * tableKeyboard;    ///< The table keyboard
                        Smartmatic::GUI::Widgets::Keyboard * keyboard; ///< Keyboard

                        bool onlyIcons; ///< true to only icons
                        int entryDivisions; ///< The entry divisions
                        int entryMaxChar; ///< The entry maximum character
                        int entryWidth; ///< Width of the entry
                        int entryHeight; ///< Height of the entry
                        bool entryVisibility; ///< true to entry visibility
                        static Smartmatic::Log::ISMTTLog * logger;	///< The logger

                    protected:
                        /**
                         * @fn  void onKeyPress ( std::string data );
                         * @brief   Method on key press.
                         * @author  Yadickson Soto
                         * @date    12/09/2013
                         * @param   data    key data.
                         */
                        void onKeyPress ( std::string data );
                        /**
                        * @fn  void onKeyEnterPress ( std::string data );
                        * @brief   Method on key enter press.
                        * @author  Eduardo Fernández
                        * @date    22/09/2017
                        */
                       void onKeyEnterPress ();
                        /**
                         * @fn  void onKeyDeletePress();
                         * @brief   Method on key delete press.
                         * @author  Yadickson Soto
                         * @date    12/09/2013
                         */
                        void onKeyDeletePress();
                        /**
                         * @fn  bool on_login_focus_in(GdkEventFocus * event);
                         * @brief   Method manager getter focus entry.
                         * @author  Yadickson Soto
                         * @date    27/03/2012
                         * @param   event   event cursor entry pos.
                         * @return  result event (true)
                         */
                        bool on_login_focus_in ( GdkEventFocus * event );
                        /**
                         * @fn	void LoginWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void initialize();
                        /**
                         * @fn  void LoginWidget::setKeyboard();
                         * @brief   Sets the keyboard.
                         * @author  eduardo.fernandez
                         * @date    22/09/2017
                         */
                        void setKeyboard();
                        /**
                         * @fn	void LoginWidget::dispose();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void dispose();
                };
            }
        }
    }
}

#endif /* LOGINWIDGET_HXX_ */
