/**
 * @file	MenuWidget.cxx
 * @brief	menu widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "MenuWidget.hxx"
#include <Functionalities/change-language-functionality.h>
#include <vector>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;

bool MenuWidget::reloadMenu = true;

MenuWidget::MenuWidget()
    : StepWidget()
{
    menu = NULL;
    initialize();
    refresh();
    this->show();
}

MenuWidget::~MenuWidget()
{
    dispose();
}

void MenuWidget::refresh()
{
    if ( reloadMenu )
    {
        reloadMenu = false;
        dispose();

        menu = new MenuWidgetmm ();
        setWidget ( menu );
    }

    show_all_children();
}

void MenuWidget::initialize()
{
}

void MenuWidget::dispose()
{
    this->remove();

    if ( menu )
    {
        delete menu;
        menu = NULL;
    }
}

void MenuWidget::SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value )
{
    menu->SetKey ( value );
}

std::string MenuWidget::getTitleText()
{
    return N_ ( "Smartmatic.SAES.GUI.Windows.MenuWindow.Title" );
}
