/**
 * @file MenuWidget.hxx
 * @brief Header menu widget
 * @date 29/03/2012
 * @author Yadickson Soto
 */

#ifndef MENUWIDGET_HXX_
#define MENUWIDGET_HXX_

#include <gtkmm.h>
#include <Widgets/StepWidget.hxx>
#include <Gui/Widgets/Menu/menu-widgetmm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	MenuWidget
                 * @brief	Menu widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class MenuWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:

                        /**
                         * @fn	MenuWidget::MenuWidget();
                         * @brief	Default constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        MenuWidget();
                        /**
                         * @fn	virtual MenuWidget::~MenuWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~MenuWidget();

                        /**
                         * @fn	virtual std::string MenuWidget::getTitleText();
                         * @brief	virtual method getter title.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @return	title.
                         */
                        virtual std::string getTitleText();
                        /**
                         * @fn	virtual void MenuWidget::initialize();
                         * @brief	Initializer method.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void initialize();
                        /**
                         * @fn	virtual void MenuWidget::dispose();
                         * @brief	virtual method stop.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void dispose();
                        /**
                         * @fn	virtual void MenuWidget::refresh();
                         * @brief	refresh widget method.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         */
                        virtual void refresh();

                        /**
                         * @fn	virtual void MenuWidget::SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value );
                         * @brief	Sets a key.
                         * @author	Yadickson Soto
                         * @date	08/06/2012
                         * @param	value	The value.
                         */
                        virtual void SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value );

                        static bool reloadMenu; ///< true to reload menu

                    private:

                        Smartmatic::SAES::GUI::Widgets::MenuWidgetmm * menu;  ///< The menu
                };
            }
        }
    }
}

#endif /* MENUWIDGET_HXX_ */
