/**
 * @file	OpenElectionStatusWidget.cxx
 * @brief	open election status widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "OpenElectionStatusWidget.hxx"
#include <Operation/MachineOperationManager.hxx>
#include <System/IO/File.h>
#include "Environment/Configurations/resource-path-provider.h"
#include <Log/ISMTTLog.h>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/Exception/XmlException.hxx>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Exception;

using namespace Smartmatic::Log;

Smartmatic::Log::ISMTTLog * OpenElectionStatusWidget::logger = Smartmatic::Log::SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.OpenElectionStatusWidget", GETTEXT_PACKAGE );

OpenElectionStatusWidget::OpenElectionStatusWidget ( ProgressWidget * progress )
    : WizardWidgetBase(), progress ( progress )
{

}

OpenElectionStatusWidget::~OpenElectionStatusWidget()
{
    this->remove();

    if ( progress )
    {
        delete progress;
        progress = NULL;
    }
}

void OpenElectionStatusWidget::initialize()
{
    m_disable_close_signal.emit();
    bool errorHappen = false;

    try
    {
        MachineOperationManager::getInstance()->openElection();
    }
    catch ( CryptoException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.OpenElectionFunctionality.ErrorOpeningElection" ) );
        logger->Debug ( "OpenElectionStatusWidget CryptoException" );
        errorHappen = true;
    }
    catch ( SignedException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.OpenElectionFunctionality.ErrorOpeningElection" ) );
        logger->Debug ( "OpenElectionStatusWidget SignedException" );
        errorHappen = true;
    }
    catch ( XmlException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.OpenElectionFunctionality.ErrorOpeningElection" ) );
        logger->Debug ( "OpenElectionStatusWidget XmlException" );
        errorHappen = true;
    }
    catch ( ... )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.OpenElectionFunctionality.ErrorOpeningElection" ) );
        logger->Debug ( "OpenElectionStatusWidget XmlException" );
        errorHappen = true;
    }

    if ( !errorHappen )
    {
        end();
    }
    else
    {
        updateLabel ( N_ ( "Smartmatic.SAES.GUI.Widgets.OpenElectionStatusWidget.ErrorMessage" ) );
    }
}

void OpenElectionStatusWidget::dispose()
{
}

void OpenElectionStatusWidget::end()
{
    emit();
}

void OpenElectionStatusWidget::updateLabel ( std::string text )
{
    progress->hideInfoTitle();
    progress->hideImage();
    progress->setInfoText ( text );
    m_stop_signal.emit();
}

bool OpenElectionStatusWidget::isEnabledButtonExitWizard()
{
    return false;
}
