/**
 * @file	PMInstallMachineWidget.hxx
 * @brief	pm install machine widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#ifndef PMINSTALLMACHINEWIDGET_HXX_
#define PMINSTALLMACHINEWIDGET_HXX_

#include <Gui/Widgets/InstallMachineWidget.hxx>
#include <Gui/Widgets/LoginWidget.hxx>
#include <Gui/Widgets/InfoMessageWidget.hxx>
#include <Gui/Widgets/PollingPlaceInfoWidget.hxx>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <Gui/Widgets/LanguageInstallWidget.hxx>
#include <Gui/Widgets/ReportInstallWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	PMInstallMachineWidget
                 * @brief	Pm install machine widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class PMInstallMachineWidget : public InstallMachineWidget
                {

                    public:
                        /**
                         * @fn	PMInstallMachineWidget::PMInstallMachineWidget ( ProgressWidget * progressWidget,
                         * 			PollingPlaceInfoWidget * pollingPlaceInfoWidget );
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	progressWidget					If non-null, the progress widget.
                         * @param [in,out]	pollingPlaceInfoWidget	If non-null, the polling place information widget.
                         */
                        PMInstallMachineWidget ( ProgressWidget * progressWidget,
                                                 PollingPlaceInfoWidget * pollingPlaceInfoWidget );

                        /**
                         * @fn	virtual PMInstallMachineWidget::~PMInstallMachineWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~PMInstallMachineWidget();
                        /**
                         * @fn	virtual void PMInstallMachineWidget::stop();
                         * @brief	Stops this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual void stop();
                        /**
                         * @fn	virtual bool PMInstallMachineWidget::isTitle();
                         * @brief	virtual method getter is visible title frame.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @return	true is title frame visible.
                         */
                        virtual bool isTitle();
                        /**
                         * @fn	virtual std::string PMInstallMachineWidget::getTitleText();
                         * @brief	virtual method getter title.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @return	title.
                         */
                        virtual std::string getTitleText();

                    protected:
                        /**
                           * @fn	void PMInstallMachineWidget::authentication();
                           * @brief	Authentications this object.
                           * @author	Youssef.tourkmani
                           * @date	01/11/2012
                           */
                        void authentication();
                        /**
                         * @fn	void PMInstallMachineWidget::error();
                         * @brief	Errors this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void error();
                        /**
                         * @fn	void PMInstallMachineWidget::language();
                         * @brief	Languages this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void language();
                        /**
                         * @fn	void PMInstallMachineWidget::info();
                         * @brief	Infoes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void info();
                        /**
                         * @fn	void PMInstallMachineWidget::print();
                         * @brief	Prints this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void print();
                        /**
                         * @fn	void PMInstallMachineWidget::install();
                         * @brief	Installs this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void install();
                        /**
                         * @fn	virtual void PMInstallMachineWidget::initialize();
                         * @brief	Initializer method.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void initialize();
                        /**
                         * @fn	virtual void PMInstallMachineWidget::dispose();
                         * @brief	virtual method stop.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void dispose();

                        /**
                         * @fn	Gtk::Widget * PMInstallMachineWidget::getMessageLoginError();
                         * @brief	Gets message login error.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	null if it fails, else the message login error.
                         */
                        Gtk::Widget * getMessageLoginError();
                        /**
                         * @fn	Gtk::Widget * PMInstallMachineWidget::getMessageLoginOk();
                         * @brief	Gets message login ok.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	null if it fails, else the message login ok.
                         */
                        Gtk::Widget * getMessageLoginOk();

                        /**
                         * @fn	void PMInstallMachineWidget::login();
                         * @brief	Login.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void login();

                        /**
                         * @fn	void PMInstallMachineWidget::onLogin();
                         * @brief	Login.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        void onLogin();
                        /**
                          * @fn	void PMInstallMachineWidget::showProgressLogin();
                          * @brief	Shows the progress login.
                          * @author	Youssef.tourkmani
                          * @date	01/11/2012
                          */
                        void showProgressLogin();
                        /**
                         * @fn	void PMInstallMachineWidget::checkData();
                         * @brief	Check data.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void checkData();
                        /**
                         * @fn	void PMInstallMachineWidget::end();
                         * @brief	Ends this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void end();

                    private:

                        bool firstTime; ///< true to first time
                        bool showImages;	///< true to show, false to hide the images
                        bool forceText; ///< true to force text
                        bool loginOk; ///< true if the login operation was a success, false if it failed
                        bool isPresent; ///< true if this object is present

                        std::string center; ///< The center
                        std::string password; ///< The password
                        bool centerCaseSensitive; ///< true to center case sensitive

                        Gtk::Label * message; ///< The message
                        Gtk::Image * image; ///< The image

                        Glib::Dispatcher signalLoginDispatcher; ///< signal login dispatcher
                        Glib::Thread * m_thread; ///< The thread pointer

                        Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget;	///< The progress widget
                        Smartmatic::SAES::GUI::Widgets::PollingPlaceInfoWidget * pollingPlaceInfoWidget;	///< The polling place information widget
                        Smartmatic::SAES::GUI::Widgets::LoginWidget * loginWidget;	///< The login widget
                        Smartmatic::SAES::GUI::Widgets::InfoMessageWidget * infoMessage;	///< Message describing the information
                        Smartmatic::SAES::GUI::Widgets::LanguageInstallWidget * languageWidget; ///< The language widget
                        Smartmatic::SAES::GUI::Widgets::ReportInstallWidget * reportInstall;	///< The report install

                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };
            }
        }
    }
}

#endif /* PMINSTALLMACHINEWIDGET_HXX_ */
