/**
 * @file PartyButtonWidget.cxx
 * @brief Body class party button widget
 * @date 12/03/2012
 * @author Yadickson Soto
 */

#include "PartyButtonWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Operation/Voting/VotingOperation.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::GUI::Operation::Voting;

PartyButtonWidget::PartyButtonWidget ( bool expand )
    : ButtonsWidget ( expand )
{
    int size;
    flagNull = false;

    if ( expand )
    {
        int spacing = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getExpandButton().getSpacing();
        size = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getExpandButton().getFontSize();
        setSpacing ( spacing );
    }
    else
    {
        int backWidth = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getSeparateButton().getBackButtonWidth();
        int nullWidth = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getSeparateButton().getNullButtonWidth();
        int acceptWidth = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getSeparateButton().getAcceptButtonWidth();
        int clearWidth = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getSeparateButton().getClearButtonWidth();
        size = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getSeparateButton().getFontSize();
        backButton->set_size_request ( backWidth, -1 );
        nullButton->set_size_request ( nullWidth, -1 );
        acceptButton->set_size_request ( acceptWidth, -1 );
        clearButton->set_size_request ( clearWidth, -1 );
    }

    std::string soundPartyS = SAESGUIConfigurationManager::getInstance()->getPartySelectSound();

    backButton->WidgetsPosition ( 5, 0, 0 );
    backButton->SetSound ( soundPartyS );
    nullButton->WidgetsPosition ( 5, 0, 0 );
    nullButton->SetSound ( soundPartyS );
    acceptButton->WidgetsPosition ( 5, 0, 0 );
    acceptButton->SetSound ( soundPartyS );
    clearButton->WidgetsPosition ( 5, 0, 0 );
    clearButton->SetSound ( soundPartyS );

    modifyFontSize ( size );

    setAccessibleName ( "PartyButtonsContainer" );
    acceptButton->setAccessibleName ( "ConfirmPartySelected" );
    clearButton->setAccessibleName ( "CancelPartySelected" );
    backButton->setAccessibleName ( "BackPartySelected" );
    nullButton->setAccessibleName ( "NullPartySelected" );
}

PartyButtonWidget::~PartyButtonWidget()
{
}

void PartyButtonWidget::update()
{
    if ( flagNull )
    {
        acceptButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.AcceptNullSelection" ) );
        clearButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.ClearNullSelection" ) );
    }
    else
    {
        acceptButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.AcceptSelection" ) );
        clearButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.ClearSelection" ) );
    }

    backButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.Back" ) );
    nullButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.NullSelection" ) );

    backButton->SetCustomName ( VotingOperation::getButtonName() );
    nullButton->SetCustomName ( VotingOperation::getButtonName() );
    acceptButton->SetCustomName ( VotingOperation::getButtonName() );
    clearButton->SetCustomName ( VotingOperation::getButtonName() );

    clearSignals();
}

void PartyButtonWidget::showNormalInfo()
{
    flagNull = false;
    acceptButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.AcceptSelection" ) );
    clearButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.ClearSelection" ) );
}

void PartyButtonWidget::showNullInfo()
{
    flagNull = true;
    acceptButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.AcceptNullSelection" ) );
    clearButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.ClearNullSelection" ) );
}
