/**
 * @file ProgressWidget.hxx
 * @brief Header class progress widget
 * @date 18/10/2011
 * @author Yadickson Soto
 */

#ifndef PROGRESSWIDGET_HXX_
#define PROGRESSWIDGET_HXX_

#include <Widgets/StepWidget.hxx>
#include <Configuration/label-configuration.h>
#include <Operation/ImageResourcePath.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicImage.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	ProgressWidget
                 * @brief	Class progress widget.
                 * @author	Yadickson Soto
                 * @date	18/10/2011
                 */
                class ProgressWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:

                        /**
                         * @fn	ProgressWidget::ProgressWidget();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        ProgressWidget();
                        /**
                         * @fn	virtual ProgressWidget::~ProgressWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        virtual ~ProgressWidget();
                        /**
                         * @fn	void ProgressWidget::dispose();
                         * @brief	Method dispose widget.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        virtual void dispose();
                        /**
                         * @fn	virtual void ProgressWidget::setImage ( std::string image );
                         * @brief	Virtual method setter image relative path to project.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         * @param	image	image relative path to project.
                         */
                        virtual void setImage ( std::string image );
                        /**
                         * @fn	virtual void ProgressWidget::setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::Resource image );
                         * @brief	Virtual method setter image type.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         * @param	image	image type.
                         */
                        virtual void setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::Resource image );
                        /**
                         * @fn	virtual void ProgressWidget::setImage ( Glib::RefPtr<Gdk::Pixbuf> pixbuff );
                         * @brief	Virtual method setter image pixbuff.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         * @param	pixbuff	The pixbuff.
                         */
                        virtual void setImage ( Glib::RefPtr<Gdk::Pixbuf> pixbuff );
                        /**
                         * @fn	void ProgressWidget::setInfoTitle ( std::string text );
                         * @brief	Method setter title text.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         * @param	text	new title text.
                         */
                        virtual void setInfoTitle ( std::string text );
                        /**
                         * @fn	void ProgressWidget::setInfoText ( std::string text );
                         * @brief	Method setter bottom text.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         * @param	text	new bottom text.
                         */
                        virtual void setInfoText ( std::string text );
                        /**
                         * @fn	void ProgressWidget::setInfoFooter ( std::string text );
                         * @brief	Method setter footer text.
                         * @author	Yadickson Soto
                         * @date	03/05/2012
                         * @param	text	new footer text.
                         */
                        virtual void setInfoFooter ( std::string text );
                        /**
                         * @fn	void ProgressWidget::setInfoTitleConfig ( Smartmatic::GUI::Configuration::LabelConfiguration labelConf );
                         * @brief	Method setter title text config.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         * @param	labelConf	label configuration.
                         */
                        void setInfoTitleConfig ( Smartmatic::GUI::Configuration::LabelConfiguration labelConf );
                        /**
                         * @fn	void ProgressWidget::setInfoTextConfig ( Smartmatic::GUI::Configuration::LabelConfiguration labelConf );
                         * @brief	Method setter bottom text config.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         * @param	labelConf	label configuration.
                         */
                        void setInfoTextConfig ( Smartmatic::GUI::Configuration::LabelConfiguration labelConf );
                        /**
                         * @fn	void ProgressWidget::setInfoFooterConfig ( Smartmatic::GUI::Configuration::LabelConfiguration labelConf );
                         * @brief	Method setter footer text config.
                         * @author	Yadickson Soto
                         * @date	03/05/2012
                         * @param	labelConf	label configuration.
                         */
                        void setInfoFooterConfig ( Smartmatic::GUI::Configuration::LabelConfiguration labelConf );
                        /**
                         * @fn	void ProgressWidget::hideInfoTitle();
                         * @brief	Method hide title.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        void hideInfoTitle();
                        /**
                         * @fn	void ProgressWidget::showInfoTitle( bool flag = true );
                         * @brief	Method show title.
                         * @param flag true for visible
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        void showInfoTitle ( bool flag = true );
                        /**
                         * @fn	void ProgressWidget::hideImage();
                         * @brief	Method hide image.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        void hideImage();
                        /**
                         * @fn	void ProgressWidget::showImage( bool flag = true );
                         * @brief	Method show image.
                         * @param flag true for visible
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        void showImage ( bool flag = true );
                        /**
                         * @fn	void ProgressWidget::hideInfoText();
                         * @brief	Method hide bottom text.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        void hideInfoText();
                        /**
                         * @fn	void ProgressWidget::showInfoText( bool flag = true );
                         * @brief	Method show bottom text.
                         * @param flag true for visible
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                        void showInfoText ( bool flag = true );
                        /**
                         * @fn	void ProgressWidget::hideInfoFooter();
                         * @brief	Method hide footer text.
                         * @author	Yadickson Soto
                         * @date	03/05/2012
                         */
                        void hideInfoFooter();
                        /**
                         * @fn	void ProgressWidget::showInfoFooter( bool flag = true );
                         * @brief	Method show footer text.
                         * @param flag true for visible
                         * @author	Yadickson Soto
                         * @date	03/05/2012
                         */
                        void showInfoFooter ( bool flag = true );

                        /**
                         * @fn	Smartmatic::GUI::Widgets::SmartmaticBasicLabel * getLabelTitle();
                         * @brief	Method getter label title widget
                         * @author	Yadickson Soto
                         * @date	03/05/2012
                         * @return label title widget
                         */
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * getLabelTitle();

                        /**
                         * @fn	Smartmatic::GUI::Widgets::SmartmaticBasicLabel * getLabelInfo();
                         * @brief	Method getter label info widget
                         * @author	Yadickson Soto
                         * @date	03/05/2012
                         * @return label info widget
                         */
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * getLabelInfo();

                        /**
                         * @fn	Smartmatic::GUI::Widgets::SmartmaticBasicLabel * getLabelFooter();
                         * @brief	Method getter label footer widget
                         * @author	Yadickson Soto
                         * @date	03/05/2012
                         * @return label footer widget
                         */
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * getLabelFooter();

                    protected:
                        /**
                         * @fn	virtual void ProgressWidget::initialize();
                         * @brief	Method initialize widget.
                         * @author	Yadickson Soto
                         * @date	18/10/2011
                         */
                         virtual void initialize();

                    private:



                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * mainFrame; ///< principal frame
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox * mainVBox; ///< principal container
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * progressImage; ///< center image
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * progressTitle; ///< top text
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * progressLabel; ///< bottom text
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * progressFooter; ///< footer text
                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                };
            }
        }
    }
}

#endif /* PROGRESSWIDGET_HXX_ */
