#ifndef	SMARTMATIC_SAES_GUI_WIDGETS_RECOUNT_RECOUNTSTATPAGE
#define	SMARTMATIC_SAES_GUI_WIDGETS_RECOUNT_RECOUNTSTATPAGE

#include "ClickAbsorber.h"
#include "RecountSession.h"
#include <Widgets/SmartmaticBasicTable.hxx>
#include <Log/SMTTLogManager.h>


namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Recount
                {

                    class RecountStatPage : public Smartmatic::GUI::Widgets::SmartmaticBasicTable, public ClickAbsorber
                    {
                        private:

                            std::list<class RecountSession *> m_recountSessions; ///< sessions container
                            class RecountSession * m_recountTotals; ///< the last session - "totals"

                            std::list<std::string> m_labelsList; ///< list of labels fetched from ECM

                            Smartmatic::Log::ISMTTLog * logger; ///< the logger

                            /**
                             * If session invalidated (needs redraw), visualizes relevant components
                             */
                            void Draw();

                            /**
                             * Notifies a session that a voice of certain type is received.
                             * If needed (when first vote for a session), creates new session.
                             */
                            void IncVote ( const int sessNum, ///< session number index, 1-based
                                           const std::string & sessName, ///< session id
                                           bool onThisStick, ///< currently unused, may be in the future
                                           const int index  ///< in-session field index, 0-based
                                         );

                            /**
                             * @brief  Adjust the size of the columns in the table.
                             * @author manuel.arguelles
                             * @date   May 16, 2018
                             */
                            void adjustTableColumnSize();

                        public:
                            /**
                             * Constructor
                             */
                            explicit RecountStatPage ( const std::list<std::string> & voterTypes ///< list of voter types
                                            );

                            /**
                             * Destructor
                             */
                            virtual ~RecountStatPage();

                            /**
                             * On button, session total or last total is pressed
                             */
                            void OnStatusChanged ( int x, ///< column
                                                   int y, ///< row
                                                   ClickAbsorber::STATE state ///< source object's state
                                                 );

                            /**
                             * Recalculates all the page sessions and totals, sets state
                             */
                            void Refresh();

                            /**
                             * Returns the labels
                             *
                             * @return list of strings - label headers
                             */
                            std::list<std::string> & GetLabelsList();

                            /**
                             * Attach element to the table
                             * Specialized wrapper around Gtk::Table::attach function
                             */
                            void Attach ( Widget & child, ///< the element to attach
                                          guint left_attach, ///< left cell to attach to
                                          guint right_attach, ///< right cell to attach to
                                          guint top_attach, ///< top cell to attach to
                                          guint bottom_attach ///< bottom cell to attach to
                                        );

                            /**
                             * Called from Functionality with elements (votes) to insert into the table for visualization
                             * Both parameters must match
                             */
                            void Update ( const std::list< std::pair<int, int> > & updates, ///< list of coordinates <el_num, sess_num>
                                          const std::list<std::string> & updateSessionNames ///< list of strings with session keys
                                        );

                            /**
                             * Returns queue with elements for erasure
                             * collects the elements from all the nested sessions
                             */
                            void GetForPersistence ( std::list< std::pair<int, int> > & deleteQueue ///< container list, reference
                                                   );

                            /**
                             * Returns number of votes remaining, not selected for erasure
                             * Forwards the call to the Totals session
                             *
                             * @return number of votes
                             */
                            int GetTotal();

                            /**
                             * Returns number of All votes counted
                             * Forwards the call to the Totals session
                             *
                             * @return number of votes
                             */
                            int GetMaxMax();

                        public:
                            static const size_t numFields; ///< so far const 15

                            static const size_t MaxVoterTypes; ///< so far const 6
                            static const size_t startOfFoundVotersIndex; ///< index of first voter type
                            static const size_t subtotalFoundIndex;	///< index of first subtotal
                            static const size_t startOfNotFoundVotersIndex; ///< index of first voter type for the "not found" group
                            static const size_t subtotalNotFoundIndex; ///< index of second subtotal
                            static const size_t totalIndex; ///< indef of the total field

                            static const char * emptyLabel; ///< signifying empty (not to be displayed) labels
                    };

                }
            }
        }
    }
}


#endif
