/**
 * @file	ReportInstallWidget.cxx
 * @brief	report install widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "ReportInstallWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Printing;

ISMTTLog * ReportInstallWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.ReportInstallWidget", GETTEXT_PACKAGE );

ReportInstallWidget::ReportInstallWidget ( GenericPrintingWidget * genericWidget )
    : Smartmatic::GUI::Widgets::StepWidget(), genericWidget ( genericWidget )
{
    this->installationPrintDocument = NULL;

    genericWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &ReportInstallWidget::end ) );
    genericWidget->getSignalError().connect ( sigc::mem_fun ( *this, &ReportInstallWidget::end ) );

    genericWidget->setTextButtons ( _ ( "Smartmatic.SAES.GUI.Widgets.ContingencyCardGenWidget.RetryButtonLabel" ), _ ( "Smartmatic.SAES.GUI.Widgets.ContingencyCardGenWidget.QuitButtonLabel" ) );

    this->setWidget ( genericWidget );
}

ReportInstallWidget::~ReportInstallWidget()
{
    dispose();
}

void ReportInstallWidget::initialize()
{
    genericWidget->setInfoTitle ( _ ( "Smartmatic.SAES.GUI.Widgets.PMInstallationWidget.PrintingReport" ) );
    genericWidget->setInfoText ( _ ( "Smartmatic.SAES.GUI.Widgets.PMInstallationWidget.TextPrintingReport" ) );
}

void ReportInstallWidget::dispose()
{
    this->remove();

    if ( installationPrintDocument )
    {
        delete installationPrintDocument;
        installationPrintDocument = NULL;
    }

    if ( genericWidget )
    {
        delete genericWidget;
        genericWidget = NULL;
    }
}

void ReportInstallWidget::start ( bool createBarcode, bool showRetry, bool showCancel )
{
    if ( installationPrintDocument )
    {
        delete installationPrintDocument;
        installationPrintDocument = NULL;
    }

    installationPrintDocument = new InstallationPrintDocument ( MachineOperationManager::getInstance()->getElectoralConfiguration(), MachineOperationManager::getInstance()->getOperationStatus(), true, createBarcode );
    installationPrintDocument->setIsPasswordRetry ( !createBarcode );

    initialize();

    genericWidget->start ( installationPrintDocument, showRetry, showCancel );
}

void ReportInstallWidget::end()
{
    if ( genericWidget->getStatus() )
    {
        MachineOperationManager::getInstance()->saveElectoralConfigurationReportPrinted();
    }

    emit();
}

bool ReportInstallWidget::getStatus()
{
    return genericWidget->getStatus();
}

std::string ReportInstallWidget::getTitleText()
{
    std::string message;

    switch ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getInfoMachine() )
    {
        case InfoMachineConfiguration::CONFIGURATION_PM:
            message = _ ( "Smartmatic.SAES.GUI.Windows.InitializationWindow.PMTitle" );
            break;

        default:
        case InfoMachineConfiguration::CONFIGURATION_VM:
            message = _ ( "Smartmatic.SAES.GUI.Windows.InitializationWindow.VMTitle" );
            break;
    }

    return message;
}
