/**
 * @file SearchAndMonitorDevicesWidget.cxx
 * @brief Body class search and mount device monitor pool widget
 * @date 05/09/2012
 * @author Yadickson Soto
 */

#include "SearchAndMonitorDevicesWidget.hxx"

#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <System/OptParser.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Operation/Devices/EurnController.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;

ISMTTLog * SearchAndMonitorDevicesWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.SearchAndMonitorDevicesWidget", GETTEXT_PACKAGE );

SearchAndMonitorDevicesWidget::SearchAndMonitorDevicesWidget ()
{
    widget = new ProgressWidget();

    widget->setInfoTitle ( "  " );
    widget->setInfoText ( "  " );
    widget->setImage ( ImageResourcePath::PROGRESS );

    endSignal.connect ( sigc::mem_fun ( *this, &SearchAndMonitorDevicesWidget::endFindDevices ) );

    widget->setAccessibleName ( "SearchAndMonitorDevices" );

    setWidget ( widget );
}

SearchAndMonitorDevicesWidget::~SearchAndMonitorDevicesWidget()
{
    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void SearchAndMonitorDevicesWidget::initialize()
{
    start ( 0 );
}

bool SearchAndMonitorDevicesWidget::process()
{
    DeviceTypeMonitorPool::getInstance()->initialize();
    endSignal.emit();
    return false;
}

void SearchAndMonitorDevicesWidget::endFindDevices()
{
    DeviceTypeMonitorPool::getInstance()->start ( 250 );
    emit();
    std::string machineModel;
    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);
    if(StringUtils::toUpper(machineModel) == "PM-NOTEBOOK-A" ||
               StringUtils::toUpper(machineModel) == "PM-NOTEBOOK-B" ||
               StringUtils::toUpper(machineModel) == "PM-VIU805" )
   {
       logger->Debug("Sending EURN PING Command to get version");
       EurnController::getInstance()->getVersion();
       logger->Debug("Finished sending EURN PING Command to get version");
   }
}
