/**
 * @file   ShutdownWidget.cxx
 * @brief  Implementation of ShutdownWidget class.
 * @author manuel.arguelles
 * @date   Oct 20, 2017
 */

#include "ShutdownWidget.hxx"
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <System/Runtime/PowerDown.hxx>
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif
#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::System::Runtime;

ShutdownWidget::ShutdownWidget()
{
    ButtonConfiguration          buttonConfiguration;
    LabelConfiguration           labelConfiguration;
    SMTTGUIConfigurationManager* manager;
    SaesGUIConfiguration*        configuration;
    int                          buttonHeight;
    int                          buttonWidth;

    manager       = SMTTGUIConfigurationManager::getInstance();
    configuration = SAESGUIConfigurationManager::getInstance()->getConfiguration();

    buttonConfiguration = configuration->getShutdownWidgetConfiguration().getButtonConfiguration();
    labelConfiguration  = SAESGUIConfigurationManager::ConvertToLabelConfiguration(
        configuration->getShutdownWidgetConfiguration().getLabelConfiguration());

    LabelConfiguration::SetLabelConfiguration(&m_label, labelConfiguration);

    buttonHeight = buttonConfiguration.getButtonHeight();
    buttonWidth  = buttonConfiguration.getButtonWidth();

    if (manager->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString(PM_VIU805))
    {
        SmartmaticGUIConfiguration* guiConfiguration = manager->getConfiguration();
        double guiReconfigurationFactor = guiConfiguration->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        buttonWidth *= guiReconfigurationFactor;
        buttonHeight *= guiReconfigurationFactor;
    }

    m_button.set_label(_("Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.ButtonShutDown"));
    m_button.set_size_request(buttonWidth, buttonHeight);
    m_button.ModifyFontSize(buttonConfiguration.getFontSize());

    m_hBox.pack_start(m_button, true, false);
    m_vBox.pack_start(m_label, true, false, 20);
    m_vBox.pack_start(m_hBox, true, false, 20);

    setWidget(&m_vBox);

    m_button.signal_clicked().connect(sigc::mem_fun(*this, &ShutdownWidget::shutDown));
}

ShutdownWidget::~ShutdownWidget()
{
}

void ShutdownWidget::setLabel(const std::string& text)
{
    m_label.set_text(text);
}

void ShutdownWidget::shutDown()
{
    PowerDown::shutdown();
}
