/**
 * @file   ShutdownWidget.hxx
 * @brief  Declaration of ShutdownWidget class.
 * @author manuel.arguelles
 * @date   Oct 20, 2017
 */

#ifndef SHUTDOWN_WIDGET_HXX
#define SHUTDOWN_WIDGET_HXX

#include <Widgets/smartmatic-button.h>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/SmartmaticBasicHBox.hxx>
#include <Widgets/StepWidget.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {

                /**
                 * @class  ShutdownWidget
                 * @brief  Shutdown widget.
                 * @author manuel.arguelles
                 * @date   Oct 20, 2017
                 */
                class ShutdownWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 20, 2017
                         */
                        ShutdownWidget();

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 20, 2017
                         */
                        virtual ~ShutdownWidget();

                    public:

                        /**
                         * @brief  Set the text of the label.
                         * @author manuel.arguelles
                         * @date   Oct 23, 2017
                         * @param  text The text of the label.
                         */
                        void setLabel(const std::string& text);

                    private:

                        /**
                         * @brief  Performs a shutdown.
                         * @author manuel.arguelles
                         * @date   Oct 23, 2017
                         */
                        void shutDown();

                    private:

                        Smartmatic::GUI::Widgets::SmartmaticButton      m_button;
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel  m_label;
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox   m_vBox;
                        Smartmatic::GUI::Widgets::SmartmaticBasicHBox   m_hBox;
                };

            } /* namespace Widgets */
        } /* namespace GUI */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* SHUTDOWN_WIDGET_HXX */
