/**
 * @file	BasicTransmissionWidget.hxx
 * @brief	Declares the basic transmission widget class.
 * @author	Yadickson Soto
 * @date	08/04/2014
 */


#ifndef _BASIC_TRANSMISSION_WIDGET_H_
#define _BASIC_TRANSMISSION_WIDGET_H_

#include <Widgets/wizard-widget-base.h>
#include <Gui/Widgets/GenericWidget.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	BasicTransmissionWidget
                 * @brief	Basic transmission widget.
                 * @author	Yadickson Soto
                 * @date	08/04/2014
                 */

                class BasicTransmissionWidget: public Smartmatic::GUI::Widgets::WizardWidgetBase
                {

                    protected:

                        /**
                         * @fn	BasicTransmissionWidget ( GenericWidget * widget, bool showCancel = false, bool showRetry = false );
                         * @brief	Constructor.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         * @param	widget	widget with label and image.
                         * @param	showCancel 	true to show, false to hide the cancel button.
                         * @param	showRetry	true to show, false to hide the retry button.
                         */
                        BasicTransmissionWidget ( GenericWidget * widget, bool showCancel = false, bool showRetry = false );

                        /**
                         * @fn	BasicTransmissionWidget::~BasicTransmissionWidget ( );
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        virtual ~BasicTransmissionWidget ( );

                        /**
                         * @fn	void showButtons ( bool showCancel = true, bool showRetry = true );
                         * @brief	Show or hide buttons.
                         * @author	Yadickson Soto
                         * @date	06/09/2013
                         * @param	showCancel 	true to show, false to hide the cancel button.
                         * @param	showRetry	true to show, false to hide the retry button.
                         */
                        void showButtons ( bool showCancel = true, bool showRetry = true );

                        /**
                         * @fn	virtual void initialize();
                         * @brief	Initializer method.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        virtual void initialize();

                        /**
                         * @fn	void errorMessage();
                         * @brief	Show error message.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        void errorMessage();

                        /**
                         * @fn	void endProcess();
                         * @brief	Completes the saving of all result files.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        void endProcess();

                        /**
                         * @fn	void errorProcess ();
                         * @brief	Updates the label because of an error in the processing.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        void errorProcess ();

                        /**
                         * @fn	void runThreadProcess();
                         * @brief	The processing thread entry point.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        void runThreadProcess();

                        /**
                         * @fn	virtual bool runProcess() = 0;
                         * @brief	Method which does the actual processing.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         * @return true if success
                         */
                        virtual bool runProcess() = 0;

                        /**
                         * @fn	virtual void showSuccessMessage() = 0;
                         * @brief	Show success message.
                         * @author	Yadickson Soto
                         * @date	02/05/2014
                         */
                        virtual void showSuccessMessage() = 0;

                        /**
                         * @fn	virtual void showErrorMessage() = 0;
                         * @brief	Show error message.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        virtual void showErrorMessage() = 0;

                        GenericWidget * widget; ///< widget with label and image

                    public:

                        /**
                         * @fn	void end();
                         * @brief	End of the widget.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        void end();

                    private:

                        Glib::Dispatcher signalEndProcess; ///< end of processing signal
                        Glib::Dispatcher signalErrorProcess; ///< error in processing signal
                        Glib::Thread * m_thread; ///< The thread pointer
                };
            }
        }
    }
}

#endif // _BASIC_TRANSMISSION_WIDGET_H_
