/**
 * @file SendResultsPackageWidget.hxx
 * @brief Header class of the send results package widget
 * @date 07/04/2014
 * @author Yadickson Soto
 */

#ifndef SENDRESULTSPACKAGEWIDGET_HXX_
#define SENDRESULTSPACKAGEWIDGET_HXX_

#include <Gui/Widgets/Transmission/BasicTransmissionWidget.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	SendResultsPackageWidget
                 * @brief	Class of the send results package widget
                 * @author	Yadickson Soto
                 * @see 	BasicTransmissionWidget
                 * @date	08/04/2014
                 */
                class SendResultsPackageWidget: public BasicTransmissionWidget
                {
                    public:
                        /**
                         * @fn	SendResultsPackageWidget ( GenericWidget * widget );
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         * @param	widget	widget with label and image.
                         */
                        SendResultsPackageWidget ( GenericWidget * widget );

                        /**
                         * @fn	virtual ~SendResultsPackageWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        virtual ~SendResultsPackageWidget();

                        /**
                         * @fn	virtual void initialize();
                         * @brief	Initializer method.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        virtual void initialize();

                        /**
                         * @fn	virtual bool runProcess();
                         * @brief	Transmits the results package
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         * @return true if success
                         */
                        virtual bool runProcess();

                        /**
                         * @fn	virtual void showSuccessMessage();
                         * @brief	Show success message.
                         * @author	Yadickson Soto
                         * @date	02/05/2014
                         */
                        virtual void showSuccessMessage();

                        /**
                         * @fn	virtual void showErrorMessage();
                         * @brief	Updates the label because of an error in the processing.
                         * @author	Yadickson Soto
                         * @date	08/04/2014
                         */
                        virtual void showErrorMessage();

                    private:

                        std::string messageResponse; ///< The message response
                };
            }
        }
    }
}

#endif /* SENDRESULTSPACKAGEWIDGET_HXX_ */
