/**
 * @file VMAuthenticationWidget.cxx
 * @brief Body class vm authentication widget
 * @date 31/03/2012
 * @author Yadickson Soto
 */

#include "VMAuthenticationWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/AlarmController.hxx>
#include <Operation/Devices/Polling/BCRPolling.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String)
#else
#define _(String) (String)
#endif

#define N_(String) (String)

using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation::Authenticator;
using namespace Smartmatic::SAES::Operation::Devices;

ISMTTLog* VMAuthenticationWidget::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.GUI.Widgets.VMAuthenticationWidget",
    GETTEXT_PACKAGE);

VMAuthenticationWidget::VMAuthenticationWidget(
    PollingPlaceInfoWidget* info,
    IAuthenticator* authenticator)
    : AuthenticationWidget(
        N_("Smartmatic.SAES.GUI.Windows.AuthenticationWindow.Title"),
        info,
        authenticator)
{
    Glib::RefPtr<Gtk::Builder> builder;

    try
    {
        builder = Gtk::Builder::create_from_file(
            ResourcePathProvider::getInstance()->GetResourcePath(
                PACKAGE_NAME,
                "ui/VMAuthenticationWidget.glade"));
    }
    catch (const Glib::FileError& ex)
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        m_logger->Debug(ss.str());
    }

    builder->get_widget("main-frame", mainFrame);
    builder->get_widget("main-vbox", mainVBox);
    builder->get_widget("info-frame", infoFrame);
    builder->get_widget("top-label", topLabel);
    builder->get_widget("key-frame", keyFrame);
    builder->get_widget("bottom-label", bottomLabel);
    builder->get_widget("alarm-label", alarmLabel);
    builder->get_widget("alignment-keypad", m_alignmentKeypad);

    if (info)
    {
        infoFrame->set_name("infoFrame");
        infoFrame->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
        infoFrame->add(*info);
        infoFrame->show_all_children();
    }
    else
    {
        infoFrame->set_shadow_type(Gtk::SHADOW_NONE);
    }

    infoFrame->show();
    keyFrame->add(*keypadTest);
    keypadTest->show();
    config();
    alarmLabel->show();
    setWidget(mainFrame);
}

VMAuthenticationWidget::~VMAuthenticationWidget()
{
    remove();
    stop();

    if (m_alignmentKeypad)
    {
        delete m_alignmentKeypad;
        m_alignmentKeypad = NULL;
    }

    AuthenticationWidget::dispose();
}

void VMAuthenticationWidget::initialize()
{
    m_logger->Debug("Start all devices");

    AlarmPolling::getInstance()->start(100);
    BCRPolling::getInstance()->start(100);
    CardPolling::getInstance()->start(100);
}

void VMAuthenticationWidget::refresh()
{
    m_logger->Debug("Refresh");

    AuthenticationWidget::refresh();

    m_connAlarm.disconnect();
    m_connAlarm = AlarmPolling::getInstance()->getSignalPressButton().connect(
        sigc::mem_fun<AlarmPolling::AlarmButtonStatus>( *this, &VMAuthenticationWidget::clearAlarm));
}

void VMAuthenticationWidget::clearAlarm(AlarmPolling::AlarmButtonStatus status)
{
    m_logger->Debug("Signal Button");

    if (status == AlarmPolling::PRESSED)
    {
        clearAlarm();
    }
}

void VMAuthenticationWidget::clearAlarm()
{
    AuthenticationWidget::clearAlarm();
    AlarmController::getInstance()->shutDownAlarm();
}

void VMAuthenticationWidget::stop()
{
    m_logger->Debug("Stop Check Alarm Button");
    m_connAlarm.disconnect();
    clearAlarm();
}
