/**
 * @file VMInitiatorWidget.cxx
 * @brief Body class voting machine initiator widget
 * @date 04/10/2012
 * @author Yadickson Soto
 */

#include "VMInitiatorWidget.hxx"
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Voting/VotingWidgetManager.hxx>
#include <Operation/Voting/VotingExperienceController.hxx>
#include <Operation/Voting/AudioVoting/AudioVotingExperienceController.hxx>
#include <Voting/OperationStatus.hxx>

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Voting;

VMInitiatorWidget::VMInitiatorWidget ( ProgressWidget * progress, IMachineInitiator * initiator )
    : InitiatorWidget ( progress, initiator )
{
    VotingWidgetManager *votingWidgetManager = VotingWidgetManager::getInstance();
    votingWidgetManager->SetElectoralConfiguration ( MachineOperationManager::getInstance()->getElectoralConfiguration() );
    votingWidgetManager->SetSaesDirectories ( MachineOperationManager::getInstance()->getSaesDirectories() );
    initiator->addCondition ( votingWidgetManager );
    VotingExperienceController *votingExperienceController = new VotingExperienceController ( MachineOperationManager::getInstance()->getElectoralConfiguration() );

    AudioVotingExperienceController *audioVotingExperienceController = new AudioVotingExperienceController();

    MachineOperationManager::getInstance()->setVotingExperienceController ( votingExperienceController );
    MachineOperationManager::getInstance()->setAudioVotingExperienceController ( audioVotingExperienceController );

    MachineOperationManager::getInstance()->setMachineInitiator ( initiator );
}

VMInitiatorWidget::~VMInitiatorWidget()
{
}
