/**
 * @file	VMInstallMachineWidget.hxx
 * @brief	voting machine install machine widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef VMINSTALLMACHINEWIDGET_HXX_
#define VMINSTALLMACHINEWIDGET_HXX_

#include <Gui/Widgets/InstallMachineWidget.hxx>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicImage.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	VMInstallMachineWidget
                 * @brief	Virtual memory install machine widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class VMInstallMachineWidget : public InstallMachineWidget
                {
                    public:
                        /**
                         * @fn	VMInstallMachineWidget::VMInstallMachineWidget ( ProgressWidget * progressWidget,
                         * 			PollingPlaceInfoWidget * pollingPlaceInfoWidget );
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	progressWidget		If non-null, the progress widget.
                         */
                        explicit VMInstallMachineWidget ( ProgressWidget * progressWidget );
                        /**
                         * @fn	virtual VMInstallMachineWidget::~VMInstallMachineWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~VMInstallMachineWidget();
                        /**
                         * @fn	virtual void VMInstallMachineWidget::stop();
                         * @brief	Stops this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual void stop();
                        /**
                         * @brief virtual method getter title
                         * @return title
                         * @date 30/03/2012
                         * @author Yadickson Soto
                         */
                        virtual std::string getTitleText();

                    private:

                        /**
                         * @fn	virtual void VMInstallMachineWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual void initialize();
                        /**
                         * @fn	virtual void VMInstallMachineWidget::dispose();
                         * @brief	virtual method stop.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void dispose();

                        /**
                         * @fn	void VMInstallMachineWidget::authentication ( const std::string& password );
                         * @brief	Authentications the given password.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	password	The password.
                         */
                        void authentication ( const std::string& password );
                        /**
                         * @fn	void VMInstallMachineWidget::showProgressLogin();
                         * @brief	Shows the progress login.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void showProgressLogin();
                        /**
                         * @fn	void VMInstallMachineWidget::checkData();
                         * @brief	Check data.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void checkData();
                        /**
                         * @fn	void VMInstallMachineWidget::showInfo();
                         * @brief	Shows the information.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void showInfo();
                        /**
                         * @fn	void VMInstallMachineWidget::onShowInfo();
                         * @brief	Shows the information.
                         * @author	Yadickson Soto
                         * @date	10/09/2013
                         */
                        void onShowInfo();
                        /**
                         * @fn	void VMInstallMachineWidget::end();
                         * @brief	Ends this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void end();

                        std::string center; ///< The center
                        std::string password; ///< The password
                        bool loginOk; ///< true if the login operation was a success, false if it failed
                        sigc::connection conn;	///< The connection

                        Glib::Dispatcher signalDispatcher; ///< Signal dispatcher
                        Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget;	///< The progress widget
                        Smartmatic::SAES::GUI::Widgets::ProgressWidget * errorWidget;   ///< The error widget
                        Glib::Thread * m_thread; ///< The thread pointer
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };
            }
        }
    }
}

#endif /* VMINSTALLMACHINEWIDGET_HXX_ */
