/**
 * @file EndVotingSequenceWidget.hxx
 * @brief Header class end voting sequence widget
 * @date 02/04/2013
 * @author Yadickson Soto
 */

#ifndef ENDVOTINGSEQUENCEWIDGET_HXX_
#define ENDVOTINGSEQUENCEWIDGET_HXX_

#include <Widgets/StepWidget.hxx>
#include <Widgets/wizard-widget-manager.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {

                    /**
                     * @class	EndVotingSequenceWidget
                     * @brief	class end voting sequence widget
                     * @author	Yadickson Soto
                     * @date	02/04/2013
                     */

                    class EndVotingSequenceWidget : public Smartmatic::GUI::Widgets::StepWidget
                    {
                        public:
                            /**
                             * @fn	EndVotingSequenceWidget();
                             * @brief	Default constructor.
                             * @author	Yadickson Soto
                             * @date	02/04/2013
                             */

                            EndVotingSequenceWidget();

                            /**
                             * @fn	virtual ~EndVotingSequenceWidget();
                             * @brief	Class destructor.
                             * @author	Yadickson Soto
                             * @date	02/04/2013
                             */
                            virtual ~EndVotingSequenceWidget();
                            /**
                             * @fn	void start();
                             * @brief	method start sequence.
                             * @author	Yadickson Soto
                             * @date	02/04/2013
                             */
                            void start();
                            /**
                             * @fn	sigc::signal<void> getSignalError();
                             * @brief	method getter signal error.
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             * @return	signal error.
                             */
                            sigc::signal<void> getSignalError();

                        protected:

                            /**
                             * @fn	virtual void refresh();
                             * @brief	refresh widget method.
                             * @author	Yadickson Soto
                             * @date	02/04/2013
                             */
                            virtual void refresh();

                            /**
                             * @fn	virtual void dispose();
                             * @brief	method dispose widget.
                             * @author	Yadickson Soto
                             * @date	02/04/2013
                             */
                            virtual void dispose();

                            /**
                             * @fn	void endSequence();
                             * @brief	method end sequence.
                             * @author	Yadickson Soto
                             * @date	02/04/2013
                             */
                            void endSequence();

                            /**
                             * @fn	void errorSequence();
                             * @brief	method error sequence.
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             */
                            void errorSequence();

                        private:

                            bool errorDetected; ///< error detected
                            sigc::signal<void> signalError; ///< signal error
                            Smartmatic::GUI::Widgets::WizardWidgetManager * manager; ///< wizard widgets
                            std::vector < Smartmatic::GUI::Widgets::WizardWidgetBase * > wizardVector; ///< The wizard vector
                            static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                    };

                }
            }
        }
    }
}
#endif /* ENDVOTINGSEQUENCEWIDGET_HXX_ */
