/**
 * @file	ICustomAttributeSelectionWidget.hxx
 * @brief	Header interface the custom attribute selection widget.
 * @author	Yadickson Soto
 * @date	13/11/2012
 */

#ifndef _ICUSTOM_ATTRIBUTE_SELECTION_WIDGET_H_
#define _ICUSTOM_ATTRIBUTE_SELECTION_WIDGET_H_

#include <Widgets/StepWidget.hxx>
#include <Voting/VotingDevice/voting-devices-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    /**
                     * @class	ICustomAttributeSelectionWidget
                     * @brief	Custom attribute selection widget.
                     * @author	Yadickson Soto
                     * @date	13/11/2012
                     */
                    class ICustomAttributeSelectionWidget: public virtual Smartmatic::GUI::Widgets::StepWidget
                    {
                        public:

                            /**
                             * @fn	ICustomAttributeSelectionWidget::ICustomAttributeSelectionWidget ( );
                             * @brief	Class constructor.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             * @param customAttr custom attribute definition
                             * @param	showConfirmation	true to show confirmation.
                             * @param	sortAlphabetically	true to sort alphabetically.
                             */
                            ICustomAttributeSelectionWidget ( Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & customAttr,
                                                              bool showConfirmation, bool sortAlphabetically );
                            /**
                             * @fn	virtual ICustomAttributeSelectionWidget::~ICustomAttributeSelectionWidget();
                             * @brief	Class destructor.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual ~ICustomAttributeSelectionWidget();
                            /**
                             * @fn	virtual void ICustomAttributeSelectionWidget::ForceSelectedOption ( unsigned int index ) = 0;
                             * @brief	Simulate the press of the button option.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             * @param	index	index of the option.
                             */
                            virtual void ForceSelectedOption ( unsigned int index ) = 0;

                            /**
                             * @typedef	sigc::signal<void, std::string> string_signal
                             * @brief	Defines an alias representing the string signal. .
                             */

                            typedef sigc::signal<void, std::string> string_signal;

                            /**
                             * @fn	string_signal ICustomAttributeSelectionWidget::signal_Answer();
                             * @brief	Raises the answer event.
                             * @author	Juan Delgado, Yadickson Soto
                             * @date	28/04/2011
                             * @return	signal
                             */

                            string_signal signal_Answer();

                        protected:

                            /**
                             * @fn void sortCustomAttribute();
                             * @brief Method sort custom attribute definition by language or alphabetically
                             * @author	Yadickson Soto
                             * @date	30/04/2013
                             */
                            void sortCustomAttribute();

                            string_signal m_answer_signal;  ///< The answer signal
                            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition customAttr; ///< current custom attribute definition
                            std::list < std::string > cadList; ///< custom attribute definition list
                            bool showConfirmation; ///< flag to show confirmation.
                            bool sortAlphabetically; ///< flag to sort alphabetically.
                            bool flagForce; ///< flagForce
                    };
                }
            }
        }
    }
}

#endif // _ICUSTOM_ATTRIBUTE_SELECTION_WIDGET_H_
