/**
 * @file PreviewWidget.cxx
 * @brief Header class preview
 * @date 13/04/2012
 * @author Yadickson Soto
 */

#include "PreviewWidget.hxx"

#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include "FactoryVotingWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Voting/set-language-functionality.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::Log;

ISMTTLog * PreviewWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.Voting.PreviewWidget", GETTEXT_PACKAGE );

PreviewWidget::PreviewWidget()
{
    view = false;
    votePreviewWidget = NULL;

    timeOutPreviewWidget = FactoryVotingWidget::createTimeOutPreviewWidget();
    timeOutPreviewWidget->getCloseClicked().connect ( sigc::mem_fun ( *this, &PreviewWidget::back ) );
    timeOutPreviewWidget->getKeepClicked().connect ( sigc::mem_fun ( *this, &PreviewWidget::viewVote ) );

    dispatcherRelease.connect ( sigc::mem_fun ( *this, &PreviewWidget::releaseTimeout ) );
    dispatcherBack.connect ( sigc::mem_fun ( *this, &PreviewWidget::back ) );

    setTitle ( false );
    setShowBorder ( true );
    setName ( SAESGUIConfigurationManager::getInstance()->GetVotingExperienceWindowName() );
}

PreviewWidget::~PreviewWidget()
{
    remove();

    if ( votePreviewWidget )
    {
        delete votePreviewWidget;
        votePreviewWidget = NULL;
    }

    if ( timeOutPreviewWidget )
    {
        delete timeOutPreviewWidget;
        timeOutPreviewWidget = NULL;
    }
}

void PreviewWidget::preview ( Vote & vote )
{
    view = true;
    this->vote = vote;
    viewVote();
}

void PreviewWidget::error()
{
    view = false;
    setTimeout();

    deleteVotePreview ();

    votePreviewWidget = FactoryVotingWidget::createVotePreviewWidget();
    votePreviewWidget->getBackClicked().connect ( sigc::mem_fun ( *this, &PreviewWidget::previewBack ) );
    votePreviewWidget->getSignalEvent().connect ( sigc::mem_fun ( *this, &PreviewWidget::setTimeout ) );

    setWidget ( votePreviewWidget );
    votePreviewWidget->error();
}

void PreviewWidget::resetTimeOut()
{
    connTimeout.disconnect();
    connTimeoutClose.disconnect();
}

void PreviewWidget::back()
{
    connTimeout.disconnect();
    connTimeoutClose.disconnect();
    emit();
}

void PreviewWidget::viewVote()
{
    setTimeout();

    deleteVotePreview ();

    votePreviewWidget = FactoryVotingWidget::createVotePreviewWidget();
    votePreviewWidget->getBackClicked().connect ( sigc::mem_fun ( *this, &PreviewWidget::previewBack ) );
    votePreviewWidget->getSignalEvent().connect ( sigc::mem_fun ( *this, &PreviewWidget::setTimeout ) );

    setWidget ( votePreviewWidget );

    if ( view )
    {
        votePreviewWidget->preview ( vote );
    }
    else
    {
        votePreviewWidget->error();
    }

    show_all_children();
}

void PreviewWidget::setTimeout()
{
    connTimeout.disconnect();
    connTimeoutClose.disconnect();
    int timeout = SAESGUIConfigurationManager::getInstance()->getWaitingForCardWindowConfiguration().getTimeoutPreviewVote() * 1000;

    if ( timeout > 0 )
    {
        connTimeout = Glib::signal_timeout().connect ( sigc::mem_fun ( *this, &PreviewWidget::release ), timeout );
    }
}

bool PreviewWidget::release ()
{
    dispatcherRelease.emit();
    return false;
}

void PreviewWidget::releaseTimeout ()
{
    deleteVotePreview ();

    setWidget ( timeOutPreviewWidget );
    connTimeoutClose.disconnect();
    int timeout = SAESGUIConfigurationManager::getInstance()->getWaitingForCardWindowConfiguration().getTimeoutPreviewVoteClose() * 1000;

    if ( timeout > 0 )
    {
        connTimeoutClose = Glib::signal_timeout().connect ( sigc::mem_fun ( *this, &PreviewWidget::releaseTimeoutClose ), timeout );
    }
}

bool PreviewWidget::releaseTimeoutClose ()
{
    deleteVotePreview ();
    dispatcherBack.emit();
    return false;
}

void PreviewWidget::previewBack ()
{
    deleteVotePreview ();
    dispatcherBack.emit();
}

void PreviewWidget::deleteVotePreview ()
{
    remove ();

    if ( votePreviewWidget )
    {
        delete votePreviewWidget;
        votePreviewWidget = NULL;
    }
}
