/**
 * @file RemoveCardWidget.hxx
 * @brief Header class card removed widget
 * @date 29/02/2012
 * @author Yadickson Soto
 */

#ifndef REMOVECARDWIDGET_HXX_
#define REMOVECARDWIDGET_HXX_

#include <Gui/Widgets/ProgressWidget.hxx>
#include <Widgets/wizard-widget-base.h>
#include <System/SafeTime.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    /**
                     * @class	RemoveCardWidget
                     * @brief	Class card removed widget.
                     * @author	Yadickson Soto
                     * @date	29/02/2012
                     */
                    class RemoveCardWidget : public Smartmatic::GUI::Widgets::WizardWidgetBase
                    {
                        public:

                            /**
                             * @fn	RemoveCardWidget::RemoveCardWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress );
                             * @brief	Class constructor.
                             * @author	Yadickson Soto
                             * @date	29/02/2012
                             * @param	progress	progress widget.
                             */
                            explicit RemoveCardWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress );
                            /**
                             * @fn	virtual RemoveCardWidget::~RemoveCardWidget();
                             * @brief	Class destructor.
                             * @author	Yadickson Soto
                             * @date	29/02/2012
                             */
                            virtual ~RemoveCardWidget();
                            /**
                             * @fn	virtual void RemoveCardWidget::refresh();
                             * @brief	refresh widget method.
                             * @author	Yadickson Soto
                             * @date	30/03/2012
                             */
                            virtual void refresh();

                        protected:

                            /**
                             * @fn	virtual void initialize();
                             * @brief	Initializer method.
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             */
                            virtual void initialize();

                            /**
                             * @fn	void endOK();
                             * @brief	Method end ok
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             */
                            void endOK();

                            /**
                             * @fn	void endError();
                             * @brief	Method end error
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             */
                            void endError();

                            /**
                             * @fn	bool checkCard ( int timer_num );
                             * @brief	Method check card
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             * @param	timer_num	The timer number.
                             * @return	true continue
                             */
                            bool checkCard ( int timer_num );

                            /**
                             * @fn	void endCount();
                             * @brief	End count
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             */
                            void endCount();

                        private:

                            int count; ///< time count
                            int timeOut; ///< timeout end voting
                            Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress;	///< The progress
                            Smartmatic::System::SafeTime safeTime; ///< safe time
                            sigc::connection connRefresh;   ///< connection refresh
                    };
                }
            }
        }
    }
}

#endif /* REMOVECARDWIDGET_HXX_ */
