/**
 * @file ScanVoteWidget.cxx
 * @brief Body class end print vote widget
 * @date 27/03/2014
 * @author Yadickson Soto
 */

#include "ScanVoteWidget.hxx"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;

ScanVoteWidget::ScanVoteWidget ( ProgressWidget * progress )
    : WizardWidgetBase(), progress ( progress )
{
    progress->setInfoTitleConfig ( SAESGUIConfigurationManager::getInstance()->GetVotingWindowEndMessagelabelConfig() );
    progress->setInfoTextConfig ( SAESGUIConfigurationManager::getInstance()->GetVotingWindowEndMessagelabelConfig() );
    progress->setImage ( "ui/scan-vote.gif" );
    setAccessibleName ( "ScanVote" );

    refresh();
    setTitle ( false );
    setShowBorder ( true );
    setWidget ( progress );
}

ScanVoteWidget::~ScanVoteWidget()
{
    this->remove();

    if ( progress )
    {
        delete progress;
        progress = NULL;
    }
}

void ScanVoteWidget::refresh()
{
    progress->setInfoTitle ( N_ ( "Beware! This vote has not been registered yet. Please head to the electronic urn and scan the QR-code of the vote to complete the voting process." ) );
    progress->setInfoText ( N_ ( "Please remove the voting card from the machine at this time." ) );
}

void ScanVoteWidget::initialize()
{
    unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_SCAN_VOTE );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &ScanVoteWidget::end ), time );
}

void ScanVoteWidget::end()
{
    emit();
}
