/**
 * @file	VotingExperienceWidget.hxx
 * @brief	voting experience widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */



#ifndef _VOTING_EXPERIENCE_WIDGET_H_
#define _VOTING_EXPERIENCE_WIDGET_H_

#include <gtkmm.h>
#include "Gui/Widgets/party-selection-widget.h"
#include "Gui/Widgets/ballot-option-selection-widget.h"
#include "Operation/Voting/VotingExperienceController.hxx"
#include "Configuration/SaesGUIConfiguration-schema.hxx"
#include "Voting/Election/election-schema.hxx"
#include "Voting/Election/ElectionWrapper.hxx"
#include "Operation/Voting/VotingWidgetManager.hxx"
#include "Gui/Widgets/Voting/ICustomAttributeSelectionWidget.hxx"
#include "Gui/Widgets/voting-frame-base.h"
#include "Gui/Widgets/referendum-selection-widget.h"
#include <Gui/Widgets/Voting/CardRemovedWidget.hxx>
#include <Gui/Widgets/Voting/PrintTimeOutWidget.hxx>
#include <Gui/Widgets/Voting/InterruptWidget.hxx>
#include <Gui/Widgets/Voting/EndVotingSequenceWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    /**
                     * @class	VotingExperienceWidget
                     * @brief	Form for viewing the voting experience.
                     * @author	Juan delgado &lt;juan.delgado@smartmatic.com&gt;
                     * @date	28/04/2011
                     */

                    class VotingExperienceWidget: public Smartmatic::GUI::Widgets::StepWidget
                    {
                        public:

                            /**
                             * @enum VotingExperienceStepType
                             * @brief defines the steps of the voting experience depending on the controller.
                             */

                            typedef enum
                            {
                              VOTING_EXPERIENCE_STEP_NO_INIT,
                              VOTING_EXPERIENCE_STEP_CUSTOM_ATTRIBUTE_WIDGET,
                              VOTING_EXPERIENCE_STEP_CONTEST_WIDGET_CHILD,
                              VOTING_EXPERIENCE_STEP_CONTEST_WIDGET,
                              VOTING_EXPERIENCE_STEP_CLOSE
                            } VotingExperienceStepType;

                            /**
                             * @fn	VotingExperienceWidget::VotingExperienceWidget();
                             * @brief	Default constructor.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            VotingExperienceWidget();

                            /**
                             * @fn	virtual VotingExperienceWidget::~VotingExperienceWidget();
                             * @brief	class destructor.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            virtual ~VotingExperienceWidget();

                            /**
                             * @fn	void VotingExperienceWidget::setActualContest ( Smartmatic::SAES::Voting::Election::Contest contestA );
                             * @brief	Sets an actual contest.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	contestA	The contest.
                             */

                            void setActualContest ( Smartmatic::SAES::Voting::Election::Contest * contestA );

                            /**
                             * @fn	void VotingExperienceWidget::setDetectCardExtractionEnable ( bool enable );
                             * @brief	Sets detect card extraction enable.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	enable	true to enable, false to disable.
                             */

                            void setDetectCardExtractionEnable ( bool enable );
                            /**
                             * @fn	static void VotingExperienceWidget::setSimulationMode ( bool enable );
                             * @brief	Sets simulation mode.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	enable	true to enable, false to disable.
                             */
                            void setSimulationMode ( bool enable );

                            /**
                             * @fn	void VotingExperienceWidget::setCardWrited ( bool result );
                             * @brief	Sets card writed.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	result	true to result.
                             */
                            void setCardWrited ( bool result );

                            /**
                             * @fn	void VotingExperienceWidget::dispose();
                             * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                             * 				unmanaged resources.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */
                            void dispose();

                            /**
                             * @fn	void VotingExperienceWidget::start();
                             * @brief	Method start process.
                             * @author	Yadickson Soto
                             * @date	16/04/2012
                             */
                            void start();

                            /**
                             * @fn	void VotingExperienceWidget::cardInsertEvent();
                             * @brief	method card insert event.
                             * @author	Yadickson Soto
                             * @date	08/03/2012
                             */
                            void cardInsertEvent();
                            /**
                             * @fn	void VotingExperienceWidget::cardRemoveEvent();
                             * @brief	method card remove event.
                             * @author	Yadickson Soto
                             * @date	08/03/2012
                             */
                            void cardRemoveEvent();

                        protected:

                            /**
                             * @fn	void VotingExperienceWidget::referendumSelection_on_back_clicked();
                             * @brief	Referendum selecton signal back clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */
                            void referendumSelection_on_back_clicked();

                            /**
                             * @fn	void VotingExperienceWidget::referendumSelection_on_accept_clicked();
                             * @brief	Referendum selecton signal accept clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */
                            void referendumSelection_on_accept_clicked();

                            /**
                             * @fn	void VotingExperienceWidget::referendumSelection_on_null_clicked();
                             * @brief	Referendum selecton signal null clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */
                            void referendumSelection_on_null_clicked();

                            /**
                             * @fn	void VotingExperienceWidget::on_ReferendumNormalVote();
                             * @brief	Referendum selecton normal vote.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */
                            void on_ReferendumNormalVote();

                            /**
                             * @fn	void VotingExperienceWidget::referendumSelection_on_ballotOption_clicked ( std::string ballotOptionCode,
                             * 			bool isSelected );
                             * @brief	Referendum selecton signal ballot option clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	ballotOptionCode	The ballot option code.
                             * @param	isSelected				true if this object is selected.
                             */
                            void referendumSelection_on_ballotOption_clicked ( std::string ballotOptionCode, bool isSelected );

                            /**
                             * @fn	void VotingExperienceWidget::partySelection_on_null_clicked();
                             * @brief	Party selection signal on clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void partySelection_on_null_clicked();

                            /**
                             * @fn	void VotingExperienceWidget::partySelection_on_back_clicked();
                             * @brief	Party selection signal on back clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void partySelection_on_back_clicked();

                            /**
                             * @fn	void VotingExperienceWidget::partySelection_on_party_clicked ( Smartmatic::SAES::Voting::Election::Party partySelected );
                             * @brief	Party selection signal on party clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	partySelected	The party selected.
                             */
                            void partySelection_on_party_clicked ( Smartmatic::SAES::Voting::Election::Party partySelected );
                            /**
                             * @fn	void VotingExperienceWidget::onProcessPartyClicked ( );
                             * @brief	Process party selection signal on party clicked.
                             * @author	Yadickson Soto
                             * @date	15/11/2012
                             */
                            void onProcessPartyClicked();
                            /**
                             * @fn	void VotingExperienceWidget::ballotOptionSelection_on_accept_clicked();
                             * @brief	ballot option selection signal on accept clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ballotOptionSelection_on_accept_clicked();

                            /**
                             * @fn	void VotingExperienceWidget::ballotOptionSelection_on_clear_clicked();
                             * @brief	ballot option selection signal on clear clicked.
                             * @author	Yadickson Soto
                             * @date	24/04/2014
                             */

                            void ballotOptionSelection_on_clear_clicked();

                            /**
                             * @fn	void VotingExperienceWidget::ballotOptionSelection_on_back_clicked();
                             * @brief	Ballot option selection signal on back clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ballotOptionSelection_on_back_clicked();

                            /**
                             * @fn	void VotingExperienceWidget::ballotOptionSelection_on_ballotOption_clicked ( std::string ballotOptionCode,
                             * 			bool isSelected );
                             * @brief	Ballot option selection signal on ballot option clicked.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	ballotOptionCode	The ballot option code.
                             * @param	isSelected				true if is selected.
                             */

                            void ballotOptionSelection_on_ballotOption_clicked ( std::string ballotOptionCode, bool isSelected );

                            /**
                             * @fn	void VotingExperienceWidget::on_AcceptVote();
                             * @brief	Handles accept vote signals.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void on_AcceptVote();

                            /**
                             * @fn	void VotingExperienceWidget::on_BackVote();
                             * @brief	Handles back vote signals.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void on_BackVote();

                            /**
                             * @fn	bool VotingExperienceWidget::onTimeout ( int timer );
                             * @brief	method called on timeout.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	timer	counter timer.
                             * @return	true for continue.
                             */

                            bool onTimeout ( int timer );

                            /**
                             * @fn	void VotingExperienceWidget::on_TimeoutContestWelcome();
                             * @brief	timeout method: Handles timeout contest welcome signals.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void on_TimeoutContestWelcome();

                            sigc::connection connectTimeout;	///< connection timeout
                            sigc::connection connection_partySelectionWidget_null_clicked;  ///< connection party selection widget null clicked
                            sigc::connection connection_partySelectionWidget_back_clicked;  ///< connection party selection widget back clicked
                            sigc::connection connection_partySelectionWidget_party_clicked; ///< connection party selection widget party clicked
                            sigc::connection connection_referendumSelectionWidget_null_clicked; ///< connection referendum selection widget null clicked
                            sigc::connection connection_referendumSelectionWidget_back_clicked; ///< connection referendum selection widget back clicked
                            sigc::connection connection_referendumSelectionWidget_accept_clicked; ///< connection referendum selection widget accept clicked
                            sigc::connection connection_ballotOptionSelectionWidget_accept_clicked; ///< connection ballot option selection widget accept clicked
                            sigc::connection connection_ballotOptionSelectionWidget_back_clicked;   ///< connection ballot option selection widget back clicked
                            sigc::connection connection_ballotOptionSelectionWidget_clear_clicked; ///< connection ballot option selection widget clear clicked
                            sigc::connection connection_ballotOptionSelectionWidget_ballotOption_clicked;   ///< connection ballot option selection widget ballot option clicked
                            sigc::connection connectionShowFinalMessage;	///< Message describing the connection show final
                            sigc::connection connection_null_voteclicked;  ///< connection vote null clicked
                            sigc::connection connection_question;   ///< connection question

                            /**
                             * @fn	void VotingExperienceWidget::on_QuestionAnswered ( std::string code );
                             * @brief	question method: Handles question answered signals.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	code	The code.
                             */

                            void on_QuestionAnswered ( std::string code );

                            /**
                             * @fn	void VotingExperienceWidget::on_ContestNormalVote();
                             * @brief	null contest question method: Handles contest normal vote signals.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void on_ContestNormalVote();

                            /**
                             * @fn	void VotingExperienceWidget::on_ContestNullVote();
                             * @brief	null contest question method: Handles contest null vote signals.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void on_ContestNullVote();

                            /**
                             * @fn	void VotingExperienceWidget::startVoting();
                             * @brief	Start voting experience.
                             * @author	Yadickson Soto
                             * @date	16/04/2012
                             */
                            void startVoting();

                        protected:

                            Smartmatic::SAES::Voting::Election::ElectionWrapper * electionWrapper;  ///< election wrapper
                            Smartmatic::SAES::Operation::Voting::VotingExperienceController * votingExperienceController;	///< voting experience controller
                            Smartmatic::SAES::GUI::Operation::Voting::VotingWidgetManager * votingWidgetManager; ///< Manager for voting widget

                            Smartmatic::SAES::GUI::Widgets::Voting::CardRemovedWidget * cardRemovedWidget;	///< card removed widget
                            Smartmatic::SAES::GUI::Widgets::Voting::PrintTimeOutWidget * printTimeOutWidget; ///< print time out widget

                            Smartmatic::SAES::GUI::Widgets::ReferendumSelectionWidget * referendumSelectionWidget; ///< referendum selection widget
                            Smartmatic::SAES::GUI::Widgets::PartySelectionWidget * partySelectionWidget; ///< party selection widget
                            Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * ballotOptionSelectionWidget;  ///< ballot option selection widget
                            Smartmatic::SAES::GUI::Widgets::Voting::ICustomAttributeSelectionWidget * customAttrWidget;	///< custom attribute widget
                            Smartmatic::SAES::GUI::Widgets::Voting::InterruptWidget * interruptWidget; ///< interrupt widget
                            Smartmatic::SAES::GUI::Widgets::Voting::EndVotingSequenceWidget * endVotingSequenceWidget; ///< end voting sequence widget

                            Smartmatic::SAES::Voting::Election::Contest * actualContest;  ///< actual contest
                            std::list<Smartmatic::SAES::Voting::Election::Contest * > contestList; ///< List of contests
                            Smartmatic::SAES::Voting::Election::Party actualParty;  ///< actual party
                            Smartmatic::SAES::Voting::Election::Party previousParty;	///< previous party
                            Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition currentcustomAttribute; ///< current custom attribute

                            Glib::Dispatcher signalDispatcherShowLoadingBallot; ///< The signal show loading ballot
                            /**
                             * @fn	virtual void refresh();
                             * @brief	refresh widget method.
                             * @author	Yadickson Soto
                             * @date	15/11/2012
                             */
                            virtual void refresh();

                            /**
                             * @fn	void VotingExperienceWidget::GetCurrentContestReources ( Glib::ustring & contestClassName,
                             * 			Glib::ustring & contestClassDescription, Glib::ustring & contestTitle,
                             * 			Glib::ustring & contestAbstract, Glib::ustring & contestHelp );
                             * @brief	core logic method: Gets the current contest reources.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	contestClassName			 	Name of the contest class.
                             * @param	contestClassDescription	Information describing the contest class.
                             * @param	contestTitle					 	The contest title.
                             * @param	contestAbstract				 	The contest abstract.
                             * @param	contestHelp						 	The contest help.
                             */

                            void GetCurrentContestReources ( Glib::ustring & contestClassName,
                                                             Glib::ustring & contestClassDescription,
                                                             Glib::ustring & contestTitle,
                                                             Glib::ustring & contestAbstract,
                                                             Glib::ustring & contestHelp );

                            /**
                             * @fn	void VotingExperienceWidget::GetContestList();
                             * @brief	core logic method: Gets the contest list.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void GetContestList();

                            /**
                             * @fn	void VotingExperienceWidget::GetNextContest();
                             * @brief	core logic method: Gets the next contest.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void GetNextContest();

                            /**
                             * @fn	void VotingExperienceWidget::GetPreviousContest();
                             * @brief	core logic method: Gets the previous contest.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void GetPreviousContest();

                            /**
                             * @fn	void VotingExperienceWidget::GetPartySelection();
                             * @brief	core logic method: Gets the party selection.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void GetPartySelection();

                            /**
                             * @fn	void VotingExperienceWidget::GetBallotOptionSelection();
                             * @brief	core logic method: Gets the ballot option selection.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void GetBallotOptionSelection();

                            /**
                             * @fn	void VotingExperienceWidget::GetAndShowPartySelection();
                             * @brief	core logic method: Gets and shows the party selection.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void GetAndShowPartySelection();

                            /**
                             * @fn	void VotingExperienceWidget::GetReferendumOrPartySelection();
                             * @brief	core logic method: Gets referendum or party selection.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void GetReferendumOrPartySelection();

                            /**
                             * @fn	void VotingExperienceWidget::GetAndShowReferendum();
                             * @brief	core logic method: Gets and shows the referendum.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */
                            void GetAndShowReferendum();

                            /**
                             * @fn	void VotingExperienceWidget::GetReferendum();
                             * @brief	core logic method: Gets the referendum.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */
                            void GetReferendum();

                            /**
                             * @fn	void VotingExperienceWidget::ShowReferendum();
                             * @brief	core logic method: Shows the referendum.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */
                            void ShowReferendum();

                            /**
                             * @fn	void VotingExperienceWidget::ConstructAndShowBallotOptionSelection();
                             * @brief	core logic method: Construct and shows ballot option selection.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ConstructAndShowBallotOptionSelection();

                            /**
                             * @fn	void VotingExperienceWidget::SetActualContestFromIndex ( int index );
                             * @brief	core logic method: Sets an actual contest from index.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @param	index	Zero-based index of the.
                             */

                            void SetActualContestFromIndex ( int index );

                            /**
                             * @fn	void VotingExperienceWidget::CreateQuestionDialog();
                             * @brief	core logic method: Creates the question dialog.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void CreateQuestionDialog();

                            /**
                             * @fn	void VotingExperienceWidget::ShowPartySelection();
                             * @brief	Shows the party selection.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ShowPartySelection();

                            /**
                             * @fn	void VotingExperienceWidget::ShowBallotOptionSelection();
                             * @brief	Shows the ballot option selection.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ShowBallotOptionSelection();

                            /**
                             * @fn	void VotingExperienceWidget::ShowWelcomeContestMessage();
                             * @brief	Shows the welcome contest message.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ShowWelcomeContestMessage();

                            /**
                             * @fn	void VotingExperienceWidget::ShowTimeOutEndVotingExperience();
                             * @brief	Shows the time out end voting experience.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ShowTimeOutEndVotingExperience();
                            /**
                             * @fn	void VotingExperienceWidget::showStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget );
                             * @brief	method view step widget.
                             * @author	Yadickson Soto
                             * @date	17/04/2012
                             * @param	widget	step widget.
                             */
                            void showStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget );

                            /**
                             * @fn	void VotingExperienceWidget::ShowLoadingBallot();
                             * @brief	Shows the loading ballot.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ShowLoadingBallot();

                            /**
                             * @fn	void VotingExperienceWidget::onShowLoadingBallot();
                             * @brief	Shows the loading ballot.
                             * @author	Yadickson Soto
                             * @date	10/09/2013
                             */
                            void onShowLoadingBallot();

                            /**
                             * @fn	void VotingExperienceWidget::ShowErrorChecksum();
                             * @brief	Shows the error checksum.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ShowErrorChecksum();

                            /**
                             * @fn	void VotingExperienceWidget::ShowErrorChecksumInternal();
                             * @brief	Shows the error checksum internal.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void ShowErrorChecksumInternal();

                            /**
                             * @fn	void VotingExperienceWidget::PerformQuestionAction();
                             * @brief	voting window logic: Perform question action.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void PerformQuestionAction();

                            /**
                             * @fn	void VotingExperienceWidget::PerformBackStep();
                             * @brief	voting window logic: Perform back step.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void PerformBackStep();

                            /**
                             * @fn	void VotingExperienceWidget::PerformBackStepInternal();
                             * @brief	voting window logic: Perform back step internal.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void PerformBackStepInternal();

                            /**
                             * @fn	virtual void VotingExperienceWidget::PerformNextStep();
                             * @brief	voting window logic: Perform next step.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            virtual void PerformNextStep();

                            /**
                             * @fn	void VotingExperienceWidget::PerformNextStepInternal();
                             * @brief	voting window logic: Perform next step internal.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void PerformNextStepInternal();

                            /**
                             * @fn	void VotingExperienceWidget::PerformTimeOutEndVotingExperience();
                             * @brief	voting window logic: Perform time out end voting experience.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void PerformTimeOutEndVotingExperience();

                            sigc::connection connAlarmAndBack;  ///< connection alarm and back
                            sigc::connection connAlarmNotRemoveCard;	///< connection alarm not remove card
                            sigc::connection possibleWelcomeTimeout;	///< possible welcome timeout

                            bool IsInInterrupt; ///< true if is in interrupt
                            bool InvalidCard;   ///< true to invalid card
                            unsigned int extractionCounter; ///< extraction counter
                            bool detectCardExtractionEnable;	///< true if detect card extraction enable
                            bool isBallotOptionSelection;	///< true if showing ballot option selection
                            bool simulationMode; ///< true to enable simulation mode, false to disable it
                            /**
                             * @fn	bool VotingExperienceWidget::CardIsValid();
                             * @brief	card detection: queries if the card is valid.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             * @return	true if it succeeds, false if it fails.
                             */

                            bool CardIsValid();

                            /**
                             * @fn	void VotingExperienceWidget::AlarmExtractionCounter ();
                             * @brief	card detection: Alarm and back.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void AlarmExtractionCounter ();

                            /**
                             * @fn	void VotingExperienceWidget::RefreshByTooManyExtracts ();
                             * @brief	card detection: Refresh by too many extracts.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void RefreshByTooManyExtracts ();

                            /**
                             * @fn	void stopByExtractProcedure ( );
                             * @brief	card detection: stop by extract procedure.
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             */
                            void stopByExtractProcedure ( );

                            /**
                             * @fn	void VotingExperienceWidget::endVoting();
                             * @brief	force end voting.
                             * @author	Youssef.tourkmani
                             * @date	17/04/2012
                             * @date	Yadickson Soto.
                             */
                            void endVoting();

                            /**
                             * @fn	void VotingExperienceWidget::errorVoting();
                             * @brief	error voting.
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             * @date	Yadickson Soto.
                             */
                            void errorVoting();

                            /**
                             * @fn	void VotingExperienceWidget::nullVoting();
                             * @brief	null voting.
                             * @author	Yadickson Soto
                             * @date	03/04/2013
                             * @date	Yadickson Soto.
                             */
                            void nullVoting();

                            /**
                             * @fn	void VotingExperienceWidget::SetLabelsText();
                             * @brief	mode of visual theme: Sets the labels text.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void SetLabelsText();

                            bool success;   ///< true if the operation was a success, false if it failed
                            bool alreadyInTransition;   ///< true if already in transition

                            sigc::connection connRefresh;   ///< connection refresh

                            int timeConnector;  ///< time connector
                            unsigned int timeConnectorLoading;  ///< time connector loading
                            int currentContestIndex;	///< current contest index
                            int totalContests;  ///< The total contests
                            bool showBackInParty;   ///< true if show, false if hide the back in party
                            int windowWidth;	///< Width of the window
                            int windowHeight;   ///< Height of the window
                            unsigned int loadingBallotsNumber;  ///< loading ballots number
                            bool showPrintImage;  ///< show print image

                            std::list<Smartmatic::SAES::Voting::Election::Contest * >::iterator  iterContest;  ///< iterator contest
                            Glib::Thread * thread;  ///< The thread
                            Glib::ustring s_CustomAttributeQuestionFormat;  ///< custom attribute question format
                            Glib::ustring s_PartyHeaderFormat;  ///< party header format
                            Glib::ustring s_PartyHeaderFormatNoQuestion;  ///< party header format no question
                            Glib::ustring s_PartyFooterFormat;  ///< party footer format
                            Glib::ustring s_FooterPartySelection;   ///<  party selection footer
                            Glib::ustring s_ContestWelcomeFormat;   ///< contest welcome format
                            Glib::ustring s_BallotHeaderFormat; ///< ballot header format
                            Glib::ustring s_BallotHeaderFormatNoQuestion; ///< ballot header format no question
                            Glib::ustring s_BallotFooterFormat; ///< ballot footer format
                            Glib::ustring s_FooterBallotSelection;  ///< footer ballot selection
                            Glib::ustring s_ConfirmSelectionMessage;	///< confirm selection message
                            Glib::ustring s_ConfirmSelectionMessageHeader;  ///< confirm selection message header
                            Glib::ustring s_ConfirmSelectionMessageFooter;  ///< confirm selection message footer
                            Glib::ustring s_ConfirmSelectionMessageBackButton;  ///< confirm selection message back button
                            Glib::ustring s_ConfirmSelectionMessageAcceptButton;	///< confirm selection message accept button
                            Glib::ustring s_TimeOutVotingExperience;	///< time out voting experience
                            Glib::ustring s_PrinterErrorOnVote; ///< printer error on vote
                            Glib::ustring s_TimeOutEndVotingExperience; ///< time out end voting experience
                            Glib::ustring s_endVotExpLabel; ///< end voting experience label
                            Glib::ustring s_finalVotExpLabel; ///< final voting experience label
                            Glib::ustring s_timeOutEndVotExpLabel;  ///< time out end voting experience label
                            Glib::ustring s_PrinterErrorOnSmartcardRemoved; ///< printer error on smartcard removed
                            Glib::ustring s_LoadingBallotOption;	///< loading ballot option
                            Glib::ustring s_checksumLabel;  ///< checksum label
                            Glib::ustring s_referendumHeaderFormat; ///< referendum header format
                            Glib::ustring s_referendumHeaderFormatNoQuestion; ///< referendum header format no question
                            Glib::ustring s_referendumFooterFormat; ///< referendum footer format
                            Glib::ustring s_errorWritingCard; ///< referendum footer format

                            VotingExperienceStepType currentStep;

                            /**
                             * @fn	void VotingExperienceWidget::CreatePixBufFromBitmapAndSaveIt();
                             * @brief	Creates the pix buffer from bitmap and save iterator.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void CreatePixBufFromBitmapAndSaveIt();

                            /**
                             * @fn	void VotingExperienceWidget::SendPixBufToVote();
                             * @brief	Sends the pix buffer to vote.
                             * @author	Youssef.tourkmani
                             * @date	01/11/2012
                             */

                            void SendPixBufToVote();

                            /**
                             * @fn	void VotingExperienceWidget::endVotingSequence();
                             * @brief	End voting sequence
                             * @author	Yadickson Soto
                             * @date	02/04/2013
                             */
                            void endVotingSequence();

                            /**
                             * @brief method to set the current step of the voting experience widget.
                             * @author gabriel.jaimes
                             * @date 01/06/2022
                             * @param step  The next step of the voting experience controller.
                             */
                            void setCurrentStep(Smartmatic::SAES::Operation::Voting::VotingExperienceController::VotingExperienceController::VotingExperienceType step);

                            Glib::Mutex syncCard; ///< The synchronise card

                            Smartmatic::GUI::Widgets::StepWidget * currentWidget; ///< current widget
                            static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                    };
                }
            }
        }
    }
}

#endif // _VOTING_EXPERIENCE_WIDGET_H_
