/**
 * @file WaitingForCardWidget.hxx
 * @brief Header of the class of the container widget which is at the beginning of the voting experience.
 * Shows the voting instructions to the elector at the beginning of the voting experience.
 * @date 11/04/2012
 * @author Yadickson Soto
 */

#ifndef WAITINGFORCARDWIDGET_HXX_
#define WAITINGFORCARDWIDGET_HXX_

#include <Gui/Widgets/ProgressWidget.hxx>
#include <Widgets/StepWidget.hxx>
#include <Widgets/SmartmaticBasicFrame.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    /**
                     * @class	WaitingForCardWidget
                     * @brief	This is the container widget which is at the beginning of the voting experience.
                     * Shows the voting instructions to the elector at the beginning of the voting experience.
                     * @author	Yadickson Soto
                     * @date	11/04/2012
                     */
                    class WaitingForCardWidget : public Smartmatic::GUI::Widgets::StepWidget
                    {
                        public:

                            /**
                             * @fn	WaitingForCardWidget::WaitingForCardWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress );
                             * @brief	Class constructor.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             * @param	progress	progress widget.
                             */
                            explicit WaitingForCardWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress );
                            /**
                             * @fn	virtual WaitingForCardWidget::~WaitingForCardWidget();
                             * @brief	Class destructor.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            virtual ~WaitingForCardWidget();
                            /**
                             * @fn	void WaitingForCardWidget::viewOnlyCard();
                             * @brief	view only card.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            void viewOnlyCard();
                            /**
                             * @fn	void WaitingForCardWidget::viewCardAndBarcode();
                             * @brief	view card and barcode.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            void viewCardAndBarcode();
                            /**
                             * @fn	void WaitingForCardWidget::viewCardAndNotBarcode();
                             * @brief	view error card and barcode.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            void viewCardAndNotBarcode();
                            /**
                             * @fn	void WaitingForCardWidget::viewReconfigureBarcode();
                             * @brief	view reconfigure barcode.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            void viewReconfigureBarcode();
                            /**
                             * @fn	void WaitingForCardWidget::viewCardValidate();
                             * @brief	view card validate.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            void viewCardValidate();
                            /**
                             * @fn	void WaitingForCardWidget::viewErrorCardValidate();
                             * @brief	view error card validate.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            void viewErrorCardValidate();
                            /**
                             * @fn	void WaitingForCardWidget::viewErrorCardAID();
                             * @brief	view error card validate.
                             * @author	Yadickson Soto
                             * @date	07/03/2013
                             */
                            void viewErrorCardAID();
                            /**
                             * @fn	void WaitingForCardWidget::viewErrorCardPIN();
                             * @brief	view error card validate.
                             * @author	Yadickson Soto
                             * @date	07/03/2013
                             */
                            void viewErrorCardPIN();
                            /**
                             * @fn	void WaitingForCardWidget::viewErrorCardGeneric();
                             * @brief	view error card validate.
                             * @author	Yadickson Soto
                             * @date	07/03/2013
                             */
                            void viewErrorCardGeneric();
                            /**
                             * @fn	void WaitingForCardWidget::viewErrorCardUsed();
                             * @brief	view error card used.
                             * @author	Yadickson Soto
                             * @date	09/01/2013
                             */
                            void viewErrorCardUsed();
                            /**
                             * @fn	void WaitingForCardWidget::update();
                             * @brief	refresh text.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            void update();
                            /**
                             * @fn	virtual void WaitingForCardWidget::refresh();
                             * @brief	refresh widget method.
                             * @author	Yadickson Soto
                             * @date	11/04/2012
                             */
                            virtual void refresh();
                            /**
                             * @fn	virtual void WaitingForCardWidget::forceNotChange();
                             * @brief	method force not change.
                             * @author	Yadickson Soto
                             * @date	21/03/2013
                             */
                            void forceNotChange();

                        private:

                            /**
                             * @enum	WaitingState
                             * @brief	state widget.
                             */
                            enum WaitingState
                            {
                                ONLY_CARD ///< state only card
                                , CARD_AND_BARCODE ///< state card and barcode
                                , CARD_AND_NOT_BARCODE ///< state card and not barcode
                                , RECONFIGURE_BARCODE ///< reconfigure barcode
                                , CARD_VALIDATE ///< card validate
                                , ERROR_CARD ///< error card validate
                                , ERROR_CARD_AID ///< error card validate
                                , ERROR_CARD_PIN ///< error card validate
                                , ERROR_CARD_GENERIC ///< error card validate
                                , ERROR_CARD_USED ///< error card used
                            };

                            bool flagNotChange; ///< flag screen generation
                            WaitingState state; ///< state widget
                            Smartmatic::SAES::GUI::Widgets::ProgressWidget * progress; ///< image frame
                            static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                            std::string insertCardMessage;
                    };
                }
            }
        }
    }
}

#endif /* WAITINGFORCARDWIDGET_HXX_ */
