/**
 * @file	VotingInfoWidget.cxx
 * @brief	Declares the ballot option info widget class, which gives "how to vote" instructions.
 * @author	Yadickson Soto
 * @date	11/03/2014
 */

#include "VotingInfoWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;

VotingInfoWidget::VotingInfoWidget ( unsigned int minimum, unsigned int maximum )
    : SmartmaticBasicVBox (), minimum ( minimum ), maximum ( maximum )
{
    instructionTitle = manage ( new SmartmaticBasicLabel () );
    instructionInfo = manage ( new SmartmaticBasicLabel () );
    footerInfo = manage ( new SmartmaticBasicLabel () );
    viewFooter = true;

    LabelConfiguration titleConfig = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getInstructionTitle() );
    LabelConfiguration infoConfig = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getInstructionInfo() );
    LabelConfiguration footerConfig = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getFooterInfo() );

    LabelConfiguration::SetLabelConfiguration ( instructionTitle, titleConfig );
    LabelConfiguration::SetLabelConfiguration ( instructionInfo, infoConfig );
    LabelConfiguration::SetLabelConfiguration ( footerInfo, footerConfig );

    pack_start ( *instructionTitle, false, false, 5 );
    pack_start ( *instructionInfo );
    pack_end ( *footerInfo, false, false, 5 );

    instructionTitle->setAccessibleName ( "InstructionTitle" );
    instructionInfo->setAccessibleName ( "InstructionInfo" );
    footerInfo->setAccessibleName ( "VotingFooterInformation" );

    setAccessibleName ( "VotingInfoWidget" );

    show_all_children();

    set_border_width ( SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration().getBorderInfo() );
}

VotingInfoWidget::~VotingInfoWidget()
{

}

void VotingInfoWidget::refresh()
{
    instructionTitle->set_text ( Glib::ustring::compose ( _ ( "Instruction Voting Title" ) , minimum, maximum ) );
    instructionInfo->set_text ( Glib::ustring::compose ( _ ( "Instruction Voting Information" ) , minimum, maximum ) );
    footerInfo->set_text ( viewFooter ? Glib::ustring::compose ( _ ( "Voting Footer Information" ) , minimum, maximum ) : " " );
}

void VotingInfoWidget::showFooterInfo ( bool show )
{
    viewFooter = show;
}
