/**
 * @file	VotingInfoWidget.hxx
 * @brief	Declares the ballot option info widget class, which gives "how to vote" instructions.
 * @author	Yadickson Soto
 * @date	11/03/2014
 */

#ifndef _VOTING_INFO_WIDGET_H_
#define _VOTING_INFO_WIDGET_H_

#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/RefreshListener.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	VotingInfoWidget
                 * @brief	Ballot option widget.
                 * @author	Yadickson Soto
                 * @date	11/03/2014
                 */

                class VotingInfoWidget: public Smartmatic::GUI::Widgets::SmartmaticBasicVBox, Smartmatic::GUI::Widgets::RefreshListener
                {
                    public:
                        /**
                         * @fn	VotingInfoWidget ();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	11/03/2014
                         * @param	minimum The minimum selection
                         * @param	maximum The maximum selection
                         */
                        VotingInfoWidget ( unsigned int minimum, unsigned int maximum );

                        /**
                         * @fn	virtual ~VotingInfoWidget();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	11/03/2014
                         */
                        virtual ~VotingInfoWidget();

                        /**
                         * @fn	void refresh();
                         * @brief	Refresh labels texts.
                         * @author	Yadickson Soto
                         * @date	11/03/2014
                         */
                        void refresh();

                        /**
                         * @fn	void showFooterInfo( bool show = true );
                         * @brief	Show footer label information
                         * @author	Yadickson Soto
                         * @date	03/04/2014
                         * @param	show if true, show footer label information
                         */
                        void showFooterInfo ( bool show = true );

                    protected:

                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * instructionTitle; ///< The instruction title
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * instructionInfo; ///< The instruction information
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * footerInfo; ///< The footer information
                        unsigned int minimum; ///< The minimum selection
                        unsigned int maximum; ///< The maximum selection
                        bool viewFooter; ///< View footer information
                };
            }
        }
    }
}

#endif // _VOTING_INFO_WIDGET_H_
