/**
 * @file WizardConfiguration.h
 * @brief Header wizard widget configuration class
 * @date 30/08/2013
 * @author Yadickson Soto
 */

#ifndef WIZARDWIDGETCONFIGURATION_H_
#define WIZARDWIDGETCONFIGURATION_H_

#include <vector>
#include <Gui/Widgets/Menu/menu-option.h>
#include "Widgets/smartmatic-button.h"
#include "Configuration/label-configuration.h"

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {

                /**
                 * @class	WizardConfiguration
                 * @brief	Wizard widget configuration.
                 * @author	Youssef.tourkmani
                 * @date	25/10/2012
                 */
                class WizardConfiguration
                {
                    public:

                        /**
                         * @fn	WizardConfiguration::WizardConfiguration();
                         * @brief	Default constructor.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         */
                        WizardConfiguration();

                        /**
                         * @fn	virtual WizardConfiguration::~WizardConfiguration();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         */
                        virtual ~WizardConfiguration();

                        /**
                         * @fn	static std::vector<MenuOption> WizardConfiguration::GetCurrentToolbox(std::vector<WizardConfiguration> configuration,
                         * 			std::string currentState);
                         * @brief	Gets current toolbox.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	configuration	The configuration.
                         * @param	currentState 	The current state.
                         * @return	The current toolbox.
                         */
                        static std::vector<MenuOption> GetCurrentToolbox ( std::vector<WizardConfiguration> configuration, std::string currentState );

                        /**
                         * @fn	static std::vector<MenuOption> WizardConfiguration::GetCurrentPrincipaButtonBox(std::vector<WizardConfiguration> configuration,
                         * 			std::string currentState);
                         * @brief	Gets current principal button box.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	configuration	The configuration.
                         * @param	currentState 	The current state.
                         * @return	The current principa button box.
                         */
                        static std::vector<MenuOption> GetCurrentPrincipaButtonBox ( std::vector<WizardConfiguration> configuration, std::string currentState );

                        /**
                         * @fn	static WizardConfiguration WizardConfiguration::GetWizardWidgetConfigurationByState(std::vector<WizardConfiguration> configuration,
                         * 			std::string currentState);
                         * @brief	Gets wizard widget configuration by state.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	configuration	The configuration.
                         * @param	currentState 	The current state.
                         * @return	The wizard widget configuration by state.
                         */
                        static WizardConfiguration GetWizardWidgetConfigurationByState ( std::vector<WizardConfiguration> configuration, std::string currentState );

                        /**
                         * @fn	bool WizardConfiguration::isState( const std::string& value );
                         * @brief	Query if 'value' is state.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         * @return	true if state, false if not.
                         */
                        bool isState ( const std::string& value );

                        /**
                         * @fn	std::vector<MenuOption> WizardConfiguration::getToolbox();
                         * @brief	Gets the toolbox.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The toolbox.
                         */
                        std::vector<MenuOption>  getToolbox();

                        /**
                         * @fn	std::vector<MenuOption> WizardConfiguration::getPrincipalOptions();
                         * @brief	Gets principal options.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The principal options.
                         */
                        std::vector<MenuOption> getPrincipalOptions();

                        /**
                         * @fn	std::string WizardConfiguration::getStateEnum();
                         * @brief	Gets state enum.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The state enum.
                         */
                        std::string getStateEnum();

                        /**
                         * @fn	std::string WizardConfiguration::getStateName();
                         * @brief	Gets state name.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The state name.
                         */
                        std::string getStateName();

                        /**
                         * @fn	std::string WizardConfiguration::getStateDescription();
                         * @brief	Gets state description.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The state description.
                         */
                        std::string getStateDescription();

                        /**
                         * @fn	std::string WizardConfiguration::getStateHelpTitleKey();
                         * @brief	Gets state help title key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The state help title key.
                         */
                        std::string getStateHelpTitleKey();

                        /**
                         * @fn	std::string WizardConfiguration::getStateHelpContentKey();
                         * @brief	Gets state help content key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The state help content key.
                         */
                        std::string getStateHelpContentKey();

                        /**
                         * @fn	std::string WizardConfiguration::getStateHelpRelativeImagePath();
                         * @brief	Gets state help relative image path.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The state help relative image path.
                         */
                        std::string getStateHelpRelativeImagePath();

                        /**
                         * @fn	std::string WizardConfiguration::getTimelineNameKey();
                         * @brief	Gets timeline name key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The timeline name key.
                         */
                        std::string getTimelineNameKey();

                        /**
                         * @fn	std::string WizardConfiguration::getTimelineDescriptionKey();
                         * @brief	Gets timeline description key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The timeline description key.
                         */
                        std::string getTimelineDescriptionKey();

                        /**
                         * @fn	std::vector < std::string > & getTimelineRelativeImagesPath();
                         * @brief	Gets the get timeline relative images path.
                         * @author	Yadickson Soto
                         * @date	07/01/2013
                         * @return	list images path
                         */
                        std::vector < std::string > & getTimelineRelativeImagesPath();

                        /**
                         * @fn	std::vector < std::string > & getTimelineCodes();
                         * @brief	Gets the get timeline codes by images
                         * @author	Yadickson Soto
                         * @date	26/04/2013
                         * @return	list codes
                         */
                        std::vector < std::string > & getTimelineCodes();

                        /**
                         * @fn	int WizardConfiguration::getToolboxElementWidth();
                         * @brief	Gets toolbox element width.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The toolbox element width.
                         */
                        int getToolboxElementWidth();

                        /**
                         * @fn	int WizardConfiguration::getToolboxElementHeight();
                         * @brief	Gets toolbox element height.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The toolbox element height.
                         */
                        int getToolboxElementHeight();

                        /**
                         * @fn	Smartmatic::GUI::Configuration::LabelConfiguration WizardConfiguration::getButtonsLabelConfiguration();
                         * @brief	Gets buttons label configuration.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The buttons label configuration.
                         */
                        Smartmatic::GUI::Configuration::LabelConfiguration getButtonsLabelConfiguration();

                        /**
                         * @fn	Smartmatic::GUI::Configuration::LabelConfiguration WizardConfiguration::getTitleHelpLabelConfiguration();
                         * @brief	Gets title help label configuration.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The title help label configuration.
                         */
                        Smartmatic::GUI::Configuration::LabelConfiguration getTitleHelpLabelConfiguration();

                        /**
                         * @fn	Smartmatic::GUI::Configuration::LabelConfiguration WizardConfiguration::getHelpLabelConfiguration();
                         * @brief	Gets help label configuration.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The help label configuration.
                         */
                        Smartmatic::GUI::Configuration::LabelConfiguration getHelpLabelConfiguration();

                        /**
                         * @fn	std::string WizardConfiguration::getHelpTitle();
                         * @brief	Gets help title.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The help title.
                         */
                        std::string getHelpTitle();

                        /**
                         * @fn	std::string WizardConfiguration::getHelpContent();
                         * @brief	Gets help content.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The help content.
                         */
                        std::string getHelpContent();

                        /**
                         * @fn	std::string getHelpTitleAccessibleName();
                         * @brief	Method getter help title accessible name.
                         * @author	Yadickson Soto
                         * @date	29/04/2013
                         * @return	accessible name.
                         */
                        std::string getHelpTitleAccessibleName();

                        /**
                         * @fn	std::string getHelpTitleAccessibleName();
                         * @brief	Method getter help content accessible name.
                         * @author	Yadickson Soto
                         * @date	29/04/2013
                         * @return	accessible name.
                         */
                        std::string getHelpContentAccessibleName();

                        /**
                         * @fn	void WizardConfiguration::setToolbox( const std::vector<MenuOption>& value );
                         * @brief	Sets a toolbox.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setToolbox ( const std::vector<MenuOption>& value );

                        /**
                         * @fn	void WizardConfiguration::setPrincipalOptions( const std::vector<MenuOption>& value );
                         * @brief	Sets principal options.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setPrincipalOptions ( const std::vector<MenuOption>& value );

                        /**
                         * @fn	void WizardConfiguration::setStateEnum( const std::string& value );
                         * @brief	Sets state enum.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setStateEnum ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setStateName( const std::string& value );
                         * @brief	Sets state name.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setStateName ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setStateDescription( const std::string& value );
                         * @brief	Sets state description.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setStateDescription ( const std::string& value );

                        /**
                         * @fn	void setAccessibleName( const std::string & value );
                         * @brief	Sets state accessible name.
                         * @author	Yadickson Soto
                         * @date	26/04/2013
                         * @param	value for accessible name
                         */
                        void setAccessibleName ( const std::string & value );

                        /**
                         * @fn	std::string getAccessibleName ();
                         * @brief	Getter state accessible name.
                         * @author	Yadickson Soto
                         * @date	26/04/2013
                         * @return	accessible name value
                         */
                        std::string getAccessibleName ();

                        /**
                         * @fn	void setTimelineAccessibleName( const std::string & value );
                         * @brief	Sets time line accessible name.
                         * @author	Yadickson Soto
                         * @date	26/04/2013
                         * @param	value for timeline accessible name
                         */
                        void setTimelineAccessibleName ( const std::string & value );

                        /**
                         * @fn	std::string getTimelineAccessibleName ();
                         * @brief	Getter time line accessible name.
                         * @author	Yadickson Soto
                         * @date	26/04/2013
                         * @return	timeline accessible name value
                         */
                        std::string getTimelineAccessibleName ();

                        /**
                         * @fn	void WizardConfiguration::setStateHelpTitleKey( const std::string& value );
                         * @brief	Sets state help title key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setStateHelpTitleKey ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setStateHelpContentKey( const std::string& value );
                         * @brief	Sets state help content key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setStateHelpContentKey ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setStateHelpRelativeImagePath( const std::string& value );
                         * @brief	Sets state help relative image path.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setStateHelpRelativeImagePath ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setTimelineNameKey( const std::string& value );
                         * @brief	Sets timeline name key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setTimelineNameKey ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setTimelineDescriptionKey( const std::string& value );
                         * @brief	Sets timeline description key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setTimelineDescriptionKey ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::addTimelineRelativeImagePath( const std::string & value, const std::string & code );
                         * @brief	Add timeline relative image path.
                         * @author	Yadickson Soto
                         * @date	07/01/2013
                         * @param	value	New relative image path
                         * @param	code code for accessible name
                         */
                        void addTimelineRelativeImagePath ( const std::string & value, const std::string & code );

                        /**
                         * @fn	void WizardConfiguration::setToolboxElementWidth(int value);
                         * @brief	Sets toolbox element width.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setToolboxElementWidth ( int value );

                        /**
                         * @fn	void WizardConfiguration::setToolboxElementHeight(int value);
                         * @brief	Sets toolbox element height.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setToolboxElementHeight ( int value );

                        /**
                         * @fn	int WizardConfiguration::getEnumIntState();
                         * @brief	Gets enum int state.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The enum int state.
                         */
                        int getEnumIntState();

                        /**
                         * @fn	void WizardConfiguration::setEnumIntState(int value);
                         * @brief	Sets enum int state.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setEnumIntState ( int value );

                        /**
                         * @fn	bool WizardConfiguration::isEnumIntState(int value);
                         * @brief	Query if 'value' is enum int state.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         * @return	true if enum int state, false if not.
                         */
                        bool isEnumIntState ( int value );

                        /**
                         * @fn	void WizardConfiguration::setButtonsLabelConfiguration(Smartmatic::GUI::Configuration::LabelConfiguration value);
                         * @brief	Sets buttons label configuration.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setButtonsLabelConfiguration ( Smartmatic::GUI::Configuration::LabelConfiguration value );

                        /**
                         * @fn	void WizardConfiguration::setTitleHelpLabelConfiguration(Smartmatic::GUI::Configuration::LabelConfiguration value);
                         * @brief	Sets title help label configuration.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setTitleHelpLabelConfiguration ( Smartmatic::GUI::Configuration::LabelConfiguration value );

                        /**
                         * @fn	void WizardConfiguration::setHelpLabelConfiguration(Smartmatic::GUI::Configuration::LabelConfiguration value);
                         * @brief	Sets help label configuration.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setHelpLabelConfiguration ( Smartmatic::GUI::Configuration::LabelConfiguration value );

                        /**
                         * @fn	std::string WizardConfiguration::getImageHelp();
                         * @brief	Gets image help.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The image help.
                         */
                        std::string getImageHelp();

                        /**
                         * @fn	std::string WizardConfiguration::getPdfHelp();
                         * @brief	Gets PDF help.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The PDF help.
                         */
                        std::string getPdfHelp();

                        /**
                         * @fn	std::string WizardConfiguration::getTitleKey();
                         * @brief	Gets title key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	The title key.
                         */
                        std::string getTitleKey();

                        /**
                         * @fn	bool WizardConfiguration::getUseHelp();
                         * @brief	Gets use help.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool getUseHelp();

                        /**
                         * @fn	void WizardConfiguration::setImageHelp( const std::string& value );
                         * @brief	Sets image help.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setImageHelp ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setPdfHelp( const std::string& value );
                         * @brief	Sets PDF help.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setPdfHelp ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setTitleKey( const std::string& value);
                         * @brief	Sets title key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void setTitleKey ( const std::string& value );

                        /**
                         * @fn	void WizardConfiguration::setUseHelp(bool value);
                         * @brief	Sets use help.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	true to value.
                         */
                        void setUseHelp ( bool value );

                        /**
                         * @fn	void setEnableFilePrinting ( bool enable );
                         * @brief	Sets enable file printing button.
                         * @author	Yadickson Soto
                         * @date	07/02/2014
                         * @param	enable	true to enable file printing button.
                         */
                        void setEnableFilePrinting ( bool enable );

                        /**
                         * @fn	bool getEnableFilePrinting ();
                         * @brief	Gets enable file printing button.
                         * @author	Yadickson Soto
                         * @date	07/02/2014
                         * @return	true if enable file printing button.
                         */
                        bool getEnableFilePrinting ();

                    private:
                        std::vector<MenuOption> options;	///< Options for controlling the operation
                        std::vector<MenuOption> toolkit;	///< The toolkit

                        std::string stateEnum;	///< The state enum
                        std::string state_Name; ///< Name of the state
                        std::string state_Description;	///< Information describing the state

                        std::string stateHelp_TitleKey; ///< The state help title key
                        std::string stateHelp_ContentKey; ///< The state help content key
                        std::string stateHelp_RelativeImagePath;	///< Full pathname of the state help relative image file

                        std::string timeline_NameKey; ///< The timeline name key
                        std::string timeline_DescriptionKey;	///< The timeline description key
                        std::vector < std::string > timelineRelativeImagesPath; ///< Full pathname of the timeline relative image file
                        std::vector < std::string > timelineCodes; ///< codes for accessible name

                        std::string helpWindow_ImageHelp; ///< The help window image help
                        std::string helpWindow_PdfHelp; ///< The help window PDF help
                        std::string helpWindow_TitleKey;	///< The help window title key
                        bool helpWindow_UseHelp;	///< true to help window use help

                        int toolboxElementWidth;	///< Width of the toolbox element
                        int toolboxElementHeight; ///< Height of the toolbox element

                        int enumIntState; ///< State of the enum int
                        bool enableFilePrinting; ///< enable printing button

                        std::string stateAccesibleName; ///< state accessible name
                        std::string timelineAccesibleName; ///< time line accessible name

                        Smartmatic::GUI::Configuration::LabelConfiguration buttonsLabelConfiguration; ///< The buttons label configuration
                        Smartmatic::GUI::Configuration::LabelConfiguration titleHelpLabelConfiguration; ///< The title help label configuration
                        Smartmatic::GUI::Configuration::LabelConfiguration helpLabelConfiguration;	///< The help label configuration
                };
            }
        }
    }
}

#endif /* WIZARDWIDGETCONFIGURATION_H_ */
