/**
 * @file	ballot-option-widget.cc
 * @brief	ballot option widget class.
 * @author	Yadickson Soto
 * @date	26/02/2014
 */

#include "ballot-option-widget.h"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>

using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Runtime::Environment;

BallotOptionWidget::BallotOptionWidget (
    const Eligible_option & elegibleOption, SaesDirectories & saesDir, bool showNumber, bool enableClick, bool showBorder, IContestLayout * layout )
    : BallotOptionInfoWidget ( elegibleOption, saesDir, enableClick, showBorder )
{
    votingWidget = NULL;

    if ( displayConfig.getType() != DisplayType::CONFIGURATION_DUAL_TEXT && displayConfig.getType() != DisplayType::CONFIGURATION_TEXT )
    {
        votingWidget = manage ( new VotingWidgetBase ( displayConfig, imagePath, labelNumber, paddingConfig, numberWidth, 100, 10, showNumber ) );
    }
    else
    {
        const bool  useFullname  = labelFirstName.empty() && labelLastName.empty();
        std::string topString    = useFullname ? labelFullName : labelFirstName;
        std::string bottomString = useFullname ? "" : labelLastName;

        votingWidget = manage ( new VotingWidgetBase (
          topString, layout->getLastnameAlignment(),
          bottomString, layout->getFirstnameAlignment(),
          labelNumber, paddingConfig, numberWidth, 100, 10, showNumber ) );
    }

    addWidget ( votingWidget );
    show_all_children();
}

void BallotOptionWidget::setSize ( int width, int height )
{
    BoxSelectionWidget::setSize ( width, height );
    votingWidget->setSize ( width, height );
    votingWidget->show();
}

BallotOptionWidget::~BallotOptionWidget()
{
}
