/**
 * @file	party-selection-widget.cc
 * @brief	party selection widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "party-selection-widget.h"
#include "System/Sounds/smartmatic-sounds-manager.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"
#include "Resources/election-string-resources-manager.h"
#include "Voting/Election/ElectionWrapper.hxx"
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Voting/VotingOperation.hxx>
#include <Operation/FormatLabel.hxx>
#include "FactoryMethodWidget.hxx"
#include <System/Utils/StringUtils.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Gtk;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::Log;

static const int ELEMENTS_HEADER = 2;

ISMTTLog * PartySelectionWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget", GETTEXT_PACKAGE );

PartySelectionWidget::PartySelectionWidget ( Contest * contest, bool viewBackButton )
{
    static PartySelectionConfiguration config = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration();

    currentBoxWidget = NULL;
    m_widgetEnabled = true;
    this->contest = contest;
    showBackButton = viewBackButton;
    contestName = contest->getDistrict().getName_key();

    ptrPartyWidget     = NULL;
    ptrPartyNullWidget = NULL;
    tableWidgets       = NULL;
    titleLabel         = NULL;
    subtitleLabel      = NULL;
    infoLabel          = NULL;

    m_isChildCustomAttribute = false;

    int x;

    SMTTGUIConfigurationManager::getInstance()->getPaddings ( x, padding );

    topHeight = config.getFrameTopHeight();
    infoHeight = config.getFrameInfoHeight();
    bottomHeight = config.getFrameBottomHeight();

    tableWidgets = manage ( new TableBoxSelectionWidget ( false, true, SAESGUIConfigurationManager::getInstance()->getVotingExperienceConfiguration().getPartySelection().getVCenter() ) );

    if ( !SAESGUIConfigurationManager::getInstance()->getVotingExperienceConfiguration().getPartySelection().getHCenter() )
    {
        tableWidgets->resize ( 1, SAESGUIConfigurationManager::getInstance()->getVotingExperienceConfiguration().getPartySelection().getColumns() );
    }

    flagShowNullButton = config.getShowNullButton();

    partyButtonWidget = FactoryMethodWidget::createPartyButtonWidget();
    partyButtonWidget->update();

    if ( SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getEnableClick() )
    {
        tableWidgets->getSignalClick().connect ( sigc::mem_fun ( this, &PartySelectionWidget::onBoxClicked ) );
        partyButtonWidget->getSignalBackButtonClicked().connect ( sigc::mem_fun ( this, &PartySelectionWidget::on_backButton_clicked ) );
        partyButtonWidget->getSignalNullButtonClicked().connect ( sigc::mem_fun ( this, &PartySelectionWidget::onAcceptClicked ) );
        partyButtonWidget->getSignalAcceptButtonClicked().connect ( sigc::mem_fun ( this, &PartySelectionWidget::onAcceptClicked ) );
        partyButtonWidget->getSignalClearButtonClicked().connect ( sigc::mem_fun ( this, &PartySelectionWidget::onClearClicked ) );
    }

    titleLabel = manage ( new SmartmaticBasicLabel() );
    titleLabel->show();
    titleLabel->set_size_request ( -1, topHeight/ELEMENTS_HEADER );

    subtitleLabel = manage(new SmartmaticBasicLabel());
    subtitleLabel->show();
    subtitleLabel->set_size_request(-1, topHeight/ELEMENTS_HEADER );

    infoLabel = manage ( new SmartmaticBasicLabel() );
    infoLabel->show();
    infoLabel->set_size_request ( -1, infoHeight );

    Smartmatic::GUI::Configuration::LabelConfiguration labelConf = getInfoLabelConfig();
    Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( infoLabel, labelConf );

    Smartmatic::GUI::Configuration::LabelConfiguration titleConfig = getTitleLabelConfig();
    Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( titleLabel, titleConfig );

    Smartmatic::GUI::Configuration::LabelConfiguration subtitleConfig = getSubtitleLabelConfig();
    Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( subtitleLabel, subtitleConfig );

    bottomFrame = manage ( new SmartmaticBasicFrame() );
    bottomFrame->set_shadow_type ( Gtk::SHADOW_NONE );
    bottomFrame->show();
    bottomFrame->add ( *partyButtonWidget );
    bottomFrame->set_size_request ( -1, bottomHeight );

    Gtk::HSeparator * separator = manage ( new Gtk::HSeparator() );
    separator->set_size_request ( -1, padding );

    externalBox = manage ( new SmartmaticBasicVBox ( false , 0 ) );
    externalBox->pack_start ( *titleLabel, false, false );
    externalBox->pack_start ( *subtitleLabel, false, false );
    externalBox->pack_start ( *separator, false, false );
    externalBox->pack_start ( *tableWidgets, true, true, padding );
    externalBox->pack_start ( *infoLabel, false, false );
    externalBox->pack_end ( *bottomFrame, false, false );

    partyButtonWidget->showButton ( false, false, showBackButton, flagShowNullButton );

    setWidget ( externalBox );
    setTitle ( false );

    setAccessibleName ( "PartySelection" );

    tableWidgets->setAccessibleName ( "PartySelectionMainTable" );
    titleLabel->setAccessibleName ( "PartySelectionTitle" );
    subtitleLabel->setAccessibleName ( "PartySelectionSubtitle" );
    infoLabel->setAccessibleName ( "PartySelectionInfo" );
    bottomFrame->setAccessibleName ( "PartySelectionBottom" );
    externalBox->setAccessibleName ( "PartySelectionContainer" );
}

PartySelectionWidget::~PartySelectionWidget()
{
    if ( partyButtonWidget )
    {
        delete partyButtonWidget;
        partyButtonWidget = NULL;
    }
}

void PartySelectionWidget::SetWidgetEnabled ( bool enabled )
{
    m_widgetEnabled = enabled;
}

void PartySelectionWidget::SetChildCustomAttribute ( bool value )
{
    m_isChildCustomAttribute = value;
}

void PartySelectionWidget::refresh()
{
    RefreshLabels();
    onClearClicked();
}

void PartySelectionWidget::ShowBackButton()
{
    showBackButton = true;
    onClearClicked();
}

std::string PartySelectionWidget::GetContestName()
{
    return ElectionStringResourcesManager::getInstance()->getResource ( contestName );
}

void PartySelectionWidget::HideBackButton()
{
    showBackButton = false;
    onClearClicked();
}

void PartySelectionWidget::SetBackButton ( bool show )
{
    showBackButton = show;
    onClearClicked();
}

std::string PartySelectionWidget::getDistrictDescription()
{
    const std::string descriptionKey = contest->getDistrict().getDescription_key();

    return ElectionStringResourcesManager::getInstance()->getResource(descriptionKey);
}

std::string PartySelectionWidget::getHelp()
{
    const std::string helpKey = contest->getHelp_key();

    return ElectionStringResourcesManager::getInstance()->getResource(helpKey);
}

void PartySelectionWidget::buildHeader()
{
  const Smartmatic::SAES::Voting::Election::Contest_class * contestClass =
     Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration()
    ->getContestClassByCode ( contest->getContest_class_code() );
  std::string contestClassName = ElectionStringResourcesManager::getInstance()->getResource(contestClass->getName_key());
  std::string subtitleValue    = "";

  FormatLabel::checkComposeString(contestClassName);
  titleLabel->set_text(contestClassName);

  if ( m_isChildCustomAttribute )
  {
    std::string contestHelp = getHelp();

    FormatLabel::checkComposeString(contestHelp);
    logger->Debug ("Showing Help Key");

    if ( StringUtils::toLowerUtf8(contestClassName) != StringUtils::toLowerUtf8(contestHelp) && !contestHelp.empty() )
      subtitleValue = contestHelp;
  }
  else
  {
    std::string districtDescription = getDistrictDescription();

    FormatLabel::checkComposeString(districtDescription);
    logger->Debug ("Showing District Description");

    if ( StringUtils::toLowerUtf8(contestClassName) != StringUtils::toLowerUtf8(districtDescription) && !districtDescription.empty() )
      subtitleValue = districtDescription;
  }

  subtitleLabel->set_text(subtitleValue);
}

void PartySelectionWidget::RefreshLabels()
{
    buildHeader();
    infoLabel->set_text ( _ ( messageInfoLabel.c_str() ) );

    partyButtonWidget->update();

    if ( ptrPartyNullWidget )
    {
        ptrPartyNullWidget->RefreshLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.NullParty" ) );
    }
}

void PartySelectionWidget::Initialize ( std::list<Ballot_option * > & partyList,
                                        Smartmatic::SAES::Runtime::Environment::SaesDirectories & saesDir )
{
    static PartySelectionConfiguration config = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration();

    float imageRelation = config.getImageRelation();
    int nCol = config.getSizeToColumns();
    int rowSpacing = config.getPartiesMargin();
    int colSpacing = config.getPartiesMargin();
    int rowsToFill = 0;
    float numberW = SAESGUIConfigurationManager::getInstance()->GetPartyWidgetConfiguration().getNumberWidth();

    int totalW, totalH;

    SMTTGUIConfigurationManager::getInstance()->getCenterSize ( totalW, totalH );
    totalH -= topHeight + infoHeight + bottomHeight + 2 * rowSpacing + 4 * padding;
    totalH -= config.getPartyButtonsSpacing();
    totalW -= config.getFixBorder();

    //calculate columns of table
    float boxW = totalW / ( float ) nCol;
    float widgetW = boxW - colSpacing;
    float imageW = widgetW - numberW;
    float imageH = imageW / imageRelation;
    float widgetH = imageH;
    float boxH = imageH + rowSpacing;
    nCol = config.getColumns();
    int nRow = totalH / boxH;
    int totalBoxes = ( int ) partyList.size() + 1;
    bool useMaxPartyNumber = config.getUseMaxPartyNumerFromAllContest();

    if ( useMaxPartyNumber )
    {
        totalBoxes = Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration()->getMaxPartiesByAllContest();
        totalBoxes += 1;
    }

    if ( nRow * nCol < totalBoxes )
    {
        nRow = totalBoxes / nCol;
        float rest = totalBoxes % nCol;
        nRow = rest > 0.0 ? nRow + 1 : nRow;
        boxH = totalH / ( float ) nRow;
        widgetH = boxH - rowSpacing;
        //imageH = widgetH;
        //imageW = imageH * imageRelation;
        //boxW = imageW + numberW;
    }

    rowsToFill = totalBoxes / nCol;
    float rem = totalBoxes % nCol;
    rowsToFill = rem > 0.0 ? rowsToFill + 1 : rowsToFill;

    tableWidgets->setSpacings ( rowSpacing, colSpacing );

    BoxSelectionWidget * box = NULL;
    static bool enabledClick = SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getEnableClick();
    static bool showBorder = SAESGUIConfigurationManager::getInstance()->GetPartyWidgetConfiguration().getShowFrame();
    static Gdk::Color selectedColor = SAESGUIConfigurationManager::ConvertToGdkColor ( SAESGUIConfigurationManager::getInstance()->GetPartyWidgetConfiguration().getSelectedColor() );
    static Gdk::Color unSelectedColor = SAESGUIConfigurationManager::ConvertToGdkColor ( SAESGUIConfigurationManager::getInstance()->GetPartyWidgetConfiguration().getUnSelectedColor() );

    int i = 0;
    int j = 0;

    std::list<Smartmatic::SAES::Voting::Election::Ballot_option * >::iterator it;  ///< The iterator

    int count;

    for ( count = 1, it = partyList.begin(); it != partyList.end(); ++it, count++ )
    {
        ptrPartyWidget = manage ( new PartyWidget ( * ( *it ), saesDir, widgetW, widgetH ) );
        ptrPartyWidget->setAccessibleName ( "PartyBoxContainer" + StringUtils::intToString ( count ) );

        box = manage ( new BoxSelectionWidget ( ( *it )->getCode(), enabledClick, showBorder ) );
        ptrPartyWidget->setAccessibleName ( "PartyBox" + StringUtils::intToString ( count ) );

        box->setSelectedColor ( selectedColor );
        box->setUnselectedColor ( unSelectedColor );
        box->addWidget ( ptrPartyWidget );
        box->set_name ( "voting_experience_party_widget" );
        box->setName ( "party_widget_event_box" );

        ptrPartyWidget->show();
        bool fillByRows = SAESGUIConfigurationManager::getInstance()->GetPartyWidgetConfiguration().getUseFillByRows();
        tableWidgets->put ( box, i, j );

        if ( fillByRows )
        {
            j++;

            if ( j == nCol )
            {
                j = 0;
                i++;
            }
        }
        else
        {
            i++;

            if ( i == rowsToFill )
            {
                i = 0;
                j++;
            }

            if ( j == nCol )
            {
                j = 0;
            }
        }
    }

    if ( config.getShowNullParty() )
    {
        ptrPartyNullWidget = manage ( new PartyWidget ( _ ( "Smartmatic.SAES.GUI.Widgets.PartySelectionWidget.NullParty" ), widgetW, widgetH ) );
        ptrPartyNullWidget->setAccessibleName ( "NullPartyBoxContainer" );

        box = manage ( new BoxSelectionWidget ( "", enabledClick, showBorder ) );
        box->setAccessibleName ( "NullPartyBox" );
        box->setSelectedColor ( selectedColor );
        box->setUnselectedColor ( unSelectedColor );
        box->addWidget ( ptrPartyNullWidget );
        box->set_name ( "voting_experience_party_widget" );
        box->setName ( "party_widget_event_box" );

        ptrPartyNullWidget->show();
        tableWidgets->put ( box, i, j );
    }
}

void PartySelectionWidget::on_backButton_clicked()
{
    if ( m_widgetEnabled == false )
    {
        return;
    }

    if ( m_back_clicked.empty() == false )
    {
        m_widgetEnabled = false;
    }

    m_back_clicked.emit();
    partyButtonWidget->clearSignals();
}

PartySelectionWidget::type_null_clicked PartySelectionWidget::null_clicked()
{
    return m_null_clicked;
}

PartySelectionWidget::type_back_clicked PartySelectionWidget::back_clicked()
{
    return m_back_clicked;
}

PartySelectionWidget::type_party_clicked PartySelectionWidget::party_clicked()
{
    return m_party_clicked;
}

Smartmatic::SAES::Voting::Election::Contest * PartySelectionWidget::getContest()
{
    return contest;
}

Smartmatic::GUI::Configuration::LabelConfiguration PartySelectionWidget::getTitleLabelConfig()
{
    return SAESGUIConfigurationManager::getInstance()->GetPartySelectionWindowTitleLabelConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration PartySelectionWidget::getSubtitleLabelConfig()
{
    return SAESGUIConfigurationManager::getInstance()->GetPartySelectionWindowSubtitleLabelConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration PartySelectionWidget::getInfoLabelConfig()
{
    return SAESGUIConfigurationManager::getInstance()->GetPartySelectionWindowBottomLabelConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration PartySelectionWidget::getBottomLabelConfig()
{
    return SAESGUIConfigurationManager::getInstance()->GetPartySelectionWindowBottomLabelConfiguration();
}

std::string PartySelectionWidget::getBottomText()
{
    std::string bottom;

    switch ( VotingOperation::getVoteMode() )
    {
        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_S:
        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_O:
        default:
            bottom = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyFooterFormat" );
            break;

        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_D:
            bottom = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyFooterFormat.Demo" );
            break;
    }

    return bottom;
}

void PartySelectionWidget::onAcceptClicked()
{
    std::string code;

    if ( currentBoxWidget )
    {
        code = currentBoxWidget->getCode();
    }

    if ( !code.empty() )
    {
        Smartmatic::SAES::Voting::Election::Ballot_option * ballotSelected =
            Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration()->getBallotOptionListFromBallotOptionCode ( contest->getCode(), code );

        Smartmatic::SAES::Voting::Election::Party partySelected = ballotSelected->getParty();

        m_party_clicked.emit ( partySelected );
    }
    else
    {
        m_null_clicked.emit();
    }
}

void PartySelectionWidget::onClearClicked()
{
    currentBoxWidget = NULL;
    messageInfoLabel = getBottomText();
    infoLabel->set_text ( _ ( messageInfoLabel.c_str() ) );
    partyButtonWidget->showButton ( false, false, showBackButton, flagShowNullButton );
    tableWidgets->clear();
    partyButtonWidget->clearSignals();
}

void PartySelectionWidget::onBoxClicked ( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list )
{
    if ( list.empty() )
    {
        onClearClicked();
        return;
    }
    else
    {
        currentBoxWidget = ( * ( list.begin() ) );
        partyButtonWidget->showButton ( true, true, false, false );
    }

    if ( currentBoxWidget->getCode().empty() )
    {
        if ( SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getAutoSelectNullParty() )
        {
            partyButtonWidget->showButton ( false, false, showBackButton, flagShowNullButton );
            onAcceptClicked();
        }
        else
        {
            messageInfoLabel = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.NullPartyInfoMessage" );
            infoLabel->set_text ( _ ( messageInfoLabel.c_str() ) );
            partyButtonWidget->showNullInfo();
        }
    }
    else
    {
        if ( SAESGUIConfigurationManager::getInstance()->GetPartySelectionConfiguration().getAutoSelectParty() )
        {
            partyButtonWidget->showButton ( false, false, showBackButton, flagShowNullButton );
            onAcceptClicked();
        }
        else
        {
            messageInfoLabel = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PartyInfoMessage" );
            infoLabel->set_text ( _ ( messageInfoLabel.c_str() ) );
            partyButtonWidget->showNormalInfo();
        }
    }
}
