/**
 * @file	party-selection-widget.h
 * @brief	Declares the party selection widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _PARTY_SELECTION_WIDGET_H_
#define _PARTY_SELECTION_WIDGET_H_

#include <gtkmm.h>
#include "Configuration/SaesGUIConfiguration-schema.hxx"
#include "Voting/Election/election-schema.hxx"
#include "Gui/Widgets/party-widget.h"
#include "Widgets/smartmatic-button.h"
#include "Operation/Voting/VotingExperienceController.hxx"
#include "Runtime/Environment/saes-directories.h"
#include "Gui/Widgets/voting-frame-base.h"
#include <Voting/Vote/vote-schema.hxx>
#include "PartyButtonWidget.hxx"
#include <Widgets/TableBoxSelectionWidget.hxx>
#include <Widgets/SmartmaticBasicFrame.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	PartySelectionWidget
                 * @brief	Party selection widget.
                 * @author	Cesar.bonilla
                 * @date	4/29/2011
                 */

                class PartySelectionWidget: public VotingFrameBase
                {
                    public:

                        /**
                         * @fn	PartySelectionWidget::PartySelectionWidget ( Smartmatic::SAES::Voting::Election::Contest * contest,
                         * 			bool viewBackButton = false );
                         * @brief	class constructor.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	contest					contest that hold parties to be displayed.
                         * @param	viewBackButton	flag view back button.
                         */

                        PartySelectionWidget ( Smartmatic::SAES::Voting::Election::Contest * contest,
                                               bool viewBackButton = false );

                        /** @brief class destructor */

                        /**
                         * @fn	virtual PartySelectionWidget::~PartySelectionWidget();
                         * @brief	Finaliser.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        virtual ~PartySelectionWidget();

                        /**
                         * @fn	void PartySelectionWidget::Initialize ( std::list<Smartmatic::SAES::Voting::Election::Ballot_option>& partyList,
                         * 			Smartmatic::SAES::Runtime::Environment::SaesDirectories & saesDir );
                         * @brief	calculate position and size of parties widgets.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	partyList	List of parties.
                         * @param	saesDir	 	The saes dir.
                         */

                        void Initialize ( std::list<Smartmatic::SAES::Voting::Election::Ballot_option * > & partyList,
                                          Smartmatic::SAES::Runtime::Environment::SaesDirectories & saesDir );

                        /**
                         * @fn	void PartySelectionWidget::ShowBackButton();
                         * @brief	Shows the back button.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void ShowBackButton();

                        /**
                         * @fn	void PartySelectionWidget::HideBackButton();
                         * @brief	Hides the back button.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void HideBackButton();

                        /**
                         * @fn	virtual void PartySelectionWidget::SetBackButton ( bool show );
                         * @brief	Sets a back button.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @param	show	true to show, false to hide.
                         */

                        virtual void SetBackButton ( bool show );
                        /**
                         * @fn	virtual void PartySelectionWidget::refresh();
                         * @brief	refresh widget method.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         */
                        virtual void refresh();

                        /**
                         * @fn	std::string PartySelectionWidget::GetContestName();
                         * @brief	search the contest name in the contest object.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @return	name of the contest.
                         */

                        std::string GetContestName();

                        /**
                         * @fn	void PartySelectionWidget::RefreshLabels();
                         * @brief	Refresh labels.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void RefreshLabels();

                        /**
                         * @fn	Smartmatic::SAES::Voting::Election::Contest PartySelectionWidget::getContest();
                         * @brief	get contest.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @return	The contest.
                         */

                        Smartmatic::SAES::Voting::Election::Contest * getContest();

                        /**
                         * @typedef	sigc::signal<void> type_null_clicked
                         * @brief	signals definitions.
                         */

                        typedef sigc::signal<void> type_null_clicked;

                        /**
                         * @typedef	sigc::signal<void> type_back_clicked
                         * @brief	Defines an alias representing the type back clicked. .
                         */

                        typedef sigc::signal<void> type_back_clicked;

                        /**
                         * @typedef	sigc::signal<void, Smartmatic::SAES::Voting::Election::Party &> type_party_clicked
                         * @brief	Defines an alias representing the type party clicked. .
                         */

                        typedef sigc::signal<void, Smartmatic::SAES::Voting::Election::Party &> type_party_clicked;

                        /**
                         * @fn	type_null_clicked PartySelectionWidget::null_clicked();
                         * @brief	signals connect methods.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @return	.
                         */

                        type_null_clicked null_clicked();

                        /**
                         * @fn	type_back_clicked PartySelectionWidget::back_clicked();
                         * @brief	Gets the back clicked.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @return	.
                         */

                        type_back_clicked back_clicked();

                        /**
                         * @fn	type_party_clicked PartySelectionWidget::party_clicked();
                         * @brief	Gets the party clicked.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         * @return	.
                         */

                        type_party_clicked party_clicked();


                        /**
                         * @fn	virtual void PartySelectionWidget::SetWidgetEnabled ( bool enabled = true );
                         * @brief	Sets widget enabled.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	enabled	(optional) the enabled.
                         */
                        virtual void SetWidgetEnabled ( bool enabled = true );

                        /**
                         * @fn	virtual void PartySelectionWidget::SetChildCustomAttribute ( bool enabled = false );
                         * @brief	Sets whether the parent object is a custom attribute widget.
                         * @author	gabriel.jaimes
                         * @date	01/06/2022
                         * @param	enabled	(optional) the enabled.
                         */
                        virtual void SetChildCustomAttribute ( bool enabled = false );

                    protected:
                        /**
                         * @brief signals declaration
                         */
                        type_null_clicked m_null_clicked;   ///< The null clicked
                        type_back_clicked m_back_clicked; ///< The back clicked
                        type_party_clicked m_party_clicked; ///< The party clicked

                        //Glib::Mutex		m_signalMutex;
                        bool m_widgetEnabled; ///< true to enable, false to disable the widget
                        bool m_isChildCustomAttribute; ///< true if the parent object is a custom attribute widget, false otherwise

                        /**
                         * @fn	void PartySelectionWidget::onBoxClicked( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list);
                         * @brief	method box click.
                         * @author	Yadickson Soto
                         * @date	14/11/2012
                         * @param list box selected
                         */
                        void onBoxClicked ( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list );

                        /**
                         * @fn	void PartySelectionWidget::on_backButton_clicked();
                         * @brief	Handles back button clicked signals.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void on_backButton_clicked();
                        /**
                         * @fn	void PartySelectionWidget::onAcceptClicked();
                         * @brief	method accept clicked.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         */
                        void onAcceptClicked();
                        /**
                         * @fn	void PartySelectionWidget::onClearClicked();
                         * @brief	method clear clicked.
                         * @author	Yadickson Soto
                         * @date	22/10/2012
                         */
                        void onClearClicked();

                        /**
                         * @fn	virtual Smartmatic::GUI::Configuration::LabelConfiguration PartySelectionWidget::getTitleLabelConfig();
                         * @brief	virtual method getter label config.
                         * @author	Gabriel Jaimes
                         * @date	27/05/2022
                         * @return	title label config.
                         */
                        virtual Smartmatic::GUI::Configuration::LabelConfiguration getTitleLabelConfig();

                        /**
                         * @fn	virtual Smartmatic::GUI::Configuration::LabelConfiguration PartySelectionWidget::getSubtitleLabelConfig();
                         * @brief	virtual method getter label config.
                         * @author	Gabriel Jaimes
                         * @date	27/05/2022
                         * @return	subtitle label config.
                         */
                        virtual Smartmatic::GUI::Configuration::LabelConfiguration getSubtitleLabelConfig();

                        /**
                         * @fn	virtual Smartmatic::GUI::Configuration::LabelConfiguration PartySelectionWidget::getInfoLabelConfig();
                         * @brief	virtual method getter label config.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @return	bottom label config.
                         */
                        virtual Smartmatic::GUI::Configuration::LabelConfiguration getInfoLabelConfig();
                        /**
                         * @fn	virtual Smartmatic::GUI::Configuration::LabelConfiguration PartySelectionWidget::getBottomLabelConfig();
                         * @brief	virtual method getter label config.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @return	bottom label config.
                         */
                        virtual Smartmatic::GUI::Configuration::LabelConfiguration getBottomLabelConfig();
                        /**
                         * @fn	virtual std::string PartySelectionWidget::getBottomText();
                         * @brief	virtual method getter info.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @return	info text.
                         */
                        virtual std::string getBottomText();
                        /**
                         * @fn	virtual std::string getHelp();
                         * @brief	virtual method getter help information.
                         * @author	Gabriel Jaimes
                         * @date	27/05/2022
                         * @return	subtitle label config.
                         */
                        virtual std::string getHelp();
                        /**
                         * @fn	virtual std::string getDistrictDescription();
                         * @brief	virtual method getter district description information.
                         * @author	Gabriel Jaimes
                         * @date	27/05/2022
                         * @return	subtitle label config.
                         */
                        virtual std::string getDistrictDescription();
                        /**
                         * @fn	virtual virtual void buildHeader();
                         * @brief	build header information.
                         * @author	Gabriel Jaimes
                         * @date	27/05/2022
                         * @return	subtitle label config.
                         */
                        virtual void buildHeader();

                    private:

                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox * externalBox; ///< The external box
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * titleLabel;	///< The title label
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * subtitleLabel;	///< The title label
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * infoLabel;	///< The header label
                        Smartmatic::GUI::Widgets::SmartmaticBasicFrame * bottomFrame;	///< The bottom frame
                        PartyButtonWidget * partyButtonWidget; ///< The party button widget
                        PartyWidget * ptrPartyWidget;	///< The pointer party widget
                        PartyWidget * ptrPartyNullWidget;	///< The pointer party widget
                        bool flagShowNullButton; ///< Flag show null button
                        bool showBackButton; ///< flag show back button

                        int padding; ///< padding between
                        int topHeight;   ///< The top height
                        int infoHeight;   ///< The info height
                        int bottomHeight;   ///< The bottom height

                        std::string messageInfoLabel; ///< message info label
                        Smartmatic::GUI::Widgets::TableBoxSelectionWidget * tableWidgets; ///< table widgets
                        Smartmatic::GUI::Widgets::BoxSelectionWidget * currentBoxWidget; ///< current box widgets

                        Smartmatic::SAES::Voting::Election::Contest * contest;	///< The contest
                        std::string contestName;	///< Name of the contest

                        Smartmatic::SAES::Voting::Vote::VoteMode votingMode; ///< The voting mode

                    private:

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif // _PARTY_SELECTION_WIDGET_H_
