/**
 * @file BarcodeReconfigureWindow.cxx
 * @brief Body class barcode reconfigure window
 * @date 19/10/2011
 * @author Yadickson Soto
 */

#include "BarcodeReconfigureWindow.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::Log;

BarcodeReconfigureWindow * BarcodeReconfigureWindow::instance = NULL;
ISMTTLog * BarcodeReconfigureWindow::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Windows.BarcodeReconfigureWindow", GETTEXT_PACKAGE );

BarcodeReconfigureWindow::BarcodeReconfigureWindow()
    : PopUpMessageWindow()
{
    widget = AbstractFactoryWidget::createBarcodeDetectionWidget();
    widget->getSignalEnd().connect ( sigc::mem_fun ( *this, &BarcodeReconfigureWindow::QuitDialog ) );

    visible = SAESFunctionalityConfigurationManager::getInstance()->getScanVoteConfiguration().getSendBarcodeConfigurationOnReconnect();

    widget->setTitle ( false );
    pushStepWidget ( widget );
}

BarcodeReconfigureWindow::~BarcodeReconfigureWindow()
{
    stop();
    delete widget;
}

void BarcodeReconfigureWindow::start()
{
    if ( visible )
    {
        logger->Debug ( "start" );

        signalChange.disconnect();
        signalChange = DeviceTypeMonitorPool::getInstance()->getDeviceChangedSignal().connect ( sigc::mem_fun ( *this, &BarcodeReconfigureWindow::deviceChange ) );
    }
}

void BarcodeReconfigureWindow::stop()
{
    logger->Debug ( "stop" );
    widget->stop ();

    signalChange.disconnect();
    QuitDialog ();
}

void BarcodeReconfigureWindow::deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor )
{
    if ( deviceTypeMonitor != NULL && deviceTypeMonitor->getType() == DeviceType::CONFIGURATION_BARCODE )
    {
        if ( deviceTypeMonitor->getNumberOfDevices() > 0 )
        {
            logger->Debug ( "Device connected" );
            widget->showReconfigure();
            widget->start ( 0 );
            ShowDialog();
        }
        else
        {
            logger->Debug ( "Device disconnected" );
        }
    }
}

BarcodeReconfigureWindow * BarcodeReconfigureWindow::getInstance()
{
    if ( !instance )
    {
        instance = new BarcodeReconfigureWindow();
    }

    return instance;
}
