/**
 * @file BarcodeReconfigureWindow.hxx
 * @brief Header class barcode reconfigure window
 * @date 24/10/2012
 * @author Yadickson Soto
 */

#ifndef BARCODE_RECONFIGURE_WINDOW_HXX_
#define BARCODE_RECONFIGURE_WINDOW_HXX_

#include <Windows/pop-up-message-window.h>
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Gui/Widgets/BarcodeDetectionWidget.h>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Windows
            {
                /**
                 * @class	BarcodeReconfigureWindow
                 * @brief	class barcode reconfigure window.
                 * @author	Yadickson Soto
                 * @date	24/10/2012
                 */
                class BarcodeReconfigureWindow : public Smartmatic::GUI::Windows::PopUpMessageWindow
                {
                    public:
                        /**
                         * @fn	virtual BarcodeReconfigureWindow::~BarcodeReconfigureWindow();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	24/10/2012
                         */
                        virtual ~BarcodeReconfigureWindow();
                        /**
                         * @fn	static BarcodeReconfigureWindow * BarcodeReconfigureWindow::getInstance();
                         * @brief	method getter instance class.
                         * @author	Yadickson Soto
                         * @date	24/10/2012
                         * @return	instance class.
                         */
                        static BarcodeReconfigureWindow * getInstance();
                        /**
                         * @fn	void BarcodeReconfigureWindow::start();
                         * @brief	method start.
                         * @author	Yadickson Soto
                         * @date	24/10/2012
                         */
                        void start();
                        /**
                         * @fn	void BarcodeReconfigureWindow::stop();
                         * @brief	method stop.
                         * @author	Yadickson Soto
                         * @date	24/10/2012
                         */
                        void stop();

                    protected:
                        /**
                         * @fn	BarcodeReconfigureWindow::BarcodeReconfigureWindow();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	24/10/2012
                         */
                        BarcodeReconfigureWindow();
                        /**
                         * @fn	void deviceChange( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                         * @brief	Occurs when the observer notify a change of the device.
                         * @author	Yadickson Soto
                         * @date	13/12/2012
                         * @param 	deviceTypeMonitor The pointer reference to device object.
                         */
                        void deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );

                    private:

                        static BarcodeReconfigureWindow * instance; ///< instance class

                        bool visible; ///< Whether the window should be shown on reconfigure barcode device if connected
                        sigc::connection signalChange; ///< The signal triggered when occurs changes of a device.
                        Smartmatic::SAES::GUI::Widgets::BarcodeDetectionWidget * widget; ///< progress widget
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /* BARCODE_RECONFIGURE_WINDOW_HXX_ */
