/**
 * @file DeviceMonitorWindow.hxx
 * @brief Declaration of the screen to view the devices information.
 * @date 13/12/2012
 * @author Yadickson Soto
 */

#ifndef DEVICE_MONITOR_WINDOW_HXX_
#define DEVICE_MONITOR_WINDOW_HXX_

#include <Windows/pop-up-message-window.h>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Gui/Widgets/DeviceMonitorWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Windows
            {
                /**
                 * @class	DeviceMonitorWindow
                 * @brief	Message screen for monitoring the devices information.
                 * @author	Yadickson Soto
                 * @date	13/12/2012
                 */
                class DeviceMonitorWindow : public Smartmatic::GUI::Windows::PopUpMessageWindow
                {
                    public:
                        /**
                         * @fn	virtual DeviceMonitorWindow::~DeviceMonitorWindow();
                         * @brief Finalizes the instance of the DeviceMonitorWindow class.
                         * @author	Yadickson Soto
                         * @date	13/12/2012
                         */
                        virtual ~DeviceMonitorWindow();
                        /**
                         * @fn	static DeviceMonitorWindow * DeviceMonitorWindow::getInstance();
                         * @brief	Gets the instance of the DeviceMonitorWindow singleton.
                         * @brief	method getter instance class.
                         * @author	Yadickson Soto
                         * @date	13/12/2012
                         * @return	The pointer reference to message window that provide access to the instance.
                         */
                        static DeviceMonitorWindow * getInstance();
                        /**
                         * @fn	void DeviceMonitorWindow::start();
                         * @brief	Starts the listen of the events associated at device monitor.
                         * @author	Yadickson Soto
                         * @date	13/12/2012
                         */
                        void start();
                        /**
                         * @fn	void DeviceMonitorWindow::stop();
                         * @brief	Stops the listen of the events associated at device monitor.
                         * @author	Yadickson Soto
                         * @date	13/12/2012
                         */
                        void stop();

                        /**
                         * @fn	virtual void QuitDialog();
                         * @brief	Method quit dialog.
                         * @author	Yadickson Soto
                         * @date	09/06/2015
                         */
                        virtual void QuitDialog();

                    protected:
                        /**
                         * @fn	DeviceMonitorWindow::DeviceMonitorWindow();
                         * @brief	Default constructor initializes an instance of the ICheckCondition class.
                         * @author	Yadickson Soto
                         * @date	13/12/2012
                         */
                        DeviceMonitorWindow();
                        /**
                         * @fn	void deviceChange( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                         * @brief	Occurs when the observer notify a change of the device.
                         * @author	Yadickson Soto
                         * @date	13/12/2012
                         * @param 	deviceTypeMonitor The pointer reference to device object.
                         */
                        void deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );

                    private:

                        static DeviceMonitorWindow * instance; ///< The pointer reference to message screen that provide access to the instance.
                        sigc::connection signalChange; ///< The signal triggered when occurs changes of a device.

                        bool visible; ///< Whether the window should be shown on device change event
                        int border; ///< The border width associated to internal widget.

                        int iconWidth; ///< The natural width associated to device icon.
                        int iconHeight; ///< The natural height associated to device icon.
                        int timeOut; ///< Timeout configured that triggers the close the screen.

                        std::list <Smartmatic::SAES::GUI::Widgets::DeviceMonitorWidget * > devices; ///< The collection the device widgets.
                        Gtk::VBox * widget; ///< The pointer reference to internal widget which contains a collection the device widgets.
                        static Smartmatic::Log::ISMTTLog * logger; ///< The reference pointer of a logger instance.
                };
            }
        }
    }
}

#endif /* DEVICE_MONITOR_WINDOW_HXX_ */
