/**
 * @file MemoryMessageWindow.hxx
 * @brief Header class show memory message on block window
 * @date 09/06/2015
 * @author Yadickson Soto
 */

#ifndef MEMORY_MESSAGE_WINDOW_HXX_
#define MEMORY_MESSAGE_WINDOW_HXX_

#include <Windows/pop-up-message-window.h>
#include <Operation/MachineOperationManager.hxx>
#include <Gui/Widgets/MemoryStateWidget.hxx>
#include <Operation/MemoryEventManager.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Windows
            {
                /**
                 * @class	MemoryMessageWindow
                 * @brief	class barcode reconfigure window.
                 * @author	Yadickson Soto
                 * @date	09/06/2015
                 */
                class MemoryMessageWindow : public Smartmatic::GUI::Windows::PopUpMessageWindow
                {
                    public:
                        /**
                         * @fn	virtual ~MemoryMessageWindow();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	09/06/2015
                         */
                        virtual ~MemoryMessageWindow();
                        /**
                         * @fn	static MemoryMessageWindow * getInstance();
                         * @brief	method getter instance class.
                         * @author	Yadickson Soto
                         * @date	09/06/2015
                         * @return	instance class.
                         */
                        static MemoryMessageWindow * getInstance();
                        /**
                         * @fn	void start();
                         * @brief	method start.
                         * @author	Yadickson Soto
                         * @date	09/06/2015
                         */
                        void start();
                        /**
                         * @fn	void stop();
                         * @brief	method stop.
                         * @author	Yadickson Soto
                         * @date	09/06/2015
                         */
                        void stop();
                        /**
                         * @fn	sigc::signal<void> getSignalBlock();
                         * @brief	method getter signal block.
                         * @author	Yadickson Soto
                         * @date	10/06/2015
                         * @return	signal.
                         */
                        sigc::signal<void> getSignalBlock();
                        /**
                         * @fn	sigc::signal<void> getSignalAllow();
                         * @brief	method getter signal allow.
                         * @author	Yadickson Soto
                         * @date	10/06/2015
                         * @return	signal.
                         */
                        sigc::signal<void> getSignalAllow();
                        /**
                         * @fn	sigc::signal<void> getSignalResume();
                         * @brief	method getter signal resume.
                         * @author	Yadickson Soto
                         * @date	10/06/2015
                         * @return	signal.
                         */
                        sigc::signal<void> getSignalResume();
                        /**
                         * @brief  Show memory check error pop up.
                         * @author manuel.arguelles
                         * @date   04/07/2017
                         */
                        void showMemoryCheckError();
                        /**
                         * @brief  Show no memory present error pop up.
                         * @author manuel.arguelles
                         * @date   04/07/2017
                         */
                         void showNoMemoryError();

                        /**
                         * @brief  Shows a message to reinsert the memory.
                         * @author manuel.arguelles
                         * @date   26/05/2022
                         */
                        void showReinsertOneMemory();

                        /**
                         * @brief  Shows a message when a memory is extracted.
                         * @author manuel.arguelles
                         * @date   20/07/2022
                         */
                        void showOneMemory();

                        /**
                         * @brief  Shows a message to reinsert both memories.
                         * @author manuel.arguelles
                         * @date   26/05/2022
                         */
                        void showReinsertBothMemories();

                    protected:
                        /**
                         * @fn	MemoryMessageWindow();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	09/06/2015
                         */
                        MemoryMessageWindow();
                        /**
                         * @fn	void onQuitDialog();
                         * @brief	Callback if cancel safe eject.
                         * @author	Yadickson Soto
                         * @date	09/06/2015
                         */
                        void onQuitDialog();
                        /**
                         * @fn	void deviceChange( Smartmatic::SAES::Operation::MemoryEventManager::MemoryEvent event );
                         * @brief	Occurs when the observer notify a change of the device.
                         * @author	Yadickson Soto
                         * @date	29/07/2015
                         * @param 	event The memory event status
                         */
                        void deviceChange ( Smartmatic::SAES::Operation::MemoryEventManager::MemoryEvent event );

                    private:
                        /**
                         * @fn void waitUntilMemoryDetection();
                         * @brief Waits until memory detection.
                         * @author eduardo.fernandez
                         * @date   24/01/2019
                         */
                        void waitUntilMemoryDetection();

                    private:

                        sigc::connection signalChange; ///< The signal triggered when occurs changes of a device.

                        sigc::signal<void> signalBlock; ///< signal block current widget
                        sigc::signal<void> signalAllow; ///< signal allow one memory
                        sigc::signal<void> signalResume; ///< signal resume current widget

                        static MemoryMessageWindow * instance; ///< instance class
                        Smartmatic::SAES::GUI::Widgets::MemoryStateWidget * widget; ///< message widget
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /* MEMORY_MESSAGE_WINDOW_HXX_ */
