/**
 * @file VoteProcessWindow.hxx
 * @brief Header class vote process window
 * @date 23/10/2012
 * @author Yadickson Soto
 */

#ifndef VOTE_PROCESS_WINDOW_HXX_
#define VOTE_PROCESS_WINDOW_HXX_

#include <Windows/pop-up-message-window.h>
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include <Operation/Voting/VotePersistence.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Windows
            {
                /**
                 * @class	VoteProcessWindow
                 * @brief	class vote process window.
                 * @author	Yadickson Soto
                 * @date	23/10/2012
                 */
                class VoteProcessWindow : public Smartmatic::GUI::Windows::PopUpMessageWindow
                {
                    public:
                        /**
                         * @fn	virtual VoteProcessWindow::~VoteProcessWindow();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        virtual ~VoteProcessWindow();
                        /**
                         * @fn	void VoteProcessWindow::showNormalMode();
                         * @brief	method show normal mode.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        void showNormalMode();
                        /**
                         * @fn	void VoteProcessWindow::showReduceMode();
                         * @brief	method show reduce mode.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        void showReduceMode();
                        /**
                         * @fn	static VoteProcessWindow * VoteProcessWindow::getInstance();
                         * @brief	method getter instance class.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         * @return	instance class.
                         */
                        static VoteProcessWindow * getInstance();
                        /**
                         * @fn	void VoteProcessWindow::start();
                         * @brief	method start.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        void start();
                        /**
                         * @fn	void VoteProcessWindow::stop();
                         * @brief	method stop.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        void stop();

                    protected:
                        /**
                         * @fn	VoteProcessWindow::VoteProcessWindow();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        VoteProcessWindow();
                        /**
                         * @fn	void VoteProcessWindow::setConfiguration ( Smartmatic::SAES::Configuration::ScanConsolidationConfiguration & config );
                         * @brief	method setter configuration window.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         * @param	config	new configuration.
                         */
                        void setConfiguration ( Smartmatic::SAES::Configuration::ScanConsolidationConfiguration & config );
                        /**
                         * @fn	void onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );
                         * @brief	Method getter status vote process.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         * @param	status	status vote process.
                         * @param	count 	count status.
                         */
                        void onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );

                    private:

                        static VoteProcessWindow * instance; ///< instance class
                        sigc::connection signalStatusVote;	///< The signal status vote

                        Gdk::Color scanFontVoteColor; ///< valid vote color
                        Gdk::Color scanFontDuplicateVoteColor; ///< duplicate vote color
                        Gdk::Color scanFontBadVoteColor; ///< bad vote color

                        Gdk::Color scanBoxVoteColor; ///< valid vote color
                        Gdk::Color scanBoxDuplicateVoteColor; ///< duplicate vote color
                        Gdk::Color scanBoxBadVoteColor; ///< bad vote color

                        bool showMessage; ///< flag show message
                        bool enabledChangeColor; ///< flag enabled change color

                        std::string messageVote; ///< message normal vote
                        std::string messageDuplicateVote; ///< message duplicate vote
                        std::string messageBadVote; ///< message bad vote

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /* VOTE_PROCESS_WINDOW_HXX_ */
