/**
 * @file AlarmMessage.cxx
 * @brief Body class alarm message text
 * @date 12/04/2012
 * @author Yadickson Soto
 */

#include "AlarmMessage.hxx"
#include <Operation/Devices/AlarmController.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/AlarmConfigurationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::Log;

ISMTTLog * AlarmMessage::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Operation.AlarmMessage", GETTEXT_PACKAGE );

AlarmMessage::AlarmMessage()
{
}

AlarmMessage::~AlarmMessage()
{
}

void AlarmMessage::alarm ( AlarmEnum alarmType )
{
    AlarmController::getInstance()->addAlarm ( alarmType );
    AlarmController::getInstance()->soundAlarm ( alarmType );

    if ( AlarmConfigurationManager::getInstance()->getCategoryOfAlarm ( alarmType ) != AlarmCategoryEnum::CONFIGURATION_NOTALARM )
    {
        std::string alarmMessage;

        switch ( alarmType )
        {
            case AlarmEnum::CONFIGURATION_PRINTERNOTREADYFORVOTE:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Widgets.WaitingCardWidget.PrinterError" );
                break;

            case AlarmEnum::CONFIGURATION_INVALIDVOTINGDATA:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Widgets.WaitingCardWidget.ErrorInfoVotingCardData" );
                break;

            case AlarmEnum::CONFIGURATION_INVALIDCONTENT:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Widgets.WaitingCardWidget.AlarmActivated" );
                break;

            case AlarmEnum::CONFIGURATION_CARDEXTRACTIONCOUNTER:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.SmartcardExtracted" );
                break;

            case AlarmEnum::CONFIGURATION_VOTEPRINTINGFAILURE:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PrinterErrorOnVote" );
                break;

            case AlarmEnum::CONFIGURATION_VOTECOMPLIANCEFAILURE:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.ComplianceFailureOnVote" );
                break;

            case AlarmEnum::CONFIGURATION_PRINTERNEOPAFTERVOTE:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PrinterNearEndOfPaperAfterVote" );
                break;

            case AlarmEnum::CONFIGURATION_CARDNOTREMOVEDAFTERVOTING:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.CardNotExtracted" );
                break;

            case AlarmEnum::CONFIGURATION_UNABLETOREAD:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Widgets.WaitingCardWidget.AlarmActivated" );
                break;

            case AlarmEnum::CONFIGURATION_CARDUSED:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Widgets.WaitingCardWidget.AlarmCardUsed" );
                break;

            case AlarmEnum::CONFIGURATION_CARDVALIDITYPERIODEXPIRED:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Widgets.WaitingCardWidget.CardValidityPeriodExpired" );
                break;

            case AlarmEnum::CONFIGURATION_UNABLETOWRITE:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Widgets.WaitingCardWidget.ErrorWriteCard" );
                break;

            case AlarmEnum::CONFIGURATION_TIMEOUT:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.TimeOutEndVotingExperience" );
                break;

            case AlarmEnum::CONFIGURATION_CRYPTOERROR:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.CryptoError" );
                break;

            case AlarmEnum::CONFIGURATION_UNEXPECTED:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.UnexpectedAlarm" );
                break;

            case AlarmEnum::CONFIGURATION_PRINTERNEOPBEFOREVOTE:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PrinterNearEndOfPaperBeforeVote" );
                break;

            case AlarmEnum::CONFIGURATION_ALARMBOXDISCONNECTED:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.AlarmBoxDisconnected" );
                break;

            case AlarmEnum::CONFIGURATION_ALARMBOXCONNECTED:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.AlarmBoxConnected" );
                break;

            case AlarmEnum::CONFIGURATION_CARDREADERUNRESPONSIVE:
                alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.CardReaderUnresponsive" );
                break;

            case AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOTCONFIGUREDFORAUDIO:
               alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.DetectedAudioCardButNotConfiguredForAudio" );
               break;

            case AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOABLEPRESENT:
               alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.DetectedAudioCardButNoAblePresent" );
               break;

            case AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOVOICESPRESENT:
               alarmMessage = N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.DetectedAudioCardButNoVoicesPresent" );
               break;

            default:
                alarmMessage = " ";
                break;
        }

        logger->Audit ( alarmMessage );
        AlarmController::getInstance()->setMessage ( alarmMessage );
    }
}
