/**
 * @file FormatLabel.hxx
 * @brief Header class format label text
 * @date 11/04/2012
 * @author Yadickson Soto
 */

#ifndef FORMATLABEL_HXX_
#define FORMATLABEL_HXX_

#include <string>
#include <glibmm.h>
#include <Voting/Election/election-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Operation
            {
                /**
                 * @class	FormatLabel
                 * @brief	class format label text.
                 * @author	Yadickson Soto
                 * @date	11/04/2012
                 */
                class FormatLabel
                {
                    protected:
                        /**
                         * @fn	FormatLabel::FormatLabel();
                         * @brief	class constructor.
                         * @author	Yadickson Soto
                         * @date	11/04/2012
                         */
                        FormatLabel();

                    public:
                        /**
                         * @fn	virtual FormatLabel::~FormatLabel();
                         * @brief	class destructor.
                         * @author	Yadickson Soto
                         * @date	11/04/2012
                         */
                        virtual ~FormatLabel();
                        /**
                         * @fn	static std::string FormatLabel::format ( std::string text, std::string sep = "\n" );
                         * @brief	method format text multi language.
                         * @author	Yadickson Soto
                         * @date	11/04/2012
                         * @param	text	text to format.
                         * @param 	sep separator
                         * @return	text format.
                         */
                        static std::string format ( std::string text, std::string sep = "\n" );
                        /**
                         * @fn	static void FormatLabel::checkComposeString ( std::string & stringToCheck );
                         * @brief	method check compose string.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param	stringToCheck	check and result text.
                         */
                        static void checkComposeString ( std::string & stringToCheck );

                        /**
                         * @fn	static Glib::ustring FormatLabel::contestText ( Smartmatic::SAES::Voting::Election::Contest & contest,
                         * 			Glib::ustring & format );
                         * @brief	Contest text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	contest	The contest.
                         * @param [in,out]	format 	Describes the format to use.
                         * @return	.
                         */
                        static Glib::ustring contestText ( Smartmatic::SAES::Voting::Election::Contest & contest, Glib::ustring & format );

                };
            }
        }
    }
}

#endif /* FORMATLABEL_HXX_ */
