/**
 * @file	GenerateXMLImage.h
 * @brief	Declares the generate XML image class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef GENERATEXMLIMAGE_H_
#define GENERATEXMLIMAGE_H_

#include <Operation/MachineOperationManager.hxx>
#include <Log/ISMTTLog.h>
#include <Voting/Screen/GeneratedImagesWrapper.hxx>
#include <Voting/Screen/generated-images-schema.hxx>
#include "Configuration/SaesGUIConfiguration-schema.hxx"
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include "Operation/Voting/VotingExperienceController.hxx"
#include <Voting/VotingDevice/VotingDeviceWrapper.hxx>
#include <Voting/Election/ElectionWrapper.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace GUI
        {
            namespace Operation
            {
                /**
                 * @class	GenerateXMLImage
                 * @brief	Generate XML image.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class GenerateXMLImage
                {
                    public:

                        /**
                         * @fn	GenerateXMLImage::GenerateXMLImage();
                         * @brief	Default constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        GenerateXMLImage();
                        /**
                         * @fn	virtual GenerateXMLImage::~GenerateXMLImage();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~GenerateXMLImage();

                        /**
                         * @fn	void GenerateXMLImage::init();
                         * @brief	method clear.
                         * @author	Yadickson Soto
                         * @date	24/05/2012
                         */
                        void init();
                        /**
                         * @fn	void GenerateXMLImage::persist ( std::string folderName );
                         * @brief	persist in folder.
                         * @author	Yadickson Soto
                         * @date	24/05/2012
                         * @param	folderName	folder name.
                         */
                        void persist ( std::string folderName );

                        /**
                         * @fn	void addImage ( std::string fileName, std::string language,
                         * 			std::string country, Smartmatic::SAES::Voting::Screen::BasicImageType type );
                         * @brief	Add image screen.
                         * @author	Yadickson Soto
                         * @date	20/03/2013
                         * @param	fileName	Filename of the file.
                         * @param	language	The language.
                         * @param	country 	The country.
                         * @param	type 		screen type
                         */
                        void addImage (
                            std::string fileName,
                            std::string language,
                            std::string country,
                            Smartmatic::SAES::Voting::Screen::BasicImageType type );

                        /**
                         * @fn	void addSelectionImage ( std::string fileName, std::string language,
                         * 			std::string country, Smartmatic::SAES::Voting::Screen::BasicImageType type, std::string selection );
                         * @brief	Adds to the language screen.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	fileName	Filename of the file.
                         * @param	language	The language.
                         * @param	country 	The country.
                         * @param 	type		screen type
                         * @param	selection 	selection
                         */
                        void addSelectionImage (
                            std::string fileName,
                            std::string language,
                            std::string country,
                            Smartmatic::SAES::Voting::Screen::BasicImageType type,
                            std::string selection );

                        /**
                         * @fn	void GenerateXMLImage::addCustomAttributes ( std::string fileName, std::string language,
                         * 			std::string country );
                         * @brief	Adds a custom attributes.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	fileName	Filename of the file.
                         * @param	language	The language.
                         * @param	country 	The country.
                         * @param	customAttribute	custom attribute
                         */
                        void addCustomAttributes ( std::string fileName, std::string language, std::string country, Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & customAttribute );

                        /**
                         * @fn	void GenerateXMLImage::addParty ( std::string fileName, std::string language,
                         * 			std::string country );
                         * @brief	Adds a party.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	fileName	Filename of the file.
                         * @param	language	The language.
                         * @param	country 	The country.
                         * @param	contest		contest
                         */
                        void addParty ( std::string fileName, std::string language, std::string country, Smartmatic::SAES::Voting::Election::Contest * contest );

                        /**
                         * @fn	void GenerateXMLImage::addBallot ( std::string fileName, std::string language,
                         * 			std::string country );
                         * @brief	Adds a ballot.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	fileName	Filename of the file.
                         * @param	language	The language.
                         * @param	country 	The country.
                         * @param	party	party
                         */
                        void addBallot ( std::string fileName, std::string language, std::string country, Smartmatic::SAES::Voting::Election::Party * party );

                    private:

                        /**
                         * @fn	void GenerateXMLImage::dispose();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	05/11/2012
                         */
                        void dispose();

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;  ///< The electoral configuration
                        Smartmatic::SAES::Operation::MachineOperationManager * machineOperationManager; ///< Manager for machine operation
                        Smartmatic::SAES::Voting::Screen::GeneratedImagesWrapper * imagesWrapper;	///< Images wrapper
                        Smartmatic::SAES::Configuration::CustomAtributtesWindowConfiguration questionConfig;	///< The question configuration
                        Smartmatic::SAES::Configuration::VotingExperienceValidationScreensFunctionalityConfiguration screenConfig;	///< The screen configuration
                };
            }
        }
    }
}

#endif /* GENERATEXMLIMAGE_H_ */
