/**
 * @file	functionality-initiator.cc
 * @brief	functionality initiator class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "functionality-initiator.h"
#include <Configuration/menu-option-type-schema.hxx>
#include "Functionalities/saes-functionality-manager.h"
#include "Functionalities/close-election-functionality.h"
#include "Functionalities/open-election-functionality.h"
#include "Functionalities/alarm-box-diagnosis-functionality.h"
#include "Functionalities/EurnDiagnosisFunctionality.hxx"
#include "Functionalities/printer-diagnosis-functionality.h"
#include "Functionalities/s-c-reader-diagnosis-functionality.h"
#include "Functionalities/calibration-diagnosis-functionality.h"
#include "Functionalities/barcode-diagnosis-functionality.h"
#include "Functionalities/all-diagnosis-functionality.h"
#include "Functionalities/flash-diagnosis-functionality.h"
#include "Functionalities/change-hour-functionality.h"
#include "Functionalities/change-language-functionality.h"
#include "Functionalities/VotingMachineInstallationFunctionality.hxx"
#include "Functionalities/vote-demo-functionality.h"
#include "Functionalities/print-preview-functionality.h"
#include "Functionalities/voting-experience-validation-functionality.h"
#include "Functionalities/close-votation-functionality.h"
#include "Functionalities/LogReportFunctionality.h"
#include "Functionalities/ShutdownMachineFunctionality.h"
#include "Functionalities/ActivationConsolidationFunctionality.h"
#include "Functionalities/DataUtilsValidatorFunctionality.hxx"
#include "Functionalities/AbleDiagnosticFunctionality.hxx"
#include "Functionalities/LanguageInstallFunctionality.hxx"
#include "Functionalities/InitialCalibrationFunctionality.hxx"
#include "Functionalities/DeviceDiagnosticResultFunctionality.hxx"
#include "Functionalities/DeviceDiagnosticPrintFunctionality.hxx"
#include "Functionalities/DeviceDiagnosticSaveFunctionality.hxx"
#include "Functionalities/SerialNumberFunctionality.hxx"
#include "Functionalities/DateDiagnosticFunctionality.hxx"
#include "Functionalities/MenuFunctionality.hxx"
#include "Functionalities/AutoDiagnosticFunctionality.hxx"
#include "Functionalities/LoginFunctionality.hxx"
#include "Functionalities/ZipCodeFunctionality.hxx"
#include "Functionalities/InitiatorFunctionality.hxx"
#include "Functionalities/AuthenticatorFunctionality.hxx"
#include "Functionalities/SimulatorVotesFunctionality.hxx"
#include "Functionalities/WaitingVotingExperienceFunctionality.hxx"
#include "Functionalities/WizardFunctionality.hxx"
#include "Functionalities/SkipVotingMachineInstallationFunctionality.hxx"
#include "Functionalities/SkipVoteProcessDemoFunctionality.hxx"
#include "Functionalities/VotingMachineProcessStatusFunctionality.hxx"
#include "Functionalities/ProcessStatusResultsShutdownFunctionality.hxx"
#include "Functionalities/ReportInstallFunctionality.hxx"
#include "Functionalities/RecountFunctionality.h"
#include "Functionalities/InfoLoginFunctionality.hxx"
#include "Functionalities/RemoveUSBFunctionality.hxx"
#include "Functionalities/SystemFilesIntegrityCheckFunctionality.hxx"
#include "Functionalities/CheckEmlFilesFunctionality.hxx"
#include "Functionalities/CheckVoteFilesFunctionality.hxx"
#include "Functionalities/ClosingElectionStatusFunctionality.hxx"
#include "Functionalities/CloseElectionStatusFunctionality.hxx"
#include "Functionalities/GenerateAllResultFunctionality.hxx"
#include "Functionalities/ReprintReportInstallFunctionality.hxx"
#include "Functionalities/AutoBarcodeDiagnosisFunctionality.hxx"
#include "Functionalities/AutoPrinterDiagnosisFunctionality.hxx"
#include "Functionalities/CustomRemarkDiagnoseFunctionality.hxx"
#include "Functionalities/CustomizableCheckListDiagnoseFunctionality.hxx"
#include "Functionalities/DiagnosticsInfoFunctionality.hxx"
#include "Functionalities/ResultsPackageTransmissionFunctionality.hxx"
#include "Functionalities/SearchAndMonitorDevicesFunctionality.hxx"
#include "Functionalities/ExternalPrintFunctionality.hxx"
#include "Functionalities/ReportPrintingVMFunctionality.hxx"
#include "Functionalities/DeviceDiagnosticResultListOnlyFunctionality.hxx"
#include "Functionalities/MouseFunctionality.hxx"

using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::Functionality::Configuration;

void FunctionalityInitiator::Register()
{
    unsigned int count = sizeof ( MenuOptionType::_xsd_MenuOptionType_indexes_ ) / sizeof ( MenuOptionType::Value );

    for ( unsigned int i = 0; i < count; i++ )
    {
        switch ( MenuOptionType::_xsd_MenuOptionType_indexes_[i] )
        {
            case MenuOptionType::CONFIGURATION_XMLMENU:
                Smartmatic::SAES::GUI::Functionality::MenuFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMABLEDIAGNOSTICFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::AbleDiagnosticFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMACTIVATIONCONSOLIDATIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ActivationConsolidationFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMALARMBOXDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::AlarmBoxDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMEURNDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::EurnDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMALLDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::AllDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMAUTHENTICATORFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::AuthenticatorFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMAUTODIAGNOSTICFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::AutoDiagnosticFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMBARCODEDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::BarcodeDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCALIBRATIONDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::CalibrationDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCHANGEHOURFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ChangeHourFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCHANGELANGUAGEFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ChangeLanguageFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCHECKEMLFILESFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::CheckEmlFilesFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCHECKVOTESFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::CheckVoteFilesFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCLOSINGELECTIONSTATUSFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ClosingElectionStatusFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCLOSEELECTIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::CloseElectionFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCLOSEELECTIONSTATUSFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::CloseElectionStatusFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCLOSEVOTATIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::CloseVotationFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMDATAUTILSVALIDATORFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::DataUtilsValidatorFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMDATEDIAGNOSTICFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::DateDiagnosticFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICPRINTFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::DeviceDiagnosticPrintFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICRESULTFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::DeviceDiagnosticResultFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICSAVEFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::DeviceDiagnosticSaveFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;


            case MenuOptionType::CONFIGURATION_XMLMFLASHDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::FlashDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMGENERATEALLRESULTFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::GenerateAllResultFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMINFOLOGINFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::InfoLoginFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMINITIALCALIBRATIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::InitialCalibrationFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMINITIATORFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::InitiatorFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMLANGUAGEINSTALLFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::LanguageInstallFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMLOGINFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::LoginFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMLOGREPORTFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::LogReportFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMOPENELECTIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::OpenElectionFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMPRINTERDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::PrinterDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMPRINTPREVIEWFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::PrintPreviewFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMPROCESSSTATUSRESULTSSHUTDOWNFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ProcessStatusResultsShutdownFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMRECOUNTFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::RecountFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMREMOVEUSBFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::RemoveUSBFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMSYSTEMFILESINTEGRITYCHECKFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::SystemFilesIntegrityCheckFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMREPORTINSTALLFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ReportInstallFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMREPRINTINSTALLFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ReprintReportInstallFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMSCREADERDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::SCReaderDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMSERIALNUMBERFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::SerialNumberFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMSHUTDOWMMACHINEFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ShutdownMachineFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMSIMULATORVOTESFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::SimulatorVotesFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMSKIPVOTEPROCESSDEMOFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::SkipVoteProcessDemoFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMSKIPVOTINGMACHINEINSTALLATIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::SkipVotingMachineInstallationFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMVOTEPROCESSDEMOFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::VoteProcessDemoFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMVOTINGEXPERIENCEVALIDATIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::VotingExperienceValidationFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMVOTINGMACHINEINSTALLATIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::VotingMachineInstallationFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMVOTINGMACHINEPROCESSSTATUSRESULTSFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::VotingMachineProcessStatusFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMWAITINGVOTINGEXPERIENCEFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::WaitingVotingExperienceFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMWIZARDFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::WizardFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMZIPCODEFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ZipCodeFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMAUTOBARCODEDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::AutoBarcodeDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMAUTOPRINTERDIAGNOSISFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::AutoPrinterDiagnosisFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCUSTOMREMARKDIAGNOSEFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::CustomRemarkDiagnoseFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMCUSTOMIZABLECHECKLISTDIAGNOSEFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::CustomizableCheckListDiagnoseFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMDIAGNOSTICSINFOFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::DiagnosticsInfoFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMRESULTSPACKAGETRANSMISSIONFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ResultsPackageTransmissionFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMMOUSEFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::MouseFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMSEARCHANDMONITORDEVICESFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::SearchAndMonitorDevicesFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMEXTERNALPRINTFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ExternalPrintFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMREPORTPRINTINGVMFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::ReportPrintingVMFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            case MenuOptionType::CONFIGURATION_XMLMDEVICEDIAGNOSTICRESULTLISTONLYFUNCTIONALITY:
                Smartmatic::SAES::GUI::Functionality::DeviceDiagnosticResultListOnlyFunctionality::getInstance()->Register ( SaesFunctionalityManager::getInstance() );
                break;

            default:
                break;
        }
    }
}

