#ifndef TEST_SAES_GUI_CONFIGURATION_MANAGER_TEST
#define TEST_SAES_GUI_CONFIGURATION_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockFile.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>
#include <tests/MockSaesGuiConfigurationManager.hxx>
#include <tests/MockProcessLauncher.hxx>
#include <tests/MockExitProcess.hxx>

class SaesGuiConfigurationTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;
        typedef Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager SAESGUIConfigurationManager;
        typedef Smartmatic::System::Runtime::ProcessLauncher ProcessLauncher;
        typedef Smartmatic::System::Runtime::ExitProcess ExitProcess;

    public:

        SaesGuiConfigurationTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testInitializeEmptyConfiguration()
        {
            MockConfigurationPersistenceManager manager;
            MockProcessLauncher launcher;
            MockExitProcess exitProcess;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            MOCKPP_CHAINER_FOR ( MockExitProcess, exitProcess ) exitProcess_mock ( &exitProcess );

            exitProcess_mock
            .expects ( MOCKPP_NS::once() );

            ExitProcess::getInstance ( &exitProcess );
            ProcessLauncher::getInstance ( &launcher );
            ConfigurationPersistenceManager::getInstance ( &manager );

            SAESGUIConfigurationManager * saesGuiManager = SAESGUIConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( NULL == saesGuiManager->getConfiguration() );

            manager.verify();
            launcher.verify();
            exitProcess.verify();

            delete saesGuiManager;
        }

        void testInitializeConfigurationFail()
        {
            MockConfigurationPersistenceManager manager;
            MockProcessLauncher launcher;
            MockExitProcess exitProcess;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            MOCKPP_CHAINER_FOR ( MockExitProcess, exitProcess ) exitProcess_mock ( &exitProcess );

            exitProcess_mock
            .expects ( MOCKPP_NS::once() );

            ExitProcess::getInstance ( &exitProcess );
            ProcessLauncher::getInstance ( &launcher );
            ConfigurationPersistenceManager::getInstance ( &manager );

            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "SMTT", "<abc>Hello</abc>" );
            SAESGUIConfigurationManager * saesGuiManager = SAESGUIConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( NULL == saesGuiManager->getConfiguration() );

            manager.verify();
            launcher.verify();
            exitProcess.verify();

            delete saesGuiManager;
        }

        void testInitializeConfigurationLoadDefault()
        {
            MockConfigurationPersistenceManager manager;
            MockProcessLauncher launcher;
            MockExitProcess exitProcess;
            MockSaesGuiConfigurationManager saesGuiManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::never() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            MOCKPP_CHAINER_FOR ( MockExitProcess, exitProcess ) exitProcess_mock ( &exitProcess );

            exitProcess_mock
            .expects ( MOCKPP_NS::never() );

            ExitProcess::getInstance ( &exitProcess );
            ProcessLauncher::getInstance ( &launcher );
            ConfigurationPersistenceManager::getInstance ( &manager );
            SAESGUIConfigurationManager::getInstance ( &saesGuiManager );

            CPPUNIT_ASSERT ( NULL != SAESGUIConfigurationManager::getInstance()->getConfiguration() );

            manager.verify();
            launcher.verify();
            exitProcess.verify();
        }

        CPPUNIT_TEST_SUITE ( SaesGuiConfigurationTest );

        CPPUNIT_TEST ( testInitializeEmptyConfiguration );
        CPPUNIT_TEST ( testInitializeConfigurationFail );
        CPPUNIT_TEST ( testInitializeConfigurationLoadDefault );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SAES_GUI_CONFIGURATION_MANAGER_TEST */
