#ifndef TEST_FILLTABLECOLUMNDYNAMICWIDGETTEST
#define TEST_FILLTABLECOLUMNDYNAMICWIDGETTEST

#ifdef ECUT_MAIN

#include <mockpp/mockpp.h>
#include <mockpp/framework/VerifyingTestCase.h>
#include <mockpp/ExpectationValue.h>
#include <mockpp/compat/AssertionFailedError.h>

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>

#include "PoorMockTableWidget.hxx"
#include "ChainMockColumnLayout.hxx"
#include <tests/MockSizeWidget.hxx>
#include "PoorMockFillTableWidget.hxx"

class FillTableColumnDynamicWidgetTest : public CppUnit::TestFixture
{
    private:

        PoorMockTableWidget * table;
        PoorMockFillTableWidget * consumer;

    public:

        FillTableColumnDynamicWidgetTest ()
        {
            table = NULL;
            consumer = NULL;
        }

        void setUp()
        {
            table = new PoorMockTableWidget();
            consumer = new PoorMockFillTableWidget ( table );
        }

        void tearDown()
        {
            delete consumer;
        }

        void testCallPrincipalMethod()
        {
            ChainMockColumnLayout mockConfig;
            createStub ( mockConfig, 1, 1, true, false );
            consumer->fill ( &mockConfig, 700, 1000 );

            CPPUNIT_ASSERT ( 1 == consumer->putPrincipalCounter );
        }

        void testCallStandInMethod()
        {
            ChainMockColumnLayout mockConfig;
            createStub ( mockConfig, 1, 1, true, false );
            consumer->fill ( &mockConfig, 700, 1000 );

            CPPUNIT_ASSERT ( 1 == consumer->putStandInCounter );
        }

        void testNotCallStandInTitleMethod()
        {
            ChainMockColumnLayout mockConfig;
            createStub ( mockConfig, 1, 1, true, false );
            consumer->fill ( &mockConfig, 700, 1000 );

            CPPUNIT_ASSERT ( 0 == consumer->putStandInTitleCounter );
        }

        void testCallStandInTitleMethod()
        {
            ChainMockColumnLayout mockConfig;
            createStub ( mockConfig, 1, 1, true, false );
            addBallot ( 0, 1 );
            consumer->fill ( &mockConfig, 700, 1000 );

            CPPUNIT_ASSERT ( 1 == consumer->putStandInTitleCounter );
        }

        void test_3Column_13Pivot_Swing_13P_10S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 13, true, false );
            addBallot ( 13, 10 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 13, table->row );

            MOCKPP_ASSERT_EQUALS ( 13, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 0, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 10, table->map [2] );
        }

        void test_3Column_15Pivot_Swing_18P_10S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 15, true, false );
            addBallot ( 18, 10 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 15, table->row );

            MOCKPP_ASSERT_EQUALS ( 9, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 9, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 10, table->map [2] );
        }

        void test_4Column_23Pivot_Swing_29P_10S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 4, 23, true, false );
            addBallot ( 29, 10 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 4, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 15, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 14, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 0, table->map [2] );
            MOCKPP_ASSERT_EQUALS ( 10, table->map [3] );
        }

        void test_4Column_23Pivot_Swing_69P_10S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 4, 23, true, false );
            addBallot ( 69, 10 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 4, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 23, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 23, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 23, table->map [2] );
            MOCKPP_ASSERT_EQUALS ( 10, table->map [3] );
        }

        void test_4Column_23Pivot_Swing_72P_16S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 4, 23, true, false );
            addBallot ( 72, 16 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 4, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 23, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 23, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 23, table->map [2] );
            MOCKPP_ASSERT_EQUALS ( 19, table->map [3] );
        }

        void test_3Column_23Pivot_Swing_48P_0S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 23, true, false );
            addBallot ( 48, 0 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 16, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 16, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 16, table->map [2] );
        }

        void test_3Column_23Pivot_Swing_49P_0S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 23, true, false );
            addBallot ( 49, 0 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 17, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 16, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 16, table->map [2] );
        }

        void test_3Column_23Pivot_Swing_50P_0S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 23, true, false );
            addBallot ( 50, 0 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 17, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 17, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 16, table->map [2] );
        }

        void test_4Column_23Pivot_Swing_72P_0S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 4, 23, true, false );
            addBallot ( 72, 0 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 4, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 18, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 18, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 18, table->map [2] );
            MOCKPP_ASSERT_EQUALS ( 18, table->map [3] );
        }

        void test_3Column_13Pivot_NotSwing_13P_10S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 13, false, false );
            addBallot ( 13, 10 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 13, table->row );

            MOCKPP_ASSERT_EQUALS ( 13, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 0, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 10, table->map [2] );
        }

        void test_3Column_13Pivot_NotSwing_14P_10S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 13, false, false );
            addBallot ( 14, 10 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 13, table->row );

            MOCKPP_ASSERT_EQUALS ( 13, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 1, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 10, table->map [2] );
        }

        void test_3Column_13Pivot_NotSwing_24P_0S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 13, false, false );
            addBallot ( 24, 0 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 13, table->row );

            MOCKPP_ASSERT_EQUALS ( 13, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 11, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 0, table->map [2] );
        }

        void test_3Column_15Pivot_PartialSwing_30P_0S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 15, false, true );
            addBallot ( 30, 0 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 15, table->row );

            MOCKPP_ASSERT_EQUALS ( 15, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 14, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 1, table->map [2] );
        }

        void test_3Column_15Pivot_PartialSwing_30P_10S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 15, false, true );
            addBallot ( 30, 10 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 15, table->row );

            MOCKPP_ASSERT_EQUALS ( 15, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 14, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 11, table->map [2] );
        }

        void test_3Column_23Pivot_PartialSwing_14P_0S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 23, false, true );
            addBallot ( 14, 0 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 14, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 0, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 0, table->map [2] );
        }

        void test_3Column_23Pivot_PartialSwing_46P_21S()
        {
            ChainMockColumnLayout mockConfig;

            createStub ( mockConfig, 3, 23, false, true );
            addBallot ( 46, 21 );

            consumer->fill ( &mockConfig, 700, 1000 );

            MOCKPP_ASSERT_EQUALS ( 3, table->column );
            MOCKPP_ASSERT_EQUALS ( 23, table->row );

            MOCKPP_ASSERT_EQUALS ( 23, table->map [0] );
            MOCKPP_ASSERT_EQUALS ( 22, table->map [1] );
            MOCKPP_ASSERT_EQUALS ( 22, table->map [2] );
        }

        CPPUNIT_TEST_SUITE ( FillTableColumnDynamicWidgetTest );

        CPPUNIT_TEST ( testCallPrincipalMethod );
        CPPUNIT_TEST ( testCallStandInMethod );
        CPPUNIT_TEST ( testNotCallStandInTitleMethod );
        CPPUNIT_TEST ( testCallStandInTitleMethod );

        CPPUNIT_TEST ( test_3Column_13Pivot_Swing_13P_10S );
        CPPUNIT_TEST ( test_3Column_15Pivot_Swing_18P_10S );
        CPPUNIT_TEST ( test_4Column_23Pivot_Swing_29P_10S );
        CPPUNIT_TEST ( test_4Column_23Pivot_Swing_69P_10S );
        CPPUNIT_TEST ( test_4Column_23Pivot_Swing_72P_16S );

        CPPUNIT_TEST ( test_3Column_23Pivot_Swing_48P_0S );
        CPPUNIT_TEST ( test_3Column_23Pivot_Swing_49P_0S );
        CPPUNIT_TEST ( test_3Column_23Pivot_Swing_50P_0S );
        CPPUNIT_TEST ( test_4Column_23Pivot_Swing_72P_0S );

        CPPUNIT_TEST ( test_3Column_13Pivot_NotSwing_13P_10S );
        CPPUNIT_TEST ( test_3Column_13Pivot_NotSwing_14P_10S );
        CPPUNIT_TEST ( test_3Column_13Pivot_NotSwing_24P_0S );

        CPPUNIT_TEST ( test_3Column_15Pivot_PartialSwing_30P_0S );
        CPPUNIT_TEST ( test_3Column_15Pivot_PartialSwing_30P_10S );
        CPPUNIT_TEST ( test_3Column_23Pivot_PartialSwing_14P_0S );
        CPPUNIT_TEST ( test_3Column_23Pivot_PartialSwing_46P_21S );

        CPPUNIT_TEST_SUITE_END();

    private:

        void createStub ( ChainMockColumnLayout & mockConfig, int column, int pivot, bool swing, bool partialSwing )
        {
            MOCKPP_CHAINER_FOR ( ChainMockColumnLayout, getColumns ) column_mock ( &mockConfig );
            MOCKPP_CHAINER_FOR ( ChainMockColumnLayout, getPivot ) pivot_mock ( &mockConfig );
            MOCKPP_CHAINER_FOR ( ChainMockColumnLayout, isSwing ) swing_mock ( &mockConfig );
            MOCKPP_CHAINER_FOR ( ChainMockColumnLayout, isPartialSwing ) partial_swing_mock ( &mockConfig );
            MOCKPP_CHAINER_FOR ( ChainMockColumnLayout, getSpacing ) spacing_mock ( &mockConfig );

            column_mock.stubs().will ( MOCKPP_NS::returnValue ( column ) );
            pivot_mock.stubs().will ( MOCKPP_NS::returnValue ( pivot ) );
            swing_mock.stubs().will ( MOCKPP_NS::returnValue ( swing ) );
            partial_swing_mock.stubs().will ( MOCKPP_NS::returnValue ( partialSwing ) );
            spacing_mock.stubs().setStub ( new MOCKPP_NS::ReturnStub<float> ( 1.0 ) );
        }

        void addBallot ( int principal, int standIn )
        {
            for ( int i = 0; i < principal; i++ )
            {
                MockSizeWidget * ballot = new MockSizeWidget();
                consumer->addPrincipal ( ballot );
            }

            for ( int i = 0; i < standIn; i++ )
            {
                MockSizeWidget * ballot = new MockSizeWidget();
                consumer->addStandIn ( ballot );
            }
        }
};

#endif /* ECUT_MAIN */

#endif /* TEST_FILLTABLECOLUMNDYNAMICWIDGETTEST */
