/**
 * @file BarcodeCRCCommandElement.hxx
 * @brief	Defines the crc barcode reader command element. One byte for crc
 * @author	Yadickson Soto
 * @date	23/06/2015
 */

#ifndef BARCODE_CRC_COMMAND_ELEMENT_HXX_
#define BARCODE_CRC_COMMAND_ELEMENT_HXX_

#include <Commands/CommandElement.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	BarcodeCRCCommandElement
                 * @brief	Class to define the crc barcode reader command element.
                 * @author	Yadickson Soto
                 * @date	23/06/2015
                 */
                class BarcodeCRCCommandElement : public CommandElement
                {
                    public:
                        /**
                         * @fn BarcodeCRCCommandElement();
                         * @brief	Constructor default.
                         * @author	Yadickson Soto
                         * @date	23/06/2015
                         */
                        BarcodeCRCCommandElement ();

                        /**
                         * @fn virtual ~BarcodeCRCCommandElement();
                         * @brief	Constructor destructorer.
                         * @author	Yadickson Soto
                         * @date	23/06/2015
                         */
                        virtual ~BarcodeCRCCommandElement();

                        /**
                         * @fn virtual void addToCommand( std::vector < unsigned char > & command );
                         * @brief Add crc command element to a full command.
                         * @date 23/06/2015
                         * @author Yadickson Soto
                         * @param	command The full command to which to add the command element..
                         */
                        virtual void addToCommand ( std::vector < unsigned char > & command );
                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* BARCODE_CRC_COMMAND_ELEMENT_HXX_ */
