/**
 * @file   BarcodeGetFactorySerialNumberCommand.hxx
 * @brief  Declaration of BarcodeGetFactorySerialNumberCommand class.
 * Queries the device for its factory serial number.
 * @author manuel.arguelles
 * @date   Apr  5, 2018
 */

#ifndef BARCODE_GET_FACTORY_SERIAL_NUMBER_COMMAND_HXX
#define BARCODE_GET_FACTORY_SERIAL_NUMBER_COMMAND_HXX

#include <Commands/CommandBaseBarcode.hxx>
#include <vector>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }

            namespace Commands
            {
                /**
                 * @class  BarcodeGetFactorySerialNumberCommand
                 * @brief  Get the factory serial number of the device.
                 * @author manuel.arguelles
                 * @date   Apr  5, 2018
                 */
                class BarcodeGetFactorySerialNumberCommand : public CommandBaseBarcode
                {
                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Apr  5, 2018
                         */
                        explicit BarcodeGetFactorySerialNumberCommand(HW::Connections::IConnection* connection);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Apr  5, 2018
                         */
                        virtual ~BarcodeGetFactorySerialNumberCommand();

                    protected:

                        /**
                         * @brief  Initialize the object.
                         * @author manuel.arguelles
                         * @date   Apr  5, 2018
                         */
                        virtual void initialize();

                        /**
                         * @brief  Process the response.
                         * @author manuel.arguelles
                         * @date   Apr  5, 2018
                         * @param  response The response.
                         * @return True if the response is ok, false if not.
                         */
                        virtual bool processResponse(const std::vector<unsigned char>& response);

                    private:

                        static Smartmatic::Log::ISMTTLog* m_logger;

                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* BARCODE_GET_FACTORY_SERIAL_NUMBER_COMMAND_HXX */
