/**
 * @file	BarcodeSendConfigCommand.hxx
 * @brief	Header to send barcode reconfiguration command.
 * @date	26/06/2015
 * @author	Yadickson Soto
 */

#ifndef _BARCODE_SEND_SERIAL_NUMBER_COMMAND_H_
#define _BARCODE_SEND_SERIAL_NUMBER_COMMAND_H_

#include <Commands/command-structure.h>
#include <vector>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	BarcodeSendConfigCommand
                 * @brief	Send barcode reconfiguration command.
                 * @author	Yadickson Soto
                 * @date	26/06/2015
                 */
                class BarcodeSendConfigCommand: public CommandStructure
                {

                    public:

                        /**
                         * @fn	BarcodeSendConfigCommand(Smartmatic::SAES::HW::Connections::IConnection * connection,
                         * const std::string & command, unsigned int timeout, unsigned int retries, bool mandatory );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param 	connection The connection provider
                         * @param	command The string command to send
                         * @param	timeout The waiting time to get response command
                         * @param	retries The tries to get good response
                         * @param	mandatory If command isn't mandatory, the execute is true
                         */
                        BarcodeSendConfigCommand ( Smartmatic::SAES::HW::Connections::IConnection * connection,
                                                   const std::string & command,
                                                   unsigned int timeout,
                                                   unsigned int retries,
                                                   bool mandatory );

                        /**
                         * @fn	virtual BarcodeSendConfigCommand::~BarcodeSendConfigCommand();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         */
                        virtual ~BarcodeSendConfigCommand();

                    protected:

                        /**
                         * @fn	virtual void BarcodeSendConfigCommand::initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         */
                        virtual void initialize();

                        /**
                         * @fn	virtual bool processResponse(const std::vector<unsigned char> & response);
                         * @brief	Processes the response and checks for its validity.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param	response	The response.
                         * @return	true if it succeeds, false if it fails.
                         */
                        virtual bool processResponse ( const std::vector<unsigned char> & response );

                        /**
                         * @fn	virtual CommandList getCommandList ();
                         * @brief	Get the complete command (list of command elements) to perform a barcode-reader operation.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @return	The command elements to make full command result.
                         */
                        virtual CommandList getCommandList ();

                    private:

                        std::string command; ///< The command to send
                        bool mandatory; ///< If true, process command always return true
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif // _BARCODE_SEND_SERIAL_NUMBER_COMMAND_H_
