/**
 * @file	CommandBaseBarcode.hxx
 * @brief	Declares the command base barcode class.
 * @author	Yadickson Soto
 * @date	23/06/2015
 */

#ifndef _COMMAND_BASE_BARCODE_H_
#define _COMMAND_BASE_BARCODE_H_

#include <Commands/command-structure.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	CommandBaseBarcode
                 * @brief	Base class for managing the barcode commands and its properties.
                 * @author	Fsilva
                 * @date	28/04/2011
                 */
                class CommandBaseBarcode: public CommandStructure
                {
                    public:

                        /**
                         * @fn	CommandBaseBarcode::CommandBaseBarcode( Smartmatic::SAES::HW::Connections::IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         * @param 	connection The connection provider
                         */
                        explicit CommandBaseBarcode ( Smartmatic::SAES::HW::Connections::IConnection * connection );

                    protected:

                        /**
                         * @fn	virtual CommandList getCommandList ();
                         * @brief	Get the complete command (list of command elements) to perform a barcode-reader operation.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @return	The command elements to make full command result.
                         */
                        virtual CommandList getCommandList ();
                };
            }
        }
    }
}

#endif // _COMMAND_BASE_BARCODE_H_
