/**
 * @file CommandElement.hxx
 * @brief Defines the basic building blocks of a full command to be sent to a device.
 * @date 17/02/2014
 * @author Yadickson Soto
 */

#ifndef COMMANDELEMENT_HXX_
#define COMMANDELEMENT_HXX_

#include <vector>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class CommandElement
                 * @brief Class to define the basic building blocks of a full command to be sent to a device.
                 * @date 17/02/2014
                 * @author Yadickson Soto
                 */
                class CommandElement
                {
                    public:
                        /**
                         * @fn virtual ~CommandElement();
                         * @brief Default destructor.
                         * @date 17/02/2014
                         * @author Yadickson Soto
                         */
                        virtual ~CommandElement() {};
                        /**
                         * @fn virtual void addToCommand( std::vector < unsigned char > & command ) = 0;
                         * @brief Add this command element to a full command.
                         * @date 17/02/2014
                         * @author Yadickson Soto
                         * @param	command The full command to which to add the command element..
                         */
                        virtual void addToCommand ( std::vector < unsigned char > & command ) = 0;
                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* COMMANDELEMENT_HXX_ */
