/**
 * @file DataCommandElement.hxx
 * @brief	Defines the data command element.
 * @author	Yadickson Soto
 * @date	17/02/2014
 */

#ifndef DATA_COMMAND_ELEMENT_HXX_
#define DATA_COMMAND_ELEMENT_HXX_

#include <Commands/CommandElement.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	DataCommandElement
                 * @brief	Defines the data command element.
                 * @author	Yadickson Soto
                 * @date	17/02/2014
                 */
                class DataCommandElement : public CommandElement
                {
                    public:

                        /**
                         * @fn	DataCommandElement ( const std::vector<unsigned char> & data );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @param	data The input data to added.
                         */
                        explicit DataCommandElement ( const std::vector<unsigned char> & data );

                        /**
                         * @fn	virtual ~DataCommandElement();
                         * @brief	Default destructor.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         */
                        virtual ~DataCommandElement();

                        /**
                         * @fn virtual void addToCommand( std::vector < unsigned char > & command );
                         * @brief Adds the data to the being-constructed command.
                         * @date 17/02/2014
                         * @author Yadickson Soto
                         * @param	command The being-constructed command, to which we want to add the data command element.
                         */
                        virtual void addToCommand ( std::vector < unsigned char > & command );

                    private:

                        const std::vector<unsigned char> & data; ///< The input data for add to command.
                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* DATA_COMMAND_ELEMENT_HXX_ */
