/**
 * @file   EurnDoorCommand.hxx
 * @brief  Declaration of E-Urn's door command.
 * @author manuel.arguelles
 * @date   Aug 28, 2017
 */

#ifndef EURN_DOOR_WITH_LEDCOMMAND_HXX
#define EURN_DOOR_WITH_LEDCOMMAND_HXX

#include "CommandBaseEurn.hxx"

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }
            namespace Commands
            {
                /**
                 * @class  EurnDoorWithLedCommand
                 * @brief  Set of commands for the e-urn's door with LED events.
                 * @author manuel.arguelles
                 * @date   Aug 28, 2017
                 */
                class EurnDoorWithLedCommand : public CommandBaseEurn
                {
                    public:

                        /**
                         * @brief  Creates an object of EurnDoorWithLedCommand class.
                         * @author manuel.arguelles
                         * @date   Aug 28, 2017
                         * @param  connection The connection provider.
                         * @param  time       The amount of seconds before auto closing the door.
                         * @param  duringExecution action to perform during execution
                         * @param  afterExecution action to perform during execution
                         */
                    EurnDoorWithLedCommand(HW::Connections::IConnection* connection, unsigned char time, unsigned char duringExecution, unsigned char afterExecution);

                        /**
                         * @brief  Destroys the object.
                         * @author manuel.arguelles
                         * @date   Aug 28, 2017
                         */
                        virtual ~EurnDoorWithLedCommand();

                    private:

                        /**
                         * @brief  Initialize the command.
                         * @author manuel.arguelles
                         * @date   Aug 28, 2017
                         */
                        virtual void initialize();


                    private:

                        unsigned char m_time;
                        unsigned char m_duringExecution;
                        unsigned char m_afterExecution;
                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* EURN_DOOR_WITH_LED_COMMAND_HXX */
