/**
 * @file   EurnRedLedCommand.hxx
 * @brief  Declaration of EurnRedLedCommand class.
 * @author manuel.arguelles
 * @date   Aug 28, 2017
 */

#ifndef EURN_RED_LED_COMMAND_HXX
#define EURN_RED_LED_COMMAND_HXX

#include "CommandBaseEurn.hxx"

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }
            namespace Commands
            {
                /**
                 * @class  EurnRedLedCommand
                 * @brief  Command sets for the red led.
                 * @author manuel.arguelles
                 * @date   Aug 28, 2017
                 */
                class EurnRedLedCommand : public CommandBaseEurn
                {
                    public:

                        /**
                         * @brief  Creates an object of EurnRedLedCommand class.
                         * @author manuel.arguelles
                         * @date   Aug 28, 2017
                         * @param  connection     The connection provider.
                         * @param  turnOn         Specify if the led should be on or off.
                         * @param  time           The amount of seconds before turning off the led.
                         * @param  afterExecution The post-execution command.
                         */
                        EurnRedLedCommand(HW::Connections::IConnection* connection, bool turnOn, unsigned char time, unsigned char afterExecution = 0x00);

                        /**
                         * @brief  Destroys the object.
                         * @author manuel.arguelles
                         * @date   Aug 28, 2017
                         */
                        ~EurnRedLedCommand();

                    private:
                                                /**
                         * @brief  Initialize the command.
                         * @author manuel.arguelles
                         * @date   Aug 28, 2017
                         */
                        virtual void initialize();

                    private:

                        bool m_turnOn;
                        unsigned char m_time;
                        unsigned char m_afterExecution;

                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */


#endif /* EURN_RED_LED_COMMAND_HXX */
