/**
 * @file   IOBEnableUsbCommand.hxx
 * @brief  Declaration of IOBEnableUsbCommand class.
 * @author eduardo.fernandez
 * @date   Jan 2, 2018
 */

#ifndef IOB_ENABLE_USB_COMMAND_HXX
#define IOB_ENABLE_USB_COMMAND_HXX

#include "CommandBaseIOB.hxx"
#include "IOBUsbPort.hxx"

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }

            namespace Commands
            {
                /**
                 * @class  IOBEnableUsbCommand
                 * @brief  Command to enable the USB ports.
                 * @author eduardo.fernandez
                 * @date   Jan 2, 2018
                 */
                class IOBEnableUsbCommand : public CommandBaseIOB
                {
                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        IOBEnableUsbCommand(HW::Connections::IConnection* connection, IOBUsbPort usbPort);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        virtual ~IOBEnableUsbCommand();

                    private:

                        /**
                         * @brief  Initialize the command.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        virtual void initialize();

                    private:

                        IOBUsbPort m_usbPort;
                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */


#endif /* IOB_ENABLE_USB_COMMAND_HXX */
