/*
 * IOBSetSerialNumberCommand.hxx
 *
 *  Created on: Jan 2, 2018
 *      Author: eduardo.fernandez
 */

#ifndef IOB_SET_SERIAL_NUMBER_COMMAND_HXX
#define IOB_SET_SERIAL_NUMBER_COMMAND_HXX

#include "CommandBaseIOB.hxx"

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }

            namespace Commands
            {
                /**
                 * @class  IOBSetSerialNumberCommand
                 * @brief  Command to set the serial number of the machine in the IOB's EEPROM.
                 * @author eduardo.fernandez
                 * @date   Jan 2, 2018
                 */
                class IOBSetSerialNumberCommand : public CommandBaseIOB
                {
                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        IOBSetSerialNumberCommand(HW::Connections::IConnection* connection, const  std::vector<unsigned char> & serialChars);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        virtual ~IOBSetSerialNumberCommand();

                    private:

                        /**
                         * @brief  Initialize the command.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         */
                        virtual void initialize();

                    private:

                        std::vector<unsigned char> m_serialChars;
                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* IOB_SET_SERIAL_NUMBER_COMMAND_HXX */
