/**
 * @file	P168ABoardModelNameCommand.hxx
 * @brief	Header the P168A board model name command class.
 * @date	12/09/2012
 * @author	Yadickson Soto
 */

#ifndef _P168A_BOARD_MODEL_NAME_COMMAND_H_
#define _P168A_BOARD_MODEL_NAME_COMMAND_H_

#include "command-base-p168-a.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	P168ABoardModelNameCommand
                 * @brief	P168A board model name command class.
                 * @author	Yadickson Soto
                 * @date	12/09/2012
                 */
                class P168ABoardModelNameCommand: public CommandBaseP168A
                {

                    public:

                        /**
                         * @fn	P168ABoardModelNameCommand::P168ABoardModelNameCommand( Smartmatic::SAES::HW::Connections::IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         * @param 	connection The connection provider
                         */
                        explicit P168ABoardModelNameCommand ( Smartmatic::SAES::HW::Connections::IConnection * connection );
                        /**
                         * @fn	virtual P168ABoardModelNameCommand::~P168ABoardModelNameCommand();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	12/09/2012
                         */
                        virtual ~P168ABoardModelNameCommand();

                    protected:

                        /**
                         * @fn	virtual void P168ABoardModelNameCommand::initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	12/09/2012
                         */
                        virtual void initialize();

                        /**
                         * @fn	virtual bool P168ABoardModelNameCommand::processResponse(const std::vector<unsigned char> & response);
                         * @brief	Processes the response and checks for its validity.
                         * @author	Yadickson Soto
                         * @date	12/09/2012
                         * @param	response	The response.
                         * @return	true if it succeeds, false if it fails.
                         */
                        virtual bool processResponse ( const std::vector<unsigned char> & response );

                        /**
                         * @fn	virtual CommandList getCommandList ();
                         * @brief	Get the complete command (list of command elements) to get the P168A model name.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @return	The command elements to make full command result.
                         */
                        virtual CommandList getCommandList ();

                    private:

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif // _P168A_BOARD_MODEL_NAME_COMMAND_H_
