/**
 * @file	P168AWriteMemoryCommand.hxx
 * @brief	Header the P168A write memory command class.
 * @date	12/09/2012
 * @author	Yadickson Soto
 */

#ifndef _P168A_WRITE_MEMORY_COMMAND_H_
#define _P168A_WRITE_MEMORY_COMMAND_H_

#include "command-base-p168-a.h"
#include <vector>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	P168AWriteMemoryCommand
                 * @brief	P168A write memory command class.
                 * @author	Yadickson Soto
                 * @date	12/09/2012
                 */
                class P168AWriteMemoryCommand: public CommandBaseP168A
                {

                    public:

                        /**
                         * @fn	P168AWriteMemoryCommand::P168AWriteMemoryCommand(Smartmatic::SAES::HW::Connections::IConnection * connection,
                         * 			unsigned char address1,
                         * 			unsigned char address2,
                         * 			std::vector <unsigned char> & data);
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         * @param 	connection The connection provider
                         * @param	address1				address 1.
                         * @param	address2				address 2.
                         * @param [in,out]	data	data length to read.
                         */
                        P168AWriteMemoryCommand ( Smartmatic::SAES::HW::Connections::IConnection * connection, unsigned char address1, unsigned char address2, std::vector <unsigned char> & data );
                        /**
                         * @fn	virtual P168AWriteMemoryCommand::~P168AWriteMemoryCommand();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	12/09/2012
                         */
                        virtual ~P168AWriteMemoryCommand();

                    protected:

                        /**
                         * @fn	virtual void P168AWriteMemoryCommand::initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	12/09/2012
                         */
                        virtual void initialize();

                    private:

                        unsigned char address1; ///< address 1 (80h ~ EFh)
                        unsigned char address2; ///< address 2 (00h ~ FFh)
                        std::vector <unsigned char> dataToWrite; ///< data to write
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif // _P168A_WRITE_MEMORY_COMMAND_H_
