/**
 * @file	alarm-ping-command.cc
 * @brief	alarm ping command class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "alarm-ping-command.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Commands/AlarmHeaderCommandElement.hxx>
#include <Commands/DataCommandElement.hxx>
#include <Commands/AlarmFooterCommandElement.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::Log;

ISMTTLog * AlarmPingCommand::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Commands.AlarmPingCommand", GETTEXT_PACKAGE );

AlarmPingCommand::AlarmPingCommand ( Smartmatic::SAES::HW::Connections::IConnection * connection )
    : CommandBaseAlarm ( connection )
{
    hasResponse_ = true;
    retriesToReceive = 1;
}

CommandStructure::CommandList AlarmPingCommand::getCommandList ()
{
    CommandList commands;

    commands.push_back ( new AlarmHeaderCommandElement() );
    commands.push_back ( new DataCommandElement ( commandByte ) );
    commands.push_back ( new DataCommandElement ( data ) );
    commands.push_back ( new AlarmFooterCommandElement() );

    return commands;
}

void AlarmPingCommand::initialize()
{
    commandByte.push_back ( 0x00 );
    data.push_back ( 0x00 );
    data.push_back ( 0x00 );
}

bool AlarmPingCommand::processResponse(const std::vector<unsigned char>& response)
{
    bool isValid = false;
    if ( response.size() >= 5)
    {
        logger->Debug("AB Ping Received: " +
                    StringUtils::arrayHexToString(response, "0x", " "));
        if (response[0] == 0x55 && response[1] == 0x00 && response[2] == 0x00 && response[3] == 0x00 && response[4] == 0xAA)
        {
            isValid = true;
        }
    }
    return isValid;
}
