/**
 * @file	alarm-ping-command.h
 * @brief	Declares the alarm ping command class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _ALARM_PING_COMMAND_H_
#define _ALARM_PING_COMMAND_H_

#include "command-base-alarm.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	AlarmPingCommand
                 * @brief	Manages the commands that control the ping of the alarm.
                 * @author	Fsilva
                 * @date	28/04/2011
                 */

                class AlarmPingCommand: public CommandBaseAlarm
                {

                    public:

                        /**
                         * @fn	AlarmPingCommand::AlarmPingCommand( Smartmatic::SAES::HW::Connections::IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         * @param 	connection The connection provider
                         */
                        explicit AlarmPingCommand ( Smartmatic::SAES::HW::Connections::IConnection * connection );

                    protected:

                        /**
                         * @fn	virtual CommandList getCommandList ();
                         * @brief	Get the complete command (list of command elements) to ping the alarm.
                         * @author	Yadickson Soto
                         * @date	17/02/2014
                         * @return	The command elements to make full command result.
                         */
                        virtual CommandList getCommandList ();

                    private:

                        /**
                         * @fn	void AlarmPingCommand::initialize();
                         * @brief	Initializes this object.
                         * @author	Fsilva
                         * @date	27/04/2011
                         */

                        void initialize();

                        /**
                         * @brief  Process the response
                         * @author manuel.arguelles
                         * @date   Oct 31, 2017
                         * @param  response The vector with the response bytes.
                         * @return True if the response was valid, false if not.
                        */
                        virtual bool processResponse(const std::vector<unsigned char>& response);

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}


#endif // _ALARM_PING_COMMAND_H_
