/**
 * @file	alarm-red-led-command.h
 * @brief	Declares the alarm red LED command class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _ALARM_RED_LED_COMMAND_H_
#define _ALARM_RED_LED_COMMAND_H_

#include "command-base-alarm.h"

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Commands
            {
                /**
                 * @class	AlarmRedLedCommand
                 * @brief	Manages the commands that control the red led of the alarm.
                 * @author	Fsilva
                 * @date	28/04/2011
                 */

                class AlarmRedLedCommand: public CommandBaseAlarm
                {

                    public:

                        /**
                         * @fn	AlarmRedLedCommand::AlarmRedLedCommand( Smartmatic::SAES::HW::Connections::IConnection * connection, unsigned char period, unsigned char repeat);
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         * @param 	connection The connection provider
                         * @param	period	The period.
                         * @param	repeat	The repeat.
                         */

                        AlarmRedLedCommand ( Smartmatic::SAES::HW::Connections::IConnection * connection, unsigned char period, unsigned char repeat );

                    private:

                        /**
                         * @fn	void AlarmRedLedCommand::initialize();
                         * @brief	Initializes this object.
                         * @author	Fsilva
                         * @date	27/04/2011
                         */

                        void initialize();

                        unsigned char period;   ///< Indicate the waiting time between each run.
                        unsigned char repeat;   ///< Indicate the amount of run.
                };
            }
        }
    }
}

#endif // _ALARM_RED_LED_COMMAND_H_
