/**
 * @file	AlarmBoxConfigurationManager.hxx
 * @brief	Declares the alarm box configuration manager class.
 * @author	Yadickson Soto
 * @date	03/07/2015
 */

#ifndef _ALARM_BOX_CONFIGURATION_MANAGER_H_
#define _ALARM_BOX_CONFIGURATION_MANAGER_H_

#include <string>
#include <Configuration/alarm-box-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Configuration
            {
                /**
                 * @class	AlarmBoxConfigurationManager
                 * @brief	Manager for alarm box configurations class.
                 * @author	Yadickson Soto
                 * @date	03/07/2015
                 */
                class AlarmBoxConfigurationManager
                {
                    public:

                        /**
                         * @fn	static AlarmBoxConfigurationManager *getInstance(AlarmBoxConfigurationManager * externManager = NULL);
                         * @brief	Gets the current alarm box Configuration Manager.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @param 	externManager external manager to unit test
                         * @return	instance of the class
                         */
                        static AlarmBoxConfigurationManager * getInstance ( AlarmBoxConfigurationManager * externManager = NULL );

                        /**
                         * @fn	bool simulationEnabled ();
                         * @brief	Get if simulation mode is enabled
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return	true if the simulation mode is enabled.
                         */
                        bool simulationEnabled ();

                        /**
                         * @fn	std::string getAlarmBoxPingFile ();
                         * @brief	Gets the simulator alarm box ping file.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return 	The simulator alarm box ping file.
                         */
                        std::string getAlarmBoxPingFile ();

                        /**
                         * @fn	std::string getAlarmBoxButtonFile ();
                         * @brief	Gets the simulator alarm box button file.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return 	The simulator alarm box button file.
                         */
                        std::string getAlarmBoxButtonFile ();

                        /**
                         * @fn	std::string getAlarmBoxBehaviorFile ();
                         * @brief	Gets the simulator alarm box behavior file.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return 	The simulator alarm box behavior file.
                         */
                        std::string getAlarmBoxBehaviorFile ();

                        /**
                         * @fn	virtual ~AlarmBoxConfigurationManager();
                         * @brief	Class destructorer
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         */
                        virtual ~AlarmBoxConfigurationManager();

                    protected:

                        /**
                         * @fn	virtual AlarmBoxConfiguration & getAlarmBoxConfiguration();
                         * @brief	Gets alarm box configuration.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return	The alarm box configuration.
                         */
                        virtual AlarmBoxConfiguration & getAlarmBoxConfiguration();

                        /**
                         * @fn	AlarmBoxConfigurationManager();
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         */
                        AlarmBoxConfigurationManager();

                        static AlarmBoxConfigurationManager * instance; ///< The singleton instance
                };
            }
        }
    }
}

#endif // _ALARM_BOX_CONFIGURATION_MANAGER_H_
