/*
 * EurnDeviceConfigurationManager.hxx
 *
 *  Created on: Oct 31, 2017
 *      Author: juanvalera
 */

#ifndef EURNDEVICECONFIGURATIONMANAGER_HXX_
#define EURNDEVICECONFIGURATIONMANAGER_HXX_

#include <string>
#include <Configuration/EurnDeviceSchema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Configuration
            {
                /**
                 * @class	EurnDeviceConfigurationManager
                 * @brief	Manager for Eurn configurations class.
                 * @author	Juan Valera
                 * @date	31/10/2017
                 */
                class EurnDeviceConfigurationManager
                {
                    public:

                        /**
                         * @fn	static EurnDeviceConfigurationManager *getInstance(EurnDeviceConfigurationManager * externManager = NULL);
                         * @brief	Gets the current Eurn Configuration Manager.
                         * @author	Juan Valera
                         * @date	31/10/2017
                         * @param 	externManager external manager to unit test
                         * @return	instance of the class
                         */
                        static EurnDeviceConfigurationManager * getInstance ( EurnDeviceConfigurationManager * externManager = NULL );

                        /**
                         * @fn	bool getIfCloseWhenInvalidVote ();
                         * @brief	Gets true if Close door command is send when an invalid vote is scanned.
                         * @author	Juan Valera
                         * @date	08/07/2015
                         * @return 	The model file path
                         */
                        bool getIfCloseWhenInvalidVote ();

                        /**
                         * @fn	virtual ~EurnDeviceConfigurationManager();
                         * @brief	Class destructorer
                         * @author	Juan Valera
                         * @date	31/10/2017
                         */
                        virtual ~EurnDeviceConfigurationManager();

                        /**
                         * @fn	virtual EurnDeviceConfiguration & getConfiguration();
                         * @brief	Gets Eurn device configuration.
                         * @author	Juan Valera
                         * @date	01/11/2017
                         * @return	The Eurn device configuration.
                         */
                        virtual EurnDeviceConfiguration & getConfiguration();

                    protected:
                        /**
                         * @fn	EurnDeviceConfigurationManager();
                         * @brief	Default constructor.
                         * @author	Juan Valera
                         * @date	31/10/2017
                         */
                        EurnDeviceConfigurationManager();

                        /**
                         * @fn	virtual EurnDeviceConfiguration & getEurnDeviceConfiguration();
                         * @brief	Gets Eurn device configuration.
                         * @author	Juan Valera
                         * @date	31/10/2017
                         * @return	The Eurn device configuration.
                         */
                        virtual EurnDeviceConfiguration & getEurnDeviceConfiguration();

                        static EurnDeviceConfigurationManager * instance; ///< The singleton instance
                };
            }
        }
    }
}
#endif /* EURNDEVICECONFIGURATIONMANAGER_HXX_ */
