/**
 * @file	P168AConfigurationManager.hxx
 * @brief	Declares the able-d configuration manager class.
 * @author	Yadickson Soto
 * @date	01/07/2015
 */

#ifndef _P168A_CONFIGURATION_MANAGER_H_
#define _P168A_CONFIGURATION_MANAGER_H_

#include <string>
#include <Configuration/p168a-device-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Configuration
            {
                /**
                 * @class	P168AConfigurationManager
                 * @brief	Manager for abler-d configurations class.
                 * @author	Yadickson Soto
                 * @date	01/07/2015
                 */
                class P168AConfigurationManager
                {
                    public:

                        /**
                         * @fn	static P168AConfigurationManager *getInstance(P168AConfigurationManager * externManager = NULL);
                         * @brief	Gets the current abler-d Configuration Manager.
                         * @author	Yadickson Soto
                         * @date	01/07/2015
                         * @param 	externManager external manager to unit test
                         * @return	instance of the class
                         */
                        static P168AConfigurationManager * getInstance ( P168AConfigurationManager * externManager = NULL );

                        /**
                         * @fn	const P168ADeviceConfiguration::ButtonTypeSequence & getButtons ();
                         * @brief	Gets the button list configuration.
                         * @author	Yadickson Soto
                         * @date	01/07/2015
                         * @return 	The button list configuration.
                         */
                        const P168ADeviceConfiguration::ButtonTypeSequence & getButtons ();

                        /**
                         * @fn	std::string getSerialMachineFile ();
                         * @brief	Gets the serial machine file to simulate device.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return 	The serial machine file path
                         */
                        std::string getSerialMachineFile ();

                        /**
                         * @fn	std::string getAbleDButtonsFile ();
                         * @brief	Gets the able-d buttons file to simulate device.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return 	The able-d buttons file path
                         */
                        std::string getAbleDButtonsFile ();

                        /**
                         * @fn	bool simulationEnabled ();
                         * @brief	Get if simulation mode is enabled
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return	true if the simulation mode is enabled.
                         */
                        bool simulationEnabled ();

                        /**
                         * @fn	virtual ~P168AConfigurationManager();
                         * @brief	Class destructorer
                         * @author	Yadickson Soto
                         * @date	01/07/2015
                         */
                        virtual ~P168AConfigurationManager();

                    protected:

                        /**
                         * @fn	virtual P168ADeviceConfiguration & getP168ADeviceConfiguration();
                         * @brief	Gets able-d configuration.
                         * @author	Yadickson Soto
                         * @date	01/07/2015
                         * @return	The able-d configuration.
                         */
                        virtual P168ADeviceConfiguration & getP168ADeviceConfiguration();

                        /**
                         * @fn	P168AConfigurationManager();
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	01/07/2015
                         */
                        P168AConfigurationManager();

                        static P168AConfigurationManager * instance; ///< The singleton instance
                };
            }
        }
    }
}

#endif // _P168A_CONFIGURATION_MANAGER_H_
