/**
 * @file	s-a-e-s-hardware-conf-manager.h
 * @brief	Declares the e s hardware conf manager class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#ifndef _S_A_E_S_HARDWARE_CONFIGURATION_MANAGER_H_
#define _S_A_E_S_HARDWARE_CONFIGURATION_MANAGER_H_

#include <string>
#include <Configuration/SaesHardwareConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Configuration
            {
                /**
                 * @class	SAESHardwareConfigurationManager
                 * @brief	Manager for saes hardware configurations.
                 * @author	Fsilva
                 * @date	28/04/2011
                 */

                class SAESHardwareConfigurationManager
                {
                    public:

                        /**
                         * @fn	static SAESHardwareConfigurationManager SAESHardwareConfigurationManager::*getInstance( SAESHardwareConfigurationManager * externConfig = NULL );
                         * @brief	Gets the current hardware configuration manager.
                         * @author	Yadickson Soto
                         * @date	10/02/2014
                         * @param 	externConfig used only in unit test
                         * @return	instances of the class.
                         */
                        static SAESHardwareConfigurationManager * getInstance ( SAESHardwareConfigurationManager * externConfig = NULL );

                        /**
                         * @fn	Smartmatic::SAES::HW::Configuration::SaesHardwareConfiguration * getConfiguration();
                         * @brief	Gets the configuration.
                         * @author	Fsilva
                         * @date	27/04/2011
                         * @return	null if it fails, else the configuration.
                         */
                        Smartmatic::SAES::HW::Configuration::SaesHardwareConfiguration * getConfiguration();

                        /**
                         * @fn	void SAESHardwareConfigurationManager::loadDefault();
                         * @brief	Loads the default configuration.
                         * @author	Fsilva
                         * @date	27/04/2011
                         */
                        void loadDefault();

                        /**
                         * @fn	virtual SAESHardwareConfigurationManager::~SAESHardwareConfigurationManager();
                         * @brief	Finaliser.
                         * @author	Fsilva
                         * @date	27/04/2011
                         */

                        virtual ~SAESHardwareConfigurationManager();

                    protected:

                        /**
                         * @fn	SAESHardwareConfigurationManager::SAESHardwareConfigurationManager();
                         * @brief	Default constructor.
                         * @author	Fsilva
                         * @date	27/04/2011
                         */
                        SAESHardwareConfigurationManager();

                        /**
                         * @fn	virtual void SAESHardwareConfigurationManager::initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	10/02/2014
                         */
                        virtual void initialize();

                        static SAESHardwareConfigurationManager * instance; ///< The singleton instance

                    private:

                        Smartmatic::SAES::HW::Configuration::SaesHardwareConfiguration * configuration; ///< The configuration
                };
            }
        }
    }
}

#endif // _S_A_E_S_HARDWARE_CONFIGURATION_MANAGER_H_
