/**
 * @file	saes-hardware-language-controller.h
 * @brief	Declares the saes hardware language controller class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#ifndef _SAES_HW_LANGUAGE_CONTROLLER_H_
#define _SAES_HW_LANGUAGE_CONTROLLER_H_

#include "Environment/Configurations/BaseLanguageConfiguration.hxx"


namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Configuration
            {
                /**
                 * @class	SAESHardwareLanguageController
                 * @brief	Manager for SAES hardware language.
                 * @author	Fsilva
                 * @date	28/04/2011
                 */

                class SAESHardwareLanguageController: public Smartmatic::Environment::BaseLanguageConfiguration
                {
                    public:

                        /**
                         * @fn	std::string SAESHardwareLanguageController::getPackageName();
                         * @brief	Gets the package name.
                         * @author	Fsilva
                         * @date	27/04/2011
                         * @return	The package name.
                         */

                        std::string getPackageName();

                        /**
                         * @fn	std::string SAESHardwareLanguageController::getLocaleDir();
                         * @brief	Gets the locale dir.
                         * @author	Fsilva
                         * @date	27/04/2011
                         * @return	The locale dir.
                         */

                        std::string getLocaleDir();

                        /**
                         * @fn	SAESHardwareLanguageController::SAESHardwareLanguageController();
                         * @brief	Default constructor.
                         * @author	Fsilva
                         * @date	27/04/2011
                         */

                        SAESHardwareLanguageController();

                        /**
                         * @fn	virtual SAESHardwareLanguageController::~SAESHardwareLanguageController();
                         * @brief	Finaliser.
                         * @author	Fsilva
                         * @date	27/04/2011
                         */

                        virtual ~SAESHardwareLanguageController();
                };
            }
        }
    }
}

#endif // _SAES_HW_LANGUAGE_CONTROLLER_H_
