/*
 * AbledInterface.hxx
 *
 *  Created on: Jan 12, 2018
 *      Author: eduardo.fernandez
 */

#ifndef ABLED_INTERFACE_HXX
#define ABLED_INTERFACE_HXX

#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Operation
            {
                class AbleDButtonState;
            }

            namespace Devices
            {
                /**
                 * @class	AbledInterface
                 * @brief	Manage the connections with the abled.
                 * @author	eduardo.fernandez
                 * @date	Jan 12, 2018
                 */
                class AbledInterface
                {
                    public:

                        /**
                         * @fn	virtual ~AbledInterface();
                         * @brief	Class destroyer.
                         * @author	eduardo.fernandez
                         * @date	Jan 12, 2018
                         */
                        virtual ~AbledInterface() {};

                        /**
                         * @fn	virtual bool open() = 0;
                         * @brief	Opens the connections.
                         * @author	eduardo.fernandez
                         * @date	Jan 12, 2018
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open(const std::string & device) = 0;

                        /**
                         * @fn	virtual void close() = 0;
                         * @brief	Closes the connections.
                         * @author	eduardo.fernandez
                         * @date	Jan 12, 2018
                         * @return  True if the connection was closed.
                         */
                        virtual bool close() = 0;

                        /**
                         * @fn  virtual bool enableNotifications(const bool enable) = 0;
                         * @brief  Enables abled notifications.
                         * @author eduardo.fernandez
                         * @date   Jan 12, 2018
                         * @param  enable Whether to enable the abled notifications.
                         * @return Whether the abled notifications are successfully enabled or disabled.
                         */
                        virtual bool enableNotifications(const bool enable) = 0;

                        /**
                         * @fn  virtual std::string getFWVersion() = 0;
                         * @brief   Gets the firmware version of the abled.
                         * @author  eduardo.fernandez
                         * @date    Jan 12, 2018
                         * @return  hex number if successful, 0x00 if not
                         */
                        virtual std::string getFWVersion() = 0;

                        /**
                         * @fn  virtual std::string getSerialNumber() = 0;
                         * @brief  Gets the serial number from the PDB's EEPROM.
                         * @author eduardo.fernandez
                         * @date   Jan 12, 2018
                         * @return The serial number is successful, empty string if not.
                         */
                        virtual std::string getSerialNumber() = 0;

                        /**
                         * @fn virtual bool setSerialNumber(const std::string& serial) = 0;
                         * @brief  Sets the serial number to the PDB's EEPROM.
                         * @author eduardo.fernandez
                         * @date   Jan 12, 2018
                         * @param  serial The serial value to set.
                         * @return True if the operation succeed, false if not.
                        */
                        virtual bool setSerialNumber(const std::string& serial) = 0;

                        /**
                         * @fn  virtual Smartmatic::SAES::HW::Operation::A4517AbleDButtonState * getA4517AbleDButtonState() = 0;
                         * @brief   Method getter able-d button status.
                         * @author  eduardo.fernandez
                         * @date    Jan 15, 2018
                         * @return AbleD button status
                         */
                        virtual Smartmatic::SAES::HW::Operation::AbleDButtonState * getA4517AbleDButtonState () = 0;
                };
            }
        }
    }
}
#endif /* ABLED_INTERFACE_HXX */
