#ifndef TEST_BARCODE_GET_SERIAL_NUMBER_COMMAND_TEST
#define TEST_BARCODE_GET_SERIAL_NUMBER_COMMAND_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Commands/BarcodeGetSerialNumberCommand.hxx>
#include <System/Utils/StringUtils.hxx>
#include <tests/StoreMockConnection.hxx>

class BarcodeGetSerialNumberCommandTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Utils::StringUtils StringUtils;
        typedef Smartmatic::System::SafeVector<unsigned char> ResponseVector;
        typedef Smartmatic::SAES::HW::Commands::BarcodeGetSerialNumberCommand BarcodeGetSerialNumberCommand;

    public:

        BarcodeGetSerialNumberCommandTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testGetSerialNumberEmptyResponse()
        {
            StoreMockConnection connection;

            BarcodeGetSerialNumberCommand getSerialNumberCommand ( &connection );
            ResponseVector response;

            MOCKPP_CHAINER_FOR_EXT ( StoreMockConnection, openConnection, ext ) Open_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, readBuffer ) Read_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, closeConnection ) Close_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, getTimeOut ) TimeOut_mock ( &connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( false == getSerialNumberCommand.execute() );
            CPPUNIT_ASSERT ( "7e0000053348303230b3" == StringUtils::arrayHexToString ( connection.sendBuffer ) );

            connection.verify();
        }

        void testGetSerialNumberNotEmptyResponseFail()
        {
            StoreMockConnection connection;

            BarcodeGetSerialNumberCommand getSerialNumberCommand ( &connection );
            ResponseVector response;

            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );

            MOCKPP_CHAINER_FOR_EXT ( StoreMockConnection, openConnection, ext ) Open_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, readBuffer ) Read_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, closeConnection ) Close_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, getTimeOut ) TimeOut_mock ( &connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( false == getSerialNumberCommand.execute() );
            CPPUNIT_ASSERT ( true == getSerialNumberCommand.getResponse().empty() );
            CPPUNIT_ASSERT ( "7e0000053348303230b3" == StringUtils::arrayHexToString ( connection.sendBuffer ) );

            connection.verify();
        }

        void testGetSerialNumberNotEmptyResponseSuccess()
        {
            StoreMockConnection connection;

            BarcodeGetSerialNumberCommand getSerialNumberCommand ( &connection );
            ResponseVector response;

            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( '1' );
            response.push_back ( '2' );
            response.push_back ( '3' );
            response.push_back ( 0x00 );

            MOCKPP_CHAINER_FOR_EXT ( StoreMockConnection, openConnection, ext ) Open_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, readBuffer ) Read_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, closeConnection ) Close_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, getTimeOut ) TimeOut_mock ( &connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( true == getSerialNumberCommand.execute() );
            CPPUNIT_ASSERT ( "123" ==  StringUtils::arrayHexToAsciiString ( getSerialNumberCommand.getResponse() ) );
            CPPUNIT_ASSERT ( "7e0000053348303230b3" == StringUtils::arrayHexToString ( connection.sendBuffer ) );

            connection.verify();
        }

        CPPUNIT_TEST_SUITE ( BarcodeGetSerialNumberCommandTest );

        CPPUNIT_TEST ( testGetSerialNumberEmptyResponse );
        CPPUNIT_TEST ( testGetSerialNumberNotEmptyResponseFail );
        CPPUNIT_TEST ( testGetSerialNumberNotEmptyResponseSuccess );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_BARCODE_GET_SERIAL_NUMBER_COMMAND_TEST */
