#ifndef TEST_BARCODE_LENGTH_COMMAND_ELEMENT_TEST
#define TEST_BARCODE_LENGTH_COMMAND_ELEMENT_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Commands/BarcodeLengthCommandElement.hxx>
#include <System/Utils/StringUtils.hxx>

class BarcodeLengthCommandElementTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Utils::StringUtils StringUtils;
        typedef Smartmatic::SAES::HW::Commands::BarcodeLengthCommandElement BarcodeLengthCommandElement;

    public:

        BarcodeLengthCommandElementTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testLengthDataEmpty()
        {
            std::vector<unsigned char> command;
            std::vector<unsigned char> data;

            BarcodeLengthCommandElement length ( data );

            std::string expected = "0000";
            std::string result;

            length.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( 2 == command.size() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testLengthDataOneElement()
        {
            std::vector<unsigned char> command;
            std::vector<unsigned char> data;

            data.push_back ( 0x33 );

            BarcodeLengthCommandElement length ( data );

            std::string expected = "0001";
            std::string result;

            length.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( 2 == command.size() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testLengthDataTwoElements()
        {
            std::vector<unsigned char> command;
            std::vector<unsigned char> data;

            data.push_back ( 0x33 );
            data.push_back ( 0x25 );

            BarcodeLengthCommandElement length ( data );

            std::string expected = "0002";
            std::string result;

            length.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( 2 == command.size() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testLengthDataTenElements()
        {
            std::vector<unsigned char> command;
            std::vector<unsigned char> data;

            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );

            BarcodeLengthCommandElement length ( data );

            std::string expected = "000a";
            std::string result;

            length.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( 2 == command.size() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testLengthDataTwelveElements()
        {
            std::vector<unsigned char> command;
            std::vector<unsigned char> data;

            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );
            data.push_back ( 0x00 );
            data.push_back ( 0x01 );

            BarcodeLengthCommandElement length ( data );

            std::string expected = "0014";
            std::string result;

            length.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( 2 == command.size() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testLengthDataThreeHundredElements()
        {
            std::vector<unsigned char> command;
            std::vector<unsigned char> data;

            for ( int i = 0; i < 300 ; i++ )
            {
                data.push_back ( 0x00 );
            }

            BarcodeLengthCommandElement length ( data );

            std::string expected = "012c";
            std::string result;

            length.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( 2 == command.size() );
            CPPUNIT_ASSERT ( expected == result );
        }

        CPPUNIT_TEST_SUITE ( BarcodeLengthCommandElementTest );

        CPPUNIT_TEST ( testLengthDataEmpty );
        CPPUNIT_TEST ( testLengthDataOneElement );
        CPPUNIT_TEST ( testLengthDataTwoElements );
        CPPUNIT_TEST ( testLengthDataTenElements );
        CPPUNIT_TEST ( testLengthDataTwelveElements );
        CPPUNIT_TEST ( testLengthDataThreeHundredElements );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_BARCODE_LENGTH_COMMAND_ELEMENT_TEST */
