#ifndef TEST_CHECKSUM_COMMAND_ELEMENT_TEST
#define TEST_CHECKSUM_COMMAND_ELEMENT_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Commands/ChecksumCommandElement.hxx>
#include <System/Utils/StringUtils.hxx>

class ChecksumCommandElementTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Utils::StringUtils StringUtils;
        typedef Smartmatic::SAES::HW::Commands::ChecksumCommandElement ChecksumCommandElement;

    public:

        ChecksumCommandElementTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testInputDataEmpty()
        {
            std::vector<unsigned char> input;
            std::vector<unsigned char> command;

            ChecksumCommandElement checksum ( input );

            std::string expected = "00";
            std::string result;

            checksum.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( false == command.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testInputData_OneElement()
        {
            std::vector<unsigned char> input;
            std::vector<unsigned char> command;

            input.push_back ( 0x25 );

            ChecksumCommandElement checksum ( input );

            std::string expected = "25";
            std::string result;

            checksum.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( false == command.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testInputData_TwoElement()
        {
            std::vector<unsigned char> input;
            std::vector<unsigned char> command;

            input.push_back ( 0x25 );
            input.push_back ( 0x02 );

            ChecksumCommandElement checksum ( input );

            std::string expected = "27";
            std::string result;

            checksum.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( false == command.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testInputData_TwoCalls()
        {
            std::vector<unsigned char> input;
            std::vector<unsigned char> command;

            input.push_back ( 0x25 );
            input.push_back ( 0x02 );

            ChecksumCommandElement checksum ( input );

            std::string expected = "2727";
            std::string result;

            checksum.addToCommand ( command );
            checksum.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( false == command.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testInputData_Overload()
        {
            std::vector<unsigned char> input;
            std::vector<unsigned char> command;

            input.push_back ( 0xFF );
            input.push_back ( 0x01 );

            ChecksumCommandElement checksum ( input );

            std::string expected = "00";
            std::string result;

            checksum.addToCommand ( command );

            result = StringUtils::arrayHexToString ( command );

            CPPUNIT_ASSERT ( false == command.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        CPPUNIT_TEST_SUITE ( ChecksumCommandElementTest );

        CPPUNIT_TEST ( testInputDataEmpty );
        CPPUNIT_TEST ( testInputData_OneElement );
        CPPUNIT_TEST ( testInputData_TwoElement );
        CPPUNIT_TEST ( testInputData_TwoCalls );
        CPPUNIT_TEST ( testInputData_Overload );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_CHECKSUM_COMMAND_ELEMENT_TEST */
