#ifndef TEST_P168A_PRODUCT_VERSION_INFO_NAME_TEST
#define TEST_P168A_PRODUCT_VERSION_INFO_NAME_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Commands/P168AProductVersionInfoCommand.hxx>
#include <System/Utils/StringUtils.hxx>
#include <tests/StoreMockConnection.hxx>

class P168AProductVersionInfoCommandTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Utils::StringUtils StringUtils;
        typedef Smartmatic::System::SafeVector<unsigned char> ResponseVector;
        typedef Smartmatic::SAES::HW::Commands::P168AProductVersionInfoCommand P168AProductVersionInfoCommand;

    public:

        P168AProductVersionInfoCommandTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testGetProductVersionInfoFail()
        {
            StoreMockConnection connection;
            P168AProductVersionInfoCommand infoCommand ( &connection );
            ResponseVector response;

            response.push_back ( 0x00 );

            MOCKPP_CHAINER_FOR_EXT ( StoreMockConnection, openConnection, ext ) Open_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, readBuffer ) Read_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, closeConnection ) Close_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, getTimeOut ) TimeOut_mock ( &connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( false == infoCommand.execute() );
            CPPUNIT_ASSERT ( true == infoCommand.getResponse().empty() );
            CPPUNIT_ASSERT ( "1b01" == StringUtils::arrayHexToString ( connection.sendBuffer ) );

            connection.verify();
        }

        void testGetProductVersionInfoFail_Header_Fail()
        {
            StoreMockConnection connection;
            P168AProductVersionInfoCommand infoCommand ( &connection );
            ResponseVector response;

            response.push_back ( 0x00 );
            response.push_back ( 0x01 );
            response.push_back ( 0x01 );
            response.push_back ( 0xff );

            MOCKPP_CHAINER_FOR_EXT ( StoreMockConnection, openConnection, ext ) Open_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, readBuffer ) Read_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, closeConnection ) Close_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, getTimeOut ) TimeOut_mock ( &connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( false == infoCommand.execute() );
            CPPUNIT_ASSERT ( true == infoCommand.getResponse().empty() );
            CPPUNIT_ASSERT ( "1b01" == StringUtils::arrayHexToString ( connection.sendBuffer ) );

            connection.verify();
        }

        void testGetProductVersionInfoFail_Command_Fail()
        {
            StoreMockConnection connection;
            P168AProductVersionInfoCommand infoCommand ( &connection );
            ResponseVector response;

            response.push_back ( 0x1B );
            response.push_back ( 0x00 );
            response.push_back ( 0x01 );
            response.push_back ( 0xff );

            MOCKPP_CHAINER_FOR_EXT ( StoreMockConnection, openConnection, ext ) Open_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, readBuffer ) Read_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, closeConnection ) Close_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, getTimeOut ) TimeOut_mock ( &connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( false == infoCommand.execute() );
            CPPUNIT_ASSERT ( true == infoCommand.getResponse().empty() );
            CPPUNIT_ASSERT ( "1b01" == StringUtils::arrayHexToString ( connection.sendBuffer ) );

            connection.verify();
        }

        void testGetProductVersionInfoFail_Length_Zero()
        {
            StoreMockConnection connection;
            P168AProductVersionInfoCommand infoCommand ( &connection );
            ResponseVector response;

            response.push_back ( 0x1B );
            response.push_back ( 0x01 );
            response.push_back ( 0x00 );
            response.push_back ( 0xff );

            MOCKPP_CHAINER_FOR_EXT ( StoreMockConnection, openConnection, ext ) Open_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, readBuffer ) Read_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, closeConnection ) Close_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, getTimeOut ) TimeOut_mock ( &connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( false == infoCommand.execute() );
            CPPUNIT_ASSERT ( true == infoCommand.getResponse().empty() );
            CPPUNIT_ASSERT ( "1b01" == StringUtils::arrayHexToString ( connection.sendBuffer ) );

            connection.verify();
        }

        void testGetProductVersionInfoSuccess()
        {
            StoreMockConnection connection;
            P168AProductVersionInfoCommand infoCommand ( &connection );
            ResponseVector response;

            response.push_back ( 0x1B );
            response.push_back ( 0x01 );
            response.push_back ( 0x04 );
            response.push_back ( '0' );
            response.push_back ( '.' );
            response.push_back ( '2' );
            response.push_back ( '5' );
            response.push_back ( 0x00 );

            MOCKPP_CHAINER_FOR_EXT ( StoreMockConnection, openConnection, ext ) Open_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, readBuffer ) Read_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, closeConnection ) Close_mock ( &connection );
            MOCKPP_CHAINER_FOR ( StoreMockConnection, getTimeOut ) TimeOut_mock ( &connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( true == infoCommand.execute() );
            CPPUNIT_ASSERT ( "0.25" == StringUtils::arrayHexToAsciiString ( infoCommand.getResponse() ) );
            CPPUNIT_ASSERT ( "1b01" == StringUtils::arrayHexToString ( connection.sendBuffer ) );

            connection.verify();
        }

        CPPUNIT_TEST_SUITE ( P168AProductVersionInfoCommandTest );

        CPPUNIT_TEST ( testGetProductVersionInfoFail );
        CPPUNIT_TEST ( testGetProductVersionInfoFail_Header_Fail );
        CPPUNIT_TEST ( testGetProductVersionInfoFail_Command_Fail );
        CPPUNIT_TEST ( testGetProductVersionInfoFail_Length_Zero );
        CPPUNIT_TEST ( testGetProductVersionInfoSuccess );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_P168A_PRODUCT_VERSION_INFO_NAME_TEST */
