#ifndef TEST_BARCODE_CONF_MANAGER_TEST
#define TEST_BARCODE_CONF_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Configuration/configuration-persistence-manager.h>
#include <Configuration/BarcodeDeviceConfigurationManager.hxx>
#include <tests/MockSAESHWConfigurationManager.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>
#include <tests/StoreMockBarcodeConfManager.hxx>
#include <tests/MockProcessLauncher.hxx>
#include <tests/MockExitProcess.hxx>

class BarcodeConfManagerTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::SAES::HW::Configuration::BarcodeDeviceConfigurationManager BarcodeDeviceConfigurationManager;
        typedef Smartmatic::SAES::HW::Configuration::SAESHardwareConfigurationManager SAESHardwareConfigurationManager;
        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;
        typedef Smartmatic::SAES::HW::Configuration::BarcodeCommand BarcodeCommand;
        typedef Smartmatic::System::Runtime::ProcessLauncher ProcessLauncher;
        typedef Smartmatic::System::Runtime::ExitProcess ExitProcess;


        BarcodeConfManagerTest()
        {
        }

        void setUp()
        {
        }

        void tearDown()
        {
        }

        void testGetConfigurationList_ConfigurationFileIsEmpty()
        {
            MockConfigurationPersistenceManager manager;
            MockProcessLauncher launcher;
            MockExitProcess exitProcess;
            MockSAESHWConfigurationManager saeshwManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::never() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            MOCKPP_CHAINER_FOR ( MockExitProcess, exitProcess ) exitProcess_mock ( &exitProcess );

            exitProcess_mock
            .expects ( MOCKPP_NS::never() );

            ExitProcess::getInstance ( &exitProcess );
            ProcessLauncher::getInstance ( &launcher );
            ConfigurationPersistenceManager::getInstance ( &manager );
            SAESHardwareConfigurationManager::getInstance ( &saeshwManager );

            SAESHardwareConfigurationManager::getInstance()->getConfiguration()->getBarcodeDeviceConfiguration().reset();

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence list;

            BarcodeDeviceConfigurationManager * barcodeManager = BarcodeDeviceConfigurationManager::getInstance();

            list = barcodeManager->getBarcodeConfigurationCommands ( "configuration", "hr220" );

            CPPUNIT_ASSERT ( true == list.empty() );

            manager.verify();
            launcher.verify();
            exitProcess.verify();

            delete barcodeManager;
        }

        void testGetConfigurationList_ConfigEmpty()
        {
            StoreMockBarcodeConfManager barcodeManager;
            BarcodeDeviceConfigurationManager::getInstance ( &barcodeManager );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence list;

            list = BarcodeDeviceConfigurationManager::getInstance()->getBarcodeConfigurationCommands ( "configuration", "hr220" );

            CPPUNIT_ASSERT ( true == list.empty() );
        }

        void testGetConfigurationList_ConfigValueNotFound()
        {
            StoreMockBarcodeConfManager barcodeManager;
            BarcodeDeviceConfigurationManager::getInstance ( &barcodeManager );

            Smartmatic::SAES::HW::Configuration::BarcodeDeviceConfiguration::ConfigurationCommandsSequence commandSeq;
            Smartmatic::SAES::HW::Configuration::BarcodeCommands commands ( "config" );
            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence seq;

            seq.push_back ( BarcodeCommand ( "command", 1000, 3, false ) );

            commands.setCommand ( seq );
            commandSeq.push_back ( commands );

            barcodeManager.config.setConfigurationCommands ( commandSeq );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence list;

            list = BarcodeDeviceConfigurationManager::getInstance()->getBarcodeConfigurationCommands ( "configuration", "hr220" );

            CPPUNIT_ASSERT ( true == list.empty() );
        }

        void testGetConfigurationList_PrimaryConfigValueFound()
        {
            StoreMockBarcodeConfManager barcodeManager;
            BarcodeDeviceConfigurationManager::getInstance ( &barcodeManager );

            Smartmatic::SAES::HW::Configuration::BarcodeDeviceConfiguration::ConfigurationCommandsSequence commandSeq;
            Smartmatic::SAES::HW::Configuration::BarcodeCommands commands ( "configuration" );
            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence seq;

            seq.push_back ( BarcodeCommand ( "command", 1000, 3, false ) );

            commands.setCommand ( seq );
            commandSeq.push_back ( commands );

            barcodeManager.config.setConfigurationCommands ( commandSeq );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence list;

            list = BarcodeDeviceConfigurationManager::getInstance()->getBarcodeConfigurationCommands ( "configuration", "hr220" );

            CPPUNIT_ASSERT ( false == list.empty() );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandIterator it;
            it = list.begin();

            CPPUNIT_ASSERT ( "command" == ( *it ).getCommand() );
            CPPUNIT_ASSERT ( 1000 == ( *it ).getDelayResponse() );
            CPPUNIT_ASSERT ( 3 == ( *it ).getRetries() );
            CPPUNIT_ASSERT ( false == ( *it ).getMandatorySent() );
        }

        void testGetConfigurationList_SecundaryConfigValueFound()
        {
            StoreMockBarcodeConfManager barcodeManager;
            BarcodeDeviceConfigurationManager::getInstance ( &barcodeManager );

            Smartmatic::SAES::HW::Configuration::BarcodeDeviceConfiguration::ConfigurationCommandsSequence commandSeq;
            Smartmatic::SAES::HW::Configuration::BarcodeCommands commands ( "configuration-hr220" );
            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence seq;

            seq.push_back ( BarcodeCommand ( "command", 1000, 3, true ) );

            commands.setCommand ( seq );
            commandSeq.push_back ( commands );

            barcodeManager.config.setConfigurationCommands ( commandSeq );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence list;

            list = BarcodeDeviceConfigurationManager::getInstance()->getBarcodeConfigurationCommands ( "configuration", "hr220" );

            CPPUNIT_ASSERT ( false == list.empty() );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandIterator it;
            it = list.begin();

            CPPUNIT_ASSERT ( "command" == ( *it ).getCommand() );
            CPPUNIT_ASSERT ( 1000 == ( *it ).getDelayResponse() );
            CPPUNIT_ASSERT ( 3 == ( *it ).getRetries() );
            CPPUNIT_ASSERT ( true == ( *it ).getMandatorySent() );
        }

        void testGetConfigurationList_SecundaryConfigValueFound_TwoCommands_Find_HR220()
        {
            StoreMockBarcodeConfManager barcodeManager;
            BarcodeDeviceConfigurationManager::getInstance ( &barcodeManager );

            Smartmatic::SAES::HW::Configuration::BarcodeDeviceConfiguration::ConfigurationCommandsSequence commandSeq;
            Smartmatic::SAES::HW::Configuration::BarcodeCommands commandOne ( "configuration" );
            Smartmatic::SAES::HW::Configuration::BarcodeCommands commandTwo ( "configuration-hr220" );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence seqOne;
            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence seqTwo;

            seqOne.push_back ( BarcodeCommand ( "commandOne", 100, 2, true ) );
            seqTwo.push_back ( BarcodeCommand ( "commandTwo", 1000, 3, false ) );

            commandOne.setCommand ( seqOne );
            commandTwo.setCommand ( seqTwo );

            commandSeq.push_back ( commandOne );
            commandSeq.push_back ( commandTwo );

            barcodeManager.config.setConfigurationCommands ( commandSeq );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence list;

            list = BarcodeDeviceConfigurationManager::getInstance()->getBarcodeConfigurationCommands ( "configuration", "hr220" );

            CPPUNIT_ASSERT ( false == list.empty() );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandIterator it;
            it = list.begin();

            CPPUNIT_ASSERT ( "commandTwo" == ( *it ).getCommand() );
            CPPUNIT_ASSERT ( 1000 == ( *it ).getDelayResponse() );
            CPPUNIT_ASSERT ( 3 == ( *it ).getRetries() );
            CPPUNIT_ASSERT ( false == ( *it ).getMandatorySent() );
        }

        void testGetConfigurationList_SecundaryConfigValueFound_TwoCommands_Find_MV110()
        {
            StoreMockBarcodeConfManager barcodeManager;
            BarcodeDeviceConfigurationManager::getInstance ( &barcodeManager );

            Smartmatic::SAES::HW::Configuration::BarcodeDeviceConfiguration::ConfigurationCommandsSequence commandSeq;

            Smartmatic::SAES::HW::Configuration::BarcodeCommands commandBasic ( "configuration" );
            Smartmatic::SAES::HW::Configuration::BarcodeCommands commandOne ( "configuration-hr220" );
            Smartmatic::SAES::HW::Configuration::BarcodeCommands commandTwo ( "configuration-mv110" );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence seqBasic;
            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence seqOne;
            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence seqTwo;

            seqBasic.push_back ( BarcodeCommand ( "command", 100, 1, false ) );
            seqOne.push_back ( BarcodeCommand ( "commandOne", 1000, 3, false ) );
            seqTwo.push_back ( BarcodeCommand ( "commandTwo", 2000, 4, true ) );

            commandBasic.setCommand ( seqBasic );
            commandOne.setCommand ( seqOne );
            commandTwo.setCommand ( seqTwo );

            commandSeq.push_back ( commandBasic );
            commandSeq.push_back ( commandOne );
            commandSeq.push_back ( commandTwo );

            barcodeManager.config.setConfigurationCommands ( commandSeq );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandSequence list;

            list = BarcodeDeviceConfigurationManager::getInstance()->getBarcodeConfigurationCommands ( "configuration", "mv110" );

            CPPUNIT_ASSERT ( false == list.empty() );

            Smartmatic::SAES::HW::Configuration::BarcodeCommands::CommandIterator it;
            it = list.begin();

            CPPUNIT_ASSERT ( "commandTwo" == ( *it ).getCommand() );
            CPPUNIT_ASSERT ( 2000 == ( *it ).getDelayResponse() );
            CPPUNIT_ASSERT ( 4 == ( *it ).getRetries() );
            CPPUNIT_ASSERT ( true == ( *it ).getMandatorySent() );
        }

        void testGetDefaultConfiguration_MockConfigurationIsEmpty()
        {
            StoreMockBarcodeConfManager mock;
            BarcodeDeviceConfigurationManager * barcodeManager = BarcodeDeviceConfigurationManager::getInstance ( &mock );

            CPPUNIT_ASSERT ( false == barcodeManager->simulationEnabled() );
            CPPUNIT_ASSERT ( true == barcodeManager->getModelFile().empty() );
            CPPUNIT_ASSERT ( true == barcodeManager->getSerialFile().empty() );
            CPPUNIT_ASSERT ( true == barcodeManager->getCommandsFile().empty() );
            CPPUNIT_ASSERT ( true == barcodeManager->getBufferFile().empty() );
        }

        void testGetConfigurationValues()
        {
            StoreMockBarcodeConfManager mock;
            BarcodeDeviceConfigurationManager * barcodeManager = BarcodeDeviceConfigurationManager::getInstance ( &mock );

            mock.config.setModelFile ( "model" );
            mock.config.setSerialFile ( "" );
            mock.config.setCommandsFile ( "" );
            mock.config.setBufferFile ( "" );

            CPPUNIT_ASSERT ( false == barcodeManager->simulationEnabled() );
            CPPUNIT_ASSERT ( "model" == barcodeManager->getModelFile() );
            CPPUNIT_ASSERT ( true == barcodeManager->getSerialFile().empty() );
            CPPUNIT_ASSERT ( true == barcodeManager->getCommandsFile().empty() );
            CPPUNIT_ASSERT ( true == barcodeManager->getBufferFile().empty() );

            mock.config.setModelFile ( "model" );
            mock.config.setSerialFile ( "serial" );
            mock.config.setCommandsFile ( "" );
            mock.config.setBufferFile ( "" );

            CPPUNIT_ASSERT ( false == barcodeManager->simulationEnabled() );
            CPPUNIT_ASSERT ( "model" == barcodeManager->getModelFile() );
            CPPUNIT_ASSERT ( "serial" == barcodeManager->getSerialFile() );
            CPPUNIT_ASSERT ( true == barcodeManager->getCommandsFile().empty() );
            CPPUNIT_ASSERT ( true == barcodeManager->getBufferFile().empty() );

            mock.config.setModelFile ( "model" );
            mock.config.setSerialFile ( "serial" );
            mock.config.setCommandsFile ( "commands" );
            mock.config.setBufferFile ( "" );

            CPPUNIT_ASSERT ( false == barcodeManager->simulationEnabled() );
            CPPUNIT_ASSERT ( "model" == barcodeManager->getModelFile() );
            CPPUNIT_ASSERT ( "serial" == barcodeManager->getSerialFile() );
            CPPUNIT_ASSERT ( "commands" == barcodeManager->getCommandsFile() );
            CPPUNIT_ASSERT ( true == barcodeManager->getBufferFile().empty() );

            mock.config.setModelFile ( "model" );
            mock.config.setSerialFile ( "serial" );
            mock.config.setCommandsFile ( "commands" );
            mock.config.setBufferFile ( "buffer" );

            CPPUNIT_ASSERT ( true == barcodeManager->simulationEnabled() );
            CPPUNIT_ASSERT ( "model" == barcodeManager->getModelFile() );
            CPPUNIT_ASSERT ( "serial" == barcodeManager->getSerialFile() );
            CPPUNIT_ASSERT ( "commands" == barcodeManager->getCommandsFile() );
            CPPUNIT_ASSERT ( "buffer" == barcodeManager->getBufferFile() );
        }

        CPPUNIT_TEST_SUITE ( BarcodeConfManagerTest );

        CPPUNIT_TEST ( testGetConfigurationList_ConfigurationFileIsEmpty );
        CPPUNIT_TEST ( testGetConfigurationList_ConfigEmpty );
        CPPUNIT_TEST ( testGetConfigurationList_ConfigValueNotFound );
        CPPUNIT_TEST ( testGetConfigurationList_PrimaryConfigValueFound );
        CPPUNIT_TEST ( testGetConfigurationList_SecundaryConfigValueFound );
        CPPUNIT_TEST ( testGetConfigurationList_SecundaryConfigValueFound_TwoCommands_Find_HR220 );
        CPPUNIT_TEST ( testGetConfigurationList_SecundaryConfigValueFound_TwoCommands_Find_MV110 );

        CPPUNIT_TEST ( testGetDefaultConfiguration_MockConfigurationIsEmpty );
        CPPUNIT_TEST ( testGetConfigurationValues );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_BARCODE_CONF_MANAGER_TEST */
