#ifndef TEST_BARCODE_READER_DEVICE_TEST
#define TEST_BARCODE_READER_DEVICE_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Devices/BarcodeReaderDevice.hxx>
#include <tests/MockConnection.hxx>
#include <tests/MockSerialPortConnection.hxx>
#include <tests/MockBarcodeReaderDevice.hxx>
#include <System/Utils/StringUtils.hxx>

class BarcodeReaderDeviceTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Utils::StringUtils StringUtils;
        typedef Smartmatic::SAES::HW::Devices::BarcodeReaderDevice BarcodeReaderDevice;
        typedef Smartmatic::System::SafeVector<unsigned char> ResponseVector;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

    public:

        BarcodeReaderDeviceTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testOpenConnectionFail ()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockConnection, openConnection, ext ) Open_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, closeConnection ) Close_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            Open_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( false ) );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::never() );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            CPPUNIT_ASSERT ( false == barcodeDevice.open ( "device" ) );

            connection->verify();
        }

        void testOpenConnectionSuccess ()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockConnection, openConnection, ext ) Open_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, closeConnection ) Close_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            Open_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( true ) );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::never() );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            CPPUNIT_ASSERT ( true == barcodeDevice.open ( "device" ) );

            connection->verify();
        }

        void testCloseConnectionFail ()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockConnection, openConnection, ext ) Open_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, closeConnection ) Close_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::never() );
            Close_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( false ) );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            CPPUNIT_ASSERT ( false == barcodeDevice.close() );

            connection->verify();
        }

        void testCloseConnectionSuccess ()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockConnection, openConnection, ext ) Open_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, closeConnection ) Close_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::never() );
            Close_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( true ) );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            CPPUNIT_ASSERT ( true == barcodeDevice.close() );

            connection->verify();
        }

        void testGetDeviceModel()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            ResponseVector response;

            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 'D' );
            response.push_back ( 'e' );
            response.push_back ( 'v' );
            response.push_back ( 'i' );
            response.push_back ( 'c' );
            response.push_back ( 'e' );
            response.push_back ( ' ' );
            response.push_back ( 'I' );
            response.push_back ( 'D' );
            response.push_back ( ':' );
            response.push_back ( ' ' );
            response.push_back ( 'h' );
            response.push_back ( 'r' );
            response.push_back ( 's' );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Send_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( true ) );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( "hrs" == barcodeDevice.getDeviceModel() );

            connection->verify();
        }

        void testGetDeviceSerial()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            ResponseVector response;

            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( 0x00 );
            response.push_back ( '1' );
            response.push_back ( '2' );
            response.push_back ( '3' );
            response.push_back ( 0x00 );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Send_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( true ) );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( "123" == barcodeDevice.getDeviceSerial() );

            connection->verify();
        }

        void testSetDeviceSerialFail()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            Send_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue<bool> ( false ) );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            CPPUNIT_ASSERT ( false == barcodeDevice.setDeviceSerial ( "SERIAL" ) );

            connection->verify();
        }

        void testSetDeviceSerialSuccess()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            Send_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( true ) );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            CPPUNIT_ASSERT ( true == barcodeDevice.setDeviceSerial ( "SERIAL" ) );

            connection->verify();
        }

        void testSendCommandFail ()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockConnection, openConnection, ext ) Open_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, closeConnection ) Close_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            ResponseVector response;

            response.push_back ( 0x15 );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( true ) );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( false == barcodeDevice.sendReconfigureCommand ( "NLS0203000", 0, 1, true ) );

            connection->verify();
        }

        void testSendCommandSuccess ()
        {
            MockConnection * connection = new MockConnection();
            BarcodeReaderDevice barcodeDevice ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockConnection, openConnection, ext ) Open_mock ( connection );
            MOCKPP_CHAINER_FOR_EXT ( MockConnection, sendCommand, ext ) Send_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, closeConnection ) Close_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            ResponseVector response;

            response.push_back ( 0x06 );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::once() ).will ( MOCKPP_NS::returnValue<bool> ( true ) );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::atLeastOnce() ).will ( MOCKPP_NS::returnValue < unsigned int > ( 0 ) );

            CPPUNIT_ASSERT ( true == barcodeDevice.sendReconfigureCommand ( "NLS0203000", 0, 1, true ) );

            connection->verify();
        }

        void testReadBufferSuccess ()
        {
            MockConnection * connection = new MockConnection();
            MockSerialPortConnection * serialPort = new MockSerialPortConnection ( connection );
            MockBarcodeReaderDevice barcodeDevice ( serialPort, false, false );
            serialPort->setListener ( &barcodeDevice );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, openConnection, ext ) Open_mock ( serialPort );
            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, sendCommand, ext ) Send_mock ( serialPort );
            MOCKPP_CHAINER_FOR ( MockSerialPortConnection, closeConnection ) Close_mock ( serialPort );

            ResponseVector response;

            response.push_back ( 'H' );
            response.push_back ( 'e' );
            response.push_back ( 'l' );
            response.push_back ( 'l' );
            response.push_back ( 'o' );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            SafeBuffer buffer = barcodeDevice.readBuffer();

            CPPUNIT_ASSERT ( true == ( buffer.GetSize() > 0 ) );

            ResponseVector vector;

            vector.insert ( vector.end(), buffer.GetData(), buffer.GetData() + buffer.GetSize() );

            CPPUNIT_ASSERT ( "Hello" == StringUtils::arrayHexToAsciiString ( vector ) );

            connection->verify();
        }

        void testReadBufferOnlyCommandResponse ()
        {
            MockConnection * connection = new MockConnection();
            MockSerialPortConnection * serialPort = new MockSerialPortConnection ( connection );
            MockBarcodeReaderDevice barcodeDevice ( serialPort, false, false );
            serialPort->setListener ( &barcodeDevice );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, openConnection, ext ) Open_mock ( serialPort );
            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, sendCommand, ext ) Send_mock ( serialPort );
            MOCKPP_CHAINER_FOR ( MockSerialPortConnection, closeConnection ) Close_mock ( serialPort );

            ResponseVector response;

            response.push_back ( 0x06 );  // success response

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            SafeBuffer buffer = barcodeDevice.readBuffer();

            CPPUNIT_ASSERT ( true == ( buffer.GetSize() == 0 ) );

            connection->verify();
        }

        void testReadBufferSendCommandSuccess ()
        {
            MockConnection * connection = new MockConnection();
            MockSerialPortConnection * serialPort = new MockSerialPortConnection ( connection );
            MockBarcodeReaderDevice barcodeDevice ( serialPort, false, true );
            serialPort->setListener ( &barcodeDevice );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, openConnection, ext ) Open_mock ( serialPort );
            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, sendCommand, ext ) Send_mock ( serialPort );
            MOCKPP_CHAINER_FOR ( MockSerialPortConnection, closeConnection ) Close_mock ( serialPort );

            ResponseVector response;

            response.push_back ( 0x06 );  // success response
            response.push_back ( 'H' );
            response.push_back ( 'e' );
            response.push_back ( 'l' );
            response.push_back ( 'l' );
            response.push_back ( 'o' );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            SafeBuffer buffer = barcodeDevice.readBuffer();

            CPPUNIT_ASSERT ( true == ( buffer.GetSize() > 0 ) );

            ResponseVector vector;

            vector.insert ( vector.end(), buffer.GetData(), buffer.GetData() + buffer.GetSize() );

            CPPUNIT_ASSERT ( "Hello" == StringUtils::arrayHexToAsciiString ( vector ) );

            connection->verify();
        }

        void testReadBufferSendCommandFail ()
        {
            MockConnection * connection = new MockConnection();
            MockSerialPortConnection * serialPort = new MockSerialPortConnection ( connection );
            MockBarcodeReaderDevice barcodeDevice ( serialPort, false, true );
            serialPort->setListener ( &barcodeDevice );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, openConnection, ext ) Open_mock ( serialPort );
            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, sendCommand, ext ) Send_mock ( serialPort );
            MOCKPP_CHAINER_FOR ( MockSerialPortConnection, closeConnection ) Close_mock ( serialPort );

            ResponseVector response;

            response.push_back ( 0x15 );  // success response
            response.push_back ( 'H' );
            response.push_back ( 'e' );
            response.push_back ( 'l' );
            response.push_back ( 'l' );
            response.push_back ( 'o' );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            SafeBuffer buffer = barcodeDevice.readBuffer();

            CPPUNIT_ASSERT ( true == ( buffer.GetSize() > 0 ) );

            ResponseVector vector;

            vector.insert ( vector.end(), buffer.GetData(), buffer.GetData() + buffer.GetSize() );

            CPPUNIT_ASSERT ( "Hello" == StringUtils::arrayHexToAsciiString ( vector ) );

            connection->verify();
        }

        void testReadBufferSendCommandOtherResponse ()
        {
            MockConnection * connection = new MockConnection();
            MockSerialPortConnection * serialPort = new MockSerialPortConnection ( connection );
            MockBarcodeReaderDevice barcodeDevice ( serialPort, false, true );
            serialPort->setListener ( &barcodeDevice );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, openConnection, ext ) Open_mock ( serialPort );
            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, sendCommand, ext ) Send_mock ( serialPort );
            MOCKPP_CHAINER_FOR ( MockSerialPortConnection, closeConnection ) Close_mock ( serialPort );

            ResponseVector response;

            response.push_back ( '.' );  // success response
            response.push_back ( 'H' );
            response.push_back ( 'e' );
            response.push_back ( 'l' );
            response.push_back ( 'l' );
            response.push_back ( 'o' );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            SafeBuffer buffer = barcodeDevice.readBuffer();

            CPPUNIT_ASSERT ( true == ( buffer.GetSize() > 0 ) );

            ResponseVector vector;

            vector.insert ( vector.end(), buffer.GetData(), buffer.GetData() + buffer.GetSize() );

            CPPUNIT_ASSERT ( ".Hello" == StringUtils::arrayHexToAsciiString ( vector ) );

            connection->verify();
        }

        void testReadBufferProcessCommand ()
        {
            MockConnection * connection = new MockConnection();
            MockSerialPortConnection * serialPort = new MockSerialPortConnection ( connection );
            MockBarcodeReaderDevice barcodeDevice ( serialPort, true, false );
            serialPort->setListener ( &barcodeDevice );

            MOCKPP_CHAINER_FOR ( MockConnection, readBuffer ) Read_mock ( connection );
            MOCKPP_CHAINER_FOR ( MockConnection, getTimeOut ) TimeOut_mock ( connection );

            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, openConnection, ext ) Open_mock ( serialPort );
            MOCKPP_CHAINER_FOR_EXT ( MockSerialPortConnection, sendCommand, ext ) Send_mock ( serialPort );
            MOCKPP_CHAINER_FOR ( MockSerialPortConnection, closeConnection ) Close_mock ( serialPort );

            ResponseVector response;

            response.push_back ( 0x00 );

            Open_mock.expects ( MOCKPP_NS::never() );
            Send_mock.expects ( MOCKPP_NS::never() );
            Read_mock.expects ( MOCKPP_NS::once() ).will ( new MOCKPP_NS::ReturnStub< ResponseVector > ( response ) );
            Close_mock.expects ( MOCKPP_NS::never() );
            TimeOut_mock.expects ( MOCKPP_NS::never() );

            SafeBuffer buffer = barcodeDevice.readBuffer();

            CPPUNIT_ASSERT ( true == ( buffer.GetSize() == 0 ) );

            connection->verify();
        }

        CPPUNIT_TEST_SUITE ( BarcodeReaderDeviceTest );

        CPPUNIT_TEST ( testOpenConnectionFail );
        CPPUNIT_TEST ( testOpenConnectionSuccess );

        CPPUNIT_TEST ( testCloseConnectionFail );
        CPPUNIT_TEST ( testCloseConnectionSuccess );

        CPPUNIT_TEST ( testGetDeviceModel );
        CPPUNIT_TEST ( testGetDeviceSerial );

        CPPUNIT_TEST ( testSetDeviceSerialFail );
        CPPUNIT_TEST ( testSetDeviceSerialSuccess );

        CPPUNIT_TEST ( testSendCommandFail );
        CPPUNIT_TEST ( testSendCommandSuccess );

        CPPUNIT_TEST ( testReadBufferSuccess );
        CPPUNIT_TEST ( testReadBufferOnlyCommandResponse );
        CPPUNIT_TEST ( testReadBufferSendCommandSuccess );
        CPPUNIT_TEST ( testReadBufferSendCommandFail );
        CPPUNIT_TEST ( testReadBufferSendCommandOtherResponse );

        CPPUNIT_TEST ( testReadBufferProcessCommand );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_BARCODE_READER_DEVICE_TEST */
