#ifndef TEST_ABLE_D_BUTTON_STATE_TEST
#define TEST_ABLE_D_BUTTON_STATE_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Operation/AbleDButtonState.hxx>

class AbleDButtonStateTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::HW::Operation::AbleDButtonState AbleDButtonState;

    public:

        AbleDButtonStateTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testGetDefaultButtonTypeValue ()
        {
            CPPUNIT_ASSERT ( 2 == AbleDButtonState::HELP );
            CPPUNIT_ASSERT ( 3 == AbleDButtonState::SELECT );
            CPPUNIT_ASSERT ( 4 == AbleDButtonState::UP );
            CPPUNIT_ASSERT ( 5 == AbleDButtonState::DOWN );
            CPPUNIT_ASSERT ( 6 == AbleDButtonState::RIGHT );
            CPPUNIT_ASSERT ( 7 == AbleDButtonState::LEFT );
            CPPUNIT_ASSERT ( 8 == AbleDButtonState::PAUSE );
            CPPUNIT_ASSERT ( 9 == AbleDButtonState::SPEED_UP );
            CPPUNIT_ASSERT ( 10 == AbleDButtonState::SPEED_DOWN );
            CPPUNIT_ASSERT ( 11 == AbleDButtonState::VOLUME_UP );
            CPPUNIT_ASSERT ( 12 == AbleDButtonState::VOLUME_DOWN );
            CPPUNIT_ASSERT ( 13 == AbleDButtonState::ACTIVATE );
        }

        void testGetDefaultButtonStateValue ()
        {
            CPPUNIT_ASSERT ( 1 == AbleDButtonState::NONE );
            CPPUNIT_ASSERT ( 2 == AbleDButtonState::PRESS );
            CPPUNIT_ASSERT ( 3 == AbleDButtonState::RELEASE );
            CPPUNIT_ASSERT ( 4 == AbleDButtonState::HOLD );
        }

        void testGetDefaultState ()
        {
            AbleDButtonState manager;

            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::HELP ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::SELECT ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::UP ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::DOWN ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::RIGHT ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::LEFT ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::PAUSE ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::SPEED_UP ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::SPEED_DOWN ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::VOLUME_UP ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::VOLUME_DOWN ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::ACTIVATE ) );
        }

        void testUpdateDefaultState ()
        {
            AbleDButtonState manager;

            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.getStatus ( AbleDButtonState::HELP ) );

            manager.updateStatus ( AbleDButtonState::HELP, AbleDButtonState::PRESS );
            CPPUNIT_ASSERT ( AbleDButtonState::PRESS == manager.getStatus ( AbleDButtonState::HELP ) );

            manager.updateStatus ( AbleDButtonState::HELP, AbleDButtonState::HOLD );
            CPPUNIT_ASSERT ( AbleDButtonState::HOLD == manager.getStatus ( AbleDButtonState::HELP ) );

            manager.updateStatus ( AbleDButtonState::HELP, AbleDButtonState::RELEASE );
            CPPUNIT_ASSERT ( AbleDButtonState::RELEASE == manager.getStatus ( AbleDButtonState::HELP ) );
        }

        void testCheckChangeState ()
        {
            AbleDButtonState manager;

            CPPUNIT_ASSERT ( false == manager.isChanged() );

            manager.updateStatus ( AbleDButtonState::HELP, AbleDButtonState::PRESS );

            CPPUNIT_ASSERT ( true == manager.isChanged() );
            CPPUNIT_ASSERT ( false == manager.isChanged() );

            manager.updateStatus ( AbleDButtonState::UP, AbleDButtonState::HOLD );

            CPPUNIT_ASSERT ( true == manager.isChanged() );
            CPPUNIT_ASSERT ( false == manager.isChanged() );
        }

        void testStringToButtonType ()
        {
            AbleDButtonState manager;

            CPPUNIT_ASSERT ( AbleDButtonState::HELP == manager.stringToButtonType ( " help " ) );
            CPPUNIT_ASSERT ( AbleDButtonState::HELP == manager.stringToButtonType ( " Help " ) );
            CPPUNIT_ASSERT ( AbleDButtonState::HELP == manager.stringToButtonType ( "Help" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::HELP == manager.stringToButtonType ( "HeLp" ) );

            CPPUNIT_ASSERT ( AbleDButtonState::SELECT == manager.stringToButtonType ( "SELECT" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::UP == manager.stringToButtonType ( "UP" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::DOWN == manager.stringToButtonType ( "DOWN" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::RIGHT == manager.stringToButtonType ( "RIGHT" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::LEFT == manager.stringToButtonType ( "LEFT" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::PAUSE == manager.stringToButtonType ( "PAUSE" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::SPEED_UP == manager.stringToButtonType ( "SPEED_UP" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::SPEED_DOWN == manager.stringToButtonType ( "SPEED_DOWN" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::VOLUME_UP == manager.stringToButtonType ( "VOLUME_UP" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::ACTIVATE == manager.stringToButtonType ( "ACTIVATE" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::VOLUME_DOWN == manager.stringToButtonType ( "VOLUME_DOWN" ) );

            CPPUNIT_ASSERT ( AbleDButtonState::UNKNOW == manager.stringToButtonType ( "x" ) );
        }

        void testStringToButtonState ()
        {
            AbleDButtonState manager;

            CPPUNIT_ASSERT ( AbleDButtonState::PRESS == manager.stringToButtonState ( " press " ) );
            CPPUNIT_ASSERT ( AbleDButtonState::PRESS == manager.stringToButtonState ( " Press " ) );
            CPPUNIT_ASSERT ( AbleDButtonState::PRESS == manager.stringToButtonState ( "Press" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::PRESS == manager.stringToButtonState ( "PrEsS" ) );

            CPPUNIT_ASSERT ( AbleDButtonState::HOLD == manager.stringToButtonState ( "HOLD" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::RELEASE == manager.stringToButtonState ( "RELEASE" ) );
            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.stringToButtonState ( "NONE" ) );

            CPPUNIT_ASSERT ( AbleDButtonState::NONE == manager.stringToButtonState ( "x" ) );
        }

        CPPUNIT_TEST_SUITE ( AbleDButtonStateTest );

        CPPUNIT_TEST ( testGetDefaultButtonTypeValue );
        CPPUNIT_TEST ( testGetDefaultButtonStateValue );
        CPPUNIT_TEST ( testGetDefaultState );
        CPPUNIT_TEST ( testUpdateDefaultState );
        CPPUNIT_TEST ( testCheckChangeState );

        CPPUNIT_TEST ( testStringToButtonType );
        CPPUNIT_TEST ( testStringToButtonState );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_ABLE_D_BUTTON_STATE_TEST */
