/**
 * @file	SmartmaticConfigurationManager.hxx
 *
 * @brief	Declares the smartmatic configuration manager class.
 */

#ifndef _SMARTMATIC_CONFIGURATION_MANAGER_H_
#define _SMARTMATIC_CONFIGURATION_MANAGER_H_

#include <string>
#include <vector>
#include <Configuration/SmartmaticConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace Configuration
    {
        /**
         * @class	SmartmaticConfigurationManager
         * @brief	Manager for smartmatic configurations class.
         * @author	Rafael.gasperi
         * @date	28/04/2011
         */
        class SmartmaticConfigurationManager
        {
            public:

                /**
                 * @fn	static SmartmaticConfigurationManager SmartmaticConfigurationManager::*getInstance( SmartmaticConfigurationManager * externManager = NULL );
                 * @brief	Gets the current Smartmatic Configuration Manager.
                 * @author	Yadickson Soto
                 * @date	05/02/2014
                 * @param 	externManager external manager to unit test
                 * @return	instance of the class
                 */
                static SmartmaticConfigurationManager * getInstance ( SmartmaticConfigurationManager * externManager = NULL );

                /**
                 * @fn	Smartmatic::Configuration::SmartmaticConfiguration * getConfiguration();
                 * @brief	Gets the configuration.
                 * @author	Youssef.tourkmani
                 * @date	29/10/2012
                 * @return	null if it fails, else the configuration.
                 */
                Smartmatic::Configuration::SmartmaticConfiguration * getConfiguration();

                /**
                 * @fn	void SmartmaticConfigurationManager::loadDefault();
                 * @brief	Loads the default configuration.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                void loadDefault();

                /**
                 * @fn	virtual SmartmaticConfigurationManager::~SmartmaticConfigurationManager();
                 * @brief	Class destructorer
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                virtual ~SmartmaticConfigurationManager();

            protected:

                /**
                 * @fn	SmartmaticConfigurationManager::SmartmaticConfigurationManager();
                 * @brief	Default constructor.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                SmartmaticConfigurationManager();

                /**
                 * @fn	virtual void SmartmaticConfigurationManager::initialize();
                 * @brief	Initializes this object.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                virtual void initialize();

            private:

                static SmartmaticConfigurationManager * instance; ///< The singleton instance
                Smartmatic::Configuration::SmartmaticConfiguration * configuration; ///< The configuration
        };
    }
}

#endif // _SMARTMATIC_CONFIGURATION_MANAGER_H_
