/**
 * @file	configuration-persistence-manager.h
 *
 * @brief	Header file to the configuration persistence manager class.
 */

#ifndef _CONFIGURATION_PERSISTENCE_MANAGER_H_
#define _CONFIGURATION_PERSISTENCE_MANAGER_H_

#include <string>
#include <Configuration/ConfigurationBase-schema.hxx>

namespace Smartmatic
{
    namespace Configuration
    {
        /**
         * @class	ConfigurationPersistenceManager
         * @brief	Manager for configuration persistences class.
         * @author	Rafael.gasperi
         * @date	28/04/2011
         */
        class ConfigurationPersistenceManager
        {
            public:

                /**
                 * @fn	~ConfigurationPersistenceManager();
                 * @brief	Destroy
                 * @author	Yadickson Soto
                 * @date	02/12/2013
                 */
                virtual ~ConfigurationPersistenceManager();

                /**
                     * @fn	static ConfigurationPersistenceManager * ConfigurationPersistenceManager::getInstance( ConfigurationPersistenceManager * externManager = NULL );
                     * @brief	Gets the current Configuration Persistence Manager.
                     * @author	Rafael.gasperi
                     * @date	28/04/2011
                     * @param 	externManager extern  manager only to unit test
                     * @return	The Configuration Persistence Manager.
                     */
                static ConfigurationPersistenceManager * getInstance ( ConfigurationPersistenceManager * externManager = NULL );

                /**
                 * @fn	std::string ConfigurationPersistenceManager::GetLibraryConfigFile( const std::string & nameOfLibrary );
                 * @brief	Gets a library configuration file.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @param	nameOfLibrary	Name of the library.
                 * @return	The library configuration file.
                 */
                std::string GetLibraryConfigFile ( const std::string & nameOfLibrary );

                /**
                 * @fn	void ConfigurationPersistenceManager::SetLibraryConfigFile( const std::string & nameOfLibrary,
                 * 			const std::string & on);
                 * @brief	Sets a library configuration file.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @param	nameOfLibrary	Name of the library.
                 * @param	on					 	The on.
                 */
                void SetLibraryConfigFile ( const std::string & nameOfLibrary, const std::string & on );

                /**
                 * @fn	bool SaveFile();
                 * @brief	Saves the file.
                 * @author	Yadickson Soto
                 * @date	02/12/2013
                 * @return 	true if save file is successfully
                 */
                bool SaveFile();

                static std::string filePath;  ///< Full pathname of the file

            protected:

                /**
                 * @fn	ConfigurationPersistenceManager::ConfigurationPersistenceManager();
                 * @brief	Default constructor.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                ConfigurationPersistenceManager();

                /**
                 * @fn	virtual void Initialize ( const std::string & filePath );
                 * @brief	Initializes this object.
                 * @author	Yadickson Soto
                 * @date	02/12/2013
                 * @param	filePath	Full pathname of the file.
                 */
                virtual void Initialize ( const std::string & filePath );

                /**
                 * @fn	Smartmatic::Configuration::ConfigurationBase * getConfiguration();
                 * @brief	Gets configuration base
                 * @author	Yadickson Soto
                 * @date	02/12/2013
                 * @return configuration base
                 */
                Smartmatic::Configuration::ConfigurationBase * getConfiguration();

            private:

                Smartmatic::Configuration::ConfigurationBase * configuration; ///< The configuration
                static ConfigurationPersistenceManager * instance;	///< The singleton instance
        };
    }
}

#endif // _CONFIGURATION_PERSISTENCE_MANAGER_H_
