/**
 * @file	base-language-controller.h
 *
 * @brief	Header file to the base language controller class.
 */

#ifndef _BASE_LANGUAGE_CONTROLLER_H_
#define _BASE_LANGUAGE_CONTROLLER_H_

#include <string>

namespace Smartmatic
{
    namespace Environment
    {
        /**
         * @class	BaseLanguageController
         * @brief	Base language controller class.
         * @author	Yadickson Soto
         * @date	28/04/2011
         */
        class BaseLanguageController
        {
            public:

                /**
                 * @fn	static void BaseLanguageController::LocalizeApplication(const std::string & newLanguage, const std::string & newCountry);
                 * @brief	Localize application Language.
                 * @author	Yadickson Soto
                 * @date	27/11/2013
                 * @param	newLanguage	The new language to be set.
                 * @param	newCountry 	The new country to be set.
                 */
                static void LocalizeApplication ( const std::string & newLanguage, const std::string & newCountry );

                /**
                 * @fn	static void BaseLanguageController::FullLocalizeApplication(std::string fullLanguage);
                 * @brief	Full localize application Language.
                 * @author	Yadickson.soto
                 * @date	14/05/2011
                 * @param	fullLanguage	The new language to be set.
                 */
                static void FullLocalizeApplication ( std::string fullLanguage );

                /**
                 * @fn	static void BaseLanguageController::GetActualLanguage(std::string& actualLanguage,
                 * 			std::string& actualCountry);
                 * @brief	Gets the actual application Language.
                 * @author	Yadickson Soto
                 * @date	28/04/2011
                 * @param	actualLanguage	The actual language found .
                 * @param	actualCountry 	The actual country found.
                 */
                static void GetActualLanguage ( std::string & actualLanguage, std::string & actualCountry );

                /**
                 * @fn	static std::string BaseLanguageController::getActualLanguage();
                 * @brief	Gets the actual application Language.
                 * @author	Yadickson Soto yadickson.soto@gmail.com
                 * @date	07/05/2011
                 * @return	actual application Language in format ll_CC.
                 */
                static std::string getActualLanguage();

                /**
                 * @fn	static bool isSetLanguage();
                 * @brief	Gets if set language is true.
                 * @author	Yadickson Soto yadickson.soto@gmail.com
                 * @date	27/11/2013
                 * @return	true if set language is successfully
                 */
                static bool isSetLanguage();
        };
    }
}

#endif // _BASE_LANGUAGE_CONTROLLER_H_
