/**
 * @file	resource-path-provider.cc
 *
 * @brief	resource path provider class.
 */

#include "resource-path-provider.h"
#include <Environment/Configurations/LanguageController.hxx>
#include <System/IO/Path.h>
#include <System/IO/File.h>
#include <System/IO/Directory.h>
#include <System/OptParser.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;

ResourcePathProvider * ResourcePathProvider::singleton_instance = NULL; ///< The resource path provider singleton instance

ResourcePathProvider::ResourcePathProvider()
{

}

ResourcePathProvider::~ResourcePathProvider()
{
    singleton_instance = NULL;
}

std::string ResourcePathProvider::GetResourcePath ( const std::string & libDirectory, const std::string & relativePath )
{
    std::string currentPath = Directory::getInstance()->getCurrentDirectory();
    std::string pathDirectory = Path::Combine ( currentPath, libDirectory );
    std::string pathFile;

    if ( Directory::getInstance()->DirectoryExists ( pathDirectory ) )
    {
        pathFile = Path::Combine ( pathDirectory, relativePath );
        pathFile = getResourcePath ( pathFile );
    }

    if ( !File::getInstance()->canOpenFile ( pathFile ) )
    {
        pathFile = OptParser::getInstance()->isSet ( OptParser::SHARE_DIR ) && OptParser::getInstance()->getArg ( OptParser::SHARE_DIR, pathFile ) ? pathFile : DATADIR;
        pathFile = Path::Combine ( pathFile, Path::Combine ( libDirectory, relativePath ) );
        pathFile = getResourcePath ( pathFile );
    }

    return pathFile;
}

std::string ResourcePathProvider::getResourcePath ( const std::string & path )
{
    std::string lang = LanguageController::getInstance()->getCurrentLanguage();
    std::string country = LanguageController::getInstance()->getCurrentCountry();
    return getResourcePathByLanguage ( path, lang, country );
}

std::string ResourcePathProvider::getResourcePathByLanguage ( const std::string & path, const std::string & lang, const std::string & country )
{
    std::string fullFile = Path::combineFileName ( path, lang + "_" + country );
    std::string singleFile = Path::combineFileName ( path, lang );
    std::string resultFile = path;

    if ( File::getInstance()->canOpenFile ( fullFile ) )
    {
        resultFile = fullFile;
    }
    else if ( File::getInstance()->canOpenFile ( singleFile ) )
    {
        resultFile = singleFile;
    }

    return resultFile;
}

ResourcePathProvider * ResourcePathProvider::getInstance ( ResourcePathProvider * externResource )
{
    if ( singleton_instance == NULL )
    {
        singleton_instance = ( externResource == NULL ? new ResourcePathProvider() : externResource );
    }

    return singleton_instance;
}
