/**
 * @file   AudioPlayerImpl.hxx
 * @brief  Declaration of AudioPlayerImpl class.
 * @author manuel.arguelles
 * @date   Oct  5, 2017
 */

#ifndef AUDIO_PLAYER_IMPL_HXX
#define AUDIO_PLAYER_IMPL_HXX

#include "Audio.hxx"
#include <string>
#include <ao/ao.h>
#include <glibmm.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Audio
        {
            /**
             * @class  AudioPlayerImpl
             * @brief  Audio player implementation.
             * @author manuel.arguelles
             * @date   Oct  5, 2017
             */
            class AudioPlayerImpl
            {
                public:

                    /**
                     * @brief  Creates and instance of the class.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     */
                    AudioPlayerImpl();

                    /**
                     * @brief  Destroys and instance of the class.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     */
                    ~AudioPlayerImpl();

                public:

                    /**
                     * @brief  Plays the audio.
                     * @author manuel.arguelles
                     * @date   Sep 29, 2017
                     * @throw  AudioException If the device is busy or cannot
                     * be opened.
                     * @param  audio The audio samples to play.
                     */
                    void play(const Audio& audio);

                    /**
                     * @brief  Opens the audio device.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @throw  AudioException If the device cannot be opened.
                     */
                    void openDevice();

                    /**
                     * @brief  Closes the audio device.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     */
                    void closeDevice();

                    /**
                     * @brief  Sets the output device to use.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @param  device The name of the device to use.
                     */
                    void setOutputDevice(const std::string& device);

                private:

                    Glib::Mutex      m_playLock;
                    ao_device*       m_device;
                    ao_option*       m_options;
                    ao_sample_format m_format;
            };

        } /* namespace Audio */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* AUDIO_PLAYER_IMPL_HXX */
