/**
 * @file   MixerControl.cxx
 * @brief  Implementation of MixerControl class.
 * @author manuel.arguelles
 * @date   Dec 13, 2017
 */

#include "MixerControl.hxx"
#include "MixerControlImpl.hxx"

using namespace Smartmatic::System::Audio;

static const int VOLUME_INCREMENT = 5;

MixerControl::MixerControl(const std::string& cardName, const int defaultVolume)
    : m_impl(new MixerControlImpl(cardName, defaultVolume))
{
}

MixerControl::~MixerControl()
{
}

void MixerControl::setPlaybackVolume(const int volume)
{
    m_impl->setPlaybackVolume(volume);
}

int MixerControl::getPlaybackVolume()
{
    return m_impl->getPlaybackVolume();
}

void MixerControl::increasePlaybackVolume()
{
    int currentVolume = getPlaybackVolume();

    setPlaybackVolume(currentVolume + VOLUME_INCREMENT);
}

void MixerControl::decreasePlaybackVolume()
{
    int currentVolume = getPlaybackVolume();

    setPlaybackVolume(currentVolume - VOLUME_INCREMENT);
}

std::string MixerControl::getPlaybackDeviceName() const
{
    return m_impl->getPlaybackDeviceName();
}
