/**
 * @file   ITextToSpeech.hxx
 * @brief  Declaration of TextToSpeech interface.
 * @author manuel.arguelles
 * @date   Oct 10, 2017
 */

#ifndef I_TEXT_TO_SPEECH_HXX
#define I_TEXT_TO_SPEECH_HXX

#include <string>

namespace Smartmatic
{
    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {

                /**
                 * @class  ITextToSpeech
                 * @brief  TextToSpeech interface class.
                 * @author manuel.arguelles
                 * @date   Oct 10, 2017
                 */
                class ITextToSpeech
                {
                    public:

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 10, 2017
                         */
                        virtual ~ITextToSpeech() {}

                    public:

                        /**
                         * @brief  Set the language to use.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  language The language to use.
                         * @return True on success, false on error.
                         */
                        virtual bool setLanguage(const std::string& language) = 0;

                        /**
                         * @brief  Sets the speech rate.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  rate The speed rate in percentage, 100 is
                         * normal. Valid values are 50-400.
                         */
                        virtual bool setSpeechRate(int rate) = 0;

                        /**
                         * @brief  Speak the given text.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  text The text to speak.
                         */
                        virtual void speak(const std::string& text) = 0;

                        /**
                         * @brief  Stops the processing of text.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         */
                        virtual void stop() = 0;

                        /**
                         * @brief  Tell if the engine is speaking.
                         * @author manuel.arguelles
                         * @date   Oct 10, 2017
                         * @return True if speaking, false if not.
                         */
                        virtual bool isSpeaking() const = 0;

                        /**
                         * @brief  Wait while speaking.
                         * @author manuel.arguelles
                         * @date   Dec 18, 2017
                         */
                        virtual void waitWhileSpeaking() const = 0;
                };

            } /* namespace TextToSpeech */
        } /* namespace Audio */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* I_TEXT_TO_SPEECH_HXX */
