/**
 * @file	ICompression.hxx
 * @brief	Header compression interface.
 * @author	Yadickson Soto
 * @date	08/11/2012
 */

#ifndef _ICOMPRESION_H_
#define _ICOMPRESION_H_

#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Compression
        {
            /**
             * @class	ICompression
             * @brief	Compression interface.
             * @author	Yadickson Soto
             * @date	08/11/2012
             */
            class ICompression
            {
                public:

                    /**
                     * @brief	Class constructor
                     * @author	Yadickson Soto
                     * @date	08/11/2012
                     */
                    ICompression();

                    /**
                     * @brief	Class destructor
                     * @author	Yadickson Soto
                     * @date	08/11/2012
                     */
                    virtual ~ICompression();

                    /**
                     * @fn	virtual bool ICompression::Inflate( Smartmatic::System::SafeBuffer & dataIn,
                     *		Smartmatic::System::SafeBuffer & dataOut, int & avail) = 0;
                     * @brief	Inflates.
                     * @author	Yadickson Soto
                     * @date	08/11/2012
                     * @param	dataIn				The data in.
                     * @param	dataOut				The data out result.
                     * @param	avail				availability data.
                     * @return	true if success
                     */
                    virtual bool Inflate ( Smartmatic::System::SafeBuffer & dataIn,
                                           Smartmatic::System::SafeBuffer & dataOut,
                                           int & avail ) = 0;

                    /**
                     * @fn	virtual bool ICompression::Deflate ( Smartmatic::System::SafeBuffer & dataIn,
                     *              Smartmatic::System::SafeBuffer & dataOut ) = 0;
                     * @brief	Deflates.
                     * @author	Yadickson Soto
                     * @date	08/11/2012
                     * @param	dataIn				The data in.
                     * @param	dataOut				The data out result.
                     * @return	true if success
                     */
                    virtual bool Deflate ( Smartmatic::System::SafeBuffer & dataIn,
                                           Smartmatic::System::SafeBuffer & dataOut ) = 0;
            };
        }
    }
}
#endif // _ICOMPRESION_H_
