/**
 * @file Base64Encoder.cpp
 * @brief Body class base64 encoder
 * @date 05/05/2011
 * @author Yadickson Soto
 */

#include "Base64Encoder.h"
#include "B64.hxx"
#include <stdlib.h>
#include <stdint.h>

using namespace Smartmatic::System::Encoding;

bool Base64Encoder::ConvertToBase64 ( const Smartmatic::System::SafeBuffer & buffer, std::string & result )
{
    bool encoder = false;
    result.clear();

    const unsigned char * data = buffer.GetData();
    unsigned int dataLen = buffer.GetSize();

    if ( data != NULL && dataLen > 0 )
    {
        B64 b64;
        unsigned int outputLen;
        unsigned char * base64_buff = b64.encode ( data, dataLen, outputLen );

        if ( base64_buff )
        {
            result.assign ( ( const char * ) base64_buff, outputLen );
            free ( base64_buff );
            encoder = true;
        }
    }

    return encoder;
}

bool Base64Encoder::ConvertFromBase64 ( const std::string & base64String, Smartmatic::System::SafeBuffer & buffer )
{
    bool encoder = false;
    buffer.Clear();

    if ( TestBase64String ( base64String ) )
    {
        B64 b64;
        unsigned int sizeData = 0;
        unsigned char * data = b64.decode ( base64String.c_str(), base64String.length(), sizeData );
        buffer.Clear();

        if ( data )
        {
            buffer.AddData ( data, sizeData );
            free ( data );
            data = NULL;
            encoder = true;
        }
    }

    return encoder;
}

bool Base64Encoder::TestBase64String ( const std::string & testString )
{
    unsigned int bufferResultsize = testString.size();
    unsigned int sizeData = 0;

    if ( bufferResultsize > 0 )
    {
        B64 b64;
        unsigned char * data = ( unsigned char * ) b64.decode ( testString.c_str(), testString.length(), sizeData );

        if ( data != NULL )
        {
            free ( data );
            data = NULL;
        }
    }

    return sizeData > 0;
}
