/**
 * @file Base64Encoder.h
 * @brief Header class base64 encoder
 * @date 05/05/2011
 * @author Yadickson Soto
 */

#ifndef BASE64ENCODER_H_
#define BASE64ENCODER_H_

#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Encoding
        {
            /**
             * @class	Base64Encoder
             * @brief	Class definition file.
             * @author	Freddy Gomez &lt;fgomez@smartmatic.com&gt;
             * @date	05-05-2011
             */
            class Base64Encoder
            {
                public:

                    /**
                     * @fn	static bool Base64Encoder::ConvertToBase64(const Smartmatic::System::SafeBuffer & buffer, std::string & result);
                     * @brief	Encode a byte array in a base 64 string.
                     * @author	Yadickson Soto
                     * @date	05/05/2011
                     * @param	buffer	The buffer.
                     * @param	result result string base 64
                     * @return	true if success convert to base64
                     */
                    static bool ConvertToBase64 ( const Smartmatic::System::SafeBuffer & buffer, std::string & result );

                    /**
                     * @fn	static bool Base64Encoder::ConvertFromBase64(const std::string & base64String,
                     * 			Smartmatic::System::SafeBuffer & buffer);
                     * @brief	Decode a base 64 string .
                     * @author	Freddy Gomez
                     * @date	04/05/2011
                     * @param [in]	base64String	a base 64 strign to decode.
                     * @param	buffer result.
                     * @return	true whether the string is in base 64 format.
                     */
                    static bool ConvertFromBase64 ( const std::string & base64String, Smartmatic::System::SafeBuffer & buffer );

                    /**
                     * @fn	static bool Base64Encoder::TestBase64String( const std::string & testString );
                     * @brief	This method run the process specified in the command line using the popen/pclose function.
                     * @author	Freddy Gomez
                     * @date	04/05/2011
                     * @param [in]	testString	A string to test.
                     * @return	true whether the string is in base 64 format.
                     */
                    static bool TestBase64String ( const std::string & testString );
            };
        }
    }
}
#endif /* BASE64ENCODER_H_ */
