/**
 * @file   AudioException.hxx
 * @brief  Declaration of AudioException class.
 * @author manuel.arguelles
 * @date   Oct  5, 2017
 */

#ifndef AUDIO_EXCEPTION_HXX
#define AUDIO_EXCEPTION_HXX

#include "BaseException.hxx"

namespace Smartmatic
{
    namespace System
    {
        namespace Exception
        {
            /**
             * @class  AudioException
             * @brief  Audio exceptions.
             * @author manuel.arguelles
             * @date   Oct  5, 2017
             */
            class AudioException : public BaseException
            {
                public:

                    /**
                     * @brief  Creates an instance of the class.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @param  message The exception message.
                     * @param  code    The exception code.
                     * @param  logger  The logger.
                     */
                    AudioException(const std::string& message,
                                   const std::string& code,
                                   Smartmatic::Log::ISMTTLog* logger = NULL)
                        : BaseException(message, code, logger) {}

                    /**
                     * @brief  Destroys an instance of the class.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     */
                    virtual ~AudioException() throw() {}
            };

        } /* namespace Exception */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* AUDIO_EXCEPTION_HXX */
