/**
 * @file	INotifyFile.hxx
 * @brief	Interface to the check changes in file
 * @date 	05/05/2015
 * @author 	Yadickson Soto
 */

#ifndef INOTIFY_FILE_HXX_
#define INOTIFY_FILE_HXX_

#include <string>
#include <System/SafeList.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace System
    {
        namespace IO
        {
            /**
             * @class	INotifyFile
             * @brief	Class util to check change in files.
             * @author	Yadickson Soto
             * @date	05/05/2015
             */
            class INotifyFile
            {
                public:

                    /**
                     * @enum Status
                     * @brief All status collection to check
                     * @author	Yadickson Soto
                     * @date	05/05/2015
                     */
                    enum Status
                    {
                        NONE	= 0x00,
                        CREATE	= 0x01,
                        MODIFY	= 0x02,
                        DELETE	= 0x04
                    };

                    /**
                     * @struct	StatusNotify
                     * @brief	Structure to make status by file path.
                     * @author	Yadickson Soto
                     * @date	05/05/2015
                     */
                    struct StatusNotify
                    {
                        StatusNotify ( const std::string & path, int status ) :
                            path ( path ), status ( status ) { }

                        std::string path;
                        int status;
                    };

                    /**
                     * @fn	INotifyFile();
                     * @brief	Default Constructor.
                     * @author	Yadickson Soto
                     * @date	05/05/2015
                     */
                    INotifyFile();

                    /**
                     * @fn	virtual ~INotifyFile();
                     * @brief	Class destructor.
                     * @author	Yadickson Soto
                     * @date	05/05/2015
                     */
                    virtual ~INotifyFile();

                    /**
                     * @fn	virtual void initialize();
                     * @brief	Initialize inotify controller
                     * @author	Yadickson Soto
                     * @date	05/05/2015
                     */
                    virtual void initialize();

                    /**
                     * @fn	virtual void dispose();
                     * @brief	Clear all watch dog
                     * @author	Yadickson Soto
                     * @date	05/05/2015
                     */
                    virtual void dispose();

                    /**
                     * @fn	void addPath ( const std::string & path );
                     * @brief	Add path file to check status.
                     * @param	path path file to check status
                     * @author	Yadickson Soto
                     * @date	05/05/2015
                     */
                    virtual void addPath ( const std::string & path );

                    /**
                     * @fn	virtual Smartmatic::System::SafeList < StatusNotify > getStatus ();
                     * @brief	Gets all file path status all watch dog
                     * @author	Yadickson Soto
                     * @date	05/05/2015
                     */
                    virtual Smartmatic::System::SafeList < StatusNotify > getStatus ();

                private:

                    int handle; ///< The handle to initialize the services
                    std::list < int > watchDogList; ///< The watch dog list from add path event
                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger
            };
        }
    }
}

#endif /* INOTIFY_FILE_HXX_ */
