/**
 * @file ProcessMonitor.cpp
 * @brief Body process monitor class
 * @date 04/05/2011
 * @author Freddy Gomez
 */

#include "System/Runtime/ProcessMonitor.h"
#include "System/Runtime/ProcessLauncher.h"
#include <sstream>
#include <unistd.h>

using namespace Smartmatic::System::Runtime;

int ProcessMonitor::CountProcessRunning ( const std::string & processSearchPattern )
{
    std::string command = "ps -A";

    if ( processSearchPattern.size() > 0 && processSearchPattern != "*" )
    {
        command.append ( " | grep " );
        command.append ( processSearchPattern );
    }

    std::list<std::string> out = ProcessLauncher::getInstance()->RunProcessThroughPipe ( command );
    return out.size();
}

void ProcessMonitor::KillProcess ( int pid )
{
    std::stringstream command;
    command << "kill -9 " << pid;
    ProcessLauncher::getInstance()->RunProcessThroughSystem ( command.str() );
}

void ProcessMonitor::KillProcess ( const std::string & processName, bool background )
{
    std::stringstream command;
    command << "pkill -9 " << processName;
    ProcessLauncher::getInstance()->RunProcessThroughSystem ( command.str(), background );
}

void ProcessMonitor::KillProcessAndWait ( const std::string & processName, int count )
{
    do
    {
        KillProcess ( processName );
        sleep ( 1 );
        count--;
    }
    while ( count > 0 && CountProcessRunning ( processName ) > 0 );
}
