/**
 * @file SafeThread.cxx
 * @brief Body class thread safe
 * @date 07/06/2015
 * @author Yadickson Soto
 */

#include "SafeThread.hxx"
#include <unistd.h>

using namespace Smartmatic::System;

SafeThread::SafeThread()
{
    this->flagRun = false;
    this->flagThreadRun = false;
    this->m_thread = NULL;
    this->valueTime = 0;
}

SafeThread::~SafeThread()
{
    stop ();
}

void SafeThread::start ( unsigned int valueTime )
{
    this->valueTime = valueTime;

    if ( !Glib::thread_supported() )
    {
        Glib::thread_init();
    }

    stop();

    flagThreadRun = true;
    m_thread = Glib::Thread::create ( sigc::mem_fun ( *this, &SafeThread::onThread ), true );
}

void SafeThread::stop()
{
    flagThreadRun = false;
    Glib::Mutex::Lock lock ( mutex );

    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }
}

bool SafeThread::isRunning()
{
    return flagRun;
}

void SafeThread::onThread ()
{
    Glib::Mutex::Lock lock ( mutex );

    preStart();
    flagRun = true;

    while ( flagThreadRun )
    {
        if ( !process() )
        {
            break;
        }

        usleep ( valueTime * 1000 );
    }

    flagRun = false;
    postStop();
}

void SafeThread::preStart ()
{

}

void SafeThread::postStop ()
{

}
