/**
 * @file SafeTime.hxx
 * @brief Header safe time class
 * @date 12/03/2013
 * @author Yadickson Soto
 */

#ifndef SAFETIME_HXX_
#define SAFETIME_HXX_

#include <string>
#include <time.h>

namespace Smartmatic
{
    namespace System
    {

        /**
         * @class SafeTime
         * @brief safe time class
         * @date 12/03/2013
         * @author Yadickson Soto
         */
        class SafeTime
        {
            public:

                /**
                 * @fn SafeTime();
                 * @brief Class constructor
                 * @date 12/03/2013
                 * @author Yadickson Soto
                 */
                SafeTime();

                /**
                 * @fn virtual ~SafeTime();
                 * @brief Class destructor
                 * @date 12/03/2013
                 * @author Yadickson Soto
                 */
                virtual ~SafeTime();

                /**
                 * @fn	void SafeTime::start();
                 * @brief	Method start time process
                 * @author	Yadickson Soto
                 * @date	27/03/2013
                 */
                void start();

                /**
                 * @fn	void SafeTime::stop();
                 * @brief	Method stop time process
                 * @author	Yadickson Soto
                 * @date	27/03/2013
                 */
                void stop();

                /**
                 * @fn	std::string SafeTime::getStringTime ();
                 * @brief	Method getter string time process
                 * @author	Yadickson Soto
                 * @date	27/03/2013
                 * @return time process
                 */
                std::string getStringTime ();

                /**
                 * @fn	unsigned int getHour ();
                 * @brief	Method getter hour
                 * @author	Yadickson Soto
                 * @date	03/04/2013
                 * @return hour
                 */
                unsigned int getHour ();

                /**
                 * @fn	unsigned int getMinute ();
                 * @brief	Method getter minutes
                 * @author	Yadickson Soto
                 * @date	03/04/2013
                 * @return minute
                 */
                unsigned int getMinute ();

                /**
                 * @fn	unsigned int getSecond ();
                 * @brief	Method getter seconds
                 * @author	Yadickson Soto
                 * @date	03/04/2013
                 * @return seconds
                 */
                unsigned int getSecond ();

                /**
                 * @fn	unsigned int getAllSeconds ();
                 * @brief	Method getter all seconds
                 * @author	Yadickson Soto
                 * @date	03/04/2013
                 * @return seconds
                 */
                unsigned int getAllSeconds ();

            private:

                time_t initTime; ///< init time
                time_t endTime; ///< end time
                time_t resultTime; ///< result time
        };
    }
}

#endif /* SAFETIME_HXX_ */
