/**
 * @file ComposeKey.hxx
 * @brief Header class utility for compose key
 * @date 06/02/2013
 * @author Yadickson Soto
 */

#ifndef COMPOSEKEY_HXX_
#define COMPOSEKEY_HXX_

#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            /**
             * @class ComposeKey
             * @brief Class utility for compose key
             * @date 06/02/2013
             * @author Yadickson Soto
             */
            class ComposeKey
            {
                public:

                    /**
                     * @fn ComposeKey( );
                     * @brief Class constructor
                     * @date 06/02/2013
                     * @author Yadickson Soto
                     */
                    ComposeKey ( );

                    /**
                     * @fn ComposeKey( const Smartmatic::System::SafeBuffer & key );
                     * @param key key to compose
                     * @brief Class constructor
                     * @date 06/02/2013
                     * @author Yadickson Soto
                     */
                    explicit ComposeKey ( const Smartmatic::System::SafeBuffer & key );

                    /**
                     * @fn virtual ~ComposeKey();
                     * @brief Class destructor
                     * @date 06/02/2013
                     * @author Yadickson Soto
                     */
                    virtual ~ComposeKey();

                    /**
                     * @fn void setKey ( const unsigned char key [], unsigned int length );
                     * @param key key to compose
                     * @param length length
                     * @brief Method setter key
                     * @date 06/02/2013
                     * @author Yadickson Soto
                     */
                    void setKey ( const unsigned char key [], unsigned int length );

                    /**
                     * @fn void setKey ( const Smartmatic::System::SafeBuffer & key );
                     * @param key key to compose
                     * @brief Method setter key
                     * @date 06/02/2013
                     * @author Yadickson Soto
                     */
                    void setKey ( const Smartmatic::System::SafeBuffer & key );

                    /**
                     * @fn void getCompose ( unsigned int length, Smartmatic::System::SafeBuffer & result );
                     * @param length length result
                     * @param result result key
                     * @brief Method getter compose key
                     * @date 06/02/2013
                     * @author Yadickson Soto
                     */
                    void getCompose ( unsigned int length, Smartmatic::System::SafeBuffer & result );

                private:

                    Smartmatic::System::SafeBuffer key; ///< key
            };
        }
    }
}

#endif /* COMPOSEKEY_HXX_ */
