/**
 * @file CertificateExtensionPEM.h
 * @brief Header class certificate PEM extension.
 * @date 18/12/2013
 * @author Beatriz Avila
 */

#ifndef CERTIFICATEEXTENSIONPEM_HXX_
#define CERTIFICATEEXTENSIONPEM_HXX_

#include <System/Security/Cryptography/CertificateExtensionBase.hxx>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	CertificateExtensionPEM
                 * @brief	Class for CertificateExtensionPEM.
                 * @author	Beatriz Avila
                 * @date	18/12/2013
                 */
                class CertificateExtensionPEM : public CertificateExtensionBase
                {
                    public:

                        /**
                         * @fn	CertificateExtensionPEM::CertificateExtensionPEM( const Smartmatic::System::SafeBuffer & certificateBuffer,
                         * 		const std::string & certPassword = "");
                         * @brief	Class constructor.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @param	certificateBuffer	SafeBuffer with certificate.
                         * @param	certPassword		password.
                         */
                        CertificateExtensionPEM ( const Smartmatic::System::SafeBuffer & certificateBuffer, const std::string & certPassword = "" );

                        /**
                         * @fn	virtual CertificateExtensionPEM::~CertificateExtensionPEM();
                         * @brief	Class destructor.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         */
                        virtual ~CertificateExtensionPEM ( );

                    public:

                        /**
                         * @fn     CertificateExtensionPEM* clone() const;
                         * @brief  Duplicates the instance of the CertificateExtensionPEM class.
                         * @return The new instance.
                         * @author manuel.arguelles
                         * @date   08/08/2022
                         */
                        virtual CertificateExtensionPEM* clone() const;

                    protected:
                        /**
                         * @fn virtual	X509 * CertificateExtensionPEM::LoadCertifiedX509FromBIO();
                         * @brief	Method load certified file from BIO.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @return	certificate.
                         */
                        virtual X509 * LoadCertifiedX509FromBIO ( );

                        /**
                         * @fn virtual	EVP_PKEY * CertificateExtensionPEM::LoadCertifiedPrivateKeyFromBIO();
                         * @brief	Method load private key from BIO.
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @return	private key.
                         */
                        virtual EVP_PKEY * LoadCertifiedPrivateKeyFromBIO ( );

                        /**
                         * @fn virtual	stack_st_X509 * CertificateExtensionPEM::getCertificateAuthoritiesFromBIO();
                         * @brief	Method get certificate authorities from BIO
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @return	certificate authorities.
                         */
                        virtual stack_st_X509 * getCertificateAuthoritiesFromBIO ( );

                        /**
                         * @fn static int CertificateExtensionPEM::passw_callback ( char * buf, int bufsiz, int verify, PW_CB_DATA * cb_tmp )
                         * @brief method pass callback
                         * @author	Beatriz Avila
                         * @date	18/12/2013
                         * @param buf buffer data
                         * @param bufsiz buffer size
                         * @param verify flag
                         * @param password password
                         * @return status
                         */
                        static int passw_callback ( char * buf, int bufsiz, int verify, char * password );

                    private:
                        static Smartmatic::Log::ISMTTLog * logger; ///<logger
                };
            }
        }
    }
}

#endif /* CERTIFICATEEXTENSIONPEM_HXX_ */
