/**
 * @file PBEKeyDerivatorHMAC.h
 * @brief Header Pbe key derivator hmac class
 * @date 10/11/2011
 * @author Freddy Gomez
 */


#ifndef PBEKEYDERIVATORHMAC_H_
#define PBEKEYDERIVATORHMAC_H_

#include <string>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/DigestAlgorithmLookup.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class  PBEKeyDerivatorHMAC
                 * @brief  Pbe key derivator hmac class.
                 * @author freddyg
                 * @date   28/04/2011
                 */
                class PBEKeyDerivatorHMAC
                {
                    public:

                        /**
                         * @brief  Default constructor.
                         * @author freddyg
                         * @date   28/04/2011
                         */
                        PBEKeyDerivatorHMAC();

                        /**
                         * @brief  Finaliser.
                         * @author freddyg
                         * @date   28/04/2011
                         */
                        virtual ~PBEKeyDerivatorHMAC();

                        /**
                         * @brief  Generates a key.
                         * @author Freddy Gomez, Yadickson Soto
                         * @date   28/04/2011
                         * @throw  CryptoException If error occurs of empty passphrase is passed.
                         * @param  passPhrase The pass phrase.
                         * @param  outkey     If non-null, the outkey.
                         * @param  keyLen     Length of the key.
                         * @param  iterations The iterations.
                         * @param  mdAlg      The md algorithm.
                         */
                        void GenerateKey (
                            const Smartmatic::System::SafeBuffer & passPhrase,
                            Smartmatic::System::SafeBuffer & outkey,
                            MsgDigestUtil::DigestAlgType mdAlg,
                            unsigned int keyLen = 32,
                            unsigned int iterations = 2048 );

                        /**
                         * @brief  Sets a salt.
                         * @author Yadickson Soto
                         * @date   21/01/2014
                         * @param salt The salt.
                         */
                        void SetSalt ( const Smartmatic::System::SafeBuffer & salt );

                    private:

                        Smartmatic::System::SafeBuffer salt; ///< The salt value

                };
            }
        }
    }
}
#endif /* PBEKEYDERIVATORHMAC_H_ */
