/**
 * @file OpenSSLCertificate.hxx
 * @brief Header class openssl certificate utility
 * @date 08/02/2013
 * @author Yadickson Soto
 */

#ifndef OPENSSLCERTIFICATE_HXX_
#define OPENSSLCERTIFICATE_HXX_

#include <System/SafeBuffer.h>
#include <xsec/enc/OpenSSL/OpenSSLCryptoX509.hpp>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            /**
             * @class OpenSSLCertificate
             * @brief class detached signature
             * @date 08/02/2013
             * @author Yadickson Soto
             */
            class OpenSSLCertificate
            {
                public:

                    /**
                     * @fn OpenSSLCertificate ( Smartmatic::System::SafeBuffer & buffer );
                     * @brief Class constructor
                     * @date 08/02/2013
                     * @author Yadickson Soto
                     */
                    explicit OpenSSLCertificate ( Smartmatic::System::SafeBuffer & buffer );

                    /**
                     * @fn virtual ~OpenSSLCertificate();
                     * @brief Class destructor
                     * @date 08/02/2013
                     * @author Yadickson Soto
                     */
                    virtual ~OpenSSLCertificate();

                    /**
                     * @fn	OpenSSLCryptoX509 * getX509 ( );
                     * @brief	method getter x509 certificate
                     * @author	Yadickson Soto
                     * @date	09/02/2013
                     * @return	x509 certificate
                     */
                    OpenSSLCryptoX509 * getX509 ( );

                private:

                    /**
                     * @brief  Disable copy constructor.
                     * @author manuel.arguelles
                     * @date   Sep 12, 2017
                     */
                    OpenSSLCertificate(const OpenSSLCertificate&);

                    /**
                     * @brief  Disable copy.
                     * @author manuel.arguelles
                     * @date   Sep 12, 2017
                     */
                    OpenSSLCertificate& operator=(const OpenSSLCertificate&);

                private:

                    OpenSSLCryptoX509 * x509; ///< x509 certificate
            };
        }
    }
}

#endif /* OPENSSLCERTIFICATE_HXX_ */
