/**
 * @file	beep-manager.h
 *
 * @brief	Header file to the beep manager class.
 */

#ifndef _BEEP_DESCRIPTOR_H_
#define _BEEP_DESCRIPTOR_H_

#include <vector>
#include "System/Sounds/beep-descriptor.h"

namespace Smartmatic
{
    namespace System
    {
        namespace Sounds
        {
            /**
             * @class	BeepManager
             * @brief	Manager for beeps class.
             * @author	Rafael.gasperi
             * @date	28/04/2011
             */

            class BeepManager
            {
                public:

                    /**
                     * @fn	void BeepManager::AddBeep(BeepDescriptor beepDescriptor);
                     * @brief	Adds a beep.
                     * @author	Rafael.gasperi
                     * @date	28/04/2011
                     * @param	beepDescriptor	The beep descriptor.
                     */
                    void AddBeep ( BeepDescriptor beepDescriptor );

                    /**
                     * @fn	void BeepManager::Beep();
                     * @brief	Beeps this object.
                     * @author	Rafael.gasperi
                     * @date	28/04/2011
                     */
                    void Beep();

                    /**
                     * @fn	void BeepManager::Clear();
                     * @brief	Clears this object to its blank/initial state.
                     * @author	Rafael.gasperi
                     * @date	28/04/2011
                     */
                    void Clear();

                    /**
                     * @fn	void Stop ();
                     * @brief	Stop service sounds.
                     * @author	Yadickson Soto
                     * @date	20/06/2013
                     */
                    void Stop ();

                private:

                    std::vector<BeepDescriptor> beeps; ///< The beeps
            };
        }
    }
}

#endif // _BEEP_DESCRIPTOR_H_
