/**
 * @file DateTimeUtils.cxx
 * @brief
 * @date 15/07/2013
 * @author Yadickson Soto
 */

#include "DateTimeUtils.hxx"
#include "StringUtils.hxx"
#include <string.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <time.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::System::Utils;
using namespace Smartmatic::Log;

ISMTTLog * DateTimeUtils::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Utils.DateTimeUtils", GETTEXT_PACKAGE );

DateTimeUtils::DateTimeUtils ( const std::string & day, const std::string & month, const std::string & year )
{
    input.append ( StringUtils::intToString ( StringUtils::stringToInt ( day ), 2 ) );
    input.append ( "/" );
    input.append ( StringUtils::intToString ( StringUtils::stringToInt ( month ), 2 ) );
    input.append ( "/" );
    input.append ( StringUtils::intToString ( StringUtils::stringToInt ( year ), 4 ) );

    parse ( input );
}

DateTimeUtils::DateTimeUtils ( const std::string & stringTime )
{
    input = StringUtils::replaceAll ( stringTime, "-", "/" );
    parse ( input );
}

DateTimeUtils::~DateTimeUtils()
{

}

void DateTimeUtils::parse ( const std::string & stringTime )
{
    logger->Debug ( "Parse : " + stringTime );

    std::string strTime = StringUtils::replaceAll ( stringTime, "T", " " );

    _time.tm_hour = 0;
    _time.tm_min = 0;
    _time.tm_sec = 0;
    _time.tm_year = 0;
    _time.tm_mon = 0;
    _time.tm_wday = 0;
    _time.tm_mday = 1;
    _time.tm_zone = ( const char * ) "";

    if ( strptime ( strTime.c_str(), "%Y/%m/%d %H:%M:%S", &_time ) != NULL )
    {
        logger->Debug ( "Format %Y/%m/%d %H:%M:%S" );
    }
    else if ( strptime ( strTime.c_str(), "%d/%m/%Y", &_time ) != NULL )
    {
        logger->Debug ( "Format %d/%m/%Y" );
        _time.tm_hour = 0;
        _time.tm_min = 0;
        _time.tm_sec = 0;
    }
    else if ( strptime ( strTime.c_str(), "%Y/%m/%d", &_time ) != NULL )
    {
        logger->Debug ( "Format %Y/%m/%d" );
        _time.tm_hour = 0;
        _time.tm_min = 0;
        _time.tm_sec = 0;
    }
    else if ( strptime ( strTime.c_str(), "%H:%M:%S", &_time ) != NULL )
    {
        logger->Debug ( "Format %H:%M:%S" );
        _time.tm_year = 0;
        _time.tm_mon = 0;
        _time.tm_wday = 0;
        _time.tm_mday = 0;
    }
    else
    {
        _time.tm_hour = 0;
        _time.tm_min = 0;
        _time.tm_sec = 0;
        _time.tm_year = 0;
        _time.tm_mon = 0;
        _time.tm_wday = 0;
        _time.tm_mday = 1;
    }

    _time.tm_isdst = -1;

    mktime ( &_time );
}

std::string DateTimeUtils::getDate()
{
    char buffer [20];
    strftime ( buffer, sizeof ( buffer ), "%d/%m/%Y", &_time );
    return std::string ( buffer );
}

std::string DateTimeUtils::getTime()
{
    char buffer [20];
    strftime ( buffer, sizeof ( buffer ), "%H:%M:%S", &_time );
    return std::string ( buffer );
}

std::string DateTimeUtils::getDay()
{
    std::vector < std::string > vector;
    StringUtils::splitToVector ( getDate(), "/", vector );
    return vector[0];
}

std::string DateTimeUtils::getMonth()
{
    std::vector < std::string > vector;
    StringUtils::splitToVector ( getDate(), "/", vector );
    return vector[1];
}

std::string DateTimeUtils::getYear()
{
    std::vector < std::string > vector;
    StringUtils::splitToVector ( getDate(), "/", vector );
    return vector[2];
}

bool DateTimeUtils::validate()
{
    std::vector < std::string > vector;
    StringUtils::splitToVector ( input, "/", vector );
    std::string temp;

    if ( vector.size() >= 3 )
    {
        temp.append ( StringUtils::intToString ( StringUtils::stringToInt ( vector[0] ), 2 ) );
        temp.append ( "/" );
        temp.append ( StringUtils::intToString ( StringUtils::stringToInt ( vector[1] ), 2 ) );
        temp.append ( "/" );
        temp.append ( StringUtils::intToString ( StringUtils::stringToInt ( vector[2] ), 4 ) );
    }

    bool result = !temp.empty() && temp == getDate();

    logger->Debug ( "Validate : " +  input + " - " + getDate() + " " + getTime() + std::string ( result ? " (true)" : " (false)" ) );

    return result;
}

std::string DateTimeUtils::getCurrentDateTime()
{
    time_t dateTemp;       // Stores seconds elapsed since 01-01-1970
    char dateOut[25];      // Output string

    time ( &dateTemp );
    struct tm * dateFormat = localtime ( &dateTemp ); // Saves in Date format

    strftime ( dateOut, sizeof ( dateOut ), "%d-%m-%YT%H:%M:%S%Z", dateFormat );

    return std::string ( dateOut );
}
