/**
 * @file SafeBufferUtils.hxx
 * @brief Header of the utility to split and find elements over any buffer.
 * @date 06/02/2014
 * @date Yadickson Soto
 */

#ifndef SAFEBUFFERUTILS_HXX_
#define SAFEBUFFERUTILS_HXX_

#include <vector>
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Utils
        {
            /**
             * @class	SafeBufferUtils
             * @brief 	Represents the utility to split and find elements over any buffer.
             * @author	Yadickson Soto
             * @date	15/05/2015
             */
            class SafeBufferUtils
            {
                public:
                    /**
                     * @fn	SafeBufferUtils ( Smartmatic::System::SafeBuffer & buffer );
                     * @brief   Class constructor to initialize buffer class to process.
                     * @param	buffer buffer to proccess.
                     * @author	Yadickson Soto
                     * @date	14/05/2015
                     */
                    explicit SafeBufferUtils ( const Smartmatic::System::SafeBuffer & buffer );

                    /**
                     * @fn	virtual ~SafeBufferUtils();
                     * @brief	class destructor
                     * @author	Yadickson Soto
                     * @date	14/05/2015
                     */
                    virtual ~SafeBufferUtils ();

                    /**
                     * @fn	std::vector < Smartmatic::System::SafeBuffer > splitByEndLine ();
                     * @brief	Method split buffer by end line characters, if end line not found,
                     * simply return a result vector containing the entire buffer in its first element.
                     * @author	Yadickson Soto
                     * @date	14/05/2015
                     * @return	vector lines from buffer
                     */
                    std::vector < Smartmatic::System::SafeBuffer > splitByEndLine ();

                    /**
                     * @fn	std::vector < SafeBuffer > splitByBuffer ( const SafeBuffer & findBuffer );
                     * @brief	Method to split buffer after each occurence of "findBuffer"; if it is never found,
                     * simply return a result vector containing the entire buffer in its first element.
                     * @author	Yadickson Soto
                     * @date	14/05/2015
                     * @return	vector lines from buffer
                     */
                    std::vector < SafeBuffer > splitByBuffer ( const SafeBuffer & findBuffer );

                    /**
                     * @fn	bool SafeBuffer::findData ( unsigned long index, const SafeBuffer & findBuffer,
                     * 			unsigned long & postResult );
                     * @brief	Method find data in buffer.
                     * @author	Yadickson Soto
                     * @date	25/02/2012
                     * @param	index				index to begin find data.
                     * @param	findBuffer		Data to find.
                     * @param	postResult	Post found moreData.
                     * @return	true is found.
                     */
                    bool findData ( unsigned long index, const Smartmatic::System::SafeBuffer & findBuffer, unsigned long & postResult );

                private:

                    std::vector<unsigned char>& m_buffer;
            };

        } /* namespace Utils */
    } /* namespace System */
} /* namespace Smartmatic */
#endif /* SAFEBUFFERUTILS_HXX_ */
