#ifndef TEST_CONFIGURATION_PERSISTENCE_MANAGER_TEST
#define TEST_CONFIGURATION_PERSISTENCE_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockFile.hxx>
#include <tests/StoreMockFile.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>

class ConfigurationPersistenceManagerTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;
        typedef Smartmatic::System::IO::File File;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

    public:

        ConfigurationPersistenceManagerTest ()
        {

        }

        void setUp()
        {
        }

        void tearDown()
        {

        }

        void testDefaultConfigurationFileName()
        {
            std::string expected = "configuration-miscellaneous.xml";

            CPPUNIT_ASSERT ( expected == ConfigurationPersistenceManager::filePath );
        }

        void testSaveEmptyConfigurationSuccessfully()
        {
            MockConfigurationPersistenceManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &file );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            ConfigurationPersistenceManager::getInstance ( &manager );

            CPPUNIT_ASSERT ( true == ConfigurationPersistenceManager::getInstance()->SaveFile() );

            manager.verify();
            file.verify();
        }

        void testSaveEmptyConfigurationFail()
        {
            MockConfigurationPersistenceManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &file );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            ConfigurationPersistenceManager::getInstance ( &manager );

            CPPUNIT_ASSERT ( false == ConfigurationPersistenceManager::getInstance()->SaveFile() );

            manager.verify();
            file.verify();
        }

        void testGetEmptyConfiguration()
        {
            MockConfigurationPersistenceManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &manager );

            CPPUNIT_ASSERT ( true == ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "type" ).empty() );

            manager.verify();
        }

        void testSetAndGetSameConfiguration()
        {
            MockConfigurationPersistenceManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            std::string expected = "content";

            ConfigurationPersistenceManager::getInstance ( &manager );
            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "type", "content" );

            CPPUNIT_ASSERT ( expected == ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "type" ) );

            manager.verify();
        }

        void testTwoSetInSameConfiguration()
        {
            MockConfigurationPersistenceManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            std::string expected = "content2";

            ConfigurationPersistenceManager::getInstance ( &manager );
            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "type", "content1" );
            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "type", "content2" );

            CPPUNIT_ASSERT ( expected == ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "type" ) );

            manager.verify();
        }

        void testTwoSetConfiguration()
        {
            MockConfigurationPersistenceManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            std::string expected1 = "content1";
            std::string expected2 = "content2";

            ConfigurationPersistenceManager::getInstance ( &manager );
            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "type1", "content1" );
            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "type2", "content2" );

            CPPUNIT_ASSERT ( expected1 == ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "type1" ) );
            CPPUNIT_ASSERT ( expected2 == ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "type2" ) );

            manager.verify();
        }

        void testSetOneConfigurationAndSave()
        {
            MockConfigurationPersistenceManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &file );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            ConfigurationPersistenceManager::getInstance ( &manager );
            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "type", "content" );

            CPPUNIT_ASSERT ( true == ConfigurationPersistenceManager::getInstance()->SaveFile() );

            manager.verify();
            file.verify();
        }

        void testSetTwoConfigurationAndSave()
        {
            MockConfigurationPersistenceManager manager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &file );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            ConfigurationPersistenceManager::getInstance ( &manager );
            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "type1", "content1" );
            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "type2", "content2" );

            CPPUNIT_ASSERT ( true == ConfigurationPersistenceManager::getInstance()->SaveFile() );

            manager.verify();
            file.verify();
        }

        void testLoadEmptyConfigurationFile()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), true ) );

            ConfigurationPersistenceManager * manager = ConfigurationPersistenceManager::getInstance();
            std::string result = ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "SMTT" );

            CPPUNIT_ASSERT ( true == result.empty() );

            delete manager;
        }

        void testLoadConfigurationFile()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            std::string xml = "<ConfigurationBaseRoot xmlns=\"http://www.smartmatic.com/Configuration\">" \
                              "<ConfigurationType name=\"SMTT\">" \
                              "<Configuration>Hello</Configuration>" \
                              "</ConfigurationType>" \
                              "</ConfigurationBaseRoot>";

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( xml ), true ) );

            ConfigurationPersistenceManager * manager = ConfigurationPersistenceManager::getInstance();
            std::string result = ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "SMTT" );

            CPPUNIT_ASSERT ( false ==  result.empty() );
            CPPUNIT_ASSERT ( "Hello" ==  result );

            delete manager;
        }

        CPPUNIT_TEST_SUITE ( ConfigurationPersistenceManagerTest );

        CPPUNIT_TEST ( testDefaultConfigurationFileName );
        CPPUNIT_TEST ( testSaveEmptyConfigurationSuccessfully );
        CPPUNIT_TEST ( testSaveEmptyConfigurationFail );

        CPPUNIT_TEST ( testGetEmptyConfiguration );
        CPPUNIT_TEST ( testSetAndGetSameConfiguration );
        CPPUNIT_TEST ( testTwoSetInSameConfiguration );
        CPPUNIT_TEST ( testTwoSetConfiguration );

        CPPUNIT_TEST ( testSetOneConfigurationAndSave );
        CPPUNIT_TEST ( testSetTwoConfigurationAndSave );

        CPPUNIT_TEST ( testLoadEmptyConfigurationFile );
        CPPUNIT_TEST ( testLoadConfigurationFile );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_CONFIGURATION_PERSISTENCE_MANAGER_TEST */
