#ifndef TEST_BASE_LANGUAGE_CONTROLLER_TEST
#define TEST_BASE_LANGUAGE_CONTROLLER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Runtime/ProcessLauncher.h>
#include <Environment/Configurations/base-language-controller.h>
#include <tests/StoreMockProcessLauncher.hxx>
#include "MockBasicLanguageController.hxx"

class BaseLanguageControllerTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::Environment::LanguageController LanguageController;
        typedef Smartmatic::Environment::BaseLanguageController BaseLanguageController;
        typedef Smartmatic::System::Runtime::ProcessLauncher ProcessLauncher;

    public:

        BaseLanguageControllerTest ()
        {

        }

        void setUp()
        {
        }

        void tearDown()
        {

        }

        void testSetLanguage()
        {
            MockBasicLanguageController controller;
            LanguageController::getInstance ( &controller );

            MOCKPP_CHAINER_FOR ( MockBasicLanguageController, getLangSystem ) getLangSystem_mock ( &controller );
            MOCKPP_CHAINER_FOR_EXT ( MockBasicLanguageController, setLangSystem, ext ) setLangSystem_mock ( &controller );

            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            launcher.listReturn.push_back ( "fr_BE" );
            launcher.listReturn.push_back ( "fr_FR" );

            getLangSystem_mock
            .expects ( MOCKPP_NS::once() );

            setLangSystem_mock
            .expects ( MOCKPP_NS::once() );

            BaseLanguageController::LocalizeApplication ( "fr", "fr" );

            std::string langExpected = "fr";
            std::string countryExpected = "FR";
            std::string fullExpected = "fr_FR";

            std::string lang, country;
            BaseLanguageController::GetActualLanguage ( lang, country );

            CPPUNIT_ASSERT ( true == LanguageController::getInstance()->isSetLang() );
            CPPUNIT_ASSERT ( langExpected == lang );
            CPPUNIT_ASSERT ( countryExpected == country );
            CPPUNIT_ASSERT ( fullExpected == BaseLanguageController::getActualLanguage() );

            controller.verify();
        }

        void testSingleLanguage()
        {
            MockBasicLanguageController controller;
            LanguageController::getInstance ( &controller );

            MOCKPP_CHAINER_FOR ( MockBasicLanguageController, getLangSystem ) getLangSystem_mock ( &controller );
            MOCKPP_CHAINER_FOR_EXT ( MockBasicLanguageController, setLangSystem, ext ) setLangSystem_mock ( &controller );

            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            launcher.listReturn.push_back ( "fr_BE" );
            launcher.listReturn.push_back ( "fr_FR" );

            getLangSystem_mock
            .expects ( MOCKPP_NS::once() );

            setLangSystem_mock
            .expects ( MOCKPP_NS::once() );

            BaseLanguageController::FullLocalizeApplication ( "fr" );

            std::string langExpected = "fr";
            std::string countryExpected = "FR";
            std::string fullExpected = "fr_FR";

            std::string lang, country;
            BaseLanguageController::GetActualLanguage ( lang, country );

            CPPUNIT_ASSERT ( true == LanguageController::getInstance()->isSetLang() );
            CPPUNIT_ASSERT ( langExpected == lang );
            CPPUNIT_ASSERT ( countryExpected == country );
            CPPUNIT_ASSERT ( fullExpected == BaseLanguageController::getActualLanguage() );

            controller.verify();
        }

        void testFullLanguage()
        {
            MockBasicLanguageController controller;
            LanguageController::getInstance ( &controller );

            MOCKPP_CHAINER_FOR ( MockBasicLanguageController, getLangSystem ) getLangSystem_mock ( &controller );
            MOCKPP_CHAINER_FOR_EXT ( MockBasicLanguageController, setLangSystem, ext ) setLangSystem_mock ( &controller );

            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            launcher.listReturn.push_back ( "fr_BE" );
            launcher.listReturn.push_back ( "fr_FR" );

            getLangSystem_mock
            .expects ( MOCKPP_NS::once() );

            setLangSystem_mock
            .expects ( MOCKPP_NS::once() );

            BaseLanguageController::FullLocalizeApplication ( "fr_BE" );

            std::string langExpected = "fr";
            std::string countryExpected = "BE";
            std::string fullExpected = "fr_BE";

            std::string lang, country;
            BaseLanguageController::GetActualLanguage ( lang, country );

            CPPUNIT_ASSERT ( true == BaseLanguageController::isSetLanguage() );
            CPPUNIT_ASSERT ( langExpected == lang );
            CPPUNIT_ASSERT ( countryExpected == country );
            CPPUNIT_ASSERT ( fullExpected == BaseLanguageController::getActualLanguage() );

            controller.verify();
        }

        CPPUNIT_TEST_SUITE ( BaseLanguageControllerTest );

        CPPUNIT_TEST ( testSetLanguage );
        CPPUNIT_TEST ( testSingleLanguage );
        CPPUNIT_TEST ( testFullLanguage );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_BASE_LANGUAGE_CONTROLLER_TEST */
