/**
 * @file   MockStorageDevice.hxx
 * @brief  Declaration of StorageDevice mocked object.
 * @author manuel.arguelles
 * @date   Aug  8, 2017
 */

#ifndef MOCK_STORAGE_DEVICE_HXX
#define MOCK_STORAGE_DEVICE_HXX

#include <mockpp/mockpp.h>
#include <mockpp/chaining/ChainableMockObject.h>
#include <mockpp/chaining/ChainingMockObjectSupport.h>
#include <System/Runtime/IStorageDevice.hxx>

class MockStorageDevice : public Smartmatic::System::Runtime::IStorageDevice
                        , public MOCKPP_NS::ChainableMockObject
{
    public:

        MockStorageDevice(const std::string& devicePath)
            : MOCKPP_NS::ChainableMockObject(MOCKPP_PCHAR("MockStorageDevice" ), 0)
            , MOCKPP_CONSTRUCT_MEMBERS_FOR_CHAINABLE0(isMounted)
            , MOCKPP_CONSTRUCT_MEMBERS_FOR_CHAINABLE0(mount)
            , MOCKPP_CONSTRUCT_MEMBERS_FOR_CHAINABLE0(unmount)
            , MOCKPP_CONSTRUCT_MEMBERS_FOR_CHAINABLE0(getMountPoint)
            , MOCKPP_CONSTRUCT_MEMBERS_FOR_CHAINABLE0(getDevicePath)
            , MOCKPP_CONSTRUCT_MEMBERS_FOR_CHAINABLE1(isSameDevice)
        {
        }

        virtual ~MockStorageDevice()
        {
        }

        MOCKPP_CONST_CHAINABLE0(MockStorageDevice, bool, isMounted);

        MOCKPP_CHAINABLE0(MockStorageDevice, bool, mount);

        MOCKPP_CHAINABLE0(MockStorageDevice, bool, unmount);

        MOCKPP_CONST_CHAINABLE0(MockStorageDevice, std::string, getMountPoint);

        MOCKPP_CONST_CHAINABLE0(MockStorageDevice, std::string, getDevicePath);

        MOCKPP_CONST_CHAINABLE1(
            MockStorageDevice,
            bool,
            isSameDevice,
            Smartmatic::System::Runtime::IStorageDevice);
};

#endif /* MOCK_STORAGE_DEVICE_HXX */
