#ifndef TEST_EXCEPTION_TEST
#define TEST_EXCEPTION_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Exception/BaseException.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/Exception/RSAException.h>
#include <System/Exception/GuIdException.h>
#include <System/Exception/MessageDigestException.h>
#include <System/Exception/XmlException.hxx>
#include <tests/MockSMTTLog.h>

using namespace Smartmatic::System::Exception;

class AllExceptionTest : public CPPUNIT_NS::TestFixture
{
    public:

        AllExceptionTest()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testBaseExceptionNullLogger()
        {
            BaseException ex ( "message", "code", NULL );

            CPPUNIT_ASSERT ( "message" == ex.getMessage() );
            CPPUNIT_ASSERT ( "message" == std::string ( ex.what() ) );
            CPPUNIT_ASSERT ( "code" == ex.getCode() );
        }

        void testBaseExceptionWithLogger()
        {
            MockSMTTLog mock;

            MOCKPP_CHAINER_FOR_EXT ( MockSMTTLog, Debug, ext ) debug_mock ( &mock );
            MOCKPP_CHAINER_FOR_EXT ( MockSMTTLog, Audit, ext ) audit_mock ( &mock );
            MOCKPP_CHAINER_FOR_EXT ( MockSMTTLog, Error, ext ) error_mock ( &mock );

            debug_mock.expects ( MOCKPP_NS::never() );
            audit_mock.expects ( MOCKPP_NS::never() );
            error_mock.expects ( MOCKPP_NS::once() );

            BaseException ex ( "message", "code", &mock );

            CPPUNIT_ASSERT ( "message" == ex.getMessage() );
            CPPUNIT_ASSERT ( "message" == std::string ( ex.what() ) );
            CPPUNIT_ASSERT ( "code" == ex.getCode() );

            mock.verify();
        }

        void testCryptoException()
        {
            CryptoException ex ( "message", "code", NULL );

            CPPUNIT_ASSERT ( "message" == ex.getMessage() );
            CPPUNIT_ASSERT ( "message" == std::string ( ex.what() ) );
            CPPUNIT_ASSERT ( "code" == ex.getCode() );
        }

        void testSignedException()
        {
            SignedException ex ( "message", "code", NULL );

            CPPUNIT_ASSERT ( "message" == ex.getMessage() );
            CPPUNIT_ASSERT ( "message" == std::string ( ex.what() ) );
            CPPUNIT_ASSERT ( "code" == ex.getCode() );
        }

        void testRSAException()
        {
            RSAException ex ( "message", "code", NULL );

            CPPUNIT_ASSERT ( "message" == ex.getMessage() );
            CPPUNIT_ASSERT ( "message" == std::string ( ex.what() ) );
            CPPUNIT_ASSERT ( "code" == ex.getCode() );
        }

        void testGuIdException()
        {
            GuIdException ex ( "message", "code", NULL );

            CPPUNIT_ASSERT ( "message" == ex.getMessage() );
            CPPUNIT_ASSERT ( "message" == std::string ( ex.what() ) );
            CPPUNIT_ASSERT ( "code" == ex.getCode() );
        }

        void testMessageDigestException()
        {
            MessageDigestException ex ( "message", "code", NULL );

            CPPUNIT_ASSERT ( "message" == ex.getMessage() );
            CPPUNIT_ASSERT ( "message" == std::string ( ex.what() ) );
            CPPUNIT_ASSERT ( "code" == ex.getCode() );
        }

        void testXmlException()
        {
            XmlException ex ( "message", "code", NULL );

            CPPUNIT_ASSERT ( "message" == ex.getMessage() );
            CPPUNIT_ASSERT ( "message" == std::string ( ex.what() ) );
            CPPUNIT_ASSERT ( "code" == ex.getCode() );
        }

        CPPUNIT_TEST_SUITE ( AllExceptionTest );

        CPPUNIT_TEST ( testBaseExceptionNullLogger );
        CPPUNIT_TEST ( testBaseExceptionWithLogger );
        CPPUNIT_TEST ( testCryptoException );
        CPPUNIT_TEST ( testSignedException );
        CPPUNIT_TEST ( testRSAException );
        CPPUNIT_TEST ( testGuIdException );
        CPPUNIT_TEST ( testMessageDigestException );
        CPPUNIT_TEST ( testXmlException );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_EXCEPTION_TEST */
