#ifndef TEST_PATH_TEST
#define TEST_PATH_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/IO/Path.h>

class PathTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::IO::Path Path;
        std::string separator;

    public:

        PathTest ()
        {
        }

        void setUp()
        {
            separator = Path::getPathSeparator();
        }

        void tearDown()
        {

        }

        void testGetPathSeparator()
        {
            std::string expected ( separator );
            std::string result;

            result = Path::getPathSeparator();

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineEmptyParameters()
        {
            std::string p1;
            std::string p2;

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testCombineFirstEmptyParameter()
        {
            std::string p1;
            std::string p2 = "p2";

            std::string expected;

            expected.append ( "p2" );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineSecondEmptyParameter()
        {
            std::string p1 = "p1";
            std::string p2;

            std::string expected;

            expected.append ( "p1" );
            expected.append ( separator );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineFirstEmptySecondWithEndPathSeparatorParameter()
        {
            std::string p1;
            std::string p2;

            p2.append ( "p2" );
            p2.append ( separator );

            std::string expected;

            expected.append ( "p2" );
            expected.append ( separator );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineSecondEmptyFirstWithEndPathSeparatorParameter()
        {
            std::string p1;
            std::string p2;

            p1.append ( "p1" );
            p1.append ( separator );

            std::string expected;

            expected.append ( "p1" );
            expected.append ( separator );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineFirstEmptySecondWithInitPathSeparatorParameter()
        {
            std::string p1;
            std::string p2;

            p2.append ( separator );
            p2.append ( "p2" );

            std::string expected;

            expected.append ( "p2" );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineSecondEmptyFirstWithInitPathSeparatorParameter()
        {
            std::string p1;
            std::string p2;

            p1.append ( separator );
            p1.append ( "p1" );

            std::string expected;

            expected.append ( separator );
            expected.append ( "p1" );
            expected.append ( separator );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineTwoPathSeparatorParameters()
        {
            std::string p1;
            std::string p2;

            p1.append ( separator );
            p2.append ( separator );

            std::string expected;

            expected.append ( separator );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineTwoRelativePathsParameters()
        {
            std::string p1;
            std::string p2;

            p1.append ( "p1" );
            p2.append ( "p2" );

            std::string expected;

            expected.append ( "p1" );
            expected.append ( separator );
            expected.append ( "p2" );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineTwoFixPathsParameters()
        {
            std::string p1;
            std::string p2;

            p1.append ( separator );
            p1.append ( "p1" );
            p2.append ( separator );
            p2.append ( "p2" );

            std::string expected;

            expected.append ( separator );
            expected.append ( "p1" );
            expected.append ( separator );
            expected.append ( "p2" );

            std::string result;

            result = Path::Combine ( p1, p2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetFileNameEmptyFileName()
        {
            std::string file;
            std::string result;

            result = Path::GetFileName ( file );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetFileNameWithRelativePath()
        {
            std::string file = "path/path_two/file";
            std::string expected = "file";

            std::string result;

            result = Path::GetFileName ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetFileNameWithFixPath()
        {
            std::string file = "/path/path_two/file.txt";
            std::string expected = "file.txt";

            std::string result;

            result = Path::GetFileName ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetFileNameToDirectoryPath()
        {
            std::string path = "path/path_two/";
            std::string result;

            result = Path::GetFileName ( path );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetFileExtensionEmptyFileName()
        {
            std::string file;
            std::string result;

            result = Path::GetFileExtension ( file );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetFileExtensionFileNameWithoutExt()
        {
            std::string file = "file";
            std::string result;

            result = Path::GetFileExtension ( file );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetFileExtensionWithRelativePath()
        {
            std::string file = "path/file.txt";
            std::string expected = ".txt";

            std::string result;

            result = Path::GetFileExtension ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetFileExtensionWithFixPath()
        {
            std::string file = "/path/file.txt";
            std::string expected = ".txt";

            std::string result;

            result = Path::GetFileExtension ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetFileExtensionToDirectoryPath()
        {
            std::string path = "path/path_two/";
            std::string result;

            result = Path::GetFileExtension ( path );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetFileExtension()
        {
            std::string file = "/path/file";
            std::string result;

            result = Path::GetFileExtension ( file );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetFileNameWithOutExtensionToEmptyParam()
        {
            std::string path;
            std::string result;

            result = Path::GetFileNameWithOutExtension ( path );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetFileNameWithOutExtensionParamWithoutExt()
        {
            std::string file = "path/file";
            std::string expected = "file";
            std::string result;

            result = Path::GetFileNameWithOutExtension ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetFileNameWithOutExtensionParamWithExt()
        {
            std::string file = "path/file.txt";
            std::string expected = "file";
            std::string result;

            result = Path::GetFileNameWithOutExtension ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetDirectoryToEmptyParam()
        {
            std::string file;
            std::string result;

            result = Path::GetDirectory ( file );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetDirectoryWithFixPath()
        {
            std::string file = "/path/file";
            std::string expected = "/path/";

            std::string result;

            result = Path::GetDirectory ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetDirectoryToDirectoryPath()
        {
            std::string path = "/path/path_two/";
            std::string expected = "/path/path_two/";

            std::string result;

            result = Path::GetDirectory ( path );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetDirectoryToRelativePath()
        {
            std::string path = "path/path_two/file";
            std::string expected = "path/path_two/";

            std::string result;

            result = Path::GetDirectory ( path );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetFullFileNameWithOutExtensionToEmptyParam()
        {
            std::string path;
            std::string result;

            result = Path::GetFullFileNameWithOutExtension ( path );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testGetFullFileNameWithOutExtensionParamWithoutExt()
        {
            std::string file = "/path/path_two/file";
            std::string expected = "/path/path_two/file";
            std::string result;

            result = Path::GetFullFileNameWithOutExtension ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testGetFullFileNameWithOutExtensionParamWithExt()
        {
            std::string file = "path/path_two/file.txt";
            std::string expected = "path/path_two/file";
            std::string result;

            result = Path::GetFullFileNameWithOutExtension ( file );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineFileNameEmptyCombine()
        {
            std::string file = "path/path_two/file.txt";
            std::string combine;

            std::string expected = "path/path_two/file.txt";

            std::string result;

            result = Path::combineFileName ( file, combine );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineFileNameWithoutExt()
        {
            std::string file = "path/path_two/file";
            std::string combine = "FR";

            std::string expected = "path/path_two/fileFR";

            std::string result;

            result = Path::combineFileName ( file, combine );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineFileNameWithtExt()
        {
            std::string file = "path/path_two/file.txt";
            std::string combine = "FR";

            std::string expected = "path/path_two/fileFR.txt";

            std::string result;

            result = Path::combineFileName ( file, combine );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        void testCombineDotToSecondPath()
        {
            std::string path1 = "path";
            std::string path2 = ".";

            std::string expected = "path/";

            std::string result;

            result = Path::Combine ( path1, path2 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( expected == result );
        }

        CPPUNIT_TEST_SUITE ( PathTest );

        CPPUNIT_TEST ( testGetPathSeparator );

        CPPUNIT_TEST ( testCombineEmptyParameters );

        CPPUNIT_TEST ( testCombineFirstEmptyParameter );
        CPPUNIT_TEST ( testCombineSecondEmptyParameter );

        CPPUNIT_TEST ( testCombineFirstEmptySecondWithEndPathSeparatorParameter );
        CPPUNIT_TEST ( testCombineSecondEmptyFirstWithEndPathSeparatorParameter );
        CPPUNIT_TEST ( testCombineFirstEmptySecondWithInitPathSeparatorParameter );
        CPPUNIT_TEST ( testCombineSecondEmptyFirstWithInitPathSeparatorParameter );

        CPPUNIT_TEST ( testCombineTwoPathSeparatorParameters );
        CPPUNIT_TEST ( testCombineTwoRelativePathsParameters );
        CPPUNIT_TEST ( testCombineTwoFixPathsParameters );

        CPPUNIT_TEST ( testGetFileNameEmptyFileName );
        CPPUNIT_TEST ( testGetFileNameWithRelativePath );
        CPPUNIT_TEST ( testGetFileNameWithFixPath );
        CPPUNIT_TEST ( testGetFileNameToDirectoryPath );

        CPPUNIT_TEST ( testGetFileNameWithOutExtensionToEmptyParam );
        CPPUNIT_TEST ( testGetFileNameWithOutExtensionParamWithoutExt );
        CPPUNIT_TEST ( testGetFileNameWithOutExtensionParamWithExt );

        CPPUNIT_TEST ( testGetFileExtensionEmptyFileName );
        CPPUNIT_TEST ( testGetFileExtensionFileNameWithoutExt );
        CPPUNIT_TEST ( testGetFileExtensionWithRelativePath );
        CPPUNIT_TEST ( testGetFileExtensionWithFixPath );
        CPPUNIT_TEST ( testGetFileExtensionToDirectoryPath );
        CPPUNIT_TEST ( testGetFileExtension );

        CPPUNIT_TEST ( testGetDirectoryToEmptyParam );
        CPPUNIT_TEST ( testGetDirectoryWithFixPath );
        CPPUNIT_TEST ( testGetDirectoryToDirectoryPath );
        CPPUNIT_TEST ( testGetDirectoryToRelativePath );

        CPPUNIT_TEST ( testGetFullFileNameWithOutExtensionToEmptyParam );
        CPPUNIT_TEST ( testGetFullFileNameWithOutExtensionParamWithoutExt );
        CPPUNIT_TEST ( testGetFullFileNameWithOutExtensionParamWithExt );

        CPPUNIT_TEST ( testCombineFileNameEmptyCombine );
        CPPUNIT_TEST ( testCombineFileNameWithoutExt );
        CPPUNIT_TEST ( testCombineFileNameWithtExt );

        CPPUNIT_TEST ( testCombineDotToSecondPath );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_PATH_TEST */
