#ifndef TEST_PROCESS_MONITOR_TEST_
#define TEST_PROCESS_MONITOR_TEST_

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Runtime/ProcessMonitor.h>
#include <System/Runtime/ProcessLauncher.h>
#include "MockRunProcessLauncher.hxx"
#include "StoreMockProcessLauncher.hxx"

class ProcessMonitorTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Runtime::ProcessLauncher ProcessLauncher;
        typedef Smartmatic::System::Runtime::ProcessMonitor ProcessMonitor;

    public:

        ProcessMonitorTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testCallCountProcessNotRunning()
        {
            MockRunProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            int result;
            int expected = 0;

            std::string expectedCommand = "ps -A | grep process";

            result = ProcessMonitor::CountProcessRunning ( "process" );

            CPPUNIT_ASSERT ( expected == result );
            CPPUNIT_ASSERT ( expectedCommand == launcher.RunProcessThroughPipeCommandLine );

            launcher.verify();
        }

        void testCallCountAllProcessRunning()
        {
            MockRunProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            launcher.list.push_back ( "1234" );
            launcher.list.push_back ( "1235" );

            int result;
            int expected = 2;

            std::string expectedCommand = "ps -A";

            result = ProcessMonitor::CountProcessRunning ( "*" );

            CPPUNIT_ASSERT ( expected == result );
            CPPUNIT_ASSERT ( expectedCommand == launcher.RunProcessThroughPipeCommandLine );

            launcher.verify();
        }

        void testCallCountProcessRunning()
        {
            MockRunProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            launcher.list.push_back ( "1234" );

            int result;
            int expected = 1;

            result = ProcessMonitor::CountProcessRunning ( "process" );

            CPPUNIT_ASSERT ( expected == result );

            launcher.verify();
        }

        void testCallKillProcessByInt()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "kill -9 1234";

            ProcessMonitor::KillProcess ( 1234 );

            CPPUNIT_ASSERT ( expected == launcher.RunProcessThroughSystemCommandLine );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testCallKillProcessByString()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "pkill -9 dsort";

            ProcessMonitor::KillProcess ( "dsort" );

            CPPUNIT_ASSERT ( expected == launcher.RunProcessThroughSystemCommandLine );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        CPPUNIT_TEST_SUITE ( ProcessMonitorTest );

        CPPUNIT_TEST ( testCallCountProcessNotRunning );
        CPPUNIT_TEST ( testCallCountProcessRunning );
        CPPUNIT_TEST ( testCallCountAllProcessRunning );
        CPPUNIT_TEST ( testCallKillProcessByInt );
        CPPUNIT_TEST ( testCallKillProcessByString );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_PROCESS_MONITOR_TEST_ */
