#ifndef TEST_CRYPTOPKCS7TEST
#define TEST_CRYPTOPKCS7TEST

#ifdef ECUT_MAIN

#include "System/Security/Cryptography/CryptoPKCS7.hxx"
#include <cppunit/extensions/HelperMacros.h>
#include <System/Security/OpenSSLSecurity.hxx>
#include <System/Encoding/Base64Encoder.h>
#include <System/SafeBuffer.h>
#include <System/IO/File.h>
#include <tests/StoreMockFile.hxx>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/safestack.h>

using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security::Cryptography;

static std::string validCertificate_PKCS12 = "MIIJ4QIBAzCCCacGCSqGSIb3DQEHAaCCCZgEggmUMIIJkDCCBo8GCSqGSIb3DQEH"
        "BqCCBoAwggZ8AgEAMIIGdQYJKoZIhvcNAQcBMBwGCiqGSIb3DQEMAQYwDgQIsyAM"
        "Tby61EMCAggAgIIGSNA54zRywSYK7JFdm8Gzfmib0u4oCdZf05yZp/guiXXou7N4"
        "GslmIdVhxtzIiv3XniG3kxWZccv6fs9S77VjyhS+HYi1OsR6TrGWc/sQ+vX+Xa2m"
        "LW2WuZLfjPonKLlY4pmT1saXv46xxgg9PFHiKZ54ao6F0g+24BGP/hXTGYLo9rDy"
        "3/Z6W8E1FX2BuEk1M79zTMMwDkbg5ivO8P9W6tOoMwgOqTnVKuZPmdqjF/BN58By"
        "PiFkBjMuDkRi5RlwhzbQqmd1Fgfq59JTcGrY8uDCcYCzAO7lBZS/bcZy1k9eY1eU"
        "XwUqmlzyszuly2x/0TP+Ycrws/p4hUHKbHeqN+3orxlcJsr/EMwnyipV/VASPjEM"
        "aZ+BKV8Pfr7PlZGyahhfSyHvx3EBFx4ONHTLsg5lzXYCquP41CZRhopgU41Nbu1K"
        "9dz4zC9uUkmA7zloJIbClWigRETS3cqbVUzgbr4wiIfbwdW2XnycAvh8v0GBMcu1"
        "GmWu0hXEEYGXnm30Wr8vxoXoZvfx/Euu0erU+Eo2QH7UL6FfMOYGbKSMzeaRcvnx"
        "gIg1dQkIUThyOYiJ34WELh9QQikJ7iY1uu0nxvWN/X+tH3ufVOIMZ7FFcXH2yLKZ"
        "/f4qFj5JOzKRfbxjB660ccUVXCHJYWeS3eWuRFYEUry6mfmcmXtT2ly2dMBjop2j"
        "t7GFlZBbKelazk9Zn/urrcusZZgaWDZns85ojMqBP9kbnZwBj+LvN3BLSbVhjU5N"
        "Tb2biKaAKhkf2gMO/F/REEs+uzB3vZOg1un0tBcC0enfqynq3wpQV6QDw4n7QIU+"
        "yz9zyif2PQ1IVexau4Me7N9Ot958OSlkVnSwx6rUz5NmAT4wzO7y4HQaJwmp/JDw"
        "EJHMXi5cvekN2ek75a4imk057pNQo4+HuMH1BUZxAIBZ+ylbe8+NjBNzbWDMu6gg"
        "UXTz0B4454ZzJnGO1Qr6neBGGpFQ5DKmmzSZUDCC9TzZG4JuxRzkKPgae66Uugin"
        "xi7F3MnWSI+YAjCow6UGZv/PmCCBVFVxM6ASvS+Y6rPNYOWkn0NfZH+Xb+t/aSAj"
        "3SVs9lPSFZ7q3ng5H0SsXe6cSmAu3UXOt2htM+lypmNduRD18HBaINz6DApu9kQ9"
        "5Q+Q6b27X178VP2ymnStfw0VLAQGFD/LaSpeLU3tW0p+se1jgkL3GEyJ7OhvyeWy"
        "JWTbm3WwtpmUaBTJRyC4AZu/76Uyk72EXYdVSpQXrKFWd3k2ykxyOKFnUOWxtlyX"
        "0AwdUIVooRwTewXzs5+RRtrPd41aTjPhMv7HDIjSHpdhDy5DGj67gWYdKSzJ6Zig"
        "SDCxqqUod5h7paXUFMbPGCPakmIBPdgfraLkK20NXBIzCOrZmBw2EE2oIqQtJ/f6"
        "l1jUUe85iWEVStxlQhgxF1Z9HOjgaWKm8LNjBaq0pPEWHFngIANefHkiibRTA882"
        "MKHuco1bQXYpKat4bKTT6hid1IDojLRo+6FWQptE7qX0WfriUA4Qsr7hmMT10/Kf"
        "bziAHlMIMhCUZvRt7JqR4wyhUq0AWkFA8/aH0FQNs0n6gWO4bUG7KT831XZvr21C"
        "cknButntjTOg0GV8K0EicT90jgY1OawsR4/lRFrMcT+1R9YecCdzpIzqbmLodZG+"
        "ZDrs9XHRNfXpcnz1X6P/8U3aa/x26/ER6bmezznWF60NVrA3u8ACUcymjh2mC+6/"
        "4uQeca8K7v+jvlrjK9jpXEsNPaWfDsTm5Nf39BnmE8WrJTc5BU8GlvarZdWzblSY"
        "JXzi6BlAc/e7u3+NCIlXXlow86upuKxAnGBu142lRe1s2ZnCT/cL2ubab9nHJUUe"
        "CpsW6egPL+rXSVagOHD9d55X8r2USzhTGyVTq6G/wkY1VPAjMfH0+SZ7rbgkCg6z"
        "7wrRuQYWqG5QgtqJ4vXhIZffpyd3TWZhF+iFaXkCZNM48K4CmkpzecmgfgYkIvya"
        "Zqcw8aA0aQARVREnP58xrb2SrdLygpx12jMu6mZiTwxy3BfHmUJUeC15X7Nn7gxH"
        "jXPFsJmuHGKZIa1Wrq3GeImQgFGts/DuynFmV+H/5U6UD9hS4tHvbIbO6cmnyMZ7"
        "x4OkKFvp13nusDiyuzCdQllYx0G06kFzxFuchui23ms0zAp0LjCCAvkGCSqGSIb3"
        "DQEHAaCCAuoEggLmMIIC4jCCAt4GCyqGSIb3DQEMCgECoIICpjCCAqIwHAYKKoZI"
        "hvcNAQwBAzAOBAiwRsjl2i/JBgICCAAEggKAo67L/gzuTYIVlwuVsUF63kISwQ2d"
        "MRwlFQQc5VD9O+lnIksCyZfpcHcSYJGl4AeOy9glAeOn0o13ynkA5oCe1DlemadM"
        "njui44MSLivmNTJleGNqCtoUT88/ADGcGoeOneMCl4yTP42klGZaF2hXoT7tAZkE"
        "vurLAw7VBK7JWT/8tXXFBN2CvNGkyhSQWasfzQUyc44nABsOAprbwGzj7LpOJ4jb"
        "J7FvPcn3lJXR3xPdDNWTg8nLxiALpJMoB0vPFanyxDmhG8LdhxA8E0kOBacF9c9v"
        "165RxCIA/9M1ri1/H7iDpeCwjSPb+yyZTPGEsyuQQdjBiph6YXPQcBs17TkGgjIm"
        "TjKJnnubyKSf2CryfOHlvBTHN84XUzasqkRNMPyk4mhboJCBZ0sduxYnLmbcwBAW"
        "aXdxiGGnhenfnHYZ9NWDeiYXofHn+UZrqjfE3xWxg70h//Nj6ISu9s6u6YR343vB"
        "ivi8UFJu3kpwXkm1yZLtgZb64QH4VvGXE5H/vLVtedRThqq6kSGwKcKJCl31VrF9"
        "Xy+7u4t9amQVpVDIq9IEpwGzqpGxNSpVavUSmzfTbiAwZtkdCgPd2W0lrXSbznxo"
        "RqpQmnejBwy5jUkM6IPn3DovMUWpoxZt2klj286ZqJM7iF11y9H9S0aOt3W88Ulp"
        "sqvA9C0Jp6N3+OftZg4+pJX/MZrfBt0Yk27tA5AzT4AbxmOtKdLtnjMiK0UteTsu"
        "SeriPybTxXJyExR5WAhHU5K5MuURZGpn+dN2viwEwSdMbPGxJijCpnCqdf0iaJWI"
        "DbP8L+x7QUH81sIBZpDXC13hQMjnBOyru71H5tYzPhrvisuxLudqqwi8wzElMCMG"
        "CSqGSIb3DQEJFTEWBBS2X3q5lo9YxXfYjtyJLisz+f5mKTAxMCEwCQYFKw4DAhoF"
        "AAQUd9qIdcVDXDs3XI/8DniFdZhZKpYECGXJksfazXjZAgIIAA==";

static std::string anotherValidCertificate_PKCS12 = "MIILEQIBAzCCCtcGCSqGSIb3DQEHAaCCCsgEggrEMIIKwDCCB78GCSqGSIb3DQEH"
        "BqCCB7AwggesAgEAMIIHpQYJKoZIhvcNAQcBMBwGCiqGSIb3DQEMAQYwDgQILMip"
        "vKjxNhACAggAgIIHeBGTreHijloBYyY5xhcO4UgR+5pzBPBU1IrxxqXgKwyGkDOe"
        "DoAbjfRoYuY0bLBV4GUK6eqnmtJbJEKNpkchc1DyHRINX7n4RfBmkzHRUeOj13Um"
        "TishDAxcLpCESLnVsWumf0fCFY30a0Ms+OLlnLFnLIcjt2R7Ex+O2+KhXXXFdesG"
        "6QuqSreTYCgT7ose74X//aqtLVliIDJnv9Lz9oC25vHtTyE2C0P4wkK/8cIZz7Xy"
        "YmmarSE18C1cvQCaR7BKwTOJuXcR7Q9neEgQ0UfDZjtUAGlX/wiztR2LVG9QXWy0"
        "S6krXb7nlhxKHFJ87S+OXfMi17XteIhu3clQiBA8sNLG4FoFWu+tWjZEydJW00hq"
        "bFToGY6b5ormjDGpZTdFxTyC1iVOhtUyuZSPNSV8I5NEXmRJnljPCwbTbwmdpNnL"
        "QRIwVFZt/L9BZRp7XYRJwSqdNLdDkPEq3WCxeVbl6uT6EpAffu4/d4/PbbWUDbdT"
        "i7tlXW68fdXwT010VXx7xJNDxI4+KppqhTEj+DE44MKrQFqFHFt/S9KUU3AT5Kog"
        "LFK6zbQSGFKQ3YFAI6JSRzKRMQAr5d7SUCLWlGTbOpRhTRLPpTVM3RXRWOQWIzHt"
        "8y8xq/Z0nOvNd28mNqEi+mwFLiunXwCUZhvfZWVJU1wr+iGVidMcpKnp1tHwAe1H"
        "6lFKbGkdBkBZQ0qCxlfuctCtPDeYiqib1R29caV+vsuVry0JSuTUwskMmrmWK6tz"
        "DLaoYhsIEHzj4zjfu277G2t2ceP3kGH4kbLhuAV4AzDVZGpWWOoBYS03XmuJdVTj"
        "bAYikL2sM8jJhiw71R8OZQU7KkUqM9vupw8HGzvn5d1Udy7k7BKSwcqw//18QSBb"
        "eIQbJP515M5ywYU7X+QAhg6uqzQ0Xhsk4tS3eITG901FLVkqePINVhGMPAfJZpY7"
        "FrI+LLcQ5K4Q1Mw2rVKwsd4qe3FF83JZ+W17Ut7DZBWUx4zKb0lv/es4ugZODbGB"
        "W0MeaB976InrC/VJjyEBs2fDK4n02cXlJXCkFsB7ASnZhRt9wLm46EuCM296g7Zq"
        "J5eXoaGzPHOXy9KgDgtJih1AAIYHQxl6T7/ZK7BrCwLVgfZW6pf+ndrrtqhPqWfc"
        "K75GHAJ3MNcBIQe7Xr1Z77TS4IjPZpvy7zBADxmbiCYqOUpDkQ/ptGMGdlw3K4pt"
        "SRBW8wdSUjsD6Qsx9N6LfTfruYjQxiJnpyGWJT2sPAnMy19Hsjply8jRqC5RCh1L"
        "00k+n8U+XSnyrV//xkO3Wxbv5NQkD40mCd4/RZ2vx9CeA2M6C2cQ7xS3/zsWmuOG"
        "xap5Z54JnjXcCq+cmbmQNrKhaJ2L2sFE6+FHA7dni/0ZYB38SPme0/DTjIWhyz6t"
        "/LZ3n3iyoZLdEMy2j2K6VMjQIJ3CxY/F2r1HtGFYWKs+aZNQYhWRjROIz/+AxS4q"
        "j8r/QZDfo8Qa50MwbBnAJkB757tJxNhym9pGETbXJ/TJcnmpSx2Ze+dN/knisWIf"
        "IY7w31akW6Ls53mYzld1gmLIyUmvz9QuzACkvK72YooKnq8IFmcXRAva5CToRNxo"
        "i+6n8EaD05xj91yoRDxYp1OJKk0IvgG8jgRN6MRjjxWw1J1zOQtv05aBhopIZD0/"
        "7z9hFi42/C9fd4kn8o+vKBPP+nOw4s72b3oXCWD+sjVH8pYvmz8OJ8Q4dzcpqQes"
        "soDSvq3rcOEOVxqgOJGdVXaIzIhUrfc/v9Mpsh2/fzLJPycd2+JEHb8lsQbdWEJ2"
        "r2hjj9TNaY+/FvF3vAIP/25iyzXZkzFR2eqyPYKEe1khT0ZRWMMk2wNJhrdc+QI5"
        "aiVY40v1yvVVCD4VofMbLwrqLnMYo0cg3vCG5IG7pwLVcvqMCCU/XZSVQT22/ZhP"
        "U+xyaF1a4ixuvXvfzDK50QxXmJuocgpk5j13z7Vh9zvL1LEpIPsUwKHXpDAt0ME/"
        "6/EgXDjTUrtOhX43CYy+ST9vzkJTKeTU/OwfnzhN8M0H5amZ5EjZWzEeICnnsSHL"
        "l7FgIIHm3mkoLJV40toG8Jr3N6kCGcVqUDhWpaHlDTRyZRJe7rO9OV2QRnN/u/rL"
        "6as+JyXB4a4G0wqFgqUANla/BEjWTty5eS8GhmXONAkhwBgf26dCl6mdmZ0P/JpA"
        "7iDB4lwB3+BUOO8xlzUUxqJ4HFkZ4M0nlC2cejxdLkjSBWNeTHaNq8onD4IYINAb"
        "RdFNsa8r7aH8pnj99MGHP6TSiojYS6krrad5u4IA9yGLiLo6LhPxqHOQU6RWkN0g"
        "6f7ScH+a4j07TpA9m76rZnTImTeQ1u3pDdyJx7msM7NbjumhUGnS5TS3GDfqASCA"
        "TZBRhRVtEDHwJp7hBbEpi+kUFRq0M3Y7Olcvd8OIgzWAG0Oxjod2CEdh9yhcPGQA"
        "aN+o8MS8FuG6Z9TAEcvGCApIm6XlT6DZRHXdKQHaNoPA7aR46k6RUAdGupZ4At6m"
        "D8di1MEHpn9ZUHYT34ETSJD1NRgsRalNtr71bQsz2XZ0Eoj9+r+2I5krAU/wb23A"
        "1voojI0wggL5BgkqhkiG9w0BBwGgggLqBIIC5jCCAuIwggLeBgsqhkiG9w0BDAoB"
        "AqCCAqYwggKiMBwGCiqGSIb3DQEMAQMwDgQIO3ja4yqBTXgCAggABIICgKMMM8K2"
        "Cg2Re3rFfcBFUf3APPSUjj/GWKzvNHOCukMTtym38+xKdiAU6KwkTNY9/AZq/2d8"
        "yY0EQSREPeTCg4Z1KeDqDaV83aYNZFomQBmQgk9zY0U7W9bsOaTVuK9SDagW1JQc"
        "UEGgF55ELgv3lrIXB4vRQfs6BewCPnzCYUvv0kn5BPtzw9e4fh7Sy189MYdmB0uX"
        "KeDkRShG5E486ZlliRuL/JlYe1hKmJT5hpzdUpQBz9z6KUL9CyOScu9vW6pjqRji"
        "8vhRG7aOoZvyfULbtce/3CBiXoVaf9f1WYW2/JmSMdMkRqDhFfbzUeD5yQwPLJqj"
        "NJt69yUgLmleb2PAsUk+/rKw/VIX2F/RV3YGcKGk0qlcbjsUNsPJQ9OoWgUqu/tu"
        "P9zlR1v703Dp1sQeoWLwtL0sjsgtUNR95bYTKxOxWOT66wKnt5qNQe5vzGy0EmyO"
        "SFIeC/8bJQ6zg17E198r/4Ci//tgH/Czy7ntvYSJl+9wLehf7U2GN+VWMl3hEcEv"
        "BOa7jDMVNiSYA2U8BvMXwqBsZddPtju2ygV0HAKG0F7jZczq4kh/Ma7kXciB8zL/"
        "GmzzRSopPBJw4uAzy4QYKMElAh5n1v0T1ib04+618V5IQe/wLgu15EVIl+1PUvkg"
        "lScmGLg9s+LpotcxVxd30embcMcnHJ4ue5/zABQKBKPssvJQjcaYop69op+VRqU5"
        "uZyozWf/7Tnlk2nc5rQNRisf774SNyu0mGU8Ro0eRYIoOUJdX17h6SPHbx6QnsKf"
        "ADUaxwktk76wWehAuv6i9xcrGc6t57rIkTZZghCs43incag5+64wzHzasPOoUbqi"
        "ewEX6bogzVQ3knMxJTAjBgkqhkiG9w0BCRUxFgQUmAfrzuaxctAKQ865xtkaJnHF"
        "STYwMTAhMAkGBSsOAwIaBQAEFEUsUKLrBKKRNC/54ClLPulgAYkpBAiwclqKnOBt"
        "LQICCAA=";

static std::string validPrivateKey_PEM = "-----BEGIN ENCRYPTED PRIVATE KEY-----\n"
        "MIICxjBABgkqhkiG9w0BBQ0wMzAbBgkqhkiG9w0BBQwwDgQIiV8C9Nv/IOcCAggA\n"
        "MBQGCCqGSIb3DQMHBAggVWwspYiQfgSCAoCnX7fmbIA4YMDX+2F26injvxB8Vwz5\n"
        "en5MomnpXGiPFKcVaZgj/vaHo7qPHh3/Xvpj9SboRwrNkO0iY+KCJjaE5GezbJ6z\n"
        "nvElXO3Gs6/mzqFnIc2ApK6ShxZ1mVJEGtUP/D5996kiQZ3/+ks/Ne94B0AxQ6Jt\n"
        "Koi28hdW9YH7lp0iwFfIBrV+0Y7+I/IXYV0fNPWCrI6gaval89XZPfoF8wvK4sO0\n"
        "KDbMMv59X+L07KD5RLBIxLbcq0GVzYsC53MU6aU2UkrSCPEPi39tdzkvF0tPEBbY\n"
        "vME6TrJONkSxaV8nzID/BO+ZNvl0RVUhePbpf/CQhxZZUTWY5mgdZ7Ru0hdl9jJ+\n"
        "zyddWAg24Te56rpsyjhfAJhRYZK1UwQUOLhRaaUvZpBM2QYHA9NSHOGv9IU6krP+\n"
        "iTFOKw2XbGMZkuCJz1Jn8h4jfGHfw2SMZvRcmqNW2OuQQn40kMlOoKp1xcZ2pTR4\n"
        "B0dl1Y4tLe7s4zC9JdyGsYiwlAwHwDSgATg+G6c1cUbxHwlPBAl8fNpmNdp7OGZk\n"
        "DhHoC2rldV3D04HwukarR82OP8VUj7H3ykSEKTEnUX8vGpkruPPXpe3lF9pZrqX5\n"
        "QAvdZuz+jDWy+OOt3NNLR7pEUK96geuDNJegWracsUB2/DwRKhWOWjxG3nko0H+n\n"
        "y1P4TG0xeT/59oJOLvscHpO+IW3p3rjiTvN8hgFt1bLSi+ZKJOxtJNNsQCMokgV9\n"
        "UiB3/sJnLM0f+yRODsR0iscy51rssuSs8P8XjsyEiG+qep8VfGrCAwIw4dMxyZ/P\n"
        "jtuZe1bhjON3Zva27eF47FKRpwiU5KBFBwbSx92lGZ6Gin5dYP045LOf\n"
        "-----END ENCRYPTED PRIVATE KEY-----\n";

static std::string validCertificate_PEM = "-----BEGIN CERTIFICATE-----\n"
        "MIIC9DCCAl2gAwIBAgICEAAwDQYJKoZIhvcNAQEFBQAweTELMAkGA1UEBhMCQkUx\n"
        "ETAPBgNVBAgMCEJydXNzZWxzMRIwEAYDVQQHDAlFdHRlcmJlZWsxEzARBgNVBAoM\n"
        "ClNtYXJ0bWF0aWMxHDAaBgNVBAsME0VsZWN0b3JhbCBTb2x1dGlvbnMxEDAOBgNV\n"
        "BAMMB1JPT1QgQ0EwHhcNMTMxMjI2MTAzNjM0WhcNMTQxMjI2MTAzNjM0WjCBiDEL\n"
        "MAkGA1UEBhMCQkUxETAPBgNVBAgMCEJydXNzZWxzMRMwEQYDVQQKDApTbWFydG1h\n"
        "dGljMRwwGgYDVQQLDBNFbGVjdG9yYWwgU29sdXRpb25zMQ4wDAYDVQQDDAVVc2Vy\n"
        "MTEjMCEGCSqGSIb3DQEJARYUdXNlcjFAc21hcnRtYXRpYy5jb20wgZ8wDQYJKoZI\n"
        "hvcNAQEBBQADgY0AMIGJAoGBALkfis573u0A27LW+n1q9URLy6fFoJ8XIAn1bYrw\n"
        "YQgZPfuLGk0fSFzPf3skMKhvueWgNg9IbOi29Ta8/y4T55SYdTCjumezcjo5kXfP\n"
        "J5dUFNMSnACwP+xH8MGrCaUBn9WEGO4IE0dbiTK4koH0kCrnbOqR+IaNW/TDFJvH\n"
        "mWQ5AgMBAAGjezB5MAkGA1UdEwQCMAAwLAYJYIZIAYb4QgENBB8WHU9wZW5TU0wg\n"
        "R2VuZXJhdGVkIENlcnRpZmljYXRlMB0GA1UdDgQWBBQb0Vt+kvOcMX8GKNS2Nrp6\n"
        "fJ8RWTAfBgNVHSMEGDAWgBR7ieRpADfz3zAXXjayYT4sn4hRCjANBgkqhkiG9w0B\n"
        "AQUFAAOBgQARS2/N2ySflGKV4bbNgcu9RJspQ7P6wTFgLivsjcLKgJvLgJQUNdsr\n"
        "eoY4T0Jjm3yg+tN6y8831u4JUgZOuvjWmHmIkYFPqqkzEHkPyslBZFHBhE/60xNn\n"
        "p+xjtpQJ/RB9eC4OeaXpgaCLYP5jQPxN00A8HEdLNjqz6sIvWyCzjg==\n"
        "-----END CERTIFICATE-----\n";

static std::string correctPasswordPKCS12 = "passwordpkcs12";
static std::string correctPrivateKeyPassword = "passworduser1";
static std::string filePath = "filePath";
static std::string message = "Hello world!";

const static EVP_CIPHER * cipher = EVP_aes_128_cbc();

class CryptoPKCS7Test : public CPPUNIT_NS::TestFixture
{
    public:

        CryptoPKCS7Test()
        {

        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testCryptoWithoutCertificate ()
        {
            SafeBuffer originalData ( message );
            SafeBuffer encryptedData ( "exit" );

            CryptoPKCS7 cryptoP7 ( cipher );

            CPPUNIT_ASSERT_THROW ( cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::SMIME ), CryptoException );
            CPPUNIT_ASSERT ( 0 == encryptedData.GetSize() );
        }

        void testDecryptoWithoutCertificate ()
        {
            SafeBuffer originalData ( message );
            SafeBuffer encryptedData ( "exit" );

            CryptoPKCS7 cryptoP7 ( cipher );

            CPPUNIT_ASSERT_THROW ( cryptoP7.decryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::SMIME ), CryptoException );
            CPPUNIT_ASSERT ( 0 == encryptedData.GetSize() );
        }

        void testCryptoAndDecryptoBuffer_SMIME ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::SMIME );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( encryptedData != originalData );

            cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::SMIME );

            CPPUNIT_ASSERT ( 0 != decryptedData.GetSize() );
            CPPUNIT_ASSERT ( encryptedData != decryptedData );
            CPPUNIT_ASSERT ( originalData == decryptedData );
        }

        void testCryptoAndDecryptoBuffer_PEM ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::PEM );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::PEM );

            CPPUNIT_ASSERT ( 0 != decryptedData.GetSize() );
            CPPUNIT_ASSERT ( encryptedData != decryptedData );
            CPPUNIT_ASSERT ( originalData == decryptedData );
        }

        void testCryptoAndDecryptoBuffer_DER ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::DER );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::DER );

            CPPUNIT_ASSERT ( 0 != decryptedData.GetSize() );
            CPPUNIT_ASSERT ( encryptedData != decryptedData );
            CPPUNIT_ASSERT ( originalData == decryptedData );
        }

        void testNullCipherMode ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            CryptoPKCS7 cryptoP7 ( NULL );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::PEM );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::PEM );

            CPPUNIT_ASSERT ( 0 != decryptedData.GetSize() );
            CPPUNIT_ASSERT ( encryptedData != decryptedData );
            CPPUNIT_ASSERT ( originalData == decryptedData );
        }

        void testCryptoWithX509Null ()
        {
            SafeBuffer certificateBuffer ( validPrivateKey_PEM );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PEM, correctPrivateKeyPassword );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData;

            CPPUNIT_ASSERT_THROW ( cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::PEM ), CryptoException );
            CPPUNIT_ASSERT ( 0 == encryptedData.GetSize() );
        }

        void testCryptoAndDecryptoLoadTwoCertificates ()
        {
            SafeBuffer certificateBufferPEM ( validCertificate_PEM );
            SafeBuffer certificateBufferPrivatePEM ( validPrivateKey_PEM );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBufferPEM, true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBufferPrivatePEM, true ) );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PEM );

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::PEM );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PEM, correctPrivateKeyPassword );
            cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::PEM );

            CPPUNIT_ASSERT ( 0 != decryptedData.GetSize() );
            CPPUNIT_ASSERT ( encryptedData != decryptedData );
            CPPUNIT_ASSERT ( originalData == decryptedData );
        }

        void testCryptoAndDecryptoLoadTwoCertificatesFail ()
        {
            SafeBuffer certificateBufferPEM ( validCertificate_PEM );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBufferPEM, true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBufferPEM, true ) );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PEM );

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::PEM );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PEM );

            CPPUNIT_ASSERT_THROW ( cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::PEM ), CryptoException );
            CPPUNIT_ASSERT ( 0 == decryptedData.GetSize() );
        }

        void testCryptoAndDecryptoPEMFail ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::PEM );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            encryptedData.removeData ( 0, 1 );

            CPPUNIT_ASSERT_THROW ( cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::PEM ), CryptoException );
            CPPUNIT_ASSERT ( 0 == decryptedData.GetSize() );
        }

        void testCryptoAndDecryptoSMIMEFail ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::SMIME );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            encryptedData.removeData ( 192, 1 );

            CPPUNIT_ASSERT_THROW ( cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::SMIME ) , CryptoException );
            CPPUNIT_ASSERT ( 0 == decryptedData.GetSize() );
        }

        void testCryptoAndDecryptoDERFail ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::DER );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            encryptedData.removeData ( 0, 1 );

            CPPUNIT_ASSERT_THROW ( cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::DER ), CryptoException );
            CPPUNIT_ASSERT ( 0 == decryptedData.GetSize() );
        }

        void testNullOriginalData ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            SafeBuffer originalData;
            SafeBuffer encryptedData;

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::SMIME );
            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
        }

        void testDecryptoBuffer_DifferentCertificate ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            SafeBuffer anotherCertificateBuffer;
            Base64Encoder::ConvertFromBase64 ( anotherValidCertificate_PKCS12, anotherCertificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( anotherCertificateBuffer, true ) );

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::SMIME );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            cryptoP7.SetCertificate ( filePath, Cryptography::CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            CPPUNIT_ASSERT_THROW ( cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::SMIME ), CryptoException );
            CPPUNIT_ASSERT ( 0 == decryptedData.GetSize() );
        }

        void testDecryptoFail ()
        {
            SafeBuffer certificateBuffer;
            Base64Encoder::ConvertFromBase64 ( validCertificate_PKCS12, certificateBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( certificateBuffer, true ) );

            CryptoPKCS7 cryptoP7 ( cipher );
            cryptoP7.SetCertificate ( filePath, CertUtil::CertFormat_PKCS12, correctPasswordPKCS12 );

            SafeBuffer originalData ( message );
            SafeBuffer encryptedData, decryptedData;

            cryptoP7.encryptBuffer ( originalData, encryptedData, Cryptography::PKCS7Base::SMIME );

            CPPUNIT_ASSERT ( 0 != encryptedData.GetSize() );
            CPPUNIT_ASSERT ( originalData != encryptedData );

            CPPUNIT_ASSERT_THROW ( cryptoP7.decryptBuffer ( encryptedData, decryptedData, Cryptography::PKCS7Base::PEM ), CryptoException );
            CPPUNIT_ASSERT ( 0 == decryptedData.GetSize() );
        }

        CPPUNIT_TEST_SUITE ( CryptoPKCS7Test );

        CPPUNIT_TEST ( testCryptoWithoutCertificate );
        CPPUNIT_TEST ( testDecryptoWithoutCertificate );
        CPPUNIT_TEST ( testCryptoAndDecryptoBuffer_SMIME );
        CPPUNIT_TEST ( testCryptoAndDecryptoBuffer_PEM );
        CPPUNIT_TEST ( testCryptoAndDecryptoBuffer_DER );
        CPPUNIT_TEST ( testNullCipherMode );
        CPPUNIT_TEST ( testCryptoWithX509Null );
        CPPUNIT_TEST ( testCryptoAndDecryptoLoadTwoCertificates );
        CPPUNIT_TEST ( testCryptoAndDecryptoLoadTwoCertificatesFail );
        CPPUNIT_TEST ( testCryptoAndDecryptoPEMFail );
        CPPUNIT_TEST ( testCryptoAndDecryptoSMIMEFail );
        CPPUNIT_TEST ( testCryptoAndDecryptoDERFail );
        CPPUNIT_TEST ( testNullOriginalData );
        CPPUNIT_TEST ( testNullCipherMode );
        CPPUNIT_TEST ( testDecryptoBuffer_DifferentCertificate );
        CPPUNIT_TEST ( testDecryptoFail );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_TEST_CRYPTOPKCS7TEST */
