#ifndef TEST_STRING_UTILS_TEST
#define TEST_STRING_UTILS_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Utils/StringUtils.hxx>

class StringUtilsTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::System::Utils::StringUtils StringUtils;

        StringUtilsTest()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testRemoveInputEmpty()
        {
            std::string input;
            std::string find = "a";

            std::string result = StringUtils::remove ( input, find );
            std::string expected = "";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testRemoveInputAndFindEmpty()
        {
            std::string input;
            std::string find;

            std::string result = StringUtils::remove ( input, find );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testRemoveFindEmpty()
        {
            std::string input = "abcd";
            std::string expected = "abcd";
            std::string find;

            std::string result = StringUtils::remove ( input, find );

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testRemoveFindNotFound()
        {
            std::string input = "abcd-abcd-abcd";
            std::string expected = "abcd-abcd-abcd";
            std::string find = "xyz";

            std::string result = StringUtils::remove ( input, find );

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testRemove()
        {
            std::string input = "abcdxxxxabcd";
            std::string expected = "xxxxabcd";
            std::string find = "abcd";

            std::string result = StringUtils::remove ( input, find );

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
        }

        void testRemoveAllIfInputEmpty()
        {
            std::string input;
            std::string find = "a";

            std::string result = StringUtils::removeAll ( input, find );
            std::string expected = "";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testRemoveAllIfInputAndFindEmpty()
        {
            std::string input;
            std::string find;

            std::string result = StringUtils::removeAll ( input, find );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testRemoveAllIfFindEmpty()
        {
            std::string input = "abcd";
            std::string expected = "abcd";
            std::string find;

            std::string result = StringUtils::removeAll ( input, find );

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testRemoveAllIfFindNotFound()
        {
            std::string input = "abcd-abcd-abcd";
            std::string expected = "abcd-abcd-abcd";
            std::string find = "xyz";

            std::string result = StringUtils::removeAll ( input, find );

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testRemoveAll()
        {
            std::string input = "abcd-abcd-abcd";
            std::string expected = "--";
            std::string find = "abcd";

            std::string result = StringUtils::removeAll ( input, find );

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceInputEmpty()
        {
            std::string input;
            std::string find = "a";
            std::string replace = "b";

            std::string result = StringUtils::replace ( input, find, replace );
            std::string expected = "";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testReplaceInputAndFindEmpty()
        {
            std::string input;
            std::string find;
            std::string replace = "b";

            std::string result = StringUtils::replace ( input, find, replace );
            std::string expected = "";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testReplaceFindEmpty()
        {
            std::string input = "xyz";
            std::string find;
            std::string replace = "b";

            std::string result = StringUtils::replace ( input, find, replace );
            std::string expected = "xyz";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceFindNotFound()
        {
            std::string input = "xyz";
            std::string find = "a";
            std::string replace = "b";

            std::string result = StringUtils::replace ( input, find, replace );
            std::string expected = "xyz";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceDestEmpty()
        {
            std::string input = "xyz";
            std::string find = "x";
            std::string replace;

            std::string result = StringUtils::replace ( input, find, replace );
            std::string expected = "yz";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplace()
        {
            std::string input = "abcabc";
            std::string find = "a";
            std::string replace = "b";

            std::string result = StringUtils::replace ( input, find, replace );
            std::string expected = "bbcabc";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceLastFindEmpty()
        {
            std::string input = "abc-abc";
            std::string find;
            std::string replace = "b";

            std::string result = StringUtils::replaceLast ( input, find, replace );
            std::string expected = "abc-abc";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceLastFindNotFound()
        {
            std::string input = "abc-abc";
            std::string find = "x";
            std::string replace = "b";

            std::string result = StringUtils::replaceLast ( input, find, replace );
            std::string expected = "abc-abc";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceLast()
        {
            std::string input = "abc-abc";
            std::string find = "c";
            std::string replace = "b";

            std::string result = StringUtils::replaceLast ( input, find, replace );
            std::string expected = "abc-abb";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceAllIfInputEmpty()
        {
            std::string input;
            std::string find = "a";
            std::string replace = "b";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testReplaceAllIfInputAndFindEmpty()
        {
            std::string input;
            std::string find;
            std::string replace = "b";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testReplaceAllIfFindEmpty()
        {
            std::string input = "xyz";
            std::string find;
            std::string replace = "b";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "xyz";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceAllIfFindNotFound()
        {
            std::string input = "xyz";
            std::string find = "n";
            std::string replace = "b";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "xyz";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceAllDestEmpty()
        {
            std::string input = "xyzxyz";
            std::string find = "x";
            std::string replace;

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "yzyz";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceAll()
        {
            std::string input = "xyzxyz";
            std::string find = "x";
            std::string replace = "m";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "myzmyz";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceAllNear()
        {
            std::string input = "xyzxxxxyz";
            std::string find = "x";
            std::string replace = "m";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "myzmmmmyz";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceAllScape()
        {
            std::string input = "abcdabcd";
            std::string find = "a";
            std::string replace = "\\";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "\\bcd\\bcd";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceAllLastCaracter()
        {
            std::string input = "ABCD;";
            std::string find = ";";
            std::string replace = "\\;";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "ABCD\\;";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testReplaceAllEndLine()
        {
            std::string input = "abcd\nabcd";
            std::string find = "\n";
            std::string replace = " ";

            std::string result = StringUtils::replaceAll ( input, find, replace );
            std::string expected = "abcd abcd";

            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
            CPPUNIT_ASSERT ( false == result.empty() );
        }

        void testSplitToListInputEmpty()
        {
            std::string input;
            std::string find = "x";
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );

            CPPUNIT_ASSERT ( 1 == list.size() );
            CPPUNIT_ASSERT ( true == ( * ( list.begin() ) ).empty() );
        }

        void testSplitToListInputAndFindEmpty()
        {
            std::string input;
            std::string find;
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );

            CPPUNIT_ASSERT ( 1 == list.size() );
            CPPUNIT_ASSERT ( true == ( * ( list.begin() ) ).empty() );
        }

        void testSplitToListAndFindEmpty()
        {
            std::string input = "abcdef";
            std::string find;
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );
            std::string expected = "abcdef";

            CPPUNIT_ASSERT ( 1 == list.size() );
            CPPUNIT_ASSERT ( 0 == ( * ( list.begin() ) ).compare ( expected ) );
        }

        void testSplitToListAndFindNotFound()
        {
            std::string input = "abcdef";
            std::string find = "x";
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );
            std::string expected = "abcdef";

            CPPUNIT_ASSERT ( 1 == list.size() );
            CPPUNIT_ASSERT ( 0 == ( * ( list.begin() ) ).compare ( expected ) );
        }

        void testSplitFirstFoundToList()
        {
            std::string input = "xabcdefxcd";
            std::string find = "x";
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );

            CPPUNIT_ASSERT ( 3 == list.size() );

            std::list < std::string >::iterator it = list.begin();

            CPPUNIT_ASSERT ( true == ( *it ).empty() );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "abcdef" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "cd" ) );
        }

        void testSplitLastFoundToList()
        {
            std::string input = "abcdefxcdx";
            std::string find = "x";
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );

            CPPUNIT_ASSERT ( 3 == list.size() );

            std::list < std::string >::iterator it = list.begin();

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "abcdef" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "cd" ) );
            ++it;

            CPPUNIT_ASSERT ( true == ( *it ).empty() );
        }

        void testSplitToListUnion()
        {
            std::string input = "abcdefxxcdxfx";
            std::string find = "x";
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );

            CPPUNIT_ASSERT ( 5 == list.size() );

            std::list < std::string >::iterator it = list.begin();

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "abcdef" ) );
            ++it;

            CPPUNIT_ASSERT ( true == ( *it ).empty() );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "cd" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "f" ) );
            ++it;

            CPPUNIT_ASSERT ( true == ( *it ).empty() );
        }

        void testSplitToListUnion2()
        {
            std::string input = "abcxxxdxexfxxcdxfxls";
            std::string find = "x";
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );

            CPPUNIT_ASSERT ( 10 == list.size() );

            std::list < std::string >::iterator it = list.begin();

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "abc" ) );
            ++it;

            CPPUNIT_ASSERT ( true == ( *it ).empty() );
            ++it;

            CPPUNIT_ASSERT ( true == ( *it ).empty() );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "d" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "e" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "f" ) );
            ++it;

            CPPUNIT_ASSERT ( true == ( *it ).empty() );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "cd" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "f" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "ls" ) );
        }

        void testSplitToList()
        {
            std::string input = "abcdefxabxcdxf";
            std::string find = "x";
            std::list < std::string > list;

            StringUtils::splitToList ( input, find, list );

            CPPUNIT_ASSERT ( 4 == list.size() );

            std::list < std::string >::iterator it = list.begin();

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "abcdef" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "ab" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "cd" ) );
            ++it;

            CPPUNIT_ASSERT ( 0 == ( *it ).compare ( "f" ) );
        }

        void testSplitToVector()
        {
            std::string input = "abcdefxabxcdxf";
            std::string find = "x";
            std::vector < std::string > vector;

            StringUtils::splitToVector ( input, find, vector );

            CPPUNIT_ASSERT ( 4 == vector.size() );
            CPPUNIT_ASSERT ( 0 == vector[0].compare ( "abcdef" ) );
            CPPUNIT_ASSERT ( 0 == vector[1].compare ( "ab" ) );
            CPPUNIT_ASSERT ( 0 == vector[2].compare ( "cd" ) );
            CPPUNIT_ASSERT ( 0 == vector[3].compare ( "f" ) );
        }

        void testFoundCharacterInStringEmpty()
        {
            std::string input;
            char character = 'a';

            CPPUNIT_ASSERT ( false == StringUtils::isFound ( input, character ) );
        }

        void testFoundCharacterInStringAndNotCharacter()
        {
            std::string input = "bcde";
            char character = 'a';

            CPPUNIT_ASSERT ( false == StringUtils::isFound ( input, character ) );
        }

        void testFoundCharacterInString()
        {
            std::string input = "abcde";
            char character = 'a';

            CPPUNIT_ASSERT ( true == StringUtils::isFound ( input, character ) );
        }

        void testRegexStringInputEmpty()
        {
            std::string regex = "abc";
            std::string input;

            CPPUNIT_ASSERT ( false == StringUtils::regex ( regex, input ) );
        }

        void testRegexStringInputAndFindEmpty()
        {
            std::string regex;
            std::string input;

            CPPUNIT_ASSERT ( true == StringUtils::regex ( regex, input ) );
        }

        void testRegexStringFindEmpty()
        {
            std::string regex = "";
            std::string input = "ab abd abc axy";

            CPPUNIT_ASSERT ( true == StringUtils::regex ( regex, input ) );
        }

        void testRegexStringFindNotFound()
        {
            std::string regex = "xxy";
            std::string input = "ab abd abc axy";

            CPPUNIT_ASSERT ( false == StringUtils::regex ( regex, input ) );
        }

        void testRegexStringWrong()
        {
            std::string regex = "[";
            std::string input = "ab abd abc axy";

            CPPUNIT_ASSERT ( false == StringUtils::regex ( regex, input ) );
        }

        void testRegexString()
        {
            std::string regex = "ab";
            std::string input = "ab abd abc axy";

            CPPUNIT_ASSERT ( true == StringUtils::regex ( regex, input ) );
        }

        void testRegexStringGroupNotFound()
        {
            std::string regex = "/dev/ttyUSB[0-9]+";
            std::string input = "/dev/ttyUSB";

            CPPUNIT_ASSERT ( false == StringUtils::regex ( regex, input ) );
        }

        void testRegexStringGroupFound()
        {
            std::string regex = "/dev/ttyUSB[0-9]+";
            std::string input = "/dev/ttyUSB1";

            CPPUNIT_ASSERT ( true == StringUtils::regex ( regex, input ) );
        }

        void testRegexStringOrFoundFirst()
        {
            std::string regex = "SMTT|SMI Corporation";
            std::string input = "SMTT";

            CPPUNIT_ASSERT ( true == StringUtils::regex ( regex, input ) );
        }

        void testRegexStringOrFoundSecond()
        {
            std::string regex = "SMTT|SMI Corporation";
            std::string input = "SMI Corporation";

            CPPUNIT_ASSERT ( true == StringUtils::regex ( regex, input ) );
        }

        void testTrimInputEmpty()
        {
            std::string input;
            CPPUNIT_ASSERT ( true == StringUtils::trim ( input ).empty() );
        }

        void testTrimInputNotSpace()
        {
            std::string input = "abcdef";
            std::string expected = "abcdef";

            CPPUNIT_ASSERT ( false == expected.empty() );
            CPPUNIT_ASSERT ( 0 == ( StringUtils::trim ( input ) ).compare ( expected ) );
        }

        void testTrimInputOneStartSpace()
        {
            std::string input = " abcdef";
            std::string expected = "abcdef";

            CPPUNIT_ASSERT ( false == expected.empty() );
            CPPUNIT_ASSERT ( 0 == ( StringUtils::trim ( input ) ).compare ( expected ) );
        }

        void testTrimInputMoreStartSpace()
        {
            std::string input = "     abcdef";
            std::string expected = "abcdef";

            CPPUNIT_ASSERT ( false == expected.empty() );
            CPPUNIT_ASSERT ( 0 == ( StringUtils::trim ( input ) ).compare ( expected ) );
        }

        void testTrimInputEndSpace()
        {
            std::string input = "abcdef ";
            std::string expected = "abcdef";

            CPPUNIT_ASSERT ( false == expected.empty() );
            CPPUNIT_ASSERT ( 0 == ( StringUtils::trim ( input ) ).compare ( expected ) );
        }

        void testTrimInputMoreEndSpace()
        {
            std::string input = "abcdef     ";
            std::string expected = "abcdef";

            CPPUNIT_ASSERT ( false == expected.empty() );
            CPPUNIT_ASSERT ( 0 == ( StringUtils::trim ( input ) ).compare ( expected ) );
        }

        void testTrimInterSpace()
        {
            std::string input = "abc def";
            std::string expected = "abc def";

            CPPUNIT_ASSERT ( false == expected.empty() );
            CPPUNIT_ASSERT ( 0 == ( StringUtils::trim ( input ) ).compare ( expected ) );
        }

        void testTrim()
        {
            std::string input = "      abc def      ";
            std::string expected = "abc def";

            CPPUNIT_ASSERT ( false == expected.empty() );
            CPPUNIT_ASSERT ( 0 == ( StringUtils::trim ( input ) ).compare ( expected ) );
        }

        void testStringToIntInputEmpty()
        {
            std::string input;

            CPPUNIT_ASSERT ( 0 == StringUtils::stringToInt ( input ) );
        }

        void testStringToIntInputNotNumeric()
        {
            std::string input = "abcd";

            CPPUNIT_ASSERT ( 0 == StringUtils::stringToInt ( input ) );
        }

        void testStringToIntInputAlfaNumeric()
        {
            std::string input = "ab0cd";

            CPPUNIT_ASSERT ( 0 == StringUtils::stringToInt ( input ) );
        }

        void testStringToIntPositive()
        {
            std::string input = "25";

            CPPUNIT_ASSERT ( 25 == StringUtils::stringToInt ( input ) );
        }

        void testStringToIntNegative()
        {
            std::string input = "-25";

            CPPUNIT_ASSERT ( -25 == StringUtils::stringToInt ( input ) );
        }

        void testStringToIntZero()
        {
            std::string input = "0";

            CPPUNIT_ASSERT ( 0 == StringUtils::stringToInt ( input ) );
        }

        void testStringToLongInputEmpty()
        {
            std::string input;

            CPPUNIT_ASSERT ( 0L == StringUtils::stringToLong ( input ) );
        }

        void testStringToLongInputNotNumeric()
        {
            std::string input = "abcd";

            CPPUNIT_ASSERT ( 0L == StringUtils::stringToLong ( input ) );
        }

        void testStringToLongInputAlfaNumeric()
        {
            std::string input = "ab0cd";

            CPPUNIT_ASSERT ( 0 == StringUtils::stringToLong ( input ) );
        }

        void testStringToLongPositive()
        {
            std::string input = "2545269";

            CPPUNIT_ASSERT ( 2545269 == StringUtils::stringToLong ( input ) );
        }

        void testStringToLongNegative()
        {
            std::string input = "-2545269";

            CPPUNIT_ASSERT ( -2545269 == StringUtils::stringToLong ( input ) );
        }

        void testStringToLongZero()
        {
            std::string input = "0";

            CPPUNIT_ASSERT ( 0 == StringUtils::stringToLong ( input ) );
        }

        void testStringToDoubleInputEmpty()
        {
            std::string input;

            CPPUNIT_ASSERT ( 0.0 == StringUtils::stringToDouble ( input ) );
        }

        void testStringToDoubleInputNotNumeric()
        {
            std::string input = "abcd";

            CPPUNIT_ASSERT ( 0.0 == StringUtils::stringToDouble ( input ) );
        }

        void testStringToDoubleInputAlfaNumeric()
        {
            std::string input = "ab0cd";

            CPPUNIT_ASSERT ( 0.0 == StringUtils::stringToDouble ( input ) );
        }

        void testStringToDoublePositive()
        {
            std::string input = "2545269";

            CPPUNIT_ASSERT ( 2545269 == StringUtils::stringToDouble ( input ) );
        }

        void testStringToDoubleNegative()
        {
            std::string input = "-2545269";

            CPPUNIT_ASSERT ( -2545269 == StringUtils::stringToDouble ( input ) );
        }

        void testStringToDoubleZero()
        {
            std::string input = "0";

            CPPUNIT_ASSERT ( 0 == StringUtils::stringToDouble ( input ) );
        }

        void testIntToStringPositive()
        {
            int input = 250;
            std::string expected = "250";

            CPPUNIT_ASSERT ( 0 == StringUtils::intToString ( input ).compare ( expected ) );
        }

        void testIntToStringNegative()
        {
            int input = -250;
            std::string expected = "-250";

            CPPUNIT_ASSERT ( 0 == StringUtils::intToString ( input ).compare ( expected ) );
        }

        void testIntToStringZero()
        {
            int input = 0;
            std::string expected = "0";

            CPPUNIT_ASSERT ( 0 == StringUtils::intToString ( input ).compare ( expected ) );
        }

        void testIntToStringGrouped()
        {
            int input = 2;
            std::string expected = "02";

            CPPUNIT_ASSERT ( 0 == StringUtils::intToString ( input, 2 ).compare ( expected ) );
        }

        void testLongToStringPositive()
        {
            long long input = 250L;
            std::string expected = "250";

            CPPUNIT_ASSERT ( 0 == StringUtils::longToString ( input ).compare ( expected ) );
        }

        void testLongToStringNegative()
        {
            long long input = -250L;
            std::string expected = "-250";

            CPPUNIT_ASSERT ( 0 == StringUtils::longToString ( input ).compare ( expected ) );
        }

        void testLongToStringZero()
        {
            long long input = 0;
            std::string expected = "0";

            CPPUNIT_ASSERT ( 0 == StringUtils::longToString ( input ).compare ( expected ) );
        }

        void testLongToStringGroup()
        {
            long long input = 2L;
            std::string expected = "002";

            CPPUNIT_ASSERT ( 0 == StringUtils::longToString ( input, 3 ).compare ( expected ) );
        }

        void testToUpper()
        {
            std::string input = "abCdEfG";
            std::string expected = "ABCDEFG";

            CPPUNIT_ASSERT ( 0 == StringUtils::toUpper ( input ).compare ( expected ) );
        }

        void testToLower()
        {
            std::string input = "abCdEfG";
            std::string expected = "abcdefg";

            CPPUNIT_ASSERT ( 0 == StringUtils::toLower ( input ).compare ( expected ) );
        }

        void testArrayToStringWithInputEmpty()
        {
            std::vector < unsigned char > input;
            std::string expected = "";

            std::string result = StringUtils::arrayHexToString ( input );

            CPPUNIT_ASSERT ( true == result.empty() );
            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
        }

        void testArrayToStringWithoutPrefixAndSeparator()
        {
            std::vector < unsigned char > input;
            std::string expected = "2542";

            input.push_back ( 0x25 );
            input.push_back ( 0x42 );

            std::string result = StringUtils::arrayHexToString ( input );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
        }

        void testArrayToString()
        {
            std::vector < unsigned char > input;
            std::string expected = "0x25,0x42";

            input.push_back ( 0x25 );
            input.push_back ( 0x42 );

            std::string result = StringUtils::arrayHexToString ( input, "0x", "," );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
        }

        void testArrayToAsciiString()
        {
            std::vector < unsigned char > input;
            std::string expected = "Hello 1234";

            input.push_back ( 0x48 );
            input.push_back ( 0x65 );
            input.push_back ( 0x6C );
            input.push_back ( 0x6C );
            input.push_back ( 0x6F );
            input.push_back ( 0x20 );
            input.push_back ( 0x31 );
            input.push_back ( 0x32 );
            input.push_back ( 0x33 );
            input.push_back ( 0x34 );

            std::string result = StringUtils::arrayHexToAsciiString ( input );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
        }

        void testAsciiStringToArray ()
        {
            std::string input = "Hello 1234";
            std::vector < unsigned char > result;

            StringUtils::asciiStringToArrayHex ( input, result );

            CPPUNIT_ASSERT ( 10 == result.size() );
            CPPUNIT_ASSERT ( 0x48 == result[0] );
            CPPUNIT_ASSERT ( 0x65 == result[1] );
            CPPUNIT_ASSERT ( 0x6C == result[2] );
            CPPUNIT_ASSERT ( 0x6C == result[3] );
            CPPUNIT_ASSERT ( 0x6F == result[4] );
            CPPUNIT_ASSERT ( 0x20 == result[5] );
            CPPUNIT_ASSERT ( 0x31 == result[6] );
            CPPUNIT_ASSERT ( 0x32 == result[7] );
            CPPUNIT_ASSERT ( 0x33 == result[8] );
            CPPUNIT_ASSERT ( 0x34 == result[9] );
        }

        void testEmptyBufferHexToString ()
        {
            Smartmatic::System::SafeBuffer input;
            std::string result;

            result = StringUtils::bufferHexToString ( input, "0x", "," );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testBufferHexToString ()
        {
            Smartmatic::System::SafeBuffer input ( "Hello" );
            std::string expected = "0x48,0x65,0x6c,0x6c,0x6f";
            std::string result;

            result = StringUtils::bufferHexToString ( input, "0x", "," );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected ) == 0 ) );
        }

        void testEmptyStringHexToBuffer ()
        {
            std::string input;
            Smartmatic::System::SafeBuffer result;

            StringUtils::stringToBufferHex ( input, result, "0x", "," );

            CPPUNIT_ASSERT ( 0 == result.GetSize() );
        }

        void testStringHexToBuffer ()
        {
            std::string input = "0x48,0x65,0x6c,0x6c,0x6f";
            Smartmatic::System::SafeBuffer result;

            StringUtils::stringToBufferHex ( input, result, "0x", "," );

            CPPUNIT_ASSERT ( 5 == result.GetSize() );
            CPPUNIT_ASSERT ( 0x48 == result.GetData() [0] );
            CPPUNIT_ASSERT ( 0x65 == result.GetData() [1] );
            CPPUNIT_ASSERT ( 0x6C == result.GetData() [2] );
            CPPUNIT_ASSERT ( 0x6C == result.GetData() [3] );
            CPPUNIT_ASSERT ( 0x6F == result.GetData() [4] );
        }

        void testStringToArray ()
        {
            std::string input = "0x48,0x65,0x6C,0x6C,0x6F";
            std::vector < unsigned char > result;

            StringUtils::stringToArrayHex ( input, result, "0x", "," );

            CPPUNIT_ASSERT ( 5 == result.size() );
            CPPUNIT_ASSERT ( 0x48 == result[0] );
            CPPUNIT_ASSERT ( 0x65 == result[1] );
            CPPUNIT_ASSERT ( 0x6C == result[2] );
            CPPUNIT_ASSERT ( 0x6C == result[3] );
            CPPUNIT_ASSERT ( 0x6F == result[4] );
        }

        void testConvertEndLine()
        {
            std::string input = "text\\ntext";
            std::string expected = "text\ntext";

            std::string result = StringUtils::convertEndLine ( input );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
        }

        void testToHexStringDefault()
        {
            long long value = 25321;
            std::string expected = "000062e9";

            std::string result = StringUtils::toHexString ( value );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
        }

        void testToHexStringDefaultWithPrefixAndGroup16()
        {
            long long value = 25321;
            std::string expected = "0x00000000000062e9";

            std::string result = StringUtils::toHexString ( value, "0x", 16 );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( 0 == expected.compare ( result ) );
        }

        void testCapitalizeUtf8()
        {
            std::string value1 = "DUFOURNY Dominique";
            std::string value2 = "VAN GOIDSENHOVEN Gaëtan";
            std::string value3 = "LEFÈVRE Frédéric";
            std::string value4 = "TÁÉÍÓÚÀÈÌÒÙÜÑ";
            std::string expected1 = "Dufourny Dominique";
            std::string expected2 = "Van Goidsenhoven Gaëtan";
            std::string expected3 = "Lefèvre Frédéric";
            std::string expected4 = "Táéíóúàèìòùüñ";

            CPPUNIT_ASSERT ( 0 == expected1.compare ( StringUtils::capitalizeUtf8(value1) ) );
            CPPUNIT_ASSERT ( 0 == expected2.compare ( StringUtils::capitalizeUtf8(value2) ) );
            CPPUNIT_ASSERT ( 0 == expected3.compare ( StringUtils::capitalizeUtf8(value3) ) );
            CPPUNIT_ASSERT ( 0 == expected4.compare ( StringUtils::capitalizeUtf8(value4) ) );
        }

        void testToLowerUtf8()
        {
            std::string value1 = "DUFOURNY Dominique";
            std::string value2 = "VAN GOIDSENHOVEN Gaëtan";
            std::string value3 = "LEFÈVRE Frédéric";
            std::string value4 = "TÁÉÍÓÚÀÈÌÒÙÜÑ";
            std::string value5 = "ÁÉÍÓÚ ÀÈÌÒÙ ÄËÏÖÜ ßA§µ";
            std::string expected1 = "dufourny dominique";
            std::string expected2 = "van goidsenhoven gaëtan";
            std::string expected3 = "lefèvre frédéric";
            std::string expected4 = "táéíóúàèìòùüñ";
            std::string expected5 = "áéíóú àèìòù äëïöü ßa§µ";

            CPPUNIT_ASSERT ( 0 == expected1.compare ( StringUtils::toLowerUtf8(value1) ) );
            CPPUNIT_ASSERT ( 0 == expected2.compare ( StringUtils::toLowerUtf8(value2) ) );
            CPPUNIT_ASSERT ( 0 == expected3.compare ( StringUtils::toLowerUtf8(value3) ) );
            CPPUNIT_ASSERT ( 0 == expected4.compare ( StringUtils::toLowerUtf8(value4) ) );
            CPPUNIT_ASSERT ( 0 == expected5.compare ( StringUtils::toLowerUtf8(value5) ) );
        }

        CPPUNIT_TEST_SUITE ( StringUtilsTest );

        CPPUNIT_TEST ( testRemoveInputEmpty );
        CPPUNIT_TEST ( testRemoveInputAndFindEmpty );
        CPPUNIT_TEST ( testRemoveFindEmpty );
        CPPUNIT_TEST ( testRemoveFindNotFound );
        CPPUNIT_TEST ( testRemove );

        CPPUNIT_TEST ( testRemoveAllIfInputEmpty );
        CPPUNIT_TEST ( testRemoveAllIfInputAndFindEmpty );
        CPPUNIT_TEST ( testRemoveAllIfFindEmpty );
        CPPUNIT_TEST ( testRemoveAllIfFindNotFound );
        CPPUNIT_TEST ( testRemoveAll );

        CPPUNIT_TEST ( testReplaceInputEmpty );
        CPPUNIT_TEST ( testReplaceInputAndFindEmpty );
        CPPUNIT_TEST ( testReplaceFindEmpty );
        CPPUNIT_TEST ( testReplaceFindNotFound );
        CPPUNIT_TEST ( testReplaceDestEmpty );
        CPPUNIT_TEST ( testReplace );
        CPPUNIT_TEST ( testReplaceLastFindEmpty );
        CPPUNIT_TEST ( testReplaceLastFindNotFound );
        CPPUNIT_TEST ( testReplaceLast );

        CPPUNIT_TEST ( testReplaceAllIfInputEmpty );
        CPPUNIT_TEST ( testReplaceAllIfInputAndFindEmpty );
        CPPUNIT_TEST ( testReplaceAllIfFindEmpty );
        CPPUNIT_TEST ( testReplaceAllIfFindNotFound );
        CPPUNIT_TEST ( testReplaceAllDestEmpty );
        CPPUNIT_TEST ( testReplaceAll );
        CPPUNIT_TEST ( testReplaceAllNear );
        CPPUNIT_TEST ( testReplaceAllScape );
        CPPUNIT_TEST ( testReplaceAllEndLine );
        CPPUNIT_TEST ( testReplaceAllLastCaracter );

        CPPUNIT_TEST ( testSplitToListInputEmpty );
        CPPUNIT_TEST ( testSplitToListInputAndFindEmpty );
        CPPUNIT_TEST ( testSplitToListAndFindEmpty );
        CPPUNIT_TEST ( testSplitToListAndFindNotFound );

        CPPUNIT_TEST ( testSplitFirstFoundToList );
        CPPUNIT_TEST ( testSplitLastFoundToList );

        CPPUNIT_TEST ( testSplitToListUnion );
        CPPUNIT_TEST ( testSplitToListUnion2 );

        CPPUNIT_TEST ( testSplitToList );
        CPPUNIT_TEST ( testSplitToVector );

        CPPUNIT_TEST ( testFoundCharacterInStringEmpty );
        CPPUNIT_TEST ( testFoundCharacterInStringAndNotCharacter );
        CPPUNIT_TEST ( testFoundCharacterInString );

        CPPUNIT_TEST ( testRegexStringInputEmpty );
        CPPUNIT_TEST ( testRegexStringInputAndFindEmpty );
        CPPUNIT_TEST ( testRegexStringFindEmpty );
        CPPUNIT_TEST ( testRegexStringFindNotFound );
        CPPUNIT_TEST ( testRegexStringWrong );
        CPPUNIT_TEST ( testRegexString );

        CPPUNIT_TEST ( testRegexStringGroupNotFound );
        CPPUNIT_TEST ( testRegexStringGroupFound );

        CPPUNIT_TEST ( testRegexStringOrFoundFirst );
        CPPUNIT_TEST ( testRegexStringOrFoundSecond );

        CPPUNIT_TEST ( testTrimInputEmpty );
        CPPUNIT_TEST ( testTrimInputNotSpace );
        CPPUNIT_TEST ( testTrimInputOneStartSpace );
        CPPUNIT_TEST ( testTrimInputMoreStartSpace );
        CPPUNIT_TEST ( testTrimInputEndSpace );
        CPPUNIT_TEST ( testTrimInputMoreEndSpace );
        CPPUNIT_TEST ( testTrimInterSpace );
        CPPUNIT_TEST ( testTrim );

        CPPUNIT_TEST ( testStringToIntInputEmpty );
        CPPUNIT_TEST ( testStringToIntInputNotNumeric );
        CPPUNIT_TEST ( testStringToIntInputAlfaNumeric );
        CPPUNIT_TEST ( testStringToIntPositive );
        CPPUNIT_TEST ( testStringToIntNegative );
        CPPUNIT_TEST ( testStringToIntZero );

        CPPUNIT_TEST ( testStringToLongInputEmpty );
        CPPUNIT_TEST ( testStringToLongInputNotNumeric );
        CPPUNIT_TEST ( testStringToLongInputAlfaNumeric );
        CPPUNIT_TEST ( testStringToLongPositive );
        CPPUNIT_TEST ( testStringToLongNegative );
        CPPUNIT_TEST ( testStringToLongZero );

        CPPUNIT_TEST ( testStringToDoubleInputEmpty );
        CPPUNIT_TEST ( testStringToDoubleInputNotNumeric );
        CPPUNIT_TEST ( testStringToDoubleInputAlfaNumeric );
        CPPUNIT_TEST ( testStringToDoublePositive );
        CPPUNIT_TEST ( testStringToDoubleNegative );
        CPPUNIT_TEST ( testStringToDoubleZero );

        CPPUNIT_TEST ( testIntToStringPositive );
        CPPUNIT_TEST ( testIntToStringNegative );
        CPPUNIT_TEST ( testIntToStringZero );
        CPPUNIT_TEST ( testIntToStringGrouped );

        CPPUNIT_TEST ( testLongToStringPositive );
        CPPUNIT_TEST ( testLongToStringNegative );
        CPPUNIT_TEST ( testLongToStringZero );
        CPPUNIT_TEST ( testLongToStringGroup );

        CPPUNIT_TEST ( testToUpper );
        CPPUNIT_TEST ( testToLower );

        CPPUNIT_TEST ( testArrayToStringWithInputEmpty );
        CPPUNIT_TEST ( testArrayToStringWithoutPrefixAndSeparator );
        CPPUNIT_TEST ( testArrayToString );

        CPPUNIT_TEST ( testArrayToAsciiString );
        CPPUNIT_TEST ( testAsciiStringToArray );
        CPPUNIT_TEST ( testStringToArray );

        CPPUNIT_TEST ( testEmptyBufferHexToString );
        CPPUNIT_TEST ( testBufferHexToString );
        CPPUNIT_TEST ( testEmptyStringHexToBuffer );
        CPPUNIT_TEST ( testStringHexToBuffer );
        CPPUNIT_TEST ( testConvertEndLine );

        CPPUNIT_TEST ( testToHexStringDefault );
        CPPUNIT_TEST ( testToHexStringDefaultWithPrefixAndGroup16 );

        CPPUNIT_TEST ( testCapitalizeUtf8 );
        CPPUNIT_TEST ( testToLowerUtf8 );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_STRING_UTILS_TEST */
