#ifndef TEST_XML_ROFF_TRANSFORMER_TEST
#define TEST_XML_ROFF_TRANSFORMER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/XSLFO/XSLFOTransformer.h>
#include <System/XSLFO/XMLRoffTransformer.h>
#include <System/Runtime/ProcessLauncher.h>
#include <System/IO/Directory.h>
#include "MockProcessLauncher.hxx"
#include "StoreMockProcessLauncher.hxx"

class XMLRoffTransformerTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Runtime::ProcessLauncher ProcessLauncher;
        typedef Smartmatic::System::XSLFO::XMLRoffTransformer XMLRoffTransformer;
        typedef Smartmatic::System::XSLFO::XSLFOTransformer XSLFOTransformer;

        XSLFOTransformer * trasnformer;

    public:

        XMLRoffTransformerTest ()
        {
            trasnformer = NULL;
        }

        void setUp()
        {
            trasnformer = new XMLRoffTransformer();
        }

        void tearDown()
        {
            delete trasnformer;
            trasnformer = NULL;
        }

        void testBasicTransformerSuccessful()
        {
            MockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            bool expected = true;
            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( expected == result );

            launcher.verify();
        }

        void testBasicTransformerFail()
        {
            MockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<int> ( 1 ) );

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            bool expected = false;
            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( expected == result );

            launcher.verify();
        }

        void testBasicTransformerCommandExpectedDefault()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=auto --backend=cairo --compat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpectedFormatAuto()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=auto --backend=cairo --compat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setFormat ( XMLRoffTransformer::TRANSFORM_AUTO );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpectedFormatPdf()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=pdf --backend=cairo --compat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setFormat ( XMLRoffTransformer::TRANSFORM_PDF );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpectedFormatPostscript()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=postscript --backend=cairo --compat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setFormat ( XMLRoffTransformer::TRANSFORM_POSTSCRIPT );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpectedFormatSvg()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=svg --backend=cairo --compat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setFormat ( XMLRoffTransformer::TRANSFORM_SVG );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpectedBackendCairo()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=auto --backend=cairo --compat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setBackend ( XMLRoffTransformer::TRANSFORM_CAIRO );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpectedBackendGp()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=auto --backend=gp --compat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setBackend ( XMLRoffTransformer::TRANSFORM_GP );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpectedCompat()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=auto --backend=cairo --compat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setCompat ( XMLRoffTransformer::TRANSFORM_COMPAT );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpectedNotCompat()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=auto --backend=cairo --nocompat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setCompat ( XMLRoffTransformer::TRANSFORM_NOCOMPAT );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        void testBasicTransformerCommandExpected()
        {
            StoreMockProcessLauncher launcher;
            ProcessLauncher::getInstance ( &launcher );

            std::string expected = "xmlroff -o output --format=pdf --backend=gp --nocompat input template";

            std::string outputFile = "output";
            std::string inputFile = "input";
            std::string templateFile = "template";

            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setFormat ( XMLRoffTransformer::TRANSFORM_PDF );
            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setBackend ( XMLRoffTransformer::TRANSFORM_GP );
            ( static_cast < XMLRoffTransformer * > ( trasnformer ) )->setCompat ( XMLRoffTransformer::TRANSFORM_NOCOMPAT );

            bool result;

            result = trasnformer->transformFile ( outputFile, inputFile, templateFile );

            CPPUNIT_ASSERT ( true == result );
            CPPUNIT_ASSERT ( 0 == expected.compare ( launcher.RunProcessThroughSystemCommandLine ) );
            CPPUNIT_ASSERT ( false == launcher.RunProcessThroughSystemBackground );
        }

        CPPUNIT_TEST_SUITE ( XMLRoffTransformerTest );

        CPPUNIT_TEST ( testBasicTransformerSuccessful );
        CPPUNIT_TEST ( testBasicTransformerFail );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedDefault );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedFormatAuto );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedFormatPdf );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedFormatPostscript );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedFormatSvg );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedBackendCairo );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedBackendGp );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedCompat );
        CPPUNIT_TEST ( testBasicTransformerCommandExpectedNotCompat );
        CPPUNIT_TEST ( testBasicTransformerCommandExpected );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_XML_ROFF_TRANSFORMER_TEST */
