/**
 * @file	smartmaticgui-language-controller.h
 * @brief	Declares the smartmaticgui language controller class.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmatic
 * Copyright (C)  2011 <>
 *
 * smartmatic is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmatic is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _SMARTMATIC_GUI_LANGUAGE_CONTROLLER_H_
#define _SMARTMATIC_GUI_LANGUAGE_CONTROLLER_H_

#include "Environment/Configurations/BaseLanguageConfiguration.hxx"

namespace Smartmatic
{
    namespace GUI
    {
        namespace Configuration
        {

            /**
             * @class	SmartmaticGUILanguageController
             * @brief	Smartmatic graphical user interface language controller.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class SmartmaticGUILanguageController: public Smartmatic::Environment::BaseLanguageConfiguration
            {
                public:

                    /**
                     * @fn	std::string SmartmaticGUILanguageController::getPackageName();
                     * @brief	Gets the package name.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The package name.
                     */
                    std::string getPackageName();

                    /**
                     * @fn	std::string SmartmaticGUILanguageController::getLocaleDir();
                     * @brief	Gets the locale dir.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	The locale dir.
                     */
                    std::string getLocaleDir();

                    /**
                     * @fn	SmartmaticGUILanguageController::SmartmaticGUILanguageController();
                     * @brief	Default constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    SmartmaticGUILanguageController();

                    /**
                     * @fn	virtual SmartmaticGUILanguageController::~SmartmaticGUILanguageController();
                     * @brief	Finaliser.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~SmartmaticGUILanguageController();
            };
        }
    }
}

#endif // _SMARTMATIC_GUI_LANGUAGE_CONTROLLER_H_
