/**
 * @file PrintScreenManager.h
 * @brief Header class print screen manager
 * @date 07/04/2011
 * @author Gerardo Hernandez, Yadickson Soto
 */

#ifndef PRINTSCREENMANAGER_H_
#define PRINTSCREENMANAGER_H_

#include <string>
#include "ICapture.hxx"

namespace Smartmatic
{
    namespace Graphics
    {
        namespace Capture
        {
            /**
             * @class	PrintScreenManager
             * @brief	Manager for print screens.
             * @author	Yadickson Soto
             * @date	14/03/2013
             */
            class PrintScreenManager
            {
                private:

                    /**
                     * @fn	PrintScreenManager::PrintScreenManager();
                     * @brief	Default constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    PrintScreenManager();

                    PrintScreenManager(const PrintScreenManager&);

                    static PrintScreenManager * instance; ///< instance class
                    Smartmatic::Graphics::Capture::ICapture * iclass; ///< interface

                public:

                    /**
                     * @fn	virtual PrintScreenManager::~PrintScreenManager();
                     * @brief	Class destructor
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~PrintScreenManager();

                    /**
                     * @fn	void PrintScreenManager::capture( std::string imageFullPath );
                     * @brief	Captures.
                     * @author	Yadickson Soto
                     * @date	14/03/2013
                     * @param	imageFullPath	Full pathname of the image full file.
                     */
                    void capture ( const std::string& imageFullPath );

                    /**
                     * @fn	static PrintScreenManager * getInstance();
                     * @brief	Method getter instance class.
                     * @author	Yadickson Soto
                     * @date	14/03/2013
                     * @return 	instace class
                     */
                    static PrintScreenManager * getInstance();
            };
        }
    }
}

#endif /* PRINTSCREENMANAGER_H_ */
