/**
 * @file AccessibleManager.hxx
 * @brief Header Class Accessible Manager
 * @date 24/04/2013
 * @author Yadickson Soto
 */

#ifndef ACCESSIBLEMANAGER_HXX_
#define ACCESSIBLEMANAGER_HXX_

#include <gtkmm.h>
#include <string>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Operation
        {

            /**
             * @class AccessibleManager
             * @brief Class Accessible Manager
             * @date 24/04/2013
             * @author Yadickson Soto
             */
            class AccessibleManager
            {
                private:
                    /**
                     * @fn AccessibleManager();
                     * @brief Class Constructor
                     * @date 24/04/2013
                     * @author Yadickson Soto
                     */
                    AccessibleManager();

                public:
                    /**
                     * @fn virtual ~AccessibleManager();
                     * @brief Class destructor
                     * @date 24/04/2013
                     * @author Yadickson Soto
                     */
                    virtual ~AccessibleManager();

                    /**
                     * @fn static void setName ( Gtk::Widget * widget, const std::string & name );
                     * @brief Method setter accessible name to widget
                     * @date 24/04/2013
                     * @author Yadickson Soto
                     * @param widget widget
                     * @param name new accessible name
                     */
                    static void setName ( Gtk::Widget * widget, const std::string & name );

                    /**
                     * @fn static void setDescription ( Gtk::Widget * widget, const std::string & description );
                     * @brief Method setter accessible description to widget
                     * @date 24/04/2013
                     * @author Yadickson Soto
                     * @param widget widget
                     * @param description new accessible description
                     */
                    static void setDescription ( Gtk::Widget * widget, const std::string & description );
            };

        } /* namespace Operation */
    } /* namespace GUI */
} /* namespace Smartmatic */
#endif /* ACCESSIBLEMANAGER_HXX_ */
