/**
 * @file BoxSelectionWidget.hxx
 * @brief Header box selection widget class
 * @date 14/11/2012
 * @author Yadickson Soto
 */

#ifndef BOXSELECTIONWIDGET_HXX_
#define BOXSELECTIONWIDGET_HXX_

#include <gtkmm.h>
#include <Widgets/SmartmaticBasicFrame.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/ISizeWidget.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {

            /**
             * @class BoxSelectionWidget
             * @brief box selection widget class
             * @date 14/11/2012
             * @author Yadickson Soto
             */
            class BoxSelectionWidget : public ISizeWidget, public SmartmaticBasicFrame
            {
                public:
                    /**
                     * @fn BoxSelectionWidget( const std::string & code, bool enableClick = true, bool showBorder = true, bool setColors = true );
                     * @brief Class constructor
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param code code
                     * @param enableClick true if enable click
                     * @param showBorder true is show border
                     * @param setColors set colors
                     */
                    BoxSelectionWidget ( const std::string & code, bool enableClick = true, bool showBorder = true, bool setColors = true );
                    /**
                     * @fn BoxSelectionWidget( const std::string & code, const std::string & optionalValue, bool enableClick = true, bool showBorder = true );
                     * @brief Class constructor
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param code code
                     * @param optionalValue optional value
                     * @param enableClick true if enable click
                     * @param showBorder true is show border
                     * @param setColors set colors
                     */
                    BoxSelectionWidget ( const std::string & code, const std::string & optionalValue, bool enableClick = true, bool showBorder = true, bool setColors = true );
                    /**
                     * @fn virtual ~BoxSelectionWidget();
                     * @brief Class destructor
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     */
                    virtual ~BoxSelectionWidget();
                    /**
                     * @fn void addWidget( Gtk::Widget * widget );
                     * @brief Method add widget
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param widget new widget
                     */
                    void addWidget ( Gtk::Widget * widget );
                    /**
                     * @fn void setSelectedColor( Gdk::Color color );
                     * @brief Method setter selected color
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param color new selected color
                     */
                    void setSelectedColor ( Gdk::Color color );
                    /**
                     * @fn void setUnselectedColor( Gdk::Color color );
                     * @brief Method setter unselected color
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param color new unselected color
                     */
                    void setUnselectedColor ( Gdk::Color color );
                    /**
                     * @fn void forceClick();
                     * @brief Method force click
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     */
                    void forceClick();
                    /**
                     * @fn virtual void clear();
                     * @brief Method clear
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     */
                    virtual void clear();
                    /**
                     * @fn sigc::signal<void, bool> getSignalClick();
                     * @brief Method getter signal click
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @return signal
                     */
                    sigc::signal<void, BoxSelectionWidget *> getSignalClick();
                    /**
                     * @fn SmartmaticBasicEventBox * getEventBox();
                     * @brief Method getter event box widget
                     * @date 25/04/2013
                     * @author Yadickson Soto
                     * @return event box widget
                     */
                    SmartmaticBasicEventBox * getEventBox();
                    /**
                     * @fn std::string getCode();
                     * @brief Method getter code
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @return code
                     */
                    std::string getCode();
                    /**
                     * @fn bool getStatus();
                     * @brief Method getter status
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @return success if selected
                     */
                    bool getStatus();
                    /**
                     * @fn void setName( const std::string & name );
                     * @brief Method setter name widget
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param name name widget
                     */
                    void setName ( const std::string & name );
                    /**
                     * @fn std::string getOptionalValue();
                     * @brief Method getter optional value
                     * @date 26/04/2013
                     * @author Yadickson Soto
                     * @return optional value
                     */
                    std::string getOptionalValue();
                    /**
                     * @fn	virtual void setSize ( int width, int height );
                     * @brief	Sets the instance's size.
                     * @author	Yadickson Soto
                     * @date	25/02/2014
                     * @param	width 	Widget's width.
                     * @param	height	Widget's height.
                     */
                    virtual void setSize ( int width, int height );
                    /**
                     * @fn	virtual void setAccessibleName ( const std::string & name );
                     * @brief	setter accessible name method.
                     * @author	Yadickson Soto
                     * @date	14/03/2014
                     * @param name new accessible name
                     */
                    virtual void setAccessibleName ( const std::string & name );

                protected:

                    /**
                     * @fn	virtual bool onClicked ( GdkEventButton * event );
                     * @brief	methods that receives event box button press mask.
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param	event	the event.
                     * @return	true if it succeeds
                     */
                    virtual bool onClicked ( GdkEventButton * event );
                    /**
                     * @fn void click();
                     * @brief Method click
                     * @date 28/11/2012
                     * @author Yadickson Soto
                     */
                    void click();
                    /**
                     * @fn void initialize( const std::string & code, const std::string & optionalValue, bool enableClick, bool showBorder, bool setColors );
                     * @brief Method initialize
                     * @date 26/04/2013
                     * @author Yadickson Soto
                     * @param code code
                     * @param optionalValue optional value
                     * @param enableClick true if enable click
                     * @param showBorder true is show border
                     * @param setColors set colors
                     */
                    void initialize ( const std::string & code, const std::string & optionalValue, bool enableClick, bool showBorder, bool setColors );

                private:

                    std::string code; ///< code
                    std::string optionalValue; ///< optional value
                    std::string accessibleName; ///< accessible name

                    bool flagForce; ///< flag force
                    bool flagSelected; ///< flag selected
                    bool setColors; ///< flag set colors

                    Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * eventbox; ///< event box container
                    Gdk::Color selectedColor; ///< selected color
                    Gdk::Color unselectedColor; ///< unselected color
                    sigc::signal<void, BoxSelectionWidget *> signalClick; ///< signal click
            };

        } /* namespace Widgets */
    } /* namespace GUI */
} /* namespace Smartmatic */
#endif /* BOXSELECTIONWIDGET_HXX_ */
