/**
 * @file EventboxSelectionWidget.cpp
 * @brief Header class event box selection widget
 * @date 05/09/2011
 * @author Juan Delgado
 */

#include "EventboxSelectionWidget.h"
#include "SmartmaticBasicLabel.hxx"
#include "SmartmaticBasicEventBox.hxx"
#include <sstream>

using namespace Gtk;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;

static const int EXTRA_PADDING = 12;

EventboxSelectionWidget::EventboxSelectionWidget ( std::list< std::map < std::string, std::list < std::map < std::string, std::string > > > > & selectionMap, std::string titleAccessibleName, const std::string& boxAccessibleName )
    : SmartmaticBasicVBox ()
{
    widthRequested = -1;
    selectedBoxColor = Gdk::Color ( "white" );
    unselectedBoxColor = Gdk::Color ( "white" );
    selectedFontColor = Gdk::Color ( "black" );
    unselectedFontColor = Gdk::Color ( "black" );
    this->selectionMap = selectionMap;
    eventboxMap.clear();
    labelMap.clear();
    namesSelected.clear();
    valuesSelected.clear();
    titleLabelList.clear();
    namesSelected.clear();
    valuesSelected.clear();
    m_initializing = true;

    std::list< std::map < std::string, std::list < std::map < std::string, std::string > > > >::iterator iter;

    for ( iter = selectionMap.begin();
            iter != selectionMap.end();
            ++iter )
    {
        std::map < std::string, std::list < std::map < std::string, std::string > > >::iterator iterTitle;
        std::map < std::string, std::list < std::map < std::string, std::string > > > & mapTitle ( *iter );

        for ( iterTitle = mapTitle.begin();
                iterTitle != mapTitle.end();
                ++iterTitle )
        {
            SmartmaticBasicLabel * labelTitle = manage ( new SmartmaticBasicLabel ( iterTitle->first ) );
            labelTitle->show();
            pack_start ( *labelTitle, false, false );
            titleLabelList.push_back ( labelTitle );
            VBox * insideVbox = manage ( new VBox ( true, 1 ) );
            insideVbox->show();
            pack_start ( *insideVbox );

            std::list < std::map < std::string, std::string > >::iterator iterItems;
            std::list < std::map < std::string, std::string > > & mapItems ( iterTitle->second );
            int index;

            for ( iterItems = mapItems.begin(), index = 0;
                    iterItems != mapItems.end();
                    ++iterItems, ++index )
            {
                std::map < std::string, std::string >::iterator iter2;
                std::map < std::string, std::string > & textItem ( *iterItems );

                for ( iter2 = textItem.begin();
                        iter2 != textItem.end();
                        ++iter2 )
                {
                    Frame * frame = manage ( new Frame() );
                    frame->show();
                    insideVbox->pack_start ( *frame );
                    SmartmaticBasicEventBox * eventbox = manage ( new SmartmaticBasicEventBox() );
                    eventbox->show();
                    eventbox->set_events ( Gdk::BUTTON_PRESS_MASK );
                    sigc::connection connection = eventbox->signal_button_release_event().connect ( sigc::bind<std::string> (
                                sigc::mem_fun ( *this, &EventboxSelectionWidget::on_event_selected ),
                                iter2->first ) );
                    m_connectionMap.insert(std::pair<std::string, sigc::connection>(iter2->first, connection));
                    frame->add ( *eventbox );
                    eventboxMap[iter2->first] = eventbox;
                    SmartmaticBasicLabel * label = manage ( new SmartmaticBasicLabel ( iter2->second ) );
                    label->show();
                    eventbox->add ( *label );
                    labelMap[iter2->first] = label;

                    std::stringstream indexToStringStream;
                    indexToStringStream << boxAccessibleName << index;
                    eventbox->setAccessibleName ( indexToStringStream.str() );
                    label->setAccessibleName ( indexToStringStream.str() );
                }
            }

            labelTitle->setAccessibleName ( titleAccessibleName );
        }
    }

    show();
}

EventboxSelectionWidget::~EventboxSelectionWidget()
{
}

void EventboxSelectionWidget::SetFirstSelected()
{
    std::list< std::map < std::string, std::list < std::map < std::string, std::string > > > >::iterator iter;

    for ( iter = selectionMap.begin();
            iter != selectionMap.end();
            ++iter )
    {
        std::map < std::string, std::list < std::map < std::string, std::string > > >::iterator iterTitle;
        std::map < std::string, std::list < std::map < std::string, std::string > > > & mapTitle ( *iter );

        for ( iterTitle = mapTitle.begin();
                iterTitle != mapTitle.end();
                ++iterTitle )
        {
            std::list < std::map < std::string, std::string > >::iterator iterItems;
            std::list < std::map < std::string, std::string > > & mapItems ( iterTitle->second );

            for ( iterItems = mapItems.begin();
                    iterItems != mapItems.end();
                    ++iterItems )
            {
                std::map < std::string, std::string >::iterator iter2;
                std::map < std::string, std::string > & textItem ( *iterItems );

                for ( iter2 = textItem.begin();
                        iter2 != textItem.end();
                        ++iter2 )
                {
                    on_event_selected ( NULL, iter2->first );
                    return;
                }
            }
        }
    }
}

void EventboxSelectionWidget::disconnectSignals(const bool emitDisconnectionSignal)
{
    std::map<std::string, sigc::connection>::iterator it = m_connectionMap.begin();

    for (; it != m_connectionMap.end(); ++it)
    {
        it->second.disconnect();
    }

    if (emitDisconnectionSignal)
    {
        m_disconnectionSignal.emit(m_initializing);
    }
}

EventboxSelectionWidget::DisconnectionSignal EventboxSelectionWidget::getDisconnectionSignal()
{
    return m_disconnectionSignal;
}

void EventboxSelectionWidget::reconnectSignals()
{
    std::map<std::string, sigc::connection>::iterator it = m_connectionMap.begin();

    for (; it != m_connectionMap.end(); ++it)
    {
        it->second = eventboxMap[it->first]->signal_button_release_event().connect ( sigc::bind<std::string> (
            sigc::mem_fun ( *this, &EventboxSelectionWidget::on_event_selected ),
            it->first ) );
    }
}

bool EventboxSelectionWidget::on_event_selected ( GdkEventButton * event, const std::string& value )
{
    std::list< std::map < std::string, std::list < std::map < std::string, std::string > > > >::iterator iter;

    for ( iter = selectionMap.begin();
            iter != selectionMap.end();
            ++iter )
    {
        std::map < std::string, std::list < std::map < std::string, std::string > > >::iterator iterTitle;
        std::map < std::string, std::list < std::map < std::string, std::string > > > & mapTitle ( *iter );

        for ( iterTitle = mapTitle.begin();
                iterTitle != mapTitle.end();
                ++iterTitle )
        {
            std::list < std::map < std::string, std::string > >::iterator iterItems;
            std::list < std::map < std::string, std::string > > & mapItems ( iterTitle->second );

            for ( iterItems = mapItems.begin();
                    iterItems != mapItems.end();
                    ++iterItems )
            {
                std::map < std::string, std::string >::iterator iter2;
                std::map < std::string, std::string > & textItem ( *iterItems );

                for ( iter2 = textItem.begin();
                        iter2 != textItem.end();
                        ++iter2 )
                {
                    if ( iter2->first == value )
                    {
                        SetEventboxSelected ( true, labelMap[iter2->first], eventboxMap[iter2->first] );
                        namesSelected[iterTitle->first] = iter2->second;
                        valuesSelected[iterTitle->first] = iter2->first;
                    }
                    else
                    {
                        SetEventboxSelected ( false, labelMap[iter2->first], eventboxMap[iter2->first] );
                    }
                }
            }
        }
    }

    disconnectSignals();

    m_type_box_clicked.emit ( GetValuesSelected() );
    return true;
}

void EventboxSelectionWidget::set_size_request ( int width, int height )
{
    VBox::set_size_request ( width, height );
    widthRequested = width - 50;

    for ( std::map< std::string, Label * >::iterator iter = labelMap.begin();
            iter != labelMap.end(); ++iter )
    {
        iter->second->set_size_request ( widthRequested, -1 );
    }

    for ( std::list <Gtk::Label * >::iterator iter = titleLabelList.begin();
            iter != titleLabelList.end(); ++iter )
    {
        ( *iter )->set_size_request ( widthRequested, -1 );
        ( *iter )->set_line_wrap ( true );
    }
}

void EventboxSelectionWidget::SetEventboxSelected ( bool isSelected, Label * label, EventBox * eventbox )
{
    if ( isSelected )
    {
        eventbox->modify_bg ( Gtk::STATE_NORMAL, selectedBoxColor );
        LabelConfiguration::SetLabelConfiguration ( label, selectedLabelConfiguration );
        label->modify_fg ( Gtk::STATE_NORMAL, selectedFontColor );
    }
    else
    {
        eventbox->modify_bg ( Gtk::STATE_NORMAL, unselectedBoxColor );
        LabelConfiguration::SetLabelConfiguration ( label, unselectedLabelConfiguration );
        label->modify_fg ( Gtk::STATE_NORMAL, unselectedFontColor );
    }

    Glib::signal_timeout().connect_once(sigc::mem_fun(*this, &EventboxSelectionWidget::adjustOptionSize), 1);
}

void EventboxSelectionWidget::adjustOptionSize()
{
    std::map<std::string, Gtk::Label*>::iterator it;

    for (it = labelMap.begin(); it != labelMap.end(); ++it)
    {
        Gtk::Label* label = it->second;
        Glib::RefPtr<Pango::Layout> layout = label->get_layout();

        layout->set_width((widthRequested * Pango::SCALE) - 1);

        if (layout->get_line_count() > 1)
        {
            int maxLineSize = 0;

            for (int i = 0; i < layout->get_line_count(); ++i)
            {
                Pango::Rectangle logical = layout->get_line(i)->get_pixel_logical_extents();

                if (maxLineSize < logical.get_width())
                {
                    maxLineSize = logical.get_width();
                }
            }

            int padding = ((widthRequested - maxLineSize) / 2) + EXTRA_PADDING;

            label->set_line_wrap(true);
            label->set_padding(padding, 0);
        }
    }
}

std::list< std::string > EventboxSelectionWidget::GetNamesSelected()
{
    std::list< std::string > returnList;

    for ( std::map< std::string, std::string >::iterator iter = namesSelected.begin();
            iter != namesSelected.end(); ++iter )
    {
        returnList.push_back ( iter->second );
    }

    return returnList;
}

std::list< std::string > EventboxSelectionWidget::GetValuesSelected()
{
    std::list< std::string > returnList;

    for ( std::map< std::string, std::string >::iterator iter = valuesSelected.begin();
            iter != valuesSelected.end(); ++iter )
    {
        returnList.push_back ( iter->second );
    }

    return returnList;
}

void EventboxSelectionWidget::SetSelectedBoxColor ( Gdk::Color color )
{
    selectedBoxColor = color;
}

void EventboxSelectionWidget::SetUnselectedBoxColor ( Gdk::Color color )
{
    unselectedBoxColor = color;
}

void EventboxSelectionWidget::SetSelectedFontColor ( Gdk::Color color )
{
    selectedFontColor = color;
}

void EventboxSelectionWidget::SetUnselectedFontColor ( Gdk::Color color )
{
    unselectedFontColor = color;
}

void EventboxSelectionWidget::SetSelectedFontDescription ( LabelConfiguration & labelConfiguration )
{
    selectedLabelConfiguration = labelConfiguration;
}

void EventboxSelectionWidget::SetUnselectedFontDescription ( LabelConfiguration & labelConfiguration )
{
    unselectedLabelConfiguration = labelConfiguration;
}

void EventboxSelectionWidget::SetTitlesFontDescription ( LabelConfiguration & labelConfiguration )
{
    for ( std::list <Gtk::Label * >::iterator iter = titleLabelList.begin();
            iter != titleLabelList.end(); ++iter )
    {
        LabelConfiguration::SetLabelConfiguration ( ( *iter ), labelConfiguration );
    }
}

void EventboxSelectionWidget::SetTitlesHeight ( int height )
{
    for ( std::list <Gtk::Label * >::iterator iter = titleLabelList.begin();
            iter != titleLabelList.end(); ++iter )
    {
        ( *iter )->set_size_request ( widthRequested, height );
    }
}

EventboxSelectionWidget::type_box_clicked EventboxSelectionWidget::signal_box_clicked()
{
    return m_type_box_clicked;
}

void EventboxSelectionWidget::setInitializing(const bool initializing)
{
    m_initializing = initializing;
}
