/**
 * @file HelpExitButtons.h
 * @brief Header class help exit buttons
 * @date 31/05/2011
 * @author Juan Delgado
 */

#ifndef HELPEXITBUTTONS_H_
#define HELPEXITBUTTONS_H_

#include <gtkmm.h>
#include "Widgets/exit-button-listener.h"
#include "Widgets/help-button-listener.h"
#include "Widgets/smartmatic-button.h"
#include <Widgets/SmartmaticBasicHBox.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {

            /**
             * @class	HelpExitButtons
             * @brief	Help exit buttons.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class HelpExitButtons : public SmartmaticBasicHBox
            {
                public:


                    /**
                     * @fn	HelpExitButtons::HelpExitButtons(ExitButtonListener *exitListener, bool showExitButton,
                     * 			HelpButtonListener* helpListener, bool showHelpButton);
                     * @brief	Constructor.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	exitListener		If non-null, the exit listener.
                     * @param	showExitButton	true to show, false to hide the exit button.
                     * @param	helpListener		If non-null, the help listener.
                     * @param	showHelpButton	true to show, false to hide the help button.
                     */
                    HelpExitButtons ( ExitButtonListener * exitListener, bool showExitButton,
                                      HelpButtonListener * helpListener, bool showHelpButton );

                    /**
                     * @fn	HelpExitButtons::~HelpExitButtons();
                     * @brief	Destructor.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    ~HelpExitButtons();

                    /**
                     * @fn	void HelpExitButtons::ExitButton_SetSensitive(bool sensitive);
                     * @brief	Set sensitive.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	sensitive	true to sensitive.
                     */
                    void ExitButton_SetSensitive ( bool sensitive );

                    /**
                     * @fn	void HelpExitButtons::ExitButton_SetVisible(bool visible);
                     * @brief	Set visible the button.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	visible	true to show, false to hide.
                     */
                    void ExitButton_SetVisible ( bool visible );

                    /**
                     * @fn	void HelpExitButtons::ExitButton_SetName(std::string name);
                     * @brief	Set name to the button.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	name	The name.
                     */
                    void ExitButton_SetName ( std::string name );

                    /**
                     * @fn	void HelpExitButtons::HelpButton_SetSensitive(bool sensitive);
                     * @brief	Set sensitive.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	sensitive	true to sensitive.
                     */
                    void HelpButton_SetSensitive ( bool sensitive );

                    /**
                     * @fn	void HelpExitButtons::HelpButton_SetVisible(bool visible);
                     * @brief	Set visible the button.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	visible	true to show, false to hide.
                     */
                    void HelpButton_SetVisible ( bool visible );

                    /**
                     * @fn	void HelpExitButtons::HelpButton_SetName(std::string name);
                     * @brief	Set name to the button.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	name	The name.
                     */
                    void HelpButton_SetName ( std::string name );

                protected:
                    SmartmaticButton * exitButton; ///< The exit button
                    SmartmaticButton * helpButton; ///< The help button

                    ExitButtonListener * exitListener; ///< The exit listener
                    HelpButtonListener * helpListener; ///< The help listener

                    /**
                     * @fn	void HelpExitButtons::on_exitButton_clicked();
                     * @brief	Handles exit button clicked signals.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void on_exitButton_clicked();

                    /**
                     * @fn	void HelpExitButtons::on_helpButton_clicked();
                     * @brief	Handles help button clicked signals.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void on_helpButton_clicked();

            };
        }
    }
}

#endif /* HELPEXITBUTTONS_H_ */
