/**
 * @file SmartmaticBasicVBox.hxx
 * @brief Header basic vbox widget class
 * @date 25/04/2013
 * @author Yadickson Soto
 */

#ifndef _SMARTMATIC_BASICVBOX_H_
#define _SMARTMATIC_BASICVBOX_H_

#include <gtkmm.h>
#include <Operation/AccessibleNameWidget.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {
            /**
             * @class	SmartmaticBasicVBox
             * @brief	Smartmatic basic vbox.
             * @date 25/04/2013
             * @author Yadickson Soto
             */
            class SmartmaticBasicVBox: public Gtk::VBox, public Smartmatic::GUI::Operation::AccessibleNameWidget
            {

                public:

                    /**
                     * @fn	SmartmaticBasicVBox( bool homogeneous = false, int spacing = 0 );
                     * @brief	Default constructor.
                     * @date 25/04/2013
                     * @author Yadickson Soto
                     * @param homogeneous Whether each widget in the VBox should have the same
                     * height.  If set, a PACK_SHRINK argument to pack_start() or pack_end() is ignored.
                     * @param spacing Determines the space in pixels between child widgets.
                     */
                    SmartmaticBasicVBox ( bool homogeneous = false, int spacing = 0 );

                    /**
                     * @fn	SmartmaticBasicVBox(BaseObjectType* cobject,
                     * 			const Glib::RefPtr<Gtk::Builder>& refBuilder=Gtk::Builder::create());
                     * @brief	Constructor.
                     * @date 25/04/2013
                     * @author Yadickson Soto
                     * @param	cobject			If non-null, the cobject.
                     * @param	refBuilder	(optional) the reference builder.
                     */
                    SmartmaticBasicVBox ( BaseObjectType * cobject, const Glib::RefPtr<Gtk::Builder> & refBuilder = Gtk::Builder::create() );

                    /**
                     * @fn	virtual ~SmartmaticBasicVBox();
                     * @brief	Class destructor.
                     * @date 25/04/2013
                     * @author Yadickson Soto
                     */
                    virtual ~SmartmaticBasicVBox();
            };
        }
    }
}

#endif // _SMARTMATIC_BASICVBOX_H_
