/**
 * @file SmartmaticRadioButton.hxx
 * @brief Header class smartmatic radio button
 * @date 21/09/2012
 * @author Yadickson Soto
 */

#ifndef SMARTMATICRADIOBUTTON_HXX_
#define SMARTMATICRADIOBUTTON_HXX_

#include <Widgets/SmartmaticBasicRadioButton.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {
            /**
             * @class	SmartmaticRadioButton
             * @brief	class smartmatic radio button.
             * @author	Yadickson Soto
             * @date	21/09/2012
             */
            class SmartmaticRadioButton : public SmartmaticBasicRadioButton
            {
                public:
                    /**
                     * @fn	SmartmaticRadioButton::SmartmaticRadioButton();
                     * @brief	class constructor.
                     * @author	Yadickson Soto
                     * @date	16/10/2013
                     */
                    SmartmaticRadioButton ();
                    /**
                     * @fn	SmartmaticRadioButton::SmartmaticRadioButton(const std::string & name,
                     * 			const std::string & code);
                     * @brief	class constructor.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @param	name	name.
                     * @param	code	code.
                     */
                    SmartmaticRadioButton ( const std::string & name, const std::string & code );
                    /**
                     * @fn	SmartmaticRadioButton::SmartmaticRadioButton(BaseObjectType* cobject,
                     * 			const Glib::RefPtr<Gtk::Builder>& refBuilder=Gtk::Builder::create());
                     * @brief	class constructor.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @param	cobject			object.
                     * @param	refBuilder	reference to builder.
                     */
                    SmartmaticRadioButton ( BaseObjectType * cobject, const Glib::RefPtr<Gtk::Builder> & refBuilder = Gtk::Builder::create() );
                    /**
                     * @fn	virtual SmartmaticRadioButton::~SmartmaticRadioButton();
                     * @brief	class destructor.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     */
                    virtual ~SmartmaticRadioButton();
                    /**
                     * @fn	std::string SmartmaticRadioButton::getName();
                     * @brief	method getter name.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @return	name.
                     */
                    std::string getName();
                    /**
                     * @fn	std::string SmartmaticRadioButton::getCode();
                     * @brief	method getter code.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @return	code.
                     */
                    std::string getCode();
                    /**
                     * @fn	std::string getAttribute ();
                     * @brief	method getter the optional attribute.
                     * @author	Yadickson Soto
                     * @date	30/01/2014
                     * @return	The optional attribute.
                     */
                    std::string getAttribute ();

                    /**
                     * @fn	void SmartmaticRadioButton::setName(const std::string & name);
                     * @brief	method setter name.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @param	name	name.
                     */
                    void setName ( const std::string & name );
                    /**
                     * @fn	void SmartmaticRadioButton::setCode(const std::string & code);
                     * @brief	method setter code.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @param	code	code.
                     */
                    void setCode ( const std::string & code );
                    /**
                     * @fn	void setAttributes ( const std::string & attribute );
                     * @brief	method setter the optional attribute.
                     * @author	Yadickson Soto
                     * @date	30/01/2014
                     * @param	attribute	The optional attribute.
                     */
                    void setAttribute ( const std::string & attribute );
                    /**
                     * @fn	void setFontSize ( int fontSize );
                     * @brief	method modified font size.
                     * @author	Yadickson Soto
                     * @date	16/10/2013
                     * @param	fontSize font size.
                     */
                    void setFontSize ( int fontSize );

                private:

                    std::string name; ///< visual name name
                    std::string code; ///< code
                    std::string attribute; ///< The optional atribute
                    int fontSize;	///< the font size
            };
        }
    }
}

#endif /* SMARTMATICRADIOBUTTON_HXX_ */
