/**
 * @file SmartmaticRadioButtonGroup.hxx
 * @brief Header class smartmatic radio button group
 * @date 21/09/2012
 * @author Yadickson Soto
 */

#ifndef SMARTMATICRADIOBUTTONGROUP_HXX_
#define SMARTMATICRADIOBUTTONGROUP_HXX_

#include <gtkmm.h>
#include <set>
#include <Widgets/SmartmaticRadioButton.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {
            /**
             * @class	SmartmaticRadioButtonGroup
             * @brief	class smartmatic radio button group.
             * @author	Yadickson Soto
             * @date	21/09/2012
             */
            class SmartmaticRadioButtonGroup
            {
                public:
                    /**
                     * @fn	SmartmaticRadioButtonGroup::SmartmaticRadioButtonGroup();
                     * @brief	class constructor.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     */
                    SmartmaticRadioButtonGroup();
                    /**
                     * @fn	virtual SmartmaticRadioButtonGroup::~SmartmaticRadioButtonGroup();
                     * @brief	class destructor.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     */
                    virtual ~SmartmaticRadioButtonGroup();
                    /**
                     * @fn	void SmartmaticRadioButtonGroup::addRadioButton(SmartmaticRadioButton * radioButton, bool grouped = true);
                     * @brief	method add new radio button.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @param	radioButton	new radio button.
                     * @param 	grouped grouped radio buttons.
                     */
                    void addRadioButton ( SmartmaticRadioButton * radioButton, bool grouped = true );
                    /**
                     * @fn	SmartmaticRadioButton * SmartmaticRadioButtonGroup::getRadioButtonSelected();
                     * @brief	method getter current radio button selected.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @return	NULL if not selected radio button.
                     */
                    SmartmaticRadioButton * getRadioButtonSelected();
                    /**
                     * @fn	sigc::signal<void, SmartmaticRadioButton *> SmartmaticRadioButtonGroup::getSignalChange();
                     * @brief	method getter signal change.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     * @return	signal change.
                     */
                    sigc::signal<void, SmartmaticRadioButton *> getSignalChange();
                    /**
                     * @fn	void SmartmaticRadioButtonGroup::start();
                     * @brief	method start emit signal.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     */
                    void start();
                    /**
                     * @fn	void SmartmaticRadioButtonGroup::clearAll();
                     * @brief	method unselected all options.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     */
                    void clearAll();

                private:

                    /**
                     * @class	GroupButton
                     * @brief	Group button class.
                     * @author	Yadickson Soto
                     * @date	21/09/2012
                     */
                    class GroupButton: public Gtk::RadioButtonGroup
                    {
                        public:

                            /**
                             * @fn	GroupButton::GroupButton();
                             * @brief	class constructor.
                             * @author	Yadickson Soto
                             * @date	21/09/2012
                             */
                            GroupButton();
                    };


                    /**
                     * @fn	void SmartmaticRadioButtonGroup::changeOption(Gtk::StateType type);
                     * @brief	Change option.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @param	type	The type.
                     */
                    void changeOption ( Gtk::StateType type );

                    bool emitSignal; ///< flag emit signal
                    SmartmaticRadioButton * current; ///< current radio button

                    GroupButton group; ///< group
                    std::set <SmartmaticRadioButton *> radioButtonList; ///< radio button list
                    sigc::signal<void, SmartmaticRadioButton *> signalChange; ///< signal change
                    std::list <sigc::connection> connectionList; ///< connection list

                protected:

                    /**
                     * @fn	virtual void add ( SmartmaticRadioButton * widget ) = 0;
                     * @brief	Add new radio button widget.
                     * @author	Yadickson Soto
                     * @date	16/10/2013
                     * @param	widget new widget to add
                     */
                    virtual void add ( SmartmaticRadioButton * widget ) = 0;
            };
        }
    }
}

#endif /* SMARTMATICRADIOBUTTONGROUP_HXX_ */
