/**
 * @file TableCenterWidget.cxx
 * @brief Body class table center widget
 * @date 13/03/2012
 * @author Yadickson Soto
 */

#include "TableCenterWidget.hxx"
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;

ISMTTLog * TableCenterWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.GUI.Widgets.TableCenterWidget", GETTEXT_PACKAGE );

TableCenterWidget::TableCenterWidget ( bool homogeneous, bool verticalCenter )
    : SmartmaticBasicEventBox()
{
    this->mainEventBox = NULL;
    this->mainHbox = NULL;
    this->mainVbox = NULL;
    this->tableBox = NULL;
    this->table = NULL;
    mainEventBox = new Smartmatic::GUI::Widgets::SmartmaticBasicEventBox();
    mainHbox = new Smartmatic::GUI::Widgets::SmartmaticBasicHBox();
    mainHbox->set_homogeneous ( true );
    mainVbox = new Smartmatic::GUI::Widgets::SmartmaticBasicVBox();
    mainVbox->set_homogeneous ( false );
    tableBox = new Smartmatic::GUI::Widgets::SmartmaticBasicEventBox();

    mainVbox->pack_start ( *tableBox, verticalCenter, false );
    mainHbox->pack_start ( *mainVbox, false, false );
    mainEventBox->add ( *mainHbox );
    table = new TableWidget();
    tableBox->add ( *table );
    table->getTable()->set_homogeneous ( homogeneous );

    add ( *mainEventBox );
    show_all_children();
}

TableCenterWidget::~TableCenterWidget()
{
    dispose();
}

void TableCenterWidget::initialize()
{
}

void TableCenterWidget::dispose()
{
    remove();

    if ( mainEventBox )
    {
        delete mainEventBox;
        mainEventBox = NULL;
    }

    if ( mainHbox )
    {
        delete mainHbox;
        mainHbox = NULL;
    }

    if ( mainVbox )
    {
        delete mainVbox;
        mainVbox = NULL;
    }

    if ( tableBox )
    {
        delete tableBox;
        tableBox = NULL;
    }

    if ( table )
    {
        delete table;
        table = NULL;
    }
}

void TableCenterWidget::resize ( unsigned int row, unsigned int column )
{
    table->resize ( row, column );
}

void TableCenterWidget::put ( void * widget, int row, int column )
{
    Gtk::Widget * w = static_cast < Gtk::Widget * > ( widget );
    table->put ( w, row, column );
}

void TableCenterWidget::put ( Gtk::Widget * widget, int row, int column, Gtk::AttachOptions xAlign, Gtk::AttachOptions yAlign )
{
    table->put ( widget, row, column, xAlign, yAlign );
}

void TableCenterWidget::setSpacings ( int row, int column )
{
    table->setSpacings ( row, column );
}

unsigned int TableCenterWidget::getCountElements()
{
    return table->getCountElements();
}

void TableCenterWidget::clear()
{
    table->clear();
}

SmartmaticBasicTable * TableCenterWidget::getTable()
{
    return table->getTable();
}
