/**
 * @file TableCenterWidget.hxx
 * @brief Header class table center widget
 * @date 13/03/2012
 * @author Yadickson Soto
 */

#ifndef TABLECENTERWIDGET_HXX_
#define TABLECENTERWIDGET_HXX_

#include <Widgets/TableWidget.hxx>
#include <Widgets/ITableWidget.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/SmartmaticBasicTable.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/SmartmaticBasicHBox.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace GUI
    {
        namespace Widgets
        {
            /**
             * @class	TableCenterWidget
             * @brief	Class TableCenterWidget.
             * @author	Yadickson Soto
             * @date	13/03/2012
             */
            class TableCenterWidget : public SmartmaticBasicEventBox, public ITableWidget
            {
                public:
                    /**
                     * @fn	TableCenterWidget::TableCenterWidget(bool homogeneous = true, bool verticalCenter = false);
                     * @brief	Class constructor.
                     * @author	Yadickson Soto
                     * @date	16/05/2012
                     * @param	homogeneous			column homogeneous.
                     * @param	verticalCenter	vertical center.
                     */
                    TableCenterWidget ( bool homogeneous = true, bool verticalCenter = false );
                    /**
                     * @fn	virtual TableCenterWidget::~TableCenterWidget();
                     * @brief	Class destructor.
                     * @author	Yadickson Soto
                     * @date	13/03/2012
                     */
                    virtual ~TableCenterWidget();
                    /**
                     * @fn	virtual void TableCenterWidget::initialize();
                     * @brief	method initialize class.
                     * @author	Yadickson Soto
                     * @date	13/03/2012
                     */
                    virtual void initialize();
                    /**
                     * @fn	virtual void TableCenterWidget::dispose();
                     * @brief	method dispose class.
                     * @author	Yadickson Soto
                     * @date	13/03/2012
                     */
                    virtual void dispose();
                    /**
                     * @fn	virtual void TableCenterWidget::setSpacings(int row, int column);
                     * @brief	method set spacings table.
                     * @author	Yadickson Soto
                     * @date	16/03/2012
                     * @param	row			row spacings.
                     * @param	column	column spacings.
                     */
                    virtual void setSpacings ( int row, int column );
                    /**
                     * @fn	virtual void TableCenterWidget::resize(unsigned int row, unsigned int column);
                     * @brief	method resize table.
                     * @author	Yadickson Soto
                     * @date	16/03/2012
                     * @param	row			row number.
                     * @param	column	column number.
                     */
                    virtual void resize ( unsigned int row, unsigned int column );
                    /**
                     * @fn	virtual void TableCenterWidget::put( void * widget, int row, int column);
                     * @brief	method put new element in table.
                     * @author	Yadickson Soto
                     * @date	16/03/2012
                     * @param	widget	new widget.
                     * @param	row			row position.
                     * @param	column	column position.
                     */
                    virtual void put ( void * widget, int row, int column );
                    /**
                     * @fn	void TableCenterWidget::put(Gtk::Widget * widget, int row, int column,
                     * 			Gtk::AttachOptions xAlign, Gtk::AttachOptions yAlign);
                     * @brief	method put new element in table.
                     * @author	Yadickson Soto
                     * @date	16/03/2012
                     * @param	widget	new widget.
                     * @param	row			row position.
                     * @param	column	column position.
                     * @param	xAlign	x align.
                     * @param	yAlign	y align.
                     */
                    void put ( Gtk::Widget * widget, int row, int column, Gtk::AttachOptions xAlign, Gtk::AttachOptions yAlign );
                    /**
                     * @fn	unsigned int TableCenterWidget::getCountElements();
                     * @brief	method getter count elements.
                     * @author	Yadickson Soto
                     * @date	18/05/2012
                     * @return	count elements.
                     */
                    virtual unsigned int getCountElements();
                    /**
                     * @fn	void TableCenterWidget::clear();
                     * @brief	method clear elements.
                     * @author	Yadickson Soto
                     * @date	21/05/2012
                     */
                    void clear();
                    /**
                     * @fn	SmartmaticBasicTable * getTable();
                     * @brief	method getter table.
                     * @author	Yadickson Soto
                     * @date	25/04/2013
                     * @return	table.
                     */
                    SmartmaticBasicTable * getTable();

                private:

                    Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * mainEventBox; ///< The main event box frame
                    Smartmatic::GUI::Widgets::SmartmaticBasicHBox * mainHbox; ///< The main hbox
                    Smartmatic::GUI::Widgets::SmartmaticBasicVBox * mainVbox; ///< The main vbox
                    Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * tableBox; ///< table box

                    Smartmatic::GUI::Widgets::TableWidget * table; ///< table elements
                    static Smartmatic::Log::ISMTTLog * logger;  ///< logger
            };
        }
    }
}

#endif /* TABLECENTERWIDGET_HXX_ */
