/**
 * @file WizardManager.cxx
 * @brief Implementation of the wizard manager class.
 * @date 08/04/2014
 * @author Yadickson Soto
 */

#include "WizardManager.hxx"

using namespace Smartmatic::GUI::Widgets;

WizardManager::WizardManager()
{
    manager = new WizardWidgetManager ();
    setWidget ( manager );
}

WizardManager::~WizardManager()
{
    this->remove();

    connectionManager.disconnect();

    std::vector < sigc::connection >::iterator itc;

    for ( itc = connections.begin();
            itc != connections.end();
            ++itc )
    {
        ( *itc ).disconnect();
    }

    connections.clear();

    std::vector < WizardWidgetBase * >::iterator it;

    for ( it = wizardVector.begin();
            it != wizardVector.end();
            ++it )
    {
        delete ( *it );
    }

    wizardVector.clear();

    if ( manager )
    {
        delete manager;
        manager = NULL;
    }
}

void WizardManager::add ( WizardWidgetBase * widget )
{
    wizardVector.push_back ( widget );
    connections.push_back ( widget->on_stop_signal_emit().connect ( sigc::mem_fun ( *this, &WizardManager::error ) ) );
    connections.push_back ( widget->on_cancel_signal_emit().connect ( sigc::mem_fun ( *this, &WizardManager::cancel ) ) );
}

void WizardManager::initialize()
{
    connectionManager.disconnect();
    connectionManager = manager->on_finish_signal_emit().connect ( sigc::mem_fun ( *this, &WizardManager::emit ) );
    manager->Start ( wizardVector );
}

void WizardManager::error()
{
    on_stop_signal_emit().emit();
}

void WizardManager::cancel()
{
    on_cancel_signal_emit().emit();
}
