/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmaticgui
 * Copyright (C) Juan Delgado 2011 <juan.delgado@smartmatic.com>
 *
 * smartmaticgui is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmaticgui is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @file image-selection-widget.h
 * @brief widget that displays images, and one can be selected
 * @author Juan delgado <juan.delgado@smartmatic.com>
 */

#ifndef _IMAGE_SELECTION_WIDGET_H_

#define _IMAGE_SELECTION_WIDGET_H_

#include <gtkmm.h>
#include <Widgets/SmartmaticBasicHBox.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {

            /**
             * @class	ImageSelectionWidget
             * @brief	Image selection widget.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class ImageSelectionWidget: public SmartmaticBasicHBox
            {
                public:

                    /**
                     * @fn	ImageSelectionWidget::ImageSelectionWidget(std::vector<std::string> imagesPath,
                     * 			std::vector<std::string> imagesCode);
                     * @brief	constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	imagesPath	path of images.
                     * @param	imagesCode	path of images.
                     */
                    ImageSelectionWidget ( std::vector<std::string> imagesPath, std::vector<std::string> imagesCode );

                    /**
                     * @fn	virtual ImageSelectionWidget::~ImageSelectionWidget();
                     * @brief	destructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~ImageSelectionWidget();

                    /**
                     * @typedef	sigc::signal<void, std::string> type_clicked
                     * @brief	signal definition.
                     */
                    typedef sigc::signal<void, std::string> type_clicked;

                    /**
                     * @fn	type_clicked ImageSelectionWidget::clicked();
                     * @brief	signal connect method.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	signal
                     */
                    type_clicked clicked();

                    /**
                     * @fn	std::vector<Smartmatic::GUI::Widgets::SmartmaticBasicEventBox *> & getEventBoxs();
                     * @brief	Method getter all event box create
                     * @author	Yadickson Soto
                     * @date	25/04/2013
                     * @return	vector with all event box
                     */
                    std::vector<Smartmatic::GUI::Widgets::SmartmaticBasicEventBox *> & getEventBoxs();

                protected:
                    /**
                     * @brief signals declaration
                     */
                    type_clicked m_clicked; ///< The clicked

                    /**
                     * @fn	bool ImageSelectionWidget::on_image_clicked(GdkEventButton* event, std::string imagePath);
                     * @brief	event box click connect method.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	event		 	If non-null, the event.
                     * @param	imagePath	Full pathname of the image file.
                     * @return	true if it succeeds, false if it fails.
                     */

                    bool on_image_clicked ( GdkEventButton * event, std::string imagePath );

                private:
                    /**
                     * child widgets
                     */
                    std::vector<Smartmatic::GUI::Widgets::SmartmaticBasicEventBox *> events; ///< The events
                    std::vector<Gtk::Image *> images;	///< The images
            };
        }
    }
}

#endif // _IMAGE_SELECTION_WIDGET_H_
