/**
 * @file smartmatic-label.h
 * @brief Header label widget class
 * @date 29/10/2012
 * @author Juan Delgado, Yadickson Soto
 */

#ifndef _SMARTMATIC_LABEL_H_
#define _SMARTMATIC_LABEL_H_

#include <gtkmm.h>
#include <Widgets/SmartmaticBasicLabel.hxx>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {

            /**
             * @class	SmartmaticLabel
             * @brief	Smartmatic label.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class SmartmaticLabel: public SmartmaticBasicLabel
            {

                public:

                    /**
                     * @fn	SmartmaticLabel::SmartmaticLabel();
                     * @brief	Default constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    SmartmaticLabel();

                    /**
                     * @fn	SmartmaticLabel::SmartmaticLabel(const Glib::ustring& label, bool mnemonic=false);
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	label			The label.
                     * @param	mnemonic	true to mnemonic.
                     */
                    SmartmaticLabel ( const Glib::ustring & label, bool mnemonic = false );

                    /**
                     * @fn	SmartmaticLabel::SmartmaticLabel(const Glib::ustring& label, float xalign, float yalign,
                     * 			bool mnemonic=false);
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	label			The label.
                     * @param	xalign		The xalign.
                     * @param	yalign		The yalign.
                     * @param	mnemonic	true to mnemonic.
                     */
                    SmartmaticLabel ( const Glib::ustring & label, float xalign, float yalign, bool mnemonic = false );

                    /**
                     * @fn	SmartmaticLabel::SmartmaticLabel(const Glib::ustring& label, Gtk::AlignmentEnum xalign,
                     * 			Gtk::AlignmentEnum yalign=Gtk::ALIGN_CENTER, bool mnemonic=false);
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	label			The label.
                     * @param	xalign		The xalign.
                     * @param	yalign		The yalign.
                     * @param	mnemonic	true to mnemonic.
                     */
                    SmartmaticLabel ( const Glib::ustring & label, Gtk::AlignmentEnum xalign, Gtk::AlignmentEnum yalign = Gtk::ALIGN_CENTER, bool mnemonic = false );

                    /**
                     * @fn	SmartmaticLabel::SmartmaticLabel(BaseObjectType* cobject,
                     * 			const Glib::RefPtr<Gtk::Builder>& refBuilder=Gtk::Builder::create());
                     * @brief	Constructor.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	cobject			If non-null, the cobject.
                     * @param	refBuilder	(optional) the reference builder.
                     */
                    SmartmaticLabel ( BaseObjectType * cobject, const Glib::RefPtr<Gtk::Builder> & refBuilder = Gtk::Builder::create() );

                    /**
                     * @fn	virtual SmartmaticLabel::~SmartmaticLabel();
                     * @brief	Finaliser.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~SmartmaticLabel();

                    /**
                     * @fn	void SmartmaticLabel::SetCustomWrapMode();
                     * @brief	Sets custom wrap mode.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void SetCustomWrapMode();

                    /**
                     * @fn	void SmartmaticLabel::SetSize(int size);
                     * @brief	Sets a size.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	size	The size.
                     */
                    void SetSize ( int size );

                    /**
                     * @fn	void SmartmaticLabel::SetBold(bool bold);
                     * @brief	Sets a bold.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	bold	true to bold.
                     */
                    void SetBold ( bool bold );

                    /**
                     * @fn	static void SmartmaticLabel::SetAutoWrap(Label &label, int width, int height);
                     * @brief	Sets automatically wrap for a label.
                     * @author	Juan.Delgado
                     * @date	02/08/2011
                     * @param	label 	The label.
                     * @param	width 	The width of the label.
                     * @param	height	The height of the label.
                     */
                    static void SetAutoWrap ( Label & label, int width, int height );

                protected:
                    //Overrides:

                    /**
                     * @fn	virtual void SmartmaticLabel::on_size_allocate(Gtk::Allocation& allocation);
                     * @brief	Handles size allocate signals.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	allocation	The allocation.
                     */

                    virtual void on_size_allocate ( Gtk::Allocation & allocation );

                    /**
                     * @fn	void SmartmaticLabel::Initialize();
                     * @brief	Initializes this object.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void Initialize();

                private:
                    bool customWrapMode;	///< true to enable custom wrap mode, false to disable it
                    bool isRecursive; ///< true if this object is recursive


            };
        }
    }
}

#endif // _SMARTMATIC_LABEL_H_
