/**
 * @file	wizard-widget-manager.cc
 * @brief	wizard widget manager class.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmaticgui
 * Copyright (C) Juan Delgado 2011 <juan.delgado@smartmatic.com>
 *
 * smartmaticgui is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmaticgui is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "wizard-widget-manager.h"

using namespace Gtk;
using namespace Smartmatic::GUI::Widgets;

WizardWidgetManager::WizardWidgetManager ()
    : StepWidget ()
{
    setAccessibleName ( "WizardWidgetManager" );
}

WizardWidgetManager::WizardWidgetManager ( const std::vector<WizardWidgetBase *>& widgets )
: StepWidget ()
, widgetsVector(widgets)
, iter(widgetsVector.begin())
{
    setAccessibleName ( "WizardWidgetManager" );
}

WizardWidgetManager::~WizardWidgetManager()
{
    widgetConnect.disconnect();
    widgetErrorConnect.disconnect();
    nextConnect.disconnect();
}

void WizardWidgetManager::ShowWidget()
{
    if ( !widgetsVector.empty() && *iter )
    {
        widgetConnect = ( *iter )->getSignalEnd().connect ( sigc::mem_fun ( this, &WizardWidgetManager::NextWidget ) );
        widgetErrorConnect = ( *iter )->getSignalError().connect ( sigc::mem_fun ( this, &WizardWidgetManager::onError ) );
        ( *iter )->initialize();
        ( *iter )->refresh();
        setWidget ( ( *iter ) );
        show_all_children();
    }
}

void WizardWidgetManager::NextWidget()
{
    widgetConnect.disconnect();
    widgetErrorConnect.disconnect();

    if ( !widgetsVector.empty() )
    {
        ( *iter )->dispose();
    }

    m_transition_signal.emit();
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &WizardWidgetManager::onNextWidgetInternal ), 0 );
}

void WizardWidgetManager::onError()
{
    widgetConnect.disconnect();
    widgetErrorConnect.disconnect();

    if ( !widgetsVector.empty() )
    {
        ( *iter )->dispose();
    }

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &WizardWidgetManager::onErrorInternal ), 0 );
}

void WizardWidgetManager::onNextWidgetInternal()
{
    signalNextWidget.emit();
}

void WizardWidgetManager::onErrorInternal()
{
    m_error_signal.emit();
}

void WizardWidgetManager::NextWidgetInternal()
{
    if ( !widgetsVector.empty() && ! ( *iter )->StopWizard() )
    {
        ++iter;

        if ( iter != this->widgetsVector.end() )
        {
            Glib::signal_timeout().connect_once ( sigc::mem_fun ( this, &WizardWidgetManager::ShowWidget ), 0 );
        }
        else
        {
            m_finish_signal.emit();
        }
    }
    else
    {
        m_finish_signal.emit();
    }
}

WizardWidgetManager::type_void_signal WizardWidgetManager::on_finish_signal_emit()
{
    return m_finish_signal;
}

WizardWidgetManager::type_void_signal WizardWidgetManager::on_transition_signal_emit()
{
    return m_transition_signal;
}

WizardWidgetManager::type_void_signal WizardWidgetManager::on_error_signal_emit()
{
    return m_error_signal;
}

void WizardWidgetManager::Start()
{
    if ( !widgetsVector.empty() )
    {
        this->iter = this->widgetsVector.begin();
    }

    nextConnect.disconnect();

    nextConnect = signalNextWidget.connect ( sigc::mem_fun ( *this, &WizardWidgetManager::NextWidgetInternal ) );

    ShowWidget();
}

void WizardWidgetManager::Start ( const std::vector<WizardWidgetBase *>& widgetsVector )
{
    this->widgetsVector = widgetsVector;
    Start();
}

void WizardWidgetManager::Stop()
{
    nextConnect.disconnect();
}

WizardWidgetBase * WizardWidgetManager::getCurrentWidget()
{
    return ( *iter );
}

void WizardWidgetManager::refresh()
{
    if ( !widgetsVector.empty() && *iter )
    {
        ( *iter )->refresh();
    }
}
