/**
 * @file	pop-up-message-window.h
 * @brief	Declares the POP up message Windows Form.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmaticgui
 * Copyright (C) Juan Delgado 2011 <juan.delgado@smartmatic.com>
 *
 * smartmaticgui is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmaticgui is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _POP_UP_MESSAGE_WINDOW_H_

#define _POP_UP_MESSAGE_WINDOW_H_

#include <gtkmm.h>
#include "basic-window.h"
#include <Configuration/label-configuration.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace GUI
    {
        namespace Windows
        {

            /**
             * @class	PopUpMessageWindow
             * @brief	Form for viewing the POP up message.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class PopUpMessageWindow: public BasicWindow
            {
                public:

                    /**
                     * @fn	PopUpMessageWindow::PopUpMessageWindow();
                     * @brief	class constructor.
                     * @author	Yadickson Soto
                     * @date	23/10/2012
                     */
                    PopUpMessageWindow();

                    /**
                     * @fn	PopUpMessageWindow::PopUpMessageWindow( Glib::ustring text,
                     * 			Smartmatic::GUI::Configuration::LabelConfiguration & labelConf, int timerToClose,
                     * 			int opacity = 100);
                     * @brief	Constructor.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	text					The text.
                     * @param	labelConf			The label conf.
                     * @param	timerToClose	The timer to close.
                     * @param	opacity				(optional) the opacity.
                     */
                    PopUpMessageWindow (
                        Glib::ustring text,
                        Smartmatic::GUI::Configuration::LabelConfiguration & labelConf,
                        int timerToClose,
                        int opacity = 100 );

                    /**
                     * @fn	PopUpMessageWindow::PopUpMessageWindow(Gtk::Widget* child, int timerToClose);
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	child					If non-null, the child.
                     * @param	timerToClose	The timer to close.
                     */
                    PopUpMessageWindow ( Gtk::Widget * child, int timerToClose );

                    /**
                     * @fn	virtual PopUpMessageWindow::~PopUpMessageWindow();
                     * @brief	Finaliser.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~PopUpMessageWindow();

                    /**
                     * @fn	void PopUpMessageWindow::refreshText(Glib::ustring text);
                     * @brief	Refresh text.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	text	The text.
                     */
                    void refreshText ( Glib::ustring text );

                    /**
                     * @fn	void PopUpMessageWindow::setLabelConfig(Smartmatic::GUI::Configuration::LabelConfiguration & labelConf);
                     * @brief	Sets label configuration.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	labelConf	The label conf.
                     */
                    void setLabelConfig ( Smartmatic::GUI::Configuration::LabelConfiguration & labelConf );

                    /**
                     * @fn	void PopUpMessageWindow::setFontColorLabel(Gdk::Color & color);
                     * @brief	Sets font color label.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	color	The color.
                     */
                    void setFontColorLabel ( Gdk::Color & color );

                    /**
                     * @fn	void PopUpMessageWindow::setBoxColorLabel(Gdk::Color & color);
                     * @brief	Sets box color label.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	color	The color.
                     */
                    void setBoxColorLabel ( Gdk::Color & color );

                    /**
                     * @fn	virtual void PopUpMessageWindow::ShowDialog();
                     * @brief	Method  show the dialog.
                     * @author	Yadickson Soto
                     * @date	27/04/2011
                     */
                    virtual void ShowDialog();

                    /**
                     * @fn	virtual void PopUpMessageWindow::QuitDialog();
                     * @brief	Method quit dialog.
                     * @author	Yadickson Soto
                     * @date	27/04/2011
                     */
                    virtual void QuitDialog();

                    /**
                     * @fn	void restart ();
                     * @brief	Method restart timeout
                     * @author	Yadickson Soto
                     * @date	13/12/2012
                     */
                    void restart ();

                protected:

                    /**
                     * @fn	void PopUpMessageWindow::setTimerToClose(int timerToClose);
                     * @brief	Sets the timer that triggers the close the screen.
                     * @author	Yadickson Soto
                     * @date	23/10/2012
                     * @param	timerToClose	Is number of milliseconds between calls to that method.
                     */
                    void setTimerToClose ( int timerToClose );
                    /**
                     * @fn	void PopUpMessageWindow::setOpacity(int opacity);
                     * @brief	Method setter opacity.
                     * @author	Yadickson Soto
                     * @date	23/10/2012
                     * @param	opacity	opacity.
                     */
                    void setOpacity ( int opacity );
                    /**
                     * @fn	void PopUpMessageWindow::setText(Glib::ustring text);
                     * @brief	Method setter text.
                     * @author	Yadickson Soto
                     * @date	23/10/2012
                     * @param	text	text.
                     */
                    void setText ( Glib::ustring text );

                    /**
                     * @fn	void PopUpMessageWindow::setContainer(Gtk::Widget* child);
                     * @brief	Sets a container.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	child	If non-null, the child.
                     */
                    void setContainer ( Gtk::Widget * child );

                    /**
                     * @fn	bool PopUpMessageWindow::hideWindow(GdkEventButton* event);
                     * @brief	Hides the window.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     * @param	event	If non-null, the event.
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool hideWindow ( GdkEventButton * event );

                    sigc::connection conn_hide; ///< The connection hide
                    Gtk::EventBox * eventbox; ///< The eventbox
                    Gtk::Label * label; ///< The label

                    /**
                     * @fn	virtual void PopUpMessageWindow::on_show();
                     * @brief	Handles show signals.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */

                    virtual void on_show();
                    /**
                     * @fn	bool PopUpMessageWindow::myHide();
                     * @brief	method hide window.
                     * @author	Yadickson Soto
                     * @date	30/03/2012
                     * @return	false.
                     */
                    bool myHide();

                private:

                    Gtk::Widget * child; ///< The child
                    int timerToClose;   ///< The timer to close
                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger
            };
        }
    }
}

#endif // _POP_UP_MESSAGE_WINDOW_H_
