#ifndef TEST_SMTT_GUI_CONFIGURATION_MANAGER_TEST
#define TEST_SMTT_GUI_CONFIGURATION_MANAGER_TEST

#ifdef ECUT_MAIN

#include <Configuration/SmartmaticGUIConfiguration-schema.hxx>
#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockFile.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>
#include <tests/MockSmttGuiConfigurationManager.hxx>
#include <tests/MockProcessLauncher.hxx>
#include <tests/MockExitProcess.hxx>

class SmttGuiConfigurationTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;
        typedef Smartmatic::GUI::Configuration::SMTTGUIConfigurationManager SMTTGUIConfigurationManager;
        typedef Smartmatic::GUI::Configuration::SmartmaticGUIConfiguration SmartmaticGUIConfiguration;
        typedef Smartmatic::GUI::Configuration::SmartmaticGUIMachineModelConfiguration SmartmaticGUIMachineModelConfiguration;
        typedef Smartmatic::System::Runtime::ProcessLauncher ProcessLauncher;
        typedef Smartmatic::System::Runtime::ExitProcess ExitProcess;

    public:

        SmttGuiConfigurationTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testInitializeEmptyConfiguration()
        {
            MockConfigurationPersistenceManager manager;
            MockProcessLauncher launcher;
            MockExitProcess exitProcess;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            MOCKPP_CHAINER_FOR ( MockExitProcess, exitProcess ) exitProcess_mock ( &exitProcess );

            exitProcess_mock
            .expects ( MOCKPP_NS::once() );

            ExitProcess::getInstance ( &exitProcess );
            ProcessLauncher::getInstance ( &launcher );
            ConfigurationPersistenceManager::getInstance ( &manager );

            SMTTGUIConfigurationManager * smttManager = SMTTGUIConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( NULL == smttManager->getConfiguration() );

            manager.verify();
            launcher.verify();
            exitProcess.verify();

            delete smttManager;
        }

        void testInitializeConfigurationFail()
        {
            MockConfigurationPersistenceManager manager;
            MockProcessLauncher launcher;
            MockExitProcess exitProcess;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            MOCKPP_CHAINER_FOR ( MockExitProcess, exitProcess ) exitProcess_mock ( &exitProcess );

            exitProcess_mock
            .expects ( MOCKPP_NS::once() );

            ExitProcess::getInstance ( &exitProcess );
            ProcessLauncher::getInstance ( &launcher );
            ConfigurationPersistenceManager::getInstance ( &manager );

            ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "SMTT", "<abc>Hello</abc>" );
            SMTTGUIConfigurationManager * smttGuiManager = SMTTGUIConfigurationManager::getInstance();

            CPPUNIT_ASSERT ( NULL == smttGuiManager->getConfiguration() );

            manager.verify();
            launcher.verify();
            exitProcess.verify();

            delete smttGuiManager;
        }

        void testInitializeConfigurationLoadDefault()
        {
            MockConfigurationPersistenceManager manager;
            MockProcessLauncher launcher;
            MockExitProcess exitProcess;
            MockSmttGuiConfigurationManager smttGuiManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            MOCKPP_CHAINER_FOR_EXT ( MockProcessLauncher, RunProcessThroughSystem, ext ) RunProcessThroughSystem_mock ( &launcher );

            RunProcessThroughSystem_mock
            .expects ( MOCKPP_NS::never() )
            .will ( MOCKPP_NS::returnValue<int> ( 0 ) );

            MOCKPP_CHAINER_FOR ( MockExitProcess, exitProcess ) exitProcess_mock ( &exitProcess );

            exitProcess_mock
            .expects ( MOCKPP_NS::never() );

            ExitProcess::getInstance ( &exitProcess );
            ProcessLauncher::getInstance ( &launcher );
            ConfigurationPersistenceManager::getInstance ( &manager );
            SMTTGUIConfigurationManager::getInstance ( &smttGuiManager );

            CPPUNIT_ASSERT ( NULL != SMTTGUIConfigurationManager::getInstance()->getConfiguration() );

            manager.verify();
            launcher.verify();
            exitProcess.verify();
        }

        void testSmartmaticGUIMachineModelConfigurationWithoutValues()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSmttGuiConfigurationManager confManager;
            SMTTGUIConfigurationManager::getInstance ( &confManager );

            SmartmaticGUIMachineModelConfiguration::MachineModelSequence machineModelSequence;
            Smartmatic::GUI::Configuration::MachineModelType machineModel;
            machineModel.setId( "PM-VIU805" );
            Smartmatic::GUI::Configuration::WindowSizeType windowSize;
            windowSize.setHeight( 0 );
            windowSize.setWidth( 0 );
            machineModel.setWindowSize( windowSize );
            machineModel.setTitleHeight( 0 );
            Smartmatic::GUI::Configuration::WindowPaddingType windowPadding;
            windowPadding.setHorizontalPadding( 0 );
            windowPadding.setVerticalPadding( 0 );
            machineModel.setWindowPadding( windowPadding );
            machineModelSequence.push_back( machineModel );

            SmartmaticGUIMachineModelConfiguration & smartmaticGUIMachineModelConfiguration( SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration() );
            smartmaticGUIMachineModelConfiguration.setMachineModel(machineModelSequence);

            int w, h;

            SMTTGUIConfigurationManager::getInstance()->getWindowSize ( w, h );

            CPPUNIT_ASSERT ( 0 == w );
            CPPUNIT_ASSERT ( 0 == h );

            SMTTGUIConfigurationManager::getInstance()->getCenterSize ( w, h );

            CPPUNIT_ASSERT ( 0 == w );
            CPPUNIT_ASSERT ( 0 == h );

            SMTTGUIConfigurationManager::getInstance()->getHeaderSize ( w, h );

            CPPUNIT_ASSERT ( 0 == w );
            CPPUNIT_ASSERT ( 0 == h );

            SMTTGUIConfigurationManager::getInstance()->getCenterSizeWithHeader ( w, h );

            CPPUNIT_ASSERT ( 0 == w );
            CPPUNIT_ASSERT ( 0 == h );
        }

        void testSmartmaticGUIMachineModelConfigurationValues()
        {
            MockConfigurationPersistenceManager persistenceManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &persistenceManager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &persistenceManager );

            MockSmttGuiConfigurationManager confManager;
            SMTTGUIConfigurationManager::getInstance ( &confManager );

            SmartmaticGUIMachineModelConfiguration::MachineModelSequence machineModelSequence;
            Smartmatic::GUI::Configuration::MachineModelType machineModel;
            machineModel.setId( "PM-VIU805" );
            Smartmatic::GUI::Configuration::WindowSizeType windowSize;
            windowSize.setHeight( 600 );
            windowSize.setWidth( 1000 );
            machineModel.setWindowSize( windowSize );
            machineModel.setTitleHeight( 20 );
            Smartmatic::GUI::Configuration::WindowPaddingType windowPadding;
            windowPadding.setHorizontalPadding( 5 );
            windowPadding.setVerticalPadding( 10 );
            machineModel.setWindowPadding( windowPadding );
            machineModelSequence.push_back( machineModel );

            SmartmaticGUIMachineModelConfiguration & smartmaticGUIMachineModelConfiguration( SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration() );
            smartmaticGUIMachineModelConfiguration.setMachineModel(machineModelSequence);

            int w, h;

            SMTTGUIConfigurationManager::getInstance()->getWindowSize ( w, h );

            CPPUNIT_ASSERT ( 1000 == w );
            CPPUNIT_ASSERT ( 600 == h );

            SMTTGUIConfigurationManager::getInstance()->getCenterSize ( w, h );

            CPPUNIT_ASSERT ( 990 == w );
            CPPUNIT_ASSERT ( 590 == h );

            SMTTGUIConfigurationManager::getInstance()->getHeaderSize ( w, h );

            CPPUNIT_ASSERT ( 990 == w );
            CPPUNIT_ASSERT ( 20 == h );

            SMTTGUIConfigurationManager::getInstance()->getCenterSizeWithHeader ( w, h );

            CPPUNIT_ASSERT ( 990 == w );
            CPPUNIT_ASSERT ( 560 == h );
        }

        CPPUNIT_TEST_SUITE ( SmttGuiConfigurationTest );

        CPPUNIT_TEST ( testInitializeEmptyConfiguration );
        CPPUNIT_TEST ( testInitializeConfigurationFail );
        CPPUNIT_TEST ( testInitializeConfigurationLoadDefault );
        CPPUNIT_TEST ( testSmartmaticGUIMachineModelConfigurationWithoutValues );
        CPPUNIT_TEST ( testSmartmaticGUIMachineModelConfigurationValues );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SMTT_GUI_CONFIGURATION_MANAGER_TEST */
