/**
 * @file    ISMTTLog.h
 * @brief   Declares the ismtt log class.
 * @date 8/07/2011
 * @author gerardohernandez
 */

#ifndef ISMTTLOG_H_
#define ISMTTLOG_H_

#include <string>

namespace Smartmatic
{
    namespace Log
    {
        /**
         * @class   ISMTTLog
         * @brief   Interface to define the contract for the SMTT log.
         * @author  Fsilva
         * @date    28/04/2011
         */

        class ISMTTLog
        {
            public:

                /**
                 * @fn  virtual ISMTTLog::~ISMTTLog ();
                 * @brief   Class destructor.
                 * @author  Yadickson Soto
                 * @date    20/01/2012
                 */
                virtual ~ISMTTLog () { };

                /**
                 * @fn  virtual void ISMTTLog::Debug ( const std::string & messageCode ) = 0;
                 * @brief   Log the debug message.
                 * @author  Fsilva
                 * @date    27/04/2011
                 * @param   messageCode The message code.
                 */
                virtual void Debug ( const std::string & messageCode ) = 0;

                /**
                 * @fn  virtual void ISMTTLog::Audit ( const std::string & messageCode ) = 0;
                 * @brief   Log the audit message.
                 * @author  Fsilva
                 * @date    27/04/2011
                 * @param   messageCode The message code.
                 */
                virtual void Audit ( const std::string & messageCode ) = 0;

                /**
                 * @fn  virtual void ISMTTLog::Error ( const std::string & messageCode ) = 0;
                 * @brief   Log the error message.
                 * @author  Fsilva
                 * @date    27/04/2011
                 * @param   messageCode The message code.
                 */
                virtual void Error ( const std::string & messageCode ) = 0;
        };
    }
}

#endif /* ISMTTLOG_H_ */
