/**
 * @file LogInterface.hxx
 * @brief Declaration of the interface that abstracts to the entity logs base.
 * @date 18/07/2011
 * @author Yadickson Soto
 */

#ifndef LOGINTERFACE_HXX_
#define LOGINTERFACE_HXX_

#include <string>

namespace Smartmatic
{
    namespace Log
    {
        /**
         * @class   LogInterface
         * @brief   Interface that abstracts to the log base entity. This class contains the code of the log instance.
         * @author  Yadickson Soto
         * @date    18/07/2011
         */
        class LogInterface
        {
            public:

                /**
                 * @fn  LogInterface::LogInterface ( const std::string & code );
                 * @brief   Default constructor initializes an instance of the LogInterface Class.
                 * @author  Yadickson Soto
                 * @date    18/07/2011
                 * @param   code    code of interface.
                 */
                explicit LogInterface ( const std::string & code );

                /**
                 * @fn  LogInterface::~LogInterface ()
                 * @brief   Finalizes the instance of the LogInterface Class.
                 * @author  Yadickson Soto
                 * @date    31/10/2012
                 */
                virtual ~LogInterface ();

                /**
                 * @fn  std::string LogInterface::getCode ();
                 * @brief   Gets the code that identifies the instance.
                 * @author  Yadickson Soto
                 * @date    18/07/2011
                 * @return  The code of interface.
                 */
                std::string getCode () const;

            protected:
                std::string code; ///< The code that identifies the instance.
        };
    }
}

#endif /* LOGINTERFACE_HXX_ */
